package edu.bu.signstream.codingScheme.ui.importLegacyCodingScheme;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.dnd.DnDConstants;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import java.util.logging.Level;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTree;

import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Database;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Track;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.ui.ButtonPanel;
import edu.bu.signstream.ui.SS3SignStreamApplication;

public class CodingSchemeUpdatePanel extends JPanel implements ActionListener {

   private TreeDragSource ds;

   private TreeDropTarget dt;

   private JTree sourceTree;

   private JTree targetTree;

   private JDialog parentDialog;

   private SignStreamSegmentPanel ssPanel;

   private CodingSchemaService service;

   private boolean isSchemasIdentical = false;

   private JButton okBtn = new JButton("OK");
   private JButton cancelBtn = new JButton("Cancel");
   private JButton helpBtn = new JButton("Help");

   private final int OK = 0;
   private final int CANCEL = 1;
   private final int HELP = 2;

   public CodingSchemeUpdatePanel(
      HashMap lCodingSchema,
      SignStreamSegmentPanel ssPanel,
      JDialog parentDialog) {

      this.ssPanel = ssPanel;
      this.parentDialog = parentDialog;

      setSize(600, 400);

      init(lCodingSchema);
   }

   public void init(HashMap lCodingSchema) {

	  // GD: 2021-04-21 leaving this call to get default scheme in, as it's legacy code
      SS3CodingScheme defaultCS =
         SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();

      service = new CodingSchemaService();
      service.loadMasterSchema(defaultCS);

      LegacyCodingSchemaFormBean fb = service.compare(lCodingSchema);
      ArrayList difIDs = fb.getDifFieldIDs();

      isSchemasIdentical = difIDs.isEmpty();

      SS3SignStreamApplication.logger.log(
         Level.INFO,
         "Transform Legacy Coding Scheme to Sign Stream 3 Coding cheme format.");

      SS3CodingScheme legacySchema =
         service.translateToCodingSchema(fb, lCodingSchema, difIDs);

      CSDelegate sourceDelegate = new CSDelegate(legacySchema, service);
      CSDelegate targetDelegate = new CSDelegate(defaultCS, service);
      SortableTreeNode sourceNode = sourceDelegate.getSortableTreeNode();
      SortableTreeNode targetNode = targetDelegate.getSortableTreeNode();
      sourceTree = new JTree(sourceNode);
      targetTree = new JTree(targetNode);

      ButtonPanel buttonPanel = new ButtonPanel();
      buttonPanel.add(helpBtn);
      buttonPanel.add(cancelBtn);
      buttonPanel.add(okBtn);

      JPanel legacyPanel = new JPanel();
      legacyPanel.setLayout(new BorderLayout());
      JScrollPane jsp1 = new JScrollPane(sourceTree);
      jsp1.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
      legacyPanel.add(jsp1, BorderLayout.CENTER);
      legacyPanel.setBorder(
         BorderFactory.createTitledBorder("Legacy coding scheme"));

      JPanel defaultPanel = new JPanel();
      defaultPanel.setLayout(new BorderLayout());
      JScrollPane jsp2 = new JScrollPane(targetTree);
      jsp2.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
      defaultPanel.add(jsp2, BorderLayout.CENTER);
      defaultPanel.setBorder(
         BorderFactory.createTitledBorder("Default coding scheme"));

      JPanel migrationPanel = new JPanel();
      migrationPanel.setLayout(new GridLayout(0, 2, 20, 20));
      migrationPanel.add(legacyPanel);
      migrationPanel.add(defaultPanel);

      ComparedCodingSchemaPanel comparePanel =
         new ComparedCodingSchemaPanel(fb, parentDialog);

      JTabbedPane tabbedPane = new JTabbedPane();
      tabbedPane.add(comparePanel, "Legacy Data Compare Tool");
      tabbedPane.add(migrationPanel, "Legacy Data Migration Tool");

      setLayout(new BorderLayout());
      ///add(mb, BorderLayout.NORTH);
      add(tabbedPane, BorderLayout.CENTER);
      add(buttonPanel, BorderLayout.SOUTH);

      // If we only support move operations...
      // ds = new TreeDragSource(tree, DnDConstants.ACTION_MOVE);
      ds = new TreeDragSource(sourceTree, DnDConstants.ACTION_COPY_OR_MOVE);
      dt = new TreeDropTarget(targetTree, parentDialog);

      okBtn.setActionCommand("" + OK);
      cancelBtn.setActionCommand("" + CANCEL);
      helpBtn.setActionCommand("" + HELP);

      okBtn.addActionListener(this);
      cancelBtn.addActionListener(this);
      helpBtn.addActionListener(this);
   }

   @Override
public void actionPerformed(ActionEvent ae) {
      String str = ae.getActionCommand();

      switch (Integer.parseInt(str)) {
         case OK :
            if (!isSchemasIdentical && !ds.isTreeEmpty()) {
               int n =
                  JOptionPane.showConfirmDialog(
                     parentDialog,
                     "Not all data are transferred to new coding scheme. \n Do you want to continue?",
                     "Warning message",
                     JOptionPane.YES_NO_OPTION);

               if (n == JOptionPane.NO_OPTION)
                  return;
            }
            // update value and field refrences....
            SS3SignStreamApplication.logger.log(
               Level.INFO,
               "Updtaing SignStream 2 database fields and values references to SignStream 3 coding scheme...");
            updateSS3PanelCodingScheme();

            SS3SignStreamApplication.logger.log(
               Level.INFO,
               "SignStream 2 database references had been updated to SignStream 3 coding scheme...");
            parentDialog.dispose();
            break;
         case CANCEL :
            SS3SignStreamApplication.logger.log(
               Level.INFO,
               "Coding scheme transfer had been canceled.");
            parentDialog.dispose();
            break;
         case HELP :
            String warningTxt =
               isSchemasIdentical
                  ? "<HTML>Legacy Coding Schema and Default Coding Schema are identical.</HTML>"
                  : "<HTML>Legacy Coding Schema can not be transferred automatically. <BR>"
                     + "Please update Legacy Coding Schema with Sign Stream 2 and reload the file. <BR>"
                     + "Fields that should be updated are listed in 'Legacy Data Compare Tool'. <BR>"
                     + "If updae is not possible, then create new Coding Schema using our Legacy Data Migration Tool. </HTML>";
            
            SS3Singleton.getSignStreamApplication().confirmationDialog("Help", warningTxt);
            break;
      }
   }

   @Deprecated
   public void updateSS3PanelCodingScheme() {
      // add new coding scheme
      SS3CodingScheme cScheme = dt.getSS3CodingScheme();
	  // GD: 2021-04-21 leaving this call to get default scheme in, as it's legacy code
      SS3CodingScheme ssCs = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();
      ssCs.setFields(cScheme.getFields());

      // add legacy coding schema
      SS3CodingScheme lScheme = ds.getSS3CodingScheme();
      HashMap fields = lScheme.getFields();

      Set set = fields.keySet(); //.entrySet();
      Iterator iter = set.iterator();
      while (iter.hasNext()) {
         SS3Field field = (SS3Field) fields.get(iter.next());
         ssCs.addField(field);
      }

      // add CS to the database
      ssPanel.getSS3Database().setSS3CodingScheme(ssCs);

	  // GD: 2021-04-21 leaving this call to set local scheme in, as it's legacy code
      // add legacy CS to the database
      ssPanel.getSS3Database().setLocalSS3CodingScheme(lScheme);

      // update loaded database object
      updateDatabaseReferencesToCodingScheme(cScheme);
      //}
   }

   private void updateDatabaseReferencesToCodingScheme(SS3CodingScheme cScheme) {
      // update value references first
      updateDatabaseValueReferences(cScheme);
      // then update field references
      updateDatabaseFieldReferences(cScheme);
   }

   private void updateDatabaseFieldReferences(SS3CodingScheme cScheme) {
      SS3Database db = ssPanel.getSS3Database();
      HashMap fields = cScheme.getFields();
      Set set = fields.keySet();
      Iterator iterator = set.iterator();

      while (iterator.hasNext()) {
         String key = (String) iterator.next();
         Object value = fields.get(key);
         SS3Field field = (SS3Field) value;

         String legacyID = field.getLegacyID();
         if (legacyID != null && legacyID.length() > 0) {
            updateFieldReference(db, field);
         }
      }
   }

   private void updateDatabaseValueReferences(SS3CodingScheme cScheme) {
      SS3Database db = ssPanel.getSS3Database();
      HashMap fields = cScheme.getFields();
      Set set = fields.keySet();
      Iterator iterator = set.iterator();

      while (iterator.hasNext()) {
         String key = (String) iterator.next();
         Object value = fields.get(key);
         SS3Field field = (SS3Field) value;

         ArrayList values = field.getValuesAsList();
         for (int i = 0; i < values.size(); i++) {
            SS3FieldValue fValue = (SS3FieldValue) values.get(i);
            String legacyValueID = fValue.getLegacyID();

            if (legacyValueID != null && legacyValueID.length() > 0) {
               updateValueReference(db, fValue);
            }
         }
      }
   }

   private void updateFieldReference(SS3Database db, SS3Field field) {
      String legacyID = field.getLegacyID();
      SS3Track track = getSS3Track(db, legacyID);

      if (track != null)
         track.setId(field.getId());
   }

   private SS3Track getSS3Track(SS3Database db, String id) {
      ArrayList list = db.getTracksList();

      for (int i = 0; i < list.size(); i++) {
         SS3Track track = (SS3Track) list.get(i);
         String tID = track.getId();

         if (tID.equalsIgnoreCase(id))
            return track;
      }

      return null;
   }

   private void updateValueReference(SS3Database db, SS3FieldValue value) {
      String legacyFieldID = value.getParentFieldID();
      String legacyValueID = value.getLegacyID();

      SS3Track track = getSS3Track(db, legacyFieldID);

      if (track != null) {
         ArrayList entities = track.getEntities();
         for (int i = 0; i < entities.size(); i++) {
            SS3Entity entity = (SS3Entity) entities.get(i);
            ArrayList items = entity.getItemsAsList();
            for (int j = 0; j < items.size(); j++) {
               SS3Item item = (SS3Item) items.get(j);
               if (item.getId().equalsIgnoreCase(legacyValueID)) {
                  item.setId(value.getId());
                  return;
               }
            }
         }
      }
   }
}
