package edu.bu.signstream.common.db;

import java.io.File;
import java.io.FileOutputStream;
import java.io.FilePermission;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import java.util.logging.Level;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import edu.bu.signstream.common.util.vo.CodingSchemaField;
import edu.bu.signstream.common.util.vo.CodingSchemaValue;
import edu.bu.signstream.common.util.vo.MediaFile;
import edu.bu.signstream.common.util.vo.Participant;
import edu.bu.signstream.common.util.vo.Profile;
import edu.bu.signstream.common.util.vo.Segment;
import edu.bu.signstream.common.util.vo.SignStreamDocument;
import edu.bu.signstream.common.util.vo.Track;
import edu.bu.signstream.common.util.vo.TrackValue;
import edu.bu.signstream.common.util.vo.Utterance;
import edu.bu.signstream.ui.SS3SignStreamApplication;

public class XMLAdaptor {  
   
  public Document createNewXMLDocument(String path) {
	Document document = null;
		
	try {	  
	  DocumentBuilderFactory domFactory = DocumentBuilderFactory.newInstance();
	  DocumentBuilder domBuilder = domFactory.newDocumentBuilder();
	  document = domBuilder.newDocument();
	} catch(Exception ex) {
		SS3SignStreamApplication.logger.log(Level.INFO, ex.getMessage());
	}
	return document;	
  }
  
  
  public void saveDocument(SignStreamDocument doc, String fileName) throws Exception {
	Document document = createNewXMLDocument(fileName);
	
	try {	 
	  Element root = document.createElement("SIGNSTREAM-DATABASE");
	  Element distributor = document.createElement("DISTRIBUTOR");
	  Element author = document.createElement("AUTHOR");
	  Element participants = document.createElement("PARTICIPANTS");
	  Element codingScheme = document.createElement("CODING-SCHEME");
	  Element mediaFiles = document.createElement("MEDIA-FILES");
	  Element utterances = document.createElement("UTTERANCES");
	  
	  Profile header = doc.getProfile();
	  
	  if(header != null) {
	    root.setAttribute("SIGNSTREAM-VERSION", "" + header.getSignStreamVersion());
	    root.setAttribute("SOURCE", ""+header.getSourcePath());
	    root.setAttribute("DATABASE-VERSION", ""+header.getDatabaseVersion());
	  }  
	    root.appendChild(distributor);
	    root.appendChild(author);
	    root.appendChild(participants);
	    root.appendChild(codingScheme);
	    root.appendChild(mediaFiles);
	    root.appendChild(utterances);
	  
	  if(header != null) {
	    distributor.appendChild(document.createTextNode(header.getDistributor()));
	    author.appendChild(document.createTextNode(header.getAuthor()));
	  }
	  
	  // PARTICIPANTS
      HashMap prtpMap = doc.getParticipants();
      Set prtpSet = prtpMap.keySet();
      Iterator prtpIt = prtpSet.iterator();
      while(prtpIt.hasNext()) {
      	String key = (String)prtpIt.next();
  
        Participant part = (Participant)prtpMap.get(key);
		Element participant = document.createElement("PARTICIPANT");
		participant.setAttribute("ID", "" + part.getParticipantId());
		participant.setAttribute("NAME", "" + part.getName());
		participant.setAttribute("LABEL", "" + part.getLabel());
		participant.setAttribute("AGE", "" + part.getAge());
		participant.setAttribute("GENDER", "" + part.getGender());
		participant.setAttribute("LANGUAGE", "" + part.getLanguage());
		
		ArrayList bckgrList = part.getBackground();
		for(int i = 0; i < bckgrList.size(); i ++) {
		  Element background = document.createElement("BACKGROUND");
		  background.appendChild(document.createTextNode((String)bckgrList.get(i)));
		  participant.appendChild(background);
		}
		participants.appendChild(participant);
      }
      
      // CODING SCHEME
	  HashMap schemeMap = doc.getCodingScheme();
	  Set schemeSet = schemeMap.keySet();
	  Object[] objects = schemeSet.toArray();
	  for(int i = 0; i < objects.length; i ++) {
		String key = (String) objects[i]; 
		CodingSchemaField csField = (CodingSchemaField) schemeMap.get(key);
		Element field = document.createElement("FIELD");
		codingScheme.appendChild(field);
		field.setAttribute("ID", ""+csField.getFieldId());
		field.setAttribute("NAME", csField.getName());
		
		if(csField.getLabel() != null) 
		  field.setAttribute("LABEL", csField.getLabel());
		
		if(csField.getPrefix() != null) 
		  field.setAttribute("PREFIX", csField.getPrefix());
		  
		if(csField.getConstraint() != null)  
		  field.setAttribute("CONSTRAINT", csField.getConstraint());
		  
		if(csField.getTimeAlignment() != null)  
		  field.setAttribute("TIME-ALIGNMENT", csField.getTimeAlignment());
		
		if(csField.getHexColor() != null)
		  field.setAttribute("COLOR", csField.getHexColor());
		
		ArrayList fieldArray = csField.getCodingSchemeValues();
		for(int k = 0; k < fieldArray.size(); k ++) {
		  CodingSchemaValue shemeValue = (CodingSchemaValue)fieldArray.get(k);
		  Element value = document.createElement("VALUE");
		  value.setAttribute("ID", ""+shemeValue.getValueId());
		  value.setAttribute("NAME", shemeValue.getName());
		  value.setAttribute("LABEL", shemeValue.getLabel());
		  field.appendChild(value);
		}
		
		ArrayList defFieldArray = csField.getDefaultCodingSchemeValues();
		for(int l = 0; l < defFieldArray.size(); l ++) {
		  CodingSchemaValue shemeValue = (CodingSchemaValue)defFieldArray.get(l);
		  Element value = document.createElement("VALUE");
		  value.setAttribute("ID", ""+shemeValue.getValueId());
		  value.setAttribute("NAME", shemeValue.getName());
		  field.appendChild(value);
		}		
	 }
		 
	 //	MEDIA-FILES
	 HashMap mediaMap = doc.getMediaFiles();
	 Set mediaSet = mediaMap.keySet();
	 Iterator mediaIt = mediaSet.iterator();
	 while(mediaIt.hasNext()) {
	   String key = (String) mediaIt.next();
	   MediaFile mFile = (MediaFile)mediaMap.get(key);
	   Element media = document.createElement("MEDIA-FILE");
	   mediaFiles.appendChild(media);
	   media.setAttribute("ID", mFile.getId());
	   media.setAttribute("LEGACY-PATH", mFile.getLegacyPath());
	 }
	 
     // UTTERANCES
	 HashMap utterMap = doc.getUtterances();
	 Set utterSet = utterMap.keySet();
	 Iterator utterIt = utterSet.iterator();
	 while(utterIt.hasNext()) {
	   String key = (String)utterIt.next(); 
	   Utterance utter = (Utterance)utterMap.get(key);
	   Element utterance = document.createElement("UTTERANCE");
	   utterances.appendChild(utterance);
	   utterance.setAttribute("ID", utter.getId());
	   utterance.setAttribute("EXCERPT",utter.getExcerpt());
	   utterance.setAttribute("S", ""+utter.getStartTimeInfo().doubleValue());
	   utterance.setAttribute("E", ""+utter.getEndTimeInfo().doubleValue());
	   
	   ArrayList notesAr = utter.getNotes();
	   Iterator noteIt = notesAr.iterator();
	   while(noteIt.hasNext()) {
	   	 String note = (String)noteIt.next();
		 Element notes = document.createElement("NOTES");
		 notes.appendChild(document.createTextNode(note));	
		 utterance.appendChild(notes);	 
	   }	   
	   
	   HashMap mediaRefMap = utter.getMediaFiles();
	   Set mediaRefSet = mediaRefMap.keySet();
	   Iterator mediaRefIt = mediaRefSet.iterator();
	   while(mediaRefIt.hasNext()) {
		 Element mediaRef = document.createElement("MEDIA-REF");
		 utterance.appendChild(mediaRef);
		 String nkey = (String)mediaRefIt.next(); 
		 MediaFile mFile = (MediaFile)mediaRefMap.get(nkey);		
		 mediaRef.setAttribute("ID", mFile.getId());
	   }
	   
	   ArrayList segmentsMap = utter.getSegments();
	   Iterator segmentsIt = segmentsMap.iterator();
	   while(segmentsIt.hasNext()) {
	   	 Segment segm = (Segment)segmentsIt.next();
		 Element segment = document.createElement("SEGMENT");
		 utterance.appendChild(segment);
		 segment.setAttribute("PARTICIPANT-ID", segm.getParticipantId());
		 segment.setAttribute("PRIMARY", ""+segm.isPrimaryParticipant());
			 
		 HashMap trackMap = segm.getTracks();
		 Set trackSet = trackMap.keySet();
		 Iterator trackIt = trackSet.iterator();
		 	 
		 while(trackIt.hasNext()) {
		   String tkey = (String)trackIt.next(); 
		   Track trck = (Track)trackMap.get(tkey);
		   Element track = document.createElement("TRACK");
		    segment.appendChild(track);
		    track.setAttribute("FID", trck.getFieldId());
		    
			ArrayList trackValues = trck.getTrackValues();
			for(int j = 0; j < trackValues.size(); j ++) {
			  TrackValue trackValue = (TrackValue)trackValues.get(j);
			  Element value = document.createElement("A");
			  track.appendChild(value);
			  value.setAttribute("S", ""+trackValue.getStartTimeInfo().doubleValue());
			  value.setAttribute("E", ""+trackValue.getEndTimeInfo().doubleValue());
			  
			  int valueID = trackValue.getValueId();
			  if(valueID > -1) {
			    value.setAttribute("VID", ""+valueID);
			  } 

			  String text = trackValue.getTextValue();
			  if(text != null && text.length() > 0) 			  
			    value.appendChild(document.createTextNode(text));
			}			
		 }		  
	   }	   
	 }
	 
	 saveDocument(root, fileName);
     	  
	} catch(Exception ex){      
		SS3SignStreamApplication.logger.log(Level.INFO, "Can't save xml file. " + ex.getMessage());

     throw new Exception("Can't save xml file");
	}
  }
  
  public void saveDocument(Element root, String fileName) {
	try {
	  File file = new File(fileName);
	  
	  if(!file.exists()) {
	    file.createNewFile();
	  }
	  
	  FilePermission permission = new FilePermission(fileName, "read,write");	  
	  OutputStream outputStream = new FileOutputStream(file);
	  TransformerFactory tFactory = TransformerFactory.newInstance();
	  Transformer transformer = tFactory.newTransformer();

	  DOMSource source = new DOMSource(root);
	  StreamResult result = new StreamResult(outputStream);
	  transformer.transform(source, result);

	  outputStream.flush();
	  outputStream.close();
    } catch(Exception ex) {
    	SS3SignStreamApplication.logger.log(Level.INFO, ex.getMessage());
    }
  } 
}
