package edu.bu.signstream.common.db.ss3;

import static edu.bu.signstream.common.db.ss4.SS4Util.readLongOpt;

import java.util.Comparator;
import java.util.Optional;

import org.w3c.dom.Element;

public abstract class ValidatorConverter {
	SS3ValidatorConverterReturns returns = new SS3ValidatorConverterReturns();
	private String sysadminMsg1 = "The following error has been detected in your collection file: [";
	private String sysadminMsg2 = "] For assistance, please refer to the online documentation.";
    
    boolean isBlank(String val) {
    	return val == null || val.isBlank();
    }
    
    boolean isNotBlank(String val) {
    	return val != null && !val.isBlank();
    }

    boolean expect(boolean cond, int errorCode, String msgError, String msgPublicError) {
        if (!cond) {
        		Tuple tupleError = new Tuple(errorCode, msgError, true);
        		returns.errors.add(tupleError);
        		
        		if (isBlank(msgPublicError))
        			msgPublicError = sysadminMsg1 + msgError + sysadminMsg2;

        		Tuple tuplePublicError = new Tuple(errorCode, msgPublicError, true);
        		returns.publicErrors.add(tuplePublicError);
        }
        return cond;
    }
    boolean expect(boolean cond, int errorCode, String msgError, String msgPublicError, boolean makePublic) {
    	if (!cond) {
    		Tuple tupleError = new Tuple(errorCode, msgError, makePublic);
    		returns.errors.add(tupleError);

    		if (isBlank(msgPublicError)) {
    			msgPublicError = sysadminMsg1 + msgError + sysadminMsg2;
    		}

    		if (makePublic) {
    			Tuple tuplePublicError = new Tuple(errorCode, msgPublicError, makePublic);
    			returns.publicErrors.add(tuplePublicError);
    		}
    	}
    	return cond;
    }
    <T> boolean expect(Optional<T> opt, int errorCode, String msgError, String msgPublicError) {
        if (opt.isEmpty()) {
    		Tuple tupleError = new Tuple(errorCode, msgError, true);
        		returns.errors.add(tupleError);
        		
        		if (isBlank(msgPublicError))
        			msgPublicError = sysadminMsg1 + msgError + sysadminMsg2;

        		Tuple tuplePublicError = new Tuple(errorCode, msgPublicError, true);
        		returns.publicErrors.add(tuplePublicError);        				
        }
        return opt.isPresent();
    }

    interface Testable {
        void test() throws Exception;
    }
    boolean test(Testable test, int errorCode, String msgError, String msgPublicError) {
        try {
            test.test();
            return true;
        } catch (Exception e) {
    		Tuple tupleError = new Tuple(errorCode, msgError);
            returns.errors.add(tupleError);
            
            if (isBlank(msgPublicError))
            		msgPublicError = sysadminMsg1 + msgError + sysadminMsg2;

    		Tuple tuplePublicError = new Tuple(errorCode, msgPublicError);
            	returns.publicErrors.add(tuplePublicError);
            	
            return false;
        }
    }
	
	class ElementComparator implements Comparator<Element> {
		@Override
		public int compare(Element o1, Element o2) {
            var o1start = readLongOpt(o1, "S").orElse(null);
            var o1end = readLongOpt(o1, "E").orElse(null);
            var o2start = readLongOpt(o2, "S").orElse(null);
            var o2end = readLongOpt(o2, "E").orElse(null);
            
            var startCompare = o1start.compareTo(o2start);
            var endCompare = o1end.compareTo(o2end);
            if (startCompare == endCompare)
            	return startCompare;
            else {
            	if (startCompare == 0)
            		return endCompare;
            	else if (endCompare == 0)
            		return startCompare;
            }
			
			return 0;
		}
    }
	ElementComparator comp = new ElementComparator();

    public String repr(Element el, String... attrs) {
        var b = new StringBuilder();
        b.append("(");

        int n = attrs.length;
        for (int i = 0; i < n; i++) {
            b.append(attrs[i]);
            b.append("=\"");
            String str = el.getAttribute(attrs[i]);
            str = str.replace("\"","\\\"").replace("\\", "\\\\");
            b.append(str); 
            if (i != n - 1)
                b.append("\", ");
            else
                b.append("\"");
        }

        b.append(")");
        return b.toString();
    }
    
    public String repr(String str) {
    	var b = new StringBuilder();
    	b.append("\"");
    	
    	str = str.replace("\"","\\\"").replace("\\", "\\\\");
    	b.append(str);
    	
    	b.append("\"");
    	
    	return b.toString();
    }
    
    public void addToErrors(int errorCode, String msgError, String msgPublicError, boolean makePublic) {
		Tuple tupleError = new Tuple(errorCode, msgError, makePublic);
    	returns.errors.add(tupleError);
    	
    	if (isBlank(msgPublicError)) {
    		msgPublicError = sysadminMsg1 + msgError + sysadminMsg2;
    	}
    	
    	if (makePublic) {
    		Tuple tuplePublicError = new Tuple(errorCode, msgPublicError, makePublic);
    		returns.publicErrors.add(tuplePublicError);
    	}
    }

    public void addToErrors(int errorCode, String msgError, String msgPublicError) {
		Tuple tupleError = new Tuple(errorCode, msgError, true);
    	returns.errors.add(tupleError);
    	
    	if (isBlank(msgPublicError)) {
    		msgPublicError = sysadminMsg1 + msgError + sysadminMsg2;
    	}

		Tuple tuplePublicError = new Tuple(errorCode, msgPublicError, true);
    	returns.publicErrors.add(tuplePublicError);
    }
    
    public void addToModifications(int modificationCode, String msgModification, String msgPublicModification) {
    	boolean isPublic = false;
    	
    	if (isNotBlank(msgPublicModification)) {
    		isPublic = true;
			Tuple tuplePublicModification = new Tuple(modificationCode, msgPublicModification, isPublic);
			returns.publicModifications.add(tuplePublicModification);
    	}
    	
    	Tuple tupleModification = new Tuple(modificationCode, msgModification, isPublic);
    	returns.modifications.add(tupleModification);
    }
}
