package edu.bu.signstream.common.db.ss4;

import edu.bu.signstream.common.db.UtilityFilesXMLWriter;
import edu.bu.signstream.common.files.SSResources;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.Excerpt;
import edu.bu.signstream.common.util.vo.ss3.Author;
import edu.bu.signstream.common.util.vo.ss3.Reference;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.*;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field.Alignment;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.handShapes.CSHandShapeItem;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.handShapes.CSHandShapePalette;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.handShapes.CSHandShapeTemplate;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.handShapes.CSHandShapeTemplates;
import edu.bu.signstream.common.util.vo.ss3.db.*;
import edu.bu.signstream.common.util.vo.ss3.note.SS3Note;
import edu.bu.signstream.common.util.vo.ss3.participants.ParticipantsLibrary;
import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.grepresentation.fields.gestureField.signTypeField.GestureCategory;
import edu.bu.signstream.grepresentation.fields.gestureField.signTypeField.GestureType;
import edu.bu.signstream.grepresentation.fields.gestureField.signTypeField.GestureTypeCategoryObject;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.IPAKeyboard;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations.LocationConfig;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations.LocationImage;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations.LocationType;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations.Region;
import edu.bu.signstream.time.*;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureEntityProperties;
import edu.bu.signstream.grepresentation.fields.nonManualField.GlossEntityProperties;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.signbank.SignBankCollection;
import edu.bu.signstream.grepresentation.view.signbank.SignBankOccurrence;
import edu.bu.signstream.ui.SS3SignStreamApplication;
import org.w3c.dom.CharacterData;
import org.w3c.dom.*;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import java.awt.*;
import java.util.*;
import java.util.logging.Level;
import java.util.stream.Collectors;

import static edu.bu.signstream.common.db.ss4.SS4Util.*;

/**
 * The SS3ZipXmlFilesParser class parses SignStream files of format  filename.ss3 or fileName.ss3.zip
 * fileName.ss3 ot filename.ss3.zip is a zipped directory that contains one or more macroUnit.xml files
 * and optionally utility files.
 */

public class SS4Reader {
    private void parseUnits(SS3Database db, Element dbEl) {
        readChildList(dbEl, TIMEUNIT_LIST_TAG, TIMEUNIT_TAG)
            .map(el -> readTimeUnitEl(el))
            .forEach(db::addTimeUnit);
    }

    private void parseRanges(SS3Database db, Element dbEl) {
        readChildList(dbEl, TIMERANGE_LIST_TAG, TIMERANGE_TAG)
            .map(el -> readTimeRangeEl(db, el))
            .forEach(range -> {
                if (range.equals(TimeRange.ABSOLUTE)) {
                    // If reading the ABSOLUTE TimeRange from the XML, apply its settings to the real one.
                    TimeRange.ABSOLUTE.step(range.step());
                } else {
                    db.addTimeRange(range);
                }
            });
    }

    private void parseMedia(SS3Database db, Element dbEl) {
        var media = readChild(dbEl, "MEDIA");

        readChildListOrdered(media, VIDEO_LIST_TAG, VIDEO_TAG)
            .map(e -> readVideoEl(db, e))
            .forEach(v-> db.addVideo(v, v.order()));

        readChildListOrdered(media, AUDIO_LIST_TAG, AUDIO_TAG)
            .map(e -> readAudioEl(db, e))
            .forEach(a -> db.addAudio(a, a.order()));

        readChildListOrdered(media, GRAPH_LIST_TAG, GRAPH_TAG)
            .map(e -> readGraphEl(db, e))
            .forEach(db::addGraph);
    }

	/**
     * Parses database xml file.
     * @param  fileName a unzipped file name of an xml file.
     * @return  populated SS3Database object
     */
    public SS3Database readDatabase(String fileName) {
        try {
            DocumentBuilder builder = DocumentBuilderFactory.newInstance()
                    .newDocumentBuilder();
            Document _doc = null;
            try {
                _doc = builder.parse(fileName);
            } catch (java.net.MalformedURLException ex) {
                String space = "%20";
                StringBuffer sb = new StringBuffer();
                StringTokenizer tokenizer = new StringTokenizer(fileName);
                sb.append(tokenizer.nextElement());

                while (tokenizer.hasMoreTokens()) {
                    sb.append(space);
                    sb.append(tokenizer.nextElement());
                }
                _doc = builder.parse(sb.toString());
            }
            // Make it final so it can be used in lambdas...
            var doc = _doc;

            NodeList nodes1 = doc.getElementsByTagName("SIGNSTREAM-DATABASE");
            Element dbEl = (Element) nodes1.item(0);

            /* TODO: compare database and app versions
            String ssVersion = excEl.getAttribute("SIGNSTREAM-VERSION");
            String currentVersion = SS3Singleton.getSignStreamApplication().getVersion();
            if (currentVersion > ssVersion) {
                fix problems with old collection
            } else if (currentVersion < ssVersion) {
                you opened something from the future how
            }
            */


            // TODO: Parse actual range, account for TYPE

            SS3Database db = new SS3Database();

            parseUnits(db, dbEl);
            parseRanges(db, dbEl);
            parseMedia(db, dbEl);

            db.setFileName(fileName);

            String excerpt = dbEl.getAttribute("EXCERPT");
            String excerptID = dbEl.getAttribute("EXCERPT_NUMBER");
            db.setExcerpt(excerpt);
            db.setNumber(excerptID);
            db.setExcerptObj(new Excerpt(excerptID, excerpt));

            for (int i = 0; i < nodes1.getLength(); i++) {
                Element el1 = (Element) nodes1.item(i);
                NodeList refList = el1.getElementsByTagName("REFERENCES");

                Element el2 = (Element) refList.item(0);
                NodeList cSchRefList = el2
                        .getElementsByTagName("CODING-SCHEMES");
                NodeList prtRefList = el2.getElementsByTagName("PARTICIPANTS");

                NodeList antRefList = el2.getElementsByTagName("ANNOTATORS");

                ArrayList<SS3Participant> selectedParticipants = new ArrayList<SS3Participant>();
                Element prtElement0 = (Element) prtRefList.item(0);

                if (prtElement0 != null) {
                    NodeList prtcpNode = prtElement0
                            .getElementsByTagName("PARTICIPANT");
                    ParticipantsLibrary participantsLibrary = SS3Singleton
                            .getManager().getLoadedParticipantsLibrary();

                    for (int ii = 0; ii < prtcpNode.getLength(); ii++) {
                        Element prtElement = (Element) prtcpNode.item(ii);
                        SS3Participant participant = null;
                        String participantID = prtElement.getAttribute("ID");
                        String participantLabel = prtElement.getAttribute("LABEL");
                        
                        if (prtElement.getAttributes().getLength() > 2) {
                            participant = new SS3Participant();

                            participant.setId(participantID);
                            participant.setLabel(participantLabel);
                            participant.setBackground(prtElement.getAttribute("BACKGROUND"));
                            participant.setComments(prtElement.getAttribute("COMMENTS"));
                            participant.setDominantHand(prtElement.getAttribute("DOMINANT_HAND"));
                            participant.setFullName(prtElement.getAttribute("FULL_NAME"));
                            participant.setGender(prtElement.getAttribute("GENDER"));
                            participant.setNativeLanguage(prtElement.getAttribute("NATIVE_LANGUAGE"));
                            participant.setParentsInfo(prtElement.getAttribute("PARENT_INFO"));
                            participant.setYearOfBirth(prtElement.getAttribute("YEAR_OF_BIRTH"));

                            participantsLibrary.addLocalParticipant(participant);
                            new UtilityFilesXMLWriter().saveParticipantsLibrary(participantsLibrary);
                        } else if (participantID != null && participantID.length() > 0) {
                            participant = participantsLibrary.getParticipant(participantID);
                        }
                        if (participant != null) {
                            String graphFileID = readRef(prtElement, "GRAPH");
                            if (!graphFileID.isEmpty()) {
                                participant.setGraphFileID(graphFileID);
                            }
                            selectedParticipants.add(participant);
                        }
                    }
                }

                ArrayList<SS3Participant> selectedAnnotators = new ArrayList<SS3Participant>();
                Element antElement0 = (Element) antRefList.item(0);

                if (antElement0 != null) {
                    NodeList antNode = antElement0.getElementsByTagName("ANNOTATOR");

                    // check, had not been loaded
                    ParticipantsLibrary annotatorsLibrary = SS3Singleton
                            .getManager().getLoadedAnnotatorsLibrary();

                    for (int jj = 0; jj < antNode.getLength(); jj++) {
                        Element antElement = (Element) antNode.item(jj);
                        String annotatorID = antElement.getAttribute("ID");
                        String annotatorLabel = antElement.getAttribute("LABEL");
                        
                        SS3Participant annotator = new SS3Participant();

                        annotator.setId(annotatorID);
                        annotator.setLabel(annotatorLabel);

                        annotatorsLibrary.addLocalParticipant(annotator);
                        new UtilityFilesXMLWriter().saveAnnotatorsLibrary(annotatorsLibrary);

                        selectedAnnotators.add(annotator);
                    }
                }

                ArrayList<Reference> cShRef = parseCodingSchemes(cSchRefList);
                db.setCodingSchemeRef(cShRef);
                db.setSelectedParticipants(selectedParticipants);
                db.setSelectedAnnotators(selectedAnnotators);


                SS3CodingScheme cScheme = new SS3CodingScheme();
                NodeList lCodingSchemeList = el1
                        .getElementsByTagName("LOCAL-CODING-SCHEME");
                cScheme.setFields(parseLocalFields(lCodingSchemeList));
                db.setLocalSS3CodingScheme(cScheme);

                NodeList dbList = el1.getElementsByTagName("DATABASE");

                for (int l = 0; l < dbList.getLength(); l++) {
                    Element el10 = (Element) dbList.item(i);

                    NodeList tracksList = el10.getElementsByTagName("TRACKS");

                    ArrayList<SS3Track> tracks = parseTracks(tracksList, db);
                    for (SS3Track track : tracks) {
                        db.addTrack(track);
                    }

                    SS3GlossWindow glossWinow = parseSS3GlossWindow(el10, db);

                    db.setSS3GlossWindow(glossWinow);

//                    NodeList ndsList = el10.getElementsByTagName("NOTES");
//                    ArrayList<SS3Note> notes = parseNotes(ndsList);
//                    for (int k = 0; k < notes.size(); k++) {
//                        SS3Note note = notes.get(k);
//                        db.addNote(note);
//                    }
                }
            }
            return db;
        } catch (Exception e) {
            e.printStackTrace();
            SS3SignStreamApplication.logger.log(Level.INFO, e.getMessage());
        }
        return null;
    }

    /*
     * This method parses gloss-window node of a database (xml) file. Bellow is an example of a node.
     *
     * <GLOSS-WINDOW>
     *     <DISPLAY-TIME-PERIOD/>
     *     <TEMPORAL-PARTITIONS>
     *         <TEMPORAL-PARTITION ID="499147" LABEL="Temporal Partition 111">
     *             <SEGMENT-TIER ID="499148" LABEL="Segment 1">
     *                 <SEGMENT-TIER-UTTERANCE E="-1" ID="6046711" LABEL="U1" S="-1">
     *                     <STATEMENT-FIELD ENTITY-IDS="129010308179213, 129010309531615" FIELD-ID="10000"/>
     *                     <STATEMENT-FIELD ENTITY-IDS="129010314238016, 129010316766137" FIELD-ID="27"/>
     *                 </SEGMENT-TIER-UTTERANCE>
     *             </SEGMENT-TIER>
     *         </TEMPORAL-PARTITION>
     *     </TEMPORAL-PARTITIONS>
     * </GLOSS-WINDOW>
     */
    private SS3GlossWindow parseSS3GlossWindow(Element el, SS3Database db) {
        SS3GlossWindow glsWindow = new SS3GlossWindow();

        readRefs(el, "VISIBLE", "FIELD")
            .forEach(id -> glsWindow.addPermanentFieldAsID(id));

        NodeList displayTimePeriodNode = el
                .getElementsByTagName("DISPLAY-TIME-PERIOD");
        Element displayTimePeriodElement = (Element) displayTimePeriodNode
                .item(0);
        SS3GlossWindowSegment displayTimePeriod = new SS3GlossWindowSegment();
        displayTimePeriod.setLabel("Display Time Period");
        glsWindow.setDisplayTimePeriod(displayTimePeriod);

        NodeList temporalPartitionsNode = el
                .getElementsByTagName("TEMPORAL-PARTITIONS");
        Element temporalPartitionsElement = (Element) temporalPartitionsNode
                .item(0);
        NodeList temporalPartitionNode = temporalPartitionsElement
                .getElementsByTagName("TEMPORAL-PARTITION");

        for (int j = 0; j < temporalPartitionNode.getLength(); j++) {
            Element temporalPartitionEl = (Element) temporalPartitionNode
                    .item(j);

            SS3TemporalPartition temporalPartition = new SS3TemporalPartition();
            String temporalPartitionLabel = temporalPartitionEl
                    .getAttribute("LABEL");

            temporalPartition.setID(temporalPartitionEl.getAttribute("ID"));
            temporalPartition.setLabel(temporalPartitionLabel);
            glsWindow.addTemporalPartition(temporalPartition);

            var segmentItems = new HashMap<Time, SS3GlossWindowItem>();

            NodeList temporalPartitionSegmentNode = temporalPartitionEl
                    .getElementsByTagName("SEGMENT-TIER");

            for (int l = 0; l < temporalPartitionSegmentNode.getLength(); l++) {
                Element temporalPartitionSegmentEl = (Element) temporalPartitionSegmentNode
                        .item(l);

                SS3GlossWindowSegment temporalPartitionSegment = new SS3GlossWindowSegment();
                String temporalPartitionSegmentID = temporalPartitionSegmentEl
                        .getAttribute("ID");

                SS3Participant part = db.getSelectedParticipant(
                        readRef(temporalPartitionSegmentEl, "PARTICIPANT"));
                String isDef = temporalPartitionSegmentEl.getAttribute("IS_DEFAULT");
                String indexFieldID = readRef(temporalPartitionSegmentEl, "INDEX_FIELD");

                String currentGestureCounter = temporalPartitionSegmentEl.getAttribute("CURRENT_GESTURE_COUNTER");

                String showAllGestureCountersStr = temporalPartitionSegmentEl.getAttribute("SHOW_ALL_GESTURE_COUNTERS");
                boolean showAllGestureCounters = false;
                if (showAllGestureCountersStr != null && showAllGestureCountersStr.trim().equalsIgnoreCase("T"))
                    showAllGestureCounters = true;

                String defaultShowGestureCountersStr = temporalPartitionSegmentEl.getAttribute("DEFAULT_SHOW_GESTURE_COUNTERS");
                boolean defaultShowGestureCounters = false;
                if (defaultShowGestureCountersStr != null && defaultShowGestureCountersStr.trim().equalsIgnoreCase("T"))
                    defaultShowGestureCounters = true;

                readRefs(temporalPartitionSegmentEl, "VISIBLE", "FIELD")
                    .forEach(id -> temporalPartitionSegment.addPermanentFieldAsID(id));

                var panelIDs = readRefs(temporalPartitionSegmentEl, "VISIBLE", PANEL_TAG)
                    .collect(Collectors.toSet());
                temporalPartitionSegment.setVisiblePanelIDs(panelIDs);

                String temporalPartitionSegmentLabel = temporalPartitionSegmentEl
                        .getAttribute("LABEL");

                String note = temporalPartitionSegmentEl.getAttribute("NOTE");
                if(note != null && !note.isEmpty()) {
                    SS3GlossWindowItemNote noteItem = new SS3GlossWindowItemNote();
                    noteItem.setNote(note);
                    temporalPartitionSegment.setNote(noteItem);
                }

                boolean isDefault = isDef.equals("T") ? true : false;	// will not throw exception

                temporalPartitionSegment.setID(temporalPartitionSegmentID);
                temporalPartitionSegment.setParticipant(part);
                int indexOfSeparator = temporalPartitionSegmentLabel.indexOf(Util.SEGMENT_TIER_SEPARATOR);
                temporalPartitionSegment.setLabel(part != null ? temporalPartitionSegmentLabel : temporalPartitionSegmentLabel.substring(indexOfSeparator + 1));
                temporalPartitionSegment.setIndexFieldID(indexFieldID);
                currentGestureCounter = currentGestureCounter.isBlank() ? "1" : currentGestureCounter;
                temporalPartitionSegment.setCurrentGestureCounter(Integer.parseInt(currentGestureCounter));
                temporalPartitionSegment.setShowAllGestureCounters(showAllGestureCounters);
                temporalPartitionSegment.setDefaultShowGestureCounters(defaultShowGestureCounters);

                temporalPartition.addTemporalPartitionSegment(temporalPartitionSegment);

                // always try to set the 0th segment tier to the default, in case none of them are set to true
                if (isDefault || l == 0) {
                    temporalPartition.setDefaultSegment(temporalPartitionSegment);
                }

                // BEGIN copied
                NodeList statementFieldNode1 = temporalPartitionSegmentEl
                        .getElementsByTagName("STATEMENT-FIELD");

                for (int s = 0; s < statementFieldNode1.getLength(); s++) {
                    Element statementFieldEl = (Element) statementFieldNode1
                            .item(s);

                    SS3StatementField statementField = new SS3StatementField();
                    statementField.setId(read(statementFieldEl, "ID"));

                    String color = statementFieldEl.getAttribute("COLOR");
                    if(color != null && !color.isEmpty()) {
                        int rgbColor = Integer.parseInt(color);
                        statementField.setColor(new Color(rgbColor));
                    }

                    readRefOpt(statementFieldEl, TIMERANGE_TAG)
                        .ifPresent(id -> statementField.setTimeRange(db.getTimeRange(id)));

                    ArrayList<SS3Entity> statementFieldEntitiesList = new ArrayList<SS3Entity>();
                    var track = db.getTrack(statementField.getId());
                    readRefsOpt(statementFieldEl, "ENTITY").ifPresent(ids ->
                        ids.forEach(id -> statementFieldEntitiesList.add(track.getSS3Entity(id))));
                    statementField.setSS3Entities(statementFieldEntitiesList);

                    // END copied

                    temporalPartitionSegment.addField(statementField);
                }

                NodeList temporalPartitionSegmentGlossWindowItemNode = temporalPartitionSegmentEl
                        .getElementsByTagName("SEGMENT-TIER-UTTERANCE");

                for (int k = 0; k < temporalPartitionSegmentGlossWindowItemNode
                        .getLength(); k++) {
                    Element temporalPartitionSegmentGlossWindowItemEl = (Element) temporalPartitionSegmentGlossWindowItemNode
                            .item(k);

                    SS3GlossWindowItem temporalPartitionSegmentWindowItem = new SS3GlossWindowItem(
                            temporalPartition.getLabel(),
                            temporalPartitionSegment.getLabel());

                    temporalPartitionSegmentWindowItem
                            .setId(temporalPartitionSegmentGlossWindowItemEl
                                    .getAttribute("ID"));
                    temporalPartitionSegmentWindowItem
                            .setLabel(temporalPartitionSegmentGlossWindowItemEl
                                    .getAttribute("LABEL"));

                    readRefs(temporalPartitionSegmentGlossWindowItemEl, "VISIBLE", "FIELD")
                        .forEach(id -> temporalPartitionSegmentWindowItem.addPermanentlyVisibleFieldAsID(id));

                    var interval = readInterval(db, temporalPartitionSegmentGlossWindowItemEl);
                    SS3GlossWindowItem enteredItem = segmentItems.get(interval.t0);

                    if (enteredItem == null) {
                        temporalPartitionSegmentWindowItem.setStartTimeInfo(interval.t0);
                        temporalPartitionSegmentWindowItem.setEndTimeInfo(interval.t1);

                        segmentItems.put(interval.t0, temporalPartitionSegmentWindowItem);
                    } else {
                        temporalPartitionSegmentWindowItem
                            .setStartTimeInfo(enteredItem.getStartTimeInfo());
                        temporalPartitionSegmentWindowItem
                            .setEndTimeInfo(enteredItem.getEndTimeInfo());
                    }

                    temporalPartitionSegment
                            .addSS3GlossWindowItem(temporalPartitionSegmentWindowItem);

                    NodeList statementFieldNode = temporalPartitionSegmentGlossWindowItemEl
                            .getElementsByTagName("STATEMENT-FIELD");

                    for (int s = 0; s < statementFieldNode.getLength(); s++) {
                        Element statementFieldEl = (Element) statementFieldNode
                                .item(s);

                        SS3StatementField statementField = new SS3StatementField();
                        statementField.setId(statementFieldEl
                                .getAttribute("FIELD-ID"));

                        String color = statementFieldEl.getAttribute("COLOR");
                        if(color != null && !color.isEmpty()) {
                            int rgbColor = Integer.parseInt(color);
                            statementField.setColor(new Color(rgbColor));
                        }

                        readRefOpt(statementFieldEl, TIMERANGE_TAG)
                            .ifPresent(id -> statementField.setTimeRange(db.getTimeRange(id)));

                        ArrayList<SS3Entity> statementFieldEntitiesList = new ArrayList<SS3Entity>();
                        var track = db.getTrack(statementField.getId());
                        readRefsOpt(statementFieldEl, "ENTITY").ifPresent(ids ->
                            ids.forEach(id -> statementFieldEntitiesList.add(track.getSS3Entity(id))));
                        statementField.setSS3Entities(statementFieldEntitiesList);

                        // GD 2021-12-01: believe this is all old/outdated code
//                        //Fix for disappearing translation fields. Only happens when no translation field is found for an utterance.
//                        if (statementField.getId().equals(Util.FREE_TEXT_FIELD_ID) && statementField.getSS3Entities().size() == 0) {
//                            outer:for (SS3Entity e : db.getTrack(Util.FREE_TEXT_FIELD_ID).getEntities()) {
//
//                                //Check to make sure Entity doesn't already have a parent.
//                                for (SS3GlossWindowItem i : segmentItems.values()) {
//                                    if (i.getSS3StatementField(Util.FREE_TEXT_FIELD_ID) != null && i.getSS3StatementField(Util.FREE_TEXT_FIELD_ID).getSS3Entities().size() > 0 && i.getSS3StatementField(Util.FREE_TEXT_FIELD_ID).getSS3Entities().get(0).getID().equals(e.getID())) {
//                                        continue outer;
//                                    }
//                                }
//
//                                //1. Check if utterance ID matches and translation's parent entity ID.
//                                if (e.getParentEntityID().equals(temporalPartitionSegmentWindowItem.getId())) {
//                                    statementField.addSS3Entity(e);
//                                    e.getItems().get(0).setStartTimeInfo(temporalPartitionSegmentWindowItem.getStartTimeInfo());
//                                    e.getItems().get(0).setEndTimeInfo(temporalPartitionSegmentWindowItem.getEndTimeInfo());
//                                    break;
//                                //2. Check if End Movie time matches any translation's end movie time.
//                                } else if (e.getItems().get(0).getEndTimeInfo().doubleValue() == temporalPartitionSegmentWindowItem.getEndTimeInfo().doubleValue()) {
//                                    statementField.addSS3Entity(e);
//                                    e.getItems().get(0).setStartTimeInfo(temporalPartitionSegmentWindowItem.getStartTimeInfo());
//                                    e.getItems().get(0).setEndTimeInfo(temporalPartitionSegmentWindowItem.getEndTimeInfo());
//                                    break;
//                                    // JACK TODO: Can't compare times before videos are loaded.
//                                //3. Check if End Movie time of any translation falls within utterance's range (between start/end time).
////									} else if (e.getItems().get(0).getEndTimeInfo().doubleValue() + 2000 ==  temporalPartitionSegmentWindowItem.getEndTimeInfo().doubleValue()
////											&& e.getItems().get(0).getStartTimeInfo().doubleValue() + 2000 == temporalPartitionSegmentWindowItem.getStartTimeInfo().doubleValue()) {
////										statementField.addSS3Entity(e);
////										e.getItems().get(0).setStartTimeInfo(temporalPartitionSegmentWindowItem.getStartTimeInfo());
////										e.getItems().get(0).setEndTimeInfo(temporalPartitionSegmentWindowItem.getEndTimeInfo());
////										break;
//                                }
//                            }
//                        }

                        // to be removed
                        temporalPartitionSegmentWindowItem
                            .addField(statementField);
                    }
                }

                SS3GlossWindowSegment listOfSegmentItems = new SS3GlossWindowSegment();

                var items = segmentItems.values().stream()
                    .collect(Collectors.toMap(SS3GlossWindowItem::getLabel, i -> i, (a, b) -> a));
                for (var item : items.values()) {
                    SS3GlossWindowItem tpItem = new SS3GlossWindowItem(
                        temporalPartitionLabel,
                        temporalPartitionSegmentLabel);
                    tpItem.setLabel(item.getLabel());
                    tpItem.setStartTimeInfo(item.getStartTimeInfo());
                    tpItem.setEndTimeInfo(item.getEndTimeInfo());
                    listOfSegmentItems.addSS3GlossWindowItem(tpItem);
                }

                temporalPartition.setGlossWindowSegment(listOfSegmentItems);
            }
        }

        return glsWindow;
    }

    private ArrayList<SS3Note> parseNotes(NodeList nodes) {
        ArrayList<SS3Note> list = new ArrayList<SS3Note>();
        Element element = (Element) nodes.item(0);
        NodeList noteNode = element.getElementsByTagName("NOTE");

        for (int i = 0; i < noteNode.getLength(); i++) {
            SS3Note note = new SS3Note();
            Element line = (Element) noteNode.item(i);
            note.setID(line.getAttribute("ID"));
            note.setCoderID(readRef(line, "CODER"));
            note.setTimeCreated(line.getAttribute("TIME_CREATED"));
            note.setTimeUpdated(line.getAttribute("TIME_UPDATED"));

            NodeList sNode = line.getElementsByTagName("SUBJECT");
            for (int j = 0; j < sNode.getLength(); j++) {
                Element sEl = (Element) sNode.item(j);
                note.setSubject(getCharacterDataFromElement(sEl));
            }

            NodeList bNode = line.getElementsByTagName("BODY");
            for (int l = 0; l < bNode.getLength(); l++) {
                Element bEl = (Element) bNode.item(l);
                note.setBody(getCharacterDataFromElement(bEl));
            }

            list.add(note);
        }
        return list;
    }

    private HashMap<String, SS3Field> parseFields(NodeList nodes) {
        HashMap<String, SS3Field> fields = new HashMap<String, SS3Field>();
        Element element = (Element) nodes.item(0);
        NodeList fNode = element.getElementsByTagName("FIELD");

        for (int i = 0; i < fNode.getLength(); i++) {
            SS3Field fld = new SS3Field();
            Element fEl = (Element) fNode.item(i);
            fld.setId(fEl.getAttribute("ID"));
            fld.setColor(fEl.getAttribute("COLOR"));
            fld.setLabel(fEl.getAttribute("LABEL"));
            fld.setName(fEl.getAttribute("NAME"));
            fld.setPrefix(fEl.getAttribute("PREFIX"));
            fld.setType(fEl.getAttribute("TYPE"));
            fld.setGroup(fEl.getAttribute("GROUP"));
            fld.setOrder(fEl.getAttribute("ORDER"));

            NodeList valNode = fEl.getElementsByTagName("VALUE");
            for (int j = 0; j < valNode.getLength(); j++) {
                SS3FieldValue val = new SS3FieldValue();
                Element valEl = (Element) valNode.item(j);
                val.setId(valEl.getAttribute("ID"));
                val.setIid(valEl.getAttribute("IMG_ID"));
                val.setLabel(valEl.getAttribute("LABEL"));
                val.setName(valEl.getAttribute("NAME"));
                val.setType(valEl.getAttribute("TYPE"));
                fld.addValue(val);
            }
            fields.put(fld.getId(), fld);

        }
        return fields;
    }

    private HashMap<String, SS3Field> parseLocalFields(NodeList nodes) {
    	SS3CodingScheme defScheme = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();

        HashMap<String, SS3Field> fields = new HashMap<String, SS3Field>();
        Element element = (Element) nodes.item(0);
        
        var fieldsEl = readChildOpt(element, "FIELDS");
        if (fieldsEl.isPresent()) {
        	readChildren(fieldsEl.get(), "FIELD").forEach(fieldEl -> {
        		SS3Field field = null;
        		
        		String fieldID = read(fieldEl, "ID");
        		SS3Field defaultField = defScheme.getField(fieldID);
        		if (defaultField != null) {
        			field = defaultField.cloneField();
        		} else {
        			field = new SS3Field();
        			field.setId(fieldID);
        		}
        		
				String deletedField = readOpt(fieldEl, "DELETED").orElse(null);
				if (deletedField != null && !deletedField.isEmpty()) {
					field.setDeleted("T".equals(deletedField));
				}
				
				if (!field.isDeleted()) {
	        		Element fieldAttrsEl = readChildOpt(fieldEl, "FIELD-ATTRIBUTES").orElse(null);
	        		if (fieldAttrsEl != null) {
	        			for (Element fieldAttrEl: readChildren(fieldAttrsEl, "FIELD-ATTRIBUTE").collect(Collectors.toList())) {
	        				String name = readOpt(fieldAttrEl, "NAME").orElse(null);
	        				if (name != null && !name.isEmpty()) field.setName(name);
	        				String label = readOpt(fieldAttrEl, "LABEL").orElse(null);
	        				if (label != null && !label.isEmpty()) field.setLabel(label);
	        				String prefix = readOpt(fieldAttrEl, "PREFIX").orElse(null);
	        				if (prefix != null && !prefix.isEmpty()) field.setPrefix(prefix);
	        				String color = readOpt(fieldAttrEl, "COLOR").orElse(null);
	        				if (color != null && !color.isEmpty()) field.setColor(color);
	        				String type = readOpt(fieldAttrEl, "TYPE").orElse(null);
	        				if (type != null && !type.isEmpty()) field.setType(type);
	        				String alignment = readOpt(fieldAttrEl, "ALIGNMENT").orElse(null);
	        				if (alignment != null && !alignment.isEmpty()) {
	        					Alignment alignmentEnum = Alignment.CENTER;
	        					
	        					if (alignment.equals("RIGHT"))
	        						alignmentEnum = Alignment.RIGHT;
	        					else if (alignment.equals("LEFT"))
	        						alignmentEnum = Alignment.LEFT;
	        					else if (alignment.equals("RIGHT_LEFT"))
	        						alignmentEnum = Alignment.RIGHT_LEFT;
	        					else if (alignment.equals("CENTER"))
	        						alignmentEnum = Alignment.CENTER;
	        						
	        					field.setAlignment(alignmentEnum);
	        				}
	        				String group = readOpt(fieldAttrEl, "GROUP").orElse(null);
	        				if (group != null && !group.isEmpty()) {
	        					switch (group) {
	        					case Constants.HEAD_GROUP_NAME_CAPS:
	        						field.setGroup(Constants.HEAD_GROUP_NAME_CAPS);
	        						field.setGroupID(Constants.HEAD_GROUP_ID);
	        						break;
	        					case Constants.FACE_GROUP_NAME_CAPS:
	        						field.setGroup(Constants.FACE_GROUP_NAME_CAPS);
	        						field.setGroupID(Constants.FACE_GROUP_ID);
	        						break;
	        					case Constants.BODY_GROUP_NAME_CAPS:
	        						field.setGroup(Constants.BODY_GROUP_NAME_CAPS);
	        						field.setGroupID(Constants.BODY_GROUP_ID);
	        						break;
	        					case Constants.GRAMMATICAL_GROUP_NAME_CAPS:
	        						field.setGroup(Constants.GRAMMATICAL_GROUP_NAME_CAPS);
	        						field.setGroupID(Constants.GRAMMATICAL_GROUP_ID);
	        						break;
	        					case Constants.SPEECH_GROUP_NAME_CAPS:
	        						field.setGroup(Constants.SPEECH_GROUP_NAME_CAPS);
	        						field.setGroupID(Constants.SPEECH_GROUP_ID);
	        						break;
	        					case Constants.GENERIC_GROUP_NAME:
	        						field.setGroup(Constants.GENERIC_GROUP_NAME);
	        						field.setGroupID(Constants.GENERIC_GROUP_ID);
	        						break;
	        					case Constants.MANUAL_GROUP_NAME_CAPS:
	        						field.setGroup(Constants.MANUAL_GROUP_NAME_CAPS);
	        						field.setGroupID(Constants.MANUAL_GROUP_ID);
	        						break;
	        					case Constants.GESTURE_GROUP_NAME_CAPS:
	        						field.setGroup(Constants.GESTURE_GROUP_NAME_CAPS);
	        						field.setGroupID(Constants.GESTURE_GROUP_ID);
	        						break;
	        					case Constants.GESTURE_DOM_CATEGORY_GROUP_NAME_CAPS:
	        						field.setGroup(Constants.GESTURE_DOM_CATEGORY_GROUP_NAME_CAPS);
	        						field.setGroupID(Constants.GESTURE_DOM_CATEGORY_GROUP_ID);
	        						break;
	        					case Constants.GESTURE_NDOM_CATEGORY_GROUP_NAME_CAPS:
	        						field.setGroup(Constants.GESTURE_NDOM_CATEGORY_GROUP_NAME_CAPS);
	        						field.setGroupID(Constants.GESTURE_NDOM_CATEGORY_GROUP_ID);
	        						break;
	        					}
	        				}
	        				String order = readOpt(fieldAttrEl, "ORDER").orElse(null);
	        				if (order != null && !order.isEmpty()) field.setOrder(order);
	        			}
	        		}
	        		
	        		Element fieldValuesEl = readChildOpt(fieldEl, "FIELD-VALUES").orElse(null);
	        		if (fieldValuesEl != null) {
	        			for (Element fieldValueEl: readChildren(fieldValuesEl, "FIELD-VALUE").collect(Collectors.toList())) {
	                		SS3FieldValue fieldValue = null;
	                		
	                		String fieldValueID = read(fieldValueEl, "ID");
	                		SS3FieldValue defaultFieldValue = null;
	                		if (defaultField != null) {
	                			defaultFieldValue = defaultField.getFieldValue(fieldValueID);
	                		}
	                		
	                		if (defaultFieldValue != null) {
	                			fieldValue = defaultFieldValue.cloneFieldValue();
	                		} else {
	                			fieldValue = new SS3FieldValue();
	                			fieldValue.setId(fieldValueID);
	                		}
	                		
	        				String deletedValue = readOpt(fieldValueEl, "DELETED").orElse(null);
	        				if (deletedValue != null && !deletedValue.isEmpty()) {
	        					if ("T".equals(deletedValue)) {
	        						field.remove(fieldValue);
	        						continue;
	        					}
	        				}
	        				
	        				Element fieldValueAttrsEl = readChildOpt(fieldValueEl, "FIELD-VALUE-ATTRIBUTES").orElse(null);
	        				if (fieldValueAttrsEl != null) {
	        					for (Element fieldValueAttrEl: readChildren(fieldValueAttrsEl, "FIELD-VALUE-ATTRIBUTE").collect(Collectors.toList())) {
	        						String vName = readOpt(fieldValueAttrEl, "NAME").orElse(null);
	        						if (vName != null && !vName.isEmpty()) fieldValue.setName(vName);
	        						String vLabel = readOpt(fieldValueAttrEl, "LABEL").orElse(null);
	        						if (vLabel != null && !vLabel.isEmpty()) fieldValue.setLabel(vLabel);
	        						String vType = readOpt(fieldValueAttrEl, "TYPE").orElse(null);
	        						if (vType != null && !vType.isEmpty()) fieldValue.setType(vType);
	        						String vIID = readOpt(fieldValueAttrEl, "IID").orElse(null);
	        						if (vIID != null && !vIID.isEmpty()) fieldValue.setIid(vIID);
	        					}
	        				}
		        			
		        			field.addValue(fieldValue);
	        			}
	        		}
				}
				
				fields.put(fieldID, field);
        	});
        }
        
        return fields;
    }

    private ArrayList<Reference> parseCodingSchemes(NodeList nodes) {
        ArrayList<Reference> list = new ArrayList<Reference>();
        Element element = (Element) nodes.item(0);
        NodeList prtcpNode = element.getElementsByTagName("CODING-SCHEME");

        for (int i = 0; i < prtcpNode.getLength(); i++) {
            Reference ref = new Reference();
            Element line = (Element) prtcpNode.item(i);
            ref.setId(line.getAttribute("ID"));
            ref.setFileName(line.getAttribute("FILE_NAME"));
            ref.setSignature(line.getAttribute("SIGNATURE"));
            list.add(ref);
        }
        return list;
    }

    private HashMap<String, java.awt.Point> parseMFilesOnScreenLocation(
            NodeList nodes) {
        HashMap<String, java.awt.Point> locations = new HashMap<String, java.awt.Point>();

        SS3SignStreamApplication.logger.log(Level.INFO,
                "Starting parsing media on screen locations");

        Object obj = nodes.item(0);
        if (obj != null) {
            Element element = (Element) obj;
            NodeList mfNode = element.getElementsByTagName("MEDIA-FILE");

            for (int i = 0; i < mfNode.getLength(); i++) {
                Element line = (Element) mfNode.item(i);
                String fileName = line.getAttribute("FILE_NAME");
                String x = line.getAttribute("LOCATION_X");
                String y = line.getAttribute("LOCATION_Y");

                try {
                    int X = Integer.parseInt(x);
                    int Y = Integer.parseInt(y);
                    locations.put(fileName, new java.awt.Point(X, Y));
                } catch (NullPointerException ex) {
                    SS3SignStreamApplication.logger.log(Level.INFO,
                            "Parsing media on screen location: fileName = "
                                    + fileName + " could not parse properly");
                }

                SS3SignStreamApplication.logger.log(Level.INFO,
                        "Parsed media on screen location: fileName = "
                                + fileName + ", x = " + x + ", y = " + y);
            }
        }

        SS3SignStreamApplication.logger.log(Level.INFO,
                "Media on screen locations parsed");
        return locations;
    }

    // point nodes to "TRACKS"
    private ArrayList<SS3Track> parseTracks(NodeList nodes, SS3Database db) {
        ArrayList<SS3Track> tracks = new ArrayList<SS3Track>();
        Element element = (Element) nodes.item(0);
        NodeList tNode = element.getElementsByTagName("TRACK");

        // dom handshape
        SS3Track track1000 = null;

        // ndom handshape
        SS3Track track2000 = null;

        // dom hand
        SS3Track track10501 = null;

        // ndom hand
        SS3Track track10502 = null;

        // gloss1
        SS3Track track10000 = null;

        // gloss2
        SS3Track track10001 = null;

        boolean setGlossDependencies = false;

        for (int i = 0; i < tNode.getLength(); i++) {
            SS3Track track = new SS3Track();
            Element tEl = (Element) tNode.item(i);

            String fieldID = tEl.getAttribute("ID");

            track.setId(fieldID);

            NodeList ncNode = tEl.getElementsByTagName("NON_COMPLETE_EVENTS");
            for (int nc = 0; nc < ncNode.getLength(); nc++) {
                Element entEl = (Element) ncNode.item(nc);
                NodeList ncItmNode = entEl.getElementsByTagName("ITEM");

                for (int nci = 0; nci < ncItmNode.getLength(); nci++) {
                    SS3Item ncItem = new SS3Item();

                    Element ncItmEl = (Element) ncItmNode.item(nci);

                    var interval = readInterval(db, ncItmEl);
                    ncItem.setStartTimeInfo(interval.t0);
                    ncItem.setEndTimeInfo(interval.t1);

                    ncItem.setText(getCharacterDataFromElement(ncItmEl));
                    String sf = ncItmEl.getAttribute("SF");
                    String ef = ncItmEl.getAttribute("EF");
                    ncItem.setStartTimeFinal(Util.isTrue(sf));
                    ncItem.setEndTimeFinal(Util.isTrue(ef));
                    String startEntID = readRef(ncItmEl, "COMP_START_ENTITY");
                    String endEntID = readRef(ncItmEl, "COMP_END_ENTITY");
                    ncItem.setStartCompoundEntityID(startEntID);
                    ncItem.setEndCompoundEntityID(endEntID);
                    track.addNonCompleteEvent(ncItem);
                }
            }

            NodeList etNode = tEl.getElementsByTagName("ENTERED_TEXT");
            String fid = tEl.getAttribute("ID");
            for (int et = 0; et < etNode.getLength(); et++) {
                Element etEl = (Element) etNode.item(et);
                NodeList etItmNode = etEl.getElementsByTagName("ITEM");

                for (int eti = 0; eti < etItmNode.getLength(); eti++) {
                    Element etItmEl = (Element) etItmNode.item(eti);

                    var interval = readInterval(db, etItmEl);

                    String enteredTxt = getCharacterDataFromElement(etItmEl);
                    if (enteredTxt.trim().length() != 0)
                        track.addEnteredTextEvent(new SS3Item(enteredTxt, interval.t0));
                }
            }

            NodeList entNode = tEl.getElementsByTagName("ENTITY");
            for (int j = 0; j < entNode.getLength(); j++) {
                SS3Entity entity = new SS3Entity();
                Element entEl = (Element) entNode.item(j);
                entity.setID(entEl.getAttribute("ID"));
                entity.setRefEntityId(entEl.getAttribute("PARENT_ENTITY_ID"));
//                entity.setParentEntityID(readRef(entEl, "SEGMENT-TIER-UTTERANCE"));		// see below - GLOSS-WINDOW-ITEM-REFS

                String compStartEntityID = readRef(entEl, "COMP_START_ENTITY");
                if (compStartEntityID != null) {
                    entity.setStartCompoundEntityID(compStartEntityID);
                }

                String compEndEntityID = readRef(entEl, "COMP_END_ENTITY");
                if (compEndEntityID != null) {
                    entity.setEndCompoundEntityID(compEndEntityID);
                }

                entity.setReference(entEl.getAttribute("REF"));

                if (fieldID.equalsIgnoreCase(Util.DOM_HANDSHAPE_FIELD_ID)
                        || fieldID.equalsIgnoreCase(Util.NDOM_HANDSHAPE_FIELD_ID)
                        || fieldID.equalsIgnoreCase(Util.DOM_HAND_FIELD_ID)
                        || fieldID.equalsIgnoreCase(Util.NDOM_HAND_FIELD_ID)) {

                    String lockedGlossPhoneAlignment = entEl.getAttribute("LOCKED_HAND_ALIGNMENT");
                    boolean isLockedGlossPhoneAlignment = "T".equalsIgnoreCase(lockedGlossPhoneAlignment);
                    entity.setLockedGlossPhoneAlignments(isLockedGlossPhoneAlignment);

                    if(fieldID.equalsIgnoreCase(Util.DOM_HANDSHAPE_FIELD_ID)) {
                        track1000 = track;
                    }

                    if(fieldID.equalsIgnoreCase(Util.NDOM_HANDSHAPE_FIELD_ID)) {
                        track2000 = track;
                    }

                    if(fieldID.equalsIgnoreCase(Util.DOM_HAND_FIELD_ID)) {
                        track10501 = track;
                    }

                    if(fieldID.equalsIgnoreCase(Util.NDOM_HAND_FIELD_ID)) {
                        track10502 = track;
                    }
                } else if (fieldID.equalsIgnoreCase(Util.DOM_GESTURE_HANDSHAPE_FIELD_ID)
                        || fieldID.equalsIgnoreCase(Util.NDOM_GESTURE_HANDSHAPE_FIELD_ID)
                        || fieldID.equalsIgnoreCase(Util.DOM_GESTURE_HAND_FIELD_ID)
                        || fieldID.equalsIgnoreCase(Util.NDOM_GESTURE_HAND_FIELD_ID)) {

                    String lockedGesturePhoneAlignment = entEl.getAttribute("LOCKED_HAND_ALIGNMENT");
                    boolean isLockedGesturePhoneAlignment = "T".equalsIgnoreCase(lockedGesturePhoneAlignment);
                    entity.setLockedGesturePhoneAlignments(isLockedGesturePhoneAlignment);
                }

                String depGlsID = null, depApexID, depHandID, depHandShapeID, depOrientationID,
                        depLocationID, depSignTypeID, depGestureTypeID;

                if ((fieldID.equalsIgnoreCase(Util.DOM_GLOSS_FIELD_ID) || fieldID.equalsIgnoreCase(Util.NDOM_GLOSS_FIELD_ID))) {

                    if(fieldID.equalsIgnoreCase(Util.DOM_GLOSS_FIELD_ID)) {
                        track10000 = track;
                    }

                    if(fieldID.equalsIgnoreCase(Util.NDOM_GLOSS_FIELD_ID)) {
                        track10001 = track;
                    }

                    depGlsID = readRef(entEl, "DEPENDENT_GLOSS_ENTITY");
                    depHandID = readRef(entEl, "DEPENDENT_HAND_ENTITY");
                    depHandShapeID = readRef(entEl, "DEPENDENT_HAND_SHAPE_ENTITY");
                    depLocationID = readRef(entEl, "DEPENDENT_LOCATION_ENTITY");
                    depSignTypeID = readRef(entEl, "DEPENDENT_SIGN_TYPE_ENTITY");

                    if(depGlsID == null || depGlsID.length() == 0) {
                        depGlsID = readRef(entEl, "CORRESPONDING_GLOSS_ENTITY");
                    }

                    entity.setDependentGlossEntityID(depGlsID);
                    entity.setDependentHandEntityID(depHandID);
                    entity.setDependentHandShapeEntityID(depHandShapeID);
                    entity.setDependentLocationEntityID(depLocationID);
                    entity.setDependentSignTypeEntityID(depSignTypeID);

                    String primaryEntryStr = entEl.getAttribute("PRIMARY_ENTRY");
                    entity.setPrimaryEntry(primaryEntryStr);

                    GlossEntityProperties properties = entity
                            .getGlossEntityProperties();

                    if (properties == null) {
                        properties = new GlossEntityProperties();
                        entity.setGlossEntityProperties(properties);
                    }

                    String dominance = entEl.getAttribute("DOMINANCE");
                    String markedNumberOfHands = entEl
                            .getAttribute("MARKED_NUMBER_OF_HANDS");

                    properties
                            .setMarkedNumberOfHandsString(markedNumberOfHands);


                    /**
                     * <ENTITY> <PROPERTIES SIGN=
                     * "FINGERSPELLED/LOAN/LEXICAL/CLASSIFIER/GESTURE/NUMBER"
                     * NAME_SIGN="T/F" PASSIVE_BASE_ARM="T/F" NO_OF_HANDS="1/2"
                     *
                     * TWOHANDED_HADSHAPES="SAME/ALTERNATING_SAME/DIFFERENT"
                     * START_END_HANDSHAPES="SAME/DIFFERENT"
                     * NDOM_HAND_START_END_HANDSHAPE="SAME/DIFFERENT"
                     * DOM_HAND_START_END_HANDSHAPE="SAME/DIFFERENT" />
                     * <DOMINANCE="L/R"/> <MARKED_NUMBER_OF_HANDS="T/F"/>
                     * </ENTITY>
                     */
                    NodeList propNode = entEl
                            .getElementsByTagName("PROPERTIES");
                    for (int jj = 0; jj < propNode.getLength(); jj++) {
                        // SS3Entity entity = new SS3Entity();
                        Element propEl = (Element) propNode.item(jj);
                        String sign = propEl.getAttribute("SIGN");
                        String nameSign = propEl.getAttribute("NAME_SIGN");
                        String passiveBaseArm = propEl
                                .getAttribute("PASSIVE_BASE_ARM");
                        String numberOfHands = propEl
                                .getAttribute("NO_OF_HANDS");

                        String twoHanded = propEl
                                .getAttribute("TWOHANDED_HADSHAPES");

                        String startEndHandshape = propEl
                                .getAttribute("START_END_HANDSHAPES");
                        String ndomHandStartEndHandshape = propEl
                                .getAttribute("NDOM_HAND_START_END_HANDSHAPE");
                        String domHandStartEndHandshape = propEl
                                .getAttribute("DOM_HAND_START_END_HANDSHAPE");

                        properties.setSignTypeAsString(sign);
                        properties.setNameSign(nameSign.trim()
                                .equalsIgnoreCase("T"));
                        properties.setPassiveBaseArm(passiveBaseArm.trim()
                                .equalsIgnoreCase("T"));

                        properties.setTwoHanded(!entity.getDependentGlossEntityID().trim().isEmpty() ||
                                numberOfHands.trim().equalsIgnoreCase("2"));

                        properties
                                .setTwoHandedHandshapesClassification(twoHanded);
                        properties.setStartEndHandshape(startEndHandshape);

                        properties
                                .setStartEndHandshapeNdomHand(ndomHandStartEndHandshape);
                        properties
                                .setStartEndHandshapeDomHand(domHandStartEndHandshape);

                        // if(twoHanded.trim().equalsIgnoreCase("SAME")) {
                        // properties.setTwoHandedHandshapesClassification(GlossEntityProperties.SAME_HANDSHAPES_DOM_NDOM);
                        // } else
                        // if(twoHanded.trim().equalsIgnoreCase("ALTERNATING_SAME"))
                        // {
                        // properties.setTwoHandedHandshapesClassification(GlossEntityProperties.ALTERNATING_SAME_HANDSHAPES_DOM_NDOM);
                        // } else
                        // if(twoHanded.trim().equalsIgnoreCase("DIFFERENT")) {
                        // properties.setTwoHandedHandshapesClassification(GlossEntityProperties.DIFFERENT_HANDSHAPES_DOM_NDOM);
                        // } else {
                        // properties.setTwoHandedHandshapesClassification("");
                        // }
                        /*
                         * <ENTITY CODER-ID="" COMP_END_ENTITY_ID=""
                         * COMP_START_ENTITY_ID="" DOMINANCE="R"
                         * MARKED_NUMBER_OF_HANDS="T" > <PROPERTIES
                         * NAME_SIGN="T" NO_OF_HANDS="2" PASSIVE_BASE_ARM="T"
                         * SIGN="" START_END_HANDSHAPES=""
                         * TWOHANDED_HADSHAPES="DIFFERENT HANDSHAPES LR"
                         * NDOM_HAND_START_END_HANDSHAPE
                         * ="DIFFERENT START/END HANDSHAPE"
                         * DOM_HAND_START_END_HANDSHAPE
                         * ="SAME START/END HANDSHAPE" />
                         */
                        // if(startEndHandshape.trim().equalsIgnoreCase("SAME"))
                        // {
                        // properties.setStartEndHandshape(GlossEntityProperties.SAME_START_END_HANDSHAPE);
                        // } else
                        // if(startEndHandshape.trim().equalsIgnoreCase("DIFFERENT"))
                        // {
                        // properties.setStartEndHandshape(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
                        // } else {
                        // properties.setStartEndHandshape("");
                        // }
                        //
                        // if(ndomHandStartEndHandshape.trim().equalsIgnoreCase("SAME"))
                        // {
                        // properties.setStartEndHandshapeNdomHand(GlossEntityProperties.SAME_START_END_HANDSHAPE);
                        // } else
                        // if(ndomHandStartEndHandshape.trim().equalsIgnoreCase("DIFFERENT"))
                        // {
                        // properties.setStartEndHandshapeNdomHand(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
                        // } else {
                        // properties.setStartEndHandshapeNdomHand("");
                        // }
                        //
                        // if(domHandStartEndHandshape.trim().equalsIgnoreCase("SAME"))
                        // {
                        // properties.setStartEndHandshapeDomHand(GlossEntityProperties.SAME_START_END_HANDSHAPE);
                        // } else
                        // if(domHandStartEndHandshape.trim().equalsIgnoreCase("DIFFERENT"))
                        // {
                        // properties.setStartEndHandshapeDomHand(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
                        // } else {
                        // properties.setStartEndHandshapeNdomHand("");
                        // }
                        //
                        // System.out.println("Entity ID = " + entity.getID() +
                        // ", properties = " + properties);
                        //
                        entity.setGlossEntityProperties(properties);
                    }
                }	// glosses

                if ((fieldID.equalsIgnoreCase(Util.DOM_GESTURE_FIELD_ID) || fieldID.equalsIgnoreCase(Util.NDOM_GESTURE_FIELD_ID))) {

                    depGlsID = readRef(entEl, "DEPENDENT_GESTURE_ENTITY");
                    depApexID = readRef(entEl, "DEPENDENT_APEX_ENTITY");
                    depHandID = readRef(entEl, "DEPENDENT_HAND_ENTITY");
                    depHandShapeID = readRef(entEl, "DEPENDENT_HAND_SHAPE_ENTITY");
                    depOrientationID = readRef(entEl, "DEPENDENT_ORIENTATION_ENTITY");
                    depLocationID = readRef(entEl, "DEPENDENT_LOCATION_ENTITY");
                    depGestureTypeID = readRef(entEl, "DEPENDENT_GESTURE_TYPE_ENTITY");
                    String incompleteGesture = entEl.getAttribute("INCOMPLETE");
                    if (incompleteGesture != null && incompleteGesture.trim().equalsIgnoreCase("T"))
                        entity.setIncompleteGesture(true);
                    else
                        entity.setIncompleteGesture(false);

                    String showUniqueNumber = entEl.getAttribute("SHOW_UNIQUE_NUMBER");
                    if (showUniqueNumber != null && showUniqueNumber.trim().equalsIgnoreCase("T"))
                        entity.setShowUniqueNumber(true);
                    else
                        entity.setShowUniqueNumber(false);

                    String uniqueNumber = entEl.getAttribute("UNIQUE_NUMBER");
                    entity.setUniqueNumber(uniqueNumber);

                    if(depGlsID == null || depGlsID.length() == 0) {
                        depGlsID = readRef(entEl, "CORRESPONDING_GESTURE_ENTITY");
                    }

                    entity.setDependentGestureEntityID(depGlsID);
                    entity.setDependentApexEntityID(depApexID);
                    entity.setDependentHandEntityID(depHandID);
                    entity.setDependentHandShapeEntityID(depHandShapeID);
                    entity.setDepenentOrientationEntityID(depOrientationID);
                    entity.setDependentLocationEntityID(depLocationID);
                    entity.setDependentSignTypeEntityID(depGestureTypeID);

                    GestureEntityProperties properties = entity
                            .getGestureEntityProperties();

                    if (properties == null) {
                        properties = new GestureEntityProperties();
                        entity.setGestureEntityProperties(properties);
                    }

                    String dominance = entEl.getAttribute("DOMINANCE");
                    String markedNumberOfHands = entEl
                            .getAttribute("MARKED_NUMBER_OF_HANDS");

                    properties.setDominamtHand(dominance);
                    properties
                            .setMarkedNumberOfHandsString(markedNumberOfHands);

                    /**
                     * <ENTITY> <PROPERTIES SIGN=
                     * "FINGERSPELLED/LOAN/LEXICAL/CLASSIFIER/GESTURE/NUMBER"
                     * NAME_SIGN="T/F" PASSIVE_BASE_ARM="T/F" NO_OF_HANDS="1/2"
                     *
                     * TWOHANDED_HADSHAPES="SAME/ALTERNATING_SAME/DIFFERENT"
                     * START_END_HANDSHAPES="SAME/DIFFERENT"
                     * NDOM_HAND_START_END_HANDSHAPE="SAME/DIFFERENT"
                     * DOM_HAND_START_END_HANDSHAPE="SAME/DIFFERENT" />
                     * <DOMINANCE="L/R"/> <MARKED_NUMBER_OF_HANDS="T/F"/>
                     * </ENTITY>
                     */
                    NodeList propNode = entEl
                            .getElementsByTagName("PROPERTIES");
                    for (int jj = 0; jj < propNode.getLength(); jj++) {
                        // SS3Entity entity = new SS3Entity();
                        Element propEl = (Element) propNode.item(jj);
                        String sign = propEl.getAttribute("SIGN");
                        String nameSign = propEl.getAttribute("NAME_SIGN");
                        String passiveBaseArm = propEl
                                .getAttribute("PASSIVE_BASE_ARM");
                        String incomplete = propEl
                                .getAttribute("INCOMPLETE");
                        String numberOfHands = propEl
                                .getAttribute("NO_OF_HANDS");

                        String twoHanded = propEl
                                .getAttribute("TWOHANDED_HADSHAPES");

                        String startEndHandshape = propEl
                                .getAttribute("START_END_HANDSHAPES");
                        String ndomHandStartEndHandshape = propEl
                                .getAttribute("NDOM_HAND_START_END_HANDSHAPE");
                        String domHandStartEndHandshape = propEl
                                .getAttribute("DOM_HAND_START_END_HANDSHAPE");

                        properties.setSignTypeAsString(sign);
                        properties.setNameSign(nameSign.trim()
                                .equalsIgnoreCase("T"));
                        properties.setPassiveBaseArm(passiveBaseArm.trim()
                                .equalsIgnoreCase("T"));

                        properties.setTwoHanded(!entity.getDependentGestureEntityID().trim().isEmpty() ||
                                numberOfHands.trim().equalsIgnoreCase("2"));

                        properties
                                .setTwoHandedHandshapesClassification(twoHanded);
                        properties.setStartEndHandshape(startEndHandshape);

                        properties
                                .setStartEndHandshapeNdomHand(ndomHandStartEndHandshape);
                        properties
                                .setStartEndHandshapeDomHand(domHandStartEndHandshape);

                        entity.setGestureEntityProperties(properties);
                    }
                }	// gestures
//
//					if ((fieldID.equalsIgnoreCase(Util.DOM_GESTURE_TYPE_FIELD_ID) 
//							|| fieldID.equalsIgnoreCase(Util.NDOM_GESTURE_TYPE_FIELD_ID))) {
//						
//					}	// gesture types

                Element glossWindowItemRefsEl = readChild(entEl, "GLOSS-WINDOW-ITEM-REFS");
                ArrayList<String> parentEntityIDs = new ArrayList<String>();
                for(Element glossWindowItemRefEl : readChildren(glossWindowItemRefsEl, "REF").collect(Collectors.toList())) {
                    if (glossWindowItemRefEl != null) {
                        String parentEntityID = read(glossWindowItemRefEl, "GLOSS-WINDOW-ITEM_ID");
                        parentEntityIDs.add(parentEntityID);
                    }
                }
                entity.setParentEntityIDs(parentEntityIDs);

                ArrayList<Element> nodeList = 
                		readChildrenOrdered(entEl, "ITEM").collect(Collectors.toCollection(ArrayList::new));

                for (int l = 0; l < nodeList.size(); l++) {
                    SS3Item item = new SS3Item();
                    Element itmEl = (Element) nodeList.get(l);
                    item.setId(itmEl.getAttribute("ID"));
					item.setImageID(readRef(itmEl, "IMG"));

                    String textOnly = itmEl.getAttribute("TEXT_ONLY");
                    if (textOnly != null
                            & textOnly.trim().equalsIgnoreCase("T")) {
                        item.setTextOnly(true);
                    }

					item.setLocationRegionID(readRef(itmEl, "LOCATION-REGION"));
					String locationTypeIDStr = readRef(itmEl, "LOCATION-TYPE");
                    if (locationTypeIDStr != null && !locationTypeIDStr.isEmpty()) {
                        int locationTypeID = Integer.parseInt(locationTypeIDStr);
                        if (SS3Singleton.getLocationConfig().getLocationType(locationTypeID) == null) {
                            continue;
                        }
                        item.setLocationTypeID(locationTypeID);
                    }


					item.setNoteID(readRef(itmEl, "NOTE"));

                    var interval = readInterval(db, itmEl);
                    item.setStartTimeInfo(interval.t0);
                    item.setEndTimeInfo(interval.t1);

                    String txt = getCharacterDataFromElement(itmEl);
                    item.setText(txt);
                    entity.addItem(item);

                    /* TODO
                     * This will need changing after moving to variable timeunit scheme
                     */
//						if (fieldID.equals(Util.FREE_TEXT_FIELD_ID)) {
//							String validTime = entEl.getAttribute("VALID_TIME");
//							boolean isValidTime = validTime != null && validTime.equals("T");
//							if (!isValidTime) {	// if time is wrong in db, then set it to the correct time
//								double packetDuration = db.getPacketDuration();
//								double buffer = packetDuration*2;
//
//								double newStartMoveTime = (start.doubleValue() - buffer);
//								double newEndMovieTime = (end.doubleValue() - buffer);
//
//								start = SS3Singleton.getVideoTimeRange().time(Math.round(newStartMoveTime), TimeUnit.TIMESTAMP);
//                                end = SS3Singleton.getVideoTimeRange().time(Math.round(newEndMovieTime), TimeUnit.TIMESTAMP);
//							}
//							entity.setValidTime(true);	// if we went through the block, or not, set this to valid time
//						}
                }

                String lockedDomNdomAlignment = entEl.getAttribute("LOCKED_DOM_NDOM_ALIGNMENT");
                String lockedGlossPhoneAlignment = entEl.getAttribute("LOCKED_HAND_ALIGNMENT");
                String lockedGesturePhoneAlignment = entEl.getAttribute("LOCKED_HAND_ALIGNMENT");

                if(lockedGlossPhoneAlignment.trim().length() == 0) {
                    setGlossDependencies = true;
                }

                boolean isLockedDomNdomAlignment = "T".equalsIgnoreCase(lockedDomNdomAlignment);
                boolean isLockedGlossPhoneAlignment = "T".equalsIgnoreCase(lockedGlossPhoneAlignment);
                boolean isLockedGesturePhoneAlignment = "T".equalsIgnoreCase(lockedGesturePhoneAlignment);

                // need to move this logic down here - for locking information
                // check if lockedDomNdom is correct when compared to the dependent gloss (it has been known to be wrong in the db)
                if (depGlsID != null) {
                    SS3Track depTrack = null;
                    if (track.equals(track10000)) {
                        depTrack = track10001;
                    } else if (track.equals(track10001)) {
                        depTrack = track10000;
                    }

                    if (depTrack != null) {
                        SS3Entity depGls = depTrack.getSS3Entity(depGlsID);
                        if (depGls != null) {
//								double entityStartTime = entity.getStartTime();
//								double entityEndTime = entity.getEndTime();
//								double depGlsStartTime = depGls.getStartTime();
//								double depGlsEndTime = depGls.getEndTime();
//
//								if (entityStartTime != depGlsStartTime || entityEndTime != depGlsEndTime) {
//									isLockedDomNdomAlignment = false;
//
//									depGls.setLockedDomNdomAlignments(isLockedDomNdomAlignment);
//								}
                        }
                    }
                }

                entity.setLockedGlossPhoneAlignments(isLockedGlossPhoneAlignment);
                entity.setLockedDomNdomAlignments(isLockedDomNdomAlignment);
                entity.setLockedGesturePhoneAlignments(isLockedGesturePhoneAlignment);

                // Check to make sure entity start time is <= end time.
                // if not then don't add to track.
                if (!entity.getItems().isEmpty()) {
//						double entityStartTime = entity.getItems().get(0).getStartTimeInfo().doubleValue();
//						double entityEndTime = entity.getItems().get(entity.getItems().size() - 1).getEndTimeInfo().doubleValue();
//						if (entityStartTime <= entityEndTime) {
                        track.addEntity(entity);
//						}
                }
            }
            tracks.add(track);
        }

        if(setGlossDependencies) {
//				SS3Track track1000 = null;
//				SS3Track track2000 = null;
//				SS3Track track10501 = null;
//				SS3Track track10502 = null;
//				// gloss1
//				SS3Track track10000 = null;
//				// gloss2
//				SS3Track track10001 = null

            if(track1000 != null) {
                ArrayList<SS3Entity> entities = track1000.getEntities();
                for(SS3Entity entity : entities) {
                    //entity.getStartTime();
                    entity.getItems();
                }
            }
        }

        // fix signs saved previously into annotation file
        syncTracksToSignBank(track10000, track10001, track1000, track2000);
        syncTracksToSignBank(track10001, track10000, track1000, track2000);

        return tracks;
    }

    private String getCharacterDataFromElement(Element e) {
        Node child = e.getFirstChild();
        if (child instanceof CharacterData) {
            CharacterData cd = (CharacterData) child;
            return cd.getData();
        }
        return "";
    }

    /**
     * <pre>
     * Helper method: for all the entities within entitySourceHandTrack (just parsed)
     * determine whether their sign bank properties (if they exist for a given entity)
     * are up to date with the current sign bank collection (also should be parsed at this point).
     *
     * If an entity was added/edited via the SignBank interface, it was given a SIGNBANKDATA element in the annotation XML file.
     * We determine here whether, since the last time this annotation file was opened, the entity now differs
     * from the SignBank variant (read: occurrence) that it was sourced from. This would mean differing in terms of
     * handshapes and any other data that is coded via the Morph-Phon window.
     *
     * This method is meant to be called twice, once with gloss1 as entitySource and once with gloss2 as entitySource.
     * NOTE: Thus there will be inevitably some duplication of work since the gloss data being checked
     * is common between the gloss1 and gloss2 fields. This is unavoidable because there is a GlossEntityProperties object
     * for each GlossChainedEventsEntity object (thus, a two handed sign is composed of two GCEE objects, each with
     * their own GEP object). Because it remains possible to add a one-handed sign exclusively in the nondominant (gloss2) field,
     * we cannot only loop through gloss1 and assume to hit all the signs in the annotation file.
     * </pre>
     *
     * @param entitySourceHandTrack The SS3Track to loop through and sync each entity found therein
     * @param oppositeHandTrack The SS3Track on the opposite hand to entitySourceHandTrack (e.g. if entitySourceHandTrack
     * is gloss1, then oppositeHandTrack must be gloss2 -- this will be checked first, and if incorrect input will return false).
     * @param domHSTrack The SS3Track for the dominant handshape field (dom handshape field)
     * @param nondomHSTrack The SS3Track for the nondominant handshape field (ndom handshape field)
     * @return false if for any reason any of the entities encounters an error; true otherwise.
     */
    private boolean syncTracksToSignBank(SS3Track entitySourceHandTrack, SS3Track oppositeHandTrack,
            SS3Track domHSTrack, SS3Track nondomHSTrack) {
        if (entitySourceHandTrack == null || oppositeHandTrack == null
                || domHSTrack == null || nondomHSTrack == null)
            return false;

        // first sanity check the field id's on the tracks are all as expected given the inputs
        if ( !domHSTrack.getId().equals(Util.DOM_HANDSHAPE_FIELD_ID))
            return false;

        if ( !nondomHSTrack.getId().equals(Util.NDOM_HANDSHAPE_FIELD_ID))
            return false;

        if (entitySourceHandTrack.getId().equals(Util.DOM_GLOSS_FIELD_ID)
                && !oppositeHandTrack.getId().equals(Util.NDOM_GLOSS_FIELD_ID))
            return false;

        if (entitySourceHandTrack.getId().equals(Util.NDOM_GLOSS_FIELD_ID)
                && !oppositeHandTrack.getId().equals(Util.DOM_GLOSS_FIELD_ID))
            return false;

        // then proceed to loop through the entitySourceHandTrack and sync all entities
        for (SS3Entity entity: entitySourceHandTrack.getEntities()) {
            if (entity.getGlossEntityProperties().isVerifiedFromSignBank()) {
                for (SS3Item item: entity.getItems()) {
                    String itemText = item.getText();
                    if (itemText != null && !itemText.isEmpty()) {
                        if (!itemText.equals("Final Hold") && !itemText.equals("Initial Hold")
                                && !itemText.equals("Onset") && !itemText.equals("Offset")) {	// ignore these events
                            if (!syncEntityToSignBank(entity, itemText,
                                    entitySourceHandTrack, oppositeHandTrack, domHSTrack, nondomHSTrack)) {
                                continue;
                            }

                            break;	// should only need to sync for one SS3Item per SS3Entity
                        }
                    }
                }
            }
        }

        return true;
    }

    /**
     * <pre>
     * Helper method for syncTracksToSignBank()
     * Take in a single SS3Entity with its label, and compare the sign data found in its GlossEntityProperies
     * against the sign data from the SignBank variant (read: occurrence) which it came from.
     * If any discrepancies are found, then the entity is marked as differing.
     * </pre>
     *
     * @param entity The SS3Entity to compare
     * @param entityLabel The label of the SS3Entity
     * @param entitySourceHandTrack The track (either gloss1 or gloss2) which this SS3Entity came from
     * @param oppositeHandTrack  The opposite track (gloss1 or gloss2) of what this SS3Entity came from
     * @param domHSTrack The SS3Track for the dominant handshape field (dom handshape field)
     * @param nondomHSTrack The SS3Track for the nondominant handshape field (ndom handshape field)
     * @return false if for any reason the sync encounters an error; true otherwise.
     */
    private boolean syncEntityToSignBank(SS3Entity entity, String entityLabel,
            SS3Track entitySourceHandTrack, SS3Track oppositeHandTrack,
            SS3Track domHSTrack, SS3Track nondomHSTrack) {
        SignBankCollection signBankCollection = SS3Singleton.getSignStreamApplication().getSignBankCollection();
        SS3CodingScheme cs = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();

        GlossEntityProperties properties = entity.getGlossEntityProperties();

        String mainGlossID = properties.getMainGlossID();
        String variantID = properties.getVariantID();
        String occurrenceID = properties.getOccurrenceID();

        SignBankOccurrence entityOccurrence = signBankCollection.getOccurrenceWithID(mainGlossID, variantID, occurrenceID);

        if (entityOccurrence == null)
            return false;

        // NEXT get occurrence data
        String occSourceResource = entityOccurrence.getSIgnBankResourceFileName();

        String occDomStartHS = entityOccurrence.getDomStartHS();
        String occNondomStartHS = entityOccurrence.getNondomStartHS();
        String occDomEndHS = entityOccurrence.getDomEndHS();
        String occNondomEndHS = entityOccurrence.getNondomEndHS();

        String occLeftRightHS = entityOccurrence.getLeftRightHS();
        String occStartEndHS = entityOccurrence.getStartEndHS();
        String occStartEndLeftHS = entityOccurrence.getStartEndLeftHS();
        String occStartEndRightHS = entityOccurrence.getStartEndRightHS();

        boolean occTwoHanded = entityOccurrence.isTwoHanded();
        boolean occMarkedHands = entityOccurrence.isMarkedHands();
        boolean occPassiveBaseArm = entityOccurrence.isPassiveBaseArm();

        String occSignType = entityOccurrence.getSignType();

        // NEXT get entity data
        String leftRightHS = properties.getTwoHandedHandshapesClassification();
        String startEndHS = properties.getStartEndHandshape();
        String startEndLeftHS = properties.getStartEndHandshapeNdomHand();
        String startEndRightHS = properties.getStartEndHandshapeDomHand();

        boolean twoHanded = properties.isTwoHanded();
        boolean markedHands = properties.isMarkedNumberOfHands();
        boolean passiveBaseArm = properties.isPassiveBaseArm();

        String signType = null;
        if (properties.isLexical())
            signType = "LEXICAL";
        else if (properties.isFingerspelled())
            signType = "FINGERSPELLED";
        else if (properties.isLoan())
            signType = "LOAN";
        else if (properties.isNumber())
            signType = "NUMBER";
        else if (properties.isClassifier())
            signType = "CLASSIFER";
        else if (properties.isGesture())
            signType = "GESTURE";

        // GLOSS1 is DOM
        // GLOSS2 is NONDOM
        SS3Track domGlossTrack = null, nondomGlossTrack = null;
        String domGlossEntityID = null, nondomGlossEntityID = null, domHSEntityID = null, nondomHSEntityID = null;
        String domStartHS = null, nondomStartHS = null, domEndHS = null, nondomEndHS = null;

        if (entitySourceHandTrack.getId().equals(Util.DOM_GLOSS_FIELD_ID)) {	// if entity came from gloss1/dom field
            domGlossTrack = entitySourceHandTrack;
            nondomGlossTrack = oppositeHandTrack;

            domGlossEntityID = entity.getID();
            domHSEntityID = entity.getDependentHandShapeEntityID();

            nondomGlossEntityID = entity.getDependentGlossEntityID();
        } else if (entitySourceHandTrack.getId().equals(Util.NDOM_GLOSS_FIELD_ID)) {	// if entity came from gloss2/nondom field
             nondomGlossTrack = entitySourceHandTrack;
             domGlossTrack = oppositeHandTrack;

            nondomGlossEntityID = entity.getID();
            nondomHSEntityID = entity.getDependentHandShapeEntityID();

            domGlossEntityID = entity.getDependentGlossEntityID();
        }

        // get dom labels	from the coding scheme
        if (domGlossEntityID != null && !domGlossEntityID.isEmpty()) {
            if (domHSEntityID == null || domHSEntityID.isEmpty()) {
                for (SS3Entity dGlossEntity: domGlossTrack.getEntities()) {
                    if (dGlossEntity.getID().equals(domGlossEntityID)) {
                        domHSEntityID = dGlossEntity.getDependentHandShapeEntityID();
                        break;
                    }
                }
            }

            if (domHSEntityID != null && !domHSEntityID.isEmpty()) {	// check again in case hs entity was not found in track
                for (SS3Entity dHSEntity: domHSTrack.getEntities()) {
                    if (dHSEntity.getID().equals(domHSEntityID)) {
                        ArrayList<SS3Item> domHSItems = dHSEntity.getItems();
                        if (!domHSItems.isEmpty()) {
                            int domHSItemsSize = domHSItems.size();
                            SS3Item domStartHSItem = domHSItems.get(0);
                            SS3Item domEndHSItem = domHSItems.get(domHSItemsSize - 1);

                            String domStartHSItemID = domStartHSItem.getId();
                            String domEndHSItemID = domEndHSItem.getId();

                            SS3FieldValue domStartHSValue =
                                    cs.getFieldsMapSS3FieldValue(Util.DOM_HANDSHAPE_FIELD_ID, domStartHSItemID);
                            domStartHS = domStartHSValue.getLabel();

                            SS3FieldValue domEndHSValue =
                                    cs.getFieldsMapSS3FieldValue(Util.DOM_HANDSHAPE_FIELD_ID, domEndHSItemID);
                            domEndHS = domEndHSValue.getLabel();

                            break;
                        }
                    }
                }
            }
        }

        // get nondom labels from the coding scheme
        if (nondomGlossEntityID != null && !nondomGlossEntityID.isEmpty()) {
            if (nondomHSEntityID == null || nondomHSEntityID.isEmpty()) {
                for (SS3Entity ndGlossEntity: nondomGlossTrack.getEntities()) {
                    if (ndGlossEntity.getID().equals(nondomGlossEntityID)) {
                        nondomHSEntityID = ndGlossEntity.getDependentHandShapeEntityID();
                        break;
                    }
                }
            }

            if (nondomHSEntityID != null && !nondomHSEntityID.isEmpty()) {	// check again in case hs entity was not found in track
                for (SS3Entity ndHSEntity: nondomHSTrack.getEntities()) {
                    if (ndHSEntity.getID().equals(nondomHSEntityID)) {
                        ArrayList<SS3Item> nondomHSItems = ndHSEntity.getItems();
                        if (!nondomHSItems.isEmpty()) {
                            int nondomHSItemsSize = nondomHSItems.size();
                            SS3Item nondomStartHSItem = nondomHSItems.get(0);
                            SS3Item nondomEndHSItem = nondomHSItems.get(nondomHSItemsSize - 1);

                            String nondomStartHSItemID = nondomStartHSItem.getId();
                            String nondomEndHSItemID = nondomEndHSItem.getId();

                            SS3FieldValue nondomStartHSValue =
                                    cs.getFieldsMapSS3FieldValue(Util.NDOM_HANDSHAPE_FIELD_ID, nondomStartHSItemID);
                            nondomStartHS = nondomStartHSValue.getLabel();

                            SS3FieldValue nondomEndHSValue =
                                    cs.getFieldsMapSS3FieldValue(Util.NDOM_HANDSHAPE_FIELD_ID, nondomEndHSItemID);
                            nondomEndHS = nondomEndHSValue.getLabel();

                            break;
                        }
                    }
                }
            }
        }

        // NEXT begin to compare the sign-bank-defined occurrence data to the saved entity data
        boolean differingFromOccurrence = false;

        // first with the hs labels
        if (occDomStartHS != null && domStartHS != null && !occDomStartHS.equals(domStartHS))
            differingFromOccurrence = true;
        if (occNondomStartHS != null && nondomStartHS != null && !occNondomStartHS.equals(nondomStartHS))
            differingFromOccurrence = true;
        if (occDomEndHS != null && domEndHS != null && !occDomEndHS.equals(domEndHS))
            differingFromOccurrence = true;
        if (occNondomEndHS != null && nondomEndHS != null && !occNondomEndHS.equals(nondomEndHS))
            differingFromOccurrence = true;

        // then with the hs same/diff classifications
        if (occLeftRightHS != null) {
            if (twoHanded && occLeftRightHS.equals("SAME")
                    && !leftRightHS.equals(GlossEntityProperties.SAME_HANDSHAPES_DOM_NDOM)) {
                differingFromOccurrence = true;

                if (occStartEndHS != null) {
                    if (occStartEndHS.equals("SAME") &&
                            !startEndHS.equals(GlossEntityProperties.SAME_START_END_HANDSHAPE))
                        differingFromOccurrence = true;
                    else if (occStartEndHS.equals("DIFFERENT") &&
                            !startEndHS.equals(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE))
                        differingFromOccurrence = true;
                }
            } else if (twoHanded && occLeftRightHS.equals("DIFFERENT") &&
                    !leftRightHS.equals(GlossEntityProperties.DIFFERENT_HANDSHAPES_DOM_NDOM)) {
                differingFromOccurrence = true;

                if (occStartEndLeftHS != null) {
                    if (occStartEndLeftHS.equals("SAME") &&
                            !startEndLeftHS.equals(GlossEntityProperties.SAME_START_END_HANDSHAPE))
                        differingFromOccurrence = true;
                    else if (occStartEndLeftHS.equals("DIFFERENT") &&
                            !startEndLeftHS.equals(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE))
                        differingFromOccurrence = true;
                }

                if (occStartEndRightHS != null) {
                    if (occStartEndRightHS.equals("SAME") &&
                            !startEndRightHS.equals(GlossEntityProperties.SAME_START_END_HANDSHAPE))
                        differingFromOccurrence = true;
                    else if (occStartEndRightHS.equals("DIFFERENT") &&
                            !startEndRightHS.equals(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE))
                        differingFromOccurrence = true;
                }
            }
        }

        if (occTwoHanded != twoHanded)
            differingFromOccurrence = true;

        if (occMarkedHands != markedHands)
            differingFromOccurrence = true;

        if (occPassiveBaseArm != passiveBaseArm)
            differingFromOccurrence = true;

        if ( !occSignType.equals(signType))
            differingFromOccurrence = true;

        properties.setDifferingFromOccurrence(differingFromOccurrence);

        // regardless of differing, always set the source resource on the entity properties to the now current source resource
        properties.setSourceResource(occSourceResource);
        properties.setVerifiedFromSignBank(true);	// should already be set anyways

        return true;
    }

    public SS3CodingScheme parseCodingScheme(String fileName) {
        SS3CodingScheme cScheme = new SS3CodingScheme();

        try {
            DocumentBuilder builder = DocumentBuilderFactory.newInstance()
                .newDocumentBuilder();
            Document doc = builder.parse(fileName);

            NodeList nodes1 = doc.getElementsByTagName("SIGNSTREAM-DATABASE");
            for (int i = 0; i < nodes1.getLength(); i++) {
                Element el1 = (Element) nodes1.item(i);
                cScheme.setSignStreamVersion(el1.getAttribute("SIGNSTREAM-VERSION"));
                cScheme.setDbVersion(el1.getAttribute("DATABASE-VERSION"));
                NodeList refList = el1.getElementsByTagName("REFERENCES");

                for (int j = 0; j < refList.getLength(); j++) {
                    Element el2 = (Element) refList.item(j);
                    NodeList cSchRefList = el2
                        .getElementsByTagName("CODING-SCHEMES");
                    NodeList prtRefList = el2
                        .getElementsByTagName("PARTICIPANTS");

                    cScheme.setCodingSchemeRef(parseReferences(cSchRefList));
                    cScheme.setParticipantRef(parseReferences(prtRefList));
                }

                NodeList authorList = el1.getElementsByTagName("AUTHORS");
                cScheme.setAuthors(parseAuthors(authorList));

                NodeList csList = el1.getElementsByTagName("CODING-SCHEME");

                for (int l = 0; l < csList.getLength(); l++) {
                    Element el10 = (Element) csList.item(l);

                    NodeList imgsList = el10.getElementsByTagName("IMGS");
                    ArrayList<SS3FieldTypeImg> imgs = parseHandshapeImages(imgsList);
                    for (SS3FieldTypeImg img : imgs) {
                        cScheme.addImage(img);
                    }

                    var panels = readChildListOrdered(el10, PANEL_LIST_TAG, PANEL_TAG)
                        .map(SS4Util::readPanelEl)
                        .collect(Collectors.toList());
                    cScheme.setPanels(panels);

                    var viewModes = readChildListOrdered(el10, VIEW_MODE_LIST_TAG, VIEW_MODE_TAG)
                        .map(el -> readViewModeEl(cScheme, el))
                        .collect(Collectors.toList());
                    cScheme.setViewModes(viewModes);

                    var defViewMode = readRef(readChild(el10, VIEW_MODE_LIST_TAG), "DEFAULT", VIEW_MODE_TAG);
                    cScheme.setDefaultViewMode(cScheme.viewMode(defViewMode));

                        Element locationConfigEl = (Element) el10.getElementsByTagName("LOCATION_CONFIG").item(0);
                    parseLocationImages((Element) locationConfigEl.getElementsByTagName("IMAGES").item(0));

                    Element regionsEl = (Element) locationConfigEl.getElementsByTagName("REGIONS").item(0);
                    Element regionAreasEl = (Element) locationConfigEl.getElementsByTagName("REGION_AREAS").item(0);
                    parseRegions(regionsEl, regionAreasEl);
                    parseLocationTypes((Element) locationConfigEl.getElementsByTagName("TYPES").item(0));

                    NodeList tmpltsList = el10
                        .getElementsByTagName("TEMPLATES");
                    Element tmpltsEl = (Element) tmpltsList.item(0);
                    CSHandShapeTemplates templates = parseHandShapeTemplates(tmpltsEl);
                    cScheme.setHandShapeTemplates(templates);

                    NodeList fldsList = el10
                        .getElementsByTagName("FIELD_TYPES");
                    ArrayList<SS3FieldType> flds = parseFieldTypes(fldsList);
                    for (int k = 0; k < flds.size(); k++) {
                        cScheme.addFieldType(flds.get(k));
                    }

                    // BEGIN: gesture types/categories
                    GestureTypeCategoryObject gestureTypeObject = new GestureTypeCategoryObject();

                    NodeList gstList = el10.getElementsByTagName("GESTURE_TYPES");
                    HashMap<String, GestureType> gestTypes = parseGestureTypes(gstList);

                    NodeList gstCatList = el10.getElementsByTagName("GESTURE_CATEGORIES");
                    HashMap<String, GestureCategory> gestureCategories = parseGestureCategories(gstCatList, gestTypes);

                    gestureTypeObject.setGestureCategories(gestureCategories);

                    cScheme.setGestureTypeObject(gestureTypeObject);
                    // END: gesture types/categories

                    NodeList vcNode = el10
                        .getElementsByTagName("VALUE_COLLECTIONS");
                    ValueCollections col = parseValueCollections(vcNode);
                    cScheme.setValueCollections(col);

                    NodeList stmtList = el10.getElementsByTagName("FIELDS");

                    Element valEl = (Element) stmtList.item(0);
                    cScheme.setColor(valEl.getAttribute("DFLT_COLOR"));
                    cScheme.setFields(parseFields(stmtList));

                    NodeList dList = el10
                        .getElementsByTagName("DEFAULT-SEGMENT");

                    ArrayList<String> defaultSegmentList = new ArrayList<String>();

                    Element pEl = (Element) dList.item(0);
                    NodeList vNode = pEl.getElementsByTagName("F");

                    for (int j = 0; j < vNode.getLength(); j++) {
                        Element vEl = (Element) vNode.item(j);
                        String id = vEl.getAttribute("ID");
                        defaultSegmentList.add(id);
                    }

                    cScheme.setDefaultSegment(defaultSegmentList);

                    Element ipaCharsEl = (Element) el10.getElementsByTagName("IPA-CHARS").item(0);
                    parseIPAChars(ipaCharsEl);
                }
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            SS3SignStreamApplication.logger.setLevel(Level.SEVERE);
            SS3SignStreamApplication.logger.info(ex.getMessage());
        }

        return cScheme;
    }

    private ArrayList<Reference> parseReferences(NodeList nodes) {
        ArrayList<Reference> list = new ArrayList<Reference>();
        Element element = (Element) nodes.item(0);
        NodeList prtcpNode = element.getElementsByTagName("REF");

        for (int i = 0; i < prtcpNode.getLength(); i++) {
            Reference ref = new Reference();
            Element line = (Element) prtcpNode.item(i);
            ref.setId(line.getAttribute("ID"));
            ref.setFileName(line.getAttribute("FILE_NAME"));
            ref.setSignature(line.getAttribute("SIGNATURE"));
            list.add(ref);
        }
        return list;
    }

    private ArrayList<Author> parseAuthors(NodeList nodes) {
        ArrayList<Author> list = new ArrayList<Author>();
        Element element = (Element) nodes.item(0);
        NodeList authorNode = element.getElementsByTagName("AUTHOR");

        for (int i = 0; i < authorNode.getLength(); i++) {
            Author author = new Author();
            Element line = (Element) authorNode.item(i);
            author.setParticipantID(line.getAttribute("PARTICIPANT_ID"));
            list.add(author);
        }
        return list;
    }

    private void parseIPAChars(Element ipaCharsEl) {
        Element symbolsEl = (Element) ipaCharsEl.getElementsByTagName("SYMBOLS").item(0);
        NodeList symbolsList = symbolsEl.getElementsByTagName("SYMBOL");
        String[] symbols = new String[symbolsList.getLength()];
        for (int i = 0; i < symbolsList.getLength(); i++) {
            Element symbolEl = (Element) symbolsList.item(i);
            symbols[i] = Character.toString((char) Integer.parseInt(symbolEl.getTextContent(), 16));
        }
        Element diasEl = (Element) ipaCharsEl.getElementsByTagName("DIACRITICS").item(0);
        NodeList diaList = diasEl.getElementsByTagName("DIACRITIC");
        String[] diacritics = new String[diaList.getLength()];
        for (int i = 0; i < diaList.getLength(); i++) {
            Element diaEl = (Element) diaList.item(i);
            Element symbolEl = (Element) diaEl.getElementsByTagName("SYMBOL").item(0);
            Element previewCharEl = (Element) diaEl.getElementsByTagName("PREVIEW-CHAR").item(0);
            diacritics[i] = previewCharEl.getTextContent() + Character.toString((char) Integer.parseInt(symbolEl.getTextContent(), 16));
        }
        IPAKeyboard.setSymbols(symbols);
        IPAKeyboard.setDiacritics(diacritics);
    }

    /**
     * <TEMPLATES> <TEMPLATE ID="0" NAME="American Sign Language" LABEL = "ASL">
     * <PALETTE ID="1" NAME="Letters">
     * <ITEM ID="HSH_014" DESCRIPTION="A" UNMARKED="Y" FINGERSPELLED="Y"/>
     * <ITEM ID="HSH_020" DESCRIPTION="B" UNMARKED="Y" FINGERSPELLED="Y"/>
     * </PALETTE>
     */
    private CSHandShapeTemplates parseHandShapeTemplates(Element element) {
        CSHandShapeTemplates templates = new CSHandShapeTemplates();

        NodeList templateList = element.getElementsByTagName("TEMPLATE");

        for (int n = 0; n < templateList.getLength(); n++) {
            Element templateEl = (Element) templateList.item(n);

            CSHandShapeTemplate template = new CSHandShapeTemplate();

            template.setId(templateEl.getAttribute("ID"));
            template.setName(templateEl.getAttribute("NAME"));
            template.setLabel(templateEl.getAttribute("LABEL"));

            NodeList paletteList = templateEl.getElementsByTagName("PALETTE");

            for (int p = 0; p < paletteList.getLength(); p++) {
                Element paletteEl = (Element) paletteList.item(p);

                CSHandShapePalette palette = new CSHandShapePalette();

                palette.setId(paletteEl.getAttribute("ID"));
                palette.setName(paletteEl.getAttribute("NAME"));

                NodeList itemList = paletteEl.getElementsByTagName("ITEM");

                for (int i = 0; i < itemList.getLength(); i++) {
                    Element itemEl = (Element) itemList.item(i);

                    CSHandShapeItem item = new CSHandShapeItem();

                    item.setId(itemEl.getAttribute("ID"));
                    item.setDescription(itemEl.getAttribute("DESCRIPTION"));
                    item.setUnmarked(Util.isTrue(itemEl
                        .getAttribute("UNMARKED")));
                    item.setFingerspelled(Util.isTrue(itemEl
                        .getAttribute("FINGERSPELLED")));
                    palette.addHandShapeItem(item);
                }
                template.addHandShapePalette(palette);
            }
            templates.addHandShapeTemplate(template);
        }

        return templates;
    }

    // point to VALUE_COLLECTIONS
    private ValueCollections parseValueCollections(NodeList cNode) {
        ValueCollections col = new ValueCollections();

        Element pEl = (Element) cNode.item(0);
        NodeList vNode = pEl.getElementsByTagName("VALUE_COLLECTION");

        for (int i = 0; i < vNode.getLength(); i++) {
            Element vEl = (Element) vNode.item(i);
            String id = vEl.getAttribute("ID");
            ValueCollection vc = new ValueCollection(id);

            NodeList node = vEl.getElementsByTagName("VALUE");
            for (int j = 0; j < node.getLength(); j++) {
                Element e = (Element) node.item(j);
                SS3FieldValue value = new SS3FieldValue();
                value.setId(e.getAttribute("ID"));
                value.setName(e.getAttribute("NAME"));
                value.setLabel(e.getAttribute("LABEL"));
                vc.addValue(value);
            }

            col.addCollection(vc);
        }
        return col;
    }

    // point to STATEMENTS
    private ArrayList<SS3GlossWindowItem> parseStatements(NodeList nodes) {
        ArrayList<SS3GlossWindowItem> statements = new ArrayList<SS3GlossWindowItem>();

        Element element = (Element) nodes.item(0);
        NodeList tNode = element.getElementsByTagName("STATEMENT");

        for (int i = 0; i < tNode.getLength(); i++) {
            SS3GlossWindowItem statement = new SS3GlossWindowItem("", "");

            Element tEl = (Element) tNode.item(i);

            NodeList mediaOnScreenLocationList = tEl
                .getElementsByTagName("MEDIA_FILES_ON_SCREEN_LOCATION");
            statement
                .setVideoOnScreenLocations(parseMFilesOnScreenLocation(mediaOnScreenLocationList));
            statement.setId(tEl.getAttribute("ID"));
            statement.setNoteID(tEl.getAttribute("NOTE_ID"));

            statement.setLabel(tEl.getAttribute("EXCERPT"));
            Time strTimeInfo = SS3Singleton.getVideoTimeRange().time(Integer.parseInt(tEl.getAttribute("S")), TimeUnit.TIMESTAMP);
            Time endTimeInfo = SS3Singleton.getVideoTimeRange().time(Integer.parseInt(tEl.getAttribute("E")), TimeUnit.TIMESTAMP);
            statement.setStartTimeInfo(strTimeInfo);
            statement.setEndTimeInfo(endTimeInfo);

            NodeList pnl = tEl.getElementsByTagName("PARTICIPANT");
            ArrayList<String> participants = new ArrayList<String>();
            for (int r = 0; r < pnl.getLength(); r++) {
                Element pel = (Element) pnl.item(r);
                participants.add(pel.getAttribute("ID"));
            }
            statement.setParticipants(participants);

            NodeList cnl = tEl.getElementsByTagName("CODER");
            ArrayList<String> coders = new ArrayList<String>();
            for (int c = 0; c < cnl.getLength(); c++) {
                Element cel = (Element) cnl.item(c);
                coders.add(cel.getAttribute("ID"));
            }
            statement.setCoders(coders);

            NodeList fnls = tEl.getElementsByTagName("FIELDS");
            for (int j = 0; j < fnls.getLength(); j++) {
                Element fel = (Element) fnls.item(j);
                NodeList fnl = fel.getElementsByTagName("FIELD");

                for (int l = 0; l < fnl.getLength(); l++) {
                    Element fEl = (Element) fnl.item(l);
                    SS3StatementField field = new SS3StatementField();
                    field.setId(fEl.getAttribute("ID"));

    				ArrayList<SS3Entity> statementFieldEntitiesList = new ArrayList<SS3Entity>();

                    NodeList ent = fEl.getElementsByTagName("ENTITIES");
                    if (ent != null && ent.getLength() > 0) {
                        Element eEl = (Element) ent.item(0);
                        NodeList eNl = eEl.getElementsByTagName("ENTITY");
                        for (int e = 0; e < eNl.getLength(); e++) {
                            // TODO: copy to here the functionality from SS3ZipXmlFilesParser.parseSS3GlossWindow()
                            Element entEl = (Element) eNl.item(e);
                            SS3Entity ss3Entity = new SS3Entity();
                            ss3Entity.setID(entEl.getAttribute("ID"));
                            statementFieldEntitiesList.add(ss3Entity);
                        }
                    }
                    field.setSS3Entities(statementFieldEntitiesList);
                    statement.addField(field);
                }
            }
            statements.add(statement);
        }
        return statements;
    }

    // point to PRESENTATIONS
    private ArrayList<SS3Selection> parseSelections(NodeList nodes) {
        ArrayList<SS3Selection> selections = new ArrayList<SS3Selection>();

        Element element = (Element) nodes.item(0);
        NodeList sNode = element.getElementsByTagName("SELECTION");

        for (int i = 0; i < sNode.getLength(); i++) {
            SS3Selection selection = new SS3Selection();

            Element sEl = (Element) sNode.item(i);
            selection.setId(sEl.getAttribute("ID"));
            selection.setLabel(sEl.getAttribute("LABEL"));

            NodeList pnl = sEl.getElementsByTagName("STATEMENT");
            for (int r = 0; r < pnl.getLength(); r++) {
                Element pel = (Element) pnl.item(r);
                selection.addStatementID(pel.getAttribute("ID"));
            }
            selections.add(selection);
        }
        return selections;
    }

    // point to IMGS
    /**
     * <IMG ID="HSH_001" PATH1="big-1.jpg" PATH2="1.jpg"> <SELECTED_FIINGERS
     * ID="1"> <BASE_JOINTS ID="1"> <SPREAD ID="1"/> </BASE_JOINTS>
     * <NONBASE_JOINTS ID="1"/> </SELECTED_FIINGERS> <NONSELECTED_FINGERS
     * ID="1"/> <THUMB_CONTACT ID="1"/> </IMG>
     *
     * <IMG ID="HSH_001" PATH1="big-1.jpg" PATH2="1.jpg" slct_fingers="1"
     * base_joints="1" spread="1" non_base_joints="1" non_slct_fingers="1"
     * thumb_contact="1"/>
     */
    private ArrayList<SS3FieldTypeImg> parseHandshapeImages(NodeList imgsNode) {
        ArrayList<SS3FieldTypeImg> images = new ArrayList<SS3FieldTypeImg>();

        Element element = (Element) imgsNode.item(0);
        NodeList imgNode = element.getElementsByTagName("IMG");

        for (int j = 0; j < imgNode.getLength(); j++) {
            SS3FieldTypeImg img = new SS3FieldTypeImg();
            Element imgEl = (Element) imgNode.item(j);
            img.setId(imgEl.getAttribute("ID"));
            img.setPath1(imgEl.getAttribute("PATH1"));
            img.setPath2(imgEl.getAttribute("PATH2"));
            img.setLabel(imgEl.getAttribute("LABEL"));
            img.setSelectedFingersID(imgEl.getAttribute("slct_fingers"));
            img.setBaseJointsID(imgEl.getAttribute("base_joints"));
            img.setSpreadID(imgEl.getAttribute("spread"));
            img.setNonBaseJointsID(imgEl.getAttribute("non_base_joints"));
            img.setNonSelectedFingersID(imgEl.getAttribute("non_slct_fingers"));
            img.setThumbContactID(imgEl.getAttribute("thumb_contact"));

            images.add(img);
        }
        return images;
    }

    private void parseLocationImages(Element imagesEl) {
        NodeList imagesList = imagesEl.getElementsByTagName("IMAGE");
        String basePath = imagesEl.getAttribute("BASE_PATH");
        LocationConfig.IMAGE_BASE_PATH = SSResources.resourcesPath + "/" + basePath;
        for (int i = 0; i < imagesList.getLength(); i++) {
            Element imageEl = (Element) imagesList.item(i);
            String id = imageEl.getAttribute("ID");
            String path = imageEl.getAttribute("PATH");
            String dominance = imageEl.getAttribute("DOMINANCE");

            SS3Singleton.getLocationConfig().addImage(new LocationImage(id, path, dominance));
        }
    }

    private void parseRegions(Element regionsEl, Element regionAreasEl) {
        LocationConfig locationConfig = SS3Singleton.getLocationConfig();

        NodeList regionsList = regionsEl.getElementsByTagName("REGION");

        HashMap<String, String> regionAreasMap = new HashMap<String, String>();
        for (int i = 0; i < regionsList.getLength(); i++) {
            Element regionEl = (Element) regionsList.item(i);

            String id = regionEl.getAttribute("ID");
            String name = regionEl.getAttribute("NAME");
            String abbr = regionEl.getAttribute("ABBR");
            boolean selectable = false;
            String selectableAttr = regionEl.getAttribute("SELECTABLE");
            if (selectableAttr != null && !selectableAttr.isEmpty()) {
                selectable = selectableAttr.equals("1");
            }

            Region region = new Region(id,name,abbr);
            region.setSelectable(selectable);

            locationConfig.addRegion(region);

            String regionAreaId = regionEl.getAttribute("REGION_AREA_ID");
            if (regionAreaId != null && !regionAreaId.isEmpty()) {
                regionAreasMap.put(regionAreaId, id);
            }
        }

        NodeList regionAreasList = regionAreasEl.getElementsByTagName("REGION_AREA");
        for (int i = 0; i < regionAreasList.getLength(); i++) {
            Element regionAreaEl = (Element) regionAreasList.item(i);

            String regionAreaID = regionAreaEl.getAttribute("ID");
            String imageID = regionAreaEl.getAttribute("IMAGE_ID");

            Region parentRegion = locationConfig.getRegion(regionAreasMap.get(regionAreaID));
            parentRegion.setImageID(imageID);

            NodeList subregionsList = regionAreaEl.getElementsByTagName("SUB_REGION_AREA");
            final HashMap<String, Integer> subregionOrders = new HashMap<String, Integer>();
            for (int j = 0; j < subregionsList.getLength(); j++) {
                Element subregionEl = (Element) subregionsList.item(j);

                String subregionID = subregionEl.getAttribute("REGION_ID");
                int subregionOrder = Integer.parseInt(subregionEl.getAttribute("ORDER"));
                subregionOrders.put(subregionID, subregionOrder);

                Region subregion = locationConfig.getRegion(subregionID);
                subregion.setParent(parentRegion);

                NodeList polygonsList = subregionEl.getElementsByTagName("POLYGON");
                ArrayList<Polygon> polygons = new ArrayList<Polygon>();
                for (int k = 0; k < polygonsList.getLength(); k++) {
                    Element polygonEl = (Element) polygonsList.item(k);

                    String[] xcoordsStr = polygonEl.getAttribute("X").split(",");
                    String[] ycoordsStr = polygonEl.getAttribute("Y").split(",");

                    int[] xcoords = new int[xcoordsStr.length];
                    for (int l = 0; l < xcoordsStr.length; l++) {
                        xcoords[l] = Integer.parseInt(xcoordsStr[l]);
                    }
                    int[] ycoords = new int[ycoordsStr.length];
                    for (int l = 0; l < ycoordsStr.length; l++) {
                        ycoords[l] = Integer.parseInt(ycoordsStr[l]);
                    }

                    polygons.add(new Polygon(xcoords, ycoords, xcoords.length));
                }

                parentRegion.addSubregion(subregion, subregionOrder, polygons);
            }

        }
    }

    private void parseLocationTypes(Element typesEl) {
        NodeList typesList = typesEl.getElementsByTagName("TYPE");
        for (int i = 0; i < typesList.getLength(); i++) {
            Element typeEl = (Element) typesList.item(i);
            String type = typeEl.getAttribute("NAME");
            int id = Integer.parseInt(typeEl.getAttribute("ID"));
            int order = Integer.parseInt(typeEl.getAttribute("ORDER"));
            SS3Singleton.getLocationConfig().addLocationType(new LocationType(type, id, order));
        }
    }

    //nodes should reference GESTURE_CATEGORIES
    private HashMap<String, GestureCategory> parseGestureCategories(NodeList nodes, HashMap<String, GestureType> gestureTypes) {
        HashMap<String, GestureCategory> gCategories = new HashMap<String, GestureCategory>();

        Element element = (Element) nodes.item(0);
        NodeList tNode = element.getElementsByTagName("GESTURE_CATEGORY");
        for (int i = 0; i < tNode.getLength(); i++) {
            GestureCategory gc = new GestureCategory();
            Element tEl = (Element) tNode.item(i);
            gc.setId(tEl.getAttribute("ID"));
            gc.setName(tEl.getAttribute("NAME"));
            gc.setParentCategoryId(tEl.getAttribute("PARENT_GESTURE_CATEGORY_ID"));
            gc.setDomFieldID(tEl.getAttribute("DOM_FIELD_ID"));
            gc.setNDomFieldID(tEl.getAttribute("NDOM_FIELD_ID"));

            NodeList types = tEl.getElementsByTagName("GESTURE_TYPE");
            for (int j = 0; j < types.getLength(); j++) {
                Element type = (Element) types.item(j);
                String id = type.getAttribute("GESTURE_TYPE_ID");
                GestureType gestureType = gestureTypes.get(id);
                gc.addGestureType(gestureType);
                gestureType.setParentCategory(gc);
            }
            gCategories.put(gc.getId(), gc);
        }

        return gCategories;
    }

    //nodes should reference GESTURE_TYPES
    private HashMap<String, GestureType> parseGestureTypes(NodeList nodes) {
        HashMap<String, GestureType> gTypes = new HashMap<String, GestureType>();

        Element element = (Element) nodes.item(0);
        NodeList tNode = element.getElementsByTagName("GESTURE_TYPE");
        for (int i = 0; i < tNode.getLength(); i++) {
            GestureType gt = new GestureType();
            Element tEl = (Element) tNode.item(i);
            gt.setId(tEl.getAttribute("ID"));
            gt.setName(tEl.getAttribute("NAME"));
            gt.setAbbr(tEl.getAttribute("ABBR"));
            gTypes.put(gt.getId(), gt);
        }

        return gTypes;
    }

    // nodes should referense FIELD_TYPES
    private ArrayList<SS3FieldType> parseFieldTypes(NodeList nodes) {
        ArrayList<SS3FieldType> fTypes = new ArrayList<SS3FieldType>();
        HashMap<String, SS3FieldType> hTypes = new HashMap<String, SS3FieldType>();

        Element element = (Element) nodes.item(0);
        NodeList tNode = element.getElementsByTagName("FIELD_TYPE");

        for (int i = 0; i < tNode.getLength(); i++) {
            SS3FieldType ft = new SS3FieldType();
            Element tEl = (Element) tNode.item(i);
            ft.setType(tEl.getAttribute("TYPE"));
            ft.setLabel(tEl.getAttribute("LABEL"));
            ft.setDecsription(tEl.getAttribute("DESCRIPTION"));
            ft.setRefType(tEl.getAttribute("REF"));
            ft.setDepType(tEl.getAttribute("DEP_TYPE"));

            fTypes.add(ft);
            hTypes.put(ft.getType(), ft);
        }

        for (int j = 0; j < fTypes.size(); j++) {
            SS3FieldType ft = fTypes.get(j);
            String refType = ft.getRefType();
            Object obj = hTypes.get(refType);
            if (obj != null) {
                SS3FieldType ftm = (SS3FieldType) obj;
                ftm.addDependentField(ft);
            }
        }

        return fTypes;
    }
}
