package edu.bu.signstream.common.db.ss4;

import edu.bu.signstream.common.db.XMLWriter;
import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.Reference;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.common.util.vo.ss3.db.*;
import edu.bu.signstream.common.util.vo.ss3.media.SS3MediaLibrary;
import edu.bu.signstream.common.util.vo.ss3.note.SS3Note;
import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureEntityProperties;
import edu.bu.signstream.grepresentation.fields.nonManualField.GlossEntityProperties;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.ui.SS3SignStreamApplication;
import edu.bu.signstream.ui.SerchableDirectory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import java.util.logging.Level;
import java.util.stream.Collectors;

import static edu.bu.signstream.common.db.ss4.SS4Util.*;

/**
 * Temporary class to save SS3 database.
 */
public class SS4Writer {
    private Element createDistributorElement(SS3DBProfile prf, Document document) {
        Element root = document.createElement("DISTRIBUTOR");
        root.appendChild(document.createTextNode(prf.getDistributor()));
        return root;
    }

    private Element createReferencesElement(SS3Database db, Document document) {
		Element root = document.createElement("REFERENCES");
		root.appendChild(createCodingSchemasElement(db, document));
		root.appendChild(createParticipantsElement(db, document));
		root.appendChild(createAnnotatorsElement(db, document));
		return root;
	}

	private Element createCodingSchemasElement(SS3Database db, Document document) {
		Element root = document.createElement("CODING-SCHEMES");
		ArrayList csRef = db.getCodingSchemeRef();
		for (int i = 0; i < csRef.size(); i++) {
			Reference ref = (Reference) csRef.get(i);
			Element rEl = document.createElement("CODING-SCHEME");
			rEl.setAttribute("ID", "" + ref.getId());
			rEl.setAttribute("FILE_NAME", "" + ref.getFileName());
			rEl.setAttribute("SIGNATURE", "" + ref.getSignature());
			root.appendChild(rEl);
		}
		return root;
	}

	private Element createParticipantsElement(SS3Database db, Document document) {
		Element root = document.createElement("PARTICIPANTS");
		ArrayList<SS3Participant> participants = db.getSelectedParticipants();

		for (SS3Participant participant : participants) {
			Element pEl = document.createElement("PARTICIPANT");
			pEl.setAttribute("ID", "" + participant.getId());
			pEl.setAttribute("LABEL", "" + participant.getLabel());
			pEl.setAttribute("BACKGROUND", "" + participant.getBackground());
			pEl.setAttribute("COMMENTS", "" + participant.getComments());
			pEl.setAttribute("DOMINANT_HAND", "" + participant.getDominantHand());
			pEl.setAttribute("FULL_NAME", "" + participant.getFullName());
			pEl.setAttribute("GENDER", "" + participant.getGender());
			pEl.setAttribute("NATIVE_LANGUAGE", "" + participant.getNativeLanguage());
			pEl.setAttribute("PARENT_INFO", "" + participant.getParentsInfo());
			pEl.setAttribute("YEAR_OF_BIRTH", "" + participant.getYearOfBirth());

			if (participant.getGraphFileID() != null && !participant.getGraphFileID().isEmpty()) {
				writeRef(pEl, "GRAPH", participant.getGraphFileID());
			}
			root.appendChild(pEl);
		}
		return root;
	}

	private Element createAnnotatorsElement(SS3Database db, Document document) {
		Element root = document.createElement("ANNOTATORS");
		ArrayList<SS3Participant> annotators = db.getSelectedAnnotators();

		for (SS3Participant annotator : annotators) {
			Element aEl = document.createElement("ANNOTATOR");
			aEl.setAttribute("ID", "" + annotator.getId());
			aEl.setAttribute("LABEL", "" + annotator.getLabel());
			root.appendChild(aEl);
		}
		return root;
	}

	private Element createCodingSchemeElement(SS3Database db, Document document) {

		Element root = document.createElement("LOCAL-CODING-SCHEME");
		SS3CodingScheme localScheme = db.getLocalSS3CodingScheme();
		SS3CodingScheme defScheme = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();

		if (localScheme == null)
			return root;

		Element fieldsEl = document.createElement("FIELDS");
		
		HashMap fields = localScheme.getFields();

		Set keys = fields.keySet();
		Iterator it = keys.iterator();
		while (it.hasNext()) {
			Object obj = fields.get(it.next());
			SS3Field localField = (SS3Field) obj;
			
			SS3Field defaultField = defScheme.getField(localField.getId());

			if (localField.equals(defaultField))
			    continue;

			String name = localField.getName() == null ? "" : localField.getName();
			String label = localField.getLabel() == null ? "" : localField.getLabel();
			String prefix = localField.getPrefix() == null ? "" : localField.getPrefix();
			String color = localField.getColor() == null ? "" : ""
					+ Util.color2HexString(localField.getColor());
			String type = localField.getType() == null ? "" : localField.getType();
			String alignment = localField.getAlignment() == null ? "" : localField.getAlignment().toString();
			String group = localField.getGroup() == null ? "" : localField.getGroup();
			String order = localField.getOrder() == -1 ? "" : String.valueOf(localField.getOrder());
			boolean deleted = localField.isDeleted();

			boolean appendFieldEl = false;
			Element fieldEl = document.createElement("FIELD");
			fieldEl.setAttribute("ID", localField.getId());
			
			boolean appendFieldAttrsEl = false;
			Element fieldAttrsEl = document.createElement("FIELD-ATTRIBUTES");
			
			if ( ! name.isEmpty() && (defaultField == null || ! name.equals(defaultField.getName()))) {
				Element fieldAttrEl = document.createElement("FIELD-ATTRIBUTE");
				fieldAttrEl.setAttribute("NAME", name);
				fieldAttrsEl.appendChild(fieldAttrEl);
				
				appendFieldEl = true;
				appendFieldAttrsEl = true;
			}
			if ( ! label.isEmpty() && (defaultField == null || ! label.equals(defaultField.getLabel()))) {
				Element fldAttrEl = document.createElement("FIELD-ATTRIBUTE");
				fldAttrEl.setAttribute("LABEL", label);
				fieldAttrsEl.appendChild(fldAttrEl);
				
				appendFieldEl = true;
				appendFieldAttrsEl = true;
			}
			if ( ! prefix.isEmpty() && (defaultField == null || ! prefix.equals(defaultField.getPrefix()))) {
				Element fldAttrEl = document.createElement("FIELD-ATTRIBUTE");
				fldAttrEl.setAttribute("PREFIX", prefix);
				fieldAttrsEl.appendChild(fldAttrEl);
				
				appendFieldEl = true;
				appendFieldAttrsEl = true;
			}
			if ( ! color.isEmpty()) {
				boolean writeIt = false;
				
				if (defaultField != null) {
					var defaultFieldColor = defaultField.getColor();
					if (defaultFieldColor != null) {
						var defaultFieldHexColor = Util.color2HexString(defaultFieldColor);
						if ( ! color.equals(defaultFieldHexColor))
							writeIt = true;
					}
				} else {
					if (! color.equals(Util.color2HexString(SS3Field.defaultColor)))
						writeIt = true;
				}
				
				if (writeIt) {
					Element fldAttrEl = document.createElement("FIELD-ATTRIBUTE");
					fldAttrEl.setAttribute("COLOR", color);
					fieldAttrsEl.appendChild(fldAttrEl);
					
					appendFieldEl = true;
					appendFieldAttrsEl = true;
				}
			}
			if ( ! type.isEmpty() && (defaultField == null || ! type.equals(defaultField.getType()))) {
				Element fldAttrEl = document.createElement("FIELD-ATTRIBUTE");
				fldAttrEl.setAttribute("TYPE", type);
				fieldAttrsEl.appendChild(fldAttrEl);
				
				appendFieldEl = true;
				appendFieldAttrsEl = true;
			}
			if ( ! alignment.isEmpty() && (defaultField == null || ! alignment.equals(defaultField.getAlignment().toString()))) {
				Element fldAttrEl = document.createElement("FIELD-ATTRIBUTE");
				fldAttrEl.setAttribute("ALIGNMENT", alignment);
				fieldAttrsEl.appendChild(fldAttrEl);
				
				appendFieldEl = true;
				appendFieldAttrsEl = true;
			}
			if ( ! group.isEmpty() && (defaultField == null || ! group.equals(defaultField.getGroup()))) {
				Element fldAttrEl = document.createElement("FIELD-ATTRIBUTE");
				fldAttrEl.setAttribute("GROUP", group);
				fieldAttrsEl.appendChild(fldAttrEl);
				
				appendFieldEl = true;
				appendFieldAttrsEl = true;
			}
			if ( ! order.isEmpty() && (defaultField == null || ! order.equals(String.valueOf(defaultField.getOrder())))) {
				Element fldAttrEl = document.createElement("FIELD-ATTRIBUTE");
				fldAttrEl.setAttribute("ORDER", order);
				fieldAttrsEl.appendChild(fldAttrEl);
				
				appendFieldEl = true;
				appendFieldAttrsEl = true;
			}
			if (defaultField != null && deleted != defaultField.isDeleted()) {
				fieldEl.setAttribute("DELETED", "T");
				
				appendFieldEl = true;
			}
			
			if (appendFieldAttrsEl) {
				fieldEl.appendChild(fieldAttrsEl);
			}
			
			boolean appendFieldValuesEl = false;
			Element fieldValuesEl = document.createElement("FIELD-VALUES");
			
			ArrayList<SS3FieldValue> localFieldValues = localField.getValuesAsList();
			ArrayList<SS3FieldValue> defaultFieldValues = null;
			ArrayList<SS3FieldValue> deletedFieldValues = new ArrayList<SS3FieldValue>();
			
			if (defaultField != null) {
				defaultFieldValues = defaultField.getValuesAsList();

				defaultFieldValues.stream()
	                    .filter(val -> localField.getFieldValue(val.getId()) == null)
	                    .forEach(val -> deletedFieldValues.add(val));
			}
			
			for (SS3FieldValue localFieldValue : localFieldValues) {
				SS3FieldValue defaultFieldValue = null;
				if (defaultField != null)
					defaultFieldValue = defaultField.getFieldValue(localFieldValue.getId());
				
				if (localFieldValue.equals(defaultFieldValue))
					continue;

				String vName = localFieldValue.getName() == null ? "" : localFieldValue.getName();
				String vLabel = localFieldValue.getLabel() == null ? "" : localFieldValue.getLabel();
				String vType = localFieldValue.getType() == null ? "" : localFieldValue.getType();
				String iid = localFieldValue.getIid() == null ? "" : localFieldValue.getIid();

				boolean appendFieldValueEl = false;
				Element fieldValueEl = document.createElement("FIELD-VALUE");
				fieldValueEl.setAttribute("ID", localFieldValue.getId());

				boolean appendFieldValueAttrsEl = false;
				Element fieldValueAttrsEl = document.createElement("FIELD-VALUE-ATTRIBUTES");
				
				if ( ! vName.isEmpty() && (defaultFieldValue == null || ! vName.equals(defaultFieldValue.getName()))) {
					Element fldValAttrEl = document.createElement("FIELD-VALUE-ATTRIBUTE");
					fldValAttrEl.setAttribute("NAME", vName);
					fieldValueAttrsEl.appendChild(fldValAttrEl);
					
					appendFieldValueEl = true;
					appendFieldValueAttrsEl = true;
				}
				if ( ! vLabel.isEmpty() && (defaultFieldValue == null || ! vLabel.equals(defaultFieldValue.getLabel()))) {
					Element fldValAttrEl = document.createElement("FIELD-VALUE-ATTRIBUTE");
					fldValAttrEl.setAttribute("LABEL", vLabel);
					fieldValueAttrsEl.appendChild(fldValAttrEl);
					
					appendFieldValueEl = true;
					appendFieldValueAttrsEl = true;
				}
				if ( ! vType.isEmpty() && (defaultFieldValue == null || ! vType.equals(defaultFieldValue.getType()))) {
					Element fldValAttrEl = document.createElement("FIELD-VALUE-ATTRIBUTE");
					fldValAttrEl.setAttribute("TYPE", vType);
					fieldValueAttrsEl.appendChild(fldValAttrEl);
					
					appendFieldValueEl = true;
					appendFieldValueAttrsEl = true;
				}
				if ( ! iid.isEmpty() && (defaultFieldValue == null || ! iid.equals(defaultFieldValue.getIid()))) {
					Element fieldValueAttrEl = document.createElement("FIELD-VALUE-ATTRIBUTE");
					fieldValueAttrEl.setAttribute("IID", iid);
					fieldValueAttrsEl.appendChild(fieldValueAttrEl);
					
					appendFieldValueEl = true;
					appendFieldValueAttrsEl = true;
				}
				
				if (appendFieldValueAttrsEl) {
					fieldValueEl.appendChild(fieldValueAttrsEl);
					
					appendFieldValuesEl = true;
					appendFieldEl = true;
				}
				
				if (appendFieldValueEl) {
					fieldValuesEl.appendChild(fieldValueEl);
					
					appendFieldValuesEl = true;
					appendFieldEl = true;
				}
			}
			
			for (SS3FieldValue deletedFieldValue: deletedFieldValues) {
				Element fieldValueEl = document.createElement("FIELD-VALUE");
				fieldValueEl.setAttribute("ID", deletedFieldValue.getId());
				fieldValueEl.setAttribute("DELETED", "T");
				
				// below is as if appendFieldValueEl == true
				fieldValuesEl.appendChild(fieldValueEl);
				
				appendFieldValuesEl = true;
				appendFieldEl = true;
			}
			
			if (appendFieldValuesEl) {
				fieldEl.appendChild(fieldValuesEl);
			}
			
			if (appendFieldEl) {
				fieldsEl.appendChild(fieldEl);
			}
		}
		
		root.appendChild(fieldsEl);
		
		return root;
	}

	private Element createMediaElement(SS3Database db, Document doc) {
        Element root = doc.createElement("MEDIA");
       
        var videosList = db.getVideos().stream().sorted(
        		Comparator.comparing(n -> n.order())).collect(Collectors.toList());
       
        var audiosList = db.getAudios().stream().sorted(
        		Comparator.comparing(n -> n.order())).collect(Collectors.toList());
       
        writeChildListOrdered(root, VIDEO_LIST_TAG, videosList.stream().map(v -> writeVideoEl(doc, v)));
        writeChildListOrdered(root, AUDIO_LIST_TAG, audiosList.stream().map(a -> writeAudioEl(doc, a)));
        writeChildList(root, GRAPH_LIST_TAG, db.getGraphs().stream().map(g -> writeGraphEl(doc, g)));
        return root;
    }

	private Element createDatabaseElement(SS3Database db, Document document) {
		Element root = document.createElement("DATABASE");

		root.appendChild(createTracksElement(db, document));
//		root.appendChild(createNotesElement(db, document));

        SS3GlossWindow window = db.getSS3GlossWindow();
        ArrayList<SS3GlossWindowItem> items = window.getDisplayTimePeriod()
            .getSS3GlossWindowItems();

        writeRefs(root, "VISIBLE", "FIELD", window.getPermanentFieldIDs().stream());

        Element tpEl = document.createElement("TEMPORAL-PARTITIONS");
        root.appendChild(tpEl);

        ArrayList<SS3TemporalPartition> temporalPartitions = window
            .getTemporalPartitions();

        for (SS3TemporalPartition temporalPartition : temporalPartitions) {
            Element tp1El = document.createElement("TEMPORAL-PARTITION");
            tp1El.setAttribute("ID", temporalPartition.getID());
            tp1El.setAttribute("LABEL", temporalPartition.getLabel());
            tpEl.appendChild(tp1El);

            ArrayList<SS3GlossWindowSegment> segments = temporalPartition
                .getTemporalPartitionSegments();
            for (SS3GlossWindowSegment segment : segments) {
                Element sEl = document
                    .createElement("SEGMENT-TIER");
                sEl.setAttribute("ID", segment.getID());
                sEl.setAttribute("LABEL", segment.getLabel());
				String isdefault = segment.isDefault() ? "T" : "F";
				sEl.setAttribute("IS_DEFAULT", "" + isdefault);
                writeRef(sEl, "INDEX", "FIELD", segment.getIndexFieldID());
                String participantID = segment.getParticipant() != null ? segment.getParticipant().getId() : "";
                writeRef(sEl, "PARTICIPANT", participantID);

                sEl.setAttribute("CURRENT_GESTURE_COUNTER",
                    "" + segment.getCurrentGestureCounter());
                boolean showAllGestureCounters = segment.getShowAllGestureCounters();
                if (showAllGestureCounters)
                    sEl.setAttribute("SHOW_ALL_GESTURE_COUNTERS", "T");
                else
                    sEl.setAttribute("SHOW_ALL_GESTURE_COUNTERS", "F");

                boolean defaultShowGestureCounters = segment.getDefaultShowGestureCounters();
                if (defaultShowGestureCounters)
                    sEl.setAttribute("DEFAULT_SHOW_GESTURE_COUNTERS", "T");
                else
                    sEl.setAttribute("DEFAULT_SHOW_GESTURE_COUNTERS", "F");

                writeRefs(sEl, "VISIBLE", "FIELD", segment.getPermanentFieldIDs().stream());

                writeRefs(sEl, "VISIBLE", PANEL_TAG, segment.getVisiblePanelIDs().stream());

                SS3GlossWindowItemNote note = segment.getNote();
                if (note != null) {
                    String noteStr = note.getNote();
                    if (noteStr != null && !noteStr.isEmpty()) {
                        sEl.setAttribute("NOTE", noteStr);
                    }
                }

                // statement fields exist here, under the segment tier object
                ArrayList<SS3StatementField> statementFields1 = segment
                    .getStatementFields();
                for (SS3StatementField statementField : statementFields1) {
                    // STATEMENT-FIELD
                    Element stmtEl = document
                        .createElement("STATEMENT-FIELD");
                    write(stmtEl, "ID", statementField.getId());

                    Color fieldColor = statementField.getColor();
                    if (fieldColor != null) {
                        stmtEl.setAttribute("COLOR",
                            "" + fieldColor.getRGB());
                    }

                    var timeRange = statementField.getTimeRange();
                    if (timeRange != null)
                        writeRef(stmtEl, TIMERANGE_TAG, timeRange.id());

                    if (!statementField.getSS3Entities().isEmpty())
                        writeRefs(stmtEl, "ENTITY", statementField.getSS3Entities().stream().map(SS3Entity::getID));

                    sEl.appendChild(stmtEl);
                }

                tp1El.appendChild(sEl);

                ArrayList<SS3GlossWindowItem> segmentItems = segment
                    .getSS3GlossWindowItems();
                for (SS3GlossWindowItem segmentItem : segmentItems) {
                    segmentItem.synchronize();

                    Element wEl = document.createElement("SEGMENT-TIER-UTTERANCE");
                    wEl.setAttribute("ID", segmentItem.getId());
                    wEl.setAttribute("LABEL", segmentItem.getLabel());

                    writeRefs(wEl, "VISIBLE", "FIELD", segmentItem.getPermanentVisibleFieldIDs().stream());

                    writeInterval(wEl, segmentItem.interval());

                    sEl.appendChild(wEl);

                }
            }
        }

		return root;
	}

	private Element createTracksElement(SS3Database db, Document document) {
		Element root = document.createElement("TRACKS");
		ArrayList tracks = db.getTracksList();
		for (int i = 0; i < tracks.size(); i++) {
			SS3Track track = (SS3Track) tracks.get(i);

			Element tEl = document.createElement("TRACK");
			tEl.setAttribute("ID", "" + track.getId());
			ArrayList<SS3Entity> entities = track.getEntities();
			for (SS3Entity entity : entities) {
				Element eEl = document.createElement("ENTITY");
				eEl.setAttribute("ID", entity.getID());
				eEl.setAttribute("PARENT_ENTITY_ID", entity.getRefEntityId());
//				writeRef(eEl, "SEGMENT-TIER-UTTERANCE", entity.getParentEntityID());		// see below - GLOSS-WINDOW-ITEM-REFS
				writeRef(eEl, "COMP_START", "ENTITY", entity.getStartCompoundEntityID());
				writeRef(eEl, "COMP_END", "ENTITY", entity.getEndCompoundEntityID());

				GlossEntityProperties properties = entity
						.getGlossEntityProperties();

				GestureEntityProperties gestureProperties = entity
						.getGestureEntityProperties();

				if (track.getId().equalsIgnoreCase(Util.DOM_HANDSHAPE_FIELD_ID)
						|| track.getId().equalsIgnoreCase(Util.NDOM_HANDSHAPE_FIELD_ID)
						|| track.getId().equalsIgnoreCase(Util.DOM_HAND_FIELD_ID)
						|| track.getId().equalsIgnoreCase(Util.NDOM_HAND_FIELD_ID)) {
					
					boolean locked = entity.isLockedGlossPhoneAlignments();
					if(locked) {
						eEl.setAttribute("LOCKED_HAND_ALIGNMENT", "T");
					} else {
						eEl.setAttribute("LOCKED_HAND_ALIGNMENT", "F");
					}
				} else if (track.getId().equalsIgnoreCase(Util.DOM_GESTURE_HANDSHAPE_FIELD_ID)
						|| track.getId().equalsIgnoreCase(Util.NDOM_GESTURE_HANDSHAPE_FIELD_ID)
						|| track.getId().equalsIgnoreCase(Util.DOM_GESTURE_HAND_FIELD_ID)
						|| track.getId().equalsIgnoreCase(Util.NDOM_GESTURE_HAND_FIELD_ID)) {
					
					boolean locked = entity.isLockedGesturePhoneAlignments();
					if(locked) {
						eEl.setAttribute("LOCKED_HAND_ALIGNMENT", "T");
					} else {
						eEl.setAttribute("LOCKED_HAND_ALIGNMENT", "F");
					}
				}

				if ((track.getId().equalsIgnoreCase(Util.DOM_GLOSS_FIELD_ID) || track.getId()
						.equalsIgnoreCase(Util.NDOM_GLOSS_FIELD_ID))) {
					if (properties != null) {

						Element pEl = document.createElement("PROPERTIES");
						pEl.setAttribute("SIGN", properties.getSignType() == null ? "" : properties.getSignType().name());
						pEl.setAttribute("NAME_SIGN", properties.getNameSign());
						pEl.setAttribute("PASSIVE_BASE_ARM", properties.getPassiveBaseArm());
						pEl.setAttribute("NO_OF_HANDS", properties.getNumberOfHands());

						pEl.setAttribute("TWOHANDED_HADSHAPES", properties.getTwoHandedHandshapesClassification());
						pEl.setAttribute("START_END_HANDSHAPES", properties.getStartEndHandshape());
						pEl.setAttribute("NDOM_HAND_START_END_HANDSHAPE", properties.getStartEndHandshapeNdomHand());
						pEl.setAttribute("DOM_HAND_START_END_HANDSHAPE",
								properties.getStartEndHandshapeDomHand());

						eEl.appendChild(pEl);

						eEl.setAttribute("MARKED_NUMBER_OF_HANDS",
								properties.getMarkedNumberOfHands());
						writeRef(eEl, "CORRESPONDING_GLOSS", "ENTITY", entity.getDependentGlossEntityID());
					}

					boolean lockedGlossPhoneAlignments = entity
							.isLockedGlossPhoneAlignments();
					boolean lockedDomNdomAlignments = entity
							.isLockedDomNdomAlignments();

					String lockedGlossPhonAlignment = lockedGlossPhoneAlignments ? "T"
							: "F";
					String lockedDomNdomAlignment = lockedDomNdomAlignments ? "T"
							: "F";

					String depGlossID = entity.getDependentGlossEntityID();
					String depHandID = entity.getDependentHandEntityID();
					String depHandShapeID = entity
							.getDependentHandShapeEntityID();
					String depLocationID = entity.getDependentLocationEntityID();
					String depSignTypeID = entity.getDependentSignTypeEntityID();
					String primaryEntry = entity.getPrimaryEntry();
				
					eEl.setAttribute("LOCKED_DOM_NDOM_ALIGNMENT",
							lockedDomNdomAlignment);
					eEl.setAttribute("LOCKED_HAND_ALIGNMENT",
							lockedGlossPhonAlignment);

					writeRef(eEl, "DEPENDENT_GLOSS", "ENTITY", depGlossID);
					writeRef(eEl, "DEPENDENT_HAND", "ENTITY", depHandID);
					writeRef(eEl, "DEPENDENT_HAND_SHAPE", "ENTITY", depHandShapeID);
					writeRef(eEl, "DEPENDENT_LOCATION", "ENTITY", depLocationID);
					writeRef(eEl, "DEPENDENT_SIGN_TYPE", "ENTITY", depSignTypeID);
					
					eEl.setAttribute("PRIMARY_ENTRY", primaryEntry);
				} else if ( track.getId().equalsIgnoreCase(Util.DOM_GESTURE_FIELD_ID) || 
						track.getId().equalsIgnoreCase(Util.NDOM_GESTURE_FIELD_ID)) {
					
					boolean incompleteGesture = entity.isIncompleteGesture();
					if (incompleteGesture)
						eEl.setAttribute("INCOMPLETE", "T");
					else
						eEl.setAttribute("INCOMPLETE", "F");
					
					boolean showUniqueNumber = entity.getShowUniqueNumber();
					if (showUniqueNumber)
						eEl.setAttribute("SHOW_UNIQUE_NUMBER", "T");
					else
						eEl.setAttribute("SHOW_UNIQUE_NUMBER", "F");
					
					eEl.setAttribute("UNIQUE_NUMBER", entity.getUniqueNumber());
					
					if (gestureProperties != null) {
						Element pEl = document.createElement("PROPERTIES");
						pEl.setAttribute("SIGN", gestureProperties.getSignType() == null ? "" : gestureProperties.getSignType().name());
						pEl.setAttribute("NAME_SIGN", gestureProperties.getNameSign());
						pEl.setAttribute("PASSIVE_BASE_ARM", gestureProperties.getPassiveBaseArm());
						pEl.setAttribute("NO_OF_HANDS", gestureProperties.getNumberOfHands());

						pEl.setAttribute("TWOHANDED_HADSHAPES", gestureProperties.getTwoHandedHandshapesClassification());
						pEl.setAttribute("START_END_HANDSHAPES", gestureProperties.getStartEndHandshape());
						pEl.setAttribute("NDOM_HAND_START_END_HANDSHAPE", gestureProperties.getStartEndHandshapeNdomHand());
						pEl.setAttribute("DOM_HAND_START_END_HANDSHAPE",
								gestureProperties.getStartEndHandshapeDomHand());

						eEl.appendChild(pEl);

						eEl.setAttribute("DOMINANCE",
								gestureProperties.getDominamtHand());
						eEl.setAttribute("MARKED_NUMBER_OF_HANDS",
								gestureProperties.getMarkedNumberOfHands());
						writeRef(eEl, "CORRESPONDING_GESTURE", "ENTITY", entity.getDependentGestureEntityID());
					}

					boolean lockedGesturePhoneAlignments = entity
							.isLockedGesturePhoneAlignments();
					boolean lockedDomNdomAlignments = entity
							.isLockedDomNdomAlignments();

					String lockedGesturePhonAlignment = lockedGesturePhoneAlignments ? "T"
							: "F";
					String lockedDomNdomAlignment = lockedDomNdomAlignments ? "T"
							: "F";
					
					String depGestureID = entity.getDependentGestureEntityID();
					String depApexID = entity.getDependentApexEntityID();
					String depHandID = entity.getDependentHandEntityID();
					String depHandShapeID = entity.getDependentHandShapeEntityID();
					String depOrientationID = entity.getDependentOrientationEntityID();
					String depGestureTypeID = entity.getDependentSignTypeEntityID();
				
					eEl.setAttribute("LOCKED_DOM_NDOM_ALIGNMENT",
							lockedDomNdomAlignment);
					eEl.setAttribute("LOCKED_HAND_ALIGNMENT",
							lockedGesturePhonAlignment);
					
					writeRef(eEl, "DEPENDENT_GESTURE", "ENTITY", depGestureID);
					writeRef(eEl, "DEPENDENT_APEX", "ENTITY", depApexID);
					writeRef(eEl, "DEPENDENT_HAND", "ENTITY", depHandID);
					writeRef(eEl, "DEPENDENT_HAND_SHAPE", "ENTITY", depHandShapeID);
					writeRef(eEl, "DEPENDENT_ORIENTATION", "ENTITY", depOrientationID);
					writeRef(eEl, "DEPENDENT_GESTURE_TYPE", "ENTITY", depGestureTypeID);
					
				} else if (track.getId().equalsIgnoreCase(Util.DOM_GESTURE_TYPE_FIELD_ID)
						|| track.getId().equalsIgnoreCase(Util.NDOM_GESTURE_TYPE_FIELD_ID)) {
//					System.out.println(eEl);
				} else if (track.getId().equalsIgnoreCase(Util.FREE_TEXT_FIELD_ID)) {
					if (entity.isValidTime()) {
						eEl.setAttribute("VALID_TIME", "T");
					}
				}

				/**
				 * <ENTITY> <PROPERTIES
				 * SIGN="FINGERSPELLED/LOAN/LEXICAL/CLASSIFIER/GESTURE/NUMBER"
				 * NAME_SIGN="T/F" PASSIVE_BASE_ARM="T/F" NO_OF_HANDS="1/2"
				 *
				 * TWOHANDED_HADSHAPES="SAME/ALTERNATING_SAME/DIFFERENT"
				 * START_END_HANDSHAPES="SAME/DIFFERENT"
				 * NDOM_HAND_START_END_HANDSHAPE="SAME/DIFFERENT"
				 * DOM_HAND_START_END_HANDSHAPE="SAME/DIFFERENT" />
				 * <DOMINANCE="L/R"/> <MARKED_NUMBER_OF_HANDS="T/F"/> </ENTITY>
				 */

				ArrayList<SS3Item> items = entity.getItemsAsList();
				int order = 0;
				for (SS3Item item : items) {
					Element iEl = document.createElement("ITEM");
					iEl.setAttribute("ID", item.getId());
					
					String imageID = item.getImageID();
					if (imageID != null && !imageID.isBlank())
						writeRef(iEl, "IMG", imageID);

					if (item.isTextOnly()) {
						iEl.setAttribute("TEXT_ONLY", "T");
					}

					if (item.getLocationRegionID() != null && !item.getLocationRegionID().isEmpty()) {
						writeRef(iEl, "LOCATION-REGION", item.getLocationRegionID());
						writeRef(iEl, "LOCATION-TYPE", String.valueOf(item.getLocationTypeID()));
					}

					if (item.getNoteID() != null
							&& item.getNoteID().length() > 0) {
						writeRef(iEl, "NOTE", item.getNoteID());
					}

					writeInterval(iEl, item.interval());
					write(iEl, "ORDER", String.valueOf(++order));

					if (item.getText() != null && item.getText().length() > 0) {

						// String str = item.getText();
						// System.out.println("TEXT = [" + str + "]");

						iEl.appendChild(document.createTextNode(item.getText()));
					}
					eEl.appendChild(iEl);
				}
				
				// parent utterances of this entity
				Element glossWindowItemRefsEl = document.createElement("GLOSS-WINDOW-ITEM-REFS");

				ArrayList<String> parentEntityIDs = entity.getParentEntityIDs();
				if (parentEntityIDs != null && ! parentEntityIDs.isEmpty()) {
					for (String parentEntityID: parentEntityIDs) {
						Element glossWindowItemRefEl = document.createElement("REF");
						write(glossWindowItemRefEl, "GLOSS-WINDOW-ITEM_ID", parentEntityID);
						glossWindowItemRefsEl.appendChild(glossWindowItemRefEl);
					}
				}
				
				eEl.appendChild(glossWindowItemRefsEl);
				
				tEl.appendChild(eEl);
			}

			ArrayList<SS3Item> nonCompleteEventsList = track
					.getNonCompleteEvents();
			if (nonCompleteEventsList.size() > 0) {
				Element neEl = document.createElement("NON_COMPLETE_EVENTS");
				for (int n = 0; n < nonCompleteEventsList.size(); n++) {
					SS3Item item = nonCompleteEventsList.get(n);

					Element iEl = document.createElement("ITEM");

                    writeInterval(iEl, item.interval());

					String sf = item.isStartTimeFinal() ? "T" : "F";
					iEl.setAttribute("SF", sf);
					String ef = item.isEndTimeFinal() ? "T" : "F";
					iEl.setAttribute("EF", ef);
					if (item.getText() != null && item.getText().length() > 0) {
						iEl.appendChild(document.createTextNode(item.getText()));
					}

					writeRef(iEl, "COMP_START", "ENTITY", item.getStartCompoundEntityID());
					writeRef(iEl, "COMP_END", "ENTITY", item.getEndCompoundEntityID());

					neEl.appendChild(iEl);
				}
				tEl.appendChild(neEl);
			}

			ArrayList<SS3Item> enteredTxtList = track.getEnteredTextEvents();
			if (enteredTxtList.size() > 0) {
				Element etEl = document.createElement("ENTERED_TEXT");
				for (int t = 0; t < enteredTxtList.size(); t++) {
					SS3Item item1 = enteredTxtList.get(t);

					Element iEl = document.createElement("ITEM");

					writeInterval(iEl, item1.interval());

					if (item1.getText() != null && item1.getText().length() > 0) {
						iEl.appendChild(document.createTextNode(item1.getText()));
					}
					etEl.appendChild(iEl);
				}
				tEl.appendChild(etEl);
			}
			root.appendChild(tEl);
		}
		return root;
	}

	private Element createNotesElement(SS3Database db, Document document) {
		Element root = document.createElement("NOTES");
		ArrayList<SS3Note> notes = db.getNotesList();
		for (SS3Note note : notes) {
			Element nEl = document.createElement("NOTE");
			nEl.setAttribute("ID", note.getID());
			writeRef(nEl, "CODER", note.getCoderID());
			nEl.setAttribute("TIME_CREATED", note.getTimeCreated());
			nEl.setAttribute("TIME_UPDATED", note.getTimeUpdated());
			Element sEl = document.createElement("SUBJECT");
			sEl.appendChild(document.createTextNode(note.getSubject()));
			nEl.appendChild(sEl);
			Element bEl = document.createElement("BODY");
			bEl.appendChild(document.createTextNode(note.getBody()));
			nEl.appendChild(bEl);
			root.appendChild(nEl);
		}
		return root;
	}

	public void saveMediaLibrary(SS3MediaLibrary library) throws Exception {
		Document document = XMLWriter.createNewXMLDocument(library
				.getFileName());

		try {
			Element root = document.createElement("SIGNSTREAM-DATABASE");
//			Element distributorElement = document.createElement("DISTRIBUTOR");
//			distributorElement
//					.appendChild(document
//							.createTextNode("American Sign Language Linguistic Research Project, Boston University, Boston, MA."));
			// document.createTextNode(prf.getDistributor()));
//			root.appendChild(distributorElement);

			Element searchLocationElement = document
					.createElement("SEARCH_LOCATIONS");
			root.appendChild(searchLocationElement);

			int priority = 1;
			ArrayList<SerchableDirectory> searchLocations = library
					.getDirectoriesToSearch();
			for (SerchableDirectory directory : searchLocations) {
				Element searchElement = document
						.createElement("SEARCH_LOCATION");
				File dirFile = new File(directory.getDirectory());
				File baseDir = new File(System.getProperty("user.dir"));
				String relativizedPath = Util.relativizePath(baseDir, dirFile);
				searchElement.setAttribute("PATH", relativizedPath);
				String searchable = directory.isSearchable() ? "Y" : "N";
				searchElement.setAttribute("LOOKUP", searchable);
				searchElement.setAttribute("PRIORITY", "" + priority);
				int numberOfSearches = directory.isSearchable() ? directory
						.getNumberOfSearches() + 1 : directory
						.getNumberOfSearches();
				searchElement.setAttribute("SEARCH_COUNT", ""
						+ numberOfSearches);
				priority++;
				searchLocationElement.appendChild(searchElement);
			}

			XMLWriter.saveDocument(root, library.getFileName());

		} catch (Exception ex) {

			SS3SignStreamApplication.logger.log(Level.INFO,
					"Media library [" + library.getFileName()
							+ "] had NOT been saved. " + ex.getStackTrace());

			// ex.printStackTrace();

			throw new Exception("Media library [" + library.getFileName()
					+ "] had NOT been saved.");
		}
	}

	public boolean deleteDatabase(SS3Database db) {
		String directory = "SignStreamDatabases" + File.separator
				+ db.getDirectory();
		String fileName = db.getFileName();

		if (db.getReadFromXMLFile() != null) {
			fileName = db.getReadFromXMLFile().getPath();
		} else {
			fileName = directory + File.separator + db.getFileName();
		}

		File file = new File(fileName);
		return file.delete();
	}

	/**
	 * Saves the database to a temporary directory.
	 * Sets the database's file extension if it hasn't been set.
	 * If saving a database that has the extension "[...].ss3.ss3" you must set the file extension
	 * yourself before passing it to this function.
	 * @param db
	 * @return True if the files were saved to the temporary directory successfully and False
	 * otherwise.
	 * @throws Exception
	 */
	public boolean writeDatabase(SS3Database db) throws Exception {
		String dbDirectory = db.getParentDirectoryPath();

		String dir = db.getDirectory();

		if(!dir.startsWith(SS3Database.tempDirectoryPrefix))
    		dir = SS3Database.tempDirectoryPrefix + dir;
    	if(!dir.endsWith(SS3Database.dbFileExtension))
    		dir += SS3Database.dbFileExtension;
    	db.setDirectory(dir);
		
		String directory = dbDirectory + File.separator + dir;


		SS3SignStreamApplication.logger.log(Level.INFO, "Saving database ["
				+ directory + "]...");

		String collectionName = SS3Singleton.getSignStreamApplication().getTitle();

		File databasesFile = new File(dbDirectory);
		if (!databasesFile.exists()) {
			try {
				boolean createDirectory = databasesFile.mkdir();

				if (!createDirectory) {
					String[] str = new String[4];
					str[0] = "Collection " + collectionName
							+ " could not be saved ";
					str[1] = "because its parent directory could not be created.";
					str[2] = "Please make sure that only legal characters are used.";
					str[3] = "Also try saving the collection in a different directory.";

					SS3Singleton.getErrorMessages()
							.showSaveCollectionErrorMessage(str);
					return false;
				}
			} catch (SecurityException sEx) {
				String[] str = new String[4];
				str[0] = "Collection " + collectionName
						+ " could not be saved ";
				str[1] = "The Security manager does not permit the named";
				str[2] = "directory to be created.  Please verify write";
				str[3] = "permissions and try again.";

				SS3Singleton.getErrorMessages().showSaveCollectionErrorMessage(
						str);
				return false;
			}
		}

		File directoryFile = new File(directory);

		if (!directoryFile.exists()) {
			try {
				boolean createFile = directoryFile.mkdir();
				if (!createFile) {

					String[] str = new String[4];
					str[0] = "Collection " + collectionName
							+ " could not be saved ";
					str[1] = "because collection directory was not created. ";
					str[2] = "Please make sure that only legal characters had been used.";
					str[3] = "Also try saving the collection in different directory.";

					SS3Singleton.getErrorMessages()
							.showSaveCollectionErrorMessage(str);
					return false;
				}
			} catch (SecurityException sEx) {

				String[] str = new String[4];
				str[0] = "Collection " + collectionName
						+ " could not be saved. ";
				str[1] = "The Security manager does not permit the named";
				str[2] = "directory to be created.  Please verify write";
				str[3] = "permissions and try again.";

				SS3Singleton.getErrorMessages().showSaveCollectionErrorMessage(
						str);
				return false;
			}
		}

		String name0 = db.getFileName();

		int t = name0.indexOf(SS3Database.macroFileExtension);
		if (t == -1) {
			t = name0.indexOf(".xml");
		}

		String name = name0;
		if (t > -1) {
			name = name0.substring(0, t);
		}

		// check if filename is valid
		String newFileName = "";
		char[] chars = name.toCharArray();

		for (int i = 0; i < chars.length; i++) {
			char c = chars[i];
			if (Character.isLetterOrDigit(c)) {
				newFileName = newFileName + c;
			} else if (Character.isWhitespace(c)) {
				newFileName.trim();
				newFileName = newFileName + c;
			} else {
				String str = new String("" + c);
				if (str.equalsIgnoreCase("-") || str.equalsIgnoreCase("+")) {
					newFileName = newFileName + str;
				} else if (str.equalsIgnoreCase("#")) {
					newFileName = newFileName + "loan";
				} else if (str.equalsIgnoreCase(File.pathSeparator)) {
					newFileName = newFileName + "or";
				}
			}
		}

		String fileName = directory + File.separator + newFileName + SS3Database.macroFileExtension;

		File file = new File(fileName);

		if(file.exists()) {
			file.delete();
		}

		Document _doc = null;
		try {
			_doc = XMLWriter.createNewXMLDocument(fileName);
		} catch (Exception ex) {

			String[] str = new String[4];
			str[0] = "Collection " + collectionName + " could not be saved. ";
			str[1] = ex.getMessage();
			str[2] = "";
			str[3] = "";

			SS3Singleton.getErrorMessages().showSaveCollectionErrorMessage(str);
			return false;
		}
		// Make it final so it can be used in lambdas...
		var doc = _doc;

		try {
			SS3DBProfile prf = db.getProfile();
			Element root = doc.createElement("SIGNSTREAM-DATABASE");
			root.setAttribute("SIGNSTREAM-VERSION",
					"" + prf.getSignStreamVersion());
			root.setAttribute("DATABASE-VERSION", "" + prf.getDatabaseVersion());
			root.setAttribute("EXCERPT", db.getExcerpt());
			root.setAttribute("EXCERPT_NUMBER", db.getNumber());

			writeChildList(root, TIMEUNIT_LIST_TAG, db.getTimeUnits().stream().map(u -> createTimeUnitEl(doc, u)));
            writeChildList(root, TIMERANGE_LIST_TAG, db.getTimeRanges().stream().map(r -> createTimeRangeEl(doc, r)));

            root.appendChild(createMediaElement(db, doc));

			root.appendChild(createDistributorElement(prf, doc));
			root.appendChild(createReferencesElement(db, doc));
			// root.appendChild(createAuthorsElement(db, document));
			root.appendChild(createCodingSchemeElement(db, doc));
			root.appendChild(createDatabaseElement(db, doc));

			boolean savedCoolection = XMLWriter.saveDocument(root, fileName);
			if (!savedCoolection) {

				String[] str = new String[4];
				str[0] = "Collection " + collectionName
						+ " could not be saved. ";
				str[1] = "Please manually delete the directory and/or file";
				str[2] = "with the collection name and try again.";
				str[3] = "";

				SS3Singleton.getErrorMessages().showSaveCollectionErrorMessage(
						str);
				return false;
			}

			// showCollectionSavedMessage();
			return true;
		} catch (Exception ex) {
			SS3SignStreamApplication.logger.log(Level.INFO, "Collection ["
					+ dbDirectory
					+ "] had NOT been saved. Error in Macro Unit [" + fileName
					+ "]" + ex.getStackTrace());

			String[] str = new String[4];
			str[0] = "Collection " + collectionName + " could not be saved. ";
			str[1] = ex.getMessage();
			str[2] = "";
			str[3] = "";

			SS3Singleton.getErrorMessages().showSaveCollectionErrorMessage(str);
			ex.printStackTrace();
			return false;
		}
	}

	private JPopupMenu collectionSavedPopup = null;
	private JLabel label = new JLabel("Collection had been saved.");

	private void showCollectionSavedMessage() {

		if (collectionSavedPopup != null) {
			UserInterfaceUtil.setLabelLNF(label);
			collectionSavedPopup = new JPopupMenu();

			final JButton okBtn = new JButton("OK");

			JPanel msgPanel = new JPanel();
			GridBagConstraints gbc = new GridBagConstraints();

			msgPanel.setLayout(new GridBagLayout());
			gbc.anchor = GridBagConstraints.WEST;

			gbc.gridwidth = 3;
			gbc.gridx = 0;
			gbc.gridy = 0;
			msgPanel.add(label, gbc);

			gbc.gridy = 1;
			gbc.gridx = 0;
			msgPanel.add(new JLabel(" "), gbc);

			gbc.gridwidth = 1;
			gbc.anchor = GridBagConstraints.EAST;
			gbc.gridy = 3;
			gbc.gridx = 0;
			msgPanel.add(new JLabel(" "), gbc);
			gbc.gridx = 2;
			msgPanel.add(okBtn, gbc);

			okBtn.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					collectionSavedPopup.setVisible(false);
				}
			});

			SwingUtilities.invokeLater(new Runnable() {
				@Override
				public void run() {
					okBtn.requestFocus();
				}
			});

			Dimension d = new Dimension();
			int width = msgPanel.getPreferredSize().width + 30;
			int height = msgPanel.getPreferredSize().height + 20;

			d.setSize(width, height);

			collectionSavedPopup.add(msgPanel);
			collectionSavedPopup.setPopupSize(d);
		}

		JPanel panel = SS3Singleton.getSignStreamApplication()
				.getSignStreamSegmentPanel().getUtteranceView();
		// popup.show(field.getSignStreamSegmentPanel().getUtteranceView(), 250,
		// 250);
		if (panel != null) {
			panel = SS3Singleton.getSignStreamApplication()
					.getSignStreamSegmentPanel().getDatabaseView();
		}
		collectionSavedPopup.show(panel, 250, 250);
		collectionSavedPopup.setVisible(true);
	}
}
