package edu.bu.signstream.common.tree;

import edu.bu.signstream.common.util.Pair;
import edu.bu.signstream.common.util.Util;

import javax.swing.tree.DefaultMutableTreeNode;
import java.util.stream.Stream;

public class Node<T> implements Transferable {
    public final T value;

    private Node<T> parent;
    private final DefaultMutableTreeNode node;

    public Node(T value, boolean leaf) {
        this.value = value;
        this.node = new DefaultMutableTreeNode(new NodeWrapper<>(this), !leaf);
    }
    public Node(T value) { this(value, false); }

    DefaultMutableTreeNode node() { return node; }


    public Node<T> add(Node<T> child, int i) {
        if (child.parent != null)
            child.parent.node().remove(child.node());

        child.parent = this;
        if (i == -1 || i > node.getChildCount())
            node.add(child.node());
        else
            node.insert(child.node(), i);

        return this;
    }
    public Node<T> add(Node<T> child) {
        if (child.parent != null)
            child.parent.node().remove(child.node());

        node.add(child.node());
        child.parent = this;

        return this;
    }

    public Node<T> parent() { return parent; }
    public int index() { return parent.node().getIndex(node); }

    public Stream<Node<T>> children() {
        return Util.stream(node.children()).map(NodeWrapper::node);
    }
    public Stream<Node<T>> dfs() {
        return Stream.concat(Stream.of(this), children().flatMap(Node::dfs));
    }


    @Override
    public boolean equals(Object obj) {
        return obj instanceof Node && value.equals(((Node)obj).value);
    }

    @Override
    public Class<? extends Transferable> clazz() { return getClass(); }
}
