package edu.bu.signstream.common.util;

import java.awt.*;
import java.awt.geom.AffineTransform;
import java.io.File;
import java.io.IOException;
import java.net.URLDecoder;
import java.util.*;
import java.util.stream.Stream;
import java.util.stream.StreamSupport;

import edu.bu.signstream.common.files.SSResources;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.time.Duration;
import edu.bu.signstream.time.TimeRange;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;

public class Util {
    private static TimeRange SELECTED_RANGE = null;

	/*
	 * these would be how many of the current units to move in
	 */
	public enum UnitStep {
	    DEFAULT(0),
		ONE(1),
		TEN(10),
		ONE_HUNDRED(100),
		ONE_THOUSAND(1000),
		TEN_THOUSAND(10000),
		ONE_HUNDRED_THOUSAND(100000),
        ONE_MILLION(1000000),
        TEN_MILLION(10000000),
        ONE_HUNDRED_MILLION(100000000);

		public final int value;
		UnitStep(int value) { this.value = value; }
	}
	
	public static UnitStep CURRENT_UNIT_STEP = UnitStep.DEFAULT;

	private static int counter;
	private static int INTEGER_MAX_VALUE = Integer.MAX_VALUE;
	
	public static final String DOM_GLOSS_FIELD_ID = "10000";
	public static final String NDOM_GLOSS_FIELD_ID = "10001";

	public static final String DOM_HAND_FIELD_ID = "10501";
	public static final String NDOM_HAND_FIELD_ID = "10502";

	public static final String DOM_HANDSHAPE_FIELD_ID = "1000";
	public static final String NDOM_HANDSHAPE_FIELD_ID = "2000";

	public static final String DOM_LOCATION_FIELD_ID = "25000";
	public static final String NDOM_LOCATION_FIELD_ID = "25001";
	
	public static final String DOM_SIGN_TYPE_FIELD_ID = "26000";
	public static final String NDOM_SIGN_TYPE_FIELD_ID = "26001";
	public static final String FREE_TEXT_FIELD_ID = "30000";
	
	public static final String DOM_GESTURE_FIELD_ID = "27001";
	public static final String NDOM_GESTURE_FIELD_ID = "27002";
	
	public static final String DOM_GESTURE_APEX_FIELD_ID = "27003";
	public static final String NDOM_GESTURE_APEX_FIELD_ID = "27004";

	public static final String DOM_GESTURE_HAND_FIELD_ID = "27005";
	public static final String NDOM_GESTURE_HAND_FIELD_ID = "27006";

	public static final String DOM_GESTURE_HANDSHAPE_FIELD_ID = "27007";
	public static final String NDOM_GESTURE_HANDSHAPE_FIELD_ID = "27008";
	
	public static final String DOM_GESTURE_ORIENTATION_FIELD_ID = "27009";
	public static final String NDOM_GESTURE_ORIENTATION_FIELD_ID = "27010";

	public static final String DOM_GESTURE_LOCATION_FIELD_ID = "75000";		// TODO: this should be removed
	public static final String NDOM_GESTURE_LOCATION_FIELD_ID = "75001";		// TODO: this should be removed
	
	public static final String DOM_GESTURE_TYPE_FIELD_ID = "27011";
	public static final String NDOM_GESTURE_TYPE_FIELD_ID = "27012";
	public static final String GESTURE_FREE_TEXT_FIELD_ID = "80000";		// TODO: this should be removed
	
	public static final String SPEECH_WORDS_FIELD_ID = "90000";
	public static final String SPEECH_SYLLABLES_FIELD_ID = "90001";
	public static final String SPEECH_PHONES_FIELD_ID = "90002";
	public static final String SPEECH_POINTS_FIELD_ID = "90003";
	
	public static final String RIGHT_HAND = "R";
	
	public static int MILLISECONDS_WAIT_LISTENERS = 300;

	public static final String VIDEO_FILE_MOV = "mov";
	public static final String VIDEO_FILE_MPG = "mpg";
	public static final String VIDEO_FILE_AVI = "avi";
	
	public final static char SEGMENT_TIER_SEPARATOR = ':';
	
	public final static String GRAPH_FILE_DELIMITER_LINE = ",";
	public final static String GRAPH_FILE_DELIMITER_ITEM = ":";
	public final static int GRAPH_FILE_LINE_ITEM_COUNT = 4;
	public final static int GRAPH_FILE_ITEM_COUNT = 2;
	public final static String FIELD_NAME = "field_name";
	public final static String FIELD_VALUE = "field_value";
	public final static String CSV_REGEX_PATTERN = ",(?=(?:[^\"]*\"[^\"]*\")*[^\"]*$)";

	public static final String SIGN_BANK_DIR = SSResources.signbankPath;
	public static final String SIGN_BANK_PRIORITIES_FILE = "priorityList.xml";
	public static final String SIGN_BANK_DEFAULT_LOCAL_FILE = "localSignBank.xml";
	public static final String SIGN_BANK_DEFAULT_GLOBAL_FILE = "globalSignBank.xml";
	
	public static int MTB_TEXT_BUFFER = 5;
    
    public static final String DNI_HANDSHAPE_IMAGE_ID = "HSH_999";
    
    public static final String DNI_ORIENTATION_IMAGE_ID = "ORN_999";
    
    public static final String VALIDATOR_CODES_DELIMITER = "---";
    
    public static final String EXTRA_REGEX_CHARACTERS = "!@#$%^&*()\\-_+={}\\[\\]\\\\|;:'\",./<>? ";	// the last character is an intentional space
    public static final String LABEL_REGEX_PATTERN = "^[a-zA-Z0-9" + EXTRA_REGEX_CHARACTERS + "]+$";
   
   private static SearchPaths searchPaths = new SearchPaths();

   private static boolean dragging = false;
   public static void dragging(boolean isDragging) { dragging = isDragging; }
   public static boolean dragging() { return dragging; }

   public static void setSelectedRange(TimeRange range) {
       SELECTED_RANGE = range;
   }

   public static TimeRange getSelectedRange() {
       return SELECTED_RANGE;
   }

    public static TimeRange getCurrentRange(Field field, TimeRange def) {
        var bar = SS3Singleton.getMediaToolBar(SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel());

        if (SELECTED_RANGE != null)
            return SELECTED_RANGE;
        else if (field != null)
            return field.getWrapper().timeRange();
        else
            return def;
    }

    public static TimeRange getCurrentRange(Field field) {
        return getCurrentRange(field, SS3Singleton.getVideoTimeRange());
    }

    public static TimeRange getCurrentRange() {
        return getCurrentRange(SS3Singleton.getMediaToolBar().selectedField());
    }

    public static TimeRange getCurrentRange(TimeRange def) {
        return getCurrentRange(null, def);
    }

    public static UnitStep getUnitStep() {
       return CURRENT_UNIT_STEP;
    }

    public static void setUnitStep(UnitStep unitStep) {
        CURRENT_UNIT_STEP = unitStep;
    }

    public static void nextUnitStep() {
       var steps = Arrays.asList(UnitStep.values());
       var next = steps.indexOf(CURRENT_UNIT_STEP) + 1;
       CURRENT_UNIT_STEP = steps.get(next % steps.size());
    }

    public static void prevUnitStep() {
        var steps = Arrays.asList(UnitStep.values());
        var next = steps.indexOf(CURRENT_UNIT_STEP) - 1;
        // Offset by size to handle negative wrap
        CURRENT_UNIT_STEP = steps.get((next + steps.size()) % steps.size());
    }

    public static Duration getStep() {
       return getStep(Util.getCurrentRange());
   }

    public static Duration getStep(TimeRange range) {
//        var bar = SS3Singleton.getMediaToolBar();
//        if (bar.selectedEvent() != null)
//            range = bar.selectedEvent().getStartTimeInfo().range();
//        else if (bar.activeField() != null)
//            range = bar.activeField().timeRange();

        if (CURRENT_UNIT_STEP == UnitStep.DEFAULT)
            return new Duration(range.step(), range.unit());
        else
            return new Duration(CURRENT_UNIT_STEP.value, range.unit());
    }

    public static Duration getStepInRange(TimeRange range) {
        if (CURRENT_UNIT_STEP == UnitStep.DEFAULT)
            return new Duration(range.step(), range.unit());
        else
            return new Duration(CURRENT_UNIT_STEP.value, range.unit());
    }

    public static TimeRange getStepRange() {
        var bar = SS3Singleton.getMediaToolBar();
        if (bar.selectedEvent() != null && bar.selectedEvent().getStartTimeInfo() != null)
            return bar.selectedEvent().getStartTimeInfo().range();
        else if (bar.activeField() != null)
            return bar.activeField().timeRange();
        else
            return Util.getCurrentRange();
    }
    
   public static SearchPaths getSearchPaths() {
      return searchPaths;    
   }

   public static String getCurrentDayNTime() {
	      Calendar c = Calendar.getInstance();
	      StringBuffer sb = new StringBuffer();
	      sb.append(c.get(Calendar.MONTH));
	      sb.append("/");
	      sb.append(c.get(Calendar.DAY_OF_MONTH));
	      sb.append("/");
	      sb.append(c.get(Calendar.YEAR));
	      sb.append(" ");
	      sb.append(c.get(Calendar.HOUR_OF_DAY));
	      sb.append(":");
	      sb.append(c.get(Calendar.MINUTE));
	      int am = c.get(Calendar.AM_PM);
	      if (am == 0) {
	         sb.append(" AM");
	      } else {
	         sb.append(" PM");
	      }
	      return sb.toString();
   }
   
   public static boolean isNotEmpty(String str) {
      return str != null && str.trim().length() > 0;
   }

   @Deprecated
	public static String getUniqueNumber() {
		Calendar calendar = Calendar.getInstance();
		counter++;
		
		long longtime = calendar.getTimeInMillis() + counter;
		String timeStr;
		String substr;
		
		// keep truncating the long until its less than INTEGER_MAX_VALUE
		do {	
			timeStr = "" + longtime;
			substr = timeStr.substring(5);
			longtime = Long.parseLong(substr);
			substr = "" + longtime;		// re-cast to a string in case parseLong chopped off any leading zero's
		} while (longtime > INTEGER_MAX_VALUE);
		
		return substr;
	}

	@Deprecated
	public static String getLongUniqueNumber() {
		Calendar calendar = Calendar.getInstance();
		// do not remove counter!!! Should be here to ensure that number is unique
		counter++;
		return "" + calendar.getTimeInMillis() + counter;
	}

	public static String uuid() {
       return UUID.randomUUID().toString();
    }

	public static ArrayList<String> getAdjustedLabels(String str,
			FontMetrics fontMetrics, int maxLength) {
		ArrayList<String> list = new ArrayList<String>();

		String str0 = str;
		String str1 = "";

		while (str0.length() > 1) {
			str1 = getAdjustedLabel(str0, fontMetrics, maxLength);
			list.add(str1);
			str0 = str0.substring(str1.length());
		}

		return list;
	}

	public static String getAdjustedLabel(String str, FontMetrics fontMetrics,
			int maxLength) {

		if (str == null)
			return "";

		str = str.trim();

		if (fontMetrics.stringWidth(str) <= maxLength)
			return str;

		String strToDraw = str;

		while (fontMetrics.stringWidth(strToDraw) > maxLength) {
//		- fontMetrics.stringWidth("...")) {
			if (strToDraw.length() > 1) {
				strToDraw = strToDraw.substring(0, strToDraw.length() - 1);
			} else {
				break;
			}
		}

		return strToDraw;
	}

	public static boolean str2Bool(String str) {
		return str.equalsIgnoreCase("true");
	}

	public static String color2HexString(Color color) {
		StringBuffer sbColor = new StringBuffer();

		sbColor.append(toHexString(color.getRed()));
		sbColor.append(toHexString(color.getGreen()));
		sbColor.append(toHexString(color.getBlue()));

		return sbColor.toString();
	}

	private static String toHexString(int i) {
		if (i > 0xFF || i < 0)
			throw new RuntimeException(
					"Color component is outside expected range: " + i);
		if (i <= 0x0F)
			return "0" + Integer.toHexString(i);
		else
			return Integer.toHexString(i);
	}

	public static Color hexString2Color(String hexColor) {
		if (hexColor.length() < 6)
			return Color.BLACK;
		// throw new RuntimeException("Color component is outside expected
		// range: 6");

		String red = hexColor.substring(0, 2);
		String green = hexColor.substring(2, 4);
		String blue = hexColor.substring(4, 6);

      Color color = Color.BLACK;
      
      try {
         color = new Color(Integer.parseInt(red, 16), Integer.parseInt(green, 16), Integer.parseInt(blue, 16));
      } catch(Exception ex) {
         ;
      }

		return color;
	}
	
	public static boolean isXML3File(String fileName) {
		var s = fileName.lastIndexOf(".3.xml");
		
		return (s > -1);
	}

	public static boolean isXMLFile(String fileName) {
		String ext = getExtension(fileName);

		if ("xml".equalsIgnoreCase(ext))
			return true;
		return false;
	}

	public static boolean isMovieFile(File file) {
		String ext = getExtension(file);

		if (VIDEO_FILE_MOV.equals(ext) || VIDEO_FILE_MPG.equals(ext)
				|| VIDEO_FILE_AVI.equals(ext))
			return true;
		return false;
	}

	public static boolean isMovieFile(String fileName) {
		String ext = getExtension(fileName);

		if (VIDEO_FILE_MOV.equals(ext) || VIDEO_FILE_MPG.equals(ext)
				|| VIDEO_FILE_AVI.equals(ext))
			return true;
		return false;
	}

	/*
	 * Get the extension of a file.
	 */
	public static String getExtension(String fileName) {
		String ext = null;
		String s = getName(fileName);
		int i = s.lastIndexOf('.');

		if (i > 0 && i < s.length() - 1) {
			ext = s.substring(i + 1).toLowerCase();
		}
		return ext;
	}

	public static String getExtension(File file) {
		String ext = null;
		String s = file.getName();
		int i = s.lastIndexOf('.');

		if (i > 0 && i < s.length() - 1) {
			ext = s.substring(i + 1).toLowerCase();
		}
		return ext;
	}

	public static boolean isTextFile(String fileName) {
		String extension = getExtension(fileName);
		return "txt".equals(extension);
	}

	public static String getName(String fileName) {
		File file = new File(fileName);
		return file.getName();
	}

	public static String getNameNoExtension(String fileName) {
		String name = getName(fileName);
		String extension = getExtension(fileName);
		int s = name.length() - extension.length() - 1;
		String fileNameNoExtension = name.substring(0, s);
		return fileNameNoExtension;
	}

	public static boolean isBinaryFile(String fileName) {
		if (isMovieFile(fileName) && isTextFile(fileName))
			return true;

		return false;
	}

	public static boolean removeDirectory(String dirName) {
		File dir = new File(dirName);
		File[] children = dir.listFiles();

		// remove files
		if (children != null) {
			for (int i = 0; i < children.length; i++) {
				File file = children[i];
				if (file.isDirectory()) {
					removeDirectory(file.getAbsolutePath());
				} else {
					file.delete();
				}
			}
		}
		dir.delete();
		return true;
	}

	/**
	 * Given a base directory, returns the path of newDir relative to
	 * the baseDir.
	 * @param baseDir Directory to which the returned path should be relative
	 * @param newDir Directory whose path is to be relativized
	 * @return the path of newDir relative to the baseDir.
	 */
	public static String relativizePath(File baseDir, File newDir) {
		try {
			if (baseDir.getCanonicalPath().equals(newDir.getCanonicalPath())) {
				return ".";
			}

			String _relativized = baseDir.toURI().relativize(newDir.toURI()).toString();
			if (_relativized.startsWith("file:")) {
				_relativized = _relativized.substring(5);
				_relativized = URLDecoder.decode(_relativized, "UTF-8");
			}
			String relativePath = new File(_relativized).getPath();
			if (relativePath.equals(newDir.getAbsolutePath())) {
				String absPath = baseDir.getAbsolutePath();
				int pos = 0;
				for (int i = 0; i < Math.min(relativePath.length(), absPath.length()); i++) {
					if (relativePath.charAt(i) != absPath.charAt(i)) {
						while (relativePath.charAt(i - 1) != File.separatorChar) {
							i--;
						}
						pos = i;
						break;
					}
				}
				absPath = absPath.substring(pos);
				relativePath = relativePath.substring(pos);

				int numBackup = 1;
				for (char c : absPath.toCharArray()) {
					if (c == File.separatorChar)
						numBackup++;
				}

				for (int i = 0; i < numBackup; i++) {
					relativePath = "../" + relativePath;
				}

				return relativePath;
			}
			return relativePath;
		} catch (IOException e) {
			return null;
		}
	}
   
   public static boolean isTrue(String y) {
      return (y.equalsIgnoreCase("y") || y.equalsIgnoreCase("yes") || y.equalsIgnoreCase("true") || y.equalsIgnoreCase("1") || y.equalsIgnoreCase("t"));
   }
   
   public static String getBooleanAsString(boolean b) {
      if(b) {
         return "T";
      }
      return "F";
   }

   public static void flipHorizontal(int width, Graphics2D g) {
	   AffineTransform flipTransform = AffineTransform.getScaleInstance(-1, 1);
	   flipTransform.translate(-width, 0);
	   g.transform(flipTransform);
   }
   

	/**
	 * Checks for a match between occurrenceLabel and any in the list of SS3FieldValue objects.
	 * Intended for use only by clients of sign bank classes/functionality.
	 * @param fieldValues The field values taken from one field of the coding schema
	 * @param occurrenceLabel The label from the occurrence to compare to the coding schema labels
	 * @return SS3FieldValue The SS3FieldValue whose label matches occurrenceLabel, or null if none
	 */
	public static SS3FieldValue matchSignBank(ArrayList<SS3FieldValue> fieldValues, String occurrenceLabel) {
		
		// the coding scheme does not find the "big-" images in the directory, so use the others of similar name
		// hack for now
		String alternativeHSImageTag = "big-";
		int tagSize = alternativeHSImageTag.length();
		
		if (occurrenceLabel.length() > tagSize && occurrenceLabel.substring(0, tagSize).equals(alternativeHSImageTag)) {
			occurrenceLabel = occurrenceLabel.substring(tagSize);
		}
		
		// exact match check
		for (SS3FieldValue value: fieldValues) {
			String valueLabel = value.getLabel();

			if (valueLabel != null && !valueLabel.isEmpty()) {	
				if (valueLabel.equals(occurrenceLabel)) {
					return value;
				}
			}
		}
		
		// case where labels have a slash
		for (SS3FieldValue value: fieldValues) {
			String valueLabel = value.getLabel();
			
			if (valueLabel != null && !valueLabel.isEmpty()) {			
				
				if (valueLabel.contains("/")) {
					int index = valueLabel.indexOf("/");
					String valueLabel1 = null, valueLabel2 = null;
					valueLabel1 = valueLabel.substring(0, index);
					valueLabel2 = valueLabel.substring(index + 1);
					
					if (valueLabel1.equals(occurrenceLabel) || valueLabel2.equals(occurrenceLabel)) {
						return value;
					}
				}
				
				if (occurrenceLabel.contains("/")) {
					int index = occurrenceLabel.indexOf("/");
					String occurrenceLabel1 = null, occurrenceLabel2 = null;
					occurrenceLabel1 = occurrenceLabel.substring(0, index);
					occurrenceLabel2 = occurrenceLabel.substring(index + 1);
					
					if (valueLabel.equals(occurrenceLabel1) || valueLabel.equals(occurrenceLabel2)) {
						return value;
					}
				}
			}			
		}
		
		return null;
	}
	
	/**
	 * Checks for a match between schemaLabel and occurrenceLabel
	 * Intended for use only by clients of sign bank classes/functionality.
	 * @param schemaLabel The label based on the coding schema, which could contain two labels separated by a slash (e.g. "F/9")
	 * @param occurrenceLabel The label from the occurrence to compare to the coding schema labels
	 * @return boolean whether the labels match or schemaLabel contains occurrenceLabel
	 */
	public static boolean matchSignBank(String schemaLabel, String occurrenceLabel) {
		
		if (schemaLabel != null && !schemaLabel.isEmpty()) {			
			if (schemaLabel.contains("/")) {
				int index = schemaLabel.indexOf("/");
				String valueLabel1 = null, valueLabel2 = null;
				valueLabel1 = schemaLabel.substring(0, index);
				valueLabel2 = schemaLabel.substring(index + 1);
				
				if ( !valueLabel1.equals(occurrenceLabel) && !valueLabel2.equals(occurrenceLabel) && !schemaLabel.equals(occurrenceLabel)) {
					return false;
				}
			} else {
				if ( !schemaLabel.equals(occurrenceLabel)) {
					return false;
				}
			}
		} else {
//			return false;
		}
		
		return true;
	}
	
	public static String trimAsterisksAndPlusSigns(String input) {
		if (input == null)
			return null;

		String output = input;
		
		//Remove leading + and * symbols
		while(output.length() > 0 && (output.charAt(0) == '+' || output.charAt(0) == '*')) {
			output = output.substring(1);
		}
		
		//Remove trailing + and * symbols
		while (output.length() > 0 && (output.charAt(output.length()-1) == '+' || output.charAt(output.length()-1) == '*')) {
			output = output.substring(0, output.length()-1);
		}
				
		return output;
	}

    public static String basename(String fileName) {
        return fileName.replaceFirst("[.][^.]+$", "");
    }

    public static <T> Stream<T> stream(Enumeration<T> e) {
	    return StreamSupport.stream(Spliterators.spliteratorUnknownSize(e.asIterator(), Spliterator.ORDERED), false);
    }
}
