package edu.bu.signstream.common.util.signbank;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import edu.bu.signstream.ui.SS3SignStreamApplication;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import edu.bu.signstream.common.files.SSResources;
import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.Version;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.signbank.SignBankCollection;
import edu.bu.signstream.grepresentation.view.signbank.SignBankMainGloss;
import edu.bu.signstream.grepresentation.view.signbank.SignBankOccurrence;
import edu.bu.signstream.grepresentation.view.signbank.SignBankVariant;

public class SignBankParser {

	public static Logger logger = Logger.getLogger("edu.bu.signstream.ui");
	
	public SignBankCollection parseSignBank(JFrame parent, boolean showVersionError) {

		SignBankResourceCollection resourceCollection = createSignBankResourceCollection();
		SignBankCollection collection = createSignBankCollection(parent, resourceCollection, showVersionError);

		logger.log(Level.INFO, "Finished parsing sign bank.");
		
		return collection;
	}
	
	private SignBankResourceCollection createSignBankResourceCollection() {
		SignBankResourceCollection resourceCollection = new SignBankResourceCollection();
		ArrayList<SignBankResource> resourcesList = new ArrayList<SignBankResource>();
		
		// first getting the File path to "/signbank" in the deployment directory
		File signBankDir = new File(Util.SIGN_BANK_DIR);

		logger.log(Level.INFO, "Accessing sign bank directory: " + signBankDir);
		
		if (signBankDir.exists() && signBankDir.isDirectory()) {
			// first get the priorities file and make sure that it exists
			String prioritiesListStringPath = Util.SIGN_BANK_DIR + "/" + Util.SIGN_BANK_PRIORITIES_FILE;
			File prioritiesListFile = new File(prioritiesListStringPath);
			
			if ( !prioritiesListFile.exists()) {
				logger.log(Level.INFO, "Priorities file does not exist: " + prioritiesListFile);
				// BREAK, file should exist
			}

			logger.log(Level.INFO, "Reading priorities file: " + prioritiesListFile);

			// process the priorities file and add each filename to an arraylist to preserve order
			ArrayList<String> signBankFileNamesByPriority = new ArrayList<String>(); 
			
			DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder docBuilder = null;
			Document doc = null;
			try {
				docBuilder = docFactory.newDocumentBuilder();
				doc = docBuilder.parse(prioritiesListFile);
			} catch (ParserConfigurationException e2) {
				e2.printStackTrace();
				SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", e2);
			} catch (SAXException e2) {
				e2.printStackTrace();
				SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", e2);
			} catch (IOException e2) {
				e2.printStackTrace();
				SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", e2);
			}
			
			logger.log(Level.INFO, "Parsing priorities file: " + prioritiesListFile);
			
			NodeList priorityFilesNodeList = doc.getElementsByTagName("PRIORITY_FILES");
			Element priorityFilesElement = (Element) priorityFilesNodeList.item(0);
			if (priorityFilesElement == null)
				return null;
			NodeList fileNodeList = priorityFilesElement.getElementsByTagName("FILE");
			for (int i = 0; i < fileNodeList.getLength(); i++) {
				Element fileElement = (Element) fileNodeList.item(i);
				String fileName = fileElement.getAttribute("NAME").trim();
				String priority = fileElement.getAttribute("PRIORITY");
				signBankFileNamesByPriority.add(fileName);
			}
			
			logger.log(Level.INFO, "Creating resource collection from file list: " + signBankFileNamesByPriority);

			// loop through each xml file found within signBankDir and create a resource obj out of it
			for (String signBankFileName: signBankFileNamesByPriority) {
				if ( !signBankFileName.endsWith(".xml")) {
					continue;
				}

				String stringPath = Util.SIGN_BANK_DIR + "/" + signBankFileName;
				File file = new File(stringPath);

				logger.log(Level.INFO, "Trying to access resource from file: " + file);
				
				if ( !file.exists()) {
					continue;
					// should exist - if not, then the priorities file listed a file that wasn't actually there, somehow
				}

				logger.log(Level.INFO, "Creating sign bank resource from file: " + file);
				
				
				SignBankResource resource = new SignBankResource(signBankFileName, file.getAbsolutePath());
				resourcesList.add(resource);

				logger.log(Level.INFO, "Created sign bank resource from file: " + file);
			}
			
			resourceCollection.setResourcesList(resourcesList);	// set the resource collection to this ordered list of resources
		}
		
		logger.log(Level.INFO, "Finished creating sign bank resource collection.");
		
		return resourceCollection;
	}
	
	/**
	 * Merge the incoming list of SignBankResource objects into one DOM object
	 * @param resources The list of SignBankResource objects sorted by priority
	 * @return
	 */
	private Document createSignBanksDocument(JFrame parent, SignBankResourceCollection resourceCollection, boolean showVersionError)
			throws ParserConfigurationException, SAXException, IOException {
		
		ArrayList<SignBankResource> resources = resourceCollection.getResourcesList();	// list of resources sorted by priority high to low
		ArrayList<SignBankResource> updatedResourcesList = new ArrayList<SignBankResource>();
		
		Collections.reverse(resources);	// resources is now sorted starting at lowest priority and ending in highest priority
		// we do this so that the last document added to the group is the highest priority and overwrites all the others
		// GREG TODO: does the above need to be done this way?
		
		// ensuring the sign bank files we have are parse-able
		String signbankMinVersionStr = null;
		try {
			Properties properties = new Properties();
			InputStream input = new FileInputStream(SSResources.infoPath);
			properties.load(input);
			signbankMinVersionStr = properties.getProperty("signbank_schema_min_version");
		} catch (IOException e) {
			e.printStackTrace();
		}

		String signbankMaxVersionStr = null;
		try {
			Properties properties = new Properties();
			InputStream input = new FileInputStream(SSResources.infoPath);
			properties.load(input);
			signbankMaxVersionStr = properties.getProperty("signbank_schema_max_version");
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		Version signBankMinVersion = new Version(signbankMinVersionStr);
		Version signBankMaxVersion = new Version(signbankMaxVersionStr);

		DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
		DocumentBuilder docBuilder = docFactory.newDocumentBuilder();
		Document doc = docBuilder.newDocument();
		Element rootSetElement = doc.createElement("SIGNBANKS");
		Node rootSetNode = doc.appendChild(rootSetElement);

		File resourceFile = null;
		Document document = null;
		Node tempNode = null;
		for (SignBankResource resource: resources) {
			resourceFile = new File(resource.getSignBankAbsolutePath());
			try {
				document = docBuilder.parse(resourceFile);
			} catch (SAXParseException e) {
				System.out.println("Could not parse resource file: " + resourceFile);
				System.out.println("Skipping resource file: " + resourceFile);
				
				continue;
			}
			// tag each signbank element with the resource filename, then get it later and pass it to the occurrence objects
			tempNode = doc.importNode(document.getElementsByTagName("SIGNBANK").item(0), true);
			String resourceFileName = resource.getSignBankFileName();
			( (Element) tempNode).setAttribute("RESOURCE_FILENAME", resourceFileName);
			
			String signBankCurrentVersionStr = null;

			// compare min, max, and current versions 
			boolean signBankVersionTooHigh = false;
			boolean signBankVersionTooLow = false;
			boolean cannotParseSignBank = false;
			
			var schemaVersion = tempNode.getAttributes().getNamedItem("SCHEMA_VERSION");
			if (schemaVersion != null) {
				var nodeValue = schemaVersion.getNodeValue();
				if ( ! nodeValue.isBlank())
					signBankCurrentVersionStr = schemaVersion.getNodeValue();

				Version signBankCurrentVersion = null;
				if (signBankCurrentVersionStr == null) {
					signBankCurrentVersion = new Version(signbankMinVersionStr);
				} else {
					signBankCurrentVersion = new Version(signBankCurrentVersionStr);
				}
				
				if (signBankCurrentVersion.greaterThanCompatible(signBankMaxVersion)) {
					cannotParseSignBank = true;
					signBankVersionTooHigh = true;
				} else if (signBankCurrentVersion.lessThanCompatible(signBankMinVersion)) {
					cannotParseSignBank = true;
					signBankVersionTooLow = true;
				}
			}
			
			// showVersionError is only true on the call from SS3App main
			if (cannotParseSignBank && showVersionError) {
				String failure = "Your current version of SignStream is unable to parse the installed " + resourceFileName;
				
				String reason = null;
				
				if (signBankVersionTooHigh)
					reason = "Reason: the " + resourceFileName + " is too new. Upgrade SignStream to use this sign bank.";

				if (signBankVersionTooLow) {
					reason = "Reason: the " + resourceFileName + " is too old. Upgrade the SignBank in order to use it.";
					
					// TODO: in the future, for local sign bank file, this will need to operate with a "Converter" class
					// similar to how it is done for collection files - see DatabaseDelegate.addFile() for more details.
//					if (resourceFileName.equals(Util.SIGN_BANK_DEFAULT_LOCAL_FILE)) {
//						// TODO: converter
//					}
					
				}
				
				showPopup(parent, failure, reason);
				
			} else if ( ! cannotParseSignBank){
				// add resource back to resource list
				updatedResourcesList.add(resource);
				
				rootSetNode.appendChild(tempNode);
			}
		}

		resourceCollection.setResourcesList(updatedResourcesList);	// update the resource collection to this potentially lessened list of resources
		
		logger.log(Level.INFO, "Ready to parse sign bank.");

		return doc;
	}
	
	private void showPopup(JFrame parent, String failure, String reason) {
		JLabel failureLabel = new JLabel(failure);
		JLabel reasonLabel = new JLabel(reason);
		
		JDialog dialog = new JDialog(parent, true);
		dialog.setTitle("Warning message");
		JPopupMenu popup = new JPopupMenu();
		
		final JButton okBtn = new JButton("OK");

		JPanel msgPanel = new JPanel();
		GridBagConstraints gbc = new GridBagConstraints();

		msgPanel.setLayout(new GridBagLayout());
		gbc.anchor = GridBagConstraints.WEST;
		gbc.gridwidth = 3;
		gbc.gridx = 0;
		gbc.gridy = 0;
		msgPanel.add(new JLabel(" "), gbc);
		gbc.gridy = 1;
		msgPanel.add(failureLabel, gbc);

		gbc.gridy = 2;
		gbc.gridx = 0;
		msgPanel.add(reasonLabel, gbc);
		
		gbc.gridy = 3;
		gbc.gridx = 2;
		gbc.anchor = GridBagConstraints.EAST;
		msgPanel.add(new JLabel(" "), gbc);
		gbc.gridy = 4;
		msgPanel.add(okBtn, gbc);
		gbc.gridy = 5;
		msgPanel.add(new JLabel(" "), gbc);		

		okBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				dialog.setVisible(false);
			}
		});

		SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				okBtn.requestFocus();
			}
		});

		Dimension d = new Dimension();
		int width = msgPanel.getPreferredSize().width + 30;
		int height = msgPanel.getPreferredSize().height + 20;

		d.setSize(width, height);
		
		dialog.setMinimumSize(d);
		dialog.setPreferredSize(d);
		dialog.getContentPane().add(msgPanel);

		dialog.setLocation(250, 250);
		dialog.setLocationRelativeTo(parent);
		dialog.setVisible(true);
	}
	
	/**
	 * Create a SignBankCollection object from merging all of the resources found in the SignBankResourcCollection param
	 * @param resourceCollection The list of SignBankResource objects to merge into one collection
	 * @return A new SIgnBankCollection, created from the resourceCollection passed in.
	 */
	private SignBankCollection createSignBankCollection(JFrame parent, SignBankResourceCollection resourceCollection, boolean showVersionError) {
		
		Document doc = null;
		try {
			doc = createSignBanksDocument(parent, resourceCollection, showVersionError);
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", e);
		} catch (SAXException e) {
			e.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", e);
		} catch (IOException e) {
			e.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", e);
		}

		// resourceCollection might have been modified above
		SignBankCollection collection = new SignBankCollection(resourceCollection);
		ArrayList<SignBankResource> resources = resourceCollection.getResourcesList();

		logger.log(Level.INFO, "Parsing sign bank.");

		HashMap<String, SignBankMainGloss> mainGlosses = collection.getMainGlossesList();
		HashMap<String, SignBankVariant> variants;
		HashMap<String, SignBankOccurrence> occurrences;
		
		SignBankResource resource = null;
		SignBankMainGloss mainGloss = null;
		SignBankVariant variant = null;
		SignBankOccurrence occurrence = null;
		
		NodeList signBanksNodeList = doc.getElementsByTagName("SIGNBANKS");
		Element signBanksElement = (Element) signBanksNodeList.item(0);
		if (signBanksElement == null)
			return null;
		NodeList signBankNodeList = signBanksElement.getElementsByTagName("SIGNBANK");
		
//		// checking that the merging of resource files into one document went smoothly
//		int signBanks = signBankNodeList.getLength();
//		if (signBanks != resources.size())
//			return null;	// do something else here	
		// removing this check, as we might have removed a resource, or multiple, before parsing below
		
		for (int i = 0; i < signBankNodeList.getLength(); i++) {
			Element signBankElement = (Element) signBankNodeList.item(i);
			String signBankAuthor = signBankElement.getAttribute("AUTHOR").trim();
			String dateCreated = signBankElement.getAttribute("DATE_CREATED").trim();
			String dateModified = signBankElement.getAttribute("DATE_MODIFIED").trim();
			String redirect = signBankElement.getAttribute("REDIRECT").trim();
			String resourceFileName = signBankElement.getAttribute("RESOURCE_FILENAME").trim();
			
			for (SignBankResource res: resources) {
				if (res.getSignBankFileName().equals(resourceFileName)) {
					resource = res;
					break;
				}
			}
			
			if (resource != null) {
				resource.setSignBankAuthor(signBankAuthor);
				resource.setDateCreated(dateCreated);
				resource.setDateModified(dateModified);
				resource.setRedirect(redirect);
			}
			
			NodeList mainGlossesNodeList = signBankElement.getElementsByTagName("MAIN_GLOSSES");
			Element mainGlossesElement = (Element) mainGlossesNodeList.item(0);
			if (mainGlossesElement == null)
				continue;
			NodeList mainGlossNodeList = mainGlossesElement.getElementsByTagName("MAIN_GLOSS");
			for (int j = 0 ; j < mainGlossNodeList.getLength(); j++) {
				Element mainGlossElement = (Element) mainGlossNodeList.item(j);

				String mainGlossID = mainGlossElement.getAttribute("ID").trim();
				String defaultVariantID = mainGlossElement.getAttribute("DEFAULT_VARIANT_ID").trim();
				
				NodeList mainGlossLabelNodeList = mainGlossElement.getElementsByTagName("LABEL");
				Element mainGlossLabelElement = (Element) mainGlossLabelNodeList.item(0);
				if (mainGlossLabelElement == null)
					continue;
				String mainGlossLabel = mainGlossLabelElement.getTextContent().trim();
				
				// if gloss label already exists, use that main gloss, otherwise create a new one
				if (mainGlosses.get(mainGlossLabel) != null) {
					mainGloss = mainGlosses.get(mainGlossLabel);
				} else {
					mainGloss = new SignBankMainGloss(mainGlossID, mainGlossLabel, defaultVariantID);
					mainGlosses.put(mainGloss.getMainGlossLabel(), mainGloss);
				}

				mainGloss.setDefaultVariantID(defaultVariantID);
				variants = mainGloss.getVariantsList();
				
				NodeList variantsNodeList = mainGlossElement.getElementsByTagName("VARIANTS");
				Element variantsElement = (Element) variantsNodeList.item(0);
				if (variantsElement == null)
					continue;
				NodeList variantNodeList = variantsElement.getElementsByTagName("VARIANT");
				for (int k = 0; k < variantNodeList.getLength(); k++) {
					Element variantElement = (Element) variantNodeList.item(k);
					
					String variantID = variantElement.getAttribute("ID").trim();
					String defaultOccurrenceID = variantElement.getAttribute("DEFAULT_OCCURRENCE_ID").trim();
					String occurrenceCount = variantElement.getAttribute("OCCURRENCE_COUNT").trim();
					
					NodeList variantLabelNodeList = variantElement.getElementsByTagName("LABEL");
					Element variantLabelElement = (Element) variantLabelNodeList.item(0);
					String variantLabel = variantLabelElement.getTextContent().trim();
					
					// if variant label already exists, use that variant, otherwise create a new one
					if (variants.get(variantLabel) != null) {
						variant = variants.get(variantLabel);
					} else {
						variant = new SignBankVariant(mainGloss, variantID, variantLabel, defaultOccurrenceID, occurrenceCount);
						variants.put(variant.getVariantLabel(), variant);
					}

					variant.setDefaultOccurrenceID(defaultOccurrenceID);
					occurrences = variant.getOccurrencesList();
					
					NodeList occurrencesNodeList = variantElement.getElementsByTagName("OCCURRENCES");
					Element occurrencesElement = (Element) occurrencesNodeList.item(0);
					if (occurrencesElement == null)
						continue;
					NodeList occurrenceNodeList = occurrencesElement.getElementsByTagName("OCCURRENCE");
					for (int l = 0; l < occurrenceNodeList.getLength(); l++) {
						Element occurrenceElement = (Element) occurrenceNodeList.item(l);
						
						String occurrenceID = occurrenceElement.getAttribute("ID").trim();

						// if default occurrence id was not set for the variant, set it to the first occurrence in this list
						if (defaultOccurrenceID == null || defaultOccurrenceID.isEmpty()) {
							if (l == 0) {
								variant.setDefaultOccurrenceID(occurrenceID);
							}
						}
						
						// surround this in a try/catch, some mechanism to not break for these (optional) things TODO
						Element domStartHSEl = (Element) occurrenceElement.getElementsByTagName("D_START_HS").item(0);
						String domStartHS = domStartHSEl.getTextContent().trim();
						Element nondomStartHSEl = (Element) occurrenceElement.getElementsByTagName("ND_START_HS").item(0);
						String nondomStartHS = nondomStartHSEl.getTextContent().trim();
						Element domEndHSEl = (Element) occurrenceElement.getElementsByTagName("D_END_HS").item(0);
						String domEndHS = domEndHSEl.getTextContent().trim();
						Element nondomEndHSEl = (Element) occurrenceElement.getElementsByTagName("ND_END_HS").item(0);
						String nondomEndHS = nondomEndHSEl.getTextContent().trim();

						ArrayList<String> imageURLList = new ArrayList<String>();
						String startURL = null;
						String endURL = null;
						HashMap<Integer, String> orderedImageURLList = new HashMap<Integer, String>();
						HashMap<String, String> correspondingType = new HashMap<String, String>();
						Element imagesElement = (Element) occurrenceElement.getElementsByTagName("IMAGES").item(0);
						NodeList imagesNodeList = imagesElement.getElementsByTagName("IMAGE");
						for (int m = 0; m < imagesNodeList.getLength(); m++) {
							Element imageNode = (Element) imagesNodeList.item(m);
							String imageID = imageNode.getAttribute("ID").trim();
							String imageOrder = imageNode.getAttribute("ORDER").trim();
							String imageURL = imageNode.getAttribute("URL").trim();
							if (imageOrder != null && !imageOrder.isEmpty()) {
								orderedImageURLList.put(Integer.parseInt(imageOrder), imageURL);
							}
							imageURLList.add(imageURL);
							if (imageNode.getAttribute("TYPE").trim().equals("START")) {
								startURL = imageURL;
							} else if (imageNode.getAttribute("TYPE").trim().equals("END")) {
								endURL = imageURL;
							}
						}
						
						ArrayList<String> videoURLList = new ArrayList<String>();
						Element videosElement = (Element) occurrenceElement.getElementsByTagName("VIDEOS").item(0);
						NodeList videosNodeList = videosElement.getElementsByTagName("VIDEO");
						for (int m = 0; m < videosNodeList.getLength(); m++) {
							Element videoNode = (Element) videosNodeList.item(m);
							String videoID = videoNode.getAttribute("ID").trim();
							String videoURL = videoNode.getAttribute("URL").trim();
							videoURLList.add(videoURL);
						}
						
						Element participantEl = (Element) occurrenceElement.getElementsByTagName("PARTICIPANT").item(0);
						String participant = participantEl.getTextContent().trim();
						Element signTypeEl = (Element) occurrenceElement.getElementsByTagName("SIGN_TYPE").item(0);
						String signType = signTypeEl.getTextContent().trim();
						Element twoHandedEl = (Element) occurrenceElement.getElementsByTagName("TWO_HANDED").item(0);
						String twoHanded = twoHandedEl.getTextContent().trim();
						Element passiveBaseArmEl = (Element) occurrenceElement.getElementsByTagName("PASSIVE_BASE_ARM").item(0);
						String passiveBaseArm = passiveBaseArmEl.getTextContent().trim();
						Element markedHandsEl = (Element) occurrenceElement.getElementsByTagName("MARKED_HANDS").item(0);
						String markedHands = markedHandsEl.getTextContent().trim();
						Element leftRightHSEl = (Element) occurrenceElement.getElementsByTagName("LEFT_RIGHT_HS").item(0);
						String leftRightHS = leftRightHSEl.getTextContent().trim();
						Element startEndHSEl = (Element) occurrenceElement.getElementsByTagName("START_END_HS").item(0);
						String startEndHS = startEndHSEl.getTextContent().trim();
						Element startEndLeftHSEl = (Element) occurrenceElement.getElementsByTagName("START_END_LEFT_HS").item(0);
						String startEndLeftHS = startEndLeftHSEl.getTextContent().trim();
						Element startEndRightHSEl = (Element) occurrenceElement.getElementsByTagName("START_END_RIGHT_HS").item(0);
						String startEndRightHS = startEndRightHSEl.getTextContent().trim();

						// do not parse signs among the types: [FINGERSPELLED, COMPOUND]
						if (signType.equals("FINGERSPELLED") || signType.equals("COMPOUND"))
							continue;
						
						occurrence = new SignBankOccurrence(variant, occurrenceID);

						// store all parsed occurrence data points 
						occurrence.setDomStartHS(domStartHS);
						occurrence.setNondomStartHS(nondomStartHS);
						occurrence.setDomEndHS(domEndHS);
						occurrence.setNondomEndHS(nondomEndHS);
						
						// hack up the images for now, both will work, but preference for order tags over ID tags
						ArrayList<Integer> order = new ArrayList<Integer>(orderedImageURLList.keySet());
						if (!order.isEmpty()) {
							Collections.sort(order);
							for (int m = 0; m < order.size(); m++) {
								Integer key = order.get(m);
								String url = orderedImageURLList.get(key);
								occurrence.addFrameImageURL(url);
							}
						} else {
							for (int m = 0; m < imageURLList.size(); m++) {
								String url = imageURLList.get(m);
								occurrence.addFrameImageURL(url);
							}
						}
						
						for (int m = 0; m < videoURLList.size(); m++) {
							String url = videoURLList.get(m);
							occurrence.addSignVideoURL(url);
						}
						
						occurrence.addStartImageURL(startURL);
						occurrence.addEndImageURL(endURL);
												
						occurrence.setParticipant(participant);
						occurrence.setSignType(signType);
						occurrence.setTwoHanded(twoHanded);
						occurrence.setPassiveBaseArm(passiveBaseArm);
						occurrence.setMarkedHands(markedHands);
						occurrence.setLeftRightHS(leftRightHS);
						occurrence.setStartEndHS(startEndHS);
						occurrence.setStartEndLeftHS(startEndLeftHS);
						occurrence.setStartEndRightHS(startEndRightHS);
						
						// assign to this occurrence the resource/file it came from
						occurrence.setSignBankResourceFileName(resourceFileName);
						
						occurrences.put(occurrence.getOccurrenceID(), occurrence);
						
					}
				}
				
			}
		}
		
		resourceCollection.setResourcesList(resources);
		
		return collection;
	}

}
