package edu.bu.signstream.common.util.vo.ss3.codingScheme;

import java.util.*;
import java.awt.Color;
import java.io.Serializable;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;

public class SS3Field implements Serializable, Comparable<Object> {

	private static final long serialVersionUID = 1L;
	
	public static Color defaultColor = Color.black;
	
	private String id;
	private String name;
	private String label;
	private String prefix;
	private String type;
	private Alignment alignment = Alignment.CENTER;
	private boolean deleted = false;

	// these attributes are saved separately from the field element in xml, as display attributes
	private int order;
	private Color color = defaultColor;


	public enum Alignment {
		RIGHT("R"),
		LEFT("L"),
		CENTER("C"),
		RIGHT_LEFT("RL");

		private String label;

		Alignment(String label) {
			this.label = label;
		}

		public String getLabel() {
			return label;
		}
	}
	
	private String group = "";

	private String comment = "";

	private String legacyID;

	private HashMap<String, SS3FieldValue> values = new HashMap<String, SS3FieldValue>();
	
	private int groupID = -1;
	
	public SS3Field() {}

	public int getGroupID() {
		return groupID;
	}

	public void setGroupID(int groupID) {
		this.groupID = groupID;
	}

	public String getGroup() {
		return group;
	}

	public void setGroup(String group) {
		this.group = group;
		
		switch (group) {
		case Constants.HEAD_GROUP_NAME_CAPS:
			this.setGroupID(Constants.HEAD_GROUP_ID);
			break;
		case Constants.FACE_GROUP_NAME_CAPS:
			this.setGroupID(Constants.FACE_GROUP_ID);
			break;
		case Constants.BODY_GROUP_NAME_CAPS:
			this.setGroupID(Constants.BODY_GROUP_ID);
			break;
		case Constants.GRAMMATICAL_GROUP_NAME_CAPS:
			this.setGroupID(Constants.GRAMMATICAL_GROUP_ID);
			break;
		case Constants.SPEECH_GROUP_NAME_CAPS:
			this.setGroupID(Constants.SPEECH_GROUP_ID);
			break;
		case Constants.GENERIC_GROUP_NAME:
			this.setGroupID(Constants.GENERIC_GROUP_ID);
			break;
		case Constants.MANUAL_GROUP_NAME_CAPS:
			this.setGroupID(Constants.MANUAL_GROUP_ID);
			break;
		case Constants.GESTURE_GROUP_NAME_CAPS:
			this.setGroupID(Constants.GESTURE_GROUP_ID);
			break;
		case Constants.GESTURE_DOM_CATEGORY_GROUP_NAME_CAPS:
			this.setGroupID(Constants.GESTURE_DOM_CATEGORY_GROUP_ID);
			break;
		case Constants.GESTURE_NDOM_CATEGORY_GROUP_NAME_CAPS:
			this.setGroupID(Constants.GESTURE_NDOM_CATEGORY_GROUP_ID);
			break;
		}
	}

	public SS3Field cloneField() {
		SS3Field field = new SS3Field();
		field.setId(getId());
		field.setGroup(getGroup());
		field.setGroupID(getGroupID());
		field.setName(getName());
		field.setLabel(getLabel());
		field.setPrefix(getPrefix());
		field.setOrder(getOrder());
		field.setColor(getColor());
		field.setType(getType());
		field.setAlignment(getAlignment());
		field.setDeleted(isDeleted());

		ArrayList<SS3FieldValue> lValues = getValuesAsList();

		for (int i = 0; i < lValues.size(); i++) {
			field.addValue(lValues.get(i).cloneFieldValue());
		}

		return field;
	}	

	public void setLegacyID(String legacyID) {
		this.legacyID = legacyID;
	}

	public String getLegacyID() {
		return legacyID;
	}

	public void addValue(SS3FieldValue value) {
		values.put(value.getId(), value);
	}

	public void remove(SS3FieldValue value) {
		values.remove(value.getId());
	}

	public void setAlignment(Alignment alignment) {
		this.alignment = alignment;
	}

	public Alignment getAlignment() {
		return alignment;
	}

	public Color getColor() {
		return color;
	}

	public void setColor(String color) {
		this.color = Util.hexString2Color(color);
	}

	public String getId() {
		return id;
	}

	public String getLabel() {
		return label;
	}

	// returns label if available, othervise - name
	public String getLabelOrName() {
		if (label != null && label.length() > 0)
			return label;
		return name;
	}

	public String getName() {
		return name;
	}

	public String getPrefix() {
		return prefix;
	}

	public String getType() {
		return type;
	}
	
	public int getOrder() {
		return order;
	}
	
	public boolean isDeleted() {
		return deleted;
	}
	
	public SS3FieldValue getFieldValue(String valueID) {
		return values.get(valueID);
	}

	public HashMap<String, SS3FieldValue> getValues() {
		return values;
	}

	@Override
	public int compareTo(Object obj) {
		if (! (obj instanceof SS3Field))
			return -1;
		
		SS3Field other = (SS3Field) obj;
		
		return this.getOrder() - other.getOrder();
	}

	public ArrayList<SS3FieldValue> getValuesAsList() {
		Set<String> set = values.keySet();
		Iterator<String> it = set.iterator();
		ArrayList<SS3FieldValue> list = new ArrayList<SS3FieldValue>();

		while (it.hasNext()) {
			SS3FieldValue value = values.get(it.next());
			list.add(value);
		}
		return list;
	}

	public void setColor(Color color) {
		this.color = color;
	}

	public void setId(String string) {
		id = string;
	}

	public void setLabel(String string) {
		label = string;
	}

	public void setName(String string) {
		name = string;
	}

	public void setPrefix(String string) {
		prefix = string;
	}

	public void setType(String string) {
		type = string;
	}
	
	public void setOrder(int order) {
		this.order = order;
	}
	
	public void setOrder(String string) {
		try {
			this.order = Integer.parseInt(string);
		} catch (NumberFormatException e) {
			return;
		}
	}
	
	public void setDeleted(boolean deleted) {
		this.deleted = deleted;
	}
	
	public void setValues(ArrayList<SS3FieldValue> list) {
		for (int i = 0; i < list.size(); i++) {
			addValue(list.get(i));
		}
	}
	
	public void removeAllValues() {
		values.clear();
	}

	@Override
	public String toString() {
		return name + " <" + label + ">" + comment;
	}
	
	@Override
	public boolean equals(Object other) {
		SS3Field otherField = null;
		if (other instanceof SS3Field)
			otherField = (SS3Field) other;
		else
			return false;
		
		if (this.isDeleted() != otherField.isDeleted())
			return false;
		
		if (this.getId() == null && otherField.getId() != null)
			return false;
		if (this.getId() != null && otherField.getId() == null)
			return false;
		if (this.getId() != null && !this.getId().equals(otherField.getId()))
			return false;
		
		if (this.getName() == null && otherField.getName() != null)
			return false;
		if (this.getName() != null && otherField.getName() == null)
			return false;
		if (this.getName() != null && !this.getName().equals(otherField.getName()))
			return false;
		
		if (this.getLabel() == null && otherField.getLabel() != null)
			return false;
		if (this.getLabel() != null && otherField.getLabel() == null)
			return false;
		if (this.getLabel() != null && !this.getLabel().equals(otherField.getLabel()))
			return false;
		
		if (this.getPrefix() == null && otherField.getPrefix() != null)
			return false;
		if (this.getPrefix() != null && otherField.getPrefix() == null)
			return false;
		if (this.getPrefix() != null && !this.getPrefix().equals(otherField.getPrefix()))
			return false;
		
		if (this.getType() == null && otherField.getType() != null)
			return false;
		if (this.getType() != null && otherField.getType() == null)
			return false;
		if (this.getType() != null && !this.getType().equals(otherField.getType()))
			return false;
		
		if (this.getAlignment() == null && otherField.getAlignment() != null)
			return false;
		if (this.getAlignment() != null && otherField.getAlignment() == null)
			return false;
		if (this.getAlignment() != null && !this.getAlignment().equals(otherField.getAlignment()))
			return false;

		if (this.getOrder() == -1 && otherField.getOrder() != -1)
			return false;
		if (this.getOrder() != -1 && otherField.getOrder() == -1)
			return false;
		if (this.getOrder() != -1 && this.getOrder() != otherField.getOrder())
			return false;

		if (this.getColor() == null && otherField.getColor() != null)
			return false;
		if (this.getColor() != null && otherField.getColor() == null)
			return false;
		if (this.getColor() != null && !this.getColor().equals(otherField.getColor()))
			return false;
		
		if (this.getGroup() == null && otherField.getGroup() != null)
			return false;
		if (this.getGroup() != null && otherField.getGroup() == null)
			return false;
		if (this.getGroup() != null && !this.getGroup().equals(otherField.getGroup()))
			return false;
		
		if (this.getGroupID() == -1 && otherField.getGroupID() != -1)
			return false;
		if (this.getGroupID() != -1 && otherField.getGroupID() == -1)
			return false;
		if (this.getGroupID() != -1 && ! (this.getGroupID() == otherField.getGroupID()))
			return false;
		
		if (this.getValues().size() != otherField.values.size())
			return false;
		
		if (this.getValues().size() > 0) {
			for (SS3FieldValue value: this.getValues().values()) {
				SS3FieldValue otherValue = otherField.getFieldValue(value.getId());
				
				if (otherValue == null)
					return false;
				
				if ( !value.equals(otherValue))
					return false;
			}
		}
		
		if (otherField.getValues().size() > 0) {
			for (SS3FieldValue otherValue: otherField.getValues().values()) {
				SS3FieldValue thisValue = this.getFieldValue(otherValue.getId());
				
				if (thisValue == null)
					return false;
				
				if ( !otherValue.equals(thisValue))		// GREG: TODO: might be redundant
					return false;
			}
		}
		
		return true;
	}
}
