package edu.bu.signstream.grepresentation.fields;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.util.ArrayList;

import javax.swing.JPanel;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.vo.Track;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Track;
import edu.bu.signstream.time.Interval;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.time.TimeRange;

public abstract class Field extends JPanel {
	
	private static final long serialVersionUID = 1L;
	FieldWrapper wrapper;
	protected double Y;
	protected Track track; // String fieldID;
	protected SS3Track ss3Track;
	protected Color color = null;
	protected SignStreamSegmentPanel segmentPanel;
	protected PresentationField parentField = null;

	protected String segmentID = "";

	public FieldWrapper getWrapper() {
	    return wrapper;
    }
	
	public abstract ArrayList getEntities();

	// JACK TODO: This is just a hack since getEntities uses a raw ArrayList...
	public ArrayList<EventsEntity> getAllEntities() {
	    return new ArrayList<EventsEntity>(getEntities());
    }
	
	public ArrayList<EventsEntity> getEntitiesBetween(Time startMovieTimeInfo, Time endMovieTimeInfo, boolean exclusive) {
		ArrayList entities = getEntities();	// however this Field-inherited class returns all entities
		ArrayList<EventsEntity> entitiesBetween = new ArrayList<EventsEntity>();

		for (Object obj: entities) {
			EventsEntity entity = (EventsEntity) obj;
			
			if (entity.getStartTimeInfo() == null || entity.getEndTimeInfo() == null)
				continue;
			
			// if either startMovieTime or endMovieTime fall within the bounds of this entity, add it 
			if ( !exclusive) {
				double startMovieCoord = segmentPanel.getZoomer()
						.getCoordinateTimeConvertor().convertTimeInfoToCoordinate(startMovieTimeInfo);
				double endMovieCoord = segmentPanel
						.getZoomer().getCoordinateTimeConvertor().convertTimeInfoToCoordinate(endMovieTimeInfo);
				if (entity.contains(startMovieCoord) || entity.contains(endMovieCoord)) {
					entitiesBetween.add(entity);
					continue;
				}
			}
			
			Time entStartTimeInfo = entity.getStartTimeInfo();
			Time entEndTimeInfo = entity.getEndTimeInfo();
			
			if (startMovieTimeInfo.lessThanOrEq(entStartTimeInfo) && entEndTimeInfo.lessThanOrEq(endMovieTimeInfo)) {
				entitiesBetween.add(entity);
			}
		}
		
		return entitiesBetween;
	}

	public TimeRange timeRange() {
	    return wrapper.timeRange();
    }

	public abstract EventsEntity getEntityWithRefID(String refEntityID);

	public abstract EventsEntity getEntity(String entityID);

	public abstract void highlight(boolean highlight, String refID);

	public abstract Event getEventAt(double x);

	public abstract EventsEntity getEntityAt(double x);

	public abstract void deleteEntity(EventsEntity entity);

	public abstract int getFieldSize();

	public abstract void deleteEmptyEntities();

    /**
     * Returns whether the interval is overlapping with any entities in the field.
     */
	public abstract boolean isIntervalEmpty(Interval interval);

	public abstract void addEventsEntity(EventsEntity entity);

	public abstract double getStartTimeCoordinate();

	public abstract double getEndTimeCoordinate();

	public abstract void unselect();

	public abstract Event getSelectedEvent();

	public abstract EventsEntity getSelectedEntity();
	
	public Field(SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		this.segmentPanel = segmentPanel;
		this.parentField = parentField;
	}

	public SignStreamSegmentPanel getSignStreamSegmentPanel() {
		return segmentPanel;
	}
	
	public void setSegmentID(String segmentID) {
		this.segmentID = segmentID;
	}

	public String getSegmentID() {
		return segmentID;
	}

	public String getFieldID() {
		if (track != null)
			return track.getFieldId(); // fieldID;
		if (ss3Track != null)
			return ss3Track.getId();
		return null;
	}

	// update creation && update time here
	public SS3Track getSS3Track() {
		if (ss3Track != null) 
			ss3Track.removeEntities();
		
		ArrayList<EventsEntity> entities = getEntities();

		for (EventsEntity entity : entities) {
			ss3Track.addEntity(entity.getSS3Entity());
		}

		return ss3Track;
	}

	@Override
	public int getY() {
		return (int) Y;
	}

	public Event selectEvent(double x) {
		Event event = getEventAt(x);
		if (event != null)
			event.select();
		return event;
	}

	public boolean contains(double y) {
		double y1 = (Y - SegmentGraphReprUtil.axesTopHeight - 1);
		double y2 = (Y + SegmentGraphReprUtil.valueHeight + 2);
		return y1 <= y && y <= y2;
	}

	public void paint(Graphics2D g2d, double Y, Font font) {
		this.Y = Y;
	}

	public double getStartTime() {
		return ss3Track.getStartTime();
	}

	public double getEndTime() {
		return ss3Track.getEndTime();
	}

	public Color getColor() {
		return color;
	}

	public void setColor(Color color) {
		this.color = color;
	}
	
	public PresentationField getParentField() {
		return parentField;
	}
	
	public void setParentField(PresentationField parentField) {
		this.parentField = parentField;
	}
}
