package edu.bu.signstream.grepresentation.fields;

import java.awt.Point;
import java.util.ArrayList;

import javax.swing.JPanel;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.time.Rounding;
import edu.bu.signstream.time.TimeRange;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.time.TimeUnit;

public abstract class FieldWrapper {
   protected boolean visible = true;
   protected int viewLineNumber = -1;
   protected Field field;
   private String type;

    protected TimeRange defaultTimeRange;
	protected TimeRange timeRange;

   protected int MOVE_DIRECTION;
   public static final int MOVE_START_POINT = 1;
   public static final int MOVE_END_POINT = 2;

   private boolean updatable = false;   
   private boolean selectedNHighlighted = false;
   
   protected ArrayList<FieldWrapper> dependentWrappers = new ArrayList<FieldWrapper>();
   protected SignStreamSegmentPanel segmentPanel;

    public TimeRange defaultTimeRange() { return defaultTimeRange; }
    public void setDefaultTimeRange(TimeRange range) { defaultTimeRange = range; }

    public TimeRange timeRange() { return timeRange; }
    public void setTimeRange(TimeRange range) { timeRange = range; }

   public boolean isSelectedNHighlighted() {
	   return selectedNHighlighted;
   }

   public void setSelectedNHighlighted(boolean selectedNHighlighted) {
	   this.selectedNHighlighted = selectedNHighlighted;
   }

   public FieldWrapper(Field field, SignStreamSegmentPanel segmentPanel, TimeRange defaultTimeRange) {
       field.wrapper = this;
      this.field = field;
      this.segmentPanel = segmentPanel;
      this.defaultTimeRange = defaultTimeRange;
      this.timeRange = defaultTimeRange;
   }

   public SignStreamSegmentPanel getSignStreamSegmentPanel() {
      return segmentPanel;
   }
   
   public void setMoveDirection(int direction) {
      this.MOVE_DIRECTION = direction;
   }
   
   public int getMoveDirection() {
      return MOVE_DIRECTION;
   }

   public void repaint() {
      field.repaint();
   }

   public boolean isUpdatable() {
      return updatable;
   }

   public void setUpdatable(boolean updatable) {
      this.updatable = updatable;
   }

 //  public abstract void setDependencies();

   public ArrayList getDependentWrappers() {
      return dependentWrappers;
   }

   public void addDependentWrapper(FieldWrapper wrapper) {
      String fieldID = wrapper.getField().getFieldID();
      FieldWrapper wrp = getDependentFieldWrapper(fieldID);

      if (wrp == null)
         dependentWrappers.add(wrapper);
   }

   public FieldWrapper getDependentFieldWrapper(String fieldID) {
      for (int i = 0; i < dependentWrappers.size(); i++) {
         FieldWrapper wrp = dependentWrappers.get(i);
         String fID = wrp.getField().getFieldID();
         if (fieldID.equalsIgnoreCase(fID))
            return wrp;
      }
      return null;
   }

   public FieldWrapper getDependentFieldWrapperByType(String type) {
       SS3CodingScheme dbScheme = segmentPanel.getSS3Database().getSS3CodingScheme();

      for (int i = 0; i < dependentWrappers.size(); i++) {
         FieldWrapper wrp = dependentWrappers.get(i);
         String fieldID = wrp.getField().getFieldID();
         SS3Field f = dbScheme.getField(fieldID);
         String ftype = f.getType();

         if (ftype.equalsIgnoreCase(type))
            return wrp;
      }
      return null;
   }

  // public abstract void adjustEntity(int entityAt, Event event);

   //public abstract void highlight(boolean highlight, int eventAt);
   public abstract void highlight(boolean highlight, String refID);

   public abstract void addEventsEntity(EventsEntity entity);

   public EventsEntity getEntityAt(int entityAt) {
      return field.getEntityAt(entityAt);
   }

   public void delete(int eventAt) {
      EventsEntity entity = field.getEntityAt(eventAt);
      if (entity != null)
         field.deleteEntity(entity);
   }

   public void setType(String type) {
      this.type = type;
   }

   public String getType() {
      return type;
   }

   public boolean isVisible() {
      return visible;
   }

   public void setVisible(boolean visible) {
      this.visible = visible;
   }

   public Field getField() {
      return field;
   }

   public void setViewLineNumber(int viewLineNumber) {
      this.viewLineNumber = viewLineNumber;
   }

   protected boolean isEntitySelected() {
      try {
         return segmentPanel.isEntityLevelSelected();
      } catch (NullPointerException ex) {
         return false;
      }
   }

   //public abstract int getViewLineNumber();

   public abstract boolean contains(int Y);

   // MouseListener
   public abstract void mouseClicked(Point point, JPanel panel);
   public abstract void mouseClickedAlt(Point point);
   public abstract void mousePressedAlt(Point point, JPanel panel);
   public abstract void mouseRightClicked(Point point);
   public abstract void mouseLeftClicked(Point point);
   //public abstract void mouseDblClicked(Point point);
   //blic abstract void mouseReleased();
   public abstract void mouseReleased(Point point, JPanel panel);
   public abstract void mouseHoldReleased(Point point, JPanel panel);
   public abstract void mouseDblLeftClicked(Point point, JPanel panel);
   public abstract void mouseDblRightClicked(Point point, JPanel panel);

   // MouseMotionListener
   public abstract void mouseMoved(Point point, JPanel panel);
   public void mouseDragged(Point p, JPanel panel) {
       var time = TimeRange.ABSOLUTE.time(p.x, TimeUnit.COORDINATE)
           .in(Util.getCurrentRange(getField()), Rounding.FLOOR);

       SS3Singleton.getMediaPlayer().seek(time);
   };

   // KeyListener
   //public abstract void keyPressed(KeyEvent e);

   public abstract void unselectAll();

   public double getStartTime() {
      return field.getStartTime();
   }

   public double getEndTime() {
      return field.getEndTime();
   }

   public double getStartTimeCoordinate() {
      return field.getStartTimeCoordinate();
   }

   public double getEndTimeCoordinate() {
      return field.getEndTimeCoordinate();
   }

   public int getViewLineNumber() {
      if (viewLineNumber >= 0)
         return viewLineNumber;
      return Integer.parseInt(field.getFieldID());
   }

   public Event getSelectedEvent() {
      return field.getSelectedEvent();
   }

   public EventsEntity getSelectedEntity() {
      return field.getSelectedEntity();
   }

   public void selectItemIfEntitySelected() {
      EventsEntity entity = getSelectedEntity();
      if (entity != null)
         entity.selectEventIfEntitySelected();
   }

   public void selectEntityIfItemSelected() {
      Event event = getSelectedEvent();
      if (event != null) {
         double coord =
            event.getStartTimeCoordinate() + event.getEndTimeCoordinate() / 2;
         EventsEntity entity = field.getEntityAt(coord);
         if (entity != null)
            entity.select();
      }
   }
}