package edu.bu.signstream.grepresentation.fields;

import java.awt.*;
import java.awt.geom.Line2D;
import java.util.ArrayList;
import java.util.StringTokenizer;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldTypeImg;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.grepresentation.fields.apexField.ApexChainedApexEvent;
import edu.bu.signstream.grepresentation.fields.apexField.ApexChainedApexEventsEntity;
import edu.bu.signstream.grepresentation.fields.apexField.ChainedApexEvent;
import edu.bu.signstream.grepresentation.fields.apexField.ChainedApexEventsCollection;
import edu.bu.signstream.grepresentation.fields.apexField.ChainedApexEventsEntity;
import edu.bu.signstream.grepresentation.fields.apexField.NewChainedApexEvent;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureCompoundable;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureChainedEvent;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureChainedEventsCollection;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureEntityProperties;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureEvent;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureEventsEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureField;
import edu.bu.signstream.grepresentation.fields.gestureField.NewGestureChainedEvent;
import edu.bu.signstream.grepresentation.fields.glossField.Compoundable;
import edu.bu.signstream.grepresentation.fields.glossField.EnteredWord;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEvent;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.glossField.GlossField;
import edu.bu.signstream.grepresentation.fields.glossField.Text;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeEntity;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeEvent;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeSubEntity;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapesImageManager;
import edu.bu.signstream.grepresentation.fields.handShapeField.NewHandShapeEntity;
import edu.bu.signstream.grepresentation.fields.handShapes.joints.JointsEvent;
import edu.bu.signstream.grepresentation.fields.handShapes.joints.JointsEventsEntity;
import edu.bu.signstream.grepresentation.fields.movementField.MovementEntity;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEvent;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsCollection;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.nonManualField.NewChainedEvent;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationEvent;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationEventsEntity;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;

public class Painter {
	
	private SS3CodingScheme scheme = null;

	private Color entitySelectedColor = Color.GREEN;
	private Color eventSelectedColor = Color.RED;
	private Color eventHighlightedColor = Color.BLUE;
	private Color onsetUnselectedColor = new Color(14, 140, 58);
	private Color offsetUnselectedColor = Color.BLUE;
	
	private final String TWO_HANDED_STRING = "(2h)";
	private final String INCOMPLETE_STRING = "INC-";
	private final String ALT_HANDS_STRING = "alt.";

	public Painter(SS3CodingScheme scheme) {
		this.scheme = scheme;
	}

	public void drawNewEvent(NewChainedEvent event, Graphics2D g2d, double Y) {
		double X1 = event.getStartTimeCoordinate();
		double X2 = event.getEndTimeCoordinate();

		g2d.fillOval((int) X1, (int) Y - 5, 4, 4);
		if (X2 > -1)
			g2d.draw(new Line2D.Double(X1, Y - 5, X2, Y - 5));
	}
	
	public void drawNewEvent(NewChainedApexEvent event, Graphics2D g2d, double Y) {
		double X1 = event.getStartTimeCoordinate();
		double X2 = event.getEndTimeCoordinate();

		g2d.fillOval((int) X1, (int) Y - 5, 4, 4);
		if (X2 > -1)
			g2d.draw(new Line2D.Double(X1, Y - 5, X2, Y - 5));
	}

	public void drawEnteredWord(EnteredWord word, Graphics2D g2d, double Y,
			FontMetrics fontMetrics) {
		g2d.setColor(Color.darkGray);
		g2d.drawString(word.getText(), word.getStartCoord(), (int) Y - 1);
	}
	
	public void drawEnteredGlossText(ArrayList<Text> textObjects,
			Graphics2D g2d, double Y, FontMetrics fontMetrics,
			GlossField glsField) {

		g2d.setFont(fontMetrics.getFont());

		for (Text text : textObjects) {
			if (text.isSelected()) {
				g2d.setColor(Color.RED);
			} else {
				g2d.setColor(Color.DARK_GRAY);
			}

			double endCoord = text.getStartCoord() + fontMetrics.stringWidth(text.getText());
			text.setEndCoord(endCoord);
			g2d.drawString(text.getText(), (int) text.getStartCoord(), (int) Y - 1);
		}
	}
// @ TODO:  
	public void drawEnteredGlossTextNew(ArrayList<Text> textObjects,
			Graphics2D g2d, double Y, FontMetrics fontMetrics,
			GlossField glsField) {

		g2d.setFont(fontMetrics.getFont());

		for (Text text : textObjects) {
			if (text.isSelected()) {
				g2d.setColor(Color.RED);
			} else {
				g2d.setColor(Color.DARK_GRAY);
			}
			
			double textStartCoord = text.getStartCoord();
			
			int textWidth = fontMetrics.stringWidth(text.getText());

			double nextEventStartCoord = glsField
					.getNextEntityOrTextStartCoord(text.getStartCoord());			
			
			if(nextEventStartCoord < 0) {
				double endCoord = text.getStartCoord() + fontMetrics.stringWidth(text.getText());
				text.setEndCoord(endCoord);
				g2d.drawString(text.getText(), (int) text.getStartCoord(), (int) Y - 1);
				return;
			}
			
			double availableWidth = nextEventStartCoord - textStartCoord;
//			
			System.out.println("textStartCoord = " + textStartCoord + 
					", textWidth = " + textWidth + ", nextEventStartCoord = " + nextEventStartCoord + 
					", availableWidth = " + availableWidth);
//			
			if(availableWidth < textWidth) {
				//
				System.err.println("availableWidth = " + availableWidth);
				//
				
				double prevEventStartCoord = glsField.getPrevEntityOrTextStartCoord(text.getStartCoord());
				
				if(prevEventStartCoord < text.getStartCoord()) {
					double newStartCoord = prevEventStartCoord + 5;
					text.setStartCoord(newStartCoord);
					
					//if() {
						
					//}
					
				} else {				
					String shortTxt = "&";
					double endCoord = text.getStartCoord() + fontMetrics.stringWidth(shortTxt);
					double endShortCoord = text.getStartCoord() + endCoord;
					text.setEndCoord(endShortCoord);
					g2d.drawString(shortTxt, (int) text.getStartCoord(), (int) Y - 1);
				}

			} else {
				double endCoord = text.getStartCoord() + fontMetrics.stringWidth(text.getText());
				text.setEndCoord(endCoord);
				g2d.drawString(text.getText(), (int) text.getStartCoord(), (int) Y - 1);
			}
		}
	}
	
	public void drawApexEntity(ApexChainedApexEventsEntity entity,
			Graphics2D g2d, double Y, FontMetrics fontMetrics, String fieldID,
			EventsEntity previousEntity, EventsEntity followingEntity) {
		Field field = entity.getField();

		ChainedApexEventsCollection collection = entity
				.getChainedApexEventsCollection();
		ChainedApexEvent onset = collection.getOnsetEvent();
		ChainedApexEvent value = collection.getValueEvent();
		ApexChainedApexEvent glossValue = collection.getTextValueEvent();
		ChainedApexEvent offset = collection.getOffsetEvent();

		boolean selected = entity.isSelected();

		// paint compounds
//		if (entity.isStartCompound()) {
//			Compoundable cmp = entity.getStartCompound();
//			double coord = cmp.getEndTimeCoordinate();
//
//			double x = coord + (entity.getStartTimeCoordinate() - coord) / 2;
//			Color saveColor = g2d.getColor();
//            g2d.setColor(Color.RED);
//			g2d.drawString("+", (int) x - 4, (int) Y);
//			g2d.setColor(saveColor);
//		}

		if (onset != null) {
			g2d.setColor(getHoldColor(onset, selected));
			drawOnset(onset, g2d, Y);
		}

		if (glossValue != null) {

			if (entity.getField() != null
					&& entity.getField().getColor() != null) {
				g2d.setColor(entity.getField().getColor());
			} else {
				g2d.setColor(getValueColor(glossValue, selected, entity
						.isHighlighted(), fieldID, field));
			}
			drawTextValue(entity, glossValue, g2d, Y, fontMetrics, fieldID,
					previousEntity, followingEntity);

//			boolean isLockedDomNdomAlignments = entity
//					.isLockedDomNdomAlignments();
			// boolean isLockedGlossPhonAlignments =
			// entity.isLockedGlossPhonAlignments();
			// drawLock(g2d, glossValue.getStartTimeCoordinate(), (int) Y,
			// Color.BLUE, isLockedDomNdomAlignments,
			// isLockedGlossPhonAlignments);
			// drawLock(g2d, glossValue.getEndTimeCoordinate(), (int) Y,
			// Color.BLUE, isLockedDomNdomAlignments,
			// isLockedGlossPhonAlignments);

//			drawLock(g2d, (int) glossValue.getStartTimeCoordinate(), (int) Y,
//					Color.BLUE, isLockedDomNdomAlignments);
//			drawLock(g2d, (int) glossValue.getEndTimeCoordinate(), (int) Y,
//					Color.BLUE, isLockedDomNdomAlignments);
		}

		if (value != null) {
			Color color = null;
			if (entity.getField() != null
					&& entity.getField().getColor() != null) {
				color = entity.getField().getColor();
			} else {
				color = getValueColor(value, selected, entity.isHighlighted(),
						fieldID, field);
			}
			g2d.setColor(color);
			drawValue(value, g2d, Y);
			drawValueLabel(value, entity, g2d, Y, fontMetrics, fieldID,
					previousEntity, followingEntity);

//			boolean isLockedDomNdomAlignments = entity
//					.isLockedDomNdomAlignments();
//			boolean isLockedGlossPhonAlignments = entity
//					.isLockedGlossPhonAlignments();
//
//			drawLock(g2d, (int) glossValue.getStartTimeCoordinate(), (int) Y,
//					Color.BLUE, isLockedDomNdomAlignments,
//					isLockedGlossPhonAlignments);
//			drawLock(g2d, (int) glossValue.getEndTimeCoordinate(), (int) Y,
//					Color.BLUE, isLockedDomNdomAlignments,
//					isLockedGlossPhonAlignments);

			// if (entity.isLockedDomNdomAlignments()) {
			// drawLock(g2d, value.getStartTimeCoordinate(), (int) Y,
			// eventHighlightedColor);
			// drawLock(g2d, value.getEndTimeCoordinate(), (int) Y,
			// eventHighlightedColor);
			// }
		}
		
		if (offset != null) {
			g2d.setColor(getHoldColor(offset, selected));
			drawOffset(offset, g2d, Y);
		}
	}

	public void drawGlossEntity(GlossChainedEventsEntity entity,
			Graphics2D g2d, double Y, FontMetrics fontMetrics, String fieldID,
			EventsEntity previousEntity, EventsEntity followingEntity) {

		Field field = entity.getField();

		ChainedEventsCollection collection = entity
				.getChainedEventsCollection();
		ChainedEvent onset = collection.getOnsetEvent();
		ChainedEvent initialHold = collection.getInitialHoldEvent();
		ChainedEvent value = collection.getValueEvent();
		GlossChainedEvent glossValue = collection.getTextValueEvent();
		ChainedEvent finalHold = collection.getFinalHoldEvent();
		ChainedEvent offset = collection.getOffsetEvent();

		boolean selected = entity.isSelected();

		// paint compounds
		if (entity.isStartCompound()) {
			Compoundable cmp = entity.getStartCompound();
			double coord = cmp.getEndTimeCoordinate();

			double x = coord + (entity.getStartTimeCoordinate() - coord) / 2;
			Color saveColor = g2d.getColor();
            g2d.setColor(Color.RED);
			g2d.drawString("+", (int) x - 4, (int) Y);
			g2d.setColor(saveColor);
		}

		if (onset != null) {
			g2d.setColor(getHoldColor(onset, selected));
			drawOnset(onset, g2d, Y);
		}

		if (initialHold != null) {
			g2d.setColor(getHoldColor(initialHold, selected));
			drawInitialHold(initialHold, g2d, Y);
		}

		if (glossValue != null) {

			if (entity.getField() != null
					&& entity.getField().getColor() != null) {
				g2d.setColor(entity.getField().getColor());
			} else {
				g2d.setColor(getValueColor(glossValue, selected, entity
						.isHighlighted(), fieldID, field));
			}
			drawTextValue(entity, glossValue, g2d, Y, fontMetrics, fieldID,
					previousEntity, followingEntity);

			boolean isLockedDomNdomAlignments = entity
					.isLockedDomNdomAlignments();
			// boolean isLockedGlossPhonAlignments =
			// entity.isLockedGlossPhonAlignments();
			// drawLock(g2d, glossValue.getStartTimeCoordinate(), (int) Y,
			// Color.BLUE, isLockedDomNdomAlignments,
			// isLockedGlossPhonAlignments);
			// drawLock(g2d, glossValue.getEndTimeCoordinate(), (int) Y,
			// Color.BLUE, isLockedDomNdomAlignments,
			// isLockedGlossPhonAlignments);

			drawLock(g2d, (int) glossValue.getStartTimeCoordinate(), (int) Y,
					Color.BLUE, isLockedDomNdomAlignments);
			drawLock(g2d, (int) glossValue.getEndTimeCoordinate(), (int) Y,
					Color.BLUE, isLockedDomNdomAlignments);
		}

		if (value != null) {
			Color color = null;
			if (entity.getField() != null
					&& entity.getField().getColor() != null) {
				color = entity.getField().getColor();
			} else {
				color = getValueColor(value, selected, entity.isHighlighted(),
						fieldID, field);
			}
			g2d.setColor(color);
			drawValue(value, g2d, Y);
			drawValueLabel(value, entity, g2d, Y, fontMetrics, fieldID,
					previousEntity, followingEntity);

			boolean isLockedDomNdomAlignments = entity
					.isLockedDomNdomAlignments();
			boolean isLockedGlossPhonAlignments = entity
					.isLockedGlossPhonAlignments();

			drawLock(g2d, (int) glossValue.getStartTimeCoordinate(), (int) Y,
					Color.BLUE, isLockedDomNdomAlignments,
					isLockedGlossPhonAlignments);
			drawLock(g2d, (int) glossValue.getEndTimeCoordinate(), (int) Y,
					Color.BLUE, isLockedDomNdomAlignments,
					isLockedGlossPhonAlignments);

			// if (entity.isLockedDomNdomAlignments()) {
			// drawLock(g2d, value.getStartTimeCoordinate(), (int) Y,
			// eventHighlightedColor);
			// drawLock(g2d, value.getEndTimeCoordinate(), (int) Y,
			// eventHighlightedColor);
			// }
		}

		if (finalHold != null) {
			g2d.setColor(getHoldColor(finalHold, selected));
			drawFinalHold(finalHold, g2d, Y);
		}

		if (offset != null) {
			g2d.setColor(getHoldColor(offset, selected));
			drawOffset(offset, g2d, Y);
		}
	}
	
	public void drawEntity(ChainedApexEventsEntity entity, Graphics2D g2d,
			double Y, FontMetrics fontMetrics, String fieldID,
			EventsEntity previousEntity, EventsEntity followingEntity) {
		
	}

	public void drawEntity(ChainedEventsEntity entity, Graphics2D g2d,
			double Y, FontMetrics fontMetrics, String fieldID,
			EventsEntity previousEntity, EventsEntity followingEntity) {

		Field field = entity.getField();

		ChainedEventsCollection collection = entity
				.getChainedEventsCollection();
		ChainedEvent onset = collection.getOnsetEvent();
		ChainedEvent initialHold = collection.getInitialHoldEvent();
		ChainedEvent value = collection.getValueEvent();
		GlossChainedEvent glossValue = collection.getTextValueEvent();
		ChainedEvent finalHold = collection.getFinalHoldEvent();
		ChainedEvent offset = collection.getOffsetEvent();

		boolean selected = entity.isSelected();

		if (onset != null) {
			g2d.setColor(getHoldColor(onset, selected));
			drawOnset(onset, g2d, Y);
		}

		if (initialHold != null) {
			g2d.setColor(getHoldColor(initialHold, selected));
			drawInitialHold(initialHold, g2d, Y);
		}

		if (glossValue != null) {

			if (entity.getField() != null
					&& entity.getField().getColor() != null) {
				g2d.setColor(entity.getField().getColor());
			} else {
				g2d.setColor(getValueColor(glossValue, selected, entity
						.isHighlighted(), fieldID, field));
			}
			drawTextValue(entity, glossValue, g2d, Y, fontMetrics, fieldID,
					previousEntity, followingEntity);
		}

		if (value != null) {
			Color color = null;
			if (entity.getField() != null
					&& entity.getField().getColor() != null) {
				color = entity.getField().getColor();
			} else {
				color = getValueColor(value, selected, entity.isHighlighted(),
						fieldID, field);
			}
			g2d.setColor(color);
			drawValue(value, g2d, Y);
			drawValueLabel(value, entity, g2d, Y, fontMetrics, fieldID,
					previousEntity, followingEntity);
		}

		if (finalHold != null) {
			g2d.setColor(getHoldColor(finalHold, selected));
			drawFinalHold(finalHold, g2d, Y);
		}

		if (offset != null) {
			g2d.setColor(getHoldColor(offset, selected));
			drawOffset(offset, g2d, Y);
		}

		 if((fieldID.equalsIgnoreCase(Util.DOM_HAND_FIELD_ID) ||
		 fieldID.equalsIgnoreCase(Util.NDOM_HAND_FIELD_ID)) &&
		 entity.isLockedGlossPhonAlignments()) {
			 
			 Event v = entity.getValueOrTextValueEvent();
			 if(v != null) {}
			 	drawLock(g2d, (int) v.getStartTimeCoordinate(), (int) Y, Color.BLUE,
			 			entity.isLockedGlossPhonAlignments());
			 	drawLock(g2d, (int) v.getEndTimeCoordinate(), (int) Y, Color.BLUE,
			 			entity.isLockedGlossPhonAlignments());
		 }
	}
	
	public void drawEntity(GestureChainedEventsEntity entity, Graphics2D g2d,
			double Y, FontMetrics fontMetrics, String fieldID,
			GestureChainedEventsEntity previousEntity, GestureChainedEventsEntity followingEntity) {

		Field field = entity.getField();

		GestureChainedEventsCollection collection = entity
				.getGestureChainedEventsCollection();
		GestureChainedEvent onset = collection.getPreparationEvent();
		GestureChainedEvent initialHold = collection.getPrestrokeHoldEvent();
		GestureChainedEvent value = collection.getValueEvent();
		GestureEvent glossValue = collection.getTextValueEvent();
		GestureChainedEvent finalHold = collection.getPoststrokeHoldEvent();
		GestureChainedEvent offset = collection.getRetractionEvent();
		GestureChainedEvent relaxed = collection.getRelaxedEvent();

		boolean selected = entity.isSelected();

		if (onset != null) {
			g2d.setColor(getHoldColor(onset, selected));
			drawOnset(onset, g2d, Y);
		}

		if (initialHold != null) {
			g2d.setColor(getHoldColor(initialHold, selected));
			drawInitialHold(initialHold, g2d, Y);
		}

		if (glossValue != null) {

			if (entity.getField() != null
					&& entity.getField().getColor() != null) {
				g2d.setColor(entity.getField().getColor());
			} else {
				g2d.setColor(getValueColor(glossValue, selected, entity
						.isHighlighted(), fieldID, field));
			}
			drawTextValue(entity, glossValue, g2d, Y, fontMetrics, fieldID,
					previousEntity, followingEntity);
		}

		if (value != null) {
			Color color = null;
			if (entity.getField() != null
					&& entity.getField().getColor() != null) {
				color = entity.getField().getColor();
			} else {
				color = getValueColor(value, selected, entity.isHighlighted(),
						fieldID, field);
			}
			g2d.setColor(color);
			drawValue(value, g2d, Y);
			drawValueLabel(value, entity, g2d, Y, fontMetrics, fieldID,
					previousEntity, followingEntity);
		}

		if (finalHold != null) {
			g2d.setColor(getHoldColor(finalHold, selected));
			drawFinalHold(finalHold, g2d, Y);
		}

		if (offset != null) {
			g2d.setColor(getHoldColor(offset, selected));
			drawOffset(offset, g2d, Y);
		}
		
		if (relaxed != null) {
			g2d.setColor(getHoldColor(relaxed, selected));
			drawRelaxed(relaxed, g2d, Y);
		}

		 if((fieldID.equalsIgnoreCase(Util.DOM_GESTURE_HAND_FIELD_ID) ||
		 fieldID.equalsIgnoreCase(Util.NDOM_GESTURE_HAND_FIELD_ID)) &&
		 entity.isLockedGesturePhonAlignments()) {
			 
			 Event v = entity.getValueOrTextValueEvent();
			 if(v != null) {}
			 	drawLock(g2d, (int) v.getStartTimeCoordinate(), (int) Y, Color.BLUE,
			 			entity.isLockedGesturePhonAlignments());
			 	drawLock(g2d, (int) v.getEndTimeCoordinate(), (int) Y, Color.BLUE,
			 			entity.isLockedGesturePhonAlignments());
		 }
	}

	public void drawLock(Graphics2D g2d, int x, int y, Color color,
			boolean locked) {
		if (locked) {
			Color saveColor = g2d.getColor();
			g2d.setColor(color);
			g2d.drawLine(x, y - 14, x, y - 8);

			g2d.drawLine(x - 3, y - 11, x, y - 8);
			g2d.drawLine(x + 3, y - 11, x, y - 8);

			g2d.drawLine(x - 3, y - 11, x, y - 14);
			g2d.drawLine(x + 3, y - 11, x, y - 14);
			
			g2d.setColor(saveColor);
		}
	}

	public void drawLock(Graphics2D g2d, int x, int y, Color color,
			boolean isLockedDomNdomAlignments,
			boolean isLockedGlossPhonAlignments) {
		Color saveColor = g2d.getColor();

		if (isLockedDomNdomAlignments || isLockedGlossPhonAlignments) {
			g2d.setColor(color);
			g2d.drawLine(x, y - 14, x, y - 8);
			g2d.setColor(saveColor);
		}

		if (isLockedDomNdomAlignments) {
			g2d.drawLine(x - 3, y - 11, x, y - 8);
			g2d.drawLine(x + 3, y - 11, x, y - 8);
		}

		if (isLockedGlossPhonAlignments) {
			g2d.drawLine(x - 3, y - 11, x, y - 14);
			g2d.drawLine(x + 3, y - 11, x, y - 14);
		}
	}

	public void drawPresentationEntity(PresentationEventsEntity entity,
			Graphics2D g2d, double Y, FontMetrics fontMetrics) {

		PresentationEvent event = entity.getEvent();
		g2d.setColor(Color.BLACK);

		if (entity.isSelected())
			g2d.setColor(Color.RED);

		double X1 = event.getStartTimeCoordinate();
		double X2 = event.getEndTimeCoordinate();

		g2d.draw(new Line2D.Double(X1, Y, X2, Y));
		g2d.draw(new Line2D.Double(X1, Y + 1, X2, Y + 1));
		g2d.draw(new Line2D.Double(X1, Y - 2, X1, Y + 3));
		g2d.draw(new Line2D.Double(X2, Y - 2, X2, Y + 3));

		String label = event.getText();
		label = Util.getAdjustedLabel(event.getText(), fontMetrics, (int) (X2 - X1));

		g2d.setColor(Color.DARK_GRAY);

		label = label == null ? "" : label;

		int lenght = fontMetrics.stringWidth(label);
		double dX = (X2 - X1 - lenght) / 2 + X1;
		g2d.drawString(label, (int) dX, (int) Y - 1);
	}

	public void drawTextValue(EventsEntity entity,
			GlossChainedEvent glossValue, Graphics2D g2d, double Y,
			FontMetrics fontMetrics, String fieldID,
			EventsEntity previousEntity, EventsEntity followingEntity) {

		Field field = entity.getField();

		Color color = getValueColor(glossValue, entity.isSelected(), entity
				.isHighlighted(), fieldID, field);
		g2d.setColor(color);

		drawValue(glossValue, g2d, Y);
		drawValueLabel(glossValue, entity, g2d, Y, fontMetrics, fieldID,
				previousEntity, followingEntity);
	}
	
	public void drawTextValue(GestureChainedEventsEntity entity,
			GestureEvent glossValue, Graphics2D g2d, double Y,
			FontMetrics fontMetrics, String fieldID,
			GestureChainedEventsEntity previousEntity, GestureChainedEventsEntity followingEntity) {

		Field field = entity.getField();

		Color color = getValueColor(glossValue, entity.isSelected(), entity
				.isHighlighted(), fieldID, field);
		g2d.setColor(color);

		drawValue(glossValue, g2d, Y);
		drawValueLabel(glossValue, entity, g2d, Y, fontMetrics, fieldID,
				previousEntity, followingEntity);
	}
	
	public void drawTextValue(EventsEntity entity,
			ApexChainedApexEvent glossValue, Graphics2D g2d, double Y,
			FontMetrics fontMetrics, String fieldID,
			EventsEntity previousEntity, EventsEntity followingEntity) {

		Field field = entity.getField();

		Color color = getValueColor(glossValue, entity.isSelected(), entity
				.isHighlighted(), fieldID, field);
		g2d.setColor(color);

		drawValue(glossValue, g2d, Y);
		drawValueLabel(glossValue, entity, g2d, Y, fontMetrics, fieldID,
				previousEntity, followingEntity);
	}

	public void drawMovementEntity(MovementEntity entity, Graphics2D g2d,
			FontMetrics fontMetrics, double Y, String fieldID) {

		boolean selected = entity.isSelected();

		if (entity.getFirstEvent() == null)
			return;

		Color color = getValueColor(entity, selected, entity.isHighlighted(),
				fieldID, entity.getField());

		g2d.setColor(color);

		ArrayList events = entity.getEvents();
		for (int i = 0; i < events.size(); i++) {
			ChainedEvent event = (ChainedEvent) events.get(i);
			String id = event.getID();
			StringBuffer sbLabel = new StringBuffer();
			StringTokenizer tk = new StringTokenizer(id, "#");
			String vID = "0";

			while (tk.hasMoreTokens()) {
				String token = tk.nextToken();
				StringTokenizer tk1 = new StringTokenizer(token, ".");
				String cID = tk1.nextToken();
				if (cID.equalsIgnoreCase(Constants.VC_MOVEMENT_CONTOUR)) {
					vID = tk1.nextToken();
					break;
				}
			}

			SS3FieldValue value = scheme.getValueCollectionSS3FieldValue(
					Constants.VC_MOVEMENT_CONTOUR, vID);
			sbLabel.append(value.getLabelOrName());

			if (event.isSelected()) {
				g2d.setColor(eventSelectedColor);
			} else {
				g2d.setColor(color);
			}

			double X1 = event.getStartTimeCoordinate();
			double X2 = event.getEndTimeCoordinate();
			g2d.draw(new Line2D.Double(X1, Y, X2, Y));
			g2d.draw(new Line2D.Double(X1, Y + 1, X2, Y + 1));
			g2d.draw(new Line2D.Double(X1, Y - 2, X1, Y + 3));
			g2d.draw(new Line2D.Double(X2, Y - 2, X2, Y + 3));

			g2d.setColor(Color.DARK_GRAY);

			String label = sbLabel.toString();

			int lenght = fontMetrics.stringWidth(label);
			double dX = (X2 - X1 - lenght) / 2 + X1;
			g2d.drawString(label, (int) dX, (int) Y - 1);
		}
	}

	public void drawEntityWithMultipleEvents(JointsEventsEntity entity,
			Graphics2D g2d, double Y, FontMetrics fontMetrics, String fieldID) {

		ArrayList events = entity.getEvents();
		boolean selected = entity.isSelected();

		Color color = entity.getColor();
		if (color != null)
			g2d.setColor(color);

		g2d.draw(new Line2D.Double(entity.getStartTimeCoordinate(), Y + 1,
				entity.getEndTimeCoordinate(), Y + 1));

		for (int i = 0; i < events.size(); i++) {
			JointsEvent event = (JointsEvent) events.get(i);

			if (event != null) {
				color = getValueColor(event, selected, entity.isHighlighted(),
						fieldID, entity.getField());
				g2d.setColor(color);

				double X1 = event.getStartTimeCoordinate();
				double X2 = X1 + 10;

				if (i == events.size() - 1)
					X1 = X1 - 10;

				if (event.isSelected()) {
					g2d.setColor(Color.red);
				} else {
					g2d.setColor(Color.DARK_GRAY);
				}

				g2d.fillRect((int) X1, (int) Y - 1, 10, 3);
				event.setRectangle(new Rectangle((int) (X1 - 1), 1, 10, 30));

				String valueID = event.getID();

				SS3FieldValue csValue = scheme.getField(fieldID) != null ? scheme
						.getField(fieldID).getFieldValue(valueID)
						: null;

				if (csValue != null) {
					event.setText(csValue.getLabelOrName());
					event.setToolTipText(csValue.getName());
					g2d.setColor(Color.BLACK);
				}

				String label = event.getText() == null ? "" : event.getText();

				int lenght = fontMetrics.stringWidth(label);
				double dX = (X2 - X1 - lenght) / 2 + X1;
				g2d.drawString(label, (int) dX, (int) Y - 2);
			}
		}
	}

	public void drawHandShapeEntity(HandShapeEntity entity, Graphics2D g2d,
			double Y, String fieldID) {

		if (entity.getFirstEvent() == null)
			return;

		if (!entity.hasSubEntities())
			return;

		// LOAD IMAGES
		loadHandShapeIcons(entity, fieldID);

        // check if one framed entity.
        if(entity.getFirstEvent().getStartTimeInfo().doubleValue() == entity.getLastHandShapeEvent().getEndTimeInfo().doubleValue()) {

            ArrayList<HandShapeSubEntity> subEntities = entity.getSubEntities();

            // iterator returns sorted events
            HandShapeSubEntity superEntity = subEntities.get(0);

            if (superEntity.getEvents().isEmpty())
                return;

            Color entityColor = getValueColor(entity, entity.isSelected(),
                    entity.isHighlighted(), fieldID, entity.getField());

            Color color = entityColor;

            if (superEntity.isSelected()) {
                color = Color.RED;
            }

            superEntity.getFirstEvent();

            HandShapeEvent event = (HandShapeEvent) superEntity.getFirstEvent();
            g2d.setColor(color);

            drawHandShapeEvent(event, entity, entity.getEventsIterator(), LINE_UP_HANDSHAPE_ICON_MIDDLE_TIME, g2d, Y);

            return;
        }

		if (!drawTwoIdenticalHandshapes(entity, fieldID, g2d, Y)) {
			EventsIterator eventsIterator = entity.getEventsIterator();

			HandShapeEvent magnifiedEvent = null;
			Color entityColor = getValueColor(entity, entity.isSelected(),
					entity.isHighlighted(), fieldID, entity.getField());

			Color color = entityColor;
			g2d.setColor(color);

			ArrayList<HandShapeSubEntity> subEntities = entity.getSubEntities();

			// iterator returns sorted events
			HandShapeSubEntity superEntity = subEntities.get(0);

			if (superEntity.getEvents().isEmpty())
				return;

			if (superEntity.isSelected()) {
				color = Color.RED;
			} else {
				color = entityColor;
			}

			g2d.draw(new Line2D.Double(superEntity.getStartTimeCoordinate(),
					Y - 5, superEntity.getEndTimeCoordinate(), Y - 5));

			// draw entity base line
			g2d.draw(new Line2D.Double(
					superEntity.getStartTimeCoordinate() + 2, Y - 5,
					superEntity.getEndTimeCoordinate() - 2, Y - 5));

			// draw super entity icons
			ArrayList<HandShapeEvent> superEvents = superEntity.getEvents();

			HandShapeEvent superEntityLastEvent = (HandShapeEvent) superEntity
					.getLastEvent();

			g2d.setColor(color);

			for (int i = 0; i < superEvents.size() - 1; i++) {
				HandShapeEvent event = superEvents.get(i);
				// boolean lineByStartCoord = true;
				// if (event == superEntityLastEvent)
				// lineByStartCoord = false;
				// g2d.setColor(color);

				boolean drawImageOriginalSize = drawHandShapeEvent(event,
						entity, eventsIterator, LINE_UP_HANDSHAPE_ICON_START_TIME, g2d, Y);

				if (event.isMagnifiedImage() && !drawImageOriginalSize) {
					magnifiedEvent = event;
				}
			}

			// g2d.setColor(color);
			boolean drawImageOriginalSize1 = drawHandShapeEvent(
					superEntityLastEvent, entity, eventsIterator, LINE_UP_HANDSHAPE_ICON_END_TIME, g2d, Y);

			if (superEntityLastEvent.isMagnifiedImage()
					&& !drawImageOriginalSize1) {
				magnifiedEvent = superEntityLastEvent;
			}

			// draw new sub entity
			NewHandShapeEntity newEntity = entity.getNewHandShapeEntity();

			if (newEntity != null) {
				g2d.setColor(color);

				drawSubEntityBaseLine(newEntity.getStartTimeCoord(), newEntity
						.getEndTimeCoord(), (int) Y, g2d);
			}

			for (int i = 1; i < subEntities.size(); i++) {
				HandShapeSubEntity subEntity = subEntities.get(i);

				if (subEntity.isSelected()) {
					color = Color.RED;
				} else {
					color = entityColor;
				}

				g2d.setColor(color);

				// draw sub entity base line
				drawSubEntityBaseLine((int) subEntity.getStartTimeCoordinate(),
						(int) subEntity.getEndTimeCoordinate(), (int) Y, g2d);

				Event subEntityLastEvent = subEntity.getLastEvent();
				ArrayList subEvents = subEntity.getEvents();

				for (int j = 0; j < subEvents.size(); j++) {
					HandShapeEvent sEvent = (HandShapeEvent) subEvents.get(j);
					int lineByStartCoord = LINE_UP_HANDSHAPE_ICON_START_TIME;
					if (sEvent == subEntityLastEvent)
						lineByStartCoord = LINE_UP_HANDSHAPE_ICON_END_TIME;

                    g2d.setColor(color);

					boolean drawImageOriginalSize = drawHandShapeEvent(sEvent,
							entity, eventsIterator, lineByStartCoord, g2d, Y);

					if (sEvent.isMagnifiedImage() && !drawImageOriginalSize) {
						magnifiedEvent = sEvent;
					}
				}
			}

			if (magnifiedEvent != null) {
				Image img = magnifiedEvent.getLargeImage();

				Image image = img.getScaledInstance(img.getWidth(entity), 23,
						Image.SCALE_AREA_AVERAGING);

				int width1 = image.getWidth(entity) + 1;
				int height1 = image.getHeight(entity) + 1;

				double w = magnifiedEvent.getEndTimeCoordinate()
						- magnifiedEvent.getStartTimeCoordinate();
				double w1 = magnifiedEvent.getStartTimeCoordinate() + (w / 2);
				double startCoord = w1 - width1 / 2;

				g2d.drawRect((int) (startCoord - 1), (int) Y - 18 - 1, width1, height1);
				g2d.drawImage(image, (int) startCoord, (int) Y - 18, Color.WHITE,
						entity);
			}
		}
	}

	private boolean drawTwoIdenticalHandshapes(HandShapeEntity entity,
			String fieldID, Graphics2D g2d, double Y) {
		ArrayList events = entity.getEvents();

		if (events.size() != 2) {
			return false;
		}
		// start here!
		HandShapeEvent event1 = (HandShapeEvent) events.get(0);
		HandShapeEvent event2 = (HandShapeEvent) events.get(1);

		if (!event1.getImageID().equalsIgnoreCase(event2.getImageID())) {
			return false;
		}

		// LOAD IMAGES
		loadHandShapeIcons(entity, fieldID);

		double lenght = entity.getEndTimeCoordinate()
				- entity.getStartTimeCoordinate();

		Image imgL = event1.getLargeImage();
		Image imgS = event1.getSmallImage();
		int lL = imgL.getWidth(entity);
		int lS = imgS.getWidth(entity);
		// int lLR1 = (int)(lL*0.8);
		int lLR2 = (int) (lL * 1.3);
		int lSR1 = (int) (lS * 0.8);
		int lSR2 = (int) (lS * 1.2);

		if (lenght == lS) {
			// draw one small handshape
			g2d.drawImage(imgS, (int) entity.getStartTimeCoordinate(), (int) Y - 18,
					Color.WHITE, entity);
			event1.setRectangle(new Rectangle(
					(int) entity.getStartTimeCoordinate() - 1, 1, (int) lenght, 23));
			event2.setRectangle(new Rectangle(
					(int) entity.getStartTimeCoordinate() - 1, 1, (int) lenght, 23));
			return true;
		} else if (lenght < lS) {
			if (lenght >= lSR1) {
				// draw resized small image
				Image resizedSmallImage = imgS.getScaledInstance((int) lenght, 23,
						Image.SCALE_AREA_AVERAGING);
				g2d.drawImage(resizedSmallImage, (int) entity
						.getStartTimeCoordinate(), (int) Y - 18, Color.WHITE,
						entity);
				event1.setRectangle(new Rectangle((int) entity
						.getStartTimeCoordinate() - 1, 1, (int) lenght, 23));
				event2.setRectangle(new Rectangle((int) entity
						.getStartTimeCoordinate() - 1, 1, (int) lenght, 23));
				return true;
			} else {
				// draw rectangles
				return false;
			}
		} else if (lenght < lL) {
			if (lenght <= lSR2) {
				Image resizedSmallImage = imgS.getScaledInstance((int) lenght, 23,
						Image.SCALE_AREA_AVERAGING);
				g2d.drawImage(resizedSmallImage, (int) entity
						.getStartTimeCoordinate(), (int) Y - 18, Color.WHITE,
						entity);
				event1.setRectangle(new Rectangle((int) entity
						.getStartTimeCoordinate() - 1, 1, (int) lenght, 23));
				event2.setRectangle(new Rectangle((int) entity
						.getStartTimeCoordinate() - 1, 1, (int) lenght, 23));
				return true;
			} else {
				Image resizedLargeImage = imgL.getScaledInstance((int) lenght, 23,
						Image.SCALE_AREA_AVERAGING);
				g2d.drawImage(resizedLargeImage, (int) entity
						.getStartTimeCoordinate(), (int) Y - 18, Color.WHITE,
						entity);
				event1.setRectangle(new Rectangle((int) entity
						.getStartTimeCoordinate() - 1, 1, (int) lenght, 23));
				event2.setRectangle(new Rectangle((int) entity
						.getStartTimeCoordinate() - 1, 1, (int) lenght, 23));
				return true;
			}
		} else if (lenght == lL) {
			// draw large handshape
			g2d.drawImage(imgL, (int) entity.getStartTimeCoordinate(), (int) Y - 18,
					Color.WHITE, entity);
			event1.setRectangle(new Rectangle(
					(int) entity.getStartTimeCoordinate() - 1, 1, (int) lenght, 23));
			return true;
		} else if (lenght < 2 * lS) {
			if (lenght <= lLR2) {
				Image resizedLargeImage = imgL.getScaledInstance((int) lenght, 23,
						Image.SCALE_AREA_AVERAGING);
				g2d.drawImage(resizedLargeImage, (int) entity
						.getStartTimeCoordinate(), (int) Y - 18, Color.WHITE,
						entity);
				event1.setRectangle(new Rectangle((int) entity
						.getStartTimeCoordinate() - 1, 1, (int) lenght, 23));
				event2.setRectangle(new Rectangle((int) entity
						.getStartTimeCoordinate() - 1, 1, (int) lenght, 23));
				return true;
			} else {
				// draw a rectangle (black)
				int height = imgL.getHeight(null);
				g2d.setColor(Color.black);
				g2d.fillRect((int) entity.getStartTimeCoordinate(), (int) Y - 18,
						(int) lenght, height); // 23);

				double startCoord = entity.getStartTimeCoordinate()
						+ (lenght - lL) / 2;
				g2d.drawImage(imgL, (int) startCoord, (int) Y - 18, Color.WHITE,
						entity);
				event1.setRectangle(new Rectangle((int) entity
						.getStartTimeCoordinate() - 1, 1, (int) lenght, 23));
				event2.setRectangle(new Rectangle((int) entity
						.getStartTimeCoordinate() - 1, 1, (int) lenght, 23));
				return true;
			}
		}

		return false;
		/*
		 * else if(lenght > 2*lL) { // draw two large handshapes } else
		 * if(lenght > 2*lS) { // draw two small handshapes }
		 * 
		 * // if (lenght < img.getWidth(entity)) { if (lenght == 0) lenght = 1;
		 * 
		 * Image image = event1.getLargeImage(); //.getSmallImage(); //Image
		 * image = img2.getScaledInstance(lenght, 23, //
		 * Image.SCALE_AREA_AVERAGING
		 * 
		 * g2d.drawImage(image, entity.getStartTimeCoordinate(), (int) Y - 18,
		 * Color.WHITE, entity);
		 * 
		 * event1.setRectangle(new Rectangle(entity.getStartTimeCoordinate() -
		 * 1, 1, lWidth, 23)); event2.setRectangle(new
		 * Rectangle(entity.getStartTimeCoordinate() - 1 + lWidth, 1, lWidth,
		 * 23)); // g2d.setColor(Color.RED); //
		 * g2d.drawRect(entity.getStartTimeCoordinate() - 1, (int) Y - 18, //
		 * lWidth, 23); // g2d.drawRect(entity.getStartTimeCoordinate() - 1 +
		 * lWidth, (int) Y - // 18, lWidth, 23); return true;
		 */
	}

	private void drawSubEntityBaseLine(int x1, int x2, int Y, Graphics2D g2d) {
		g2d.draw(new Line2D.Double(x1 + 2, Y - 2, x2 - 2, Y - 2));
		g2d.draw(new Line2D.Double(x1 + 2, Y - 8, x2 - 2, Y - 8));
	}

	private static final int LINE_UP_HANDSHAPE_ICON_START_TIME = 0;
    private static final int LINE_UP_HANDSHAPE_ICON_END_TIME = 1;
    private static final int LINE_UP_HANDSHAPE_ICON_MIDDLE_TIME = 2;

    private boolean drawHandShapeEvent(HandShapeEvent event,
			HandShapeEntity entity, EventsIterator eventsIterator, int lineUp, Graphics2D g2d, double Y) {

		boolean drawSmallImage = false;

		if (event.isSelected())
			g2d.setColor(Color.RED);

		double startTimeCoordinate = event.getStartTimeCoordinate();

        Event previousEvent = null;
        Event nextEvent = null;

        if(eventsIterator != null) {
            previousEvent = eventsIterator.previous(event);
            nextEvent = eventsIterator.next(event);
        }

		Image img = event.getLargeImage();

		Image image = img.getScaledInstance(img.getWidth(entity), 23,
				Image.SCALE_AREA_AVERAGING);

		if (lineUp == LINE_UP_HANDSHAPE_ICON_START_TIME) {
			double w = 10000;
			if (nextEvent != null) {
				w = nextEvent.getStartTimeCoordinate()
						- event.getStartTimeCoordinate() - 5;
			}
			if (w < event.getSmallImage().getWidth(entity)) {
				g2d.drawRect((int) startTimeCoordinate - 1, (int) Y - 18 - 1, 2, 24);
				event.setRectangle(new Rectangle((int) startTimeCoordinate - 1, 1, 2, 24));
				return false;
			} else if (w < image.getWidth(entity)) {
				img = event.getSmallImage();
				image = img.getScaledInstance(img.getWidth(entity), 23,
						Image.SCALE_AREA_AVERAGING);
			}
		} else if(lineUp == LINE_UP_HANDSHAPE_ICON_END_TIME) {

			double w = event.getLargeImage().getWidth(entity);

			if (previousEvent != null) {
				w = event.getStartTimeCoordinate()
						- previousEvent.getStartTimeCoordinate() - 25;
			}

			if (w < event.getSmallImage().getWidth(entity)) {
				g2d.drawRect((int) startTimeCoordinate - 1, (int) Y - 18 - 1, 2, 24);
				event.setRectangle(new Rectangle((int) startTimeCoordinate - 1, 1, 2, 24));
				return false;
			} else if (w < image.getWidth(entity)) {
				image = event.getSmallImage();
				drawSmallImage = true;
			}
            startTimeCoordinate = event.getStartTimeCoordinate()
					- image.getWidth(entity);
		} else if(lineUp == LINE_UP_HANDSHAPE_ICON_MIDDLE_TIME) {
            // used only for one framed events
            double w = event.getLargeImage().getWidth(entity);

            if (previousEvent != null) {
                w = event.getStartTimeCoordinate()
                        - previousEvent.getStartTimeCoordinate() - 25;
            }

            if (w < event.getSmallImage().getWidth(entity)) {
                g2d.drawRect((int) startTimeCoordinate - 1, (int) Y - 18 - 1, 2, 24);
                event.setRectangle(new Rectangle((int) startTimeCoordinate - 1, 1, 2, 24));
                return false;
            } else if (w < image.getWidth(entity)) {
                image = event.getSmallImage();
                drawSmallImage = true;
            }
            startTimeCoordinate = event.getStartTimeCoordinate()
                    - image.getWidth(entity)/2;
        }

		int width = image.getWidth(entity) + 1;
		int height = image.getHeight(entity) + 1;

		g2d.drawRect((int) startTimeCoordinate - 1, (int) Y - 18 - 1, width, height);
		event.setRectangle(new Rectangle((int) startTimeCoordinate, 1, width, height));
		g2d.drawImage(image, (int) startTimeCoordinate, (int) Y - 18, Color.WHITE, entity);
		return !drawSmallImage;
	}

	private void loadHandShapeIcons(HandShapeEntity entity, String fieldID) {
		ArrayList hShapes = entity.getHandShapeEvents();
		HandShapesImageManager hsManager = SS3Singleton
				.getHandShapesImageManager();

		SS3Field csField = scheme.getField(fieldID);
		for (int i = 0; i < hShapes.size(); i++) {
			HandShapeEvent event = (HandShapeEvent) hShapes.get(i);

			if (event.getLargeImage() == null || event.getSmallImage() == null) {
				String id = event.getID();
				SS3FieldValue value = csField.getFieldValue(id);

				SS3FieldTypeImg ftImg = scheme.getImage(value.getIid());

				if (event.getLargeImage() == null) {

					Image lImage = hsManager.getImage(ftImg.getPath1());
					Image imgL = lImage.getScaledInstance(lImage
							.getWidth(entity), 23, Image.SCALE_AREA_AVERAGING);
					event.setLargeImage(imgL);
				}

				if (event.getSmallImage() == null) {

					Image sImage = hsManager.getImage(ftImg.getPath2());
					Image imgS = sImage.getScaledInstance(sImage
							.getWidth(entity), 23, Image.SCALE_AREA_AVERAGING);
					event.setSmallImage(imgS);
				}
			}
		}
	}

	public void drawOnset(ChainedEvent event, Graphics2D g2d, double Y) {
		double X1 = event.getStartTimeCoordinate();
		double X2 = event.getEndTimeCoordinate();

		Color color = g2d.getColor();
		
		if(!event.isSelected()) {
			g2d.setColor(onsetUnselectedColor);
		}
		
		if (X2 - X1 <= 6) {
			double X3 = X2 - 5;
			double Y1 = Y - 8;
			g2d.draw(new Line2D.Double(X2, Y1, X2, Y - 3));

			g2d.draw(new Line2D.Double(X2, Y1 + 3, X3, Y1 + 3));
			g2d.draw(new Line2D.Double(X3 + 3, Y1, X3, Y1 + 3));
			g2d.draw(new Line2D.Double(X3 + 3, Y1 + 6, X3, Y1 + 3));
		} else {
			g2d.draw(new Line2D.Double(X1 + 5, Y - 3, X1, Y));
			g2d.draw(new Line2D.Double(X1 + 5, Y + 3, X1, Y));

			g2d.draw(new Line2D.Double(X1, Y, X2, Y));
		}
		
		if(!event.isSelected()) {
			g2d.setColor(color);
		}
	}

	public void drawOffset(ChainedEvent event, Graphics2D g2d, double Y) {
		double X1 = event.getStartTimeCoordinate();
		double X2 = event.getEndTimeCoordinate();

		Color color = g2d.getColor();
		
		if(!event.isSelected()) {
			g2d.setColor(offsetUnselectedColor);
		}
		
		if (X2 - X1 <= 6) {
			double X3 = X1 + 5;
			double Y1 = Y - 8;
			g2d.draw(new Line2D.Double(X1, Y1, X1, Y - 3));

			g2d.draw(new Line2D.Double(X1, Y1 + 3, X3, Y1 + 3));
			g2d.draw(new Line2D.Double(X3 - 3, Y1, X3, Y1 + 3));
			g2d.draw(new Line2D.Double(X3 - 3, Y1 + 6, X3, Y1 + 3));
		} else {
			g2d.draw(new Line2D.Double(X2, Y, X2 - 5, Y - 3));
			g2d.draw(new Line2D.Double(X2, Y, X2 - 5, Y + 3));
			g2d.draw(new Line2D.Double(X1, Y, X2, Y));
		}

		if(!event.isSelected()) {
			g2d.setColor(color);
		}
	}
	
	public void drawOnset(GestureChainedEvent event, Graphics2D g2d, double Y) {
		double X1 = event.getStartTimeCoordinate();
		double X2 = event.getEndTimeCoordinate();

		Color color = g2d.getColor();
		
		if(!event.isSelected()) {
			g2d.setColor(onsetUnselectedColor);
		}
		
		if (X2 - X1 <= 6) {
			double X3 = X2 - 5;
			double Y1 = Y - 8;
			g2d.draw(new Line2D.Double(X2, Y1, X2, Y - 3));

			g2d.draw(new Line2D.Double(X2, Y1 + 3, X3, Y1 + 3));
			g2d.draw(new Line2D.Double(X3 + 3, Y1, X3, Y1 + 3));
			g2d.draw(new Line2D.Double(X3 + 3, Y1 + 6, X3, Y1 + 3));
		} else {
			g2d.draw(new Line2D.Double(X1 + 5, Y - 3, X1, Y));
			g2d.draw(new Line2D.Double(X1 + 5, Y + 3, X1, Y));

			g2d.draw(new Line2D.Double(X1, Y, X2, Y));
		}
		
		if(!event.isSelected()) {
			g2d.setColor(color);
		}
	}

	public void drawOffset(GestureChainedEvent event, Graphics2D g2d, double Y) {
		double X1 = event.getStartTimeCoordinate();
		double X2 = event.getEndTimeCoordinate();

		Color color = g2d.getColor();
		
		if(!event.isSelected()) {
			g2d.setColor(offsetUnselectedColor);
		}
		
		if (X2 - X1 <= 6) {
			double X3 = X1 + 5;
			double Y1 = Y - 8;
			g2d.draw(new Line2D.Double(X1, Y1, X1, Y - 3));

			g2d.draw(new Line2D.Double(X1, Y1 + 3, X3, Y1 + 3));
			g2d.draw(new Line2D.Double(X3 - 3, Y1, X3, Y1 + 3));
			g2d.draw(new Line2D.Double(X3 - 3, Y1 + 6, X3, Y1 + 3));
		} else {
			g2d.draw(new Line2D.Double(X2, Y, X2 - 5, Y - 3));
			g2d.draw(new Line2D.Double(X2, Y, X2 - 5, Y + 3));
			g2d.draw(new Line2D.Double(X1, Y, X2, Y));
		}

		if(!event.isSelected()) {
			g2d.setColor(color);
		}
	}
	
	public void drawRelaxed(GestureChainedEvent event, Graphics2D g2d, double Y) {
		double X1 = event.getStartTimeCoordinate();
		double X2 = event.getEndTimeCoordinate();

		double cX = X1;
		while (cX < X2 - 5) {
			g2d.draw(new Line2D.Double(cX, Y, cX + 2, Y));
			cX += 4;
		}

		g2d.draw(new Line2D.Double(X2, Y, X2 - 5, Y - 3));
		g2d.draw(new Line2D.Double(X2, Y, X2 - 5, Y + 3));
		//g2d.draw(new Line2D.Double(X1, Y + 3, X2, Y + 3));
		//g2d.draw(new Line2D.Double(X1, Y + 5, X1, Y - 2));
		//g2d.draw(new Line2D.Double(X2, Y + 5, X2, Y - 3));
	}
	
	public void drawOnset(ChainedApexEvent event, Graphics2D g2d, double Y) {
		double X1 = event.getStartTimeCoordinate();
		double X2 = event.getEndTimeCoordinate();

		Color color = g2d.getColor();
		
		if(!event.isSelected()) {
			g2d.setColor(onsetUnselectedColor);
		}
		
		if (X2 - X1 <= 6) {
			double X3 = X2 - 5;
			double Y1 = Y - 8;
			g2d.draw(new Line2D.Double(X2, Y1, X2, Y - 3));

			g2d.draw(new Line2D.Double(X2, Y1 + 3, X3, Y1 + 3));
			g2d.draw(new Line2D.Double(X3 + 3, Y1, X3, Y1 + 3));
			g2d.draw(new Line2D.Double(X3 + 3, Y1 + 6, X3, Y1 + 3));
		} else {
			g2d.draw(new Line2D.Double(X1 + 5, Y - 3, X1, Y));
			g2d.draw(new Line2D.Double(X1 + 5, Y + 3, X1, Y));

			g2d.draw(new Line2D.Double(X1, Y, X2, Y));
		}
		
		if(!event.isSelected()) {
			g2d.setColor(color);
		}
	}

	public void drawOffset(ChainedApexEvent event, Graphics2D g2d, double Y) {
		double X1 = event.getStartTimeCoordinate();
		double X2 = event.getEndTimeCoordinate();

		Color color = g2d.getColor();
		
		if(!event.isSelected()) {
			g2d.setColor(offsetUnselectedColor);
		}
		
		if (X2 - X1 <= 6) {
			double X3 = X1 + 5;
			double Y1 = Y - 8;
			g2d.draw(new Line2D.Double(X1, Y1, X1, Y - 3));

			g2d.draw(new Line2D.Double(X1, Y1 + 3, X3, Y1 + 3));
			g2d.draw(new Line2D.Double(X3 - 3, Y1, X3, Y1 + 3));
			g2d.draw(new Line2D.Double(X3 - 3, Y1 + 6, X3, Y1 + 3));
		} else {
			g2d.draw(new Line2D.Double(X2, Y, X2 - 5, Y - 3));
			g2d.draw(new Line2D.Double(X2, Y, X2 - 5, Y + 3));
			g2d.draw(new Line2D.Double(X1, Y, X2, Y));
		}

		if(!event.isSelected()) {
			g2d.setColor(color);
		}
	}

	public void drawHold(ChainedEvent event, Graphics2D g2d, double Y) {
		double X1 = event.getStartTimeCoordinate();
		double X2 = event.getEndTimeCoordinate();

		double cX = X1;
		while (cX < X2 - 5) {
			g2d.draw(new Line2D.Double(cX, Y, cX + 5, Y));
			cX += 10;
		}

		g2d.draw(new Line2D.Double(X2, Y, X2 - 5, Y - 3));
		g2d.draw(new Line2D.Double(X2, Y, X2 - 5, Y + 3));
		g2d.draw(new Line2D.Double(X1, Y + 3, X2, Y + 3));
		g2d.draw(new Line2D.Double(X1, Y + 5, X1, Y - 2));
		g2d.draw(new Line2D.Double(X2, Y + 5, X2, Y - 3));
	}

	private Color getValueColor(Event event, boolean selected,
			boolean eventHighlighted, String fieldID, Field field) {

		if (selected) {
			return entitySelectedColor;
		} else if (event != null && event.isSelected()) {
			return eventSelectedColor;
		} else if (eventHighlighted || event.isHighlighted()) {
			return eventHighlightedColor;
		} else if (event.getColor() != null) {
			return event.getColor();
		} else if (field != null && field.getColor() != null) {
			return field.getColor();
		} else {
			SS3Field csField = scheme.getField(fieldID);
			if (csField != null)
				return csField.getColor();
			return scheme.getColor();
		}
	}

	private Color getHoldColor(Event event, boolean entitySelected) {
		if (entitySelected) {
			return entitySelectedColor;
		} else if (event.isSelected()) {
			return eventSelectedColor;
		} else if (event.isHighlighted()) {
			return eventHighlightedColor;
		} else {
			return scheme.getColor();
		}
	}

	public void drawInitialHold(ChainedEvent event, Graphics2D g2d, double Y) {

		double X1 = event.getStartTimeCoordinate();
		double X2 = event.getEndTimeCoordinate();

		if (X2 - X1 <= 6) {
			double X3 = X1 + 5;
			double Y1 = Y - 6;

			g2d.draw(new Line2D.Double(X1, Y1 + 3, X3, Y1 + 3));
			g2d.draw(new Line2D.Double(X1, Y1 + 2, X3, Y1 + 2));
			g2d.draw(new Line2D.Double(X1, Y1 + 1, X3, Y1 + 1));

			g2d.draw(new Line2D.Double(X1, Y1, X1, Y + 3));
			g2d.draw(new Line2D.Double(X2, Y1, X2, Y + 3));
		} else {

			double cX = X1;
			while (cX < X2 - 5) {
				g2d.draw(new Line2D.Double(cX, Y, cX + 5, Y));
				cX += 10;
			}

			g2d.draw(new Line2D.Double(X1, Y, X1 + 5, Y - 3));
			g2d.draw(new Line2D.Double(X1, Y, X1 + 5, Y + 3));

			g2d.draw(new Line2D.Double(X1, Y + 3, X2, Y + 3));
			g2d.draw(new Line2D.Double(X1, Y + 5, X1, Y - 2));
			g2d.draw(new Line2D.Double(X2, Y + 5, X2, Y - 3));
		}
	}
	
	public void drawInitialHold(GestureChainedEvent event, Graphics2D g2d, double Y) {

		double X1 = event.getStartTimeCoordinate();
		double X2 = event.getEndTimeCoordinate();

		if (X2 - X1 <= 6) {
			double X3 = X1 + 5;
			double Y1 = Y - 6;

			g2d.draw(new Line2D.Double(X1, Y1 + 3, X3, Y1 + 3));
			g2d.draw(new Line2D.Double(X1, Y1 + 2, X3, Y1 + 2));
			g2d.draw(new Line2D.Double(X1, Y1 + 1, X3, Y1 + 1));

			g2d.draw(new Line2D.Double(X1, Y1, X1, Y + 3));
			g2d.draw(new Line2D.Double(X2, Y1, X2, Y + 3));
		} else {

			double cX = X1;
			while (cX < X2 - 5) {
				g2d.draw(new Line2D.Double(cX, Y, cX + 5, Y));
				cX += 10;
			}

			g2d.draw(new Line2D.Double(X1, Y, X1 + 5, Y - 3));
			g2d.draw(new Line2D.Double(X1, Y, X1 + 5, Y + 3));

			g2d.draw(new Line2D.Double(X1, Y + 3, X2, Y + 3));
			g2d.draw(new Line2D.Double(X1, Y + 5, X1, Y - 2));
			g2d.draw(new Line2D.Double(X2, Y + 5, X2, Y - 3));
		}
	}

	public void drawFinalHold(ChainedEvent event, Graphics2D g2d, double Y) {

		double X1 = event.getStartTimeCoordinate();
		double X2 = event.getEndTimeCoordinate();

		if (X2 - X1 <= 6) {
			double X3 = X2 - 5;
			double Y1 = Y - 6;

			g2d.draw(new Line2D.Double(X2, Y1 + 3, X3, Y1 + 3));
			g2d.draw(new Line2D.Double(X2, Y1 + 2, X3, Y1 + 2));
			g2d.draw(new Line2D.Double(X2, Y1 + 1, X3, Y1 + 1));

			g2d.draw(new Line2D.Double(X1, Y1, X1, Y + 3));
			g2d.draw(new Line2D.Double(X2, Y1, X2, Y + 3));
		} else {

			double cX = X1;
			while (cX < X2 - 5) {
				g2d.draw(new Line2D.Double(cX, Y, cX + 5, Y));
				cX += 10;
			}

			g2d.draw(new Line2D.Double(X2, Y, X2 - 5, Y - 3));
			g2d.draw(new Line2D.Double(X2, Y, X2 - 5, Y + 3));
			g2d.draw(new Line2D.Double(X1, Y + 3, X2, Y + 3));
			g2d.draw(new Line2D.Double(X1, Y + 5, X1, Y - 2));
			g2d.draw(new Line2D.Double(X2, Y + 5, X2, Y - 3));
		}
	}
	
	public void drawFinalHold(GestureChainedEvent event, Graphics2D g2d, double Y) {

		double X1 = event.getStartTimeCoordinate();
		double X2 = event.getEndTimeCoordinate();

		if (X2 - X1 <= 6) {
			double X3 = X2 - 5;
			double Y1 = Y - 6;

			g2d.draw(new Line2D.Double(X2, Y1 + 3, X3, Y1 + 3));
			g2d.draw(new Line2D.Double(X2, Y1 + 2, X3, Y1 + 2));
			g2d.draw(new Line2D.Double(X2, Y1 + 1, X3, Y1 + 1));

			g2d.draw(new Line2D.Double(X1, Y1, X1, Y + 3));
			g2d.draw(new Line2D.Double(X2, Y1, X2, Y + 3));
		} else {

			double cX = X1;
			while (cX < X2 - 5) {
				g2d.draw(new Line2D.Double(cX, Y, cX + 5, Y));
				cX += 10;
			}

			g2d.draw(new Line2D.Double(X2, Y, X2 - 5, Y - 3));
			g2d.draw(new Line2D.Double(X2, Y, X2 - 5, Y + 3));
			g2d.draw(new Line2D.Double(X1, Y + 3, X2, Y + 3));
			g2d.draw(new Line2D.Double(X1, Y + 5, X1, Y - 2));
			g2d.draw(new Line2D.Double(X2, Y + 5, X2, Y - 3));
		}
	}

	public void drawValue(ChainedEvent event, Graphics2D g2d, double Y) {
		double X1 = event.getStartTimeCoordinate();
		double X2 = event.getEndTimeCoordinate();

		if (event.isSelected()) {
			g2d.setColor(Color.red);
		} else {
			g2d.setColor(event.getColor());
		}

		g2d.draw(new Line2D.Double(X1, Y, X2, Y));

		if (X1 == X2) {
			g2d.draw(new Line2D.Double(X1, Y + 1, X2 + 10, Y + 1));
			g2d.fillOval((int) X1, (int) Y - 1, 6, 4);
		} else {
			g2d.draw(new Line2D.Double(X1, Y + 1, X2, Y + 1));
			g2d.draw(new Line2D.Double(X1, Y - 2, X1, Y + 3));
			g2d.draw(new Line2D.Double(X2, Y - 2, X2, Y + 3));
		}
	}
	
	public void drawValue(GestureChainedEvent event, Graphics2D g2d, double Y) {
		double X1 = event.getStartTimeCoordinate();
		double X2 = event.getEndTimeCoordinate();

		if (event.isSelected()) {
			g2d.setColor(Color.red);
		} else {
			g2d.setColor(event.getColor());
		}

		g2d.draw(new Line2D.Double(X1, Y, X2, Y));

		if (X1 == X2) {
			g2d.draw(new Line2D.Double(X1, Y + 1, X2 + 10, Y + 1));
			g2d.fillOval((int) X1, (int) Y - 1, 6, 4);
		} else {
			g2d.draw(new Line2D.Double(X1, Y + 1, X2, Y + 1));
			g2d.draw(new Line2D.Double(X1, Y - 2, X1, Y + 3));
			g2d.draw(new Line2D.Double(X2, Y - 2, X2, Y + 3));
		}
	}
	
	public void drawValue(ChainedApexEvent event, Graphics2D g2d, double Y) {
		double X1 = event.getStartTimeCoordinate();
		double X2 = event.getEndTimeCoordinate();

		if (event.isSelected()) {
			g2d.setColor(Color.red);
		} else {
			g2d.setColor(event.getColor());
		}

		g2d.draw(new Line2D.Double(X1, Y, X2, Y));

		if (X1 == X2) {
			g2d.draw(new Line2D.Double(X1, Y + 1, X2 + 10, Y + 1));
			g2d.fillOval((int) X1, (int) Y - 1, 6, 4);
		} else {
			g2d.draw(new Line2D.Double(X1, Y + 1, X2, Y + 1));
			g2d.draw(new Line2D.Double(X1, Y - 2, X1, Y + 3));
			g2d.draw(new Line2D.Double(X2, Y - 2, X2, Y + 3));
		}
	}

	private void drawValueLabel(ChainedEvent event, EventsEntity entity,
			Graphics2D g2d, double Y, FontMetrics fontMetrics, String fieldID,
			EventsEntity previousEntity, EventsEntity followingEntity) {

		double X1 = event.getStartTimeCoordinate();
		double X2 = event.getEndTimeCoordinate();

		String valueID = event.getID();

		SS3Field currentField = scheme.getField(fieldID);
				
		SS3Field.Alignment alignment = currentField.getAlignment();
		SS3FieldValue csValue = currentField.getFieldValue(valueID);

		if (csValue != null && !(event instanceof GlossChainedEvent)) {
			event.setText(csValue.getLabelOrName());
			event.setToolTipText(csValue.getName());
			g2d.setColor(Color.BLACK);
		}

		String label = event.getText() == null ? "" : event.getText();
		String prefix = currentField.getPrefix();

		g2d.setColor(Color.DARK_GRAY);

		label = (prefix == null || prefix.isEmpty()) ? label : prefix + ":" + label;

		int length = fontMetrics.stringWidth(label);

		if (alignment == SS3Field.Alignment.RIGHT_LEFT) {
			g2d.drawString(label, (int) X1, (int) Y - 1);
			g2d.drawString(label, (int) (X2 - length), (int) Y - 1);
		} else {
			double dX;

			if (alignment == SS3Field.Alignment.CENTER) {
				dX = (X2 - X1 - length) / 2 + X1;
			} else if (alignment == SS3Field.Alignment.RIGHT) {
				dX = X2 - length;
			} else {
				dX = X1;
			}

			g2d.drawString(label, (int) dX, (int) Y - 1);
		}
	}
	
	private void drawValueLabel(GestureChainedEvent event, GestureChainedEventsEntity entity,
			Graphics2D g2d, double Y, FontMetrics fontMetrics, String fieldID,
			EventsEntity previousEntity, EventsEntity followingEntity) {

		double X1 = event.getStartTimeCoordinate();
		double X2 = event.getEndTimeCoordinate();

		String valueID = event.getID();

		SS3Field currentField = scheme.getField(fieldID);
				
		SS3Field.Alignment alignment = currentField.getAlignment();
		SS3FieldValue csValue = currentField.getFieldValue(valueID);

		if (csValue != null && !(event instanceof GestureEvent)) {
			event.setText(csValue.getLabelOrName());
			event.setToolTipText(csValue.getName());
			g2d.setColor(Color.BLACK);
		}

		String label = event.getText() == null ? "" : event.getText();
		String prefix = currentField.getPrefix();

		g2d.setColor(Color.DARK_GRAY);
		
		GestureEventsEntity entityForLabelling = null;

		label = (prefix == null || prefix.isEmpty()) ? label : prefix + ":" + label;
		
		if (entity instanceof GestureEventsEntity)
			entityForLabelling = (GestureEventsEntity) entity;
		else {
			String refEntityID = entity.getRefEntityId();
			
			FieldWrappersCollection fwc = SS3Singleton.getSignStreamApplication()
			.getSignStreamSegmentPanel().getUtteranceView()
			.getSpeechView().getFieldWrappersCollection();
			
			FieldWrapper domGestureFieldWrapper = fwc
					.getFieldWrapperWithFieldID(Util.DOM_GESTURE_FIELD_ID);
			
			FieldWrapper ndomGestureFieldWrapper = fwc
					.getFieldWrapperWithFieldID(Util.NDOM_GESTURE_FIELD_ID);
			
			if (domGestureFieldWrapper != null && domGestureFieldWrapper.getField() != null)
				entityForLabelling = (GestureEventsEntity) domGestureFieldWrapper.getField().getEntity(refEntityID);	
			
			if (entityForLabelling == null && 
					ndomGestureFieldWrapper != null && ndomGestureFieldWrapper.getField() != null)
				entityForLabelling = (GestureEventsEntity) ndomGestureFieldWrapper.getField().getEntity(refEntityID);			
		}
		
		if (entityForLabelling != null) {
			boolean isTwoHanded = entityForLabelling.getGestureEntityProperties().isTwoHanded();
			boolean isIncompleteGesture = entityForLabelling.getIncompleteGesture();
			boolean isAlternating = entityForLabelling.getGestureEntityProperties()
				.getTwoHandedHandshapesClassification()
				.equalsIgnoreCase(GestureEntityProperties.ALTERNATING_SAME_HANDSHAPES_DOM_NDOM);
			boolean showUniqueNumber = entityForLabelling.getShowUniqueNumber();
			String uniqueNumber = entityForLabelling.getUniqueNumber();
			
			if (isAlternating) {
				label = ALT_HANDS_STRING + label;
			}
			
			if (isTwoHanded) {
				label = TWO_HANDED_STRING + label;
			}
			
			if (isIncompleteGesture) {
				label = INCOMPLETE_STRING + label;
			}
			
			if (showUniqueNumber && uniqueNumber != null && !uniqueNumber.isEmpty()) {
				label = label + "-" + uniqueNumber;
			}
		}

		int length = fontMetrics.stringWidth(label);

		if (alignment == SS3Field.Alignment.RIGHT_LEFT) {
			g2d.drawString(label, (int) X1, (int) Y - 1);
			g2d.drawString(label, (int) (X2 - length), (int) Y - 1);
		} else {
			double dX;

			if (alignment == SS3Field.Alignment.CENTER) {
				dX = (X2 - X1 - length) / 2 + X1;
			} else if (alignment == SS3Field.Alignment.RIGHT) {
				dX = X2 - length;
			} else {
				dX = X1;
			}

			g2d.drawString(label, (int) dX, (int) Y - 1);
		}
	}
	
	private void drawValueLabel(ChainedApexEvent event, EventsEntity entity,
			Graphics2D g2d, double Y, FontMetrics fontMetrics, String fieldID,
			EventsEntity previousEntity, EventsEntity followingEntity) {

		double X1 = event.getStartTimeCoordinate();
		double X2 = event.getEndTimeCoordinate();

		String valueID = event.getID();

		SS3Field currentField = scheme.getField(fieldID);
				
		SS3Field.Alignment alignment = currentField.getAlignment();
		SS3FieldValue csValue = currentField.getFieldValue(valueID);

		if (csValue != null && !(event instanceof ApexChainedApexEvent)) {
			event.setText(csValue.getLabelOrName());
			event.setToolTipText(csValue.getName());
			g2d.setColor(Color.BLACK);
		}

		String label = event.getText() == null ? "" : event.getText();
		String prefix = currentField.getPrefix();

		g2d.setColor(Color.DARK_GRAY);

		label = (prefix == null || prefix.isEmpty()) ? label : prefix + ":" + label;

		int length = fontMetrics.stringWidth(label);

		if (alignment == SS3Field.Alignment.RIGHT_LEFT) {
			g2d.drawString(label, (int) X1, (int) Y - 1);
			g2d.drawString(label, (int) (X2 - length), (int) Y - 1);
		} else {
			double dX;

			if (alignment == SS3Field.Alignment.CENTER) {
				dX = (X2 - X1 - length) / 2 + X1;
			} else if (alignment == SS3Field.Alignment.RIGHT) {
				dX = X2 - length;
			} else {
				dX = X1;
			}

			g2d.drawString(label, (int) dX, (int) Y - 1);
		}
	}

	public void drawFreeTextLabel(ChainedEvent event, Graphics2D g2d, double Y,
			FontMetrics fontMetrics, String fieldID) {

		GlossChainedEvent ftEvent = (GlossChainedEvent) event;

        double x = ftEvent.getStartTimeCoordinate();

        if(x < 5) {
           x = 10;
           Time start = ftEvent.getStartTimeInfo();
           start = event.getSignStreamSegmentPanel().getZoomer().getCoordinateTimeConvertor()
           		.convertCoordinateToTimeInfo(10);
           ftEvent.setStartTimeInfo(start);
        }
        g2d.setColor(Color.BLACK);
		g2d.drawString(ftEvent.getText(), (int) x, (int)Y - 1);
		return;
	}

	public Color getHighlightedColor() {
		return Color.BLUE;
	}

	public void drawEnteredGestureText(
			ArrayList<edu.bu.signstream.grepresentation.fields.gestureField.GestureText> enteredText, Graphics2D g2d, double Y,
			FontMetrics fontMetrics, GestureField gestField) {
		g2d.setFont(fontMetrics.getFont());

		for (edu.bu.signstream.grepresentation.fields.gestureField.GestureText text : enteredText) {
			if (text.isSelected()) {
				g2d.setColor(Color.RED);
			} else {
				g2d.setColor(Color.DARK_GRAY);
			}

			double endCoord = text.getStartCoord() + fontMetrics.stringWidth(text.getText());
			text.setEndCoord(endCoord);
			g2d.drawString(text.getText(), (int) text.getStartCoord(), (int) Y - 1);
		}
	}

	public void drawNewEvent(NewGestureChainedEvent newEvent, Graphics2D g2d, double Y) {
		double X1 = newEvent.getStartTimeCoordinate();
		double X2 = newEvent.getEndTimeCoordinate();

		g2d.fillOval((int) X1, (int) Y - 5, 4, 4);
		if (X2 > -1)
			g2d.draw(new Line2D.Double(X1, Y - 5, X2, Y - 5));
	}

	public void drawGestureEntity(GestureEventsEntity entity,
			Graphics2D g2d, double Y, FontMetrics fontMetrics, String fieldID,
			GestureEventsEntity previousEntity, GestureEventsEntity followingEntity) {
		Field field = entity.getField();

		GestureChainedEventsCollection collection = entity
				.getGestureChainedEventsCollection();
		GestureChainedEvent preparation = collection.getPreparationEvent();
		GestureChainedEvent prestrokeHold = collection.getPrestrokeHoldEvent();
		GestureChainedEvent value = collection.getValueEvent();
		GestureEvent gestureValue = collection.getTextValueEvent();
		GestureChainedEvent poststrokeHold = collection.getPoststrokeHoldEvent();
		GestureChainedEvent retraction = collection.getRetractionEvent();
		GestureChainedEvent relaxed = collection.getRelaxedEvent();

		boolean selected = entity.isSelected();

		// paint compounds
		if (entity.isStartCompound()) {
			GestureCompoundable cmp = entity.getStartCompound();
			double coord = cmp.getEndTimeCoordinate();

			double x = coord + (entity.getStartTimeCoordinate() - coord) / 2;
            g2d.setColor(Color.RED);
			g2d.drawString("+", (int) (x - 4), (int) Y);
		}

		if (preparation != null) {
			g2d.setColor(getHoldColor(preparation, selected));
			drawOnset(preparation, g2d, Y);
		}

		if (prestrokeHold != null) {
			g2d.setColor(getHoldColor(prestrokeHold, selected));
			drawInitialHold(prestrokeHold, g2d, Y);
		}
		
		if (gestureValue != null) {

			if (entity.getField() != null
					&& entity.getField().getColor() != null) {
				g2d.setColor(entity.getField().getColor());
			} else {
				g2d.setColor(getValueColor(gestureValue, selected, entity
						.isHighlighted(), fieldID, field));
			}
			
			drawTextValue(entity, gestureValue, g2d, Y, fontMetrics, fieldID,
					previousEntity, followingEntity);

			boolean isLockedDomNdomAlignments = entity
					.isLockedDomNdomAlignments();
			// boolean isLockedGlossPhonAlignments =
			// entity.isLockedGlossPhonAlignments();
			// drawLock(g2d, glossValue.getStartTimeCoordinate(), (int) Y,
			// Color.BLUE, isLockedDomNdomAlignments,
			// isLockedGlossPhonAlignments);
			// drawLock(g2d, glossValue.getEndTimeCoordinate(), (int) Y,
			// Color.BLUE, isLockedDomNdomAlignments,
			// isLockedGlossPhonAlignments);

			drawLock(g2d, (int) gestureValue.getStartTimeCoordinate(), (int) Y,
					Color.BLUE, isLockedDomNdomAlignments);
			drawLock(g2d, (int) gestureValue.getEndTimeCoordinate(), (int) Y,
					Color.BLUE, isLockedDomNdomAlignments);
		}

		if (value != null) {
			Color color = null;
			if (entity.getField() != null
					&& entity.getField().getColor() != null) {
				color = entity.getField().getColor();
			} else {
				color = getValueColor(value, selected, entity.isHighlighted(),
						fieldID, field);
			}
			g2d.setColor(color);
			drawValue(value, g2d, Y);
			drawValueLabel(value, entity, g2d, Y, fontMetrics, fieldID,
					previousEntity, followingEntity);

			boolean isLockedDomNdomAlignments = entity
					.isLockedDomNdomAlignments();
			boolean isLockedGesturePhonAlignments = entity
					.isLockedGesturePhonAlignments();

			drawLock(g2d, (int) gestureValue.getStartTimeCoordinate(), (int) Y,
					Color.BLUE, isLockedDomNdomAlignments,
					isLockedGesturePhonAlignments);
			drawLock(g2d, (int) gestureValue.getEndTimeCoordinate(), (int) Y,
					Color.BLUE, isLockedDomNdomAlignments,
					isLockedGesturePhonAlignments);

			// if (entity.isLockedDomNdomAlignments()) {
			// drawLock(g2d, value.getStartTimeCoordinate(), (int) Y,
			// eventHighlightedColor);
			// drawLock(g2d, value.getEndTimeCoordinate(), (int) Y,
			// eventHighlightedColor);
			// }
		}

		if (poststrokeHold != null) {
			g2d.setColor(getHoldColor(poststrokeHold, selected));
			drawFinalHold(poststrokeHold, g2d, Y);
		}

		if (retraction != null) {
			g2d.setColor(getHoldColor(retraction, selected));
			drawOffset(retraction, g2d, Y);
		}
		
		if (relaxed != null) {
			g2d.setColor(getHoldColor(relaxed, selected));
			drawRelaxed(relaxed, g2d, Y);
		}
	}
}
