package edu.bu.signstream.grepresentation.fields.apexField;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;

import edu.bu.signstream.common.util.CoordinateTimeConverter;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.TrackValue;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.fields.CollectionManager;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.ui.ConfirmationPanel;

/**
 * Manages collection of ChainedApexEventsEntity objects.
 */
public class ChainedApexEventsCollectionManager extends CollectionManager {

	/**
	 * collection of ChainedApexEventsEntity objects to manage
	 */
	private ChainedApexEventsCollection collection = null;
	private SignStreamSegmentPanel segmentPanel = null;
	private CoordinateTimeConverter convertor = null;

	/**
	 * 
	 * @param collection
	 *            must not be null
	 */
	public ChainedApexEventsCollectionManager(ChainedApexEventsCollection collection,
			SignStreamSegmentPanel segmentPanel) {
		this.collection = collection;
		this.segmentPanel = segmentPanel;

		this.convertor = segmentPanel.getZoomer().getCoordinateTimeConvertor();
	}
	
	public void selectChainedApexEvent(ChainedApexEvent event) {
		if (event != null)
			event.select();
	}

	public void unselectChainedApexEvent(ChainedApexEvent event) {
		event.unselect();
	}

	@Override
	public void unselectEntity() {
		if (collection.getOnsetEvent() != null)
			collection.getOnsetEvent().unselect();
		if (collection.getInitialHoldEvent() != null)
			collection.getInitialHoldEvent().unselect();
		if (collection.getValueEvent() != null)
			collection.getValueEvent().unselect();
		if (collection.getTextValueEvent() != null)
			collection.getTextValueEvent().unselect();
		if (collection.getFinalHoldEvent() != null)
			collection.getFinalHoldEvent().unselect();
		if (collection.getOffsetEvent() != null)
			collection.getOffsetEvent().unselect();
	}

	@Override
	public Event getSelectedEvent() {
		if (collection.getOnsetEvent() != null
				&& collection.getOnsetEvent().isSelected())
			return collection.getOnsetEvent();
		if (collection.getInitialHoldEvent() != null
				&& collection.getInitialHoldEvent().isSelected())
			return collection.getInitialHoldEvent();
		if (collection.getValueEvent() != null
				&& collection.getValueEvent().isSelected())
			return collection.getValueEvent();
		if (collection.getTextValueEvent() != null
				&& collection.getTextValueEvent().isSelected())
			return collection.getTextValueEvent();
		if (collection.getFinalHoldEvent() != null
				&& collection.getFinalHoldEvent().isSelected())
			return collection.getFinalHoldEvent();
		if (collection.getOffsetEvent() != null
				&& collection.getOffsetEvent().isSelected())
			return collection.getOffsetEvent();
		return null; // throw Exception
	}

//	public void setStartTimeOrRemoveEvent(int time) {
//		Event firstEvent = getFirstEvent();
//		int event1StartTime = firstEvent.getStartTimeInfo().getMovieTime();
//
//		if (time < event1StartTime) {
//			firstEvent.getStartTimeInfo().setMovieTime(time);
//		} else {
//			if (collection.getOnsetEvent() != null) {
//				Event onset = collection.getOnsetEvent();
//				if (onset.getEndTimeInfo().getMovieTime() < time) {
//					collection.setOnsetEvent(null);
//					setStartTimeOrRemoveEvent(time);
//				} else if (onset.getEndTimeInfo().getMovieTime() == time) {
//					collection.setOnsetEvent(null);
//				} else {
//					collection.getOnsetEvent().getStartTimeInfo()
//							.setMovieTime(time);
//				}
//			} else if (collection.getInitialHoldEvent() != null) {
//				Event hold = collection.getInitialHoldEvent();
//				if (hold.getEndTimeInfo().getMovieTime() < time) {
//					collection.setInitialHoldEvent(null);
//					setStartTimeOrRemoveEvent(time);
//				} else if (hold.getEndTimeInfo().getMovieTime() == time) {
//					collection.setInitialHoldEvent(null);
//				} else {
//					collection.getInitialHoldEvent().getStartTimeInfo()
//							.setMovieTime(time);
//				}
//			} else {
//				Event value = collection.getValueEvent();
//				if (value.getEndTimeInfo().getMovieTime() <= time) {
//					;
//				}
//			}
//		}
//	}

//	public void setEndTimeOrRemoveEvent(int time) {
//		Event lastEvent = getLastEvent();
//		int event1EndTime = lastEvent.getStartTimeInfo().getMovieTime();
//
//		if (time > event1EndTime) {
//			lastEvent.getEndTimeInfo().setMovieTime(time);
//		} else {
//			if (collection.getOffsetEvent() != null) {
//				Event offset = collection.getOffsetEvent();
//				if (offset.getStartTimeInfo().getMovieTime() > time) {
//					collection.setOffsetEvent(null);
//					setEndTimeOrRemoveEvent(time);
//				} else if (offset.getStartTimeInfo().getMovieTime() == time) {
//					collection.setOffsetEvent(null);
//				} else {
//					collection.getOffsetEvent().getEndTimeInfo()
//							.setMovieTime(time);
//				}
//			} else if (collection.getFinalHoldEvent() != null) {
//				Event hold = collection.getFinalHoldEvent();
//				if (hold.getStartTimeInfo().getMovieTime() > time) {
//					collection.setFinalHoldEvent(null);
//					setEndTimeOrRemoveEvent(time);
//				} else if (hold.getStartTimeInfo().getMovieTime() == time) {
//					collection.setFinalHoldEvent(null);
//				} else {
//					collection.getFinalHoldEvent().getEndTimeInfo()
//							.setMovieTime(time);
//				}
//			} else {
//				Event value = collection.getValueEvent();
//				if (value.getEndTimeInfo().getMovieTime() <= time) {
//					;
//				}
//			}
//		}
//	}

	// public Event getEventAt(int x) {
	@Override
	public Event getEventAt(double x) {
		deleteUnattachedEvents();

		ChainedApexEvent event = null;
		if ((event = collection.getOnsetEvent()) != null && event.contains(x))
			return event;
		if ((event = collection.getInitialHoldEvent()) != null
				&& event.contains(x))
			return event;
		if ((event = collection.getValueEvent()) != null && event.contains(x))
			return event;
		if ((event = collection.getTextValueEvent()) != null
				&& event.contains(x))
			return event;
		if ((event = collection.getFinalHoldEvent()) != null
				&& event.contains(x))
			return event;
		if ((event = collection.getOffsetEvent()) != null && event.contains(x))
			return event;
		return null;
	}

	/**
	 * @return left existing ChainedApexEvent of the ChainedApexEventsEntity
	 */
	@Override
	public Event getFirstEvent() {

		deleteUnattachedEvents();

		ChainedApexEvent event = null;

		if ((event = collection.getOnsetEvent()) != null && event.isExist())
			return event;

		if ((event = collection.getInitialHoldEvent()) != null
				&& event.isExist())
			return event;

		if ((event = collection.getValueEvent()) != null && event.isExist())
			return event;

		if ((event = collection.getTextValueEvent()) != null && event.isExist())
			return event;
		return null;
	}

	private void deleteUnattachedEvents() {
		ChainedApexEvent txtEvent = collection.getValueOrTextValueEvent();

		if (txtEvent == null)
			return;

		Time txtStartMovieTimeInfo = txtEvent.getStartTimeInfo();
		Time txtEndMovieTimeInfo = txtEvent.getEndTimeInfo();

		if (collection.getInitialHoldEvent() != null) {
			Time ihEndMovieTimeInfo = collection.getInitialHoldEvent()
					.getEndTimeInfo();
			if (! ihEndMovieTimeInfo.equals(txtStartMovieTimeInfo)) {
				collection.setInitialHoldEvent(null);
				collection.setOnsetEvent(null);
			} else if (collection.getOnsetEvent() != null) {
				Time ihStartMovieTimeInfo = collection.getInitialHoldEvent()
						.getStartTimeInfo();
				Time onsetEndMovieTimeInfo = collection.getOnsetEvent()
						.getEndTimeInfo();
				if (! ihStartMovieTimeInfo.equals(onsetEndMovieTimeInfo)) {
					collection.setOnsetEvent(null);
				}
			}
		} else if (collection.getOnsetEvent() != null) {
			Time onsetEndMovieTimeInfo = collection.getOnsetEvent().getEndTimeInfo();
			if (! onsetEndMovieTimeInfo.equals(txtStartMovieTimeInfo)) {
				collection.setOnsetEvent(null);
			}
		}

		if (collection.getFinalHoldEvent() != null) {
			Time fhStartMovieTimeInfo = collection.getFinalHoldEvent()
					.getStartTimeInfo();
			if (! txtEndMovieTimeInfo.equals(fhStartMovieTimeInfo)) {
				collection.setFinalHoldEvent(null);
				collection.setOffsetEvent(null);
			} else {
				Time offsetStartMovieTimeInfo = collection.getFinalHoldEvent()
						.getStartTimeInfo();
				if (! txtEndMovieTimeInfo.equals(offsetStartMovieTimeInfo)) {
					collection.setOffsetEvent(null);
				}
			}
		} else if (collection.getOffsetEvent() != null) {
			Time offsetStartMovieTimeInfo = collection.getOffsetEvent()
					.getStartTimeInfo();
			if (! txtEndMovieTimeInfo.equals(offsetStartMovieTimeInfo)) {
				collection.setOffsetEvent(null);
			}
		}
	}

	/**
	 * @return right existing ChainedApexEvent of the ChainedApexEventsEntity
	 */
	@Override
	public Event getLastEvent() {
		deleteUnattachedEvents();
		ChainedApexEvent event = null;

		// clean entity - sometimes it may have wrong HOLD transfered from
		// Legacy file

		if ((event = collection.getOffsetEvent()) != null && event.isExist())
			return event;

		if ((event = collection.getFinalHoldEvent()) != null && event.isExist()) {
			if (collection.getValueOrTextValueEvent().getEndTimeInfo()
					.value() != collection.getFinalHoldEvent()
					.getStartTimeInfo().value()) {
				collection.setFinalHoldEvent(null);
			} else {
				return event;
			}
		}

		if ((event = collection.getValueEvent()) != null && event.isExist())
			return event;

		if ((event = collection.getTextValueEvent()) != null && event.isExist())
			return event;

		return null;
	}

	/**
	 * delete entire entity
	 */
	public boolean deleteEntity() {
		collection.setOnsetEvent(null);
		collection.setInitialHoldEvent(null);
		collection.setValueEvent(null);
		collection.setTextValueEvent(null);
		collection.setFinalHoldEvent(null);
		collection.setOffsetEvent(null);
		return true;
	}

	/**
	 * @param event
	 *            ChainedApexEvent to be deleted
	 */
	public void deleteChainedApexEvent(ChainedApexEvent event) {
		// if(event != null)
		// event.delete();
		event = null;
	}

	public ChainedApexEventsCollection getChainedApexEventsCollection() {
		return collection;
	}

	@Override
	public void resetAll() {
		ChainedApexEvent event;
		if ((event = collection.getOnsetEvent()) != null)
			event.reset();
		if ((event = collection.getInitialHoldEvent()) != null)
			event.reset();
		if ((event = collection.getValueEvent()) != null)
			event.reset();
		if ((event = collection.getTextValueEvent()) != null)
			event.reset();
		if ((event = collection.getFinalHoldEvent()) != null)
			event.reset();
		if ((event = collection.getOffsetEvent()) != null)
			event.reset();
	}

	public boolean isAnyEventSelected() {
		return getSelectedEvent() != null ? true : false;
	}

	@Override
	public boolean adjustEntity(double startTime,double endTime) {
		ChainedApexEvent onset = collection.getOnsetEvent();
		ChainedApexEvent offset = collection.getOffsetEvent();
		ChainedApexEvent finalHold = collection.getFinalHoldEvent();
		ChainedApexEvent value = collection.getValueOrTextValueEvent();

		var range = value.getEndTimeInfo().range();
		var endTimeInfo = range.time(Math.round(endTime), TimeUnit.TIMESTAMP);
        var startTimeInfo = range.time(Math.round(startTime), TimeUnit.TIMESTAMP);

		if (value == null)
			return false;

		if (startTime < value.getStartTimeInfo().value()
				&& endTime > value.getEndTimeInfo().value()) {
			deleteEntity();
			return true;
		} else if (startTime < value.getStartTimeInfo().value()
				&& endTime < value.getEndTimeInfo().value()) {
			deleteOnsetEvent();
			deleteInitialHoldEvent();
			collection.getValueEvent().setStartTimeInfo(endTimeInfo);
			return true;
		} else if (endTime < value.getEndTimeInfo().value()
				&& endTime > value.getStartTimeInfo().value()) {
			deleteFinalHoldEvent();
			deleteOffsetEvent();
			collection.getValueEvent().setEndTimeInfo(endTimeInfo);
			return true;
		} else if (offset != null
				&& startTime > offset.getStartTimeInfo().value()) {
			deleteOffsetEvent();
			collection.getInitialHoldEvent().setStartTimeInfo(startTimeInfo);
			return true;
		} else if (onset != null
				&& startTime > onset.getStartTimeInfo().value()) {
            collection.getOnsetEvent().setStartTimeInfo(startTimeInfo);
			return true;
		} else if (finalHold != null
				&& endTime < finalHold.getEndTimeInfo().value()) {
			deleteFinalHoldEvent();
			collection.getFinalHoldEvent().setStartTimeInfo(endTimeInfo);
			return true;
		} else if (offset != null
				&& endTime < onset.getEndTimeInfo().value()) {
			collection.getOffsetEvent().setStartTimeInfo(endTimeInfo);
			return true;
		}
		return false;
	}

	public boolean deleteOnsetEvent() {
		String message = "<HTML>This action will delete ONSET value. <BR> Do you want to proceed? </HTML>";

		final JDialog dialog = new JDialog(SS3Singleton
				.getSignStreamApplication(), "Confirm Dialog", true);
		JButton proceedBtn = new JButton("Proceed");
		JButton cancelBtn = new JButton("Cancel");

		proceedBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				collection.setOnsetEvent(null);
				dialog.dispose();
			}
		});

		cancelBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				dialog.dispose();
			}
		});

		ConfirmationPanel panel = new ConfirmationPanel(message, proceedBtn,
				cancelBtn);

		Container contentPane = dialog.getContentPane();
		contentPane.setLayout(new BorderLayout(40, 40));
		contentPane.add(panel, BorderLayout.CENTER);

		dialog.setLocation(new Point(150, 150));
		dialog.pack();
		dialog.setVisible(true);

		return false;
	}

	public boolean deleteOnsetEventNoWarning() {
		collection.setOnsetEvent(null);
		return true;
	}

	public boolean deleteOffsetEventNoWarning() {
		collection.setOffsetEvent(null);
		return true;
	}

	public boolean deleteFinalHoldEventNoWarning() {
		collection.setFinalHoldEvent(null);

		if (collection.getOffsetEvent() != null) {
			Event value = collection.getValueOrTextValueEvent();
			collection.getOffsetEvent().setStartTimeInfo(value.getStartTimeInfo());
		}
		return true;
	}

	public boolean deleteInitialHoldEventNoWarning() {
		collection.setInitialHoldEvent(null);

		if (collection.getOnsetEvent() != null) {
			Event value = collection.getValueOrTextValueEvent();
			collection.getOnsetEvent().setEndTimeInfo(value.getEndTimeInfo());
		}
		return true;
	}

	public boolean deleteInitialHoldEvent() {
		String message = "<HTML>This action will delete INITIAL HOLD value. <BR> Do you want to proceed? </HTML>";

		final JDialog dialog = new JDialog(SS3Singleton
				.getSignStreamApplication(), "Confirm Dialog", true);
		JButton proceedBtn = new JButton("Proceed");
		JButton cancelBtn = new JButton("Cancel");

		proceedBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				collection
					.getValueOrTextValueEvent()
					.setStartTimeInfo(collection.getInitialHoldEvent().getStartTimeInfo());
				deleteInitialHoldEventNoWarning();
				dialog.dispose();
			}
		});

		cancelBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				dialog.dispose();
			}
		});

		ConfirmationPanel panel = new ConfirmationPanel(message, proceedBtn,
				cancelBtn);

		Container contentPane = dialog.getContentPane();
		contentPane.setLayout(new BorderLayout(40, 40));
		contentPane.add(panel, BorderLayout.CENTER);

		dialog.setLocation(new Point(150, 150));
		dialog.pack();
		dialog.setVisible(true);

		return false;
	}

	public boolean deleteValueEvent() {
		String message = "<HTML>This action will delete selected event. <BR> Do you want to proceed?</HTML>";

		final JDialog dialog = new JDialog(SS3Singleton
				.getSignStreamApplication(), "Confirm Dialog", true);
		JButton proceedBtn = new JButton("Proceed");
		JButton cancelBtn = new JButton("Cancel");

		proceedBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				deleteEntity();
				dialog.dispose();
			}
		});

		cancelBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				dialog.dispose();
			}
		});

		ConfirmationPanel panel = new ConfirmationPanel(message, proceedBtn,
				cancelBtn);

		Container contentPane = dialog.getContentPane();
		contentPane.setLayout(new BorderLayout(40, 40));
		contentPane.add(panel, BorderLayout.CENTER);

		dialog.setLocation(new Point(150, 150));
		dialog.pack();
		dialog.setVisible(true);

		return false;
	}

	public boolean deleteTextValueEvent() {
		String message = "<HTML>This action will delete selected event. <BR> Do you want to proceed?</HTML>";

		final JDialog dialog = new JDialog(SS3Singleton
				.getSignStreamApplication(), "Confirm Dialog", true);
		JButton proceedBtn = new JButton("Proceed");
		JButton cancelBtn = new JButton("Cancel");

		proceedBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				deleteEntity();
				dialog.dispose();
			}
		});

		cancelBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				dialog.dispose();
			}
		});

		ConfirmationPanel panel = new ConfirmationPanel(message, proceedBtn,
				cancelBtn);

		Container contentPane = dialog.getContentPane();
		contentPane.setLayout(new BorderLayout(40, 40));
		contentPane.add(panel, BorderLayout.CENTER);

		dialog.setLocation(new Point(150, 150));
		dialog.pack();
		dialog.setVisible(true);

		return false;
	}

	public boolean deleteFinalHoldEvent() {
		String message = "<HTML>This action will delete FINAL HOLD value. <BR> Do you want to proceed? </HTML>";

		final JDialog dialog = new JDialog(SS3Singleton.getSignStreamApplication(), "Confirm Dialog", true);
		JButton proceedBtn = new JButton("Proceed");
		JButton cancelBtn = new JButton("Cancel");

		proceedBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				if (collection.getOffsetEvent() != null)
					collection
					.getOffsetEvent()
                    .setStartTimeInfo(collection.getFinalHoldEvent().getStartTimeInfo());
				// collection.getFinalHoldEvent().delete();
				collection.setFinalHoldEvent(null);
				dialog.dispose();
			}
		});

		cancelBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				dialog.dispose();
			}
		});

		ConfirmationPanel panel = new ConfirmationPanel(message, proceedBtn,
				cancelBtn);

		Container contentPane = dialog.getContentPane();
		contentPane.setLayout(new BorderLayout(40, 40));
		contentPane.add(panel, BorderLayout.CENTER);

		dialog.setLocation(new Point(150, 150));
		dialog.pack();
		dialog.setVisible(true);

		return false;
	}

	public boolean deleteOffsetEvent() {
		String message = "<HTML>This action will delete OFFSET value. <BR> Do you want to proceed?</HTML>";

		final JDialog dialog = new JDialog(SS3Singleton.getSignStreamApplication(), "Confirm Dialog", true);
		JButton proceedBtn = new JButton("Proceed");
		JButton cancelBtn = new JButton("Cancel");

		proceedBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				collection.setOffsetEvent(null);
				dialog.dispose();
			}
		});

		cancelBtn.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				dialog.dispose();
			}
		});

		ConfirmationPanel panel = new ConfirmationPanel(message, proceedBtn,
				cancelBtn);

		Container contentPane = dialog.getContentPane();
		contentPane.setLayout(new BorderLayout(40, 40));
		contentPane.add(panel, BorderLayout.CENTER);

		dialog.setLocation(new Point(150, 150));
		dialog.pack();
		dialog.setVisible(true);

		return false;
	}

//	public boolean setInitialHoldStartMovieTimeNoWarning(int movieTime) {
//		ChainedApexEvent initialHold = collection.getInitialHoldEvent();
//		ChainedApexEvent onset = collection.getOnsetEvent();
//
//		if (initialHold == null)
//			return false;
//
//		int maxHoldStartMovieTime = convertor
//				.getPreviousFrameMovieTime(initialHold.getEndTimeInfo()
//						.getMovieTime());
//
//		if (movieTime <= maxHoldStartMovieTime) {
//			if (onset != null) {
//				int onsetMinEndMovieTime = convertor
//						.getNextFrameMovieTime(onset.getStartTimeInfo()
//								.getMovieTime());
//
//				if (movieTime >= onsetMinEndMovieTime) {
//					onset.getEndTimeInfo().setMovieTime(movieTime);
//					initialHold.getStartTimeInfo().setMovieTime(movieTime);
//					return true;
//				} else {
//					onset.getEndTimeInfo().setMovieTime(onsetMinEndMovieTime);
//					initialHold.getStartTimeInfo().setMovieTime(
//							onsetMinEndMovieTime);
//					// collection.setOnsetEvent(null);
//					// initialHold.setStartTimeCoordinate(coord);
//					return true;
//				}
//			} else {
//				initialHold.getStartTimeInfo().setMovieTime(movieTime);
//				return true;
//			}
//		} else {
//			initialHold.getStartTimeInfo().setMovieTime(maxHoldStartMovieTime);
//			// collection.setInitialHoldEvent(null);
//			if (onset != null)
//				onset.getEndTimeInfo().setMovieTime(
//						collection.getValueOrTextValueEvent()
//								.getStartTimeInfo().getMovieTime());
//			return true;
//		}
//	}
//
//	public boolean setValueStartMovieTimeNoWarning(int movieTime) {
//		ChainedApexEvent onset = collection.getOnsetEvent();
//		ChainedApexEvent initialHold = collection.getInitialHoldEvent();
//		ChainedApexEvent value = collection.getValueOrTextValueEvent();
//
//		if (value == null)
//			return false;
//
//		int valueMaxStartMovieTime = convertor.getPreviousFrameMovieTime(value
//				.getEndTimeInfo().getMovieTime());
//
//		if (movieTime <= valueMaxStartMovieTime) {
//			if (initialHold != null) {
//				int holdMinEndMovieTime = convertor
//						.getNextFrameMovieTime(initialHold.getStartTimeInfo()
//								.getMovieTime());
//				if (movieTime >= holdMinEndMovieTime) {
//					initialHold.getEndTimeInfo().setMovieTime(movieTime);
//					value.getStartTimeInfo().setMovieTime(movieTime);
//					return true;
//				} else {
//					// collection.setInitialHoldEvent(null);
//					// value.setStartTimeCoordinate(coord);
//					initialHold.getEndTimeInfo().setMovieTime(
//							holdMinEndMovieTime);
//					value.getStartTimeInfo().setMovieTime(holdMinEndMovieTime);
//					return true;
//				}
//			} else if (onset != null) {
//				int onsetMinEndMovieTime = convertor
//						.getNextFrameMovieTime(onset.getStartTimeInfo()
//								.getMovieTime());
//				if (movieTime > onsetMinEndMovieTime) {
//					onset.getEndTimeInfo().setMovieTime(movieTime);
//					value.getStartTimeInfo().setMovieTime(movieTime);
//					return true;
//				} else {
//					onset.getEndTimeInfo().setMovieTime(onsetMinEndMovieTime);
//					value.getStartTimeInfo().setMovieTime(onsetMinEndMovieTime);
//					// collection.setOnsetEvent(null);
//					// value.setStartTimeCoordinate(coord);
//					return true;
//				}
//			} else {
//				value.getStartTimeInfo().setMovieTime(movieTime);
//				return true;
//			}
//		} else {
//			return deleteEntity();
//		}
//	}
//
//	public boolean setOnsetStartMovieTimeNoWarning(int movieTime) {
//		ChainedApexEvent onset = collection.getOnsetEvent();
//
//		if (movieTime > onset.getStartTimeInfo().getMovieTime()) {
//
//			int maxStartMovieTime = convertor.getPreviousFrameMovieTime(onset
//					.getEndTimeInfo().getMovieTime());
//
//			if (movieTime <= maxStartMovieTime) {
//				onset.getStartTimeInfo().setMovieTime(movieTime);
//			} else {
//				onset.getStartTimeInfo().setMovieTime(maxStartMovieTime);
//			}
//		} else {
//			onset.getStartTimeInfo().setMovieTime(movieTime);
//		}
//		return true;
//	}
//
//	public Response checkOnsetStartMovieTimeUpdatePossibility(int movieTime) {
//		Response response = new Response();
//		ChainedApexEvent onset = collection.getOnsetEvent();
//
//		response.setUpdatePossible(true);
//		response.setMovieTime(movieTime);
//		response.setState(Response.OK);
//
//		if (movieTime > onset.getStartTimeInfo().getMovieTime()) {
//			int maxStartTime = convertor.getPreviousFrameMovieTime(onset
//					.getEndTimeInfo().getMovieTime());
//
//			if (movieTime > maxStartTime) {
//				response.setUpdatePossible(true);
//				response.setState(Response.HOLD_DELETE);
//				response.setErrorMessage("Onset event will be deleted.");
//				response.setMovieTime(maxStartTime);
//			}
//		}
//
//		return response;
//	}
//
//	public Response checkValueStartMovieTimeUpdatePossibility(int movieTime) {
//		ChainedApexEvent onset = collection.getOnsetEvent();
//		ChainedApexEvent initialHold = collection.getInitialHoldEvent();
//		ChainedApexEvent value = collection.getValueOrTextValueEvent();
//
//		Response response = new Response();
//		response.setState(Response.OK);
//		response.setUpdatePossible(true);
//		response.setMovieTime(movieTime);
//
//		int valueMaxStartMovieTime = convertor.getPreviousFrameMovieTime(value
//				.getEndTimeInfo().getMovieTime());
//
//		if (movieTime <= valueMaxStartMovieTime) {
//			if (initialHold != null) {
//				int holdMinEndMovieTime = convertor
//						.getNextFrameMovieTime(initialHold.getStartTimeInfo()
//								.getMovieTime());
//				if (movieTime >= holdMinEndMovieTime) {
//					;
//				} else {
//					response.setState(Response.HOLD_DELETE);
//					response.setErrorMessage("Initial Hold will be deleted");
//					response.setMovieTime(holdMinEndMovieTime);
//				}
//			} else if (onset != null) {
//				int onsetMinEndMovieTime = convertor
//						.getNextFrameMovieTime(onset.getStartTimeInfo()
//								.getMovieTime());
//				if (movieTime > onsetMinEndMovieTime) {
//					;
//				} else {
//					response.setState(Response.HOLD_DELETE);
//					response.setErrorMessage("Onset will be deleted");
//					response.setMovieTime(onsetMinEndMovieTime);
//				}
//			} else {
//				;
//			}
//		} else {
//			response.setState(Response.ENTITY_DELETE);
//			response.setErrorMessage("Entity will be deleted");
//			response.setMovieTime(valueMaxStartMovieTime);
//		}
//		return response;
//	}
//
//	public Response checkValueEndMovieTimeUpdatePossibility(int movieTime) {
//		ChainedApexEvent offset = collection.getOffsetEvent();
//		ChainedApexEvent finalHold = collection.getFinalHoldEvent();
//		ChainedApexEvent value = collection.getValueOrTextValueEvent();
//
//		Response response = new Response();
//		response.setUpdatePossible(true);
//		response.setState(Response.OK);
//		response.setMovieTime(movieTime);
//
//		int valueMinEndMovieTime = convertor.getNextFrameMovieTime(value
//				.getStartTimeInfo().getMovieTime());
//
//		if (movieTime >= valueMinEndMovieTime) {
//			if (finalHold != null) {
//				int maxHoldStartMovieTime = convertor
//						.getPreviousFrameMovieTime(finalHold.endTimeInfo
//								.getMovieTime());
//
//				if (maxHoldStartMovieTime >= movieTime) {
//					;
//				} else {
//					response.setState(Response.HOLD_DELETE);
//					response.setErrorMessage("Final Hold will be deleted");
//					response.setMovieTime(maxHoldStartMovieTime);
//				}
//
//			} else if (offset != null) {
//				int maxOffsetStartMovieTime = convertor
//						.getPreviousFrameMovieTime(offset.getEndTimeInfo()
//								.getMovieTime());
//
//				if (maxOffsetStartMovieTime > movieTime) {
//					;
//				} else {
//					response.setState(Response.HOLD_DELETE);
//					response.setErrorMessage("Offset will be deleted");
//					response.setMovieTime(maxOffsetStartMovieTime);
//				}
//			} else {
//				;
//			}
//		} else {
//			response.setState(Response.ENTITY_DELETE);
//			response.setErrorMessage("Entity will be deleted");
//			response.setMovieTime(valueMinEndMovieTime);
//		}
//		return response;
//	}
//
//	public boolean setOffsetEndMovieTimeNoWarning(int movieTime) {
//		ChainedApexEvent offset = collection.getOffsetEvent();
//
//		int offsetMinEndMovieTime = convertor.getNextFrameMovieTime(offset
//				.getStartTimeInfo().getMovieTime());
//
//		if (movieTime >= offsetMinEndMovieTime) {
//			offset.getEndTimeInfo().setMovieTime(movieTime);
//			return true;
//		} else {
//			offset.getEndTimeInfo().setMovieTime(offsetMinEndMovieTime);
//			// collection.setOffsetEvent(null);
//			return false;
//		}
//	}

//	public Response checkOffsetEndMovieTimeUpdatePossibility(int movieTime) {
//		ChainedApexEvent offset = collection.getOffsetEvent();
//
//		Response response = new Response();
//		response.setUpdatePossible(true);
//		response.setState(Response.OK);
//		response.setMovieTime(movieTime);
//
//		int offsetMinEndMovieTime = convertor.getNextFrameMovieTime(offset
//				.getStartTimeInfo().getMovieTime());
//
//		if (movieTime <= offsetMinEndMovieTime) {
//			response.setErrorMessage("Offset will be deleted");
//			response.setState(Response.HOLD_DELETE);
//			response.setMovieTime(offsetMinEndMovieTime);
//		}
//
//		return response;
//	}
//
//	public Response checkInitialHoldStartMovieTimeUpdatePossibility(
//			int movieTime) {
//		ChainedApexEvent onset = collection.getOnsetEvent();
//		ChainedApexEvent initialHold = collection.getInitialHoldEvent();
//
//		Response response = new Response();
//		response.setUpdatePossible(true);
//		response.setState(Response.OK);
//		response.setMovieTime(movieTime);
//
//		int maxHoldStartMovieTime = convertor
//				.getPreviousFrameMovieTime(initialHold.getEndTimeInfo()
//						.getMovieTime());
//
//		if (movieTime < maxHoldStartMovieTime) {
//			if (onset != null) {
//				int onsetMinEndMovieTime = convertor
//						.getNextFrameMovieTime(onset.getStartTimeInfo()
//								.getMovieTime());
//				if (movieTime >= onsetMinEndMovieTime) {
//					;
//				} else {
//					response.setErrorMessage("Onset will be deleted");
//					response.setState(Response.HOLD_DELETE);
//					response.setMovieTime(onsetMinEndMovieTime);
//				}
//			} else {
//				;
//			}
//		} else {
//			response.setErrorMessage("Initial Hold will be deleted");
//			response.setState(Response.HOLD_DELETE);
//			response.setMovieTime(maxHoldStartMovieTime);
//		}
//
//		return response;
//	}
//
//	public Response checkFinalHoldEndMovieTimeUpdatePossibility(int movieTime) {
//		ChainedApexEvent offset = collection.getOffsetEvent();
//		ChainedApexEvent finalHold = collection.getFinalHoldEvent();
//
//		Response response = new Response();
//		response.setUpdatePossible(true);
//		response.setState(Response.OK);
//		response.setMovieTime(movieTime);
//
//		int holdMinEndMovieTime = convertor.getNextFrameMovieTime(finalHold
//				.getStartTimeInfo().getMovieTime());
//
//		if (movieTime > holdMinEndMovieTime) {
//			if (offset != null) {
//				int maxOffsetStartMovieTime = convertor
//						.getPreviousFrameMovieTime(offset.getEndTimeInfo()
//								.getMovieTime());
//
//				if (maxOffsetStartMovieTime > movieTime) {
//					;
//				} else {
//					response.setErrorMessage("Offset will be deleted");
//					response.setState(Response.HOLD_DELETE);
//					// response.setMovieTime();
//				}
//			}
//		} else {
//			response.setErrorMessage("Final Hold will be deleted");
//			response.setState(Response.HOLD_DELETE);
//			response.setMovieTime(holdMinEndMovieTime);
//		}
//
//		return response;
//	}
//
//	public boolean setFinalHoldEndMovieTimeNoWarning(int movieTime) {
//		ChainedApexEvent finalHold = collection.getFinalHoldEvent();
//
//		if (finalHold == null)
//			return true;
//
//		ChainedApexEvent offset = collection.getOffsetEvent();
//
//		int holdMinEndMovieTime = convertor.getNextFrameMovieTime(finalHold
//				.getStartTimeInfo().getMovieTime());
//
//		if (movieTime >= holdMinEndMovieTime) {
//			if (offset != null) {
//				int maxOffsetStartMovieTime = convertor
//						.getPreviousFrameMovieTime(offset.getEndTimeInfo()
//								.getMovieTime());
//
//				if (maxOffsetStartMovieTime > movieTime) {
//					finalHold.getEndTimeInfo().setMovieTime(movieTime);
//					offset.getStartTimeInfo().setMovieTime(movieTime);
//					return true;
//				} else {
//					finalHold.getEndTimeInfo().setMovieTime(
//							maxOffsetStartMovieTime);
//					offset.getStartTimeInfo().setMovieTime(
//							maxOffsetStartMovieTime);
//					return true;
//				}
//			} else {
//				finalHold.getEndTimeInfo().setMovieTime(movieTime);
//				return true;
//			}
//		} else {
//			finalHold.getEndTimeInfo().setTimeBase(holdMinEndMovieTime);
//			return true;
//		}
//	}
//
//	public boolean setValueEndMovieTimeNoWarning(int movieTime) {
//		ChainedApexEvent offset = collection.getOffsetEvent();
//		ChainedApexEvent finalHold = collection.getFinalHoldEvent();
//		ChainedApexEvent value = collection.getValueOrTextValueEvent();
//
//		if (value == null)
//			return false;
//
//		int valueMinEndMovieTime = convertor.getNextFrameMovieTime(value
//				.getStartTimeInfo().getMovieTime());
//
//		if (movieTime >= valueMinEndMovieTime) {
//			if (finalHold != null) {
//				int maxHoldStartMovieTime = convertor
//						.getPreviousFrameMovieTime(finalHold.getEndTimeInfo()
//								.getMovieTime());
//
//				if (maxHoldStartMovieTime >= movieTime) {
//					value.getEndTimeInfo().setMovieTime(movieTime);
//					finalHold.getStartTimeInfo().setMovieTime(movieTime);
//					return true;
//				} else {
//					value.getEndTimeInfo().setMovieTime(maxHoldStartMovieTime);
//					finalHold.getStartTimeInfo().setMovieTime(
//							maxHoldStartMovieTime);
//					return true;
//				}
//
//			} else if (offset != null) {
//
//				int maxOffsetStartMovieTime = convertor
//						.getPreviousFrameMovieTime(offset.getEndTimeInfo()
//								.getMovieTime());
//
//				if (maxOffsetStartMovieTime > movieTime) {
//					value.getEndTimeInfo().setMovieTime(movieTime);
//					offset.getStartTimeInfo().setMovieTime(movieTime);
//					return true;
//				} else {
//					try {
//						finalHold.getStartTimeInfo().setMovieTime(
//								maxOffsetStartMovieTime);
//						value.getEndTimeInfo().setMovieTime(
//								maxOffsetStartMovieTime);
//					} catch (NullPointerException ex) {
//						ex.printStackTrace();
//					}
//					return true;
//				}
//			} else {
//				value.getEndTimeInfo().setMovieTime(movieTime);
//				return true;
//			}
//		} else {
//			return deleteEntity();
//		}
//	}

	// add onset, one frame long approximately 1/30 second
	public void addOnsetEvent() {
		ChainedApexEvent event = collection.getInitialHoldEvent();
		if (event == null)
			event = collection.getValueOrTextValueEvent();

		try {
			String id = "" + Constants.APEX_ONSET_CODE;
			
			Time endTimeInfo = event.getStartTimeInfo();
			Time startTimeInfo = segmentPanel.getMultipleMovieController().getPrecedingFrameTimeInfo(endTimeInfo);
			
			TrackValue newValue = new TrackValue(id, "", startTimeInfo, endTimeInfo);
			collection.setOnsetEvent(new ChainedApexEvent(newValue, segmentPanel));
		} catch (NullPointerException ex) {
			// @TODO:
			// "Not valid argument Exception will be thrown!!! ChainedApexEventsCollectionManager");
		}
	}

	public void addOffsetEvent() {
		ChainedApexEvent event = collection.getFinalHoldEvent();
		if (event == null)
			event = collection.getValueOrTextValueEvent();

		try {
			String id = "" + Constants.APEX_OFFSET_CODE;
			
			Time startTimeInfo = event.getEndTimeInfo();
			Time endTimeInfo = segmentPanel.getMultipleMovieController().getFollowingFrameTimeInfo(startTimeInfo);
			
			TrackValue newValue = new TrackValue(id, "", startTimeInfo, endTimeInfo);
			collection.setOffsetEvent(new ChainedApexEvent(newValue, segmentPanel));
		} catch (NullPointerException ex) {
			// @TODO:
			// "Not valid argument Exception will be thrown!!! ChainedApexEventsCollectionManager");
		}
	}

	public void addInitialHoldEvent() {
		ChainedApexEvent onset = collection.getOnsetEvent();
		ChainedApexEvent event = collection.getValueOrTextValueEvent();

		try {
			String id = "" + Constants.APEX_INITIAL_HOLD_CODE;
			
			Time endTimeInfo = event.getStartTimeInfo();
			Time startTimeInfo = segmentPanel.getMultipleMovieController().getPrecedingFrameTimeInfo(endTimeInfo);
			
			TrackValue newValue = new TrackValue(id, "", startTimeInfo, endTimeInfo);
			collection.setInitialHoldEvent(new ChainedApexEvent(newValue, segmentPanel));

			if (onset != null) {
				onset.setEndTimeInfo(startTimeInfo);
				// onset.getEndTimeInfo().setMovieTime(onset.getEndTimeInfo().getMovieTime()-minFrameDurationToAdd);
			}
		} catch (NullPointerException ex) {
			// @TODO:
			// "Not valid argument Exception will be thrown!!! ChainedApexEventsCollectionManager");
		}
	}

	public void addInitialHoldEvent(ChainedApexEvent hold) {
		collection.setInitialHoldEvent(hold);
	}

	public void addFinalHoldEvent(ChainedApexEvent hold) {
		collection.setFinalHoldEvent(hold);
	}

	public void addFinalHoldEvent() {
		ChainedApexEvent offset = collection.getOffsetEvent();
		ChainedApexEvent event = collection.getValueOrTextValueEvent();

		try {
			String id = "" + Constants.APEX_FINAL_HOLD_CODE;
			
			Time startTimeInfo = event.getEndTimeInfo();
			Time endTimeInfo = segmentPanel.getMultipleMovieController().getFollowingFrameTimeInfo(startTimeInfo);
			
			TrackValue newValue = new TrackValue(id, "", startTimeInfo, endTimeInfo);
			collection.setFinalHoldEvent(new ChainedApexEvent(newValue, segmentPanel));

			if (offset != null) {
				offset.setStartTimeInfo(endTimeInfo);
			}
		} catch (NullPointerException ex) {
			// @TODO:
			// "Not valid argument Exception will be thrown!!! ChainedApexEventsCollectionManager");
		}
	}
}
