package edu.bu.signstream.grepresentation.fields.apexField;

import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.util.ArrayList;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.time.Duration;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.FieldWrappersCollection;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureEventsEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureFieldWrapper;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationEventsEntity;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class ChainedApexEventsEntity extends EventsEntity {

	private static final long serialVersionUID = 1L;

	protected ChainedApexEventsCollectionManager manager;

	private ChainedApexEventsEntity previousEntity = null;
	private ChainedApexEventsEntity nextEntity = null;
	
	private boolean isLockedApexPhonAlignments = false;

	public ChainedApexEventsEntity(ChainedApexEventsCollection collection,
			SignStreamSegmentPanel segmentPanel, Field field) {

		super(segmentPanel, field);
		setID(Util.getLongUniqueNumber());
		this.manager = new ChainedApexEventsCollectionManager(collection,
				segmentPanel);
	}
	
	public boolean isLockedApexPhonAlignments() {
		return isLockedApexPhonAlignments;
	}

	public void setLockedApexPhonAlignments(boolean isLockedApexPhonAlignments) {
		this.isLockedApexPhonAlignments = isLockedApexPhonAlignments;
	}
	
    // returns an error code that will be matched with an error message or null if success. 
	public String setSelectedEventStartMovieTime(Time timeInfo) {
		FieldWrappersCollection gestureWrappers = segmentPanel.getUtteranceView().getSpeechView().getFieldWrappersCollection();
		GestureFieldWrapper gestureWrapper;
		if (getField().getFieldID().equals(Util.DOM_GESTURE_APEX_FIELD_ID)) {
			gestureWrapper = (GestureFieldWrapper) gestureWrappers.getFieldWrapperWithFieldID(Util.DOM_GESTURE_FIELD_ID);
		} else {
			gestureWrapper = (GestureFieldWrapper) gestureWrappers.getFieldWrapperWithFieldID(Util.NDOM_GESTURE_FIELD_ID);
		}
		GestureEventsEntity parentGestureEntity = (GestureEventsEntity) gestureWrapper.getGestureField()
				.getEntityAt((getStartTimeCoordinate() + getEndTimeCoordinate()) / 2);
		
		Event selectedEvent =  SS3Singleton.getMediaToolBar(null).selectedEvent();
        Time originalSelectedStartTimeInfo = selectedEvent.getStartTimeInfo();
        Time originalSelectedEndTimeInfo = selectedEvent.getEndTimeInfo();

        Time maxStartMovieTimeInfo = originalSelectedEndTimeInfo.subtract(new Duration(1, originalSelectedEndTimeInfo.unit()));
        
        Time minStartMovieTimeInfo = parentGestureEntity.getValueOrTextValueEvent().getStartTimeInfo();

		// apex value events are allowed to have the same start/end time
		if (this instanceof ApexChainedApexEventsEntity
				&& selectedEvent.equals(manager.getChainedApexEventsCollection().getValueOrTextValueEvent())) {
            maxStartMovieTimeInfo = maxStartMovieTimeInfo.add(new Duration(1, originalSelectedStartTimeInfo.unit()));
		}

		if (timeInfo.greaterThan(maxStartMovieTimeInfo)) {
			return ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME;
		}
		
		if (timeInfo.lessThan(minStartMovieTimeInfo)) {
            return ErrorMessages.APEX_EXTENDS_PAST_GESTURE;
        }

		Event prevEvent = manager.getChainedApexEventsCollection().getPreviousEvent(selectedEvent);
		
		if (prevEvent == null) {
			// |<--
			if (selectedEvent.getStartTimeInfo().greaterThan(timeInfo)) {
				if (previousEntity == null
						|| previousEntity.getEndTimeInfo().lessThanOrEq(timeInfo)) {
		            selectedEvent.setStartTimeInfo(timeInfo);
				} else {
					return ErrorMessages.CHAINS_CANNOT_OVERLAP;
				}
			} else {
	            selectedEvent.setStartTimeInfo(timeInfo);
			}			
		} else if (selectedEvent.getStartTimeInfo().greaterThan(timeInfo)) {
			
			//ChainedApexEvent preparationEvent = manager.getChainedApexEventsCollection().getPreparationEvent();
			ChainedApexEvent prestrokeHoldEvent = manager.getChainedApexEventsCollection().getInitialHoldEvent();
			ChainedApexEvent event = manager.getChainedApexEventsCollection().getValueOrTextValueEvent();
			ChainedApexEvent poststrokeHoldEvent = manager.getChainedApexEventsCollection().getFinalHoldEvent();
			ChainedApexEvent retractionEvent = manager.getChainedApexEventsCollection().getOffsetEvent();

			if ( (prevEvent == event && timeInfo.lessThan(prevEvent.getStartTimeInfo())) 
					|| (prevEvent != event && timeInfo.lessThanOrEq(prevEvent.getStartTimeInfo())) ) {
				
				if(selectedEvent.equals(prestrokeHoldEvent)) {
					return ErrorMessages.SET_START_PRESTROKE_HOLD_CANT_DELETE_PREPARATION;
				} else if(selectedEvent.equals(event)) {
					if(prestrokeHoldEvent != null && prevEvent.equals(prestrokeHoldEvent)) {
						return ErrorMessages.SET_START_VALUE__CANT_DELETE_PRESTROKE_HOLD;
					} else  {
						return ErrorMessages.SET_START_VALUE_CANT_DELETE_PREPARATION;
					}
				} else if(selectedEvent.equals(poststrokeHoldEvent)) {
					return ErrorMessages.SET_START_POSTSTROKE_HOLD_CANT_DELETE_VALUE;
				} else if(selectedEvent.equals(retractionEvent)) {
					if(poststrokeHoldEvent != null && prevEvent.equals(event)) {
						return ErrorMessages.SET_START_RETRACTION_CANT_DELETE_POSTSTROKE_HOLD; 
					} else {
						return ErrorMessages.SET_START_RETRACTION_CANT_DELETE_VALUE;
					}
				}
			} else {
	            selectedEvent.setStartTimeInfo(timeInfo);

				prevEvent.setEndTimeInfo(timeInfo);
			}
		} else {
			// selectedEvent.getStartTimeInfo().setMovieTime(movieTime);
			// prevEvent.getEndTimeInfo().setMovieTime(movieTime);
            //
            selectedEvent.setStartTimeInfo(timeInfo);

            prevEvent.setEndTimeInfo(timeInfo);
		}

		// check for one-frame logic of gloss events
		if(this instanceof ApexChainedApexEventsEntity
				&& selectedEvent.equals(manager.getChainedApexEventsCollection().getValueOrTextValueEvent())) {
			ApexChainedApexEventsEntity  glsEntity = (ApexChainedApexEventsEntity)this;
			
			ApexEntityProperties properties = glsEntity.getApexEntityProperties();
			
			// current start and end times same? it means we are moving to a 1 or more frame difference, so set oneFrameApex to false
			if (originalSelectedStartTimeInfo.equals(originalSelectedEndTimeInfo)) {
				properties.setOneFrameApex(false);
			}
		}
		
		return null;
	}

	public String setSelectedEventEndMovieTime(Time timeInfo) {
		FieldWrappersCollection gestureWrappers = segmentPanel.getUtteranceView().getSpeechView().getFieldWrappersCollection();
		GestureFieldWrapper gestureWrapper;
		if (getField().getFieldID().equals(Util.DOM_GESTURE_APEX_FIELD_ID)) {
			gestureWrapper = (GestureFieldWrapper) gestureWrappers.getFieldWrapperWithFieldID(Util.DOM_GESTURE_FIELD_ID);
		} else {
			gestureWrapper = (GestureFieldWrapper) gestureWrappers.getFieldWrapperWithFieldID(Util.NDOM_GESTURE_FIELD_ID);
		}
		GestureEventsEntity parentGestureEntity = (GestureEventsEntity) gestureWrapper.getGestureField()
				.getEntityAt((getStartTimeCoordinate() + getEndTimeCoordinate()) / 2);
		
		Event selectedEvent =  SS3Singleton.getMediaToolBar(null).selectedEvent();
        Time originalSelectedStartTimeInfo = selectedEvent.getStartTimeInfo();
        Time originalSelectedEndTimeInfo = selectedEvent.getEndTimeInfo();
		
		Time maxEndMovieTimeInfo = parentGestureEntity.getValueOrTextValueEvent().getEndTimeInfo();
        
		Time minEndMovieTimeInfo = originalSelectedStartTimeInfo.add(new Duration(1, originalSelectedStartTimeInfo.unit()));

		// apex value events are allowed to have the same start/end time
		if (this instanceof ApexChainedApexEventsEntity
				&& selectedEvent.equals(manager.getChainedApexEventsCollection().getValueOrTextValueEvent())) {
			minEndMovieTimeInfo = minEndMovieTimeInfo.subtract(new Duration(1, originalSelectedStartTimeInfo.unit()));
		}

		if (minEndMovieTimeInfo.greaterThan(timeInfo)) {
			return ErrorMessages.EVENT_END_TIME_FOLLOW_START_TIME;
		}
		
		if (timeInfo.greaterThan(maxEndMovieTimeInfo)) {
			return ErrorMessages.APEX_EXTENDS_PAST_GESTURE;
		}
		
		Event nextEvent = manager.getChainedApexEventsCollection().getNextEvent(selectedEvent);
		
		// new
		if (nextEvent == null) {
			if (selectedEvent.getEndTimeInfo().lessThan(timeInfo)) {
				if (nextEntity == null
						|| nextEntity.getStartTimeInfo().greaterThanOrEq(timeInfo)) {
					selectedEvent.setEndTimeInfo(timeInfo);
				} else {
					return ErrorMessages.CHAINS_CANNOT_OVERLAP;
				}
			} else {
				selectedEvent.setEndTimeInfo(timeInfo);
			}
		} else if (selectedEvent.getEndTimeInfo().lessThan(timeInfo)) {

			ChainedApexEvent preparation = manager.getChainedApexEventsCollection().getOnsetEvent();
			ChainedApexEvent prestrokeHold = manager.getChainedApexEventsCollection().getInitialHoldEvent();
			ChainedApexEvent event = manager.getChainedApexEventsCollection().getValueOrTextValueEvent();
			ChainedApexEvent poststrokeHold = manager.getChainedApexEventsCollection().getFinalHoldEvent();	

			if ( (nextEvent == event && timeInfo.greaterThan(nextEvent.getEndTimeInfo())) 
					|| (nextEvent != event && timeInfo.greaterThanOrEq(nextEvent.getEndTimeInfo())) ) {

				if(selectedEvent.equals(preparation)) {
					if(prestrokeHold!= null) {
						return ErrorMessages.SET_END_PREPARATION_CANT_DELETE_PRESTROKE_HOLD;
					} else {
						return ErrorMessages.SET_END_PREPARATION_CANT_DELETE_VALUE;
					}
				} else if(selectedEvent.equals(prestrokeHold)) {
					return ErrorMessages.SET_END_PRESTROKE_HOLD_CANT_DELETE_VALUE;
				} else if(selectedEvent.equals(event)) {
					if(poststrokeHold != null) {
						return ErrorMessages.SET_END_VALUE_CANT_DELETE_POSTSTROKE_HOLD;
					} else {
						return ErrorMessages.SET_END_VALUE_CANT_DELETE_RETRACTION;
					}
				} else {
					return ErrorMessages.SET_END_POSTSTROKE_HOLD_CANT_DELETE_RETRACTION;
				}
			} else {
				selectedEvent.setEndTimeInfo(timeInfo);
				
				nextEvent.setStartTimeInfo(timeInfo);
			}
		} else {
			selectedEvent.setEndTimeInfo(timeInfo);
			
			nextEvent.setStartTimeInfo(timeInfo);
		}

		// check for one-frame logic of gloss events
		if(this instanceof ApexChainedApexEventsEntity
				&& selectedEvent.equals(manager.getChainedApexEventsCollection().getValueOrTextValueEvent())) {
			ApexChainedApexEventsEntity  glsEntity = (ApexChainedApexEventsEntity)this;
			
			ApexEntityProperties properties = glsEntity.getApexEntityProperties();
			
			// current start and end times same? it means we are moving to a 1 or more frame difference, so set oneFrameApex to false
			if (originalSelectedStartTimeInfo.equals(originalSelectedEndTimeInfo)) {
				properties.setOneFrameApex(false);
			}
		}
		
		return null;
	}
		
	public ChainedApexEventsEntity getPreviousEntity() {
		return previousEntity;
	}

	public void setPreviousEntity(ChainedApexEventsEntity previousEntity) {
		this.previousEntity = previousEntity;
	}

	public ChainedApexEventsEntity getNextEntity() {
		return nextEntity;
	}

	public void setNextEntity(ChainedApexEventsEntity nextEntity) {
		this.nextEntity = nextEntity;
	}

	public String getText() {
		return getTextValue().getText();
	}

	public Event getNextEvent(Event event) {
		return manager.getChainedApexEventsCollection().getNextEvent(event);
	}

	public Event getPreviousEvent(Event event) {
		return manager.getChainedApexEventsCollection().getPreviousEvent(event);
	}

	public void setField(ApexField nmField) {
		super.setField(nmField);
	}

	public ChainedApexEventsEntity clone(String fieldID) {
		ChainedApexEventsCollection cll = manager.getChainedApexEventsCollection()
				.cloneChainedApexEventsCollection();
		ChainedApexEventsEntity ret = new ChainedApexEventsEntity(cll, segmentPanel, getField());
		ret.updateParentEntity();
		return ret;
	}

	@Override
	public SS3Entity getSS3Entity() {
		SS3Entity entity = new SS3Entity();
		String id = getID();
		
		String entID = id.trim().length() > 0 ? id : Util.getLongUniqueNumber();
		entity.setID(entID);
		if (getParentEntities() != null) {
			entity.setParentEntities(getParentEntities());
		}
		entity.setRefEntityId(getRefEntityId());

		ChainedApexEventsCollection collection = manager
				.getChainedApexEventsCollection();

		if (collection.getOnsetEvent() != null) {
			SS3Item onsetItem = createSS3Item(collection.getOnsetEvent());
			entity.addItem(onsetItem);
		}

		if (collection.getValueEvent() != null) {
			SS3Item valueItem = createSS3Item(collection.getValueEvent());
			entity.addItem(valueItem);
		}

		if (collection.getTextValueEvent() != null) {
			SS3Item txtValueItem = createSS3Item(collection.getTextValueEvent());
			entity.addItem(txtValueItem);
		}

		if (collection.getOffsetEvent() != null) {
			SS3Item offsetItem = createSS3Item(collection.getOffsetEvent());
			entity.addItem(offsetItem);
		}
		
//		boolean lockedApexPhoneAlignments = isLockedApexPhonAlignments();
//		entity.setLockedApexPhoneAlignments(lockedApexPhoneAlignments);
//		
//		if(this instanceof ApexChainedApexEventsEntity) {
//			ApexChainedApexEventsEntity  glsEntity = (ApexChainedApexEventsEntity)this;
//			boolean isLockedDomNdomAlignments = glsEntity.isLockedDomNdomAlignments();
//			
//			ApexChainedApexEventsEntity dependentApexEntity = glsEntity.getDependentApexEntity();
//						
//			entity.setLockedDomNdomAlignments(isLockedDomNdomAlignments);
//			if(dependentApexEntity != null) {
//				entity.setDependentApexEntityID(dependentApexEntity.getID());
//			}
//
//            CompoundableApex startCompound = glsEntity.getStartCompound();
//            if(startCompound != null) {
//                entity.setStartCompaundEntityID(startCompound.getID());
//            }
//
//            CompoundableApex endCompound = glsEntity.getEndCompound();
//            if(endCompound != null) {
//                entity.setEndCompaundEntityID(endCompound.getID());
//            }
//			
//			ApexEntityProperties properties = glsEntity.getApexEntityProperties();
//			entity.setApexEntityProperties(properties);
//		}

		return entity;
	}

	private SS3Item createSS3Item(ChainedApexEvent event) {
		SS3Item item = new SS3Item();
		item.setId(event.getID());
		item.setText(event.getText());
		item.setStartTimeInfo(event.getStartTimeInfo());
		item.setEndTimeInfo(event.getEndTimeInfo());
		return item;
	}

	public Event getValueOrTextValueEvent() {
		return manager.getChainedApexEventsCollection().getValueOrTextValueEvent();
	}

	public ChainedApexEventsCollection getChainedApexEventsCollection() {
		return manager.getChainedApexEventsCollection();
	}

	public ChainedApexEvent getValue() {
		return manager.getChainedApexEventsCollection().getValueEvent();
	}

	public ApexChainedApexEvent getTextValue() {
		return manager.getChainedApexEventsCollection().getTextValueEvent();
	}

	public ApexChainedApexEvent getApexChainedApexEvent() {
		return manager.getChainedApexEventsCollection().getTextValueEvent();
	}

	@Deprecated
	public void setStartTime(double startTime) {
		ChainedApexEvent event = (ChainedApexEvent) manager.getFirstEvent();
		if (event != null)
            event.setStartTimeInfo(getEndTimeInfo().range().time(Math.round(startTime), TimeUnit.TIMESTAMP));
	}

	@Override
	public void setStartTimeInfo(Time startTimeInfo) {
		ChainedApexEvent event = (ChainedApexEvent) manager.getFirstEvent();
		if (event != null)
			event.setStartTimeInfo(startTimeInfo);
	}

	@Override
	public void setEndTimeInfo(Time endTimeInfo) {
		ChainedApexEvent event = (ChainedApexEvent) manager.getLastEvent();
		if (event != null)
			event.setEndTimeInfo(endTimeInfo);
	}

	@Deprecated
	public void setEndTime(double endTime) {
		ChainedApexEvent event = (ChainedApexEvent) manager.getLastEvent();
		if (event != null)
		    event.setEndTimeInfo(getEndTimeInfo().range().time(Math.round(endTime), TimeUnit.TIMESTAMP));
	}

	@Override
	public Event getFirstEvent() {
		return manager.getFirstEvent();
	}

	public ChainedApexEvent getInitialHold() {
		return manager.getChainedApexEventsCollection().getInitialHoldEvent();
	}

	public ChainedApexEvent getFinalHold() {
		return manager.getChainedApexEventsCollection().getFinalHoldEvent();
	}

	public ChainedApexEvent getOnset() {
		return manager.getChainedApexEventsCollection().getOnsetEvent();
	}

	public void deleteOnset() {
		manager.deleteOnsetEventNoWarning();
	}

	public void deleteInitialHold() {
		manager.deleteInitialHoldEventNoWarning();
	}

	public void deleteFinalHold() {
		manager.deleteFinalHoldEventNoWarning();
	}

	public void deleteOffset() {
		manager.deleteOffsetEventNoWarning();
	}

	public ChainedApexEvent getOffset() {
		return manager.getChainedApexEventsCollection().getOffsetEvent();
	}

	@Override
	public Event getLastEvent() {
		return manager.getLastEvent();
	}

	@Override
	public Time getStartTimeInfo() {
		ChainedApexEvent event = (ChainedApexEvent) manager.getFirstEvent();
		if (event == null)
			return null;
		return event.getStartTimeInfo();
	}

	@Override
	public Time getEndTimeInfo() {
		ChainedApexEvent event = (ChainedApexEvent) manager.getLastEvent();
		if (event == null)
			return null;
		return event.getEndTimeInfo();
	}

	@Override
	public boolean selectEntityIfEventSelected() {
		if (manager.getSelectedEvent() != null) {
			select();
			return true;
		}
		return false;
	}

	@Override
	public boolean selectEventIfEntitySelected() {
		if (selected) {
			manager.unselectEntity();
			ChainedApexEvent valueEvent = manager.getChainedApexEventsCollection()
					.getValueEvent();
			manager.selectChainedApexEvent(valueEvent);
			selected = false;
			return true;
		}
		return false;
	}

	@Override
	public boolean select() {
		selected = true;
		return selected;
	}

	@Override
	public boolean unselect() {
		selected = false;
		unhighlight();
		manager.unselectEntity();
		return true;
	}

	@Override
	public boolean adjustEntity(int x1, int x2) {
		double pixelInTimeFrame = segmentPanel.getZoomer()
				.getPixelsInTimeFrame();
		double startTime = x1 / pixelInTimeFrame;
		double endTime = x2 / pixelInTimeFrame;
		return manager.adjustEntity(startTime, endTime);
	}

	@Override
	public Event getSelectedEvent() {		
		if (selected)
			return this;
		return manager.getSelectedEvent();
	}

	@Override
	public void reset() {
		selected = false;
		// mToolBar.setEnable(false);
		manager.resetAll();
	}

	@Override
	public boolean delete() {
		return delete(false);
	}

	public boolean delete(boolean showWarningMsg) {
		return manager.deleteEntity();
	}

	public void setValueID(String valueID) {
		ChainedApexEventsCollection collection = manager
				.getChainedApexEventsCollection();
		ChainedApexEvent event = collection.getValueEvent();
		event.setID(valueID);
	}

	@Override
	public Event getEventAt(double x) {
		return manager.getEventAt(x);
	}

	@Override
	public ArrayList<String> getEventsToAdd() {
		ArrayList<String> list = new ArrayList<String>();

		ChainedApexEventsCollection collection = manager
				.getChainedApexEventsCollection();

		if (collection.getOnsetEvent() == null)
			list.add(Constants.ONSET);

		if (collection.getInitialHoldEvent() == null)
			list.add(Constants.INITIAL_HOLD);

		if (collection.getFinalHoldEvent() == null)
			list.add(Constants.FINAL_HOLD);

		if (collection.getOffsetEvent() == null)
			list.add(Constants.OFFSET);

		return list;
	}
	
	@Override
	public ArrayList<Event> getEventsList() {
		return new ArrayList<Event>(getChainedApexEventsCollection().getCollection());
	}

	@Override
	public String getToolTipText(int x) {
		ChainedApexEvent event = (ChainedApexEvent) manager.getEventAt(x);

		if (event != null)
			return event.getToolTipText();
		return null;
	}

	@Override
	public boolean selectEvent(double x) {
		Event event = manager.getEventAt(x);
		return event.select();
	}

	@Override
	public void paint(Graphics2D g2d, double Y, Font font) {
		g2d.setFont(font);
		FontMetrics fontMetrics = getFontMetrics(font);

		segmentPanel.getPainter().drawEntity(this, g2d, Y, fontMetrics,
				getField().getFieldID(), previousEntity, nextEntity);
	}

	public void setInitialHoldEvent(ChainedApexEvent event) {
		if (event != null)
			event.setID("" + Constants.APEX_INITIAL_HOLD_CODE);
		manager.getChainedApexEventsCollection().setInitialHoldEvent(event);
	}

	public void setFinalHoldEvent(ChainedApexEvent event) {
		if (event != null)
			event.setID("" + Constants.APEX_FINAL_HOLD_CODE);
		manager.getChainedApexEventsCollection().setFinalHoldEvent(event);
	}

	public void setOnsetEvent(ChainedApexEvent event) {
		if (event != null)
			event.setID("" + Constants.APEX_ONSET_CODE);
		manager.getChainedApexEventsCollection().setOnsetEvent(event);
	}

	public void setOffsetEvent(ChainedApexEvent event) {
		if (event != null)
			event.setID("" + Constants.APEX_OFFSET_CODE);
		manager.getChainedApexEventsCollection().setOffsetEvent(event);
	}

	public String addOnsetEvent() {
		if (previousEntity != null) {
			Time endMovieTimeInfo = previousEntity.getEndTimeInfo();

			ChainedApexEvent event = (ChainedApexEvent) manager.getFirstEvent();
			Time selectedEventStartTimeInfo = segmentPanel.getMultipleMovieController()
					.getPrecedingFrameTimeInfo(event.getStartTimeInfo());

			if (selectedEventStartTimeInfo.lessThan(endMovieTimeInfo)) {
				return ErrorMessages.CHAINS_CANNOT_OVERLAP;
			}
		}

		manager.addOnsetEvent();
		return null;
	}

	public String addInitialHoldEvent() {
		ChainedApexEvent onset = getOnset();
		if (onset != null) {
			Time onsetStartTimeInfo = onset.getStartTimeInfo();
			Time onsetEndTimeInfo = onset.getEndTimeInfo();
			Time precedingFrameEndTimeInfo = segmentPanel.getMultipleMovieController()
				.getPrecedingFrameTimeInfo(onsetEndTimeInfo);

			if (precedingFrameEndTimeInfo.equals(onsetStartTimeInfo)) {
				return ErrorMessages.ADD_INITIAL_HOLD_CANT_ADJUST_ONSET;
			} else {
				manager.addInitialHoldEvent();
				return null;
			}
		}

		if (previousEntity != null) {
			Time endMovieTimeInfo = previousEntity.getEndTimeInfo();

			ChainedApexEvent event = (ChainedApexEvent) manager.getFirstEvent();
			Time selectedEventStartTimeInfo = segmentPanel.getMultipleMovieController()
					.getPrecedingFrameTimeInfo(event.getStartTimeInfo());

			if (selectedEventStartTimeInfo.lessThan(endMovieTimeInfo)) {
				return ErrorMessages.CHAINS_CANNOT_OVERLAP;
			}
		}

		manager.addInitialHoldEvent();
		return null;
	}

	public String addFinalHoldEvent() {
		// GREG TODO: should the order of the 2 below if-stmts be reversed, like in above addInitialHoldEvent ?
		if (nextEntity != null) {
			Time startMovieTimeInfo = nextEntity.getStartTimeInfo();

			ChainedApexEvent event = (ChainedApexEvent) manager.getLastEvent();
			Time selectedEventEndTimeInfo = segmentPanel.getMultipleMovieController()
					.getFollowingFrameTimeInfo(event.getEndTimeInfo());

			if (selectedEventEndTimeInfo.greaterThan(startMovieTimeInfo)) {
				return ErrorMessages.CHAINS_CANNOT_OVERLAP;
			}
		}

		ChainedApexEvent offset = getOffset();
		if (offset != null) {
			Time onsetStartTimeInfo = offset.getStartTimeInfo();
			Time onsetEndTimeInfo = offset.getEndTimeInfo();
			Time precedingFrameEndTimeInfo = segmentPanel.getMultipleMovieController()
				.getPrecedingFrameTimeInfo(onsetEndTimeInfo);

			if (precedingFrameEndTimeInfo.equals(onsetStartTimeInfo)) {
				return ErrorMessages.ADD_FINAL_HOLD_CANT_ADJUST_OFFSET;
			}
		}

		manager.addFinalHoldEvent();
		return null;
	}

	public String addOffsetEvent() {
		if (nextEntity != null) {
			Time startMovieTimeInfo = nextEntity.getStartTimeInfo();

			ChainedApexEvent event = (ChainedApexEvent) manager.getFirstEvent();
			Time selectedEventEndTimeInfo = segmentPanel.getMultipleMovieController()
					.getFollowingFrameTimeInfo(event.getEndTimeInfo());

			if (selectedEventEndTimeInfo.greaterThan(startMovieTimeInfo)) {
				return ErrorMessages.CHAINS_CANNOT_OVERLAP;
			}
		}

		manager.addOffsetEvent();
		return null;
	}
}
