package edu.bu.signstream.grepresentation.fields.apexField;

import java.awt.Font;
import java.awt.Graphics2D;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.stream.Collectors;

import edu.bu.signstream.common.util.Sorter;
import edu.bu.signstream.common.util.vo.TrackValue;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.time.Interval;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.time.TimeRange;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Painter;
import edu.bu.signstream.grepresentation.fields.glossField.EnteredText;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class ChainedApexEventsEntityCollection {
	private ArrayList<ChainedApexEventsEntity> entities = new ArrayList<ChainedApexEventsEntity>();
	private SignStreamSegmentPanel segmentPanel;

	private NewChainedApexEvent newEvent;
	private EnteredText enteredText;

	public ChainedApexEventsEntityCollection(
			ArrayList<ChainedApexEventsEntity> entities,
			SignStreamSegmentPanel segmentPanel) {
		this.entities = entities;
		this.segmentPanel = segmentPanel;

        sortEntities();
	}

	public EnteredText getEnteredText() {
		return enteredText;
	}

	public void setEnteredText(EnteredText enteredText) {
		this.enteredText = enteredText;
	}

	public ChainedApexEventsEntity getNextEntity(Time endTimeInfo) {
		ChainedApexEventsEntity nextEntity = null;
		for (ChainedApexEventsEntity entity : entities) {
			if(entity.getStartTimeInfo().greaterThan(endTimeInfo)) {
				if(nextEntity == null) {
					nextEntity = entity;
				} else if(entity.getStartTimeInfo().lessThan(nextEntity.getStartTimeInfo())) {
					nextEntity = entity;
				}
			}
		}
		return nextEntity;
	}
	
	public ChainedApexEventsEntity getPreviousEntity(Time startTimeInfo) {
		ChainedApexEventsEntity prevEntity = null;
		for (ChainedApexEventsEntity entity : entities) {
			if(entity.getStartTimeInfo().lessThan(startTimeInfo)) {
				if(prevEntity == null) {
					prevEntity = entity;
				} else if(entity.getStartTimeInfo().greaterThan(prevEntity.getStartTimeInfo())) {
					prevEntity = entity;
				}
			}
		}
		return prevEntity;
	}

	public ArrayList<ChainedApexEventsEntity> getEntitiesInInterval(Interval interval) {
        return entities.stream()
                .filter(e -> e.overlaps(interval))
                .collect(Collectors.toCollection(ArrayList::new));
	}

	public void removeEmptyEntities() {
		ArrayList<ChainedApexEventsEntity> entitiesToDelete = new ArrayList<ChainedApexEventsEntity>();

		for (ChainedApexEventsEntity entity : entities) {
			if (entity.getStartTimeInfo() == null || entity.getEndTimeInfo() == null) {
				entitiesToDelete.add(entity);
			}
		}

		for (ChainedApexEventsEntity entityToDelete : entitiesToDelete) {
			deleteEntity(entityToDelete);
		}
	}
	
	/**
	 * Cleanup method: if there are hanging references to a deleted field value, which is residing in this 
	 * ChainedApexEventsEntityCollection with an invalid field value ID, then it has since been deleted from the
	 * coding schema, but not yet form this collection.
	 * @param fieldID The Field ID of the field to get from the coding scheme
	 * @return true if succesful, false otherwise
	 */
	boolean deleteInvalidEntities(String fieldID) {
		ArrayList<ChainedApexEventsEntity> entitiesToDelete = new ArrayList<ChainedApexEventsEntity>();
		
		SS3CodingScheme dbScheme = segmentPanel.getSS3Database().getSS3CodingScheme();
		SS3Field ss3Field = dbScheme.getField(fieldID);
		ArrayList<SS3FieldValue> ss3FieldValues = ss3Field.getValuesAsList();
		ArrayList<String> ss3FieldValueIDs = new ArrayList<String>();
		
		for (SS3FieldValue ss3FieldValue: ss3FieldValues) {
			String valueID = ss3FieldValue.getId();
			if (valueID == null || valueID.isEmpty())
				continue;
			
			ss3FieldValueIDs.add(valueID);
		}
		
		if (ss3FieldValueIDs.isEmpty()) {
			return false;
		}
		
		for (ChainedApexEventsEntity entity: entities) {
			String valueID = entity.getValueOrTextValueEvent().getID();
			if ( !ss3FieldValueIDs.contains(valueID)) {
				entitiesToDelete.add(entity);
			}
		}

		for (ChainedApexEventsEntity entityToDelete : entitiesToDelete) {
			deleteEntity(entityToDelete);
		}		
		
		return true;
	}

	public ChainedApexEventsEntity getEntityWithRefID(String refEntityId) {
		for (ChainedApexEventsEntity entity : entities) {
			if (refEntityId != null
					&& refEntityId.equalsIgnoreCase(entity.getRefEntityId()))
				return entity;
		}
		return null;
	}

	public ChainedApexEventsEntity getEntity(String entityID) {
		for (ChainedApexEventsEntity entity : entities) {
			if (entity.getID().equalsIgnoreCase(entityID))
				return entity;
		}
		return null;
	}

	public ArrayList<ChainedApexEventsEntity> getEntities() {
		return entities;
	}

	public int size() {
		return entities.size();
	}

	public void deleteEntity(ChainedApexEventsEntity entity) {
		ChainedApexEventsEntity previousEntity = entity.getPreviousEntity();
		ChainedApexEventsEntity nextEntity = entity.getNextEntity();

		if (previousEntity != null) {
			previousEntity.setNextEntity(nextEntity);
		}

		if (nextEntity != null) {
			nextEntity.setPreviousEntity(previousEntity);
		}

		boolean removed = entities.remove(entity);

        if(!removed) {
            Time startMovieTimeInfo = entity.getStartTimeInfo();
            Time endMovieTimeInfo = entity.getEndTimeInfo();

            ChainedApexEventsEntity entityToRemove = null;

            for(ChainedApexEventsEntity entity1 : entities) {
                Time startMovieTime1Info = entity1.getStartTimeInfo();
                Time endMovieTime1Info = entity1.getEndTimeInfo();

                if(startMovieTimeInfo.equals(startMovieTime1Info) && endMovieTimeInfo.equals(endMovieTime1Info)) {
                    entityToRemove = entity1;
                }
            }

            entities.remove(entityToRemove);
        }
	}

	public double getStartTime() {
		removeEmptyEntities();
		double startTime = 0;
		for (ChainedApexEventsEntity entity : entities) {
			startTime = startTime < entity.getStartTimeInfo().value() ? startTime
					: entity.getStartTimeInfo().value();
		}
		return startTime;
	}

	public double getEndTime() {
		removeEmptyEntities();
		double endTime = 0;
		for (ChainedApexEventsEntity entity : entities) {
			endTime = endTime > entity.getEndTimeInfo().value() ? endTime
					: entity.getEndTimeInfo().value();
		}
		return endTime;
	}
	
	public Time getStartTimeInfo() {
		removeEmptyEntities();
		
		if (entities.size() == 0)
			return TimeRange.ABSOLUTE.zero();
		
		Time earliestStartTime = entities.get(0).getStartTimeInfo();
		
		for (ChainedApexEventsEntity entity : entities) {
			earliestStartTime = earliestStartTime.value() < entity.getStartTimeInfo().value() ? earliestStartTime
					: entity.getStartTimeInfo();
		}
		
		return earliestStartTime;
	}
	
	public Time getEndTimeInfo() {
		removeEmptyEntities();
		
		if (entities.size() == 0)
			return TimeRange.ABSOLUTE.zero();
		
		Time endTimeInfo = entities.get(0).getStartTimeInfo();
		
		for (ChainedApexEventsEntity entity : entities) {
			endTimeInfo = endTimeInfo.value() > entity.getStartTimeInfo().value() ? endTimeInfo
					: entity.getStartTimeInfo();
		}
		
		return endTimeInfo;
	}

	public double getStartTimeCoordinate() {
		return SS3Singleton.getVideoControlManager().getZoomer()
				.getCoordinateTimeConvertor().convertTimeInfoToCoordinate(getStartTimeInfo());
	}

	public double getEndTimeCoordinate() {
		return SS3Singleton.getVideoControlManager().getZoomer()
				.getCoordinateTimeConvertor().convertTimeInfoToCoordinate(getEndTimeInfo());
	}

	public void unselect() {
		for (ChainedApexEventsEntity entity : entities) {
			entity.unselect();
		}
	}

	public void reset() {
		for (ChainedApexEventsEntity entity : entities) {
			entity.reset();
		}
	}

	public void addEntity(ChainedApexEventsEntity entity) {
		
		double startMovieTime = entity.getStartTimeInfo().value();
		
		ChainedApexEventsEntity previousEntity = null;
		ChainedApexEventsEntity nextEntity = null;

		for (ChainedApexEventsEntity ent : entities) {
			double entMovieStartTime = ent.getStartTimeInfo().value();
			
			// exit function in case that this event is already in the list
			if (entMovieStartTime == startMovieTime)
				return;

			if (entMovieStartTime < startMovieTime) {
				if (previousEntity == null) {
					previousEntity = ent;
				} else if (entMovieStartTime > previousEntity
						.getStartTimeInfo().value()) {
					previousEntity = ent;
				}
			} else {
				if (nextEntity == null) {
					nextEntity = ent;
				} else if (entMovieStartTime < nextEntity.getStartTimeInfo()
						.value()) {
					nextEntity = ent;
				}
			}
		}
		
		if (previousEntity != null) {
			ChainedApexEventsEntity next = previousEntity.getNextEntity();
			if (next != null) {
				entity.setNextEntity(next);
				next.setPreviousEntity(entity);
			}
			previousEntity.setNextEntity(entity);
			entity.setPreviousEntity(previousEntity);

		} else if (nextEntity != null) {
			entity.setNextEntity(nextEntity);
			nextEntity.setPreviousEntity(entity);
		}

		this.entities.add(entity);
	}

	public void selectEntity(double x) {
		EventsEntity entity = getChainedApexEventsEntityAt(x);
		if (entity != null)
			entity.select();
	}

	public ChainedApexEventsEntity getChainedApexEventsEntityAt(double coordX) {
		for (ChainedApexEventsEntity entity : entities) {
			if (entity.contains(coordX))
				return entity;
		}
		return null;
	}

	public Event getSelectedEvent() {
		for (ChainedApexEventsEntity entity : entities) {
			Event event = entity.getSelectedEvent();
			if(event != null) { 
				return event;
			}
		}
		return null;
	}

	public ChainedApexEventsEntity getSelectedEntity() {
		for (ChainedApexEventsEntity entity : entities) {
			Event event = entity.getSelectedEvent();
			if(event != null) {
				return entity;
			}
		}
		return null;
	}

	public ChainedApexEvent getEventAt(double x) {
		ChainedApexEventsEntity entity = getChainedApexEventsEntityAt(x);
		if (entity == null)
			return null;

		return entity.getChainedApexEventsCollection().getEventAt(x);
	}

	public void setSelectedEvementValueID(String valueID) {
		Event event = getSelectedEvent();

		if (event instanceof ChainedApexEvent) {
			ChainedApexEvent chainedEvent = (ChainedApexEvent) event;
			chainedEvent.setID(valueID);
		}
	}

	public NewChainedApexEvent getNewChainedApexEvent() {
		return newEvent;
	}

	public void setNewChainedApexEvent(NewChainedApexEvent newEvent) {
		this.newEvent = newEvent;
	}

	public ChainedApexEventsCollection createNewChainedApexEventCollection(
			String valueID) {
		if (valueID != null && newEvent != null) {
			TrackValue trackValue = new TrackValue();
			trackValue.setValueId(valueID);

			trackValue.setStartTimeInfo(newEvent.getStartTimeInfo());
			trackValue.setEndTimeInfo(newEvent.getEndTimeInfo());

			ChainedApexEvent event = new ChainedApexEvent(trackValue, segmentPanel);

			ChainedApexEventsCollection collection = new ChainedApexEventsCollection();
			collection.setValueEvent(event);
			newEvent = null;

			return collection;
		}
		return null;
	}

    public void sortEntities() {
        int entitiesSize = entities.size();

        if(entitiesSize == 1)
            return;

        double[] startTimes = new double[entitiesSize];
        HashMap<String, ChainedApexEventsEntity> map = new HashMap<String, ChainedApexEventsEntity>();

        int i = 0;
        for(ChainedApexEventsEntity entity : entities) {
            double movieTime = entity.getStartTimeInfo().value();
            map.put(""+movieTime, entity);
            startTimes[i] = movieTime;
            i++;
        }

        Sorter.sort(startTimes);

        i = 0;
        while(i < entitiesSize-1) {
            ChainedApexEventsEntity glsEntity = map.get(""+startTimes[i]);
            i++;
            ChainedApexEventsEntity nextGlsEntity = map.get(""+startTimes[i]);
            glsEntity.setNextEntity(nextGlsEntity);
            nextGlsEntity.setPreviousEntity(glsEntity);
        }
    }

	public void sort(ArrayList<ChainedApexEventsEntity> entitiesToSort) {
		removeEmptyEntities();

		HashMap<String, ChainedApexEventsEntity> map = new HashMap<String, ChainedApexEventsEntity>();

		double[] startTimes = new double[entitiesToSort.size()];

		ArrayList<ChainedApexEventsEntity> toRemove = new ArrayList<ChainedApexEventsEntity>();

		for (int i = 0; i < entitiesToSort.size(); i++) {
			ChainedApexEventsEntity entity = entitiesToSort.get(i);
			if (entity != null && entity.getStartTimeInfo() != null) {
				double startTime = entity.getStartTimeInfo().value();
				map.put("" + startTime, entity);
				startTimes[i] = startTime;
			} else {
				toRemove.add(entity);
			}
		}

		for (ChainedApexEventsEntity cEnt : toRemove) {
			entitiesToSort.remove(cEnt);
		}

		Sorter.sort(startTimes);
		entitiesToSort.clear();

		for (int j = 0; j < startTimes.length; j++) {
			ChainedApexEventsEntity obj = map.get("" + startTimes[j]);
			entitiesToSort.add(obj);
		}
	}

	public void paint(Graphics2D g2d, double Y, Font font) {
		for (ChainedApexEventsEntity entity : entities) {
			entity.paint(g2d, Y, font);
		}

		if (newEvent != null) {
			Painter painter = segmentPanel.getPainter();
			painter.drawNewEvent(newEvent, g2d, Y);
		}
	}
}
