package edu.bu.signstream.grepresentation.fields.gestureField;

import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.util.ArrayList;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.time.Duration;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.apexField.ApexChainedApexEventsEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.handshapes.GestureHandShapeEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.handshapes.GestureHandShapeField;
import edu.bu.signstream.grepresentation.fields.gestureField.handshapes.GestureHandShapeFieldWrapper;
import edu.bu.signstream.grepresentation.fields.gestureField.locationField.GestureLocationEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.orientationField.GestureOrientationEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.orientationField.GestureOrientationField;
import edu.bu.signstream.grepresentation.fields.gestureField.orientationField.GestureOrientationFieldWrapper;
import edu.bu.signstream.grepresentation.fields.gestureField.signTypeField.GestureTypeEntity;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolbarCommons;

public class GestureChainedEventsEntity extends EventsEntity {

	private static final long serialVersionUID = 1L;

	protected GestureChainedEventsCollectionManager manager;

	private GestureChainedEventsEntity previousEntity = null;
	private GestureChainedEventsEntity nextEntity = null;
	
	private boolean isLockedGesturePhonAlignments = false;
    
    private boolean incompleteGesture = false;
    private boolean showUniqueNumber = false;
    private String uniqueNumber = null;

	public GestureChainedEventsEntity(GestureChainedEventsCollection collection,
			SignStreamSegmentPanel segmentPanel, Field field) {

		super(segmentPanel, field);
		setID(Util.getLongUniqueNumber());
		this.manager = new GestureChainedEventsCollectionManager(collection,
				segmentPanel);
	}
	
	public boolean isLockedGesturePhonAlignments() {
		return isLockedGesturePhonAlignments;
	}

	public void setLockedGesturePhonAlignments(boolean isLockedGesturePhonAlignments) {
		this.isLockedGesturePhonAlignments = isLockedGesturePhonAlignments;
	}
	
    // returns an error code that will be matched with an error message or null if success. 
	public String canSetSelectedEventStartMovieTime(Time timeInfo) {
		Event selectedEvent =  SS3Singleton.getMediaToolBar(null).selectedEvent();
		
        Time originalSelectedEndTimeInfo = selectedEvent.getEndTimeInfo();

        Time maxStartMovieTimeInfo = originalSelectedEndTimeInfo.subtract(new Duration(1, originalSelectedEndTimeInfo.unit()));

		if (maxStartMovieTimeInfo.lessThan(timeInfo)) {
			return ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME;
		}

		if (previousEntity != null) {
			if (previousEntity.getEndTimeInfo().greaterThan(timeInfo)) {
				return ErrorMessages.CHAINS_CANNOT_OVERLAP;
			}
		}

		boolean becomingOneFrame = timeInfo.equals(originalSelectedEndTimeInfo);
		if (becomingOneFrame) {
			if ((nextEntity != null && nextEntity.getStartTimeInfo().equals(timeInfo) && nextEntity.getEndTimeInfo().equals(timeInfo)) 
				|| (previousEntity != null && previousEntity.getStartTimeInfo().equals(timeInfo) && previousEntity.getEndTimeInfo().equals(timeInfo))) {
				return ErrorMessages.SET_ONE_FRAMES_CANNOT_OVERLAP;				
			}
		}

		SS3Singleton.getMediaToolBar().update();
		return null;
	}

	public String canSetSelectedEventEndMovieTime(Time timeInfo) {
		Event selectedEvent = SS3Singleton.getMediaToolBar(null).selectedEvent();
		
        Time originalSelectedStartTimeInfo = selectedEvent.getStartTimeInfo();

        Time minEndMovieTimeInfo = originalSelectedStartTimeInfo.add(new Duration(1, originalSelectedStartTimeInfo.unit()));

		if (minEndMovieTimeInfo.greaterThan(timeInfo)) {
			return ErrorMessages.EVENT_END_TIME_FOLLOW_START_TIME;
		}
		
		if (nextEntity != null) {
			if (nextEntity.getStartTimeInfo().lessThan(timeInfo)) {
				return ErrorMessages.CHAINS_CANNOT_OVERLAP;
			}
		}

		boolean becomingOneFrame = timeInfo.equals(originalSelectedStartTimeInfo);
		if (becomingOneFrame) {
			if ((nextEntity != null && nextEntity.getStartTimeInfo().equals(timeInfo) && nextEntity.getEndTimeInfo().equals(timeInfo)) 
				|| (previousEntity != null && previousEntity.getStartTimeInfo().equals(timeInfo) && previousEntity.getEndTimeInfo().equals(timeInfo))) {
				return ErrorMessages.SET_ONE_FRAMES_CANNOT_OVERLAP;				
			}
		}
		
		SS3Singleton.getMediaToolBar().update();		
		return null;
	}

	@Deprecated
	public void setStartTime(double startTime) {
		GestureChainedEvent event = (GestureChainedEvent) manager.getFirstEvent();
		if (event != null)
		    event.setStartTimeInfo(SS3Singleton.getVideoTimeRange().time(Math.round(startTime), TimeUnit.TIMESTAMP));
	}

	@Override
	public void setStartTimeInfo(Time startTimeInfo) {
		GestureChainedEvent event = (GestureChainedEvent) manager.getFirstEvent();
		if (event != null)
			event.setStartTimeInfo(startTimeInfo);
		
		GestureHandShapeEntity depHsEntity = null;
		GestureOrientationEntity depOrnEntity = null;
		if (getField().getFieldID().equals(Util.DOM_GESTURE_HAND_FIELD_ID) || getField().getFieldID().equals(Util.NDOM_GESTURE_HAND_FIELD_ID)) {
            ArrayList depWrappers = segmentPanel.getSelectedFieldWrapper().getDependentWrappers();
            if (!depWrappers.isEmpty()) {
                GestureHandShapeFieldWrapper hsFieldWrapper = (GestureHandShapeFieldWrapper) depWrappers.get(0);
                GestureHandShapeField hsField = hsFieldWrapper.getGestureHandShapeField();
                depHsEntity = (GestureHandShapeEntity) hsField.getEntityWithRefID(getID());
                
                GestureOrientationFieldWrapper ornFieldWrapper = (GestureOrientationFieldWrapper) depWrappers.get(1);
                GestureOrientationField ornField = ornFieldWrapper.getGestureOrientationField();
                depOrnEntity = (GestureOrientationEntity) ornField.getEntityWithRefID(getID());
            }
		}

		if (depHsEntity != null) {
			MediaToolbarCommons.setGestureHandshapeStartTime(depHsEntity, startTimeInfo);
		}
		if (depOrnEntity != null) {
			MediaToolbarCommons.setGestureOrientationStartTime(depOrnEntity, startTimeInfo);
		}
	}

	@Override
	public void setEndTimeInfo(Time endTimeInfo) {
		GestureChainedEvent event = (GestureChainedEvent) manager.getLastEvent();
		if (event != null)
			event.setEndTimeInfo(endTimeInfo);
		
		GestureHandShapeEntity depHsEntity = null;
		GestureOrientationEntity depOrnEntity = null;
		if (getField().getFieldID().equals(Util.DOM_GESTURE_HAND_FIELD_ID) || getField().getFieldID().equals(Util.NDOM_GESTURE_HAND_FIELD_ID)) {
            ArrayList depWrappers = segmentPanel.getSelectedFieldWrapper().getDependentWrappers();
            if (!depWrappers.isEmpty()) {
                GestureHandShapeFieldWrapper hsFieldWrapper = (GestureHandShapeFieldWrapper) depWrappers.get(0);
                GestureHandShapeField hsField = hsFieldWrapper.getGestureHandShapeField();
                depHsEntity = (GestureHandShapeEntity) hsField.getEntityWithRefID(getID());
                
                GestureOrientationFieldWrapper ornFieldWrapper = (GestureOrientationFieldWrapper) depWrappers.get(1);
                GestureOrientationField ornField = ornFieldWrapper.getGestureOrientationField();
                depOrnEntity = (GestureOrientationEntity) ornField.getEntityWithRefID(getID());
            }
		}

		if (depHsEntity != null) {
			MediaToolbarCommons.setGestureHandshapeEndTime(depHsEntity, endTimeInfo);
		}
		if (depOrnEntity != null) {
			MediaToolbarCommons.setGestureOrientationEndTime(depOrnEntity, endTimeInfo);
		}
	}

	@Deprecated
	public void setEndTime(double endTime) {
		GestureChainedEvent event = (GestureChainedEvent) manager.getLastEvent();
		if (event != null)
            event.setEndTimeInfo(SS3Singleton.getVideoTimeRange().time(Math.round(endTime), TimeUnit.TIMESTAMP));
	}

	public GestureChainedEventsEntity getPreviousEntity() {
		return previousEntity;
	}

	public void setPreviousEntity(GestureChainedEventsEntity previousEntity) {
		this.previousEntity = previousEntity;
	}

	public GestureChainedEventsEntity getNextEntity() {
		return nextEntity;
	}

	public void setNextEntity(GestureChainedEventsEntity nextEntity) {
		this.nextEntity = nextEntity;
	}

	public String getText() {
		return getTextValue().getText();
	}

	public Event getNextEvent(Event event) {
		return manager.getGestureChainedEventsCollection().getNextEvent(event);
	}

	public Event getPreviousEvent(Event event) {
		return manager.getGestureChainedEventsCollection().getPreviousEvent(event);
	}

	public void setField(GestureField nmField) {
		super.setField(nmField);
	}

	public GestureChainedEventsEntity clone(String fieldID) {
		GestureChainedEventsCollection cll = manager.getGestureChainedEventsCollection()
				.cloneGestureChainedEventsCollection();
		GestureChainedEventsEntity ret = new GestureChainedEventsEntity(cll, segmentPanel, getField());
		ret.updateParentEntity();
		return ret;
	}

	@Override
	public SS3Entity getSS3Entity() {
		SS3Entity entity = new SS3Entity();
		String id = getID();
		
		String entID = id.trim().length() > 0 ? id : Util.getLongUniqueNumber();
		entity.setID(entID);
		if (getParentEntities() != null)
			entity.setParentEntities(getParentEntities());
		entity.setRefEntityId(getRefEntityId());

		GestureChainedEventsCollection collection = manager
				.getGestureChainedEventsCollection();

		if (collection.getPreparationEvent() != null) {
			SS3Item onsetItem = createSS3Item(collection.getPreparationEvent());
			entity.addItem(onsetItem);
		}

		if (collection.getPrestrokeHoldEvent() != null) {
			SS3Item inHoldItem = createSS3Item(collection.getPrestrokeHoldEvent());
			entity.addItem(inHoldItem);
		}

		if (collection.getValueEvent() != null) {
			SS3Item valueItem = createSS3Item(collection.getValueEvent());
			entity.addItem(valueItem);
		}

		if (collection.getTextValueEvent() != null) {
			SS3Item txtValueItem = createSS3Item(collection.getTextValueEvent());
			entity.addItem(txtValueItem);
		}

		if (collection.getPoststrokeHoldEvent() != null) {
			SS3Item fnHoldItem = createSS3Item(collection.getPoststrokeHoldEvent());
			entity.addItem(fnHoldItem);
		}

		if (collection.getRetractionEvent() != null) {
			SS3Item offsetItem = createSS3Item(collection.getRetractionEvent());
			entity.addItem(offsetItem);
		}
		
		if (collection.getRelaxedEvent() != null) {
			SS3Item relaxedItem = createSS3Item(collection.getRelaxedEvent());
			entity.addItem(relaxedItem);
		}
		
		boolean lockedGesturePhoneAlignments = isLockedGesturePhonAlignments();
		entity.setLockedGesturePhoneAlignments(lockedGesturePhoneAlignments);
		
		if(this instanceof GestureEventsEntity) {
			GestureEventsEntity  gestEntity = (GestureEventsEntity)this;
			
			entity.setIncompleteGesture(gestEntity.getIncompleteGesture());
			entity.setShowUniqueNumber(gestEntity.getShowUniqueNumber());
			entity.setUniqueNumber(gestEntity.getUniqueNumber());
			
			boolean isLockedDomNdomAlignments = gestEntity.isLockedDomNdomAlignments();
			entity.setLockedDomNdomAlignments(isLockedDomNdomAlignments);
			
			GestureChainedEventsEntity dependentGestureEntity = gestEntity.getDependentGestureEntity();
			GestureChainedEventsEntity dependentHandEntity = gestEntity.getDependentHandEntity();
			ApexChainedApexEventsEntity dependentApexEntity = gestEntity.getDependentApexEntity();
			GestureHandShapeEntity dependentHandShapeEntity = gestEntity.getDependentHandShapeEntity();
			GestureOrientationEntity dependentOrientationEntity = gestEntity.getDependentOrientationEntity();
			GestureLocationEntity dependentLocationEntity = gestEntity.getDependentLocationEntity();
			GestureTypeEntity dependentSignTypeEntity = gestEntity.getDependentGestureTypeEntity();
						
			if(dependentGestureEntity != null) {
				entity.setDependentGestureEntityID(dependentGestureEntity.getID());
			}
			if (dependentApexEntity != null) {
				entity.setDependentApexEntityID(dependentApexEntity.getID());
			}
			if(dependentHandEntity != null) {
				entity.setDependentHandEntityID(dependentHandEntity.getID());
			}
			if(dependentHandShapeEntity != null) {
				//String hndID = dependentHandShapeEntity.getID();
				entity.setDependentHandShapeEntityID(dependentHandShapeEntity.getID());
			}
			if (dependentOrientationEntity != null) {
				entity.setDepenentOrientationEntityID(dependentOrientationEntity.getID());
			}
			if (dependentLocationEntity != null) {
				entity.setDependentLocationEntityID(dependentLocationEntity.getID());
			}
			if (dependentSignTypeEntity != null) {
				entity.setDependentSignTypeEntityID(dependentSignTypeEntity.getID());
			}

            GestureCompoundable startCompound = gestEntity.getStartCompound();
            if(startCompound != null) {
                entity.setStartCompoundEntityID(startCompound.getID());
            }

            GestureCompoundable endCompound = gestEntity.getEndCompound();
            if(endCompound != null) {
                entity.setEndCompoundEntityID(endCompound.getID());
            }
			
			GestureEntityProperties properties = gestEntity.getGestureEntityProperties();
			entity.setGestureEntityProperties(properties);
		}
		return entity;
	}

	private SS3Item createSS3Item(GestureChainedEvent event) {
		SS3Item item = new SS3Item();
		item.setId(event.getID());
		item.setText(event.getText());
		item.setStartTimeInfo(event.getStartTimeInfo());
		item.setEndTimeInfo(event.getEndTimeInfo());
		return item;
	}

	public Event getValueOrTextValueEvent() {
		return manager.getGestureChainedEventsCollection().getValueOrTextValueEvent();
	}

	public GestureChainedEventsCollection getGestureChainedEventsCollection() {
		return manager.getGestureChainedEventsCollection();
	}

	public GestureChainedEvent getValue() {
		return manager.getGestureChainedEventsCollection().getValueEvent();
	}

	public GestureChainedEvent getTextValue() {
		return manager.getGestureChainedEventsCollection().getTextValueEvent();
	}

	public GestureChainedEvent getGestureChainedEvent() {
		return manager.getGestureChainedEventsCollection().getTextValueEvent();
	}

	@Override
	public Event getFirstEvent() {
		return manager.getFirstEvent();
	}

	public GestureChainedEvent getPrestrokeHold() {
		return manager.getGestureChainedEventsCollection().getPrestrokeHoldEvent();
	}

	public GestureChainedEvent getPoststrokeHold() {
		return manager.getGestureChainedEventsCollection().getPoststrokeHoldEvent();
	}

	public GestureChainedEvent getPreparation() {
		return manager.getGestureChainedEventsCollection().getPreparationEvent();
	}
	
	public GestureChainedEvent getRetraction() {
		return manager.getGestureChainedEventsCollection().getRetractionEvent();
	}
	
	public GestureChainedEvent getRelaxed() {
		return manager.getGestureChainedEventsCollection().getRelaxedEvent();
	}

	public void deletePreparation() {
		manager.deletePreparationEventNoWarning();
	}

	public void deletePrestrokeHold() {
		manager.deletePrestrokeHoldEventNoWarning();
	}

	public void deletePoststrokeHold() {
		manager.deletePoststrokeHoldEventNoWarning();
	}

	public void deleteRetraction() {
		manager.deleteRetractionEventNoWarning();
	}
	
	public void deleteRelaxed() {
		manager.deleteRelaxedEventNoWarning();
	}

	@Override
	public Event getLastEvent() {
		return manager.getLastEvent();
	}

	@Override
	public Time getStartTimeInfo() {
		GestureChainedEvent event = (GestureChainedEvent) manager.getFirstEvent();
		if (event == null)
			return null;
		return event.getStartTimeInfo();
	}

	@Override
	public Time getEndTimeInfo() {
		GestureChainedEvent event = (GestureChainedEvent) manager.getLastEvent();
		if (event == null)
			return null;
		return event.getEndTimeInfo();
	}

	@Override
	public boolean selectEntityIfEventSelected() {
		if (manager.getSelectedEvent() != null) {
			select();
			return true;
		}
		return false;
	}

	@Override
	public boolean selectEventIfEntitySelected() {
		if (selected) {
			manager.unselectEntity();
			GestureChainedEvent valueEvent = manager.getGestureChainedEventsCollection()
					.getValueEvent();
			manager.selectGestureChainedEvent(valueEvent);
			selected = false;
			return true;
		}
		return false;
	}

	@Override
	public boolean select() {
		selected = true;
		return selected;
	}

	@Override
	public boolean unselect() {
		selected = false;
		unhighlight();
		manager.unselectEntity();
		return true;
	}

	@Override
	public boolean adjustEntity(int x1, int x2) {
		double pixelInTimeFrame = segmentPanel.getZoomer()
				.getPixelsInTimeFrame();
		double startTime = x1 / pixelInTimeFrame;
		double endTime = x2 / pixelInTimeFrame;
		return manager.adjustEntity(startTime, endTime);
	}

	@Override
	public Event getSelectedEvent() {		
		if (selected)
			return this;
		return manager.getSelectedEvent();
	}

	@Override
	public void reset() {
		selected = false;
		// mToolBar.setEnable(false);
		manager.resetAll();
	}

	@Override
	public boolean delete() {
		return delete(false);
	}

	public boolean delete(boolean showWarningMsg) {
		return manager.deleteEntity();
	}

	public void setValueID(String valueID) {
		GestureChainedEventsCollection collection = manager
				.getGestureChainedEventsCollection();
		GestureChainedEvent event = collection.getValueEvent();
		event.setID(valueID);
	}

	@Override
	public Event getEventAt(double x) {
		return manager.getEventAt(x);
	}

	@Override
	public ArrayList<String> getEventsToAdd() {
		ArrayList<String> list = new ArrayList<String>();

		GestureChainedEventsCollection collection = manager
				.getGestureChainedEventsCollection();

		if (collection.getPreparationEvent() == null)
			list.add(Constants.PREPARATION);

		if (collection.getPrestrokeHoldEvent() == null)
			list.add(Constants.PRESTROKE_HOLD);

		if (collection.getPoststrokeHoldEvent() == null)
			list.add(Constants.POSTSTROKE_HOLD);

		if (collection.getRetractionEvent() == null)
			list.add(Constants.RETRACTION);
		
		if (collection.getRelaxedEvent() == null)
			list.add(Constants.RELAXED);

		return list;
	}

	@Override
	public ArrayList<Event> getEventsList() {
		return new ArrayList<Event>(getGestureChainedEventsCollection().getCollection());
    }

	@Override
	public String getToolTipText(int x) {
		GestureChainedEvent event = (GestureChainedEvent) manager.getEventAt(x);

		if (event != null)
			return event.getToolTipText();
		return null;
	}

	@Override
	public boolean selectEvent(double x) {
		Event event = manager.getEventAt(x);
		return event.select();
	}

	@Override
	public void paint(Graphics2D g2d, double Y, Font font) {
		g2d.setFont(font);
		FontMetrics fontMetrics = getFontMetrics(font);
		segmentPanel.getPainter().drawEntity(this, g2d, Y, fontMetrics,
				getField().getFieldID(), previousEntity, nextEntity);
	}

	public void setPrestrokeHoldEvent(GestureChainedEvent event) {
		if (event != null)
			event.setID("" + Constants.GESTURE_PRESTROKE_HOLD_CODE);
		manager.getGestureChainedEventsCollection().setPrestrokeHoldEvent(event);
	}

	public void setPoststrokeHoldEvent(GestureChainedEvent event) {
		if (event != null)
			event.setID("" + Constants.GESTURE_POSTSTROKE_HOLD_CODE);
		manager.getGestureChainedEventsCollection().setPoststrokeHoldEvent(event);
	}

	public void setPreparationEvent(GestureChainedEvent event) {
		if (event != null)
			event.setID("" + Constants.GESTURE_PREPARATION_CODE);
		manager.getGestureChainedEventsCollection().setPreparationEvent(event);
	}

	public void setRetractionEvent(GestureChainedEvent event) {
		if (event != null)
			event.setID("" + Constants.GESTURE_RETRACTION_CODE);
		manager.getGestureChainedEventsCollection().setRetractionEvent(event);
	}
	
	public void setRelaxedEvent(GestureChainedEvent event) {
		if (event != null)
			event.setID("" + Constants.GESTURE_RELAXED_CODE);
		manager.getGestureChainedEventsCollection().setRelaxedEvent(event);
	}

	public String addPreparationEvent() {
		if (previousEntity != null) {
			Time endMovieTimeInfo = previousEntity.getEndTimeInfo();

			GestureChainedEvent event = (GestureChainedEvent) manager.getFirstEvent();
			Time selectedEventStartTimeInfo = segmentPanel.getMultipleMovieController()
					.getPrecedingFrameTimeInfo(event.getStartTimeInfo());

			if (selectedEventStartTimeInfo.lessThan(endMovieTimeInfo)) {
				return ErrorMessages.GESTURE_CHAINS_CANNOT_OVERLAP;
			}
		}

		manager.addPreparationEvent();
		return null;
	}

	public String addPrestrokeHoldEvent() {
		GestureChainedEvent preparation = getPreparation();
		if (preparation != null) {
			Time preparationStartTimeInfo = preparation.getStartTimeInfo();
			Time preparationEndTimeInfo = preparation.getEndTimeInfo();
			Time precedingFrameEndTimeInfo = segmentPanel.getMultipleMovieController()
					.getPrecedingFrameTimeInfo(preparationEndTimeInfo);

			if (precedingFrameEndTimeInfo.equals(preparationStartTimeInfo)) {
				return ErrorMessages.ADD_PRESTROKE_HOLD_CANT_ADJUST_PREPARATION;
			} else {
				manager.addPrestrokeHoldEvent();
				return null;
			}
		}

		if (previousEntity != null) {
			Time endMovieTimeInfo = previousEntity.getEndTimeInfo();

			GestureChainedEvent event = (GestureChainedEvent) manager.getFirstEvent();
			Time selectedEventStartTimeInfo = segmentPanel.getMultipleMovieController()
					.getPrecedingFrameTimeInfo(event.getStartTimeInfo());

			if (selectedEventStartTimeInfo.lessThan(endMovieTimeInfo)) {
				return ErrorMessages.GESTURE_CHAINS_CANNOT_OVERLAP;
			}
		}

		manager.addPrestrokeHoldEvent();
		return null;
	}

	public String addPoststrokeHoldEvent() {
		// GREG TODO: should the order of the 2 below if-stmts be reversed, like in above addPrestrokeHoldEvent ?
		if (nextEntity != null) {
			Time startMovieTimeInfo = nextEntity.getStartTimeInfo();

			GestureChainedEvent event = (GestureChainedEvent) manager.getLastEvent();
			Time selectedEventEndTimeInfo = segmentPanel.getMultipleMovieController()
					.getFollowingFrameTimeInfo(event.getEndTimeInfo());

			if (selectedEventEndTimeInfo.greaterThan(startMovieTimeInfo)) {
				return ErrorMessages.GESTURE_CHAINS_CANNOT_OVERLAP;
			}
		}

		GestureChainedEvent retraction = getRetraction();
		if (retraction != null) {
			Time retractionStartTimeInfo = retraction.getStartTimeInfo();
			Time retractionEndTimeInfo = retraction.getEndTimeInfo();
			Time precedingFrameEndTimeInfo = segmentPanel.getMultipleMovieController()
				.getPrecedingFrameTimeInfo(retractionEndTimeInfo);

			if (precedingFrameEndTimeInfo.equals(retractionStartTimeInfo)) {
				return ErrorMessages.ADD_POSTSTROKE_HOLD_CANT_ADJUST_RETRACTION;
			}
		}

		manager.addPoststrokeHoldEvent();
		return null;
	}

	public String addRetractionEvent() {
		if (nextEntity != null) {
			Time startMovieTimeInfo = nextEntity.getStartTimeInfo();

			GestureChainedEvent event = (GestureChainedEvent) manager.getFirstEvent();
			Time selectedEventEndTimeInfo = segmentPanel.getMultipleMovieController()
					.getFollowingFrameTimeInfo(event.getEndTimeInfo());

			if (selectedEventEndTimeInfo.greaterThan(startMovieTimeInfo)) {
				return ErrorMessages.GESTURE_CHAINS_CANNOT_OVERLAP;
			}
		}

		manager.addRetractionEvent();
		return null;
	}
	
	public String addRelaxedEvent() {
		if (nextEntity != null) {
			Time startMovieTimeInfo = nextEntity.getStartTimeInfo();

			GestureChainedEvent event = (GestureChainedEvent) manager.getFirstEvent();
			Time selectedEventEndTimeInfo
				= segmentPanel.getMultipleMovieController().getFollowingFrameTimeInfo(event.getEndTimeInfo());

			if (selectedEventEndTimeInfo.greaterThan(startMovieTimeInfo)) {
				return ErrorMessages.GESTURE_CHAINS_CANNOT_OVERLAP;
			}
		}

		manager.addRelaxedEvent();
		return null;
	}
	
	public void setIncompleteGesture(boolean incompleteGesture) {
		this.incompleteGesture = incompleteGesture;
	}
	
	public boolean getIncompleteGesture() {
		return this.incompleteGesture;
	}

	public boolean getShowUniqueNumber() {
		return showUniqueNumber;
	}

	public void setShowUniqueNumber(boolean showUniqueNumber) {
		this.showUniqueNumber = showUniqueNumber;
	}

	public String getUniqueNumber() {
		return uniqueNumber;
	}

	public void setUniqueNumber(String uniqueNumber) {
		this.uniqueNumber = uniqueNumber;
	}
}
