package edu.bu.signstream.grepresentation.fields.gestureField;

import java.awt.Font;
import java.awt.Graphics2D;
import java.util.ArrayList;
import java.util.HashMap;

import edu.bu.signstream.common.util.Sorter;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class GestureChainedEventsEntityCollection {
	private ArrayList<GestureChainedEventsEntity> entities = new ArrayList<GestureChainedEventsEntity>();
	private SignStreamSegmentPanel segmentPanel;

	private GestureEnteredText enteredText;

	public GestureChainedEventsEntityCollection(
			ArrayList<GestureChainedEventsEntity> entities,
			SignStreamSegmentPanel segmentPanel) {
		this.entities = entities;
		this.segmentPanel = segmentPanel;

        sortEntities();
	}

	public GestureEnteredText getEnteredText() {
		return enteredText;
	}

	public void setEnteredText(GestureEnteredText enteredText) {
		this.enteredText = enteredText;
	}

	public GestureChainedEventsEntity getNextEntity(int coord) {
		GestureChainedEventsEntity nextEntity = null;
		for (GestureChainedEventsEntity entity : entities) {
			if(entity.getStartTimeCoordinate() > coord) {
				if(nextEntity == null) {
					nextEntity = entity;
				} else if(entity.getStartTimeCoordinate() < nextEntity.getStartTimeCoordinate()) {
					nextEntity = entity;
				}
			}
		}
		return nextEntity;
	}
	
	public GestureChainedEventsEntity getPreviousEntity(int coord) {
		GestureChainedEventsEntity prevEntity = null;
		for (GestureChainedEventsEntity entity : entities) {
			if(entity.getStartTimeCoordinate() < coord) {
				if(prevEntity == null) {
					prevEntity = entity;
				} else if(entity.getStartTimeCoordinate() > prevEntity.getStartTimeCoordinate()) {
					prevEntity = entity;
				}
			}
		}
		return prevEntity;
	}

	/**
	 * This method expects startCoord and endCoord to be coordinates, not times.
	 * @param startCoord Start coordinate to search within
	 * @param endCoord End coordinate to search within
	 * @return List of GlossChainedEventsEntity objects within the interval provided
	 */
	public ArrayList<GestureChainedEventsEntity> getEntitiesInInterval(double startCoord, double endCoord) {
		ArrayList<GestureChainedEventsEntity> list = new ArrayList<GestureChainedEventsEntity>();
		
		for (GestureChainedEventsEntity entity : entities) {
			if ((entity.isEntityInsideElement(startCoord, endCoord))
					|| (entity.contains(startCoord) || entity.contains(endCoord))) {
				list.add(entity);
			}
		}

		return list;
	}

	public void removeEmptyEntities() {
		ArrayList<GestureChainedEventsEntity> entitiesToDelete = new ArrayList<GestureChainedEventsEntity>();

		for (GestureChainedEventsEntity entity : entities) {
			if (entity.getStartTimeInfo() == null || entity.getEndTimeInfo() == null) {
				entitiesToDelete.add(entity);
			}
		}

		for (GestureChainedEventsEntity entityToDelete : entitiesToDelete) {
			deleteEntity(entityToDelete);
		}
	}
	
	/**
	 * Cleanup method: if there are hanging references to a deleted field value, which is residing in this 
	 * GestureChainedEventsEntityCollection with an invalid field value ID, then it has since been deleted from the
	 * coding schema, but not yet form this collection.
	 * @param fieldID The Field ID of the field to get from the coding scheme
	 * @return true if succesful, false otherwise
	 */
	boolean deleteInvalidEntities(String fieldID) {
		ArrayList<GestureChainedEventsEntity> entitiesToDelete = new ArrayList<GestureChainedEventsEntity>();
		
		SS3CodingScheme dbScheme = segmentPanel.getSS3Database().getSS3CodingScheme();
		SS3Field ss3Field = dbScheme.getField(fieldID);
		ArrayList<SS3FieldValue> ss3FieldValues = ss3Field.getValuesAsList();
		ArrayList<String> ss3FieldValueIDs = new ArrayList<String>();
		
		for (SS3FieldValue ss3FieldValue: ss3FieldValues) {
			String valueID = ss3FieldValue.getId();
			if (valueID == null || valueID.isEmpty())
				continue;
			
			ss3FieldValueIDs.add(valueID);
		}
		
		if (ss3FieldValueIDs.isEmpty()) {
			return false;
		}
		
		for (GestureChainedEventsEntity entity: entities) {
			String valueID = entity.getValueOrTextValueEvent().getID();
			if ( !ss3FieldValueIDs.contains(valueID)) {
				entitiesToDelete.add(entity);
			}
		}

		for (GestureChainedEventsEntity entityToDelete : entitiesToDelete) {
			deleteEntity(entityToDelete);
		}		
		
		return true;
	}

	public GestureChainedEventsEntity getEntityWithRefID(String refEntityId) {
		for (GestureChainedEventsEntity entity : entities) {
			if (refEntityId != null
					&& refEntityId.equalsIgnoreCase(entity.getRefEntityId()))
				return entity;
		}
		return null;
	}

	public GestureChainedEventsEntity getEntity(String entityID) {
		for (GestureChainedEventsEntity entity : entities) {
			if (entity.getID().equalsIgnoreCase(entityID))
				return entity;
		}
		return null;
	}

	public ArrayList<GestureChainedEventsEntity> getEntities() {
		return entities;
	}

	public int size() {
		return entities.size();
	}

	public void deleteEntity(GestureChainedEventsEntity entity) {
		GestureChainedEventsEntity previousEntity = entity.getPreviousEntity();
		GestureChainedEventsEntity nextEntity = entity.getNextEntity();

		if (previousEntity != null) {
			previousEntity.setNextEntity(nextEntity);
		}

		if (nextEntity != null) {
			nextEntity.setPreviousEntity(previousEntity);
		}

		boolean removed = entities.remove(entity);

        if(!removed) {
            Time startMovieTimeInfo = entity.getStartTimeInfo();
            Time endMovieTimeInfo = entity.getEndTimeInfo();

            GestureChainedEventsEntity entityToRemove = null;

            for(GestureChainedEventsEntity entity1 : entities) {
                Time startMovieTimeInfo1 = entity1.getStartTimeInfo();
                Time endMovieTimeInfo1 = entity1.getEndTimeInfo();

                if(startMovieTimeInfo.equals(startMovieTimeInfo1) && endMovieTimeInfo.equals(endMovieTimeInfo1)) {
                    entityToRemove = entity1;
                }
            }

            entities.remove(entityToRemove);
        }
	}

	public Time getStartTimeInfo() {
		removeEmptyEntities();
		
		if (entities.isEmpty()) 
			return SS3Singleton.getVideoTimeRange().zero();
		
		Time earliestStartTime = entities.get(0).getStartTimeInfo();
				
		for (GestureChainedEventsEntity entity : entities) {
			earliestStartTime = (earliestStartTime.doubleValue() < entity.getStartTimeInfo().doubleValue()
					? earliestStartTime
					: entity.getStartTimeInfo());
		}
		
		return earliestStartTime;
	}

	public Time getEndTimeInfo() {
		removeEmptyEntities();
		
		if (entities.isEmpty())
			return SS3Singleton.getVideoTimeRange().zero();
		
		Time latestEndTime = entities.get(0).getEndTimeInfo();
		
		for (GestureChainedEventsEntity entity : entities) {
			latestEndTime = (latestEndTime.doubleValue() < entity.getStartTimeInfo().doubleValue()
						? latestEndTime
						: entity.getEndTimeInfo());
		}
		
		return latestEndTime;
	}

	public double getStartTimeCoordinate() {
		Time time = getStartTimeInfo();

		return SS3Singleton.getVideoControlManager().getZoomer()
				.getCoordinateTimeConvertor().convertTimeInfoToCoordinate(time);
	}

	public double getEndTimeCoordinate() {
		Time time = getEndTimeInfo();

		return SS3Singleton.getVideoControlManager().getZoomer()
				.getCoordinateTimeConvertor().convertTimeInfoToCoordinate(time);
	}

	public void unselect() {
		for (GestureChainedEventsEntity entity : entities) {
			entity.unselect();
		}
	}

	public void reset() {
		for (GestureChainedEventsEntity entity : entities) {
			entity.reset();
		}
	}

	public void addEntity(GestureChainedEventsEntity entity) {

		Time startMovieTimeInfo = entity.getStartTimeInfo();
		
		GestureChainedEventsEntity previousEntity = null;
		GestureChainedEventsEntity nextEntity = null;

		for (GestureChainedEventsEntity ent : entities) {

            Time entMovieStartTimeInfo = ent.getStartTimeInfo();
			
//			// exit function in case that this event is already in the list
//			if (entMovieStartTimeInfo.equals(startMovieTimeInfo))
//				return;
            // GD commenting during merge - I could be wrong though
            
            	// GD: I believe this function is fine as is, since gestures cannot be one-frame, negating the need for all the wacky logic in GlossChainedEventsEntityCollection

			if (entMovieStartTimeInfo.lessThanOrEq(startMovieTimeInfo)) {
				if (previousEntity == null) {
					previousEntity = ent;
				} else if (entMovieStartTimeInfo.greaterThan(previousEntity.getStartTimeInfo())) {
					previousEntity = ent;
				}
			} else {
				if (nextEntity == null) {
					nextEntity = ent;
				} else if (entMovieStartTimeInfo.lessThan(nextEntity.getStartTimeInfo())) {
					nextEntity = ent;
				}
			}
		}
		
		if (previousEntity != null) {
			GestureChainedEventsEntity next = previousEntity.getNextEntity();
			if (next != null) {
				entity.setNextEntity(next);
				next.setPreviousEntity(entity);
			}
			previousEntity.setNextEntity(entity);
			entity.setPreviousEntity(previousEntity);

		} else if (nextEntity != null) {
			entity.setNextEntity(nextEntity);
			nextEntity.setPreviousEntity(entity);
		}

		this.entities.add(entity);
	}

	public void selectEntity(int x) {
		EventsEntity entity = getGestureChainedEventsEntityAt(x);
		if (entity != null)
			entity.select();
	}

	public GestureChainedEventsEntity getGestureChainedEventsEntityAt(int coordX) {
		for (GestureChainedEventsEntity entity : entities) {
			if (entity.contains(coordX))
				return entity;
		}
		return null;
	}

	public Event getSelectedEvent() {
		for (GestureChainedEventsEntity entity : entities) {
			Event event = entity.getSelectedEvent();
			if(event != null) { 
				return event;
			}
		}
		return null;
	}

	public GestureChainedEventsEntity getSelectedEntity() {
		for (GestureChainedEventsEntity entity : entities) {
			Event event = entity.getSelectedEvent();
			if(event != null) {
				return entity;
			}
		}
		return null;
	}

	public GestureChainedEvent getEventAt(int x) {
		GestureChainedEventsEntity entity = getGestureChainedEventsEntityAt(x);
		if (entity == null)
			return null;

		return entity.getGestureChainedEventsCollection().getEventAt(x);
	}

	public void setSelectedEvementValueID(String valueID) {
		Event event = getSelectedEvent();

		if (event instanceof GestureChainedEvent) {
			GestureChainedEvent chainedEvent = (GestureChainedEvent) event;
			chainedEvent.setID(valueID);
		}
	}

//	public NewGestureChainedEvent getNewGestureChainedEvent() {
//		return newEvent;
//	}
//
//	public void setNewGestureChainedEvent(NewGestureChainedEvent newEvent) {
//		this.newEvent = newEvent;
//	}

//	public GestureChainedEventsCollection createNewGestureChainedEventCollection(
//			String valueID) {
//		if (valueID != null && newEvent != null) {
//			TrackValue trackValue = new TrackValue();
//			trackValue.setValueId(valueID);
//
//			trackValue.setStartTimeInfo(newEvent.getStartTimeInfo());
//			trackValue.setEndTimeInfo(newEvent.getEndTimeInfo());
//
//			GestureChainedEvent event = new GestureChainedEvent(trackValue, segmentPanel);
//
//			GestureChainedEventsCollection collection = new GestureChainedEventsCollection();
//			collection.setValueEvent(event);
//			newEvent = null;
//
//			return collection;
//		}
//		return null;
//	}

    public void sortEntities() {
        int entitiesSize = entities.size();

        if(entitiesSize == 1)
            return;

        double[] startTimes = new double[entitiesSize];
        HashMap<String, GestureChainedEventsEntity> map = new HashMap<String, GestureChainedEventsEntity>();

        int i = 0;
        for(GestureChainedEventsEntity entity : entities) {
            double movieTime = entity.getStartTimeInfo().doubleValue();
            map.put(""+movieTime, entity);
            startTimes[i] = movieTime;
            i++;
        }

        Sorter.sort(startTimes);

        i = 0;
        while(i < entitiesSize-1) {
            GestureChainedEventsEntity gestEntity = map.get(""+startTimes[i]);
            i++;
            GestureChainedEventsEntity nextGestEntity = map.get(""+startTimes[i]);
            gestEntity.setNextEntity(nextGestEntity);
            nextGestEntity.setPreviousEntity(gestEntity);
        }
    }

	public void sort(ArrayList<GestureChainedEventsEntity> entitiesToSort) {
		removeEmptyEntities();

		HashMap<String, GestureChainedEventsEntity> map = new HashMap<String, GestureChainedEventsEntity>();

		double[] startTimes = new double[entitiesToSort.size()];

		ArrayList<GestureChainedEventsEntity> toRemove = new ArrayList<GestureChainedEventsEntity>();

		for (int i = 0; i < entitiesToSort.size(); i++) {
			GestureChainedEventsEntity entity = entitiesToSort.get(i);
			if (entity != null && entity.getStartTimeInfo() != null) {
				double startTime = entity.getStartTimeInfo().doubleValue();
				map.put("" + startTime, entity);
				startTimes[i] = startTime;
			} else {
				toRemove.add(entity);
			}
		}

		for (GestureChainedEventsEntity cEnt : toRemove) {
			entitiesToSort.remove(cEnt);
		}

		Sorter.sort(startTimes);
		entitiesToSort.clear();

		for (int j = 0; j < startTimes.length; j++) {
			GestureChainedEventsEntity obj = map.get("" + startTimes[j]);
			entitiesToSort.add(obj);
		}
	}

	public void paint(Graphics2D g2d, double Y, Font font) {
		for (GestureChainedEventsEntity entity : entities) {
			entity.paint(g2d, Y, font);
		}

//		if (newEvent != null) {
//			Painter painter = segmentPanel.getPainter();
//			painter.drawNewEvent(newEvent, g2d, Y);
//		}
	}
}
