package edu.bu.signstream.grepresentation.fields.gestureField;

import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.HeadlessException;
import java.util.ArrayList;

import javax.swing.JOptionPane;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.time.Duration;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.apexField.ApexChainedApexEventsEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.handshapes.GestureHandShapeEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.locationField.GestureLocationEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.signTypeField.GestureTypeEntity;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.fields.gestureField.orientationField.GestureOrientationEntity;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolbarCommons;

// NOTE: GestureEventsEntity akin to GlossChainedEventsEntity
// NOTE: GestureChainedEventsEntity akin to ChainedEventsEntity
public class GestureEventsEntity extends GestureChainedEventsEntity implements GestureCompoundable {

	private static final long serialVersionUID = 1L;
	
	private boolean isLockedDomNdomAlignments = false;
	
	private GestureChainedEventsEntity dependentHandEntity = null;
	private GestureHandShapeEntity dependentHandShapeEntity = null;
	private GestureOrientationEntity dependentOrientationEntity = null;
	private GestureEventsEntity dependentGestureEntity = null;
	private ApexChainedApexEventsEntity dependentApexEntity = null;
	
	private GestureLocationEntity dependentLocationEntity = null;
	private GestureTypeEntity dependentSignTypeEntity = null;

    private String dependentGestureEntityID;
    private String dependentHandEntityID;
    private String dependentHandShapeEntityID;
    private String dependentOrientationEntityID;
    private String dependentApexEntityID;
    private String dependentLocationEntityID;
    private String dependentSignTypeEntityID;

    private GestureEventsEntity previousEntity = null;
    private GestureEventsEntity nextEntity = null;

    private GestureChainedEventsCollection chainedEventsCollection = null;

    protected GestureChainedEventsCollectionManager manager;

    private String correspondingGestureEntityID = "";

    private GestureCompoundable startCompound = null;
    private GestureCompoundable endCompound = null;

    private GestureEntityProperties glossProperties = new GestureEntityProperties();
    
    public static final String DEFAULT_TEXT = "STROKE";

    public String getDependentGestureEntityID() {
		return dependentGestureEntityID;
	}

	public void setDependentGestureEntityID(String dependentGestureEntityID) {
		this.dependentGestureEntityID = dependentGestureEntityID;
	}

	public String getDependentHandEntityID() {
		return dependentHandEntityID;
	}

	public void setDependentHandEntityID(String dependentHandEntityID) {
		this.dependentHandEntityID = dependentHandEntityID;
	}

	public String getDependentHandShapeEntityID() {
		return dependentHandShapeEntityID;
	}

	public void setDependentHandShapeEntityID(String dependentHandShapeEntityID) {
		this.dependentHandShapeEntityID = dependentHandShapeEntityID;
	}
	
	public String getDependentOrientationEntityID() {
		return dependentOrientationEntityID;
	}

	public void setDependentOrientationEntityID(String dependentOrientationEntityID) {
		this.dependentOrientationEntityID = dependentOrientationEntityID;
	}

	public String getDependentLocationEntityID() {
		return dependentLocationEntityID;
	}

	public void setDependentLocationEntityID(String dependentLocationEntityID) {
		this.dependentLocationEntityID = dependentLocationEntityID;
	}

	public String getDependentSignTypeEntityID() {
		return dependentSignTypeEntityID;
	}

	public void setDependentSignTypeEntityID(String dependentSignTypeEntityID) {
		this.dependentSignTypeEntityID = dependentSignTypeEntityID;
	}
	
	public String getDependentApexEntityID() {
		return dependentApexEntityID;
	}

	public void setDependentApexEntityID(String dependentApexEntityID) {
		this.dependentApexEntityID = dependentApexEntityID;
	}

	public GestureEventsEntity(GestureChainedEventsCollection collection, SignStreamSegmentPanel segmentPanel, Field field) {
		super(collection, segmentPanel, field);
		setID(Util.getLongUniqueNumber());
		this.manager = new GestureChainedEventsCollectionManager(collection,
				segmentPanel);

		this.chainedEventsCollection = collection;

//        this.getSS3Entity();
	}
	
	public boolean isLockedDomNdomAlignments() {
		return isLockedDomNdomAlignments;
	}
	
	public void setLockedDomNdomAlignments(boolean isLockedDomNdomAlignments) {
		this.isLockedDomNdomAlignments = isLockedDomNdomAlignments;
	}
	
	@Override
	public String canSetSelectedEventStartMovieTime(Time timeInfo) {
		Event selectedEvent =  SS3Singleton.getMediaToolBar(null).selectedEvent();
		
        Time originalSelectedStartTimeInfo = selectedEvent.getStartTimeInfo();
        Time originalSelectedEndTimeInfo = selectedEvent.getEndTimeInfo();

        Time maxStartMovieTimeInfo = originalSelectedEndTimeInfo.subtract(new Duration(1, originalSelectedEndTimeInfo.unit()));

		GestureChainedEvent prestrokeHold = manager.getGestureChainedEventsCollection().getPrestrokeHoldEvent();
		GestureChainedEvent value = manager.getGestureChainedEventsCollection().getValueOrTextValueEvent();
		GestureChainedEvent poststrokeHold = manager.getGestureChainedEventsCollection().getPoststrokeHoldEvent();
		GestureChainedEvent retraction = manager.getGestureChainedEventsCollection().getRetractionEvent();
		GestureChainedEvent relaxed = manager.getGestureChainedEventsCollection().getRelaxedEvent();

		// check for apex timing conflicts
		GestureEventsEntity gestEntity = (GestureEventsEntity) this;
		ApexChainedApexEventsEntity apexEntity = gestEntity.getDependentApexEntity();

		GestureEventsEntity depGestEntity = gestEntity.getDependentGestureEntity();
		boolean gestLockedDomNdomAlignments = gestEntity.isLockedDomNdomAlignments();

		if (selectedEvent.equals(value)) {
			if (apexEntity != null) {
				if (timeInfo.greaterThan(apexEntity.getStartTimeInfo())) {
					return ErrorMessages.GESTURE_APEX_BOUNDS;
				}
			}

			if (depGestEntity != null) {
				ApexChainedApexEventsEntity depApexEntity = depGestEntity.getDependentApexEntity();

				if (gestLockedDomNdomAlignments && depApexEntity != null) {
					if (timeInfo.greaterThan(depApexEntity.getStartTimeInfo())) {
						return ErrorMessages.GESTURE_APEX_BOUNDS;
					}
				}
			}
		} else if (selectedEvent.equals(retraction) || selectedEvent.equals(poststrokeHold) || selectedEvent.equals(relaxed) ) {
			if (apexEntity != null) {
				if (timeInfo.lessThan(apexEntity.getEndTimeInfo())) {
					return ErrorMessages.GESTURE_APEX_BOUNDS;
				}
			}

			if (depGestEntity != null) {
				ApexChainedApexEventsEntity depApexEntity = depGestEntity.getDependentApexEntity();

				if (gestLockedDomNdomAlignments && depApexEntity != null) {
					if (timeInfo.lessThan(depApexEntity.getEndTimeInfo())) {
						return ErrorMessages.GESTURE_APEX_BOUNDS;
					}
				}
			}
		}
		
		// Gesture value events are allowed to have the same start/end time
		if (selectedEvent.equals(value)) {
            maxStartMovieTimeInfo = maxStartMovieTimeInfo.add(new Duration(1, originalSelectedEndTimeInfo.unit()));
		}

		if (maxStartMovieTimeInfo.lessThan(timeInfo)) {
			return ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME;
		}

		if (selectedEvent.equals(value)) {
			// check for one-frame logic of Gesture events
			boolean currentlyOneFrame = originalSelectedStartTimeInfo.equals(originalSelectedEndTimeInfo);
			boolean becomingOneFrame = timeInfo.equals(originalSelectedEndTimeInfo);
			if (currentlyOneFrame || becomingOneFrame) {
				String retOneFrame = checkOneFrameLogic(timeInfo, currentlyOneFrame, becomingOneFrame);
				if (retOneFrame != null) {
					return retOneFrame;
				}
			}
			
			if (!becomingOneFrame) {
				// asking the dep hand entities on both hands if canMove
	            boolean gestLockedHand = gestEntity.isLockedGesturePhonAlignments();
				
				GestureChainedEventsEntity handEntity = this.getDependentHandEntity();
				GestureChainedEventsEntity depHandEntity = null;
				boolean depGestLockedHand = false;
				
				if (handEntity != null && gestLockedHand) {
					String resp = handEntity.canSetSelectedEventStartMovieTime(timeInfo);

					if (resp != null) {
						return ErrorMessages.GESTURE_CHAINS_CANNOT_OVERLAP;
					}
				}
				
				if (depGestEntity != null && gestLockedDomNdomAlignments) {
					GestureChainedEventsEntity depGestPrevEntity = depGestEntity.getPreviousEntity();
					if (depGestPrevEntity != null) {
						if (depGestPrevEntity.getEndTimeInfo().greaterThan(timeInfo)) {
							return ErrorMessages.GESTURE_CHAINS_CANNOT_OVERLAP;
						}
					}
					
					// TODO: for glosses make sure the most external if condntion here also includes final/initial holds
					depHandEntity = depGestEntity.getDependentHandEntity();
					depGestLockedHand = depGestEntity.isLockedGesturePhonAlignments();
					if (depHandEntity != null && depGestLockedHand) {
						String resp = depHandEntity.canSetSelectedEventStartMovieTime(timeInfo);

						if (resp != null) {
							return ErrorMessages.GESTURE_CHAINS_CANNOT_OVERLAP;
						}
					}
				}
			}
		}

		if (previousEntity != null) {
			if (previousEntity.getEndTimeInfo().greaterThan(timeInfo)) {
				return ErrorMessages.GESTURE_CHAINS_CANNOT_OVERLAP;
			}
		}
		
		Event prevEvent = manager.getGestureChainedEventsCollection().getPreviousEvent(selectedEvent);
		
		if (prevEvent == null) {
			// |<--
			if (selectedEvent.getStartTimeInfo().greaterThan(timeInfo)) {
				if (previousEntity == null
						|| previousEntity.getEndTimeInfo().lessThanOrEq(timeInfo)) {
//		            selectedEvent.setStartTimeInfo(timeInfo);
		            return null;
//                    // selectedEvent.getStartTimeInfo().setMovieTime(movieTime);
				} else {
					return ErrorMessages.GESTURE_CHAINS_CANNOT_OVERLAP;
				}
			} else {
//	            selectedEvent.setStartTimeInfo(timeInfo);
				return null;
//                // selectedEvent.getStartTimeInfo().setMovieTime(movieTime);
			}			
		} else if (selectedEvent.getStartTimeInfo().greaterThan(timeInfo)) {
			Time prevStartTimeInfo = prevEvent.getStartTimeInfo();
			Time timeInfoLessOneDuration = timeInfo.subtract(new Duration(1, timeInfo.unit()));
			if (timeInfoLessOneDuration.lessThan(prevStartTimeInfo)) {
				
				if(selectedEvent.equals(prestrokeHold)) {
					return ErrorMessages.SET_START_PRESTROKE_HOLD_CANT_DELETE_PREPARATION;
				} else if(selectedEvent.equals(value)) {
					if(prestrokeHold != null && prevEvent.equals(prestrokeHold)) {
						return ErrorMessages.SET_START_VALUE__CANT_DELETE_PRESTROKE_HOLD;
					} else  {
						return ErrorMessages.SET_START_VALUE_CANT_DELETE_PREPARATION;
					}
				} else if(selectedEvent.equals(poststrokeHold)) {
					return ErrorMessages.SET_START_POSTSTROKE_HOLD_CANT_DELETE_VALUE;
				} else if(selectedEvent.equals(retraction)) {
					if(poststrokeHold != null && prevEvent.equals(poststrokeHold)) {
						return ErrorMessages.SET_START_RETRACTION_CANT_DELETE_POSTSTROKE_HOLD; 
					} else {
						return ErrorMessages.SET_START_RETRACTION_CANT_DELETE_VALUE;
					}
				} else if (selectedEvent.equals(relaxed)) {
					if (retraction != null && prevEvent.equals(retraction)) {
						return ErrorMessages.SET_START_RELAXED_CANT_DELETE_RETRACTION;
					} else if (poststrokeHold != null && prevEvent.equals(poststrokeHold)) {
						return ErrorMessages.SET_START_RELAXED_CANT_DELETE_POSTSTROKE_HOLD;
					} else {
						return ErrorMessages.SET_START_RELAXED_CANT_DELETE_VALUE;
					}
				}
			} else {
//	            selectedEvent.setStartTimeInfo(timeInfo);
//
//				prevEvent.setEndTimeInfo(timeInfo);
				return null;
			}
		} else {
//            selectedEvent.setStartTimeInfo(timeInfo);
//
//            prevEvent.setEndTimeInfo(timeInfo);
			return null;
		}

		SS3Singleton.getMediaToolBar().update();
		return null;
	}
	
	@Override
	public String canSetSelectedEventEndMovieTime(Time timeInfo) {
		Event selectedEvent = SS3Singleton.getMediaToolBar(null).selectedEvent();
		
        Time originalSelectedStartTimeInfo = selectedEvent.getStartTimeInfo();
        Time originalSelectedEndTimeInfo = selectedEvent.getEndTimeInfo();

        Time minEndMovieTimeInfo = originalSelectedStartTimeInfo.add(new Duration(1, originalSelectedStartTimeInfo.unit()));

		GestureChainedEvent preparation = manager.getGestureChainedEventsCollection().getPreparationEvent();
		GestureChainedEvent prestrokeHold = manager.getGestureChainedEventsCollection().getPrestrokeHoldEvent();
		GestureChainedEvent value = manager.getGestureChainedEventsCollection().getValueOrTextValueEvent();
		GestureChainedEvent poststrokeHold = manager.getGestureChainedEventsCollection().getPoststrokeHoldEvent();
		GestureChainedEvent retraction = manager.getGestureChainedEventsCollection().getRetractionEvent();
		GestureChainedEvent relaxed = manager.getGestureChainedEventsCollection().getRelaxedEvent();

		// check for apex timing conflicts
		GestureEventsEntity gestEntity = (GestureEventsEntity) this;
		ApexChainedApexEventsEntity apexEntity = gestEntity.getDependentApexEntity();

		GestureEventsEntity depGestEntity = gestEntity.getDependentGestureEntity();
		boolean gestLockedDomNdomAlignments = gestEntity.isLockedDomNdomAlignments();

		if (selectedEvent.equals(value)) {
			if (apexEntity != null) {
				if (timeInfo.lessThan(apexEntity.getEndTimeInfo())) {
					return ErrorMessages.GESTURE_APEX_BOUNDS;
				}
			}

			if (depGestEntity != null) {
				ApexChainedApexEventsEntity depApexEntity = depGestEntity.getDependentApexEntity();

				if (gestLockedDomNdomAlignments && depApexEntity != null) {
					if (timeInfo.lessThan(depApexEntity.getEndTimeInfo())) {
						return ErrorMessages.GESTURE_APEX_BOUNDS;
					}
				}
			}
		} else if (selectedEvent.equals(preparation) || selectedEvent.equals(prestrokeHold) ) {
			if (apexEntity != null) {
				if (timeInfo.greaterThan(apexEntity.getStartTimeInfo())) {
					return ErrorMessages.GESTURE_APEX_BOUNDS;
				}
			}

			if (depGestEntity != null) {
				ApexChainedApexEventsEntity depApexEntity = depGestEntity.getDependentApexEntity();

				if (gestLockedDomNdomAlignments && depApexEntity != null) {
					if (timeInfo.greaterThan(depApexEntity.getStartTimeInfo())) {
						return ErrorMessages.GESTURE_APEX_BOUNDS;
					}
				}
			}
		}
		
		// Gesture value events are allowed to have the same start/end time
		if (selectedEvent.equals(value)) {
			minEndMovieTimeInfo = minEndMovieTimeInfo.subtract(new Duration(1, originalSelectedStartTimeInfo.unit()));
		}

		if (minEndMovieTimeInfo.greaterThan(timeInfo)) {
			return ErrorMessages.EVENT_END_TIME_FOLLOW_START_TIME;
		}
		
		if (selectedEvent.equals(value)) {
			// check for one-frame logic of Gesture events
			boolean currentlyOneFrame = originalSelectedStartTimeInfo.equals(originalSelectedEndTimeInfo);
			boolean becomingOneFrame = timeInfo.equals(originalSelectedStartTimeInfo);
			if (currentlyOneFrame || becomingOneFrame) {
				String retOneFrame = checkOneFrameLogic(timeInfo, currentlyOneFrame, becomingOneFrame);
				if (retOneFrame != null) {
					return retOneFrame;
				}
			}
			
			if (!becomingOneFrame) {
				// asking the dep hand entities on both hands if canMove
	            boolean gestLockedHand = gestEntity.isLockedGesturePhonAlignments();
				
				GestureChainedEventsEntity handEntity = this.getDependentHandEntity();
				GestureChainedEventsEntity depHandEntity = null;
				boolean depGestLockedHand = false;
				
				if (handEntity != null && gestLockedHand) {
					String resp = handEntity.canSetSelectedEventEndMovieTime(timeInfo);

					if (resp != null) {
						return ErrorMessages.GESTURE_CHAINS_CANNOT_OVERLAP;
					}
				}
				
				if (depGestEntity != null && gestLockedDomNdomAlignments) {
					GestureChainedEventsEntity depGestNextEntity = depGestEntity.getNextEntity();
					if (depGestNextEntity != null) {
						if (depGestNextEntity.getStartTimeInfo().lessThan(timeInfo)) {
							return ErrorMessages.CHAINS_CANNOT_OVERLAP;
						}
						
						// TODO: for glosses make sure the most external if condntion here also includes final/initial holds
						depHandEntity = depGestEntity.getDependentHandEntity();
						depGestLockedHand = depGestEntity.isLockedGesturePhonAlignments();
						if (depHandEntity != null && depGestLockedHand) {
							String resp = depHandEntity.canSetSelectedEventEndMovieTime(timeInfo);

							if (resp != null) {
								return ErrorMessages.GESTURE_CHAINS_CANNOT_OVERLAP;
							}
						}
					}
				}
			}
		}
		
		if (nextEntity != null) {
			if (nextEntity.getStartTimeInfo().lessThan(timeInfo)) {
				return ErrorMessages.CHAINS_CANNOT_OVERLAP;
			}
		}

		Event nextEvent = manager.getGestureChainedEventsCollection().getNextEvent(selectedEvent);

		if(nextEvent != null) {				
			Time maxNextEventStartTimeInfo = segmentPanel.getMultipleMovieController().getPrecedingFrameTimeInfo(
					nextEvent.getEndTimeInfo());
			if(maxNextEventStartTimeInfo.greaterThanOrEq(timeInfo)) {
//				selectedEvent.setEndTimeInfo(timeInfo);
//				
//				nextEvent.setStartTimeInfo(timeInfo);
				return null;
			} else {
				if(selectedEvent.equals(preparation)) {
					if(prestrokeHold!= null && nextEvent.equals(prestrokeHold)) {
						return ErrorMessages.SET_END_PREPARATION_CANT_DELETE_PRESTROKE_HOLD;
					} else {
						return ErrorMessages.SET_END_PREPARATION_CANT_DELETE_VALUE;
					}
				} else if(selectedEvent.equals(prestrokeHold)) {
					return ErrorMessages.SET_END_PRESTROKE_HOLD_CANT_DELETE_VALUE;
				} else if(selectedEvent.equals(value)) {
					if(poststrokeHold != null && nextEvent.equals(poststrokeHold)) {
						return ErrorMessages.SET_END_VALUE_CANT_DELETE_POSTSTROKE_HOLD;
					} else if (retraction != null && nextEvent.equals(retraction)) {
						return ErrorMessages.SET_END_VALUE_CANT_DELETE_RETRACTION;
					} else {
						return ErrorMessages.SET_END_VALUE_CANT_DELETE_RELAXED;
					}
				} else if (selectedEvent.equals(poststrokeHold)) {
					if (retraction != null && nextEvent.equals(retraction)) {
						return ErrorMessages.SET_END_POSTSTROKE_HOLD_CANT_DELETE_RETRACTION;
					} else if (relaxed != null && nextEvent.equals(relaxed)) {
						return ErrorMessages.SET_END_POSTSTROKE_HOLD_CANT_DELETE_RELAXED;
					}
				} else if (selectedEvent.equals(retraction)) {
					if (relaxed != null && nextEvent.equals(relaxed)) {
						return ErrorMessages.SET_END_RETRACTION_CANT_DELETE_RELAXED;
					}
				}
			}
		} else {
//			selectedEvent.setEndTimeInfo(timeInfo);
			return null;
		}

		SS3Singleton.getMediaToolBar().update();
		return null;
	}

	// GD: there's some conflicts in just straight overriding GestureChainedEventsEntity.setStartTimeInfo/setEndTimeInfo
	// mostly in the GestureFieldWrapper on dragging/creation of the entity
	public void setGestureEventsEntityStartTimeInfo(Time timeInfo) {
		Event selectedEvent = SS3Singleton.getMediaToolBar(null).selectedEvent();
		Event prevEvent = manager.getGestureChainedEventsCollection().getPreviousEvent(selectedEvent);
		
		if (selectedEvent == null) {
			super.setStartTimeInfo(timeInfo);
			return;
		}
		
		selectedEvent.setStartTimeInfo(timeInfo);
		if (prevEvent != null) {
			prevEvent.setEndTimeInfo(timeInfo);
		}
	}

	public void setGestureEventsEntityEndTimeInfo(Time timeInfo) {
		Event selectedEvent = SS3Singleton.getMediaToolBar(null).selectedEvent();
		Event nextEvent = manager.getGestureChainedEventsCollection().getNextEvent(selectedEvent);
		
		if (selectedEvent == null) {
			super.setEndTimeInfo(timeInfo);
			return;
		}
		
		selectedEvent.setEndTimeInfo(timeInfo);
		if (nextEvent != null) {
			nextEvent.setStartTimeInfo(timeInfo);
		}
	}
	
	private String checkOneFrameLogic(Time timeInfo, boolean currentlyOneFrame, boolean becomingOneFrame) {
		GestureEventsEntity  gestEntity = (GestureEventsEntity)this;

		if (becomingOneFrame) {
			if ((nextEntity != null && nextEntity.getStartTimeInfo().equals(timeInfo) && nextEntity.getEndTimeInfo().equals(timeInfo)) 
				|| (previousEntity != null && previousEntity.getStartTimeInfo().equals(timeInfo) && previousEntity.getEndTimeInfo().equals(timeInfo))) {
				return ErrorMessages.SET_ONE_FRAMES_CANNOT_OVERLAP;				
			}
		}
		
		GestureHandShapeEntity handShapeEntity = gestEntity.getDependentHandShapeEntity();
		GestureOrientationEntity orientationEntity = gestEntity.getDependentOrientationEntity();
		
		boolean isLockedDomNdom = gestEntity.isLockedDomNdomAlignments();
		boolean isGestLockedHand = gestEntity.isLockedGesturePhonAlignments();
		
		GestureEntityProperties properties = gestEntity.getGestureEntityProperties();
		
		GestureEventsEntity depGestEntity = gestEntity.getDependentGestureEntity();
		GestureHandShapeEntity depHandShapeEntity = null;
		GestureOrientationEntity depOrientationEntity = null;
		
		boolean isDepGestLockedHand = false;	
		GestureEntityProperties depProperties = null;
		
		if (depGestEntity != null) {
			depHandShapeEntity = depGestEntity.getDependentHandShapeEntity();
			depOrientationEntity = depGestEntity.getDependentOrientationEntity();
			
			isDepGestLockedHand = depGestEntity.isLockedGesturePhonAlignments();
			depProperties = depGestEntity.getGestureEntityProperties();
		}
		
		// current start and end times same? it means we are moving to a 1 or more frame difference, so set oneFrameGesture to false
		if (currentlyOneFrame) {
			properties.setOneFrameGesture(false);
			
			if (isGestLockedHand) {
				if (handShapeEntity != null)
					handShapeEntity.cloneEvents(timeInfo);
				if (orientationEntity != null)
					orientationEntity.cloneEvents(timeInfo);
			}
			
			if (isLockedDomNdom) {
				depProperties.setOneFrameGesture(false);
				
				if (isDepGestLockedHand) {
					if (depHandShapeEntity != null)
						depHandShapeEntity.cloneEvents(timeInfo);
					if (depOrientationEntity != null)
						depOrientationEntity.cloneEvents(timeInfo);
				}
			}
		}
		else {
			// start is moving into end - intention is to recreate a one frame Gesture
			if (becomingOneFrame) {	
				boolean isPossibleToMerge = true;	// can we merge handshapes at all? (i.e. is the first the same as the last handshape?)

				if (isGestLockedHand) {
					if (handShapeEntity != null) {
						isPossibleToMerge = handShapeEntity.isPossibleToMergeEvents();
					}
					if (isPossibleToMerge && orientationEntity != null) {
						isPossibleToMerge = orientationEntity.isPossibleToMergeEvents();
					}
				}
				
				if (isPossibleToMerge) {
					if (isLockedDomNdom && isDepGestLockedHand) {
						if (depHandShapeEntity != null) {
							isPossibleToMerge = depHandShapeEntity.isPossibleToMergeEvents();
						}
						if (isPossibleToMerge && depOrientationEntity != null) {
							isPossibleToMerge = depOrientationEntity.isPossibleToMergeEvents();
						}
					}
				}

				int optionPaneResult = -1;
				if (isPossibleToMerge) {
					String warningDialogText = "Warning: this action will create a one-frame Gesture\n\n"
							+ "and merge the start and end handshapes into one handshape.\n\n"
							+ "If this is okay, please press Proceed";
					String warningTitleText = "WARNING";
					int optionType = JOptionPane.YES_NO_OPTION;
					int messageType = JOptionPane.ERROR_MESSAGE;
					String options[] = {"Proceed", "Cancel"};
					// indeed this action is not an error, and is in fact a warning;
					// but nonetheless a red stop sign messageType is louder than a yellow triangle

					try {
						optionPaneResult = JOptionPane.showOptionDialog(this, warningDialogText, warningTitleText, optionType, 
								messageType, null, options,	options[1]);
					} catch (HeadlessException he) {
						System.out.println(he.getMessage());
					}
				} else {				            
					return ErrorMessages.SET_ONE_FRAME_GESTURE_DIFF_HANDSHAPES;
				}

				if (optionPaneResult == JOptionPane.OK_OPTION) {			
					// merge current start and end handshapes into one
					
					if (isGestLockedHand) {
						if (handShapeEntity != null) 
							handShapeEntity.mergeEvents(timeInfo);
						if (orientationEntity != null)
							orientationEntity.mergeEvents(timeInfo);
					}

					properties.setOneFrameGesture(true);
					properties.setStartEndHandshape(GestureEntityProperties.SAME_START_END_HANDSHAPE);
					properties.setStartEndHandshapeNdomHand(GestureEntityProperties.SAME_START_END_HANDSHAPE);
					properties.setStartEndHandshapeDomHand(GestureEntityProperties.SAME_START_END_HANDSHAPE);

					if (isLockedDomNdom) {
						depProperties.setOneFrameGesture(true);
						depProperties.setStartEndHandshape(GestureEntityProperties.SAME_START_END_HANDSHAPE);
						depProperties.setStartEndHandshapeNdomHand(GestureEntityProperties.SAME_START_END_HANDSHAPE);
						depProperties.setStartEndHandshapeDomHand(GestureEntityProperties.SAME_START_END_HANDSHAPE);
						
						if (isDepGestLockedHand) {
							if (depHandShapeEntity != null)
								depHandShapeEntity.mergeEvents(timeInfo);
							if (depOrientationEntity != null)
								depOrientationEntity.mergeEvents(timeInfo);
						}
					}

					return null;
				} else {
					return ""; // this is a dummy return code, which returns with no action being taken, and prevents a popup window in calling function
				}
			}
		}
		
		return null;
	}

	public GestureEntityProperties getGestureEntityProperties() {
		if(glossProperties == null) {
			glossProperties = new GestureEntityProperties();
		}
		return glossProperties;
	}
	
	@Override
	public GestureEventsEntity clone(String fieldID) {
		GestureChainedEventsCollection cll = manager.getGestureChainedEventsCollection().cloneGestureChainedEventsCollection();
		GestureEventsEntity ret = new GestureEventsEntity(cll, segmentPanel, getField());
		ret.updateParentEntity();
		return ret;
	}
	
	@Override
	public void deletePreparation() {
		super.deletePreparation();

		GestureEventsEntity depGestEntity = getDependentGestureEntity();
		
		if (isLockedDomNdomAlignments()) {
			if (depGestEntity != null && depGestEntity.getPreparation() != null) {
				depGestEntity.deletePreparation();
				depGestEntity.setStartTimeInfo(getStartTimeInfo());
			}
		}
		
		GestureLocationEntity gestureLocationEntity = getDependentLocationEntity();
		if (gestureLocationEntity != null) {
			gestureLocationEntity.updateTimes();
		}
		
		GestureTypeEntity gestureTypeEntity = getDependentGestureTypeEntity();
		if (gestureTypeEntity != null) {
			gestureTypeEntity.updateTimes();
		}
	}
	
	@Override
	public void deleteRetraction() {
		super.deleteRetraction();

		GestureEventsEntity depGestEntity = getDependentGestureEntity();
		
		if (isLockedDomNdomAlignments()) {
			if (depGestEntity != null && depGestEntity.getRetraction() != null) {
				depGestEntity.deleteRetraction();
				depGestEntity.setEndTimeInfo(getEndTimeInfo());
			}
		}
		
		GestureLocationEntity gestureLocationEntity = getDependentLocationEntity();
		if (gestureLocationEntity != null) {
			gestureLocationEntity.updateTimes();
		}
		
		GestureTypeEntity gestureTypeEntity = getDependentGestureTypeEntity();
		if (gestureTypeEntity != null) {
			gestureTypeEntity.updateTimes();
		}
	}
	
	@Override
	public void deleteRelaxed() {
		super.deleteRelaxed();

		GestureEventsEntity depGestEntity = getDependentGestureEntity();
		
		if (isLockedDomNdomAlignments()) {
			if (depGestEntity != null && depGestEntity.getRelaxed() != null) {
				depGestEntity.deleteRelaxed();
				depGestEntity.setEndTimeInfo(getEndTimeInfo());
			}
		}
		
		GestureLocationEntity gesureLocationEntity = getDependentLocationEntity();
		if (gesureLocationEntity != null) {
			gesureLocationEntity.updateTimes();
		}
		
		GestureTypeEntity gestureTypeEntity = getDependentGestureTypeEntity();
		if (gestureTypeEntity != null) {
			gestureTypeEntity.updateTimes();
		}
	}
	
	@Override
	public void deletePrestrokeHold() {
		super.deletePrestrokeHold();

		GestureEventsEntity depGestEntity = getDependentGestureEntity();
		GestureChainedEventsEntity handEntity = getDependentHandEntity();
		GestureHandShapeEntity handShapeEntity = getDependentHandShapeEntity();
		GestureOrientationEntity orientationEntity = getDependentOrientationEntity();
		
		Time time = getValueOrTextValueEvent().getStartTimeInfo();
		
		if (isLockedGesturePhonAlignments()) {
			if (handEntity != null) {
			    handEntity.setStartTimeInfo(time);
				if (handShapeEntity != null) {
					MediaToolbarCommons.setGestureHandshapeStartTime(
							handShapeEntity, time);
				}
				if (orientationEntity != null) {
					MediaToolbarCommons.setGestureOrientationStartTime(
							orientationEntity, time);
				}
			}
		}
		
		if (isLockedDomNdomAlignments()) {
			if (depGestEntity != null && depGestEntity.getPrestrokeHold() != null) {
				depGestEntity.deletePrestrokeHold();
			}
		}
		
		GestureLocationEntity LocationEntity = getDependentLocationEntity();
		if (LocationEntity != null) {
			LocationEntity.updateTimes();
		}
		
		GestureTypeEntity signTypeEntity = getDependentGestureTypeEntity();
		if (signTypeEntity != null) {
			signTypeEntity.updateTimes();
		}
	}
	
	@Override
	public void deletePoststrokeHold() {
		super.deletePoststrokeHold();

		GestureEventsEntity depGestEntity = getDependentGestureEntity();
		GestureChainedEventsEntity handEntity = getDependentHandEntity();
		GestureHandShapeEntity handShapeEntity = getDependentHandShapeEntity();
		GestureOrientationEntity orientationEntity = getDependentOrientationEntity();
		
		Time time = getValueOrTextValueEvent().getEndTimeInfo();
		
		if (isLockedGesturePhonAlignments()) {
			if (handEntity != null) {
			    handEntity.setEndTimeInfo(time);
				if (handShapeEntity != null) {
					MediaToolbarCommons.setGestureHandshapeEndTime(
							handShapeEntity, time);
				}
				if (orientationEntity != null) {
					MediaToolbarCommons.setGestureOrientationEndTime(
							orientationEntity, time);
				}
			}
		}
		
		if (isLockedDomNdomAlignments()) {
			if (depGestEntity != null && depGestEntity.getPoststrokeHold() != null) {
				depGestEntity.deletePoststrokeHold();
			}
		}
		
		GestureLocationEntity locationEntity = getDependentLocationEntity();
		if (locationEntity != null) {
			locationEntity.updateTimes();
		}
		
		GestureTypeEntity signTypeEntity = getDependentGestureTypeEntity();
		if (signTypeEntity != null) {
			signTypeEntity.updateTimes();
		}
	}

	public ArrayList<EventsEntity> getDependentEntities() {
		ArrayList<EventsEntity> dEntities = new ArrayList<EventsEntity>();
		
		if(dependentHandEntity != null) {
			dEntities.add(dependentHandEntity);
		}
		
		if(dependentHandShapeEntity != null) {
			dEntities.add(dependentHandShapeEntity);
		}
		
		if(dependentOrientationEntity != null) {
			dEntities.add(dependentOrientationEntity);
		}
		
		if(dependentGestureEntity != null) {
			dEntities.add(dependentGestureEntity);
			
			EventsEntity depHandEntity = dependentGestureEntity.getDependentHandEntity();
			if(depHandEntity != null) {
				dEntities.add(depHandEntity);
			}
			
			EventsEntity depHandShapeEntity = dependentGestureEntity.getDependentHandShapeEntity(); 
			if(depHandShapeEntity != null) {
				dEntities.add(depHandShapeEntity);
			}
		}
		
		if (dependentApexEntity != null) {
			dEntities.add(dependentApexEntity);
			ApexChainedApexEventsEntity depApexEntity = dependentApexEntity.getDependentApexEntity();
			if (depApexEntity != null) {
				dEntities.add(depApexEntity);
			}
		}
		
		if (dependentLocationEntity != null) {
			dEntities.add(dependentLocationEntity);
		}
		
		if (dependentSignTypeEntity != null) {
			dEntities.add(dependentSignTypeEntity);
		}
		return dEntities;
	}
	
	public void setGestureEntityProperties(GestureEntityProperties glossProperties) {
		this.glossProperties = glossProperties;
	}
	
	@Override
	public GestureChainedEventsCollection getGestureChainedEventsCollection() {
		return chainedEventsCollection;
	}

	public GestureChainedEventsEntity getDependentHandEntity() {
		return dependentHandEntity;
	}

	public void setCorrespondingGestureEntityID(String correspondingGestureEntityID) {
		this.correspondingGestureEntityID = correspondingGestureEntityID;
	}
	
	public String getCorrespondingGestureEntityID() {
		return correspondingGestureEntityID;
	}
	
	public void setDependentHandEntity(GestureChainedEventsEntity dependentHandEntity) {
		if(dependentHandEntity != null) {
	        setDependentHandEntityID(dependentHandEntity.getID());
		}
		this.dependentHandEntity = dependentHandEntity;
	}

	
	public GestureHandShapeEntity getDependentHandShapeEntity() {
		return dependentHandShapeEntity;
	}

	public void setDependentHandShapeEntity(
			GestureHandShapeEntity dependentHandShapeEntity) {
        this.dependentHandShapeEntity = dependentHandShapeEntity;
        if(dependentHandEntity != null) {
            this.setDependentHandShapeEntityID(dependentHandShapeEntity.getID());
		}
	}
	
	public GestureOrientationEntity getDependentOrientationEntity() {
		return dependentOrientationEntity;
	}

	public void setDependentOrientationEntity(
			GestureOrientationEntity dependentOrientationEntity) {
        this.dependentOrientationEntity = dependentOrientationEntity;
        if(dependentOrientationEntity != null) {
            this.setDependentHandShapeEntityID(dependentOrientationEntity.getID());
		}
	}

	public GestureEventsEntity getDependentGestureEntity() {
		return dependentGestureEntity;
	}

	public void setDependentGestureEntity(
			GestureEventsEntity dependentGestureEntity) {
		this.dependentGestureEntity = dependentGestureEntity;

        if(dependentGestureEntity != null) {
		    setCorrespondingGestureEntityID(dependentGestureEntity.getID());
        } else {
            setCorrespondingGestureEntityID(null);
        }
	}
	
	public ApexChainedApexEventsEntity getDependentApexEntity() {
		return dependentApexEntity;
	}

	public void setDependentApexEntity(
			ApexChainedApexEventsEntity dependentApexEntity) {
		this.dependentApexEntity = dependentApexEntity;

        if(dependentApexEntity != null) {
		    setDependentApexEntityID(dependentApexEntity.getID());
        } else {
            setDependentApexEntityID(null);
        }
	}
	
	public GestureTypeEntity getDependentGestureTypeEntity() {
		return dependentSignTypeEntity;
	}

	public void setDependentSignTypeEntity(GestureTypeEntity dependentSignTypeEntity) {
		this.dependentSignTypeEntity = dependentSignTypeEntity;
    	if (dependentSignTypeEntity != null) {
    		setDependentSignTypeEntityID(dependentSignTypeEntity.getID());
		} else {
			setDependentSignTypeEntityID(null);
		}
	}
	
	public GestureLocationEntity getDependentLocationEntity() {
		return dependentLocationEntity;
	}

	public void setDependentLocationEntity(GestureLocationEntity dependentLocationEntity) {
		this.dependentLocationEntity = dependentLocationEntity;
    	if (dependentLocationEntity != null) {
    		setDependentLocationEntityID(dependentLocationEntity.getID());
		}
	}

	@Override
	public GestureEventsEntity getPreviousEntity() {
		return previousEntity;
	}

	public void setPreviousEntity(GestureEventsEntity previousEntity) {
		this.previousEntity = previousEntity;
	}

	@Override
	public GestureEventsEntity getNextEntity() {
		return nextEntity;
	}

	public void setNextEntity(GestureEventsEntity nextEntity) {
		this.nextEntity = nextEntity;
	}

	@Override
	public String getText() {
        if(getTextValue() == null) {
            return "";
        } else {
		    return getTextValue().getText();
        }
	}

	@Override
	public String getText(boolean start) {
		return getText();
	}

	@Override
	public boolean isEndCompound() {
		return endCompound != null;
	}

	@Override
	public boolean isStartCompound() {
		return startCompound != null;
	}

	@Override
	public GestureCompoundable getEndCompound() {
		return endCompound;
	}

	@Override
	public GestureCompoundable getStartCompound() {
		return startCompound;
	}

	@Override
	public void setEndCompound(GestureCompoundable endCompound) {
		this.endCompound = endCompound;
	}

	@Override
	public void setStartCompound(GestureCompoundable startCompound) {
		this.startCompound = startCompound;
	}

	@Override
	public void paint(Graphics2D g2d, double Y, Font font) {
		g2d.setFont(font);
		FontMetrics fontMetrics = getFontMetrics(font);
		
		segmentPanel.getPainter().drawGestureEntity(this, g2d, Y, fontMetrics, getField().getFieldID(), previousEntity, nextEntity);
	}
}
