package edu.bu.signstream.grepresentation.fields.gestureField;

import java.awt.Font;
import java.awt.Graphics2D;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.stream.Collectors;

import edu.bu.signstream.common.util.Sorter;
import edu.bu.signstream.common.util.vo.TrackValue;
import edu.bu.signstream.time.Bound;
import edu.bu.signstream.time.Interval;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Painter;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class GestureEventsEntityCollection {

	private ArrayList<GestureEventsEntity> entities = new ArrayList<GestureEventsEntity>();
	private NewGestureChainedEvent newEvent = null;
	private GestureEnteredText enteredGestureText = null;
	private SignStreamSegmentPanel segmentPanel;

	public GestureEventsEntityCollection(
			ArrayList<GestureEventsEntity> entities,
			SignStreamSegmentPanel segmentPanel) {
		this.entities = entities;
		this.segmentPanel = segmentPanel;

        sortEntities();
	}

    public void sortEntities() {
        int entitiesSize = 0;

        for(GestureEventsEntity entity : entities) {
            // skip entered text values
            if(entity.getStartTimeInfo() != null) {
                entitiesSize++;
            }
        }

        if(entitiesSize == 1)
            return;

        double[] startTimes = new double[entitiesSize];
        HashMap<String, GestureEventsEntity> map = new HashMap<String, GestureEventsEntity>();

        int i = 0;
        for(GestureEventsEntity entity : entities) {
            if(entity.getStartTimeInfo() != null) {
                double movieTime = entity.getStartTimeInfo().doubleValue();
                map.put(""+movieTime, entity);
                startTimes[i] = movieTime;
                i++;
            }
        }

        Sorter.sort(startTimes);

        i = 0;
        while(i < entitiesSize-1) {
            GestureEventsEntity gestEntity = map.get(""+startTimes[i]);
            i++;
            GestureEventsEntity nextGestEntity = map.get(""+startTimes[i]);
            gestEntity.setNextEntity(nextGestEntity);
            nextGestEntity.setPreviousEntity(gestEntity);
        }
    }
	
	public ArrayList<GestureEventsEntity> getGestureEventsEntityWithText(String txt) {
		ArrayList<GestureEventsEntity> glosses = new ArrayList<GestureEventsEntity>();
		for(GestureEventsEntity entity : entities) {
			String text = entity.getText();
			if(text.equalsIgnoreCase(txt)) {
				glosses.add(entity);
			}
		}
		return glosses;
	}
	
	public double getNextEntityOrTextStartCoord(double coord) {
		double smalestNextCoord = -1;
		for(GestureEventsEntity entity : entities) {
			double entityCoord = entity.getStartTimeCoordinate(); 

			if(smalestNextCoord == -1) {
				if(entityCoord >= coord) {
					smalestNextCoord = entityCoord;
				}
			} else {
				if(entityCoord > coord && entityCoord < smalestNextCoord) {
					smalestNextCoord = entityCoord;
				}	
			}
		}
		
		if(enteredGestureText != null) {
			ArrayList<GestureText> entText = enteredGestureText.getEnteredText();
			for(GestureText text : entText) {
				double textStartCoord = text.getStartCoord();
				if(smalestNextCoord == -1) {
					if(textStartCoord > coord) {
						smalestNextCoord = textStartCoord;
					}
				} else {
					if(textStartCoord > coord && textStartCoord < smalestNextCoord) {
						smalestNextCoord = textStartCoord;
					}	
				}
			}
		}
		
		return smalestNextCoord;
	}

	public GestureEventsEntity getNextEntity(Time endTimeInfo) {
		GestureEventsEntity nextEntity = null;
		for (GestureEventsEntity entity : entities) {
			if(entity.getStartTimeInfo().greaterThan(endTimeInfo)) {
				if(nextEntity == null) {
					nextEntity = entity;
				} else if(entity.getStartTimeInfo().lessThan(nextEntity.getStartTimeInfo())) {
					nextEntity = entity;
				}
			}
		}
		return nextEntity;
	}
	
	public GestureEventsEntity getPreviousEntity(Time startTimeInfo) {
		GestureEventsEntity prevEntity = null;
		for (GestureEventsEntity entity : entities) {
			if(entity.getStartTimeInfo().lessThan(startTimeInfo)) {
				if(prevEntity == null) {
					prevEntity = entity;
				} else if(entity.getStartTimeInfo().greaterThan(prevEntity.getStartTimeInfo())) {
					prevEntity = entity;
				}
			}
		}
		return prevEntity;
	}
	
	public double getPrevEntityOrTextStartCoord(double coord) {
		double smalestPrevCoord = -1;
		
		for(GestureEventsEntity entity : entities) {
			double entityCoord = entity.getEndTimeCoordinate(); 

			if(smalestPrevCoord == -1) {
				if(entityCoord <= coord) {
					smalestPrevCoord = entityCoord;
				}
			} else {
				if(entityCoord < coord && entityCoord > smalestPrevCoord) {
					smalestPrevCoord = entityCoord;
				}	
			}
		}
		
		if(enteredGestureText != null) {
			ArrayList<GestureText> entText = enteredGestureText.getEnteredText();
			for(GestureText text : entText) {
				double textStartCoord = text.getStartCoord();
				if(smalestPrevCoord == -1) {
					if(textStartCoord < coord) {
						smalestPrevCoord = textStartCoord;
					}
				} else {
					if(textStartCoord < coord && textStartCoord > smalestPrevCoord) {
						smalestPrevCoord = textStartCoord;
					}	
				}
			}
		}
		
		return smalestPrevCoord;
	}

	// JACK TODO: ??????
	public ArrayList<NotCompleteGestureEvent> getNotCompleteGestureEventsInInterval(int startCoord, int endCoord) {
		return new ArrayList<NotCompleteGestureEvent>();
	}

    public ArrayList<GestureEventsEntity> getEntitiesInInterval(Interval interval) {
	    return entities.stream()
            .filter(e -> e.overlaps(interval))
            .collect(Collectors.toCollection(ArrayList::new));
    }

    public ArrayList<GestureEventsEntity> getEntitiesInInterval(Interval interval, Bound bound) {
	    return entities.stream()
            .filter(e -> e.overlaps(interval, bound))
            .collect(Collectors.toCollection(ArrayList::new));
    }

	public void removeEmptyEntities() {
		ArrayList<GestureEventsEntity> entitiesToDelete = new ArrayList<GestureEventsEntity>();

		for (GestureEventsEntity entity : entities) {
			if (entity.getStartTimeInfo() == null
					|| entity.getEndTimeInfo() == null) {
				entitiesToDelete.add(entity);
			}
		}

		for (GestureEventsEntity entityToDelete : entitiesToDelete) {
			deleteEntity(entityToDelete);
		}
	}

	public void setEnteredTextObject(GestureEnteredText enteredGestureText) {
		this.enteredGestureText = enteredGestureText;
	}

	public GestureEnteredText getEnteredText() {
		return enteredGestureText;
	}

	public GestureEventsEntity getEntityWithRefID(String refEntityId) {
		for (GestureEventsEntity entity : entities) { 
			if (refEntityId != null
					&& refEntityId.equalsIgnoreCase(entity.getRefEntityId()))
				return entity;
		}
		return null;
	}

	public GestureEventsEntity getEntity(String entityID) {
		for (GestureEventsEntity entity : entities) {
			String id = entity.getID();
			if (id.equalsIgnoreCase(entityID))
				return entity;
		}
		return null;
	}

	public ArrayList<GestureEventsEntity> getEntities() {
		return entities;
	}

	public int size() {
		return entities.size();
	}

	public void deleteEntity(GestureEventsEntity entity) {
		GestureEventsEntity previousEntity = entity.getPreviousEntity();
		GestureEventsEntity nextEntity = entity.getNextEntity();

		if (previousEntity != null) {
			previousEntity.setNextEntity(nextEntity);
		}

		if (nextEntity != null) {
			nextEntity.setPreviousEntity(previousEntity);
		}

		entities.remove(entity);
	}

	public Time getStartTimeInfo() {
		removeEmptyEntities();
		
		if (entities.isEmpty()) 
			return SS3Singleton.getVideoTimeRange().zero();
		
		Time earliestStartTime = entities.get(0).getStartTimeInfo();
				
		for (GestureEventsEntity entity : entities) {
			earliestStartTime = (earliestStartTime.doubleValue() < entity.getStartTimeInfo().doubleValue()
					? earliestStartTime
					: entity.getStartTimeInfo());
		}
		
		return earliestStartTime;
	}

	public Time getEndTimeInfo() {
		removeEmptyEntities();
		
		if (entities.isEmpty())
			return SS3Singleton.getVideoTimeRange().zero();
		
		Time latestEndTime = entities.get(0).getEndTimeInfo();
		
		for (GestureEventsEntity entity : entities) {
			latestEndTime = (latestEndTime.doubleValue() < entity.getStartTimeInfo().doubleValue()
						? latestEndTime
						: entity.getEndTimeInfo());
		}
		
		return latestEndTime;
	}

	public double getStartTimeCoordinate() {
		Time time = getStartTimeInfo();

		return SS3Singleton.getVideoControlManager().getZoomer()
				.getCoordinateTimeConvertor().convertTimeInfoToCoordinate(time);
	}

	public double getEndTimeCoordinate() {
		Time time = getEndTimeInfo();

		return SS3Singleton.getVideoControlManager().getZoomer()
				.getCoordinateTimeConvertor().convertTimeInfoToCoordinate(time);
	}

	public void unselect() {
		for (GestureEventsEntity entity : entities) {
			entity.unselect();
		}
	}

	public void reset() {
		for (GestureEventsEntity entity : entities) {
			entity.reset();
		}
	}

	public void addEntity(GestureEventsEntity entity) {

		Time startTimeInfo = entity.getStartTimeInfo();

		GestureEventsEntity previousEntity = null;
		GestureEventsEntity nextEntity = null;

		for (GestureEventsEntity ent : entities) {
			if(ent.getStartTimeInfo() != null) {

				Time entStartTimeInfo = ent.getStartTimeInfo();
				Time entEndTimeInfo = ent.getEndTimeInfo();

				if (entStartTimeInfo.lessThanOrEq(startTimeInfo)) {
					
					if (previousEntity == null) {
						previousEntity = ent;
					} else if (entStartTimeInfo.greaterThanOrEq(previousEntity.getStartTimeInfo())
							&& (entEndTimeInfo.greaterThanOrEq(previousEntity.getStartTimeInfo()))) {
							previousEntity = ent;
					}
				} else {
					if (nextEntity == null) {
						nextEntity = ent;
					} else if (entStartTimeInfo.lessThanOrEq(nextEntity.getStartTimeInfo())
							&& entEndTimeInfo.lessThanOrEq(nextEntity.getStartTimeInfo())) {
						nextEntity = ent;
					}
				}
			}
		}

		if (previousEntity != null) {
			GestureEventsEntity next = previousEntity.getNextEntity();
			if (next != null) {
				entity.setNextEntity(next);
				next.setPreviousEntity(entity);
			}
			previousEntity.setNextEntity(entity);
			entity.setPreviousEntity(previousEntity);

		} else if (nextEntity != null) {
			entity.setNextEntity(nextEntity);
			nextEntity.setPreviousEntity(entity);
		}

		this.entities.add(entity);
	}

	public void setNewGestureChainedEvent(NewGestureChainedEvent newEvent) {
		this.newEvent = newEvent;
	}

	public NewGestureChainedEvent getNewChainedEvent() {
		return newEvent;
	}

	public void selectEntity(int x) {
		EventsEntity entity = getGestureEventsEntityAt(x);
		if (entity != null)
			entity.select();
	}

	public GestureEventsEntity getGestureEventsEntityAt(double coordX) {
		for (GestureEventsEntity entity : entities) {
			if (entity.contains(coordX))
				return entity;
		}
		return null;
	}

	public Event getSelectedEvent() {
		GestureEventsEntity selectedEntity = getSelectedEntity();
		if (selectedEntity != null) {
			return selectedEntity.getSelectedEvent();
		}
		return null;
	}

	public GestureEventsEntity getSelectedEntity() {
		for (GestureEventsEntity entity : entities) {
			if (entity.getSelectedEvent() != null)
				return entity;
		}
		return null;
	}

	public GestureChainedEvent getEventAt(double x) {
		GestureEventsEntity entity = getGestureEventsEntityAt(x);
		if (entity == null)
			return null;

		return entity.getGestureChainedEventsCollection().getEventAt(x);
	}

	public void setSelectedEvementValueID(String valueID) {
		Event event = getSelectedEvent();

		if (event instanceof GestureChainedEvent) {
			GestureChainedEvent chainedEvent = (GestureChainedEvent) event;
			chainedEvent.setID(valueID);
		}
	}

	public NewGestureChainedEvent getNewGestureChainedEvent() {
		return newEvent;
	}

	@Deprecated // GD: possibly don't use...
	public GestureChainedEventsCollection createNewGestureChainedEventCollection(String valueID) {
		if (valueID != null && newEvent != null) {
			TrackValue trackValue = new TrackValue();
			trackValue.setValueId(valueID);

			trackValue.setStartTimeInfo(newEvent.getStartTimeInfo());
			trackValue.setEndTimeInfo(newEvent.getEndTimeInfo());

			GestureChainedEvent event = new GestureChainedEvent(trackValue, segmentPanel);

			GestureChainedEventsCollection collection = new GestureChainedEventsCollection();
			collection.setValueEvent(event);
			newEvent = null;

			return collection;
		}
		return null;
	}

	// JACK TODO: What in tarnation?
	public void sort(ArrayList<GestureEventsEntity> entitiesToSort) {
		removeEmptyEntities();

		HashMap<String, GestureEventsEntity> map = new HashMap<String, GestureEventsEntity>();

		double[] startTimes = new double[entitiesToSort.size()];

		ArrayList<GestureEventsEntity> toRemove = new ArrayList<GestureEventsEntity>();

		for (int i = 0; i < entitiesToSort.size(); i++) {
			GestureEventsEntity entity = entitiesToSort.get(i);
			if (entity != null && entity.getStartTimeInfo() != null) {
				double startTime = entity.getStartTimeInfo().doubleValue();
				map.put("" + startTime, entity);
				startTimes[i] = startTime;
			} else {
				toRemove.add(entity);
			}
		}

		for (GestureEventsEntity cEnt : toRemove) {
			entitiesToSort.remove(cEnt);
		}

		Sorter.sort(startTimes);
		entitiesToSort.clear();

		for (int j = 0; j < startTimes.length; j++) {
			GestureEventsEntity obj = map.get("" + startTimes[j]);
			entitiesToSort.add(obj);
		}
	}

	public void paintEntities(Graphics2D g2d, double Y, Font font) {
		for (GestureEventsEntity entity : entities) {
			entity.paint(g2d, Y, font);
		}
	}

	public void paintNewEvent(Graphics2D g2d, double Y) {
		if (newEvent != null) {
			Painter painter = segmentPanel.getPainter(); // new Painter();
			painter.drawNewEvent(newEvent, g2d, Y);
		}
	}

	public void paintEnteredGestureText(Graphics2D g2d, double Y, Font font) {
		if (enteredGestureText != null)
			enteredGestureText.paint(g2d, Y, font);
	}
/*	
	//
	public Compoundable getPreviousCompoundable(int startTime) {
		Compoundable nCmp = null;
		
		ArrayList<Compoundable> cmps = new ArrayList<Compoundable>();
		cmps.addAll(entities);
		//cmps.addAll(enteredGestureText.getNotCompleteGestureEvents());
		//cmps.addAll(enteredGestureText.getTextCollection());
		
		Iterator<Compoundable> eIt = cmps.iterator();
		while (eIt.hasNext()) {
			Compoundable cmp = eIt.next();
			if (startTime >= cmp.getEndTimeInfo().getMovieTime()) {
				if (nCmp == null) {
					nCmp = cmp;
				} else if (nCmp.getEndTimeInfo().getMovieTime() < cmp
						   .getEndTimeInfo().getMovieTime()) {
					nCmp = cmp;
				}
			}
		}
		
		return nCmp;
	}
	
	public ArrayList<Compoundable> getBeforenAfterCompounds(Event event) {
		Compoundable beforeCmp = null;
		Compoundable afterCmp = null;
		
		for (GestureChainedEventsEntity entity : entities) {
			if (entity.getEndTimeInfo().getMovieTime() < event
				.getStartTimeInfo().getMovieTime()) {
				if (beforeCmp == null) {
					beforeCmp = entity;
				} else if (entity.getEndTimeInfo().getMovieTime() > beforeCmp
						   .getEndTimeInfo().getMovieTime()) {
					beforeCmp = entity;
				}
			} else if (entity.getStartTimeInfo().getMovieTime() > event
					   .getEndTimeInfo().getMovieTime()) {
				if (afterCmp == null) {
					afterCmp = entity;
				} else if (entity.getStartTimeInfo().getMovieTime() < afterCmp
						   .getStartTimeInfo().getMovieTime()) {
					afterCmp = entity;
				}
			}
		}
		
		ArrayList<Compoundable> cmp = new ArrayList<Compoundable>();
		cmp.add(0, beforeCmp);
		cmp.add(1, afterCmp);
		
		return cmp;
	}
	
	public Compoundable getNextCompoundable(int endTime) {
		Compoundable nCmp = null;
		
		ArrayList<Compoundable> cmps = new ArrayList<Compoundable>();
		cmps.addAll(entities);
		//cmps.addAll(enteredGestureText.getNotCompleteGestureEvents());
		//cmps.addAll(enteredGestureText.getTextCollection());
		
		Iterator<Compoundable> eIt = cmps.iterator();
		while (eIt.hasNext()) {
			Compoundable cmp = eIt.next();
			if (endTime < cmp.getStartTimeInfo().getMovieTime()) {
				if (nCmp == null) {
					nCmp = cmp;
				} else if (cmp.getStartTimeInfo().getMovieTime() < nCmp
						   .getStartTimeInfo().getMovieTime()) {
					nCmp = cmp;
				}
			}
		}
		return nCmp;
	}
	//
	 */
}
