package edu.bu.signstream.grepresentation.fields.gestureField;

import java.awt.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.logging.Level;

import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JTextField;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowSegment;
import edu.bu.signstream.grepresentation.fields.GestureTextEventsEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.GestureInfoDialog;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.MorphologicalPhonologicalInfoDialog;
import edu.bu.signstream.time.Rounding;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.time.TimeRange;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.nonManualField.NonManualFieldWrapper;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.SegmentGraphicRepresentation;
import edu.bu.signstream.grepresentation.view.SegmentTemplate;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.main.ToolBarPanels;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolBar;
import edu.bu.signstream.ui.SS3SignStreamApplication;

/**
 * Gesture fields contain time-aligned text. Predefined gloss fields: the main
 * gloss field and the non-dominant hand gloss field. These can be used for
 * coding activity of the dominant and non-dominant hands, respectively.Those
 * two fields are always displayed.
 * 
 * Gesture field corresponds with Hand field and Hand Shape field.
 * */

public class GestureFieldWrapper extends NonManualFieldWrapper {
	protected GestureField gestField;

	private GestureMouseDblRightClickedDelegate rDblClickDelegate;

	private Point lastPoint = null;

	public GestureFieldWrapper(GestureField gestField,
			SignStreamSegmentPanel segmentPanel, TimeRange timeRange) {
		super(gestField, segmentPanel, timeRange);
		this.gestField = gestField;
	}

	public GestureField getGestureField() {
		return gestField;
	}

	public void mouseCtrlAltClicked(Point point) {
		EventsEntity ent = gestField.getEntityAt(point.x);
		GestureChainedEventsEntity entity = (GestureChainedEventsEntity) ent;
		Event event = gestField.getEventAt(point.x);
		GestureChainedEvent cEvent = (GestureChainedEvent) event;

        if (Arrays.stream(Window.getWindows())
            .anyMatch(w -> w.isVisible() &&
                (w instanceof MorphologicalPhonologicalInfoDialog ||
                    w instanceof GestureInfoDialog)))
            return;

		if (ent == null || entity == null)
			return;

		SS3Singleton.getMediaToolBar().select(cEvent, entity);

        SegmentTemplate template = segmentPanel.getUtteranceView()
                .getSegmentTemplate(field.getSegmentID());

		if (rDblClickDelegate == null) {
			rDblClickDelegate = new GestureMouseDblRightClickedDelegate(gestField,
					this, template.getSpeechView());
		}

		rDblClickDelegate.controlAltDblClick(cEvent, this, template.getSpeechView());
	}

	@Override
	public boolean contains(int Y) {
		return field.contains(Y);
	}

	@Override
	public void mouseClicked(Point point, JPanel panel) {

		ToolBarPanels panels = segmentPanel.getToolBarPanels();
		MediaToolBar mediaTb = panels.getMediaToolBar();

		GestureChainedEventsEntity entity = (GestureChainedEventsEntity) gestField.getEntityAt(point.x);
		Event event = gestField.getEventAt(point.x);

		if (entity != null && event == null) {
			GestureChainedEventsEntity correctEntity = null;
			ArrayList array = gestField.getEntities();
			for (int i = 0; i < array.size(); i++) {
				GestureChainedEventsEntity ent = (GestureChainedEventsEntity) array.get(i);
				if (ent.contains(point.x)) {
					Event sEvent = ent.getEventAt(point.x);
					if (sEvent != null) {
						correctEntity = ent;
						event = sEvent;

						GestureChainedEvent txtEvent = ent.getTextValue();

						if (point.x < txtEvent.getStartTimeCoordinate()) {
							GestureChainedEvent ciHold = correctEntity
									.getPrestrokeHold();
							GestureChainedEvent iHold = entity.getPrestrokeHold();
							if (ciHold != null
									&& iHold != null
									&& ciHold.getStartTimeCoordinate() == iHold
											.getStartTimeCoordinate()) {
								entity.setPrestrokeHoldEvent(null);
								entity = correctEntity;
							} else {
								GestureChainedEvent cOnset = correctEntity.getPreparation();
								GestureChainedEvent onset = entity.getPreparation();
								if (cOnset != null
										&& onset != null
										&& cOnset.getStartTimeCoordinate() == onset
												.getStartTimeCoordinate()) {
									entity.setPreparationEvent(null);
									entity = correctEntity;
								}
							}
						} else if (point.x > txtEvent.getEndTimeCoordinate()) {
							GestureChainedEvent cfHold = correctEntity.getPoststrokeHold();
							GestureChainedEvent fHold = entity.getPoststrokeHold();
							if (cfHold != null
									&& fHold != null
									&& cfHold.getStartTimeCoordinate() == fHold
											.getStartTimeCoordinate()) {
								entity.setPoststrokeHoldEvent(null);
								entity = correctEntity;
							} else {
								GestureChainedEvent cOffset = correctEntity
										.getRetraction();
								GestureChainedEvent offset = entity.getRetraction();
								if (cOffset != null
										&& offset != null
										&& cOffset.getStartTimeCoordinate() == offset
												.getStartTimeCoordinate()) {
									entity.setPreparationEvent(null);
									entity = correctEntity;
								}
							}
						}
					}
				}
			}
		}
		if (event != null) {
		    mediaTb.select(event, entity);
		} else if (entity != null) {
			event = entity.getEventAt(point.x);
		} else {
			GestureEnteredText enteredText = null;

			try {
				enteredText = gestField.getGestureEventsEntityCollection()
						.getEnteredText();
				
				enteredText.unselectAll();
				GestureText text = enteredText.getTextAt(point.x);

				if (text != null) {
				    var hack = new GestureTextEventsEntity(text, gestField);
				    mediaTb.select(hack, hack);
//					text.setSelected(true);
//					mediaTb.selectEvent(null, null, glsField);
				}
			} catch (NullPointerException ex) {
                SS3Singleton.getSignStreamApplication().logger.log(Level.WARNING, ex.getMessage());
			}
		}
		panel.repaint();
	}

	@Override
	public void mouseRightClicked(Point point) {
	}

	@Override
	public void mouseLeftClicked(Point point) {
	}

	@Override
	public void mouseDblLeftClicked(Point point, JPanel panel) {
		// 2016-11-07 removing all the below to disallow renaming glosses from the main window - must be done via Morph-Phon window.
		// with only one mechanism to change it, we can better control only "sanctioned" entries from SignBank
	}

	public void mousePressed(Point point) {

		GestureEnteredText enteredText = null;

		try {
			enteredText = gestField.getGestureEventsEntityCollection()
					.getEnteredText();

			GestureText text = enteredText.getTextAt(point.x);

			if (text != null) {
				ToolBarPanels panels = segmentPanel.getToolBarPanels();
				MediaToolBar mediaTb = panels.getMediaToolBar();

                var hack = new GestureTextEventsEntity(text, gestField);
                mediaTb.select(hack, hack);
//				text.setSelected(true);
//				mediaTb.selectEvent(null, null, glsField);
			}
		} catch (NullPointerException ex) {
            SS3Singleton.getSignStreamApplication().logger.log(Level.WARNING, ex.getMessage());
        }
	}

	@Override
	public void mouseDblRightClicked(final Point point, final JPanel panel) {
		EventsEntity ent = gestField.getEntityAt(point.x);

        if (Arrays.stream(Window.getWindows())
            .anyMatch(w -> w.isVisible() &&
                (w instanceof MorphologicalPhonologicalInfoDialog ||
                    w instanceof GestureInfoDialog)))
            return;

		if (ent != null) {
			GestureEventsEntity entity = (GestureEventsEntity) ent;

			if (rDblClickDelegate == null) {
				SegmentTemplate template = segmentPanel.getUtteranceView()
						.getSegmentTemplate(field.getSegmentID());

				rDblClickDelegate = new GestureMouseDblRightClickedDelegate(
						gestField, this, template.getSpeechView());
			}

			if (isEntitySelected()) {
				rDblClickDelegate.entityDblClick(entity, point, panel);
			} else {
				Event event = gestField.getEventAt(point.x);
				GestureChainedEvent cEvent = (GestureChainedEvent) event;
				rDblClickDelegate.dblClick(cEvent, entity, point, panel);
			}
			segmentPanel.getUtteranceView().repaint();
		} else {

			try {
				GestureEnteredText enteredText = gestField
						.getGestureEventsEntityCollection()
						.getEnteredText();

				enteredText.unselectAll();
				GestureText selectedText = enteredText.getTextAt(point.x);

				if (selectedText != null) {
					selectedText.setSelected(true);

					SegmentTemplate template = segmentPanel.getUtteranceView()
							.getSegmentTemplate(field.getSegmentID());
					SegmentGraphicRepresentation rep = template.getSignView()
							.getSegmentGraphicRep();

					// user should be able to delete text
					final JPopupMenu popup = new JPopupMenu();
					JMenuItem delete = new JMenuItem("Delete");
					UserInterfaceUtil.setMenuItemLNF(delete);
					delete.addActionListener(new GestureDeleteTextActionListener(
							selectedText, enteredText, rep));
					popup.add(delete);

					JMenuItem cancel = new JMenuItem("Cancel");
					UserInterfaceUtil.setMenuItemLNF(cancel);
					popup.add(cancel);

					popup.setPreferredSize(popup.getPreferredSize());
					popup.show(panel, point.x, point.y);
					popup.setVisible(true);
					
					SS3Singleton.getSignStreamApplication().forcePopupInvisibleOnWindowLostFocus(popup);

					segmentPanel.getUtteranceView().repaint();
				}
			} catch (NullPointerException e) {
				e.printStackTrace();
				SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", e);
			}
		}
	}

	@Override
	public void mouseReleased(Point point, final JPanel panel) {
		// clear the lastPoint since the user has finished dragging the mouse
		lastPoint = null;

		NewGestureChainedEvent newEvent = gestField.getNewGestureChainedEvent();
		if (newEvent != null) {

			SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);

			Time startTimeInfo = newEvent.getStartTimeInfo();
			Time endTimeInfo = newEvent.getEndTimeInfo();

			if (endTimeInfo.lessThan(startTimeInfo)) {
				Time temp = endTimeInfo;
				endTimeInfo = startTimeInfo;
				startTimeInfo = temp;
			}

			boolean invalidEventTiming = false;

			if (startTimeInfo.equals(endTimeInfo)) {
				invalidEventTiming = true;
			}

//			PresentationField parentField = glsField.getParentField();
//			Event parentEvent = parentField.getEventAt(point.x);
//			if (parentEvent != null) {
//				Time parentEventStartTimeInfo = parentEvent.getStartTimeInfo();
//				Time precedingEventStartTimeInfo = segmentPanel.getMultipleMovieController()
//						.getPrecedingFrameTimeInfo(parentEventStartTimeInfo);
//				Time parentEventEndTimeInfo = parentEvent.getEndTimeInfo();
//				Time followingEventEndTimeInfo = segmentPanel.getMultipleMovieController()
//						.getFollowingFrameTimeInfo(parentEventEndTimeInfo);
//				if (startTimeInfo.lessThan(precedingEventStartTimeInfo)) {
//					invalidEventTiming = true;
//				}
//				if (endTimeInfo.greaterThan(followingEventEndTimeInfo)) {
//					invalidEventTiming = true;
//				}
//			} else {
//				invalidEventTiming = true;
//			}

			if (invalidEventTiming) {
				gestField.setNewGestureChainedEvent(null);
				return;
			}

			var range = Util.getCurrentRange(getField());

			GestureEvent textEvent = new GestureEvent(segmentPanel);

			textEvent.setText(GestureEventsEntity.DEFAULT_TEXT);
			textEvent.setStartTimeInfo(startTimeInfo.in(range));
			textEvent.setEndTimeInfo(endTimeInfo.in(range));

			GestureChainedEventsCollection coll = new GestureChainedEventsCollection();
			coll.setTextValueEvent(textEvent);
			GestureEventsEntity newEntity = new GestureEventsEntity(coll, segmentPanel, gestField);
			
			SS3GlossWindowSegment currentSegment = gestField.getParentField().getSS3GlossWindowSegment();
			newEntity.setShowUniqueNumber(currentSegment.getDefaultShowGestureCounters());
			
			int uniqueNumber = currentSegment.getCurrentGestureCounter();
			currentSegment.setGestureUniqueNumber(newEntity, uniqueNumber);		// this should always succeed
			

			newEntity.setStartTimeInfo(startTimeInfo.in(range));
			newEntity.setEndTimeInfo(endTimeInfo.in(range));
			gestField.addGestureEventsEntity(newEntity);
			newEntity.updateParentEntity();

			gestField.setNewGestureChainedEvent(null);

			SS3Singleton.getMediaToolBar(null).selectEvent(textEvent, newEntity, gestField);

            SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
		}
		
		
		
		
//		if (glsField.getSelectedCompoundStartCoord() != -1
//				&& glsField.getSelectedCompoundEndCoord() != -1) {
//			ArrayList list = glsField.getEventsToCreateCompound();
//
//			if (list != null && list.size() > 0) {
//				ToolBarPanels tbPanels = segmentPanel.getToolBarPanels();
//				SegmentToolBarPanel stbp = tbPanels.getSegmentToolBarPanel();
//				stbp.activateCompoundBn(list, glsField);
//			}
//		}

//		EnteredText_XYZ enteredTxt = glsField
//				.getGestureEventsEntityCollection().getEnteredText();
//
//		if (enteredTxt != null) {
//			enteredTxt.selectTextBetweenSelectedEvents();
//			enteredTxt.setSelectedSegment(false);
//		}
	}

	@Override
	public void mouseDraggedAlt(Point p) {
        var time = TimeRange.ABSOLUTE.time(p.x, TimeUnit.COORDINATE)
            .in(Util.getCurrentRange(getField()), Rounding.FLOOR)
            .clamp(SS3Singleton.getMediaPlayer().interval());

        if (!drawNewEvent(time)) {
            GestureChainedEventsEntity entity = (GestureChainedEventsEntity) gestField.getSelectedEntity();
            if (entity != null) {
                if (MOVE_DIRECTION == MOVE_START_POINT) {
                    String resp = entity.canSetSelectedEventStartMovieTime(time);

                    if (resp != null) {
                        SS3Singleton.getErrorMessages().showErrorMessage(resp);
                        return;
                    }

                    entity.setStartTimeInfo(time);
                }

                if (MOVE_DIRECTION == MOVE_END_POINT) {
                    String resp = entity.canSetSelectedEventEndMovieTime(time);

                    if (resp != null) {
                        SS3Singleton.getErrorMessages().showErrorMessage(resp);
                        return;
                    }

                    entity.setEndTimeInfo(time);
                }
            }

            SS3Singleton.getMediaPlayer().seek(time);
        }
    }

	// Point point, final JPanel panel
	@Override
	public void mousePressedAlt(Point point, JPanel panel) {
		EventsEntity entityAt = gestField.getEntityAt(point.x);

		var time = TimeRange.ABSOLUTE.time(point.x, TimeUnit.COORDINATE)
            .in(Util.getCurrentRange(getField()), Rounding.FLOOR)
            .clamp(SS3Singleton.getMediaPlayer().interval());

		if (entityAt != null) {
//			 SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CHAINS_CANNOT_OVERLAP);
			return;
		}

		NewGestureChainedEvent newEvent = new NewGestureChainedEvent(time, segmentPanel);
		
		int numberOfEntities = gestField.getEntities().size();

		if (numberOfEntities == 0) {
			gestField.setNewGestureChainedEvent(newEvent);
		} else {
			EventsEntity nextEntity = gestField.getNextGestureEntity(time);
			EventsEntity prevEntity = gestField.getPreviousGestureEntity(time);

			if (prevEntity != null && prevEntity.getEndTimeInfo().greaterThan(time)) {
				SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CHAINS_CANNOT_OVERLAP);
				return;
			}

			newEvent.setNextEntity(nextEntity);
			newEvent.setPreviousEntity(prevEntity);

			gestField.setNewGestureChainedEvent(newEvent);
		}
		
		SS3Singleton.getApplicationStateController()
				.setLoadedCollectionUpdated(true);
	}

	@Override
	public void unselectAll() {
		super.unselectAll();

		GestureEventsEntityCollection gestCollection = gestField
				.getGestureEventsEntityCollection();
		ArrayList<GestureEventsEntity> gestEntities = gestCollection
				.getEntities();
		for (GestureEventsEntity entity : gestEntities) {
			entity.unselect();
		}

		try {
			gestField.getGestureEventsEntityCollection().getEnteredText()
					.unselectAll();
			gestField.setSelectCompoundStartCoord(-1);
			gestField.setSelectCompoundEndCoord(-1);
		} catch (NullPointerException ex) {
			;
		}
	}


	private boolean drawNewEvent(Time time) {
		NewGestureChainedEvent newEvent = gestField.getNewGestureChainedEvent();
		if (newEvent != null) {
			EventsEntity nextEntity = newEvent.getNextEntity();
			EventsEntity prevEntity = newEvent.getPreviousEntity();

            if (prevEntity != null) {
                var prevEnd = prevEntity.getEndTimeInfo();
                if (time.lessThan(prevEnd))
                    time = prevEnd;
            }

            if (nextEntity != null) {
                var nextStart = nextEntity.getStartTimeInfo();
                if (time.greaterThan(nextStart))
                    time = nextStart;
            }

			newEvent.setEndTimeInfo(time);

			SS3Singleton.getMediaPlayer().seek(time);

			return true;
		}
		return false;
	}
}

class NewTextPopupMenuListener implements PopupMenuListener {
	private ArrayList<GestureText> textInInterval = null;
	private double startCoord, endCoord;
	private GestureField gestField = null;
	private SignStreamSegmentPanel segmentPanel = null;
	private JTextField labelTextField = null;

	NewTextPopupMenuListener(JTextField labelTextField,
			ArrayList<GestureText> textInInterval, double startCoord, double endCoord,
			GestureField gestField, SignStreamSegmentPanel segmentPanel) {
		this.textInInterval = textInInterval;
		this.startCoord = startCoord;
		this.endCoord = endCoord;
		this.gestField = gestField;
		this.segmentPanel = segmentPanel;
		this.labelTextField = labelTextField;
	}

	@Override
	public void popupMenuCanceled(PopupMenuEvent popupMenuEvent) {
		gestField.getGestureEventsEntityCollection().getEnteredText()
				.synchronize(textInInterval, startCoord, endCoord);
		removeListeners(popupMenuEvent);
		segmentPanel.repaint();
	}

	@Override
	public void popupMenuWillBecomeInvisible(PopupMenuEvent popupMenuEvent) {
		removeListeners(popupMenuEvent);
	}

	private void removeListeners(PopupMenuEvent popupMenuEvent) {
		java.awt.event.KeyListener[] listeners = labelTextField
				.getKeyListeners();
		for (int i = 0; i < listeners.length; i++) {
			java.awt.event.KeyListener listener = listeners[i];
			labelTextField.removeKeyListener(listener);
		}

		labelTextField = null;

		JPopupMenu menu = (JPopupMenu) popupMenuEvent.getSource();
		menu.removePopupMenuListener(this);

		System.gc();
	}

	@Override
	public void popupMenuWillBecomeVisible(PopupMenuEvent popupMenuEvent) {
		// System.out.println("Becoming Visible");
	}
}

class UpdateTextPopupMenuListener implements PopupMenuListener {
	private JTextField labelTextField = null;

	UpdateTextPopupMenuListener(JTextField labelTextField) {
		this.labelTextField = labelTextField;
	}

	@Override
	public void popupMenuCanceled(PopupMenuEvent popupMenuEvent) {
		removeListeners(popupMenuEvent);
	}

	@Override
	public void popupMenuWillBecomeInvisible(PopupMenuEvent popupMenuEvent) {
		removeListeners(popupMenuEvent);
	}

	private void removeListeners(PopupMenuEvent popupMenuEvent) {
		java.awt.event.KeyListener[] listeners = labelTextField
				.getKeyListeners();
		for (int i = 0; i < listeners.length; i++) {
			java.awt.event.KeyListener listener = listeners[i];
			labelTextField.removeKeyListener(listener);
		}

		labelTextField = null;

		JPopupMenu menu = (JPopupMenu) popupMenuEvent.getSource();
		menu.removePopupMenuListener(this);

		System.gc();
	}

	@Override
	public void popupMenuWillBecomeVisible(PopupMenuEvent popupMenuEvent) {
		;
	}
}
