package edu.bu.signstream.grepresentation.fields.gestureField;

import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;

import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.MorphologicalPhonologicalInfoDialog;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureChainedEvent;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.GestureInfoDialog;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.nonManualField.MouseDblRightClickedDelegate;
import edu.bu.signstream.grepresentation.fields.nonManualField.NonManualField;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.GestureNRelatedView;
//import edu.bu.signstream.grepresentation.view.GestureNRelatedView;

public class GestureMouseDblRightClickedDelegate extends MouseDblRightClickedDelegate {

	private GestureFieldWrapper wrapper = null;
	private GestureNRelatedView gestView = null;

	public GestureMouseDblRightClickedDelegate(NonManualField nmField, GestureFieldWrapper wrapper, GestureNRelatedView gestureNRelatedView) {
		super(nmField);
		this.wrapper = wrapper;
		this.gestView = gestView;
	}

	/**
	 * Shows Morphological Phonological Information Dialog
	 * 
	 * @param event selected gloss text event
	 */
	public void controlAltDblClick(GestureChainedEvent event, GestureFieldWrapper wrapper, GestureNRelatedView gestView) {
		event.select();
		GestureInfoDialog dialog = SS3Singleton.getGestureFieldDialogs().getGestureInfoDialog();
		if (dialog == null) {
			dialog = new GestureInfoDialog(wrapper, gestView);
			dialog.setResizable(true);
			SS3Singleton.getGestureFieldDialogs().setGestureInfoDialog(dialog);
		} else {
			dialog.update(wrapper, gestView);
		}
		
		dialog.setVisible(true);
		dialog.setLocationRelativeTo(null);
	}

	public void dblClick(GestureChainedEvent event, final GestureEventsEntity gestEntity, final Point point,
			final JPanel panel) {

		if (event == null)
			return;

		Event value = gestEntity.getValueOrTextValueEvent();
		Event iHold = gestEntity.getPrestrokeHold();
		Event fHold = gestEntity.getPoststrokeHold();
		Event onset = gestEntity.getPreparation();
		Event offset = gestEntity.getRetraction();
		Event relaxed = gestEntity.getRelaxed();

		if (event.equals(value)) {
			SS3Singleton.getGestureFieldDialogs().getAssociatedElelmentsMenu(gestEntity, point, panel);
			//valueDblClick(event, gestEntity, point, panel);
		} else {
			final JPopupMenu popup = new JPopupMenu();
			JMenuItem mi = null;

			if (event.equals(iHold)) {
				mi = new JMenuItem("Delete Pre-stroke Hold");
				mi.addActionListener(new ActionListener() {
					@Override
					public void actionPerformed(ActionEvent e) {
						gestEntity.deletePrestrokeHold();
						SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
						panel.repaint();
					}
				});
			} else if (event.equals(fHold)) {
				mi = new JMenuItem("Delete Post-stroke Hold");
				mi.addActionListener(new ActionListener() {
					@Override
					public void actionPerformed(ActionEvent e) {
						gestEntity.deletePoststrokeHold();
						SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
						panel.repaint();
					}
				});
			} else if (event.equals(onset)) {
				mi = new JMenuItem("Delete Preparation");
				mi.addActionListener(new ActionListener() {
					@Override
					public void actionPerformed(ActionEvent e) {
						gestEntity.deletePreparation();
						SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
						panel.repaint();
					}
				});
			} else if (event.equals(offset)) {
				mi = new JMenuItem("Delete Retraction");
				mi.addActionListener(new ActionListener() {
					@Override
					public void actionPerformed(ActionEvent e) {
						gestEntity.deleteRetraction();
						SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
						panel.repaint();
					}
				});
			} else if (event.equals(relaxed)) {
				mi = new JMenuItem("Delete Relaxed");
				mi.addActionListener(new ActionListener() {
					@Override
					public void actionPerformed(ActionEvent e) {
						gestEntity.deleteRelaxed();
						SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
						panel.repaint();
					}
				});
			}
			UserInterfaceUtil.setMenuItemLNF(mi);
			popup.add(mi);
			JMenuItem cancel = new JMenuItem("Cancel");
			UserInterfaceUtil.setMenuItemLNF(cancel);
			popup.add(cancel);
			popup.setPreferredSize(popup.getPreferredSize());
			popup.show(panel, point.x, point.y);
			popup.setVisible(true);

			SS3Singleton.getSignStreamApplication().forcePopupInvisibleOnWindowLostFocus(popup);
		}
	}
	
	public void entityDblClick(
		      final GestureChainedEventsEntity entity,
		      final Point point,
		      final JPanel panel) {

		      final JPopupMenu popup = new JPopupMenu();

		      JMenuItem delete = new JMenuItem("Delete");
		      UserInterfaceUtil.setMenuItemLNF(delete);
		      //delete.addActionListener(new DeleteActionListener(entity, panel));
		      popup.add(delete);
		      
		      JMenuItem cancel = new JMenuItem("Cancel");
		      UserInterfaceUtil.setMenuItemLNF(cancel);
		      popup.add(cancel);

		      popup.setPreferredSize(popup.getPreferredSize());
		      popup.show(panel, point.x, point.y);
		      popup.setVisible(true);
				
			  SS3Singleton.getSignStreamApplication().forcePopupInvisibleOnWindowLostFocus(popup);
		   }
}
