package edu.bu.signstream.grepresentation.fields.gestureField.dialog;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map.Entry;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeSelectionModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureEventsEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.signTypeField.GestureCategory;
import edu.bu.signstream.grepresentation.fields.gestureField.signTypeField.GestureType;
import edu.bu.signstream.grepresentation.fields.gestureField.signTypeField.GestureTypeEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.signTypeField.GestureTypeEvent;
import edu.bu.signstream.grepresentation.fields.gestureField.signTypeField.GestureTypeFieldWrapper;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.GestureNRelatedView;
import edu.bu.signstream.grepresentation.view.LabelObject;

public class GestureTypePanel extends JPanel implements ItemListener, ActionListener {
	
	private GestureInfoPanel gestureInfoPanel = null;
	private GestureEventsEntity operativeGestureEntity = null;
	private GestureTypeEntity gestureTypeEntity = null;
	String currentGestureTypeFieldID = null;
	
	private CheckBoxTree jTree;

	public GestureTypePanel (GestureInfoPanel gestureInfoPanel) {
		this.gestureInfoPanel = gestureInfoPanel;
		
		DefaultMutableTreeNode top = new DefaultMutableTreeNode("Gesture Types");
		populateTree(top);
		
		jTree = new CheckBoxTree(top);
		jTree.setSelectionModel(new DefaultTreeSelectionModel() {
			@Override
			public void addSelectionPath(TreePath path) {
				DefaultMutableTreeNode node = (DefaultMutableTreeNode) path.getLastPathComponent();
				//prevent non-leaf (non checkbox) nodes from being selected
				if (node.isLeaf()) {
					super.addSelectionPath(path);
				}
			}
			@Override
			public void addSelectionPaths(TreePath[] paths) {
				if(paths != null) {
					for (TreePath path : paths) {
						TreePath[] toAdd = new TreePath[1];
						toAdd[0] = path;
						
						if (isPathSelected(path)) {
							super.removeSelectionPaths(toAdd);
						} else {
							super.addSelectionPaths(toAdd);
						}
					}
				}
			}
		});
		
		CheckBoxNodeRenderer renderer = new CheckBoxNodeRenderer();
		jTree.setCellRenderer(renderer);
		expandNodes(jTree, new TreePath(top));
		
		JScrollPane treeView = new JScrollPane(jTree);
		treeView.setPreferredSize(new Dimension(500, 450));
		add(treeView);
		
		update();
	}
	
	//sets all nodes in the tree to an expanded state
	private void expandNodes(JTree tree, TreePath parent) {
		TreeNode node = (TreeNode) parent.getLastPathComponent();
		if (node.getChildCount() >= 0) {
			for (Enumeration e = node.children(); e.hasMoreElements();) {
				TreeNode n = (TreeNode)e.nextElement();
				TreePath path = parent.pathByAddingChild(n);
				expandNodes(tree, path);
			}
			tree.expandPath(parent);
		}
	}
	
	private void populateTree(DefaultMutableTreeNode top) {
		//get categories and types from coding schema
		SS3CodingScheme dbScheme = SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel()
				.getSS3Database().getSS3CodingScheme();
		HashMap<String, GestureCategory> gestureCategories = dbScheme.getGestureTypeObject().getGestureCategories();
		
		//temporary lists to help construct the tree; one for categories, one for types
		HashMap<String, DefaultMutableTreeNode> categoryNodes = new HashMap<String, DefaultMutableTreeNode>();
		
		//make nodes for all categories 
		for (GestureCategory gc : gestureCategories.values()) {
			DefaultMutableTreeNode categoryNode = new DefaultMutableTreeNode(gc, true);
			categoryNodes.put(gc.getId(), categoryNode);
		}
		
		//form tree links between categories
		for (Entry<String, DefaultMutableTreeNode> entry: categoryNodes.entrySet()) {
			 DefaultMutableTreeNode categoryNode = entry.getValue();
			 GestureCategory gestureCategory = (GestureCategory) categoryNode.getUserObject();
			
			String parentID = gestureCategory.getParentCategoryId();

			if (parentID == null || parentID.equals("")) {
				//if no parent categories, must be the highest level categories
				top.add(categoryNode);
			} else {
				DefaultMutableTreeNode parentNode = categoryNodes.get(parentID);
				parentNode.add(categoryNode);
			}

			//add children gesture type nodes to this category node
			for (GestureType gestureType: gestureCategory.getGestureTypes().values()) {
				DefaultMutableTreeNode gestureTypeNode = new DefaultMutableTreeNode(gestureType, true);
				categoryNode.add(gestureTypeNode);
			}
		}
	}
	
	public void setSelectedGestureTypes() {
		TreePath[] selectedPaths = jTree.getSelectionPaths();
		
		GestureNRelatedView view = gestureInfoPanel.getGestureNRelatedView();
		
		ArrayList<GestureType> selectedTypes = new ArrayList<GestureType>();
		HashMap<GestureCategory, ArrayList<GestureType>> categoryTypeMap 
			= new HashMap<GestureCategory, ArrayList<GestureType>>();
		
		if (selectedPaths != null) {
			for (int i = 0; i < selectedPaths.length; i++) {
				//System.out.println(selectedPaths[i]);
				DefaultMutableTreeNode node = (DefaultMutableTreeNode)selectedPaths[i].getLastPathComponent();
				GestureType type = (GestureType)node.getUserObject();
				selectedTypes.add(type);

				// make another map: Category -> list of types, then one list of types per sub-gestureType-entity
				GestureCategory cat = type.getParentCategory();
				categoryTypeMap.putIfAbsent(cat, new ArrayList<GestureType>());		// java8 and above
				categoryTypeMap.get(cat).add(type);
			}
		}
		
		GestureEventsEntity depEntity = operativeGestureEntity.getDependentGestureEntity();

		// delete old entities indiscriminately, before recreating below
		if (gestureTypeEntity != null) {

			gestureTypeEntity.setSelectedTypes(null);
			gestureTypeEntity.getField().deleteEntity(gestureTypeEntity);		// the field method overrides, to delete dependents also

			operativeGestureEntity.setDependentSignTypeEntity(null);
			if (depEntity != null)
				depEntity.setDependentSignTypeEntity(null);
			
			gestureTypeEntity = null;
		}

		// resets operativeGestureEntity, gestureTypeEntity, currentGestureTypeFieldID
		updateGestureTypeEntity();
		
		Time startTimeInfo = operativeGestureEntity.getValueOrTextValueEvent().getStartTimeInfo();
		Time endTimeInfo = operativeGestureEntity.getValueOrTextValueEvent().getEndTimeInfo();
		
		if (selectedTypes != null && !selectedTypes.isEmpty()) {
			// (re)create gesture type entity on the current field/wrapper

			LabelObject gestObj = view.getSegment().getRootLabelObject().getGestureLabelObject();
			
			FieldWrapper topLevelWrapper = gestObj
						.getChildWithFieldID(currentGestureTypeFieldID).getWrapper();

			GestureTypeEvent topLevelEvent = new GestureTypeEvent(
					topLevelWrapper.getSignStreamSegmentPanel(),
					startTimeInfo,
					endTimeInfo
					);

			gestureTypeEntity = new GestureTypeEntity(
					topLevelEvent, topLevelWrapper.getSignStreamSegmentPanel(), topLevelWrapper.getField());
			gestureTypeEntity.setID(Util.getLongUniqueNumber());
			topLevelWrapper.addEventsEntity(gestureTypeEntity);

			gestureTypeEntity.updateParentEntity();
			gestureTypeEntity.setRefGestureEntity(operativeGestureEntity);
			gestureTypeEntity.setSelectedTypes(selectedTypes);

			operativeGestureEntity.setDependentSignTypeEntity(gestureTypeEntity);

			// reset the values to what is currently selected
			for (Entry<GestureCategory, ArrayList<GestureType>> entry: categoryTypeMap.entrySet()) {
				GestureCategory gestureCategory = entry.getKey();
				ArrayList<GestureType> gestureTypes = entry.getValue();

				FieldWrapper wrapper = gestObj
							.getChildWithFieldID(currentGestureTypeFieldID).getWrapper();

				ArrayList<LabelObject> labelObjectsWithWrappers = gestObj.getAllChildren();
				for (LabelObject obj: labelObjectsWithWrappers) {
					String categoryFieldID = currentGestureTypeFieldID.equals(Util.NDOM_GESTURE_TYPE_FIELD_ID)
							? gestureCategory.getNdomFieldID()
							: gestureCategory.getDomFieldID();
					
					if (obj.getFieldID().equalsIgnoreCase(categoryFieldID)) {
						wrapper = obj.getWrapper();
						break;
					}
				}

				GestureTypeFieldWrapper gestureTypeFieldWrapper = (GestureTypeFieldWrapper) wrapper;

				GestureTypeEvent event = new GestureTypeEvent(
						gestureTypeFieldWrapper.getSignStreamSegmentPanel(),
						startTimeInfo,
						endTimeInfo
						);
				GestureTypeEntity ent = new GestureTypeEntity(
						event, gestureTypeFieldWrapper.getSignStreamSegmentPanel(), gestureTypeFieldWrapper.getField());
				ent.updateParentEntity();
				ent.setRefGestureEntity(operativeGestureEntity);
				ent.setSelectedTypes(gestureTypes);

				ent.setID(Util.getLongUniqueNumber());
				gestureTypeFieldWrapper.addEventsEntity(ent);

				ent.setRefEntityId(gestureTypeEntity.getID());
				gestureTypeEntity.addDependentGestureTypeEntity(ent);
			}
		}
	}
	
	public void updateGestureTypeEntity() {
		GestureEventsEntity selectedEntity = (GestureEventsEntity) SS3Singleton.getMediaToolBar(null).selectedEntity();
		GestureEventsEntity depEntity = selectedEntity.getDependentGestureEntity();
		
		String selectedEntityGestureFieldID = selectedEntity.getField().getFieldID();
		boolean isNondom = selectedEntityGestureFieldID.equals(Util.NDOM_GESTURE_FIELD_ID);

		boolean isTwoHanded = gestureInfoPanel.isTwoHandedSelected();
		
//		GestureTypeEntity currentGestureTypeEntity = selectedEntity.getDependentSignTypeEntity();
//		GestureTypeEntity depGestureTypeEntity = depEntity != null ? depEntity.getDependentSignTypeEntity() : null;

		this.currentGestureTypeFieldID = Util.DOM_GESTURE_TYPE_FIELD_ID;
		
		// GREG TODO: change the conditions here
		if (isNondom) {
			if (!isTwoHanded || depEntity == null) {
				this.operativeGestureEntity = selectedEntity;

				this.currentGestureTypeFieldID = Util.NDOM_GESTURE_TYPE_FIELD_ID;
			} else {
				this.operativeGestureEntity = depEntity;
			}
		} else {
			this.operativeGestureEntity = selectedEntity;
		}
		
//		if (currentGestureTypeEntity != null)
//			this.operativeGestureEntity = selectedEntity;
//		else if (depGestureTypeEntity != null)
//			this.operativeGestureEntity = depEntity;
//		else
//			this.operativeGestureEntity = selectedEntity;	

		this.gestureTypeEntity = operativeGestureEntity.getDependentGestureTypeEntity();
	}
	
	public void update() {
		
		updateGestureTypeEntity();

		DefaultMutableTreeNode root = (DefaultMutableTreeNode) this.jTree.getModel().getRoot();
		if (this.gestureTypeEntity == null) {
			deselectAll(root);
		} else {
			jTree.clearSelection();
			setSelections(root);
		}
	}
	
	//traverses through the tree and sets all selections to null
	private void deselectAll(DefaultMutableTreeNode node) {
		for (int i = 0; i < node.getChildCount(); i++) {
			DefaultMutableTreeNode childNode = (DefaultMutableTreeNode) node.getChildAt(i);
			if (childNode.getChildCount() > 0) {
				deselectAll(childNode);
			} else {
				TreePath path = new TreePath(childNode.getPath());
				jTree.removeSelectionPath(path);
			}
		}
	}
	
	//traverses through the tree and sets selections
	private void setSelections(DefaultMutableTreeNode node) {
		for (int i = 0; i < node.getChildCount(); i++) {
			DefaultMutableTreeNode childNode = (DefaultMutableTreeNode) node.getChildAt(i);
			if (childNode.getChildCount() > 0) {
				setSelections(childNode);
			} else {
				if (childNode.getUserObject() instanceof GestureCategory) {
					continue;
				}
				GestureType currType = (GestureType)childNode.getUserObject();
				if (this.gestureTypeEntity.getSelectedTypes().contains(currType)) {
					TreePath path = new TreePath(childNode.getPath());
					jTree.addSelectionPath(path);
				}
			}
		}
	}
	
	@Override
	public void actionPerformed(ActionEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void itemStateChanged(ItemEvent e) {
		// TODO Auto-generated method stub
		
	}
	
}
