package edu.bu.signstream.grepresentation.fields.gestureField.dialog.handShapes;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.handshapeTypes.StartEndHandshapesType;
import edu.bu.signstream.grepresentation.listener.WeakActionListener;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.GestureHandShapeBean;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.GestureHandShapePanel;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.HandShapeBean;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.HandShapePanel;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapeImage;
import edu.bu.signstream.ui.ButtonPanel;

public class GestureHandshapePalette extends JPanel implements ActionListener {

	private GestureHandShapesPaletteFormBean formBean = null;
	private GestureHandShapeBean bean = null;
	private GestureGeneralHandshapePalette palette = null;
    private GestureHandShapePanel hsPanel = null;

	private JDialog dialog = null;

	private final int ENTER = 0;
	private final int CANCEL = 1;

    private final JComboBox<String> options = new JComboBox<String>(new String[] {"Basic Gestures", "ASL-All", "ASL-Letters", "ASL-Numbers", "ASL-Unmarked"});
    private final JComboBox<String> orientationOptions = new JComboBox<String>(new String[] {"Orientations"});
    
	public GestureHandshapePalette(GestureHandShapesPaletteFormBean formBean,
			GestureHandShapeBean bean, SS3CodingScheme scheme, JDialog dialog, int selectedIndex, GestureHandShapePanel hsPanel) {

		this.formBean = formBean;
		this.bean = bean;
		this.dialog = dialog;
        this.hsPanel = hsPanel;

		palette = new GestureGeneralHandshapePalette(formBean.getValues(),
				formBean.getSelectedHshImages(), formBean.getSelectedOrientationImages(), scheme, selectedIndex, hsPanel);
		palette.setBorder(new TitledBorder(formBean.getTitle()));

//        options.addItemListener(new WeakItemListener(new ItemListener() {
//            @Override
//            public void itemStateChanged(ItemEvent itemEvent) {
//                int selectedIndex = options.getSelectedIndex();
//                palette.setPalette(selectedIndex);
//            }
//        }, options));

        options.addItemListener(new ItemListener() {
            @Override
			public void itemStateChanged(ItemEvent itemEvent) {
                int selectedIndex = options.getSelectedIndex();
                palette.setPalette(selectedIndex);
            }
        });
        
        orientationOptions.addItemListener(new ItemListener() {
        	@Override
        	public void itemStateChanged(ItemEvent itemEvent) {
        		int selectedIndex = orientationOptions.getSelectedIndex();
        		palette.setOrientationPalette(selectedIndex);
        	}
        });

        options.setSelectedIndex(selectedIndex);
        orientationOptions.setSelectedIndex(0);

        setLayout(new BorderLayout());
        
        JPanel top = new JPanel(new FlowLayout(FlowLayout.CENTER));
        top.add(new JLabel("Handshapes: "));
        top.add(options);
        JPanel bottom = new JPanel(new FlowLayout(FlowLayout.CENTER));
        bottom.add(new JLabel("Orientations: "));
        bottom.add(orientationOptions);

        JPanel languagePanel = null;
        languagePanel = new JPanel(new BorderLayout());
        languagePanel.add(top, BorderLayout.NORTH);
        languagePanel.add(bottom, BorderLayout.SOUTH);

        JButton enter = new JButton("Enter");
        JButton cancel = new JButton("Cancel");

        ButtonPanel buttonPanel = new ButtonPanel();
        buttonPanel.add(cancel);
        buttonPanel.add(enter);
        add(languagePanel, BorderLayout.NORTH);
        add(palette, BorderLayout.CENTER);
        add(buttonPanel, BorderLayout.SOUTH);

		enter.setActionCommand("" + ENTER);
		cancel.setActionCommand("" + CANCEL);
		enter.addActionListener(new WeakActionListener(this, enter));
		cancel.addActionListener(new WeakActionListener(this, cancel));

		enter.setSelected(true);
	}
	
	public void update(GestureHandShapesPaletteFormBean formBean, GestureHandShapeBean bean, int selectedIndex) {
		this.bean = bean;
		this.formBean = formBean;
        options.setSelectedIndex(selectedIndex);
        orientationOptions.setSelectedIndex(0);

        palette.setSelectedHshImages(formBean.getSelectedHshImages());
        palette.setSelectedOrientationImages(formBean.getSelectedOrientationImages());
        palette.setBorder(new TitledBorder(formBean.getTitle()));
        palette.repaint();
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		String str = ae.getActionCommand();

		ArrayList<HandShapeImage> selectedHandshapes = palette.getSelectedHandshapes();
		ArrayList<OrientationImage> selectedOrientations = palette.getSelectedOrientations();

		switch (Integer.parseInt(str)) {
			case ENTER:

				// first check if this is a one-frame gloss, and if it is then disallow anything more than one handshape in either hand
				if (hsPanel.isOneFrameGloss()) {
					if (selectedHandshapes.size() > 1) {
						SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.SELECT_GESTURE_HANDSHAPES_EXACTLY_ONE);
						break;
					}
				} else if (hsPanel.getSelectedSameDifStartEndHandshapes() == StartEndHandshapesType.DIFFERENT && (selectedHandshapes.size() == 1)) {
					// conversely, if this is not a one-frame gloss and you have dif start/end selected,
					// then disallow a selection of just one handshape in either hand
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.SELECT_GESTURE_HANDSHAPES_MULTIPLE);
					break;
				} else if (selectedHandshapes.size() != selectedOrientations.size()) {
					// user must select the same number of handshape and orientation images
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.GESTURE_SELECT_HANDSHAPES_ORIENTATIONS);
					break;
				}
				
	            // if user only selected none-image for one of above lists - disallow
	            boolean onlyNoneImage = false;
	            onlyNoneImage = onlyNoneImage || 
            		(selectedHandshapes.size() > 0 && selectedHandshapes.stream().allMatch(
            				img -> img.getSS3FieldValue().getIid().equalsIgnoreCase(Util.DNI_HANDSHAPE_IMAGE_ID))) || 
            		(selectedOrientations.size() > 0 && selectedOrientations.stream().allMatch(
            				img -> img.getSS3FieldValue().getIid().equalsIgnoreCase(Util.DNI_ORIENTATION_IMAGE_ID)));
	            
	            if (onlyNoneImage) {
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.GESTURE_CANNOT_SELECT_ONLY_NONE);
	            	break;
	            }

				bean.setUpdated(true);

				if (formBean.getPaletteType() == GestureHandShapesPaletteFormBean.PaletteType.RIGHT) {
					if (hsPanel.getAssociatedFieldsPanel().isDominantHandRight()) {
						bean.setSelectedDomHandShapeImages(selectedHandshapes);
						bean.setSelectedDomOrientationImages(selectedOrientations);
					} else {
						bean.setSelectedNdomHandShapeImages(selectedHandshapes);
						bean.setSelectedNdomOrientationImages(selectedOrientations);
					}
				}

				if (formBean.getPaletteType() == GestureHandShapesPaletteFormBean.PaletteType.LEFT) {
					if (hsPanel.getAssociatedFieldsPanel().isDominantHandRight()) {
						bean.setSelectedNdomHandShapeImages(selectedHandshapes);
						bean.setSelectedNdomOrientationImages(selectedOrientations);
					} else {
						bean.setSelectedDomHandShapeImages(selectedHandshapes);
						bean.setSelectedDomOrientationImages(selectedOrientations);
					}
				}

				if (formBean.getPaletteType() == GestureHandShapesPaletteFormBean.PaletteType.RIGHT_N_LEFT) {
					bean.setSelectedDomHandShapeImages(selectedHandshapes);
					bean.setSelectedNdomHandShapeImages(selectedHandshapes);
					bean.setSelectedDomOrientationImages(selectedOrientations);
					bean.setSelectedNdomOrientationImages(selectedOrientations);
				}

				hsPanel.updateSelectedHandshapesPanels(bean.getSelectedDomHandShapeImages(),
													   bean.getSelectedNdomHandShapeImages(),
													   bean.getSelectedDomOrientationImages(),
													   bean.getSelectedNdomOrientationImages());

				dialog.setVisible(false);
				break;

			case CANCEL:
				dialog.setVisible(false);
				break;
		}
	}
	
	public GestureGeneralHandshapePalette getPalette() {
		return palette;
	}
}
