package edu.bu.signstream.grepresentation.fields.gestureField.dialog.handShapes;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.Insets;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;

import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingConstants;

import edu.bu.signstream.common.util.UIPalette;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldTypeImg;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.FieldValueWrapper;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.PalettePanel;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.SameWidhtIcon;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapeImage;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapePalletClient;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.SelectedHandshapesPanel;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;

public class OrientationPalette extends PalettePanel {

	private static final long serialVersionUID = 1L;

	private SelectedGestureHandshapesPanel hsClient;
	
	private SS3CodingScheme defaultCodingScheme;

	private HashMap<String, SS3FieldValue> hm = new HashMap<String, SS3FieldValue>();
	private ArrayList<OrientationImage> orientationImages = new ArrayList<>();
	
	private JPanel handshapesPanel = new JPanel();
	JScrollPane scrollPane = new JScrollPane(handshapesPanel);

	public OrientationPalette(ArrayList<SS3FieldValue> orientations,
			HandShapePalletClient hsClient, SS3CodingScheme defaultCodingScheme) {
		this.hsClient = (SelectedGestureHandshapesPanel) hsClient;
		this.defaultCodingScheme = defaultCodingScheme;

		initUI();
	}

	private void initUI() {
		setLayout(new GridBagLayout());
		
		Dimension prefSize = new Dimension(450, 150);

		handshapesPanel.setLayout(new GridBagLayout());
		handshapesPanel.setMinimumSize(prefSize);
		GridBagConstraints c = new GridBagConstraints();

		c.insets = new Insets(1, 1, 1, 1);

		populateHahdshapesPanel();

		//scrollPane.setPreferredSize(new Dimension(ddd.width + 10, 500));
		
		scrollPane.setPreferredSize(prefSize);
		scrollPane.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);

		c.gridwidth = 1;
		c.gridy = 1;
		c.gridx = 1;
		add(scrollPane, c);
	}

	private void populateHahdshapesPanel() {
		handshapesPanel.removeAll();
		handshapesPanel.setBackground(UIPalette.getBackground());

		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(5, 5, 5, 5);
		c.gridwidth = 1;
		c.weighty = 1;
		c.gridy = 0;
		c.anchor = GridBagConstraints.CENTER;
		
		JLabel upLabel = getGestureImageLabel("ORN_001");
		c.gridx = 0;
		handshapesPanel.add(upLabel, c);
		
		JLabel downLabel = getGestureImageLabel("ORN_002");
		c.gridx = 1;
		handshapesPanel.add(downLabel, c);
		
		JLabel leftLabel = getGestureImageLabel("ORN_003");
		c.gridx = 2;
		handshapesPanel.add(leftLabel, c);
		
		JLabel rightLabel = getGestureImageLabel("ORN_004");
		c.gridx = 3;
		handshapesPanel.add(rightLabel, c);
		
		c.gridy = 1;
		JLabel selfLabel = getGestureImageLabel("ORN_005");
		c.gridx = 0;
		handshapesPanel.add(selfLabel, c);
		
		JLabel awaySelfLabel = getGestureImageLabel("ORN_006");
		c.gridx = 1;
		handshapesPanel.add(awaySelfLabel, c);
		
		JLabel togetherLabel = getGestureImageLabel("ORN_007");
		c.gridx = 2;
		handshapesPanel.add(togetherLabel, c);
		
		JLabel towardsLabel = getGestureImageLabel("ORN_008");
		c.gridx = 3;
		handshapesPanel.add(towardsLabel, c);
		
		JLabel angledHeadLabel = getGestureImageLabel("ORN_009");
		c.gridx = 4;
		c.gridwidth = 2;
		handshapesPanel.add(angledHeadLabel, c);
		
		c.gridy = 2;
		
		c.gridx = 4;
		c.gridy = 0;
		c.gridheight = 1;
		JLabel dniGesLabel = getGestureImageLabel("ORN_999");
		handshapesPanel.add(dniGesLabel, c);
	}
	
	private JLabel getGestureImageLabel(final String iid) {
		try {
			SS3FieldTypeImg ss3Img = defaultCodingScheme.getImage(iid);
			SS3FieldValue fieldValue = new SS3FieldValue();
			fieldValue.setId(iid);
			fieldValue.setIid(iid);
			final OrientationImage image = new OrientationImage(fieldValue, defaultCodingScheme);
			String lblText = "<html><p style=\"text-align: center;\">" + ss3Img.getLabel().replaceAll("\\\\n", "<br>") + "</p></html>";
			JLabel lbl = new JLabel(lblText, new ImageIcon(image.getImage()), SwingConstants.CENTER);
			lbl.setHorizontalTextPosition(JLabel.CENTER);
			lbl.setVerticalTextPosition(JLabel.BOTTOM);
			lbl.setSize(image.getImage().getWidth(null) + 5, image.getImage().getHeight(null) + 5);
			lbl.setToolTipText(ss3Img.getLabel());
			lbl.setHorizontalTextPosition(JLabel.CENTER);
			lbl.setVerticalTextPosition(JLabel.BOTTOM);
			lbl.addMouseListener(new MouseAdapter() {
				@Override
				public void mousePressed(MouseEvent e) {
					hsClient.addOrientation(image);
//					SS3Singleton.getApplicationStateController()
//							.setLoadedCollectionUpdated(true);
				}
			});
			return lbl;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}
}

