package edu.bu.signstream.grepresentation.fields.gestureField.dialog.handShapes;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.swing.JPanel;

import edu.bu.signstream.common.util.vo.Palette;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.grepresentation.listener.WeakFocusListener;
import edu.bu.signstream.grepresentation.listener.WeakKeyListener;
import edu.bu.signstream.grepresentation.listener.WeakMouseListener;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.GestureHandShapePanel;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.HandShapePanel;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.LettersHandshapesPalette;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapeImage;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapePalletClient;

public class SelectedGestureHandshapesPanel extends JPanel implements
		HandShapePalletClient, MouseListener, KeyListener, FocusListener {

	private static final long serialVersionUID = 1L;
	private SS3CodingScheme cs;

	private ArrayList<HandShapeImage> handshapes = new ArrayList<HandShapeImage>();
	private ArrayList<OrientationImage> orientations = new ArrayList<OrientationImage>();
	private ArrayList<HandShapeImage> formBeanImages = null;
	private ArrayList<OrientationImage> formBeanOrientationImages = new ArrayList<OrientationImage>();
	private HashMap<Rectangle, String> handshapePositionMap = new HashMap<Rectangle, String>();
	private HashMap<Rectangle, String> orientationPositionMap = new HashMap<Rectangle, String>();
	private ArrayList<Point> clickedAtPoints = new ArrayList<Point>();
	private GestureHandShapePanel gestureHandShapePanel;
	
	private LettersHandshapesPalette lettersHandshapesPallet = null;

	public SelectedGestureHandshapesPanel(SS3CodingScheme cs) {
		init(cs);
	}
	
	public SelectedGestureHandshapesPanel(ArrayList<HandShapeImage> hsImages, ArrayList<OrientationImage> orientationImages, SS3CodingScheme cs, GestureHandShapePanel gestureHandShapePanel) {
		this.gestureHandShapePanel = gestureHandShapePanel;
		handshapes.addAll(hsImages);
		orientations.addAll(orientationImages);
		formBeanImages = hsImages;
		formBeanOrientationImages = orientationImages;

		gestureHandShapePanel.selectedHandshapesPanelUpdated();
		
		init(cs);
	}
	
	public void update(ArrayList<HandShapeImage> hsImages) {
		handshapes.clear();
		handshapes.addAll(hsImages);
		repaint();
	}
	
	public void updateOrientations(ArrayList<OrientationImage> hsImages) {
		orientations.clear();
		orientations.addAll(hsImages);
		repaint();
	}
	
	public void setLettersHandshapesPallet(LettersHandshapesPalette pallet) {
		this.lettersHandshapesPallet = pallet;
	}
	
	private void init(SS3CodingScheme cs) {
		this.cs = cs;
		this.addMouseListener(new WeakMouseListener(this, this));
		this.setFocusable(true);
		this.addKeyListener(new WeakKeyListener(this, this));
		this.addFocusListener(new WeakFocusListener(this, this));
	}
	
	public void clean() {
		this.removeMouseListener(this);
		this.removeKeyListener(this);
		this.removeFocusListener(this);
		
		handshapes = null;
		orientations = null;
		formBeanImages = null;
		formBeanOrientationImages = null;
		handshapePositionMap = null;
		clickedAtPoints = null;
		gestureHandShapePanel = null;
	}

	@Override
	public void mousePressed(MouseEvent e) {
	}

	@Override
	public void mouseReleased(MouseEvent e) {
	}

	@Override
	public void mouseEntered(MouseEvent e) {
	}

	@Override
	public void mouseExited(MouseEvent e) {
	}

	@Override
	public void mouseClicked(MouseEvent e) {
		if (e.isControlDown()) {
			clickedAtPoints.add(e.getPoint());
		} else if (e.isShiftDown()) {
				clickedAtPoints.add(e.getPoint());
				if (clickedAtPoints.size() >= 2) {
					Point point1 = clickedAtPoints.get(0);
					Point point2 = clickedAtPoints.get(clickedAtPoints.size() - 1);

					if (point1.x > point2.x) {
						Point point = point2;
						point2 = point1;
						point1 = point;
					}

					int id1 = -1, id2 = -1;
					Iterator i = handshapePositionMap.entrySet().iterator();
					//ArrayList<HandShapeImage> toRemove = new ArrayList<HandShapeImage>();
					while (i.hasNext()) {
						Map.Entry me = (Map.Entry) i.next();
						Rectangle rectangle = (Rectangle) me.getKey();

						if (rectangle == null)
							return;

						if (rectangle.contains(point1)) {
							String value = (String) me.getValue();
							id1 = Integer.parseInt(value);
						}
						if (rectangle.contains(point2)) {
							String value = (String) me.getValue();
							id2 = Integer.parseInt(value);
						}
					}

					if (id1 > -1 && id2 > -1) {
						clickedAtPoints.add(point1);

						Iterator iter = handshapePositionMap.entrySet()
								.iterator();
						while (iter.hasNext()) {
							Map.Entry map = (Map.Entry) iter.next();
							String value = (String) map.getValue();
							int id = Integer.parseInt(value);

							if (id1 < id && id < id2) {
								Rectangle rectangle = (Rectangle) map.getKey();
								clickedAtPoints.add(rectangle.getLocation());
							}
						}
						clickedAtPoints.add(point2);
					} else if (id1 > -1) {
						clickedAtPoints.add(point1);
					} else if (id2 > -1) {
						clickedAtPoints.add(point2);
					}
				}
		} else {
			clickedAtPoints.clear();
			clickedAtPoints.add(e.getPoint());
		}

		this.requestFocus();
		repaint();
	}

	@Override
	public void keyTyped(KeyEvent e) {
	}

	@Override
	public void keyPressed(KeyEvent e) {
	}

	@Override
	public void keyReleased(KeyEvent e) {
		int keyCode = e.getKeyCode();
		if (keyCode == 8 || keyCode == 127 || keyCode == 110) {

			// remove selected icon
			if (clickedAtPoints.isEmpty()) {
				return;
			} else {
				ArrayList<HandShapeImage> toRemoveHs = new ArrayList<HandShapeImage>();
				ArrayList<OrientationImage> toRemoveO = new ArrayList<OrientationImage>();
				
				for (Map.Entry<Rectangle, String> me : handshapePositionMap.entrySet()) {
					Rectangle rectangle = (Rectangle) me.getKey();

					if (rectangle == null)
						return;

					for (Point point : clickedAtPoints) {
						boolean contains = rectangle.contains(point);

						if (contains) {
							String value = (String) me.getValue();
							HandShapeImage image = handshapes.get(Integer
									.parseInt(value));
							toRemoveHs.add(image);
						}
					}
				}
				
				for (Map.Entry<Rectangle, String> me : orientationPositionMap.entrySet()) {
					Rectangle rectangle = (Rectangle) me.getKey();

					if (rectangle == null)
						return;

					for (Point point : clickedAtPoints) {
						boolean contains = rectangle.contains(point);

						if (contains) {
							String value = (String) me.getValue();
							OrientationImage image = orientations.get(Integer
									.parseInt(value));
							toRemoveO.add(image);
						}
					}
				}

				for (HandShapeImage remove : toRemoveHs) {
					handshapes.remove(remove);
				}
				
				for (OrientationImage remove : toRemoveO) {
					orientations.remove(remove);
				}
				
				if(this.lettersHandshapesPallet != null) {
					lettersHandshapesPallet.synchronizeHandshapeWithTextfield(handshapes);
				}

				synchronizeWithFormBean();
				clickedAtPoints.clear();
				repaint();
			}
		}
	}

	@Override
	public void setItem(Object obj) {
		SS3FieldValue value = (SS3FieldValue) obj;
		handshapes.add(new HandShapeImage(value, cs));
		synchronizeWithFormBean();

		repaint();
		revalidate();
	}
	
	public void addOrientation(OrientationImage o) {
		orientations.add(o);
		synchronizeWithFormBean();
		repaint();
		revalidate();
	}

	@Override
	public void paint(Graphics g) {
		handshapePositionMap.clear();
		orientationPositionMap.clear();

		Graphics2D g2d = (Graphics2D) g;
		int x = 5;
		int y = 5;

		g2d.setColor(Palette.BACKGROUND_COLOR);
		g2d.fillRect(0, 0, getWidth() + 50, getHeight());

		for (int i = 0; i < handshapes.size(); i++) {

			HandShapeImage hsImage = handshapes.get(i);
			Image img = hsImage.getImage();
			int width = img.getWidth(null) + 1;
			int height = img.getHeight(null) + 1;

			g2d.setColor(Color.BLACK);
			g2d.drawRect(x - 1, y - 1, width, height);
			g2d.drawImage(img, x, y, Color.WHITE, this);

			Rectangle r = new Rectangle(x - 1, y - 1, width, height);
			handshapePositionMap.put(r, "" + i);

			for (Point clickedAtPoint : clickedAtPoints) {
				if (clickedAtPoint != null && r.contains(clickedAtPoint)) {
					g2d.setColor(Color.red);
					g2d.drawRect(x - 1, y - 1, width, height);
					g2d.setColor(Color.BLACK);
				}
			}
			x += width + 3;
		}
		
		x = 5;
		y = 40;
		
		for (int i = 0; i < orientations.size(); i++) {

			OrientationImage oImg = orientations.get(i);
			Image img = oImg.getImage();
			int width = img.getWidth(null) + 1;
			int height = img.getHeight(null) + 1;

			g2d.setColor(Color.BLACK);
			g2d.drawRect(x - 1, y - 1, width, height);
			g2d.drawImage(img, x, y, Color.WHITE, this);

			Rectangle r = new Rectangle(x - 1, y - 1, width, height);
			orientationPositionMap.put(r,  "" + i);

			for (Point clickedAtPoint : clickedAtPoints) {
				if (clickedAtPoint != null && r.contains(clickedAtPoint)) {
					g2d.setColor(Color.red);
					g2d.drawRect(x - 1, y - 1, width, height);
					g2d.setColor(Color.BLACK);
				}
			}
			x += width + 3;
		}

		Dimension d = new Dimension(x, (int) getPreferredSize().getHeight());
		setPreferredSize(d);
		setMinimumSize(d);
		setMaximumSize(d);
	}

	@Override
	public void removeAll() {
		handshapes.removeAll(handshapes);
		orientations.removeAll(orientations);
		synchronizeWithFormBean();
		repaint();
		revalidate();
	}

	public ArrayList<HandShapeImage> getHandshapes() {
		return handshapes;
	}
	
	public ArrayList<OrientationImage> getOrientations() {
		return orientations;
	}

	public void clearHandshapes() {
		this.handshapes.clear();
		synchronizeWithFormBean();
	}

	public void setHandshapes(ArrayList<HandShapeImage> handshapes) {
		this.handshapes.clear();
		this.handshapes.addAll(handshapes);
        synchronizeWithFormBean();
	}
	
	public void setHandshapesAndOrientations(ArrayList<HandShapeImage> handshapes, ArrayList<OrientationImage> orientations) {
		this.handshapes.clear();
		this.handshapes.addAll(handshapes);
		this.orientations.clear();
		this.orientations.addAll(orientations);
        synchronizeWithFormBean();
	}
	
	public void setOrientations(ArrayList<OrientationImage> orientations) {
		this.orientations.clear();
		this.orientations.addAll(orientations);
        synchronizeWithFormBean();
	}

	public int getNumberOfHandShapes() {
		return handshapes.size();
	}

	private void synchronizeWithFormBean() {
		formBeanImages.clear();
		formBeanImages.addAll(handshapes);
		formBeanOrientationImages.clear();
		formBeanOrientationImages.addAll(orientations);
		if (gestureHandShapePanel != null) {
			gestureHandShapePanel.selectedHandshapesPanelUpdated();
		}
    }

	@Override
	public void setItem(Object obj, boolean bothHands) {
	}
	
	@Override
	public void focusGained(FocusEvent e) {			
	}
	
	@Override
	public void focusLost(FocusEvent e) {
		clickedAtPoints.clear();
		repaint();
		revalidate();
	}
}
