package edu.bu.signstream.grepresentation.fields.gestureField.locationField;

import edu.bu.signstream.time.Interval;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.time.TimeRange;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations.Region;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

import java.awt.*;

public class GestureLocationEvent implements Event {

	private String id;

	private SignStreamSegmentPanel segmentPanel;

	private GestureLocationType locationType;
	private Region location;

	private Time startTimeInfo;
	private Time endTimeInfo;

	public GestureLocationEvent(SignStreamSegmentPanel segmentPanel) {
		this(segmentPanel, null, Region.NONE);
	}

	public GestureLocationEvent(SignStreamSegmentPanel segmentPanel, GestureLocationType locationType, Region location) {
		this.segmentPanel = segmentPanel;
		this.locationType = locationType;
		this.location = location;
	}

	public boolean isNoneLocation() {
		return location.equals(Region.NONE);
	}

	@Override
	public Time getStartTimeInfo() {
		return startTimeInfo;
	}

	@Override
	public Time getEndTimeInfo() {
		return endTimeInfo;
	}

    @Override
    public Interval interval() {
        return new Interval(startTimeInfo, endTimeInfo);
    }

	@Override
	public void setStartTimeInfo(Time startTimeInfo) {
		this.startTimeInfo = startTimeInfo;
		this.endTimeInfo = startTimeInfo;
	}

	@Override
	public void setEndTimeInfo(Time endTimeInfo) {
		this.endTimeInfo = endTimeInfo;
		this.startTimeInfo = endTimeInfo;
	}

	@Override
	public double getStartTimeCoordinate() {
		return segmentPanel.getZoomer().getCoordinateTimeConvertor()
				.convertTimeInfoToCoordinate(startTimeInfo);
	}

	@Override
	public double getEndTimeCoordinate() {
		return segmentPanel.getZoomer().getCoordinateTimeConvertor()
				.convertTimeInfoToCoordinate(endTimeInfo);
	}

	@Override
	public void setStartTimeCoordinate(double coord) {
	    startTimeInfo = TimeRange.ABSOLUTE.time(Math.round(coord), TimeUnit.COORDINATE)
            .in(SS3Singleton.getVideoTimeRange());
	}

	@Override
	public void setEndTimeCoordinate(double coord) {
        endTimeInfo = TimeRange.ABSOLUTE.time(Math.round(coord), TimeUnit.COORDINATE)
            .in(SS3Singleton.getVideoTimeRange());
    }

    public String getText() {
        return location.getAbbreviation();
	}

	@Override
	public String getToolTipText() {
		return location.getName();
	}

	@Override
	public boolean select() {
		return false;
	}

	@Override
	public boolean unselect() {
		return false;
	}

	@Override
	public void highlight() {}

	@Override
	public void unhighlight() {}

	@Override
	public boolean isSelected() {
		return false;
	}

	@Override
	public boolean isHighlighted() {
		return false;
	}

	@Override
	public void setColor(Color color) {}

	@Override
	public Color getColor() { return null; }

	@Override
	public String getID() { return id; }

	@Override
	public void setID(String id) {
		this.id = id;
	}

	public void setSS3Item(SS3Item item) {
		setStartTimeInfo(item.getStartTimeInfo());
		setEndTimeInfo(item.getEndTimeInfo());
		locationType = new GestureLocationType(SS3Singleton.getLocationConfig().getLocationType(item.getLocationTypeID()));
		location = SS3Singleton.getLocationConfig().getRegion(item.getLocationRegionID());
	}

	public GestureLocationType getLocationType() {
		return locationType;
	}

	public Region getGestureLocation() {
		return location;
	}
}
