package edu.bu.signstream.grepresentation.fields.gestureField.mouseDelegateListeners;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.logging.Level;

import javax.swing.*;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;

import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.FieldWrappersCollection;
import edu.bu.signstream.grepresentation.fields.apexField.ApexChainedApexEventsEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureEventsEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.GestureInfoDialog;
import edu.bu.signstream.grepresentation.fields.gestureField.handshapes.GestureHandShapeEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.locationField.GestureLocationEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.signTypeField.GestureTypeEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.orientationField.GestureOrientationEntity;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.LabelObject;

public class DeleteGestureActionListener implements ActionListener {

    /**
     * Selected chained events entity
     */
    private GestureChainedEventsEntity entity = null;

    /**
     * Sets selected chained events entity.
     * @param entity selected chained events entity
     */
    public void setChainedEventsEntity(GestureChainedEventsEntity entity) {
        this.entity = entity;
    }

    @Override
	public void actionPerformed(ActionEvent e) {

        GestureInfoDialog dialog = SS3Singleton
                .getGestureFieldDialogs().getGestureInfoDialog();
        if (dialog != null && dialog.isVisible() &&
            (
                entity.equals(dialog.getCurrentGestureEntity()) ||
                entity.equals(dialog.getCurrentGestureEntity().getDependentGestureEntity())
            )) {
            dialog.discardChanges();
        }

        var app = SS3Singleton.getSignStreamApplication();

        final JPopupMenu popup = new JPopupMenu();

        JLabel msgLabel = new JLabel("Are you sure you want to delete selected entity?");
        UserInterfaceUtil.setLabelLNF(msgLabel);

        final JButton okBtn = new JButton("Proceed");
        final JButton cancelBtn = new JButton("Cancel");

        JPanel msgPanel = new JPanel();
        GridBagConstraints gbc = new GridBagConstraints();

        msgPanel.setLayout(new GridBagLayout());
        gbc.anchor = GridBagConstraints.WEST;

        gbc.gridwidth = 3;
        gbc.gridx = 0;
        gbc.gridy = 0;
        msgPanel.add(msgLabel, gbc);

        gbc.gridy = 1;
        gbc.gridx = 1;
        msgPanel.add(new JLabel(" "), gbc);

        gbc.gridwidth = 1;
        gbc.anchor = GridBagConstraints.EAST;
        gbc.gridy = 3;
        gbc.gridx = 0;
        msgPanel.add(new JLabel(" "), gbc);
        gbc.gridx = 1;
        msgPanel.add(okBtn, gbc);
        gbc.gridx = 2;
        msgPanel.add(cancelBtn, gbc);

        cancelBtn.addActionListener(new ActionListener() {
            @Override
			public void actionPerformed(ActionEvent e) {
                popup.setVisible(false);
                app.unblock();
            }
        });

        okBtn.addActionListener(new ActionListener() {
            @Override
			public void actionPerformed(ActionEvent e) {
                try {
                    entity.getField().deleteEntity(entity);

                    if (entity instanceof GestureEventsEntity) {
                        GestureEventsEntity glossChainedEventsEntity = (GestureEventsEntity) entity;
                        deleteGestureEventsEntity(glossChainedEventsEntity);

                        GestureEventsEntity depGlossChainedEventsEntity = glossChainedEventsEntity.getDependentGestureEntity();
                        if (depGlossChainedEventsEntity != null) {
                        	depGlossChainedEventsEntity.getField().deleteEntity(depGlossChainedEventsEntity);
                            deleteGestureEventsEntity(depGlossChainedEventsEntity);
                        }
                    }

                    SS3Singleton.getMediaToolBar(null).selectedField().repaint();
                    SS3Singleton.getSignStreamApplication().repaint();
                    
                    SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);	// calls SSSP.updateLoadedDatabase();
					
                    // refresh the utterance lists to reflect the absence of this gloss now
					SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().getUtteranceView().updateUtteranceLists();
                } catch (NullPointerException ex) {
                    // hide the popup and log the exception.
                	ex.printStackTrace();
                    SS3Singleton.getSignStreamApplication().logger.log(Level.WARNING, ex.getMessage());
                    popup.setVisible(false);
                }
                
                SS3Singleton.getMediaToolBar().segmentPanel().getUtteranceView().getSegmentTemplate(
                	entity.getField().getSegmentID()).getNonmanView()
                	.getCSFieldDialog().getCSFieldPanel().showAllFieldsCurrentUtterance();

                popup.setVisible(false);
                app.unblock();
            }
        });

        popup.addPopupMenuListener(new PopupMenuListener() {
            @Override public void popupMenuWillBecomeInvisible(PopupMenuEvent e) {
                app.unblock();
            }

            @Override public void popupMenuCanceled(PopupMenuEvent e) { }
            @Override public void popupMenuWillBecomeVisible(PopupMenuEvent e) { }
        });

        int width = msgPanel.getPreferredSize().width + 30;
        int height = msgPanel.getPreferredSize().height + 20;

        popup.add(msgPanel);
        popup.setPopupSize(new Dimension(width, height));
        popup.setLocation(250, 250);
        popup.setVisible(true);
        SS3Singleton.getSignStreamApplication().forcePopupInvisibleOnWindowLostFocus(popup);

        SwingUtilities.invokeLater(okBtn::requestFocus);

        app.block();
    }

    /**
     * Deletes GestureEventsEntity and if available corresponding entities from Hand Field and from Hand Shape Field.
     *
     * @param gestureChainedEventsEntity to delete
     * @param fieldID                  ID of entity's field, in current implementation it is 10000 or 10001
     * @param fieldWrappersCollection  a collection of loaded FieldWrappers
     */
    private void deleteGestureEventsEntity(GestureEventsEntity gestureChainedEventsEntity) {

    	ApexChainedApexEventsEntity dependentApexEntity = gestureChainedEventsEntity.getDependentApexEntity();
        GestureChainedEventsEntity dependentHandEntity = gestureChainedEventsEntity.getDependentHandEntity();
        GestureHandShapeEntity dependentHandShapeEntity = gestureChainedEventsEntity.getDependentHandShapeEntity();
        GestureTypeEntity dependentGestureTypeEntity = gestureChainedEventsEntity.getDependentGestureTypeEntity();
        GestureOrientationEntity dependentOrientationEntity = gestureChainedEventsEntity.getDependentOrientationEntity();
        GestureLocationEntity dependentLocationEntity = gestureChainedEventsEntity.getDependentLocationEntity();
        
        if (gestureChainedEventsEntity.isStartCompound()) {
        	gestureChainedEventsEntity.getPreviousEntity().setEndCompound(null);
        	gestureChainedEventsEntity.setStartCompound(null);
        }
        
        if (gestureChainedEventsEntity.isEndCompound()) {
        	gestureChainedEventsEntity.getNextEntity().setStartCompound(null);
        	gestureChainedEventsEntity.setEndCompound(null);
        }

        if (dependentApexEntity != null)
        	dependentApexEntity.getField().deleteEntity(dependentApexEntity);

        if (dependentHandEntity != null)
        	dependentHandEntity.getField().deleteEntity(dependentHandEntity);
        
        if (dependentHandShapeEntity != null)
        	dependentHandShapeEntity.getField().deleteEntity(dependentHandShapeEntity);

        if (dependentOrientationEntity != null)
        	dependentOrientationEntity.getField().deleteEntity(dependentOrientationEntity);

        if (dependentLocationEntity != null)
        	dependentLocationEntity.getField().deleteEntity(dependentLocationEntity);

        if (dependentGestureTypeEntity != null)
        	dependentGestureTypeEntity.getField().deleteEntity(dependentGestureTypeEntity);
    }
}
