package edu.bu.signstream.grepresentation.fields.gestureField.mouseDelegateListeners;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.time.Bound;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.apexField.ApexChainedApexEventsEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureChainedEvent;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureEventsEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureField;
import edu.bu.signstream.grepresentation.fields.gestureField.handshapes.GestureHandShapeEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.orientationField.GestureOrientationEntity;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolbarCommons;

public class LockUnlockGestureDomNdomHandAlignmentsListener implements ActionListener {

	@Override
	public void actionPerformed(ActionEvent e) {
		
		GestureEventsEntity selectedEntity  = (GestureEventsEntity)SS3Singleton.getMediaToolBar(null).selectedEntity();
		
		boolean locked = !selectedEntity.isLockedDomNdomAlignments();
		GestureEventsEntity ndEntity = selectedEntity
				.getDependentGestureEntity();

		// GD: ???
		// if we are at this point, both entities are guaranteed to exist, so enforce that they both are aware of the other hand
		selectedEntity.getGestureEntityProperties().setTwoHanded(true);
		ndEntity.getGestureEntityProperties().setTwoHanded(true);

		// move to a locked state
		if (locked) {
			// check if possible to perform the action
			GestureField ndField = (GestureField) ndEntity.getField();
			ArrayList<GestureEventsEntity> ndEntities = ndField.getEntitiesInInterval(selectedEntity.interval(), Bound.EXCLUSIVE);
			if (!ndEntities.isEmpty()) {
				boolean cannotLock = false;
				for (GestureEventsEntity entity: ndEntities) {
					if (entity != null && !entity.equals(ndEntity)) {
						cannotLock = true;
					}
				}
				
				// warning!!!
				if (cannotLock) {
					SS3Singleton.getErrorMessages().showErrorMessage(
							ErrorMessages.CANT_LOCK_GESTURES);
					return;
				}

				// check for apex on opposite hand
				Event value = selectedEntity.getValueOrTextValueEvent();
				ApexChainedApexEventsEntity ndApexEntity = ndEntity.getDependentApexEntity();
				if (value != null && ndApexEntity != null && 
						! value.interval().contains(ndApexEntity.interval(), Bound.INCLUSIVE)) {
					SS3Singleton.getErrorMessages().showErrorMessage(
							ErrorMessages.GESTURE_APEX_BOUNDS);
					return;
				}
			}
			
			// all good to proceed, do the update by setting the times
			update(selectedEntity, ndEntity);
		} 

		ndEntity.setLockedDomNdomAlignments(locked);
		selectedEntity.setLockedDomNdomAlignments(locked);
		SS3Singleton.getSignStreamApplication()
				.getSignStreamSegmentPanel().repaint();
	}

	private void update(GestureEventsEntity selectedEntity,
			GestureEventsEntity ndEntity) {

		GestureChainedEvent value = selectedEntity.getTextValue();
		GestureChainedEvent preparation = selectedEntity.getPreparation();
		GestureChainedEvent prestrokeHold = selectedEntity.getPrestrokeHold();
		GestureChainedEvent retraction = selectedEntity.getRetraction();
		GestureChainedEvent poststrokeHold = selectedEntity.getPoststrokeHold();
		GestureChainedEvent relaxed = selectedEntity.getRelaxed();

		GestureChainedEvent ndValue = ndEntity.getTextValue();
		GestureChainedEvent ndPreparation = ndEntity.getPreparation();
		GestureChainedEvent ndPrestrokeHold = ndEntity.getPrestrokeHold();
		GestureChainedEvent ndRetraction = ndEntity.getRetraction();
		GestureChainedEvent ndPoststrokeHold = ndEntity.getPoststrokeHold();
		GestureChainedEvent ndRelaxed = ndEntity.getRelaxed();

		ndValue.setStartTimeInfo(value.getStartTimeInfo());
        ndValue.setEndTimeInfo(value.getEndTimeInfo());

        // order matters for the below
        if (prestrokeHold != null) {
        	if (ndPrestrokeHold == null) {
        		ndEntity.addPrestrokeHoldEvent();
        		ndPrestrokeHold = ndEntity.getPrestrokeHold();
        	}
        	ndPrestrokeHold.setStartTimeInfo(prestrokeHold.getStartTimeInfo());
        	ndPrestrokeHold.setEndTimeInfo(prestrokeHold.getEndTimeInfo());
        } else {
        	ndEntity.deletePoststrokeHold();
        }
        
        if (preparation != null) {
        	if (ndPreparation == null) {
        		ndEntity.addPreparationEvent();
        		ndPreparation = ndEntity.getPreparation();
        	}
        	ndPreparation.setStartTimeInfo(preparation.getStartTimeInfo());
        	ndPreparation.setEndTimeInfo(preparation.getEndTimeInfo());
        } else {
        	ndEntity.deletePreparation();
        }

        if (poststrokeHold != null) {
        	if (ndPoststrokeHold == null) {
        		ndEntity.addPoststrokeHoldEvent();
        		ndPoststrokeHold = ndEntity.getPoststrokeHold();
        	}
        	ndPoststrokeHold.setStartTimeInfo(poststrokeHold.getStartTimeInfo());
        	ndPoststrokeHold.setEndTimeInfo(poststrokeHold.getEndTimeInfo());
        } else {
        	ndEntity.deletePoststrokeHold();
        }

        if (retraction != null) {
        	if (ndRetraction == null) {
        		ndEntity.addRetractionEvent();
        		ndRetraction = ndEntity.getRetraction();
        	}
        	ndRetraction.setStartTimeInfo(retraction.getStartTimeInfo());
        	ndRetraction.setEndTimeInfo(retraction.getEndTimeInfo());
        } else {
        	ndEntity.deleteRetraction();
        }

        if (relaxed != null) {
        	if (ndRelaxed == null) {
        		ndEntity.addRelaxedEvent();
        		ndRelaxed = ndEntity.getRelaxed();
        	}
        	ndRelaxed.setStartTimeInfo(relaxed.getStartTimeInfo());
        	ndRelaxed.setEndTimeInfo(relaxed.getEndTimeInfo());
        } else {
        	ndEntity.deleteRelaxed();
        }

		if (ndEntity.isLockedGesturePhonAlignments()) {
			GestureChainedEventsEntity depHandEntity = ndEntity
					.getDependentHandEntity();
			GestureHandShapeEntity depHandShapeEntity = ndEntity
					.getDependentHandShapeEntity();
			GestureOrientationEntity depOrientationEntity = ndEntity
					.getDependentOrientationEntity();

			Time startTimeInfo = ndEntity.getValueOrTextValueEvent().getStartTimeInfo();
			Time endTimeInfo = ndEntity.getValueOrTextValueEvent().getEndTimeInfo();
			
			// NOTE: we are not setting start/end times according to onsets/offsets as the handshapes do not track to those times
			// nor to the initial/final holds - only lock hand/handshape/orientaiton entities to the main stroke event
			if (depHandEntity != null) {
				depHandEntity.getValueOrTextValueEvent().setStartTimeInfo(startTimeInfo);
				depHandEntity.getValueOrTextValueEvent().setEndTimeInfo(endTimeInfo);

				if (depHandShapeEntity != null) {
					MediaToolbarCommons.setGestureHandshapeStartTime(depHandShapeEntity, startTimeInfo);
					MediaToolbarCommons.setGestureHandshapeEndTime(depHandShapeEntity, endTimeInfo);
				}

				if (depOrientationEntity != null) {
					MediaToolbarCommons.setGestureOrientationStartTime(depOrientationEntity, startTimeInfo);
					MediaToolbarCommons.setGestureOrientationEndTime(depOrientationEntity, endTimeInfo);
				}
			}
		}
	}
}
