package edu.bu.signstream.grepresentation.fields.gestureField.orientationField;

import java.awt.Font;
import java.awt.Graphics2D;
import java.util.ArrayList;

import edu.bu.signstream.time.Time;
import edu.bu.signstream.common.util.vo.TrackValue;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class GestureOrientationSubEntity extends EventsEntity {

	// collection of hand shapes
	private GestureOrientationsManager manager = new GestureOrientationsManager(segmentPanel);
	//private Color lineColor = Color.GRAY;

	public GestureOrientationSubEntity(SignStreamSegmentPanel segmentPanel, Field field) {
		super(segmentPanel, field);
	}

	@Override
	public SS3Entity getSS3Entity() {
		SS3Entity entity = new SS3Entity();
		if (getParentEntities() != null)
			entity.setParentEntities(getParentEntities());
        ArrayList<GestureOrientationEvent> events = manager.getOrientationsCollection();

		for (GestureOrientationEvent event : events) {
			if (event != null) {
				SS3Item item = new SS3Item();
				item.setId(event.getID());
				item.setStartTimeInfo(event.getStartTimeInfo());
				item.setEndTimeInfo(event.getEndTimeInfo());
				item.setText(event.getText());
				entity.addItem(item);
			}
		}
		return entity;
	}

	public GestureOrientationsManager getGestureOrientationsManager() {
		return manager;
	}

	@Override
	public boolean isSelected() {
		return false;
	}

	@Override
	public boolean isOverlapping(double coordx1, double coordx2) {
		double startCoord = getStartTimeCoordinate();
		double endCoord = getEndTimeCoordinate();

		return (coordx1 <= startCoord && coordx2 >= startCoord)
			|| (coordx1 <= endCoord && coordx2 >= endCoord);
	}

	@Override
	public double getStartTimeCoordinate() {
        ArrayList<GestureOrientationEvent> events = manager.getOrientationsCollection();

        if(events.size() < 1)
            return 0;

        double startTimeCoord = events.get(0).getStartTimeCoordinate();
		
		for (GestureOrientationEvent event : events) {
			startTimeCoord =
				startTimeCoord < event.getStartTimeCoordinate()
					? startTimeCoord
					: event.getStartTimeCoordinate();
		}
		return startTimeCoord;
	}

	@Override
	public Event getFirstEvent() {
		return manager.getFirstEvent();
	}

	@Override
	public Event getLastEvent() {
		return manager.getLastEvent();
	}

	@Override
	public double getEndTimeCoordinate() {
        ArrayList<GestureOrientationEvent> events = manager.getOrientationsCollection();

		double endTimeCoord = 0;
		for (GestureOrientationEvent event : events) {
			endTimeCoord =
				endTimeCoord > event.getStartTimeCoordinate()
					? endTimeCoord
					: event.getStartTimeCoordinate();
		}
		return endTimeCoord;
	}

	public void addOrientation(SS3Item item) {
		manager.addOrientation(item);
	}

	public void addOrientation(GestureOrientationEvent event) {
		manager.addOrientation(event);
	}

	@Deprecated
	public void addOrientation(TrackValue value) {
		manager.addOrientation(value);
	}

	@Deprecated
	public void setStartTime(int startTime) {
		Event event = manager.getFirstEvent();
        var time = SS3Singleton.getVideoTimeRange().time(startTime, TimeUnit.TIMESTAMP);
        event.setStartTimeInfo(time);
        event.setEndTimeInfo(time);
	}

	@Deprecated
	public void setEndTime(int endTime) {
		Event event = manager.getLastEvent();
        var time = SS3Singleton.getVideoTimeRange().time(endTime, TimeUnit.TIMESTAMP);
        event.setStartTimeInfo(time);
        event.setEndTimeInfo(time);
	}

	@Deprecated
	public double getEndTime() {
		return manager.getLastEvent().getEndTimeInfo().doubleValue();
	}

	@Deprecated
	public double getStartTime() {
		return manager.getFirstEvent().getStartTimeInfo().doubleValue();
	}

	@Override
	public boolean selectEntityIfEventSelected() {
		if (manager.getSelectedEvent() != null)
			select();
		return selected;
	}

	@Override
	public boolean selectEventIfEntitySelected() {
        return selected && manager.getFirstEvent().select();
    }

	@Override
	public boolean select() {
		//manager.highlightEntity(Palette.HIGHLIGHTED_ENTITY_COLOR);

		//QTMultiMovieController qtControl =
		// SSSingleton.getMultiMovieController();
		//qtControl.setStartTime((int) manager.getFirstEvent().getStartTime());
		//qtControl.setEndTime((int) manager.getLastEvent().getEndTime());
		selected = true;
     // segmentPanel.getToolBarsPanel().getMediaToolBar().enable();
		return selected;
	}

	@Override
	public boolean unselect() {
		selected = false;
		//mToolBar.setEnable(false);
		manager.unselectEntity();
		return selected;
	}

	@Override
	public boolean adjustEntity(int x1, int x2) {
		return manager.adjustEntity(x1, x2);
	}

	@Override
	public Event getSelectedEvent() {
		return manager.getSelectedEvent();
	}

	@Override
	public void reset() {
		manager.resetAll();
	}

	@Override
	public boolean delete() {
		manager.deleteEntity();
		return true;
	}

	@Override
	public Event getEventAt(double x) {
		return manager.getEventAt(x);
	}

	@Override
	public ArrayList getEventsToAdd() {
		ArrayList list = new ArrayList();
		list.add("Add Orientation");
		return list;
	}

	@Override
	public String getToolTipText(int x) {
		GestureOrientationEvent event = (GestureOrientationEvent) manager.getEventAt(x);

		if (event == null)
			return "";

		StringBuilder toolTipText = new StringBuilder();
		toolTipText.append(event.getText());

		if (event.equals(getFirstEvent())) {
			toolTipText.append(", starts at ");
		} else if (event.equals(getLastEvent())) {
			toolTipText.append(", ends at ");
		} else {
			toolTipText.append(", at ");
		}

		toolTipText.append(event.getStartTimeInfo().doubleValue());

		return toolTipText.toString();
	}

	@Override
	public boolean selectEvent(double x) {
		Event event = manager.getEventAt(x);
		if (event != null)
			return event.select();
		return false;
	}

	public ArrayList<GestureOrientationEvent> getEvents() {
		return manager.getOrientationsCollection();
	}

	@Override
	public void paint(Graphics2D g2d, double Y, Font font) {
	}


   @Override
public Time getStartTimeInfo() {
      return manager.getFirstEvent().getStartTimeInfo();
   }

   @Override
public Time getEndTimeInfo() {
      return manager.getLastEvent().getEndTimeInfo();
   }

   @Override
public void setStartTimeInfo(Time startTimeInfo) {
      manager.getFirstEvent().setStartTimeInfo(startTimeInfo);
      
   }

   @Override
public void setEndTimeInfo(Time endTimeInfo) {
      manager.getLastEvent().setEndTimeInfo(endTimeInfo);
   }

@Override
public ArrayList<Event> getEventsList() {
	   return new ArrayList<Event>(getEvents());
}
}
