package edu.bu.signstream.grepresentation.fields.gestureField.signTypeField;

import edu.bu.signstream.time.Time;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureEventsEntity;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

import java.awt.*;
import java.util.ArrayList;
import java.util.Collection;

public class GestureTypeEntity extends EventsEntity {

    private GestureTypeEvent event;
    private ArrayList<GestureType> selectedTypes;
    private ArrayList<GestureTypeEntity> dependentGestureTypeEntities;
    
    private GestureEventsEntity refDomGestureEntity;

   // use constructor then call setSelectedTypes() 
    public GestureTypeEntity(GestureTypeEvent event, SignStreamSegmentPanel segmentPanel, Field field) {
        super(segmentPanel, field);
        this.event = event;
        this.selectedTypes = new ArrayList<GestureType>();
        this.dependentGestureTypeEntities = new ArrayList<GestureTypeEntity>();
    }

    public void setRefGestureEntity(GestureEventsEntity refDomGlossEntity) {
        this.refDomGestureEntity = refDomGlossEntity;
        this.setRefEntityId(refDomGlossEntity.getID());
    }

    public void setSelectedTypes(ArrayList<GestureType> selectedTypes) {
    	if (selectedTypes == null)
    		this.selectedTypes = new ArrayList<GestureType>();
    	else
    		this.selectedTypes = selectedTypes;
    	
    	getSS3Entity();		// this will generate all the SS3Item's needed to represent each GestureType
    }
    
    public ArrayList<GestureType> getSelectedTypes() {
    	return this.selectedTypes;
    }
    public void addDependentGestureTypeEntity(GestureTypeEntity ent) {
    	this.dependentGestureTypeEntities.add(ent);
    }
    public void setDependentGestureTypeEntities(ArrayList<GestureTypeEntity> entities) {
    	this.dependentGestureTypeEntities = new ArrayList<GestureTypeEntity>(entities);
    }
    public ArrayList<GestureTypeEntity> getDependentGestureTypeEntities() {
    	return this.dependentGestureTypeEntities;
    }
    @Override
    public SS3Entity getSS3Entity() {
        SS3Entity ss3entity = new SS3Entity();

//      ss3entity.addItem(new SS3Item(getEvent()));
        for (GestureType type: selectedTypes) {
        	SS3Item typeItem = new SS3Item(type.getId(), getStartTimeInfo());
        	typeItem.setEndTimeInfo(getEndTimeInfo());
        	ss3entity.addItem(typeItem);
        }
        
        ss3entity.setID(getID());
		if (getParentEntities() != null)
			ss3entity.setParentEntities(getParentEntities());
		ss3entity.setRefEntityId(getRefEntityId());
        return ss3entity;
    }
    
    public void setSS3Entity(SS3Entity ss3Entity) {
    	for (SS3Item item: ss3Entity.getItemsAsList()) {
    		String gestureTypeID = item.getText();
    		Collection<GestureCategory> gestureCategories 
    			= segmentPanel.getSS3Database().getSS3CodingScheme()
    			.getGestureTypeObject().getGestureCategories().values();
    		for (GestureCategory gc: gestureCategories) {
    			GestureType gt = gc.getGestureTypes().get(gestureTypeID);
    			if (gt != null) {
    				selectedTypes.add(gt);
    			}
    		}
    	}
    }

    public GestureTypeEvent getEvent() {
        return event;
    }

    @Override
    public Event getFirstEvent() {
        return event;
    }

    @Override
    public Event getLastEvent() {
        return event;
    }

    @Override
    public boolean selectEntityIfEventSelected() {
        return false;
    }

    @Override
    public boolean selectEventIfEntitySelected() {
        return false;
    }

    @Override
    public Time getStartTimeInfo() {
        return event.getStartTimeInfo();
    }

    @Override
    public Time getEndTimeInfo() {
        return event.getEndTimeInfo();
    }

    @Override
    public void setStartTimeInfo(Time startTimeInfo) {
        event.setStartTimeInfo(startTimeInfo);
    }

    @Override
    public void setEndTimeInfo(Time endTimeInfo) {
        event.setEndTimeInfo(endTimeInfo);
    }

    public void updateTimes() {
        if (refDomGestureEntity != null) {
        	Time startTimeInfo = refDomGestureEntity.getValueOrTextValueEvent().getStartTimeInfo();
        	Time endTimeInfo = refDomGestureEntity.getValueOrTextValueEvent().getEndTimeInfo();
        	
            setStartTimeInfo(startTimeInfo);
            setEndTimeInfo(endTimeInfo);
            
            if (dependentGestureTypeEntities != null && !dependentGestureTypeEntities.isEmpty()) {
            	for (GestureTypeEntity depEntity: dependentGestureTypeEntities) {
                    depEntity.setStartTimeInfo(startTimeInfo);
                    depEntity.setEndTimeInfo(endTimeInfo);
            	}
            }
        }
    }

    @Override
    public boolean select() {
        return false;
    }

    @Override
    public boolean unselect() {
        return false;
    }

    @Override
    public boolean adjustEntity(int x1, int x2) {
        return false;
    }

    @Override
    public Event getSelectedEvent() {
        return null;
    }

    @Override
    public void reset() {

    }

    @Override
    public boolean delete() {
        return false;
    }

    @Override
    public Event getEventAt(double x) {
        return null;
    }

    @Override
    public ArrayList<String> getEventsToAdd() {
        return null;
    }

    @Override
    public String getToolTipText(int x) {
        return null;
    }

    @Override
    public boolean selectEvent(double x) {
        return false;
    }

    @Override
    public void paint(Graphics2D g2d, double Y, Font font) {
        int y = (int) Y;
        g2d.setFont(font);
        g2d.setColor(Color.BLACK);

        // checking dependents are empty to just not paint the parent entity
        if (selectedTypes != null && !selectedTypes.isEmpty() && dependentGestureTypeEntities.isEmpty()) {
            String gestureTypeLabel = "";
            for (GestureType gt : selectedTypes) {
            	gestureTypeLabel = gestureTypeLabel + " " + gt.getName();
            }
            double x1 = event.getStartTimeCoordinate();
            double x2 = event.getEndTimeCoordinate();
            int strWidth = g2d.getFontMetrics().stringWidth(gestureTypeLabel);

            g2d.drawString(gestureTypeLabel, (int) ((x1 + x2 - strWidth) / 2), y);
            g2d.drawLine((int)x1, y + 1, (int) x2, y + 1);
            g2d.drawLine((int)x1, y + 2, (int) x2, y + 2);
            g2d.drawLine((int)x1, y - 1, (int) x1, y + 2);
            g2d.drawLine((int)x2, y - 1, (int) x2, y + 2);
        }
    }

	@Override
	public ArrayList<Event> getEventsList() {
		ArrayList<Event> events = new ArrayList<Event>();
		events.add(getEvent());		// only one event in a SignTypeEntity
		return events;
	}
	
	@Override
	public boolean equals(Object o) {
		GestureTypeEntity other;
		if (o instanceof GestureTypeEntity)
			other =  (GestureTypeEntity) o;
		else
			return false;
		
		return (getSelectedTypes().equals(other.getSelectedTypes()))
				&& (getDependentGestureTypeEntities().equals(other.getDependentGestureTypeEntities()));
	}
}
