package edu.bu.signstream.grepresentation.fields.glossField;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;

import edu.bu.signstream.common.util.CoordinateTimeConverter;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.TrackValue;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.fields.CollectionManager;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEvent;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsCollectionManager;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.ui.ConfirmationPanel;

/**
 * Manages collection of ChainedEventsEntity objects.
 */
public class GlossChainedEventsCollectionManager extends ChainedEventsCollectionManager {

	/**
	 * collection of ChainedEventsEntity objects to manage
	 */
	private SignStreamSegmentPanel segmentPanel = null;
	private CoordinateTimeConverter convertor = null;

	/**
	 * 
	 * @param collection
	 *            must not be null
	 */
	public GlossChainedEventsCollectionManager(GlossChainedEventsCollection collection,
			SignStreamSegmentPanel segmentPanel) {
		super(collection, segmentPanel);
		this.segmentPanel = segmentPanel;

		this.convertor = segmentPanel.getZoomer().getCoordinateTimeConvertor();
	}


	// add onset, one frame long approximately 1/30 second
	@Override
	public void addOnsetEvent() {
		ChainedEvent event = collection.getInitialHoldEvent();
		if (event == null)
			event = collection.getValueOrTextValueEvent();

		try {
			String id = "" + Constants.GLOSS_ONSET_CODE;
			
			Time endTimeInfo = event.getStartTimeInfo();
			Time startTimeInfo = segmentPanel.getMultipleMovieController().getPrecedingFrameTimeInfo(endTimeInfo);
			
			TrackValue newValue = new TrackValue(id, "", startTimeInfo, endTimeInfo);
			collection.setOnsetEvent(new ChainedEvent(newValue, segmentPanel));
		} catch (NullPointerException ex) {
			// @TODO:
			// "Not valid argument Exception will be thrown!!! ChainedEventsCollectionManager");
		}
	}

	@Override
	public void addOffsetEvent() {
		ChainedEvent event = collection.getFinalHoldEvent();
		if (event == null)
			event = collection.getValueOrTextValueEvent();

		try {
			String id = "" + Constants.GLOSS_OFFSET_CODE;
			
			Time startTimeInfo = event.getEndTimeInfo();
			Time endTimeInfo = segmentPanel.getMultipleMovieController().getFollowingFrameTimeInfo(startTimeInfo);
			
			TrackValue newValue = new TrackValue(id, "", startTimeInfo, endTimeInfo);
			collection.setOffsetEvent(new ChainedEvent(newValue, segmentPanel));
		} catch (NullPointerException ex) {
			// @TODO:
			// "Not valid argument Exception will be thrown!!! ChainedEventsCollectionManager");
		}
	}

	@Override
	public void addInitialHoldEvent() {
		ChainedEvent onset = collection.getOnsetEvent();
		ChainedEvent event = collection.getValueOrTextValueEvent();

		try {
			String id = "" + Constants.GLOSS_INITIAL_HOLD_CODE;
			
			Time endTimeInfo = event.getStartTimeInfo();
			Time startTimeInfo = segmentPanel.getMultipleMovieController().getPrecedingFrameTimeInfo(endTimeInfo);
			
			TrackValue newValue = new TrackValue(id, "", startTimeInfo, endTimeInfo);
			collection.setInitialHoldEvent(new ChainedEvent(newValue, segmentPanel));

			if (onset != null) {
				onset.setEndTimeInfo(startTimeInfo);
				// onset.getEndTimeInfo().setMovieTime(onset.getEndTimeInfo().getMovieTime()-minFrameDurationToAdd);
			}
		} catch (NullPointerException ex) {
			// @TODO:
			// "Not valid argument Exception will be thrown!!! ChainedEventsCollectionManager");
		}
	}

	@Override
	public void addFinalHoldEvent() {
		ChainedEvent offset = collection.getOffsetEvent();
		ChainedEvent event = collection.getValueOrTextValueEvent();

		try {
			String id = "" + Constants.GLOSS_FINAL_HOLD_CODE;
			
			Time startTimeInfo = event.getEndTimeInfo();
			Time endTimeInfo = segmentPanel.getMultipleMovieController().getFollowingFrameTimeInfo(startTimeInfo);
			
			TrackValue newValue = new TrackValue(id, "", startTimeInfo, endTimeInfo);
			collection.setFinalHoldEvent(new ChainedEvent(newValue, segmentPanel));

			if (offset != null) {
				offset.setStartTimeInfo(endTimeInfo);
			}
		} catch (NullPointerException ex) {
			// @TODO:
			// "Not valid argument Exception will be thrown!!! ChainedEventsCollectionManager");
		}
	}
}
