package edu.bu.signstream.grepresentation.fields.glossField;

import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.HeadlessException;
import java.util.ArrayList;
import java.util.HashMap;

import javax.swing.JOptionPane;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.time.Duration;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations.LocationInfo;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations.LocationType;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations.Region;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeEntity;
import edu.bu.signstream.grepresentation.fields.locationField.LocationEntity;
import edu.bu.signstream.grepresentation.fields.locationField.LocationEvent;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEvent;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsCollection;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.nonManualField.GlossEntityProperties;
import edu.bu.signstream.grepresentation.fields.signTypeField.SignTypeEntity;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolbarCommons;

public class GlossChainedEventsEntity extends ChainedEventsEntity implements Compoundable {

	private static final long serialVersionUID = 1L;
	
	private boolean isLockedDomNdomAlignments = false;
	
	private ChainedEventsEntity dependentHandEntity = null;
	private HandShapeEntity dependentHandShapeEntity = null;
	private GlossChainedEventsEntity dependentGlossEntity = null;
	private LocationEntity dependentLocationEntity = null;
	private SignTypeEntity dependentSignTypeEntity = null;

    private String dependentGlossEntityID;
    private String dependentHandEntityID;
    private String dependentHandShapeEntityID;
    private String dependentLocationEntityID;
    private String dependentSignTypeEntityID;

    private GlossChainedEventsEntity previousEntity = null;
    private GlossChainedEventsEntity nextEntity = null;

    private GlossChainedEventsCollection chainedEventsCollection = null;

    private String correspondingGlossEntityID = "";

    private Compoundable startCompound = null;
    private Compoundable endCompound = null;

    private GlossEntityProperties glossProperties = new GlossEntityProperties();
    
    private String primaryEntry = null;

    public String getDependentGlossEntityID() {
		return dependentGlossEntityID;
	}

	public void setDependentGlossEntityID(String dependentGlossEntityID) {
		this.dependentGlossEntityID = dependentGlossEntityID;
	}

	public String getDependentHandEntityID() {
		return dependentHandEntityID;
	}

	public void setDependentHandEntityID(String dependentHandEntityID) {
		this.dependentHandEntityID = dependentHandEntityID;
	}

	public String getDependentHandShapeEntityID() {
		return dependentHandShapeEntityID;
	}

	public void setDependentHandShapeEntityID(String dependentHandShapeEntityID) {
		this.dependentHandShapeEntityID = dependentHandShapeEntityID;
	}

	public String getDependentLocationEntityID() {
		return dependentLocationEntityID;
	}

	public void setDependentLocationEntityID(String dependentLocationEntityID) {
		this.dependentLocationEntityID = dependentLocationEntityID;
	}

	public String getDependentSignTypeEntityID() {
		return dependentSignTypeEntityID;
	}

	public void setDependentSignTypeEntityID(String dependentSignTypeEntityID) {
		this.dependentSignTypeEntityID = dependentSignTypeEntityID;
	}

	public GlossChainedEventsEntity(GlossChainedEventsCollection collection, SignStreamSegmentPanel segmentPanel, Field field) {
		super(collection, segmentPanel, field);
		setID(Util.getLongUniqueNumber());
		this.manager = new GlossChainedEventsCollectionManager(collection,
				segmentPanel);

		this.chainedEventsCollection = collection;

//        this.getSS3Entity();
	}
	
	public boolean isLockedDomNdomAlignments() {
		return isLockedDomNdomAlignments;
	}
	
	public void setLockedDomNdomAlignments(boolean isLockedDomNdomAlignments) {
		this.isLockedDomNdomAlignments = isLockedDomNdomAlignments;
	}

	public GlossEntityProperties getGlossEntityProperties() {
		if(glossProperties == null) {
			glossProperties = new GlossEntityProperties();
		}
		return glossProperties;
	}
	
	@Override
	public GlossChainedEventsEntity clone(String fieldID) {
		GlossChainedEventsCollection cll = (GlossChainedEventsCollection) 
				manager.getChainedEventsCollection().cloneChainedEventsCollection();
		GlossChainedEventsEntity ret = new GlossChainedEventsEntity(cll, segmentPanel, getField());
		ret.updateParentEntity();
		return ret;
	}
	
	@Override
	public void deleteOnset() {
		super.deleteOnset();

		GlossChainedEventsEntity depGlsEntity = getDependentGlossEntity();
		LocationEntity locationEntity = getDependentLocationEntity();
		
		if (isLockedDomNdomAlignments()) {
			if (depGlsEntity != null && depGlsEntity.getOnset() != null) {
				depGlsEntity.deleteOnset();
				depGlsEntity.setStartTimeInfo(getStartTimeInfo());
			}
		}

		if (locationEntity != null) {
			locationEntity.updateTimes();
		}
		
		SignTypeEntity signTypeEntity = getDependentSignTypeEntity();
		if (signTypeEntity != null) {
			signTypeEntity.updateTimes();
		}
	}
	
	@Override
	public void deleteOffset() {
		super.deleteOffset();

		GlossChainedEventsEntity depGlsEntity = getDependentGlossEntity();
		LocationEntity locationEntity = getDependentLocationEntity();
		
		if (isLockedDomNdomAlignments()) {
			if (depGlsEntity != null && depGlsEntity.getOffset() != null) {
				depGlsEntity.deleteOffset();
				depGlsEntity.setEndTimeInfo(getEndTimeInfo());
			}
		}

		if (locationEntity != null) {
			locationEntity.updateTimes();
		}

		SignTypeEntity signTypeEntity = getDependentSignTypeEntity();
		if (signTypeEntity != null) {
			signTypeEntity.updateTimes();
		}
	}
	
	@Override
	public void deleteInitialHold() {
		super.deleteInitialHold();

		GlossChainedEventsEntity depGlsEntity = getDependentGlossEntity();
		ChainedEventsEntity handEntity = getDependentHandEntity();
		HandShapeEntity handShapeEntity = getDependentHandShapeEntity();
		LocationEntity locationEntity = getDependentLocationEntity();
		
		Time time = getValueOrTextValueEvent().getStartTimeInfo();
		
		if (isLockedGlossPhonAlignments()) {
			if (handEntity != null) {
				handEntity.setStartTimeInfo(time);
				if (handShapeEntity != null) {
					MediaToolbarCommons.setHandshapeStartTime(
							handShapeEntity, time);
				}
			}
		}
		
		if (isLockedDomNdomAlignments()) {
			if (depGlsEntity != null && depGlsEntity.getInitialHold() != null) {
				depGlsEntity.deleteInitialHold();
//				depGlsEntity.setStartTimeInfo(time);
			}
		}

		if (locationEntity != null) {
			locationEntity.updateTimes();
		}
		
		SignTypeEntity signTypeEntity = getDependentSignTypeEntity();
		if (signTypeEntity != null) {
			signTypeEntity.updateTimes();
		}
	}
	
	@Override
	public void deleteFinalHold() {
		super.deleteFinalHold();

		GlossChainedEventsEntity depGlsEntity = getDependentGlossEntity();
		ChainedEventsEntity handEntity = getDependentHandEntity();
		HandShapeEntity handShapeEntity = getDependentHandShapeEntity();
		LocationEntity locationEntity = getDependentLocationEntity();
		
		Time time = getValueOrTextValueEvent().getEndTimeInfo();
		
		if (isLockedGlossPhonAlignments()) {
			if (handEntity != null) {
				handEntity.setEndTimeInfo(time);
				if (handShapeEntity != null) {
					MediaToolbarCommons.setHandshapeEndTime(
							handShapeEntity, time);
				}
			}
		}
		
		if (isLockedDomNdomAlignments()) {
			if (depGlsEntity != null && depGlsEntity.getFinalHold() != null) {
				depGlsEntity.deleteFinalHold();
//				depGlsEntity.setEndTimeInfo(time);
			}
		}

		if (locationEntity != null) {
			locationEntity.updateTimes();
		}
		
		SignTypeEntity signTypeEntity = getDependentSignTypeEntity();
		if (signTypeEntity != null) {
			signTypeEntity.updateTimes();
		}
	}

	public ArrayList<EventsEntity> getDependentEntities() {
		ArrayList<EventsEntity> dEntities = new ArrayList<EventsEntity>();
		
		if(dependentHandEntity != null) {
			dEntities.add(dependentHandEntity);
		}
		
		if(dependentHandShapeEntity != null) {
			dEntities.add(dependentHandShapeEntity);
		}
		
		if(dependentGlossEntity != null) {
			dEntities.add(dependentGlossEntity);
			
			EventsEntity depHandEntity = dependentGlossEntity.getDependentHandEntity();
			if(depHandEntity != null) {
				dEntities.add(depHandEntity);
			}
			
			EventsEntity depHandShapeEntity = dependentGlossEntity.getDependentHandShapeEntity(); 
			if(depHandShapeEntity != null) {
				dEntities.add(depHandShapeEntity);
			}
		}

		if (dependentLocationEntity != null) {
			dEntities.add(dependentLocationEntity);
		}
		
		if (dependentSignTypeEntity != null) {
			dEntities.add(dependentSignTypeEntity);
		}
		return dEntities;
	}
	
	public void setGlossEntityProperties(GlossEntityProperties glossProperties) {
		this.glossProperties = glossProperties;
	}
	
	@Override
	public ChainedEventsCollection getChainedEventsCollection() {
		return chainedEventsCollection;
	}

	public ChainedEventsEntity getDependentHandEntity() {
		return dependentHandEntity;
	}

	public void setCorrespondingGlossEntityID(String correspondingGlossEntityID) {
		this.correspondingGlossEntityID = correspondingGlossEntityID;
	}
	
	public String getCorrespondingGlossEntityID() {
		return correspondingGlossEntityID;
	}
	
	public void setDependentHandEntity(ChainedEventsEntity dependentHandEntity) {
		if(dependentHandEntity != null) {
	        setDependentHandEntityID(dependentHandEntity.getID());
		}
		this.dependentHandEntity = dependentHandEntity;
	}

	public HandShapeEntity getDependentHandShapeEntity() {
		return dependentHandShapeEntity;
	}

	public void setDependentHandShapeEntity(
			HandShapeEntity dependentHandShapeEntity) {
        this.dependentHandShapeEntity = dependentHandShapeEntity;
        if(dependentHandEntity != null) {
            this.setDependentHandShapeEntityID(dependentHandShapeEntity.getID());
		}
	}

	public GlossChainedEventsEntity getDependentGlossEntity() {
		return dependentGlossEntity;
	}

	public void setDependentGlossEntity(
			GlossChainedEventsEntity dependentGlossEntity) {
		this.dependentGlossEntity = dependentGlossEntity;

        if(dependentGlossEntity != null) {
		    setCorrespondingGlossEntityID(dependentGlossEntity.getID());
        } else {
            setCorrespondingGlossEntityID(null);
        }
	}

	public LocationEntity getDependentLocationEntity() {
		return dependentLocationEntity;
	}

	public void setDependentLocationEntity(LocationEntity dependentLocationEntity) {
    	if (dependentLocationEntity != null) {
    		setDependentLocationEntityID(dependentLocationEntity.getID());

			ArrayList<LocationEvent> events = dependentLocationEntity.getEvents();
			HashMap<LocationType, Region> locationRegionsMap = new HashMap<LocationType, Region>();
			for (LocationEvent event : events) {
				locationRegionsMap.put(event.getLocationType(), event.getLocation());
			}
			glossProperties.setLocationInfo(new LocationInfo(locationRegionsMap));
		}
		this.dependentLocationEntity = dependentLocationEntity;
	}
	
	public SignTypeEntity getDependentSignTypeEntity() {
		return dependentSignTypeEntity;
	}

	public void setDependentSignTypeEntity(SignTypeEntity dependentSignTypeEntity) {
		this.dependentSignTypeEntity = dependentSignTypeEntity;
    	if (dependentSignTypeEntity != null) {
    		setDependentSignTypeEntityID(dependentSignTypeEntity.getID());
		}
	}

	@Override
	public GlossChainedEventsEntity getPreviousEntity() {
		return previousEntity;
	}

	public void setPreviousEntity(GlossChainedEventsEntity previousEntity) {
		this.previousEntity = previousEntity;
	}

	@Override
	public GlossChainedEventsEntity getNextEntity() {
		return nextEntity;
	}

	public void setNextEntity(GlossChainedEventsEntity nextEntity) {
		this.nextEntity = nextEntity;
	}

	@Override
	public String getText() {
        if(getTextValue() == null) {
            return "";
        } else {
		    return getTextValue().getText();
        }
	}

	@Override
	public String getText(boolean start) {
		return getText();
	}

	@Override
	public boolean isEndCompound() {
		return endCompound != null;
	}

	@Override
	public boolean isStartCompound() {
		return startCompound != null;
	}

	@Override
	public Compoundable getEndCompound() {
		return endCompound;
	}

	@Override
	public Compoundable getStartCompound() {
		return startCompound;
	}

	@Override
	public void setEndCompound(Compoundable endCompound) {
		this.endCompound = endCompound;
	}

	@Override
	public void setStartCompound(Compoundable startCompound) {
		this.startCompound = startCompound;
	}

	public String getPrimaryEntry() {
		return primaryEntry;
	}

	public void setPrimaryEntry(String primaryEntry) {
		this.primaryEntry = primaryEntry;
	}

	@Override
	public void paint(Graphics2D g2d, double Y, Font font) {
		g2d.setFont(font);
		FontMetrics fontMetrics = getFontMetrics(font);
		
		segmentPanel.getPainter().drawGlossEntity(this, g2d, Y, fontMetrics, getField().getFieldID(), previousEntity, nextEntity);
	}

	@Override
	public void setInitialHoldEvent(ChainedEvent event) {
		if (event != null)
			event.setID("" + Constants.GLOSS_INITIAL_HOLD_CODE);
		manager.getChainedEventsCollection().setInitialHoldEvent(event);
	}

	@Override
	public void setFinalHoldEvent(ChainedEvent event) {
		if (event != null)
			event.setID("" + Constants.GLOSS_FINAL_HOLD_CODE);
		manager.getChainedEventsCollection().setFinalHoldEvent(event);
	}

	@Override
	public void setOnsetEvent(ChainedEvent event) {
		if (event != null)
			event.setID("" + Constants.GLOSS_ONSET_CODE);
		manager.getChainedEventsCollection().setOnsetEvent(event);
	}

	@Override
	public void setOffsetEvent(ChainedEvent event) {
		if (event != null)
			event.setID("" + Constants.GLOSS_OFFSET_CODE);
		manager.getChainedEventsCollection().setOffsetEvent(event);
	}

	@Override
	public String addOnsetEvent() {
		if (previousEntity != null) {
			Time endMovieTimeInfo = previousEntity.getEndTimeInfo();

			ChainedEvent event = (ChainedEvent) manager.getFirstEvent();
			Time selectedEventStartTimeInfo = segmentPanel.getMultipleMovieController()
					.getPrecedingFrameTimeInfo(event.getStartTimeInfo());

			if (selectedEventStartTimeInfo.lessThan(endMovieTimeInfo)) {
				return ErrorMessages.CHAINS_CANNOT_OVERLAP;
			}
		}

		manager.addOnsetEvent();
		return null;
	}

	@Override
	public String addInitialHoldEvent() {
		ChainedEvent onset = getOnset();
		if (onset != null) {
			Time onsetStartTimeInfo = onset.getStartTimeInfo();
			Time onsetEndTimeInfo = onset.getEndTimeInfo();
			Time precedingFrameEndTimeInfo = segmentPanel.getMultipleMovieController()
				.getPrecedingFrameTimeInfo(onsetEndTimeInfo);

			if (precedingFrameEndTimeInfo.equals(onsetStartTimeInfo)) {
				return ErrorMessages.ADD_INITIAL_HOLD_CANT_ADJUST_ONSET;
			} else {
				manager.addInitialHoldEvent();
				return null;
			}
		}

		if (previousEntity != null) {
			Time endMovieTimeInfo = previousEntity.getEndTimeInfo();

			ChainedEvent event = (ChainedEvent) manager.getFirstEvent();
			Time selectedEventStartTimeInfo = segmentPanel.getMultipleMovieController()
					.getPrecedingFrameTimeInfo(event.getStartTimeInfo());

			if (selectedEventStartTimeInfo.lessThan(endMovieTimeInfo)) {
				return ErrorMessages.CHAINS_CANNOT_OVERLAP;
			}
		}

		manager.addInitialHoldEvent();
		return null;
	}

	@Override
	public String addFinalHoldEvent() {
		// GREG TODO: should the order of the 2 below if-stmts be reversed, like in above addInitialHoldEvent ?
		if (nextEntity != null) {
			Time startMovieTimeInfo = nextEntity.getStartTimeInfo();

			ChainedEvent event = (ChainedEvent) manager.getLastEvent();
			Time selectedEventEndTimeInfo = segmentPanel.getMultipleMovieController()
					.getFollowingFrameTimeInfo(event.getEndTimeInfo());

			if (selectedEventEndTimeInfo.greaterThan(startMovieTimeInfo)) {
				return ErrorMessages.CHAINS_CANNOT_OVERLAP;
			}
		}

		ChainedEvent offset = getOffset();
		if (offset != null) {
			Time onsetStartTimeInfo = offset.getStartTimeInfo();
			Time onsetEndTimeInfo = offset.getEndTimeInfo();
			Time precedingFrameEndTimeInfo = segmentPanel.getMultipleMovieController()
				.getPrecedingFrameTimeInfo(onsetEndTimeInfo);

			if (precedingFrameEndTimeInfo.equals(onsetStartTimeInfo)) {
				return ErrorMessages.ADD_FINAL_HOLD_CANT_ADJUST_OFFSET;
			}
		}

		manager.addFinalHoldEvent();
		return null;
	}

	@Override
	public String addOffsetEvent() {
		if (nextEntity != null) {
			Time startMovieTimeInfo = nextEntity.getStartTimeInfo();

			ChainedEvent event = (ChainedEvent) manager.getFirstEvent();
			Time selectedEventEndTimeInfo = segmentPanel.getMultipleMovieController()
					.getFollowingFrameTimeInfo(event.getEndTimeInfo());

			if (selectedEventEndTimeInfo.greaterThan(startMovieTimeInfo)) {
				return ErrorMessages.CHAINS_CANNOT_OVERLAP;
			}
		}

		manager.addOffsetEvent();
		return null;
	}

	
    // returns an error code that will be matched with an error message or null if success. 
	@Override
	public String canSetSelectedEventStartMovieTime(Time timeInfo) {
		Event selectedEvent =  SS3Singleton.getMediaToolBar(null).selectedEvent();
		
        Time originalSelectedStartTimeInfo = selectedEvent.getStartTimeInfo();
        Time originalSelectedEndTimeInfo = selectedEvent.getEndTimeInfo();

        Time maxStartMovieTimeInfo = originalSelectedEndTimeInfo.subtract(new Duration(1, originalSelectedEndTimeInfo.unit()));
		
		Event initialHold = manager.getChainedEventsCollection().getInitialHoldEvent();
		Event value = manager.getChainedEventsCollection().getValueOrTextValueEvent();
		Event finalHold = manager.getChainedEventsCollection().getFinalHoldEvent();
		Event offset = manager.getChainedEventsCollection().getOffsetEvent();
		
		// Gloss value events are allowed to have the same start/end time
		if (selectedEvent.equals(value)) {
            maxStartMovieTimeInfo = maxStartMovieTimeInfo.add(new Duration(1, originalSelectedEndTimeInfo.unit()));
		}

		if (maxStartMovieTimeInfo.lessThan(timeInfo)) {
			return ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME;
		}
		
		GlossChainedEventsEntity glsEntity = (GlossChainedEventsEntity) this;
		GlossChainedEventsEntity depGlsEntity = glsEntity.getDependentGlossEntity();
		boolean glsLockedDomNdomAlignments = glsEntity.isLockedDomNdomAlignments();

		if (selectedEvent.equals(value) || selectedEvent.equals(initialHold) || selectedEvent.equals(finalHold)) {
			// check for one-frame logic of Gloss events
			boolean currentlyOneFrame = originalSelectedStartTimeInfo.equals(originalSelectedEndTimeInfo);
			boolean becomingOneFrame = timeInfo.equals(originalSelectedEndTimeInfo);
			if (currentlyOneFrame || becomingOneFrame) {
				String retOneFrame = checkOneFrameLogic(timeInfo, currentlyOneFrame, becomingOneFrame);
				if (retOneFrame != null) {
					return retOneFrame;
				}
			}
			
			if (!becomingOneFrame) {
				// asking the dep hand entities on both hands if canMove
	            boolean glsLockedHand = glsEntity.isLockedGlossPhonAlignments();
				
				ChainedEventsEntity handEntity = this.getDependentHandEntity();
				ChainedEventsEntity depHandEntity = null;
				boolean depGlsLockedHand = false;
				
				if (handEntity != null && glsLockedHand) {
					String resp = handEntity.canSetSelectedEventStartMovieTime(timeInfo);

					if (resp != null) {
						return ErrorMessages.GLOSS_CHAINS_CANNOT_OVERLAP;
					}
				}
				
				if (depGlsEntity != null && glsLockedDomNdomAlignments) {
					GlossChainedEventsEntity depGlsPrevEntity = depGlsEntity.getPreviousEntity();
					if (depGlsPrevEntity != null) {
						if (depGlsPrevEntity.getEndTimeInfo().greaterThan(timeInfo)) {
							return ErrorMessages.GLOSS_CHAINS_CANNOT_OVERLAP;
						}
					}
					
					depHandEntity = depGlsEntity.getDependentHandEntity();
					depGlsLockedHand = depGlsEntity.isLockedGlossPhonAlignments();
					if (depHandEntity != null && depGlsLockedHand) {
						String resp = depHandEntity.canSetSelectedEventStartMovieTime(timeInfo);

						if (resp != null) {
							return ErrorMessages.GLOSS_CHAINS_CANNOT_OVERLAP;
						}
					}
				}
			}
		}

		if (previousEntity != null) {
			if (previousEntity.getEndTimeInfo().greaterThan(timeInfo)) {
				return ErrorMessages.GLOSS_CHAINS_CANNOT_OVERLAP;
			}
		}
		
		Event prevEvent = manager.getChainedEventsCollection().getPreviousEvent(selectedEvent);
		
		if (prevEvent == null) {
			// |<--
			if (selectedEvent.getStartTimeInfo().greaterThan(timeInfo)) {
				if (previousEntity == null
						|| previousEntity.getEndTimeInfo().lessThanOrEq(timeInfo)) {
//		            selectedEvent.setStartTimeInfo(timeInfo);
		            return null;
//                    // selectedEvent.getStartTimeInfo().setMovieTime(movieTime);
				} else {
					return ErrorMessages.GLOSS_CHAINS_CANNOT_OVERLAP;
				}
			} else {
//	            selectedEvent.setStartTimeInfo(timeInfo);
				return null;
//                // selectedEvent.getStartTimeInfo().setMovieTime(movieTime);
			}			
		} else if (selectedEvent.getStartTimeInfo().greaterThan(timeInfo)) {
			Time prevStartTimeInfo = prevEvent.getStartTimeInfo();
			Time timeInfoLessOneDuration = timeInfo.subtract(new Duration(1, timeInfo.unit()));
			if (timeInfoLessOneDuration.lessThan(prevStartTimeInfo)) {
				
				if(selectedEvent.equals(initialHold)) {
					return ErrorMessages.SET_START_INITIAL_HOLD_CANT_DELETE_ONSET;
				} else if(selectedEvent.equals(value)) {
					if(initialHold != null && prevEvent.equals(initialHold)) {
						return ErrorMessages.SET_START_VALUE__CANT_DELETE_INITIAL_HOLD;
					} else  {
						return ErrorMessages.SET_START_VALUE_CANT_DELETE_ONSET;
					}
				} else if(selectedEvent.equals(finalHold)) {
					return ErrorMessages.SET_START_FINAL_HOLD_CANT_DELETE_VALUE;
				} else if(selectedEvent.equals(offset)) {
					if(finalHold != null && prevEvent.equals(finalHold)) {
						return ErrorMessages.SET_START_OFFSET_CANT_DELETE_FINAL_HOLD; 
					} else {
						return ErrorMessages.SET_START_OFFSET_CANT_DELETE_VALUE;
					}
				}
			} else {
//	            selectedEvent.setStartTimeInfo(timeInfo);
//
//				prevEvent.setEndTimeInfo(timeInfo);
				return null;
			}
		} else {
//            selectedEvent.setStartTimeInfo(timeInfo);
//
//            prevEvent.setEndTimeInfo(timeInfo);
			return null;
		}

		SS3Singleton.getMediaToolBar().update();
		return null;
	}

	@Override
	public String canSetSelectedEventEndMovieTime(Time timeInfo) {
		Event selectedEvent = SS3Singleton.getMediaToolBar(null).selectedEvent();

        Time originalSelectedStartTimeInfo = selectedEvent.getStartTimeInfo();
        Time originalSelectedEndTimeInfo = selectedEvent.getEndTimeInfo();

        Time minEndMovieTimeInfo = originalSelectedStartTimeInfo.add(new Duration(1, originalSelectedStartTimeInfo.unit()));

		ChainedEvent onset = manager.getChainedEventsCollection().getOnsetEvent();
		ChainedEvent initialHold = manager.getChainedEventsCollection().getInitialHoldEvent();
		ChainedEvent value = manager.getChainedEventsCollection().getValueOrTextValueEvent();
		ChainedEvent finalHold = manager.getChainedEventsCollection().getFinalHoldEvent();	
		
		GlossChainedEventsEntity glsEntity = (GlossChainedEventsEntity) this;

		GlossChainedEventsEntity depGlsEntity = glsEntity.getDependentGlossEntity();
		boolean glsLockedDomNdomAlignments = glsEntity.isLockedDomNdomAlignments();
		
		// Gloss value events are allowed to have the same start/end time
		if (selectedEvent.equals(value)) {
			minEndMovieTimeInfo = minEndMovieTimeInfo.subtract(new Duration(1, originalSelectedStartTimeInfo.unit()));
		}

		if (minEndMovieTimeInfo.greaterThan(timeInfo)) {
			return ErrorMessages.EVENT_END_TIME_FOLLOW_START_TIME;
		}
		
		if (selectedEvent.equals(value) || selectedEvent.equals(initialHold) || selectedEvent.equals(finalHold)) {
			// check for one-frame logic of Gloss events
			boolean currentlyOneFrame = originalSelectedStartTimeInfo.equals(originalSelectedEndTimeInfo);
			boolean becomingOneFrame = timeInfo.equals(originalSelectedStartTimeInfo);
			if (currentlyOneFrame || becomingOneFrame) {
				String retOneFrame = checkOneFrameLogic(timeInfo, currentlyOneFrame, becomingOneFrame);
				if (retOneFrame != null) {
					return retOneFrame;
				}
			}
			
			if (!becomingOneFrame) {
				// asking the dep hand entities on both hands if canMove
	            boolean glsLockedHand = glsEntity.isLockedGlossPhonAlignments();
				
				ChainedEventsEntity handEntity = this.getDependentHandEntity();
				ChainedEventsEntity depHandEntity = null;
				boolean depGlsLockedHand = false;
				
				if (handEntity != null && glsLockedHand) {
					String resp = handEntity.canSetSelectedEventEndMovieTime(timeInfo);

					if (resp != null) {
						return ErrorMessages.GLOSS_CHAINS_CANNOT_OVERLAP;
					}
				}
				
				if (depGlsEntity != null && glsLockedDomNdomAlignments) {
					GlossChainedEventsEntity depGlsNextEntity = depGlsEntity.getNextEntity();
					if (depGlsNextEntity != null) {
						if (depGlsNextEntity.getStartTimeInfo().lessThan(timeInfo)) {
							return ErrorMessages.CHAINS_CANNOT_OVERLAP;
						}
						
						depHandEntity = depGlsEntity.getDependentHandEntity();
						depGlsLockedHand = depGlsEntity.isLockedGlossPhonAlignments();
						if (depHandEntity != null && depGlsLockedHand) {
							String resp = depHandEntity.canSetSelectedEventEndMovieTime(timeInfo);

							if (resp != null) {
								return ErrorMessages.GLOSS_CHAINS_CANNOT_OVERLAP;
							}
						}
					}
				}
			}
		}
		
		if (nextEntity != null) {
			if (nextEntity.getStartTimeInfo().lessThan(timeInfo)) {
				return ErrorMessages.CHAINS_CANNOT_OVERLAP;
			}
		}

		Event nextEvent = manager.getChainedEventsCollection().getNextEvent(selectedEvent);

		if(nextEvent != null) {				
			Time maxNextEventStartTimeInfo = segmentPanel.getMultipleMovieController().getPrecedingFrameTimeInfo(
					nextEvent.getEndTimeInfo());
			if(maxNextEventStartTimeInfo.greaterThanOrEq(timeInfo)) {
//				selectedEvent.setEndTimeInfo(timeInfo);
//				
//				nextEvent.setStartTimeInfo(timeInfo);
				return null;
			} else {
				if(selectedEvent.equals(onset)) {
					if(initialHold!= null) {
						return ErrorMessages.SET_END_ONSET_CANT_DELETE_INITIAL_HOLD;
					} else {
						return ErrorMessages.SET_END_ONSET_CANT_DELETE_VALUE;
					}
				} else if(selectedEvent.equals(initialHold)) {
					return ErrorMessages.SET_END_INITIAL_HOLD_CANT_DELETE_VALUE;
				} else if(selectedEvent.equals(value)) {
					if(finalHold != null) {
						return ErrorMessages.SET_END_VALUE_CANT_DELETE_FINAL_HOLD;
					} else {
						return ErrorMessages.SET_END_VALUE_CANT_DELETE_OFFSET;
					}
				} else if (selectedEvent.equals(finalHold)) {
					return ErrorMessages.SET_END_FINAL_HOLD_CANT_DELETE_OFFSET;
				}
			} 
		} else {
//			selectedEvent.setEndTimeInfo(timeInfo);
			return null;
		}
		
		SS3Singleton.getMediaToolBar().update();
		return null;
	}

	// GD: there's some conflicts in just straight overriding ChainedEventsEntity.setStartTimeInfo/setEndTimeInfo
	public void setGlossEventsEntityStartTimeInfo(Time timeInfo) {
		Event selectedEvent = SS3Singleton.getMediaToolBar(null).selectedEvent();
		Event prevEvent = manager.getChainedEventsCollection().getPreviousEvent(selectedEvent);
		
		if (selectedEvent == null) {
			super.setStartTimeInfo(timeInfo);
			return;
		}
		
		selectedEvent.setStartTimeInfo(timeInfo);
		if (prevEvent != null) {
			prevEvent.setEndTimeInfo(timeInfo);
		}
	}

	public void setGlossEventsEntityEndTimeInfo(Time timeInfo) {
		Event selectedEvent = SS3Singleton.getMediaToolBar(null).selectedEvent();
		Event nextEvent = manager.getChainedEventsCollection().getNextEvent(selectedEvent);
		
		if (selectedEvent == null) {
			super.setEndTimeInfo(timeInfo);
			return;
		}
		
		selectedEvent.setEndTimeInfo(timeInfo);
		if (nextEvent != null) {
			nextEvent.setStartTimeInfo(timeInfo);
		}
	}
	
	private String checkOneFrameLogic(Time timeInfo, boolean currentlyOneFrame, boolean becomingOneFrame) {
		GlossChainedEventsEntity  glsEntity = (GlossChainedEventsEntity)this;
		
		if (becomingOneFrame) {
			if ((nextEntity != null && nextEntity.getStartTimeInfo().equals(timeInfo) && nextEntity.getEndTimeInfo().equals(timeInfo)) 
				|| (previousEntity != null && previousEntity.getStartTimeInfo().equals(timeInfo) && previousEntity.getEndTimeInfo().equals(timeInfo))) {
				return ErrorMessages.SET_ONE_FRAMES_CANNOT_OVERLAP;				
			}
		}
		
		HandShapeEntity handShapeEntity = glsEntity.getDependentHandShapeEntity();
		ChainedEvent initialHold = glsEntity.getInitialHold();
		ChainedEvent finalHold = glsEntity.getFinalHold();
		
		boolean isLockedDomNdom = glsEntity.isLockedDomNdomAlignments();
		boolean isGlsLockedHand = glsEntity.isLockedGlossPhonAlignments();
		
		boolean glsCanMoveHandshapes = true;
		if (initialHold != null || finalHold != null) {
			glsCanMoveHandshapes = false;
		}
		
		GlossEntityProperties properties = glsEntity.getGlossEntityProperties();
		
		GlossChainedEventsEntity depGlsEntity = glsEntity.getDependentGlossEntity();
		HandShapeEntity depHandShapeEntity = null;
		
		boolean isDepGlsLockedHand = false;	
		boolean depGlsCanMoveHandshapes = true;
		GlossEntityProperties depProperties = null;
		
		if (depGlsEntity != null) {
			depHandShapeEntity = depGlsEntity.getDependentHandShapeEntity();
			ChainedEvent depGlsInitialHold = depGlsEntity.getInitialHold();
			ChainedEvent depGlsFinalHold = depGlsEntity.getFinalHold();
			
			if (depGlsInitialHold != null || depGlsFinalHold != null) {
				depGlsCanMoveHandshapes = false;
			}
			
			isDepGlsLockedHand = depGlsEntity.isLockedGlossPhonAlignments();
			depProperties = depGlsEntity.getGlossEntityProperties();
		}
		
		// current start and end times same? it means we are moving to a 1 or more frame difference, so set oneFrameGloss to false
		if (currentlyOneFrame) {
			properties.setOneFrameGloss(false);
			
			if (isGlsLockedHand) {
				if (handShapeEntity != null && glsCanMoveHandshapes)
					handShapeEntity.cloneEvents(timeInfo);
			}
			
			if (isLockedDomNdom) {
				depProperties.setOneFrameGloss(false);
				
				if (isDepGlsLockedHand) {
					if (depHandShapeEntity != null && depGlsCanMoveHandshapes)
						depHandShapeEntity.cloneEvents(timeInfo);
				}
			}
		}
		else {
			// start is moving into end - intention is to recreate a one frame Gloss
			if (becomingOneFrame) {	
				boolean isPossibleToMerge = true;	// can we merge handshapes at all? (i.e. is the first the same as the last handshape?)

				if (isGlsLockedHand) {
					if (handShapeEntity != null) {
						isPossibleToMerge = handShapeEntity.isPossibleToMergeEvents();
					}
				}
				
				if (isPossibleToMerge) {
					if (isLockedDomNdom && isDepGlsLockedHand) {
						if (depHandShapeEntity != null) {
							isPossibleToMerge = depHandShapeEntity.isPossibleToMergeEvents();
						}
					}
				}

				int optionPaneResult = -1;
				if (isPossibleToMerge) {
					String warningDialogText = "Warning: this action will create a one-frame Gloss\n\n"
							+ "and merge the start and end handshapes into one handshape.\n\n"
							+ "If this is okay, please press Proceed";
					String warningTitleText = "WARNING";
					int optionType = JOptionPane.YES_NO_OPTION;
					int messageType = JOptionPane.ERROR_MESSAGE;
					String options[] = {"Proceed", "Cancel"};
					// indeed this action is not an error, and is in fact a warning;
					// but nonetheless a red stop sign messageType is louder than a yellow triangle

					try {
						optionPaneResult = JOptionPane.showOptionDialog(this, warningDialogText, warningTitleText, optionType, 
								messageType, null, options,	options[1]);
					} catch (HeadlessException he) {
						System.out.println(he.getMessage());
					}
				} else {				            
					return ErrorMessages.SET_ONE_FRAME_GLOSS_DIFF_HANDSHAPES;
				}

				if (optionPaneResult == JOptionPane.OK_OPTION) {			
					// merge current start and end handshapes into one
					
					if (isGlsLockedHand) {
						if (handShapeEntity != null && glsCanMoveHandshapes) 
							handShapeEntity.mergeEvents(timeInfo);
					}

					properties.setOneFrameGloss(true);
					properties.setStartEndHandshape(GlossEntityProperties.SAME_START_END_HANDSHAPE);
					properties.setStartEndHandshapeNdomHand(GlossEntityProperties.SAME_START_END_HANDSHAPE);
					properties.setStartEndHandshapeDomHand(GlossEntityProperties.SAME_START_END_HANDSHAPE);

					if (isLockedDomNdom) {
						depProperties.setOneFrameGloss(true);
						depProperties.setStartEndHandshape(GlossEntityProperties.SAME_START_END_HANDSHAPE);
						depProperties.setStartEndHandshapeNdomHand(GlossEntityProperties.SAME_START_END_HANDSHAPE);
						depProperties.setStartEndHandshapeDomHand(GlossEntityProperties.SAME_START_END_HANDSHAPE);
						
						if (isDepGlsLockedHand) {
							if (depHandShapeEntity != null && depGlsCanMoveHandshapes)
								depHandShapeEntity.mergeEvents(timeInfo);
						}
					}

					return null;
				} else {
					return ""; // this is a dummy return code, which returns with no action being taken, and prevents a popup window in calling function
				}
			}
		}
		
		return null;
	}
}
