package edu.bu.signstream.grepresentation.fields.glossField;

import java.awt.Font;
import java.awt.Graphics2D;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.logging.Level;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.Palette;
import edu.bu.signstream.common.util.vo.Track;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Track;
import edu.bu.signstream.time.Bound;
import edu.bu.signstream.time.Interval;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEvent;
import edu.bu.signstream.grepresentation.fields.nonManualField.NonManualField;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.ui.SS3SignStreamApplication;

/**
 * Non-manual fields fall into two categories: anatomical and functional.
 * 
 * Anatomical fields refer to particular physical muscle movement.
 * Functional/interpretive fields provide a functional interpretation for a
 * particular movement or set of movements.
 * */

public class GlossField extends NonManualField { // implements ActionListener {

	private static final long serialVersionUID = 1L;

	int selectedCompoundStartCoord = -1;
	int selectedCompoundEndCoord = -1;
	
	private NewGlossChainedEvent newGlossEvent;
	private GlossChainedEventsEntityCollection collection = null;

	public GlossField(SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		super(segmentPanel, parentField);
	}

	public GlossField(Track track, SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		super(segmentPanel, parentField);
		this.track = track;
		createTrackEntity(track);
	}

	public GlossField(SS3Track ss3track, SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		super(segmentPanel, parentField);
		this.ss3Track = ss3track;
		createTrackEntity(ss3track);
	}

	public GlossChainedEventsEntityCollection getGlossChainedEventsEntityCollection() {
		return collection;
	}
	
	public double getPrevEntityOrTextStartCoord(double coord) {
		return collection.getPrevEntityOrTextStartCoord(coord);
	}
	
	public double getNextEntityOrTextStartCoord(double coord) {
		return collection.getNextEntityOrTextStartCoord(coord);
	}
	
	@Override
	public GlossChainedEventsEntity getNextEntity(Time endTimeInfo) {
		return collection.getNextEntity(endTimeInfo);
	}
	
	@Override
	public GlossChainedEventsEntity getPreviousEntity(Time startTimeInfo) {
		return collection.getPreviousEntity(startTimeInfo);
	}


    public ArrayList<GlossChainedEventsEntity> getEntitiesInInterval(Interval interval) {
        return collection.getEntitiesInInterval(interval);
    }

    public ArrayList<GlossChainedEventsEntity> getEntitiesInInterval(Interval interval, Bound bound) {
        return collection.getEntitiesInInterval(interval, bound);
    }


	@Override
	public void deleteEntity(EventsEntity entity) {
		GlossChainedEventsEntity glsEntity = (GlossChainedEventsEntity)entity;
		collection.deleteEntity(glsEntity);
	}
	
	public void setGlossChainedEventsEntityCollection(GlossChainedEventsEntityCollection collection) {
		this.collection = collection;
	}
	
	public void setSelectCompoundStartCoord(int selectedCompoundStartCoord) {
		this.selectedCompoundStartCoord = selectedCompoundStartCoord;
	
		EnteredText enteredText = null;

		try {
			enteredText = getGlossChainedEventsEntityCollection().getEnteredText();
			enteredText.unselectAll();
		} catch (NullPointerException e) {
			e.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", e);
		}
	}

	public void setSelectCompoundEndCoord(int selectedCompoundEndCoord) {
		this.selectedCompoundEndCoord = selectedCompoundEndCoord;
	}

	public int getSelectedCompoundStartCoord() {
		return selectedCompoundStartCoord;
	}

	public int getSelectedCompoundEndCoord() {
		return selectedCompoundEndCoord;
	}

	public NewGlossChainedEvent getNewGlossChainedEvent() {
		return newGlossEvent;
	}

	public void setNewGlossChainedEvent(NewGlossChainedEvent newGlossEvent) {
		this.newGlossEvent = newGlossEvent;
	}

	public ArrayList<Compoundable> getEventsToCreateCompound() {
		
		if (selectedCompoundStartCoord == -1 || selectedCompoundEndCoord == -1)
			return null;

        // JACK TODO: This is always an empty list... leaving so we can find usages of the function.
		ArrayList<NotCompleteGlossEvent> nonComplEventsInInterval = collection
				.getNotCompleteGlossEventsInInterval(
						selectedCompoundStartCoord, selectedCompoundEndCoord);

        var converter = SS3Singleton.getVideoControlManager().getZoomer().getCoordinateTimeConvertor();
        var start = converter.convertCoordinateToTimeInfo(selectedCompoundStartCoord);
        var end = converter.convertCoordinateToTimeInfo(selectedCompoundEndCoord);
        var interval = new Interval(start, end);

        ArrayList<GlossChainedEventsEntity> entitiesInInterval = collection.getEntitiesInInterval(interval);
        collection.sort(entitiesInInterval);

        return new ArrayList<Compoundable>(entitiesInInterval);

        // JACK TODO: None of this even runs since size() is always zero... commenting for now.
//		ArrayList<Compoundable> compounds = new ArrayList<Compoundable>();
//
//		collection.sort(entitiesInInterval);
//
//		for(GlossChainedEventsEntity entity : entitiesInInterval) {
//			compounds.add(entity);
//		}
//
//		if (nonComplEventsInInterval.size() == 0) {
//			return compounds;
//		} else if (nonComplEventsInInterval.size() > 2) {
//			return null;
//
//		} else {
//			NotCompleteGlossEvent presNotComplEvent = null;
//			NotCompleteGlossEvent posNotComplEvent = null;
//
//			NotCompleteGlossEvent nCmlGlsEvent1 = nonComplEventsInInterval.get(0);
//
//			if (nCmlGlsEvent1.isEndMovieTimeFinal()) {
//				presNotComplEvent = nCmlGlsEvent1;
//			} else {
//				posNotComplEvent = nCmlGlsEvent1;
//			}
//
//			if (nonComplEventsInInterval.size() == 2) {
//				NotCompleteGlossEvent nCmlGlsEvent2 = nonComplEventsInInterval
//						.get(1);
//				if (presNotComplEvent != null
//						&& nCmlGlsEvent2.isStartMovieTimeFinal()
//						&& presNotComplEvent.getEndTimeInfo().doubleValue() <= nCmlGlsEvent2
//								.getStartTimeInfo().doubleValue()) {
//					posNotComplEvent = nCmlGlsEvent2;
//				} else if (posNotComplEvent != null
//						&& nCmlGlsEvent2.isEndMovieTimeFinal()
//						&& nCmlGlsEvent2.getEndTimeInfo().doubleValue() <= posNotComplEvent
//								.getStartTimeInfo().doubleValue()) {
//					presNotComplEvent = nCmlGlsEvent2;
//				}
//			}
//
//			if (compounds.size() == 0) {
//				if (presNotComplEvent != null)
//					compounds.add(presNotComplEvent);
//				if (posNotComplEvent != null)
//					compounds.add(posNotComplEvent);
//			} else {
//				Compoundable strEntity = compounds.get(0);
//				Compoundable endEntity = compounds.get(compounds.size() - 1);
//				if (presNotComplEvent != null
//						&& strEntity.getStartTimeInfo().doubleValue() >= presNotComplEvent
//								.getEndTimeInfo().doubleValue()) {
//					compounds.add(0, presNotComplEvent);
//				}
//				if (posNotComplEvent != null
//						&& endEntity.getEndTimeInfo().doubleValue() <= posNotComplEvent
//								.getStartTimeInfo().doubleValue()) {
//					compounds.add(compounds.size(), posNotComplEvent);
//				}
//			}
//			return compounds;
//		}
	}

	@Override
	public void deleteEmptyEntities() {
		ArrayList<GlossChainedEventsEntity> entities = collection.getEntities();
		ArrayList<GlossChainedEventsEntity> entitiesToDelete = new ArrayList<GlossChainedEventsEntity>();
		for(GlossChainedEventsEntity entity : entities) {
			if(entity.getStartTimeInfo() == null || entity.getEndTimeInfo() == null || entity.getValueOrTextValueEvent() == null) {
				entitiesToDelete.add(entity);
			}
		}
		
		for(GlossChainedEventsEntity toDelete : entitiesToDelete) {
			collection.deleteEntity(toDelete);
		}
	//	collection.deleteEmptyEntities();
	//	collection.updateGlossEntities();
	}

	@Override
	public ArrayList getEntities() {
		return collection.getEntities();
	}

	@Override
	public EventsEntity getEntity(String entityID) {
		return collection.getEntity(entityID);
	}
	
	
	public void addGlossChainedEventsEntity(GlossChainedEventsEntity entity) {
		collection.addEntity(entity);	
	}

	@Override
	public Event getEventAt(double x) {
		return collection.getEventAt(x);
	}
	
	@Override
	public EventsEntity getEntityAt(double x) {
		return collection.getGlossChainedEventsEntityAt(x);
	}

	@Override
	public Track getTrack() {
		return track;
	}

	@Override
	protected void createTrackEntity(SS3Track track) {
		ArrayList<GlossChainedEventsEntity> chainedEvents = new ArrayList<GlossChainedEventsEntity >();
		ArrayList<SS3Entity> entities = track.getEntities();

		HashMap<SS3Entity, SS3Item> txtOnly = new HashMap<SS3Entity, SS3Item>();	// make tuples of entity->item
	 
		for (SS3Entity entity : entities) {
			GlossChainedEventsCollection collection = new GlossChainedEventsCollection();
			
			ArrayList<SS3Item> items = entity.getItemsAsList();
			for (SS3Item item : items) {
				String id = item.getId();

				// TODO: I don't believe the checks for track/field id here are necessary - it is obvious we are gloss field at this point
				if ((track.getId().equals(Util.DOM_GLOSS_FIELD_ID) ||
						track.getId().equals(Util.NDOM_GLOSS_FIELD_ID)) && item.isTextOnly()) {
					txtOnly.put(entity, item);
				} else {
					if (id.equalsIgnoreCase("" + Constants.GLOSS_ONSET_CODE)) {
						collection.setOnsetEvent(new ChainedEvent(item, segmentPanel));
					} else if (id.equalsIgnoreCase("" + Constants.GLOSS_OFFSET_CODE)) {
						collection.setOffsetEvent(new ChainedEvent(item, segmentPanel));
					} else if (id.equalsIgnoreCase("" + Constants.GLOSS_INITIAL_HOLD_CODE)) {
						collection.setInitialHoldEvent(new ChainedEvent(item, segmentPanel));
					} else if (id.equalsIgnoreCase("" + Constants.GLOSS_FINAL_HOLD_CODE)) {
						collection.setFinalHoldEvent(new ChainedEvent(item, segmentPanel));
					} else {
						collection.setTextValueEvent(new GlossChainedEvent(item, segmentPanel));
					}
				}
			}
			
			// means its a chained entity and not a text-only
			if (collection.getTextValueEvent() != null) {
				GlossChainedEventsEntity chainedEventEntity = new GlossChainedEventsEntity(collection, segmentPanel, this);

				chainedEventEntity.setGlossEntityProperties(entity.getGlossEntityProperties());
				chainedEventEntity.setCorrespondingGlossEntityID(entity.getDependentGlossEntityID());
				
				String dependentGlossEntityID = entity.getDependentGlossEntityID();
				String dependentHandEntityID = entity.getDependentHandEntityID();
				String dependentHandShapeEntityID = entity.getDependentHandShapeEntityID();
				String dependentLocationEntityID = entity.getDependentLocationEntityID();
				String dependentSignTypeEntityID = entity.getDependentSignTypeEntityID();
				boolean isLockedGlossPhonAlignments = entity.isLockedGlossPhoneAlignments();
				boolean isLockedDomNdomAlignments = entity.isLockedDomNdomAlignments();
							
				chainedEventEntity.setDependentGlossEntityID(dependentGlossEntityID);
				chainedEventEntity.setDependentHandEntityID(dependentHandEntityID);
				chainedEventEntity.setDependentHandShapeEntityID(dependentHandShapeEntityID);
				chainedEventEntity.setDependentLocationEntityID(dependentLocationEntityID);
				chainedEventEntity.setDependentSignTypeEntityID(dependentSignTypeEntityID);
				
				chainedEventEntity.setLockedGlossPhonAlignments(isLockedGlossPhonAlignments);
				chainedEventEntity.setLockedDomNdomAlignments(isLockedDomNdomAlignments);

				chainedEventEntity.setID(entity.getID());
				chainedEventEntity.setRefEntityId(entity.getRefEntityId());
				//chainedEventEntity.setReference(entity.getReference());
	            if (entity.getParentEntityIDs() == null || entity.getParentEntityIDs().isEmpty()) {
	            	chainedEventEntity.updateParentEntity();
	            } else {
	            	chainedEventEntity.setParentEntities(getParentField().getEntities(entity.getParentEntityIDs()));
	            }
	            
	            chainedEventEntity.setPrimaryEntry(entity.getPrimaryEntry());

//				if (entity.isValueStartTimeLocked() && entity.isValueEndTimeLocked()) {
//					chainedEventEntity.setLockedDomNdomAlignments(true);
//				}
				
//				|| entity.isInitialHoldStartTimeLocked())
//					chainedEventEntity.lockStart();
	//
//				if (entity.isValueEndTimeLocked() || entity.isFinalHoldEndTimeLocked())
//					chainedEventEntity.lockEnd();

				chainedEvents.add(chainedEventEntity);
			}
		}
// 
		// GlossChainedEventsEntityCollection 
		collection = new GlossChainedEventsEntityCollection(chainedEvents, segmentPanel);
//		ArrayList<GlossChainedEventsEntity> entities, segmentPanel);		
// 		ChainedEventsEntityCollection(chainedEvents, segmentPanel);
// 		collection.updateGlossEntities();

		// set compound mappings
		for (int c = 0; c < entities.size(); c++) {
			SS3Entity ent = entities.get(c);
			String sCmpEntID = ent.getStartCompoundEntityID();
			String eCmpEntID = ent.getEndCompoundEntityID();

			if (Util.isNotEmpty(sCmpEntID) || Util.isNotEmpty(eCmpEntID)) {
				GlossChainedEventsEntity mEnt = collection.getEntity(ent.getID());
				if (mEnt != null) {
					if (Util.isNotEmpty(sCmpEntID)) {
						GlossChainedEventsEntity sEnt = collection.getEntity(sCmpEntID);
						mEnt.setStartCompound(sEnt);
					}
					if (Util.isNotEmpty(eCmpEntID)) {
						GlossChainedEventsEntity eEnt = collection.getEntity(eCmpEntID);
						mEnt.setEndCompound(eEnt);
					}
				}
			}
		}

		EnteredText entText = new EnteredText(segmentPanel, this);
		for(SS3Entity textOnlyEntity: txtOnly.keySet()) {
			SS3Item txtOnlyItem = txtOnly.get(textOnlyEntity);
			Text text = new Text(textOnlyEntity.getID(), txtOnlyItem.getId(), txtOnlyItem.getText());
			text.setStartTime(textOnlyEntity.getStartTimeInfo().doubleValue());
			text.setEndTime(textOnlyEntity.getEndTimeInfo().doubleValue());
			entText.addText(text);
		}
		
		collection.setEnteredTextObject(entText);
	}

	@Deprecated	// legacy code involving Track, not SS3Track
	private void createTrackEntity(Track track) {
//		ArrayList<GlossChainedEventsEntity> chainedEvents = new ArrayList<GlossChainedEventsEntity>();
//		ArrayList trackValues = track.getTrackValues();
//
//		ArrayList<TrackValue> values = new ArrayList<TrackValue>();
//		HashMap<String, TrackValue> onset = new HashMap<String, TrackValue>();
//		HashMap<String, TrackValue> onsetHold = new HashMap<String, TrackValue>();
//		HashMap<String, TrackValue> offset = new HashMap<String, TrackValue>();
//		HashMap<String, TrackValue> offsetHold = new HashMap<String, TrackValue>();
//
//		for (int i = 0; i < trackValues.size(); i++) {
//			TrackValue trackValue = (TrackValue) trackValues.get(i);
//			int valueId = trackValue.getValueId();
//
//			if (valueId == Constants.GLOSS_HOLD_CODE) {
//				double startFrame = trackValue.getStartTimeInfo().doubleValue();
//				double endFrame = trackValue.getEndTimeInfo().doubleValue();
//				onsetHold.put("" + endFrame, trackValue);
//				offsetHold.put("" + startFrame, trackValue);
//			} else if (valueId == Constants.GLOSS_ONSET_CODE) {
//				double endFrame = trackValue.getEndTimeInfo().doubleValue();
//				onset.put("" + endFrame, trackValue);
//			} else if (valueId == Constants.GLOSS_OFFSET_CODE) {
//				double startFrame = trackValue.getStartTimeInfo().doubleValue();
//				offset.put("" + startFrame, trackValue);
//			} else {
//				values.add(trackValue);
//			}
//		}
//
//		for (TrackValue trackValue : values) { 
//			double startingFrame = trackValue.getStartTimeInfo().doubleValue();
//			double endingFrame = trackValue.getEndTimeInfo().doubleValue();
//
//			TrackValue onsetHoldValue = onsetHold.get(""
//					+ startingFrame);
//			TrackValue offsetHoldValue = offsetHold.get(""
//					+ endingFrame);
//			TrackValue onsetValue = null, offsetValue = null;
//
//			if (onsetHoldValue == null) {
//				onsetValue = onset.get("" + startingFrame);
//			} else {
//				onsetValue = onset.get(""
//						+ onsetHoldValue.getStartTimeInfo().doubleValue());
//			}
//
//			if (offsetHoldValue == null) {
//				offsetValue = onset.get("" + endingFrame);
//			} else {
//				offsetValue = onset.get(""
//						+ offsetHoldValue.getEndTimeInfo().doubleValue());
//			}
//
//			GlossChainedEventsCollection collection = new GlossChainedEventsCollection();
//			collection.setOnsetEvent(new ChainedEvent(onsetValue, segmentPanel));
//			collection.setInitialHoldEvent(new ChainedEvent(onsetHoldValue, segmentPanel));
//			collection.setTextValueEvent(new GlossChainedEvent(trackValue, segmentPanel));
//			collection.setFinalHoldEvent(new ChainedEvent(offsetHoldValue, segmentPanel));
//			collection.setOffsetEvent(new ChainedEvent(offsetValue, segmentPanel));
//
//			GlossChainedEventsEntity chainedEvent = new GlossChainedEventsEntity(collection, segmentPanel, this);
//			chainedEvent.determineParentEntity();
//
//			chainedEvents.add(chainedEvent);
//		}
//
//		collection = new GlossChainedEventsEntityCollection(chainedEvents, segmentPanel); 
	}

	@Override
	public void paint(Graphics2D g2d, double Y, Font font) {
		this.Y = Y;
		
		ArrayList<GlossChainedEventsEntity> glossEntities = collection.getEntities();
		
		for(GlossChainedEventsEntity entity : glossEntities) {
			entity.paint(g2d, Y, font);
		}

		if (selectedCompoundStartCoord > -1 && selectedCompoundEndCoord > -1) {
			g2d.setColor(Palette.SELECTED_BACKGROUND_COLOR);
			g2d.fillRect(selectedCompoundStartCoord, (int) (Y - 13),
					selectedCompoundEndCoord - selectedCompoundStartCoord, 19);
		}

		try {
			getGlossChainedEventsEntityCollection().getEnteredText().paint(g2d, Y, font);
		} catch (NullPointerException ex) {
			;
		}
	}
	
    @Override
    public boolean isIntervalEmpty(Interval interval) {
        return collection.getEntitiesInInterval(interval).isEmpty();
    }
}
