package edu.bu.signstream.grepresentation.fields.glossField;

import java.awt.*;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.logging.Level;

import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JTextField;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;

import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.grepresentation.fields.TextEventsEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.GestureInfoDialog;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.MorphologicalPhonologicalInfoDialog;
import edu.bu.signstream.time.Rounding;
import edu.bu.signstream.time.TimeRange;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEvent;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.nonManualField.NonManualFieldWrapper;
import edu.bu.signstream.grepresentation.fields.nonManualField.mouseDelegateListeners.DeleteTextActionListener;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.SegmentGraphicRepresentation;
import edu.bu.signstream.grepresentation.view.SegmentTemplate;
import edu.bu.signstream.grepresentation.view.main.SegmentToolBarPanel;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.main.ToolBarPanels;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolBar;
import edu.bu.signstream.ui.SS3SignStreamApplication;
import edu.bu.signstream.ui.popup.TextEntryPopupGenerator;

/**
 * Gloss fields contain time-aligned text. Predefined gloss fields: the main
 * gloss field and the non-dominant hand gloss field. These can be used for
 * coding activity of the dominant and non-dominant hands, respectively.Those
 * two fields are always displayed.
 * 
 * Gloss field corresponds with Hand field and Hand Shape field.
 * */

public class GlossFieldWrapper extends NonManualFieldWrapper {
	protected GlossField glsField;

	private GlossMouseDblRightClickedDelegate rDblClickDelegate;

	public GlossFieldWrapper(GlossField glsField,
			SignStreamSegmentPanel segmentPanel, TimeRange timeRange) {
		super(glsField, segmentPanel, timeRange);
		this.glsField = glsField;
	}

	public GlossField getGlossField() {
		return glsField;
	}

	public void mouseCtrlAltClicked(Point point) {
		EventsEntity ent = glsField.getEntityAt(point.x);
		ChainedEventsEntity entity = (ChainedEventsEntity) ent;
		Event event = glsField.getEventAt(point.x);
		ChainedEvent cEvent = (ChainedEvent) event;

        if (Arrays.stream(Window.getWindows())
            .anyMatch(w -> w.isVisible() &&
                (w instanceof MorphologicalPhonologicalInfoDialog ||
                    w instanceof GestureInfoDialog)))
            return;

        if (ent == null || entity == null)
			return;

		SS3Singleton.getMediaToolBar().select(cEvent, entity);

        SegmentTemplate template = segmentPanel.getUtteranceView()
                .getSegmentTemplate(field.getSegmentID());

		if (rDblClickDelegate == null) {
			rDblClickDelegate = new GlossMouseDblRightClickedDelegate(glsField,
					this, template.getSignView());
		}

		rDblClickDelegate.controlAltDblClick(cEvent, this, template.getSignView());
	}

	@Override
	public boolean contains(int Y) {
		return field.contains(Y);
	}

	@Override
	public void mouseClicked(Point point, JPanel panel) {

		ToolBarPanels panels = segmentPanel.getToolBarPanels();
		MediaToolBar mediaTb = panels.getMediaToolBar();

		ChainedEventsEntity entity = (ChainedEventsEntity) glsField.getEntityAt(point.x);
		Event event = glsField.getEventAt(point.x);

		if (entity != null && event == null) {
			ChainedEventsEntity correctEntity = null;
			ArrayList array = glsField.getEntities();
			for (int i = 0; i < array.size(); i++) {
				ChainedEventsEntity ent = (ChainedEventsEntity) array.get(i);
				if (ent.contains(point.x)) {
					Event sEvent = ent.getEventAt(point.x);
					if (sEvent != null) {
						correctEntity = ent;
						event = sEvent;

						ChainedEvent txtEvent = ent.getTextValue();

						if (point.x < txtEvent.getStartTimeCoordinate()) {
							ChainedEvent ciHold = correctEntity
									.getInitialHold();
							ChainedEvent iHold = entity.getInitialHold();
							if (ciHold != null
									&& iHold != null
									&& ciHold.getStartTimeCoordinate() == iHold
											.getStartTimeCoordinate()) {
								entity.setInitialHoldEvent(null);
								entity = correctEntity;
							} else {
								ChainedEvent cOnset = correctEntity.getOnset();
								ChainedEvent onset = entity.getOnset();
								if (cOnset != null
										&& onset != null
										&& cOnset.getStartTimeCoordinate() == onset
												.getStartTimeCoordinate()) {
									entity.setOnsetEvent(null);
									entity = correctEntity;
								}
							}
						} else if (point.x > txtEvent.getEndTimeCoordinate()) {
							ChainedEvent cfHold = correctEntity.getFinalHold();
							ChainedEvent fHold = entity.getFinalHold();
							if (cfHold != null
									&& fHold != null
									&& cfHold.getStartTimeCoordinate() == fHold
											.getStartTimeCoordinate()) {
								entity.setFinalHoldEvent(null);
								entity = correctEntity;
							} else {
								ChainedEvent cOffset = correctEntity
										.getOffset();
								ChainedEvent offset = entity.getOffset();
								if (cOffset != null
										&& offset != null
										&& cOffset.getStartTimeCoordinate() == offset
												.getStartTimeCoordinate()) {
									entity.setOnsetEvent(null);
									entity = correctEntity;
								}
							}
						}
					}
				}
			}
		}
		if (event != null && entity != null) {
		    mediaTb.select(event, entity);
//		} else if (entity != null) {
//			event = entity.getEventAt(point.x);
		} else {
			EnteredText enteredText = null;

			try {
				enteredText = glsField.getGlossChainedEventsEntityCollection()
						.getEnteredText();
				
				enteredText.unselectAll();
				Text text = enteredText.getTextAt(point.x);

				if (text != null) {
                    var hack = new TextEventsEntity(text, glsField);
                    mediaTb.select(hack, hack);
//                    text.setSelected(true);
//					mediaTb.selectEvent(null, null, glsField);
				}
			} catch (NullPointerException ex) {
                SS3Singleton.getSignStreamApplication().logger.log(Level.WARNING, ex.getMessage());
			}
		}
		panel.repaint();
	}

	@Override
	public void mouseRightClicked(Point point) {
	}

	@Override
	public void mouseLeftClicked(Point point) {
	}

	@Override
	public void mouseDblLeftClicked(Point point, JPanel panel) {
		// 2016-11-07 removing all the below to disallow renaming glosses from the main window - must be done via Morph-Phon window.
		// with only one mechanism to change it, we can better control only "sanctioned" entries from SignBank
	}

	public void mousePressed(Point point) {

		EnteredText enteredText = null;

		try {
			enteredText = glsField.getGlossChainedEventsEntityCollection()
					.getEnteredText();

			Text text = enteredText.getTextAt(point.x);

			if (text != null) {
				ToolBarPanels panels = segmentPanel.getToolBarPanels();
				MediaToolBar mediaTb = panels.getMediaToolBar();

				var hack = new TextEventsEntity(text, glsField);
				mediaTb.select(hack, hack);
//				text.setSelected(true);
//				mediaTb.selectEvent(null, null, glsField);
			}
		} catch (NullPointerException ex) {
            SS3Singleton.getSignStreamApplication().logger.log(Level.WARNING, ex.getMessage());
        }
	}

	@Override
	public void mouseDblRightClicked(final Point point, final JPanel panel) {
		EventsEntity ent = glsField.getEntityAt(point.x);

        if (Arrays.stream(Window.getWindows())
            .anyMatch(w -> w.isVisible() &&
                (w instanceof MorphologicalPhonologicalInfoDialog ||
                    w instanceof GestureInfoDialog)))
            return;

		if (ent != null) {
			GlossChainedEventsEntity entity = (GlossChainedEventsEntity) ent;

			if (rDblClickDelegate == null) {
				SegmentTemplate template = segmentPanel.getUtteranceView()
						.getSegmentTemplate(field.getSegmentID());

				rDblClickDelegate = new GlossMouseDblRightClickedDelegate(
						glsField, this, template.getSignView());
			}

			if (isEntitySelected()) {
				rDblClickDelegate.entityDblClick(entity, point, panel);
			} else {
				Event event = glsField.getEventAt(point.x);
				ChainedEvent cEvent = (ChainedEvent) event;
				rDblClickDelegate.dblClick(cEvent, entity, point, panel);
			}
			segmentPanel.getUtteranceView().repaint();
		} else {

			try {
				EnteredText enteredText = glsField
						.getGlossChainedEventsEntityCollection()
						.getEnteredText();

				enteredText.unselectAll();
				Text selectedText = enteredText.getTextAt(point.x);

				if (selectedText != null) {
					selectedText.setSelected(true);

					SegmentTemplate template = segmentPanel.getUtteranceView()
							.getSegmentTemplate(field.getSegmentID());
					SegmentGraphicRepresentation rep = template.getSignView()
							.getSegmentGraphicRep();

					// user should be able to delete text
					final JPopupMenu popup = new JPopupMenu();
					JMenuItem delete = new JMenuItem("Delete");
					UserInterfaceUtil.setMenuItemLNF(delete);
					delete.addActionListener(new DeleteTextActionListener(
							selectedText, enteredText, rep));
					popup.add(delete);

					JMenuItem cancel = new JMenuItem("Cancel");
					UserInterfaceUtil.setMenuItemLNF(cancel);
					popup.add(cancel);

					popup.setPreferredSize(popup.getPreferredSize());
					popup.show(panel, point.x, point.y);
					popup.setVisible(true);
					
					SS3Singleton.getSignStreamApplication().forcePopupInvisibleOnWindowLostFocus(popup);

					segmentPanel.getUtteranceView().repaint();
				}
			} catch (NullPointerException e) {
				e.printStackTrace();
				SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", e);
			}
		}
	}

	@Override
	public void mouseReleased(Point point, final JPanel panel) {

		if (glsField.getSelectedCompoundStartCoord() != -1
				&& glsField.getSelectedCompoundEndCoord() != -1) {
			ArrayList list = glsField.getEventsToCreateCompound();

			if (list != null && list.size() > 0) {
				ToolBarPanels tbPanels = segmentPanel.getToolBarPanels();
				SegmentToolBarPanel stbp = tbPanels.getSegmentToolBarPanel();
				stbp.activateCompoundBn(list, glsField);
			}
		}

		EnteredText enteredTxt = glsField
				.getGlossChainedEventsEntityCollection().getEnteredText();

		if (enteredTxt != null) {
			enteredTxt.selectTextBetweenSelectedEvents();
			enteredTxt.setSelectedSegment(false);
		}
	}

	@Override
	public void mouseDragged(Point point, JPanel panel) {
        var time = TimeRange.ABSOLUTE.time(point.x, TimeUnit.COORDINATE)
            .in(Util.getCurrentRange(getField()), Rounding.FLOOR)
            .clamp(SS3Singleton.getMediaPlayer().interval());

        SS3Singleton.getMediaPlayer().seek(time);

        EnteredText enteredText = null;

		try {
			enteredText = glsField.getGlossChainedEventsEntityCollection()
					.getEnteredText();
			Text text = enteredText.getTextAt(point.x);

			if (text != null) {
				MediaToolBar mb = SS3Singleton.getMediaToolBar(segmentPanel);
//				Event event = mb.selectedEvent();
//				EventsEntity entity = mb.selectedEntity();
//
//				if (event != null) {
//					event.unselect();
//					event.unhighlight();
//				}
//
//				if (entity != null) {
//					entity.unselect();
//					entity.unhighlight();
//				}
				// GD 2023-03-01 commenting, as the new way of doing select/deselect is in MediaToolBar, and these ways cause problems when gloss/other windows are open

				var hack = new TextEventsEntity(text, glsField);
				mb.select(hack, hack);
			}
		} catch (NullPointerException e) {
			e.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", e);
		}
		return;
	}

	// Point point, final JPanel panel
	@Override
	public void mousePressedAlt(Point point, JPanel panel) {
		EventsEntity entityAt = glsField.getEntityAt(point.x);
		if (entityAt != null)
			return;
		
		Text textAt = glsField.getGlossChainedEventsEntityCollection().getEnteredText().getTextAt(point.x);
		if (textAt != null)
			return;

        SS3Singleton.getMediaToolBar().deselect();

        var time = TimeRange.ABSOLUTE.time(point.x, TimeUnit.COORDINATE)
            .in(Util.getCurrentRange(getField()), Rounding.FLOOR)
            .clamp(SS3Singleton.getMediaPlayer().interval());

		Point loc = new Point((int)time.as(TimeUnit.COORDINATE, Rounding.NEAREST), field.getY() - 11);
		
		TextEntryPopupGenerator.getInstance("", panel, loc, label -> {
		    if (label.isBlank()) return;

            glsField.getGlossChainedEventsEntityCollection()
                .getEnteredText().addText(new Text(label, time));
			SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
            segmentPanel.repaint();
        }, () -> {});
	}

	@Override
	public void unselectAll() {
		super.unselectAll();

		GlossChainedEventsEntityCollection glsCollection = glsField
				.getGlossChainedEventsEntityCollection();
		ArrayList<GlossChainedEventsEntity> glsEntities = glsCollection
				.getEntities();
		for (GlossChainedEventsEntity entity : glsEntities) {
			entity.unselect();
		}

		try {
			glsField.getGlossChainedEventsEntityCollection().getEnteredText()
					.unselectAll();
			glsField.setSelectCompoundStartCoord(-1);
			glsField.setSelectCompoundEndCoord(-1);
		} catch (NullPointerException ex) {
			;
		}
	}
}

@Deprecated
class NewTextPopupMenuListener extends WindowAdapter {
	private ArrayList<Text> textInInterval = null;
	private double startCoord, endCoord;
	private GlossField glsField = null;
	private SignStreamSegmentPanel segmentPanel = null;
	private JTextField labelTextField = null;

	NewTextPopupMenuListener(JTextField labelTextField,
			ArrayList<Text> textInInterval, double startCoord, double endCoord,
			GlossField glsField, SignStreamSegmentPanel segmentPanel) {
		this.textInInterval = textInInterval;
		this.startCoord = startCoord;
		this.endCoord = endCoord;
		this.glsField = glsField;
		this.segmentPanel = segmentPanel;
		this.labelTextField = labelTextField;
	}

	private void removeListeners() {
		java.awt.event.KeyListener[] listeners = labelTextField
				.getKeyListeners();
		for (int i = 0; i < listeners.length; i++) {
			java.awt.event.KeyListener listener = listeners[i];
			labelTextField.removeKeyListener(listener);
		}

		labelTextField = null;

		System.gc();
	}

	@Override
	public void windowClosing(WindowEvent e) {
		removeListeners();
	}

	@Override
	public void windowClosed(WindowEvent e) {
//		glsField.getGlossChainedEventsEntityCollection().getEnteredText()
//        	.synchronize(textInInterval, startCoord, endCoord);
	    removeListeners();
	    segmentPanel.repaint();
	}
}

class UpdateTextPopupMenuListener implements PopupMenuListener {
	private JTextField labelTextField = null;

	UpdateTextPopupMenuListener(JTextField labelTextField) {
		this.labelTextField = labelTextField;
	}

	@Override
	public void popupMenuCanceled(PopupMenuEvent popupMenuEvent) {
		removeListeners(popupMenuEvent);
	}

	@Override
	public void popupMenuWillBecomeInvisible(PopupMenuEvent popupMenuEvent) {
		removeListeners(popupMenuEvent);
	}

	private void removeListeners(PopupMenuEvent popupMenuEvent) {
		java.awt.event.KeyListener[] listeners = labelTextField
				.getKeyListeners();
		for (int i = 0; i < listeners.length; i++) {
			java.awt.event.KeyListener listener = listeners[i];
			labelTextField.removeKeyListener(listener);
		}

		labelTextField = null;

		JPopupMenu menu = (JPopupMenu) popupMenuEvent.getSource();
		menu.removePopupMenuListener(this);

		System.gc();
	}

	@Override
	public void popupMenuWillBecomeVisible(PopupMenuEvent popupMenuEvent) {
		;
	}
}