package edu.bu.signstream.grepresentation.fields.glossField;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;
import javax.swing.event.MenuEvent;
import javax.swing.event.MenuListener;

import edu.bu.signstream.common.util.CoordinateTimeConverter;
import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.MorphologicalPhonologicalInfoDialog;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEvent;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.nonManualField.GlossEntityProperties;
import edu.bu.signstream.grepresentation.fields.nonManualField.NonManualFieldWrapper;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.GlossNRelatedView;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolBar;
import edu.bu.signstream.ui.video.MultipleMovieController;

public class GlsDblLeftClickPopup implements ActionListener, MenuListener {

	final JPopupMenu popup = new JPopupMenu();

	private JMenu assocFldsMenu = null;
	private JMenu compoundsMenu = null;
	private JMenu assocEventsMenu = null;
	private JMenuItem color = null;
	private JMenuItem deleteMenu = null; 

	private JMenuItem defAssocFlds = null;
	private JMenuItem lockAssocFlds = null;

	private JMenuItem addOnset = new JMenuItem("Onset");
	private JMenuItem addInitialHold = new JMenuItem("Initial Hold");
	private JMenuItem addFinalHold = new JMenuItem("Final Hold");
	private JMenuItem addOffset = new JMenuItem("Offset");

	private JMenuItem previousCompaund = new JMenuItem("");
	private JMenuItem followingCompound = new JMenuItem("");

	private final int ASSOCIATED_FIELDS = 0;
	private final int LOCK_ASSOCIATED_FIELDS = 1;
	private final int ONSET = 2;
	private final int INITIAL_HOLD = 3;
	private final int FINAL_HOLD = 4;
	private final int OFFSET = 5;
	private final int PREVIOUS_COMPOUND = 6;
	private final int FOLLOWING_COMPOUND = 7;
	private final int DELETE_MENU = 8;
	private final int DELETE_OK = 9;
	private final int DELETE_CANCEL = 10;
	private final int COLOR = 11; 
	
	private JPopupMenu deleteEntityDialog = null;
	
	 private MorphologicalPhonologicalInfoDialog assocFieldsDialog = null;
	
	private JButton deleteEntityBtn = new JButton("Proceed");
	private JButton cancelDeleteEntityBtn = new JButton("Cancel");
	
	//private GlossNRelatedView glsView = null;
	
	private ChainedEvent value = null;
	private GlossChainedEventsEntity entity = null;
	
	private JPanel panel = null; 
	private Point point = null;

	private GlossFieldWrapper wrapper = null;
	private GlossNRelatedView glsView = null;
	
	private JColorChooser colorChooser = new JColorChooser();
	private JDialog colorChooserDialog = null;
	
	public GlsDblLeftClickPopup(GlossFieldWrapper wrapper, GlossNRelatedView glsView) {
		this.wrapper = wrapper;
		this.glsView = glsView;

		assocFldsMenu = new JMenu("Associated Fields");
		popup.add(assocFldsMenu);

		defAssocFlds = new JMenuItem("Define Associated Fields");
		defAssocFlds.setActionCommand("" + ASSOCIATED_FIELDS);
		defAssocFlds.addActionListener(this);
		assocFldsMenu.add(defAssocFlds);

		lockAssocFlds = new JMenuItem("Lock Associated Fields");
		lockAssocFlds.setActionCommand("" + LOCK_ASSOCIATED_FIELDS);
		lockAssocFlds.addActionListener(this);
		assocFldsMenu.add(lockAssocFlds);

		compoundsMenu = new JMenu("Compounds");
		popup.add(compoundsMenu);

		assocEventsMenu = new JMenu("Associated Elements");
		popup.add(assocEventsMenu);

		addOnset.setActionCommand("" + ONSET);
		addInitialHold.setActionCommand("" + INITIAL_HOLD);
		addFinalHold.setActionCommand("" + FINAL_HOLD);
		addOffset.setActionCommand("" + OFFSET);
		previousCompaund.setActionCommand("" + PREVIOUS_COMPOUND);
		followingCompound.setActionCommand("" + FOLLOWING_COMPOUND);

		assocEventsMenu.addMenuListener(this);
		addOnset.addActionListener(this);
		addInitialHold.addActionListener(this);
		addFinalHold.addActionListener(this);
		addOffset.addActionListener(this);

		assocEventsMenu.add(addOnset);
		assocEventsMenu.add(addInitialHold);
		assocEventsMenu.add(addFinalHold);
		assocEventsMenu.add(addOffset);

		compoundsMenu.addMenuListener(this);
		previousCompaund.addActionListener(this);
		followingCompound.addActionListener(this);

		color = new JMenuItem("Color");
		color.setActionCommand(""+COLOR);
		color.addActionListener(this); 
		popup.add(color);

		deleteMenu = new JMenuItem("Delete");
		deleteMenu.setActionCommand(""+DELETE_MENU);
		deleteMenu.addActionListener(this);
		popup.add(deleteMenu);

		JMenuItem cancel = new JMenuItem("Cancel");
		popup.add(cancel);
	}

	@Override
	public void menuSelected(MenuEvent e) {
		if (e.getSource().equals(assocEventsMenu)) {
			if (entity.getOnset() == null) {
				addOnset.setEnabled(true);
			} else {
				addOnset.setEnabled(false);
			}

			if (entity.getInitialHold() == null) {
				addInitialHold.setEnabled(true);
			} else {
				addInitialHold.setEnabled(false);
			}

			if (entity.getFinalHold() == null) {
				addFinalHold.setEnabled(true);
			} else {
				addFinalHold.setEnabled(false);
			}

			if (entity.getOffset() == null) {
				addOffset.setEnabled(true);
			} else {
				addOffset.setEnabled(false);
			}
		} else if (e.getSource().equals(compoundsMenu)) {
			
			GlossChainedEventsEntity prevComp = entity.getPreviousEntity();
			GlossChainedEventsEntity nextComp = entity.getNextEntity();

			compoundsMenu.remove(previousCompaund);
			compoundsMenu.remove(followingCompound);

			if (prevComp != null) {
				compoundsMenu.add(previousCompaund);
				if (entity.isStartCompound()) {
					previousCompaund
							.setText("Unlink " + prevComp.getText(true));
				} else {
					previousCompaund.setText("Link " + prevComp.getText(true));
				}
			}

			if (nextComp != null) {
				compoundsMenu.add(followingCompound);
				if (entity.isEndCompound()) {
					followingCompound.setText("Unlink "
							+ nextComp.getText(false));
				} else {
					followingCompound
							.setText("Link " + nextComp.getText(false));
				}
			}
		} 
	}

	private void showDeleteObject() {

		if (deleteEntityDialog == null) {		
			deleteEntityBtn.setActionCommand("" + DELETE_OK);
			cancelDeleteEntityBtn.setActionCommand("" + DELETE_CANCEL);

			deleteEntityBtn.addActionListener(this);
			cancelDeleteEntityBtn.addActionListener(this);

			deleteEntityDialog = new JPopupMenu();

			JLabel msgLabel = new JLabel(
					"Are you sure you want to delete the object?");
			UserInterfaceUtil.setLabelLNF(msgLabel);

			JPanel msgPanel = new JPanel();
			GridBagConstraints gbc = new GridBagConstraints();

			msgPanel.setLayout(new GridBagLayout());
			gbc.anchor = GridBagConstraints.WEST;

			gbc.gridwidth = 3;
			gbc.gridx = 0;
			gbc.gridy = 0;
			msgPanel.add(msgLabel, gbc);

			gbc.gridy = 1;
			gbc.gridx = 1;
			msgPanel.add(new JLabel(" "), gbc);

			gbc.gridwidth = 1;
			gbc.anchor = GridBagConstraints.EAST;
			gbc.gridy = 3;
			gbc.gridx = 0;
			msgPanel.add(new JLabel(" "), gbc);
			gbc.gridx = 1;
			msgPanel.add(deleteEntityBtn, gbc);
			gbc.gridx = 2;
			msgPanel.add(cancelDeleteEntityBtn, gbc);

			SwingUtilities.invokeLater(new Runnable() {
				@Override
				public void run() {
					deleteEntityBtn.requestFocus();
				}
			});

			int width = msgPanel.getPreferredSize().width + 30;
			int height = msgPanel.getPreferredSize().height + 20;

			deleteEntityDialog.add(msgPanel);
			deleteEntityDialog.setPopupSize(new Dimension(width, height));
		}
		
		deleteEntityDialog.show(panel, point.x, point.y);
		deleteEntityDialog.setVisible(true);
		
		SS3Singleton.getSignStreamApplication().forcePopupInvisibleOnWindowLostFocus(deleteEntityDialog);
	}

	@Override
	public void menuDeselected(MenuEvent arg0) {
	}

	@Override
	public void menuCanceled(MenuEvent arg0) {
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		String str = e.getActionCommand();

		MediaToolBar mediaToolBar = SS3Singleton.getMediaToolBar(null);
		SignStreamSegmentPanel segmentPanel = SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel();
				//mediaToolBar.getSignStreamSegmentPanel();
		MultipleMovieController controller = segmentPanel
				.getMultipleMovieController();
		CoordinateTimeConverter convertor = segmentPanel.getZoomer()
				.getCoordinateTimeConvertor();

		EventsEntity selectedEntity = mediaToolBar.selectedEntity();

		switch (Integer.parseInt(str)) {
		case ONSET:
			Time minOnsetStartTimeInfo = controller.getPrecedingFrameTimeInfo(entity
					.getStartTimeInfo());
			double fStartTimeCoord = convertor
					.convertTimeInfoToCoordinate(minOnsetStartTimeInfo);

			if (wrapper.getField().getEventAt(fStartTimeCoord) == null) {
				entity.addOnsetEvent();
				entity.unselect();
				entity.getOnset().select();
				segmentPanel.repaint();

				mediaToolBar.selectEvent(entity.getOnset(), entity,
						wrapper.getField());

				SS3Singleton.getApplicationStateController()
				.setLoadedCollectionUpdated(true);
			} else {
				String msg = "Onset could not be added.";
				SS3Singleton.getSignStreamApplication()
				.confirmationDialog("Warning", msg);
			}
			break;
		case INITIAL_HOLD:
			if (entity.getOnset() != null) {
				Event onset = entity.getOnset();
				// int onsetDuration = onset.getEndTimeInfo().getMovieTime() -
				double onsetStartTime = onset.getStartTimeInfo().doubleValue();
				double onsetEndTime = onset.getEndTimeInfo().doubleValue();
				double onsetDuration = onsetEndTime - onsetStartTime;

				double frames = onsetDuration / controller.getPacketDuration();
				if (frames > 1) {
					// add initial hold
					entity.addInitialHoldEvent();
					ChainedEvent initialHold = entity.getInitialHold();
					onset.setEndTimeInfo(initialHold.getStartTimeInfo());
					entity.unselect();
					entity.getInitialHold().select();
					segmentPanel.repaint();

					mediaToolBar.selectEvent(initialHold, entity,
							wrapper.getField());
				} else {
					String msg = "Initial Hold could not be added. Adjust Onset start time and try again.";
					SS3Singleton.getSignStreamApplication()
							.confirmationDialog("Warning", msg);
				}
			} else {
				Time startValueTimeInfo = entity.getValueOrTextValueEvent()
						.getStartTimeInfo();
				Time startInitialHoldTimeInfo = controller
						.getPrecedingFrameTimeInfo(startValueTimeInfo);

				// add initial hold
				entity.addInitialHoldEvent();

				ChainedEvent initialHold = entity.getInitialHold();

				entity.unselect();
				initialHold.select();
				segmentPanel.repaint();

				mediaToolBar.selectEvent(initialHold, entity,
						wrapper.getField());
			}

			break;
		case FINAL_HOLD:
			StringBuffer warningMsg = new StringBuffer();
			entity.addFinalHoldEvent();
			entity.unselect();
			entity.getFinalHold().select();

//			entity.notifyObservers(entity.getFinalHold(),
//					EntityStateConstants.ADD_FINAL_HOLD, warningMsg);

			mediaToolBar.selectEvent(entity.getFinalHold(), entity,
					wrapper.getField());

			SS3Singleton.getApplicationStateController()
					.setLoadedCollectionUpdated(true);

			segmentPanel.repaint();

			if (warningMsg.length() > 0) {
				SS3Singleton
						.getSignStreamApplication()
						.confirmationDialog("Warning",
								"Final Hold could not be added.");
			}
			break;
		case OFFSET:

			double fEndTime = entity.getEndTimeInfo().doubleValue() + 2
					* convertor.getMovieFrameDuration();
			double fEndTimeCoord = convertor.convertTimeToCoordinate(fEndTime);

			if (wrapper.getField().getEventAt(fEndTimeCoord) == null) {

				entity.addOffsetEvent();
				entity.unselect();
				entity.getOffset().select();
				wrapper.getField().getSignStreamSegmentPanel().repaint();

				MediaToolBar tb = SS3Singleton.getSignStreamApplication()
						.getSignStreamSegmentPanel().getToolBarPanels()
						.getMediaToolBar();
				SS3Singleton.getApplicationStateController()
						.setLoadedCollectionUpdated(true);
				tb.selectEvent(entity.getOffset(), entity, wrapper.getField());

			} else {
				String msg = "Offset could not be added.";
				SS3Singleton.getSignStreamApplication()
						.confirmationDialog("Warning", msg);
			}
			break;
		case PREVIOUS_COMPOUND:
			GlossChainedEventsEntity prevComp = entity.getPreviousEntity();

			if (prevComp != null) {
				if (previousCompaund.getText().startsWith("Link")) {
					prevComp.setEndCompound(entity);
					entity.setStartCompound(prevComp);
				} else {
					prevComp.setEndCompound(null);
					entity.setStartCompound(null);
				}

				SS3Singleton.getApplicationStateController()
						.setLoadedCollectionUpdated(true);
				wrapper.getField().getSignStreamSegmentPanel().repaint();
			}
			break;
		case FOLLOWING_COMPOUND:
			GlossChainedEventsEntity folComp = entity.getNextEntity();
			
			if (folComp != null) {
				if (followingCompound.getText().startsWith("Link")) {
					entity.setEndCompound(folComp);
					folComp.setStartCompound(entity);
				} else {
					folComp.setStartCompound(null);
					entity.setEndCompound(null);
				}

				SS3Singleton.getApplicationStateController()
						.setLoadedCollectionUpdated(true);
				wrapper.getField().getSignStreamSegmentPanel().repaint();
			}
			break;
			
		case DELETE_MENU:
			showDeleteObject();
			break;
			
		case DELETE_OK:
			String fieldID = entity.getField().getFieldID();

			if (fieldID.equalsIgnoreCase(Util.DOM_GLOSS_FIELD_ID)) {
				GlossEntityProperties glsProperties = entity
						.getGlossEntityProperties();
				if (glsProperties.isTwoHanded()) {
					ChainedEventsEntity cEntity = entity.getDependentGlossEntity();

//					if (cEntity != null) {
//						cEntity.update(cEntity, cEntity,
//								EntityStateConstants.DELETE,
//								new StringBuffer());
//					}
				}
			} else if (fieldID.equalsIgnoreCase(Util.NDOM_GLOSS_FIELD_ID)) {
				GlossChainedEventsEntity cEntity = entity.getDependentGlossEntity();

				if (cEntity != null) {
					GlossEntityProperties glsProperties = cEntity.getGlossEntityProperties(); 
					glsProperties.setTwoHanded(false);
				}
			}

//			entity.update(entity, entity, EntityStateConstants.DELETE,
//					new StringBuffer());
			SS3Singleton.getApplicationStateController()
					.setLoadedCollectionUpdated(true);
		
			deleteEntityDialog.setVisible(false);
			wrapper.getField().getSignStreamSegmentPanel().repaint();
			break;
			
		case DELETE_CANCEL:
			deleteEntityDialog.setVisible(false);
			break;
		
		case ASSOCIATED_FIELDS:
			if(assocFieldsDialog == null) {
				//	
				//SignStreamSegmentPanel panel = SS3Singleton.getManager().getSignStreamApplication().getSignStreamSegmentPanel(); 
				//GlossNRelatedView glsView = panel.getUtteranceView().getGlossNRelatedView();
				
				//	
				assocFieldsDialog = new MorphologicalPhonologicalInfoDialog(wrapper, glsView);
				assocFieldsDialog.setResizable(false);
				assocFieldsDialog.setVisible(true);
			} else {
				//GlossNRelatedView glsView = panel.getUtteranceView().getGlossNRelatedView();
				assocFieldsDialog.update(wrapper, glsView);
				assocFieldsDialog.setVisible(true);
			}			
			break;
		
		case LOCK_ASSOCIATED_FIELDS:
		   //   StringBuffer warningMsg1 = new StringBuffer();
		      SignStreamSegmentPanel panel = wrapper.getSignStreamSegmentPanel();
		      
		      GlossChainedEventsEntity entity = (GlossChainedEventsEntity)wrapper.getSelectedEntity();
		      GlossChainedEventsEntity ndEntity = entity.getDependentGlossEntity();
		     
//		      entity.notifyObservers(
//		         entity,
//		         EntityStateConstants.LOCK_START_TIME,
//		         warningMsg1);
//		      entity.notifyObservers(
//		         entity,
//		         EntityStateConstants.LOCK_END_TIME,
//		         warningMsg1);
		      
		      if(ndEntity != null) {
		    	  Event event = entity.getValueOrTextValueEvent();
		    	  ChainedEvent onset = entity.getOnset();
		    	  ChainedEvent initialHold = entity.getInitialHold();
		    	  ChainedEvent offset = entity.getOffset();
		    	  ChainedEvent finalHold = entity.getFinalHold();
		    	  
		    	  Event ndEvent = ndEntity.getValueOrTextValueEvent();
		    	  ChainedEvent ndOnset = ndEntity.getOnset();
		    	  ChainedEvent ndInitialHold = ndEntity.getInitialHold();
		    	  ChainedEvent ndOffset = ndEntity.getOffset();
		    	  ChainedEvent ndFinalHold = ndEntity.getFinalHold();   	  

		    	  ndEvent.setStartTimeInfo(event.getStartTimeInfo());
		    	  ndEvent.setEndTimeInfo(event.getEndTimeInfo());

		    	  if(initialHold != null) {
		    		  if(ndInitialHold != null) {
		    		      ndInitialHold.setStartTimeInfo(initialHold.getStartTimeInfo());
		    		      ndInitialHold.setEndTimeInfo(initialHold.getEndTimeInfo());
		    		  } else {
		    			  ndInitialHold = new ChainedEvent(panel);
                          ndInitialHold.setStartTimeInfo(initialHold.getStartTimeInfo());
                          ndInitialHold.setEndTimeInfo(initialHold.getEndTimeInfo());
		    			  ndEntity.setInitialHoldEvent(ndInitialHold);
		    		  }
		    	  } else {
		    		  ndEntity.setInitialHoldEvent(null);
		    	  }
		    	  
		    	  if(onset != null) {
		    		  if(ndOnset != null) {
		    		      ndOnset.setStartTimeInfo(onset.getStartTimeInfo());
		    		      ndOnset.setEndTimeInfo(onset.getEndTimeInfo());
		    		  } else {
		    			  ndOnset = new ChainedEvent(panel);
                          ndOnset.setStartTimeInfo(onset.getStartTimeInfo());
                          ndOnset.setEndTimeInfo(onset.getEndTimeInfo());
		    			  ndEntity.setOnsetEvent(ndOnset);
		    		  }
		    	  } else {
		    		  ndEntity.setOnsetEvent(null);
		    	  }
		    	  
		    	  if(finalHold != null) {
		    		  if(ndFinalHold != null) {
		    		      ndFinalHold.setStartTimeInfo(finalHold.getStartTimeInfo());
		    		      ndFinalHold.setEndTimeInfo(finalHold.getEndTimeInfo());
		    		  } else {
		    			  ndFinalHold = new ChainedEvent(panel);
                          ndFinalHold.setStartTimeInfo(finalHold.getStartTimeInfo());
                          ndFinalHold.setEndTimeInfo(finalHold.getEndTimeInfo());
		    			  ndEntity.setFinalHoldEvent(ndFinalHold);
		    		  }
		    	  } else {
		    		  ndEntity.setFinalHoldEvent(null);
		    	  }
		    	  
				// ndEntity.notifyObservers(
				// entity,
				// EntityStateConstants.LOCK_START_TIME,
				// warningMsg1);
				// ndEntity.notifyObservers(
				// entity,
				// EntityStateConstants.LOCK_END_TIME,
				// warningMsg1);
		          //entity.lockStart(); //.setLockedNdEntityStartTime(true);
		          //entity.lockEnd(); //.setLockedNdEntityEndTime(true);
		      }
		      
		      SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
		         
		      panel.repaint();

		      //if (warningMsg1.length() > 0) {
			//		SS3Singleton.getManager().getSignStreamApplication().confirmationDialog("Warning", warningMsg1.toString());
		     // }
			break;
		case COLOR:
			if(colorChooserDialog == null) {
				ChangeColorListener changeColorListener = new ChangeColorListener(colorChooser, wrapper);
				colorChooserDialog =
		         JColorChooser.createDialog(
		        		 SS3Singleton.getSignStreamApplication(),
		            "Pick A Color",
		            true,
		            colorChooser,
		            changeColorListener, null);
			      
			      Color color = value.getColor();
			      if (color == null) 
			         color = Color.DARK_GRAY;
			         
			      colorChooser.setColor(color);
			      colorChooserDialog.setVisible(true);
			} else {
			      Color color = value.getColor();
			      if (color == null) 
			         color = Color.DARK_GRAY;
			         
			      colorChooser.setColor(color);
			      colorChooserDialog.setVisible(true);
			}

			break;
		}
		
		selectedEntity.updateParentEntity();
	}

	public void destroy() {

	}

	public void setInvisible() {
		popup.setVisible(false);
	}

	public void setVisible(ChainedEvent value, GlossChainedEventsEntity entity,
			JPanel panel, Point point) {
		this.value = value;
		this.entity = entity;
		this.panel = panel; 
		this.point = point;
		popup.setPreferredSize(popup.getPreferredSize());
		popup.show(panel, point.x, point.y);
		popup.setVisible(true);
		
		SS3Singleton.getSignStreamApplication().forcePopupInvisibleOnWindowLostFocus(popup);
	}
}

class ChangeColorListener implements ActionListener {
	   private JColorChooser chooser;
	   private NonManualFieldWrapper wrapper;

	   public ChangeColorListener(JColorChooser chooser, NonManualFieldWrapper wrapper) {
	      this.chooser = chooser;
	      this.wrapper = wrapper;
	   }

	   @Override
	public void actionPerformed(ActionEvent e) {
	      Color color = chooser.getColor();
	      wrapper.getSelectedEvent().setColor(color);
	      wrapper.getField().getSignStreamSegmentPanel().repaint();
	      SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);	     
	   }
}
