package edu.bu.signstream.grepresentation.fields.glossField.dialog.handShapes;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.handshapeTypes.StartEndHandshapesType;
import edu.bu.signstream.grepresentation.listener.WeakActionListener;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.HandShapeBean;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.HandShapePanel;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.LettersHandshapesPalette;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapeImage;
import edu.bu.signstream.ui.ButtonPanel;

public class HandshapePalette extends JPanel implements ActionListener {

	private HandShapesPaletteFormBean formBean = null;
	private HandShapeBean bean = null;
	private GeneralHandshapePalette palette = null;
    private HandShapePanel hsPanel = null;

	private JDialog dialog = null;

	private final int ENTER = 0;
	private final int CANCEL = 1;

    private final JComboBox<String> options = new JComboBox<String>(new String[] {"ASL - All", "ASL - Letters", "ASL - Numbers", "ASL - Unmarked"});

	public HandshapePalette(HandShapesPaletteFormBean formBean,
			HandShapeBean bean, SS3CodingScheme scheme, JDialog dialog, int selectedIndex, HandShapePanel hsPanel) {

		this.formBean = formBean;
		this.bean = bean;
		this.dialog = dialog;
        this.hsPanel = hsPanel;

		palette = new GeneralHandshapePalette(formBean.getValues(),
				formBean.getSelectedHshImages(), scheme, selectedIndex, hsPanel);
		palette.setBorder(new TitledBorder(formBean.getTitle()));

//        options.addItemListener(new WeakItemListener(new ItemListener() {
//            @Override
//            public void itemStateChanged(ItemEvent itemEvent) {
//                int selectedIndex = options.getSelectedIndex();
//                palette.setPalette(selectedIndex);
//            }
//        }, options));

        options.addItemListener(new ItemListener() {
            @Override
			public void itemStateChanged(ItemEvent itemEvent) {
                int selectedIndex = options.getSelectedIndex();
                palette.setPalette(selectedIndex);
            }
        });

        options.setSelectedIndex(selectedIndex);

        setLayout(new BorderLayout());

        JPanel languagePanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        languagePanel.add(new JLabel("Palettes: "));
        languagePanel.add(new JComboBox<String>(new String[] {"American Sign Language"}));
        languagePanel.add(options);

        JButton enter = new JButton("Enter");
        JButton cancel = new JButton("Cancel");

        ButtonPanel buttonPanel = new ButtonPanel();
        buttonPanel.add(cancel);
        buttonPanel.add(enter);
        add(languagePanel, BorderLayout.NORTH);
        add(palette, BorderLayout.CENTER);
        add(buttonPanel, BorderLayout.SOUTH);

		enter.setActionCommand("" + ENTER);
		cancel.setActionCommand("" + CANCEL);
		enter.addActionListener(new WeakActionListener(this, enter));
		cancel.addActionListener(new WeakActionListener(this, cancel));

		enter.setSelected(true);
	}
	
	public void update(HandShapesPaletteFormBean formBean, HandShapeBean bean, int selectedIndex) {
		this.bean = bean;
		this.formBean = formBean;
        options.setSelectedIndex(selectedIndex);
        
        palette.setSelectedHshImages(formBean.getSelectedHshImages());       
        if (options.getSelectedIndex() == selectedIndex) {
            palette.setPalette(selectedIndex);
        } else {
            options.setSelectedIndex(selectedIndex);
        }

        palette.setBorder(new TitledBorder(formBean.getTitle()));
        palette.repaint();
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		String str = ae.getActionCommand();

		ArrayList<HandShapeImage> selectedHandshapes = palette.getSelectedHandshapes();

		switch (Integer.parseInt(str)) {
			case ENTER:
				ArrayList<HandShapeImage> hsImages = new ArrayList<HandShapeImage>(selectedHandshapes);

				// first check if this is a one-frame gloss, and if it is then disallow anything more than one handshape in either hand
				if (hsPanel.isOneFrameGloss()) {
					if (hsImages.size() > 1) {
						SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.SELECT_HANDSHAPES_EXACTLY_ONE);
						break;
					}
				} else if (hsPanel.getSelectedSameDifStartEndHandshapes() == StartEndHandshapesType.DIFFERENT && (hsImages.size() == 1)) {
					// conversely, if this is not a one-frame gloss and you have dif start/end selected,
					// then disallow a selection of just one handshape in either hand
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.SELECT_HANDSHAPES_MULTIPLE);
					break;
				}

				bean.setUpdated(true);

				if (formBean.getPaletteType() == HandShapesPaletteFormBean.PaletteType.RIGHT) {
					if (hsPanel.getAssociatedFieldsPanel().isDominantHandRight()) {
						bean.setSelectedDomHandShapeImages(selectedHandshapes);
					} else {
						bean.setSelectedNdomHandShapeImages(selectedHandshapes);
					}
				}

				if (formBean.getPaletteType() == HandShapesPaletteFormBean.PaletteType.LEFT) {
					if (hsPanel.getAssociatedFieldsPanel().isDominantHandRight()) {
						bean.setSelectedNdomHandShapeImages(selectedHandshapes);
					} else {
						bean.setSelectedDomHandShapeImages(selectedHandshapes);
					}
				}

				if (formBean.getPaletteType() == HandShapesPaletteFormBean.PaletteType.RIGHT_N_LEFT) {
					bean.setSelectedDomHandShapeImages(selectedHandshapes);
					bean.setSelectedNdomHandShapeImages(selectedHandshapes);
				}

				hsPanel.updateSelectedHandshapesPanels(bean.getSelectedDomHandShapeImages(),
													   bean.getSelectedNdomHandShapeImages());

				dialog.setVisible(false);
				break;

			case CANCEL:
				dialog.setVisible(false);
				break;
		}
	}
	
	public GeneralHandshapePalette getPalette() {
		return palette;
	}
}
