package edu.bu.signstream.grepresentation.fields.handShapeField;

import java.awt.Font;
import java.awt.Graphics2D;
import java.util.ArrayList;
import java.util.Iterator;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.vo.Track;
import edu.bu.signstream.common.util.vo.TrackValue;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Track;
import edu.bu.signstream.time.Bound;
import edu.bu.signstream.time.Interval;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.EventsIterator;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class HandShapeField extends Field {

	private static final long serialVersionUID = 1L;
	// collection of HandShapeEntities
	private ArrayList<HandShapeEntity> entities = new ArrayList<HandShapeEntity>();

	@Deprecated
	public HandShapeField(Track track, SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		super(segmentPanel, parentField);
		this.track = track;

		HandShapeEntity entity = new HandShapeEntity(segmentPanel, this);
		entity.setField(this);

		ArrayList values = track.getTrackValues();
		for (int i = 0; i < values.size(); i++) {
			TrackValue value = (TrackValue) values.get(i);
			entity.addHandShape(value);
		}
		entity.updateParentEntity();
		
		entities.add(entity);
	}

	public HandShapeField(SS3Track track, SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		super(segmentPanel, parentField);
		this.ss3Track = track;

		ArrayList ent = ss3Track.getEntities();
		for (int j = 0; j < ent.size(); j++) {
			Object obj = ent.get(j);

			SS3Entity entity = (SS3Entity) obj;
			HandShapeEntity hsEntity = new HandShapeEntity(track.getId(),
					entity, segmentPanel, this);
			hsEntity.setField(this);

			hsEntity.setID(entity.getID());
			hsEntity.setRefEntityId(entity.getRefEntityId());
			if (entity.getParentEntityIDs() == null && entity.getParentEntityIDs().isEmpty()) {
				hsEntity.updateParentEntity();
			} else {
				hsEntity.setParentEntities(getParentField().getEntities(entity.getParentEntityIDs()));
			}

			entities.add(hsEntity);
		}
	}

	@Override
	public ArrayList getEntities() {
		return entities;
	}

	public HandShapeEntity getNextHandShapeEntity(HandShapeEntity entity) {
		HandShapeEntity nextEntity = entity;
		double endTimeCoordToCompare = entity.getEndTimeCoordinate();

		for (int i = 0; i < entities.size(); i++) {
			HandShapeEntity ent = entities.get(i);
			double coord = ent.getStartTimeCoordinate();
			if (coord >= endTimeCoordToCompare
					&& coord < nextEntity.getStartTimeCoordinate()) {
				nextEntity = ent;
			}
		}
		if (nextEntity == entity)
			return null;
		return nextEntity;
	}

	public HandShapeEntity getPreviousHandShapeEntity(HandShapeEntity entity) {
		HandShapeEntity prevEntity = entity;
		double startTimeCoordToCompare = entity.getStartTimeCoordinate();

		deleteEmptyEntities();

		for (int i = 0; i < entities.size(); i++) {
			HandShapeEntity ent = entities.get(i);
			double coord = ent.getEndTimeCoordinate();
			if (coord <= startTimeCoordToCompare
					&& coord > prevEntity.getStartTimeCoordinate()) {
				prevEntity = ent;
			}
		}
		if (prevEntity == entity)
			return null;
		return prevEntity;
	}

	@Override
	public void deleteEmptyEntities() {
		for (int i = 0; i < entities.size(); i++) {
			HandShapeEntity entity = entities.get(i);
			
			if (entity.getEvents().isEmpty() || entity.getFirstHandShapeEvent() == null || entity.getLastHandShapeEvent() == null) { //  || entity.getFirstEvent() == null || entity.getLastEvent() == null)
				entities.remove(entity);
			}
		}
	}

	public EventsIterator getEventsIterator() {
		ArrayList events = new ArrayList();

		for (int i = 0; i < entities.size(); i++) {
			HandShapeEntity entity = entities.get(i);
			events.addAll(entity.getEvents());
		}

		return new EventsIterator(events);
	}

    @Override
    public boolean isIntervalEmpty(Interval interval) {
	    // JACK TODO: This should probably be called somewhere else, but it was here in the previous impl...
	    removeEmptyEntities();

	    return entities.stream()
            .noneMatch(e -> e.overlaps(interval));
    }

	private void removeEmptyEntities() {
		ArrayList<HandShapeEntity> entitiesToDelete = new ArrayList<HandShapeEntity>();

		Iterator iter = entities.listIterator();
		while (iter.hasNext()) {
			HandShapeEntity entity = (HandShapeEntity) iter.next();
			
			try {
				entity.getStartTimeInfo().doubleValue();
				entity.getEndTimeInfo().doubleValue();
			} catch(NullPointerException ex) {
				entitiesToDelete.add(entity);
			}
		}

		for (HandShapeEntity toDelete : entitiesToDelete) {
			entities.remove(toDelete);
		}
	}

	@Override
	public void addEventsEntity(EventsEntity entity) {
		HandShapeEntity handShapeEntity = (HandShapeEntity) entity;
		entity.add(this);
		entities.add(handShapeEntity);
	}
	
	@Override
	public EventsEntity getEntity(String entityID) {
		for (int i = 0; i < entities.size(); i++) {
			HandShapeEntity entity = entities.get(i);
			if (entityID.equalsIgnoreCase(entity.getID()))
				return entity;
		}
		return null;
	}

	@Override
	public Event getSelectedEvent() {
		for (int i = 0; i < entities.size(); i++) {
			HandShapeEntity entity = entities.get(i);
			Event event = entity.getSelectedEvent();
			if (event != null)
				return event;
		}
		return null;
	}

	@Override
	public EventsEntity getEntityWithRefID(String refEntityId) {
		if (refEntityId == null)
			return null;
		for (int i = 0; i < entities.size(); i++) {
			HandShapeEntity entity = entities.get(i);
			if (refEntityId.equalsIgnoreCase(entity.getRefEntityId()))
				return entity;
		}
		return null;
	}

	@Override
	public void highlight(boolean highlight, String refID) {
		if (!highlight) {
			for (int i = 0; i < entities.size(); i++) {
				EventsEntity ent = entities.get(i);
				ent.highlight(false);
			}
		} else {
			EventsEntity entAt = getEntityWithRefID(refID);
			// getEntityAt(entityAt);
			if (entAt != null)
				entAt.highlight(true);
		}
	}

	@Override
	public int getFieldSize() {
		return entities.size();
	}

	@Override
	public void deleteEntity(EventsEntity entity) {
		entities.remove(entity);
	}
	

	@Override
	public EventsEntity getSelectedEntity() {
		for (int i = 0; i < entities.size(); i++) {
			HandShapeEntity entity = entities.get(i);
			Event event = entity.getSelectedEvent();
			if (event != null)
				return entity;
		}
		return null;
	}

	public HandShapeEntity getEntityWithNewSubEntity() {
		for (int i = 0; i < entities.size(); i++) {
			HandShapeEntity entity = entities.get(i);
			if (entity.getNewHandShapeEntity() != null)
				return entity;
		}
		return null;
	}

	public boolean setSelectedEventStartTime(int time) {
		return false;
	}

	public boolean setSelectedEventEndTime(int time) {
		return false;
	}

	// public EventsEntity getEntityAt(int x) {
	// for (int i = 0; i < entities.size(); i++) {
	// HandShapeEntity entity = (HandShapeEntity) entities.get(i);
	// if(entity.contains(x))
	// return entity;
	// }
	// return null;
	// }

	public boolean setSelectedEntityStartCoordinate(int coord) {
		EventsEntity entity = getSelectedEntity();
		EventsEntity entityAt = getEntityAt(coord);

		if (entity == null || (entityAt != null && !entity.equals(entityAt)))
			return false;

		boolean overlap = isOverlappingNotSelectedEvent(coord, entity
				.getEndTimeCoordinate());

		if (!overlap) {
			entity.getSelectedEvent().setStartTimeCoordinate(coord);
			return false;
		}
		return true;
	}

	public boolean setSelectedEntityEndCoordinate(int coord) {
		EventsEntity entity = getSelectedEntity();
		EventsEntity entityAt = getEntityAt(coord);

		if (entity == null || (entityAt != null && !entity.equals(entityAt)))
			return false;

		boolean overlap = isOverlappingNotSelectedEvent(entity
				.getStartTimeCoordinate(), coord);

		if (!overlap) {
			entity.getSelectedEvent().setEndTimeCoordinate(coord);
			return false;
		}
		return true;
	}

	//
	public boolean isOverlappingNotSelectedEvent(double coordx1, double coordx2) {
		for (int j = 0; j < entities.size(); j++) {
			HandShapeEntity entity = entities.get(j);
			if (entity.getSelectedEvent() == null) {
				if (entity.isOverlapping(coordx1, coordx2))
					return true;
			}
		}
		return false;
	}

	//
	@Override
	public Event getEventAt(double x) {
		for (int i = 0; i < entities.size(); i++) {
			HandShapeEntity entity = entities.get(i);
			// if (entity.contains(x))
			Event event = entity.getEventAt(x);
			if (event != null)
				return event;
		}
		return null;
	}

	@Override
	public EventsEntity getEntityAt(double x) {
		for (int i = 0; i < entities.size(); i++) {
			HandShapeEntity entity = entities.get(i);
			if (entity.contains(x))
				return entity;
		}
		return null;
	}

	@Override
	public void unselect() {
		for (int i = 0; i < entities.size(); i++) {
			HandShapeEntity entity = entities.get(i);
			entity.unselect();
		}
	}

	public void addHandShapeEntity(Object obj) {
		HandShapeEntity handShapeEntity = (HandShapeEntity) obj;
		handShapeEntity.setField(this);
		//
		//System.out.println("1: " + entities.size());
		//
		entities.add(handShapeEntity);
		//
		//System.out.println("2: " + entities.size());
		//
	}

	@Override
	public void paint(Graphics2D g2d, double Y, Font font) {
		super.paint(g2d, Y, font);

        for (HandShapeEntity entity : entities) {
            if (SegmentGraphReprUtil.visibleInterval().overlaps(entity.interval(), Bound.INCLUSIVE))
                entity.paint(g2d, Y, font);

//            TimeInfo onsetInfo = edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton.getCurrentTimeRange().time(Math.round(SegmentGraphReprUtil.utteranceViewVisibleMovieOnset));
//            TimeInfo offsetInfo = edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton.getCurrentTimeRange().time(Math.round(SegmentGraphReprUtil.utteranceViewVisibleMovieOffset));
//            if (startTimeInfo.greaterThan(onsetInfo)
//                && endTimeInfo.lessThan(offsetInfo)) {
//                entity.paint(g2d, Y, font);
//            }
        }
	}

	@Override
	public double getStartTimeCoordinate() {
		double startCoord = 0;
		for (int i = 0; i < entities.size(); i++) {
			HandShapeEntity entity = entities.get(i);
			startCoord = startCoord < entity.getStartTimeCoordinate() ? startCoord
					: entity.getStartTimeCoordinate();
		}
		return 0;
	}

	@Override
	public double getEndTimeCoordinate() {
		double endCoord = 0;
		for (int i = 0; i < entities.size(); i++) {
			HandShapeEntity entity = entities.get(i);
			endCoord = endCoord > entity.getEndTimeCoordinate() ? endCoord
					: entity.getEndTimeCoordinate();
		}
		return endCoord;
	}
}
