package edu.bu.signstream.grepresentation.fields.handShapeField;

import java.util.ArrayList;

import edu.bu.signstream.time.Time;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;

public class HandShapeSubEntitiesManager {

	// collection of HandShapeSubEntities objects. First subentity (collection(0)) --> super entity
	private ArrayList<HandShapeSubEntity> collection = new ArrayList<HandShapeSubEntity>();
   
   // get following event
   public Event getNextEvent(Event eventToCompare) {
      Time movieTimeToCompareInfo = eventToCompare.getStartTimeInfo();
      
      ArrayList events = getEvents();
      ArrayList followingEvents = new ArrayList();
      
      // select following events only
      for(int i = 0; i < events.size(); i++) {
         Event event = (Event)events.get(i);
         if(movieTimeToCompareInfo.lessThan(event.getStartTimeInfo())) {
            followingEvents.add(event);             
         } else if(movieTimeToCompareInfo.equals(event.getStartTimeInfo()) && !eventToCompare.equals(event)) {
            followingEvents.add(event); 
         }   
      }
      
      if(followingEvents.size()==0)
         return null;
      
      // select event with smallect start movie time 
      Event selectedEvent = (Event)followingEvents.get(0);     
      for(int j = 1; j < followingEvents.size(); j++) {
         Event fEvent = (Event)followingEvents.get(j);
         if(selectedEvent.getStartTimeInfo().greaterThan(fEvent.getStartTimeInfo())) {
            selectedEvent = fEvent;
         }
      }   
            
      return selectedEvent;
   }
   
   // get preceding event
   public Event getPreviousEvent(Event eventToCompare) {
	      Time movieTimeToCompareInfo = eventToCompare.getEndTimeInfo();
      
      ArrayList events = getEvents();
      ArrayList precedingEvents = new ArrayList();
      
      // select preceding events only
      for(int i = 0; i < events.size(); i++) {
         Event event = (Event)events.get(i);
         if(movieTimeToCompareInfo.greaterThan(event.getStartTimeInfo())) {
            precedingEvents.add(event);             
         } else if(movieTimeToCompareInfo.equals(event.getStartTimeInfo()) && !eventToCompare.equals(event)) {
            precedingEvents.add(event); 
         }   
      }
      
      if(precedingEvents.size()==0)
         return null;
      
      // select event with smallect start movie time 
      Event selectedEvent = (Event)precedingEvents.get(0);     
      for(int j = 1; j < precedingEvents.size(); j++) {
         Event fEvent = (Event)precedingEvents.get(j);
         if(selectedEvent.getStartTimeInfo().greaterThan(fEvent.getStartTimeInfo())) {
            selectedEvent = fEvent;
         }
      }   
            
      return selectedEvent;
   }
   
   public void deleteEntity() {
		collection.clear();
	}

	public HandShapeSubEntity getSuperEntity() {
		return collection.get(0);
	}

	public void addSubEntity(HandShapeSubEntity entity) {
		collection.add(entity);
	}

	public ArrayList<HandShapeSubEntity> getSubEntities() {
		return collection;
	}

	public boolean isSelected() {
		return false;
	}

//	public boolean isOverlapping(int coordx1, int coordx2) {
//		int startCoord = getStartTimeCoordinate();
//		int endCoord = getEndTimeCoordinate();
//
//		return (coordx1 <= startCoord && coordx2 >= startCoord)
//			|| (coordx1 <= endCoord && coordx2 >= endCoord);
//	}

	public double getStartTimeCoordinate() {
		return getFirstEvent().getStartTimeCoordinate();
	}

	public Event getFirstEvent() {
		if (collection.isEmpty())
			return null;

		HandShapeSubEntity subEntity = collection.get(0);
		Event event = subEntity.getFirstEvent();

		for(HandShapeSubEntity subEnt : collection) {
			Event firstEvent = subEnt.getFirstEvent();
			if (firstEvent != null && (firstEvent.getEndTimeInfo().doubleValue() < event.getEndTimeInfo().doubleValue())) //.getStartTimeInfo().getMovieTime())
				event = firstEvent;
		}

		return event;
	}

	public Event getLastEvent() {
		if (collection.isEmpty())
			return null;
		
		HandShapeSubEntity subEntity = collection.get(0);
		Event event = subEntity.getLastEvent();

		for(HandShapeSubEntity subEnt : collection) {
			
			//int startEntStartTime = subEnt.getStartTimeInfo().getMovieTime();

			Event lastEvent = subEnt.getLastEvent();

			if (lastEvent.getStartTimeInfo().doubleValue() > event.getStartTimeInfo().doubleValue())
				event = lastEvent;
		}

		return event;
	}

	public double getEndTimeCoordinate() {
		return getLastEvent().getEndTimeCoordinate();
	}

	@Deprecated
	public void setStartTime(int startTime) {
		Event event = getFirstEvent();
		var time = SS3Singleton.getVideoTimeRange().time(startTime, TimeUnit.TIMESTAMP);
		event.setStartTimeInfo(time);
		event.setEndTimeInfo(time);
	}

	@Deprecated
	public void setEndTime(int endTime) {
		Event event = getLastEvent();
        var time = SS3Singleton.getVideoTimeRange().time(endTime, TimeUnit.TIMESTAMP);
        event.setStartTimeInfo(time);
        event.setEndTimeInfo(time);
	}

	@Deprecated
	public double getEndTime() {
		return getLastEvent().getEndTimeInfo().doubleValue();
	}

	@Deprecated
	public double getStartTime() {
		return getFirstEvent().getStartTimeInfo().doubleValue();
	}

	public boolean selectEntityIfEventSelected() {
		for(HandShapeSubEntity subEnt : collection) {
			Event event = subEnt.getSelectedEvent();
			if (event != null)
				return subEnt.select();
		}

		return false;
	}

	public boolean selectEventIfEntitySelected() {
		for(HandShapeSubEntity subEnt : collection) {
			if (subEnt.isSelected())
				return subEnt.getFirstEvent().select();
		}
		return false;
	}

	//   public boolean select() {
	//      //hsEventsManager.highlightEntity(Palette.HIGHLIGHTED_ENTITY_COLOR);
	//
	//      //QTMultiMovieController qtControl =
	//      // SSSingleton.getMultiMovieController();
	//      //qtControl.setStartTime((int) hsEventsManager.getFirstEvent().getStartTime());
	//      //qtControl.setEndTime((int) hsEventsManager.getLastEvent().getEndTime());
	//      selected = true;
	//      mToolBar.setEnable(true);
	//      return selected;
	//   }

	public boolean unselectEntity() {
		for(HandShapeSubEntity subEnt : collection) {
			subEnt.unselect();
		}
		return true;
	}

	//public boolean adjustEntity(int x1, int x2) {
	// return hsEventsManager.adjustEntity(x1, x2);
	//}

	public Event getSelectedEvent() {
		for(HandShapeSubEntity subEnt : collection) {
			Event event = subEnt.getSelectedEvent();
			if (event != null)
				return event;
		}
		return null;
	}

	public void reset() {
		for(HandShapeSubEntity subEnt : collection) {
			subEnt.reset();
		}
	}

	public boolean clear() {
		collection.clear();
//		for (int i = 0; i < collection.size(); i++) {
//			HandShapeSubEntity subEnt = (HandShapeSubEntity) collection.get(i);
//			subEnt.delete();
//		}
		return true;
	}

	public Event getEventAt(double x) {
		Event event = null;
		for(HandShapeSubEntity subEnt : collection) {
			event = subEnt.getEventAt(x);
			if (event != null)
				return event;
		}
		return event;
	}

	public ArrayList getEventsToAdd() {
		ArrayList list = new ArrayList();
		list.add("Add HandShape");
		return list;
	}

	public String getToolTipText(int x) {
		HandShapeEvent event = (HandShapeEvent) getEventAt(x);

		if (event == null)
			return "";

		StringBuffer toolTipText = new StringBuffer();
		toolTipText.append(event.getText());

		if (event.equals(getFirstEvent())) {
			toolTipText.append(", starts at ");
		} else if (event.equals(getLastEvent())) {
			toolTipText.append(", ends at ");
		} else {
			toolTipText.append(", at ");
		}

		toolTipText.append(event.getStartTimeInfo().doubleValue());

		return toolTipText.toString();
	}

//	public boolean setSelectedEventStartCoordinate(int coord) {
//		Event event = getSelectedEvent();
//		event.setStartTimeCoordinate(coord);
//		return true;
//	}
//
//	public boolean setSelectedEventEndCoordinate(int coord) {
//		Event event = getSelectedEvent();
//		event.setEndTimeCoordinate(coord);
//		return true;
//	}

	public boolean selectEvent(int x) {
		Event event = getEventAt(x);
		if (event != null)
			return event.select();
		return false;
	}

	public ArrayList<HandShapeEvent> getEvents() {
		ArrayList list = new ArrayList();
		for(HandShapeSubEntity subEnt : collection) {
			list.addAll(subEnt.getEvents());
		}
		return list;
	}
}
