package edu.bu.signstream.grepresentation.fields.handShapeField;

// import java.awt.Color;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;

import edu.bu.signstream.common.util.vo.TrackValue;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.grepresentation.fields.CollectionManager;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class HandShapesManager extends CollectionManager {
	// collection of HandShapeEvent objects
	private ArrayList<HandShapeEvent> collection = new ArrayList<HandShapeEvent>();

	private SignStreamSegmentPanel segmentPanel = null;

	public HandShapesManager(SignStreamSegmentPanel segmentPanel) {
		this.segmentPanel = segmentPanel;
	}

	public void deleteEntity() {
		collection.clear();
	}

	public boolean hasEvents() {
		return !collection.isEmpty();
	}

	@Deprecated
	public void addHandShape(String vID, String toolTipText, int startTime) {
		TrackValue value = new TrackValue();
		value.setValueId(vID);
		Time timeInfo = SS3Singleton.getVideoTimeRange().time(startTime, TimeUnit.TIMESTAMP);
//      TimeInfo timeInfo = new TimeInfo(startTime, TimeUnit.VIDEO_FRAME_30_FPS);
		value.setStartTimeInfo(timeInfo);
		value.setTextValue(toolTipText);

		HandShapeEvent event = new HandShapeEvent(value, segmentPanel);
		collection.add(event);
		sortEvents();
	}

	@Deprecated
	public void addHandShape(TrackValue value) {
		HandShapeEvent event = new HandShapeEvent(value, segmentPanel);
		addHandShape(event);
	}

	public void addHandShape(SS3Item item) {
		HandShapeEvent event = new HandShapeEvent(item, segmentPanel);
      event.setImageID(item.getImageID());
      addHandShape(event);
	}

	public void addHandShape(HandShapeEvent event) {
		collection.add(event);
//		sortEvents();		// GD: remove this whole method because events in the collection should already be in "selection order"
	}
	
	public ArrayList<HandShapeEvent> getHandShapesCollection() {
		return collection;
	}

	public HandShapeEvent getSelectedHandShapeEvent() {
		for (int i = 0; i < collection.size(); i++) {
			HandShapeEvent event = collection.get(i);
			if (event.isSelected())
				return event;
		}
		return null;
	}

	@Deprecated
	public void deleteHandShape(String vID, int startTime) {
		for (int i = 0; i < collection.size(); i++) {
			HandShapeEvent event = collection.get(i);
			String vlID = "" + event.getID();
			if (vlID.equalsIgnoreCase(vID))
				collection.remove(event);
		}
		sortEvents();
	}

	// GD: deemed invalid, because there is no need to even sort events, which are already supposed to be in "selection" order
	@Deprecated
	public void sortEvents() {
		double keys[] = new double[collection.size()];
		HashMap map = new HashMap();
		
		// GD this will not sort things properly with the same time values - the argument for why to not use this method

		for (int i = 0; i < collection.size(); i++) {
			HandShapeEvent event = collection.get(i);
			map.put("" + event.getStartTimeInfo().doubleValue(), event);
			keys[i] = event.getStartTimeInfo().doubleValue();
		}

		Arrays.sort(keys);

		collection.removeAll(collection);
		for (int j = 0; j < keys.length; j++) {
			collection.add((HandShapeEvent)map.get("" + keys[j]));
		}
	}

	@Override
	public void unselectEntity() {
		for (int i = 0; i < collection.size(); i++) {
			HandShapeEvent event = collection.get(i);
			event.unselect();
		}
	}

	@Override
	public Event getSelectedEvent() {
		for (int i = 0; i < collection.size(); i++) {
			HandShapeEvent event = collection.get(i);
			if (event.isSelected())
				return event;
		}
		return null;
	}

	@Override
	public Event getEventAt(double x) {
		for (int i = 0; i < collection.size(); i++) {
			HandShapeEvent event = collection.get(i);

			if (event.contains(x))
				return event;
		}
		return null;
	}

	@Override
	public Event getFirstEvent() {
		if (collection.size() == 0) {
			return null;
		}

		HandShapeEvent event = collection.get(0);
		for (int i = 1; i < collection.size(); i++) {
			HandShapeEvent hsEvent = collection.get(i);					
			if (hsEvent.getStartTimeInfo().doubleValue() < event.getStartTimeInfo().doubleValue())
				event = hsEvent;
		}
		return event;
	}

	@Override
	public Event getLastEvent() {
		if (collection.size() <= 0)
			return null;

		HandShapeEvent event = collection.get(0);
		for (int i = 1; i < collection.size(); i++) {
			HandShapeEvent hsEvent = collection.get(i);		
			if (hsEvent.getStartTimeInfo().doubleValue() > event.getStartTimeInfo().doubleValue())
				event = hsEvent;
		}
		return event;
	}

	@Override
	public void resetAll() {
		for (int i = 0; i < collection.size(); i++) {
			HandShapeEvent event = collection.get(i);
			event.reset();
		}
	}

	@Override
	public boolean adjustEntity(double startTime, double endTime) {
		setEntityStartTime(startTime);
		setEntityEndTime(endTime);
		return true;
	}

	@Deprecated
	public boolean setEntityStartTime(double time) {
		Event event = getFirstEvent();
        var timeInfo = SS3Singleton.getVideoTimeRange().time(Math.round(time), TimeUnit.TIMESTAMP);
        event.setStartTimeInfo(timeInfo);
		return true;
	}

	@Deprecated
	public boolean setEntityEndTime(double time) {
		Event event = getLastEvent();
        var timeInfo = SS3Singleton.getVideoTimeRange().time(Math.round(time), TimeUnit.TIMESTAMP);
        event.setEndTimeInfo(timeInfo);
		return true;
	}

	@Deprecated
	public boolean setSelectedEventStartTime(double time) {
		Event event = getSelectedEvent();
        var timeInfo = SS3Singleton.getVideoTimeRange().time(Math.round(time), TimeUnit.TIMESTAMP);
        event.setStartTimeInfo(timeInfo);
		return true;
	}

	@Deprecated
	public boolean setSelectedEventEndTime(double time) {
		Event event = getSelectedEvent();
        var timeInfo = SS3Singleton.getVideoTimeRange().time(Math.round(time), TimeUnit.TIMESTAMP);
        event.setEndTimeInfo(timeInfo);
		return true;
	}
}
