package edu.bu.signstream.grepresentation.fields.locationField;

import edu.bu.signstream.common.util.CoordinateTimeConverter;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.time.Duration;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEventsEntity;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

import java.awt.*;
import java.util.ArrayList;

public class LocationEntity extends EventsEntity {
    private LocationEventsCollection events;

    private GlossChainedEventsEntity refGlossEntity;

    public LocationEntity(String id, LocationEventsCollection events, SignStreamSegmentPanel panel, Field field) {
        this(events, panel, field);
        setID(id);
    }

    public LocationEntity(LocationEventsCollection events, SignStreamSegmentPanel segmentPanel, Field field) {
        super(segmentPanel, field);
        this.events = events;
    }

    public void setEvents(ArrayList<LocationEvent> events) {
        this.events.deleteAll();
        for (LocationEvent event : events) {
            this.events.addEvent(event);
        }
    }

    public boolean isEmptyLocationEntity() {
        for (LocationEvent event : events.getEvents()) {
            if (!event.isNoneLocation()) {
                return false;
            }
        }
        return true;
    }

    public LocationEventsCollection getLocationEventsCollection() {
        return events;
    }

    @Override
    public SS3Entity getSS3Entity() {
        SS3Entity ss3Entity = new SS3Entity();
        for (LocationEvent event : events.getEvents()) {
            ss3Entity.addItem(new SS3Item(event));
        }
        ss3Entity.setID(getID());
		if (getParentEntities() != null)
			ss3Entity.setParentEntities(getParentEntities());
		if (refGlossEntity != null)
			ss3Entity.setRefEntityId(refGlossEntity.getID());
        return ss3Entity;
    }

    @Deprecated
    public void setStartTime(int startTime) {
        var time = SS3Singleton.getVideoTimeRange().time(startTime, TimeUnit.TIMESTAMP);
        getFirstEvent().setStartTimeInfo(time);
    }

    @Deprecated
    public void setEndTime(int endTime) {
        var time = SS3Singleton.getVideoTimeRange().time(endTime, TimeUnit.TIMESTAMP);
        getLastEvent().setEndTimeInfo(time);
    }

    @Override
    public double getStartTimeCoordinate() {
        return getFirstEvent().getStartTimeCoordinate();
    }

    // should return Start Time Coordinate!!!
    @Override
    public double getEndTimeCoordinate() {
        return getLastEvent().getStartTimeCoordinate();
    }

    @Override
    public Event getFirstEvent() {
        return events.getFirstEvent();
    }

    @Override
    public Event getLastEvent() {
        return events.getLastEvent();
    }

    @Deprecated
    public double getStartTime() {
        LocationEvent event = events.getFirstEvent();
        if (event == null)
            return 0;
        return event.getStartTimeInfo().doubleValue();
    }

    @Deprecated
    public double getEndTime() {
        LocationEvent event = events.getLastEvent();
        if (event == null)
            return 0;
        return event.getEndTimeInfo().doubleValue();
    }

    @Override
    public boolean selectEntityIfEventSelected() {
        return true;
    }

    public ArrayList<LocationEvent> getEvents() {
        return events.getEvents();
    }

    @Override
    public boolean selectEventIfEntitySelected() {
        return true;
    }

    @Override
    public boolean select() {
        return true;
    }

    @Override
    public boolean unselect() {
        return true;
    }

    @Override
    public boolean adjustEntity(int x1, int x2) {
        CoordinateTimeConverter convertor =
                segmentPanel.getZoomer().getCoordinateTimeConvertor();
        Time startTimeInfo = convertor.convertCoordinateToTimeInfo(x1);
        Time endTimeInfo = convertor.convertCoordinateToTimeInfo(x2);
        return events.adjustEntity(startTimeInfo, endTimeInfo);
    }

    @Override
    public Event getSelectedEvent() {
        return null;
    }

    @Override
    public void reset() {
        selected = false;
        events.resetAll();
    }

    @Override
    public boolean delete() {
        return events.deleteAll();
    }

    @Override
    public Event getEventAt(double x) {
        return null;
    }

    @Override
    public ArrayList<String> getEventsToAdd() {
        return null;
    }

    @Override
    public String getToolTipText(int x) {
        StringBuilder str = new StringBuilder("[");
        for (int i = 0; i < events.size() - 1; i++) {
            str.append(events.getEvent(i).getToolTipText()).append(" -> ");
        }
        str.append(events.getEvent(events.size() - 1).getToolTipText()).append("]");
        return str.toString();
    }

    @Override
    public boolean selectEvent(double x) {
        return true;
    }

    @Override
    public void paint(Graphics2D g2d, double Y, Font font) {
        if (isEmptyLocationEntity()) {
            return;
        }

        g2d.setFont(font);

        double X1 = getStartTimeCoordinate();
        double X2 = getEndTimeCoordinate();

        int y = (int) Y;

        ArrayList<LocationEvent> eventsList = getLocationEventsCollection().getEvents();

        g2d.setColor(Color.BLACK);

        for (int i = 0; i < eventsList.size(); i++) {
            LocationEvent locationEvent = eventsList.get(i);
            if (!locationEvent.isNoneLocation()) {
                String currentText = locationEvent.getText();
                double x;
                if (i == 0) {
                    x = X1;
                } else if (i == eventsList.size() - 1) {
                    x = X2 - g2d.getFontMetrics().stringWidth(currentText);
                } else {
                    x = locationEvent.getStartTimeCoordinate() - g2d.getFontMetrics().stringWidth(currentText) / 2;
                }
                g2d.drawString(currentText, (int) x, y);
            }
        }

        g2d.drawLine((int) X1, y + 1, (int) X2, y + 1);
        g2d.drawLine((int) X1, y + 2, (int) X2, y + 2);
    }

    @Override
    public String toString() {
        return "LocationEntity: " + super.toString();
    }

    @Override
    public Time getStartTimeInfo() {
        LocationEvent event = events.getFirstEvent();
        if (event == null)
            return null;
        return event.getStartTimeInfo();
    }

    @Override
    public Time getEndTimeInfo() {
        LocationEvent event = events.getLastEvent();
        if (event == null)
            return null;
        return event.getEndTimeInfo();
    }

    @Override
    public void setStartTimeInfo(Time startTimeInfo) {
        LocationEvent event = events.getFirstEvent();
        if (event != null)
            event.setStartTimeInfo(startTimeInfo);
    }

    @Override
    public void setEndTimeInfo(Time endTimeInfo) {
        LocationEvent event = events.getLastEvent();
        if (event != null)
            event.setEndTimeInfo(endTimeInfo);
    }

    public void updateTimes() {
        if (refGlossEntity != null) {
            setStartTimeInfo(refGlossEntity.getStartTimeInfo());
            setEndTimeInfo(refGlossEntity.getEndTimeInfo());
        }

		var startUnit = getStartTimeInfo().unit();
		var endUnit = getEndTimeInfo().unit();
        Time start = getStartTimeInfo().absolute();
        Time end = getEndTimeInfo().absolute();
        int size = events.size();

		Duration dur = end.distance(start).divide(size - 1);
		
		for (int i = 0; i < size; i++) {
			Time eventTime = null;
            LocationEvent locationEvent = events.getEvent(i);
            if (i == 0) {
            	eventTime = start;
            } else if (i == events.size() - 1) {
            	eventTime = end;
            } else {
            	eventTime = start.add(dur.multiply(i));
            }

			var s = eventTime.convert(startUnit);
			var e = eventTime.convert(endUnit);
			
            locationEvent.setStartTimeInfo(s);
            locationEvent.setEndTimeInfo(e);
		}
    }

    @Override
    public boolean contains(double x) {
        return super.contains(x);
    }

    public void setRefGlossEntity(GlossChainedEventsEntity refDomGlossEntity) {
        this.refGlossEntity = refDomGlossEntity;
        this.setRefEntityId(refDomGlossEntity.getID());
    }

	@Override
	public ArrayList<Event> getEventsList() {
		return new ArrayList<Event>(events.getEvents());
	}
}
