package edu.bu.signstream.grepresentation.fields.movementField;

import java.util.ArrayList;

import edu.bu.signstream.time.Time;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEvent;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class MovementEventsCollection {
	private ArrayList events = new ArrayList();

	private SignStreamSegmentPanel segmentPanel;

	public MovementEventsCollection(SignStreamSegmentPanel segmentPanel) {
		this.segmentPanel = segmentPanel;
	}

	public void addEvent(ChainedEvent event) {
		events.add(event);
	}

	public void remove(ChainedEvent event) {
		events.remove(event);
	}


	public ChainedEvent getFirstEvent() {
		if (events.size() == 0)
			return null;
		ChainedEvent event = (ChainedEvent) events.get(0);
		for (int i = 1; i < events.size(); i++) {
			ChainedEvent evnt = (ChainedEvent) events.get(i);
			if (evnt.getStartTimeInfo().doubleValue() < event.getStartTimeInfo().doubleValue())
				event = evnt;
		}
		return event;
	}

	public ChainedEvent getLastEvent() {
		if (events.size() == 0)
			return null;
		ChainedEvent event = (ChainedEvent) events.get(0);
		for (int i = 1; i < events.size(); i++) {
			ChainedEvent evnt = (ChainedEvent) events.get(i);
			if (evnt.getStartTimeInfo().doubleValue() > event.getStartTimeInfo().doubleValue())
				event = evnt;
		}
		return event;
	}

	public ChainedEvent getEventAt(double x) {
		for (int i = 0; i < events.size(); i++) {
			ChainedEvent event = (ChainedEvent) events.get(i);
			if (event.contains(x))
				return event;
		}
		return null;
	}

	public boolean setSelectedEventStartCoordinate(double coord) {
		double time = segmentPanel.getZoomer().getCoordinateTimeConvertor()
				.convertCoordinateToTime(coord);
		ChainedEvent event = getSelectedEvent();
		double eventDuration = event.getEndTimeInfo().doubleValue() - time;

		if (eventDuration < 33) {
			if (deleteEventConfirmDialog()) {
				return events.remove(event);
			} else {
				return false;
			}
		}

		ChainedEvent bEvent = getBeforeSelectedEvent();
		if (bEvent == null) {
			event.setStartTimeCoordinate(coord);
			return true;
		} else {
			double duration = time - bEvent.getStartTimeInfo().doubleValue();
			if (duration > 33) {
				event.setStartTimeCoordinate(coord);
				bEvent.setEndTimeCoordinate(coord);
				return true;
			} else {
				if (deleteEventConfirmDialog()) {
					events.remove(bEvent);
					event.setStartTimeCoordinate(coord);
					return true;
				}
			}
		}

		return false;
	}

	public boolean setSelectedEventEndCoordinate(double coord) {
		double time = segmentPanel.getZoomer().getCoordinateTimeConvertor()
		.convertCoordinateToTime(coord);
		ChainedEvent event = getSelectedEvent();
		double eventDuration = time - event.getStartTimeCoordinate();

		if (eventDuration < 33) {
			if (deleteEventConfirmDialog()) {
				return events.remove(event);
			} else {
				return false;
			}
		}

		ChainedEvent aEvent = getAfterSelectedEvent();
		if (aEvent == null) {
		    event.setEndTimeInfo(SS3Singleton.getVideoTimeRange().time(Math.round(time), TimeUnit.TIMESTAMP));
			return true;
		} else {
			double duration = aEvent.getEndTimeInfo().doubleValue() - time;
			if (duration > 33) {
                event.setEndTimeInfo(SS3Singleton.getVideoTimeRange().time(Math.round(time), TimeUnit.TIMESTAMP));
                aEvent.setEndTimeInfo(SS3Singleton.getVideoTimeRange().time(Math.round(time), TimeUnit.TIMESTAMP));
				return true;
			} else {
				if (deleteEventConfirmDialog()) {
                    event.setEndTimeInfo(SS3Singleton.getVideoTimeRange().time(Math.round(time), TimeUnit.TIMESTAMP));
					return events.remove(aEvent);
				}
			}
		}

		return false;
	}

	public boolean deleteEventConfirmDialog() {
//		String msg = "This action will delete selected event. \n Do you want to proceed?";
//
//		int n = JOptionPane.showConfirmDialog(null, msg, "Confirm Dialog",
//				JOptionPane.YES_NO_OPTION);
//
//		if (n == JOptionPane.YES_OPTION) {
//			// deleteEntity();
//		// trackEntity.deleteChainedEventsEntity(this);
//			return true;
//		}
		return false;
	}

	private ChainedEvent getBeforeSelectedEvent() {
		for (int i = 0; i < events.size(); i++) {
			ChainedEvent event = (ChainedEvent) events.get(i);
			if (event.isSelected() && i > 0)
				return (ChainedEvent) events.get(i - 1);
		}
		return null;
	}

	private ChainedEvent getAfterSelectedEvent() {
		for (int i = 0; i < events.size(); i++) {
			ChainedEvent event = (ChainedEvent) events.get(i);
			if (event.isSelected() && i < events.size() - 2)
				return (ChainedEvent) events.get(i + 1);
		}
		return null;
	}

	public ChainedEvent getSelectedEvent() {
		for (int i = 0; i < events.size(); i++) {
			ChainedEvent event = (ChainedEvent) events.get(i);
			if (event.isSelected())
				return event;
		}
		return null;
	}

	public void unselectEntity() {
		for (int i = 0; i < events.size(); i++) {
			ChainedEvent event = (ChainedEvent) events.get(i);
			event.unselect();
		}
	}

	public boolean adjustEntity(Time startTimeInfo, Time endTimeInfo) {
		// ChainedEvent fEvent = getFirstEvent();
		// ChainedEvent lEvent = getLastEvent();
		//
		// if (fEvent == null || lEvent == null)
		// return false;
		//
		// fEvent.setStartTime(startTime);
		// lEvent.setEndTime(endTime);
		return true;
	}

	public void resetAll() {
		for (int i = 0; i < events.size(); i++) {
			ChainedEvent event = (ChainedEvent) events.get(i);
			event.reset();
		}
	}

	public boolean deleteAll() {
		return events.removeAll(events);
	}

	public ArrayList getEvents() {
		return events;
	}
}
