package edu.bu.signstream.grepresentation.fields.nonManualField;

import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.util.ArrayList;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.time.Duration;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.freeTextField.FreeTextEntity;
import edu.bu.signstream.grepresentation.fields.glossField.Compoundable;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEvent;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeEntity;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeField;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeFieldWrapper;
import edu.bu.signstream.grepresentation.fields.locationField.LocationEntity;
import edu.bu.signstream.grepresentation.fields.signTypeField.SignTypeEntity;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolbarCommons;

public class ChainedEventsEntity extends EventsEntity {

	private static final long serialVersionUID = 1L;

	protected ChainedEventsCollectionManager manager;

	protected ChainedEventsEntity previousEntity = null;
	protected ChainedEventsEntity nextEntity = null;
	
	protected boolean isLockedGlossPhonAlignments = false;

	public ChainedEventsEntity(ChainedEventsCollection collection,
			SignStreamSegmentPanel segmentPanel, Field field) {

		super(segmentPanel, field);
		setID(Util.getLongUniqueNumber());
		this.manager = new ChainedEventsCollectionManager(collection,
				segmentPanel);
	}
	
	public boolean isLockedGlossPhonAlignments() {
		return isLockedGlossPhonAlignments;
	}

	public void setLockedGlossPhonAlignments(boolean isLockedGlossPhonAlignments) {
		this.isLockedGlossPhonAlignments = isLockedGlossPhonAlignments;
	}
	
    // returns an error code that will be matched with an error message or null if success. 
	public String canSetSelectedEventStartMovieTime(Time timeInfo) {
		Event selectedEvent =  SS3Singleton.getMediaToolBar(null).selectedEvent();
		
//        Time originalSelectedStartTimeInfo = selectedEvent.getStartTimeInfo();
        Time originalSelectedEndTimeInfo = selectedEvent.getEndTimeInfo();

        Time maxStartMovieTimeInfo = originalSelectedEndTimeInfo.subtract(new Duration(1, originalSelectedEndTimeInfo.unit()));
		
		Event initialHold = manager.getChainedEventsCollection().getInitialHoldEvent();
		Event value = manager.getChainedEventsCollection().getValueOrTextValueEvent();
		Event finalHold = manager.getChainedEventsCollection().getFinalHoldEvent();
		Event offset = manager.getChainedEventsCollection().getOffsetEvent();

		if (maxStartMovieTimeInfo.lessThan(timeInfo)) {
			return ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME;
		}

		if (previousEntity != null) {
			if (previousEntity.getEndTimeInfo().greaterThan(timeInfo)) {
				return ErrorMessages.CHAINS_CANNOT_OVERLAP;
			}
		}

		boolean becomingOneFrame = timeInfo.equals(originalSelectedEndTimeInfo);
		if (becomingOneFrame) {
			if ((nextEntity != null && nextEntity.getStartTimeInfo().equals(timeInfo) && nextEntity.getEndTimeInfo().equals(timeInfo)) 
				|| (previousEntity != null && previousEntity.getStartTimeInfo().equals(timeInfo) && previousEntity.getEndTimeInfo().equals(timeInfo))) {
				return ErrorMessages.SET_ONE_FRAMES_CANNOT_OVERLAP;				
			}
		}
		
		Event prevEvent = manager.getChainedEventsCollection().getPreviousEvent(selectedEvent);
		if (prevEvent == null) {
			// |<--
			if (selectedEvent.getStartTimeInfo().greaterThan(timeInfo)) {
				if (previousEntity == null
						|| previousEntity.getEndTimeInfo().lessThanOrEq(timeInfo)) {
//		            selectedEvent.setStartTimeInfo(timeInfo);
		            return null;
//                    // selectedEvent.getStartTimeInfo().setMovieTime(movieTime);
				} else {
					return ErrorMessages.GLOSS_CHAINS_CANNOT_OVERLAP;
				}
			} else {
//	            selectedEvent.setStartTimeInfo(timeInfo);
				return null;
//                // selectedEvent.getStartTimeInfo().setMovieTime(movieTime);
			}			
		} else if (selectedEvent.getStartTimeInfo().greaterThan(timeInfo)) {
			Time prevStartTimeInfo = prevEvent.getStartTimeInfo();
			Time timeInfoLessOneDuration = timeInfo.subtract(new Duration(1, timeInfo.unit()));
			if (timeInfoLessOneDuration.lessThan(prevStartTimeInfo)) {
				
				if(selectedEvent.equals(initialHold)) {
					return ErrorMessages.SET_START_INITIAL_HOLD_CANT_DELETE_ONSET;
				} else if(selectedEvent.equals(value)) {
					if(initialHold != null && prevEvent.equals(initialHold)) {
						return ErrorMessages.SET_START_VALUE__CANT_DELETE_INITIAL_HOLD;
					} else  {
						return ErrorMessages.SET_START_VALUE_CANT_DELETE_ONSET;
					}
				} else if(selectedEvent.equals(finalHold)) {
					return ErrorMessages.SET_START_FINAL_HOLD_CANT_DELETE_VALUE;
				} else if(selectedEvent.equals(offset)) {
					if(finalHold != null && prevEvent.equals(finalHold)) {
						return ErrorMessages.SET_START_OFFSET_CANT_DELETE_FINAL_HOLD; 
					} else {
						return ErrorMessages.SET_START_OFFSET_CANT_DELETE_VALUE;
					}
				}
			} else {
//	            selectedEvent.setStartTimeInfo(timeInfo);
//
//				prevEvent.setEndTimeInfo(timeInfo);
				return null;
			}
		} else {
//            selectedEvent.setStartTimeInfo(timeInfo);
//
//            prevEvent.setEndTimeInfo(timeInfo);
			return null;
		}

		SS3Singleton.getMediaToolBar().update();
		return null;
	}

	public String canSetSelectedEventEndMovieTime(Time timeInfo) {
		Event selectedEvent = SS3Singleton.getMediaToolBar(null).selectedEvent();

        Time originalSelectedStartTimeInfo = selectedEvent.getStartTimeInfo();

        Time minEndMovieTimeInfo = originalSelectedStartTimeInfo.add(new Duration(1, originalSelectedStartTimeInfo.unit()));

		ChainedEvent onset = manager.getChainedEventsCollection().getOnsetEvent();
		ChainedEvent initialHold = manager.getChainedEventsCollection().getInitialHoldEvent();
		ChainedEvent value = manager.getChainedEventsCollection().getValueOrTextValueEvent();
		ChainedEvent finalHold = manager.getChainedEventsCollection().getFinalHoldEvent();	

		if (minEndMovieTimeInfo.greaterThan(timeInfo)) {
			return ErrorMessages.EVENT_END_TIME_FOLLOW_START_TIME;
		}
		
		if (nextEntity != null) {
			if (nextEntity.getStartTimeInfo().lessThan(timeInfo)) {
				return ErrorMessages.CHAINS_CANNOT_OVERLAP;
			}
		}

		boolean becomingOneFrame = timeInfo.equals(originalSelectedStartTimeInfo);
		if (becomingOneFrame) {
			if ((nextEntity != null && nextEntity.getStartTimeInfo().equals(timeInfo) && nextEntity.getEndTimeInfo().equals(timeInfo)) 
				|| (previousEntity != null && previousEntity.getStartTimeInfo().equals(timeInfo) && previousEntity.getEndTimeInfo().equals(timeInfo))) {
				return ErrorMessages.SET_ONE_FRAMES_CANNOT_OVERLAP;				
			}
		}

		Event nextEvent = manager.getChainedEventsCollection().getNextEvent(selectedEvent);

		if(nextEvent != null) {				
			Time maxNextEventStartTimeInfo = segmentPanel.getMultipleMovieController().getPrecedingFrameTimeInfo(
					nextEvent.getEndTimeInfo());
			if(maxNextEventStartTimeInfo.greaterThanOrEq(timeInfo)) {
//				selectedEvent.setEndTimeInfo(timeInfo);
//				
//				nextEvent.setStartTimeInfo(timeInfo);
				return null;
			} else {
				if(selectedEvent.equals(onset)) {
					if(initialHold!= null) {
						return ErrorMessages.SET_END_ONSET_CANT_DELETE_INITIAL_HOLD;
					} else {
						return ErrorMessages.SET_END_ONSET_CANT_DELETE_VALUE;
					}
				} else if(selectedEvent.equals(initialHold)) {
					return ErrorMessages.SET_END_INITIAL_HOLD_CANT_DELETE_VALUE;
				} else if(selectedEvent.equals(value)) {
					if(finalHold != null) {
						return ErrorMessages.SET_END_VALUE_CANT_DELETE_FINAL_HOLD;
					} else {
						return ErrorMessages.SET_END_VALUE_CANT_DELETE_OFFSET;
					}
				} else if (selectedEvent.equals(finalHold)) {
					return ErrorMessages.SET_END_FINAL_HOLD_CANT_DELETE_OFFSET;
				}
			} 
		} else {
//			selectedEvent.setEndTimeInfo(timeInfo);
			return null;
		}
		
		SS3Singleton.getMediaToolBar().update();
		return null;
	}
	
	public ChainedEventsEntity getPreviousEntity() {
		return previousEntity;
	}

	public void setPreviousEntity(ChainedEventsEntity previousEntity) {
		this.previousEntity = previousEntity;
	}

	public ChainedEventsEntity getNextEntity() {
		return nextEntity;
	}

	public void setNextEntity(ChainedEventsEntity nextEntity) {
		this.nextEntity = nextEntity;
	}

	public String getText() {
		return getTextValue().getText();
	}

	public Event getNextEvent(Event event) {
		return manager.getChainedEventsCollection().getNextEvent(event);
	}

	public Event getPreviousEvent(Event event) {
		return manager.getChainedEventsCollection().getPreviousEvent(event);
	}

	public void setField(NonManualField nmField) {
		super.setField(nmField);
	}

	public ChainedEventsEntity clone(String fieldID) {
		ChainedEventsCollection cll = manager.getChainedEventsCollection()
				.cloneChainedEventsCollection();
		ChainedEventsEntity ret = new ChainedEventsEntity(cll, segmentPanel, getField());
		ret.updateParentEntity();
		return ret;
	}

	@Override
	public SS3Entity getSS3Entity() {
		SS3Entity entity = new SS3Entity();
		String id = getID();
		
		String entID = id.trim().length() > 0 ? id : Util.getLongUniqueNumber();
		entity.setID(entID);
		if (getParentEntities() != null)
			entity.setParentEntities(getParentEntities());
		entity.setRefEntityId(getRefEntityId());

		ChainedEventsCollection collection = manager
				.getChainedEventsCollection();

		if (collection.getOnsetEvent() != null) {
			SS3Item onsetItem = createSS3Item(collection.getOnsetEvent());
			entity.addItem(onsetItem);
		}

		if (collection.getInitialHoldEvent() != null) {
			SS3Item inHoldItem = createSS3Item(collection.getInitialHoldEvent());
			entity.addItem(inHoldItem);
		}

		if (collection.getValueEvent() != null) {
			SS3Item valueItem = createSS3Item(collection.getValueEvent());
			entity.addItem(valueItem);
		}

		if (collection.getTextValueEvent() != null) {
			SS3Item txtValueItem = createSS3Item(collection.getTextValueEvent());
			entity.addItem(txtValueItem);
		}

		if (collection.getFinalHoldEvent() != null) {
			SS3Item fnHoldItem = createSS3Item(collection.getFinalHoldEvent());
			entity.addItem(fnHoldItem);
		}

		if (collection.getOffsetEvent() != null) {
			SS3Item offsetItem = createSS3Item(collection.getOffsetEvent());
			entity.addItem(offsetItem);
		}
		
		boolean lockedGlossPhoneAlignments = isLockedGlossPhonAlignments();
		entity.setLockedGlossPhoneAlignments(lockedGlossPhoneAlignments);
		
		if(this instanceof GlossChainedEventsEntity) {
			GlossChainedEventsEntity  glsEntity = (GlossChainedEventsEntity)this;
			boolean isLockedDomNdomAlignments = glsEntity.isLockedDomNdomAlignments();
			
			GlossChainedEventsEntity dependentGlossEntity = glsEntity.getDependentGlossEntity();
			ChainedEventsEntity dependentHandEntity = glsEntity.getDependentHandEntity();
			HandShapeEntity dependentHandShapeEntity = glsEntity.getDependentHandShapeEntity();
			LocationEntity dependentLocationEntity = glsEntity.getDependentLocationEntity();
			SignTypeEntity dependentSignTypeEntity = glsEntity.getDependentSignTypeEntity();
						
			entity.setLockedDomNdomAlignments(isLockedDomNdomAlignments);
			if(dependentGlossEntity != null) {
				entity.setDependentGlossEntityID(dependentGlossEntity.getID());
			}
			if(dependentHandEntity != null) {
				entity.setDependentHandEntityID(dependentHandEntity.getID());
			}
			if(dependentHandShapeEntity != null) {
				//String hndID = dependentHandShapeEntity.getID();
				entity.setDependentHandShapeEntityID(dependentHandShapeEntity.getID());
			}
			if (dependentLocationEntity != null) {
				entity.setDependentLocationEntityID(dependentLocationEntity.getID());
			}

			if (dependentSignTypeEntity != null) {
				entity.setDependentSignTypeEntityID(dependentSignTypeEntity.getID());
			}

            Compoundable startCompound = glsEntity.getStartCompound();
            if(startCompound != null) {
                entity.setStartCompoundEntityID(startCompound.getID());
            }

            Compoundable endCompound = glsEntity.getEndCompound();
            if(endCompound != null) {
                entity.setEndCompoundEntityID(endCompound.getID());
            }
			
			GlossEntityProperties properties = glsEntity.getGlossEntityProperties();
			entity.setGlossEntityProperties(properties);
			
			String primaryEntry = glsEntity.getPrimaryEntry();
			if (primaryEntry != null) {
				entity.setPrimaryEntry(primaryEntry);
			}
		} else if (this instanceof FreeTextEntity) {
			FreeTextEntity freeTextEntity = (FreeTextEntity) this;
			if (freeTextEntity.isValidTime()) {
				entity.setValidTime(true);
				
//				int entityStartTime = entity.getStartTime();
//				int entityEndTime = entity.getEndTime();
//				
//				int freeTextEntityStartTime = this.getStartTimeInfo().getMovieTime();
//				int freeTextEntityEndTime = this.getEndTimeInfo().getMovieTime();
//				
//				// continue on here if we want to set valid time only if the times are aligned exactly
			}
		}

		return entity;
	}

	private SS3Item createSS3Item(ChainedEvent event) {
		SS3Item item = new SS3Item();
		item.setId(event.getID());
		item.setText(event.getText());
		item.setStartTimeInfo(event.getStartTimeInfo());
		item.setEndTimeInfo(event.getEndTimeInfo());
		return item;
	}

	public Event getValueOrTextValueEvent() {
		return manager.getChainedEventsCollection().getValueOrTextValueEvent();
	}

	public ChainedEventsCollection getChainedEventsCollection() {
		return manager.getChainedEventsCollection();
	}

	public ChainedEvent getValue() {
		return manager.getChainedEventsCollection().getValueEvent();
	}

	public ChainedEvent getTextValue() {
		return manager.getChainedEventsCollection().getTextValueEvent();
	}

	public GlossChainedEvent getGlossChainedEvent() {
		return manager.getChainedEventsCollection().getTextValueEvent();
	}

	@Deprecated
	public void setStartTime(double startTime) {
		ChainedEvent event = (ChainedEvent) manager.getFirstEvent();
		if (event != null)
		    event.setStartTimeInfo(SS3Singleton.getVideoTimeRange().time(Math.round(startTime), TimeUnit.TIMESTAMP));
	}

	@Override
	public void setStartTimeInfo(Time timeInfo) {
		Event onset = manager.getChainedEventsCollection().getOnsetEvent();
		Event initialHold = manager.getChainedEventsCollection().getInitialHoldEvent();
		Event value = manager.getChainedEventsCollection().getValueOrTextValueEvent();
		Event finalHold = manager.getChainedEventsCollection().getFinalHoldEvent();
		Event offset = manager.getChainedEventsCollection().getOffsetEvent();
		
		boolean selectingPartOfThisEntity = false;
		
		Event selectedEvent = SS3Singleton.getMediaToolBar(null).selectedEvent();
		
		if (selectedEvent.equals(onset) || selectedEvent.equals(initialHold) || selectedEvent.equals(value)
				|| selectedEvent.equals(finalHold) || selectedEvent.equals(offset)) {
			selectingPartOfThisEntity = true;
		}
		
		if (!selectingPartOfThisEntity) {
			// special case for hand entities - should really be a sub-class
			ChainedEvent event = (ChainedEvent) manager.getFirstEvent();
			if (event != null)
				event.setStartTimeInfo(timeInfo);
			
			return;
		}
		
		Event prevEvent = manager.getChainedEventsCollection().getPreviousEvent(selectedEvent);
		
		selectedEvent.setStartTimeInfo(timeInfo);
		if (prevEvent != null) {
			prevEvent.setEndTimeInfo(timeInfo);
		}

		HandShapeEntity depHsEntity = null;
		if (getField().getFieldID().equals(Util.DOM_HAND_FIELD_ID) || getField().getFieldID().equals(Util.NDOM_HAND_FIELD_ID)) {
			ArrayList depWrappers = segmentPanel.getSelectedFieldWrapper().getDependentWrappers();
			if (!depWrappers.isEmpty()) {
				HandShapeFieldWrapper hsFieldWrapper = (HandShapeFieldWrapper) depWrappers.get(0);
				HandShapeField hsField = hsFieldWrapper.getHandShapeField();
				depHsEntity = (HandShapeEntity) hsField.getEntityWithRefID(getID());
			}
		}

		if (depHsEntity != null) {
			MediaToolbarCommons.setHandshapeStartTime(depHsEntity, timeInfo);
		}
	}

	@Override
	public void setEndTimeInfo(Time timeInfo) {
		Event onset = manager.getChainedEventsCollection().getOnsetEvent();
		Event initialHold = manager.getChainedEventsCollection().getInitialHoldEvent();
		Event value = manager.getChainedEventsCollection().getValueOrTextValueEvent();
		Event finalHold = manager.getChainedEventsCollection().getFinalHoldEvent();
		Event offset = manager.getChainedEventsCollection().getOffsetEvent();
		
		boolean selectingPartOfThisEntity = false;
		
		Event selectedEvent = SS3Singleton.getMediaToolBar(null).selectedEvent();
		
		if (selectedEvent.equals(onset) || selectedEvent.equals(initialHold) || selectedEvent.equals(value)
				|| selectedEvent.equals(finalHold) || selectedEvent.equals(offset)) {
			selectingPartOfThisEntity = true;
		}
		
		if (!selectingPartOfThisEntity) {
			// special case for hand entities - should really be a sub-class
			ChainedEvent event = (ChainedEvent) manager.getLastEvent();
			if (event != null)
				event.setEndTimeInfo(timeInfo);
			
			return;			
		}
		
		Event nextEvent = manager.getChainedEventsCollection().getNextEvent(selectedEvent);
		
		selectedEvent.setEndTimeInfo(timeInfo);
		if (nextEvent != null) {
			nextEvent.setStartTimeInfo(timeInfo);
		}

		HandShapeEntity depHsEntity = null;
		if (getField().getFieldID().equals(Util.DOM_HAND_FIELD_ID) || getField().getFieldID().equals(Util.NDOM_HAND_FIELD_ID)) {
			ArrayList depWrappers = segmentPanel.getSelectedFieldWrapper().getDependentWrappers();
			if (!depWrappers.isEmpty()) {
				HandShapeFieldWrapper hsFieldWrapper = (HandShapeFieldWrapper) depWrappers.get(0);
				HandShapeField hsField = hsFieldWrapper.getHandShapeField();
				depHsEntity = (HandShapeEntity) hsField.getEntityWithRefID(getID());
			}
		}

		if (depHsEntity != null) {
			MediaToolbarCommons.setHandshapeEndTime(depHsEntity, timeInfo);
		}
	}
	
	@Deprecated
	public void setEndTime(double endTime) {
		ChainedEvent event = (ChainedEvent) manager.getLastEvent();
		if (event != null)
            event.setEndTimeInfo(SS3Singleton.getVideoTimeRange().time(Math.round(endTime), TimeUnit.TIMESTAMP));
	}

	@Override
	public Event getFirstEvent() {
		return manager.getFirstEvent();
	}

	public ChainedEvent getInitialHold() {
		return manager.getChainedEventsCollection().getInitialHoldEvent();
	}

	public ChainedEvent getFinalHold() {
		return manager.getChainedEventsCollection().getFinalHoldEvent();
	}

	public ChainedEvent getOnset() {
		return manager.getChainedEventsCollection().getOnsetEvent();
	}

	public void deleteOnset() {
		manager.deleteOnsetEventNoWarning();
	}

	public void deleteInitialHold() {
		manager.deleteInitialHoldEventNoWarning();
	}

	public void deleteFinalHold() {
		manager.deleteFinalHoldEventNoWarning();
	}

	public void deleteOffset() {
		manager.deleteOffsetEventNoWarning();
	}

	public ChainedEvent getOffset() {
		return manager.getChainedEventsCollection().getOffsetEvent();
	}

	@Override
	public Event getLastEvent() {
		return manager.getLastEvent();
	}

	@Override
	public Time getStartTimeInfo() {
		ChainedEvent event = (ChainedEvent) manager.getFirstEvent();
		if (event == null)
			return null;
		return event.getStartTimeInfo();
	}

	@Override
	public Time getEndTimeInfo() {
		ChainedEvent event = (ChainedEvent) manager.getLastEvent();
		if (event == null)
			return null;
		return event.getEndTimeInfo();
	}

	@Override
	public boolean selectEntityIfEventSelected() {
		if (manager.getSelectedEvent() != null) {
			select();
			return true;
		}
		return false;
	}

	@Override
	public boolean selectEventIfEntitySelected() {
		if (selected) {
			manager.unselectEntity();
			ChainedEvent valueEvent = manager.getChainedEventsCollection()
					.getValueEvent();
			manager.selectChainedEvent(valueEvent);
			selected = false;
			return true;
		}
		return false;
	}

	@Override
	public boolean select() {
		selected = true;
		return selected;
	}

	@Override
	public boolean unselect() {
		selected = false;
		unhighlight();
		manager.unselectEntity();
		return true;
	}

	@Override
	public boolean adjustEntity(int x1, int x2) {
		double pixelInTimeFrame = segmentPanel.getZoomer()
				.getPixelsInTimeFrame();
		double startTime = x1 / pixelInTimeFrame;
		double endTime = x2 / pixelInTimeFrame;
		return manager.adjustEntity(startTime, endTime);
	}

	@Override
	public Event getSelectedEvent() {		
		if (selected)
			return this;
		return manager.getSelectedEvent();
	}

	@Override
	public void reset() {
		selected = false;
		// mToolBar.setEnable(false);
		manager.resetAll();
	}

	@Override
	public boolean delete() {
		return delete(false);
	}

	public boolean delete(boolean showWarningMsg) {
		return manager.deleteEntity();
	}

	public void setValueID(String valueID) {
		ChainedEventsCollection collection = manager
				.getChainedEventsCollection();
		ChainedEvent event = collection.getValueEvent();
		event.setID(valueID);
	}

	@Override
	public Event getEventAt(double x) {
		return manager.getEventAt(x);
	}

	@Override
	public ArrayList<String> getEventsToAdd() {
		ArrayList<String> list = new ArrayList<String>();

		ChainedEventsCollection collection = manager
				.getChainedEventsCollection();

		if (collection.getOnsetEvent() == null)
			list.add(Constants.ONSET);

		if (collection.getInitialHoldEvent() == null)
			list.add(Constants.INITIAL_HOLD);

		if (collection.getFinalHoldEvent() == null)
			list.add(Constants.FINAL_HOLD);

		if (collection.getOffsetEvent() == null)
			list.add(Constants.OFFSET);

		return list;
	}
	
	@Override
	public ArrayList<Event> getEventsList() {
		return new ArrayList<Event>(getChainedEventsCollection().getCollection());
	}

	@Override
	public String getToolTipText(int x) {
		ChainedEvent event = (ChainedEvent) manager.getEventAt(x);

		if (event != null)
			return event.getToolTipText();
		return null;
	}

	@Override
	public boolean selectEvent(double x) {
		Event event = manager.getEventAt(x);
		return event.select();
	}

	@Override
	public void paint(Graphics2D g2d, double Y, Font font) {
		g2d.setFont(font);
		FontMetrics fontMetrics = getFontMetrics(font);

		segmentPanel.getPainter().drawEntity(this, g2d, Y, fontMetrics,
				getField().getFieldID(), previousEntity, nextEntity);
	}

	public void setInitialHoldEvent(ChainedEvent event) {
		if (event != null)
			event.setID("" + Constants.NONMANUAL_INITIAL_HOLD_CODE);
		manager.getChainedEventsCollection().setInitialHoldEvent(event);
	}

	public void setFinalHoldEvent(ChainedEvent event) {
		if (event != null)
			event.setID("" + Constants.NONMANUAL_FINAL_HOLD_CODE);
		manager.getChainedEventsCollection().setFinalHoldEvent(event);
	}

	public void setOnsetEvent(ChainedEvent event) {
		if (event != null)
			event.setID("" + Constants.NONMANUAL_ONSET_CODE);
		manager.getChainedEventsCollection().setOnsetEvent(event);
	}

	public void setOffsetEvent(ChainedEvent event) {
		if (event != null)
			event.setID("" + Constants.NONMANUAL_OFFSET_CODE);
		manager.getChainedEventsCollection().setOffsetEvent(event);
	}

	public String addOnsetEvent() {
		if (previousEntity != null) {
			Time endMovieTimeInfo = previousEntity.getEndTimeInfo();

			ChainedEvent event = (ChainedEvent) manager.getFirstEvent();
			Time selectedEventStartTimeInfo = segmentPanel.getMultipleMovieController()
					.getPrecedingFrameTimeInfo(event.getStartTimeInfo());

			if (selectedEventStartTimeInfo.lessThan(endMovieTimeInfo)) {
				return ErrorMessages.CHAINS_CANNOT_OVERLAP;
			}
		}

		manager.addOnsetEvent();
		return null;
	}

	public String addInitialHoldEvent() {
		ChainedEvent onset = getOnset();
		if (onset != null) {
			Time onsetStartTimeInfo = onset.getStartTimeInfo();
			Time onsetEndTimeInfo = onset.getEndTimeInfo();
			Time precedingFrameEndTimeInfo = segmentPanel.getMultipleMovieController()
				.getPrecedingFrameTimeInfo(onsetEndTimeInfo);

			if (precedingFrameEndTimeInfo.equals(onsetStartTimeInfo)) {
				return ErrorMessages.ADD_INITIAL_HOLD_CANT_ADJUST_ONSET;
			} else {
				manager.addInitialHoldEvent();
				return null;
			}
		}

		if (previousEntity != null) {
			Time endMovieTimeInfo = previousEntity.getEndTimeInfo();

			ChainedEvent event = (ChainedEvent) manager.getFirstEvent();
			Time selectedEventStartTimeInfo = segmentPanel.getMultipleMovieController()
					.getPrecedingFrameTimeInfo(event.getStartTimeInfo());

			if (selectedEventStartTimeInfo.lessThan(endMovieTimeInfo)) {
				return ErrorMessages.CHAINS_CANNOT_OVERLAP;
			}
		}

		manager.addInitialHoldEvent();
		return null;
	}

	public String addFinalHoldEvent() {
		// GREG TODO: should the order of the 2 below if-stmts be reversed, like in above addInitialHoldEvent ?
		if (nextEntity != null) {
			Time startMovieTimeInfo = nextEntity.getStartTimeInfo();

			ChainedEvent event = (ChainedEvent) manager.getLastEvent();
			Time selectedEventEndTimeInfo = segmentPanel.getMultipleMovieController()
					.getFollowingFrameTimeInfo(event.getEndTimeInfo());

			if (selectedEventEndTimeInfo.greaterThan(startMovieTimeInfo)) {
				return ErrorMessages.CHAINS_CANNOT_OVERLAP;
			}
		}

		ChainedEvent offset = getOffset();
		if (offset != null) {
			Time offsetStartTimeInfo = offset.getStartTimeInfo();
			Time offsetEndTimeInfo = offset.getEndTimeInfo();
			Time precedingFrameEndTimeInfo = segmentPanel.getMultipleMovieController()
				.getPrecedingFrameTimeInfo(offsetEndTimeInfo);

			if (precedingFrameEndTimeInfo.equals(offsetStartTimeInfo)) {
				return ErrorMessages.ADD_FINAL_HOLD_CANT_ADJUST_OFFSET;
			}
		}

		manager.addFinalHoldEvent();
		return null;
	}

	public String addOffsetEvent() {
		if (nextEntity != null) {
			Time startMovieTimeInfo = nextEntity.getStartTimeInfo();

			ChainedEvent event = (ChainedEvent) manager.getFirstEvent();
			Time selectedEventEndTimeInfo = segmentPanel.getMultipleMovieController()
					.getFollowingFrameTimeInfo(event.getEndTimeInfo());

			if (selectedEventEndTimeInfo.greaterThan(startMovieTimeInfo)) {
				return ErrorMessages.CHAINS_CANNOT_OVERLAP;
			}
		}

		manager.addOffsetEvent();
		return null;
	}
}
