package edu.bu.signstream.grepresentation.fields.nonManualField;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;

import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.time.*;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEvent;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.main.ToolBarPanels;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolBar;
import edu.bu.signstream.ui.panels.codingScheme.NewCSValuePanel;
import edu.bu.signstream.ui.popup.FieldValueSelectPopup;

public class NonManualFieldWrapper extends FieldWrapper {
	protected final NonManualField nmField;
	private MouseDblRightClickedDelegate rDblClickDelegate;
	protected Point lastPoint = null;

	public NonManualFieldWrapper(NonManualField nmField, SignStreamSegmentPanel segmentPanel, TimeRange timeRange) {
		super(nmField, segmentPanel, timeRange);
		this.nmField = nmField;
		rDblClickDelegate = new MouseDblRightClickedDelegate(nmField);
	}



	@Override
	public void highlight(boolean highlight, String refID) {
		field.highlight(highlight, refID);
	}

	@Override
	public void unselectAll() {
		nmField.unselect();
	}

	@Override
	public boolean contains(int Y) {
		return field.contains(Y);
	}

	// @TODO: implement
	@Override
	public void addEventsEntity(EventsEntity entity) {
		nmField.addChainedEventEntity((ChainedEventsEntity) entity);
	}

	// select event or entity located at given point
	@Override
	public void mouseClicked(Point point, JPanel panel) {

		ToolBarPanels panels = segmentPanel.getToolBarPanels();
		MediaToolBar mediaTb = panels.getMediaToolBar();

		EventsEntity entity = nmField.getEntityAt(point.x);
		Event event = nmField.getEventAt(point.x);
		if (event != null) {
//			event.select();
			// GD 2023-03-01 commenting, as the new way of doing select/deselect is in MediaToolBar, and these ways cause problems when gloss/other windows are open
			mediaTb.selectEvent(event, entity, nmField);
		}
	}

	// not functional
	@Override
	public void mouseRightClicked(Point point) {
	}

	// not functional
	@Override
	public void mouseLeftClicked(Point point) {
	}

	@Override
	public void mouseClickedAlt(Point point) {
	}

	// add new chained event
	@Override
	public void mousePressedAlt(Point point, JPanel panel) {
//		PresentationField parentField = nmField.getParentField();
//		Event parentEvent = parentField.getEventAt(point.x);

//		// if outside the bounds of any utterance, do not proceed
//		if (parentEvent == null) return;

        SS3Singleton.getMediaToolBar().deselect();

        var time = TimeRange.ABSOLUTE.time(point.x, TimeUnit.COORDINATE)
            .in(Util.getCurrentRange(getField()), Rounding.FLOOR);

		NewChainedEvent newEvent = new NewChainedEvent(time, segmentPanel);
		int numberOfEntities = nmField.getEntities().size();

		if (numberOfEntities == 0) {
			nmField.setNewChainedEvent(newEvent);
		} else {
			EventsEntity nextEntity = nmField.getNextEntity(time);
			EventsEntity prevEntity = nmField.getPreviousEntity(time);

			if (prevEntity != null && prevEntity.getEndTimeInfo().greaterThan(time)) {
				SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CHAINS_CANNOT_OVERLAP);
				return;
			}

			newEvent.setNextEntity(nextEntity);
			newEvent.setPreviousEntity(prevEntity);

			nmField.setNewChainedEvent(newEvent);
		}

		SS3Singleton.getMediaPlayer().seek(time);
	}

	// change field ID
	@Override
	public void mouseDblLeftClicked(final Point point, final JPanel panel) {
		Event event = SS3Singleton.getMediaToolBar(null).selectedEvent();

		// if event is null by standard procedure, then try the procedure from
		// mouseClicked() before returning
		if (event == null) {
			MediaToolBar mediaTb = SS3Singleton.getMediaToolBar(null);

			EventsEntity entity = nmField.getEntityAt(point.x);
			event = nmField.getEventAt(point.x);
			if (event != null) {
				event.select();
				mediaTb.selectEvent(event, entity, nmField);
			} else {
				return;
			}
		}

		if (!isEntitySelected()) {
			final ChainedEvent cEvent = (ChainedEvent) event;

			ChainedEventsEntity entity = (ChainedEventsEntity) SS3Singleton.getMediaToolBar(null)
					.selectedEntity();

			if (cEvent.equals(entity.getValue())) {
		          var popup = new FieldValueSelectPopup(getField(), segmentPanel, id -> {
		        	  cEvent.setID(id);
		        	  
		              SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
		              segmentPanel.repaint();
		          }, () -> {
		              SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
		              segmentPanel.repaint();
		          });

		          popup.display(panel, point);
			}
		}
	}

	@Override
	public void mouseDblRightClicked(Point point, JPanel panel) {
	}

	// add new event (hold. onset, offset) to selected entity
	public void mouseCtrlAltClicked(Point point, JPanel panel) {
		EventsEntity ent = nmField.getEntityAt(point.x);

		if (ent == null)
			return;

		ChainedEventsEntity entity = (ChainedEventsEntity) ent;

		if (isEntitySelected()) {
			rDblClickDelegate.entityDblClick(entity, point, panel);
		} else {
			Event event = nmField.getEventAt(point.x);
			ChainedEvent cEvent = (ChainedEvent) event;
			rDblClickDelegate.dblClick(cEvent, entity, point, panel);
		}
	}

	// MouseMotionListener
	@Override
	public void mouseMoved(Point point, JPanel panel) {
		panel.setCursor(new Cursor(getCursorType(point)));

		Event event = nmField.getEventAt(point.x);
		if (event != null) {
			panel.setToolTipText(event.getToolTipText());
		} else {
			panel.setToolTipText("");
		}
	}

	public int getCursorType(Point point) {
		Event event = nmField.getSelectedEvent();

		if (event != null) {
			double startCoord = event.getStartTimeCoordinate();
			double endCoord = event.getEndTimeCoordinate();

			if (startCoord == endCoord) {
				if (startCoord - 3 <= point.x && point.x <= startCoord - 1) {
					MOVE_DIRECTION = MOVE_START_POINT;
					return Cursor.E_RESIZE_CURSOR;
				} else if (startCoord + 1 <= point.x && point.x <= startCoord + 3) {
					MOVE_DIRECTION = MOVE_END_POINT;
					return Cursor.E_RESIZE_CURSOR;
				} else {
					return Cursor.DEFAULT_CURSOR;
				}
			} else {
				if (startCoord - 3 < point.x && point.x < startCoord + 3) {
					MOVE_DIRECTION = MOVE_START_POINT;
					return Cursor.E_RESIZE_CURSOR;
				} else if (endCoord - 3 < point.x && point.x < endCoord + 3) {
					MOVE_DIRECTION = MOVE_END_POINT;
					return Cursor.E_RESIZE_CURSOR;
				} else {
					return Cursor.DEFAULT_CURSOR;
				}
			}
		}

		return Cursor.DEFAULT_CURSOR;
	}

	private boolean drawNewEvent(Time time) {
		NewChainedEvent newEvent = nmField.getNewChainedEvent();
		if (newEvent != null) {
			EventsEntity nextEntity = newEvent.getNextEntity();
			EventsEntity prevEntity = newEvent.getPreviousEntity();

            if (prevEntity != null) {
                var prevEnd = prevEntity.getEndTimeInfo();
                if (time.lessThan(prevEnd))
                    time = prevEnd.in(Util.getCurrentRange(getField()), Rounding.CEIL);
            }

            if (nextEntity != null) {
                var nextStart = nextEntity.getStartTimeInfo();
                if (time.greaterThan(nextStart))
                    time = nextStart.in(Util.getCurrentRange(getField()), Rounding.FLOOR);
            }

			newEvent.setEndTimeInfo(time);

			SS3Singleton.getMediaPlayer().seek(time);

			return true;
		}
		return false;
	}

	public void mouseDraggedAlt(Point p) {
        var time = TimeRange.ABSOLUTE.time(p.x, TimeUnit.COORDINATE)
            .in(Util.getCurrentRange(getField()), Rounding.FLOOR);

        if (!drawNewEvent(time)) {
            ChainedEventsEntity entity = (ChainedEventsEntity) nmField.getSelectedEntity();
            if (entity != null) {
                if (MOVE_DIRECTION == MOVE_START_POINT) {
                    time = time.in(entity.getEndTimeInfo().range());
                    String resp = entity.canSetSelectedEventStartMovieTime(time);

                    if (resp != null) {
                        SS3Singleton.getErrorMessages().showErrorMessage(resp);
                        return;
                    }

                    entity.setStartTimeInfo(time);
                }

                if (MOVE_DIRECTION == MOVE_END_POINT) {
                    time = time.in(entity.getStartTimeInfo().range());
                    String resp = entity.canSetSelectedEventEndMovieTime(time);

                    if (resp != null) {
                        SS3Singleton.getErrorMessages().showErrorMessage(resp);
                        return;
                    }

                    entity.setEndTimeInfo(time);
                }
            }

            SS3Singleton.getMediaPlayer().seek(time);
        }
    }

	private void setMovieTime(Time movieTimeInfo) {
	    SS3Singleton.getMediaPlayer().seek(movieTimeInfo);
	}

	@Override
	public void mouseReleased(final Point point, final JPanel panel) {
		// clear the lastPoint since the user has finished dragging the mouse
		lastPoint = null;

		NewChainedEvent newEvent = nmField.getNewChainedEvent();
		if (newEvent != null) {

            var range = Util.getCurrentRange(getField());
			Time start = newEvent.getStartTimeInfo().in(range);
			Time end = newEvent.getEndTimeInfo().in(range);
			if (end.lessThan(start)) {
				var temp = end;
				end = start;
				start = temp;
			}

//            PresentationField parentField = nmField.getParentField();
//            Event utterance = parentField.getEventAt(point.x);
//			  if (start.equals(end) || utterance == null || !utterance.interval().contains(start.to(end), Bound.INCLUSIVE)) {
			if (start.equals(end)) {
                nmField.setNewChainedEvent(null);
                return;
            }

            final ChainedEvent event = new ChainedEvent(segmentPanel);
            event.setStartTimeInfo(start.in(range));
            event.setEndTimeInfo(end.in(range));

            ChainedEventsCollection collection = new ChainedEventsCollection();
            collection.setValueEvent(event);

            var popup = new FieldValueSelectPopup(getField(), segmentPanel, id -> {
                event.setID(id);

    			SS3CodingScheme dbScheme = segmentPanel.getSS3Database().getSS3CodingScheme();
    			SS3Field currentField = dbScheme.getField(field.getFieldID());
    			SS3FieldValue csValue = currentField.getFieldValue(id);

    			if (csValue != null && !(event instanceof GlossChainedEvent)) {
    				event.setText(csValue.getLabelOrName());
    				event.setToolTipText(csValue.getName());
    			}

                ChainedEventsEntity entity = new ChainedEventsEntity(collection, segmentPanel, nmField);
                entity.updateParentEntity();

                nmField.addChainedEventEntity(entity);
                nmField.setNewChainedEvent(null);

                SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
                segmentPanel.repaint();
            }, () -> {
                nmField.setNewChainedEvent(null);

                SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
                segmentPanel.repaint();
            });

            popup.display(panel, point);
		}
	}

	@Override
	public void mouseHoldReleased(final Point point, final JPanel panel) {
		Event event = SS3Singleton.getMediaToolBar(null).selectedEvent();

		// if event is null by standard procedure, then try the procedure from
		// mouseClicked() before returning
		if (event == null) {
			MediaToolBar mediaTb = SS3Singleton.getMediaToolBar(null);

			EventsEntity entity = nmField.getEntityAt(point.x);
			event = nmField.getEventAt(point.x);
			if (event != null) {
				event.select();
				mediaTb.selectEvent(event, entity, nmField);
			} else {
				return;
			}
		}

		if (!isEntitySelected()) {
			final String fieldID = field.getFieldID();

			SS3CodingScheme dbScheme = segmentPanel.getSS3Database().getSS3CodingScheme();

			SS3Field field = dbScheme.getField(fieldID);
			final ArrayList<SS3FieldValue> values = field.getValuesAsList();
			final JPopupMenu popup = new JPopupMenu();
			final ChainedEvent cEvent = (ChainedEvent) event;

			ChainedEventsEntity entity = (ChainedEventsEntity) SS3Singleton.getMediaToolBar(null)
					.selectedEntity();

			if (cEvent.equals(entity.getValue())) {
				for (int i = 0; i < values.size(); i++) {
					final SS3FieldValue value = values.get(i);

					final JLabel label = new JLabel(value.getName());
					label.addMouseListener(new MouseAdapter() {
						@Override
						public void mouseClicked(MouseEvent arg0) {
							if (cEvent != null) {
								String id = value.getId();
								cEvent.setID(id);
								SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
								panel.repaint();
							}
							popup.setVisible(false);
						}

						@Override
						public void mouseEntered(MouseEvent arg0) {
							label.setForeground(Color.RED);
						}

						@Override
						public void mouseExited(MouseEvent arg0) {
							label.setForeground(Color.BLACK);
						}
					});

					popup.add(label);
				}

				final JLabel addNew = new JLabel("Add new value");
				addNew.addMouseListener(new MouseAdapter() {
					@Override
					public void mouseClicked(MouseEvent arg0) {
						if (cEvent != null) {
							popup.setVisible(false);
							
							final JDialog popup1 = new JDialog();
							popup1.setUndecorated(true);
							popup1.setResizable(false);
							NewCSValuePanel valuePanel = new NewCSValuePanel(fieldID, cEvent,
									nmField.getSignStreamSegmentPanel(), popup1);

							popup1.setSize(valuePanel.getPreferredSize());
							popup1.add(valuePanel);
							Dimension d = popup1.getPreferredSize();
							int width = (int) d.getWidth();
							int x = (int) point.getX() - width / 2;
							int y = (int) point.getY();

							Point loc = new Point(x, y);
							SwingUtilities.convertPointToScreen(loc, panel);
							popup1.setLocation(loc);
							popup1.setVisible(true);

							// SS3Singleton.getSignStreamApplication().forcePopupInvisibleOnWindowLostFocus(popup1);

							SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
							panel.repaint();
						}
					}

					@Override
					public void mouseEntered(MouseEvent arg0) {
						addNew.setForeground(Color.RED);
					}

					@Override
					public void mouseExited(MouseEvent arg0) {
						addNew.setForeground(Color.BLACK);
					}
				});

				popup.add(addNew);
				popup.setPreferredSize(popup.getPreferredSize());
				popup.show(panel, (int) point.getX(), (int) point.getY());
				popup.setVisible(true);

				SS3Singleton.getSignStreamApplication().forcePopupInvisibleOnWindowLostFocus(popup);
			}
		}
	}
}
