package edu
        .bu
        .signstream
        .grepresentation
        .fields
        .nonManualField
        .mouseDelegateListeners;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.logging.Level;

import javax.swing.*;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;

import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.FieldWrappersCollection;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.MorphologicalPhonologicalInfoDialog;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeEntity;
import edu.bu.signstream.grepresentation.fields.locationField.LocationEntity;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.signTypeField.SignTypeEntity;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.LabelObject;

public class DeleteActionListener implements ActionListener {

    /**
     * Selected chained events entity
     */
    private ChainedEventsEntity entity = null;

    /**
     * Sets selected chained events entity.
     * @param entity selected chained events entity
     */
    public void setChainedEventsEntity(ChainedEventsEntity entity) {
        this.entity = entity;
    }

    /**
     * Executes when an annotator wants to delete selected chained entity.
     * If selected entity is GlossChainedEventsEntity, then all related entities will be deleted.
     *
     * Field IDs
     * gloss 10000
     * hand  10501
     * hand shape 1000
     *
     * gloss 10001
     * hand 10502
     * hand shape 2000
     */
    @Override
	public void actionPerformed(ActionEvent e) {

        MorphologicalPhonologicalInfoDialog dialog = SS3Singleton
                .getNonManuelFieldDialogs().getMorphologicalPhonologicalInfoDialog();
        if (dialog != null && dialog.isVisible() &&
            (
                entity.equals(dialog.getCurrentGlossEntity()) ||
                entity.equals(dialog.getCurrentGlossEntity().getDependentGlossEntity())
            )) {
            dialog.discardChanges();
        }

        var app = SS3Singleton.getSignStreamApplication();

        final JPopupMenu popup = new JPopupMenu();

        JLabel msgLabel = new JLabel("Are you sure you want to delete selected entity?");
        UserInterfaceUtil.setLabelLNF(msgLabel);

        final JButton okBtn = new JButton("Proceed");
        final JButton cancelBtn = new JButton("Cancel");

        JPanel msgPanel = new JPanel();
        GridBagConstraints gbc = new GridBagConstraints();

        msgPanel.setLayout(new GridBagLayout());
        gbc.anchor = GridBagConstraints.WEST;

        gbc.gridwidth = 3;
        gbc.gridx = 0;
        gbc.gridy = 0;
        msgPanel.add(msgLabel, gbc);

        gbc.gridy = 1;
        gbc.gridx = 1;
        msgPanel.add(new JLabel(" "), gbc);

        gbc.gridwidth = 1;
        gbc.anchor = GridBagConstraints.EAST;
        gbc.gridy = 3;
        gbc.gridx = 0;
        msgPanel.add(new JLabel(" "), gbc);
        gbc.gridx = 1;
        msgPanel.add(okBtn, gbc);
        gbc.gridx = 2;
        msgPanel.add(cancelBtn, gbc);

        cancelBtn.addActionListener(new ActionListener() {
            @Override
			public void actionPerformed(ActionEvent e) {
                popup.setVisible(false);
                app.unblock();
            }
        });

        okBtn.addActionListener(new ActionListener() {
            @Override
			public void actionPerformed(ActionEvent e) {
                try {
                    entity.getField().deleteEntity(entity);

                    if (entity instanceof GlossChainedEventsEntity) {
                        GlossChainedEventsEntity glossChainedEventsEntity = (GlossChainedEventsEntity) entity;
                        deleteGlossChainedEventsEntity(glossChainedEventsEntity);

                        GlossChainedEventsEntity depGlossChainedEventsEntity = glossChainedEventsEntity.getDependentGlossEntity();
                        if (depGlossChainedEventsEntity != null) {
                            depGlossChainedEventsEntity.getField().deleteEntity(depGlossChainedEventsEntity);
                            deleteGlossChainedEventsEntity(depGlossChainedEventsEntity);
                        }
                    }

                    SS3Singleton.getMediaToolBar(null).selectedField().repaint();
                    SS3Singleton.getSignStreamApplication().repaint();
                    
                    SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);	// calls SSSP.updateLoadedDatabase();
					
                    // refresh the utterance lists to reflect the absence of this gloss now
					SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().getUtteranceView().updateUtteranceLists();
	                
	                SS3Singleton.getMediaToolBar().segmentPanel().getUtteranceView().getSegmentTemplate(
	                	entity.getField().getSegmentID()).getNonmanView()
	                	.getCSFieldDialog().getCSFieldPanel().showAllFieldsCurrentUtterance();
                } catch (NullPointerException ex) {
                    // hide the popup and log the exception.
                    SS3Singleton.getSignStreamApplication().logger.log(Level.WARNING, ex.getMessage());
                    popup.setVisible(false);
                }

                popup.setVisible(false);
                app.unblock();
            }
        });

        popup.addPopupMenuListener(new PopupMenuListener() {
            @Override public void popupMenuWillBecomeInvisible(PopupMenuEvent e) {
                app.unblock();
            }

            @Override public void popupMenuCanceled(PopupMenuEvent e) { }
            @Override public void popupMenuWillBecomeVisible(PopupMenuEvent e) { }
        });

        int width = msgPanel.getPreferredSize().width + 30;
        int height = msgPanel.getPreferredSize().height + 20;

        popup.add(msgPanel);
        popup.setPopupSize(new Dimension(width, height));
        popup.setLocation(250, 250);
        popup.setVisible(true);
        SS3Singleton.getSignStreamApplication().forcePopupInvisibleOnWindowLostFocus(popup);

        SwingUtilities.invokeLater(okBtn::requestFocus);

        app.block();
    }

    /**
     * Deletes GlossChainedEventsEntity and if available corresponding entities from Hand Field and from Hand Shape Field.
     *
     * @param glossChainedEventsEntity to delete
     * @param fieldID                  ID of entity's field, in current implementation it is 10000 or 10001
     * @param fieldWrappersCollection  a collection of loaded FieldWrappers
     */
    private void deleteGlossChainedEventsEntity(GlossChainedEventsEntity glossChainedEventsEntity) {

        ChainedEventsEntity dependentHandEntity = glossChainedEventsEntity.getDependentHandEntity();
        HandShapeEntity dependentHandShapeEntity = glossChainedEventsEntity.getDependentHandShapeEntity();
        SignTypeEntity dependentSignTypeEntity = glossChainedEventsEntity.getDependentSignTypeEntity();
        LocationEntity dependentLocationEntity = glossChainedEventsEntity.getDependentLocationEntity();
        
        if (glossChainedEventsEntity.isStartCompound()) {
        	glossChainedEventsEntity.getPreviousEntity().setEndCompound(null);
        	glossChainedEventsEntity.setStartCompound(null);
        }

        if (glossChainedEventsEntity.isEndCompound()) {
        	glossChainedEventsEntity.getNextEntity().setStartCompound(null);
        	glossChainedEventsEntity.setEndCompound(null);
        }

        if (dependentHandEntity != null)
        	dependentHandEntity.getField().deleteEntity(dependentHandEntity);
        
        if (dependentHandShapeEntity != null)
        	dependentHandShapeEntity.getField().deleteEntity(dependentHandShapeEntity);

        if (dependentLocationEntity != null)
        	dependentLocationEntity.getField().deleteEntity(dependentLocationEntity);

        if (dependentSignTypeEntity != null)
        	dependentSignTypeEntity.getField().deleteEntity(dependentSignTypeEntity);
    }
}
