package edu.bu.signstream.grepresentation.fields.partOfSpeechField;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;

import edu.bu.signstream.common.util.vo.Track;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Track;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEvent;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsCollection;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsEntityCollection;
import edu.bu.signstream.grepresentation.fields.nonManualField.NonManualField;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class PartOfSpeechField extends NonManualField {
   // 'syntactic grouping' //
   private Rectangle2D.Double selectedRect = null;
   private ArrayList sGroups = new ArrayList();

   public PartOfSpeechField(Track track, SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
        super(track, segmentPanel, parentField);
     }

     public PartOfSpeechField(SS3Track ss3track, SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
        super(ss3track, segmentPanel, parentField);
        createTrackEntity(ss3track);
     }
   
   public ArrayList getSyntacticGroups() {
      return sGroups;
   }
   
   @Override
public void addEventsEntity(EventsEntity entity) {
         PartOfSpeechEntity cEntity = (PartOfSpeechEntity) entity;
         collection.addEntity(cEntity);
      }

   public void startSelect(int x) {
      selectedRect = new Rectangle2D.Double();
      selectedRect.x = x;
      selectedRect.y = Y - 20;
      selectedRect.width = 0;
      selectedRect.height = 25;
   }

   public void select(int x) {
      if (selectedRect != null)
         selectedRect.width = x - selectedRect.x;
   }

   public void discardSelection() {
      selectedRect = null;
   }

   public void createNewSyntacticGroup() {
//      if (selectedRect != null) {
//         int X1 = (int) selectedRect.x;
//         int X2 = X1 + (int) selectedRect.width;
//         ArrayList entities = collection.getEntitiesInside(X1, X2);
//
//         if (entities.size() > 0) {
//            SyntacticGroup group = new SyntacticGroup();
//            group.setEntities(entities);
//            sGroups.add(group);
//         }
//      }
//      discardSelection();
   }
   
   @Override
protected void createTrackEntity(SS3Track track) {
         ArrayList chainedEvents = new ArrayList();
         ArrayList entities = track.getEntities();

         for (int j = 0; j < entities.size(); j++) {
            SS3Entity entity = (SS3Entity) entities.get(j);
            ChainedEventsCollection collection = new ChainedEventsCollection();

            ArrayList items = entity.getItemsAsList();
            for (int i = 0; i < items.size(); i++) {
               SS3Item item = (SS3Item) items.get(i);
               String id = item.getId();

               ChainedEvent event = new ChainedEvent(item, segmentPanel);
               collection.setValueEvent(event);
            }

            PartOfSpeechEntity lEntity =
               new PartOfSpeechEntity(collection, track.getId(), segmentPanel, this);
         
            lEntity.setID(entity.getID());
            lEntity.setRefEntityId(entity.getRefEntityId());
            if (entity.getParentEntityIDs() == null || entity.getParentEntityIDs().isEmpty()) {
            	lEntity.updateParentEntity();
            } else {
            	lEntity.setParentEntities(getParentField().getEntities(entity.getParentEntityIDs()));
            }
            //lEntity.setReference(entity.getReference());
            chainedEvents.add(lEntity);
         }
         collection = new ChainedEventsEntityCollection(chainedEvents, segmentPanel);
      }

//   public PartOfSpeechField() {
//      super(new SS3Track());
//   }

   public SyntacticGroup getSyntacticGroupAt(int x) {
      for (int i = 0; i < sGroups.size(); i++) {
         SyntacticGroup group = (SyntacticGroup) sGroups.get(i);
         if (group.contains(x))
            return group;
      }
      return null;
   }

   @Override
public void paint(Graphics2D g2d, double Y, Font font) {
      //
      //System.err.println("PAINT");
      //
      for (int i = 0; i < sGroups.size(); i++) {
         SyntacticGroup group = (SyntacticGroup) sGroups.get(i);
         double startCoord = group.getStartTimeCoordinate();
         double endCoord = group.getEndTimeCoordinate();

         g2d.setColor(Color.DARK_GRAY);
         Font fnt1 =
            new Font(font.getName(), font.getStyle(), font.getSize() + 2);
         Font fnt2 =
            new Font(font.getName(), font.getStyle(), font.getSize() - 3);

         g2d.setFont(fnt1);
         g2d.drawString("[", (int) startCoord - 5, (int) Y);
         g2d.drawString("]", (int) endCoord + 2, (int) Y);
         g2d.setFont(fnt2);
         g2d.drawString(group.getLabel(), (int) endCoord + 8, (int) Y + 5);

         //			group.setLabelStartCoord(endCoord+8);
         //			group.setLabelEndCoord(endCoord+18);
      }

      //		selectedRect.x = 50;
      //		selectedRect.y = Y - 20;
      //		selectedRect.width = 300;
      //		selectedRect.height = 25;
      if (selectedRect != null) {
         g2d.setColor(Color.LIGHT_GRAY);
         g2d.fill(selectedRect);
      }

      super.paint(g2d, Y, font);
   }
}
