package edu.bu.signstream.grepresentation.fields.presentation;

import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.util.ArrayList;

import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowItem;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;


public class PresentationEventsEntity extends EventsEntity {

	private static final long serialVersionUID = 1L;

	private PresentationEvent event;

	private ArrayList<PresentationEventsEntity> dependentEntities = new ArrayList<PresentationEventsEntity>();

	public PresentationEventsEntity(PresentationEvent event,
			SignStreamSegmentPanel segmentPanel, Field field) {
		super(segmentPanel, field);
		this.event = event;
	}

	public void addDependentEntity(PresentationEventsEntity entity) {
		dependentEntities.add(entity);
	}

	public ArrayList<PresentationEventsEntity> getDependentEntities() {
		return dependentEntities;
	}

	public SS3GlossWindowItem getSS3GlossWindowItem() {
		return event.getSS3GlossWindowItem();
	}

	@Override
	public PresentationEventsEntity clone() {
		return new PresentationEventsEntity(event.clone(), 
				segmentPanel, getField());
	}

	public PresentationEvent getEvent() {
		return event;
	}
	
	// GREG TODO: there is probably a better way to go about this, but as of now (2021-03-23) it seems ID is always null
	@Override
	public String getID() {
		String ID = "";
		
		if (event != null)
			ID = event.getID();
		
		return ID;
	}

//	public boolean isUpdatePossible(SSObservableJPanel subj, Object o,
//			Object command) {
//		return true;
//	}

	@Override
	public double getStartTimeCoordinate() {
		return event.getStartTimeCoordinate();
	}

	@Override
	public double getEndTimeCoordinate() {
		return event.getEndTimeCoordinate();
	}

	@Override
	public void setStartTimeCoordinate(double coord) {
		event.setStartTimeCoordinate(coord);
	}

	@Override
	public void setEndTimeCoordinate(double coord) {
		event.setEndTimeCoordinate(coord);
	}

	public double getStartTime() {
		return event.getStartTimeInfo().doubleValue();
	}

	public double getEndTime() {
		return event.getEndTimeInfo().doubleValue();
	}

	public void setStartTime(double startTime) {
	    event.setStartTimeInfo(SS3Singleton.getVideoTimeRange().time(Math.round(startTime), TimeUnit.TIMESTAMP));
	}

	public void setEndTime(double endTime) {
        event.setEndTimeInfo(SS3Singleton.getVideoTimeRange().time(Math.round(endTime), TimeUnit.TIMESTAMP));
	}

	@Override
	public boolean selectEntityIfEventSelected() {
		if (event.isSelected())
			return select();
		return false;
	}

	@Override
	public boolean selectEventIfEntitySelected() {
		if (isSelected())
			return event.select();
		return false;
	}

	@Override
	public boolean select() {
		selected = true;
		return selected;
	}

	@Override
	public boolean unselect() {
		selected = false;
		event.unselect();
		return selected && event.isSelected();
	}

	// @TODO: implement
	@Override
	public boolean adjustEntity(int x1, int x2) {
		return true;
	}

	@Override
	public boolean isSelected() {
		return selected;
	}

	@Override
	public Event getSelectedEvent() {
		if (event.isSelected())
			return event;
		return null;
	}

	@Override
	public void reset() {
	}

	// @TODO: implement
	@Override
	public boolean delete() {
		return true;
	}

	@Override
	public Event getEventAt(double x) {
		if (event.contains(x))
			return event;
		return null;
	}


	@Override
	public String getToolTipText(int x) {
		return "ToolTip";
	}

	public boolean setSelectedEventStartCoordinate(double coord) {
		if (event.isSelected()) {
			event.setStartTimeCoordinate(coord);
			return true;
		}
		return false;
	}

	public boolean setSelectedEventEndCoordinate(double coord) {
		if (event.isSelected()) {
			event.setEndTimeCoordinate(coord);
			return true;
		}
		return false;
	}

	@Override
	public boolean selectEvent(double x) {
		if (event.contains(x))
			return event.select();
		return false;
	}

	@Override
	public Event getFirstEvent() {
		return event;
	}

	@Override
	public Event getLastEvent() {
		return event;
	}

	@Override
	public void paint(Graphics2D g2d, double Y, Font font) {
		g2d.setFont(font);
		FontMetrics fontMetrics = getFontMetrics(font);
		segmentPanel.getPainter().drawPresentationEntity(this, g2d, Y,
				fontMetrics);
	}

	@Override
	public Time getStartTimeInfo() {
		return getFirstEvent().getStartTimeInfo();
	}

	@Override
	public Time getEndTimeInfo() {
		return getLastEvent().getEndTimeInfo();
	}

	@Override
	public void setStartTimeInfo(Time startTimeInfo) {
		getFirstEvent().setStartTimeInfo(startTimeInfo);
	}

	@Override
	public void setEndTimeInfo(Time endTimeInfo) {
		getLastEvent().setEndTimeInfo(endTimeInfo);
	}

	@Override
	public SS3Entity getSS3Entity() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public ArrayList<String> getEventsToAdd() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public ArrayList<Event> getEventsList() {
		ArrayList<Event> events = new ArrayList<Event>();
		events.add(getEvent());		// only one event in a PresentationEventsEntity
		return events;
	}
}
