package edu.bu.signstream.grepresentation.fields.signTypeField;

import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.time.Interval;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

import java.awt.*;

public class SignTypeEvent implements Event {

    private String id;

    private Time startTimeInfo, endTimeInfo;

    private SignType signType;
    private boolean nameSign;

    private SignStreamSegmentPanel segmentPanel;

    public SignTypeEvent(SignStreamSegmentPanel segmentPanel, Time startTimeInfo, Time endTimeInfo) {
        this.segmentPanel = segmentPanel;
        setStartTimeInfo(startTimeInfo);
        setEndTimeInfo(endTimeInfo);
    }

    public SignTypeEvent(SignStreamSegmentPanel segmentPanel, SS3Item item) {
        this(segmentPanel, item.getStartTimeInfo(), item.getEndTimeInfo());
    }

    public SignType getSignType() {
        return signType;
    }

    public void setSignType(SignType signType) {
        this.signType = signType;
    }

    public boolean isNameSign() {
        return nameSign;
    }

    public void setNameSign(boolean nameSign) {
        this.nameSign = nameSign;
    }

    @Override
    public Time getStartTimeInfo() {
        return startTimeInfo;
    }

    @Override
    public Time getEndTimeInfo() {
        return endTimeInfo;
    }

    @Override
    public Interval interval() {
        return new Interval(startTimeInfo, endTimeInfo);
    }

    @Override
    public void setStartTimeInfo(Time startTimeInfo) {
        this.startTimeInfo = startTimeInfo;
    }

    @Override
    public void setEndTimeInfo(Time endTimeInfo) {
        this.endTimeInfo = endTimeInfo;
    }

    @Override
    public double getStartTimeCoordinate() {
        return segmentPanel.getZoomer().getCoordinateTimeConvertor()
                .convertTimeInfoToCoordinate(startTimeInfo);
    }

    @Override
    public double getEndTimeCoordinate() {
        return segmentPanel.getZoomer().getCoordinateTimeConvertor()
                .convertTimeInfoToCoordinate(endTimeInfo);
    }

    @Override
    public void setStartTimeCoordinate(double coord) {
        double newMovieTime = segmentPanel.getZoomer().getCoordinateTimeConvertor().convertCoordinateToTime(coord);
        startTimeInfo = SS3Singleton.getVideoTimeRange().time(Math.round(newMovieTime), TimeUnit.TIMESTAMP);
    }

    @Override
    public void setEndTimeCoordinate(double coord) {
        double newMovieTime = segmentPanel.getZoomer().getCoordinateTimeConvertor().convertCoordinateToTime(coord);
        endTimeInfo = SS3Singleton.getVideoTimeRange().time(Math.round(newMovieTime), TimeUnit.TIMESTAMP);
    }

    @Override
    public String getToolTipText() {
        return "Sign Type: " + signType.name().toLowerCase();
    }

    @Override
    public boolean select() {
        return false;
    }

    @Override
    public boolean unselect() {
        return false;
    }

    @Override
    public void highlight() {}

    @Override
    public void unhighlight() {}

    @Override
    public boolean isSelected() {
        return false;
    }

    @Override
    public boolean isHighlighted() {
        return false;
    }

    @Override
    public void setColor(Color color) {}

    @Override
    public Color getColor() { return null; }

    @Override
    public String getID() {
        return id;
    }

    @Override
    public void setID(String id) {
        this.id = id;
    }
}
