package edu.bu.signstream.grepresentation.listener;

import java.awt.Cursor;
import java.awt.Point;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.ArrayList;
import java.util.Timer;
import java.util.TimerTask;
import java.util.logging.Level;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.grepresentation.fields.*;
import edu.bu.signstream.grepresentation.fields.presentation.*;
import edu.bu.signstream.time.Rounding;
import edu.bu.signstream.time.TimeRange;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.SegmentGraphicRepresentation;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.main.Slider;
import edu.bu.signstream.grepresentation.view.main.ToolBarPanels;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolBar;
import edu.bu.signstream.ui.SS3SignStreamApplication;
import edu.bu.signstream.ui.video.MultipleMovieController;

public class SegmentMouseListener implements MouseListener, MouseMotionListener {
	private SignStreamSegmentPanel segmentPanel;

	private boolean mouseDraggedCtrlAlt = false;
	
	private int pressStatus = 0;
	private Timer timer;

	public SegmentMouseListener(SignStreamSegmentPanel segmentPanel) {
		this.segmentPanel = segmentPanel;
	}

	/**
	 * set resizable cursor (chained events, chained event entity, and slider)
	 * and show chained event duration
	 */
	@Override
	public void mouseMoved(MouseEvent e) {
		
		Point point = e.getPoint();
		FieldWrapper wrapper = getFieldWrapperAt(point);

		if (wrapper != null) {
            SS3Singleton.getMediaToolBar().activate(wrapper.getField());
            wrapper.mouseMoved(point, segmentPanel.getDatabaseView()
                .getSegmentGraphicRepresentation());
        } else {
		    SS3Singleton.getMediaToolBar().deactivate();
        }

		Slider slider = segmentPanel.getSlider();

		if (slider != null && point.y < SegmentGraphReprUtil.axesBottomHeight
				&& point.x - 3 < slider.getSliderPosition()
				&& point.x + 3 > slider.getSliderPosition())
			segmentPanel.getDatabaseView().getSegmentGraphicRepresentation()
					.setCursor(new Cursor(Cursor.E_RESIZE_CURSOR));
	}

    @Override
    public void mouseEntered(MouseEvent e) {

    }

    @Override
    public final void mouseExited(MouseEvent e) {
        SS3Singleton.getMediaToolBar().deactivate();
    }


    @Override
	public void mouseDragged(MouseEvent e) {
//		SS3SignStreamApplication.logger.log(Level.INFO, "Mouse Dragged");
		
		pressStatus = 0;	// pressStatus is only for a static press in order to trigger mouseHoldReleased()

        var player = SS3Singleton.getMediaPlayer();
        if (player.playing())
            player.pause();

        Point point = e.getPoint();
		FieldWrapper wrapper = getFieldWrapperAt(point);

		SegmentGraphicRepresentation rep = segmentPanel.getDatabaseView()
				.getSegmentGraphicRepresentation();

		if (isAltClick(e) || isCtrlPressed(e)) {
			mouseDraggedCtrlAlt = true;

			FieldWrapper altSelectedWrapper = getFieldWrapperWithNewEvent();
			if (altSelectedWrapper != null) {
				altSelectedWrapper.mouseDragged(point, rep);
			} else {
				if (wrapper != null) {
					wrapper.mouseDragged(point, rep);
				}
			}
		} else {
            var range = Util.getCurrentRange(wrapper == null ? null : wrapper.getField());
            var time = TimeRange.ABSOLUTE.time(point.x, TimeUnit.COORDINATE)
                .in(range, Rounding.FLOOR);
            player.seek(time);
		}
	}

	private FieldWrapper getFieldWrapperAt(Point point) {
		SegmentGraphicRepresentation rep = segmentPanel.getDatabaseView()
				.getSegmentGraphicRepresentation();
		FieldWrappersCollection masterWrappersCollection = rep
				.getFieldWrappersCollection();
		return masterWrappersCollection.getFieldWrapperAt(point.y);
	}

	private FieldWrapper getFieldWrapperWithSelectedEvent() {
		FieldWrapper wrapper = null;
		ArrayList<Segment> segments = segmentPanel.getSegments();
		for (Segment segment : segments) {
			FieldWrappersCollection collection = segment
					.getFieldWrappersCollection();
			wrapper = collection.getSelectedFieldWrapper();
		}
		return wrapper;
	}

	private FieldWrapper getFieldWrapperWithNewEvent() {
		SegmentGraphicRepresentation rep = segmentPanel.getDatabaseView()
				.getSegmentGraphicRepresentation();
		FieldWrappersCollection collection = rep.getFieldWrappersCollection();
		ArrayList<FieldWrapper> wrappers = collection.getWrappers();
		for (FieldWrapper wrp : wrappers) {
			if (wrp instanceof PresentationFieldWrapper) {
				PresentationFieldWrapper pFldWrp = (PresentationFieldWrapper) wrp;
				PresentationField pFld = (PresentationField) pFldWrp.getField();
				if (pFld.getNewEvent() != null) {
					return wrp;
				}
			}
		}
		return null;
	}

	@Override
	public void mousePressed(MouseEvent e) {
		MultipleMovieController controller = segmentPanel .getMultipleMovieController();
		controller.pause();

		SS3SignStreamApplication.logger.log(Level.INFO, "Mouse Pressed");
        Util.dragging(true);

        if (isAltClick(e) || isCtrlPressed(e)) {
			FieldWrapper wrapper = getFieldWrapperAt(e.getPoint());
			if (wrapper != null) {
				Event event = wrapper.getEntityAt(e.getX());
				if (event == null) {
					wrapper.mousePressedAlt(e.getPoint(), segmentPanel);		// attempt to begin drawing new line to create new entity
				}
			}
		} else {	// if normal click, prepare for click and hold
			pressStatus = 0;

			if (timer == null)
				timer = new Timer();

			// after a short wait trigger pressStatus - then it's checked in mouseReleased()
			timer.schedule(new TimerTask() {
				@Override
				public void run() {
					pressStatus = 1;

					if (timer != null) {
						timer.cancel();
						timer = null;
					}					
				}
			}, Util.MILLISECONDS_WAIT_LISTENERS, 1);
		}

		segmentPanel.getDatabaseView().getSegmentGraphicRepresentation()
				.repaint();
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		SS3SignStreamApplication.logger.log(Level.INFO, "Mouse Released");
		Util.dragging(false);
		
		if (timer != null) {
	        timer.cancel();
	        timer = null;
	    }

		// pressStatus gets set to 1 via the TimerTask in mouseDragged, and here trigger mouseHoldReleased
		if (pressStatus == 1) {
			pressStatus = 0;

			FieldWrapper wrapper = getFieldWrapperAt(e.getPoint());
			if (wrapper != null) {
				Event event = wrapper.getField().getEventAt(e.getPoint().x);
				if (event == null) {
					return;
				} else {
					if (wrapper instanceof PresentationFieldWrapper) {
						SegmentGraphicRepresentation rep = segmentPanel.getDatabaseView()
								.getSegmentGraphicRepresentation();
						PresentationFieldWrapper prsFldWrapper = (PresentationFieldWrapper) wrapper;
						
						prsFldWrapper.mouseHoldReleased(e.getPoint(), rep);
						return;
					}
				}
			}
		}
		
		MultipleMovieController controller = segmentPanel .getMultipleMovieController(); 
		controller.pause();
		
		SegmentGraphicRepresentation rep = segmentPanel.getDatabaseView().getSegmentGraphicRepresentation();
		
		FieldWrapper newSelectedWrapper = getFieldWrapperWithNewEvent();
		
		if (newSelectedWrapper != null) {
			mouseDraggedCtrlAlt = false;
			newSelectedWrapper.mouseReleased(e.getPoint(), rep);
		} else {
			FieldWrapper wrapper = getFieldWrapperAt(e.getPoint());

			if (wrapper != null) {
				wrapper.mouseReleased(e.getPoint(), rep);
			}

			if (rep != null)
				rep.repaint();
		}
	} 

	// select chained event or chained events entity
	@Override
	public void mouseClicked(final MouseEvent e) {

		SS3SignStreamApplication.logger.log(Level.INFO, "Mouse Clicked");
		
		MultipleMovieController controller = segmentPanel .getMultipleMovieController(); 
		controller.pause();

		SegmentGraphicRepresentation rep = segmentPanel.getDatabaseView()
				.getSegmentGraphicRepresentation();

		unselectAll();

		ToolBarPanels panels = segmentPanel.getToolBarPanels();
		MediaToolBar mediaTb = panels.getMediaToolBar();

		FieldWrapper wrapper = getFieldWrapperAt(e.getPoint());

		boolean altPressed = isAltClick(e);
		boolean ctrlPressed = isCtrlPressed(e);

		if (wrapper == null || wrapper.getField() == null)
			return;

		wrapper.mouseClicked(e.getPoint(), rep);
		
		segmentPanel.setSelectedFieldWrapper(wrapper);

		Event event = wrapper.getField().getEventAt(e.getPoint().x); // null;

		if (event == null && mediaTb != null) {
            var range = Util.getCurrentRange(wrapper.getField());
            var time = TimeRange.ABSOLUTE.time(e.getPoint().x, TimeUnit.COORDINATE)
                .in(range, Rounding.FLOOR);
            SS3Singleton.getMediaPlayer().seek(time);
		}

//		if (event != null && mediaTb != null) {
//			unselectAll();
//			event.select();
//			EventsEntity entity = wrapper.getSelectedEntity();
//			mediaTb.selectEvent(event, entity, wrapper.getField());
//		}

		if (wrapper instanceof PresentationFieldWrapper) {
		    // Temporal Partition Field
			if (event != null) {
			    if (mediaTb != null) {
			        var entity = (PresentationEventsEntity)event;
			        entity.select();
                    mediaTb.select(entity.getEvent(), entity);
                }

				PresentationFieldWrapper prsFldWrapper = (PresentationFieldWrapper) wrapper;
				if (e.getClickCount() == 2 && !ctrlPressed && !altPressed && e.getButton() == MouseEvent.BUTTON1) {
					prsFldWrapper.mouseDblLeftClicked(e.getPoint(), rep);	// open utterance, by default segment tier
					return;
				} else if (ctrlPressed || altPressed || e.getButton() == MouseEvent.BUTTON3) {
					prsFldWrapper.mouseCtrlAltClicked(e.getPoint());	// bring up menu
					return;
				}
			}
		} else if (wrapper instanceof SegmentFieldWrapper) {
		    // Segment Tier Field
            if (event != null) {
                var entity = (PresentationEventsEntity)event;
                entity.select();
                mediaTb.select(entity.getEvent(), entity);

                SegmentFieldWrapper segFldWrapper = (SegmentFieldWrapper) wrapper;
				if (e.getClickCount() == 2 && !ctrlPressed && !altPressed && e.getButton() == MouseEvent.BUTTON1) {
                    segFldWrapper.mouseDblLeftClicked(e.getPoint(), rep);    // open utterance, this (clicked) segment tier
                    return;
                }
//                } else if (ctrlPressed || altPressed || e.getButton() == MouseEvent.BUTTON3) {
//                    segFldWrapper.mouseCtrlAltClicked(e.getPoint());	// bring up menu
//                    return;
//                }
			}
		}

		if (altPressed) {
			wrapper.mouseClickedAlt(e.getPoint());
		}

		segmentPanel.repaint();
	}

	private boolean isShiftPressed(MouseEvent e) {
		boolean pressed = (e.getModifiers() & InputEvent.SHIFT_DOWN_MASK) == InputEvent.SHIFT_DOWN_MASK;
		if (pressed) {
			return pressed;
		} else {
			return (e.getModifiers() & InputEvent.SHIFT_MASK) == InputEvent.SHIFT_MASK;
		}
	}

	private boolean isAltClick(MouseEvent e) {
		return (e.getModifiers() & InputEvent.ALT_MASK) == InputEvent.ALT_MASK;
	}

	private boolean isCtrlPressed(MouseEvent e) {
		return (e.getModifiers() & InputEvent.CTRL_MASK) == InputEvent.CTRL_MASK;
	}

	private void unselectAll() {
		SegmentGraphicRepresentation rep = segmentPanel.getDatabaseView()
				.getSegmentGraphicRepresentation();
		FieldWrappersCollection masterWrappersCollection = rep
				.getFieldWrappersCollection();
		masterWrappersCollection.unselectAll();

        SS3Singleton.getMediaToolBar().deselect();
	}
}
