package edu.bu.signstream.grepresentation.reguestsHandler;

import java.util.ArrayList;

import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldType;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Track;
import edu.bu.signstream.time.TimeRange;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.apexField.ApexField;
import edu.bu.signstream.grepresentation.fields.apexField.ApexFieldWrapper;
import edu.bu.signstream.grepresentation.fields.customField.CustomIntervalField;
import edu.bu.signstream.grepresentation.fields.customField.CustomIntervalFieldWrapper;
import edu.bu.signstream.grepresentation.fields.customField.CustomPointField;
import edu.bu.signstream.grepresentation.fields.customField.CustomPointFieldWrapper;
import edu.bu.signstream.grepresentation.fields.customField.CustomSectionField;
import edu.bu.signstream.grepresentation.fields.customField.CustomSectionFieldWrapper;
import edu.bu.signstream.grepresentation.fields.domNdomField.DomNdomHandFieldWrapper;
import edu.bu.signstream.grepresentation.fields.freeTextField.FreeTextField;
import edu.bu.signstream.grepresentation.fields.freeTextField.FreeTextFieldWrapper;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureField;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureFieldWrapper;
import edu.bu.signstream.grepresentation.fields.gestureField.domNdomField.GestureDomNdomHandFieldWrapper;
import edu.bu.signstream.grepresentation.fields.gestureField.handshapes.GestureHandShapeField;
import edu.bu.signstream.grepresentation.fields.gestureField.handshapes.GestureHandShapeFieldWrapper;
import edu.bu.signstream.grepresentation.fields.gestureField.locationField.GestureLocationField;
import edu.bu.signstream.grepresentation.fields.gestureField.locationField.GestureLocationFieldWrapper;
import edu.bu.signstream.grepresentation.fields.gestureField.orientationField.GestureOrientationField;
import edu.bu.signstream.grepresentation.fields.gestureField.orientationField.GestureOrientationFieldWrapper;
import edu.bu.signstream.grepresentation.fields.gestureField.signTypeField.GestureTypeField;
import edu.bu.signstream.grepresentation.fields.gestureField.signTypeField.GestureTypeFieldWrapper;
import edu.bu.signstream.grepresentation.fields.glossField.GlossField;
import edu.bu.signstream.grepresentation.fields.glossField.GlossFieldWrapper;
import edu.bu.signstream.grepresentation.fields.graphField.GraphField;
import edu.bu.signstream.grepresentation.fields.graphField.GraphFieldWrapper;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeField;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeFieldWrapper;
import edu.bu.signstream.grepresentation.fields.handShapes.joints.JointsField;
import edu.bu.signstream.grepresentation.fields.handShapes.joints.JointsFieldWrapper;
import edu.bu.signstream.grepresentation.fields.lTranslationField.LiteralTranslationWrapper;
import edu.bu.signstream.grepresentation.fields.lTranslationField.LiteraltranslationField;
import edu.bu.signstream.grepresentation.fields.legacyGlossField.LegacyGlossFieldWrapper;
import edu.bu.signstream.grepresentation.fields.locationField.LocationField;
import edu.bu.signstream.grepresentation.fields.locationField.LocationFieldWrapper;
import edu.bu.signstream.grepresentation.fields.movementField.MovementField;
import edu.bu.signstream.grepresentation.fields.movementField.MovementFieldWrapper;
import edu.bu.signstream.grepresentation.fields.nonManualField.*;
import edu.bu.signstream.grepresentation.fields.orientationField.OrientationFieldWrapper;
import edu.bu.signstream.grepresentation.fields.partOfSpeechField.PartOfSpeechField;
import edu.bu.signstream.grepresentation.fields.partOfSpeechField.PartOfSpeechFieldWrapper;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.fields.signTypeField.SignTypeField;
import edu.bu.signstream.grepresentation.fields.signTypeField.SignTypeFieldWrapper;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class FieldsFactory {

	public FieldWrapper createFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField, TimeRange timeRange) {

	    SS3CodingScheme dbScheme = segmentPanel.getSS3Database().getSS3CodingScheme();

		String id = track.getId();
		SS3Field field = dbScheme.getField(id);

		SS3FieldType ft = dbScheme.getFieldType(field.getType());

		String type = ft != null ? ft.getType() : field.getType();

		FieldWrapper wrapper = null;
		
		if (type.equalsIgnoreCase(Constants.TYPE_GRAPH)) {

			wrapper = createGraphWrapper(track, segmentPanel, parentField, timeRange);

		}else if (type.equalsIgnoreCase(Constants.TYPE_NON_MANUAL)) {

			wrapper = createNonManualFieldWrapper(track, segmentPanel, parentField, timeRange);

		} else if (type.equalsIgnoreCase(Constants.TYPE_NON_MANUAL_FREE_TEXT)) {
			wrapper = createNonManualFreeTextFieldWrapper(track, segmentPanel, parentField, timeRange);
		} else if (type.equalsIgnoreCase(Constants.TYPE_GLOSS)
				|| type.equalsIgnoreCase(Constants.TYPE_DOM_GLOSS)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_GLOSS)) {

			wrapper = createGlossFieldWrapper(track, segmentPanel, parentField, timeRange);

			if (type.equalsIgnoreCase(Constants.TYPE_NDOM_GLOSS)) {

				ArrayList entities = wrapper.getField().getEntities();
                for (Object entity1 : entities) {
                    ChainedEventsEntity entity = (ChainedEventsEntity) entity1;
                    ChainedEvent event = entity.getTextValue();
                    SS3Entity ent = track.getSS3Entity(entity.getID());

                    if (ent.getColor() != null)
                        event.setColor(ent.getColor());
                }
			}

		} else if (type.equalsIgnoreCase(Constants.TYPE_GLOSS_LEGACY)) {

			wrapper = createLegacyGlossFieldWrapper(track, segmentPanel, parentField, timeRange);

		} else if (type.equalsIgnoreCase(Constants.TYPE_FREE_TEXT)) {

			wrapper = createFreeTextFieldWrapper(track, segmentPanel, parentField, timeRange);

		} else if (type.equalsIgnoreCase(Constants.TYPE_DOM_HAND_SHAPE)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_HAND_SHAPE)) {

			wrapper = createHandShapeFieldWrapper(track, segmentPanel, parentField, timeRange);

		} else if (type.equalsIgnoreCase(Constants.TYPE_DOM_POS_TAG)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_POS_TAG)) {

			wrapper = createPartOfSpeechFieldWrapper(track, segmentPanel, parentField, timeRange);

		} else if (type.equalsIgnoreCase(Constants.TYPE_DOM_TEXT)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_TEXT)) {

			wrapper = createLiteralTranslationFieldWrapper(track, segmentPanel, parentField, timeRange);

		} else if (type.equalsIgnoreCase(Constants.TYPE_DOM_HAND_LOCATION)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_HAND_LOCATION)) {

			wrapper = createHandShapeLocationFieldWrapper(track, segmentPanel, parentField, timeRange);

		} else if (type.equalsIgnoreCase(Constants.TYPE_DOM_HAND_MOVEMENT)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_HAND_MOVEMENT)) {

			wrapper = createHandShapeMovementFieldWrapper(track, segmentPanel, parentField, timeRange);

		} /*else if (type.equalsIgnoreCase(Constants.TYPE_DOM_HAND_ORIENTATION)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_HAND_ORIENTATION)) {

			wrapper = createHandShapeOrientationFieldWrapper(track,
					segmentPanel, parentField, timeRange);

		}*/ else if (type.equalsIgnoreCase(Constants.TYPE_DOM_HAND)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_HAND)) {

			wrapper = createDomNdomFieldWrapper(track, segmentPanel, parentField, timeRange);

		} else if (type.equalsIgnoreCase(Constants.TYPE_DOM_HAND_SEL_FINGERS)
				|| type.equalsIgnoreCase(Constants.TYPE_DOM_HAND_NON_SEL_FINGERS)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_HAND_SEL_FINGERS)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_HAND_NON_SEL_FINGERS)
				|| type.equalsIgnoreCase(Constants.TYPE_DOM_HAND_BASE_JOINTS)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_HAND_BASE_JOINTS)
				|| type.equalsIgnoreCase(Constants.TYPE_DOM_HAND_NON_BASE_JOINTS)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_HAND_NON_BASE_JOINTS)
				|| type.equalsIgnoreCase(Constants.TYPE_DOM_HAND_SPREAD)
				|| type.equalsIgnoreCase(Constants.TYPE_DOM_HAND_THUMB_CONTACT)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_HAND_SPREAD)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_HAND_THUMB_CONTACT)) {

			wrapper = createJointsFieldWrapper(track, segmentPanel, parentField, timeRange);

		} else if (type.equalsIgnoreCase(Constants.TYPE_DOM_LOCATION) || type.equalsIgnoreCase(Constants.TYPE_NDOM_LOCATION)) {
			wrapper = createLocationFieldWrapper(track, segmentPanel, parentField, timeRange);
		} else if (type.equalsIgnoreCase(Constants.TYPE_DOM_SIGN_TYPE)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_SIGN_TYPE)) {
			wrapper = createSignTypeFieldWrapper(track, segmentPanel, parentField, timeRange);
		} else if (type.equalsIgnoreCase(Constants.TYPE_DOM_GESTURE)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_GESTURE)) {
			wrapper = createGestureFieldWrapper(track, segmentPanel, parentField, timeRange);
		} else if (type.equalsIgnoreCase(Constants.TYPE_DOM_GESTURE_APEX)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_GESTURE_APEX)) {
			wrapper = createApexFieldWrapper(track, segmentPanel, parentField, timeRange);
		} else if (type.equalsIgnoreCase(Constants.TYPE_DOM_GESTURE_HANDSHAPE)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_GESTURE_HANDSHAPE)) {
			wrapper = createGestureHandShapeFieldWrapper(track, segmentPanel, parentField, timeRange);
		} else if (type.equalsIgnoreCase(Constants.TYPE_DOM_GESTURE_HAND)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_GESTURE_HAND)) {
			wrapper = createGestureDomNdomFieldWrapper(track, segmentPanel, parentField, timeRange);
		} else if (type.equalsIgnoreCase(Constants.TYPE_DOM_GESTURE_ORIENTATION)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_GESTURE_ORIENTATION)) {
			wrapper = createGestureOrientationFieldWrapper(track, segmentPanel, parentField, timeRange);
		} else if (type.equalsIgnoreCase(Constants.TYPE_DOM_GESTURE_LOCATION)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_GESTURE_LOCATION)) {
			wrapper = createGestureLocationFieldWrapper(track, segmentPanel, parentField, timeRange);
		} else if (type.equalsIgnoreCase(Constants.TYPE_DOM_GESTURE_TYPE)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_GESTURE_TYPE)
				|| type.equalsIgnoreCase(Constants.TYPE_DOM_GESTURE_CATEGORY)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_GESTURE_CATEGORY)) {
			wrapper = createGestureTypeFieldWrapper(track, segmentPanel, parentField, timeRange);
		} else if (type.equalsIgnoreCase(Constants.TYPE_GESTURE_FREE_TEXT)) {
			wrapper = createFreeTextFieldWrapper(track, segmentPanel, parentField, timeRange);
		} else if (type.equalsIgnoreCase(Constants.TYPE_CUSTOM_INTERVAL)) {
			wrapper = createCustomIntervalFieldWrapper(track, segmentPanel, parentField, timeRange);
		} else if (type.equalsIgnoreCase(Constants.TYPE_CUSTOM_SECTION)) {
			wrapper = createCustomSectionFieldWrapper(track, segmentPanel, parentField, timeRange);
		} else if (type.equalsIgnoreCase(Constants.TYPE_CUSTOM_POINT)) {
			wrapper = createCustomPointFieldWrapper(track, segmentPanel, parentField, timeRange);
		}

		if (wrapper != null)
			wrapper.setType(type);

		return wrapper;
	}
	
	public FieldWrapper createGraphWrapper(SS3Track track, SignStreamSegmentPanel segmentPanel,
			PresentationField parentField, TimeRange timeRange) {
		
		GraphField field = new GraphField(track, segmentPanel, parentField);
		GraphFieldWrapper wrapper = new GraphFieldWrapper(field, segmentPanel, timeRange);
		if (wrapper != null)
			wrapper.setType(Constants.TYPE_GRAPH);
		wrapper.setVisible(true);
		return wrapper;
	}

	private FieldWrapper createJointsFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField, TimeRange timeRange) {

		JointsField field = new JointsField(track, segmentPanel, parentField);
		JointsFieldWrapper wrapper = new JointsFieldWrapper(field, segmentPanel, timeRange);
		wrapper.setVisible(false);
		return wrapper;
	}

	private FieldWrapper createNonManualFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField, TimeRange timeRange) {
		NonManualField field = new NonManualField(track, segmentPanel, parentField);
		return new NonManualFieldWrapper(field, segmentPanel, timeRange);
	}

	private FieldWrapper createNonManualFreeTextFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField, TimeRange timeRange) {
		NonManualField field = new NonManualField(track, segmentPanel, parentField);
		return new NonManualFreeTextFieldWrapper(field, segmentPanel, timeRange);
	}

	private GlossFieldWrapper createGlossFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField, TimeRange timeRange) {
		GlossField field = new GlossField(track, segmentPanel, parentField);
		return new GlossFieldWrapper(field, segmentPanel, timeRange);
	}

	private GestureFieldWrapper createGestureFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField, TimeRange timeRange) {
		GestureField field = new GestureField(track, segmentPanel, parentField);
		return new GestureFieldWrapper(field, segmentPanel, timeRange);
	}
	
	private FieldWrapper createApexFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField, TimeRange timeRange) {
		ApexField field = new ApexField(track, segmentPanel, parentField);
		return new ApexFieldWrapper(field, segmentPanel, timeRange);
	}

	private FieldWrapper createLegacyGlossFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField, TimeRange timeRange) {
		GlossField field = new GlossField(track, segmentPanel, parentField);
		return new LegacyGlossFieldWrapper(field, segmentPanel, timeRange);
	}

	private FieldWrapper createDomNdomFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField, TimeRange timeRange) {
		 NonManualField field = new NonManualField(track, segmentPanel, parentField);
		return new DomNdomHandFieldWrapper(field, segmentPanel, timeRange);
	}
	
	private FieldWrapper createGestureDomNdomFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField, TimeRange timeRange) {
		return new GestureDomNdomHandFieldWrapper(track, segmentPanel, parentField, timeRange);
	}

	private FieldWrapper createFreeTextFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField, TimeRange timeRange) {
		FreeTextField field = new FreeTextField(track, segmentPanel, parentField);
		return new FreeTextFieldWrapper(field, segmentPanel, timeRange);
	}

	private FieldWrapper createHandShapeFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField, TimeRange timeRange) {

		HandShapeField field = new HandShapeField(track, segmentPanel, parentField);
		HandShapeFieldWrapper wrapper = new HandShapeFieldWrapper(field,
				segmentPanel, timeRange);
		wrapper.setVisible(false);
		return wrapper;
	}
	
	private FieldWrapper createGestureHandShapeFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField, TimeRange timeRange) {

		GestureHandShapeField field = new GestureHandShapeField(track, segmentPanel, parentField);
		GestureHandShapeFieldWrapper wrapper = new GestureHandShapeFieldWrapper(field,
				segmentPanel, timeRange);
		wrapper.setVisible(false);
		return wrapper;
	}

	private FieldWrapper createHandShapeLocationFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField, TimeRange timeRange) {
		LocationField field = new LocationField(track, segmentPanel, parentField);
		LocationFieldWrapper wrapper = new LocationFieldWrapper(field,
				segmentPanel, timeRange);
		wrapper.setVisible(false);
		return wrapper;
	}

	private FieldWrapper createHandShapeOrientationFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField, TimeRange timeRange) {
		HandShapeField field = new HandShapeField(track, segmentPanel, parentField);
		OrientationFieldWrapper wrapper = new OrientationFieldWrapper(field,
				segmentPanel ,timeRange);
		wrapper.setVisible(false);
		return wrapper;
	}

	private FieldWrapper createHandShapeMovementFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField, TimeRange timeRange) {
		MovementField field = new MovementField(track, segmentPanel, parentField);
		MovementFieldWrapper wrapper = new MovementFieldWrapper(field,
				segmentPanel, timeRange);
		wrapper.setVisible(false);
		return wrapper;
	}

	private FieldWrapper createPartOfSpeechFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField, TimeRange timeRange) {
		PartOfSpeechField field = new PartOfSpeechField(track, segmentPanel, parentField);
		PartOfSpeechFieldWrapper wrapper = new PartOfSpeechFieldWrapper(field,
				segmentPanel, timeRange);
		wrapper.setVisible(false);
		return wrapper;
	}

	private FieldWrapper createLiteralTranslationFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField, TimeRange timeRange) {
		LiteraltranslationField field = new LiteraltranslationField(track,
				segmentPanel, parentField);
		LiteralTranslationWrapper wrapper = new LiteralTranslationWrapper(
				field, segmentPanel, timeRange);
		wrapper.setVisible(false);
		return wrapper;
	}

	private FieldWrapper createLocationFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField, TimeRange timeRange) {
		LocationField field = new LocationField(track, segmentPanel, parentField);
		LocationFieldWrapper wrapper = new LocationFieldWrapper(field, segmentPanel, timeRange);
		wrapper.setVisible(false);
		return wrapper;
	}
	
	private FieldWrapper createSignTypeFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField, TimeRange timeRange) {
		SignTypeField field = new SignTypeField(track, segmentPanel, parentField);
		SignTypeFieldWrapper wrapper = new SignTypeFieldWrapper(field, segmentPanel, timeRange);
		wrapper.setVisible(false);
		return wrapper;
	}
	
	private FieldWrapper createGestureLocationFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField, TimeRange timeRange) {
		GestureLocationField field = new GestureLocationField(track, segmentPanel, parentField);
		GestureLocationFieldWrapper wrapper = new GestureLocationFieldWrapper(field, segmentPanel, timeRange);
		wrapper.setVisible(false);
		return wrapper;
	}
	
	private FieldWrapper createGestureTypeFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField, TimeRange timeRange) {
		GestureTypeField field = new GestureTypeField(track, segmentPanel, parentField);
		GestureTypeFieldWrapper wrapper = new GestureTypeFieldWrapper(field, segmentPanel, timeRange);
		wrapper.setVisible(false);
		return wrapper;
	}
	
	private FieldWrapper createGestureOrientationFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField, TimeRange timeRange) {
		GestureOrientationField field = new GestureOrientationField(track, segmentPanel, parentField);
		GestureOrientationFieldWrapper wrapper = new GestureOrientationFieldWrapper(field, segmentPanel, timeRange);
		wrapper.setVisible(false);
		return wrapper;
	}

	private FieldWrapper createCustomIntervalFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField, TimeRange timeRange) {
		CustomIntervalField field = new CustomIntervalField(track, segmentPanel, parentField);
		return new CustomIntervalFieldWrapper(field, segmentPanel, timeRange);
	}

	private FieldWrapper createCustomSectionFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField, TimeRange timeRange) {
		CustomSectionField field = new CustomSectionField(track, segmentPanel, parentField);
		return new CustomSectionFieldWrapper(field, segmentPanel, timeRange);
	}

	private FieldWrapper createCustomPointFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField, TimeRange timeRange) {
		CustomPointField field = new CustomPointField(track, segmentPanel, parentField);
		return new CustomPointFieldWrapper(field, segmentPanel, timeRange);
	}
}
