package edu.bu.signstream.grepresentation.reguestsHandler;

import java.awt.Point;

import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;

import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.glossField.Compoundable;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.glossField.GlossFieldWrapper;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.IPAKeyboard;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.MorphologicalPhonologicalInfoDialog;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.nonManualField.mouseDelegateListeners.AddFinalHoldActionListener;
import edu.bu.signstream.grepresentation.fields.nonManualField.mouseDelegateListeners.AddInitialHoldActionListener;
import edu.bu.signstream.grepresentation.fields.nonManualField.mouseDelegateListeners.AddOffsetActionListener;
import edu.bu.signstream.grepresentation.fields.nonManualField.mouseDelegateListeners.AddOnsetActionListener;
import edu.bu.signstream.grepresentation.fields.nonManualField.mouseDelegateListeners.DefineAssociatedFieldsActionListener;
import edu.bu.signstream.grepresentation.fields.nonManualField.mouseDelegateListeners.DeleteActionListener;
import edu.bu.signstream.grepresentation.fields.nonManualField.mouseDelegateListeners.LinkUnlinkCompound;
import edu.bu.signstream.grepresentation.fields.nonManualField.mouseDelegateListeners.LinkUnlinkMenuItem;
import edu.bu.signstream.grepresentation.fields.nonManualField.mouseDelegateListeners.LockUnlockGlossPhonAlignmentsListener;
import edu.bu.signstream.grepresentation.fields.nonManualField.mouseDelegateListeners.LockUnlockDomNdomHandAlignmentsListener;
import edu.bu.signstream.grepresentation.view.GlossNRelatedView;
import edu.bu.signstream.grepresentation.view.SegmentTemplate;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class NonManuelFieldDialogs {

	final JPopupMenu popup = new JPopupMenu();

	private JMenu assocEventsMenu = null;
	private JMenuItem addOnset = null;
	private JMenuItem addInitialHold = null;
	private JMenuItem addFinalHold = null;
	private JMenuItem addOffset = null;
	private JMenuItem delete = null;
	private JMenuItem cancel = null;
	private JMenuItem defAsFlds = null;
	private JMenu compoundsMenu = null;

	private JMenuItem lockHandAlignments = null;
	private JMenuItem lockGlossPhonAlignments = null; 

	private AddOnsetActionListener addOnsetListener = null;
	private AddInitialHoldActionListener addInitialHoldListener = null;
	private AddFinalHoldActionListener addFinalHoldListener = null;
	private AddOffsetActionListener addOffsetListener = null;
	private DeleteActionListener deleteActionListener = null;
	private DefineAssociatedFieldsActionListener defAssocFldsListener = null;
	
	private LinkUnlinkMenuItem linkStartCmp = new LinkUnlinkMenuItem();
	private LinkUnlinkMenuItem unlinkStartCmp = new LinkUnlinkMenuItem();
	private LinkUnlinkMenuItem linkEndCmp = new LinkUnlinkMenuItem();
	private LinkUnlinkMenuItem unlinkEndCmp = new LinkUnlinkMenuItem();
	
	private LinkUnlinkCompound linkUnlinkCmpListener = new LinkUnlinkCompound();
	
    // MorphologicalPhonologicalInfoDialog dialog is a singleton; keep it here.
    private MorphologicalPhonologicalInfoDialog morphologicalPhonologicalInfoDialog = null;
    
    private IPAKeyboard ipaKeyboard;

	public MorphologicalPhonologicalInfoDialog getMorphologicalPhonologicalInfoDialog() {
        return morphologicalPhonologicalInfoDialog;
    }

    public void setMorphologicalPhonologicalInfoDialog(MorphologicalPhonologicalInfoDialog morphologicalPhonologicalInfoDialog) {
        this.morphologicalPhonologicalInfoDialog = morphologicalPhonologicalInfoDialog;
    }
    
    public IPAKeyboard getIPAKeyboard() {
    	if (ipaKeyboard == null) {
    		ipaKeyboard = new IPAKeyboard();
    	}
		return ipaKeyboard;
	}

	public void getAssociatedElelmentsMenu(ChainedEventsEntity entity,
			Point point, JPanel panel) {
		
		SignStreamSegmentPanel segmentPanel = SS3Singleton
				.getSignStreamApplication().getSignStreamSegmentPanel();
		SegmentTemplate template = segmentPanel.getUtteranceView()
				.getSegmentTemplate(entity.getField().getSegmentID());
		GlossNRelatedView view = template.getSignView();
		GlossFieldWrapper wrapper = (GlossFieldWrapper) view
				.getFieldWrappersCollection().getFieldWrapperWithFieldID(
						entity.getField().getFieldID());

		if (assocEventsMenu == null) {
			assocEventsMenu = new JMenu("Associated Elements");

			UserInterfaceUtil.setMenuItemLNF(assocEventsMenu);
			popup.add(assocEventsMenu);

			addOnset = new JMenuItem("Onset");
			addInitialHold = new JMenuItem("Initial Hold");
			addFinalHold = new JMenuItem("Final Hold");
			addOffset = new JMenuItem("Offset");
			delete = new JMenuItem("Delete");
			cancel = new JMenuItem("Cancel");

			defAsFlds = new JMenuItem("Morph-Phon Info");
			
			lockHandAlignments = new JMenuItem("Lock/Unlock Dom-Ndom Gloss Alignments");
			lockGlossPhonAlignments = new JMenuItem("Lock/Unlock Dom Hand Alignments"); 
			
			compoundsMenu = new JMenu("Compounds");

			UserInterfaceUtil.setMenuItemLNF(addOnset);
			UserInterfaceUtil.setMenuItemLNF(addInitialHold);
			UserInterfaceUtil.setMenuItemLNF(addFinalHold);
			UserInterfaceUtil.setMenuItemLNF(addOffset);
			UserInterfaceUtil.setMenuItemLNF(defAsFlds);
			UserInterfaceUtil.setMenuItemLNF(lockHandAlignments);
			UserInterfaceUtil.setMenuItemLNF(lockGlossPhonAlignments);
			UserInterfaceUtil.setMenuItemLNF(compoundsMenu);
			UserInterfaceUtil.setMenuItemLNF(delete);
			UserInterfaceUtil.setMenuItemLNF(delete);
			UserInterfaceUtil.setMenuItemLNF(cancel);

			addOnsetListener = new AddOnsetActionListener();
			addInitialHoldListener = new AddInitialHoldActionListener();
			addFinalHoldListener = new AddFinalHoldActionListener();
			addOffsetListener = new AddOffsetActionListener();
			deleteActionListener = new DeleteActionListener();

			lockHandAlignments.addActionListener(new LockUnlockDomNdomHandAlignmentsListener());
			lockGlossPhonAlignments.addActionListener(new LockUnlockGlossPhonAlignmentsListener());

			addOnset.addActionListener(addOnsetListener);
			addInitialHold.addActionListener(addInitialHoldListener);
			addFinalHold.addActionListener(addFinalHoldListener);
			addOffset.addActionListener(addOffsetListener);
			delete.addActionListener(deleteActionListener);

            if(entity instanceof GlossChainedEventsEntity) {
                defAssocFldsListener = new DefineAssociatedFieldsActionListener(wrapper, view);
			    defAsFlds.addActionListener(defAssocFldsListener);
            }

			linkStartCmp.addActionListener(linkUnlinkCmpListener);
			unlinkStartCmp.addActionListener(linkUnlinkCmpListener);
			linkEndCmp.addActionListener(linkUnlinkCmpListener);
			unlinkEndCmp.addActionListener(linkUnlinkCmpListener);

			popup.add(delete);
			popup.add(cancel);
		} else {
            if(entity instanceof GlossChainedEventsEntity) {
                if(defAssocFldsListener == null) {
                    defAssocFldsListener = new DefineAssociatedFieldsActionListener(wrapper, view);
                    defAsFlds.addActionListener(defAssocFldsListener);
                } else {
			        defAssocFldsListener.setUpdatedValues(wrapper, view);
                }
            }
		}

		assocEventsMenu.remove(addOnset);
		assocEventsMenu.remove(addInitialHold);
		assocEventsMenu.remove(addFinalHold);
		assocEventsMenu.remove(addOffset);

		popup.remove(defAsFlds);
		popup.remove(lockHandAlignments);
		popup.remove(lockGlossPhonAlignments);
		popup.remove(compoundsMenu);

		if (entity.getOnset() == null) {
			addOnsetListener.setSelectedEntity(entity);
			assocEventsMenu.add(addOnset);
		}

		if (entity.getInitialHold() == null) {
			addInitialHoldListener.setSelectedEntity(entity);
			assocEventsMenu.add(addInitialHold);
		}

		if (entity.getFinalHold() == null) {
			addFinalHoldListener.setSelectedEntity(entity);
			assocEventsMenu.add(addFinalHold);
		}

		if (entity.getOffset() == null) {
			addOffsetListener.setSelectedEntity(entity);
			assocEventsMenu.add(addOffset);
		}

		deleteActionListener.setChainedEventsEntity(entity);

		if (entity instanceof GlossChainedEventsEntity) {

			GlossChainedEventsEntity glsEntity = (GlossChainedEventsEntity) entity;

			int popupCounter = 0;
			
			popup.add(defAsFlds, popupCounter);
			popupCounter ++;
			
			popupCounter ++;
			
			GlossChainedEventsEntity dGlsEntity =  glsEntity.getDependentGlossEntity();
			ChainedEventsEntity dHandEntity = glsEntity.getDependentHandEntity();
			
			if(dGlsEntity != null) {
				boolean lockedDomNdomAlignment = glsEntity.isLockedDomNdomAlignments();
				
				if(lockedDomNdomAlignment) {
					lockHandAlignments.setText("Unlock Dom-Ndom Gloss Alignments");
				} else {
					lockHandAlignments.setText("Lock Dom-Ndom Gloss Alignments");
				}
				popup.add(lockHandAlignments, popupCounter);
				popupCounter ++;
			}
			
			if(dHandEntity != null) {
				boolean lockedGlsPhonAlignment = glsEntity.isLockedGlossPhonAlignments();
				String fieldID = dHandEntity.getField().getFieldID();
				// right hand id = 10501, left hand id = 10502
				
				boolean isDomHand = fieldID.equalsIgnoreCase(Util.DOM_HAND_FIELD_ID);
				
				if(lockedGlsPhonAlignment) {
					if(isDomHand) {
						lockGlossPhonAlignments.setText("Unlock Dom Hand Alignments");
					} else {
						lockGlossPhonAlignments.setText("Unlock Ndom Hand Alignments");
					}
				} else {
					if(isDomHand) {
						lockGlossPhonAlignments.setText("Lock Dom Hand Alignments");
					} else {
						lockGlossPhonAlignments.setText("Lock Ndom Hand Alignments");
					}
				}
				popup.add(lockGlossPhonAlignments, popupCounter);
				popupCounter ++;
			}

			GlossChainedEventsEntity prevEntity = glsEntity.getPreviousEntity();
			GlossChainedEventsEntity nextEntity = glsEntity.getNextEntity();
			
//			Event utterance = entity.getParentEntity().getEvent();
//			Event prevEntityUtterance = null; 
//			Event nextEntityUtterance = null; 
//			if (prevEntity != null) {
//				prevEntityUtterance = prevEntity.getParentEntity().getEvent();
//			}
//			if (nextEntity != null) {
//				nextEntityUtterance = nextEntity.getParentEntity().getEvent();
//			}
			
			Compoundable startCompound = glsEntity.getStartCompound();
			Compoundable endCompound = glsEntity.getEndCompound();
			
//			//add compounds to the menu, only if there are other entities in the same utterance
//			//or if there are existing compounds to remove
//			if (utterance.equals(prevEntityUtterance) || utterance.equals(nextEntityUtterance) ||
//					startCompound != null || endCompound != null) {
			if (prevEntity != null || nextEntity != null
                || startCompound != null || endCompound != null) {
				
				compoundsMenu.remove(linkStartCmp);
				compoundsMenu.remove(unlinkStartCmp);
				compoundsMenu.remove(linkEndCmp);
				compoundsMenu.remove(unlinkEndCmp);

				popup.add(compoundsMenu, popupCounter);
				popupCounter ++;

//				//insert option to link to previous entity, if it's in the same utterance
//				//insert option to delink if already linked to entity in different utterance
//				if (prevEntity != null && prevEntityUtterance.equals(utterance) ||
//						startCompound != null) {
				if (prevEntity != null || startCompound != null) {
					if (startCompound == null) {
						linkStartCmp.setText("Link " + prevEntity.getText());
						linkStartCmp.setCompound(glsEntity);
						linkStartCmp.setPrevCompound(prevEntity);
						linkStartCmp.setLink(true);
						compoundsMenu.add(linkStartCmp);
					} else {
						// unlink previous entity, destroy compound
						unlinkStartCmp
								.setText("Unlink " + prevEntity.getText());
						unlinkStartCmp.setCompound(glsEntity);
						unlinkStartCmp.setPrevCompound(prevEntity);
						unlinkStartCmp.setLink(false);
						compoundsMenu.add(unlinkStartCmp);
					}
				}
//				//insert option to link to next entity, if it's in the same utterance
//				//insert option to delink if already linked to entity in different utterance
//				if (nextEntity != null && nextEntityUtterance.equals(utterance) ||
//						endCompound != null) {
				if (nextEntity != null || endCompound != null) {
					if (endCompound == null) {
						// link following entity, create a compound
						linkEndCmp.setText("Link " + nextEntity.getText());
						linkEndCmp.setCompound(nextEntity);
						linkEndCmp.setPrevCompound(glsEntity);
						linkEndCmp.setLink(true);
						compoundsMenu.add(linkEndCmp);
					} else {
						// unlink previous entity, destroy compound
						unlinkEndCmp.setText("Unlink " + nextEntity.getText());
						unlinkEndCmp.setCompound(nextEntity);
						unlinkEndCmp.setPrevCompound(glsEntity);
						unlinkEndCmp.setLink(false);
						compoundsMenu.add(unlinkEndCmp);
					}
				}
			}
		}

		popup.show(panel, point.x, point.y);
		popup.setVisible(true);
		
		SS3Singleton.getSignStreamApplication().forcePopupInvisibleOnWindowLostFocus(popup);
	}
	
}
