package edu.bu.signstream.grepresentation.reguestsHandler;

import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.ArrayList;

import edu.bu.signstream.common.codingSchema.CodingSchema;
import edu.bu.signstream.common.files.FilesLocation;
import edu.bu.signstream.common.files.MediaFilesLocator;
import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.signbank.SignBankParser;
import edu.bu.signstream.common.util.signbank.SignBankWriter;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingSchemeCollection;
import edu.bu.signstream.common.util.vo.ss3.db.SS3DatabaseCollection;
import edu.bu.signstream.grepresentation.fields.Segment;
import edu.bu.signstream.time.Duration;
import edu.bu.signstream.time.TimeRange;
import edu.bu.signstream.grepresentation.fields.gestureField.handshapes.GestureHandShapesImageManager;
import edu.bu.signstream.grepresentation.fields.gestureField.orientationField.GestureOrientationsImageManager;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations.LocationConfig;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapesImageManager;
import edu.bu.signstream.grepresentation.listener.SSKeyListener;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolBar;
import edu.bu.signstream.media.MediaPlayer;
import edu.bu.signstream.media.gstreamer.GstMediaPlayer;
import edu.bu.signstream.ui.SS3SignStreamApplication;
import edu.bu.signstream.ui.video.MultipleMovieController;
import edu.bu.signstream.ui.video.VideoControlManager;

public class SS3Singleton {

	private static Manager manager;

	private static HandShapesImageManager hsImageManager;
	
	private static GestureHandShapesImageManager gestureHSImageManager;
	
	private static GestureOrientationsImageManager gestureOrientationImageManager;

	private static FilesLocation filesLocator;

	private static MediaFilesLocator mediaFileLocator;

	private static SS3DatabaseCollection dbCollection;

	private static SS3CodingSchemeCollection csCollection;

    private static CodingSchema loadedCodingSchema = null;

	private static VideoControlManager videoControlManager;

	private static MediaPlayer mediaPlayer;

	private static MediaToolBar mediaToolBar;

	private static ApplicationStateController applicationStateController;

	private static SSKeyListener listener = new SSKeyListener();

	private static KeyListener videoControlKeyListener = null;

	private static NonManuelFieldDialogs nonManuelFieldDialogs = null;

	private static CustomFieldDialogs customFieldDialogs = null;

	private static GestureFieldDialogs gestureFieldDialogs;
	
	private static SignBankParser signBankParser = null;
	
	private static SignBankWriter signBankWriter = null;
	
	private static ErrorMessages errorMessages = null;

	private static LocationConfig regions = null;

    private static SS3SignStreamApplication application;
    
    private static TimeRange videoTimeRange;
    private static TimeRange audioTimeRange;

    private static FieldsFactory fieldsFactory = new FieldsFactory();

    private static RequestHandler requestHandler = new RequestHandler();
    
    private static String currentAnnotator = null;
    
    private static boolean dbOriginallyOldScheme = false;

	public RequestHandler getRequestHandler() {
        return requestHandler;
    }

	public static FieldsFactory getFieldsFactory() {
        return fieldsFactory;
    }

    public static void setCodingSchema(CodingSchema codingSchema) {
        loadedCodingSchema = codingSchema;
    }

    public static CodingSchema getLoadedCodingSchema() {
        return loadedCodingSchema;
    }

    public static void loadSignStreamApplication(SS3SignStreamApplication application) {
        SS3Singleton.application = application;
    }

    public static SS3SignStreamApplication getSignStreamApplication() {
        return application;
    }

    public static void setVideoTimeRange(TimeRange range) {
        SS3Singleton.videoTimeRange = range;

        var panel = SS3Singleton.getMediaToolBar().segmentPanel();

        var segments = new ArrayList<Segment>();
        segments.add(panel.selectedSegment());
        segments.addAll(panel.getSegments());

        segments.forEach(s -> {
            var root = s.getRootLabelObject();
            var pres = root.getPresentationLabelObject();
            pres.getAllChildren().forEach(o -> {
                o.getWrapper().setDefaultTimeRange(range);
                o.getWrapper().setTimeRange(range);
            });
        });
    }
    @Deprecated public static TimeRange getRealVideoTimeRange() {
        return videoTimeRange;
    }
    public static TimeRange getVideoTimeRange() {
        if (videoTimeRange == null)
            return TimeRange.ABSOLUTE;
//            throw new IllegalStateException("Accessed VideoTimeRange before init!");
        return videoTimeRange;
    }

    public static void setAudioTimeRange(TimeRange range) { SS3Singleton.audioTimeRange = range; }
    public static TimeRange getAudioTimeRange() { return audioTimeRange; }

    public static KeyListener getVideoControlKeyListener() {
		if (videoControlKeyListener == null) {
			videoControlKeyListener = new KeyListener() {
				@Override
				public void keyPressed(KeyEvent arg0) {
					int code = arg0.getKeyCode();
					switch (code) {
						case KeyEvent.VK_RIGHT:
                            SS3Singleton.getMediaToolBar(null);
                            var player1 = getMediaPlayer();
                            var step1 = Util.getStep();
                            var range1 = Util.getStepRange();
                            player1.pause();
                            player1.seek(player1.time().add(step1).in(range1));
                            break;
						case KeyEvent.VK_LEFT:
                            SS3Singleton.getMediaToolBar(null);
                            var player = getMediaPlayer();
                            var step = Util.getStep();
                            var range = Util.getStepRange();
                            player.pause();
                            player.seek(player.time().subtract(step).in(range));
                            break;
						default:
							break;
					}
				}

				@Override
				public void keyReleased(KeyEvent arg0) {
				}

				@Override
				public void keyTyped(KeyEvent arg0) {
				}
			};
		}
		return videoControlKeyListener;
	}
	
	public static ErrorMessages getErrorMessages() {
		if(errorMessages == null) {
			errorMessages = new ErrorMessages();
		}
		return errorMessages;
	}

	public static LocationConfig getLocationConfig() {
    	if (regions == null) {
    		regions = new LocationConfig();
		}
    	return regions;
	}

	public static SSKeyListener getSSKeyListener() {
		return listener;
	}

	public static ApplicationStateController getApplicationStateController() {
		if (applicationStateController == null) {
			applicationStateController = new ApplicationStateController();
		}
		return applicationStateController;
	}

	public static MediaToolBar getMediaToolBar(SignStreamSegmentPanel ssPanel) {
		if (mediaToolBar == null && ssPanel != null)
			mediaToolBar = new MediaToolBar(ssPanel);
		return mediaToolBar;
	}

	public static void setMediaToolBar(MediaToolBar bar) { mediaToolBar = bar; }
	public static MediaToolBar getMediaToolBar() {
        return mediaToolBar;
    }

	public static VideoControlManager getVideoControlManager() {
		if (videoControlManager == null) {
			videoControlManager = new VideoControlManager(
					new MultipleMovieController());
		}
		return videoControlManager;
	}

	public static MediaPlayer getMediaPlayer() {
        // TODO: Ensure Gst is initialized
        if (mediaPlayer == null)
            mediaPlayer = new GstMediaPlayer();
        return mediaPlayer;
    }
	
	public static void bringAllVideosToFront() {
		getMediaPlayer().videos().forEach(v -> v.view().toFront());
	}
	
	public static void bringAllAudiosToFront() {
		getMediaPlayer().audios().forEach(a -> a.view().toFront());
	}
	
	public static void bringAllMediasToFront() {
		getMediaPlayer().videos().forEach(v -> v.view().toFront());
		
		getMediaPlayer().audios().forEach(a -> a.view().toFront());
	}

	public static MediaFilesLocator getMediaFilesLocator() {
		if (mediaFileLocator == null)
			mediaFileLocator = new MediaFilesLocator();
		return mediaFileLocator;
	}

	public static FilesLocation getFilesLocation() {
		if (filesLocator == null)
			filesLocator = new FilesLocation();
		return filesLocator;
	}

	public static SS3CodingSchemeCollection getSS3CodingSchemeCollection() {
		if (csCollection == null)
			csCollection = new SS3CodingSchemeCollection();
		return csCollection;
	}

	public static SS3DatabaseCollection getSS3DatabaseCollection() {
		if (dbCollection == null)
			dbCollection = new SS3DatabaseCollection();
		return dbCollection;
	}

	public static Manager getManager() {
		if (manager == null)
			manager = new Manager();
		return manager;
	}

	public static HandShapesImageManager getHandShapesImageManager() {
		if (hsImageManager == null)
			hsImageManager = new HandShapesImageManager();
		return hsImageManager;
	}

	public static NonManuelFieldDialogs getNonManuelFieldDialogs() {
		if(nonManuelFieldDialogs == null) {
			nonManuelFieldDialogs = new NonManuelFieldDialogs();
		}
		return nonManuelFieldDialogs;
	}

	public static CustomFieldDialogs getCustomFieldDialogs() {
		if(customFieldDialogs == null) {
			customFieldDialogs = new CustomFieldDialogs();
		}
		return customFieldDialogs;
	}

	public static GestureFieldDialogs getGestureFieldDialogs() {
		if(gestureFieldDialogs == null) {
			gestureFieldDialogs = new GestureFieldDialogs();
		}
		return gestureFieldDialogs;
	}
	
	public static SignBankParser getSignBankParser() {
		if (signBankParser == null) {
			signBankParser = new SignBankParser();
		}
		return signBankParser;
	}
	
	public static SignBankWriter getSignBankWriter() {
		if (signBankWriter == null) {
			signBankWriter = new SignBankWriter();
		}
		return signBankWriter;
	}
	
	public static String getCurrentAnnotator() {
		return currentAnnotator;
	}
	
	public static void setCurrentAnnotator(String selectedAnnotator) {
		currentAnnotator = selectedAnnotator;
	}

	public static GestureHandShapesImageManager getGestureHandShapesImageManager() {
		if (gestureHSImageManager == null)
			gestureHSImageManager = new GestureHandShapesImageManager();
		return gestureHSImageManager;
	}
	
	public static GestureOrientationsImageManager getGestureOrientationsImageManager() {
		if (gestureOrientationImageManager == null)
			gestureOrientationImageManager = new GestureOrientationsImageManager();
		return gestureOrientationImageManager;
	}

    public static boolean isDbOriginallyOldScheme() {
		return dbOriginallyOldScheme;
	}

	public static void setDbOriginallyOldScheme(boolean old) {
		dbOriginallyOldScheme = old;
	}
}
