package edu.bu.signstream.grepresentation.view;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JTree;
import javax.swing.border.Border;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.gestureField.handshapes.GestureHandShapeFieldWrapper;
import edu.bu.signstream.grepresentation.fields.gestureField.orientationField.GestureOrientationFieldWrapper;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeFieldWrapper;

public class CustomTreeRenderer extends DefaultTreeCellRenderer {
   private Color color;
   private int w, h;

   public CustomTreeRenderer(Color color, int w, int h) {
      this.color = color;
      this.w = w;
      this.h = h;
   }

   @Override
public Component getTreeCellRendererComponent(
      JTree tree,
      Object value,
      boolean sel,
      boolean expanded,
      boolean leaf,
      int row,
      boolean hasFocus) {

      super.getTreeCellRendererComponent(
         tree,
         value,
         sel,
         expanded,
         leaf,
         row,
         hasFocus);

      Component com =
         super.getTreeCellRendererComponent(
            tree,
            value,
            sel,
            expanded,
            leaf,
            row,
            hasFocus);

      JLabel label = (JLabel) com;

      if (value != null) {
         LabelObject lb = null;

         if (value instanceof JTree.DynamicUtilTreeNode) {
            JTree.DynamicUtilTreeNode node = (JTree.DynamicUtilTreeNode) value;
            Object userObj = node.getUserObject();

            if (userObj instanceof TreeNodeVector) {
               TreeNodeVector vector = (TreeNodeVector) node.getUserObject();
               lb = vector.getObject();
            } else {
               lb = (LabelObject) node.getUserObject();
            }
         } else if (value instanceof DefaultMutableTreeNode) {
        	 	DefaultMutableTreeNode node = (DefaultMutableTreeNode) value;
        	 	Object userObj = node.getUserObject();
        	 	
        	 	if (userObj instanceof LabelObject) {
        	 		lb = (LabelObject) node.getUserObject();
        	 	}
         } else if (value instanceof LabelObject) {
             lb = (LabelObject) value;
         }
    	 	else {
              return this;
         }
         
         if (lb != null) {
        	 	// should be utilized only by LabelViewTemporalPartitions to display default segment tiers
             if (lb.isDefaultSegmentTier()) {
          	 	Border border = BorderFactory.createLineBorder(Color.RED, 1);
          	 	label.setBorder(border);
             } else {
            	 	label.setBorder(null);
             }
             
             int labelHeight = SegmentGraphReprUtil.cellHeight;
             
             FieldWrapper wrp = lb.getWrapper();
             if (wrp instanceof GestureHandShapeFieldWrapper || wrp instanceof GestureOrientationFieldWrapper || wrp instanceof HandShapeFieldWrapper) {
            	 labelHeight = SegmentGraphReprUtil.hsAndOrientationHeight;
             }
             
             Dimension d = getPreferredSize();
        	 d.setSize(d.getWidth(), labelHeight);
        	 label.setMinimumSize(d);
        	 label.setPreferredSize(d);
        	 label.setMaximumSize(d);
             // do not add case for if-selected here
             // selected is taken care of by JTree.setSelectionPath(TreePath)
         }
      }

      return this;
   }
}
