package edu.bu.signstream.grepresentation.view;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.util.ArrayList;

import javax.swing.BoundedRangeModel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.ScrollPaneConstants;
import javax.swing.tree.TreePath;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.UIPalette;
import edu.bu.signstream.grepresentation.fields.FieldWrappersCollection;
import edu.bu.signstream.grepresentation.fields.Segment;

public class GestureFieldsLabelView extends JPanel {

	private static final long serialVersionUID = 1L;

	private Segment segment;
	private LabelObject gestureRelatedObj = null;
	private GestureNRelatedView gestureFieldView = null;

	private LabelViewGestureNRelatedFields gestureRelated = null;

	private UtteranceView view = null;
	
	protected JScrollPane pictureScrollPane;
	
	private FieldWrappersCollection fwc;

	public GestureFieldsLabelView(Segment segment, UtteranceView sView,
			GestureNRelatedView view2, FieldWrappersCollection fwc) {
		this.segment = segment;
		this.view = sView;
		this.gestureFieldView = view2;
		this.fwc = fwc;
		
		gestureRelatedObj = segment.getRootLabelObject().getGestureLabelObject();
		
		ArrayList<LabelObject> children = gestureRelatedObj.getAllChildren();
		for (LabelObject obj : children) {
			obj.setVisible(true);
			fwc.addFieldWrapper(obj.getWrapper());
		}

		initUI();
	}

    public void deselect() {
	    if (!gestureRelated.updating)
            gestureRelated.getTree().clearSelection();
    }

	public void update(Segment segment) {
		this.segment = segment;
		gestureRelatedObj = segment.getRootLabelObject().getGestureLabelObject();
		
		ArrayList<LabelObject> objs = gestureRelatedObj.getVisibleObjects();
		for (LabelObject obj : objs) {
			obj.setVisible(true);
		}

		initUI();
	}

	public void showSelectedGestureFields(ArrayList<String> slctFieldIDs) {
		ArrayList<LabelObject> objs = gestureRelatedObj.getChildrenAsList();
		for (LabelObject obj : objs) {
			
			if (slctFieldIDs.contains(obj.getFieldID())) {
				obj.setVisible(true);
			} else {
				obj.setVisible(false);
			}
		}
	}

	public void initUI() {
		removeAll();

		JTree oldTree = null;

		if (gestureRelated != null)
			oldTree = gestureRelated.getTree();

		gestureRelatedObj = segment.getRootLabelObject()
				.getGestureLabelObject();

		gestureRelated = new LabelViewGestureNRelatedFields(gestureRelatedObj,
				segment, view, gestureFieldView, fwc);
		
		pictureScrollPane = new JScrollPane(gestureRelated,
				ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		pictureScrollPane.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
		pictureScrollPane.getVerticalScrollBar().addAdjustmentListener(e -> {
			gestureFieldView.setVerticalScrollValue(e.getValue());
		});
		pictureScrollPane.getVerticalScrollBar().setPreferredSize(new Dimension(0, 0));

		JPanel p = new JPanel();
		p.setBackground(UIPalette.getBackground());
		p.setPreferredSize(new Dimension(getWidth(),
				SegmentGraphReprUtil.axesBottomHeight));

		setLayout(new BorderLayout(0, 1));
		add(pictureScrollPane, BorderLayout.CENTER);

		JTree newTree = gestureRelated.getTree();

//		if (oldTree != null) {
//			for (int i = 0; i < oldTree.getRowCount(); i++) {
//				TreePath path = oldTree.getPathForRow(i);
//
//				if (oldTree.isExpanded(path)) {
//					Object obj = path.getLastPathComponent();
//					String label = obj.toString().trim();
//					expandTree(newTree, label);
//				}
//			}
//		} else {
			for (int j = 0; j < newTree.getRowCount(); j++) {
				TreePath newPath = newTree.getPathForRow(j);
				Object newObj = newPath.getLastPathComponent();
				String label = newObj.toString().trim();
				
//				if(label.equalsIgnoreCase("right hand") || label.equalsIgnoreCase("left hand") ) {
				expandTree(newTree, label);
//				}
			}
//		}

		ArrayList<LabelObject> visibleObj = gestureRelatedObj.getVisibleObjects();

		for (int j = 0; j < newTree.getRowCount(); j++) {
			TreePath newPath = newTree.getPathForRow(j);
			Object newObj = newPath.getLastPathComponent();
			String label = newObj.toString().trim();
			
			if (isVisibleObject(label, visibleObj)) {

				if (!label.equals("rhshape") && !label.equals("lhshape")) {
					;
				}
					//expandTreeLeaf(newTree, label);
			}
		}
	}

	public GestureNRelatedView getGestureFieldView() {
		return gestureFieldView;
	}

	public void newUpdate(LabelObject obj) {
		gestureRelated.showNode(obj);
	}

	@Override
	public void repaint() {
		if (gestureRelated != null)
			gestureRelated.update();
	}

	public void setPreferredHeight(int ph) {
		int size = getPreferredSize().width;
		setPreferredSize(new Dimension(size, ph));
	}

	public void setPreferredWidth(int pw) {
		int size = getPreferredSize().height;
		setPreferredSize(new Dimension(pw, size));
	}

	public ArrayList<LabelObject> getLabels() {
		return gestureRelatedObj.getChildrenAsList();
	}

	public LabelObject getParentLabelObject() {
		return gestureRelatedObj;
	}

	public void update() {
		removeAll();

		initUI();
	}

	private boolean isVisibleObject(String label,
			ArrayList<LabelObject> visibleObj) {
		for (LabelObject obj : visibleObj) {
			if (label.equals(obj.getLabel())) {
				return true;
			}
		}
		return false;
	}

	private void expandTreeLeaf(JTree tree, String label) {
		for (int i = 0; i < tree.getRowCount(); i++) {
			TreePath path = tree.getPathForRow(i);
			Object obj = path.getLastPathComponent();
			String treeLabel = obj.toString().trim();
			if (treeLabel.equals(label)) {
				tree.expandPath(path);
				return;
			}
		}
	}

	private void expandTree(JTree tree, String label) {
		for (int i = 0; i < tree.getRowCount(); i++) {
			TreePath path = tree.getPathForRow(i);
			Object obj = path.getLastPathComponent();
			String treeLabel = obj.toString().trim();
			if (treeLabel.equals(label)) {
				tree.expandRow(i);
				return;
			}
		}
	}
	public JScrollPane getJScrollPane() {
		return pictureScrollPane;
	}
	
	public void setVerticalScrollModel(BoundedRangeModel model) {
		pictureScrollPane.getVerticalScrollBar().setModel(model);
	}
	
	public void setVerticalScrollValue(int value) {
		pictureScrollPane.getVerticalScrollBar().setValue(value);
	}
}
