package edu.bu.signstream.grepresentation.view;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;

import javax.swing.*;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.Palette;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.ViewMode;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Segment;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class SegmentTemplate extends JPanel {	

	private static final long serialVersionUID = 1L;
	
	private SignStreamSegmentPanel segmentPanel = null;
	private Segment segment = null;
	private SegmentControlPanel controlPanel = null;
	
	private NonmanualFieldsView nonmanView = null;
	private CustomFieldsView customView = null;
	private GlossNRelatedView signView = null;
	private GestureNRelatedView speechView = null;
	
	private ViewMode viewMode;
	
	private JSplitPane outerJsp = null;
	private JSplitPane topJsp = null;
	private JSplitPane bottomJsp = null;
	
	private GridBagConstraints c = new GridBagConstraints();
	private boolean isHidden = false;
	private boolean selected = false;
	private JSplitPane sp;
	
	SegmentTemplate(Segment segment,
			UtteranceView view, 
			SignStreamSegmentPanel segmentPanel, PresentationField parentField,
			ViewMode viewMode) {
		this.segmentPanel = segmentPanel;
		this.segment = segment;
        this.viewMode = viewMode;

        controlPanel = new SegmentControlPanel(view, segmentPanel, segment, this);
        customView = new CustomFieldsView(segment, segmentPanel, controlPanel, parentField);
		signView = new GlossNRelatedView(segment, segmentPanel, view, parentField);
        speechView = new GestureNRelatedView(segment, segmentPanel,  view, parentField);
        nonmanView = new NonmanualFieldsView(segment, segmentPanel, controlPanel, signView,
            customView, speechView, parentField);

		topJsp = new JSplitPane(JSplitPane.VERTICAL_SPLIT, nonmanView, customView);
		topJsp.setOneTouchExpandable(true);
		topJsp.setDividerLocation(0.5);
		topJsp.setResizeWeight(0.5);
		topJsp.setDividerSize(5);		
		
		bottomJsp = new JSplitPane(JSplitPane.VERTICAL_SPLIT, signView, speechView);
		bottomJsp.setOneTouchExpandable(true);
		bottomJsp.setDividerLocation(0.5);
		bottomJsp.setResizeWeight(0.5);
		bottomJsp.setDividerSize(5);	
		
		outerJsp = new JSplitPane(JSplitPane.VERTICAL_SPLIT, topJsp, bottomJsp);
		outerJsp.setOneTouchExpandable(true);
		outerJsp.setDividerLocation(0.5);
		outerJsp.setResizeWeight(0.5);
		outerJsp.setDividerSize(5);	
		
		topJsp.setPreferredSize(new Dimension(
				(SegmentGraphReprUtil.UTTERANCE_VIEW_LABEL_PANEL_WIDTH + 
				SegmentGraphReprUtil.UTTERANCE_VIEW_GRAPH_PANEL_WIDTH),  
				(this.getHeight()/2)));
		bottomJsp.setPreferredSize(new Dimension(
				(SegmentGraphReprUtil.UTTERANCE_VIEW_LABEL_PANEL_WIDTH + 
				SegmentGraphReprUtil.UTTERANCE_VIEW_GRAPH_PANEL_WIDTH),  
				(this.getHeight()/2)));
		outerJsp.setPreferredSize(new Dimension(
				(SegmentGraphReprUtil.UTTERANCE_VIEW_LABEL_PANEL_WIDTH + 
				SegmentGraphReprUtil.UTTERANCE_VIEW_GRAPH_PANEL_WIDTH),  
				this.getHeight()));
		
		controlPanel.setPreferredSize(new Dimension(
				SegmentGraphReprUtil.UTTERANCE_VIEW_LEFT_PANEL_WIDTH, 
				this.getHeight()));
		controlPanel.setMinimumSize(new Dimension(
				SegmentGraphReprUtil.UTTERANCE_VIEW_LEFT_PANEL_WIDTH, 
				this.getHeight()));
		
		controlPanel.setBackground(Palette.BACKGROUND_COLOR);
		controlPanel.getTop().setBackground(Palette.BACKGROUND_COLOR);

		setBackground(Palette.BACKGROUND_COLOR);
		setBorder(BorderFactory.createLineBorder(Color.BLACK));
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		c.fill = GridBagConstraints.BOTH;
		c.weighty = 1.0;

		sp = new JSplitPane();
		
		sp.setLeftComponent(controlPanel);
		sp.setRightComponent(outerJsp);
		
		add(sp);
        calcViews();
	}
	
	private int viewCount() {
	    return viewMode.panels.size();
    }

    public void setViewMode(ViewMode mode) {
	    this.viewMode = mode;
	    updateViews();
    }

    public void updateViews() {
        // Update once to change visibility and prevent flashing,
        // and a second time to wait for Swing to recalculate sizing.
        calcViews();
        SwingUtilities.invokeLater(this::calcViews);
        // Please, Swing...
        SwingUtilities.invokeLater(() -> SwingUtilities.invokeLater(this::calcViews));
    }

	public void calcViews() {
	    // TODO: This will explode if we add more panels. Change layout and rewrite.

        var schema = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();

        //finally, distribute space evenly
        boolean view0 = viewMode.panels.contains(schema.panel(Constants.NONMANUAL_PANEL_ID));
        boolean view1 = viewMode.panels.contains(schema.panel(Constants.DEFAULT_PANEL_ID));
        boolean view2 = viewMode.panels.contains(schema.panel(Constants.SIGN_PANEL_ID));
        boolean view3 = viewMode.panels.contains(schema.panel(Constants.GESTURE_PANEL_ID));

        // Show/hide individual views
        topJsp.setVisible(view0 || view1);
        topJsp.getTopComponent().setVisible(view0);
        topJsp.getBottomComponent().setVisible(view1);
        bottomJsp.setVisible(view2 || view3);
        bottomJsp.getTopComponent().setVisible(view2);
        bottomJsp.getBottomComponent().setVisible(view3);

        int count = viewCount();
        if (count == 4) {
            topJsp.setDividerLocation(0.5);
            bottomJsp.setDividerLocation(0.5);
            outerJsp.setDividerLocation(0.5);
        } else if (count == 3) {
            if (view0 && view1) {
                topJsp.setDividerLocation(0.5);
                bottomJsp.setDividerLocation(view2 ? 1.0 : 0.0);
                outerJsp.setDividerLocation(0.66);
            } else if (view2 && view3) {
                topJsp.setDividerLocation(view0 ? 1.0 : 0.0);
                bottomJsp.setDividerLocation(0.5);
                outerJsp.setDividerLocation(0.33);
            }
        } else if (count == 2) {
            if (view0 && view1) {
                topJsp.setDividerLocation(0.5);
                outerJsp.setDividerLocation(1.0);
            } else if (view2 && view3) {
                bottomJsp.setDividerLocation(0.5);
                outerJsp.setDividerLocation(0.0);
            } else {
                topJsp.setDividerLocation(view0 ? 1.0 : 0.0);
                bottomJsp.setDividerLocation(view2 ? 1.0 : 0.0);
                outerJsp.setDividerLocation(0.5);
            }
        } else {
            if (view0 || view1) {
                topJsp.setDividerLocation(view0 ? 1.0 : 0.0);
                outerJsp.setDividerLocation(1.0);
            } else if (view2 || view3) {
                bottomJsp.setDividerLocation(view2 ? 1.0 : 0.0);
                outerJsp.setDividerLocation(0.0);
            }
        }

        topJsp.setDividerSize(view0 && view1 ? 5 : 0);
        bottomJsp.setDividerSize(view2 && view3 ? 5 : 0);
        outerJsp.setDividerSize(viewCount() > 1 ? 5 : 0);

        if (segmentPanel.getUtteranceView() != null)
            segmentPanel.getUtteranceView().updateScrollPane();
	}
	
	/**
	 * Is this segment template pane currently hidden? Returns boolean
	 * Called by UtteranceView.renderSegmentTemplatePanes()
	 */
	public boolean isHidden() {
		return isHidden;
	}
	
	public void selectTemplate() {
		segmentPanel.getUtteranceView().unselectAllSegmentTemplates();
		this.selected = true;
		segmentPanel.getUtteranceView().renderSegmentTemplatePanes();
	}
	
	/** 
	 * Is this segment the currently mouse-selected item?
	 * Called by UtteranceView.renderSegmentTemplatePanes()
	 * @return selected - boolean value activated by a mouse click on the panel, or default from UtteranceView
	 */
	public boolean isSelected() {
		return selected;
	}
	
	public void unselect() {
		this.selected = false;
	}
	
	public void update(Segment segment) {
		controlPanel.update(segment); 
		nonmanView.update(segment);
		signView.update(segment);
		speechView.update(segment);
		customView.update(segment);
        SS3Singleton.getMediaToolBar().deselectLabel();
	}
	
	public Segment getSegment() {
		return segment;
	}

	public String getPartitionLabel() {
	    return getSegment().getParentField().getTemporalPartition().getLabel();
    }

    public String getTierLabel() {
	    return getSegment().getParentField().getSS3GlossWindowSegment().getLabel();
    }
	
	public SegmentControlPanel getSegmentControlPanel() {
		return controlPanel;
	}

	public NonmanualFieldsView getNonmanView() {
		return nonmanView;
	}
	
	public CustomFieldsView getCustomView() {
		return customView;
	}
	
	public GlossNRelatedView getSignView() {
		return signView;
	}
	
	public GestureNRelatedView getSpeechView() {
		return speechView;
	}
	
	public JSplitPane getTopSplitPane() {
		return topJsp;
	}
	
	public JSplitPane getBottomSplitPane() {
		return bottomJsp;
	}
	
	public JSplitPane getSplitPane() {
		return sp;
	}
	
	public EventsEntity getSelectedUtterance() {
		return getSegmentControlPanel().getSelectedUtterance();
	}

	public int getSelectedUtteranceIndex() {
	    return getSegmentControlPanel().getSelectedUtteranceIndex();
    }

    public ViewMode viewMode() {
	    return viewMode;
    }
}