package edu.bu.signstream.grepresentation.view;

import java.awt.*;
import java.awt.event.ActionEvent;

import javax.swing.*;

import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class UpperLeftCorner extends Corner {
    public static final int SLIDER_MAX = 1000000000;

    private final JSlider zoomInOut = new JSlider();
	private boolean updating = false;

	private final JButton lock = new JButton("U");
	private boolean locked = false;

	public UpperLeftCorner(final SignStreamSegmentPanel segmentPanel) {

		zoomInOut.setMajorTickSpacing(SLIDER_MAX / 5);
		zoomInOut.setMinorTickSpacing(SLIDER_MAX / 20);
		zoomInOut.setPaintTicks(true);
		zoomInOut.setMaximum(SLIDER_MAX);
		zoomInOut.setMinimum(0);

		zoomInOut.getInputMap().put(KeyStroke.getKeyStroke("LEFT"), "none");
		zoomInOut.getInputMap().put(KeyStroke.getKeyStroke("RIGHT"), "none");

		zoomInOut.getInputMap().put(KeyStroke.getKeyStroke("UP"), "utteranceUp");
		zoomInOut.getInputMap().put(KeyStroke.getKeyStroke("DOWN"), "utteranceDown");
		zoomInOut.getActionMap().put("utteranceUp", new AbstractAction() {
			@Override
			public void actionPerformed(ActionEvent e) {
				// GD 2022-05-10: although deprecated, this is still in use when on the utterance view, clicking anywhere except the utterance list
				moveUtteranceList(true);
			}
		});
		zoomInOut.getActionMap().put("utteranceDown", new AbstractAction() {
			@Override
			public void actionPerformed(ActionEvent e) {
				// GD 2022-05-10: although deprecated, this is still in use when on the utterance view, clicking anywhere except the utterance list
				moveUtteranceList(false);
			}
		});

		zoomInOut.addKeyListener(SS3Singleton.getVideoControlKeyListener());

		zoomInOut.setValue((int)Math.round(segmentPanel.getZoomer().zoom() * SLIDER_MAX));
		zoomInOut.setPreferredSize(new Dimension(140, 30));
		zoomInOut.addChangeListener(event -> {
            if (updating) return;

            double value = (double)zoomInOut.getValue() / SLIDER_MAX;
            segmentPanel.zoom(value);

            if (!locked)
                lock.doClick();
        });

		locked = segmentPanel.zoomLocked();

		lock.setFocusable(false);
        UserInterfaceUtil.setButtonsLNF(lock);
        lock.addActionListener(l -> {
            locked = !locked;
            lock.setText(locked ? "L" : "U");
            lock.setToolTipText(locked ? "Lock Zoom" : "Unlock Zoom");
            segmentPanel.zoomLock(locked);
        });
        // Update text
        lock.doClick();
        lock.doClick();

		var row = new JPanel(new FlowLayout(FlowLayout.CENTER, 0, 0));
		row.add(zoomInOut);
		row.add(lock);

		setLayout(new BorderLayout(0, 0));
		add(row, BorderLayout.CENTER);
	}

	@Deprecated
	private void moveUtteranceList(boolean up) {
		SignStreamSegmentPanel segmentPanel = SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel();
		if (segmentPanel.isUtteranceLoaded()) {
			SegmentControlPanel controlPanel = segmentPanel.getUtteranceView().getSegmentTemplate().getSegmentControlPanel();
			controlPanel.moveUtteranceListSelection(up);
		}
	}

	public boolean locked() { return locked; }
	public void lock(boolean locked) {
	    if (locked != this.locked)
	        lock.doClick();
    }

	public void setZoom(double value) {
	    updating = true;
		zoomInOut.setValue((int)Math.round(value * SLIDER_MAX));
		updating = false;
	}

	@Deprecated
	public JSlider getZoomInOut() {
		return zoomInOut;
	}
}
