package edu.bu.signstream.grepresentation.view;

import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;
import java.util.StringTokenizer;
import java.util.logging.Level;
import java.util.stream.Collectors;

import javax.swing.BoundedRangeModel;
import javax.swing.DefaultBoundedRangeModel;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JViewport;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.plaf.basic.BasicSplitPaneDivider;
import javax.swing.plaf.basic.BasicSplitPaneUI;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.Palette;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.ViewMode;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Database;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowItem;
import edu.bu.signstream.time.*;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.Segment;
import edu.bu.signstream.grepresentation.fields.presentation.*;
import edu.bu.signstream.grepresentation.listener.CustomFieldsListener;
import edu.bu.signstream.grepresentation.listener.GestureNRelatedFieldsListener;
import edu.bu.signstream.grepresentation.listener.GlossNRelatedFieldsListener;
import edu.bu.signstream.grepresentation.listener.NonManualFieldsListener;
import edu.bu.signstream.grepresentation.reguestsHandler.Manager;
import edu.bu.signstream.grepresentation.reguestsHandler.RequestHandler;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.ui.SS3SignStreamApplication;
import edu.bu.signstream.ui.panels.search.SearchSavedState;

public class UtteranceView extends JPanel {

	private static final long serialVersionUID = 1L;
	
	private UtteranceViewHeader header = null;
	private UtteranceViewFooter footer = null;

	protected JScrollPane pictureScrollPane;
	protected final ChangeListener scrollListener = e -> updateUtteranceLists();
	private JPanel centerPanel = null;

	private ArrayList<Segment> segments;
	private SignStreamSegmentPanel segmentPanel;
	private ArrayList<SegmentGraphicRepresentation> repList = new ArrayList<SegmentGraphicRepresentation>();
	private ArrayList<SegmentTemplate> templates = new ArrayList<SegmentTemplate>();

	/** 
	 * This template object represents the "prototypical" example of a template currently being rendered on screen.
	 * It is technically always pointing to the last one rendered in a list of templates.
	 */
	private SegmentTemplate template = null;
	private NonmanualFieldsView nonmanView = null;
	private CustomFieldsView customView = null;
	private GlossNRelatedView signView = null;
	private GestureNRelatedView speechView = null;
	
	private SplitPaneListener controlPanelSplitPaneListener = new SplitPaneListener();
	private SplitPaneListener utteranceSplitPaneListener = new SplitPaneListener();

	private boolean showEndOnZoom = false;
	
	public UtteranceView(ArrayList<Segment> segments,
			SignStreamSegmentPanel segmentPanel) {

		this.segments = segments;
		this.segmentPanel = segmentPanel;

		header = new UtteranceViewHeader(segmentPanel, this);		
		footer = new UtteranceViewFooter(segmentPanel, this);
		

		repList.clear();

		centerPanel = new JPanel();
		centerPanel.setLayout(new GridLayout(0, 1));

		for (Segment segment1 : segments) {
			template = createNewSegmentTemplate(segment1, segment1.getParentField(), segment1.viewMode());

			if (pictureScrollPane == null) {
                var schema = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();
                var view = segment1.viewMode();

                if (view.panels.contains(schema.panel(Constants.NONMANUAL_PANEL_ID)))
                    pictureScrollPane = template.getNonmanView().getJScrollPane();
                if (view.panels.contains(schema.panel(Constants.DEFAULT_PANEL_ID)))
                    pictureScrollPane = template.getCustomView().getJScrollPane();
                if (view.panels.contains(schema.panel(Constants.SIGN_PANEL_ID)))
                    pictureScrollPane = template.getSignView().getJScrollPane();
                if (view.panels.contains(schema.panel(Constants.GESTURE_PANEL_ID)))
                    pictureScrollPane = template.getSpeechView().getJScrollPane();
            }

			var scrollModel = pictureScrollPane.getHorizontalScrollBar().getModel();
            template.getNonmanView().getJScrollPane().getHorizontalScrollBar().setModel(scrollModel);
            template.getCustomView().getJScrollPane().getHorizontalScrollBar().setModel(scrollModel);
            template.getSignView().getJScrollPane().getHorizontalScrollBar().setModel(scrollModel);
            template.getSpeechView().getJScrollPane().getHorizontalScrollBar().setModel(scrollModel);

			utteranceSplitPaneListener.addSplitPane(template.getNonmanView().getSplitPane());
			utteranceSplitPaneListener.addSplitPane(template.getCustomView().getSplitPane());
			utteranceSplitPaneListener.addSplitPane(template.getSignView().getSplitPane());
			utteranceSplitPaneListener.addSplitPane(template.getSpeechView().getSplitPane());
			
			controlPanelSplitPaneListener.addSplitPane(template.getSplitPane());
		}

        SS3Singleton.getSignStreamApplication().toggleViewMenu(true);
		SS3Singleton.getSignStreamApplication().recreateTogglingOptions(templates);
		SS3Singleton.getSignStreamApplication().toggleSearchOption(true);
		SS3Singleton.getSignStreamApplication().toggleMacroUnitMenu(false);

		updateScrollPane();

        footer.getScrollBarView().getJScrollBar().setModel(pictureScrollPane.getHorizontalScrollBar().getModel());
        header.getTimelineView().pictureScrollPane.getHorizontalScrollBar().setModel(
                footer.getScrollBarView().getJScrollBar().getModel());

        utteranceSplitPaneListener.addSplitPane(header.getTimelineView().getSplitPane());
        utteranceSplitPaneListener.addSplitPane(footer.getScrollBarView().getSplitPane());

        controlPanelSplitPaneListener.addSplitPane(header.getSplitPane());
        controlPanelSplitPaneListener.addSplitPane(footer.getSplitPane());

		setLayout(new BorderLayout(0, 0));
		add(header, BorderLayout.NORTH);
		add(centerPanel, BorderLayout.CENTER);
		add(footer, BorderLayout.SOUTH);

		updateDrawingsAreaSize();
	}

	public void updateScrollPane() {
	    if (templates.size() == 0)
	        return;

	    if (pictureScrollPane != null && pictureScrollPane.getViewport().getChangeListeners().length != 0)
	        pictureScrollPane.getViewport().removeChangeListener(scrollListener);

        // Get the pictureScrollPane from the the last segment template for calculating scroll positions.
        var template = templates.get(templates.size() - 1);
        var viewMode = template.viewMode();

        var schema = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();
        if (viewMode.panels.contains(schema.panel(Constants.NONMANUAL_PANEL_ID)))
            pictureScrollPane = template.getNonmanView().getJScrollPane();
        else if (viewMode.panels.contains(schema.panel(Constants.DEFAULT_PANEL_ID)))
            pictureScrollPane = template.getCustomView().getJScrollPane();
        else if (viewMode.panels.contains(schema.panel(Constants.SIGN_PANEL_ID)))
            pictureScrollPane = template.getSignView().getJScrollPane();
        else if (viewMode.panels.contains(schema.panel(Constants.GESTURE_PANEL_ID)))
            pictureScrollPane = template.getSpeechView().getJScrollPane();

        pictureScrollPane.getViewport().addChangeListener(scrollListener);

    }

    public NonmanualFieldsView getNonmanualFieldsView() {
		return nonmanView;
	}
	
	public CustomFieldsView getCustomView() {
		return customView;
	}
	
	public GlossNRelatedView getGlossNRelatedView() {
		return signView;
	}
	
	public GestureNRelatedView getSpeechView() {
		return speechView;
	}
	
	public Segment getSegment() {
		return segments.get(0);
	}

	public void setVisible(double startCoord, double endCoord, boolean forceScroll) {
	    if (pictureScrollPane == null)
	        return;

        var viewport = pictureScrollPane.getViewport();
        var visibleRect = viewport.getVisibleRect();
        var viewPosition = viewport.getViewPosition();

        int visibleWidth = visibleRect.width;
        int X1 = viewPosition.x;
        int X2 = X1 + visibleWidth;

        if (visibleWidth == 0)
            throw new RuntimeException("UtteranceView.pictureScrollPane visibleWidth=0");

        if (!forceScroll && X1 <= startCoord && X2 >= endCoord)
            return;

        Point scrollTo;
        if (Util.dragging() && startCoord >= X2)
            scrollTo = new Point(X1 + (int)(startCoord - X2), 0);
        else
            scrollTo = new Point( (int) (startCoord - 1 < 0 ? startCoord : startCoord - 1), 0);

        scrollRectToVisible(scrollTo);
	}

	public void updateDrawingsAreaSize() {
	    var sw = SS3Singleton.getMediaPlayer().duration()
            .as(TimeUnit.COORDINATE);

		double screenWidth = sw + DatabaseView.screenBufferAmount;

		// going through the rep's for nonman and gloss views all together (resetting them to length of movie duration)
		for (SegmentGraphicRepresentation rep : repList) {
			Dimension d = rep.getPreferredSize();
			Dimension dd = new Dimension((int) screenWidth, d.height);
			rep.setPreferredSize(dd);
			rep.setSize(dd);
			rep.revalidate();
		}
		
		LinenView l = header.getTimelineView().linenView;
		Dimension d = l.getPreferredSize();
		Dimension dd = new Dimension((int) screenWidth, d.height);
		l.setPreferredSize(dd);
		l.revalidate();
	}

	public int getWidht() {

		try {
			Rectangle visibleRect = null;
			if (template != null && template.getNonmanView() != null) {
				visibleRect = template.getNonmanView().getJScrollPane()
						.getViewport().getVisibleRect();
			}
			return visibleRect.width;
		} catch (Exception ex) {
			ex.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);
		}
		return -1;
	}

	public Point getVisiblePoint() {		
		JScrollPane sp = template.getSignView().getJScrollPane();		
		int value = sp.getHorizontalScrollBar().getValue();
		return new Point(value, 0);
	}

	public double getVisibleMiddlePointTime() {
		Rectangle visibleRect = null;
		Point viewPosition = null;

		try {
			if (template != null && template.getNonmanView() != null) {
				visibleRect = template.getNonmanView().getJScrollPane()
						.getViewport().getVisibleRect();
				viewPosition = template.getNonmanView().getJScrollPane()
						.getViewport().getViewPosition();
			}

			int visibleWidth = visibleRect.width;
			int X1 = viewPosition.x;
			int X = X1 + visibleWidth / 2;

			double time = segmentPanel.getZoomer().getCoordinateTimeConvertor()
					.convertCoordinateToTime(X);

			return time;

		} catch (Exception ex) {
			ex.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);
		}

		return -1;
	}

	public void setMidPointTimeVisible(double time) {
		double midCoord = segmentPanel.getZoomer().getCoordinateTimeConvertor()
				.convertTimeToCoordinate(time);
		Rectangle visibleRect = null;

		try {
			if (template != null && template.getNonmanView() != null) {
				visibleRect = template.getNonmanView().getJScrollPane()
						.getViewport().getVisibleRect();
			}

			double halfVisibleWidth = visibleRect.width / 2;
			double X1 = midCoord - halfVisibleWidth;

			if (X1 < 0)
				X1 = 0;

			Point scrollTo = new Point((int) X1, 0);
			
			scrollRectToVisible(scrollTo);
		} catch (Exception ex) {
			ex.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);
		}
	}

	double distance = 0;

	private double getDistanceToCoord(double coord) {
		Rectangle visibleRect = null;
		Point viewPosition = null;

		try {
			if (template != null && template.getNonmanView() != null) {
				visibleRect = template.getNonmanView().getJScrollPane()
						.getViewport().getVisibleRect();
				viewPosition = template.getNonmanView().getJScrollPane()
						.getViewport().getViewPosition();
			}

			int visibleWidth = visibleRect.width;
			int X1 = viewPosition.x;
			int X2 = X1 + visibleWidth;

			if (X1 < coord && X2 > coord) {
				distance = coord - X1;
			}
		} catch (Exception ex) {
			return distance;
		}
		return distance;
	}
	
	public void setZoomInOutValue(double value) {
		header.getTimelineView().getUpperLeftCorner().setZoom(value);
	}

	// if center of selected item is on the left - keep start coord fixed
	// if center of selected item is on the right - keep end coord fixed
	// if item is not selected, keep video slider fixed
	public synchronized void zoom(double fr) {
		segmentPanel.getZoomer().zoom(fr);
		setZoomInOutValue(fr);

		Point scrollTo = null;

		double sliderVisibleCoord = getDistanceToCoord(segmentPanel.getSlider()
				.getSliderPosition());
		Event selectedEvent = segmentPanel.getSelectedEvent();

		// JACK TODO: Maybe put this back, but the logic is broken for now.
		if (false && selectedEvent != null) {
			double eventCoord = showEndOnZoom ? selectedEvent.getEndTimeCoordinate() : selectedEvent.getStartTimeCoordinate();
			double eventVisibleCoord = getDistanceToCoord(eventCoord);

            // Recalculate the slider position based on the new zoom level
            segmentPanel.getSlider().setSliderPosition(SS3Singleton.getMediaPlayer().time());

			double X2 = eventCoord - eventVisibleCoord;

			if (X2 < 0)
				X2 = 0;

			scrollTo = new Point((int) X2, 0);
		} else if (sliderVisibleCoord > -1) {
			
			segmentPanel.getZoomer().zoom(fr);
			segmentPanel.repaint();
			
            // Recalculate the slider position based on the new zoom level
            segmentPanel.getSlider().setSliderPosition(SS3Singleton.getMediaPlayer().time());

			double newSliderPosition = segmentPanel.getSlider()
					.getSliderPosition();
			double X = newSliderPosition - sliderVisibleCoord;

			if (X < 0)
				X = 0;

			scrollTo = new Point((int) X, 0);
		} else {
			EventsEntity selectedEntity = SS3Singleton.getMediaToolBar(null).selectedEntity();
			var parentEntityIDs = selectedEntity.getParentEntities();
			if (parentEntityIDs != null && !parentEntityIDs.isEmpty()) {
				int size = parentEntityIDs.size();
				Event firstParentEvent = parentEntityIDs.get(0);
				Event lastParentEvent = parentEntityIDs.get(size - 1);
				
				if (firstParentEvent != null && lastParentEvent != null) {
					setVisible(firstParentEvent.getStartTimeCoordinate(),
							lastParentEvent.getEndTimeCoordinate(), false);
				} else {
					double midPointTime = getVisibleMiddlePointTime();
					setMidPointTimeVisible(midPointTime);
				}
			}
		}

		updateDrawingsAreaSize();
		
		scrollRectToVisible(scrollTo);

		segmentPanel.repaint();
		segmentPanel.updateUI();
	}

	public void setShowEndOnZoom(boolean showEndOnZoom) {
		this.showEndOnZoom = showEndOnZoom;
	}

	public void setVisibleCoord(int coord) {
		Point scrollTo = null;

		Rectangle visibleRect = null;
		Point viewPosition = null;

		try {
			if (template != null && template.getNonmanView() != null) {
				visibleRect = template.getNonmanView().getJScrollPane()
						.getViewport().getVisibleRect();
				viewPosition = template.getNonmanView().getJScrollPane()
						.getViewport().getViewPosition();
			}

			int visibleWidth = visibleRect.width;
			int X1 = viewPosition.x;
			int X2 = X1 + visibleWidth;

			if (X1 < coord && X2 > coord) {
				return;
			} else {
				int dWidth = visibleWidth - coord;
				int X3 = coord - dWidth / 2;

				if (X3 < 0)
					X3 = 0;

				scrollTo = new Point(X3, 0);
				
				scrollRectToVisible(scrollTo);
			}
		} catch (Exception ex) {
			ex.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);
		}
	}

	public ArrayList<Segment> getSegments() {
        return templates.stream()
            .map(SegmentTemplate::getSegment)
            .collect(Collectors.toCollection(ArrayList::new));
	}

	// 2020-09-24: turning this from public to private, since its not used
	private JScrollPane getPictureScrollPane() {
		return pictureScrollPane;
	}

	public JScrollBar getHorizontalScrollBar() {
		if (templates.size() > 0) {
			SegmentTemplate template = templates.get(templates.size() - 1);
			return template.getSignView().getJScrollPane()
					.getHorizontalScrollBar();
		} else if (pictureScrollPane != null) {
			return pictureScrollPane.getHorizontalScrollBar();
		} else {
			return null;
		}
	}

//	public double getStartTime() {
//		double startTime = -1;
//		for (Segment segment : segments) {
//			double time = segment.getStartTime();
//			if (time == -1) {
//				startTime = time;
//			} else if (time < startTime) {
//				startTime = time;
//			}
//		}
//		return startTime;
//	}

	public ArrayList<SegmentTemplate> getSegmentTemplates() {
		return templates;
	}
	
	public void clearSegmentTemplates() {
		templates.clear();
	}

	public SegmentTemplate getSegmentTemplate(String segmentID) {
	    return templates.stream()
            .filter(t -> t.getSegment().getID().equals(segmentID))
            .findFirst()
            .orElse(null);
	}

	public SegmentTemplate getSegmentTemplateBy(String segmentTierID) {
        return templates.stream()
            .filter(t -> t.getSegment()
                .getParentField()
                .getSS3GlossWindowSegment()
                .getID().equals(segmentTierID))
            .findFirst()
            .orElse(null);
    }

//	public double getEndTime() {
//		double endTime = -1;
//		for (Segment segment : segments) {
//			double time = segment.getEndTime();
//			if (time == -1) {
//				endTime = time;
//			} else if (time > endTime) {
//				endTime = time;
//			}
//		}
//		return endTime;
//	}

	public void scrollRectToVisible(Point point) {
		//System.out.println("Scroll Rect to Visible");
		for (SegmentTemplate template: templates) {
			if (template != null) {
				if (template.getNonmanView() != null) {
					template.getNonmanView().scrollRectToVisible(point);
				}
				if (template.getCustomView() != null) {
					template.getCustomView().scrollRectToVisible(point);
				}
				if (template.getSignView() != null) {
					template.getSignView().scrollRectToVisible(point);
				}
				if (template.getSpeechView() != null) {
					template.getSpeechView().scrollRectToVisible(point);
				}
			}
		}
		
		// this is the thing doing the actual left/right scrolling now - the above are just invalidating/revalidating to repaint 
		// the scrollbars of all the above plus the timeline are synced, which is why this works
		header.getTimelineView().scrollRectToVisible(point);
		
		updateUtteranceLists();
	}
	
	/**
	 * In each SegmentTemplate of this UtteranceView, programmatically click the expand button 
	 * on its NonManualFieldsView to trigger a refresh of visible nonmanual fields in the viewbox
	 */
	void showAllFields() {
		for (SegmentTemplate template: templates) {
			if (template != null) {
				if (template.getNonmanView() != null) {
					template.getNonmanView().getCSFieldDialog().getCSFieldPanel().showAllFieldsCurrentUtterance();
				}
			}
		}
	}
	
	public void updateUtteranceLists() {
		if (pictureScrollPane != null) {
			for (SegmentTemplate template: templates) {
				Rectangle rect = pictureScrollPane.getViewport().getViewRect();
				int min = (int) rect.getMinX() + 2;
				int max = (int) rect.getMaxX();

				if (min != max) {
                    SegmentGraphReprUtil.visibleStart = TimeRange.ABSOLUTE.time(min, TimeUnit.COORDINATE);
                    SegmentGraphReprUtil.visibleEnd = TimeRange.ABSOLUTE.time(max, TimeUnit.COORDINATE);
                }

				var utterances = template.getSegment()
                    .getParentField()
                    .getEntities().stream()
                    .filter(e -> SegmentGraphReprUtil.visibleInterval()
                        .overlaps(e.interval().convert(TimeUnit.COORDINATE, Rounding.NEAREST), Bound.EXCLUSIVE))
                    .collect(Collectors.toList());

				template.getSegmentControlPanel().updateUtteranceListSelectedItems(utterances);
			}
		}
	}

    public void updateUtteranceIndex() {
        for (SegmentTemplate template: templates)
            template.getSegmentControlPanel().updateUtteranceList();
    }

	public Dimension getViewportPreferredSize() {
		JViewport vp = pictureScrollPane.getViewport();
		return vp.getPreferredSize();
	}

	public int getVisibleWidth2() {
		if (pictureScrollPane == null) {
			return -1;
		} else {
			JViewport vp = pictureScrollPane.getViewport();
			Dimension d = vp.getExtentSize();
			
			return d.width;
		}
	}

	public SignStreamSegmentPanel getSignStreamSegmentPanel() {
		return segmentPanel;
	}

	public ArrayList<SegmentGraphicRepresentation> getSegmentGraphicRep() {
		return repList;
	}

	public SegmentTemplate getSegmentTemplate() {
		return template;
	}
	
	public void renderSegmentTemplatePanes() {
		centerPanel.removeAll();
		
		for (SegmentTemplate st: templates) {
			if (st.isSelected()) {
				st.getSegmentControlPanel().setBackground(Palette.BACKGROUND_COLOR.darker());
				st.getSegmentControlPanel().getTop().setBackground(Palette.BACKGROUND_COLOR.darker());
				st.getSegmentControlPanel().setRadioButtonSelected(true);
			} else {
				st.getSegmentControlPanel().setBackground(Palette.BACKGROUND_COLOR);
				st.getSegmentControlPanel().getTop().setBackground(Palette.BACKGROUND_COLOR);
				st.getSegmentControlPanel().setRadioButtonSelected(false);
			}
			
			centerPanel.add(st);
		}
		
		revalidate();
		repaint();
		
		segmentPanel.revalidate();
		segmentPanel.repaint();
	}
	
	public void unselectAllSegmentTemplates() {
		for (SegmentTemplate st: templates) {
			st.unselect();
		}
	}
	
	public SegmentTemplate getSelectedSegmentTemplate() {
		for (SegmentTemplate st: templates) {
			if (st.isSelected()) {
				return st;
			}
		}
		
		return null;
	}
	
	public int getSelectedSegmentTemplateIndex() {
		for (int i=0 ; i <templates.size() ; i++) {
			if (templates.get(i).isSelected()) {
				return i;
			}
		}
		
		return -1;
	}
	
	public void removeSelectedPane() {
		SegmentTemplate selectedSegmentTemplate = getSelectedSegmentTemplate();
		if (selectedSegmentTemplate == null) {
			return;		// if nothing is selected, don't do anything
		}
		
		if (templates.size() == 1)
			return; // don't allow there to ever be no templates remaining
		
		// save changes to db
		segmentPanel.updateLoadedDatabase();
		
		// remove the selected template row and segment object itself		
		templates.remove(selectedSegmentTemplate);

		Segment selectedSegment = selectedSegmentTemplate.getSegment();		
		segments.remove(selectedSegment);
		
		// if after deleting a template, none remains selected, force by default the 0th template to selected
		boolean noneSelected = true;		
		for (SegmentTemplate st: templates) {
			if (st.isSelected())
				noneSelected = false;
		}
		if (noneSelected) {
			SegmentTemplate template = templates.get(0);
			template.selectTemplate();
			SS3Singleton.getMediaToolBar().enterUtterance(
					template.getSegmentControlPanel().getSelectedUtterance());
		}
		
		SS3Singleton.getSignStreamApplication().recreateTogglingOptions(templates);
		
		// repaint
		renderSegmentTemplatePanes();

		//remove menu items for selected segment from menu bar
		//SS3Singleton.getSignStreamApplication().removeTogglingOptions(selectedSegment);
	}

	public void moveSelectedPaneUp() {
		int moveTo = 0, moveFrom = 0;
		
		for (int i = 0; i < templates.size(); i++) {
			if (templates.get(i).isSelected()) {
				moveFrom = i;
				moveTo = i - 1;
				break;
			}
		}
		
		if (moveFrom == 0)
			return;
		
		SegmentTemplate selectedTemplate = templates.get(moveFrom);
		SegmentTemplate previousTemplate = templates.get(moveTo);
		
		templates.set(moveFrom, previousTemplate);
		templates.set(moveTo, selectedTemplate);
		
		SS3Singleton.getSignStreamApplication().recreateTogglingOptions(templates);
		
		renderSegmentTemplatePanes();
	}

	public void moveSelectedPaneDown() {
		int moveTo = 0, moveFrom = 0;
		
		for (int i = 0; i < templates.size(); i++) {
			if (templates.get(i).isSelected()) {
				moveFrom = i;
				moveTo = i + 1;
				break;
			}
		}
		
		if (moveFrom == templates.size() - 1)
			return;
		
		if (templates.size() == 0)
			return;
		
		
		SegmentTemplate selectedTemplate = templates.get(moveFrom);
		SegmentTemplate nextTemplate = templates.get(moveTo);
		
		templates.set(moveFrom, nextTemplate);
		templates.set(moveTo, selectedTemplate);
		
		SS3Singleton.getSignStreamApplication().recreateTogglingOptions(templates);
		
		renderSegmentTemplatePanes();
	}
	
	public UtteranceViewHeader getHeader() {
		return header;
	}
	
	public UtteranceViewFooter getFooter() {
		return footer;
	}
	
	public boolean canSwitchThisSegment(Segment switchingSegment, 
			String temporalPartitionLabel, String segmentTierWithParticipantLabel) {

		String segmentTierLabel = segmentTierWithParticipantLabel;
		int index = segmentTierWithParticipantLabel.indexOf(":");
		if (index != -1)
			segmentTierLabel = segmentTierWithParticipantLabel.substring(index + 1);
				
		for (SegmentTemplate template: templates) {
			SegmentControlPanel scp = template.getSegmentControlPanel();
			
			// only compare the switchingSegment to segments other than itself
			if (scp.getSegment().equals(switchingSegment))
				continue;
			
			String tpLabel = scp.getCurrentPartitionString();
			String stLabel = scp.getCurrentSegmentString();
			index = scp.getCurrentSegmentString().indexOf(":"); 
			if (index != -1)
			 	stLabel = scp.getCurrentSegmentString().substring(index + 1);
			
			if (tpLabel.equals(temporalPartitionLabel) && stLabel.equals(segmentTierLabel)) {
				return false;
			}
		}
		
		return true;
	}

	/**
	 * Add a new segment object (utterance pane).
	 * If segmentToRemove is not null, remove it from the list of segments and replace in its position
	 */
	public void addUtterancePane(Segment segmentToRemove,
			String temporalPartitionName, String segmentTierName) {
		// 1) go through the DBview wrappers to find the one matching the segment tier label		
		ArrayList<FieldWrapper> databaseViewWrappers = 	segmentPanel.
				getDatabaseView().getSegmentGraphicRepresentation().getFieldWrappersCollection().getWrappers();		
		
		PresentationFieldWrapper partFldWrapper = null;
		SegmentFieldWrapper segFldWrapper = null;
		PresentationField segFld = null;

		for (FieldWrapper wrapper: databaseViewWrappers) {
			if (wrapper.getField().getName().equals(temporalPartitionName)
					&& wrapper instanceof PresentationFieldWrapper) {
				partFldWrapper = (PresentationFieldWrapper) wrapper;
				for (SegmentFieldWrapper wrapper2: partFldWrapper.getSegmentsFieldWrappers()) {
					if (wrapper2.getField().getName().equals(segmentTierName)) {
						segFldWrapper = wrapper2;
						segFld = (PresentationField) wrapper2.getField();
						break;
					}
				}
			}
		}
		
		if (segFldWrapper == null) {
			System.out.println("There was no field found with name " + segmentTierName);
			return;
		}

        for (var template : templates)
            template.getNonmanView().getCSFieldDialog().getCSFieldPanel().apply();

		ArrayList<PresentationEventsEntity> eventsEntitiesToLoad = segFld.getEntities();

		// 2) get the utterances of the segment tier
		ArrayList<SS3GlossWindowItem> surroundingItems = new ArrayList<SS3GlossWindowItem>();
		
		int size = eventsEntitiesToLoad.size();
		// use these indices below to load all items regardless of the clicked one
		int startIndex = 0;
		int endIndex = size;	// top bound exclusive
		
		for (PresentationEventsEntity entity: eventsEntitiesToLoad.subList(startIndex, endIndex)) {
			surroundingItems.add(entity.getSS3GlossWindowItem());
		}
		
		// 3) before proceeding, reset start/end times and save changes to db
		segmentPanel.updateLoadedDatabase();
		
		SegmentTemplate segmentToRemoveTemplate = null;
		
		boolean isSelected = true;
		if (segmentToRemove != null) {
			segmentToRemoveTemplate = this.getSegmentTemplate(segmentToRemove.getID());
			isSelected = segmentToRemoveTemplate.isSelected();
		}
		// 4) remove the Segment obj (if currently in view) and replace it with a new one 	
		int indexOfRemovedSegment = removeSegment(segmentToRemove);
		
		// 5) create the new segment out of the chosen item and reorient start/end times
		Manager mngr = SS3Singleton.getManager();
		RequestHandler handler = mngr.getRequestHandler();

		Segment newSegment = null;
		
		if (indexOfRemovedSegment != -1)
			newSegment = handler.convertMultipleItemsAndAddToSegmentPanelAtIndex(segFld.getSS3GlossWindowSegment(), segFld, indexOfRemovedSegment);
		else
			newSegment = handler.convertMultipleItemsAndAddToSegmentPanel(segFld.getSS3GlossWindowSegment(), segFld);	
		
		// 6) create the new template which will be displayed on-screen
		if (segmentToRemove != null) {
			template = createNewSegmentTemplate(newSegment, segFld, indexOfRemovedSegment, isSelected,
					segmentToRemoveTemplate.viewMode());
		} else {
			template = createNewSegmentTemplate(newSegment, segFld, indexOfRemovedSegment, isSelected);
		}

        // Get the pictureScrollPane from the the first visible view for calculating scroll positions.
        var schema = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();
		var viewMode = template.viewMode();
        if (viewMode.panels.contains(schema.panel(Constants.NONMANUAL_PANEL_ID)))
            pictureScrollPane = template.getNonmanView().getJScrollPane();
        else if (viewMode.panels.contains(schema.panel(Constants.DEFAULT_PANEL_ID)))
            pictureScrollPane = template.getCustomView().getJScrollPane();
        else if (viewMode.panels.contains(schema.panel(Constants.SIGN_PANEL_ID)))
            pictureScrollPane = template.getSignView().getJScrollPane();
        else if (viewMode.panels.contains(schema.panel(Constants.GESTURE_PANEL_ID)))
            pictureScrollPane = template.getSpeechView().getJScrollPane();

		template.getNonmanView().getJScrollPane().getHorizontalScrollBar().setModel(
				footer.getScrollBarView().getJScrollBar().getModel());
		template.getCustomView().getJScrollPane().getHorizontalScrollBar().setModel(
				footer.getScrollBarView().getJScrollBar().getModel());
		template.getSignView().getJScrollPane().getHorizontalScrollBar().setModel(
				footer.getScrollBarView().getJScrollBar().getModel());
		template.getSpeechView().getJScrollPane().getHorizontalScrollBar().setModel(
				footer.getScrollBarView().getJScrollBar().getModel());
		template.getSignView().getJScrollPane().getHorizontalScrollBar().setModel(
				footer.getScrollBarView().getJScrollBar().getModel());
		header.getTimelineView().pictureScrollPane.getHorizontalScrollBar().setModel(
				footer.getScrollBarView().getJScrollBar().getModel());
		
		utteranceSplitPaneListener.addSplitPane(template.getNonmanView().getSplitPane());
		utteranceSplitPaneListener.addSplitPane(template.getCustomView().getSplitPane());
		utteranceSplitPaneListener.addSplitPane(template.getSignView().getSplitPane());
		utteranceSplitPaneListener.addSplitPane(template.getSpeechView().getSplitPane());
		
		controlPanelSplitPaneListener.addSplitPane(template.getSplitPane());
		
		
		footer.getScrollBarView().getJScrollBar().getModel().addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				// as user scrolls, update the utterance lists, same as scrollRectToVisible would do
				updateUtteranceLists();
			}
		});

		this.renderSegmentTemplatePanes();

		// does some sort of updating so graph fields are drawn initially
		template.getSegmentControlPanel().offsetSelectedUtterance(0);

        if (SegmentGraphReprUtil.visibleStart != null) {
            double visibleMovieCoord = SegmentGraphReprUtil.visibleStart.absolute().as(TimeUnit.COORDINATE, Rounding.NEAREST);
            final Point scrollTo = new Point((int) visibleMovieCoord, 0);

            // need to invoke later to ensure that the pictureScrollPane is the correct size before actually scrolling
            SwingUtilities.invokeLater(new Runnable() {
                @Override
                public void run() {
                    scrollRectToVisible(scrollTo);
                }
            });
        }

        for (var template : templates)
            template.getNonmanView().getCSFieldDialog().getCSFieldPanel().apply();

        updateDrawingsAreaSize();

		//add new set of toggling options to the menu bar
		SS3Singleton.getSignStreamApplication().recreateTogglingOptions(templates);

		// update again, but this time with utterance loaded - added here to get the utterance index & search to reflect live results
		segmentPanel.updateLoadedDatabase();
		
	}
	
	private SegmentTemplate createNewSegmentTemplate(Segment segment, PresentationField segFld, ViewMode viewMode) {
		return createNewSegmentTemplate(segment, segFld, -1, false, viewMode);
	}
	
	private SegmentTemplate createNewSegmentTemplate(Segment segment, PresentationField segFld, int index, boolean selected) {
	    var schema = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();
		return createNewSegmentTemplate(segment, segFld, index, selected, schema.defaultViewMode());
	}
	
	private SegmentTemplate createNewSegmentTemplate(Segment segment, PresentationField segFld, int index, boolean selected,
			ViewMode viewMode) {
		template = new SegmentTemplate(segment, this, segmentPanel, segFld, viewMode);
		if (index < 0 || index >= templates.size()) {
            templates.add(template);
		} else {
			templates.add(index, template);
		}
		
		if (selected) {
			template.selectTemplate();
		}
		
		ArrayList<FieldWrapper> wrappers = segment
				.getFieldWrappersCollection().getWrappers();
		for (FieldWrapper wrapper : wrappers) {
			wrapper.getField().setSegmentID(segment.getID());
		}

		// remove SegmentMouseListener
		MouseListener[] mouseListeners = segmentPanel.getMouseListeners();
		for (int i = 0; i < mouseListeners.length; i++) {
			segmentPanel.removeMouseListener(mouseListeners[i]);
		}

		MouseMotionListener[] mouseMotionListeners = segmentPanel
				.getMouseMotionListeners();
		for (int j = 0; j < mouseMotionListeners.length; j++) {
			segmentPanel.removeMouseMotionListener(mouseMotionListeners[j]);
		}

		nonmanView = template.getNonmanView();
		customView = template.getCustomView();
		signView = template.getSignView();
		speechView = template.getSpeechView();
		
		SegmentGraphicRepresentation nonmanRep = nonmanView
				.getSegmentGraphicRep();
		
		SegmentGraphicRepresentation customRep = customView
				.getSegmentGraphicRep();
		
		SegmentGraphicRepresentation signRep = signView
				.getSegmentGraphicRep();
		
		SegmentGraphicRepresentation speechRep = speechView
				.getSegmentGraphicRep();
		
		repList.add(nonmanRep);
		repList.add(customRep);
		repList.add(signRep);
		repList.add(speechRep);

		//TODO: ADD CUSTOM REP/VIEW TO ALL LISTENERS?
		NonManualFieldsListener nmListener = new NonManualFieldsListener(
				segmentPanel, nonmanRep, signRep, speechRep, customRep, nonmanView.getLabelView(),
				signView.getLableView(), speechView.getLableView(), customView.getLabelView());
		nonmanRep.addMouseListener(nmListener);
		nonmanRep.addMouseMotionListener(nmListener);
		
		CustomFieldsListener customListener = new CustomFieldsListener(
				segmentPanel, nonmanRep, signRep, speechRep, customRep, nonmanView.getLabelView(),
				signView.getLableView(), speechView.getLableView(), customView.getLabelView());
		customRep.addMouseListener(customListener);
		customRep.addMouseMotionListener(customListener);

		GlossNRelatedFieldsListener glsListener = new GlossNRelatedFieldsListener(
				segmentPanel, nonmanRep, signRep, speechRep, customRep, nonmanView.getLabelView(),
				signView.getLableView(), speechView.getLableView(), customView.getLabelView());
		signRep.addMouseListener(glsListener);
		signRep.addMouseMotionListener(glsListener);
		
		GestureNRelatedFieldsListener speechListener = new GestureNRelatedFieldsListener(
				segmentPanel, nonmanRep, signRep, speechRep, customRep, nonmanView.getLabelView(),
				signView.getLableView(), speechView.getLableView(), customView.getLabelView());
		speechRep.addMouseListener(speechListener);
		speechRep.addMouseMotionListener(speechListener);

		return template;
	}

	// removes the specified segment if it exists and return the index it was removed from or -1 if it does not exist
	public int removeSegment(Segment segmentToRemove) {
		int indexOfSegmentToRemove = -1;
		ArrayList<Segment> segments = getSegments();	// GREG TODO: needs additional checking
		
		for (int i = 0; i < segments.size(); i++) {
			Segment segment = segments.get(i);
			if (segment.equals(segmentToRemove)) {
				indexOfSegmentToRemove = i;
			}
		}
		
		// editing (meaning, replacing) already existent segment
		if (indexOfSegmentToRemove != -1) {						
			segments.remove(indexOfSegmentToRemove);
			
			SegmentTemplate templateToRemove = getSegmentTemplate(segmentToRemove.getID());
			if (templateToRemove != null) {
				templates.remove(templateToRemove);
				centerPanel.remove(templateToRemove);
				
				//Change scroll bar model to not interfere with existing templates after removal.
				templateToRemove.getNonmanView().getJScrollPane().getHorizontalScrollBar().setModel(new DefaultBoundedRangeModel());
				templateToRemove.getCustomView().getJScrollPane().getHorizontalScrollBar().setModel(new DefaultBoundedRangeModel());
				templateToRemove.getSignView().getJScrollPane().getHorizontalScrollBar().setModel(new DefaultBoundedRangeModel());
				templateToRemove.getSpeechView().getJScrollPane().getHorizontalScrollBar().setModel(new DefaultBoundedRangeModel());
			}
			//remove toggling options for removed template
			SS3Singleton.getSignStreamApplication().removeTogglingOptions(segmentToRemove);
		}
		return indexOfSegmentToRemove;
	}
	
	public SearchSavedState saveCurrentState() {
		SS3Database database = segmentPanel.getSS3Database();
		var visibleOnsetTime = SegmentGraphReprUtil.visibleStart;
		double sliderPosition = segmentPanel.getSlider().getSliderPosition();
		double zoomValue = segmentPanel.getZoomer().zoom();
		String selectedUtteranceId = null;
		PresentationEventsEntity ent = this.getSelectedSegmentTemplate().getSegmentControlPanel().getSelectedUtterance();
		if (ent != null) {
			selectedUtteranceId = ent.getEvent().getID();
		}
		
		ArrayList<String> segmentsList = new ArrayList<String>();
		
		for (SegmentTemplate template: templates) {
			SegmentControlPanel scp = template.getSegmentControlPanel();
			
			String tpLabel = scp.getCurrentPartitionString();
			String stLabel = scp.getCurrentSegmentString();
			int index = scp.getCurrentSegmentString().indexOf(":"); 
			if (index != -1)
			 	stLabel = scp.getCurrentSegmentString().substring(index + 1);
			
			String combinedLabel = tpLabel + ";" + stLabel;
			segmentsList.add(combinedLabel);			
		}
		
		return new SearchSavedState(database, segmentsList, visibleOnsetTime, sliderPosition, zoomValue, getSelectedSegmentTemplateIndex(), selectedUtteranceId);
	}
	
	public void revertToSavedState(SearchSavedState searchSavedState) {
		// first get rid of everything
		centerPanel.removeAll();
		templates.clear();
		segments.clear();
		
		var visibleOnsetTime = searchSavedState.getVisibleOnsetTime();
		double zoomValue = searchSavedState.getZoomerValue();
//		SS3Singleton.getVideoControlManager().getZoomer().zoom(zoomValue);
		String selectedUtteranceId = searchSavedState.getSelectedUtteranceId();
		int selectedSegmentTemplateIndex = searchSavedState.getSelectedSegmentTemplateIndex();
		ArrayList<String> segmentsList = searchSavedState.getSegmentsList();

		for (String segmentString: segmentsList) {
			StringTokenizer tokSemicolon = new StringTokenizer(segmentString, ";");
			
			String partitionLabel = tokSemicolon.nextToken();
			String segmentLabel = tokSemicolon.nextToken();
			
			addUtterancePane(null, partitionLabel, segmentLabel);
		}
		
		
		SegmentTemplate t = this.getSegmentTemplates().get(selectedSegmentTemplateIndex);
		t.selectTemplate();
		t.getSegmentControlPanel().setSelectedUtterance(selectedUtteranceId);

		SS3Singleton.getMediaPlayer().seek(visibleOnsetTime);
	}
	
	public void parseSegmentStringAndReloadView(String str) {
		// first get rid of everything
		centerPanel.removeAll();
		templates.clear();
		segments.clear();

		StringTokenizer tokSemicolon = new StringTokenizer(str, ";");
		
		String partitionLabel = tokSemicolon.nextToken();
		String segmentLabel = tokSemicolon.nextToken();
		
		addUtterancePane(null, partitionLabel, segmentLabel);
		// template has just been set here

		String utteranceLabelAndGlosses = tokSemicolon.nextToken();
		StringTokenizer tokSpace = new StringTokenizer(utteranceLabelAndGlosses, " ");
		
		String utteranceLabel = tokSpace.nextToken();		
		
		ArrayList<PresentationEventsEntity> utterances = template.getSegment().getParentField().getEntities();
        for (final PresentationEventsEntity utterance : utterances) {
            SS3GlossWindowItem item = utterance.getEvent().getSS3GlossWindowItem();
            if (item.getLabel().equals(utteranceLabel)) {
                SS3Singleton.getMediaPlayer().seek(item.getStartTimeInfo());

                // need to invoke later to ensure that the pictureScrollPane is the correct size before actually scrolling
                SwingUtilities.invokeLater(new Runnable() {
                    @Override
                    public void run() {
                        segmentPanel.getUtteranceView().getSegmentTemplate().getSegmentControlPanel().setSelectedUtterance(utterance);
                    }
                });

                SS3Singleton.getMediaToolBar().enterUtterance(utterance);

                break;
            }
        }
	}
}

class SplitPaneListener implements MouseListener {
	
	private ArrayList<BasicSplitPaneDivider> components = new ArrayList<BasicSplitPaneDivider>();
	
	public void addSplitPane(JSplitPane sp) {
		BasicSplitPaneUI ui = (BasicSplitPaneUI) sp.getUI();
		BasicSplitPaneDivider divider = ui.getDivider();
		divider.addMouseListener(this);
		components.add(divider);
	}
	
	@Override
	public void mouseClicked(MouseEvent e) {
		//System.out.println("Mouse Click.");
	}

	@Override
	public void mousePressed(MouseEvent e) {
		//System.out.println("Mouse Press.");
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		Component divider = (Component) e.getSource();
		JSplitPane originalSP = (JSplitPane) divider.getParent();
		for (Component c : components) {
			if (c != e.getSource()) {
				JSplitPane newSP = (JSplitPane) c.getParent();
				newSP.setDividerLocation(originalSP.getDividerLocation());
			}
		}
	}

	@Override
	public void mouseEntered(MouseEvent e) {
		//System.out.println("Mouse Enter.");
	}

	@Override
	public void mouseExited(MouseEvent e) {
		//System.out.println("Mouse Exit.");
	}
}

