package edu.bu.signstream.grepresentation.view;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Optional;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSplitPane;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.vo.Palette;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Database;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolBar;
import edu.bu.signstream.time.TimeRange;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.ui.SS3SignStreamApplication;

public class UtteranceViewHeader extends JPanel {

	private static final long serialVersionUID = 1L;
	
	private SignStreamSegmentPanel segmentPanel;
	private UtteranceView utteranceView;
	
	private JPanel overviewWindowButtonPanel;
	private JButton overviewWindowButton;
	private TimelineView timelineView;
	
	private JSplitPane sp;
	
	public UtteranceViewHeader(SignStreamSegmentPanel segmentPanel,
			UtteranceView utteranceView) {
		this.segmentPanel = segmentPanel;
		this.utteranceView = utteranceView;	
		
		overviewWindowButtonPanel = new JPanel();
		overviewWindowButtonPanel.setAlignmentX(Component.CENTER_ALIGNMENT);
		overviewWindowButtonPanel.setBackground(Palette.BACKGROUND_COLOR);
		overviewWindowButtonPanel.setPreferredSize(new Dimension(
				SegmentGraphReprUtil.UTTERANCE_VIEW_LEFT_PANEL_WIDTH, 
				SegmentGraphReprUtil.UTTERANCE_VIEW_HEADER_FOOTER_HEIGHT));
		overviewWindowButtonPanel.setMinimumSize(new Dimension(
				SegmentGraphReprUtil.UTTERANCE_VIEW_LEFT_PANEL_WIDTH, 
				SegmentGraphReprUtil.UTTERANCE_VIEW_HEADER_FOOTER_HEIGHT));
		overviewWindowButtonPanel.setMaximumSize(new Dimension(
				SegmentGraphReprUtil.UTTERANCE_VIEW_LEFT_PANEL_WIDTH, 
				SegmentGraphReprUtil.UTTERANCE_VIEW_HEADER_FOOTER_HEIGHT));
		
		// button to return to overview window
		String overviewWindowButtonText = "Back to Database View";
		overviewWindowButton = new JButton(overviewWindowButtonText);
	    UserInterfaceUtil.setButtonsLNF(overviewWindowButton);
	    overviewWindowButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				boolean isSearchDialogVisible = SS3Singleton.getSignStreamApplication().isSearchDialogVisible();
				if (isSearchDialogVisible) {
					
					// dummy in order to get the message in front
					JFrame dummy = new JFrame();
				    dummy.setVisible(true);
				    dummy.setLocation(100, 100);
				    dummy.setAlwaysOnTop(true);
				    
					// then it's a default schema field -- make dialog to say: no, cannot delete
					String warningDialogText = "Close any open search windows before returning to Database View.";
					String warningTitleText = "Return to Database View";
					int optionType = JOptionPane.YES_NO_OPTION;
					int messageType = JOptionPane.WARNING_MESSAGE;
					String options[] = {"OK"};
					try {
						JOptionPane.showOptionDialog(dummy,
								warningDialogText,
								warningTitleText,
								optionType,
								messageType,
								null,
								options,
								options[0]);
					} catch (HeadlessException he) {
						System.out.println("");
					}
					dummy.dispose();
					
					return;
				}

                SS3Singleton.getSignStreamApplication().closeUtteranceViewDialogs();
                
                SS3Singleton.getSignStreamApplication().toggleViewMenu(false);
                SS3Singleton.getSignStreamApplication().toggleMacroUnitMenu(true);
                SS3Singleton.getSignStreamApplication().toggleSearchOption(false);
                
				loadDatabaseView();
			}	    	
	    });
	    overviewWindowButtonPanel.add(overviewWindowButton);
		
		// timeline view
		timelineView = new TimelineView(segmentPanel);
		timelineView.setPreferredSize(new Dimension(
				SegmentGraphReprUtil.UTTERANCE_VIEW_LABEL_PANEL_WIDTH + 
				SegmentGraphReprUtil.UTTERANCE_VIEW_GRAPH_PANEL_WIDTH, 
				SegmentGraphReprUtil.UTTERANCE_VIEW_HEADER_FOOTER_HEIGHT));
		
		overviewWindowButtonPanel.setAlignmentY(Component.TOP_ALIGNMENT);
		timelineView.setAlignmentY(Component.TOP_ALIGNMENT);

		sp = new JSplitPane();
		sp.setLeftComponent(overviewWindowButtonPanel);
		sp.setRightComponent(timelineView);
		
		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
	    add(sp);
	
	}
	
	public void loadDatabaseView() {
	    segmentPanel.loadDatabaseView();

        if (true) return;

//	    var pos = TimeRange.ABSOLUTE.time(
//	        utteranceView.pictureScrollPane.getViewport().getViewPosition().x,
//            TimeUnit.COORDINATE);
//	    segmentPanel.utteranceScroll(pos.as(TimeUnit.COORDINATE));
//	    segmentPanel.zoomLock(utteranceView.getHeader().getTimelineView().getUpperLeftCorner().locked());

//		utteranceView.getSegmentTemplates().forEach(template -> {
//			String temporalPartitionName = template.getSegment().getParentField().getTemporalPartition().getLabel();
//			String segmentTierName = template.getSegment().getParentField().getSS3GlossWindowSegment().getLabel();
//			segmentPanel.getViewOptionsMap().put(temporalPartitionName + ":" + segmentTierName, template.getViewOptions());
//		});

		// transplanted most code below from DbNavigationPanel
		
//        int width = segmentPanel.getDatabaseView().getVisibleWidth2();
//        segmentPanel.getZoomer().getCoordinateTimeConvertor().resize(width);

//        segmentPanel.databaseZoom()
//            .flatMap(z -> segmentPanel.zoomLocked() ? Optional.of(z) : Optional.empty())
//            .ifPresentOrElse(
//                zoom -> segmentPanel.zoom(zoom),
//                () -> segmentPanel.zoom(0));
//
//        segmentPanel.databaseScroll().ifPresentOrElse(
//            x -> segmentPanel.scroll(TimeRange.ABSOLUTE.time(Math.round(x), TimeUnit.COORDINATE)),
//            () -> {
//                var coord = SS3Singleton.getMediaPlayer().time().as(TimeUnit.COORDINATE);
//                segmentPanel.setVisible(coord, coord);
//            }
//        );

//        segmentPanel.getDatabaseView()
//            .getUpperLeft()
//            .lock(segmentPanel.zoomLocked());
//
//        segmentPanel.getUtteranceView().updateDrawingsAreaSize();
//        segmentPanel.getUtteranceView().getHeader().getTimelineView().getUpperLeftCorner().getZoomInOut().requestFocus();

		// // calculate coef that will allow users to see entire
		// // duration of a video file on a screen
//		double movieDuration = segmentPanel
//				.getMultipleMovieController().getMovieDuration();
//		double packetDuration = (int) segmentPanel
//				.getMultipleMovieController().getPacketDuration();
		
//		try {
//			MediaToolBar mToolBar = SS3Singleton.getMediaToolBar(null);
//			mToolBar.exitUtterance();
//		} catch(Exception ex) {
//			System.out.println(ex.getMessage());
//		}

		// No.
//		if (movieDuration == 0) {
//			movieDuration = db.getMovieDuration();
//			packetDuration = db.getPacketDuration();
//		}
//
//		if (movieDuration == 0) {
//			SS3GlossWindow glossWindow = db.getSS3GlossWindow();
//			ArrayList<SS3TemporalPartition> temporalPartitions = glossWindow
//					.getTemporalPartitions();
//			for (SS3TemporalPartition temporalPartition : temporalPartitions) {
//				SS3GlossWindowSegment segment = temporalPartition
//						.getGlossWindowSegment();
//				ArrayList<SS3GlossWindowItem> items = segment
//						.getSS3GlossWindowItems();
//				for (SS3GlossWindowItem item : items) {
//					int time = (int) item.getEndTimeInfo().getUnitValue();
//					if (time > movieDuration)
//						movieDuration = time;
//				}
//			}
//			packetDuration = 1001;
//			movieDuration = movieDuration + 20 * packetDuration;
//		}
//
//		segmentPanel.getZoomer()
//				.getCoordinateTimeConvertor()
//				.setPacketDuration(packetDuration);
//
//		double pixelsInTimeframe = db
//				.getTemporalPartitionPixelInTimeFrame();
//
//		int width = segmentPanel
//				.getDatabaseView().getVisibleWidth2();
//
//		if (pixelsInTimeframe <= 0) {
//			segmentPanel
//					.getZoomer()
//					.fillEntirePage(movieDuration, packetDuration,
//							width);
//		} else {
//			segmentPanel.getZoomer()
//					.getCoordinateTimeConvertor()
//					.setPixelInTimeFrame(pixelsInTimeframe);
//		}
//
//		double movieTime = SS3Singleton.getVideoControlManager()
//				.getMultipleMovieController().getCurrentTime();
//		segmentPanel.getSlider()
//				.setSliderPositionTime(movieTime);
//        segmentPanel.getZoomer().zoom(0);

//        var contr = SS3Singleton.getVideoControlManager().getMultipleMovieController();
//        segmentPanel.getZoomer().getCoordinateTimeConvertor().setTimeScale(contr.getPacketDuration(), contr.getMovieDuration());

//        segmentPanel.getSlider().setSliderPosition(SS3Singleton.getMediaPlayer().time());
	}	
	
	public TimelineView getTimelineView() {
		return timelineView;
    }

	public JSplitPane getSplitPane() {
		return sp;
	}
	
}
