package edu.bu.signstream.grepresentation.view;

import edu.bu.signstream.common.util.Pair;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationEventsEntity;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

public class UtteranceViewState {
    public final double zoom;
    public final double scroll;

    public final String selectedSegment;

    public final List<Pair<String, SegmentTemplateState>> segments;

    public UtteranceViewState(UtteranceView view) {
        this.zoom = view.getSignStreamSegmentPanel().getZoomer().zoom();
        this.scroll = view.pictureScrollPane.getViewport().getViewPosition().getX();

        this.selectedSegment = view.getSelectedSegmentTemplate()
            .getSegment()
            .getParentField()
            .getSS3GlossWindowSegment().getID();

        this.segments = view.getSegmentTemplates().stream()
            .map(t -> Pair.of(
                t.getSegment().getParentField().getSS3GlossWindowSegment().getID(),
                new SegmentTemplateState(t)
            ))
            .collect(Collectors.toList());
    }

    private UtteranceViewState(double zoom, double scroll, String selectedSegment,
                               List<Pair<String, SegmentTemplateState>> segments) {
        this.zoom = zoom;
        this.scroll = scroll;

        this.selectedSegment = selectedSegment;

        this.segments = segments;
    }

    public static UtteranceViewState from(PresentationField field, PresentationEventsEntity utterance, Optional<UtteranceViewState> old) {
        var zoom = utterance != null
            ? SS3Singleton.getVideoControlManager().getZoomer().getFit(utterance.interval().duration())
            : 0;

        if (SS3Singleton.getMediaToolBar().segmentPanel().zoomLocked() && old.isPresent())
            zoom = old.get().zoom;

        var scroll = utterance != null
            ? utterance.getStartTimeCoordinate()
            : 0;

        var segment = field.getSS3GlossWindowSegment();
        var selectedSegment = segment.getID();

        var viewMode = SS3Singleton.getMediaToolBar().segmentPanel()
            .getSegmentViewMode(selectedSegment);

        var schema = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();
        var segments = new ArrayList<Pair<String, SegmentTemplateState>>();
        segments.add(Pair.of(
            selectedSegment,
            new SegmentTemplateState(viewMode != null ? viewMode : schema.defaultViewMode(), utterance)));

        return new UtteranceViewState(zoom, scroll, selectedSegment, segments);
    }
}
