package edu.bu.signstream.grepresentation.view.mediaControl;

import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;

import javax.swing.*;

import edu.bu.signstream.common.util.Pair;
import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.GestureInfoDialog;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.MorphologicalPhonologicalInfoDialog;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationEventsEntity;
import edu.bu.signstream.grepresentation.view.LabelObject;
import edu.bu.signstream.time.Duration;
import edu.bu.signstream.time.TimeRange;
import edu.bu.signstream.grepresentation.fields.*;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.time.ui.TimeRangeEditDialog;
import edu.bu.signstream.ui.video.VideoControlManager;
import edu.bu.signstream.ui.video.VideoControlPanel;

public class MediaToolBar extends JToolBar {
    private final SignStreamSegmentPanel segmentPanel;
    public SignStreamSegmentPanel segmentPanel() { return segmentPanel; }

    private static final int TIME_COLS = 8;
    private static final int BUTTON_HEIGHT = 18;
    private static final Dimension LABEL_SZ = new Dimension(9999, 12);

//	private final JComboBox<String> annotatorSelectionComboBox;

    private final JButton playEvent = new JButton("►");
    private final JButton goEventStart = new JButton("|-");
    private final JButton goEventEnd = new JButton("-|");
    private final JButton playNextEvent = new JButton("►►");

    private final JButton startTimeForward = new JButton("+");
    private final JButton startTimeBack = new JButton("-");
    private final JTextField startTimeText = new JTextField(TIME_COLS);
    private final JLabel startTimeUnit = new JLabel("");
    private final JButton startTimeSet = new JButton("Set");

    private final Dimension TARGET_SZ = new Dimension(54, BUTTON_HEIGHT);
    private final String TARGET_TIME = "Green Bar";
    private final String TARGET_SELECT = "Selection";
    private final JButton targetToggle = new JButton(TARGET_TIME);

    private final JButton endTimeForward = new JButton("+");
    private final JButton endTimeBack = new JButton("-");
    private final JTextField endTimeText = new JTextField(TIME_COLS);
    private final JLabel endTimeUnit = new JLabel("");
    private final JButton endTimeSet = new JButton("Set");

    private final JButton playEntity = new JButton("►");
    private final JButton goEntityStart = new JButton("|-");
    private final JButton goEntityEnd = new JButton("-|");
    private final JButton playNextEntity = new JButton("►►");

    private final VideoControlPanel videoControlPanel;

    public static final int PLAY_RANGE_X = 24;
    public static final int PLAY_UTTERANCE_X = 15;
    public static final String PLAY_RANGE = "Range";
    public static final String PLAY_UTTERANCE = "Utterance";
    private final JButton playRangeUtterance = new JButton("►");
    private final JLabel playRangeUtteranceLabel = new JLabel(PLAY_RANGE);
    private final JButton fitRangeUtterance = new JButton("Fit");
    private final JButton goBeginningRangeUtterance = new JButton("|-");
    private final JButton goEndRangeUtterance = new JButton("-|");

    private static final Dimension TIME_RANGE_SZ = new Dimension(120, BUTTON_HEIGHT);
    private final JButton timeRangeCycle = new JButton();
    private final JButton timeRangeEdit = new JButton("Edit");

    private static final int STEP_COLS = 12;
    private final JButton stepCycle = new JButton("Step");
    private final JTextField stepText = new JTextField();
    private final JLabel stepUnit = new JLabel();

	private JPanel panel = new JPanel();
	private JPanel labels = new JPanel(null);

	// Current utterance
    private PresentationEventsEntity currentUtterance = null;

    private Field selectedField = null;
    private EventsEntity selectedEntity = null;
    private Event selectedEvent = null;

    private Field activeField = null;

    private LabelObject selectedLabel = null;

    // Target for set operations
    private boolean target = false;

    public MediaToolBar(final SignStreamSegmentPanel segmentPanel) {
		super("Media Tool Bar");
        setFloatable(false);

		this.segmentPanel = segmentPanel;

        // TODO JACK: This was never in the toolbar, add it?
        // this combobox must be populated with at least one item
//		String[] annotators = {"--Annotator--"};
//		annotatorSelectionComboBox = new JComboBox<>(annotators);
//		annotatorSelectionComboBox.setSelectedIndex(0);
//		addComboBoxComponent(annotatorSelectionComboBox, new AnnotatorSelectionListener(this), 1);

        var player = SS3Singleton.getMediaPlayer();

        panel.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0));

        // Event Controls
        addLabel(new JLabel("Event"), playEvent, 28);
        addButton(playEvent, new PlaySelectedEventListener(this), 0);
        playEvent.setToolTipText("Play Event");
        addButton(goEventStart, e -> {
            player.pause();
            player.seek(selectedEvent.getStartTimeInfo().in(Util.getCurrentRange()));
        }, 0);
        goEventStart.setToolTipText("Go to Event Start");
        addButton(goEventEnd, e -> {
            player.pause();
            player.seek(selectedEvent.getEndTimeInfo().in(Util.getCurrentRange()));
        }, 0);
        goEventEnd.setToolTipText("Go to Event End");
        addButton(playNextEvent, l -> {
            var start = selectedEvent.getStartTimeInfo();
            selectedField.getAllEntities().stream()
                .flatMap(e -> e.getEventsList()
                    .stream()
                    .map(ev -> Pair.of(e, ev)))
                .filter(p -> p.second.getStartTimeInfo().greaterThan(start))
                .min(Comparator.comparing(p -> p.second.getStartTimeInfo()))
                .ifPresent(p -> {
                	var origStartCoord = selectedEvent.getStartTimeCoordinate();
                    select(p.second, p.first);
                    var newStartCoord = selectedEvent.getStartTimeCoordinate();
                    // if e did not change (which would be the case if times were not equal) during the select, then don't play it
                    if (origStartCoord != newStartCoord)
                    	playEvent.doClick();
                });
        }, 3);
        playNextEvent.setToolTipText("Play Next Event");

        // Start time controls
        addLabel(new JLabel("Start"), startTimeBack, 4);
        addLabel(startTimeUnit, startTimeText, 4);
		addButton(startTimeBack, new StartFrameBackListener(this), 0);
        startTimeBack.setToolTipText("Decrease Start Time");
		addButton(startTimeForward, new StartFrameForwardListener( this), 0);
		startTimeForward.setToolTipText("Increase Start Time");
        addTextField(startTimeText, 0);
        addButton(startTimeSet, new SetStartTimeListener(this), 3);
        startTimeSet.setToolTipText("Set Start Time");

        // Time/Selection Toggle
//        addLabel(new JLabel("Set To"), targetToggle, 11);
//        addButton(targetToggle, e -> {
//            target = !target;
//            targetToggle.setText(!target ? TARGET_TIME : TARGET_SELECT);
//        }, 3);
        targetToggle.setMinimumSize(TARGET_SZ);
        targetToggle.setPreferredSize(TARGET_SZ);
        targetToggle.setMaximumSize(TARGET_SZ);

		// End time controls
        addLabel(new JLabel("End"), endTimeBack, 8);
        addLabel(endTimeUnit, endTimeText, 4);
        addButton(endTimeSet, new SetEndTimeListener(this), 0);
        endTimeSet.setToolTipText("Set End Time");
        addTextField(endTimeText, 0);
        addButton(endTimeBack, new EndFrameBackListener(this), 0);
        endTimeBack.setToolTipText("Decrease End Time");
        addButton(endTimeForward, new EndFrameForwardListener(this), 3);
        endTimeForward.setToolTipText("Increase End Time");

        // Entity Controls
        addLabel(new JLabel("Entity"), playEntity, 27);
        addButton(playEntity, new PlaySelectedEntityListener(this), 0);
        playEntity.setToolTipText("Play Entity");
        addButton(goEntityStart, e -> {
            player.pause();
            player.seek(selectedEntity.getStartTimeInfo().in(Util.getCurrentRange()));
        }, 0);
        goEntityStart.setToolTipText("Go to Entity Start");
        addButton(goEntityEnd, e -> {
            player.pause();
            player.seek(selectedEntity.getEndTimeInfo().in(Util.getCurrentRange()));
        }, 0);
        goEntityEnd.setToolTipText("Go to Entity End");
        addButton(playNextEntity, l -> {
            var start = selectedEntity.getStartTimeInfo();
            selectedField.getAllEntities().stream()
                .filter(e -> e.getStartTimeInfo().greaterThan(start))
                .min(Comparator.comparing(EventsEntity::getStartTimeInfo))
                .ifPresent(entity -> {
                	var origStartCoord = selectedEntity.getStartTimeCoordinate();
                    select(entity.getFirstEvent(), entity);
                    var newStartCoord = selectedEntity.getStartTimeCoordinate();
                    
                    // if selectedEntity did not change (which would be the case if times were not equal) during the select, then don't play it
                    if (origStartCoord != newStartCoord)
                    	playEntity.doClick();
                });
        }, 3);
        playNextEntity.setToolTipText("Play Next Entity");

        // Video controls
        // TODO JACK: Swap out with new controls
        VideoControlManager manager = SS3Singleton.getVideoControlManager();
        videoControlPanel = manager.createVideoControlPanel();
        addComponent(videoControlPanel, 3);

        // Range/Utterance controls
        addLabel(playRangeUtteranceLabel, playRangeUtterance, PLAY_RANGE_X);
        addButton(playRangeUtterance, e -> {
            player.pause();
            if (currentUtterance != null) {
                player.seek(currentUtterance.interval());
            } else {
                var range = Util.getCurrentRange();
                if (range == TimeRange.ABSOLUTE)
                    player.seek(player.interval());
                else
                    player.seek(range.interval());
            }
            player.play();
        }, 0);
		addButton(goBeginningRangeUtterance, e -> {
            player.pause();
            if (currentUtterance != null) {
                player.seek(currentUtterance.getStartTimeInfo());
            } else {
                player.seek(Util.getCurrentRange().start());
            }
        }, 0);
		addButton(goEndRangeUtterance, e -> {
            player.pause();
            if (segmentPanel.isUtteranceLoaded()) {
                player.seek(currentUtterance.getEndTimeInfo());
            } else {
                // Currently there's no robust tracking of the ABSOLUTE TimeRange's length, so we
                // just use the player duration instead.
                var range = Util.getCurrentRange();
                if (range == TimeRange.ABSOLUTE)
                    player.seek(player.end());
                else
                    player.seek(range.end());
            }
        }, 0);
		addButton(fitRangeUtterance, e -> {
            if (segmentPanel.isUtteranceLoaded() && currentUtterance != null) {
                segmentPanel.fitZoom(currentUtterance.interval().duration());
                segmentPanel.scroll(currentUtterance.getStartTimeInfo());
            } else {
                var range = Util.getCurrentRange();
                segmentPanel.fitZoom(range.duration());
                segmentPanel.scroll(range.start());
            }
        }, 3);

//		addLabel(new JLabel("Align To"), timeRangeCycle, 41);
//        addButton(timeRangeCycle, new TimeRangeCycleListener(this), 0);
//        addButton(timeRangeEdit, e -> new TimeRangeEditDialog(Util.getSelectedRange()).display(), 3);
        timeRangeEdit.setToolTipText("Edit Time Range");
        timeRangeCycle.setMinimumSize(TIME_RANGE_SZ);
        timeRangeCycle.setPreferredSize(TIME_RANGE_SZ);
        timeRangeCycle.setMaximumSize(TIME_RANGE_SZ);

//        addLabel(stepUnit, stepText, 4);
//        addButton(stepCycle, e -> {
//            Util.nextUnitStep();
//            update();
//        }, 0);
        stepCycle.setToolTipText("Change Step Duration");
//        addTextField(stepText, 3);
        stepText.addMouseListener(new MouseAdapter() {
            @Override
            public void mousePressed(MouseEvent e) {
                if (SwingUtilities.isLeftMouseButton(e))
                    Util.nextUnitStep();
                else
                    Util.prevUnitStep();
                update();
            }
        });
        stepText.setColumns(STEP_COLS);

        // For aligning labels.
        // Uncomment to add buttons to move around and change test string to determine X coordinates.
//        var test = new JLabel("Test");
//        addButton(new JButton("Set X"),
//            e -> test.setLocation(Integer.parseInt(JOptionPane.showInputDialog(null, "X", test.getX())), 0), 0);
//        addButton(new JButton("Set Text"),
//            e -> test.setText(JOptionPane.showInputDialog(null, "Text", test.getText())), 0);
//        addLabel(test, 10000);

        labels.setPreferredSize(LABEL_SZ);

        var container = new JPanel();
        container.setLayout(new BoxLayout(container, BoxLayout.Y_AXIS));
        container.add(panel);
        container.add(labels);
        container.add(videoControlPanel.getSlider());
        add(container);

        deselect();
	}

    public PresentationEventsEntity utterance() { return currentUtterance; }
    public void exitUtterance() {
        if (currentUtterance != null)
            select(currentUtterance.getEvent(), currentUtterance);

        SegmentGraphReprUtil.utteranceStart = null;
        SegmentGraphReprUtil.utteranceEnd = null;
        currentUtterance = null;
        update();
    }
    public void enterUtterance(PresentationEventsEntity utterance) {
        this.currentUtterance = utterance;
        SegmentGraphReprUtil.utteranceStart = utterance.getStartTimeInfo();
        SegmentGraphReprUtil.utteranceEnd = utterance.getEndTimeInfo();
        deselect();
    }

	public Field selectedField() { return selectedField; }
    public EventsEntity selectedEntity() { return selectedEntity; }
    public Event selectedEvent() { return selectedEvent; }

    public Field activeField() { return activeField; }

    public LabelObject selectedLabel() { return selectedLabel; }

    public void select(Event event, EventsEntity entity) {
        if (Arrays.stream(Window.getWindows())
            .anyMatch(w -> w.isVisible() &&
                (w instanceof MorphologicalPhonologicalInfoDialog ||
                 w instanceof GestureInfoDialog)))
            return;

        deselect();

        this.selectedEvent = event;
        this.selectedEntity = entity;
        this.selectedField = entity.getField();

        event.select();
        if (entity instanceof PresentationEventsEntity)
            entity.select();

        update();
    }
    public void deselect() {
        if (Arrays.stream(Window.getWindows())
            .anyMatch(w -> w.isVisible() &&
                (w instanceof MorphologicalPhonologicalInfoDialog ||
                 w instanceof GestureInfoDialog)))
            return;

        if (selectedEvent != null) {
            selectedEvent.unselect();
            selectedEntity.unselect();
        }

        this.selectedEvent = null;
        this.selectedEntity = null;
        this.selectedField = null;

        update();
    }

    public void selectLabel(LabelObject obj) {
        deselectLabel();

        this.selectedLabel = obj;

        if (selectedLabel.getWrapper() != null)
            selectedLabel.getWrapper().setSelectedNHighlighted(true);

        update();
        segmentPanel.repaint();
    }
    public void deselectLabel() {
        if (selectedLabel != null && selectedLabel.getWrapper() != null)
            selectedLabel.getWrapper().setSelectedNHighlighted(false);

        this.selectedLabel = null;

        update();
        var view = segmentPanel.getUtteranceView();
        if (view != null) {
            view.getNonmanualFieldsView().getLabelView().deselect();
            view.getCustomView().getLabelView().deselect();
            view.getGlossNRelatedView().getLableView().deselect();
            view.getSpeechView().getLableView().deselect();
        }
        segmentPanel.repaint();
    }

    public void activate(Field field) {
        this.activeField = field;

        update();
    }
    public void deactivate() {
        this.activeField = null;

        update();
    }

    public void update() {
        var selected = selectedEvent != null;
        var playing = SS3Singleton.getMediaPlayer().playing();
        var enabled = selected && !playing;

        playEvent.setEnabled(enabled);
        goEventStart.setEnabled(enabled);
        goEventEnd.setEnabled(enabled);
        playNextEvent.setEnabled(enabled);

        startTimeBack.setEnabled(enabled);
        startTimeForward.setEnabled(enabled);
        startTimeText.setEnabled(enabled);
        startTimeText.setText("");
        startTimeText.setToolTipText("");
        startTimeUnit.setText("");
        startTimeSet.setEnabled(enabled);

        endTimeBack.setEnabled(enabled);
        endTimeForward.setEnabled(enabled);
        endTimeText.setEnabled(enabled);
        endTimeText.setText("");
        endTimeText.setToolTipText("");
        endTimeUnit.setText("");
        endTimeSet.setEnabled(enabled);

        playEntity.setEnabled(enabled);
        goEntityStart.setEnabled(enabled);
        goEntityEnd.setEnabled(enabled);
        playNextEntity.setEnabled(enabled);

        	// utterance buttons do not need something selected in order to be enabled, only that media is not currently playing
        playRangeUtterance.setEnabled(!playing);
        goBeginningRangeUtterance.setEnabled(!playing);
        goEndRangeUtterance.setEnabled(!playing);
        fitRangeUtterance.setEnabled(!playing);

        if (selected) {
            var start = selectedEvent.getStartTimeInfo();
            if (start != null) {
                startTimeText.setText("" + start.value());
                startTimeText.setToolTipText(String.format("%,.2fs Absolute", start.absolute().as(TimeUnit.SECOND)));
                startTimeUnit.setText(start.unit().label());
            }

            var end = selectedEvent.getEndTimeInfo();
            if (end != null) {
                endTimeText.setText("" + end.value());
                endTimeText.setToolTipText(String.format("%,.2fs Absolute", end.absolute().as(TimeUnit.SECOND)));
                endTimeUnit.setText(end.unit().label());
            }
        }

        var rangeUtterance = currentUtterance == null ? PLAY_RANGE : PLAY_UTTERANCE;
        playRangeUtteranceLabel.setText(rangeUtterance);
        playRangeUtterance.setToolTipText("Play " + rangeUtterance);
        goBeginningRangeUtterance.setToolTipText("Go to " + rangeUtterance + " Start");
        goEndRangeUtterance.setToolTipText("Go to " + rangeUtterance + " End");
        fitRangeUtterance.setToolTipText("Fit Display to " + rangeUtterance);
        playRangeUtteranceLabel.setLocation(playRangeUtterance.getX()
            + (currentUtterance == null ? PLAY_RANGE_X : PLAY_UTTERANCE_X), 0);

        var range = Util.getSelectedRange();
        timeRangeEdit.setEnabled(range != null);

        if (range != null)
            timeRangeCycle.setText(range.name());
        else if (selectedEvent != null && selectedEvent.getEndTimeInfo() != null)
            timeRangeCycle.setText("Auto (" + selectedEvent.getEndTimeInfo().range().name() + ")");
        else if (activeField != null)
            timeRangeCycle.setText("Auto (" + activeField.timeRange().name() + ")");
        else
            timeRangeCycle.setText("Auto (" + SS3Singleton.getVideoTimeRange().name() + ")");

        var step = Util.getUnitStep();
        if (step != Util.UnitStep.DEFAULT) {
            stepText.setText(String.format("%,d", Util.getUnitStep().value));
            stepUnit.setText(Util.getStepRange().unit().label());
        } else if (selectedEvent != null && selectedEvent.getStartTimeInfo() != null) {
            var erange = selectedEvent.getStartTimeInfo().range();
            stepText.setText("Default (" + String.format("%,d", erange.step()) + ")");
            stepText.setToolTipText(String.format("%,.6fs",
                new Duration(erange.step(), erange.unit())
                    .as(TimeUnit.SECOND)));
            stepUnit.setText(erange.unit().label());
        } else if (range != null) {
            stepText.setText("Default (" + String.format("%,d", range.step()) + ")");
            stepText.setToolTipText(String.format("%,.6fs",
                new Duration(range.step(), range.unit())
                    .as(TimeUnit.SECOND)));
            stepUnit.setText(range.unit().label());
        } else if (activeField != null) {
            stepText.setText("Default (" + String.format("%,d", activeField.timeRange().step()) + ")");
            stepText.setToolTipText(String.format("%,.6fs",
                new Duration(activeField.timeRange().step(), activeField.timeRange().unit())
                    .as(TimeUnit.SECOND)));
            stepUnit.setText(activeField.timeRange().unit().label());
        } else {
            var def = SS3Singleton.getVideoTimeRange();
            stepText.setText("Default (" + String.format("%,d", def.step()) + ")");
            stepText.setToolTipText(String.format("%,.6fs",
                new Duration(def.step(), def.unit())
                    .as(TimeUnit.SECOND)));
            stepUnit.setText(SS3Singleton.getVideoTimeRange().unit().label());
        }

        var uView = segmentPanel.getUtteranceView();
        if (uView != null)
            uView.getNonmanualFieldsView().getLabelView().getButtonPanel().setHideEnabled(selectedLabel != null);
    }

    public void reset() {
        Util.setSelectedRange(null);
        Util.setUnitStep(Util.UnitStep.DEFAULT);
        deselect();
    }

    private void addLabel(JLabel label, JComponent parent, int offset) {
        labels.add(label);

        var font = label.getFont();
        label.setFont(new Font(font.getName(), Font.PLAIN, 10));

        label.setSize(label.getPreferredSize());
        label.addPropertyChangeListener("text", e ->
            label.setSize(label.getPreferredSize()));

        parent.addComponentListener(new ComponentAdapter() {
            @Override
            public void componentMoved(ComponentEvent e) {
                label.setLocation(parent.getX() + offset, 0);
            }
        });
    }

    /**
	 * Sets the look and feel of the button, attaches an ActionListener to it,
	 * and then adds the button to the JPanel specified by MediaToolBar.panel.
	 * @param button The JButton to add to this panel.
	 * @param listener The ActionListener to attach to the button.
	 * @param followingBlanks The number of blank whitespace characters to add
	 * 			after the button for use as padding.
	 */
	private void addButton(JButton button, ActionListener listener,
                           int followingBlanks) {
		UserInterfaceUtil.setButtonsLNF(button);
		button.setFocusable(false);
		button.addActionListener(listener);
		addComponent(button, followingBlanks);
	}

	private void addTextField(JTextField txtField, int followingBlanks) {
		Dimension dimension = new Dimension(70, 20);
		txtField.setMaximumSize(dimension);
		txtField.setEditable(false);
		txtField.setFocusable(false);
		addComponent(txtField, followingBlanks);
	}
	
	private void addComboBox(JComboBox combobox, ActionListener listener,
                             int followingBlanks) {
		UserInterfaceUtil.setComboBoxLNF(combobox);
		combobox.setFocusable(false);
		combobox.addActionListener(listener);
		addComponent(combobox, followingBlanks);
	}

	/**
	 * Adds a component to the panel specified by MediaToolBar.panel.
	 * @param component The component to be added to the panel specified by
	 * 			MediaToolBar.panel.
	 * @param followingBlanks The number of blank whitespace characters to add
	 * 			after the component for use as padding.
	 */
	private void addComponent(JComponent component, int followingBlanks) {
		panel.add(component);
		for (int i = 0; i < followingBlanks; i++) {
			panel.add(new JLabel(" "));
		}
	}

	@Deprecated
    // JACK: Shimmed for now, do not use. Use select() and deselect() instead.
	public void selectEvent(Event eventToSelect, EventsEntity entityToSelect,
			Field fieldToSelect) {
	    if (eventToSelect == null && entityToSelect == null && fieldToSelect == null) {
            deselect();
            return;
        }

	    if (eventToSelect == null && entityToSelect == null)
            throw new IllegalArgumentException("Use select() with {Gesture}TextEventsEntity instead of passing nulls here.");

	    if (eventToSelect == null || entityToSelect == null)
	        throw new IllegalArgumentException("Use deselet() instead of passing nulls here.");

	    select(eventToSelect, entityToSelect);
	}

    public void updateAnnotatorList(ArrayList<String> newAnnotatorList, String currentlySelectedAnnotator) {
//		annotatorSelectionComboBox.removeAllItems();
//
//		Collections.sort(newAnnotatorList);
//
//		for (String s: newAnnotatorList) {
//			annotatorSelectionComboBox.addItem(s);
//		}
//
//		if (currentlySelectedAnnotator != null) {
//			if (newAnnotatorList.contains(currentlySelectedAnnotator)) {
//				annotatorSelectionComboBox.setSelectedItem(currentlySelectedAnnotator);
//			} else if (annotatorSelectionComboBox.getItemCount() > 0) {
//				annotatorSelectionComboBox.setSelectedIndex(0);
//				SS3Singleton.setCurrentAnnotator((String) annotatorSelectionComboBox.getItemAt(0));
//			}
//		} else if (annotatorSelectionComboBox.getItemCount() > 0) {
//			annotatorSelectionComboBox.setSelectedIndex(0);
//		}
	}
	
//	public void addToAnnotatorList(String newAnnotator) {
//		annotatorSelectionComboBox.addItem(newAnnotator);
//	}
}
