package edu.bu.signstream.grepresentation.view.mediaControl;

import java.awt.FontMetrics;
import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;

import edu.bu.signstream.common.util.CoordinateTimeConverter;
import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.time.Bound;
import edu.bu.signstream.time.Duration;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.time.TimeRange;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.apexField.ApexChainedApexEventsEntity;
import edu.bu.signstream.grepresentation.fields.apexField.ApexField;
import edu.bu.signstream.grepresentation.fields.apexField.ChainedApexEventsEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureEnteredText;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureChainedEvent;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureEventsEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureField;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureText;
import edu.bu.signstream.grepresentation.fields.gestureField.domNdomField.GestureDomNdomHandField;
import edu.bu.signstream.grepresentation.fields.gestureField.handshapes.GestureHandShapeEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.locationField.GestureLocationEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.signTypeField.GestureTypeEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.orientationField.GestureOrientationEntity;
import edu.bu.signstream.grepresentation.fields.glossField.EnteredText;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEvent;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.glossField.GlossField;
import edu.bu.signstream.grepresentation.fields.glossField.Text;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeEntity;
import edu.bu.signstream.grepresentation.fields.locationField.LocationEntity;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEvent;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.nonManualField.NonManualField;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationEvent;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationEventsEntity;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.fields.signTypeField.SignTypeEntity;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.time.TimeUnit;

public class MediaToolbarCommons {

	public static Logger logger = Logger.getLogger("edu.bu.signstream.ui");
	
	public static void setHandshapeStartTime(HandShapeEntity entity, Time timeInfo) {
		Time start = timeInfo;
		Time end = entity.getEndTimeInfo();
		
		entity.resetTimes(start, end);
	}

	public static void setHandshapeEndTime(HandShapeEntity entity, Time timeInfo) {
		Time start = entity.getStartTimeInfo();
		Time end = timeInfo;
		
		entity.resetTimes(start, end);
	}
	
	public static void setGestureHandshapeStartTime(GestureHandShapeEntity entity, Time timeInfo) {
		Time start = timeInfo;
		Time end = entity.getEndTimeInfo();
		
		entity.resetTimes(start, end);
	}

	public static void setGestureHandshapeEndTime(GestureHandShapeEntity entity, Time timeInfo) {
		Time start = entity.getStartTimeInfo();
		Time end = timeInfo;
		
		entity.resetTimes(start, end);
	}
	
	public static void setGestureOrientationStartTime(GestureOrientationEntity entity, Time timeInfo) {
		Time start = timeInfo;
		Time end = entity.getEndTimeInfo();
		
		entity.resetTimes(start, end);		
	}

	public static void setGestureOrientationEndTime(GestureOrientationEntity entity, Time timeInfo) {
		Time start = entity.getStartTimeInfo();
		Time end = timeInfo;
		
		entity.resetTimes(start, end);
	}

	public static void repaintPanel(Time timeInfo, MediaToolBar bar) {
		bar.segmentPanel().getSlider().setSliderPositionTime(timeInfo.doubleValue());
		bar.segmentPanel().repaint();
	}

	public static boolean isUpdatablePresentationEntity(EventsEntity entity) {
		PresentationEventsEntity pEntity = (PresentationEventsEntity) entity;
		return !pEntity.getDependentEntities().isEmpty();
	}

	public static void createNewGlossEvent(EnteredText enteredText,
                                           Time currentTimeInfo, MediaToolBar bar) {

		ArrayList<Event> event = enteredText.createGlossChainedEvent(
				currentTimeInfo, currentTimeInfo);

		if (!event.isEmpty()) {
		    bar.select(event.get(0), (ChainedEventsEntity)event.get(1));
			bar.segmentPanel().setVisible(event.get(0));
		}
	}
	
	public static void createNewGestureEvent(GestureEnteredText enteredText,
                                             Time timeInfo, MediaToolBar bar) {
		ArrayList<Event> event = enteredText.createGestureChainedEvent(
				timeInfo, timeInfo);

		if (!event.isEmpty()) {
            bar.select(event.get(0), (ChainedEventsEntity)event.get(1));
			bar.segmentPanel().setVisible(event.get(0));
		}
	}

	// GD: unused as of 2021-07-30
	@Deprecated
	protected static void setEventEndTime(Time timeInfo, Event event,
                                          MediaToolBar bar) {

//		if (utterance && parentEntity != null) {
//			if (timeInfo.lessThan(parentEntity.getStartTimeInfo())) {
//				SS3Singleton.getErrorMessages().showErrorMessage(
//						ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE);
//				return;
//			} else if (timeInfo.greaterThan(parentEntity.getEndTimeInfo())) {
//				SS3Singleton.getErrorMessages().showErrorMessage(
//						ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE);
//				return;
//			}
//		}

		MediaToolbarCommons.setSelectedObjectEndTime(bar, timeInfo);
	}

	private static boolean setSelectedObjectEndTime(MediaToolBar bar, Time timeInfo) {
		Field field = bar.selectedField();

		EventsEntity selectedEntity = bar.selectedEntity();
//		EventsEntity parentEntity = selectedEntity.getParentEntity();

		EventsEntity entity = bar.selectedEntity();

		if (entity instanceof PresentationEventsEntity) {

//			PresentationEventsEntity pEntity = (PresentationEventsEntity) entity;
//			MediaToolbarCommons.setPresentationEventsEntityEndTime(timeInfo,
//					pEntity);
			return false;

		} else if (field instanceof GlossField) {
			
			return MediaToolbarCommons.setGlossChainedEventsEntityEntityEndTime(timeInfo,
					bar);

		} else if (field instanceof GestureField) {
			
			return MediaToolbarCommons.setGestureEventsEntityEntityEndTime(timeInfo,
					bar);

		} else if (field instanceof NonManualField) {
			NonManualField nmField = (NonManualField) field;
			return MediaToolbarCommons.setNonManualEntityEndTime(timeInfo, 
					nmField);
		} else if (field instanceof GestureDomNdomHandField) {

//			if (selectedEntity != null && parentEntity != null) {
//				if (parentEntity.getStartTimeInfo().greaterThan(timeInfo)
//						|| parentEntity.getEndTimeInfo().lessThan(timeInfo)) {
//					SS3Singleton.getErrorMessages().showErrorMessage(
//							ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE);
//					return;
//				}
//			}

			GestureChainedEventsEntity gestureHandEntity = (GestureChainedEventsEntity) selectedEntity;

			String id = gestureHandEntity.getField().getFieldID();
			if (id.equalsIgnoreCase(Util.DOM_GESTURE_HAND_FIELD_ID) 
					|| id.equalsIgnoreCase(Util.NDOM_GESTURE_HAND_FIELD_ID)) {
				if (gestureHandEntity.isLockedGesturePhonAlignments()) {
					SS3Singleton.getErrorMessages().showErrorMessage(
							ErrorMessages.UNLOCK_GESTURE_PHON_ALIGNMENTS);
					return false;
				}
			}

			String resp = gestureHandEntity.canSetSelectedEventEndMovieTime(timeInfo);

			if (resp != null) {
				SS3Singleton.getErrorMessages().showErrorMessage(resp);
				return false;
			}
			
			gestureHandEntity.setEndTimeInfo(timeInfo);
		}  else if (field instanceof ApexField) {
			ApexField nmField = (ApexField) field;
			return MediaToolbarCommons.setApexEntityEndTime(timeInfo, 
					nmField);
		}

		return true;
	}

	// GD: 2021-04-01 ok to deprecate?
	@Deprecated
    protected static void setPresentationEventsEntityStartTime(Time timeInfo,
                                                             PresentationEventsEntity entity) {
        PresentationField pField = (PresentationField) entity.getField();

        if (timeInfo.greaterThanOrEq(entity.getEndTimeInfo())) {
            SS3Singleton.getErrorMessages().showErrorMessage(
                ErrorMessages.EVENT_END_TIME_FOLLOW_START_TIME);
            return;
        } else if (timeInfo.greaterThan(entity.getStartTimeInfo())) {
            // update
            entity.setStartTimeInfo(timeInfo);
        } else {
            EventsEntity npEntity = pField.getPreviousEntity(entity.getStartTimeInfo());
            if (npEntity == null) {
                // update
                entity.setStartTimeInfo(timeInfo);
            } else if (npEntity.getEndTimeInfo().lessThan(timeInfo)) {
                // update
                entity.setStartTimeInfo(timeInfo);
            } else {
                SS3Singleton.getErrorMessages().showErrorMessage(
                    ErrorMessages.CHAINS_CANNOT_OVERLAP);
                return;
            }
        }

        SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
    }

	// GD: 2021-04-01 ok to deprecate?
	@Deprecated
	protected static void setPresentationEventsEntityEndTime(Time timeInfo,
                                                             PresentationEventsEntity entity) {
		PresentationField pField = (PresentationField) entity.getField();

		if (timeInfo.lessThanOrEq(entity.getStartTimeInfo())) {
			SS3Singleton.getErrorMessages().showErrorMessage(
					ErrorMessages.EVENT_END_TIME_FOLLOW_START_TIME);
			return;
		} else if (timeInfo.lessThan(entity.getEndTimeInfo())) {
			// update
			entity.setEndTimeInfo(timeInfo);
		} else {
			EventsEntity npEntity = pField.getNextEntity(entity.getEndTimeInfo());
			if (npEntity == null) {
				// update
				entity.setEndTimeInfo(timeInfo);
			} else if (npEntity.getStartTimeInfo().greaterThan(timeInfo)) {
				// update
				entity.setEndTimeInfo(timeInfo);
			} else {
				SS3Singleton.getErrorMessages().showErrorMessage(
						ErrorMessages.CHAINS_CANNOT_OVERLAP);
				return;
			}
		}

		SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
	}

	public static void setTextStartEndTime(GlossField field,
                                           GlossChainedEventsEntity entity,
                                           Time timeInfo, double coord, MediaToolBar bar) {
		
		EnteredText enteredText = field.getGlossChainedEventsEntityCollection()
				.getEnteredText();

		ArrayList<Text> selectedText = enteredText.getSelectedText();

		if (selectedText.isEmpty()) {
			return;
		}

		field.deleteEmptyEntities();

		ArrayList<EventsEntity> entities = field.getEntities();

		for (EventsEntity ent : entities) {
			if (ent.interval().contains(timeInfo, Bound.EXCLUSIVE)) {
				SS3Singleton.getErrorMessages().showErrorMessage(
						ErrorMessages.CHAINS_CANNOT_OVERLAP);
				return;
			}
		}

		// no intersections, so create the gloss
		MediaToolbarCommons.createNewGlossEvent(enteredText, timeInfo, bar);

		if (entity == null)
		{
			try {
				entity = (GlossChainedEventsEntity)bar.selectedEntity(); // this cast is okay - it could not be any other type
			}
			catch (Exception e)
			{
				//					Logger.logMsg(Logger.INFO, "Error caught in attempting to cast the newly created EventsEntity into a GlossChainedEventsEntity");
				logger.log(Level.INFO, "Error caught in attempting to cast the newly created EventsEntity into a GlossChainedEventsEntity");
			}
		}
		
		SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
		bar.selectedEntity().updateParentEntity();
	}
	
	public static void setTextStartEndTime(GestureField field,
                                           GestureEventsEntity entity, 
                                           Time timeInfo, double coord, MediaToolBar bar) {
		
		GestureEnteredText enteredText = field.getGestureEventsEntityCollection()
				.getEnteredText();

		ArrayList<GestureText> selectedText = enteredText.getSelectedText();

		if (selectedText.isEmpty()) {
			return;
		}

		GestureText selectedText1 = selectedText.get(0);
		GestureText selectedText2 = selectedText.get(selectedText.size() - 1);

		if (selectedText1.getStartCoord() <= coord
				&& coord <= selectedText2.getEndCoord()) {
			MediaToolbarCommons.createNewGestureEvent(enteredText, timeInfo, bar);
		} else {
			field.deleteEmptyEntities();

			if (coord < selectedText1.getStartCoord()) {

//				if (timeInfo.lessThan(parentEntity.getStartTimeInfo())) {
//					SS3Singleton.getErrorMessages().showErrorMessage(
//							ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE);
//					return;
//				}

				ArrayList<EventsEntity> entities = field.getEntities();

				for (EventsEntity ent : entities) {

					double endCoord = ent.getEndTimeCoordinate();
					if (coord < endCoord
							&& endCoord < selectedText1.getStartCoord()
							&& entity != ent) {

						SS3Singleton.getErrorMessages().showErrorMessage(
								ErrorMessages.CHAINS_CANNOT_OVERLAP);
						return;
					}
					
					double startCoord = ent.getStartTimeCoordinate();
					if ( (coord == startCoord) && (coord == endCoord)) {
						SS3Singleton.getErrorMessages().showErrorMessage(
								ErrorMessages.SET_ONE_FRAMES_CANNOT_OVERLAP);
						return;
					}
				}

				MediaToolbarCommons.createNewGestureEvent(enteredText, timeInfo, bar);
				
				if (entity == null)
				{
					try {
						entity = (GestureEventsEntity)bar.selectedEntity(); // this cast is okay - it could not be any other type
					}
					catch (Exception e)
					{
//						Logger.logMsg(Logger.INFO, "Error caught in attempting to cast the newly created EventsEntity into a GlossChainedEventsEntity");
						logger.log(Level.INFO, "Error caught in attempting to cast the newly created EventsEntity into a GlossChainedEventsEntity");
					}
				}

				GestureEventsEntity prevEntity = entity
						.getPreviousEntity();

				double prevEndCoord = 0;

				if (prevEntity != null) {
					prevEndCoord = prevEntity.getEndTimeCoordinate();
				}

				ArrayList<GestureText> textInInterval = enteredText
						.getSortedEnteredTextInInteval2(prevEndCoord,
								selectedText1.getStartCoord());

				FontMetrics fontMetrics = bar
						.getFontMetrics(SegmentGraphReprUtil.getGraphReprFont());

				if (!textInInterval.isEmpty()) {
					double prevStartCoord = entity.getStartTimeCoordinate();
					String txt1 = entity.getText();
					int width1 = fontMetrics.stringWidth(txt1);
					prevStartCoord = prevStartCoord - width1 / 2;

					int commulativeWidht = 0;
					int dCommulativeWidht = Util.MTB_TEXT_BUFFER;

					int textSize = textInInterval.size();

					for (int i = textSize - 1; i >= 0; i--) {
						GestureText t = textInInterval.get(i);
						String txt = t.getText();

						int width = fontMetrics.stringWidth(txt);
						prevStartCoord = prevStartCoord - Util.MTB_TEXT_BUFFER - width;
						t.setStartCoord(prevStartCoord);

						commulativeWidht = commulativeWidht + width;
						dCommulativeWidht = dCommulativeWidht + Util.MTB_TEXT_BUFFER;
					}

					dCommulativeWidht = dCommulativeWidht + Util.MTB_TEXT_BUFFER;

					double availableWidht = entity.getStartTimeCoordinate()
							- prevEndCoord - dCommulativeWidht;

					int requestedWidht = commulativeWidht;

					if (availableWidht < requestedWidht) {

						if (availableWidht <= 0) {

							for (GestureText t : textInInterval) {
								double c1 = entity.getStartTimeCoordinate()
										- prevEndCoord;
								double c2 = c1 / 2;
								double c3 = prevEndCoord + c2;
								t.setStartCoord(c3);
							}

							return;
						}

						double d1 = availableWidht;
						double d2 = requestedWidht;
						double d = d1 / d2;

						for (GestureText t : textInInterval) {
							String txt = t.getText();

							int txtWidth = (fontMetrics.stringWidth(txt));

							double w = txtWidth * d;
							int width = (int) w;

							prevEndCoord = prevEndCoord + Util.MTB_TEXT_BUFFER;

							t.setStartCoord(prevEndCoord);
							prevEndCoord = prevEndCoord + width;
						}
					}

					return;
				}
			} else if (coord > selectedText2.getEndCoord()) {

//				if (timeInfo.greaterThan(parentEntity.getEndTimeInfo())) {
//					SS3Singleton.getErrorMessages().showErrorMessage(
//							ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE);
//					return;
//				}

				ArrayList<EventsEntity> entities = field.getEntities();

				for (EventsEntity ent : entities) {

					double startCoord = ent.getStartTimeCoordinate();
					if (coord > startCoord
							&& startCoord > selectedText2.getEndCoord()
							&& entity != ent) {

						SS3Singleton.getErrorMessages().showErrorMessage(
								ErrorMessages.CHAINS_CANNOT_OVERLAP);
						return;
					}
					
					double endCoord = ent.getEndTimeCoordinate();
					if ( (coord == startCoord) && (coord == endCoord)) {
						SS3Singleton.getErrorMessages().showErrorMessage(
								ErrorMessages.SET_ONE_FRAMES_CANNOT_OVERLAP);
						return;
					}
				}

				MediaToolbarCommons.createNewGestureEvent(enteredText, timeInfo, bar);
				
				if (entity == null)
				{
					try {
						entity = (GestureEventsEntity)bar.selectedEntity(); // this cast is okay - it could not be any other type
					}
					catch (Exception e)
					{
						logger.log(Level.INFO, "Error caught in attempting to cast the newly created EventsEntity into a GlossChainedEventsEntity");
					}
				}
				
				GestureEventsEntity nextEntity = entity.getNextEntity();

//				double nextStartCoord = parentEntity.getEndTimeCoordinate();
				double nextStartCoord = TimeRange.ABSOLUTE.end().value();		// GREG TODO: not sure if valid

				if (nextEntity != null) {
					nextStartCoord = nextEntity.getStartTimeCoordinate();
				}

				ArrayList<GestureText> textInInterval = enteredText
						.getSortedEnteredTextInInteval2(
								selectedText2.getEndCoord(), nextStartCoord);

				if (!textInInterval.isEmpty()) {
					FontMetrics fontMetrics = bar
							.getFontMetrics(SegmentGraphReprUtil
									.getGraphReprFont());

					double prevEndCoord = entity.getEndTimeCoordinate();

					String txt1 = entity.getText();
					int width1 = fontMetrics.stringWidth(txt1);
					prevEndCoord = prevEndCoord + width1 / 2;

					int commulativeWidht = 0;
					int dCommulativeWidht = Util.MTB_TEXT_BUFFER;

					for (GestureText t : textInInterval) {
						String txt = t.getText();
						int width = fontMetrics.stringWidth(txt);
						prevEndCoord = prevEndCoord + Util.MTB_TEXT_BUFFER;
						t.setStartCoord(prevEndCoord);

						prevEndCoord = prevEndCoord + width;

						commulativeWidht = commulativeWidht + width;
						dCommulativeWidht = dCommulativeWidht + Util.MTB_TEXT_BUFFER;
					}

					dCommulativeWidht = dCommulativeWidht + Util.MTB_TEXT_BUFFER;

					double startMoveInInterval = prevEndCoord;
					double endMoveInInterval = entity.getEndTimeCoordinate();
					double textWidht = commulativeWidht + dCommulativeWidht;
					double dWidht = endMoveInInterval - startMoveInInterval;

					if (dWidht < textWidht) {
						;
					}
				}
			}
		}
		SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
		bar.selectedEntity().updateParentEntity();
	}

	protected static boolean setNonManualEntityEndTime(Time timeInfo,
                                                    NonManualField field) {

//		if (parentEntity != null) {
//			if (parentEntity.getStartTimeInfo().greaterThan(timeInfo)
//					|| parentEntity.getEndTimeInfo().lessThan(timeInfo)) {
//				SS3Singleton.getErrorMessages().showErrorMessage(
//						ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE);
//				return;
//			}
//		}

		ChainedEventsEntity selectedEntity = (ChainedEventsEntity) field
				.getSelectedEntity();

		String id = selectedEntity.getField().getFieldID();
		if (id.equalsIgnoreCase(Util.DOM_HAND_FIELD_ID) || id.equalsIgnoreCase(Util.NDOM_HAND_FIELD_ID)) {
			if (selectedEntity.isLockedGlossPhonAlignments()) {
				SS3Singleton.getErrorMessages().showErrorMessage(
						ErrorMessages.UNLOCK_GLOSS_PHON_ALIGNMENTS);
				return false;
			}
		}

		String resp = selectedEntity.canSetSelectedEventEndMovieTime(timeInfo);

		if (resp != null) {
			SS3Singleton.getErrorMessages().showErrorMessage(resp);
			return false;
		}
		
		selectedEntity.setEndTimeInfo(timeInfo);

		return true;
	}
	
	protected static boolean setApexEntityEndTime(Time timeInfo,
			ApexField field) {

//		if (parentEntity != null) {
//			if (parentEntity.getStartTimeInfo().greaterThan(timeInfo)
//					|| parentEntity.getEndTimeInfo().lessThan(timeInfo)) {
//				SS3Singleton.getErrorMessages().showErrorMessage(
//						ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE);
//				return;
//			}
//		}

		ChainedApexEventsEntity selectedEntity = (ChainedApexEventsEntity) field
				.getSelectedEntity();

		String resp = selectedEntity.setSelectedEventEndMovieTime(timeInfo);

		if (resp != null) {
			SS3Singleton.getErrorMessages().showErrorMessage(resp);
			return false;
		}

		return true;
	}

	protected static boolean setGlossChainedEventsEntityEntityEndTime(Time timeInfo,
                                                                   MediaToolBar bar) {
		Field field = bar.selectedField();
		Event event = bar.selectedEvent();
		EventsEntity entity = bar.selectedEntity();

//		EventsEntity selectedEntity = bar.getSelectedEventsEntity();		
//		EventsEntity parentEntity = selectedEntity.getParentEntity();

		GlossField glsField = (GlossField) field;

		if (entity == null && event != null) {
			entity = glsField.getEntityAt(event.getStartTimeCoordinate());
		}

//		if (parentEntity != null) {
//			if (parentEntity.getStartTimeInfo().greaterThan(timeInfo)
//					| parentEntity.getEndTimeInfo().lessThan(timeInfo)) {
//
//				SS3Singleton.getErrorMessages().showErrorMessage(
//						ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE);
//				return;
//			}
//		}

		GlossChainedEventsEntity glsEntity = (GlossChainedEventsEntity) entity;

		double moveFromCoord = glsEntity.getEndTimeCoordinate();
		double entityEndTime = glsEntity.getEndTimeInfo().doubleValue();
		double eventEndTime = glsEntity.getSelectedEvent().getEndTimeInfo()
				.doubleValue();

		if (true) {
			
			CoordinateTimeConverter convertor = bar.segmentPanel().getZoomer()
					.getCoordinateTimeConvertor();			
			double moveToCoord = convertor.convertTimeInfoToCoordinate(timeInfo);
			
			moveTextForward(moveFromCoord, moveToCoord,
					glsField, glsEntity, bar);
			
			String resp = glsEntity.canSetSelectedEventEndMovieTime(timeInfo);

			if (resp != null) {
				SS3Singleton.getErrorMessages().showErrorMessage(resp);
				return false;
			}

			double packetDuration = bar.segmentPanel()
					.getMultipleMovieController().getPacketDuration();

			ChainedEvent onset = glsEntity.getOnset();
			ChainedEvent offset = glsEntity.getOffset();
			ChainedEvent initialHold = glsEntity.getInitialHold();
			ChainedEvent finalHold = glsEntity.getFinalHold();
			ChainedEvent text = glsEntity.getTextValue();

			GlossChainedEventsEntity depGlsEntity = glsEntity
					.getDependentGlossEntity();
			ChainedEventsEntity handEntity = glsEntity.getDependentHandEntity();
			HandShapeEntity handShapeEntity = glsEntity
					.getDependentHandShapeEntity();

			boolean glsLockedGlossPhoneAlignments = glsEntity
					.isLockedGlossPhonAlignments();
			boolean glsLockedDomNdomAlignments = glsEntity
					.isLockedDomNdomAlignments();

			ChainedEvent depGlsOffset = null;
			ChainedEvent depGlsFinalHold = null;

			if (depGlsEntity != null) {
				depGlsOffset = depGlsEntity.getOffset();
				depGlsFinalHold = depGlsEntity.getFinalHold();
			}

			if (event.equals(onset)) {
//				double minOnsetEndTime = onset.getStartTimeInfo().doubleValue()
//						+ packetDuration;
//				double maxOnsetEndTime = text.getEndTimeInfo().doubleValue()
//						- packetDuration;
//
//				if (initialHold != null) {
//					maxOnsetEndTime = initialHold.getEndTimeInfo()
//							.doubleValue() - packetDuration;
//				}

				if (timeInfo.lessThan(onset.getStartTimeInfo())) {
					SS3Singleton.getErrorMessages().showErrorMessage(
							ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME);
					return false;
				} else if (timeInfo.greaterThan(text.getEndTimeInfo())) {
					if (initialHold != null) {
						SS3Singleton
								.getErrorMessages()
								.showErrorMessage(
										ErrorMessages.SET_END_ONSET_CANT_DELETE_INITIAL_HOLD);
						return false;
					} else {
						SS3Singleton
								.getErrorMessages()
								.showErrorMessage(
										ErrorMessages.SET_END_INITIAL_HOLD_CANT_DELETE_VALUE);
						return false;
					}
				} else {
					onset.setEndTimeInfo(timeInfo);

					if (initialHold != null) {
						initialHold.setStartTimeInfo(timeInfo);
					} else {
						text.setStartTimeInfo(timeInfo);
					}

					if (glsLockedDomNdomAlignments && depGlsEntity != null
							&& depGlsEntity.getOnset() != null) {

						ChainedEvent ndOnset = depGlsEntity.getOnset();
						ChainedEvent ndInitialHold = depGlsEntity
								.getInitialHold();
						ChainedEvent ndText = depGlsEntity.getTextValue();

						if (timeInfo.lessThan(ndOnset.getStartTimeInfo())) {
							SS3Singleton
									.getErrorMessages()
									.showErrorMessage(
											ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME);
							return false;
						} else if (timeInfo.greaterThan(ndText.getEndTimeInfo())) {
							if (ndInitialHold != null) {
								SS3Singleton
										.getErrorMessages()
										.showErrorMessage(
												ErrorMessages.SET_END_ONSET_CANT_DELETE_INITIAL_HOLD);
								return false;
							} else {
								SS3Singleton
										.getErrorMessages()
										.showErrorMessage(
												ErrorMessages.SET_END_INITIAL_HOLD_CANT_DELETE_VALUE);
								return false;
							}
						} else {
							ndOnset.setEndTimeInfo(timeInfo);

							if (ndInitialHold != null) {
								ndInitialHold.setStartTimeInfo(timeInfo);
							} else {
								ndText.setStartTimeInfo(timeInfo);
							}


							if (depGlsEntity.isLockedGlossPhonAlignments()) {
								ChainedEventsEntity ndHandEntity = depGlsEntity
										.getDependentHandEntity();
								HandShapeEntity ndHandShapeEntity = depGlsEntity
										.getDependentHandShapeEntity();

								if (ndHandEntity != null) {
									ChainedEventsEntity prevEntity = ndHandEntity
											.getPreviousEntity();
									if (prevEntity != null
											&& prevEntity.getEndTimeInfo().greaterThan(timeInfo)) {
										SS3Singleton
												.getErrorMessages()
												.showErrorMessage(
														ErrorMessages.GLOSS_CHAINS_CANNOT_OVERLAP);
										return false;
									} else {
										ndHandEntity.setStartTimeInfo(timeInfo);

										if (ndHandShapeEntity != null) {
											setHandshapeStartTime(
													ndHandShapeEntity, timeInfo);
										}
									}
								}

							}
						}
					}
					
					if (glsLockedGlossPhoneAlignments) {
						if (handEntity != null) {
							ChainedEventsEntity prevEntity = handEntity
									.getPreviousEntity();
							if (prevEntity != null
									&& prevEntity.getEndTimeInfo().greaterThan(timeInfo)) {
								SS3Singleton
										.getErrorMessages()
										.showErrorMessage(
												ErrorMessages.GLOSS_CHAINS_CANNOT_OVERLAP);
								return false;
							} else {
								handEntity.setStartTimeInfo(timeInfo);
								if (handShapeEntity != null) {
									MediaToolbarCommons.setHandshapeStartTime(
											handShapeEntity, timeInfo);
								}
							}
						}
					}
				}

			} else if (event.equals(initialHold)) {

				if (glsLockedDomNdomAlignments && depGlsEntity != null
						&& depGlsEntity.getInitialHold() != null) {
					ChainedEvent ndInitialHold = depGlsEntity.getInitialHold();
					ChainedEvent ndText = depGlsEntity.getTextValue();

					double ndMinEndTime = ndInitialHold.getStartTimeInfo()
							.doubleValue() + packetDuration;
					double ndMaxEndTime = ndText.getEndTimeInfo().doubleValue()
							- packetDuration;

					if (ndInitialHold.getStartTimeInfo().greaterThan(timeInfo) || ndText.getEndTimeInfo().lessThan(timeInfo)) {
						SS3Singleton
								.getErrorMessages()
								.showErrorMessage(
										ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME);
						return false;
					}

					ndInitialHold.setEndTimeInfo(timeInfo);
					ndText.setStartTimeInfo(timeInfo);
				}
			} else if (event.equals(text)) {
				if (glsLockedDomNdomAlignments && depGlsEntity != null) {
					if (depGlsFinalHold != null) {
						// depGlsEntity.setFinalHoldEvent(depGlsFinalHold);
						double maxStartTime = depGlsFinalHold.getEndTimeInfo()
								.doubleValue() - packetDuration;

						if (depGlsFinalHold.getEndTimeInfo().lessThan(timeInfo)) {
							depGlsEntity.setFinalHoldEvent(depGlsFinalHold);
							SS3Singleton
									.getErrorMessages()
									.showErrorMessage(
											ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME);
							return false;
						} else {
							depGlsEntity.getTextValue().setEndTimeInfo(timeInfo);
							depGlsFinalHold.setStartTimeInfo(timeInfo);
							depGlsEntity.setFinalHoldEvent(depGlsFinalHold);
						}
					} else if (depGlsOffset != null) {
						double maxEndTime = depGlsOffset.getEndTimeInfo()
								.doubleValue() - packetDuration;

						if (depGlsOffset.getEndTimeInfo().lessThan(timeInfo)) {
							SS3Singleton
									.getErrorMessages()
									.showErrorMessage(
											ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME);
							return false;
						} else {
							depGlsEntity.getTextValue().setEndTimeInfo(timeInfo);
							depGlsOffset.setStartTimeInfo(timeInfo);
							depGlsEntity.setOffsetEvent(depGlsOffset);

							if (depGlsEntity.isLockedGlossPhonAlignments()) {
								ChainedEventsEntity depHandEntity = depGlsEntity
										.getDependentHandEntity();
								HandShapeEntity depHandShapeEntity = depGlsEntity
										.getDependentHandShapeEntity();
								if (depHandEntity != null)
									depHandEntity.setEndTimeInfo(timeInfo);

								if (depHandShapeEntity != null) {
									MediaToolbarCommons.setHandshapeEndTime(
											depHandShapeEntity, timeInfo);
								}
							}
						}
					} else {
						GlossChainedEventsEntity ndNextEntity = depGlsEntity
								.getNextEntity();
						if (ndNextEntity != null
								&& ndNextEntity.getStartTimeInfo().lessThan(timeInfo)) {
							SS3Singleton.getErrorMessages().showErrorMessage(
									ErrorMessages.CHAINS_CANNOT_OVERLAP);
							return false;
					 	} else {
							depGlsEntity.getTextValue().setEndTimeInfo(timeInfo);
							if (depGlsEntity.isLockedGlossPhonAlignments()) {
								ChainedEventsEntity depHandEntity = depGlsEntity
										.getDependentHandEntity();
								HandShapeEntity depHandShapeEntity = depGlsEntity
										.getDependentHandShapeEntity();

								if (depHandEntity != null) {
									depHandEntity.setEndTimeInfo(timeInfo);

									if (depHandShapeEntity != null) {
										MediaToolbarCommons
												.setHandshapeEndTime(
														depHandShapeEntity,
														timeInfo);
									}
								}
							}
						}
					}
				}
				
				if (finalHold == null && glsLockedGlossPhoneAlignments) {
					if (handEntity != null) {
						ChainedEventsEntity nextEntity = handEntity
								.getNextEntity();
						if (nextEntity != null) {
							double nextTimeStartTime = nextEntity
									.getStartTimeInfo().doubleValue();

							if (nextEntity.getStartTimeInfo().greaterThanOrEq(timeInfo)) {
								handEntity.setEndTimeInfo(timeInfo);
							} else {
								MediaToolbarCommons.setHandshapeEndTime(
										handShapeEntity, timeInfo);
							}
						}
						if(handEntity != null) {
							handEntity.setEndTimeInfo(timeInfo);
						}
						if (handShapeEntity != null) {
							MediaToolbarCommons.setHandshapeEndTime(
									handShapeEntity, timeInfo);
						}
					}
				}

			} else if (event.equals(finalHold)) {

				if (glsLockedDomNdomAlignments) {
					if (depGlsEntity != null
							&& depGlsEntity.getFinalHold() != null) {

						ChainedEvent ndFinalHold = depGlsEntity.getFinalHold();
						double ndMinFinalHoldEndTime = ndFinalHold
								.getStartTimeInfo().doubleValue()
								+ packetDuration;

						if (ndFinalHold.getStartTimeInfo().greaterThan(timeInfo)) {
							SS3Singleton
									.getErrorMessages()
									.showErrorMessage(
											ErrorMessages.EVENT_END_TIME_FOLLOW_START_TIME);
							return false;
						} else if (depGlsEntity.getOffset() != null) {
							ChainedEvent ndOffset = depGlsEntity.getOffset();
							double ndMaxFinalHoldEndTime = ndOffset
									.getEndTimeInfo().doubleValue()
									- packetDuration;
							if (ndOffset.getEndTimeInfo().lessThan(timeInfo)) {
								SS3Singleton
										.getErrorMessages()
										.showErrorMessage(
												ErrorMessages.EVENT_END_TIME_FOLLOW_START_TIME);
								return false;
							} else {
								ndFinalHold.setEndTimeInfo(timeInfo);
								ndOffset.setStartTimeInfo(timeInfo);

								if (depGlsEntity.isLockedGlossPhonAlignments()) {
									ChainedEventsEntity ndHandEntity = depGlsEntity
											.getDependentHandEntity();
									HandShapeEntity ndHandShapeEntity = depGlsEntity
											.getDependentHandShapeEntity();

									if (ndHandEntity != null) {
										ChainedEventsEntity ndNextEntity = ndHandEntity
												.getNextEntity();
										if (ndNextEntity != null
												&& ndNextEntity.getStartTimeInfo().lessThan(timeInfo)) {
											SS3Singleton
													.getErrorMessages()
													.showErrorMessage(
															ErrorMessages.CHAINS_CANNOT_OVERLAP);
											return false;
										} else {
											ndHandEntity.setEndTimeInfo(timeInfo);
											if (ndHandShapeEntity != null) {
												MediaToolbarCommons
														.setHandshapeEndTime(
																ndHandShapeEntity,
																timeInfo);
											}
										}
									}
								}
							}
						} else {
							GlossChainedEventsEntity nextGlsEntity = depGlsEntity
									.getNextEntity();

							if (nextGlsEntity != null
									&& nextGlsEntity.getStartTimeInfo().lessThan(timeInfo)) {
								SS3Singleton
										.getErrorMessages()
										.showErrorMessage(
												ErrorMessages.CHAINS_CANNOT_OVERLAP);
								return false;
							} else {
								ndFinalHold.setEndTimeInfo(timeInfo);

								if (depGlsEntity.isLockedGlossPhonAlignments()) {
									ChainedEventsEntity ndHandEntity = depGlsEntity
											.getDependentHandEntity();
									HandShapeEntity ndHandShapeEntity = depGlsEntity
											.getDependentHandShapeEntity();

									if (ndHandEntity != null) {
										ChainedEventsEntity ndNextEntity = ndHandEntity
												.getNextEntity();
										if (ndNextEntity != null
												&& ndNextEntity
														.getStartTimeInfo().lessThan(timeInfo)) {
											SS3Singleton
													.getErrorMessages()
													.showErrorMessage(
															ErrorMessages.CHAINS_CANNOT_OVERLAP);
											return false;
										} else {
											ndHandEntity.setEndTimeInfo(timeInfo);
											if (ndHandShapeEntity != null) {
												MediaToolbarCommons
														.setHandshapeEndTime(
																ndHandShapeEntity,
																timeInfo);
											}
										}
									}
								}
							}
						}
					}
				}
				
				if (glsLockedGlossPhoneAlignments) {
					if (handEntity != null) {
						ChainedEventsEntity nextEntity = handEntity
								.getNextEntity();
						if (nextEntity != null
								&& nextEntity.getStartTimeInfo().lessThan(timeInfo)) {
							SS3Singleton.getErrorMessages().showErrorMessage(
									ErrorMessages.CHAINS_CANNOT_OVERLAP);
							return false;
						} else {
							handEntity.setEndTimeInfo(timeInfo);
							if (handShapeEntity != null) {
								MediaToolbarCommons.setHandshapeEndTime(
										handShapeEntity, timeInfo);
							}
						}
					}
				}

			} else if (event.equals(offset)) {
				if (glsLockedDomNdomAlignments && depGlsEntity != null && depGlsEntity.getOffset() != null) {
					GlossChainedEventsEntity nextEntity = depGlsEntity
							.getNextEntity();
					if (nextEntity != null
							&& nextEntity.getStartTimeInfo().lessThan(timeInfo)) {
						SS3Singleton.getErrorMessages().showErrorMessage(
								ErrorMessages.CHAINS_CANNOT_OVERLAP);
						return false;
					} else {
						if (depGlsEntity != null) {
							depGlsEntity.getOffset().setEndTimeInfo(timeInfo);
						}
					}
				}
			}

			glsEntity.setGlossEventsEntityEndTimeInfo(timeInfo);

			LocationEntity depLocEntity = glsEntity.getDependentLocationEntity();
			if (depLocEntity != null) {
				depLocEntity.updateTimes();
			} else if (glsLockedDomNdomAlignments && depGlsEntity != null) {
				depLocEntity = depGlsEntity.getDependentLocationEntity();
				if (depLocEntity != null) {
					depLocEntity.updateTimes();
				}
			}
			SignTypeEntity signTypeEntity = glsEntity.getDependentSignTypeEntity();
			if (signTypeEntity != null) {
				signTypeEntity.updateTimes();
			} else if (glsLockedDomNdomAlignments && depGlsEntity != null) {
				signTypeEntity = depGlsEntity.getDependentSignTypeEntity();
				if (signTypeEntity != null) {
					signTypeEntity.updateTimes();
				}
			}
		}

		return true;
	}

	private static void moveTextForward(double from, double to, GlossField glsField,
			GlossChainedEventsEntity entity, MediaToolBar bar) {
		// from: where entity end point started (old from)
		// to: where entity end point is desired to go
		// entity: its end point has already moved from --> to, so the end is here the (new from)
		
		FontMetrics fontMetrics = bar.getFontMetrics(SegmentGraphReprUtil
				.getGraphReprFont());
		
		EnteredText enteredText = glsField
				.getGlossChainedEventsEntityCollection().getEnteredText();

//		EventsEntity selectedEntity = bar.getSelectedEventsEntity();		
//		EventsEntity parentEntity = selectedEntity.getParentEntity();		
		GlossChainedEventsEntity nextEntity = entity.getNextEntity();
		
		ArrayList<Text> textInEndPointsInterval = enteredText
				.getSortedEnteredTextInInteval2(from, to);		// interval between old end point and new end point

		if (!textInEndPointsInterval.isEmpty()) {
//			double maxEndCoord = parentEntity
//					.getEndTimeCoordinate();
			double maxEndCoord = TimeRange.ABSOLUTE.end().value();		// GREG TODO: not sure if valid
			if (nextEntity != null) {
				maxEndCoord = nextEntity.getStartTimeCoordinate();	
				// changed this from endCoord to startCoord - should be no difference because no text should be within a gloss
			}

			ArrayList<Text> textInInterval = enteredText
					.getSortedEnteredTextInInteval2(from, maxEndCoord);

			int cumulativeWidth = 0;
			int dCumulativeWidth = Util.MTB_TEXT_BUFFER;
			int textSize = textInInterval.size();

			// loop through texts to get their cumulative width
			for (int i = textSize - 1; i >= 0; i--) {
				Text t = textInInterval.get(i);
				String txt = t.getText();
				int width = fontMetrics.stringWidth(txt);
				cumulativeWidth = cumulativeWidth + width;
				dCumulativeWidth = dCumulativeWidth + Util.MTB_TEXT_BUFFER;
			}

			dCumulativeWidth = dCumulativeWidth + Util.MTB_TEXT_BUFFER;

			double availableWidth = maxEndCoord - to;
			double requestedWidth = cumulativeWidth;
			double prevEndCoord = to + Util.MTB_TEXT_BUFFER;

			if (availableWidth >= requestedWidth) {
				for (Text text : textInInterval) {
					text.setStartCoord(prevEndCoord);
					double endCoord = prevEndCoord
							+ fontMetrics.stringWidth(text.getText());
					text.setEndCoord(endCoord);
					prevEndCoord = prevEndCoord
							+ fontMetrics.stringWidth(text.getText()) + Util.MTB_TEXT_BUFFER;
				}
			} 
			// this else block is now obsolete due to the return block above involving newDiffBetweenEntityCoords
			// if I am wrong, then uncomment this
//			else {
//				double d1 = availableWidth;
//				double d2 = requestedWidth;
//				double d = d1 / d2;
//
//				for (Text text : textInInterval) {
//					text.setStartCoord(prevEndCoord);
//					double adjustedTextWidth = fontMetrics.stringWidth(text
//							.getText()) * d - 3;
//					prevEndCoord = prevEndCoord + BUFFER + (int) adjustedTextWidth;
//				}
//			}
		}
	}
	
	protected static boolean setGestureEventsEntityEntityEndTime(Time timeInfo,
                                                              MediaToolBar bar) {
		Field field = bar.selectedField();
		Event event = bar.selectedEvent();
		EventsEntity entity = bar.selectedEntity();

//		EventsEntity selectedEntity = bar.getSelectedEventsEntity();		
//		EventsEntity parentEntity = selectedEntity.getParentEntity();

		GestureField gestField = (GestureField) field;

		if (entity == null && event != null) {
			entity = gestField.getEntityAt(event.getStartTimeCoordinate());
		}

//		if (parentEntity != null) {
//			if (parentEntity.getStartTimeInfo().greaterThan(timeInfo)
//					| parentEntity.getEndTimeInfo().lessThan(timeInfo)) {
//
//				SS3Singleton.getErrorMessages().showErrorMessage(
//						ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE);
//				return;
//			}
//		}

		GestureEventsEntity gestEntity = (GestureEventsEntity) entity;

		double moveFromCoord = gestEntity.getEndTimeCoordinate();
		double entityEndTime = gestEntity.getEndTimeInfo().doubleValue();
		double eventEndTime = gestEntity.getSelectedEvent().getEndTimeInfo()
				.doubleValue();

		if (true) {

			CoordinateTimeConverter convertor = bar.segmentPanel().getZoomer()
					.getCoordinateTimeConvertor();			
			double moveToCoord = convertor.convertTimeInfoToCoordinate(timeInfo);
			
			moveTextForward(moveFromCoord, moveToCoord,
					gestField, gestEntity, bar);
			
			String resp = gestEntity.canSetSelectedEventEndMovieTime(timeInfo);


			if (resp != null) {
				SS3Singleton.getErrorMessages().showErrorMessage(resp);
				return false;
			}

			double packetDuration = bar.segmentPanel()
					.getMultipleMovieController().getPacketDuration();

			GestureChainedEvent preparation = gestEntity.getPreparation();
			GestureChainedEvent retraction = gestEntity.getRetraction();
			GestureChainedEvent prestrokeHold = gestEntity.getPrestrokeHold();
			GestureChainedEvent poststrokeHold = gestEntity.getPoststrokeHold();
			GestureChainedEvent relaxed = gestEntity.getRelaxed();
			GestureChainedEvent text = gestEntity.getTextValue();
			
			ApexChainedApexEventsEntity apexEntity = gestEntity.getDependentApexEntity();

			GestureEventsEntity depGestEntity = gestEntity
					.getDependentGestureEntity();
			GestureChainedEventsEntity handEntity = gestEntity.getDependentHandEntity();
			GestureHandShapeEntity handShapeEntity = gestEntity
					.getDependentHandShapeEntity();
			GestureOrientationEntity orientationEntity = gestEntity
					.getDependentOrientationEntity();

			boolean gestLockedGlossPhoneAlignments = gestEntity
					.isLockedGesturePhonAlignments();
			boolean gestLockedDomNdomAlignments = gestEntity
					.isLockedDomNdomAlignments();

			GestureChainedEvent depGestPreparation = null;
			GestureChainedEvent depGestRetraction = null;
			GestureChainedEvent depGestPrestrokeHold = null;
			GestureChainedEvent depGestPoststrokeHold = null;
			GestureChainedEvent depGestRelaxed = null;
			ApexChainedApexEventsEntity depApexEntity = null;

			if (depGestEntity != null) {
				depGestPreparation = depGestEntity.getPreparation();
				depGestRetraction = depGestEntity.getRetraction();
				depGestPrestrokeHold = depGestEntity.getPrestrokeHold();
				depGestPoststrokeHold = depGestEntity.getPoststrokeHold();
				depGestRelaxed = depGestEntity.getRelaxed();
				depApexEntity = depGestEntity.getDependentApexEntity();
			}

			if (event.equals(preparation)) {
				
				if (apexEntity != null) {
					if (timeInfo.greaterThan(apexEntity.getStartTimeInfo())) {
						SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.GESTURE_APEX_BOUNDS);
						return false;
					}
				}
				
				if (gestLockedDomNdomAlignments && depApexEntity != null) {
					if (timeInfo.greaterThan(depApexEntity.getStartTimeInfo())) {
						SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.GESTURE_APEX_BOUNDS);
						return false;
					}
				}
				
				if (timeInfo.lessThan(preparation.getStartTimeInfo())) {
					SS3Singleton.getErrorMessages().showErrorMessage(
							ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME);
					return false;
				} else if (timeInfo.greaterThan(text.getEndTimeInfo())) {
					if (prestrokeHold != null) {
						SS3Singleton
								.getErrorMessages()
								.showErrorMessage(
										ErrorMessages.SET_END_PREPARATION_CANT_DELETE_PRESTROKE_HOLD);
						return false;
					} else {
						SS3Singleton
								.getErrorMessages()
								.showErrorMessage(
										ErrorMessages.SET_END_PRESTROKE_HOLD_CANT_DELETE_VALUE);
						return false;
					}
				} else {
					preparation.setEndTimeInfo(timeInfo);

					if (prestrokeHold != null) {
						prestrokeHold.setStartTimeInfo(timeInfo);
					} else {
						text.setStartTimeInfo(timeInfo);
					}

					if (gestLockedDomNdomAlignments && depGestEntity != null
							&& depGestEntity.getPreparation() != null) {

						GestureChainedEvent ndPreparation = depGestEntity.getPreparation();
						GestureChainedEvent ndPrestrokeHold = depGestEntity
								.getPrestrokeHold();
						GestureChainedEvent ndText = depGestEntity.getTextValue();

						if (timeInfo.lessThan(ndPreparation.getStartTimeInfo())) {
							SS3Singleton
									.getErrorMessages()
									.showErrorMessage(
											ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME);
							return false;
						} else if (timeInfo.greaterThan(ndText.getEndTimeInfo())) {
							if (ndPrestrokeHold != null) {
								SS3Singleton
										.getErrorMessages()
										.showErrorMessage(
												ErrorMessages.SET_END_PREPARATION_CANT_DELETE_PRESTROKE_HOLD);
								return false;
							} else {
								SS3Singleton
										.getErrorMessages()
										.showErrorMessage(
												ErrorMessages.SET_END_PRESTROKE_HOLD_CANT_DELETE_VALUE);
								return false;
							}
						} else {
							ndPreparation.setEndTimeInfo(timeInfo);

							if (ndPrestrokeHold != null) {
								ndPrestrokeHold.setStartTimeInfo(timeInfo);
							} else {
								ndText.setStartTimeInfo(timeInfo);

								if (depGestEntity.isLockedGesturePhonAlignments()) {
									GestureChainedEventsEntity ndHandEntity = depGestEntity
											.getDependentHandEntity();
									GestureHandShapeEntity ndHandShapeEntity = depGestEntity
											.getDependentHandShapeEntity();
									GestureOrientationEntity ndOrientationEntity = depGestEntity
											.getDependentOrientationEntity();

									if (ndHandEntity != null) {
										GestureChainedEventsEntity prevEntity = ndHandEntity
												.getPreviousEntity();
										if (prevEntity != null
												&& prevEntity.getEndTimeInfo().greaterThan(timeInfo)) {
											SS3Singleton
													.getErrorMessages()
													.showErrorMessage(
															ErrorMessages.GESTURE_CHAINS_CANNOT_OVERLAP);
											return false;
										} else {
											ndHandEntity.setStartTimeInfo(timeInfo);

											if (ndHandShapeEntity != null) {
												setGestureHandshapeStartTime(
														ndHandShapeEntity, timeInfo);
											}
											if (ndOrientationEntity != null) {
												setGestureOrientationStartTime(
														ndOrientationEntity, timeInfo);
											}
										}
									}

								}
							}
						}
					}
					
					if (gestLockedGlossPhoneAlignments && prestrokeHold == null) {
						if (handEntity != null) {
							GestureChainedEventsEntity prevEntity = handEntity
									.getPreviousEntity();
							if (prevEntity != null
									&& prevEntity.getEndTimeInfo().greaterThan(timeInfo)) {
								SS3Singleton
										.getErrorMessages()
										.showErrorMessage(
												ErrorMessages.GESTURE_CHAINS_CANNOT_OVERLAP);
								return false;
							} else {
								handEntity.setStartTimeInfo(timeInfo);
								if (handShapeEntity != null) {
									MediaToolbarCommons.setGestureHandshapeStartTime(
											handShapeEntity, timeInfo);
								}
								if (orientationEntity != null) {
									MediaToolbarCommons.setGestureOrientationStartTime(
											orientationEntity, timeInfo);
								}
							}
						}
					}
				}

			} else if (event.equals(prestrokeHold)) {
				
				if (apexEntity != null) {
					if (timeInfo.greaterThan(apexEntity.getStartTimeInfo())) {
						SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.GESTURE_APEX_BOUNDS);
						return false;
					}
				}
				
				if (gestLockedDomNdomAlignments && depApexEntity != null) {
					if (timeInfo.greaterThan(depApexEntity.getStartTimeInfo())) {
						SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.GESTURE_APEX_BOUNDS);
						return false;
					}
				}

				if (gestLockedDomNdomAlignments && depGestEntity != null
						&& depGestEntity.getPrestrokeHold() != null) {
					GestureChainedEvent ndPrestrokeHold = depGestEntity.getPrestrokeHold();
					GestureChainedEvent ndText = depGestEntity.getTextValue();

					double ndMinEndTime = ndPrestrokeHold.getStartTimeInfo()
							.doubleValue() + packetDuration;
					double ndMaxEndTime = ndText.getEndTimeInfo().doubleValue()
							- packetDuration;

					if (ndPrestrokeHold.getStartTimeInfo().greaterThan(timeInfo) || ndText.getEndTimeInfo().lessThan(timeInfo)) {
						SS3Singleton
								.getErrorMessages()
								.showErrorMessage(
										ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME);
						return false;
					}

					ndPrestrokeHold.setEndTimeInfo(timeInfo);
					ndText.setStartTimeInfo(timeInfo);

					if (depGestEntity.isLockedGesturePhonAlignments()) {
						GestureChainedEventsEntity ndHandEntity = depGestEntity
								.getDependentHandEntity();
						GestureHandShapeEntity ndHandShapeEntity = depGestEntity
								.getDependentHandShapeEntity();
						GestureOrientationEntity ndOrientationEntity = depGestEntity
								.getDependentOrientationEntity();

						if (ndHandEntity != null) {
							GestureChainedEventsEntity prevEntity = ndHandEntity
									.getPreviousEntity();
							if (prevEntity != null
									&& prevEntity.getEndTimeInfo().greaterThan(timeInfo)) {
								SS3Singleton
										.getErrorMessages()
										.showErrorMessage(
												ErrorMessages.GESTURE_CHAINS_CANNOT_OVERLAP);
								return false;
							} else {
								ndHandEntity.setStartTimeInfo(timeInfo);

								if (ndHandShapeEntity != null) {
									setGestureHandshapeStartTime(
											ndHandShapeEntity, timeInfo);
								}
								if (ndOrientationEntity != null) {
									setGestureOrientationStartTime(
											ndOrientationEntity, timeInfo);
								}
							}
						}

					}
				}
				
				if (gestLockedGlossPhoneAlignments) {
					if (handEntity != null) {
						GestureChainedEventsEntity nextEntity = handEntity
								.getNextEntity();
						if (nextEntity != null
								&& nextEntity.getStartTimeInfo().lessThan(timeInfo)) {
							SS3Singleton.getErrorMessages().showErrorMessage(
									ErrorMessages.GESTURE_CHAINS_CANNOT_OVERLAP);
							return false;
						} else {
							handEntity.setStartTimeInfo(timeInfo);
							if (handShapeEntity != null) {
								MediaToolbarCommons.setGestureHandshapeStartTime(
										handShapeEntity, timeInfo);
							}
							if (orientationEntity != null) {
								MediaToolbarCommons.setGestureOrientationStartTime(
										orientationEntity, timeInfo);
							}
						}
					}
				}
			} else if (event.equals(text)) {
				
				if (apexEntity != null) {
					if (timeInfo.lessThan(apexEntity.getEndTimeInfo())) {
						SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.GESTURE_APEX_BOUNDS);
						return false;
					}
				}
				
				if (gestLockedDomNdomAlignments && depApexEntity != null) {
					if (timeInfo.lessThan(depApexEntity.getEndTimeInfo())) {
						SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.GESTURE_APEX_BOUNDS);
						return false;
					}
				}
				
				if (gestLockedDomNdomAlignments && depGestEntity != null) {
					if (depGestPoststrokeHold != null) {

						if (depGestPoststrokeHold.getEndTimeInfo().lessThan(timeInfo)) {
							depGestEntity.setPoststrokeHoldEvent(depGestPoststrokeHold);
							SS3Singleton
									.getErrorMessages()
									.showErrorMessage(
											ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME);
							return false;
						} else {
							depGestEntity.getTextValue().setEndTimeInfo(timeInfo);
							depGestPoststrokeHold.setStartTimeInfo(timeInfo);
							depGestEntity.setPoststrokeHoldEvent(depGestPoststrokeHold);

							if (depGestEntity.isLockedGesturePhonAlignments()) {
								GestureChainedEventsEntity depHandEntity = depGestEntity
										.getDependentHandEntity();
								GestureHandShapeEntity depHandShapeEntity = depGestEntity
										.getDependentHandShapeEntity();
								GestureOrientationEntity depOrientationEntity = depGestEntity
										.getDependentOrientationEntity();
								if (depHandEntity != null)
									depHandEntity.setEndTimeInfo(timeInfo);

								if (depHandShapeEntity != null) {
									MediaToolbarCommons.setGestureHandshapeEndTime(
											depHandShapeEntity, timeInfo);
								}
								if (depOrientationEntity != null) {
									MediaToolbarCommons.setGestureOrientationEndTime(
											depOrientationEntity, timeInfo);
								}
							}
						}
					} else if (depGestRetraction != null) {
						double maxEndTime = depGestRetraction.getEndTimeInfo()
								.doubleValue() - packetDuration;

						if (depGestRetraction.getEndTimeInfo().lessThan(timeInfo)) {
							SS3Singleton
									.getErrorMessages()
									.showErrorMessage(
											ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME);
							return false;
						} else {
							depGestEntity.getTextValue().setEndTimeInfo(timeInfo);
							depGestRetraction.setStartTimeInfo(timeInfo);
							depGestEntity.setRetractionEvent(depGestRetraction);

							if (depGestEntity.isLockedGesturePhonAlignments()) {
								GestureChainedEventsEntity depHandEntity = depGestEntity
										.getDependentHandEntity();
								GestureHandShapeEntity depHandShapeEntity = depGestEntity
										.getDependentHandShapeEntity();
								GestureOrientationEntity depOrientationEntity = depGestEntity
										.getDependentOrientationEntity();
								if (depHandEntity != null)
									depHandEntity.setEndTimeInfo(timeInfo);

								if (depHandShapeEntity != null) {
									MediaToolbarCommons.setGestureHandshapeEndTime(
											depHandShapeEntity, timeInfo);
								}
								if (depOrientationEntity != null) {
									MediaToolbarCommons.setGestureOrientationEndTime(
											depOrientationEntity, timeInfo);
								}
							}
						}
					} else if (depGestRelaxed != null) {
						double maxEndTime = depGestRelaxed.getEndTimeInfo()
								.doubleValue() - packetDuration;

						if (depGestRelaxed.getEndTimeInfo().lessThan(timeInfo)) {
							SS3Singleton
									.getErrorMessages()
									.showErrorMessage(
											ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME);
							return false;
						} else {
							depGestEntity.getTextValue().setEndTimeInfo(timeInfo);
							depGestRelaxed.setStartTimeInfo(timeInfo);
							depGestEntity.setRelaxedEvent(depGestRelaxed);

							if (depGestEntity.isLockedGesturePhonAlignments()) {
								GestureChainedEventsEntity depHandEntity = depGestEntity
										.getDependentHandEntity();
								GestureHandShapeEntity depHandShapeEntity = depGestEntity
										.getDependentHandShapeEntity();
								GestureOrientationEntity depOrientationEntity = depGestEntity
										.getDependentOrientationEntity();
								if (depHandEntity != null)
									depHandEntity.setEndTimeInfo(timeInfo);

								if (depHandShapeEntity != null) {
									MediaToolbarCommons.setGestureHandshapeEndTime(
											depHandShapeEntity, timeInfo);
								}
								if (depOrientationEntity != null) {
									MediaToolbarCommons.setGestureOrientationEndTime(
											depOrientationEntity, timeInfo);
								}
							}
						}
					} else {
						GestureEventsEntity ndNextEntity = depGestEntity
								.getNextEntity();
						if (ndNextEntity != null
								&& ndNextEntity.getStartTimeInfo().lessThan(timeInfo)) {
							SS3Singleton.getErrorMessages().showErrorMessage(
									ErrorMessages.GESTURE_CHAINS_CANNOT_OVERLAP);
							return false;
					 	} else {
							depGestEntity.getTextValue().setEndTimeInfo(timeInfo);
							if (depGestEntity.isLockedGesturePhonAlignments()) {
								GestureChainedEventsEntity depHandEntity = depGestEntity
										.getDependentHandEntity();
								GestureHandShapeEntity depHandShapeEntity = depGestEntity
										.getDependentHandShapeEntity();
								GestureOrientationEntity depOrientationEntity = depGestEntity
										.getDependentOrientationEntity();

								if (depHandEntity != null) {
									depHandEntity.setEndTimeInfo(timeInfo);

									if (depHandShapeEntity != null) {
										MediaToolbarCommons
												.setGestureHandshapeEndTime(
														depHandShapeEntity,
														timeInfo);
									}
									if (depOrientationEntity != null) {
										MediaToolbarCommons
											.setGestureOrientationEndTime(
													depOrientationEntity,
													timeInfo);
									}
								}
							}
						}
					}
				}
				
				if (gestLockedGlossPhoneAlignments) {
					if (handEntity != null) {
						GestureChainedEventsEntity nextEntity = handEntity
								.getNextEntity();
						if (nextEntity != null) {
							double nextTimeStartTime = nextEntity
									.getStartTimeInfo().doubleValue();

							if (nextEntity.getStartTimeInfo().greaterThanOrEq(timeInfo)) {
								handEntity.setEndTimeInfo(timeInfo);
							} else {
								MediaToolbarCommons.setGestureHandshapeEndTime(
										handShapeEntity, timeInfo);
							}
						}
						if(handEntity != null) {
							handEntity.setEndTimeInfo(timeInfo);
						}
						if (handShapeEntity != null) {
							MediaToolbarCommons.setGestureHandshapeEndTime(
									handShapeEntity, timeInfo);
						}
						if (orientationEntity != null) {
							MediaToolbarCommons.setGestureOrientationEndTime(
									orientationEntity, timeInfo);
						}
					}
				}
			} else if (event.equals(poststrokeHold)) {

				if (gestLockedDomNdomAlignments) {
					if (depGestEntity != null
							&& depGestEntity.getPoststrokeHold() != null) {

						GestureChainedEvent ndPoststrokeHold = depGestEntity.getPoststrokeHold();
						double ndMinFinalHoldEndTime = ndPoststrokeHold
								.getStartTimeInfo().doubleValue()
								+ packetDuration;

						if (ndPoststrokeHold.getStartTimeInfo().greaterThan(timeInfo)) {
							SS3Singleton
									.getErrorMessages()
									.showErrorMessage(
											ErrorMessages.EVENT_END_TIME_FOLLOW_START_TIME);
							return false;
						} else if (depGestEntity.getRetraction() != null) {
							GestureChainedEvent ndRetraction = depGestEntity.getRetraction();
							if (ndRetraction.getEndTimeInfo().lessThan(timeInfo)) {
								SS3Singleton
										.getErrorMessages()
										.showErrorMessage(
												ErrorMessages.EVENT_END_TIME_FOLLOW_START_TIME);
								return false;
							} else {
								ndPoststrokeHold.setEndTimeInfo(timeInfo);
								ndRetraction.setStartTimeInfo(timeInfo);

								if (depGestEntity.isLockedGesturePhonAlignments()) {
									GestureChainedEventsEntity ndHandEntity = depGestEntity
											.getDependentHandEntity();
//									HandShapeEntity_XYZ ndHandShapeEntity = depGlsEntity
//											.getDependentHandShapeEntity();
//									OrientationEntity_XYZ ndOrientationEntity = depGlsEntity
//											.getDependentOrientationEntity();

									if (ndHandEntity != null) {
										GestureChainedEventsEntity ndNextEntity = ndHandEntity
												.getNextEntity();
										if (ndNextEntity != null
												&& ndNextEntity
														.getStartTimeInfo().lessThan(timeInfo)) {
											SS3Singleton
													.getErrorMessages()
													.showErrorMessage(
															ErrorMessages.GESTURE_CHAINS_CANNOT_OVERLAP);
											return false;
										} else {
//											ndHandEntity.setEndTimeInfo(timeInfo);
//											
//											if (ndHandShapeEntity != null) {
//												MediaToolbarCommons
//														.setHandshape_XYZEndTime(
//																ndHandShapeEntity,
//																timeInfo);
//											}
//											
//											if (ndOrientationEntity != null) {
//												MediaToolbarCommons
//														.setOrientation_XYZEndTime(
//																ndOrientationEntity,
//																timeInfo);
//											}
										}
									}
								}
							}
						} else if (depGestEntity.getRelaxed() != null) {
							GestureChainedEvent ndRelaxed = depGestEntity.getRelaxed();
							double ndMaxFinalHoldEndTime = ndRelaxed
									.getEndTimeInfo().doubleValue()
									- packetDuration;
							if (ndRelaxed.getEndTimeInfo().lessThan(timeInfo)) {
								SS3Singleton
										.getErrorMessages()
										.showErrorMessage(
												ErrorMessages.EVENT_END_TIME_FOLLOW_START_TIME);
								return false;
							} else {
								ndPoststrokeHold.setEndTimeInfo(timeInfo);
								ndRelaxed.setStartTimeInfo(timeInfo);

								if (depGestEntity.isLockedGesturePhonAlignments()) {
									GestureChainedEventsEntity ndHandEntity = depGestEntity
											.getDependentHandEntity();
//									HandShapeEntity_XYZ ndHandShapeEntity = depGlsEntity
//											.getDependentHandShapeEntity();
//									OrientationEntity_XYZ ndOrientationEntity = depGlsEntity
//											.getDependentOrientationEntity();

									if (ndHandEntity != null) {
										GestureChainedEventsEntity ndNextEntity = ndHandEntity
												.getNextEntity();
										if (ndNextEntity != null
												&& ndNextEntity
														.getStartTimeInfo().lessThan(timeInfo)) {
											SS3Singleton
													.getErrorMessages()
													.showErrorMessage(
															ErrorMessages.GESTURE_CHAINS_CANNOT_OVERLAP);
											return false;
										} else {
//											ndHandEntity.setEndTimeInfo(timeInfo);
//											
//											if (ndHandShapeEntity != null) {
//												MediaToolbarCommons
//														.setHandshape_XYZEndTime(
//																ndHandShapeEntity,
//																timeInfo);
//											}
//											
//											if (ndOrientationEntity != null) {
//												MediaToolbarCommons
//														.setOrientation_XYZEndTime(
//																ndOrientationEntity,
//																timeInfo);
//											}
										}
									}
								}
							}
						} else {
							GestureEventsEntity nextGestEntity = depGestEntity
									.getNextEntity();

							if (nextGestEntity != null
									&& nextGestEntity.getStartTimeInfo().lessThan(timeInfo)) {
								SS3Singleton
										.getErrorMessages()
										.showErrorMessage(
												ErrorMessages.GESTURE_CHAINS_CANNOT_OVERLAP);
								return false;
							} else {
								ndPoststrokeHold.setEndTimeInfo(timeInfo);

//								if (depGlsEntity.isLockedGesturePhonAlignments()) {
////									GestureChainedEventsEntity ndHandEntity = depGlsEntity
////											.getDependentHandEntity();
////									HandShapeEntity_XYZ ndHandShapeEntity = depGlsEntity
////											.getDependentHandShapeEntity();
////									OrientationEntity_XYZ ndOrientationEntity = depGlsEntity
////											.getDependentOrientationEntity();
////
////									if (ndHandEntity != null) {
////										GestureChainedEventsEntity ndNextEntity = ndHandEntity
////												.getNextEntity();
//////										if (ndNextEntity != null
//////												&& ndNextEntity
//////														.getStartTimeInfo().lessThan(timeInfo)) {
//////											SS3Singleton
//////													.getErrorMessages()
//////													.showErrorMessage(
//////															ErrorMessages.GESTURE_CHAINS_CANNOT_OVERLAP);
//////											return false;
//////										} else {
//////											// GD 2021-03-06: do not update gesture hand/shape entities to the final/initial holds
////////											ndHandEntity.setEndTimeInfo(timeInfo);
////////											if (ndHandShapeEntity != null) {
////////												MediaToolbarCommons
////////														.setHandshape_XYZEndTime(
////////																ndHandShapeEntity,
////////																timeInfo);
////////											}
//////										}
////									}
//								}
							}
						}
					}
				}
				
				if (gestLockedGlossPhoneAlignments) {
					if (handEntity != null) {
						GestureChainedEventsEntity nextEntity = handEntity
								.getNextEntity();
						if (nextEntity != null
								&& nextEntity.getStartTimeInfo().lessThan(timeInfo)) {
							SS3Singleton.getErrorMessages().showErrorMessage(
									ErrorMessages.CHAINS_CANNOT_OVERLAP);
							return false;
						} else {
							handEntity.setEndTimeInfo(timeInfo);
							if (handShapeEntity != null) {
								MediaToolbarCommons.setGestureHandshapeEndTime(
										handShapeEntity, timeInfo);
							}
							if (orientationEntity != null) {
								MediaToolbarCommons.setGestureOrientationEndTime(
										orientationEntity, timeInfo);
							}
						}
					}
				}

			} else if (event.equals(retraction)) {
				if (gestLockedDomNdomAlignments && depGestEntity != null && depGestEntity.getRetraction() != null) {
					GestureEventsEntity nextEntity = depGestEntity
							.getNextEntity();
					if (nextEntity != null
							&& nextEntity.getStartTimeInfo().lessThan(timeInfo)) {
						SS3Singleton.getErrorMessages().showErrorMessage(
								ErrorMessages.GESTURE_CHAINS_CANNOT_OVERLAP);
						return false;
					} else {
						if (depGestEntity != null) {
							depGestEntity.getRetraction().setEndTimeInfo(timeInfo);
							
							if (depGestRelaxed != null) {
								depGestRelaxed.setStartTimeInfo(timeInfo);
							}
						}
					}
				}
			} else if (event.equals(relaxed)) {
				if (gestLockedDomNdomAlignments && depGestEntity != null && depGestEntity.getRelaxed() != null) {
					GestureEventsEntity nextEntity = depGestEntity
							.getNextEntity();
					if (nextEntity != null
							&& nextEntity.getStartTimeInfo().lessThan(timeInfo)) {
						SS3Singleton.getErrorMessages().showErrorMessage(
								ErrorMessages.GESTURE_CHAINS_CANNOT_OVERLAP);
						return false;
					} else {
						if (depGestEntity != null) {
							depGestEntity.getRelaxed().setEndTimeInfo(timeInfo);
						}
					}
				}
			}
			
			gestEntity.setGestureEventsEntityEndTimeInfo(timeInfo);
			
			GestureLocationEntity depLocEntity = gestEntity.getDependentLocationEntity();
			if (depLocEntity != null) {
				depLocEntity.updateTimes();
			} else if (gestLockedDomNdomAlignments && depGestEntity != null) {
				depLocEntity = depGestEntity.getDependentLocationEntity();
				if (depLocEntity != null) {
					depLocEntity.updateTimes();
				}
			}
			GestureTypeEntity gestureTypeEntity = gestEntity.getDependentGestureTypeEntity();
			if (gestureTypeEntity != null) {
				gestureTypeEntity.updateTimes();
			} else if (gestLockedDomNdomAlignments && depGestEntity != null) {
				gestureTypeEntity = depGestEntity.getDependentGestureTypeEntity();
				if (gestureTypeEntity != null) {
					gestureTypeEntity.updateTimes();
				}
			}

		}
		
		return true;
	}

	private static void moveTextForward(double from, double to, GestureField gestField,
			GestureEventsEntity entity, MediaToolBar bar) {
		// from: where entity end point started (old from)
		// to: where entity end point is desired to go
		// entity: its end point has already moved from --> to, so the end is here the (new from)
		
		FontMetrics fontMetrics = bar.getFontMetrics(SegmentGraphReprUtil
				.getGraphReprFont());
		
		GestureEnteredText enteredText = gestField
				.getGestureEventsEntityCollection().getEnteredText();

//		EventsEntity selectedEntity = bar.getSelectedEventsEntity();		
//		EventsEntity parentEntity = selectedEntity.getParentEntity();		
		GestureEventsEntity nextEntity = entity.getNextEntity();
		
		ArrayList<GestureText> textInEndPointsInterval = enteredText
				.getSortedEnteredTextInInteval2(from, to);		// interval between old end point and new end point

		if (!textInEndPointsInterval.isEmpty()) {
//			double maxEndCoord = parentEntity
//					.getEndTimeCoordinate();
			double maxEndCoord = TimeRange.ABSOLUTE.end().value();		// GREG TODO: not sure if valid
			if (nextEntity != null) {
				maxEndCoord = nextEntity.getStartTimeCoordinate();	
				// changed this from endCoord to startCoord - should be no difference because no text should be within a gloss
			}

			ArrayList<GestureText> textInInterval = enteredText
					.getSortedEnteredTextInInteval2(from, maxEndCoord);

			int cumulativeWidth = 0;
			int dCumulativeWidth = Util.MTB_TEXT_BUFFER;
			int textSize = textInInterval.size();

			// loop through texts to get their cumulative width
			for (int i = textSize - 1; i >= 0; i--) {
				GestureText t = textInInterval.get(i);
				String txt = t.getText();
				int width = fontMetrics.stringWidth(txt);
				cumulativeWidth = cumulativeWidth + width;
				dCumulativeWidth = dCumulativeWidth + Util.MTB_TEXT_BUFFER;
			}

			dCumulativeWidth = dCumulativeWidth + Util.MTB_TEXT_BUFFER;

			double availableWidth = maxEndCoord - to;
			double requestedWidth = cumulativeWidth;
			double prevEndCoord = to + Util.MTB_TEXT_BUFFER;

			if (availableWidth >= requestedWidth) {
				for (GestureText text : textInInterval) {
					text.setStartCoord(prevEndCoord);
					double endCoord = prevEndCoord
							+ fontMetrics.stringWidth(text.getText());
					text.setEndCoord(endCoord);
					prevEndCoord = prevEndCoord
							+ fontMetrics.stringWidth(text.getText()) + Util.MTB_TEXT_BUFFER;
				}
			} 
			// this else block is now obsolete due to the return block above involving newDiffBetweenEntityCoords
			// if I am wrong, then uncomment this
//			else {
//				double d1 = availableWidth;
//				double d2 = requestedWidth;
//				double d = d1 / d2;
//
//				for (Text text : textInInterval) {
//					text.setStartCoord(prevEndCoord);
//					double adjustedTextWidth = fontMetrics.stringWidth(text
//							.getText()) * d - 3;
//					prevEndCoord = prevEndCoord + BUFFER + (int) adjustedTextWidth;
//				}
//			}
		}
	}

	private static void moveTextBackward(double from, double to, GestureField gestField,
			GestureEventsEntity entity, MediaToolBar bar) {
		// from: where entity end point started (old from)
		// to: where entity end point is desired to go
		// entity: its end point has already moved from --> to, so the end is here the (new from)
				
		FontMetrics fontMetrics = bar.getFontMetrics(SegmentGraphReprUtil
				.getGraphReprFont());
		
		GestureEnteredText enteredText = gestField
				.getGestureEventsEntityCollection().getEnteredText();

//		EventsEntity selectedEntity = bar.getSelectedEventsEntity();		
//		EventsEntity parentEntity = selectedEntity.getParentEntity();
		GestureEventsEntity prevEntity = entity.getPreviousEntity();
		
		ArrayList<GestureText> textInEndPointsInterval = enteredText
				.getSortedEnteredTextInInteval2(to, from); // since text going backwards, need to reverse from/to

		if (!textInEndPointsInterval.isEmpty()) {
//			double minStartCoord = parentEntity
//					.getStartTimeCoordinate();
			double minStartCoord = TimeRange.ABSOLUTE.start().value();		// GREG TODO: not sure if valid
			if (prevEntity != null)  {
				minStartCoord = prevEntity.getEndTimeCoordinate();
				// changed this from startCoord to endCoord - should be no difference because no text should be within a gloss
			}

			ArrayList<GestureText> textInInterval = enteredText
					.getSortedEnteredTextInInteval2(minStartCoord, from);

			int cumulativeWidth = 0;
			int dCumulativeWidth = Util.MTB_TEXT_BUFFER;
			int textSize = textInInterval.size();
			
			// loop through texts to get their cumulative width
			for (int i = 0; i < textSize; i++) {
				GestureText t = textInInterval.get(i);
				String txt = t.getText();
				int width = fontMetrics.stringWidth(txt);
				cumulativeWidth = cumulativeWidth + width;
				dCumulativeWidth = dCumulativeWidth + Util.MTB_TEXT_BUFFER;
			}

			dCumulativeWidth = dCumulativeWidth + Util.MTB_TEXT_BUFFER;

			double availableWidth = to - minStartCoord;
			double requestedWidth = cumulativeWidth;
			double prevStartCoord = to - Util.MTB_TEXT_BUFFER;			

			if (availableWidth >= requestedWidth) {
				for (int i = textSize - 1; i >= 0; i--) {	// need to loop texts in reverse in setting start/end times
					GestureText text = textInInterval.get(i);
					text.setEndCoord(prevStartCoord);
					double startCoord = prevStartCoord
							- fontMetrics.stringWidth(text.getText());
					text.setStartCoord(startCoord);
					prevStartCoord = prevStartCoord
							- fontMetrics.stringWidth(text.getText()) - Util.MTB_TEXT_BUFFER;
				}
			} 
			// this else block is now obsolete due to the return block above involving newDiffBetweenEntityCoords
			// if I am wrong, then uncomment this
//			else {
//				double d1 = availableWidth;
//				double d2 = requestedWidth;
//				double d = d1 / d2;
//
//				for (Text text : textInInterval) {
//					text.setEndCoord(prevStartCoord);
//					double adjustedTextWidth = fontMetrics.stringWidth(text
//							.getText()) * d - 3;
//					prevStartCoord = prevStartCoord - BUFFER - (int) adjustedTextWidth;
//				}
//			}
		}
	}

	private static void moveTextBackward(double from, double to, GlossField glsField,
			GlossChainedEventsEntity entity, MediaToolBar bar) {
		// from: where entity end point started (old from)
		// to: where entity end point is desired to go
		// entity: its end point has already moved from --> to, so the end is here the (new from)
				
		FontMetrics fontMetrics = bar.getFontMetrics(SegmentGraphReprUtil
				.getGraphReprFont());
		
		EnteredText enteredText = glsField
				.getGlossChainedEventsEntityCollection().getEnteredText();

//		EventsEntity selectedEntity = bar.getSelectedEventsEntity();		
//		EventsEntity parentEntity = selectedEntity.getParentEntity();
		GlossChainedEventsEntity prevEntity = entity.getPreviousEntity();
		
		ArrayList<Text> textInEndPointsInterval = enteredText
				.getSortedEnteredTextInInteval2(to, from); // since text going backwards, need to reverse from/to

		if (!textInEndPointsInterval.isEmpty()) {
//			double minStartCoord = parentEntity
//					.getStartTimeCoordinate();
			double minStartCoord = TimeRange.ABSOLUTE.start().value();		// GREG TODO: not sure if valid
			if (prevEntity != null)  {
				minStartCoord = prevEntity.getEndTimeCoordinate();
				// changed this from startCoord to endCoord - should be no difference because no text should be within a gloss
			}

			ArrayList<Text> textInInterval = enteredText
					.getSortedEnteredTextInInteval2(minStartCoord, from);

			int cumulativeWidth = 0;
			int dCumulativeWidth = Util.MTB_TEXT_BUFFER;
			int textSize = textInInterval.size();
			
			// loop through texts to get their cumulative width
			for (int i = 0; i < textSize; i++) {
				Text t = textInInterval.get(i);
				String txt = t.getText();
				int width = fontMetrics.stringWidth(txt);
				cumulativeWidth = cumulativeWidth + width;
				dCumulativeWidth = dCumulativeWidth + Util.MTB_TEXT_BUFFER;
			}

			dCumulativeWidth = dCumulativeWidth + Util.MTB_TEXT_BUFFER;

			double availableWidth = to - minStartCoord;
			double requestedWidth = cumulativeWidth;
			double prevStartCoord = to - Util.MTB_TEXT_BUFFER;			

			if (availableWidth >= requestedWidth) {
				for (int i = textSize - 1; i >= 0; i--) {	// need to loop texts in reverse in setting start/end times
					Text text = textInInterval.get(i);
					text.setEndCoord(prevStartCoord);
					double startCoord = prevStartCoord
							- fontMetrics.stringWidth(text.getText());
					text.setStartCoord(startCoord);
					prevStartCoord = prevStartCoord
							- fontMetrics.stringWidth(text.getText()) - Util.MTB_TEXT_BUFFER;
				}
			} 
			// this else block is now obsolete due to the return block above involving newDiffBetweenEntityCoords
			// if I am wrong, then uncomment this
//			else {
//				double d1 = availableWidth;
//				double d2 = requestedWidth;
//				double d = d1 / d2;
//
//				for (Text text : textInInterval) {
//					text.setEndCoord(prevStartCoord);
//					double adjustedTextWidth = fontMetrics.stringWidth(text
//							.getText()) * d - 3;
//					prevStartCoord = prevStartCoord - BUFFER - (int) adjustedTextWidth;
//				}
//			}
		}
	}

	public static boolean setSelectedEventStartTime(Time time) {
        var bar = SS3Singleton.getMediaToolBar();

		boolean success = MediaToolbarCommons.setSelectedObjectStartTime(bar, time);

        if (success)
    		SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);

		bar.selectedEntity().updateParentEntity();

		return success;
	}

	
	public static boolean setSelectedEventEndTime(Time time) {
        var bar = SS3Singleton.getMediaToolBar();

        boolean success = MediaToolbarCommons.setSelectedObjectEndTime(bar, time);

        if (success)
    		SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);

        bar.selectedEntity().updateParentEntity();

        return success;
    }


	/**
	 * This method sets selected object's start time to current movie time. If
	 * this operation cannot be performed, warning message will be shown.
	 */
	private static boolean setSelectedObjectStartTime(MediaToolBar bar, Time timeInfo) {

		Field field = bar.selectedField();
		Event event = bar.selectedEvent();
		EventsEntity selectedEntity = bar.selectedEntity();
//		EventsEntity parentEntity = selectedEntity.getParentEntity();
		SignStreamSegmentPanel panel = bar.segmentPanel();

		if (selectedEntity instanceof PresentationEventsEntity) {

//            PresentationEventsEntity pEntity = (PresentationEventsEntity) selectedEntity;
//            MediaToolbarCommons.setPresentationEventsEntityStartTime(timeInfo,
//                pEntity);

		} else if (field instanceof GlossField) {
			GlossField glsField = (GlossField) field;
//			if (parentEntity != null) {
//				if (parentEntity.getStartTimeInfo().greaterThan(timeInfo)
//						|| parentEntity.getEndTimeInfo().lessThan(timeInfo)) {
//					SS3Singleton.getErrorMessages().showErrorMessage(
//							ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE);
//					return false;
//				}
//			}

			GlossChainedEventsEntity glsEntity = (GlossChainedEventsEntity) selectedEntity;

			double moveFromCoord = glsEntity.getStartTimeCoordinate();

			if (true) {
				
				CoordinateTimeConverter convertor = bar.segmentPanel().getZoomer()
						.getCoordinateTimeConvertor();			
				double moveToCoord = convertor.convertTimeInfoToCoordinate(timeInfo);
				
				moveTextBackward(moveFromCoord, moveToCoord,
						glsField, glsEntity, bar);
				
				String resp = glsEntity.canSetSelectedEventStartMovieTime(timeInfo);

				if (resp != null) {
					SS3Singleton.getErrorMessages().showErrorMessage(resp);
					return false;
				}
				
				ChainedEvent onset = glsEntity.getOnset();
				ChainedEvent offset = glsEntity.getOffset();
				ChainedEvent initialHold = glsEntity.getInitialHold();
				ChainedEvent finalHold = glsEntity.getFinalHold();
				ChainedEvent text = glsEntity.getTextValue();
				GlossChainedEventsEntity depGlsEntity = glsEntity
						.getDependentGlossEntity();

				ChainedEventsEntity handEntity = glsEntity
						.getDependentHandEntity();

				HandShapeEntity handShapeEntity = glsEntity
						.getDependentHandShapeEntity();

				ChainedEvent ndOffset = null;

				boolean glsLockedGlossPhoneAlignments = glsEntity
						.isLockedGlossPhonAlignments();
				boolean glsLockedDomNdomAlignments = glsEntity
						.isLockedDomNdomAlignments();
				boolean ndGlsLockedGlossPhoneAlignments = false;
				if (depGlsEntity != null) {
					ndGlsLockedGlossPhoneAlignments = depGlsEntity
							.isLockedGlossPhonAlignments();
					ndOffset = glsEntity.getDependentGlossEntity().getOffset();
				}

				if (event.equals(onset)) {
					if (glsLockedDomNdomAlignments && depGlsEntity != null
							&& depGlsEntity.getOnset() != null) {
						ChainedEvent ndOnset = depGlsEntity.getOnset();
						GlossChainedEventsEntity prevEntity2 = depGlsEntity
								.getPreviousEntity();

						if (prevEntity2 != null) {
							double mTime = prevEntity2.getEndTimeInfo()
									.doubleValue();
							if (prevEntity2.getEndTimeInfo().greaterThan(timeInfo)) {
								SS3Singleton
										.getErrorMessages()
										.showErrorMessage(
												ErrorMessages.CHAINS_CANNOT_OVERLAP);
								return false;
							} else {
								ndOnset.setStartTimeInfo(timeInfo);
							}
						} else {
							ndOnset.setStartTimeInfo(timeInfo);
						}
					}
				} else if (event.equals(initialHold)) {
                    Time ndMaxInitialHoldStartTimeInfo =
                        initialHold.getEndTimeInfo().subtract(Duration.ts(panel.getMultipleMovieController().getPacketDuration()));
//					TimeInfo ndMaxInitialHoldStartTimeInfo
//						= new TimeInfo(initialHold.getEndTimeInfo().value()
//							- panel.getMultipleMovieController().getPacketDuration(),
//							initialHold.getEndTimeInfo().unit());
					if (timeInfo.greaterThan(ndMaxInitialHoldStartTimeInfo)) {
						SS3Singleton
								.getErrorMessages()
								.showErrorMessage(
										ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME);
						return false;
					}
					
					if (glsLockedDomNdomAlignments) {
						if (depGlsEntity != null
								&& depGlsEntity.getInitialHold() != null) {
							ChainedEvent ndInitialHold = depGlsEntity
									.getInitialHold();
							ChainedEvent ndOnset = depGlsEntity.getOnset();
							if (ndOnset != null) {
                                Time minOnsetEndTimeInfo =
                                    ndOnset.getStartTimeInfo().add(Duration.ts(panel.getMultipleMovieController().getPacketDuration()));
//								TimeInfo minOnsetEndTimeInfo = new TimeInfo(
//										ndOnset.getStartTimeInfo().value()
//										+ panel.getMultipleMovieController().getPacketDuration(),
//										ndOnset.getStartTimeInfo().unit());

								if (minOnsetEndTimeInfo.greaterThan(timeInfo)) {
									SS3Singleton
											.getErrorMessages()
											.showErrorMessage(
													ErrorMessages.ADD_INITIAL_HOLD_CANT_ADJUST_ONSET);
									return false;
								} else {
									ndOnset.setEndTimeInfo(timeInfo);
									ndInitialHold.setStartTimeInfo(timeInfo);

									if (depGlsEntity
											.isLockedGlossPhonAlignments()) {
										ChainedEventsEntity depHandEntity = depGlsEntity
												.getDependentHandEntity();
										HandShapeEntity depHandShapeEntity = depGlsEntity
												.getDependentHandShapeEntity();

										if (depHandEntity != null) {

											ChainedEvent textValue = depHandEntity
													.getTextValue();
											ChainedEventsEntity prevHandEntity = depHandEntity
													.getPreviousEntity();

											if (prevHandEntity == null) {
												textValue.setStartTimeInfo(timeInfo);
											} else {
												double prevEntMovieTime = prevHandEntity
														.getEndTimeInfo()
														.doubleValue();
												if (timeInfo.lessThan(prevHandEntity.getEndTimeInfo())) {
													SS3Singleton
															.getErrorMessages()
															.showErrorMessage(
																	ErrorMessages.CHAINS_CANNOT_OVERLAP);
													return false;
												} else {
													textValue.setStartTimeInfo(timeInfo);
												}
											}
											if (depHandShapeEntity != null) {
												setHandshapeStartTime(
														depHandShapeEntity,
														timeInfo);
											}
										}
									}
								}
							} else {
								GlossChainedEventsEntity prevEntity1 = depGlsEntity
										.getPreviousEntity();
								if (prevEntity1 != null) {
									double prevEndMovieTime = prevEntity1
											.getEndTimeInfo().doubleValue();
									if (timeInfo.lessThan(prevEntity1.getEndTimeInfo())) {
										SS3Singleton
												.getErrorMessages()
												.showErrorMessage(
														ErrorMessages.CHAINS_CANNOT_OVERLAP);
										return false;
									} else {
										ndInitialHold.setStartTimeInfo(timeInfo);
									}
								} else {
									ndInitialHold.setStartTimeInfo(timeInfo);
								}

								if (ndGlsLockedGlossPhoneAlignments) {
									ChainedEventsEntity depHandEntity = depGlsEntity
											.getDependentHandEntity();
									HandShapeEntity depHandShapeEntity = depGlsEntity
											.getDependentHandShapeEntity();

									if (depHandEntity != null) {
										ChainedEvent textValue = depHandEntity
												.getTextValue();
										ChainedEventsEntity prevHandEntity = depHandEntity
												.getPreviousEntity();
										if (prevHandEntity == null) {
											textValue.setStartTimeInfo(timeInfo);
										} else {
											double prevEntMovieTime = prevHandEntity
													.getEndTimeInfo()
													.doubleValue();
											if (timeInfo.lessThan(prevHandEntity.getEndTimeInfo())) {
												SS3Singleton
														.getErrorMessages()
														.showErrorMessage(
																ErrorMessages.CHAINS_CANNOT_OVERLAP);
												return false;
											}

											textValue.setStartTimeInfo(timeInfo);
										}

										if (depHandShapeEntity != null) {
											setHandshapeStartTime(
													depHandShapeEntity, timeInfo);
										}
									}
								}
							}
						}
					}
					
					if (glsLockedGlossPhoneAlignments) {
						if (handEntity != null) {
							ChainedEvent textValue = handEntity.getTextValue();
							ChainedEventsEntity prevHandEntity = handEntity
									.getPreviousEntity();

							if (prevHandEntity == null) {
								textValue.setStartTimeInfo(timeInfo);
							} else {
								double prevEntMovieTime = prevHandEntity
										.getEndTimeInfo().doubleValue();
								if (timeInfo.lessThan(prevHandEntity.getEndTimeInfo())) {
									SS3Singleton
											.getErrorMessages()
											.showErrorMessage(
													ErrorMessages.CHAINS_CANNOT_OVERLAP);
									return false;
								} else {
									textValue.setStartTimeInfo(timeInfo);
								}
							}
						}
						if (handShapeEntity != null) {
							setHandshapeStartTime(handShapeEntity, timeInfo);
						}
					}
				} else if (event.equals(text)) {
					if (glsLockedDomNdomAlignments && depGlsEntity != null) {

						ChainedEvent depOnset = depGlsEntity.getOnset();
						ChainedEvent depInitialHold = depGlsEntity
								.getInitialHold();

						if (depInitialHold != null) {
							double minEndTime = depInitialHold.getStartTimeInfo()
									.doubleValue()
									+ panel.getMultipleMovieController()
											.getPacketDuration();
                            Time minEndTimeInfo =
                                depInitialHold.getStartTimeInfo().add(Duration.ts(panel.getMultipleMovieController().getPacketDuration()));
//							TimeInfo minEndTimeInfo = new TimeInfo(
//									depInitialHold.getStartTimeInfo().value()
//										+ panel.getMultipleMovieController().getPacketDuration(),
//									depInitialHold.getStartTimeInfo().unit());
							if (minEndTimeInfo.greaterThan(timeInfo)) {
								SS3Singleton
										.getErrorMessages()
										.showErrorMessage(
												ErrorMessages.SET_START_VALUE__CANT_DELETE_INITIAL_HOLD);
								return false;
							} else {
								depGlsEntity.getTextValue().setStartTimeInfo(timeInfo);
								depInitialHold.setEndTimeInfo(timeInfo);
							}
						} else if (depOnset != null) {
							double minEndTime = depOnset.getStartTimeInfo()
									.doubleValue()
									+ panel.getMultipleMovieController()
											.getPacketDuration();
							Time minEndTimeInfo =
                                depOnset.getStartTimeInfo().add(Duration.ts(panel.getMultipleMovieController().getPacketDuration()));
//							TimeInfo minEndTimeInfo = new TimeInfo(
//									depOnset.getStartTimeInfo().value()
//										+ panel.getMultipleMovieController().getPacketDuration(),
//									depOnset.getStartTimeInfo().unit());
							if (minEndTimeInfo.greaterThan(timeInfo)) {
								SS3Singleton
										.getErrorMessages()
										.showErrorMessage(
												ErrorMessages.SET_START_VALUE_CANT_DELETE_ONSET);
								return false;
							} else {
								depGlsEntity.getTextValue().setStartTimeInfo(timeInfo);
								depOnset.setEndTimeInfo(timeInfo);

								ChainedEventsEntity depHandEntity = depGlsEntity
										.getDependentHandEntity();
								HandShapeEntity depHandShapeEntity = depGlsEntity
										.getDependentHandShapeEntity();

								if (depGlsEntity.isLockedGlossPhonAlignments()) {
									if (depHandEntity != null) {
										ChainedEventsEntity depHandPrevEntity = depHandEntity
												.getPreviousEntity();
										if (depHandPrevEntity == null) {
											depHandEntity.setStartTimeInfo(timeInfo);
										} else if (depHandPrevEntity
												.getEndTimeInfo().greaterThan(timeInfo)) {
											SS3Singleton
													.getErrorMessages()
													.showErrorMessage(
															ErrorMessages.CHAINS_CANNOT_OVERLAP);
											return false;
										} else {
											depHandEntity.setStartTimeInfo(timeInfo);
										}

										if (depHandShapeEntity != null) {
											setHandshapeStartTime(
													depHandShapeEntity, timeInfo);
										}
									}
								}
							}
						} else {
							depGlsEntity.getTextValue().setStartTimeInfo(timeInfo);
							if (depGlsEntity.isLockedGlossPhonAlignments()) {
								ChainedEventsEntity depHandEntity = depGlsEntity
										.getDependentHandEntity();
								HandShapeEntity depHandShapeEntity = depGlsEntity
										.getDependentHandShapeEntity();

								if (depHandEntity != null) {
									ChainedEventsEntity depHandPrevEntity = depHandEntity
											.getPreviousEntity();
									if (depHandPrevEntity == null) {
										depHandEntity.setStartTimeInfo(timeInfo);
									} else if (depHandPrevEntity
											.getEndTimeInfo().greaterThan(timeInfo)) {
										SS3Singleton
												.getErrorMessages()
												.showErrorMessage(
														ErrorMessages.CHAINS_CANNOT_OVERLAP);
										return false;
									} else {
										depHandEntity.setStartTimeInfo(timeInfo);
									}

									if (depHandShapeEntity != null) {
										setHandshapeStartTime(
												depHandShapeEntity, timeInfo);
									}
								}
							}
						}
					}
					
					if (initialHold == null && glsLockedGlossPhoneAlignments) {
						if (handEntity != null) {
							ChainedEventsEntity prevEntity3 = handEntity
									.getPreviousEntity();
							if (prevEntity3 == null) {
								handEntity.setStartTimeInfo(timeInfo);
							} else if (prevEntity3.getEndTimeInfo().greaterThan(timeInfo)) {
								SS3Singleton
								.getErrorMessages()
								.showErrorMessage(
										ErrorMessages.SET_START_VALUE__CANT_DELETE_INITIAL_HOLD);
								return false;
							} else {
								handEntity.setStartTimeInfo(timeInfo);
							}

							if (handShapeEntity != null) {
								setHandshapeStartTime(handShapeEntity, timeInfo);
							}
						}
					}
				} else if (event.equals(offset)) {

					if (depGlsEntity != null && ndOffset != null
							&& glsLockedDomNdomAlignments) {
						ChainedEvent ndFinalHold = depGlsEntity.getFinalHold();
						GlossChainedEvent ndValue = depGlsEntity
								.getGlossChainedEvent();

						int packetDuration = panel
								.getMultipleMovieController()
								.getPacketDuration();

						double maxStartTimcketDuration;
						Time maxStartTimeInfo =
                            ndOffset.getEndTimeInfo().subtract(Duration.ts(packetDuration));
//						TimeInfo maxStartTimeInfo = new TimeInfo(
//								ndOffset.getEndTimeInfo().value()
//									- packetDuration,
//								ndOffset.getEndTimeInfo().unit());

						if (timeInfo.lessThanOrEq(maxStartTimeInfo)) {
							if (ndFinalHold != null) {
								Time minHoldEndTimeInfo =
                                    ndFinalHold.getStartTimeInfo().add(Duration.ts(packetDuration));
//								TimeInfo minHoldEndTimeInfo = new TimeInfo(
//										ndFinalHold.getStartTimeInfo().value()
//											+ packetDuration,
//										ndFinalHold.getStartTimeInfo().unit());
								if (minHoldEndTimeInfo.lessThanOrEq(timeInfo)) {
									ndOffset.setStartTimeInfo(timeInfo);
									ndFinalHold.setEndTimeInfo(timeInfo);

									if (depGlsEntity
											.isLockedGlossPhonAlignments()) {
										ChainedEventsEntity ndDepHandEntity = depGlsEntity
												.getDependentHandEntity();
										HandShapeEntity ndDepHandShapeEntity = depGlsEntity
												.getDependentHandShapeEntity();
										if (ndDepHandEntity != null) {
											ChainedEventsEntity ndNextEntity = ndDepHandEntity
													.getNextEntity();
											if (ndNextEntity == null) {
												ndDepHandEntity.getTextValue().setEndTimeInfo(timeInfo);
												if (ndDepHandShapeEntity != null) {
													setHandshapeEndTime(
															ndDepHandShapeEntity,
															timeInfo);
												}
											} else {
												double nextEntityMovieTime = ndNextEntity
														.getStartTimeInfo()
														.doubleValue();
												if (ndNextEntity.getStartTimeInfo().lessThan(timeInfo)) {
													SS3Singleton
															.getErrorMessages()
															.showErrorMessage(
																	ErrorMessages.GLOSS_CHAINS_CANNOT_OVERLAP);
													return false;
												} else {
													ndDepHandEntity
															.getTextValue().setEndTimeInfo(timeInfo);
													if (ndDepHandShapeEntity != null) {
														setHandshapeStartTime(
																ndDepHandShapeEntity,
																timeInfo);
													}
												}
											}
										}
									}
								} else {
									SS3Singleton
											.getErrorMessages()
											.showErrorMessage(
													ErrorMessages.SET_START_OFFSET_CANT_DELETE_FINAL_HOLD);
									return false;
								}
							} else {
                                Time minEndTimeInfo =
                                    ndValue.getStartTimeInfo().add(Duration.ts(packetDuration));
								if (minEndTimeInfo.lessThanOrEq(timeInfo)) {
									ndOffset.setStartTimeInfo(timeInfo);

									if (depGlsEntity
											.isLockedGlossPhonAlignments()) {

										ChainedEventsEntity ndDepHandEntity = depGlsEntity
												.getDependentHandEntity();
										HandShapeEntity ndDepHandShapeEntity = depGlsEntity
												.getDependentHandShapeEntity();
										if (ndDepHandEntity != null) {
											ChainedEventsEntity ndNextEntity = ndDepHandEntity
													.getNextEntity();
											if (ndNextEntity == null) {
												ndDepHandEntity.getTextValue().setEndTimeInfo(timeInfo);
												if (ndDepHandShapeEntity != null) {
													setHandshapeEndTime(
															ndDepHandShapeEntity,
															timeInfo);
												}
											} else {
												double nextEntityMovieTime = ndNextEntity
														.getStartTimeInfo()
														.doubleValue();
												if (ndNextEntity.getStartTimeInfo().lessThan(timeInfo)) {
													SS3Singleton
															.getErrorMessages()
															.showErrorMessage(
																	ErrorMessages.GLOSS_CHAINS_CANNOT_OVERLAP);
													return false;
												} else {
													ndDepHandEntity
															.getTextValue().setEndTimeInfo(timeInfo);
													if (ndDepHandShapeEntity != null) {
														setHandshapeStartTime(
																ndDepHandShapeEntity,
																timeInfo);
													}
												}
											}
										}
									}

									ndValue.setEndTimeInfo(timeInfo);
									depGlsEntity.setOffsetEvent(ndOffset);
								} else {
									SS3Singleton
											.getErrorMessages()
											.showErrorMessage(
													ErrorMessages.SET_START_OFFSET_CANT_DELETE_VALUE);
									return false;
								}
							}
						} else {
							SS3Singleton
									.getErrorMessages()
									.showErrorMessage(
											ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME);
							return false;
						}
					}

					if (glsLockedGlossPhoneAlignments) {
						if (handEntity != null) {
							ChainedEventsEntity nextHandEntity = handEntity
									.getNextEntity();
							if (nextHandEntity == null) {
								handEntity.setEndTimeInfo(timeInfo);
							} else if (nextHandEntity.getStartTimeInfo().lessThan(timeInfo)) {
								SS3Singleton
										.getErrorMessages()
										.showErrorMessage(
												ErrorMessages.GLOSS_CHAINS_CANNOT_OVERLAP);
								return false;
							} else {
								handEntity.setStartTimeInfo(timeInfo);
							}

							if (handShapeEntity != null) {
								setHandshapeEndTime(handShapeEntity, timeInfo);
							}
						}
					}
				} else if (event.equals(finalHold)) {
					if (glsLockedDomNdomAlignments && depGlsEntity != null
							&& depGlsEntity.getFinalHold() != null) {
						ChainedEvent ndFinalHold = depGlsEntity.getFinalHold();
						GlossChainedEvent ndValue = depGlsEntity
								.getGlossChainedEvent();

						int packetDuration = panel
								.getMultipleMovieController()
								.getPacketDuration();

						Time minValueEndTimeInfo =
                            ndValue.getStartTimeInfo().add(Duration.ts(packetDuration));
//						TimeInfo minValueEndTimeInfo = new TimeInfo(
//								ndValue.getStartTimeInfo().value()
//									+ packetDuration,
//								ndValue.getStartTimeInfo().unit());
						Time maxHoldStartTimeInfo =
                            ndFinalHold.getEndTimeInfo().subtract(Duration.ts(packetDuration));
//						TimeInfo maxHoldStartTimeInfo = new TimeInfo(
//								ndFinalHold.getEndTimeInfo().value()
//									- packetDuration,
//								ndFinalHold.getEndTimeInfo().unit());

						if (minValueEndTimeInfo.greaterThan(timeInfo)) {
							SS3Singleton
									.getErrorMessages()
									.showErrorMessage(
											ErrorMessages.SET_START_FINAL_HOLD_CANT_DELETE_VALUE);
							return false;
						} else if (timeInfo.greaterThan(maxHoldStartTimeInfo)) {
							SS3Singleton
									.getErrorMessages()
									.showErrorMessage(
											ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME);
							return false;
						} else {
							ndFinalHold.setStartTimeInfo(timeInfo);
							ndValue.setEndTimeInfo(timeInfo);
						}
					}
				}

				glsEntity.setGlossEventsEntityStartTimeInfo(timeInfo);

				LocationEntity depLocEntity = glsEntity.getDependentLocationEntity();
				if (depLocEntity != null) {
					depLocEntity.updateTimes();
				} else if (glsLockedDomNdomAlignments && depGlsEntity != null) {
					depLocEntity = depGlsEntity.getDependentLocationEntity();
					if (depLocEntity != null) {
						depLocEntity.updateTimes();
					}
				}
				SignTypeEntity signTypeEntity = glsEntity.getDependentSignTypeEntity();
				if (signTypeEntity != null) {
					signTypeEntity.updateTimes();
				} else if (glsLockedDomNdomAlignments && depGlsEntity != null) {
					signTypeEntity = depGlsEntity.getDependentSignTypeEntity();
					if (signTypeEntity != null) {
						signTypeEntity.updateTimes();
					}
				}
			}
		} else if (field instanceof GestureField) {
			GestureField gestField = (GestureField) field;
//			if (parentEntity != null) {
//				if (parentEntity.getStartTimeInfo().greaterThan(timeInfo)
//						|| parentEntity.getEndTimeInfo().lessThan(timeInfo)) {
//					SS3Singleton.getErrorMessages().showErrorMessage(
//							ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE);
//					return false;
//				}
//			}

			GestureEventsEntity gestEntity = (GestureEventsEntity) selectedEntity;

			double moveFromCoord = gestEntity.getStartTimeCoordinate();

			if (true) {
				CoordinateTimeConverter convertor = bar.segmentPanel().getZoomer()
						.getCoordinateTimeConvertor();			
				double moveToCoord = convertor.convertTimeInfoToCoordinate(timeInfo);
				
				moveTextBackward(moveFromCoord, moveToCoord,
						gestField, gestEntity, bar);
				
				String resp = gestEntity.canSetSelectedEventStartMovieTime(timeInfo);

				if (resp != null) {
					SS3Singleton.getErrorMessages().showErrorMessage(resp);
					return false;
				}
				
				GestureChainedEvent preparation = gestEntity.getPreparation();
				GestureChainedEvent retraction = gestEntity.getRetraction();
				GestureChainedEvent prestrokeHold = gestEntity.getPrestrokeHold();
				GestureChainedEvent poststrokeHold = gestEntity.getPoststrokeHold();
				GestureChainedEvent relaxed = gestEntity.getRelaxed();
				GestureChainedEvent text = gestEntity.getTextValue();
				ApexChainedApexEventsEntity apexEntity = gestEntity.getDependentApexEntity();

				GestureEventsEntity depGestEntity = gestEntity
						.getDependentGestureEntity();

				GestureChainedEventsEntity handEntity = gestEntity
						.getDependentHandEntity();

				GestureHandShapeEntity handShapeEntity = gestEntity
						.getDependentHandShapeEntity();

				GestureOrientationEntity orientationEntity = gestEntity
						.getDependentOrientationEntity();

				GestureChainedEvent ndPreparation = null;
				GestureChainedEvent ndRetraction = null;
				GestureChainedEvent ndPrestrokeHold = null;
				GestureChainedEvent ndPoststrokeHold = null;
				GestureChainedEvent ndRelaxed = null;
				GestureChainedEvent ndText = null;
				ApexChainedApexEventsEntity ndApexEntity = null;

				boolean gestLockedGlossPhoneAlignments = gestEntity
						.isLockedGesturePhonAlignments();
				boolean gestLockedDomNdomAlignments = gestEntity
						.isLockedDomNdomAlignments();
				boolean ndGestLockedGesturePhoneAlignments = false;
				if (depGestEntity != null) {
					ndGestLockedGesturePhoneAlignments = depGestEntity.isLockedGesturePhonAlignments();
					ndPreparation = depGestEntity.getPreparation();
					ndRetraction = depGestEntity.getRetraction();
					ndPrestrokeHold = depGestEntity.getPrestrokeHold();
					ndPoststrokeHold = depGestEntity.getPoststrokeHold();
					ndRelaxed = depGestEntity.getRelaxed();
					ndText = depGestEntity.getTextValue();
					ndApexEntity = depGestEntity.getDependentApexEntity();
				}

				if (event.equals(preparation)) {
					if (gestLockedDomNdomAlignments && depGestEntity != null
							&& depGestEntity.getPreparation() != null) {
						GestureEventsEntity prevEntity2 = depGestEntity
								.getPreviousEntity();

						if (prevEntity2 != null) {
							double mTime = prevEntity2.getEndTimeInfo()
									.doubleValue();
							if (prevEntity2.getEndTimeInfo().greaterThan(timeInfo)) {
								SS3Singleton
										.getErrorMessages()
										.showErrorMessage(
												ErrorMessages.GESTURE_CHAINS_CANNOT_OVERLAP);
								return false;
							} else {
								ndPreparation.setStartTimeInfo(timeInfo);
							}
						} else {
							ndPreparation.setStartTimeInfo(timeInfo);
						}
					}
				} else if (event.equals(prestrokeHold)) {
					Time ndMaxInitialHoldStartTimeInfo =
                        prestrokeHold.getEndTimeInfo().subtract(Duration.ts(panel.getMultipleMovieController().getPacketDuration()));
//					TimeInfo ndMaxInitialHoldStartTimeInfo
//						= new TimeInfo(initialHold.getEndTimeInfo().value()
//							- panel.getMultipleMovieController().getPacketDuration(),
//							initialHold.getEndTimeInfo().unit());
					if (timeInfo.greaterThan(ndMaxInitialHoldStartTimeInfo)) {
						SS3Singleton
								.getErrorMessages()
								.showErrorMessage(
										ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME);
						return false;
					}
					
					if (gestLockedDomNdomAlignments) {
						if (depGestEntity != null
								&& depGestEntity.getPrestrokeHold() != null) {
							if (ndPreparation != null) {
								Time minOnsetEndTimeInfo =
                                    ndPreparation.getStartTimeInfo().add(Duration.ts(panel.getMultipleMovieController().getPacketDuration()));
//								TimeInfo minOnsetEndTimeInfo = new TimeInfo(
//										ndOnset.getStartTimeInfo().value()
//										+ panel.getMultipleMovieController().getPacketDuration(),
//										ndOnset.getStartTimeInfo().unit());

								if (minOnsetEndTimeInfo.greaterThan(timeInfo)) {
									SS3Singleton
											.getErrorMessages()
											.showErrorMessage(
													ErrorMessages.ADD_PRESTROKE_HOLD_CANT_ADJUST_PREPARATION);
									return false;
								} else {
									ndPreparation.setEndTimeInfo(timeInfo);
									ndPrestrokeHold.setStartTimeInfo(timeInfo);

									if (depGestEntity
											.isLockedGesturePhonAlignments()) {
										GestureChainedEventsEntity depHandEntity = depGestEntity
												.getDependentHandEntity();
										GestureHandShapeEntity depHandShapeEntity = depGestEntity
												.getDependentHandShapeEntity();
										GestureOrientationEntity depOrientationEntity = depGestEntity
												.getDependentOrientationEntity();

										// GD 2021-03-06: do not update gesture hand/shape entities to the final/initial holds
//										if (depHandEntity != null) {
//
//											GestureChainedEvent textValue = depHandEntity
//													.getTextValue();
//											GestureChainedEventsEntity prevHandEntity = depHandEntity
//													.getPreviousEntity();
//
//											if (prevHandEntity == null) {
//												textValue.setStartTimeInfo(timeInfo);
//											} else {
//												double prevEntMovieTime = prevHandEntity
//														.getEndTimeInfo()
//														.doubleValue();
//												if (timeInfo.lessThan(prevHandEntity.getEndTimeInfo())) {
//													SS3Singleton
//															.getErrorMessages()
//															.showErrorMessage(
//																	ErrorMessages.CHAINS_CANNOT_OVERLAP);
//													return false;
//												} else {
//													textValue.setStartTimeInfo(timeInfo);
//												}
//											}
//											if (depHandShapeEntity != null) {
//												setHandshape_XYZStartTime(
//														depHandShapeEntity,
//														timeInfo);
//											}
//										}
									}
								}
							} else {
								GestureEventsEntity prevEntity1 = depGestEntity
										.getPreviousEntity();
								if (prevEntity1 != null) {
									double prevEndMovieTime = prevEntity1
											.getEndTimeInfo().doubleValue();
									if (timeInfo.lessThan(prevEntity1.getEndTimeInfo())) {
										SS3Singleton
												.getErrorMessages()
												.showErrorMessage(
														ErrorMessages.GESTURE_CHAINS_CANNOT_OVERLAP);
										return false;
									} else {
										ndPrestrokeHold.setStartTimeInfo(timeInfo);
									}
								} else {
									ndPrestrokeHold.setStartTimeInfo(timeInfo);
								}

//								if (ndGlsLockedGlossPhoneAlignments) {
//									GestureChainedEventsEntity depHandEntity = depGlsEntity
//											.getDependentHandEntity();
//									HandShapeEntity_XYZ depHandShapeEntity = depGlsEntity
//											.getDependentHandShapeEntity();
//									OrientationEntity_XYZ depOrientationEntity = depGlsEntity
//											.getDependentOrientationEntity();
//
//									if (depHandEntity != null) {
//										depHandEntity.setStartTimeInfo(timeInfo);
//
//										if (depHandShapeEntity != null) {
//											setHandshape_XYZStartTime(
//													depHandShapeEntity, timeInfo);
//										}
//										if (depOrientationEntity != null) {
//											setOrientation_XYZStartTime(
//													depOrientationEntity, timeInfo);
//										}
//									}
//								}
							}
						}
					}
					
//					if (glsLockedGlossPhoneAlignments) {
//						if (handEntity != null) {
//							GestureChainedEvent textValue = handEntity.getTextValue();
//							GestureChainedEventsEntity prevHandEntity = handEntity
//									.getPreviousEntity();
//
//							if (prevHandEntity == null) {
//								textValue.setStartTimeInfo(timeInfo);
//							} else {
//								double prevEntMovieTime = prevHandEntity
//										.getEndTimeInfo().doubleValue();
//								if (timeInfo.lessThan(prevHandEntity.getEndTimeInfo())) {
//									SS3Singleton
//											.getErrorMessages()
//											.showErrorMessage(
//													ErrorMessages.GESTURE_CHAINS_CANNOT_OVERLAP);
//									return false;
//								} else {
//									textValue.setStartTimeInfo(timeInfo);
//								}
//							}
//						}
//						if (handShapeEntity != null) {
//							setHandshape_XYZStartTime(handShapeEntity, timeInfo);
//						}
//						if (orientationEntity != null) {
//							setOrientation_XYZStartTime(orientationEntity, timeInfo);
//						}
//					}
				} else if (event.equals(text)) {
					
					if (apexEntity != null) {
						if (timeInfo.greaterThan(apexEntity.getStartTimeInfo())) {
							SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.GESTURE_APEX_BOUNDS);
							return false;
						}
					}
					
					if (gestLockedDomNdomAlignments && ndApexEntity != null) {
						if (timeInfo.greaterThan(ndApexEntity.getStartTimeInfo())) {
							SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.GESTURE_APEX_BOUNDS);
							return false;
						}
					}
					
					if (gestLockedDomNdomAlignments && depGestEntity != null) {

						GestureChainedEvent depOnset = depGestEntity.getPreparation();
						GestureChainedEvent depPrestrokeHold = depGestEntity
								.getPrestrokeHold();

						if (depPrestrokeHold != null) {
							double minEndTime = depPrestrokeHold.getStartTimeInfo()
									.doubleValue()
									+ panel.getMultipleMovieController()
											.getPacketDuration();
							Time minEndTimeInfo =
                                depPrestrokeHold.getStartTimeInfo().add(Duration.ts(panel.getMultipleMovieController().getPacketDuration()));
//							TimeInfo minEndTimeInfo = new TimeInfo(
//									depInitialHold.getStartTimeInfo().value()
//										+ panel.getMultipleMovieController().getPacketDuration(),
//									depInitialHold.getStartTimeInfo().unit());
							if (minEndTimeInfo.greaterThan(timeInfo)) {
								SS3Singleton
										.getErrorMessages()
										.showErrorMessage(
												ErrorMessages.SET_START_VALUE__CANT_DELETE_PRESTROKE_HOLD);
								return false;
							} else {
								depGestEntity.getTextValue().setStartTimeInfo(timeInfo);
								depPrestrokeHold.setEndTimeInfo(timeInfo);

								if (depGestEntity.isLockedGesturePhonAlignments()) {
									GestureChainedEventsEntity depHandEntity = depGestEntity
											.getDependentHandEntity();
									GestureHandShapeEntity depHandShapeEntity = depGestEntity
											.getDependentHandShapeEntity();
									GestureOrientationEntity depOrientationEntity = depGestEntity
											.getDependentOrientationEntity();
									if (depHandEntity != null)
										depHandEntity.setStartTimeInfo(timeInfo);

									if (depHandShapeEntity != null) {
										MediaToolbarCommons.setGestureHandshapeStartTime(
												depHandShapeEntity, timeInfo);
									}
									if (depOrientationEntity != null) {
										MediaToolbarCommons.setGestureOrientationStartTime(
												depOrientationEntity, timeInfo);
									}
								}
							}
						} else if (depOnset != null) {
							double minEndTime = depOnset.getStartTimeInfo()
									.doubleValue()
									+ panel.getMultipleMovieController()
											.getPacketDuration();
							Time minEndTimeInfo =
                                depOnset.getStartTimeInfo().add(Duration.ts(panel.getMultipleMovieController().getPacketDuration()));
//							TimeInfo minEndTimeInfo = new TimeInfo(
//									depOnset.getStartTimeInfo().value()
//										+ panel.getMultipleMovieController().getPacketDuration(),
//									depOnset.getStartTimeInfo().unit());
							if (minEndTimeInfo.greaterThan(timeInfo)) {
								SS3Singleton
										.getErrorMessages()
										.showErrorMessage(
												ErrorMessages.SET_START_VALUE_CANT_DELETE_PREPARATION);
								return false;
							} else {
								depGestEntity.getTextValue().setStartTimeInfo(timeInfo);
								depOnset.setEndTimeInfo(timeInfo);

								GestureChainedEventsEntity depHandEntity = depGestEntity
										.getDependentHandEntity();
								GestureHandShapeEntity depHandShapeEntity = depGestEntity
										.getDependentHandShapeEntity();
								GestureOrientationEntity depOrientationEntity = depGestEntity
										.getDependentOrientationEntity();

								if (depGestEntity.isLockedGesturePhonAlignments()) {
									if (depHandEntity != null) {
										GestureChainedEventsEntity depHandPrevEntity = depHandEntity
												.getPreviousEntity();
										if (depHandPrevEntity == null) {
											depHandEntity.setStartTimeInfo(timeInfo);
										} else if (depHandPrevEntity
												.getEndTimeInfo().greaterThan(timeInfo)) {
											SS3Singleton
													.getErrorMessages()
													.showErrorMessage(
															ErrorMessages.GESTURE_CHAINS_CANNOT_OVERLAP);
											return false;
										} else {
											depHandEntity.setStartTimeInfo(timeInfo);
										}

										if (depOrientationEntity != null) {
											setGestureOrientationStartTime(
													depOrientationEntity, timeInfo);
										}

										if (depHandShapeEntity != null) {
											setGestureHandshapeStartTime(
													depHandShapeEntity, timeInfo);
										}
									}
								}
							}
						} else {
							depGestEntity.getTextValue().setStartTimeInfo(timeInfo);
							if (depGestEntity.isLockedGesturePhonAlignments()) {
								GestureChainedEventsEntity depHandEntity = depGestEntity
										.getDependentHandEntity();
								GestureHandShapeEntity depHandShapeEntity = depGestEntity
										.getDependentHandShapeEntity();
								GestureOrientationEntity depOrientationEntity = depGestEntity
										.getDependentOrientationEntity();

								if (depHandEntity != null) {
									GestureChainedEventsEntity depHandPrevEntity = depHandEntity
											.getPreviousEntity();
									if (depHandPrevEntity == null) {
										depHandEntity.setStartTimeInfo(timeInfo);
									} else if (depHandPrevEntity
											.getEndTimeInfo().greaterThan(timeInfo)) {
										SS3Singleton
												.getErrorMessages()
												.showErrorMessage(
														ErrorMessages.GESTURE_CHAINS_CANNOT_OVERLAP);
										return false;
									} else {
										depHandEntity.setStartTimeInfo(timeInfo);
									}

									if (depHandShapeEntity != null) {
										setGestureHandshapeStartTime(
												depHandShapeEntity, timeInfo);
									}

									if (depOrientationEntity != null) {
										setGestureOrientationStartTime(
												depOrientationEntity, timeInfo);
									}
								}
							}
						}
					}
					
					if (gestLockedGlossPhoneAlignments) {
						if (handEntity != null) {
								GestureChainedEventsEntity prevEntity3 = handEntity
										.getPreviousEntity();
								if (prevEntity3 == null) {
									handEntity.setStartTimeInfo(timeInfo);
								} else if (prevEntity3.getEndTimeInfo().greaterThan(timeInfo)) {
									SS3Singleton
											.getErrorMessages()
											.showErrorMessage(
													ErrorMessages.SET_START_VALUE__CANT_DELETE_PRESTROKE_HOLD);
									return false;
								} else {
									handEntity.setStartTimeInfo(timeInfo);
								}

								if (handShapeEntity != null) {
									setGestureHandshapeStartTime(handShapeEntity, timeInfo);
								}
								if (orientationEntity != null) {
									setGestureOrientationStartTime(orientationEntity, timeInfo);
								}
						}
					}

				} else if (event.equals(retraction)) {
					
					if (apexEntity != null) {
						if (timeInfo.lessThan(apexEntity.getEndTimeInfo())) {
							SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.GESTURE_APEX_BOUNDS);
							return false;
						}
					}
					
					if (gestLockedDomNdomAlignments && ndApexEntity != null) {
						if (timeInfo.lessThan(ndApexEntity.getEndTimeInfo())) {
							SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.GESTURE_APEX_BOUNDS);
							return false;
						}
					}

					if (depGestEntity != null && ndRetraction != null
							&& gestLockedDomNdomAlignments) {
						GestureChainedEvent ndValue = depGestEntity
								.getGestureChainedEvent();

						int packetDuration = panel
								.getMultipleMovieController()
								.getPacketDuration();

						double maxStartTimcketDuration;
						Time maxStartTimeInfo =
                            ndRetraction.getEndTimeInfo().subtract(Duration.ts(packetDuration));
//						TimeInfo maxStartTimeInfo = new TimeInfo(
//								ndOffset.getEndTimeInfo().value()
//									- packetDuration,
//								ndOffset.getEndTimeInfo().unit());

						if (timeInfo.lessThanOrEq(maxStartTimeInfo)) {
							if (ndPoststrokeHold != null) {
								Time minHoldEndTimeInfo =
                                    ndPoststrokeHold.getStartTimeInfo().add(Duration.ts(packetDuration));
//								TimeInfo minHoldEndTimeInfo = new TimeInfo(
//										ndFinalHold.getStartTimeInfo().value()
//											+ packetDuration,
//										ndFinalHold.getStartTimeInfo().unit());
								if (minHoldEndTimeInfo.lessThanOrEq(timeInfo)) {
									ndRetraction.setStartTimeInfo(timeInfo);
									ndPoststrokeHold.setEndTimeInfo(timeInfo);

//									if (depGlsEntity
//											.isLockedGesturePhonAlignments()) {
//										GestureChainedEventsEntity ndDepHandEntity = depGlsEntity
//												.getDependentHandEntity();
//										HandShapeEntity_XYZ ndDepHandShapeEntity = depGlsEntity
//												.getDependentHandShapeEntity();
//										OrientationEntity_XYZ ndDepOrientationEntity = depGlsEntity
//												.getDependentOrientationEntity();
//										if (ndDepHandEntity != null) {
//											GestureChainedEventsEntity ndNextEntity = ndDepHandEntity
//													.getNextEntity();
//											if (ndNextEntity == null) {
//												ndDepHandEntity.getTextValue().setEndTimeInfo(timeInfo);
//												if (ndDepHandShapeEntity != null) {
//													setHandshape_XYZEndTime(
//															ndDepHandShapeEntity,
//															timeInfo);
//												}
//												if (ndDepOrientationEntity != null) {
//													setOrientation_XYZEndTime(
//															ndDepOrientationEntity,
//															timeInfo);
//												}
//											} else {
//												double nextEntityMovieTime = ndNextEntity
//														.getStartTimeInfo()
//														.doubleValue();
//												if (ndNextEntity.getStartTimeInfo().lessThan(timeInfo)) {
//													SS3Singleton
//															.getErrorMessages()
//															.showErrorMessage(
//																	ErrorMessages.GLOSS_CHAINS_CANNOT_OVERLAP);
//													return false;
//												} else {
//													ndDepHandEntity
//															.getTextValue().setEndTimeInfo(timeInfo);
//													if (ndDepHandShapeEntity != null) {
//														setHandshape_XYZStartTime(
//																ndDepHandShapeEntity,
//																timeInfo);
//													}
//													if (ndDepOrientationEntity != null) {
//														setOrientation_XYZStartTime(
//																ndDepOrientationEntity, 
//																timeInfo);
//													}
//												}
//											}
//										}
//									}
								} else {
									SS3Singleton
											.getErrorMessages()
											.showErrorMessage(
													ErrorMessages.SET_START_RETRACTION_CANT_DELETE_POSTSTROKE_HOLD);
									return false;
								}
							} else {
								Time minEndTimeInfo = ndValue.getStartTimeInfo().add(Duration.ts(packetDuration));
//								TimeInfo minEndTimeInfo = new TimeInfo(
//										ndValue.getStartTimeInfo().value()
//											+ packetDuration,
//										ndValue.getStartTimeInfo().unit());
								if (minEndTimeInfo.lessThanOrEq(timeInfo)) {
									ndRetraction.setStartTimeInfo(timeInfo);

									if (depGestEntity
											.isLockedGesturePhonAlignments()) {

										GestureChainedEventsEntity ndDepHandEntity = depGestEntity
												.getDependentHandEntity();
										GestureHandShapeEntity ndDepHandShapeEntity = depGestEntity
												.getDependentHandShapeEntity();
										GestureOrientationEntity ndDepOrientationEntity = depGestEntity
												.getDependentOrientationEntity();
										if (ndDepHandEntity != null) {
											GestureChainedEventsEntity ndNextEntity = ndDepHandEntity
													.getNextEntity();
											if (ndNextEntity == null) {
												ndDepHandEntity.getTextValue().setEndTimeInfo(timeInfo);
												if (ndDepHandShapeEntity != null) {
													setGestureHandshapeEndTime(
															ndDepHandShapeEntity,
															timeInfo);
												}
												if (ndDepOrientationEntity != null) {
													setGestureOrientationEndTime(
															ndDepOrientationEntity,
															timeInfo);
												}
											} else {
												double nextEntityMovieTime = ndNextEntity
														.getStartTimeInfo()
														.doubleValue();
												if (ndNextEntity.getStartTimeInfo().lessThan(timeInfo)) {
													SS3Singleton
															.getErrorMessages()
															.showErrorMessage(
																	ErrorMessages.GESTURE_CHAINS_CANNOT_OVERLAP);
													return false;
												} else {
													ndDepHandEntity
															.getTextValue().setEndTimeInfo(timeInfo);
													if (ndDepHandShapeEntity != null) {
														setGestureHandshapeStartTime(
																ndDepHandShapeEntity,
																timeInfo);
													}
													if (ndDepOrientationEntity != null) {
														setGestureOrientationStartTime(
																ndDepOrientationEntity, 
																timeInfo);
													}
												}
											}
										}
									}

									ndText.setEndTimeInfo(timeInfo);
									depGestEntity.setRetractionEvent(ndRetraction);
								} else {
									SS3Singleton
											.getErrorMessages()
											.showErrorMessage(
													ErrorMessages.SET_START_RETRACTION_CANT_DELETE_VALUE);
									return false;
								}
							}
						} else {
							SS3Singleton
									.getErrorMessages()
									.showErrorMessage(
											ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME);
							return false;
						}
					}

					if (gestLockedGlossPhoneAlignments && poststrokeHold == null) {
						if (handEntity != null) {
							GestureChainedEventsEntity nextHandEntity = handEntity
									.getNextEntity();
							if (nextHandEntity == null) {
								handEntity.setEndTimeInfo(timeInfo);
							} else if (nextHandEntity.getStartTimeInfo().lessThan(timeInfo)) {
								SS3Singleton
										.getErrorMessages()
										.showErrorMessage(
												ErrorMessages.GESTURE_CHAINS_CANNOT_OVERLAP);
								return false;
							} else {
								handEntity.setStartTimeInfo(timeInfo);
							}

							if (handShapeEntity != null) {
								setGestureHandshapeEndTime(handShapeEntity, timeInfo);
							}
							if (orientationEntity != null) {
								setGestureOrientationEndTime(orientationEntity, timeInfo);
							}
						}
					}
				} else if (event.equals(poststrokeHold)) {
					
					if (apexEntity != null) {
						if (timeInfo.lessThan(apexEntity.getEndTimeInfo())) {
							SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.GESTURE_APEX_BOUNDS);
							return false;
						}
					}
					
					if (gestLockedDomNdomAlignments && ndApexEntity != null) {
						if (timeInfo.lessThan(ndApexEntity.getEndTimeInfo())) {
							SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.GESTURE_APEX_BOUNDS);
							return false;
						}
					}
					
					if (gestLockedDomNdomAlignments && depGestEntity != null
							&& depGestEntity.getPoststrokeHold() != null) {
						GestureChainedEvent ndValue = depGestEntity
								.getGestureChainedEvent();

						int packetDuration = panel
								.getMultipleMovieController()
								.getPacketDuration();

						Time minValueEndTimeInfo =
                            ndValue.getStartTimeInfo().add(Duration.ts(packetDuration));
//						TimeInfo minValueEndTimeInfo = new TimeInfo(
//								ndValue.getStartTimeInfo().value()
//									+ packetDuration,
//								ndValue.getStartTimeInfo().unit());
						Time maxHoldStartTimeInfo =
                            ndPoststrokeHold.getEndTimeInfo().subtract(Duration.ts(packetDuration));
//						TimeInfo maxHoldStartTimeInfo = new TimeInfo(
//								ndFinalHold.getEndTimeInfo().value()
//									- packetDuration,
//								ndFinalHold.getEndTimeInfo().unit());

						if (minValueEndTimeInfo.greaterThan(timeInfo)) {
							SS3Singleton
									.getErrorMessages()
									.showErrorMessage(
											ErrorMessages.SET_START_POSTSTROKE_HOLD_CANT_DELETE_VALUE);
							return false;
						} else if (timeInfo.greaterThan(maxHoldStartTimeInfo)) {
							SS3Singleton
									.getErrorMessages()
									.showErrorMessage(
											ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME);
							return false;
						} else {
							ndPoststrokeHold.setStartTimeInfo(timeInfo);
							ndValue.setEndTimeInfo(timeInfo);
							
	                        GestureChainedEventsEntity ndHandEntity = depGestEntity
	                                .getDependentHandEntity();
	                        GestureHandShapeEntity ndHandShapeEntity = depGestEntity
	                                .getDependentHandShapeEntity();
	                        GestureOrientationEntity ndOrientationEntity = depGestEntity
	                                .getDependentOrientationEntity();

	                        if (ndHandEntity != null) {
	                            GestureChainedEventsEntity nextEntity = ndHandEntity
	                                    .getNextEntity();
	                            if (nextEntity != null
	                                    && nextEntity.getStartTimeInfo().lessThan(timeInfo)) {
	                                SS3Singleton
	                                        .getErrorMessages()
	                                        .showErrorMessage(
	                                                ErrorMessages.GESTURE_CHAINS_CANNOT_OVERLAP);
	                                return false;         
	                            } else {
	                                ndHandEntity.setEndTimeInfo(timeInfo);
	                                
	                                if (ndHandShapeEntity != null) {
	                                    setGestureHandshapeEndTime( 
	                                            ndHandShapeEntity, timeInfo);
	                                }           
	                                if (ndOrientationEntity != null) {
	                                    setGestureOrientationEndTime( 
	                                            ndOrientationEntity, timeInfo);
	                                }           
	                            }   
	                        }   

						}
					}

					if (gestLockedGlossPhoneAlignments) {
						if (handEntity != null) {
							GestureChainedEvent textValue = handEntity.getTextValue();
							GestureChainedEventsEntity prevHandEntity = handEntity
									.getPreviousEntity();

							if (prevHandEntity == null) {
								textValue.setEndTimeInfo(timeInfo);
							} else {
								double prevEntMovieTime = prevHandEntity
										.getEndTimeInfo().doubleValue();
								if (timeInfo.lessThan(prevHandEntity.getEndTimeInfo())) {
									SS3Singleton
											.getErrorMessages()
											.showErrorMessage(
													ErrorMessages.GESTURE_CHAINS_CANNOT_OVERLAP);
									return false;
								} else {
									textValue.setEndTimeInfo(timeInfo);
								}
							}
							
							handEntity.setEndTimeInfo(timeInfo);
							
							if (handShapeEntity != null) {
								setGestureHandshapeEndTime(handShapeEntity, timeInfo);
							}
							if (orientationEntity != null) {
								setGestureOrientationEndTime(orientationEntity, timeInfo);
							}
						}
					}
				} else if (event.equals(relaxed)) {
					
					if (apexEntity != null) {
						if (timeInfo.lessThan(apexEntity.getEndTimeInfo())) {
							SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.GESTURE_APEX_BOUNDS);
							return false;
						}
					}
					
					if (gestLockedDomNdomAlignments && ndApexEntity != null) {
						if (timeInfo.lessThan(ndApexEntity.getEndTimeInfo())) {
							SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.GESTURE_APEX_BOUNDS);
							return false;
						}
					}
					
					if (depGestEntity != null && ndRelaxed != null
							&& gestLockedDomNdomAlignments) {
						GestureChainedEvent ndValue = depGestEntity
								.getGestureChainedEvent();

						int packetDuration = panel
								.getMultipleMovieController()
								.getPacketDuration();

						double maxStartTimcketDuration;
						Time maxStartTimeInfo =
								ndRelaxed.getEndTimeInfo().subtract(Duration.ts(packetDuration));
//						TimeInfo maxStartTimeInfo = new TimeInfo(
//								ndOffset.getEndTimeInfo().value()
//									- packetDuration,
//								ndOffset.getEndTimeInfo().unit());

						if (timeInfo.lessThanOrEq(maxStartTimeInfo)) {
							if (ndRetraction != null) {
								Time minHoldEndTimeInfo =
										ndRetraction.getStartTimeInfo().add(Duration.ts(packetDuration));
//								TimeInfo minHoldEndTimeInfo = new TimeInfo(
//										ndFinalHold.getStartTimeInfo().value()
//											+ packetDuration,
//										ndFinalHold.getStartTimeInfo().unit());
								if (minHoldEndTimeInfo.lessThanOrEq(timeInfo)) {
									ndRelaxed.setStartTimeInfo(timeInfo);
									ndRetraction.setEndTimeInfo(timeInfo);

//									if (depGlsEntity
//											.isLockedGesturePhonAlignments()) {
//										GestureChainedEventsEntity ndDepHandEntity = depGlsEntity
//												.getDependentHandEntity();
//										HandShapeEntity_XYZ ndDepHandShapeEntity = depGlsEntity
//												.getDependentHandShapeEntity();
//										OrientationEntity_XYZ ndDepOrientationEntity = depGlsEntity
//												.getDependentOrientationEntity();
//										if (ndDepHandEntity != null) {
//											GestureChainedEventsEntity ndNextEntity = ndDepHandEntity
//													.getNextEntity();
//											if (ndNextEntity == null) {
//												ndDepHandEntity.getTextValue().setEndTimeInfo(timeInfo);
//												if (ndDepHandShapeEntity != null) {
//													setHandshape_XYZEndTime(
//															ndDepHandShapeEntity,
//															timeInfo);
//												}
//												if (ndDepOrientationEntity != null) {
//													setOrientation_XYZEndTime(
//															ndDepOrientationEntity,
//															timeInfo);
//												}
//											} else {
//												double nextEntityMovieTime = ndNextEntity
//														.getStartTimeInfo()
//														.doubleValue();
//												if (ndNextEntity.getStartTimeInfo().lessThan(timeInfo)) {
//													SS3Singleton
//															.getErrorMessages()
//															.showErrorMessage(
//																	ErrorMessages.GESTURE_CHAINS_CANNOT_OVERLAP);
//													return false;
//												} else {
//													ndDepHandEntity
//															.getTextValue().setEndTimeInfo(timeInfo);
//													if (ndDepHandShapeEntity != null) {
//														setHandshape_XYZStartTime(
//																ndDepHandShapeEntity,
//																timeInfo);
//													}
//													if (ndDepOrientationEntity != null) {
//														setOrientation_XYZStartTime(
//																ndDepOrientationEntity, 
//																timeInfo);
//													}
//												}
//											}
//										}
//									}
								} else {
									SS3Singleton
											.getErrorMessages()
											.showErrorMessage(
													ErrorMessages.SET_START_RELAXED_CANT_DELETE_VALUE);
									return false;
								}
								
							} else if (ndPoststrokeHold != null) {
								Time minHoldEndTimeInfo =
                                    ndPoststrokeHold.getStartTimeInfo().add(Duration.ts(packetDuration));
//								TimeInfo minHoldEndTimeInfo = new TimeInfo(
//										ndFinalHold.getStartTimeInfo().value()
//											+ packetDuration,
//										ndFinalHold.getStartTimeInfo().unit());
								if (minHoldEndTimeInfo.lessThanOrEq(timeInfo)) {
									ndRelaxed.setStartTimeInfo(timeInfo);
									ndPoststrokeHold.setEndTimeInfo(timeInfo);

//									if (depGlsEntity
//											.isLockedGesturePhonAlignments()) {
//										GestureChainedEventsEntity ndDepHandEntity = depGlsEntity
//												.getDependentHandEntity();
//										HandShapeEntity_XYZ ndDepHandShapeEntity = depGlsEntity
//												.getDependentHandShapeEntity();
//										OrientationEntity_XYZ ndDepOrientationEntity = depGlsEntity
//												.getDependentOrientationEntity();
//										if (ndDepHandEntity != null) {
//											GestureChainedEventsEntity ndNextEntity = ndDepHandEntity
//													.getNextEntity();
//											if (ndNextEntity == null) {
//												ndDepHandEntity.getTextValue().setEndTimeInfo(timeInfo);
//												if (ndDepHandShapeEntity != null) {
//													setHandshape_XYZEndTime(
//															ndDepHandShapeEntity,
//															timeInfo);
//												}
//												if (ndDepOrientationEntity != null) {
//													setOrientation_XYZEndTime(
//															ndDepOrientationEntity,
//															timeInfo);
//												}
//											} else {
//												double nextEntityMovieTime = ndNextEntity
//														.getStartTimeInfo()
//														.doubleValue();
//												if (ndNextEntity.getStartTimeInfo().lessThan(timeInfo)) {
//													SS3Singleton
//															.getErrorMessages()
//															.showErrorMessage(
//																	ErrorMessages.GLOSS_CHAINS_CANNOT_OVERLAP);
//													return false;
//												} else {
//													ndDepHandEntity
//															.getTextValue().setEndTimeInfo(timeInfo);
//													if (ndDepHandShapeEntity != null) {
//														setHandshape_XYZStartTime(
//																ndDepHandShapeEntity,
//																timeInfo);
//													}
//													if (ndDepOrientationEntity != null) {
//														setOrientation_XYZStartTime(
//																ndDepOrientationEntity, 
//																timeInfo);
//													}
//												}
//											}
//										}
//									}
								} else {
									SS3Singleton
											.getErrorMessages()
											.showErrorMessage(
													ErrorMessages.SET_START_RELAXED_CANT_DELETE_POSTSTROKE_HOLD);
									return false;
								}
							} else {
								Time minEndTimeInfo = ndValue.getStartTimeInfo().add(Duration.ts(packetDuration));
//								TimeInfo minEndTimeInfo = new TimeInfo(
//										ndValue.getStartTimeInfo().value()
//											+ packetDuration,
//										ndValue.getStartTimeInfo().unit());
								if (minEndTimeInfo.lessThanOrEq(timeInfo)) {
									ndRelaxed.setStartTimeInfo(timeInfo);

									if (depGestEntity
											.isLockedGesturePhonAlignments()) {

										GestureChainedEventsEntity ndDepHandEntity = depGestEntity
												.getDependentHandEntity();
										GestureHandShapeEntity ndDepHandShapeEntity = depGestEntity
												.getDependentHandShapeEntity();
										GestureOrientationEntity ndDepOrientationEntity = depGestEntity
												.getDependentOrientationEntity();
										if (ndDepHandEntity != null) {
											GestureChainedEventsEntity ndNextEntity = ndDepHandEntity
													.getNextEntity();
											if (ndNextEntity == null) {
												ndDepHandEntity.getTextValue().setEndTimeInfo(timeInfo);
												if (ndDepHandShapeEntity != null) {
													setGestureHandshapeEndTime(
															ndDepHandShapeEntity,
															timeInfo);
												}
												if (ndDepOrientationEntity != null) {
													setGestureOrientationEndTime(
															ndDepOrientationEntity,
															timeInfo);
												}
											} else {
												double nextEntityMovieTime = ndNextEntity
														.getStartTimeInfo()
														.doubleValue();
												if (ndNextEntity.getStartTimeInfo().lessThan(timeInfo)) {
													SS3Singleton
															.getErrorMessages()
															.showErrorMessage(
																	ErrorMessages.GESTURE_CHAINS_CANNOT_OVERLAP);
													return false;
												} else {
													ndDepHandEntity
															.getTextValue().setEndTimeInfo(timeInfo);
													if (ndDepHandShapeEntity != null) {
														setGestureHandshapeStartTime(
																ndDepHandShapeEntity,
																timeInfo);
													}
													if (ndDepOrientationEntity != null) {
														setGestureOrientationStartTime(
																ndDepOrientationEntity, 
																timeInfo);
													}
												}
											}
										}
									}

									ndText.setEndTimeInfo(timeInfo);
									depGestEntity.setRelaxedEvent(ndRelaxed);
								} else {
									SS3Singleton
											.getErrorMessages()
											.showErrorMessage(
													ErrorMessages.SET_START_RELAXED_CANT_DELETE_VALUE);
									return false;
								}
							}
						} else {
							SS3Singleton
									.getErrorMessages()
									.showErrorMessage(
											ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME);
							return false;
						}
					}

					if (gestLockedGlossPhoneAlignments && poststrokeHold == null && retraction == null) {
						if (handEntity != null) {
							GestureChainedEventsEntity nextHandEntity = handEntity
									.getNextEntity();
							if (nextHandEntity == null) {
								handEntity.setEndTimeInfo(timeInfo);
							} else if (nextHandEntity.getStartTimeInfo().lessThan(timeInfo)) {
								SS3Singleton
										.getErrorMessages()
										.showErrorMessage(
												ErrorMessages.GESTURE_CHAINS_CANNOT_OVERLAP);
								return false;
							} else {
								handEntity.setEndTimeInfo(timeInfo);
							}

							if (handShapeEntity != null) {
								setGestureHandshapeEndTime(handShapeEntity, timeInfo);
							}
							if (orientationEntity != null) {
								setGestureOrientationEndTime(orientationEntity, timeInfo);
							}
						}
					}
				}
				
				gestEntity.setGestureEventsEntityStartTimeInfo(timeInfo);

				GestureLocationEntity depLocEntity = gestEntity.getDependentLocationEntity();
				if (depLocEntity != null) {
					depLocEntity.updateTimes();
				} else if (gestLockedDomNdomAlignments && depGestEntity != null) {
					depLocEntity = depGestEntity.getDependentLocationEntity();
					if (depLocEntity != null) {
						depLocEntity.updateTimes();
					}
				}
				
				GestureTypeEntity gestureTypeEntity = gestEntity.getDependentGestureTypeEntity();
				if (gestureTypeEntity != null) {
					gestureTypeEntity.updateTimes();
				} else if (gestLockedDomNdomAlignments && depGestEntity != null) {
					gestureTypeEntity = depGestEntity.getDependentGestureTypeEntity();
					if (gestureTypeEntity != null) {
						gestureTypeEntity.updateTimes();
					}
				}
			}
		} else if (field instanceof NonManualField) {
//			if (selectedEntity != null && parentEntity != null) {
//				if ( !(selectedEntity instanceof CustomChainedEventsEntity)) {
//					if (parentEntity.getStartTimeInfo().greaterThan(timeInfo)
//							|| parentEntity.getEndTimeInfo().lessThan(timeInfo)) {
//						SS3Singleton.getErrorMessages().showErrorMessage(
//								ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE);
//						return false;
//					}
//				}
//			}

			ChainedEventsEntity nmEntity = (ChainedEventsEntity) selectedEntity;
			String id = nmEntity.getField().getFieldID();
			if (id.equalsIgnoreCase(Util.DOM_HAND_FIELD_ID) || id.equalsIgnoreCase(Util.NDOM_HAND_FIELD_ID)) {
				if (nmEntity.isLockedGlossPhonAlignments()) {
					SS3Singleton.getErrorMessages().showErrorMessage(
							ErrorMessages.UNLOCK_GLOSS_PHON_ALIGNMENTS);

					return false;
				}
			}

			String errorCode = nmEntity.canSetSelectedEventStartMovieTime(timeInfo);
			if (errorCode != null) {
				SS3Singleton.getErrorMessages().showErrorMessage(errorCode);
				return false;
			}
			
			nmEntity.setStartTimeInfo(timeInfo);

			// set start time
		} else if (field instanceof GestureDomNdomHandField) {
			
//			if (selectedEntity != null && parentEntity != null) {
//				if (parentEntity.getStartTimeInfo().greaterThan(timeInfo)
//						|| parentEntity.getEndTimeInfo().lessThan(timeInfo)) {
//					SS3Singleton.getErrorMessages().showErrorMessage(
//							ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE);
//					return false;
//				}
//			}

			GestureChainedEventsEntity gestureHandEntity = (GestureChainedEventsEntity) selectedEntity;
			
			String id = gestureHandEntity.getField().getFieldID();
			if (id.equalsIgnoreCase(Util.DOM_GESTURE_HAND_FIELD_ID) 
					|| id.equalsIgnoreCase(Util.NDOM_GESTURE_HAND_FIELD_ID)) {
				if (gestureHandEntity.isLockedGesturePhonAlignments()) {
					SS3Singleton.getErrorMessages().showErrorMessage(
							ErrorMessages.UNLOCK_GESTURE_PHON_ALIGNMENTS);

					return false;
				}
			}

			String resp = gestureHandEntity.canSetSelectedEventStartMovieTime(timeInfo);
			if (resp != null) {
				SS3Singleton.getErrorMessages().showErrorMessage(resp);
				return false;
			}
			
			gestureHandEntity.setStartTimeInfo(timeInfo);

			// set start time
		} else if (field instanceof ApexField) {
//			if (selectedEntity != null) {
//				if (parentEntity.getStartTimeInfo().greaterThan(timeInfo)
//						|| parentEntity.getEndTimeInfo().lessThan(timeInfo)) {
//					SS3Singleton.getErrorMessages().showErrorMessage(
//							ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE);
//					return false;
//				}
//			}

			ChainedApexEventsEntity nmEntity = (ChainedApexEventsEntity) selectedEntity;

			String errorCode = nmEntity.setSelectedEventStartMovieTime(timeInfo);
			if (errorCode != null) {
				SS3Singleton.getErrorMessages().showErrorMessage(errorCode);
				return false;
			}

			// set start time
		}

		return true;
	}

	public static boolean setPresentationEventEndTime(Time timeInfo, Event event,
                                                   EventsEntity entity) {
	    var field = entity.getField();

	    if (!isUpdatablePresentationEntity(entity)) {
            var interval = event.interval();
            var mid = interval.t0.add(interval.duration().divide(2.0));

            var parentField = field.getParentField();
            if (parentField != null) {
                var parent = (PresentationEventsEntity)parentField.getEntityAt(mid.absolute().as(TimeUnit.COORDINATE));
                return setPresentationEventEndTime(timeInfo, parent.getEvent(), parent);
            } else {
                var parent = (PresentationEventsEntity)field.getEntityAt(mid.absolute().as(TimeUnit.COORDINATE));
                return setPresentationEventEndTime(timeInfo, parent.getEvent(), parent);
            }
        }

		PresentationField presentationField = (PresentationField) field;
		EventsEntity nextEntity = presentationField.getNextEntity(event.getEndTimeInfo());

		if (timeInfo.greaterThan(SS3Singleton.getMediaPlayer().end())) {

			SS3Singleton.getErrorMessages().showErrorMessage(
					ErrorMessages.UTTERANCE_END_AFTER_MOVIE_END);
			return false;

		} else if (event.getEndTimeInfo().greaterThan(timeInfo) && timeInfo.lessThanOrEq(event.getStartTimeInfo())) {

			SS3Singleton.getErrorMessages().showErrorMessage(
					ErrorMessages.EVENT_END_TIME_FOLLOW_START_TIME);
			return false;

		} else if (event.getEndTimeInfo().lessThan(timeInfo)) {

			if (nextEntity != null && nextEntity != entity && timeInfo.greaterThan(nextEntity.getStartTimeInfo())) {
				SS3Singleton.getErrorMessages().showErrorMessage(
						ErrorMessages.CHAINS_CANNOT_OVERLAP);
				return false;
			}
		}

		PresentationEventsEntity pEntity = (PresentationEventsEntity) entity;
		ArrayList<PresentationEventsEntity> entities = pEntity
				.getDependentEntities();

		for (PresentationEventsEntity dEntity : entities) {
			dEntity.getSS3GlossWindowItem().resetTranslationEndTime(dEntity.getID(), timeInfo);
		}

		PresentationEvent pEvent = (PresentationEvent) event;
		entity.setEndTimeInfo(timeInfo);
		pEvent.setEndTimeInfo(timeInfo);
        presentationField.getSS3GlossWindowSegment()
            .getSS3GlossWindowItem(entity.getID())
            .setEndTimeInfo(timeInfo);

        for (PresentationEventsEntity dEntity : entities) {
            dEntity.setEndTimeInfo(timeInfo);

            var item = ((PresentationField)dEntity.getField())
                .getSS3GlossWindowSegment()
                .getSS3GlossWindowItem(entity.getID());
            var dItem = ((PresentationField)dEntity.getField())
                .getSS3GlossWindowSegment()
                .getSS3GlossWindowItem(dEntity.getID());

            if (item != null) {
                item.setEndTimeInfo(timeInfo);
            } else if (dItem != null) {
                dItem.setEndTimeInfo(timeInfo);
            }
        }

		SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
		
		return true;
	}

	public static boolean setPresentationEventStartTime(Time timeInfo, Event event, EventsEntity entity) {
	    var field = entity.getField();

        if (!isUpdatablePresentationEntity(entity)) {
            var interval = event.interval();
            var mid = interval.t0.add(interval.duration().divide(2.0));

            var parentField = field.getParentField();
            if (parentField != null) {
                var parent = (PresentationEventsEntity)parentField.getEntityAt(mid.absolute().as(TimeUnit.COORDINATE));
                return setPresentationEventStartTime(timeInfo, parent.getEvent(), parent);
            } else {
                var parent = (PresentationEventsEntity)field.getEntityAt(mid.absolute().as(TimeUnit.COORDINATE));
                return setPresentationEventStartTime(timeInfo, parent.getEvent(), parent);
            }
        }

		PresentationField presentationField = (PresentationField) field;
		EventsEntity prevEntity = presentationField.getPreviousEntity(event.getStartTimeInfo());

		if (timeInfo.lessThan(SS3Singleton.getMediaPlayer().start())) {

			SS3Singleton.getErrorMessages().showErrorMessage(
					ErrorMessages.UTTERANCE_START_BEFORE_MOVIE_START);
			return false;

		} else if (event.getStartTimeInfo().lessThan(timeInfo) && timeInfo.greaterThanOrEq(event.getEndTimeInfo())) {

			SS3Singleton.getErrorMessages().showErrorMessage(
					ErrorMessages.EVENT_END_TIME_FOLLOW_START_TIME);
			return false;

		} else if (event.getStartTimeInfo().greaterThan(timeInfo)) {
			
			if (prevEntity != null && prevEntity != entity && prevEntity.getEndTimeInfo().greaterThan(timeInfo)) {

				SS3Singleton.getErrorMessages().showErrorMessage(
						ErrorMessages.CHAINS_CANNOT_OVERLAP);
				return false;
			}
		}

		PresentationEventsEntity pEntity = (PresentationEventsEntity) entity;		
		ArrayList<PresentationEventsEntity> entities = pEntity
				.getDependentEntities();

		for (PresentationEventsEntity dEntity : entities) {
			dEntity.getSS3GlossWindowItem().resetTranslationStartTime(dEntity.getID(), timeInfo);
		}

		PresentationEvent pEvent = (PresentationEvent) event;
		entity.setStartTimeInfo(timeInfo);
        pEvent.setStartTimeInfo(timeInfo);
        presentationField.getSS3GlossWindowSegment()
            .getSS3GlossWindowItem(entity.getID())
            .setStartTimeInfo(timeInfo);

        for (PresentationEventsEntity dEntity : entities) {
            dEntity.setStartTimeInfo(timeInfo);

            var item = ((PresentationField)dEntity.getField())
                .getSS3GlossWindowSegment()
                .getSS3GlossWindowItem(entity.getID());
            var dItem = ((PresentationField)dEntity.getField())
                .getSS3GlossWindowSegment()
                .getSS3GlossWindowItem(dEntity.getID());

            if (item != null) {
                item.setStartTimeInfo(timeInfo);
            } else if (dItem != null) {
                dItem.setStartTimeInfo(timeInfo);
            }
        }

		SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);

		return true;
	}
}

