package edu.bu.signstream.media.ui;

import edu.bu.signstream.time.TimeRange;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.media.MediaPlayer;
import edu.bu.signstream.media.Video;
import edu.bu.signstream.media.ui.icon.SizeLargeIconDark;
import edu.bu.signstream.media.ui.icon.SizeSmallIconDark;

import javax.swing.*;
import java.awt.*;

public class VideoView extends MediaView {
    private final JComponent image;

    private final ToggleButton resize = new ToggleButton(b -> b ? new SizeLargeIconDark(true) : new SizeSmallIconDark(true), false);

    private final Dimension originalSize;
    private final Dimension smallerSize;

    @Override
    protected JComponent view() { return image; }

    @Override
    protected JPanel controls(JPanel panel) {
        panel.add(resize);
        return panel;
    }

    @Override
    protected TimeRange defaultTimeRange() { return SS3Singleton.getVideoTimeRange(); }

    public VideoView(MediaPlayer player, Video video, JComponent image, Runnable onClose) {
        super(player, video, onClose);

        originalSize = new Dimension(video.width(), video.height());
        smallerSize = new Dimension(video.width() / 2, video.height() / 2);

        this.image = image;
        image.setPreferredSize(originalSize);

        resize.addActionListener(l -> {
        	SS3Singleton.getMediaPlayer().videos()
        		.forEach(v -> ((VideoView) v.view()).toggle());
        });
        resize.setToolTipText("Smaller Size");

        build();
    }
    
    public void toggle() {
        var size = resize.state() ? originalSize : smallerSize;
        image.setPreferredSize(size);
        image.setSize(size);
        resize.toggle();

        resize.setToolTipText(resize.state() ? "Original Size" : "Smaller Size");
        pack();
    }
}
