package edu.bu.signstream.ui;

import java.io.*;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.nio.file.StandardCopyOption;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.HashMap;
import java.awt.*;
import java.awt.desktop.QuitEvent;
import java.awt.desktop.QuitHandler;
import java.awt.desktop.QuitResponse;
import java.awt.event.*;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.HashSet;
import java.util.Map;
import java.util.Properties;
import java.util.logging.FileHandler;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.stream.Collectors;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import javax.swing.*;
import javax.swing.filechooser.FileView;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import edu.bu.signstream.common.EnterKeyListener;
import edu.bu.signstream.common.db.parser.SS3Parser;
import edu.bu.signstream.common.db.ss3.SS3Reader;
import edu.bu.signstream.common.db.ss4.SS4Reader;
import edu.bu.signstream.common.files.FilesLocation;
import edu.bu.signstream.common.files.SSResources;
import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.ImageMaker;
import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.Version;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.Excerpt;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.ViewMode;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.Panel;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Database;
import edu.bu.signstream.common.util.vo.ss3.db.SS3DatabaseCollection;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindow;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowSegment;
import edu.bu.signstream.common.util.vo.ss3.db.SS3TemporalPartition;
import edu.bu.signstream.common.util.vo.ss3.media.SS3MediaFile;
import edu.bu.signstream.common.util.vo.ss3.media.SS3MediaLibrary;
import edu.bu.signstream.common.util.vo.ss3.participants.ParticipantsLibrary;
import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.time.*;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.FieldWrappersCollection;
import edu.bu.signstream.grepresentation.fields.Segment;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.MorphologicalPhonologicalInfoDialog;
import edu.bu.signstream.grepresentation.fields.presentation.*;
import edu.bu.signstream.grepresentation.fields.presentation.dialog.RenameSegmentDialog;
import edu.bu.signstream.grepresentation.fields.presentation.dialog.RenameTemporalPartitionDialog;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.LabelObject;
import edu.bu.signstream.grepresentation.view.SegmentTemplate;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolBar;
import edu.bu.signstream.grepresentation.view.signbank.SignBankCollection;
import edu.bu.signstream.media.fileNavigation.LoadedMediaFile;
import edu.bu.signstream.navigation.DatabaseDelegate;
import edu.bu.signstream.navigation.DbNavigationPanel;
import edu.bu.signstream.ui.frame.SSFrame;
import edu.bu.signstream.ui.frame.dialog.SaveDialog;
import edu.bu.signstream.ui.panels.AnnotatorSelectionDialog;
import edu.bu.signstream.ui.panels.newDatabase.CreateUpdateMacroUnit;
import edu.bu.signstream.ui.panels.newDatabase.CreateUpdateMacroUnitDialog;
import edu.bu.signstream.ui.panels.search.SearchDialog;

/**
 * DO not use JOptionPane as it disables MenuItems on Mac.
 * */
public class SS3SignStreamApplication extends SSFrame implements ActionListener, KeyListener {

	private static final long serialVersionUID = 1L; 

	public static Logger logger = Logger.getLogger("edu.bu.signstream.ui");

	public final int NEW_COLLECTION = 0;
	public final int CLOSE_DB = 3;
	public final int SAVE_DB = 4;
	private final int SAVE_AS_DB = 5;
	public final int OPEN_AS_SS3 = 6;
	private final int NEW_SEARCH = 50;
	private final int SAVED_SEARCH = 51;

	private final int NEW_MACRO_UNIT = 7;
	private final int DELETE_MACRO_UNIT = 8;
	private final int RENAME_MACRO_UNIT = 9;

	private final int CREATE_SEGMENT = 10;
	private final int RENAME_SEGMENT = 11;
	private final int RENAME_PARTICIPANT = 12;
	private final int DELETE_SEGMENT = 13;
	private final int SET_DEFAULT_SEGMENT = 14;
	
	private final int CREATE_TEMPORAL_PARTITION = 20;
	private final int RENAME_TEMPORAL_PARTITION = 21;
	private final int DELETE_TEMPORAL_PARTITION = 22;
	private final int RENUMBER_UTTERANCES = 23;

    private final int SET_VIEW_MODE = 39;
    private final int TOGGLE_PANEL = 40;

//	private final int TOGGLE_NONMANUAL = 40;
//	private final int TOGGLE_CUSTOM = 41;
//	private final int TOGGLE_SIGN = 42;
//	private final int TOGGLE_SPEECH = 43;

	private final int HELP = 100;
	private final int UPDATE = 150;
	private final int UPDATE_SIGN_BANK = 151;
	private final int INSTALL_SIGN_BANK = 152;
	private final int UNINSTALL_SIGN_BANK = 153;
	private final int VIEW_ERRORS = 154;
	private final int VIEW_MODIFICATIONS = 155;

	private final int SHOW_IPA = 200;

	private final int EXIT = 30;
	
	private final int MENU_NUM_ITEMS = 5;

	private SS3Parser parser = null;
	private DatabaseDelegate dbDelegate = null;
	private SearchDialog searchDialog = null;

    private final SignStreamSegmentPanel panel;

	private SignBankCollection signBankCollection = null;

	public static String path = "newXMLFiles";
	public static String globalParticipantsFileName = "globalParticipants.xml";
	public static String localParticipantsFileName = "localParticipants.xml";
	public static String globalAnnotatorsFileName = "globalAnnotators.xml";
	public static String localAnnotatorsFileName = "localAnnotators.xml";
	public static String mediaLibraryFileName = "mediaLibrary.xml";

	private String scPath = null;

	JMenu fileMenu = new JMenu("Collection");
	JMenuItem newItem = new JMenuItem(" New Blank Collection ");
	JMenuItem openAsItem = new JMenuItem(" Open...");
	JMenuItem openLegacyItem = new JMenuItem(" Open Legacy...");
	JMenuItem closeItem = new JMenuItem(" Close ");
	JMenuItem saveItem = new JMenuItem(" Save ");
	JMenuItem saveAsItem = new JMenuItem(" Save As... ");
	JMenuItem newSearch = new JMenuItem("New Search");
	JMenuItem savedSearch = new JMenuItem("Open Saved Search");

	private CreateUpdateMacroUnitDialog macroUnitDialog = null;

	public static int SAVE = 4;
	public static int SAVE_AS = 5;

	private final String BASH_UPDATE_SCRIPT = SSResources.rootPath + "/Update.command";
	private final String BASH_SIGN_BANK_SCRIPT = SSResources.rootPath + "/GetSignBank.command";
 	public final static String VALIDATION_ERRORS = SSResources.rootPath + "/SignStreamValidationErrors.txt";
 	public final static String VALIDATION_MODIFICATIONS = SSResources.rootPath + "/SignStreamValidationModifications.txt";
 	
 	private JDialog validationErrorsDialog = null;
 	private JDialog validationModificationsDialog = null;
 	
	private final int BASH_EXIT_CODE_YES = 0;
	private final int BASH_EXIT_CODE_NO = 1;

	private boolean cannotParseSignBank = false;

	public static void main(String[] args) {
        var log = new LogStream(System.out);
        System.setOut(log);
        System.setErr(log);

        SwingUtilities.invokeLater(new Runnable() {
            @Override
            public void run() {
                SS3SignStreamApplication t = new SS3SignStreamApplication();

                t.setExtendedState(JFrame.MAXIMIZED_HORIZ);
                t.setSize(1450, 640);
                t.setVisible(true);
                t.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);

                t.loadNewMacroUnitDialog(new SS3Database());
            }
        });
    }

    private static class LogStream extends PrintStream {
        private final OutputStream log;

        public LogStream(OutputStream out) {
            super(out);

            try {
                log = new FileOutputStream(new File(SSResources.rootPath + "/SignStreamOut.txt"));
            } catch (FileNotFoundException e) {
                throw new RuntimeException(e);
            }
        }

        @Override
        public void write(byte[] buf, int off, int len) {
            super.write(buf, off, len);
            try {
                log.write(buf, off, len);
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }

        @Override
        public void close() {
            super.close();
            try {
                log.close();
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }
    }

	public SS3SignStreamApplication() {
        Thread.setDefaultUncaughtExceptionHandler((thread, exception) -> {
            System.out.println("====================================================================================");
            System.out.println("SEVERE: Uncaught exception on thread '" + thread.getName() + "':");
            exception.printStackTrace();
            System.out.println("====================================================================================");
        });

		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception ex) {
			;
			logger.log(Level.INFO, "Could not set system look and feel", ex);
		}

		ToolTipManager.sharedInstance().setInitialDelay(400);

		System.setProperty("apple.laf.useScreenMenuBar", "true");

		"com.sun.java.swing.plaf.windows.WindowsLookAndFeel"
				.equalsIgnoreCase(UIManager.getSystemLookAndFeelClassName());

		System.setProperty("com.apple.eawt.CocoaComponent.CompatibilityMode",
				"false");

		System.setProperty("jna.library.path", SSResources.libPath);
		System.setProperty("jna.debug_load", "true");

		KeyStroke nextFrameKeyStroke = KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT, 0);
		KeyStroke prevFrameKeyStroke = KeyStroke.getKeyStroke(KeyEvent.VK_LEFT, 0);	
		KeyStroke prevUtteranceKeyStroke = KeyStroke.getKeyStroke(KeyEvent.VK_UP, 0);	
		KeyStroke nextUtteranceKeyStroke = KeyStroke.getKeyStroke(KeyEvent.VK_DOWN, 0);	

		getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
				nextFrameKeyStroke, "nextFrame");
		
		getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
				prevFrameKeyStroke, "prevFrame");
		
		getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
				nextUtteranceKeyStroke, "nextUtterance");
		
		getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
				prevUtteranceKeyStroke, "prevUtterance");
	
		getRootPane().getActionMap().put("nextFrame", new AbstractAction() {
			@Override
			public void actionPerformed(ActionEvent e) {
                SS3Singleton.getMediaToolBar(null);
                var player = SS3Singleton.getMediaPlayer();
                var step = Util.getStep();
                var range = Util.getStepRange();
                player.pause();
                player.seek(player.time().add(step).in(range));
            }
		});
		
		getRootPane().getActionMap().put("prevFrame", new AbstractAction() {
			@Override
			public void actionPerformed(ActionEvent e) {
                SS3Singleton.getMediaToolBar(null);
                var player = SS3Singleton.getMediaPlayer();
                var step = Util.getStep();
                var range = Util.getStepRange();
                player.pause();
                player.seek(player.time().subtract(step).in(range));
            }
		});
		
		getRootPane().getActionMap().put("nextUtterance", new AbstractAction() {
			@Override
			public void actionPerformed(ActionEvent e) {
				if (getSignStreamSegmentPanel().isUtteranceLoaded()) {
					panel.getUtteranceView().getSelectedSegmentTemplate().getSegmentControlPanel().offsetSelectedUtterance(1);
				}
			}
		});
		
		getRootPane().getActionMap().put("prevUtterance", new AbstractAction() {
			@Override
			public void actionPerformed(ActionEvent e) {
				if (getSignStreamSegmentPanel().isUtteranceLoaded()) {
					panel.getUtteranceView().getSelectedSegmentTemplate().getSegmentControlPanel().offsetSelectedUtterance(-1);
				}
			}
		});

		deleteTempFiles();

//		try {
//			FileHandler fh = new FileHandler(SSResources.rootPath + "/SignStreamLog.txt");
//			logger.addHandler(fh);
//		} catch (Exception ex) {
//			ex.printStackTrace();
//			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);
//		}

		Toolkit.getDefaultToolkit().addAWTEventListener( new AWTEventListener() {
		    @Override
			public void eventDispatched(AWTEvent e) {
		        if (e.getID() == KeyEvent.KEY_PRESSED) {
                    if (((KeyEvent) e).getKeyCode() == KeyEvent.VK_ESCAPE) {
                        Component comp = (Component) e.getSource();
                        while (!(comp.getParent() == null || comp instanceof Window)) {
                            comp = comp.getParent();
                        }
                        if (comp instanceof Window) {
                            if (!Constants.importantDialogs.contains(comp.getClass()) &&
                                    (!(comp instanceof JDialog && ((JDialog)comp).isModal()))) {
                                comp.dispatchEvent(new WindowEvent((Window) comp, WindowEvent.WINDOW_CLOSING));
                            }
                        }
                    }
                }
		    }
        }, AWTEvent.KEY_EVENT_MASK);

		KeyboardFocusManager.getCurrentKeyboardFocusManager()
				.addPropertyChangeListener("permanentFocusOwner", new PropertyChangeListener() {
			@Override
			public void propertyChange(final PropertyChangeEvent e){
				if (e.getNewValue() instanceof JButton) {
					JButton btn = (JButton) e.getNewValue();
					for (KeyListener keyListener : btn.getKeyListeners()) {
						if (keyListener instanceof EnterKeyListener) {
							return;
						}
					}
					btn.addKeyListener(new EnterKeyListener(btn));
				}
			}
		});

		logger.log(Level.INFO, "Starting the Sign Stream application... ");

		String dv = System.getProperty("file.separator", "/");
		path = SSResources.xmlPath + dv;
		scPath = path + "defCodingScheme.xml";

		this.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);

		parser = new SS3Parser();
		
		SS3Singleton.getSS3CodingSchemeCollection().setDefaultCodingScheme(parseDefaultSS3CodingScheme());

		// TODO: Generate these ranges from the videos
//		var videoRange = TimeRange.ABSOLUTE.subrange("Video", TimeUnit.TIMESTAMP);
//		var audioRange = TimeRange.ABSOLUTE.subrange("Audio", TimeUnit.MICROSECOND);
//		SS3Singleton.setVideoTimeRange(videoRange);
//		SS3Singleton.setAudioTimeRange(audioRange);

		SS3Database ss3db = new SS3Database();
//		SS3Database ss3db = new SS3Database(
//				new TimeInfo(0, TimeUnit.VIDEO_FRAME_30_FPS),
//				new TimeInfo(0, TimeUnit.VIDEO_FRAME_30_FPS));

		ss3db.setSS3GlossWindow(createNewGlossWindow());
		Excerpt excerpt = new Excerpt(Util.getUniqueNumber(),
				ss3db.getFileName());
		ss3db.setExcerptObj(excerpt);

		boolean menuInstallSignBank = false; // set to 'uninstall' by default, true means 'install'

		// Begin: Startup Sign Bank check
		// On startup, check if we are running the latest sign bank file version
		Process proc = null;
		int ret = BASH_EXIT_CODE_NO;
		
		try {
			ProcessBuilder p = new ProcessBuilder("/bin/bash", BASH_SIGN_BANK_SCRIPT, "--check_asllrp");
			ret = p.start().waitFor();
		} catch (Exception e) {
			System.out.println("Error running " + BASH_SIGN_BANK_SCRIPT + " --check_asllrp");
			logger.log(Level.INFO, "Error running " + BASH_SIGN_BANK_SCRIPT + " --check_asllrp");
			logger.log(Level.INFO, "Exception: " + e.getMessage());
			
			menuInstallSignBank = true;
		}
		
		if (ret == BASH_EXIT_CODE_YES) {	// sign bank is from ASLLRP
			ret = BASH_EXIT_CODE_NO;
			try {	
				ProcessBuilder p = new ProcessBuilder("/bin/bash", BASH_SIGN_BANK_SCRIPT, "--check_outdated");
				ret = p.start().waitFor();
			} catch (Exception e) {
				System.out.println("Error running " + BASH_SIGN_BANK_SCRIPT + " --check_outdated");
				logger.log(Level.INFO, "Error running " + BASH_SIGN_BANK_SCRIPT + " --check_outdated");
				logger.log(Level.INFO, "Exception: " + e.getMessage());
				
				menuInstallSignBank = true;
			}
		} else if (ret == BASH_EXIT_CODE_NO) {	// sign bank is not from ASLLRP
			menuInstallSignBank = true;
		}

		if (ret == BASH_EXIT_CODE_YES) {	// if we are at a 'yes' here it means to do an update of the sign bank
			try {				
				JFrame dummyFrame = new JFrame();

				dummyFrame.setVisible(false);		// false because we don't want an extra frame in space
				dummyFrame.setLocationRelativeTo(null);		// centers the panel
				dummyFrame.setAlwaysOnTop(true);
				dummyFrame.toFront();

				String warningDialogText = "SignStream® will launch momentarily."
						+ "\n\nThe ASLLRP Sign Bank first needs to be updated (or deleted). "
						+ "\nPlease follow the steps in the Terminal window to do that."
						+ "\n\nThanks very much!";
				String warningTitleText = "Sign Bank Check";
				int optionType = JOptionPane.YES_NO_OPTION;
				int messageType = JOptionPane.WARNING_MESSAGE;
				String options[] = {"OK"};
				try {
					JOptionPane.showOptionDialog(dummyFrame,
							warningDialogText,
							warningTitleText,
							optionType,
							messageType,
							null,
							options,
							options[0]);
				} catch (HeadlessException he) {
					System.out.println("");
				}
				
				String currentDir = System.getProperty("user.dir");
				String command1 = 
						"tell application \"Terminal\" to do script \"" + BASH_SIGN_BANK_SCRIPT + "; exit; \"";
				String command2 = "tell application \"Terminal\" to activate";	// bring the Terminal window to the front
				String[] commands = {"osascript",  "-e",  command1, "-e", command2};

				ProcessBuilder p = new ProcessBuilder(commands);
//				p.inheritIO().start();	// leave commented - for debugging	
				Thread.sleep(1000);		// sleep for 1 second, give time for script to detect the signstream is running
				p.start().waitFor();
			} catch (Exception e) {
				System.out.println("'" + BASH_SIGN_BANK_SCRIPT + "' failed");
				logger.log(Level.INFO, "'" + BASH_SIGN_BANK_SCRIPT + "' failed");
				logger.log(Level.INFO, "Exception: " + e.getMessage());
				
				menuInstallSignBank = true;
			}
		}
		
		// test again to make sure the process went through and the user did not close the Terminal window
		ret = BASH_EXIT_CODE_NO;
		
		try {
			ProcessBuilder p = new ProcessBuilder("/bin/bash", BASH_SIGN_BANK_SCRIPT, "--check_asllrp");
			ret = p.start().waitFor();
		} catch (Exception e) {
			System.out.println("Error running " + BASH_SIGN_BANK_SCRIPT + " --check_asllrp");
			logger.log(Level.INFO, "Error running " + BASH_SIGN_BANK_SCRIPT + " --check_asllrp");
			logger.log(Level.INFO, "Exception: " + e.getMessage());
		}

		if (ret == BASH_EXIT_CODE_YES) {	// sign bank is from ASLLRP
			ret = BASH_EXIT_CODE_NO;
			try {	
				ProcessBuilder p = new ProcessBuilder("/bin/bash", BASH_SIGN_BANK_SCRIPT, "--check_outdated");
				ret = p.start().waitFor();

				// if we are at a 'yes' here on this second pass, it means the user closed the Terminal window in the first pass, so now close SignStream
				if (ret == BASH_EXIT_CODE_YES) {	
					System.exit(0);
				}		
			} catch (Exception e) {
				System.out.println("Error running " + BASH_SIGN_BANK_SCRIPT + " --check_outdated");
				logger.log(Level.INFO, "Error running " + BASH_SIGN_BANK_SCRIPT + " --check_outdated");
				logger.log(Level.INFO, "Exception: " + e.getMessage());
			}
		}
		// End: Startup Sign Bank check
		
		// At this point, we have either an empty global sign bank file, or an up-to-date global sign bank file
		
		// parse sign bank collection on startup as well
		parseSignBank(this, true);
		
		
		MenuItemsCollection menuItemsCollection = new MenuItemsCollection();
		JMenuBar mb = new JMenuBar();
		JMenu fileMenu = createFileMenu(menuItemsCollection);
		mb.add(fileMenu);

		JMenu muMenu = new JMenu("Macro Unit");
		JMenuItem newItem = new JMenuItem(" New Macro Unit ");
		JMenuItem renameItem = new JMenuItem(" Edit Macro Unit ");
		
		JMenu viewMenu = new JMenu("View");

		JMenu helpMenu = new JMenu("Help");
		JMenuItem helpItem = new JMenuItem(" Help... ");
		JMenuItem updateItem = new JMenuItem("Update SignStream...");

//		JMenuItem ipaItem = new JMenuItem("Show IPA Keyboard");
		
		// logic for what to display in menu for sign bank
		JMenuItem updateSignBankItem = null;
		if (menuInstallSignBank) {
			updateSignBankItem = new JMenuItem("Install/Update ASLLRP Sign Bank...");
			updateSignBankItem.setActionCommand(""+INSTALL_SIGN_BANK);
		} else {
			updateSignBankItem = new JMenuItem("Uninstall ASLLRP Sign Bank...");
			updateSignBankItem.setActionCommand(""+UNINSTALL_SIGN_BANK);
		}
		
		JMenuItem viewModifications = new JMenuItem("View Collection File Modifications");
		JMenuItem viewErrors = new JMenuItem("View Collection File Errors");
		
		//helpMenu.add(helpItem);
		helpItem.setActionCommand("" + HELP);
		helpItem.addActionListener(this);
		helpMenu.add(updateItem);
		updateItem.setActionCommand("" + UPDATE);
		updateItem.addActionListener(this);
		helpMenu.add(updateSignBankItem);
		updateSignBankItem.addActionListener(this);
		updateSignBankItem.setEnabled(true);
		
//		ipaItem.setActionCommand("" + SHOW_IPA);
//		ipaItem.addActionListener(this);
//		ipaItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_I, Event.META_MASK));
//		helpMenu.add(ipaItem);
		
		viewModifications.setActionCommand("" + VIEW_MODIFICATIONS);
		viewModifications.addActionListener(this);
		helpMenu.add(viewModifications);
		
		viewErrors.setActionCommand("" + VIEW_ERRORS);
		viewErrors.addActionListener(this);
		helpMenu.add(viewErrors);

		menuItemsCollection.setMacroUnitMenu(fileMenu);
		menuItemsCollection.setNewMacroUnitItem(newItem);
		menuItemsCollection.setEditMacroUnitItem(renameItem);
		muMenu.add(renameItem);

		newItem.setActionCommand("" + NEW_MACRO_UNIT);
		renameItem.setActionCommand("" + RENAME_MACRO_UNIT);

		newItem.addActionListener(this);
		renameItem.addActionListener(this);
		
		muMenu.addSeparator();
		
		JMenu sMenu = createSegmentMenu(menuItemsCollection);
		muMenu.add(sMenu);

		muMenu.addSeparator();
		
		JMenu tpMenu = createTemporalPartitionMenu(menuItemsCollection);
		muMenu.add(tpMenu);
			
		mb.add(muMenu);

		mb.add(viewMenu);
		
		mb.add(helpMenu);

		setJMenuBar(mb);

        toggleViewMenu(false);

		//If user is on Mac, add listener to ask user to save before quitting (via Command + Q or SignStream -> Quit)
		if (System.getProperty("os.name").toLowerCase().contains("mac")) {
			addQuitHandler();
		}

		panel = new SignStreamSegmentPanel(null, this); //SignStreamSegmentPanel(excerpt, ss3db, null, this);
		dbDelegate = panel.getDbNavigationPanel().getDatabaseDelegate();

		logger.log(Level.INFO, "Starting the Sign Stream application... 1");

		// handles save/dont-save on clicking 'x' to close window
        addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent e) {
                exit();
            }
        });

		logger.log(Level.INFO, "Starting the Sign Stream application... 2");

		logger.log(Level.INFO, "Starting the Sign Stream application... 3");

		SS3Singleton.loadSignStreamApplication(this);

		logger.log(Level.INFO, "Starting the Sign Stream application... 4");

        child().add(panel);

        logger.log(Level.INFO,
        		"The Sign Stream application started successfully");
	}

	@Override
	public void block() {
        super.block();

        if (!panel.isUtteranceLoaded())
            toggleMacroUnitMenu(!blocked());

        for (var m : SS3Singleton.getMediaPlayer().media())
            if (m.displayed())
                m.view().block();
    }

    @Override
    public void unblock() {
        super.unblock();

        if (!panel.isUtteranceLoaded())
            toggleMacroUnitMenu(!blocked());

        for (var m : SS3Singleton.getMediaPlayer().media())
            if (m.displayed())
                m.view().unblock();
    }

    @Override
    public boolean isFocusable() {
        return true;
    }

    /**
     * Due to a recurrent problem on OSX applications, JPopupMenu objects do not disappear when
     * the user switches to another application, and remains visible until the user switches back to the
     * Signstream application and clicks away from the JPopupMenu, which closes it.
     * This method is meant to be called by any client class in Signstream which uses a JPopupMenu object
     * and is experiencing this issue - here it is forced to invisible when the window focus switches.
     * @param popup The JPopupMenu which requires a forced setVisible(false) call on window focus switch
     */
    public void forcePopupInvisibleOnWindowLostFocus(final JPopupMenu popup) {
        final JFrame frame = this;	// need a final reference to this SS3SignStreamApplication object for the inner class below
        frame.addWindowFocusListener(new WindowFocusListener() {
            @Override public void windowGainedFocus(WindowEvent arg0) {}
            @Override public void windowLostFocus(WindowEvent arg0) {
                popup.setVisible(false);
                frame.removeWindowFocusListener(this);
            }
        });

    }

    public SignStreamSegmentPanel getSignStreamSegmentPanel() {
        return panel;
    }

    public DatabaseDelegate getDatabaseDelegate() {
        return dbDelegate;
    }

    public CreateUpdateMacroUnitDialog getMacroUnitDialog() {
        return macroUnitDialog;
    }

    public SS3CodingScheme parseDefaultSS3CodingScheme() {
        var reader = new SS3Reader();
        return reader.parseCodingScheme(scPath);
	}

	private void loadNewMacroUnitDialog(SS3Database ss3db) {
		Point location = new Point(300, 100);

		if (macroUnitDialog != null) {
			macroUnitDialog.setVisible(false);
			location = macroUnitDialog.getLocation();
			macroUnitDialog.dispose();
			macroUnitDialog = null;
		}

		String defCollectionCreatedOnLoad = resetTitleToDefault();
		
		SS3Singleton.getFilesLocation().setLastOpenedDatabaseLocation(
				FilesLocation.defaultDatabaseLocation
				+ File.separator
				+ defCollectionCreatedOnLoad
				+ SS3Database.COLLECTION_EXT);
		
		DbNavigationPanel navigation = panel.getDbNavigationPanel();
		navigation.getDatabaseDelegate().addNewDatabase(
				defCollectionCreatedOnLoad);
		navigation.refresh(navigation.getDatabaseDelegate()
				.getMutableTreeNode());
		navigation.getJTree().repaint();

		String globalParticipantsFileName = path + "globalParticipants.xml";
		String localParticipantsFileName = path + "localParticipants.xml";
		String globalAnnotatorsFileName = path + "globalAnnotators.xml";
		String localAnnotatorsFileName = path + "localAnnotators.xml";
		String mediaLibraryFileName = path + "mediaLibrary.xml";
		
		SS3Singleton.getManager().loadParticipantsLibrary(globalParticipantsFileName, localParticipantsFileName);
		SS3Singleton.getManager().loadAnnotatorsLibrary(globalAnnotatorsFileName, localAnnotatorsFileName);	
		
		ParticipantsLibrary participantsLibrary = SS3Singleton.getManager().getLoadedParticipantsLibrary();
		ParticipantsLibrary annotatorsLibrary = SS3Singleton.getManager().getLoadedAnnotatorsLibrary();

		SS3MediaLibrary mLibrary = parser
				.parseMediaLibrary(mediaLibraryFileName);

		if(macroUnitDialog == null) {
			macroUnitDialog = new CreateUpdateMacroUnitDialog(
					CreateUpdateMacroUnit.FUNCTION_ADD, ss3db,
					ss3db.getExcerptObj(), new ArrayList<LoadedMediaFile>(), new ArrayList<LoadedMediaFile>(),
					mLibrary, participantsLibrary, annotatorsLibrary,
					signBankCollection.getSignBankResourceCollection(),
					panel, this);
		} else {
			macroUnitDialog.update(
					CreateUpdateMacroUnit.FUNCTION_ADD, ss3db,
					ss3db.getExcerptObj(), new ArrayList<LoadedMediaFile>(), new ArrayList<>(),
					mLibrary, participantsLibrary, annotatorsLibrary,
					signBankCollection.getSignBankResourceCollection());
		}

		macroUnitDialog.setVisible(true);
		macroUnitDialog.setLocation(location);
	}

	@Override
	public void keyTyped(KeyEvent e) {
		displayInfo(e, "KEY TYPED: ");
	}

	/** Handle the key-pressed event from the text field. */
	@Override
	public void keyPressed(KeyEvent e) {
		displayInfo(e, "KEY PRESSED: ");
	}

	/** Handle the key-released event from the text field. */
	@Override
	public void keyReleased(KeyEvent e) {
		displayInfo(e, "KEY RELEASED: ");
	}

	private void displayInfo(KeyEvent e, String keyStatus) {
		int modifiersEx = e.getModifiersEx();
		String modString = "extended modifiers = " + modifiersEx;
		String tmpString = InputEvent.getModifiersExText(modifiersEx);
		if (tmpString.length() > 0) {
			modString += " (" + tmpString + ")";
		} else {
			modString += " (no extended modifiers)";
		}

		String actionString = "action key? ";
		if (e.isActionKey()) {
			actionString += "YES";
		} else {
			actionString += "NO";
		}

		String locationString = "key location: ";
		int location = e.getKeyLocation();
		if (location == KeyEvent.KEY_LOCATION_STANDARD) {
			locationString += "standard";
		} else if (location == KeyEvent.KEY_LOCATION_LEFT) {
			locationString += "left";
		} else if (location == KeyEvent.KEY_LOCATION_RIGHT) {
			locationString += "right";
		} else if (location == KeyEvent.KEY_LOCATION_NUMPAD) {
			locationString += "numpad";
		} else { // (location == KeyEvent.KEY_LOCATION_UNKNOWN)
			locationString += "unknown";
		}
	}

	private JMenu createSegmentMenu(MenuItemsCollection menuItemsCollection) {
		JMenu sMenu = new JMenu("Segment Tier");
		JMenuItem newItem = new JMenuItem(" New Segment Tier of Selected Temporal Partition");
		JMenuItem renameItem = new JMenuItem(" (Re)name Selected Segment Tier");
		JMenuItem renameParticipantItem = new JMenuItem(" (Re)name Selected Participant");
		JMenuItem deleteItem = new JMenuItem(" Delete Selected Segment Tier");
		JMenuItem setDefaultItem = new JMenuItem(" Set Selected Segment Tier as Default");

		menuItemsCollection.setSegmentMenu(sMenu);
		menuItemsCollection.setNewSegmentItem(newItem);
		menuItemsCollection.setRenameSegmentItem(renameItem);
		menuItemsCollection.setRenameParticipantItem(renameParticipantItem);
		menuItemsCollection.setDeleteSegmentItem(deleteItem);
		menuItemsCollection.setSetDefaultSegmentItem(setDefaultItem);

		sMenu.add(newItem);
		sMenu.add(renameItem);
		sMenu.add(renameParticipantItem);
		sMenu.add(deleteItem);
		sMenu.add(setDefaultItem);

		newItem.setActionCommand("" + CREATE_SEGMENT);
		renameItem.setActionCommand("" + RENAME_SEGMENT);
		renameParticipantItem.setActionCommand("" + RENAME_PARTICIPANT);
		deleteItem.setActionCommand("" + DELETE_SEGMENT);
		setDefaultItem.setActionCommand("" + SET_DEFAULT_SEGMENT);

		newItem.addActionListener(this);
		renameItem.addActionListener(this);
		renameParticipantItem.addActionListener(this);
		deleteItem.addActionListener(this);
		setDefaultItem.addActionListener(this);

		return sMenu;
	}
	
	private JMenu createTemporalPartitionMenu(MenuItemsCollection menuItemsCollection) {
		JMenu tpMenu = new JMenu("Temporal Partition");
		JMenuItem newItem = new JMenuItem(" New Temporal Partition");
		JMenuItem renameItem = new JMenuItem(" (Re)name Selected Temporal Partition");
		JMenuItem deleteItem = new JMenuItem(" Delete Selected Temporal Partition");
		JMenuItem renumberItem = new JMenuItem(" Renumber Utterances of Selected Temporal Partition");

		menuItemsCollection.setSegmentMenu(tpMenu);
		menuItemsCollection.setNewSegmentItem(newItem);
		menuItemsCollection.setRenameSegmentItem(renameItem);
		menuItemsCollection.setDeleteSegmentItem(deleteItem);

		tpMenu.add(newItem);
		tpMenu.add(renameItem);
		tpMenu.add(deleteItem);
		tpMenu.add(renumberItem);

		newItem.setActionCommand("" + CREATE_TEMPORAL_PARTITION);
		deleteItem.setActionCommand("" + DELETE_TEMPORAL_PARTITION);
		renameItem.setActionCommand("" + RENAME_TEMPORAL_PARTITION);
		renumberItem.setActionCommand("" + RENUMBER_UTTERANCES);

		newItem.addActionListener(this);
		renameItem.addActionListener(this);
		deleteItem.addActionListener(this);
		renumberItem.addActionListener(this);

		return tpMenu;
	}

	private JMenu createFileMenu(MenuItemsCollection menuItemsCollection) {

		menuItemsCollection.setCollectionMenu(fileMenu);
		menuItemsCollection.setNewCollectionItem(newItem);
		menuItemsCollection.setOpenCoolectionItem(openAsItem);
		menuItemsCollection.setOpenLegacyItem(openLegacyItem);
		menuItemsCollection.setCloseCollectionItem(closeItem);
		menuItemsCollection.setSaveCollectionItem(saveItem);
		menuItemsCollection.setSaveAsCollectionItem(saveAsItem);
		menuItemsCollection.setNewSearchCollectionItem(newSearch);
		menuItemsCollection.setSavedSearchCollectionItem(savedSearch);


		fileMenu.add(newItem);
		fileMenu.add(openAsItem);
		fileMenu.addSeparator();
		fileMenu.add(closeItem);
		fileMenu.add(saveItem);

		saveItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S,
                																			java.awt.Event.META_MASK));
		fileMenu.add(saveAsItem);
		fileMenu.addSeparator();
		fileMenu.add(newSearch);
		fileMenu.add(savedSearch);
		newSearch.setEnabled(false);
		savedSearch.setEnabled(false);

		newItem.setActionCommand("" + NEW_COLLECTION);
		newItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_N,
																						java.awt.Event.META_MASK));
		
		openAsItem.setActionCommand("" + OPEN_AS_SS3);

        openAsItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_O,
                																				java.awt.Event.META_MASK));
		closeItem.setActionCommand("" + CLOSE_DB);
		closeItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_W,
																						java.awt.Event.META_MASK));
		saveItem.setActionCommand("" + SAVE_DB);
		saveAsItem.setActionCommand("" + SAVE_AS_DB);
		newSearch.setActionCommand("" + NEW_SEARCH);
		savedSearch.setActionCommand("" + SAVED_SEARCH);

		newItem.addActionListener(this);
		openAsItem.addActionListener(this);
		closeItem.addActionListener(this);
		saveItem.addActionListener(this);
		saveAsItem.addActionListener(this);
		newSearch.addActionListener(this);
		savedSearch.addActionListener(this);

		return fileMenu;
	}

	private SS3GlossWindow createNewGlossWindow() {
		SS3GlossWindow glsWindow = new SS3GlossWindow();

		SS3GlossWindowSegment displayTimePeriod = new SS3GlossWindowSegment();
		displayTimePeriod.setLabel("Display Time Period");
		glsWindow.setDisplayTimePeriod(displayTimePeriod);

		SS3TemporalPartition temporalPartition = new SS3TemporalPartition();
		temporalPartition.setLabel("Temporal Partition 1");

		SS3GlossWindowSegment tpSegment = new SS3GlossWindowSegment();
		tpSegment.setLabel("Segment Tier 1");
		temporalPartition.addTemporalPartitionSegment(tpSegment);
		temporalPartition.setDefaultSegment(tpSegment);

		ArrayList<SS3TemporalPartition> partitions = new ArrayList<SS3TemporalPartition>();
		partitions.add(temporalPartition);
		glsWindow.setTemporalPartitions(partitions);

		return glsWindow;
	}

	public File openZipDatabase() {

		File selectedFile = null;
		FilesLocation location = SS3Singleton.getFilesLocation();

		if ("com.sun.java.swing.plaf.windows.WindowsLookAndFeel"
				.equalsIgnoreCase(UIManager.getSystemLookAndFeelClassName())) {
			JFileChooser fc = new JFileChooser();
			fc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES); // .DIRECTORIES_ONLY);
																			// //
																			// .FILES_AND_DIRECTORIES);
			String dbLocation = location.getLastOpenedDatabaseLocation();

			fc.addChoosableFileFilter(new SignStream3FilesFilter());

			File file = new File(dbLocation);
			String absPath = file.getAbsolutePath();

			fc.setSelectedFile(new File(absPath));
			fc.setFileView(new ImageFileView());

			int returnVal = fc.showOpenDialog(this);

			if (returnVal == JFileChooser.APPROVE_OPTION) {
				selectedFile = fc.getSelectedFile();

				logger.log(Level.INFO, "Opening [" + selectedFile.getName()
						+ "]");
			} else {
				logger.log(Level.INFO, "Open command cancelled by user.");
			}
		} else {

			FileDialog fd = new FileDialog(this, "Open a Collection",
					FileDialog.LOAD);
			String dbLocation = location.getLastOpenedDatabaseLocation();
			File file = new File(dbLocation);
			String absPath = file.getAbsolutePath();
			fd.setFilenameFilter(new ZipFormatFilenameFilter());
			fd.setDirectory(absPath);
			fd.setFile(absPath);

            getJMenuBar().setEnabled(false);
			fd.setVisible(true);
			getJMenuBar().setEnabled(true);

			System.setProperty("apple.awt.fileDialogForDirectories", "false");

			if (fd.getFile() != null) {
				String selectedFileName = fd.getDirectory() + File.separator
						+ fd.getFile();
				selectedFile = new File(selectedFileName);
			}
		}
		return selectedFile;
	}

	private void saveChangesToTheCollection(final boolean loadNewDatabase) {
		final JTree tree = panel.getDbNavigationPanel().getJTree();

		final DefaultMutableTreeNode rootNode = dbDelegate.getMutableTreeNode();
		
		Runnable run = () -> {
			SS3DatabaseCollection dbCol = SS3Singleton
					.getSS3DatabaseCollection();
			dbCol.removeAll();
			SS3Singleton.getApplicationStateController()
			.setLoadedCollectionUpdated(false);

			panel.closeAllLoadedVideoFiles();

			rootNode.removeAllChildren();
			((DefaultTreeModel) tree.getModel()).reload(rootNode);

			SS3Singleton.getSS3DatabaseCollection().removeAll();

			if (loadNewDatabase)
				panel.loadDatabase(null);

			System.exit(0);
		};
		
		if (SS3Singleton.getApplicationStateController().isLoadedCollectionUpdated())
            new SaveDialog(run, () -> {}).display();
        else
            run.run();
	}

	private void openCollection() {
        var tree = panel.getDbNavigationPanel().getJTree();
        dbDelegate = panel.getDbNavigationPanel().getDatabaseDelegate();
        var root = dbDelegate.getMutableTreeNode();
        DefaultMutableTreeNode node = null;
        if (root.getChildCount() > 0) {
            node = (DefaultMutableTreeNode) root.getChildAt(0);
            if (node.getChildCount() > 0) {
                DefaultMutableTreeNode dNode = (DefaultMutableTreeNode) node.getChildAt(0);
                if (dNode.getUserObject() instanceof String)
                    node = dNode;
            }
        }
        var fnode = node;

        Runnable open = () -> {
            var file = openZipDatabase();
            if (file == null && macroUnitDialog != null && getSignStreamSegmentPanel().getSS3Database().isEmpty())
                macroUnitDialog.setVisible(true);
            else if (file == null)
                return;

			closeCollection();

            // Workaround for a problem where SAVE_AS hangs after OPEN.
            // New clears the problem, but don't know why.
            newCollection();
            macroUnitDialog.setVisible(false);

            var opened = openDatabaseSS3(file, tree, root, fnode);
//            unblock();

            if (opened) {
                getSignStreamSegmentPanel().getDatabaseView()
                    .getUpperLeft()
                    .setZoom(0);
                getSignStreamSegmentPanel().getDatabaseView()
                    .getUpperLeft()
                    .lock(false);
                getSignStreamSegmentPanel().zoomLock(false);

                // select annotator screen
                AnnotatorSelectionDialog annotatorSelectionDialog = new AnnotatorSelectionDialog(
                    SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().getSS3Database().getSelectedAnnotators(),
                    macroUnitDialog.getMacroUnitPanel().getAnnotatorsDialog(), this);
                annotatorSelectionDialog.setLocation(250, 250);
                annotatorSelectionDialog.setVisible(true);

    			SS3Singleton.bringAllVideosToFront();
            } else if (macroUnitDialog != null && getSignStreamSegmentPanel().getSS3Database().isEmpty()) {
                macroUnitDialog.setVisible(true);
            }
        };

        if (SS3Singleton.getApplicationStateController().isLoadedCollectionUpdated())
            new SaveDialog(open, () -> {}).display();
        else
            open.run();
    }
	
	public void closeUtteranceViewDialogs() {
        if (panel.isUtteranceLoaded()) {
            if (panel.isUtteranceLoaded()) {
                for (var template : panel.getUtteranceView().getSegmentTemplates()) {
                    template.getNonmanView().getCSFieldDialog().setVisible(false);
                }
            }
        }

        if (searchDialog != null)
            searchDialog.setVisible(false);
        searchDialog = null;
        
        	if (validationErrorsDialog != null)
        		validationErrorsDialog.setVisible(false);
        	validationErrorsDialog = null;
        	
        	if (validationModificationsDialog != null)
        		validationModificationsDialog.setVisible(false);
        	validationModificationsDialog = null;
        	
        	deleteValidationFiles();

        if (SS3Singleton.getNonManuelFieldDialogs().getMorphologicalPhonologicalInfoDialog() != null)
            SS3Singleton.getNonManuelFieldDialogs().getMorphologicalPhonologicalInfoDialog().discardChanges();
        
        if (SS3Singleton.getGestureFieldDialogs().getGestureInfoDialog() != null)
        	SS3Singleton.getGestureFieldDialogs().getGestureInfoDialog().discardChanges();
	}
	
	private String resetTitleToDefault() {
		Calendar rightNow = Calendar.getInstance();
		int dayOfMonth = rightNow.get(Calendar.DAY_OF_MONTH);
		int month = rightNow.get(Calendar.MONTH) + 1;
		int year = rightNow.get(Calendar.YEAR);
		int timeHrs = rightNow.get(Calendar.HOUR_OF_DAY);
		int timeMin = rightNow.get(Calendar.MINUTE);
		int timeSec = rightNow.get(Calendar.SECOND);

		String defCollectionCreatedOnLoad = "Collection_" + month + "_"
				+ dayOfMonth + "_" + year + "_" + timeHrs + "_" + timeMin + "_"
				+ timeSec;

		setTitle(defCollectionCreatedOnLoad);
		
		return defCollectionCreatedOnLoad;
	}

	private void closeCollection() {
	    unblockAll();

        if (macroUnitDialog != null)
            macroUnitDialog.dispose();
        
        if (validationErrorsDialog != null)
        		validationErrorsDialog.dispose();

		File validationErrorsFile = new File(VALIDATION_ERRORS);
		if (validationErrorsFile.exists()) {
			validationErrorsFile.delete();
		}
        
        closeUtteranceViewDialogs();

        if (panel.isUtteranceLoaded()) {
            toggleSearchOption(false);
            toggleMacroUnitMenu(true);
            toggleViewMenu(false);
        }
        
        if (panel.isUtteranceLoaded())
            panel.getUtteranceView().getHeader().loadDatabaseView();

        SS3Singleton.getMediaPlayer().close();
        SS3Singleton.getMediaToolBar().reset();

        panel.loadDatabase(null);
        panel.resetPainter();
        panel.getSegments().clear();
        if (panel.getUtteranceView() != null)
        		panel.getUtteranceView().clearSegmentTemplates();
        
        SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(false);

		resetTitleToDefault();

        block();
    }

	private void newCollection() {
	    closeUtteranceViewDialogs();
        
        if (validationErrorsDialog != null)
        		validationErrorsDialog.dispose();

	    if (panel.isUtteranceLoaded()) {
            SS3Singleton.getSignStreamApplication().toggleViewMenu(false);
            SS3Singleton.getSignStreamApplication().toggleMacroUnitMenu(true);
            SS3Singleton.getSignStreamApplication().toggleSearchOption(false);
        }

        unblockAll();

        var tree = panel.getDbNavigationPanel().getJTree();
        dbDelegate = panel.getDbNavigationPanel().getDatabaseDelegate();
        var root = dbDelegate.getMutableTreeNode();

        root.removeAllChildren();
        ((DefaultTreeModel) tree.getModel()).reload(root);
        SS3Singleton.getSS3DatabaseCollection().removeAll();

        // below taken from constructor - mimicking functionality
        parser = new SS3Parser();

        SS3Database ss3db = new SS3Database();

        ss3db.setSS3GlossWindow(createNewGlossWindow());
        Excerpt excerpt = new Excerpt(Util.getUniqueNumber(),
            ss3db.getFileName());
        ss3db.setExcerptObj(excerpt);

        panel.loadDatabase(null);
        panel.loadDatabase(ss3db);
        
        // parse sign bank collection on startup as well
        parseSignBank(this);

        SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(false);

        getSignStreamSegmentPanel().getDatabaseView()
            .getUpperLeft()
            .setZoom(0);
        getSignStreamSegmentPanel().getDatabaseView()
            .getUpperLeft()
            .lock(false);
        getSignStreamSegmentPanel().zoomLock(false);

        loadNewMacroUnitDialog(ss3db);
    }
	
	public File saveCollection(int saveType) {
		return saveCollection(saveType, false);
	}

    public File saveCollection(int saveType, boolean makeBackup) {
        dbDelegate = panel.getDbNavigationPanel().getDatabaseDelegate();
        var root = dbDelegate.getMutableTreeNode();

        DefaultMutableTreeNode node = null;
        if (root.getChildCount() > 0) {
            node = (DefaultMutableTreeNode) root.getChildAt(0);

            if (node.getChildCount() > 0) {
                DefaultMutableTreeNode dNode = (DefaultMutableTreeNode) node
                    .getChildAt(0);
                Object uo = dNode.getUserObject();
                if (uo instanceof String) {
                    node = dNode;
                }
            }
        }

        return saveDatabaseToFile(node, saveType, makeBackup);
    }

    /**
     * Deals with event handling when user selects a menu option from the top menu bar.
     */
    @Override
    public void actionPerformed(ActionEvent ae) {
        String str = ae.getActionCommand();
        Integer intstr = Integer.parseInt(str);

        if (intstr != VIEW_ERRORS && intstr != VIEW_MODIFICATIONS) {
        		panel.updateLoadedDatabase();
        }

        final JTree tree = panel.getDbNavigationPanel().getJTree();
		dbDelegate = panel.getDbNavigationPanel().getDatabaseDelegate();
		final DefaultMutableTreeNode rootNode = dbDelegate.getMutableTreeNode();
		int childCount = rootNode.getChildCount();
		DefaultMutableTreeNode node = null;

		if (childCount > 0) {
			node = (DefaultMutableTreeNode) rootNode.getChildAt(0);

			if (node.getChildCount() > 0) {
				DefaultMutableTreeNode dNode = (DefaultMutableTreeNode) node
						.getChildAt(0);
				Object uo = dNode.getUserObject();
				if (uo instanceof String) {
					node = dNode;
				}
			}
		}

		MorphologicalPhonologicalInfoDialog dialog;

		switch (intstr) {

		case OPEN_AS_SS3:
			logger.log(Level.INFO, "Opening a collection...");
			openCollection();

			break;

		case NEW_COLLECTION:
			
			Runnable run = () -> {
				closeCollection();
				newCollection();
			};
			
			if (SS3Singleton.getApplicationStateController().isLoadedCollectionUpdated())
	            new SaveDialog(run, () -> {}).display();
	        else
	            run.run();

			break;
		case CLOSE_DB:

			if (node == null) {
				confirmationDialog("Save Resource", "Collection not available.");
				return;
			}

			run = () -> {
				closeCollection();
			};

			if (SS3Singleton.getApplicationStateController().isLoadedCollectionUpdated())
				new SaveDialog(run, () -> {}).display();
			else
				run.run();

			break;

		case SAVE_DB:
			if (node == null || !node.children().hasMoreElements()) {
				confirmationDialog("Save Resource", "Collection is not available.");
				return;
			}

			closeUtteranceViewDialogs();
			saveCollection(SS3SignStreamApplication.SAVE);
			unblock();
			break;

		case SAVE_AS_DB:
			if (node == null || !node.children().hasMoreElements()) {
				confirmationDialog("Save Resource", "Collection not available.");
				return;
			}

			closeUtteranceViewDialogs();
			saveCollection(SS3SignStreamApplication.SAVE_AS);
			unblock();
			break;
			
		case NEW_SEARCH:
			if (searchDialog == null || searchDialog.getSearchPanel() == null) {
				searchDialog = new SearchDialog(this);
				searchDialog.setVisible(true);
			}
			break;

		case CREATE_SEGMENT:
			logger.log(Level.INFO, "Create Segment");	
			if (node == null) {
				confirmationDialog("Create Segment Warning",
						"Please select a Macro Unit and try again");
			} else {
				
				DefaultMutableTreeNode labelViewSelectedNode 
					= panel.getDatabaseView().getLabelView().getSelectedNode();
				DefaultMutableTreeNode prevSiblingNode
					= labelViewSelectedNode.getPreviousSibling();
				int level = labelViewSelectedNode.getLevel();
				
				if (prevSiblingNode == null && level == 1) {
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_SELECT_DISPLAY_TIME_PERIOD);
					return;
				}
				ArrayList<SS3Participant> participants = panel.getSS3Database().getSelectedParticipants();
				if (participants.size() == 1) {
					panel.createSegment(participants.get(0));
				}
				else {					
					// select participant screen	
					AnnotatorSelectionDialog annotatorSelectionDialog = new AnnotatorSelectionDialog(
							participants, 
							macroUnitDialog.getMacroUnitPanel().getParticipantsDialog(), macroUnitDialog);
					annotatorSelectionDialog.setLocation(250, 250);
					annotatorSelectionDialog.setVisible(true);
					
					// get last selected item in list of participants before confirmation, then pass it along to createSegment()
					SS3Participant participant = annotatorSelectionDialog.getAnnotatorSelectionPanel().getSelectedAnnotator();
					
					panel.createSegment(participant);	
					
					SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
				}
			}
			break;

		case RENAME_PARTICIPANT:
			logger.log(Level.INFO, "Rename Selected Participant");	
			if (node == null) {
				confirmationDialog("Rename Selected Participant Warning",
						"Please select a Macro Unit and try again");
			} else {
				
				int level = panel.getDatabaseView().getLabelView().getSelectedNode().getLevel();
				if (level != 2) {
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_EDIT_NON_SEGMENT_TIER);
					return;
				}
				else {		
					// select participant screen	
					AnnotatorSelectionDialog annotatorSelectionDialog = new AnnotatorSelectionDialog(
							SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().getSS3Database().getSelectedParticipants(), 
							macroUnitDialog.getMacroUnitPanel().getParticipantsDialog(), this);
					annotatorSelectionDialog.setLocation(250, 250);
					annotatorSelectionDialog.setVisible(true);
					
					// get last selected item in list of participants before confirmation, then pass it along to createSegment()
					SS3Participant participant = annotatorSelectionDialog.getAnnotatorSelectionPanel().getSelectedAnnotator();
					
					panel.renameParticipant(participant);
				}
			}
			break;
		
		case RENAME_SEGMENT:
			logger.log(Level.INFO, "Rename Selected Segment");
			
			if (node == null) {
				confirmationDialog("Rename Selected Segment Warning",
						"Please select a Macro Unit and try again");
			} else {
				
				int level = panel.getDatabaseView().getLabelView().getSelectedNode().getLevel();
				if (level != 2) {
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_EDIT_NON_SEGMENT_TIER);
				}
				else {
					RenameSegmentDialog renameDialog = new RenameSegmentDialog(panel);
					renameDialog.setLocation(150, 150);
					renameDialog.setVisible(true);
					
					if (renameDialog.getText() == null) {
						SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.VALUE_MUST_MATCH_CHARACTERS);
						break;
					}
					
					String txt = renameDialog.getText().trim();
					
					if (txt.isEmpty()) {
						SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.VALUE_MUST_MATCH_CHARACTERS);
						break;
					}
					
					boolean matches = txt.matches(Util.LABEL_REGEX_PATTERN);
					if (!matches) {
						SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.VALUE_MUST_MATCH_CHARACTERS);
						break;
					}
					
					panel.renameSegment(txt);	
					SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
				}
			}

			break;

		case DELETE_SEGMENT:
			logger.log(Level.INFO, "Delete Segment");	
			if (node == null) {
				confirmationDialog("Delete Segment Warning",
						"Please select a Macro Unit and try again");
			} else {
				int level = panel.getDatabaseView().getLabelView().getSelectedNode().getLevel();
				if (level != 2) {
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_EDIT_NON_SEGMENT_TIER);
				}
				else {
					DefaultMutableTreeNode parentOfSelectedNode = 
							(DefaultMutableTreeNode) panel.getDatabaseView().getLabelView().getSelectedNode().getParent();
				
					// there is only one segment tier child of this temporal partition - can't delete it
					if(parentOfSelectedNode.getChildCount() == 1)	{
						confirmationDialog("Delete Segment Tier",
								"You cannot delete the last Segment Tier of any given Temporal Partition");
					} 
					else {
						LabelObject selectedLabelObject = (LabelObject) panel.getDatabaseView().getLabelView().getSelectedNode().getUserObject();
						String label = selectedLabelObject.getLabel();
						
						String warningDialogText =
								"All annotated data for segment \""+ label + "\" will be DELETED.\n\n"
								+ "Are you absolutely sure you want to delete segment \"" + label + "\"?";
						String warningTitleText = "WARNING";
						int optionType = JOptionPane.YES_NO_OPTION;
						int messageType = JOptionPane.ERROR_MESSAGE;
						String options[] = {"Proceed", "Cancel"};
						
						int result = -1;
						try {
							result = JOptionPane.showOptionDialog(this, 
									warningDialogText, 
									warningTitleText, 
									optionType, 
									messageType, 
									null, 
									options, 
									options[1]);
						} catch (HeadlessException he) {
							logger.log(Level.INFO, he.getMessage());
						}
						
						if (result == JOptionPane.OK_OPTION)
							panel.deleteSegment();
							SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
					}					
				}
			}
			break;
			
		case SET_DEFAULT_SEGMENT:
			logger.log(Level.INFO, "Set Default Segment");
			if (node == null) {
				confirmationDialog("Set Default Segment Warning",
						"Please select a Macro Unit and try again");
			} else {
				
				int level = panel.getDatabaseView().getLabelView().getSelectedNode().getLevel();
				if (level != 2) {
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_EDIT_NON_SEGMENT_TIER);
				}
				else {
					panel.setDefaultSegment();
				}
			}
			
			break;

		case CREATE_TEMPORAL_PARTITION:
			logger.log(Level.INFO, "Create Temporal Partition");	
			if (node == null) {
				confirmationDialog("Create Temporal Partition Warning",
						"Please select a Macro Unit and try again");
			} else {
				
				DefaultMutableTreeNode labelViewSelectedNode 
					= panel.getDatabaseView().getLabelView().getSelectedNode();
				DefaultMutableTreeNode prevSiblingNode
					= labelViewSelectedNode.getPreviousSibling();
				int level = labelViewSelectedNode.getLevel();
				
				if (prevSiblingNode == null && level == 1) {
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_SELECT_DISPLAY_TIME_PERIOD);
					return;
				}
				else {
					ArrayList<SS3Participant> participants = panel.getSS3Database().getSelectedParticipants();
					if (participants.size() == 1) {
						panel.createTemporalPartition(participants.get(0));
						
						SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
					}
					else {					
						// select participant screen	
						AnnotatorSelectionDialog annotatorSelectionDialog = new AnnotatorSelectionDialog(
								participants, 
								macroUnitDialog.getMacroUnitPanel().getParticipantsDialog(), this);
						annotatorSelectionDialog.setLocation(250, 250);
						annotatorSelectionDialog.setVisible(true);
						
						// get last selected item in list of participants before confirmation, then pass it along to createSegment()
						SS3Participant participant = annotatorSelectionDialog.getAnnotatorSelectionPanel().getSelectedAnnotator();
						
						panel.createTemporalPartition(participant);		
						
						SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
					}			
				}
			}
			break;
		case RENAME_TEMPORAL_PARTITION:
			logger.log(Level.INFO, "Rename Selected Temporal Partition");
			
			if (node == null) {
				confirmationDialog("Rename Selected Temporal Partition Warning",
						"Please select a Macro Unit and try again");
			} else {
				
				int level = panel.getDatabaseView().getLabelView().getSelectedNode().getLevel();
				if (level != 1 && level !=2) {
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_EDIT_NON_TEMPORAL_PARTITION);
				}
				else {
					DefaultMutableTreeNode labelViewSelectedNode = panel.getDatabaseView().getLabelView().getSelectedNode();
					DefaultMutableTreeNode prevSiblingNode = null;
					if(labelViewSelectedNode.getLevel() == 1)
						prevSiblingNode = labelViewSelectedNode.getPreviousSibling();
					else{
						DefaultMutableTreeNode parent = (DefaultMutableTreeNode)labelViewSelectedNode.getParent();
						prevSiblingNode = parent.getPreviousSibling();
					}
					
					if( prevSiblingNode == null )
					{
						confirmationDialog("Rename Selected Temporal Partiton",
								"You cannot rename the Display Time Period");
					}
					else
					{
						RenameTemporalPartitionDialog renameDialog = new RenameTemporalPartitionDialog(panel);
						renameDialog.setLocation(150, 150);
						renameDialog.setVisible(true);
						
						
						if (renameDialog.getText() == null) {
							SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.VALUE_MUST_MATCH_CHARACTERS);
							break;
						}
						
						String txt = renameDialog.getText().trim();
						
						if (txt.isEmpty()) {
							SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.VALUE_MUST_MATCH_CHARACTERS);
							break;
						}
						
						boolean matches = txt.matches(Util.LABEL_REGEX_PATTERN);
						if (!matches) {
							SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.VALUE_MUST_MATCH_CHARACTERS);
							break;
						}
						
						if(panel.getSS3Database().getSS3GlossWindow().getTemporalPartitionByLabel(txt) != null)
						{
							confirmationDialog("Rename Selected Temporal Partiton",
									"The name '" + txt + "' has already been used.  Please choose a different name");
						}
						else{
							panel.renameTemporalPartition(txt);
							SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
						}
					}
				}
			}

			break;
		case DELETE_TEMPORAL_PARTITION:
			logger.log(Level.INFO, "Delete Selected Temporal Partiton");	
			if (node == null) {
				confirmationDialog("Delete Selected Temporal Partiton",
						"Please select a Macro Unit and try again");
			} else {
				
				DefaultMutableTreeNode labelViewSelectedNode = panel.getDatabaseView().getLabelView().getSelectedNode();
				DefaultMutableTreeNode prevSiblingNode = null;
				if(labelViewSelectedNode.getLevel() == 1)
					prevSiblingNode = labelViewSelectedNode.getPreviousSibling();
				else{
					DefaultMutableTreeNode parent = (DefaultMutableTreeNode)labelViewSelectedNode.getParent();
					prevSiblingNode = parent.getPreviousSibling();
				}

			
				DefaultMutableTreeNode databaseViewRootNode = (DefaultMutableTreeNode)panel.getDatabaseView().getLabelView().getSelectedNode().getRoot();
				if(databaseViewRootNode.getChildCount() <= 2 || prevSiblingNode == null)
				{
					confirmationDialog("Delete Selected Temporal Partiton",
							"You cannot delete the last Temporatoral Partition or the Display Time Period");
				}
				else
				{	
					int level = panel.getDatabaseView().getLabelView().getSelectedNode().getLevel();
					if (level != 1 && level !=2) {
						SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_EDIT_NON_TEMPORAL_PARTITION);
					}
					else {
						LabelObject selectedLabelObject = null;
						String label = "";
						
						if(level == 2)
						{
							DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode) panel.getDatabaseView().getLabelView().getSelectedNode().getParent();
							selectedLabelObject = (LabelObject)parentNode.getUserObject();
							
							label = selectedLabelObject.getLabel();
						}
						else
						{
							selectedLabelObject = (LabelObject) panel.getDatabaseView().getLabelView().getSelectedNode().getUserObject();
							label = selectedLabelObject.getLabel();
						}
						
						String warningDialogText =
								"All annotated data for Temporal Partition  \""+ label + "\" will be DELETED.\n\n"
								+ "Are you absolutely sure you want to delete Temporal Partition \"" + label + "\"?";
						String warningTitleText = "WARNING";
						int optionType = JOptionPane.YES_NO_OPTION;
						int messageType = JOptionPane.ERROR_MESSAGE;
						String options[] = {"Proceed", "Cancel"};
						
						int result = -1;
						try {
							result = JOptionPane.showOptionDialog(this, 
									warningDialogText, 
									warningTitleText, 
									optionType, 
									messageType, 
									null, 
									options, 
									options[1]);
						} catch (HeadlessException he) {
							logger.log(Level.INFO, he.getMessage());
						}
						
						if (result == JOptionPane.OK_OPTION)
							panel.deleteTemporalPartition();
							SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
					}
				}
			}
			break;

		case RENUMBER_UTTERANCES:
			SignStreamSegmentPanel segmentPanel = getSignStreamSegmentPanel();
			FieldWrappersCollection collection = segmentPanel.getDatabaseView().getSegmentGraphicRepresentation()
					.getFieldWrappersCollection();

			DefaultMutableTreeNode selectedNode = segmentPanel.getDatabaseView().getLabelView().getSelectedNode();
			LabelObject partitionObj;
			if (selectedNode.getLevel() == 2) {
				partitionObj = (LabelObject) ((DefaultMutableTreeNode) selectedNode.getParent()).getUserObject();
			} else {
				partitionObj = (LabelObject) selectedNode.getUserObject();
			}
			SS3TemporalPartition partition = segmentPanel.getSS3Database().getSS3GlossWindow()
					.getTemporalPartitionByLabel(partitionObj.getLabel());
			if (partition == null) {
				break;
			}

			int result = JOptionPane.showOptionDialog(
					this,
					"You are about to renumber all utterances\nin the current temporal partition. This\n" +
							"action is irreversible. Are you sure you\nwant to continue?",
					"Renumber Utterances?", JOptionPane.OK_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE, null,
					new Object[] { "Cancel", "OK" }, "Cancel"
			);
			if (result == 0) {
				break;
			}

			ArrayList<SS3GlossWindowSegment> segmentTiers = partition.getTemporalPartitionSegments();

			for (FieldWrapper wrapper : collection.getWrappers()) {
				PresentationField field = (PresentationField) wrapper.getField();
				ArrayList<PresentationEventsEntity> entities = field.getEntities();

				if (field.getName().equals(partitionObj.getLabel()) ||
						segmentTiers.contains(field.getSS3GlossWindowSegment())) {
					for (int i = 0; i < entities.size(); i++) {
						PresentationEvent event = entities.get(i).getEvent();
						event.setText("U" + (i + 1));
					}
				}
			}
			segmentPanel.getDatabaseView().getSegmentGraphicRepresentation().repaint();
			break;
			
		case EXIT:
			exit();
			break;
			
		case HELP:
			confirmationDialog("Help Dialog",
					"<HTML>"  
					+ "     <center><h3>Summary of SignStream Terms </h3></center>    <br>"  
					+ "     <b>Temporal Partition: </b>  A tier for a macro unit containing non-overlapping utterances. <br>"  
					+ "     <b>Segment Tier: </b> Shown below a temporal partition, corresponding to a unique participant. <br>"  
					+ "     <b>Utterance: </b>  A time unit that is represented by the Utterance Bar in the temporal partition. <br>"  
					+ "     <b>Segment: </b>  A time unit under a segment tier with the same boundaries as its corresponding utterance. <br>"  
					+ "     <b>Entity (Chained Event): </b>  An Entity includes the main event (its value is represented by a label)<br> plus any associated elements that are tied to it. <br>"  
					+ "</HTML>");
			break;
			
		case UPDATE:
			try {
				String currentDir = System.getProperty("user.dir");
				
				String command1 = 
						"tell application \"Terminal\" to do script \"'" + BASH_UPDATE_SCRIPT + "' --patch --install latest; exit; \"";
				String command2 = "tell application \"Terminal\" to activate";	// bring the Terminal window to the front
				String[] commands = {"osascript",  "-e",  command1, "-e", command2};
				
				ProcessBuilder p = new ProcessBuilder(commands);
//				p.inheritIO().start();	// leave commented - for debugging
				p.start();
			} catch (IOException e) {
				System.out.println("'" + BASH_UPDATE_SCRIPT + " --patch --install latest' failed");
				logger.log(Level.INFO, "'" + BASH_UPDATE_SCRIPT + " --patch --install latest' failed");
				logger.log(Level.INFO, "Exception: " + e.getMessage());
			}
			break;
		case INSTALL_SIGN_BANK:
			try {
				String currentDir = System.getProperty("user.dir");

				String command1 = 
						"tell application \"Terminal\" to do script \"'" + BASH_SIGN_BANK_SCRIPT + "'; exit; \"";
				String command2 = "tell application \"Terminal\" to activate";	// bring the Terminal window to the front
				String[] commands = {"osascript",  "-e",  command1, "-e", command2};

				ProcessBuilder p = new ProcessBuilder(commands);
//				p.inheritIO().start();	// leave commented - for debugging	
				p.start();
			} catch (IOException e) {
				System.out.println("'" + BASH_SIGN_BANK_SCRIPT + "' failed");
				logger.log(Level.INFO, "'" + BASH_SIGN_BANK_SCRIPT + "' failed");
				logger.log(Level.INFO, "Exception: " + e.getMessage());
			}
			break;
		case UNINSTALL_SIGN_BANK:
			try {
				String currentDir = System.getProperty("user.dir");

				String command1 = 
						"tell application \"Terminal\" to do script \"'" + BASH_SIGN_BANK_SCRIPT + "' --uninstall; exit; \"";
				String command2 = "tell application \"Terminal\" to activate";	// bring the Terminal window to the front
				String[] commands = {"osascript",  "-e",  command1, "-e", command2};

				ProcessBuilder p = new ProcessBuilder(commands);
//				p.inheritIO().start();	// leave commented - for debugging	
				p.start();
			} catch (IOException e) {
				System.out.println("'" + BASH_SIGN_BANK_SCRIPT + " --uninstall' failed");
				logger.log(Level.INFO, "'" + BASH_SIGN_BANK_SCRIPT + " --uninstall' failed");
				logger.log(Level.INFO, "Exception: " + e.getMessage());
			}
			break;		
		case VIEW_ERRORS:

			File validationErrorsFile = new File(VALIDATION_ERRORS);
			if (validationErrorsFile.exists()) {
				try {
					if (validationErrorsDialog != null) {
						validationErrorsDialog.dispose();
						validationErrorsDialog = null;
					}
					
					validationErrorsDialog = new JDialog();
					validationErrorsDialog = new JDialog(this, "Collection Errors");
					
					ScrollablePanel panel = new ScrollablePanel();
					BoxLayout layout = new BoxLayout(panel, BoxLayout.Y_AXIS);
					panel.setLayout(layout);
					
					BufferedReader br = new BufferedReader(new FileReader(validationErrorsFile));
					String aLineFromFile = null;

					while ((aLineFromFile = br.readLine()) != null) {
						var tuple = aLineFromFile.split(Util.VALIDATOR_CODES_DELIMITER);
						
						if (tuple.length != 2) {
							continue;		// something weird happened and it's malformed
						}
						
						String errorCode = tuple[0].trim();
						String errorMsg = tuple[1].trim();
						
						JPanel jp = new JPanel();
						GridBagConstraints gbc = new GridBagConstraints();
						jp.setLayout(new GridBagLayout());
						gbc.anchor = GridBagConstraints.WEST;
						gbc.fill = GridBagConstraints.HORIZONTAL;
						gbc.weightx = 0.0;
						gbc.gridx = 0;
						
						JButton jb = new JButton("Help - Error Code " + errorCode);
						jp.add(jb, gbc);

						JTextArea jta = new JTextArea(errorMsg, 0, 50);
						jta.setLineWrap(true);
						jta.setWrapStyleWord(true);
						jta.setOpaque(false);
						jta.setEditable(false);
						gbc.weightx = 1.0;
						gbc.gridx = 1;
						jp.add(jta, gbc);
						
						jb.addActionListener(new ActionListener() {
							@Override
							public void actionPerformed(ActionEvent e) {
								try {
									String DAI_PROTOCOL = "https";
									String DAI_SITE = "signstream.cs.rutgers.edu";
									String DAI_PATH = "/docs/errors/" + errorCode;

									String manualURI = DAI_PROTOCOL + "://" + DAI_SITE + DAI_PATH;

									URI uri = new URI(manualURI);

									Desktop.getDesktop().browse(uri);
									
								} catch(MalformedURLException exception) {
									System.out.println("Invalid URL Format. " + exception);
								} catch (IOException exception) {
									System.out.println("Error opening URL using browser: " + exception);
								} catch (URISyntaxException exception) {
									System.out.println("Error converting URL to URI: " + exception);
								}
							}
						});
						
						panel.add(jp);
						
						panel.add(Box.createHorizontalStrut(5));
						panel.add(new JSeparator(SwingConstants.HORIZONTAL));
					}
					
					JScrollPane jsp = new JScrollPane(panel,
							JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);

					Dimension d = panel.getPreferredSize();
					Dimension nD = new Dimension(d.width + 20, d.height + 40);
					jsp.setPreferredSize(nD);

					Container contentPane = validationErrorsDialog.getContentPane();
//					contentPane.setLayout(new BorderLayout(40, 40));
					contentPane.setLayout(new BorderLayout());
					contentPane.add(jsp, BorderLayout.CENTER);

					validationErrorsDialog.setLocation(new Point(150, 150));
					validationErrorsDialog.pack();
					validationErrorsDialog.setVisible(true);

					br.close();
				} catch (Exception e) {
					System.out.println("Failed to open: " + VALIDATION_ERRORS);
					logger.log(Level.INFO, "\"Failed to open: \" + VALIDATION_ERRORS");
				}
			} else {
				JOptionPane.showMessageDialog(null, "No errors in this collection!");
			}
			
			break;		
		case VIEW_MODIFICATIONS:

			File validationModificationsFile = new File(VALIDATION_MODIFICATIONS);
			if (validationModificationsFile.exists()) {
				try {
					if (validationModificationsDialog != null) {
						validationModificationsDialog.dispose();
						validationModificationsDialog = null;
					}
					
					validationModificationsDialog = new JDialog();
					validationModificationsDialog = new JDialog(this, "Collection Modifications");
					
					ScrollablePanel panel = new ScrollablePanel();
					BoxLayout layout = new BoxLayout(panel, BoxLayout.Y_AXIS);
					panel.setLayout(layout);
					
					BufferedReader br = new BufferedReader(new FileReader(validationModificationsFile));
					String aLineFromFile = null;

					while ((aLineFromFile = br.readLine()) != null) {
						var tuple = aLineFromFile.split(Util.VALIDATOR_CODES_DELIMITER);
						
						if (tuple.length != 2) {
							continue;		// something weird happened and it's malformed
						}
						
						String modificationsCode = tuple[0].trim();
						String modificationMsg = tuple[1].trim();
						
						JPanel jp = new JPanel();
						GridBagConstraints gbc = new GridBagConstraints();
						jp.setLayout(new GridBagLayout());
						gbc.anchor = GridBagConstraints.WEST;
						gbc.fill = GridBagConstraints.HORIZONTAL;
						gbc.weightx = 0.0;
						gbc.gridx = 0;
						
						JButton jb = new JButton("Help - Modification Code " + modificationsCode);
						jp.add(jb, gbc);

						JTextArea jta = new JTextArea(modificationMsg, 0, 50);
						jta.setLineWrap(true);
						jta.setWrapStyleWord(true);
						jta.setOpaque(false);
						jta.setEditable(false);
						gbc.weightx = 1.0;
						gbc.gridx = 1;
						jp.add(jta, gbc);
						
						
						jb.addActionListener(new ActionListener() {
							@Override
							public void actionPerformed(ActionEvent e) {
								try {
									String DAI_PROTOCOL = "https";
									String DAI_SITE = "signstream.cs.rutgers.edu";
									String DAI_PATH = "/docs/modifications/" + modificationsCode;

									String manualURI = DAI_PROTOCOL + "://" + DAI_SITE + DAI_PATH;

									URI uri = new URI(manualURI);

									Desktop.getDesktop().browse(uri);
									
								} catch(MalformedURLException exception) {
									System.out.println("Invalid URL Format. " + exception);
								} catch (IOException exception) {
									System.out.println("Error opening URL using browser: " + exception);
								} catch (URISyntaxException exception) {
									System.out.println("Error converting URL to URI: " + exception);
								}
							}
						});
						
						panel.add(jp);
						
						panel.add(Box.createHorizontalStrut(5));
						panel.add(new JSeparator(SwingConstants.HORIZONTAL));
					}
					
					JScrollPane jsp = new JScrollPane(panel,
							JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);

					Dimension d = panel.getPreferredSize();
					Dimension nD = new Dimension(d.width + 20, d.height + 40);
					jsp.setPreferredSize(nD);

					Container contentPane = validationModificationsDialog.getContentPane();
					contentPane.setLayout(new BorderLayout());
					contentPane.add(jsp, BorderLayout.CENTER);

					validationModificationsDialog.setLocation(new Point(150, 150));
					validationModificationsDialog.pack();
					validationModificationsDialog.setVisible(true);

					br.close();
				} catch (Exception e) {
					System.out.println("Failed to open: " + VALIDATION_MODIFICATIONS);
					logger.log(Level.INFO, "\"Failed to open: \" + VALIDATION_MODIFICATIONS");
				}
			} else {
				JOptionPane.showMessageDialog(null, "No modifications for this collection!");
			}
			
			break;
		case NEW_MACRO_UNIT:
			SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.FUNCTIONALITY_NOT_YET_IMPLEMENTED);
			break;

		case RENAME_MACRO_UNIT:
			if (macroUnitDialog != null && macroUnitDialog.isVisible()) {
				break;
			}

			Object sObject1 = tree.getLastSelectedPathComponent();

			if (sObject1 == null) {
				confirmationDialog("Rename Macro Unit Warning",
						"Please select a Macro Unit to rename.");
				return;
			}

			DefaultMutableTreeNode node6 = (DefaultMutableTreeNode) sObject1;

			if (!node6.isLeaf()) {
				confirmationDialog("Rename Macro Unit Warning",
						"Please select a Macro Unit to rename.");
				return;
			}

			Object robj = node6.getUserObject();
			Excerpt excerpt1 = (Excerpt) robj;

			SS3DatabaseCollection dbCol123 = SS3Singleton
					.getSS3DatabaseCollection();
			SS3Database db = dbCol123.getSS3Database(excerpt1);
			db.setExcerpt(excerpt1.getExcerpt());

            var videoFiles = new ArrayList<LoadedMediaFile>();
            for (var v : db.getVideos()) {
                var media = new SS3MediaFile();

                media.setId(v.uri().toString());
                media.setFile(new File(v.uri()));
                media.setVideo(v);

                var file = new LoadedMediaFile();
                file.setMediaFile(media);
                file.setDisplay(v.displayed());
                file.setLoaded(true);
                file.setLocated(true);

                videoFiles.add(file);
            }

            var audioFiles = new ArrayList<LoadedMediaFile>();
            for (var a : db.getAudios()) {
                var media = new SS3MediaFile();

                media.setId(a.uri().toString());
                media.setFile(new File(a.uri()));
                media.setAudio(a);

                var file = new LoadedMediaFile();
                file.setMediaFile(media);
                file.setDisplay(a.displayed());
                file.setLoaded(true);
                file.setLocated(true);
                file.setAudio();

                audioFiles.add(file);
            }

			String globalParticipantsFileName1 = path
					+ "globalParticipants.xml";
			String localParticipantsFileName1 = path + "localParticipants.xml";
			String globalAnnotatorsFileName1 = path + "globalAnnotators.xml";
			String localAnnotatorsFileName1 = path + "localAnnotators.xml";
			String mediaLibraryFileName1 = path + "mediaLibrary.xml";
			
			ParticipantsLibrary participantsLibrary1 = parser
					.parseParticipantsLibrary(globalParticipantsFileName1,
							localParticipantsFileName1);
			ParticipantsLibrary annotatorsLibrary1 = parser
					.parseAnnotatorsLibrary(globalAnnotatorsFileName1,
							localAnnotatorsFileName1);
			SS3MediaLibrary mLibrary1 = parser
					.parseMediaLibrary(mediaLibraryFileName1);

			macroUnitDialog = new CreateUpdateMacroUnitDialog(
					CreateUpdateMacroUnit.FUNCTION_UPDATE, db, excerpt1,
					videoFiles, audioFiles, mLibrary1, participantsLibrary1,
					annotatorsLibrary1,
					signBankCollection.getSignBankResourceCollection(),
					panel, this);

			macroUnitDialog.setVisible(true);
			break;

		case DELETE_MACRO_UNIT:
			SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.FUNCTIONALITY_NOT_YET_IMPLEMENTED);
			break;
			
		case SHOW_IPA:
			SS3Singleton.getNonManuelFieldDialogs().getIPAKeyboard().open();
            break;

        case SET_VIEW_MODE:
            var schema = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();
            var viewMenu = getJMenuBar().getMenu(2);
            var item = (JCheckBoxMenuItem)ae.getSource();

            var mode = (ViewMode)item.getClientProperty("mode");
            var segment = (Segment)item.getClientProperty("segment");
            var index = (int)item.getClientProperty("index");

            // Disable all the other modes
            var numItems = schema.viewModes().size() + 3;
            for (int i = 0; i < numItems - 3; i++)
                viewMenu.getItem(index * numItems + i + 2).setSelected(false);
            // Enable this mode
            item.setSelected(true);

            // Enable the correct custom menu items
            var customMenu = (JMenu)viewMenu.getItem(index * numItems + (numItems - 1));
            for (int i = 0; i < customMenu.getItemCount(); i++) {
                var panelItem = (JCheckBoxMenuItem)customMenu.getItem(i);
                var panel = panelItem.getClientProperty("panel");
                panelItem.setSelected(mode.panels.contains(panel));
            }

            var template = panel.getUtteranceView().getSegmentTemplate(segment.getID());
            template.setViewMode(mode);
            break;

        case TOGGLE_PANEL:
            schema = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();
            viewMenu = getJMenuBar().getMenu(2);
            item = (JCheckBoxMenuItem)ae.getSource();

            segment = (Segment)item.getClientProperty("segment");
            template = panel.getUtteranceView().getSegmentTemplate(segment.getID());
            index = (int)item.getClientProperty("index");
            var panel = (Panel)item.getClientProperty("panel");

            var newPanels = new HashSet<>(template.viewMode().panels);
            if (item.isSelected())
                newPanels.add(panel);
            else
                newPanels.remove(panel);
            var newMode = new ViewMode(newPanels);

            if (!newMode.panels.isEmpty()) {
                // Enable any mode that now matches the selected custom items
                numItems = schema.viewModes().size() + 3;
                for (int i = 0; i < numItems - 3; i++) {
                    var modeItem = viewMenu.getItem(index * numItems + i + 2);
                    var matches = ((ViewMode)modeItem.getClientProperty("mode")).matches(newMode);
                    modeItem.setSelected(matches);
                }

                template.setViewMode(newMode);

            } else {
                item.setSelected(true);
            }
            break;
		}

		fileMenu.setEnabled(true);
		newItem.setEnabled(true);
		openAsItem.setEnabled(true);
		openLegacyItem.setEnabled(true);

		closeItem.setEnabled(true);
		saveItem.setEnabled(true);
		saveAsItem.setEnabled(true);

		repaint();

		fileMenu.setEnabled(true);
		newItem.setEnabled(true);
		openAsItem.setEnabled(true);
		openLegacyItem.setEnabled(true);

		closeItem.setEnabled(true);
		saveItem.setEnabled(true);
		saveAsItem.setEnabled(true);
	}
	
	public void recreateTogglingOptions(java.util.List<SegmentTemplate> templates) {
		JMenu viewMenu = this.getJMenuBar().getMenu(2);
		viewMenu.removeAll();
		for (int i = 0; i < templates.size(); i++) {
			addTogglingOptions(templates.get(i), i);
		}
	}
	
	public void toggleSearchOption(boolean enabled) {
		JMenu viewMneu = this.getJMenuBar().getMenu(0);
		newSearch.setEnabled(enabled);

	}

	public void toggleMacroUnitMenu(boolean enabled) {
        this.getJMenuBar().getMenu(1).setEnabled(enabled);
    }

    private JMenu tempView;
	public void toggleViewMenu(boolean enabled) {
	    if (!enabled) {
            tempView = getJMenuBar().getMenu(2);
            getJMenuBar().remove(2);
        } else {
            getJMenuBar().add(tempView, 2);
            tempView = null;
        }
    }

	private void addTogglingOptions(SegmentTemplate template, int index) {
		Segment segment = template.getSegment();
		addTogglingOptions(template, segment.getParentField().getTemporalPartition().getLabel(), segment.getParentField().getSS3GlossWindowSegment().getLabel(), index);
	}
	
	private void addTogglingOptions(SegmentTemplate template, String temporalPartitionName, String segmentTierName, int index) {
		Segment segment = template.getSegment();

        JMenu viewMenu = this.getJMenuBar().getMenu(2);
        viewMenu.addSeparator();
        JMenuItem label = new JMenuItem(" " + temporalPartitionName + " - " + segmentTierName);
        label.setEnabled(false);
        viewMenu.add(label);

		var schema = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();
		var segmentMode = template.viewMode();
		for (var mode : schema.viewModes()) {
		    var item = new JCheckBoxMenuItem(mode.label, mode.matches(segmentMode));
		    item.setActionCommand("" + SET_VIEW_MODE);
		    item.addActionListener(this);
		    item.putClientProperty("mode", mode);
            item.putClientProperty("segment", segment);
            item.putClientProperty("index", index);
            viewMenu.add(item);
        }

		JMenu customMenu = new JMenu("Custom");
		viewMenu.add(customMenu);

        for (var panel : schema.panels()) {
            var item = new JCheckBoxMenuItem(panel.label, segmentMode.panels.contains(panel));
            item.setActionCommand("" + TOGGLE_PANEL);
            item.addActionListener(this);
            item.putClientProperty("panel", panel);
            item.putClientProperty("segment", segment);
            item.putClientProperty("index", index);
            customMenu.add(item);
        }
	}
	
	public void removeTogglingOptions(Segment targetSegment) {
		JMenu viewMenu = this.getJMenuBar().getMenu(2);
		
		for (int i = 1; i < viewMenu.getItemCount(); i++) {
			//if the menu component is not an item
			if (viewMenu.getItem(i) == null) {
				continue;
			}
			Segment segment = (Segment)viewMenu.getItem(i).getClientProperty("segment");
			if (segment != null) {
				if (segment == targetSegment) {
					//remove the gesture option, manual option, custom option, nonmanual option, menu label and separator
					
//					// need to get rid of magic numbers here...
//					viewMenu.remove(i+3);
//					viewMenu.remove(i+2);
					
					viewMenu.remove(i+1);
					viewMenu.remove(i);
					viewMenu.remove(i-1);
					viewMenu.remove(i-2);
					return;
				}
			}
		}
	}

	/**
	 * ((This is a package level method for now, with the intention of only being used by SS3SignStreamApplication and contained classes.))
	 * This code was originally called in the OPEN_AS_SS3 block of the local actionPerformed().
	 * Similar code was found in the SAVE and DONT_SAVE blocks of the SaveAndOpenCollectionDialog listener,
	 * so this method is standardized between them all for easy readability. 
	 * @param tree JTree from the db navigational panel
	 * @param rootNode Root node from the db navigational panel
	 * @param node Sub node from the db navigational panel, under the root node
	 */
	boolean openDatabaseSS3(File file, JTree tree, DefaultMutableTreeNode rootNode, DefaultMutableTreeNode node) {
		if(file != null) {
            // When a new file is selected/opened, close any existing morph-phon dialog
			MorphologicalPhonologicalInfoDialog dialog;
			if ((dialog = SS3Singleton.getNonManuelFieldDialogs().getMorphologicalPhonologicalInfoDialog()) != null) {
				dialog.discardChanges(); // discard changes to the dialog
			}

			SS3Singleton.getMediaToolBar().reset();

			File unzippedFiles = unzipDirectory(file);
			if (unzippedFiles != null) {
				
				panel.closeAllLoadedVideoFiles();
				rootNode.removeAllChildren();
				
				((DefaultTreeModel) tree.getModel()).reload(rootNode);
				panel.loadDatabase(null);
				SS3Singleton.getSS3DatabaseCollection().removeAll();

				try {
					dbDelegate.openDb(unzippedFiles, file);

					var databases = SS3Singleton.getSS3DatabaseCollection().getDatabases().entrySet();
					if (databases.size() == 0)
						return false;
						
					var odb = databases.iterator().next().getValue();

					SS3Singleton.getFilesLocation()
							.setLastOpenedDatabaseLocation(file.getPath());
					panel.getDbNavigationPanel().refresh(
							dbDelegate.getMutableTreeNode());

					if (rootNode.getChildCount() > 0) {
						DefaultMutableTreeNode node1 = (DefaultMutableTreeNode) rootNode.getChildAt(0);
                        DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode) node1.getFirstChild();

                        panel.getDbNavigationPanel().getJTree().requestFocus();

                        panel.getDbNavigationPanel().getJTree().expandRow(0);
                        panel.getDbNavigationPanel().getJTree().expandRow(1);
                        panel.getDbNavigationPanel().getJTree().setSelectionRow(2);

                        SS3SignStreamApplication.logger.log(Level.INFO, "Node [" + node
                                + "] had been selected.");

                        SignStreamSegmentPanel segmentPanel = SS3Singleton
                                .getSignStreamApplication().getSignStreamSegmentPanel();

                        if (selectedNode != null) {
                            // JACK TODO: Why zero here??
//                            SegmentGraphReprUtil.utteranceMovieTimeOnset = 0;
//                            SegmentGraphReprUtil.utteranceMovieTimeOffset = -1;
                            SegmentGraphReprUtil.utteranceStart = TimeRange.ABSOLUTE.zero();
                            SegmentGraphReprUtil.utteranceEnd = null;

                            Object obj = selectedNode.getUserObject();

                            if (obj instanceof Excerpt) {
                                Excerpt excerpt = (Excerpt) obj;
                                SS3DatabaseCollection dbCol = SS3Singleton
                                        .getSS3DatabaseCollection();
                                SS3Database db = dbCol.getSS3Database(excerpt);

                                var player = SS3Singleton.getMediaPlayer();
                                player.close();

                                var missing = false;

                                for (var a : db.getAudios())
                                    if (!Paths.get(a.uri()).toFile().exists())
                                        missing = true;

                                for (var v : db.getVideos())
                                    if (!Paths.get(v.uri()).toFile().exists())
                                        missing = true;

                                panel.loadDatabase(db);

                                var video = odb.getTimeRange(TimeRange.VIDEO_ID);
                                if (video == null) {
                                    var template = db.getVideos().get(0).range();
                                    video = TimeRange.ABSOLUTE.subrange(TimeRange.VIDEO_ID, "Video",
                                        template.unit(), template.offset(), template.duration(), template.step());
                                }
                                SS3Singleton.setVideoTimeRange(video);

                                if (!db.getAudios().isEmpty()) {
                                    var audio = odb.getTimeRange(TimeRange.AUDIO_ID);
                                    if (audio == null) {
                                        var template = db.getAudios().get(0).range();
                                        audio = TimeRange.ABSOLUTE.subrange(TimeRange.AUDIO_ID, "Audio",
                                            template.unit(), template.offset(), template.duration(), template.step());
                                    }
                                    SS3Singleton.setAudioTimeRange(audio);
                                }

                                if (!missing) { // if all files are present, load them
                                	int order = 1;
                                    for (var v : db.getVideos()) {
                                        var vv = player.video(v.uri()).orElse(null);
                                        if (vv == null)
                                            vv = player.addVideo(v.uri(), order);
                                        order++;

                                        if (!vv.displayed())
                                            vv.display();
                                    }
                                    order = 1;
                                    for (var a : db.getAudios()) {
                                        var aa = player.audio(a.uri()).orElse(null);
                                        if (aa == null)
                                            aa = player.addAudio(a.uri(), order);
                                        order++;

                                        if (!aa.displayed())
                                            aa.display();
                                    }
                                }

                                int width = panel.getDatabaseView().getVisibleWidth2();
                                panel.getZoomer().getCoordinateTimeConvertor().resize(width);

                                if (missing) {
                                    var event = new ActionEvent(this, 0, "" + RENAME_MACRO_UNIT);
                                    actionPerformed(event);
                                } else {
                                    player.preroll();

                                    var v = player.videos().get(0);
                                    var timestep = v.timestep();
                                    var timebase = v.timebase();
                                    var framerate = v.framerate();
                                    var duration = player.duration().as(TimeUnit.SECOND) * framerate * timestep;
                                    SS3Singleton.getVideoControlManager()
                                        .getZoomer()
                                        .getCoordinateTimeConvertor()
                                        .setTimeScale(timebase, timestep, duration);

                                    segmentPanel.updateSize();
                                }

                                try {
                                    MediaToolBar mToolBar = SS3Singleton.getMediaToolBar(null);
                                    mToolBar.exitUtterance();
                                } catch(Exception ex) {
                                    ;
                                }

                                var schema = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();
                                for (var partition : db.getSS3GlossWindow().getTemporalPartitions()) {
                                    for (var segment : partition.getTemporalPartitionSegments()) {
                                        var panels = segment.getVisiblePanelIDs().stream()
                                            .map(id -> schema.panel(id))
                                            .collect(Collectors.toSet());
                                        var mode = new ViewMode(panels);

                                        segmentPanel.setSegmentViewMode(segment.getID(), mode);
                                    }
                                }
                            }

						Object uo = node1.getUserObject();
						if (uo instanceof String) {
							setTitle((String) uo);
						}
					}
                    }
					SS3Singleton.getApplicationStateController()
							.setLoadedCollectionUpdated(false);

                    panel.resetPainter();

                    logger.log(Level.INFO, "Database [" + file.getName()
							+ "] had been loaded.");
					
					System.out.println(unzippedFiles.getPath());
					
					Util.removeDirectory(unzippedFiles.getPath());	// remove the unzipped "tmp_<file>.ss3" directory

					return true;
					
				} catch (Exception ex) {
					logger.log(Level.SEVERE, "Could not open collection ["
							+ file.getName() + "].", ex);
                    JOptionPane.showMessageDialog(
                        this,
                        "Failed to open collection file!\n"
                            + "Apologies for the inconvenience. Please restart SignStream.",
                        "Error",
                        JOptionPane.ERROR_MESSAGE);

                    return false;
				}
			} else {
				logger.log(Level.INFO, "Wrong file format.");
				
				String txt = "<HTML>SignStream is unable to open this file.  Either it was not " + "<BR>" +
						"created by SignStream or it has been corrupted.</HTML>";
				
				confirmationDialog("File format", txt);
				return false;
			}
		} else {
		    return false;
        }
	}

	public SS3Database openDatabaseSS3() {

		File file = openZipDatabase();
		File unzippedFiles = unzipDirectory(file);

		if (unzippedFiles != null) {
			try {
				dbDelegate.openDb(unzippedFiles, file);

				// set correct path!!!
				SS3Singleton.getFilesLocation().setLastOpenedDatabaseLocation(
						file.getPath());

				logger.log(Level.INFO, "Database [" + file.getName()
						+ "] had been loaded.");
				
				// find db which was created in dbDelegate.openDB
				for (SS3Database db: SS3Singleton.getSS3DatabaseCollection().getDatabases().values()) {
					String str1 = db.getReadFromXMLFile().toString();
					int index = str1.indexOf("/");
					String str2 = str1.substring(0, index);
					
					if (unzippedFiles.getPath().equals(str2)) {
						return db;
					}
				}
				
			} catch (Exception ex) {
				logger.log(Level.SEVERE,
						"Could not open collection [" + file.getName() + "].",
						ex);
			}
		}
		
		return null;
	}
	/**
	 * Saves the collection to a ".ss3" file.
	 * Does this by saving the macro units to a temporary directory and then zipping up that
	 * directory into a ".ss3" archive.
	 * @param node - Node to represent the collection
	 * @param saveType - Used to distinguish between a regular "Save" and a "Save as"
	 */
	private File saveDatabaseToFile(DefaultMutableTreeNode node, int saveType, boolean makeBackup) {
		if(panel.isUtteranceLoaded()) {
			panel.updateLoadedDatabase();
		}

		String newArchivePath = null;
		String collectionName = (String) node.getUserObject();
		FilesLocation lastLocation = SS3Singleton.getFilesLocation();
		
		// Getting the database object
		Excerpt ex = (Excerpt) node.getFirstLeaf().getUserObject();
		SS3DatabaseCollection dbCol = SS3Singleton.getSS3DatabaseCollection();
		SS3Database db = dbCol.getSS3Database(ex);
		// Collection name does not have the ".ss3" file extension so you must add it here
		db.setNumber(ex.getID());
		db.setExcerpt(ex.getExcerpt());
		db.setFileName(ex.getExcerpt() + SS3Database.macroFileExtension);
		
		// Because of validation on opening collection, a "copy_" was made, and should be deleted now
		File dir = new File(db.getParentDirectoryPath());
		final String copyFileName = collectionName;
		File [] files = dir.listFiles(new FilenameFilter() {
			@Override
		    public boolean accept(File dir, String name) {
				return name.startsWith(SS3Database.copyFilePrefix)
						&& name.endsWith(SS3Database.dbFileExtension)
						&& name.contains(copyFileName);
			}
		});
		for (File file : files) {
			Util.removeDirectory(file.getAbsolutePath());
		}
		
		String dbOrigVersionStr = db.getVersion();
		String dbOrigCollectionName = collectionName; 
		
		if(saveType == SAVE_AS){
			String parentDirectoryName = null;
			if (!makeBackup) {
				FileDialog fd = new FileDialog(this, "Save Collection As...", FileDialog.SAVE);
				
				fd.setFilenameFilter(new ZipFormatFilenameFilter());
				
				// Sets the FileDialog to open the location of the last opened database
				File lastOpened = new File(lastLocation.getLastOpenedDatabaseLocation());
				fd.setDirectory(lastOpened.getParent());
				fd.setFile(Util.basename(lastOpened.getName()));
				
				fd.setVisible(true);
				collectionName = fd.getFile();
				parentDirectoryName = fd.getDirectory();
			} else {
//                var format = DateTimeFormatter.ofPattern("yyyyMMddHHmmss");
//                var backup = "bkp_" + LocalDateTime.now().format(format) + "_" + db.getDirectory();
//
//			    var src = Paths.get(db.getParentDirectoryPath(), db.getDirectory());
//			    var dst = Paths.get(db.getParentDirectoryPath(), backup);
//
//                try {
//                    Files.copy(src, dst, StandardCopyOption.REPLACE_EXISTING);
//                } catch (IOException e) {
//                    e.printStackTrace();
//                    JOptionPane.showMessageDialog(
//                        null,
//                        "Failed to create backup file!",
//                        "Error",
//                        JOptionPane.ERROR_MESSAGE);
//                }
//
//                return new File(dst.toString());
            }
			
			if (collectionName == null) {
				return null;
			}
			
			// Set the parent directory, the name of the temporary directory, and the name of
			// the path of the new archive file
			db.setParentDirectoryPath(parentDirectoryName);
			
			if (!collectionName.endsWith(SS3Database.COLLECTION_EXT))
				collectionName += SS3Database.COLLECTION_EXT;
			
			db.setDirectory(SS3Database.tempDirectoryPrefix + collectionName);
			newArchivePath = parentDirectoryName + File.separator + collectionName;
		} else {
			// Collection name does not have the ".ssc" file extension so you must add it here
			db.setDirectory(
					SS3Database.tempDirectoryPrefix
					+ collectionName
					+ SS3Database.COLLECTION_EXT);
		}
		
		// Below section is for after validation/conversion has occurred for a 3.0 file into 3.1
		double dbCurrentVersion = 3.1;
		if (SS3Singleton.isDbOriginallyOldScheme()) {
//		if (dbOrigVersionStr != null) {
//			dbCurrentVersion = Double.parseDouble(dbOrigVersionStr);
//			if (dbOrigVersionStr != null) {
				
				if (collectionName.endsWith(SS3Database.dbFileExtension))
					collectionName = collectionName.substring(0, collectionName.length() - SS3Database.dbFileExtension.length());
				
				if (!collectionName.endsWith(SS3Database.COLLECTION_EXT))
					collectionName += SS3Database.COLLECTION_EXT;
				
				if (!dbOrigCollectionName.endsWith(SS3Database.COLLECTION_EXT))
					dbOrigCollectionName += SS3Database.COLLECTION_EXT;
				
//				if (dbCurrentVersion < 3.1 && dbOrigCollectionName.equals(collectionName)) {
				if (dbOrigCollectionName.equals(collectionName)) {
					if (JOptionPane.showConfirmDialog(
							null,
							"You are about to save this SignStream® collection, which was previously saved with an older\n"
							+ "version of the application. It will be saved with a completely new file format and extension, (\".ssc\"), meaning\n"
							+ "it will not be possible to open it with versions of the SignStream® software prior to 3.4.0.\n\n"
							+ "Your original collection with its original file extension, (\".ss3\"), remains unaffected.",
							"Database Version Warning",
							JOptionPane.OK_CANCEL_OPTION
							) == JOptionPane.CANCEL_OPTION) {
						return null;
					}
				}
//			}	

				SS3Singleton.setDbOriginallyOldScheme(false);
		}

		// Saves the macro units to a temporary directory
		boolean saved = saveDatabase(db);
		if(!saved) {
			return null;
		}
		
		// Zips the temporary directory into a ".ss3" archive and then deletes it
		String tempDirectoryPath = db.getParentDirectoryPath() + File.separator + db.getDirectory();

		String cDirName = tempDirectoryPath;
		if (db.getDirectory().startsWith(SS3Database.tempDirectoryPrefix)) {
			var newDir = db.getDirectory().substring(4);
			cDirName = db.getParentDirectoryPath() + File.separator + newDir;
		}
		File dirFile = new File(cDirName);
		if ( (dbOrigVersionStr != null && dbCurrentVersion < 3.1) && dirFile.exists() && saveType == SAVE) {
			if (JOptionPane.showConfirmDialog(
					null,
					"You are about to overwrite an existing collection. Are you sure you wish to proceed?",
					"Overwrite Database Warning",
					JOptionPane.OK_CANCEL_OPTION
					) == JOptionPane.CANCEL_OPTION) {
				
				Util.removeDirectory(tempDirectoryPath);
				
				db.setVersion("" + dbCurrentVersion);
				
				return null;
			}
		} else if (dirFile.exists() && saveType == SAVE_AS) {
			if (JOptionPane.showConfirmDialog(
					null,
					"You are about to overwrite an existing collection. Are you sure you wish to proceed?",
					"Overwrite Database Warning",
					JOptionPane.OK_CANCEL_OPTION
					) == JOptionPane.CANCEL_OPTION) {
				
				Util.removeDirectory(tempDirectoryPath);
				
				return null;
			}
		}

		File collectionFile = zipDirectory(tempDirectoryPath);
		boolean zipped = collectionFile != null;
		// If zipping fails, delete the temporary directory
		if (!zipped) {
			Util.removeDirectory(tempDirectoryPath);
		}

		// If everything was successful, update the title of the application frame if necessary
		// and show a confirmation message.
		if (saved && zipped && !makeBackup) {
			if(saveType == SAVE_AS){
				String nodeName = collectionName
						.substring(0, collectionName.lastIndexOf(SS3Database.COLLECTION_EXT));
				node.setUserObject(nodeName);
				panel.getDbNavigationPanel().getJTree().repaint();
				setTitle(nodeName);
				lastLocation.setLastOpenedDatabaseLocation(newArchivePath);
			}
			confirmationDialog("Saved Resource", "Collection " + node.getUserObject()
					+ " has been saved.");
			SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(false);
		}
		
		return collectionFile;
	}

	public File unzipDirectory(File zFile) {

		try {
			String name = zFile.getName();

			if (name.endsWith(SS3Database.zipFileExtension)) {
				name = name.substring(0, name.length() - 4);
			}

			FileInputStream fin = new FileInputStream(zFile.getPath());
			// create temporary directory
			File tmpDir = new File(SS3Database.tempDirectoryPrefix + name);
			tmpDir.mkdir();

			// create ZipInputStream from FileInputStream object
			ZipInputStream zin = new ZipInputStream(fin);

			int i = 0;
			ZipEntry entry = null;
			boolean confFileDetected = false;
			while ((entry = zin.getNextEntry()) != null) {
				// create OutputStream to extract the entry from zip file
				String fileName = tmpDir.getName() + File.separator;

				String zipEntryName = entry.getName();
				if (zipEntryName != null && zipEntryName.equals(SS3Database.CONF_FILE)) {
					fileName += SS3Database.CONF_FILE;
					confFileDetected = true;
				} else {
					fileName += "file" + i + ".xml";
				}
				
				OutputStream os = new FileOutputStream(fileName);

				byte[] buffer = new byte[1024];
				int length;

				// read the entry from zip file and extract it to disk
				while ((length = zin.read(buffer)) > 0) {
					os.write(buffer, 0, length);
				}

				// close the streams
				os.close();
				i++;
			}
			// close the zip file
			zin.close();
			
			if (confFileDetected) {
				var confFile = Paths.get(tmpDir.getAbsolutePath(), SS3Database.CONF_FILE).toFile();
	            if (!confFile.exists())
	                throw new RuntimeException("Missing config!");
	            
	            var conf = readConf(confFile);
	            System.out.println("BEGIN CONFIG");
	            for (var e : conf.entrySet())
	                System.out.println("\"" + e.getKey() + "\": \"" + e.getValue() + "\"");
	            System.out.println("END CONFIG");

	            var version = conf.get(SS3Database.CONF_VERSION_KEY);
	            double currentVersion = Double.parseDouble(version);
	            double maxVersion = Double.parseDouble(SS3Database.COLLECTION_VERSION);
	            if (currentVersion > maxVersion) {
	            	confirmationDialog("Collection Error", "Collection cannot be parsed. A newer version of SignStream is required to open this collection.");
	            	return null;
	            }
			}

			return tmpDir;
		} catch (IOException e) {
			System.out.println("IOException :" + e);
			return null;
		}
	}

	/**
	 * Zips a directory into a ".ss3" archive file.
	 * Assumes the input has already had its ".ss3" extension added.
	 * @param dirName
	 * @return True if directory was zipped and deleted and false if an exception was thrown.
	 */
	public File zipDirectory(String dirName) {
		File collectionFile = null;
		
		try {
			File dirFile = new File(dirName);
			String cDirName = dirFile.getName();

			if (cDirName.startsWith(SS3Database.tempDirectoryPrefix)) {
				cDirName = cDirName.substring(4);
			}

			String zipFile = dirFile.getParentFile().getPath()
					+ File.separatorChar + cDirName;

			File xml = null;
			File[] childFiles = dirFile.listFiles();
			if (childFiles != null && childFiles.length == 1) {
				var check = childFiles[0];
				if (check.isDirectory()) {
					File[] secondLevelChildFiles = check.listFiles();
					if (secondLevelChildFiles != null && secondLevelChildFiles.length == 1) {
						xml = secondLevelChildFiles[0];
					}
				} else {
					xml = childFiles[0];
				}
			}
			
			var conf = new HashMap<String, String>();
			conf.put(SS3Database.CONF_VERSION_KEY, SS3Database.COLLECTION_VERSION);
			if (xml != null) conf.put(SS3Database.CONF_MACRO_UNIT_KEY, xml.getName());
			var confFile = Paths.get(dirFile.getAbsolutePath(), SS3Database.CONF_FILE).toFile();
			writeConf(conf, confFile);

			byte[] buffer = new byte[1024];

			try {
				FileOutputStream fout = new FileOutputStream(zipFile);
				ZipOutputStream zout = new ZipOutputStream(fout);
				
				File[] children = dirFile.listFiles();

				if (children != null) {
					for (int i = 0; i < children.length; i++) {
						File file = children[i];
						
						if (file.isDirectory()) {
							File[] secondLevelChildren = file.listFiles();
							for (int j = 0; j < secondLevelChildren.length; j++) {
								File secondLevelFile = secondLevelChildren[j];
								
								FileInputStream fin = new FileInputStream(secondLevelFile);
								zout.putNextEntry(new ZipEntry(file.getName() + File.separator + secondLevelFile.getName()));
								int length;

								while ((length = fin.read(buffer)) > 0) {
									zout.write(buffer, 0, length);
								}

								zout.closeEntry();
								fin.close();
							}
						} else {
							FileInputStream fin = new FileInputStream(file);
							zout.putNextEntry(new ZipEntry(file.getName()));
							int length;

							while ((length = fin.read(buffer)) > 0) {
								zout.write(buffer, 0, length);
							}

							zout.closeEntry();
							fin.close();
						}
					}
				}

				// close the ZipOutputStream
				zout.close();
				fout.close();

				// remove unzipped files
				Util.removeDirectory(dirFile.getAbsolutePath());

				if (zipFile != null) {
					String fileToDelete = zipFile + SS3Database.zipFileExtension;
					File file = new File(fileToDelete);
					if (file.exists()) {
						file.delete();
					}
				}
				
				collectionFile = new File(zipFile);

			} catch (FileNotFoundException ex) {
				ex.printStackTrace();
				SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);

				String[] str = new String[4];
				str[0] = "Collection could not be saved as .ss3 file";
				str[1] = "because access is denied.";
				str[2] = "Please close zip file and try again.";
				str[3] = "";

				SS3Singleton.getErrorMessages()
						.showSaveCollectionErrorMessage(str);
				return null;
			}
		} catch (IOException ioe) {
			ioe.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ioe);
			
			String[] str = new String[4];
			str[0] = "Collection could not be saved as .ss3 file";
			str[1] = ioe.getMessage();
			str[2] = "";
			str[3] = "";

			SS3Singleton.getErrorMessages()
					.showSaveCollectionErrorMessage(str);
			
			return null;
		} catch (NullPointerException e) {
			String[] str = new String[4];
			str[0] = "Collection could not be saved as it does not exist yet.";
			str[1] = "Please Confirm your settings in the Macro Unit Dialog";
			str[2] = "before saving.";
			str[3] = "";
			SS3Singleton.getErrorMessages()
				.showSaveCollectionErrorMessage(str);
			return null;
		}
		return collectionFile;
	}

	private Map<String, String> readConf(File file) throws IOException {
		var map = new HashMap<String, String>();
		Files.lines(Paths.get(file.getAbsolutePath()))
		.forEach(l -> {
			var kv = l.split(SS3Database.CONF_SEPARATOR, 2);
			if (kv.length == 2)
				map.put(kv[0].trim(), kv[1].trim());
		});
		return map;
	}

	private void writeConf(Map<String, String> entries, File file) throws IOException {
		Files.write(file.toPath(), (Iterable<String>) entries.entrySet().stream()
				.map(e -> e.getKey() + SS3Database.CONF_SEPARATOR + e.getValue())::iterator);
	}

	public void confirmationDialog(String title, String msg) {
		final JDialog collectionSavedDialog = new JDialog(this, title, true);
		JButton okBtn = new JButton("OK");

		ButtonPanel collectionSavedButtonPanel = new ButtonPanel();
		collectionSavedButtonPanel.add(okBtn);

		JPanel collectionSavedPanel = new JPanel();
		collectionSavedPanel.setLayout(new BorderLayout());
		JLabel collectionSavedLabel = new JLabel(msg);

		okBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				collectionSavedDialog.dispose();
			}
		});

		collectionSavedPanel.add(collectionSavedLabel, BorderLayout.CENTER);
		collectionSavedPanel
				.add(collectionSavedButtonPanel, BorderLayout.SOUTH);

		collectionSavedPanel.add(new JLabel("  "), BorderLayout.WEST);

		Dimension d = collectionSavedPanel.getPreferredSize();
		Dimension nD = new Dimension(d.width + 20, d.height + 40);
		collectionSavedPanel.setPreferredSize(nD);

		Container contentPane = collectionSavedDialog.getContentPane();
		contentPane.setLayout(new BorderLayout(40, 40));
		contentPane.add(collectionSavedPanel, BorderLayout.CENTER);

		collectionSavedDialog.setLocation(new Point(150, 150));
		collectionSavedDialog.pack();
		collectionSavedDialog.setVisible(true);

	}

	private boolean saveDatabase(SS3Database db) {
		try {
			return db.save();
		} catch (Exception e) {
			e.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", e);
			return false;
		}
	}

	public void exit() {
	    var option = JOptionPane.showConfirmDialog(this,
            "Quit SignStream?", "Confirm Quit",
            JOptionPane.OK_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE);

	    if (option == JOptionPane.CANCEL_OPTION)
	        return;

		if (SS3Singleton.getApplicationStateController()
				.isLoadedCollectionUpdated()) {
			saveChangesToTheCollection(false);
		} else {
			dispose();
			System.exit(0);
		}

		return;
	}

	private void deleteTempFiles() {
		try {
			File currentDir = new File(".");
			File[] tmpDirs = currentDir.listFiles(new FileFilter() {
				@Override
				public boolean accept(File pathname) {
					return pathname.isDirectory() &&
							pathname.getName().startsWith(SS3Database.tempDirectoryPrefix) &&
							(pathname.getName().endsWith(SS3Database.dbFileExtension) || pathname.getName().endsWith(SS3Database.COLLECTION_EXT));
				}
			});
			
			currentDir = new File(SSResources.collectionPath);
			File[] dbTmpDirs = currentDir.listFiles(new FileFilter() {
				@Override
				public boolean accept(File pathname) {
					return pathname.isDirectory() &&
							pathname.getName().startsWith(SS3Database.tempDirectoryPrefix) &&
							(pathname.getName().endsWith(SS3Database.dbFileExtension) || pathname.getName().endsWith(SS3Database.COLLECTION_EXT));
				}
			});
			
			ArrayList<File> allTmpDirs = new ArrayList<File>();
			Collections.addAll(allTmpDirs, tmpDirs);
			Collections.addAll(allTmpDirs, dbTmpDirs);
			
			if (allTmpDirs != null && allTmpDirs.size() > 0) {
				for (File tmpDir : allTmpDirs) {
					deleteDir(tmpDir);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private void deleteDir(File file) {
		File[] contents = file.listFiles();
		if (contents != null) {
			for (File f : contents) {
				deleteDir(f);
			}
		}
		file.delete();
	}

	private void addQuitHandler() {
		QuitHandler quitHandler = new QuitHandler() {
			
			@Override
			public void handleQuitRequestWith(QuitEvent event, final QuitResponse res) {
				deleteTempFiles();
				
				Runnable run = () -> {
					res.performQuit();
				};
				
				Runnable cancel = () -> {
					res.cancelQuit();
				};
				
				if (SS3Singleton.getApplicationStateController().isLoadedCollectionUpdated())
		            new SaveDialog(run, cancel).display();
		        else
		            run.run();
			}
		};
		Desktop.getDesktop().setQuitHandler(quitHandler);
	}
	
	public SignBankCollection getSignBankCollection() {
		return signBankCollection;
	}
	
	public SignBankCollection parseSignBank() {
		return parseSignBank(this);
	}
	
	public SignBankCollection parseSignBank(JFrame parent) {
		return parseSignBank(parent, false);
	}
	
	public SignBankCollection parseSignBank(JFrame parent, boolean showVersionError) {
		signBankCollection =  SS3Singleton.getSignBankParser().parseSignBank(parent, showVersionError);
		return signBankCollection;
	}
	
	public SignBankCollection parseBlankSignBank() {
		signBankCollection = new SignBankCollection();
		return signBankCollection;
	}
	
	public void writeSignBankCollection() {
		SS3Singleton.getSignBankWriter().writeCollectionToSignBank(signBankCollection);
	}
	
	public boolean isSearchDialogVisible() {
		return searchDialog != null && searchDialog.isVisible();
	}
	
	public static void deleteValidationFiles() {
	      File errorsFile = new File(SS3SignStreamApplication.VALIDATION_ERRORS);
	      errorsFile.delete();
	      
	      File modificationsFile = new File(SS3SignStreamApplication.VALIDATION_MODIFICATIONS);
	      modificationsFile.delete();
	}
}

class ImageFileView extends FileView {

	@Override
	public String getName(File f) {
		return null; 
	}

	@Override
	public String getDescription(File f) {
		return null; 
	}

	@Override
	public Boolean isTraversable(File f) {
		return null; 
	}

	@Override
	public String getTypeDescription(File f) {
		String type = null;

		if (f.isDirectory()) {
			FileFilter filter = new SignStreamFileFilter();
			File[] filteredFiles = f.listFiles(filter);
			if (filteredFiles != null && filteredFiles.length > 0) {
				type = "SignStream file";
			}
		}

		return type;
	}

	@Override
	public Icon getIcon(File f) {
		Icon icon = null;

		if (f.isDirectory()) {
			FileFilter filter = new SignStreamFileFilter();
			File[] filteredFiles = f.listFiles(filter);

			if (filteredFiles != null && filteredFiles.length > 0) {
				icon = new ImageIcon(ImageMaker.PALETTE_ICON, "SignStream");
			} 
		}

		return icon;
	}
}

class SignStreamFileFilter implements FileFilter {
	@Override
	public boolean accept(File file) {
		if (Util.isXML3File(file.getName())) {
			return true;
		}
		return false;
	}
}

class SignStream3FilesFilter extends javax.swing.filechooser.FileFilter {

	@Override
	public boolean accept(File f) {
		if (f.isDirectory()) {
			return true;
		} else {
			return f.getName().endsWith(".ss3.zip");
		}
	}

	@Override
	public String getDescription() {
		return "Accepts SignStream3 files only.";
	}
}

class ZipFormatFilenameFilter implements FilenameFilter {
	String ext1;
	String ext2;
	String ext3;

	public ZipFormatFilenameFilter() {
		this.ext1 = ".ss3.zip";
		this.ext2 = SS3Database.dbFileExtension;
		this.ext3 = SS3Database.COLLECTION_EXT;
	}

	@Override
	public boolean accept(File dir, String name) {
		if (name.endsWith(ext1) || name.endsWith(ext2) || name.endsWith(ext3))
			return true;
		else
			return false;
	}
}

class ScrollablePanel extends JPanel implements Scrollable{
    @Override
	public Dimension getPreferredScrollableViewportSize() {
        return super.getPreferredSize(); //tell the JScrollPane that we want to be our 'preferredSize' - but later, we'll say that vertically, it should scroll.
    }

    @Override
	public int getScrollableUnitIncrement(Rectangle visibleRect, int orientation, int direction) {
        return 10;
    }

    @Override
	public int getScrollableBlockIncrement(Rectangle visibleRect, int orientation, int direction) {
        return 10;
    }

    @Override
	public boolean getScrollableTracksViewportWidth() {
        return true;	//track the width, and re-size as needed.
    }

    @Override
	public boolean getScrollableTracksViewportHeight() {
        return false; //we don't want to track the height, because we want to scroll vertically.
    }
}
