package edu.bu.signstream.ui.panels;

import edu.bu.signstream.common.files.SSResources;
import edu.bu.signstream.common.graph.Graph;
import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Database;
import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.time.Duration;
import edu.bu.signstream.time.TimeRange;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.ui.panels.newDatabase.CreateUpdateMacroUnit;
import edu.bu.signstream.ui.panels.newDatabase.ParticipantView;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

public class GraphFileDialog extends JDialog implements ActionListener {

    private JLabel title = new JLabel("Change Graph File:");
    private JLabel filename = null;

    private JButton selectBtn = new JButton("Select");
    private static final String SELECT = "SELECT";

    private JButton clearBtn = new JButton("Clear");
    private static final String CLEAR = "CLEAR";

    private JButton cancelBtn = new JButton("Cancel");

    private ParticipantView participantView;

    private SS3Database db;
    private CreateUpdateMacroUnit cumuPanel;

    public GraphFileDialog(ParticipantView participantView) {
        super(SS3Singleton.getSignStreamApplication().getMacroUnitDialog(), true);
        this.participantView = participantView;

        db = SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().getSS3Database();
        cumuPanel = SS3Singleton.getSignStreamApplication().getMacroUnitDialog().getMacroUnitPanel();

        selectBtn.setActionCommand(SELECT);
        clearBtn.setActionCommand(CLEAR);

        selectBtn.addActionListener(this);
        clearBtn.addActionListener(this);
        cancelBtn.addActionListener(this);

        initUI();
    }

    private void initUI() {

        setMinimumSize(new Dimension(300, 150));

        String graphFileID = participantView.getParticipant().getGraphFileID();
        filename = new JLabel(graphFileID == null ? "None" : cumuPanel.getGraph(graphFileID).name());

        JPanel buttonPanel = new JPanel(new FlowLayout());
        buttonPanel.add(selectBtn);
        buttonPanel.add(clearBtn);
        buttonPanel.add(cancelBtn);

        setLayout(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        c.insets = new Insets(1, 1, 1, 1);
        c.gridx = 0;
        c.gridy = 0;
        add(title, c);

        c.gridy = 1;
        add(filename, c);

        c.gridy = 2;
        add(buttonPanel, c);

        pack();
    }

    @Override
    public void actionPerformed(ActionEvent event) {
        SS3Participant participant = participantView.getParticipant();

        String s = event.getActionCommand();
        if (SELECT.equals(s)) {
            File graphFile = UserInterfaceUtil.selectFile(
                    SS3Singleton.getSignStreamApplication(),
                    new File(SSResources.rootPath),
                    null,
                    UserInterfaceUtil.GRAPH_FILES_FILTER,
                    "Locate Graph File"
            );
            if (graphFile != null) {
                if (participant.getGraphFileID() != null) {
                    cumuPanel.removeGraph(participant.getGraphFileID());
//                    cumuPanel.removeGraphFile(participant.getGraphFileID());
                }

                var name = graphFile.getName();
                var uri = graphFile.toURI();
                var range = TimeRange.ABSOLUTE.subrange("Graph " + name, TimeUnit.NANOSECOND, Duration.ZERO, Duration.ZERO, 10_000_000);

                var graph = new Graph(name, uri, range);
                cumuPanel.addGraph(graph);

//                SS3MediaFile gFile = new SS3MediaFile();
//                gFile.setFile(graphFile);
//                gFile.setId(Util.getUniqueNumber());
//                gFile.setName(graphFile.getName());
//                cumuPanel.addGraphFile(gFile);

                participant.setGraphFileID(graph.id());
                participantView.setGraphFileLblText(name);

                filename.setText(graphFile.getName());
            }
        } else if (CLEAR.equals(s)) {
            if (participant.getGraphFileID() != null) {
                cumuPanel.removeGraph(participant.getGraphFileID());
//                cumuPanel.removeGraphFile(participant.getGraphFileID());
                participant.setGraphFileID(null);
                filename.setText("None");
                participantView.setGraphFileLblText("None");
            }
        }
        setVisible(false);
    }
}
