package edu.bu.signstream.ui.panels.codingScheme;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;

import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;

import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Track;
import edu.bu.signstream.time.TimeRange;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.Segment;
import edu.bu.signstream.grepresentation.fields.customField.CustomIntervalField;
import edu.bu.signstream.grepresentation.fields.customField.CustomIntervalFieldWrapper;
import edu.bu.signstream.grepresentation.fields.customField.CustomPointFieldWrapper;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.LabelObject;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.ui.ButtonPanel;
import edu.bu.signstream.ui.panels.DocumentModelListener;
import edu.bu.signstream.ui.panels.colorChooser.ColorChooser;

public class AddOrUpdateCustomField extends JPanel implements ActionListener, ItemListener {

	private static final long serialVersionUID = 1L;

	private JDialog parent = null;
	// private SS3CodingScheme scheme = null;
	private SignStreamSegmentPanel segmentPanel;
	private JPanel fieldPanel = null;
	private Segment segment = null;
	private PresentationField parentField = null;

	private JLabel fieldNameLbl = new JLabel("Field Name: ");
	private JLabel fieldLabelLbl = new JLabel("Field Label: ");
	private JLabel fieldPrefixLbl = new JLabel("Value Prefix: ");
	private JLabel fieldColorLbl = new JLabel("Field Color: ");
	private JLabel fieldGroupLbl = new JLabel("Field Group: ");
	private JLabel fieldRangeLbl = new JLabel("Field Auto Range: ");

	private JTextField fieldNameTxt = new JTextField(13);
	private JTextField fieldLabelTxt = new JTextField(7);
	private JTextField fieldPrefixTxt = new JTextField(7);

	private ColorChooser fieldColorChooser = new ColorChooser();
	private JButton fieldDefaultColorBtn = new JButton("Default");

	private String[] fieldGroups = { Constants.SPEECH_GROUP_NAME, Constants.GENERIC_GROUP_NAME };
	private JComboBox<String> fieldGroupCb = new JComboBox<>(fieldGroups);

	private JComboBox<TimeRange> fieldRangeCb;

	private JRadioButton inputText = new JRadioButton(" Text ");
	private JRadioButton inputTextPoint = new JRadioButton(" Point ");
	private JRadioButton inputTextInterval = new JRadioButton(" Interval ");

	private JRadioButton rightAlignment = new JRadioButton("");
	private JRadioButton leftAlignment = new JRadioButton("");
	private JRadioButton centerAlignment = new JRadioButton("");
	private JRadioButton rightNLeftAlignment = new JRadioButton("");

	private JButton doneBtn = new JButton("Done");
	private JButton cancelBtn = new JButton("Cancel");

	private boolean updateFieldInfo = true;

	private final int INPUT_TEXT = 0;
	private final int INPUT_TEXT_POINT = 1;
	private final int INPUT_TEXT_LINE = 2;
	private final int DONE = 4;
	private final int CANCEL = 5;

	private GridBagConstraints gbc = new GridBagConstraints();
	private JPanel fieldTypeInputPanel = null;
	private JPanel valueAlignmentPanel = null;

	private SS3Field field = null;
	private SS3Field defaultField = null;

	private DocumentModelListener dmListener = null;

	public AddOrUpdateCustomField(SignStreamSegmentPanel segmentPanel, Segment segment, int group, SS3Field field,
			JPanel fieldPanel, JDialog dialog, PresentationField parentField) {
		this.segmentPanel = segmentPanel;
		this.segment = segment;
		this.field = field;
		this.fieldPanel = fieldPanel;
		this.parent = dialog;
		this.parentField = parentField;

        var ranges = new ArrayList<TimeRange>();
        if (SS3Singleton.getRealVideoTimeRange() != null)
            ranges.add(SS3Singleton.getVideoTimeRange());
        if (SS3Singleton.getAudioTimeRange() != null)
            ranges.add(SS3Singleton.getAudioTimeRange());
        for (var g : segmentPanel.getSS3Database().getGraphs())
            ranges.add(g.range());
        ranges.add(TimeRange.ABSOLUTE);

        fieldRangeCb = new JComboBox<>(ranges.toArray(TimeRange[]::new));
        if (field != null) {
            var wrapper = segment.getRootLabelObject().getCustomLabelObject()
                .getChildWithFieldID(field.getId())
                .getWrapper();
            fieldRangeCb.setSelectedIndex(ranges.indexOf(wrapper.timeRange()));
        } else {
            if (SS3Singleton.getAudioTimeRange() != null)
                fieldRangeCb.setSelectedIndex(1);
            else
                fieldRangeCb.setSelectedIndex(0);
        }

		UserInterfaceUtil.setLabelLNF(fieldNameLbl);
		UserInterfaceUtil.setLabelLNF(fieldLabelLbl);
		UserInterfaceUtil.setLabelLNF(fieldPrefixLbl);
		UserInterfaceUtil.setLabelLNF(fieldColorLbl);
		UserInterfaceUtil.setLabelLNF(fieldGroupLbl);
        UserInterfaceUtil.setLabelLNF(fieldRangeLbl);

		UserInterfaceUtil.setTextFieldLNF(fieldNameTxt);
		UserInterfaceUtil.setTextFieldLNF(fieldLabelTxt);
		UserInterfaceUtil.setTextFieldLNF(fieldPrefixTxt);

		UserInterfaceUtil.setRadioBtnLNF(inputText);
		UserInterfaceUtil.setRadioBtnLNF(inputTextPoint);
		UserInterfaceUtil.setRadioBtnLNF(inputTextInterval);
		UserInterfaceUtil.setRadioBtnLNF(rightAlignment);
		UserInterfaceUtil.setRadioBtnLNF(leftAlignment);
		UserInterfaceUtil.setRadioBtnLNF(centerAlignment);
		UserInterfaceUtil.setRadioBtnLNF(rightNLeftAlignment);
		UserInterfaceUtil.setComboBoxLNF(fieldGroupCb);
        UserInterfaceUtil.setComboBoxLNF(fieldRangeCb);

		Dimension btnSize = doneBtn.getPreferredSize();
		doneBtn.setPreferredSize(btnSize);
		cancelBtn.setPreferredSize(btnSize);

		doneBtn.setActionCommand("" + DONE);
		doneBtn.addActionListener(this);

		cancelBtn.setActionCommand("" + CANCEL);
		cancelBtn.addActionListener(this);

		fieldTypeInputPanel = createFieldTypeInputPanel();
		valueAlignmentPanel = createValuesAlignmentPanel();

		doneBtn.setEnabled(false);
		dmListener = new DocumentModelListener(fieldNameTxt, doneBtn);
		fieldNameTxt.getDocument().addDocumentListener(dmListener);

        if (group != CSFieldPanel.BAD_GROUP_NUMBER)
            fieldGroupCb.setSelectedIndex(group);
        else
            fieldGroupCb.setSelectedIndex(0);

		SS3CodingScheme defaultCodingScheme = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();
		if (field != null) {
			defaultField = defaultCodingScheme.getField(field.getId());
			if (defaultField != null) {
				fieldGroupCb.setEnabled(false);
			}
		}

		setEnabled(updateFieldInfo);
		initRadioButtons();
		initUI();

		centerAlignment.setSelected(true);
		
		fieldDefaultColorBtn.addActionListener(e -> {
			fieldColorChooser.setColor(SS3Field.defaultColor);
		});

		if (field != null) {
			fieldNameTxt.setText(field.getName());
			fieldLabelTxt.setText(field.getLabel());
			fieldPrefixTxt.setText(field.getPrefix());
			fieldColorChooser.setColor(field.getColor());

			inputText.setSelected(true);
			
			SS3Field.Alignment alignment = field.getAlignment();
			if (alignment == SS3Field.Alignment.RIGHT) {
				rightAlignment.setSelected(true);
			} else if (alignment == SS3Field.Alignment.LEFT) {
				leftAlignment.setSelected(true);
			} else if (alignment == SS3Field.Alignment.CENTER) {
				centerAlignment.setSelected(true);
			} else {
				rightNLeftAlignment.setSelected(true);
			}

			ArrayList<SS3FieldValue> values = field.getValuesAsList();
			if (Constants.TYPE_CUSTOM_INTERVAL.equals(field.getType()) || values.size() > 0) {
				inputTextInterval.setSelected(true);
			} else if (Constants.TYPE_CUSTOM_POINT.equals(field.getType())) {
				inputTextPoint.setSelected(true);
			}
			inputTextInterval.setEnabled(false);
			inputTextPoint.setEnabled(false);
		}
	}

	private void initRadioButtons() {
		inputText.setSelected(true);
		// inputTextFree.setSelected(true);
		inputTextInterval.setSelected(true);

		if (!inputText.isSelected()) {
			inputTextPoint.setEnabled(false);
			inputTextInterval.setEnabled(false);
		}
		
		if (field != null) {
			inputText.setEnabled(false);
			inputTextPoint.setEnabled(false);
			inputTextInterval.setEnabled(false);
		}

		ButtonGroup grp1 = new ButtonGroup();
		ButtonGroup grp2 = new ButtonGroup();
		ButtonGroup grp3 = new ButtonGroup();

		grp1.add(inputText);

		grp2.add(inputTextPoint);
		grp2.add(inputTextInterval);

		grp3.add(rightAlignment);
		grp3.add(leftAlignment);
		grp3.add(centerAlignment);
		grp3.add(rightNLeftAlignment);

		inputText.addItemListener(this);
		inputTextPoint.addItemListener(this);
		inputTextInterval.addItemListener(this);

		inputText.setActionCommand("" + INPUT_TEXT);
		inputTextPoint.setActionCommand("" + INPUT_TEXT_POINT);
		inputTextInterval.setActionCommand("" + INPUT_TEXT_LINE);

//		inputText.setEnabled(false);
//		inputTextFree.setEnabled(false);
//		inputTextMenu.setEnabled(false);
//		inputGraphic.setEnabled(false);
//
//		rightAlignment.setEnabled(false);
//		leftAlignment.setEnabled(false);
//		centerAlignment.setEnabled(false);
//		rightNLeftAlignment.setEnabled(false);
		
		if (field != null) {
			SS3Field.Alignment alignment = field.getAlignment();
			if (alignment == SS3Field.Alignment.RIGHT) {
				rightAlignment.setSelected(true);
			} else if (alignment == SS3Field.Alignment.LEFT) {
				leftAlignment.setSelected(true);
			} else if (alignment == SS3Field.Alignment.CENTER) {
				centerAlignment.setSelected(true);
			} else {
				rightNLeftAlignment.setSelected(true);
			}
		}
	}

	private void initUI() {
		setLayout(new GridBagLayout());

		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.anchor = GridBagConstraints.WEST;

		gbc.gridy = 0;
		gbc.gridx = 0;
		gbc.gridwidth = 1;
		add(fieldGroupLbl, gbc);
		gbc.gridx = 1;
		gbc.gridwidth = 4;
		add(fieldGroupCb, gbc);
		gbc.gridwidth = 1;
		gbc.gridx = 5;
		add(doneBtn, gbc);

        gbc.gridy = 1;
        gbc.gridx = 0;
        add(fieldRangeLbl, gbc);
        gbc.gridx = 1;
        gbc.gridwidth = 4;
        add(fieldRangeCb, gbc);

		gbc.gridy = 2;
		gbc.gridx = 0;
		gbc.gridwidth = 1;
		add(fieldNameLbl, gbc);
		gbc.gridx = 1;
		gbc.gridwidth = 4;
		add(fieldNameTxt, gbc);
		gbc.gridwidth = 1;
		gbc.gridx = 5;
		add(cancelBtn, gbc);

		gbc.gridy = 3;
		gbc.gridx = 0;
		add(fieldLabelLbl, gbc);
		gbc.gridx = 1;
		gbc.gridwidth = 4;
		add(fieldLabelTxt, gbc);
		gbc.gridwidth = 1;

		gbc.gridy = 4;
		gbc.gridx = 0;
		add(fieldPrefixLbl, gbc);
		gbc.gridx = 1;
		gbc.gridwidth = 4;
		add(fieldPrefixTxt, gbc);
		gbc.gridwidth = 1;

		gbc.gridy = 5;
		gbc.gridx = 0;
		add(fieldColorLbl, gbc);
		gbc.gridx = 1;
		gbc.gridwidth = 3;
		add(fieldColorChooser, gbc);
		gbc.gridwidth = 1;
		gbc.gridx = 4;
		add(fieldDefaultColorBtn, gbc);

		gbc.gridy = 6;
		gbc.gridx = 0;
		gbc.gridwidth = 2;
		add(fieldTypeInputPanel, gbc);
		gbc.gridx = 2;
		gbc.gridwidth = 3;
		add(valueAlignmentPanel, gbc);

		if (inputTextInterval.isSelected()) {
			gbc.gridy = 10;
			gbc.gridx = 0;
			gbc.gridwidth = 5;
			gbc.gridheight = 15;
//			add(valuesTablePane, gbc);
			gbc.gridwidth = 1;
			gbc.gridheight = 1;

			gbc.gridy = 15;
			gbc.gridx = 5;
//			add(newValueBtn, gbc);
			gbc.gridy = 17;
//			add(deleteValueBtn, gbc);

		}
	}

	public String getNameTxt() {
		return fieldNameTxt.getText();
	}

	@Override
	public void itemStateChanged(ItemEvent event) {
		JRadioButton obj = (JRadioButton) event.getSource();
		String str = obj.getActionCommand();

		switch (Integer.parseInt(str)) {
		case INPUT_TEXT:
			if (inputText.isSelected()) {
				inputTextPoint.setEnabled(true);
				inputTextInterval.setEnabled(true);
			}
			break;

		case INPUT_TEXT_POINT:
			break;

		case INPUT_TEXT_LINE:
			break;
		}
		parent.pack();
	}

	private JPanel createFieldTypeInputPanel() {
		JPanel panel = new JPanel();
		panel.setLayout(new GridBagLayout());

		GridBagConstraints gbc = new GridBagConstraints();
		gbc.anchor = GridBagConstraints.LINE_START;
		gbc.gridwidth = GridBagConstraints.REMAINDER;

		gbc.gridwidth = 1;
		gbc.insets = new Insets(1, 1, 1, 1);
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		gbc.fill = GridBagConstraints.NONE;
		panel.add(inputText, gbc);

		int hStrut = 25;
		panel.add(Box.createHorizontalStrut(hStrut));
		panel.add(inputTextPoint, gbc);
		panel.add(Box.createHorizontalStrut(hStrut));
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		gbc.fill = GridBagConstraints.NONE;
		panel.add(inputTextInterval, gbc);

		gbc.gridwidth = 1;
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		gbc.fill = GridBagConstraints.NONE;
//		panel.add(inputGraphic, gbc);

		panel.setBorder(new TitledBorder("Field input type"));
		return panel;
	}

	private JPanel createValuesAlignmentPanel() {
		JPanel panel = new JPanel();
		Dimension d = new Dimension(120, 17);

		ValueAlignmentPanel v1 = new ValueAlignmentPanel(ValueAlignmentPanel.ALGMNT_RIGHT);
		ValueAlignmentPanel v2 = new ValueAlignmentPanel(ValueAlignmentPanel.ALGMNT_LEFT);
		ValueAlignmentPanel v3 = new ValueAlignmentPanel(ValueAlignmentPanel.ALGMNT_CENTER);
		ValueAlignmentPanel v4 = new ValueAlignmentPanel(ValueAlignmentPanel.ALGMNT_RIGHT_N_LEFT);

		v1.setPreferredSize(d);
		v2.setPreferredSize(d);
		v3.setPreferredSize(d);
		v4.setPreferredSize(d);

		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		panel.setLayout(gbl);

		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.gridwidth = GridBagConstraints.REMAINDER;

		gbc.gridwidth = 1;
		gbc.insets = new Insets(1, 1, 1, 1);
		panel.add(rightAlignment, gbc);
		panel.add(Box.createHorizontalStrut(10));
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		panel.add(v1, gbc);

		gbc.gridwidth = 1;
		panel.add(leftAlignment, gbc);
		panel.add(Box.createHorizontalStrut(10));
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		panel.add(v2, gbc);

		gbc.gridwidth = 1;
		panel.add(centerAlignment, gbc);
		panel.add(Box.createHorizontalStrut(10));
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		panel.add(v3, gbc);

		gbc.gridwidth = 1;
		panel.add(rightNLeftAlignment, gbc);
		panel.add(Box.createHorizontalStrut(10));
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		panel.add(v4, gbc);

		panel.setBorder(new TitledBorder("Alignment of values"));
		return panel;
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		String str = ae.getActionCommand();

		switch (Integer.parseInt(str)) {
		case DONE:
			if (fieldNameTxt.getText().length() == 0) {
				String label = "<HTML>The field's name and/or label has no text.<BR>Please provide some text for it or else<BR>use the Cancel button to exit this editor.</HTML>";
				SS3Singleton.getSignStreamApplication().confirmationDialog("Warning", label);

				return;

			} else if (fieldGroupCb.getSelectedIndex() == CSFieldPanel.BAD_GROUP_NUMBER) {
				String label = "<HTML>The field must belong to a specific group of existing fields.<BR>Please select a group from the dropdown.</HTML>";
				SS3Singleton.getSignStreamApplication().confirmationDialog("Warning", label);

				return;

			} else {
				int groupID = fieldGroupCb.getSelectedIndex();
				String fieldName = fieldNameTxt.getText();
				String fieldLabel = fieldLabelTxt.getText();

				CSFieldPanel csFieldPanel = null;

				if (fieldPanel instanceof CSFieldPanel) {
					csFieldPanel = (CSFieldPanel) fieldPanel;
				}

				if (csFieldPanel != null) {
					// not unique
					if (field == null && !csFieldPanel.isCustomFieldNameAndLabelUnique(fieldName, fieldLabel, groupID)) {
						StringBuffer nameNotUniqueBfr = new StringBuffer();
						nameNotUniqueBfr.append("<HTML>The field name and/or label are already in use by<BR>");
						nameNotUniqueBfr.append("another field. Please change it or use the Cancel <BR>");
						nameNotUniqueBfr.append("button to exit this editor.</HTML>");

						SS3Singleton.getSignStreamApplication().confirmationDialog("Warning",
								nameNotUniqueBfr.toString());

						fieldNameTxt.requestFocus();
						return;

					} else {

						if (field == null) {
							csFieldPanel.addNewCustomField(getCreatedOrUpdatedSS3Field(field));
							parent.setVisible(false);
						} else {
							SS3Field updatedField = getCreatedOrUpdatedSS3Field(field);
							csFieldPanel.updateCustomField(updatedField);
							parent.setVisible(false); // .dispose();
						}
					}
				}
			}
			discard();
			break;
		case CANCEL:
			StringBuffer cancelBfr = new StringBuffer();
			cancelBfr.append("<HTML>If proceeded, all entered data will be lost. <BR><BR>");
			cancelBfr.append("Are you sure you want to continue?</HTML>");

			final JDialog dialog = new JDialog(parent, "Save Resource", true);
			final JButton cancelBtn = new JButton("Cancel");
			final JButton continueBtn = new JButton("Continue");

			continueBtn.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent arg0) {
					ActionListener[] al = cancelBtn.getActionListeners();
					for (int i = 0; i < al.length; i++) {
						cancelBtn.removeActionListener(al[i]);
					}

					ActionListener[] cl = continueBtn.getActionListeners();
					for (int j = 0; j < cl.length; j++) {
						continueBtn.removeActionListener(cl[j]);
					}

					dialog.dispose();
					parent.setVisible(false);
					parent.dispose();
					discard();
				}

			});

			cancelBtn.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent arg0) {
					ActionListener[] al = cancelBtn.getActionListeners();
					for (int i = 0; i < al.length; i++) {
						cancelBtn.removeActionListener(al[i]);
					}

					ActionListener[] cl = continueBtn.getActionListeners();
					for (int j = 0; j < cl.length; j++) {
						continueBtn.removeActionListener(cl[j]);
					}

					dialog.dispose();
				}
			});

			ButtonPanel buttonPanel = new ButtonPanel();
			buttonPanel.add(cancelBtn);
			buttonPanel.add(continueBtn);

			JPanel panel = new JPanel();
			panel.setLayout(new BorderLayout());
			JLabel l1 = new JLabel(cancelBfr.toString());
			panel.add(l1, BorderLayout.CENTER);
			panel.add(buttonPanel, BorderLayout.SOUTH);

			panel.add(new JLabel("  "), BorderLayout.WEST);

			Dimension d = panel.getPreferredSize();
			Dimension nD = new Dimension(d.width + 10, d.height + 40);
			panel.setPreferredSize(nD);

			Container contentPane = dialog.getContentPane();
			contentPane.setLayout(new BorderLayout(40, 40));
			contentPane.add(panel, BorderLayout.CENTER);

			dialog.setLocation(new Point(150, 150));
			dialog.pack();
			dialog.setVisible(true);

			break;
		}
	}

	private SS3Field getCreatedOrUpdatedSS3Field(SS3Field csField) {
		boolean newField = false;

		if (csField == null) {
			newField = true;
			csField = new SS3Field();
			csField.setId(Util.getUniqueNumber());
		}

		boolean textPointSelected = inputTextPoint.isSelected();
		boolean textIntervalSelected = inputTextInterval.isSelected();

		boolean rAlgnmnt = rightAlignment.isSelected();
		boolean lAlgnmnt = leftAlignment.isSelected();
		boolean cAlgnmnt = centerAlignment.isSelected();
		boolean rlAlgnmnt = rightNLeftAlignment.isSelected();
		
		int origFieldGroupID = csField.getGroupID();

		csField.setGroupID(fieldGroupCb.getSelectedIndex());
		csField.setName(fieldNameTxt.getText());
		csField.setLabel(fieldLabelTxt.getText());
		csField.setPrefix(fieldPrefixTxt.getText());
		csField.setColor(fieldColorChooser.getColor());

		int groupIndex = fieldGroupCb.getSelectedIndex();

		switch (groupIndex) {
		case Constants.SPEECH_GROUP_ID:
			csField.setGroup(Constants.SPEECH_GROUP_NAME_CAPS);
			break;
        case Constants.GENERIC_GROUP_ID:
            csField.setGroup(Constants.GENERIC_GROUP_NAME_CAPS);
		}

		if (textPointSelected)
			csField.setType(Constants.TYPE_CUSTOM_POINT);

		if (textIntervalSelected) {
			csField.setType(Constants.TYPE_CUSTOM_INTERVAL);
//			Vector data = valuesTableModel.getDataVector();
//
//			csField.removeAllValues(); // need to remove all existing values in case something was deleted by user
//			// that user-deletion is so far only in the valuesTableModel object (and now the
//			// data vector) and any deletions should
//			// be remembered so that the csField does not keep old (deleted) values
//
//			for (int i = 0; i < data.size(); i++) {
//				Vector obj = (Vector) data.get(i);
//				SS3FieldValue fValue = new SS3FieldValue();
//				fValue.setId((String) obj.get(2));
//				fValue.setName((String) obj.get(0));
//				fValue.setLabel((String) obj.get(1));
//				csField.addValue(fValue);
//			}
		}

		if (rAlgnmnt)
			csField.setAlignment(SS3Field.Alignment.RIGHT);

		if (lAlgnmnt)
			csField.setAlignment(SS3Field.Alignment.LEFT);

		if (cAlgnmnt)
			csField.setAlignment(SS3Field.Alignment.CENTER);

		if (rlAlgnmnt)
			csField.setAlignment(SS3Field.Alignment.RIGHT_LEFT);

        var timeRange = (TimeRange)fieldRangeCb.getSelectedItem();

		if (newField) {
			SS3CodingScheme dbScheme = segmentPanel.getSS3Database().getSS3CodingScheme();
			dbScheme.addFieldAndOrder(csField);

			SS3Track track = new SS3Track();
			track.setId(csField.getId());
			
			// need to add new field to all segment tiers simultaneously
			for (Segment segment: segmentPanel.getSegments()) {
				CustomIntervalField customField = new CustomIntervalField(track, segmentPanel, parentField);
				customField.setName(csField.getLabelOrName());
				customField.setColor(csField.getColor());
				
				FieldWrapper customWrapper = null;
				if (csField.getType().equals(Constants.TYPE_CUSTOM_INTERVAL)) {
					customWrapper = new CustomIntervalFieldWrapper(customField, segmentPanel, timeRange);
				} else if (csField.getType().equals(Constants.TYPE_CUSTOM_POINT)) {
					customWrapper = new CustomPointFieldWrapper(customField, segmentPanel, timeRange);
				}
				
				var parent = segment.getRootLabelObject().getCustomLabelObject();
				LabelObject lObj = new LabelObject(parent, csField.getId(), csField.getLabelOrName());
				lObj.setWrapper(customWrapper);
				lObj.setPermanentlyVisibleAcrossSegmentTier(true);

				parent.addChild(lObj);

				// only display immediately in the segment which was clicked to open this dialog
				if ( !segment.equals(this.segment)) {
					lObj.setVisible(false);
					lObj.getWrapper().setVisible(false);
				}
			}
		} else {
			// if field already exists, then edit the name of the label, name of the
			// wrapper, and color of the field
			// for all segment tiers
			for (Segment segment: segmentPanel.getSegments()) {
				ArrayList<LabelObject> labelObjs = segment.getRootLabelObject().getCustomLabelObject()
						.getChildrenAsList();
				for (LabelObject obj : labelObjs) {
					if (obj.getFieldID().equals(csField.getId())) {
						obj.setLabel(csField.getLabelOrName());
						obj.getWrapper().getField().setName(csField.getLabelOrName());
						obj.getWrapper().getField().setColor(csField.getColor());
						obj.getWrapper().setTimeRange(timeRange);
					}
				}
			}

			SS3CodingScheme dbScheme = segmentPanel.getSS3Database().getSS3CodingScheme();
			if (origFieldGroupID != csField.getGroupID()) {
				dbScheme.removeField(csField);
				dbScheme.addFieldAndOrder(csField);
			} else {
				dbScheme.addField(csField);
			}
		}

		// SS3GlossWindowItem item = segment.getSS3GlossWindowItem();
		// SS3StatementField statementField =
		// item.getSS3StatementField(csField.getId());
		// if (statementField != null) {
		// statementField.setColor(csField.getColor());
		// }

		return csField;
	}

	public void discard() {
		doneBtn.removeActionListener(this);
		cancelBtn.removeActionListener(this);

		fieldNameTxt.getDocument().removeDocumentListener(dmListener);

		inputText.removeItemListener(this);
		inputTextPoint.removeItemListener(this);
		inputTextInterval.removeItemListener(this);

		removeAll();
	}
}
