package edu.bu.signstream.ui.panels.codingScheme;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.*;
import java.util.List;
import java.util.function.BiConsumer;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.swing.*;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.table.*;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.FieldsGroup;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowItem;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowSegment;
import edu.bu.signstream.common.util.vo.ss3.db.SS3StatementField;
import edu.bu.signstream.common.util.vo.ss3.db.SS3TemporalPartition;
import edu.bu.signstream.grepresentation.view.*;
import edu.bu.signstream.time.Bound;
import edu.bu.signstream.ui.textgrid.OpenTextGridDialog;

import org.jdesktop.swingx.JXTreeTable;
import org.jdesktop.swingx.treetable.AbstractMutableTreeTableNode;
import org.jdesktop.swingx.treetable.DefaultTreeTableModel;
import org.jdesktop.swingx.treetable.MutableTreeTableNode;
import org.jdesktop.swingx.treetable.TreeTableNode;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.Segment;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.ui.panels.codingScheme.CSFieldPanel.CodingSchemaFieldNode;

// TODO: Refactor this. The actual logic here should take 10x less code.
public class CSFieldPanel extends JPanel implements ActionListener {
	
	private static final long serialVersionUID = 1L;
	private final CSFieldDialog dialog;

    protected final int ARRAY_INDEX_FIELD_NAME = 0;
    protected final int ARRAY_INDEX_FIELD_LABEL = 1;
    protected final int ARRAY_INDEX_DISPLAY_LABEL = 2;
    protected final int ARRAY_INDEX_UTTERANCE_VISIBLE = 3;
    protected final int ARRAY_INDEX_SEGMENT_VISIBLE = 4;
    protected final int ARRAY_INDEX_GLOBAL_VISIBLE = 5;
    protected final int ARRAY_INDEX_ANNOTATED = 6;
    protected final int ARRAY_INDEX_COLOR = 7;
    protected final int ARRAY_INDEX_PREVIEW = 8;
    protected final int ARRAY_INDEX_FIELD_ID = 9;

    protected final int ARRAY_SIZE = ARRAY_INDEX_FIELD_ID + 1;

    private SignStreamSegmentPanel segmentPanel;
	private Segment segment;
	private PresentationField parentField;
	private NonmanualFieldsView nonManualView;
	private GlossNRelatedView glossNRelatedView;
	private CustomFieldsView customView;
	private GestureNRelatedView gestureView;
	private SegmentGraphicRepresentation rep;
	private CodingSchemaFieldNode nonManualFieldsRootNode = new CodingSchemaFieldNode(new Object[ARRAY_SIZE]);
	private CodingSchemaFieldNode glossFieldsRootNode = new CodingSchemaFieldNode(new Object[ARRAY_SIZE], true, false, false);
	private CodingSchemaFieldNode customFieldsRootNode = new CodingSchemaFieldNode(new Object[ARRAY_SIZE], false, true, false);
	private CodingSchemaFieldNode gestureFieldsRootNode = new CodingSchemaFieldNode(new Object[ARRAY_SIZE], false, false, true);
	private JXTreeTable nonManualFieldsTable;
	private JXTreeTable glossFieldsTable;
	private JXTreeTable customFieldsTable;
	private JXTreeTable gestureFieldsTable;
	private JScrollPane scrollPane;

	private JDialog editFieldDialog = null;
	private AddOrUpdateField editFieldPanel = null;
	private AddOrUpdateCustomField editCustomFieldPanel = null;
	private GraphFieldDialog graphFieldDialog = null;

	private SelectionListener selectionListener = null;
	private manualSelectionListener manualSelectionListener = null;
	private CustomSelectionListener customSelectionListener = null;
	private GestureSelectionListener gestureSelectionListener = null;
	private MutableTreeTableNode selectedNode = null;
	private String selectedNodeID = null;

    private static final String PARTITION_LABEL_PREFIX = "Temporal Partition: ";
    private final JLabel partitionLabel = new JLabel();

	private static final String TIER_LABEL_PREFIX = "Tier: ";
	private final JLabel tierLabel = new JLabel();

    private static final String UTTERANCE_LABEL_PREFIX = "Utterance: ";
    private final JLabel utteranceLabel = new JLabel();

	private final JButton newFieldBtn = new JButton("New Nonmanual Field");
	private final JButton newCustomFieldBtn = new JButton("New Default Field");
	private JButton importTextGridBtn = new JButton("Import TextGrid...");

	private JButton deleteBtn = new JButton("Delete");
	private JButton editBtn = new JButton("Edit Field");
	private JButton cancelBtn = new JButton("Cancel");
	private JButton doneBtn = new JButton("Done");
	private JButton indexBtn = new JButton("Set Field to Utterance Index");
	private JButton resetIndexBtn = new JButton("Reset Utterance Index");
	private JButton graphBtn = new JButton("Graph Fields");
	
	public final static int BAD_GROUP_NUMBER = 99;
	
	private HashMap<Time, Time> timeTuples = new HashMap<Time, Time>();

	private final int NEW_FIELD = 8;
	private final int NEW_CUSTOM_FIELD = 88;
	private final int DELETE = 9;
	private final int EDIT = 10;
	private final int CANCEL = 11;
	private final int DONE = 12;
	private final int INDEX = 13;
	private final int RESET_INDEX = 14;
	private final int GRAPH = 15;

	private GridBagConstraints gbc = new GridBagConstraints();

	// fields recently created and added to the coding schema
	private ArrayList<SS3Field> recentlyAddedFields = new ArrayList<SS3Field>();

	private JDialog newFieldDialog = null;
	private AddOrUpdateField newFieldPanel = null;
	private AddOrUpdateCustomField newCustomFieldPanel = null;

    private SS3TemporalPartition partition;
	private SS3GlossWindowSegment tier;
	private SS3GlossWindowItem utterance;

	public CSFieldPanel(SignStreamSegmentPanel segmentPanel, Segment segment, NonmanualFieldsView nonManualView,
			CSFieldDialog dialog, GlossNRelatedView glossNRelatedView, CustomFieldsView customView,
			GestureNRelatedView gestureView, PresentationField parentField) {
		this.segmentPanel = segmentPanel;
		this.nonManualView = nonManualView;
		this.glossNRelatedView = glossNRelatedView;
		this.customView = customView;
		this.gestureView = gestureView;
		this.segment = segment;
		this.partition = segment.getParentField().getTemporalPartition();
		this.tier = segment.getParentField().getSS3GlossWindowSegment();
		updateCurrentUtterance();
		this.dialog = dialog;
		this.parentField = parentField;
		
		this.graphFieldDialog = new GraphFieldDialog(this.segmentPanel, this.segment, nonManualView, this.parentField);

		initUI();
	}

	public CSFieldDialog dialog() { return dialog; }

	public GraphFieldDialog getGraphDialog() {
		return graphFieldDialog;
	}

	private void initUI() {
		deleteBtn.setEnabled(false);
		editBtn.setEnabled(false);

		newFieldBtn.setActionCommand("" + NEW_FIELD);
		for (var l : newFieldBtn.getActionListeners())
            newFieldBtn.removeActionListener(l);
		newFieldBtn.addActionListener(this);

		newCustomFieldBtn.setActionCommand("" + NEW_CUSTOM_FIELD);
        for (var l : newCustomFieldBtn.getActionListeners())
            newCustomFieldBtn.removeActionListener(l);
		newCustomFieldBtn.addActionListener(this);

        for (var l : importTextGridBtn.getActionListeners())
            importTextGridBtn.removeActionListener(l);
		importTextGridBtn.addActionListener(a -> importTextGrid());

		deleteBtn.setActionCommand("" + DELETE);
        for (var l : deleteBtn.getActionListeners())
            deleteBtn.removeActionListener(l);
		deleteBtn.addActionListener(this);

		editBtn.setActionCommand("" + EDIT);
        for (var l : editBtn.getActionListeners())
            editBtn.removeActionListener(l);
		editBtn.addActionListener(this);

		cancelBtn.setActionCommand("" + CANCEL);
        for (var l : cancelBtn.getActionListeners())
            cancelBtn.removeActionListener(l);
		cancelBtn.addActionListener(this);

		doneBtn.setActionCommand("" + DONE);
        for (var l : doneBtn.getActionListeners())
            doneBtn.removeActionListener(l);
		doneBtn.addActionListener(this);

		indexBtn.setActionCommand("" + INDEX);
        for (var l : indexBtn.getActionListeners())
            indexBtn.removeActionListener(l);
		indexBtn.addActionListener(this);
		
		resetIndexBtn.setActionCommand("" + RESET_INDEX);
		for (var l: resetIndexBtn.getActionListeners())
			resetIndexBtn.removeActionListener(l);
		resetIndexBtn.addActionListener(this);
		
		graphBtn.setActionCommand("" + GRAPH);
		for (var l : graphBtn.getActionListeners())
			graphBtn.removeActionListener(l);
		graphBtn.addActionListener(this);
		

		SS3CodingScheme dbScheme = segmentPanel.getSS3Database().getSS3CodingScheme();
		selectionListener = new SelectionListener(this, dbScheme, segmentPanel);
		manualSelectionListener = new manualSelectionListener(this, dbScheme, segmentPanel);
		customSelectionListener = new CustomSelectionListener(this, dbScheme, segmentPanel);
		gestureSelectionListener = new GestureSelectionListener(this, dbScheme, segmentPanel);

		scrollPane = getTreeTableScrollPanel();
		Dimension d = new Dimension(scrollPane.getPreferredSize().width + 260, 250);

		scrollPane.setPreferredSize(d);
		scrollPane.setMinimumSize(d);
		scrollPane.setMaximumSize(d);

		setLayout(new GridBagLayout());

		gbc.fill = GridBagConstraints.BOTH;
		gbc.weightx = 1;
		gbc.weighty = 1;
		gbc.gridheight = 4;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.insets = new Insets(1, 1, 1, 1);

		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.gridwidth = 9;
		add(scrollPane, gbc);

		var labelsPanel = new JPanel();
		labelsPanel.setLayout(new BoxLayout(labelsPanel, BoxLayout.Y_AXIS));
        partitionLabel.setText(PARTITION_LABEL_PREFIX + partition.getLabel());
        tierLabel.setText(TIER_LABEL_PREFIX + tier.getLabel());
        labelsPanel.add(partitionLabel);
		labelsPanel.add(tierLabel);
		labelsPanel.add(utteranceLabel);
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.weightx = 0;
        gbc.weighty = 1;
        gbc.gridwidth = 1;
        gbc.gridx = 10;
        gbc.gridy = 0;
        gbc.gridheight = 1;
        add(labelsPanel, gbc);
        
        gbc.gridy = 1;
//        add(indexBtn, gbc);
        
//        gbc.gridy = 2;
//        add(resetIndexBtn, gbc);

        var newFieldPanel = new JPanel();
        newFieldPanel.setLayout(new BoxLayout(newFieldPanel, BoxLayout.Y_AXIS));
        newFieldPanel.add(newFieldBtn);
//        newFieldPanel.add(newCustomFieldBtn);
//        newFieldPanel.add(importTextGridBtn);

		gbc.gridy = 3;
		add(newFieldPanel, gbc);

		gbc.gridx = 0;
		gbc.gridy = 14;
		add(editBtn, gbc);

		gbc.gridx = 1;
        add(deleteBtn, gbc);

		gbc.gridx = 2;
		gbc.gridx = 3;
		add(Box.createHorizontalStrut(365), gbc);
		gbc.gridx = 7;
		add(graphBtn, gbc);
		
		gbc.gridx = 8;
		add(cancelBtn, gbc);
		gbc.gridx = 10;
		add(doneBtn, gbc);

		// gestureFieldsTable has the longest name column - just doing this to get it looking nice
		for (int i = ARRAY_INDEX_FIELD_NAME; i <= ARRAY_INDEX_FIELD_LABEL; i++) {
		    var width = gestureFieldsTable.getColumnModel().getColumn(i).getPreferredWidth();
			nonManualFieldsTable.getColumnModel().getColumn(i).setPreferredWidth(width);
			glossFieldsTable.getColumnModel().getColumn(i).setPreferredWidth(width);
			customFieldsTable.getColumnModel().getColumn(i).setPreferredWidth(width);
		}

        nonManualFieldsTable.getColumnModel().getColumn(ARRAY_INDEX_DISPLAY_LABEL).setMaxWidth(120);
        glossFieldsTable.getColumnModel().getColumn(ARRAY_INDEX_DISPLAY_LABEL).setMaxWidth(120);
        customFieldsTable.getColumnModel().getColumn(ARRAY_INDEX_DISPLAY_LABEL).setMaxWidth(120);
        gestureFieldsTable.getColumnModel().getColumn(ARRAY_INDEX_DISPLAY_LABEL).setMaxWidth(120);

        nonManualFieldsTable.getColumnModel().getColumn(ARRAY_INDEX_UTTERANCE_VISIBLE).setMaxWidth(60);
        glossFieldsTable.getColumnModel().getColumn(ARRAY_INDEX_UTTERANCE_VISIBLE).setMaxWidth(60);
        customFieldsTable.getColumnModel().getColumn(ARRAY_INDEX_UTTERANCE_VISIBLE).setMaxWidth(60);
        gestureFieldsTable.getColumnModel().getColumn(ARRAY_INDEX_UTTERANCE_VISIBLE).setMaxWidth(60);

        for (int i = ARRAY_INDEX_SEGMENT_VISIBLE; i <= ARRAY_INDEX_GLOBAL_VISIBLE; i++) {
            nonManualFieldsTable.getColumnModel().getColumn(i).setMaxWidth(45);
            glossFieldsTable.getColumnModel().getColumn(i).setMaxWidth(45);
            customFieldsTable.getColumnModel().getColumn(i).setMaxWidth(45);
			gestureFieldsTable.getColumnModel().getColumn(i).setMaxWidth(45);
        }

        nonManualFieldsTable.getColumnModel().getColumn(ARRAY_INDEX_ANNOTATED).setMaxWidth(65);
        glossFieldsTable.getColumnModel().getColumn(ARRAY_INDEX_ANNOTATED).setMaxWidth(65);
        customFieldsTable.getColumnModel().getColumn(ARRAY_INDEX_ANNOTATED).setMaxWidth(65);
        gestureFieldsTable.getColumnModel().getColumn(ARRAY_INDEX_ANNOTATED).setMaxWidth(65);

        nonManualFieldsTable.getColumnModel().getColumn(ARRAY_INDEX_COLOR).setMaxWidth(40);
        glossFieldsTable.getColumnModel().getColumn(ARRAY_INDEX_COLOR).setMaxWidth(40);
        customFieldsTable.getColumnModel().getColumn(ARRAY_INDEX_COLOR).setMaxWidth(40);
        gestureFieldsTable.getColumnModel().getColumn(ARRAY_INDEX_COLOR).setMaxWidth(40);
	}

	public void close() {
        setSelectedTreeTableNode(null);
        dialog.dispose();
	}

	public void updateCurrentUtterance() {
	    var view = segmentPanel.getUtteranceView();
	    if (view != null) {
	        var template = view.getSegmentTemplate(segment.getID());
	        if (template != null) {
                var i = template.getSelectedUtteranceIndex();
                utterance = i == -1 || i > segment.getSS3GlossWindowItems().size() - 1
                    ? null
                    : segment.getSS3GlossWindowItems().get(i);
            } else {
	            utterance = null;
            }
        } else {
	        utterance = null;
        }

	    utteranceLabel.setText(UTTERANCE_LABEL_PREFIX + (utterance == null ? "None" : utterance.getLabel()));
    }

    public CodingSchemaFieldNode getSelectedNode() {
	    return (CodingSchemaFieldNode)selectedNode;
    }

	public void setSelectedTreeTableNode(CodingSchemaFieldNode selectedNode) {
	    if (selectedNode != null)
            this.selectedNode = selectedNode;

		deleteBtn.setEnabled(false);
		editBtn.setEnabled(false);
		indexBtn.setEnabled(false);

		if (selectedNode == null)
		    return;

		CodingSchemaFieldNode rootNode;
		boolean isNonManualNode = false;
		boolean isManualNode = selectedNode.isManualNode();
		boolean isCustomNode = selectedNode.isCustomNode();
		boolean isGestureNode = selectedNode.isGestureNode();
		if (isManualNode) {
			nonManualFieldsTable.clearSelection();
			customFieldsTable.clearSelection();
			gestureFieldsTable.clearSelection();
			
			rootNode = glossFieldsRootNode;
		} else if (isCustomNode) {
			nonManualFieldsTable.clearSelection();
			glossFieldsTable.clearSelection();
			gestureFieldsTable.clearSelection();
			
			rootNode = customFieldsRootNode;
		} else if (isGestureNode) {
			nonManualFieldsTable.clearSelection();
			glossFieldsTable.clearSelection();
			customFieldsTable.clearSelection();
			
			rootNode = gestureFieldsRootNode;
		} else {
			glossFieldsTable.clearSelection();
			customFieldsTable.clearSelection();
			gestureFieldsTable.clearSelection();
			
			isNonManualNode = true;
			rootNode = nonManualFieldsRootNode;
		}

		if (selectedNode != null && selectedNode.isLeaf() && !selectedNode.isGroup()) {
			if (rootNode.equals(selectedNode.getParent())) {
				// GREG: i have no idea when this would be called, but whatever
				cancelBtn.setEnabled(true);
				doneBtn.setEnabled(true);
			} else {
				if (isNonManualNode) {
					boolean doNext = true;
					while (doNext) {
						TreeTableNode pNode = selectedNode.getParent();
						if (rootNode.equals(pNode.getParent())) {
							doNext = false;
							int selectedGroup = getGroup(pNode);
							if (selectedGroup != CSFieldPanel.BAD_GROUP_NUMBER) {
								editBtn.setEnabled(true);
								deleteBtn.setEnabled(true);
								indexBtn.setEnabled(true);
							}
						}
					}
				} else if (isCustomNode) {
					boolean doNext = true;
					while (doNext) {
						TreeTableNode pNode = selectedNode.getParent();
						if (rootNode.equals(pNode.getParent())) {
							doNext = false;
							int selectedGroup = getCustomGroup(pNode);
							if (selectedGroup != CSFieldPanel.BAD_GROUP_NUMBER) {
								editBtn.setEnabled(true);
								deleteBtn.setEnabled(true);
								indexBtn.setEnabled(true);
							}
						}
					}
				}
			}
		}
    }

	public void setSelectedNodeID(String id) {
		if (id != null) {
			this.selectedNodeID = id;
		}
	}
	
	public boolean isFieldNameAndLabelUnique(String fieldName, String fieldLabel, int groupID) {
		MutableTreeTableNode node = groupID >= 0 && groupID < nonManualFieldsRootNode.getChildCount()
				? (MutableTreeTableNode) nonManualFieldsRootNode.getChildAt(groupID)
				: nonManualFieldsRootNode;

		int numberOfChildren = node.getChildCount();
		for (int i = 0; i < numberOfChildren; i++) {
			MutableTreeTableNode childNode = (MutableTreeTableNode) node.getChildAt(i);
			String existingFieldName = (String) childNode.getValueAt(ARRAY_INDEX_FIELD_NAME);
			String existingFieldLabel = (String) childNode.getValueAt(ARRAY_INDEX_FIELD_LABEL);

			if (existingFieldName.equals(fieldName) || existingFieldLabel.equals(fieldLabel)) {
				return false;
			}
		}
		return true;
	}
	
	public boolean isCustomFieldNameAndLabelUnique(String fieldName, String fieldLabel, int groupID) {
		MutableTreeTableNode node = groupID >= 0 && groupID < customFieldsRootNode.getChildCount()
				? (MutableTreeTableNode) customFieldsRootNode.getChildAt(groupID)
				: customFieldsRootNode;

		int numberOfChildren = node.getChildCount();
		for (int i = 0; i < numberOfChildren; i++) {
			MutableTreeTableNode childNode = (MutableTreeTableNode) node.getChildAt(i);
			String existingFieldName = (String) childNode.getValueAt(ARRAY_INDEX_FIELD_NAME);
			String existingFieldLabel = (String) childNode.getValueAt(ARRAY_INDEX_FIELD_LABEL);

			if (existingFieldName.equals(fieldName) || existingFieldLabel.equals(fieldLabel)) {
				return false;
			}
		}
		return true;
	}

	public void addNewField(SS3Field field) {
		recentlyAddedFields.add(field);

		int groupID = field.getGroupID();

		DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) nonManualFieldsTable.getTreeTableModel();
		MutableTreeTableNode node = groupID >= 0 && groupID < CSFieldPanel.BAD_GROUP_NUMBER
				? (MutableTreeTableNode) nonManualFieldsRootNode.getChildAt(groupID)
				: nonManualFieldsRootNode;

		Object[] newFieldObject = new Object[ARRAY_SIZE];
		newFieldObject[ARRAY_INDEX_FIELD_NAME] = field.getName();
		newFieldObject[ARRAY_INDEX_FIELD_LABEL] = field.getLabel();
		newFieldObject[ARRAY_INDEX_UTTERANCE_VISIBLE] = false;
        newFieldObject[ARRAY_INDEX_SEGMENT_VISIBLE] = false;
        newFieldObject[ARRAY_INDEX_GLOBAL_VISIBLE] = false;
		newFieldObject[ARRAY_INDEX_ANNOTATED] = "";
		newFieldObject[ARRAY_INDEX_COLOR] = field.getColor();
		newFieldObject[ARRAY_INDEX_PREVIEW] = false;
		newFieldObject[ARRAY_INDEX_FIELD_ID] = "" + field.getId();

		CodingSchemaFieldNode newFieldNode = new CodingSchemaFieldNode(newFieldObject);
		fieldsModel.insertNodeInto(newFieldNode, node, node.getChildCount());

        newFieldNode.setValueAt(true, ARRAY_INDEX_SEGMENT_VISIBLE);

        scrollToAndSelectNode(newFieldNode);
	}

	public void addNewCustomField(SS3Field field) {
		recentlyAddedFields.add(field);

		int groupID = field.getGroupID();

		DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) customFieldsTable.getTreeTableModel();
		MutableTreeTableNode node = groupID >= 0 && groupID < CSFieldPanel.BAD_GROUP_NUMBER
				? (MutableTreeTableNode) customFieldsRootNode.getChildAt(groupID)
				: customFieldsRootNode;

		Object[] newFieldObject = new Object[ARRAY_SIZE];
		newFieldObject[ARRAY_INDEX_FIELD_NAME] = field.getName();
		newFieldObject[ARRAY_INDEX_FIELD_LABEL] = field.getLabel();
		newFieldObject[ARRAY_INDEX_UTTERANCE_VISIBLE] = false;
        newFieldObject[ARRAY_INDEX_SEGMENT_VISIBLE] = false;
        newFieldObject[ARRAY_INDEX_GLOBAL_VISIBLE] = false;
		newFieldObject[ARRAY_INDEX_ANNOTATED] = "";
		newFieldObject[ARRAY_INDEX_COLOR] = field.getColor();
		newFieldObject[ARRAY_INDEX_PREVIEW] = false;
		newFieldObject[ARRAY_INDEX_FIELD_ID] = "" + field.getId();
		
		CodingSchemaFieldNode newFieldNode = new CodingSchemaFieldNode(newFieldObject, false, true, false);
		fieldsModel.insertNodeInto(newFieldNode, node, node.getChildCount());

		newFieldNode.setValueAt(true, ARRAY_INDEX_SEGMENT_VISIBLE);

		scrollToAndSelectCustomNode(newFieldNode);
	}

	public void updateField(SS3Field field) {
		DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) nonManualFieldsTable.getTreeTableModel();

		var realField = segment.getRootLabelObject()
            .getNonManuelLabelObject()
            .getChildWithFieldID(field.getId());

		// check if field is still in the same group
		int legacyGroupID = getGroup(selectedNode.getParent());
		int groupID = field.getGroupID();

		SS3CodingScheme dbScheme = segmentPanel.getSS3Database().getSS3CodingScheme();
		dbScheme.addField(field);

		fieldsModel.setValueAt(field.getName(), selectedNode, ARRAY_INDEX_FIELD_NAME);
		fieldsModel.setValueAt(field.getLabel(), selectedNode, ARRAY_INDEX_FIELD_LABEL);
		fieldsModel.setValueAt(field.getColor(), selectedNode, ARRAY_INDEX_COLOR);

		if (legacyGroupID != groupID) {
			fieldsModel.removeNodeFromParent(selectedNode);
			addNewField(field);
		}
	}

	public void updateCustomField(SS3Field field) {
		DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) customFieldsTable.getTreeTableModel();

		// check if field is still in the same group
		int legacyGroupID = getCustomGroup(selectedNode.getParent());
		int groupID = field.getGroupID();

		SS3CodingScheme dbScheme = segmentPanel.getSS3Database().getSS3CodingScheme();
		dbScheme.addField(field);

        fieldsModel.setValueAt(field.getName(), selectedNode, ARRAY_INDEX_FIELD_NAME);
        fieldsModel.setValueAt(field.getLabel(), selectedNode, ARRAY_INDEX_FIELD_LABEL);
        fieldsModel.setValueAt(field.getColor(), selectedNode, ARRAY_INDEX_COLOR);

		if (legacyGroupID != groupID) {
			fieldsModel.removeNodeFromParent(selectedNode);
			addNewCustomField(field);
		}
	}

	private void createNewFieldDialog() {
		if (newFieldDialog == null) {
			newFieldDialog = new JDialog(dialog, "Field Specifications Editor", true) {
                @Override
                public void setVisible(boolean b) {
                    if (b)
                        dialog.block();
                    else
                        dialog.unblock();
                    super.setVisible(b);
                }

                @Override
                public void dispose() {
                    super.dispose();
                    dialog.unblock();
                }
            };
		}

		newFieldPanel = new AddOrUpdateField(segmentPanel, segment, getGroup(selectedNode), null, this, newFieldDialog, parentField);
//		newFieldPanel.setBorder(BorderFactory.createEmptyBorder(15, 15, 15, 15));

		newFieldDialog.pack();
		newFieldDialog.setLayout(new BorderLayout());
		newFieldDialog.add(newFieldPanel, BorderLayout.CENTER);

		newFieldDialog.addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent we) {
				if (newFieldPanel != null) {
					newFieldPanel.discard();
					newFieldPanel = null;
				}

				dialog.unblock();
//				newFieldDialog.dispose();
			}
		});

		newFieldDialog.setSize(newFieldDialog.getPreferredSize());
		newFieldDialog.setLocation(150, 150);
		newFieldDialog.setVisible(true);
	}

	private void createNewCustomFieldDialog() {
		if (newFieldDialog == null) {
			newFieldDialog = new JDialog(dialog, "Default Field Specifications Editor", true) {
                @Override
                public void setVisible(boolean b) {
                    if (b)
                        dialog.block();
                    else
                        dialog.unblock();
                    super.setVisible(b);
                }

                @Override
                public void dispose() {
                    super.dispose();
                    dialog.unblock();
                }
            };
		}

		newCustomFieldPanel = new AddOrUpdateCustomField(segmentPanel, segment, getCustomGroup(selectedNode), null, this, newFieldDialog, parentField);
//		newCustomFieldPanel.setBorder(BorderFactory.createEmptyBorder(15, 15, 15, 15));

		newFieldDialog.pack();
		newFieldDialog.setLayout(new BorderLayout());
		newFieldDialog.add(newCustomFieldPanel, BorderLayout.CENTER);

		newFieldDialog.addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent we) {
				if (newCustomFieldPanel != null) {
					newCustomFieldPanel.discard();
					newCustomFieldPanel = null;
				}

				dialog.unblock();

//				newFieldDialog.dispose();
			}
		});

		newFieldDialog.setSize(newFieldDialog.getPreferredSize());
		newFieldDialog.setLocation(150, 150);
		newFieldDialog.setVisible(true);
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		String str = ae.getActionCommand();

		SS3CodingScheme dbScheme = segmentPanel.getSS3Database().getSS3CodingScheme();
		
		switch (Integer.parseInt(str)) {
		case NEW_FIELD:
			createNewFieldDialog();
			break;
		case NEW_CUSTOM_FIELD:
			createNewCustomFieldDialog();
			break;

		case EDIT:

			SS3Field fieldToEdit = null;
			if (selectedNode != null) {
				String id = (String) selectedNode.getValueAt(ARRAY_INDEX_FIELD_ID);
				fieldToEdit = dbScheme.getField(id);
			}
			
			CodingSchemaFieldNode cNode = (CodingSchemaFieldNode) selectedNode;

			if (editFieldDialog == null) {
				editFieldDialog = new JDialog(dialog, "Field Specifications Editor", true);
			}

			if (cNode.isNonManualNode()) {
				editFieldPanel = new AddOrUpdateField(segmentPanel, segment, getGroup(cNode.getParent()),
						fieldToEdit, this, editFieldDialog, parentField);
//				editFieldPanel.setBorder(BorderFactory.createEmptyBorder(15, 15, 15, 15));

				editFieldDialog.pack();
				editFieldDialog.setLayout(new BorderLayout());
				editFieldDialog.add(editFieldPanel, BorderLayout.CENTER);
			} else if (cNode.isCustomNode()) {
				editCustomFieldPanel = new AddOrUpdateCustomField(segmentPanel, segment, 
						getCustomGroup(cNode.getParent()),
						fieldToEdit, this, editFieldDialog, parentField);
//				editCustomFieldPanel.setBorder(BorderFactory.createEmptyBorder(15, 15, 15, 15));

				editFieldDialog.pack();
				editFieldDialog.setLayout(new BorderLayout());
				editFieldDialog.add(editCustomFieldPanel, BorderLayout.CENTER);
			} else {
				break;
			}			

			editFieldDialog.addWindowListener(new WindowAdapter() {
				@Override
				public void windowClosing(WindowEvent we) {
					if (editFieldPanel != null) {
						editFieldPanel.discard();
						editFieldPanel = null;
					}
					if (editCustomFieldPanel != null) {
						editCustomFieldPanel.discard();
						editCustomFieldPanel = null;
					}
//					editFieldDialog.dispose();
				}
			});

			editFieldDialog.setSize(editFieldDialog.getPreferredSize());
			editFieldDialog.setLocation(150, 150);
			editFieldDialog.setVisible(true);

			break;

		case DELETE:
			
			cNode = (CodingSchemaFieldNode) selectedNode;

			fieldToEdit = null;
			if (selectedNode == null)
				return;

			String id = (String) selectedNode.getValueAt(ARRAY_INDEX_FIELD_ID);
			
			fieldToEdit = dbScheme.getField(id);
			SS3CodingScheme defaultCodingScheme = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();
			SS3Field defaultField = defaultCodingScheme.getField(id);
			
			if (defaultField != null) {
				// then it's a default schema field -- make dialog to say: no, cannot delete
				String warningDialogText = "You cannot delete default fields.";
				String warningTitleText = "Delete Field";
				int optionType = JOptionPane.YES_NO_OPTION;
				int messageType = JOptionPane.WARNING_MESSAGE;
				String options[] = {"OK"};
				try {
					JOptionPane.showOptionDialog(this,
							warningDialogText,
							warningTitleText,
							optionType,
							messageType,
							null,
							options,
							options[0]);
				} catch (HeadlessException he) {
					System.out.println("");
				}

				return;
			}

			
			String name = fieldToEdit.getLabelOrName();
			String warningDialogText =
					"All annotated data for field \""+ name + "\" "
					+ "from all segment tiers will be DELETED.\n\n"
					+ "Are you absolutely sure you want to delete field \"" + name + "\"?";
			String warningTitleText = "WARNING";
			int optionType = JOptionPane.YES_NO_OPTION;
			int messageType = JOptionPane.ERROR_MESSAGE;
			String options[] = {"Proceed", "Cancel"};
			
			int result = -1;
			try {
				result = JOptionPane.showOptionDialog(this, 
						warningDialogText, 
						warningTitleText, 
						optionType, 
						messageType, 
						null, 
						options, 
						options[1]);
			} catch (HeadlessException he) {
				System.out.println("");
			}
			
			if (result != JOptionPane.OK_OPTION)
				return;

            segmentPanel.getSS3Database().getSS3CodingScheme().removeField(fieldToEdit);
            segmentPanel.getSS3Database().removeTrackByFieldID(fieldToEdit.getId());
            
            // delete this field from the root label object on all segment tiers
            for (Segment segment: segmentPanel.getSegments()) {
            		String fieldID = fieldToEdit.getId();
                RootLabelObject rootLabelObj = segment.getRootLabelObject();

                LabelObject objToRemove = null;
                if (cNode.isNonManualNode()) {
                	objToRemove = rootLabelObj.getNonManuelLabelObject()
                			.getChildWithFieldID(fieldID);
                	
                	rootLabelObj.getNonManuelLabelObject().removeChild(objToRemove);
                } else if (cNode.isCustomNode()) {
                	objToRemove = rootLabelObj.getCustomLabelObject()
                			.getChildWithFieldID(fieldID);
                	rootLabelObj.getCustomLabelObject().removeChild(objToRemove);
                }

                // remove all entities from the field which is getting deleted - this just avoids unnecessary nullptr paint errors
                if (objToRemove != null) {
                	Field fieldToRemove = objToRemove.getField();
                	for (EventsEntity entity: fieldToRemove.getAllEntities()) {
                		fieldToRemove.deleteEntity(entity);
                	}
                }
                
                // remove the entire statement field from the segment tier as well - this avoids saving old/deleted data in the collection file
                SS3GlossWindowSegment segmentTier = segment.getParentField().getSS3GlossWindowSegment();
                segmentTier.removeField(fieldID);	// remove statement field
            }

			// Remove it from the tree
            setSelectedTreeTableNode((CodingSchemaFieldNode) cNode.getParent());
            if (cNode.isNonManualNode()) {
                var model = (DefaultTreeTableModel) nonManualFieldsTable.getTreeTableModel();
                model.removeNodeFromParent(cNode);
            } else if (cNode.isCustomNode()) {
                var model = (DefaultTreeTableModel) customFieldsTable.getTreeTableModel();
                model.removeNodeFromParent(cNode);
            }

    		// need to refresh all segment tiers' panels simultaneously to pick up the deletion
            if (segment.getSignStreamSegmentPanel().getUtteranceView() != null
            		&& segment.getSignStreamSegmentPanel().getUtteranceView().getSegmentTemplates() != null) {
                for (SegmentTemplate st: segment.getSignStreamSegmentPanel().getUtteranceView().getSegmentTemplates()) {
                	st.getNonmanView().getCSFieldDialog().getCSFieldPanel().discard();
                	st.getNonmanView().getCSFieldDialog().getCSFieldPanel().apply();
                	st.repaint();
                }
            }
            
            close();
			break;	
			
		case CANCEL:
		    discard();
		    close();
			break;
		case DONE:
            apply();
            close();
			break;

		case INDEX:
			if (selectedNodeID != null) {
				SegmentControlPanel segControlPanel = segmentPanel.getUtteranceView().getNonmanualFieldsView().getLabelView().getControlPanel();
				segControlPanel.updateUtteranceList(selectedNodeID);
			}
			break;
		case RESET_INDEX:
			SegmentControlPanel segControlPanel = segmentPanel.getUtteranceView().getNonmanualFieldsView().getLabelView().getControlPanel();
			segControlPanel.updateUtteranceList(null);
			break;
		case GRAPH:
			graphFieldDialog.setVisible(true, null);
		}
			
	}

    public JScrollPane getTreeTableScrollPanel() {
	    var root = segment.getRootLabelObject();
        var scheme = segmentPanel.getSS3Database().getSS3CodingScheme();

        populateFields(nonManualFieldsRootNode, root.getNonManuelLabelObject(), scheme.loadNonManualGroups());
        populateFields(glossFieldsRootNode, root.getGlossLabelObject(), scheme.loadGlossFieldGroups());
        populateFields(gestureFieldsRootNode, root.getGestureLabelObject(), scheme.loadGestureFieldGroups());
        populateFields(customFieldsRootNode, root.getCustomLabelObject(), scheme.loadCustomFieldGroups());

		String[] cNames = new String[ARRAY_SIZE];
		cNames[ARRAY_INDEX_FIELD_NAME] = "Field Names";
		cNames[ARRAY_INDEX_FIELD_LABEL] = "Field Labels";
		cNames[ARRAY_INDEX_DISPLAY_LABEL] = "Display by Default: ";
		cNames[ARRAY_INDEX_UTTERANCE_VISIBLE] = "Utterance";
        cNames[ARRAY_INDEX_SEGMENT_VISIBLE] = "Tier";
        cNames[ARRAY_INDEX_GLOBAL_VISIBLE] = "Global";
		cNames[ARRAY_INDEX_ANNOTATED] = "Annotated";
		cNames[ARRAY_INDEX_COLOR] = "Color";
		cNames[ARRAY_INDEX_PREVIEW] = "Preview";
		cNames[ARRAY_INDEX_FIELD_ID] = " ";

		DefaultTreeTableModel nonManualFieldsModel = new DefaultTreeTableModel(nonManualFieldsRootNode, Arrays.asList(cNames)) {
            @Override
			public Class<?> getColumnClass(int column) {

				if (column == ARRAY_INDEX_UTTERANCE_VISIBLE
                    || column == ARRAY_INDEX_SEGMENT_VISIBLE
                    || column == ARRAY_INDEX_GLOBAL_VISIBLE
                    || column == ARRAY_INDEX_PREVIEW) {
					return Boolean.class;
				} else if (column == ARRAY_INDEX_COLOR) {
					return Color.class;
				} else {
					return String.class;
				}
			}

			@Override
			public boolean isCellEditable(Object node, int column) {
                return column == ARRAY_INDEX_COLOR
                    || (column == ARRAY_INDEX_UTTERANCE_VISIBLE && utterance != null)
                    || column == ARRAY_INDEX_SEGMENT_VISIBLE
                    || column == ARRAY_INDEX_GLOBAL_VISIBLE;
			}



			@Override
			public int getChildCount(Object parent) {
				// Only override count if at the root of the tree.
//				if (parent == nonManualFieldsRootNode) {
//					CodingSchemaFieldNode root = (CodingSchemaFieldNode) parent;
//					int children = 0;
//					for (int i = 0; i < root.getChildCount(); i++) {
//						TreeTableNode child = root.getChildAt(i);
//						// Nonmanual Fields contain an ArrayList of child IDs in their last column
//						if (child.getValueAt(ARRAY_INDEX_FIELD_ID) instanceof ArrayList)
//							children++;
//					}
//					return children;
//				} else
					return super.getChildCount(parent);
			}
		};

		nonManualFieldsTable = new JXTreeTable(nonManualFieldsModel) {
			private static final long serialVersionUID = 1L;

			@Override
			public TableCellRenderer getCellRenderer(int row, int column) {
				if (column == ARRAY_INDEX_COLOR) {
                    return new ColorRenderer(true);
				} else if (column == ARRAY_INDEX_PREVIEW) {
					//return new RadioButtonRenderer();
				}
				// else...
				return super.getCellRenderer(row, column);
			}
		};
		
		final DefaultTreeTableModel customFieldsModel = new DefaultTreeTableModel(customFieldsRootNode, Arrays.asList(cNames)) {
			@Override
			public Class<?> getColumnClass(int column) {
                if (column == ARRAY_INDEX_UTTERANCE_VISIBLE
                    || column == ARRAY_INDEX_SEGMENT_VISIBLE
                    || column == ARRAY_INDEX_GLOBAL_VISIBLE
                    || column == ARRAY_INDEX_PREVIEW) {
					return Boolean.class;
				} else if (column == ARRAY_INDEX_COLOR) {
					return Color.class;
				}
				return String.class;
			}

			@Override
			public boolean isCellEditable(Object node, int column) {
				return column == ARRAY_INDEX_COLOR
                    || (column == ARRAY_INDEX_UTTERANCE_VISIBLE && utterance != null)
                    || column == ARRAY_INDEX_SEGMENT_VISIBLE
                    || column == ARRAY_INDEX_GLOBAL_VISIBLE;
			}
		};
		customFieldsTable = new JXTreeTable(customFieldsModel) {
			private static final long serialVersionUID = 1L;
			
			@Override
			public TableCellRenderer getCellRenderer(int row, int column) {
				if (column == ARRAY_INDEX_COLOR) {
					return new ColorRenderer(true);
				} else if (column == ARRAY_INDEX_PREVIEW) {
					//return new RadioButtonRenderer();
				}
				return super.getCellRenderer(row,column);
			}
		};
		customFieldsTable.expandAll();
		
		final DefaultTreeTableModel manualFieldsModel = new DefaultTreeTableModel(glossFieldsRootNode, Arrays.asList(cNames)) {
			@Override
			public Class<?> getColumnClass(int column) {
                if (column == ARRAY_INDEX_UTTERANCE_VISIBLE
                    || column == ARRAY_INDEX_SEGMENT_VISIBLE
                    || column == ARRAY_INDEX_GLOBAL_VISIBLE
                    || column == ARRAY_INDEX_PREVIEW) {
					return Boolean.class;
				} else if (column == ARRAY_INDEX_COLOR) {
					return Color.class;
				}
				return String.class;
			}

			@Override
			public boolean isCellEditable(Object node, int column) {
                return column == ARRAY_INDEX_COLOR
                    || (column == ARRAY_INDEX_UTTERANCE_VISIBLE && utterance != null)
                    || column == ARRAY_INDEX_SEGMENT_VISIBLE
                    || column == ARRAY_INDEX_GLOBAL_VISIBLE;
            }
        };
        glossFieldsTable = new JXTreeTable(manualFieldsModel) {
			private static final long serialVersionUID = 1L;
			
			@Override
			public TableCellRenderer getCellRenderer(int row, int column) {
				if (column == ARRAY_INDEX_COLOR) {
					return new ColorRenderer(true);
				} else if (column == ARRAY_INDEX_PREVIEW) {
					//return new RadioButtonRenderer();
				}
				return super.getCellRenderer(row,column);
			}
		};
		glossFieldsTable.expandAll();

		DefaultTreeTableModel gestureFieldsModel = new DefaultTreeTableModel(gestureFieldsRootNode, Arrays.asList(cNames)) {
			@Override
			public Class<?> getColumnClass(int column) {

				if (column == ARRAY_INDEX_UTTERANCE_VISIBLE
                    || column == ARRAY_INDEX_SEGMENT_VISIBLE
                    || column == ARRAY_INDEX_GLOBAL_VISIBLE
                    || column == ARRAY_INDEX_PREVIEW) {
					return Boolean.class;
				} else if (column == ARRAY_INDEX_COLOR) {
					return Color.class;
				} else {
					return String.class;
				}
			}

			@Override
			public boolean isCellEditable(Object node, int column) {
                return column == ARRAY_INDEX_COLOR
                    || (column == ARRAY_INDEX_UTTERANCE_VISIBLE && utterance != null)
                    || column == ARRAY_INDEX_SEGMENT_VISIBLE
                    || column == ARRAY_INDEX_GLOBAL_VISIBLE;
			}

			@Override
			public int getChildCount(Object parent) {
				// Only override count if at the root of the tree.
//				if (parent == gestureFieldsRootNode) {
//					CodingSchemaFieldNode root = (CodingSchemaFieldNode) parent;
//					int children = 0;
//					for (int i = 0; i < root.getChildCount(); i++) {
//						TreeTableNode child = root.getChildAt(i);
//						// Nonmanual Fields contain an ArrayList of child IDs in their 6th column
//						if (child.getValueAt(ARRAY_INDEX_FIELD_ID) instanceof ArrayList)
//							children++;
//					}
//					return children;
//				} else
					return super.getChildCount(parent);
			}
		};
		
		gestureFieldsTable = new JXTreeTable(gestureFieldsModel) {
			private static final long serialVersionUID = 1L;

			@Override
			public TableCellRenderer getCellRenderer(int row, int column) {
				if (column == ARRAY_INDEX_COLOR) {
					return new ColorRenderer(true);
				} else if (column == ARRAY_INDEX_PREVIEW) {
					//return new RadioButtonRenderer();
				}
				// else...
				return super.getCellRenderer(row, column);
			}
		};
		gestureFieldsTable.expandAll();

		TableColumn customCol = nonManualFieldsTable.getColumnModel().getColumn(ARRAY_INDEX_COLOR);
		customCol.setCellEditor(new ColorEditor());

		TableColumn customColorCol = customFieldsTable.getColumnModel().getColumn(ARRAY_INDEX_COLOR);
		customColorCol.setCellEditor(new ColorEditor());

		TableColumn manualColorCol = glossFieldsTable.getColumnModel().getColumn(ARRAY_INDEX_COLOR);
		manualColorCol.setCellEditor(new ColorEditor());

		TableColumn gestureColorCol = gestureFieldsTable.getColumnModel().getColumn(ARRAY_INDEX_COLOR);
		gestureColorCol.setCellEditor(new ColorEditor());
		
		//next 2 blocks - adding RadioButtonEditors
		TableColumn previewCol = nonManualFieldsTable.getColumnModel().getColumn(ARRAY_INDEX_PREVIEW);
		RadioButtonEditor nmRadioButtonEditor = new RadioButtonEditor(new JCheckBox());
		previewCol.setCellEditor(nmRadioButtonEditor);
		
		TableColumn customPreviewCol = customFieldsTable.getColumnModel().getColumn(ARRAY_INDEX_PREVIEW);
		RadioButtonEditor customRadioButtonEditor = new RadioButtonEditor(new JCheckBox());
		customPreviewCol.setCellEditor(customRadioButtonEditor);
		
		TableColumn manualPreviewCol = glossFieldsTable.getColumnModel().getColumn(ARRAY_INDEX_PREVIEW);
		RadioButtonEditor manRadioButtonEditor = new RadioButtonEditor(new JCheckBox());
		manualPreviewCol.setCellEditor(manRadioButtonEditor);
		
		TableColumn gesturePreviewCol = gestureFieldsTable.getColumnModel().getColumn(ARRAY_INDEX_PREVIEW);
		RadioButtonEditor gestureRadioButtonEditor = new RadioButtonEditor(new JCheckBox());
		gesturePreviewCol.setCellEditor(gestureRadioButtonEditor);
		
		nonManualFieldsTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		nonManualFieldsTable.setHorizontalScrollEnabled(true);
		nonManualFieldsTable.setFillsViewportHeight(false);
		nonManualFieldsTable.setRootVisible(false);

		customFieldsTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		customFieldsTable.setHorizontalScrollEnabled(true);
		customFieldsTable.setFillsViewportHeight(false);
		customFieldsTable.setRootVisible(false);

		glossFieldsTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		glossFieldsTable.setHorizontalScrollEnabled(true);
		glossFieldsTable.setFillsViewportHeight(false);
		glossFieldsTable.setRootVisible(false);

		gestureFieldsTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		gestureFieldsTable.setHorizontalScrollEnabled(true);
		gestureFieldsTable.setFillsViewportHeight(false);
		gestureFieldsTable.setRootVisible(false);

		// do not show field id column
		nonManualFieldsTable.getColumnModel().removeColumn(nonManualFieldsTable.getColumnModel().getColumn(ARRAY_INDEX_FIELD_ID));
		nonManualFieldsTable.setAutoCreateColumnsFromModel(false);

		customFieldsTable.getColumnModel().removeColumn(customFieldsTable.getColumnModel().getColumn(ARRAY_INDEX_FIELD_ID));
		customFieldsTable.setAutoCreateColumnsFromModel(false);

		glossFieldsTable.getColumnModel().removeColumn(glossFieldsTable.getColumnModel().getColumn(ARRAY_INDEX_FIELD_ID));
		glossFieldsTable.setAutoCreateColumnsFromModel(false);

		gestureFieldsTable.getColumnModel().removeColumn(gestureFieldsTable.getColumnModel().getColumn(ARRAY_INDEX_FIELD_ID));
		gestureFieldsTable.setAutoCreateColumnsFromModel(false);
		
		//next 2 blocks - do not show temp radio button column
		nonManualFieldsTable.getColumnModel().removeColumn(nonManualFieldsTable.getColumnModel().getColumn(ARRAY_INDEX_PREVIEW));
		nonManualFieldsTable.setAutoCreateColumnsFromModel(false);
		
		customFieldsTable.getColumnModel().removeColumn(customFieldsTable.getColumnModel().getColumn(ARRAY_INDEX_PREVIEW));
		customFieldsTable.setAutoCreateColumnsFromModel(false);
		
		glossFieldsTable.getColumnModel().removeColumn(glossFieldsTable.getColumnModel().getColumn(ARRAY_INDEX_PREVIEW));
		glossFieldsTable.setAutoCreateColumnsFromModel(false);
		
		gestureFieldsTable.getColumnModel().removeColumn(gestureFieldsTable.getColumnModel().getColumn(ARRAY_INDEX_PREVIEW));
		gestureFieldsTable.setAutoCreateColumnsFromModel(false);

		UIDefaults uiDefs = UIManager.getDefaults();
		uiDefs.put("Tree.leftChildIndent", new Integer(3));

		nonManualFieldsTable.setOpenIcon(null);
		nonManualFieldsTable.setClosedIcon(null);
		nonManualFieldsTable.setLeafIcon(null);

		customFieldsTable.setOpenIcon(null);
		customFieldsTable.setClosedIcon(null);
		customFieldsTable.setLeafIcon(null);

		glossFieldsTable.setOpenIcon(null);
		glossFieldsTable.setClosedIcon(null);
		glossFieldsTable.setLeafIcon(null);

		gestureFieldsTable.setOpenIcon(null);
		gestureFieldsTable.setClosedIcon(null);
		gestureFieldsTable.setLeafIcon(null);


		nonManualFieldsTable.setRowHeight(SegmentGraphReprUtil.treeTableCellHeight);
		nonManualFieldsTable.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		nonManualFieldsTable.setDragEnabled(false);

		customFieldsTable.setRowHeight(SegmentGraphReprUtil.treeTableCellHeight);
		customFieldsTable.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

		glossFieldsTable.setRowHeight(SegmentGraphReprUtil.treeTableCellHeight);
		glossFieldsTable.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

		gestureFieldsTable.setRowHeight(SegmentGraphReprUtil.treeTableCellHeight);
		gestureFieldsTable.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

		nonManualFieldsTable.addTreeSelectionListener(selectionListener);
		customFieldsTable.addTreeSelectionListener(customSelectionListener);
		glossFieldsTable.addTreeSelectionListener(manualSelectionListener);
		gestureFieldsTable.addTreeSelectionListener(gestureSelectionListener);
		
		nonManualFieldsTable.packAll();
		customFieldsTable.packAll();
		glossFieldsTable.packAll();
		gestureFieldsTable.packAll();

		JPanel tablesPanel = new JPanel();
		BoxLayout layout = new BoxLayout(tablesPanel, BoxLayout.PAGE_AXIS);
		tablesPanel.setLayout(layout);

		Dimension margin = new Dimension(0,10);

        tablesPanel.add(nonManualFieldsTable.getTableHeader());
        tablesPanel.add(nonManualFieldsTable);
//		tablesPanel.add(Box.createRigidArea(margin));
//        tablesPanel.add(customFieldsTable.getTableHeader());
//		tablesPanel.add(customFieldsTable);
		tablesPanel.add(Box.createRigidArea(margin));
        tablesPanel.add(glossFieldsTable.getTableHeader());
		tablesPanel.add(glossFieldsTable);
//		tablesPanel.add(Box.createRigidArea(margin));
//        tablesPanel.add(gestureFieldsTable.getTableHeader());
//		tablesPanel.add(gestureFieldsTable);

		JScrollPane jsp = new JScrollPane(tablesPanel);
		jsp.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);

		return jsp;
	}

	public int getGroup(TreeTableNode node) {
		if (node == null)
			return CSFieldPanel.BAD_GROUP_NUMBER;

		DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) nonManualFieldsTable.getTreeTableModel();
		TreeTableNode rootNode = fieldsModel.getRoot();
		TreeTableNode headNode = rootNode.getChildAt(0);
		TreeTableNode faceNode = rootNode.getChildAt(1);
		TreeTableNode bodyNode = rootNode.getChildAt(2);
		TreeTableNode gramNode = rootNode.getChildAt(3);

		if (node.equals(headNode))
			return 0;
		if (node.equals(faceNode))
			return 1;
		if (node.equals(bodyNode))
			return 2;
		if (node.equals(gramNode))
			return 3;
		return CSFieldPanel.BAD_GROUP_NUMBER;
	}

	public int getCustomGroup(TreeTableNode node) {
		if (node == null)
			return CSFieldPanel.BAD_GROUP_NUMBER;

        DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) customFieldsTable.getTreeTableModel();
        TreeTableNode rootNode = fieldsModel.getRoot();
        TreeTableNode speechNode = rootNode.getChildAt(0);
        TreeTableNode genericNode = rootNode.getChildAt(1);

		// right now there is only one custom group - maybe we would have 3 - for as many different types
        if (node.equals(speechNode))
            return 0;
        if (node.equals(genericNode))
            return 1;
        return CSFieldPanel.BAD_GROUP_NUMBER;
	}

	
	
	
	/***************/
	class CodingSchemaFieldNode extends AbstractMutableTreeTableNode {
		protected boolean manualNode;
		protected boolean customNode;
		protected boolean gestureNode;

		public CodingSchemaFieldNode(Object[] data) {
			this(data, false, false, false);
		}

		public CodingSchemaFieldNode(Object[] data, boolean manualNode, boolean customNode, boolean gestureNode) {
			super(data);
			if (data == null) {
				throw new IllegalArgumentException("Node data cannot be null");
			}
			this.manualNode = manualNode;
			this.customNode = customNode;
			this.gestureNode = gestureNode;
		}
		
		public boolean isNonManualNode() {
			return !manualNode && !customNode && !gestureNode;
		}
		
		public boolean isManualNode() {
			return manualNode;
		}
		
		public boolean isCustomNode() {
			return customNode;
		}
		
		public boolean isGestureNode() {
			return gestureNode;
		}

		public DefaultTreeTableModel getFieldsModel() {
			DefaultTreeTableModel fieldsModel;
			if (manualNode) {
				fieldsModel = (DefaultTreeTableModel) glossFieldsTable.getTreeTableModel();
			} else if (customNode) {
				fieldsModel = (DefaultTreeTableModel) customFieldsTable.getTreeTableModel();
			} else if (gestureNode) {
				fieldsModel = (DefaultTreeTableModel) gestureFieldsTable.getTreeTableModel();
			} else {
				fieldsModel = (DefaultTreeTableModel) nonManualFieldsTable.getTreeTableModel();
			}
			
			return fieldsModel;
		}
		@Override
		public int getColumnCount() {
			return getData().length;
		}

		@Override
		public Object getValueAt(int column) {
			return getData()[column];
		}

		public Object[] getData() {
			return (Object[]) getUserObject();
		}

		public boolean isGroup() { return isGroup(this); }

		private boolean isGroup(TreeTableNode node) {
		    return node.getValueAt(ARRAY_INDEX_FIELD_ID) instanceof ArrayList;

//			DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) nonManualFieldsTable.getTreeTableModel();
//
//			TreeTableNode rootNode = fieldsModel.getRoot();
//			TreeTableNode headNode = rootNode.getChildAt(0);
//			TreeTableNode faceNode = rootNode.getChildAt(1);
//			TreeTableNode bodyNode = rootNode.getChildAt(2);
//			TreeTableNode gramNode = rootNode.getChildAt(3);
//
//			TreeTableNode manualNode = glossFieldsRootNode.getChildAt(0);
//
//			TreeTableNode customNode = customFieldsRootNode.getChildAt(0);
//
//			TreeTableNode gestureNode = gestureFieldsRootNode.getChildAt(0);
//			TreeTableNode domGestureCategoryNode = gestureFieldsRootNode.getChildAt(1);
//			TreeTableNode ndomGestureCategoryNode = gestureFieldsRootNode.getChildAt(2);
//
//			return node.equals(headNode) || node.equals(faceNode) || node.equals(bodyNode) || node.equals(gramNode)
//					|| node.equals(manualNode) || node.equals(customNode)
//					|| node.equals(gestureNode) || node.equals(domGestureCategoryNode)
//					|| node.equals(ndomGestureCategoryNode);
		}

		// Set value without invoking any custom functionality
		public void setValueAtSelf(Object value, int column) {
		    super.setValueAt(value, column);
		    getData()[column] = value;
        }

		@Override
		public void setValueAt(Object value, int column) {
			super.setValueAt(value, column);
			if (column == ARRAY_INDEX_UTTERANCE_VISIBLE
                || column == ARRAY_INDEX_SEGMENT_VISIBLE
                || column == ARRAY_INDEX_GLOBAL_VISIBLE) {
				Boolean isChecked = (Boolean) value;
				getData()[column] = isChecked;

				// for head, face, body, and grammatical nodes - select/unselect
				// all children if group is selected/unselected
				if (isGroup(this)) {
					int childCount = getChildCount();
					for (int i = 0; i < childCount; i++) {
						TreeTableNode node = getChildAt(i);
						if (node != null) {
						    var id = (String)node.getValueAt(ARRAY_INDEX_FIELD_ID);
						    if (!Constants.isStaticField(id))
                                node.setValueAt(value, column);
                        }
					}
				}
			}

			if (column != ARRAY_INDEX_UTTERANCE_VISIBLE
                && column != ARRAY_INDEX_SEGMENT_VISIBLE
                && column != ARRAY_INDEX_GLOBAL_VISIBLE) {
				getData()[column] = value;
				// TODO: not sure why just calling super.setValueAt(value, column) does not work here
			} else {
//				// Below logic is for checkbox column behavior only
				if (isLeaf()) {
					// For leaf nodes, the parent should be checked only if all children are
					var parent = (CodingSchemaFieldNode)getParent();
					var all = true;
					for (int i = 0; i < parent.getChildCount(); i++) {
						var child = parent.getChildAt(i);
						all = all && (boolean)child.getValueAt(column);
					}
					parent.setValueAtSelf(all, column);
				}
			}
		}
	}

    private LinkedHashMap<String, CodingSchemaFieldNode> getFieldNodes(SegmentGraphicRepresentation rep, Collection<SS3Field> allFields, LabelObject root) {
        var fields = new ArrayList<SS3Field>();
        for (var f : allFields)
            if (!Constants.isStaticField(f.getId()))
                fields.add(f);

        ArrayList<LabelObject> leaves = root.getAllChildren();
        var available = new HashSet<String>();
        var utteranceVisible = new HashSet<String>();
        var segmentVisible = new HashSet<String>();
        var globalVisible = new HashSet<String>();

        for (var obj : root.getAllChildren()) {
            var wrapper = obj.getWrapper();
            var id = obj.getFieldID();

            if (Constants.isStaticField(id))
                continue;

            if (obj.isUtterancePermanentlyVisible(utterance))
                utteranceVisible.add(id);

			if (obj.isPermanentlyVisibleAcrossSegmentTier())
			    segmentVisible.add(id);

			if (obj.isPermanentlyVisibleGlobally() || Constants.isStaticField(id))
			    globalVisible.add(id);

            if (wrapper.getField().getFieldSize() > 0)
                available.add(id);
        }

        var nodes = new LinkedHashMap<String, CodingSchemaFieldNode>();

//        var keys = fields.keySet().stream()
//            .sorted(Comparator.comparing(Integer::parseInt))
//            .collect(Collectors.toList());

        for (var field : fields) {
            String id = field.getId();

            Object[] values = new Object[ARRAY_SIZE];
            values[ARRAY_INDEX_FIELD_NAME] = field.getName();
            values[ARRAY_INDEX_FIELD_LABEL] = field.getLabel();
            values[ARRAY_INDEX_UTTERANCE_VISIBLE] = utteranceVisible.contains(id);
            values[ARRAY_INDEX_SEGMENT_VISIBLE] = segmentVisible.contains(id);
            values[ARRAY_INDEX_GLOBAL_VISIBLE] = globalVisible.contains(id);
            values[ARRAY_INDEX_ANNOTATED] = available.contains(id) ? "  *  " : "";
            values[ARRAY_INDEX_COLOR] = field.getColor();
            values[ARRAY_INDEX_PREVIEW] = false;
            values[ARRAY_INDEX_FIELD_ID] = id;

            // TODO: Fix this crap
            boolean isGloss = root == segment.getRootLabelObject().getGlossLabelObject();
            boolean isGesture = root == segment.getRootLabelObject().getGestureLabelObject();
            boolean isCustom = root == segment.getRootLabelObject().getCustomLabelObject();

            nodes.put(id, new CodingSchemaFieldNode(values, isGloss, isCustom, isGesture));
        }

        return nodes;
    }

    private void populateFields(CodingSchemaFieldNode rootNode, LabelObject rootLabel, List<FieldsGroup> groups) {
        var scheme = segmentPanel.getSS3Database().getSS3CodingScheme();
//        var fields = scheme.getDependentRootFields();
        List<SS3Field> values = new ArrayList<SS3Field>();
        values.addAll(scheme.getSortedFields());
        var nodes = getFieldNodes(rep, values, rootLabel);

        while (rootNode.getChildCount() != 0)
            rootNode.remove(0);

        for (var group : groups) {
            var children = group.getFieldIDs().stream()
                .filter(id -> !Constants.isStaticField(id))
                .map(nodes::get)
                .collect(Collectors.toList());

            Object[] value = new Object[ARRAY_SIZE];
            value[ARRAY_INDEX_FIELD_NAME] = group.getGroupName();
            value[ARRAY_INDEX_FIELD_LABEL] = "";
            value[ARRAY_INDEX_UTTERANCE_VISIBLE] = !children.isEmpty() && children.stream().allMatch(n -> (boolean)n.getValueAt(ARRAY_INDEX_UTTERANCE_VISIBLE));
            value[ARRAY_INDEX_SEGMENT_VISIBLE] = !children.isEmpty() && children.stream().allMatch(n -> (boolean)n.getValueAt(ARRAY_INDEX_SEGMENT_VISIBLE));
            value[ARRAY_INDEX_GLOBAL_VISIBLE] = !children.isEmpty() && children.stream().allMatch(n -> (boolean)n.getValueAt(ARRAY_INDEX_GLOBAL_VISIBLE));
            value[ARRAY_INDEX_ANNOTATED] = "";
            value[ARRAY_INDEX_COLOR] = Color.BLACK;
            value[ARRAY_INDEX_PREVIEW] = false;
            value[ARRAY_INDEX_FIELD_ID] = group.getFieldIDs();

            // TODO: Fix this crap
            boolean isGloss = rootLabel == segment.getRootLabelObject().getGlossLabelObject();
            boolean isGesture = rootLabel == segment.getRootLabelObject().getGestureLabelObject();
            boolean isCustom = rootLabel == segment.getRootLabelObject().getCustomLabelObject();

            CodingSchemaFieldNode groupNode = new CodingSchemaFieldNode(value, isGloss, isCustom, isGesture);

            children.forEach(groupNode::add);
            rootNode.add(groupNode);
        }

//        var keys = fields.keySet().stream()
//            .sorted(Comparator.comparing(Integer::parseInt))
//            .collect(Collectors.toList());
//
//        for (var id : keys) {
//            SS3Field childField = fields.get(id);
//            addChild(rootNode, childField);
//        }
    }

//	private void addChild(CodingSchemaFieldNode parent, SS3Field childField) {
//		CodingSchemaFieldNode child = createFieldNode(childField);
//		parent.add(child);
//		SS3CodingScheme dbScheme = segmentPanel.getSS3Database().getSS3CodingScheme();
//		ArrayList<SS3Field> depFields = dbScheme.getDependentFields(childField.getId());
//
//		for (int i = 0; i < depFields.size(); i++) {
//			SS3Field f = depFields.get(i);
//			addChild(child, f);
//		}
//	}

//	private CodingSchemaFieldNode createFieldNode(SS3Field field) {
//		LabelObject object = segment.getRootLabelObject().getGlossLabelObject();
//		ArrayList<LabelObject> list = object.getAllChildren();
//		ArrayList<String> available = new ArrayList<String>();
//		ArrayList<String> visible = new ArrayList<String>();
//
//		for (int i = 0; i < list.size(); i++) {
//			LabelObject obj = list.get(i);
//			FieldWrapper wrapper = obj.getWrapper();
//			String id = wrapper.getField().getFieldID();
//
//			if (obj.isVisible())
//				visible.add(id);
//
//			if (!available.contains(id) && wrapper.getField().getFieldSize() > 0)
//				available.add(id);
//		}
//
//		String fieldID = "" + field.getId();
//		boolean checked = visible.contains(fieldID) ? true : false;
//		boolean data = available.contains(fieldID);
//		String dataStr = data ? "*" : "";
//		boolean preview = false;
//		Object[] fieldNodeObj = new Object[ARRAY_SIZE];
//		fieldNodeObj[ARRAY_INDEX_FIELD_NAME] = field.getName();
//		fieldNodeObj[ARRAY_INDEX_FIELD_LABEL] = field.getLabel();
//		fieldNodeObj[ARRAY_INDEX_UTTERANCE_VISIBLE] = checked;
//        fieldNodeObj[ARRAY_INDEX_SEGMENT_VISIBLE] = checked;
//        fieldNodeObj[ARRAY_INDEX_GLOBAL_VISIBLE] = checked;
//		fieldNodeObj[ARRAY_INDEX_DATA] = dataStr;
//		fieldNodeObj[ARRAY_INDEX_COLOR] = field.getColor();
//		fieldNodeObj[ARRAY_INDEX_PREVIEW] = preview;
//		fieldNodeObj[ARRAY_INDEX_FIELD_ID] = field.getId();
//
//		return new CodingSchemaFieldNode(fieldNodeObj);
//	}

	public void setSelectedField(String fieldID, boolean selected) {
//		if (glossAndHandFieldIDs.contains(fieldID)) {
//			// don't change selected state of gloss/hand/translation fields in the table
//			return;
//		}
//
//		if (customPredefinedFieldIDs.contains(fieldID)) {
//			// don't change selected state of custom fields in the table
//			return;
//		}
//
//		if (gestureAndHandFieldIDs.contains(fieldID)) {
//			// don't change selected state of gesture/hand fields in the table
//			return;
//		}
//
//		DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) nmFieldsTable.getTreeTableModel();
//		TreeTableNode fieldsRootNode = fieldsModel.getRoot();
//
//		for (int groupNumber = 0; groupNumber < fieldsRootNode.getChildCount(); groupNumber++) {
//			MutableTreeTableNode groupNode = (MutableTreeTableNode) fieldsRootNode.getChildAt(groupNumber);
//			int childCount = groupNode.getChildCount();
//			for (int i = 0; i < childCount; i++) {
//				CodingSchemaFieldNode childNode = (CodingSchemaFieldNode) groupNode.getChildAt(i);
//				String nodeID = (String) childNode.getValueAt(ARRAY_INDEX_FIELD_ID);
//
//				if (nodeID.equals(fieldID)) {
//					// this will either check or uncheck the field, then refresh the panel to
//					// reflect the change
//					//childNode.setValueAt(selected, ARRAY_INDEX_PREVIEW);
//					childNode.setValueAt(selected, ARRAY_INDEX_UTTERANCE_VISIBLE);
//					return;
//				}
//			}
//		}
//
//		DefaultTreeTableModel customFieldsModel = (DefaultTreeTableModel) customFieldsTable.getTreeTableModel();
//		TreeTableNode customFieldsRootNode = customFieldsModel.getRoot();
//
//		for (int groupNumber = 0; groupNumber < customFieldsRootNode.getChildCount(); groupNumber++) {
//			MutableTreeTableNode groupNode = (MutableTreeTableNode) customFieldsRootNode.getChildAt(groupNumber);
//			int childCount = groupNode.getChildCount();
//			for (int i = 0; i < childCount; i++) {
//				CodingSchemaFieldNode childNode = (CodingSchemaFieldNode) groupNode.getChildAt(i);
//				String nodeID = (String) childNode.getValueAt(ARRAY_INDEX_FIELD_ID);
//
//				if (nodeID.equals(fieldID)) {
//					// this will either check or uncheck the field, then refresh the panel to
//					// reflect the change
//					//childNode.setValueAt(selected, ARRAY_INDEX_PREVIEW);
//					childNode.setValueAt(selected, ARRAY_INDEX_UTTERANCE_VISIBLE);
//					return;
//				}
//			}
//		}
//
//		DefaultTreeTableModel gestureFieldsModel = (DefaultTreeTableModel) gestureFieldsTable.getTreeTableModel();
//		TreeTableNode gestureFieldsRootNode = gestureFieldsModel.getRoot();
//
//		// GREG TODO: make a better way to get number of groups
//		for (int groupNumber = 0; groupNumber < gestureFieldsRootNode.getChildCount(); groupNumber++) {
//			MutableTreeTableNode groupNode = (MutableTreeTableNode) gestureFieldsRootNode.getChildAt(groupNumber);
//			int childCount = groupNode.getChildCount();
//			for (int i = 0; i < childCount; i++) {
//				CodingSchemaFieldNode childNode = (CodingSchemaFieldNode) groupNode.getChildAt(i);
//				String nodeID = (String) childNode.getValueAt(ARRAY_INDEX_FIELD_ID);
//
//				if (nodeID.equals(fieldID)) {
//					// this will either check or uncheck the field, then refresh the panel to
//					// reflect the change
//					//childNode.setValueAt(selected, ARRAY_INDEX_PREVIEW);
//					childNode.setValueAt(selected, ARRAY_INDEX_UTTERANCE_VISIBLE);
//					return;
//				}
//			}
//		}
//
//		TreeTableNode manualNode = glossFieldsRootNode.getChildAt(0);
//		for (int i = 0; i < manualNode.getChildCount(); i++) {
//			CodingSchemaFieldNode child = ((CodingSchemaFieldNode) manualNode.getChildAt(i));
//			String childFieldID = (String) child.getValueAt(ARRAY_INDEX_FIELD_ID);
//			if (fieldID.equalsIgnoreCase(childFieldID)) {
//				//child.setValueAt(selected, ARRAY_INDEX_PREVIEW);
//				child.setValueAt(selected, ARRAY_INDEX_UTTERANCE_VISIBLE);
//			}
//		}
	}

	public void scrollToAndSelectFieldByID(String fieldID) {
		if (fieldID == null || fieldID.isEmpty()) {
			scrollPane.getVerticalScrollBar().setValue(0);	// scroll to top
			return;
		}

		DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) nonManualFieldsTable.getTreeTableModel();
		TreeTableNode fieldsRootNode = fieldsModel.getRoot();

		for (int groupNumber = 0; groupNumber < fieldsRootNode.getChildCount(); groupNumber++) {
			MutableTreeTableNode groupNode = (MutableTreeTableNode) fieldsRootNode.getChildAt(groupNumber);
			int childCount = groupNode.getChildCount();
			for (int i = 0; i < childCount; i++) {
				CodingSchemaFieldNode childNode = (CodingSchemaFieldNode) groupNode.getChildAt(i);
				String nodeID = (String) childNode.getValueAt(ARRAY_INDEX_FIELD_ID);

				if (nodeID.equals(fieldID)) {
					scrollToAndSelectNode(childNode);
				}
			}
		}

		DefaultTreeTableModel customFieldsModel = (DefaultTreeTableModel) customFieldsTable.getTreeTableModel();
		TreeTableNode customFieldsRootNode = customFieldsModel.getRoot();

		for (int groupNumber = 0; groupNumber < customFieldsRootNode.getChildCount(); groupNumber++) {
			MutableTreeTableNode groupNode = (MutableTreeTableNode) customFieldsRootNode.getChildAt(groupNumber);
			int childCount = groupNode.getChildCount();
			for (int i = 0; i < childCount; i++) {
				CodingSchemaFieldNode childNode = (CodingSchemaFieldNode) groupNode.getChildAt(i);
				String nodeID = (String) childNode.getValueAt(ARRAY_INDEX_FIELD_ID);

				if (nodeID.equals(fieldID)) {
					scrollToAndSelectCustomNode(childNode);
				}
			}
		}
	}

	private void scrollToAndSelectNode(MutableTreeTableNode node) {
		DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) nonManualFieldsTable.getTreeTableModel();
		TreeNode[] nodes = fieldsModel.getPathToRoot(node);
		TreePath path = new TreePath(nodes);

		nonManualFieldsTable.expandPath(path.getParentPath());
		int row = nonManualFieldsTable.getRowForPath(path);
		nonManualFieldsTable.scrollRowToVisible(row);
		nonManualFieldsTable.scrollPathToVisible(path);
		nonManualFieldsTable.getTreeSelectionModel().setSelectionPath(path);
		setSelectedTreeTableNode((CodingSchemaFieldNode)path.getLastPathComponent());
	}

	private void scrollToAndSelectCustomNode(MutableTreeTableNode node) {
		DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) customFieldsTable.getTreeTableModel();
		TreeNode[] nodes = fieldsModel.getPathToRoot(node);
		TreePath path = new TreePath(nodes);

		customFieldsTable.expandPath(path.getParentPath());
		int row = customFieldsTable.getRowForPath(path);
		customFieldsTable.scrollRowToVisible(row);
		customFieldsTable.scrollPathToVisible(path);
        customFieldsTable.getTreeSelectionModel().setSelectionPath(path);
        setSelectedTreeTableNode((CodingSchemaFieldNode)path.getLastPathComponent());
	}

	public HashMap<String, Color> getSelectedColors() {
		HashMap<String, Color> fieldColors = new HashMap<String, Color>();
		DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) nonManualFieldsTable.getTreeTableModel();
		TreeTableNode fieldsRootNode = fieldsModel.getRoot();

		for (int groupNumber = 0; groupNumber < fieldsRootNode.getChildCount(); groupNumber++) {
			MutableTreeTableNode groupNode = (MutableTreeTableNode) fieldsRootNode.getChildAt(groupNumber);
			int childCount = groupNode.getChildCount();
			for (int i = 0; i < childCount; i++) {
				MutableTreeTableNode childNode = (MutableTreeTableNode) groupNode.getChildAt(i);

				String fieldID = (String) childNode.getValueAt(ARRAY_INDEX_FIELD_ID);
				Color color = (Color) childNode.getValueAt(ARRAY_INDEX_COLOR);
				fieldColors.put(fieldID, color);
			}
		}

		return fieldColors;
	}

	public ArrayList<String> getSelectedFieldIDsNUpdateCodingSchema() {
		ArrayList<String> visibleFieldIDs = new ArrayList<String>();

		DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) nonManualFieldsTable.getTreeTableModel();
		TreeTableNode fieldsRootNode = fieldsModel.getRoot();

		for (int groupNumber = 0; groupNumber < fieldsRootNode.getChildCount(); groupNumber++) {
			MutableTreeTableNode groupNode = (MutableTreeTableNode) fieldsRootNode.getChildAt(groupNumber);
			int childCount = groupNode.getChildCount();
			for (int i = 0; i < childCount; i++) {
				MutableTreeTableNode childNode = (MutableTreeTableNode) groupNode.getChildAt(i);
				Boolean display = (Boolean) childNode.getValueAt(ARRAY_INDEX_UTTERANCE_VISIBLE);

				if (display.booleanValue()) {
					visibleFieldIDs.add((String) childNode.getValueAt(ARRAY_INDEX_FIELD_ID));
				}
			}
		}

		// always set gloss fields to visible
		getSelectedGlossFldIDs(4, visibleFieldIDs);
		getSelectedGlossFldIDs(5, visibleFieldIDs);
		getSelectedHandFields(6, visibleFieldIDs);
		getSelectedHandFields(7, visibleFieldIDs);

		TreeTableNode manualGroupNode = glossFieldsRootNode.getChildAt(0);
		for (int i = 0; i < manualGroupNode.getChildCount(); i++) {
			TreeTableNode fieldNode = manualGroupNode.getChildAt(i);
			String fieldID = (String) fieldNode.getValueAt(ARRAY_INDEX_FIELD_ID);
			visibleFieldIDs.remove(fieldID);
			if ((Boolean) fieldNode.getValueAt(ARRAY_INDEX_UTTERANCE_VISIBLE) && !visibleFieldIDs.contains(fieldID)) {
				visibleFieldIDs.add(fieldID);
			}
		}

		return visibleFieldIDs;
	}

	private void getSelectedGlossFldIDs(int index, ArrayList<String> ids) {
		MutableTreeTableNode node = (MutableTreeTableNode) nonManualFieldsRootNode.getChildAt(index);
		Object obj = node.getValueAt(ARRAY_INDEX_UTTERANCE_VISIBLE);
		Boolean display = (Boolean) obj;
		if (display.booleanValue()) {
			ids.add((String) node.getValueAt(ARRAY_INDEX_FIELD_ID));
			int childCount = node.getChildCount();
			for (int count = 0; count < childCount; count++) {
				MutableTreeTableNode cNode = (MutableTreeTableNode) node.getChildAt(count);
				Boolean cDisplay = (Boolean) cNode.getValueAt(ARRAY_INDEX_UTTERANCE_VISIBLE);

				if (cDisplay.booleanValue()) {
					ids.add((String) cNode.getValueAt(ARRAY_INDEX_FIELD_ID));
				}
			}
		}
	}

	private void getSelectedHandDependentFieldIDs(MutableTreeTableNode node, ArrayList<String> ids) {
		int childCount = node.getChildCount();
		for (int count = 0; count < childCount; count++) {
			MutableTreeTableNode cNode = (MutableTreeTableNode) node.getChildAt(count);
			Boolean cDisplay = (Boolean) cNode.getValueAt(ARRAY_INDEX_UTTERANCE_VISIBLE);

			if (cDisplay.booleanValue()) {
				ids.add((String) cNode.getValueAt(ARRAY_INDEX_FIELD_ID));
				if (cNode.getChildCount() > 0) {
					getSelectedHandDependentFieldIDs(cNode, ids);
				}
			}
		}
	}

	private void getSelectedHandFields(int index, ArrayList<String> ids) {
		MutableTreeTableNode node = (MutableTreeTableNode) nonManualFieldsRootNode.getChildAt(index);
		Object obj = node.getValueAt(ARRAY_INDEX_UTTERANCE_VISIBLE);
		Boolean display = (Boolean) obj;
		if (display.booleanValue()) {
			ids.add((String) node.getValueAt(ARRAY_INDEX_FIELD_ID));
			int childCount = node.getChildCount();
			for (int count = 0; count < childCount; count++) {
				MutableTreeTableNode cNode = (MutableTreeTableNode) node.getChildAt(count);
				Boolean cDisplay = (Boolean) cNode.getValueAt(ARRAY_INDEX_UTTERANCE_VISIBLE);

				if (cDisplay.booleanValue()) {
					ids.add((String) cNode.getValueAt(ARRAY_INDEX_FIELD_ID));
					getSelectedHandDependentFieldIDs(cNode, ids);
				}
			}
		}
	}

    private Stream<TreeTableNode> stream(JXTreeTable table) {
	    var nodes = new ArrayList<TreeTableNode>();
        var root = (TreeTableNode)table.getTreeTableModel().getRoot();
        for (int i = 0; i < root.getChildCount(); i++) {
            var group = root.getChildAt(i);		// GREG TODO: might not be group here
            for (int j = 0; j < group.getChildCount(); j++) {
                nodes.add(group.getChildAt(j));
            }
        }
        return nodes.stream();
    }
//
//    // Iterate through
//
//    public void select(String id, boolean selected) {
//        if (glossAndHandFieldIDs.contains(id)) {
//            // don't change selected state of gloss/hand/translation fields in the table
//            return;
//        }
//        
//        if (gestureAndHandFieldIDs.contains(id)) {
//            // don't change selected state of gesture/hand fields in the table
//        	return;
//        }
//
//        Consumer<TreeTableNode> selectIfMatching = node -> {
//            if (node.getValueAt(ARRAY_INDEX_FIELD_ID).equals(id))
//                node.setValueAt(selected, 2);
//        };
//
//        iterTable(nmFieldsTable, selectIfMatching);
//        iterTable(manualFieldsTable, selectIfMatching);
//        iterTable(gestureFieldsTable, selectIfMatching);
//    }

	// Discard any changes made in the dialog by re-populating the tree with the underlying field values
    // TODO: This is really dumb. The dialog should just be recreated.
	public void discard() {
//	    initUI();
        var root = segment.getRootLabelObject();
        var scheme = segmentPanel.getSS3Database().getSS3CodingScheme();

        populateFields(nonManualFieldsRootNode, root.getNonManuelLabelObject(), scheme.loadNonManualGroups());
        ((DefaultTreeTableModel)nonManualFieldsTable.getTreeTableModel()).setRoot(nonManualFieldsRootNode);

        populateFields(glossFieldsRootNode, root.getGlossLabelObject(), scheme.loadGlossFieldGroups());
        ((DefaultTreeTableModel)glossFieldsTable.getTreeTableModel()).setRoot(glossFieldsRootNode);

        populateFields(gestureFieldsRootNode, root.getGestureLabelObject(), scheme.loadGestureFieldGroups());
        ((DefaultTreeTableModel)gestureFieldsTable.getTreeTableModel()).setRoot(gestureFieldsRootNode);

        populateFields(customFieldsRootNode, root.getCustomLabelObject(), scheme.loadCustomFieldGroups());
        ((DefaultTreeTableModel)customFieldsTable.getTreeTableModel()).setRoot(customFieldsRootNode);

        glossFieldsTable.expandAll();
        gestureFieldsTable.expandAll();
        customFieldsTable.expandAll();
	}

	// Apply any changes made in the dialog to the underlying field values
    public void apply() {
	    SS3CodingScheme dbScheme = segmentPanel.getSS3Database().getSS3CodingScheme();

        BiConsumer<TreeTableNode, LabelObject> updateVisibility = (node, child) -> {
            if ((boolean)node.getValueAt(ARRAY_INDEX_UTTERANCE_VISIBLE))
                child.setUtterancePermanentlyVisible(utterance);
            else
                child.removeUtterancePermanentlyVisible(utterance);

            child.setPermanentlyVisibleAcrossSegmentTier((boolean)node.getValueAt(ARRAY_INDEX_SEGMENT_VISIBLE));
            child.setPermanentlyVisibleGlobally((boolean)node.getValueAt(ARRAY_INDEX_GLOBAL_VISIBLE));
        };

        BiConsumer<TreeTableNode, LabelObject> updateColor = (node, child) -> {
            var id = child.getFieldID();
            var color = (Color)node.getValueAt(ARRAY_INDEX_COLOR);

            child.getWrapper().getField().setColor(color);

            var currentField = dbScheme.getField(id);
            currentField.setColor(color);
        };

        var nonManualLabels = segment.getRootLabelObject().getNonManuelLabelObject();
        stream(nonManualFieldsTable).forEach(node -> {
            for (var child : nonManualLabels.getAllChildren()) {
                if (child.getFieldID().equals(node.getValueAt(ARRAY_INDEX_FIELD_ID))) {
                    updateVisibility.accept(node, child);
                    updateColor.accept(node, child);
                }
            }
        });

        var manualLabels = segment.getRootLabelObject().getGlossLabelObject();
        stream(glossFieldsTable).forEach(node -> {
            for (var child : manualLabels.getAllChildren()) {
                if (child.getFieldID().equals(node.getValueAt(ARRAY_INDEX_FIELD_ID))) {
                    updateVisibility.accept(node, child);
                    updateColor.accept(node, child);
                }
            }
        });

        var customLabels = segment.getRootLabelObject().getCustomLabelObject();
        stream(customFieldsTable).forEach(node -> {
            for (var child : customLabels.getAllChildren()) {
                if (child.getFieldID().equals(node.getValueAt(ARRAY_INDEX_FIELD_ID))) {
                    updateVisibility.accept(node, child);
                    updateColor.accept(node, child);
                }
            }
        });

        var gestureLabels = segment.getRootLabelObject().getGestureLabelObject();
        stream(gestureFieldsTable).forEach(node -> {
            for (var child : gestureLabels.getAllChildren()) {
                if (child.getFieldID().equals(node.getValueAt(ARRAY_INDEX_FIELD_ID))) {
                    updateVisibility.accept(node, child);
                    updateColor.accept(node, child);
                }
            }
        });

        syncFieldPanels();

        for (var template : segmentPanel.getUtteranceView().getSegmentTemplates()) {
            template.getNonmanView()
                .getCSFieldDialog()
                .getCSFieldPanel()
                .showAllFieldsCurrentUtterance();
        }

        segmentPanel.updateLoadedDatabase();

        graphFieldDialog.enter();
    }

    public void redraw() {
        nonManualView.update(segment);
        glossNRelatedView.update(segment);
        customView.update(segment);
        gestureView.update(segment);
        SS3Singleton.getMediaToolBar().deselectLabel();

        nonManualView.getLabelView().loadWrappers();
        customView.getLabelView().loadWrappers();

        nonManualView.getLabelView().updateUI();
        glossNRelatedView.getLableView().updateUI();
        customView.getLabelView().updateUI();
        gestureView.getLableView().updateUI();

        nonManualView.getLabelView().repaint();
        glossNRelatedView.getLableView().repaint();
        customView.getLabelView().repaint();
        gestureView.getLableView().repaint();

//        glossNRelatedView.setSelectedField(null);
//        gestureView.setSelectedField(null);
    }

    private void syncFieldPanels() {
        var nodes = this.segment.getRootLabelObject().getAllLabelObjects().stream()
            .collect(Collectors.toMap(
                o -> o.getFieldID(),
                o -> o
            ));

        for (var template : segmentPanel.getUtteranceView().getSegmentTemplates()) {
            var panel = template.getNonmanView()
                .getCSFieldDialog()
                .getCSFieldPanel();

            if (panel == this)
                continue;

            panel.getSegment().getRootLabelObject().getAllLabelObjects()
                .forEach(o -> {
                    var self = nodes.get(o.getFieldID());
                    if (self != null)
                        o.setPermanentlyVisibleGlobally(self.isPermanentlyVisibleGlobally());
                });
        }

        for (var template : segmentPanel.getUtteranceView().getSegmentTemplates()) {
            template.getNonmanView()
                .getCSFieldDialog()
                .getCSFieldPanel()
                .discard();
        }
    }

	public Segment getSegment() {
		return segment;
	}

	private void setVisible(LabelObject obj, boolean visible) {
	    var id = obj.getFieldID();
	    if (Constants.isStaticField(id))
	        return;

        obj.setVisible(visible);
        if (obj.getWrapper() != null)
            obj.getWrapper().setVisible(visible);

        if (visible && obj.getParent() != null)
            setVisible(obj.getParent(), true);
    }
	
	private void clearTimeTuples() {
		timeTuples.clear();
	}
	
	private void addToTimeTuples(Time start, Time end) {
		timeTuples.put(start, end);
		// TODO: as times are added, transform many smaller ranges, if intersecting, into larger unified ranges
	}
	
	public void showAllFieldsCurrentUtterance() {
		discard();
		clearTimeTuples();
		
		ArrayList<SS3GlossWindowItem> utterances = new ArrayList<SS3GlossWindowItem>();

		if (utterance != null) {
            addToTimeTuples(utterance.getStartTimeInfo(), utterance.getEndTimeInfo());
            utterances.add(utterance);
        } else if (SegmentGraphReprUtil.visibleStart != null && SegmentGraphReprUtil.visibleEnd != null) {
		    addToTimeTuples(SegmentGraphReprUtil.visibleStart, SegmentGraphReprUtil.visibleEnd);
        }

		refreshVisibleFieldsHelper(utterances, false);
	}

	private void refreshVisibleFields(boolean onlyAnnotated) {
        discard();
        clearTimeTuples();
        
		ArrayList<SS3GlossWindowItem> utterances = new ArrayList<SS3GlossWindowItem>();

        // set time tuples to entire visible range, plus include all (even adjacently) visible utterances
		var visibleStart = SegmentGraphReprUtil.visibleStart;
		var visibleEnd = SegmentGraphReprUtil.visibleEnd;
		if (visibleStart != null && visibleEnd != null) {
		    addToTimeTuples(visibleStart, visibleEnd);

		    for (var utterance : segment.getSS3GlossWindowItems()) {
		        if (visibleStart.to(visibleEnd).overlaps(utterance.interval(), Bound.INCLUSIVE)) {
                    addToTimeTuples(utterance.getStartTimeInfo(), utterance.getEndTimeInfo());
                    utterances.add(utterance);
                }
            }
        }

		refreshVisibleFieldsHelper(utterances, onlyAnnotated);
    }
    public void refreshVisibleFields() { refreshVisibleFields(false); }
    public void refreshVisibleAnnotatedFields() { refreshVisibleFields(true); }

	private void refreshVisibleFieldsHelper(ArrayList<SS3GlossWindowItem> utterances, boolean onlyAnnotated) {
		RootLabelObject root = getSegment().getRootLabelObject();
		LabelObject nonManualLabelObject = root.getNonManuelLabelObject();
		LabelObject glossLabelObject = root.getGlossLabelObject();
		LabelObject gestureLabelObject = root.getGestureLabelObject();
		LabelObject customLabelObject = root.getCustomLabelObject();

        var subs = new ArrayList<LabelObject>();
        subs.add(nonManualLabelObject);
        subs.add(glossLabelObject);
        subs.add(customLabelObject);
        subs.add(gestureLabelObject);

        // Hide all fields
        for (var sub : subs) {
            for (var obj : sub.getAllChildren()) {
                setVisible(obj, false);
                if (obj.isLeaf())
                    obj.getWrapper().setSelectedNHighlighted(false);
            }
        }
        
        if (onlyAnnotated || segmentPanel.displayAnnotated()) {
            // Show all non empty fields
    		ArrayList<LabelObject> nonEmptyFieldLabels = nonManualLabelObject.getNonEmptyObjectsAcrossTimeTuples(timeTuples);
    		nonEmptyFieldLabels.addAll(glossLabelObject.getNonEmptyObjectsAcrossTimeTuples(timeTuples));
    		nonEmptyFieldLabels.addAll(gestureLabelObject.getNonEmptyObjectsAcrossTimeTuples(timeTuples));
    		nonEmptyFieldLabels.addAll(customLabelObject.getNonEmptyObjectsAcrossTimeTuples(timeTuples));
    		for (var field : nonEmptyFieldLabels)
    		    setVisible(field, true);        	
        }
        
        if (!onlyAnnotated && segmentPanel.displaySelected()) {
    		// Show all permanently shown fields
    		for (var sub : subs) {
                for (var obj : sub.getAllChildren()) {
                    if (obj.isPermanentlyVisibleAcrossSegmentTier()
                            || obj.isPermanentlyVisibleGlobally()) {
                        setVisible(obj, true);
                    }
                    
                    for (SS3GlossWindowItem utterance: utterances) {
                    	if (obj.isUtterancePermanentlyVisible(utterance)) {
                    		setVisible(obj, true);
                    	}
                    }
                }
            }
        }
		
		LabelObject gphLabelObject = graphFieldDialog.getSegment().getRootLabelObject().getGraphLabelObject();
		List<LabelObject> gphFields = gphLabelObject.getAllChildren();
		
		for (LabelObject fieldLabel : gphFields) {
			fieldLabel.getWrapper().setSelectedNHighlighted(false);
			graphFieldDialog.setSelectedField(fieldLabel.getFieldID(), fieldLabel.isVisible());
		}

		redraw();
	}

    private void importTextGrid() {
	    new OpenTextGridDialog(this).display(this);
    }
}

class GestureSelectionListener implements TreeSelectionListener {
    private CSFieldPanel panel = null;
    private SS3CodingScheme scheme = null;

    public GestureSelectionListener(CSFieldPanel panel, SS3CodingScheme scheme, SignStreamSegmentPanel segmentPanel) {
        this.panel = panel;
		this.scheme = scheme;
	}
	
	@Override
	public void valueChanged(TreeSelectionEvent se) {
		JTree tree = (JTree) se.getSource();
		MutableTreeTableNode selectedNode = (MutableTreeTableNode) tree.getLastSelectedPathComponent();

		if (selectedNode == null) {
			return;
		}

		panel.setSelectedTreeTableNode((CodingSchemaFieldNode) selectedNode);
		
		if (selectedNode.isLeaf()) {
			String fieldID = (String) selectedNode.getValueAt(panel.ARRAY_INDEX_FIELD_ID);
			SS3Field field = scheme.getField(fieldID);

			panel.setSelectedNodeID(fieldID);
			
			Color selectedColor = (Color) selectedNode.getValueAt(panel.ARRAY_INDEX_COLOR);
			Segment segment = panel.getSegment();
			field.setColor(selectedColor);

			LabelObject object = segment.getRootLabelObject().getGestureLabelObject();
			ArrayList<LabelObject> leafsOnly = object.getAllChildren();

			for (int i = 0; i < leafsOnly.size(); i++) {
				LabelObject obj = leafsOnly.get(i);
				FieldWrapper wrapper = obj.getWrapper();
				String id = wrapper.getField().getFieldID();
				if (id.equalsIgnoreCase(fieldID)) {
					wrapper.getField().setColor(selectedColor);
				}
			}

//			LabelObject gObject = segment.getRootLabelObject().getGlossNRelatedLabelObject();
//			ArrayList<LabelObject> gLeafsOnly = gObject.getAllChildren();
//
//			for (int i = 0; i < gLeafsOnly.size(); i++) {
//				LabelObject obj = gLeafsOnly.get(i);
//				FieldWrapper wrapper = obj.getWrapper();
//				String id = wrapper.getField().getFieldID();
//				if (id.equalsIgnoreCase(fieldID)) {
//					wrapper.getField().setColor(selectedColor);
//				}
//			}

			if (selectedNode.isLeaf())
				selectedNode = (MutableTreeTableNode) selectedNode.getParent();
		}
	}
}

class CustomSelectionListener implements TreeSelectionListener {
    private CSFieldPanel panel = null;
    private SS3CodingScheme scheme = null;

    public CustomSelectionListener(CSFieldPanel panel, SS3CodingScheme scheme, SignStreamSegmentPanel segmentPanel) {
        this.panel = panel;
		this.scheme = scheme;
	}
	
	@Override
	public void valueChanged(TreeSelectionEvent se) {
		JTree tree = (JTree) se.getSource();
		MutableTreeTableNode selectedNode = (MutableTreeTableNode) tree.getLastSelectedPathComponent();

		if (selectedNode == null) {
			return;
		}
		
		panel.setSelectedTreeTableNode((CodingSchemaFieldNode) selectedNode);

		var isGroup = ((CodingSchemaFieldNode)selectedNode).isGroup();
		
		if (!isGroup) {
			String fieldID = (String) selectedNode.getValueAt(panel.ARRAY_INDEX_FIELD_ID);
			SS3Field field = scheme.getField(fieldID);

			panel.setSelectedNodeID(fieldID);
			
			Color selectedColor = (Color) selectedNode.getValueAt(panel.ARRAY_INDEX_COLOR);
			Segment segment = panel.getSegment();
			field.setColor(selectedColor);

			LabelObject object = segment.getRootLabelObject().getCustomLabelObject();
			ArrayList<LabelObject> leafsOnly = object.getAllChildren();

			for (int i = 0; i < leafsOnly.size(); i++) {
				LabelObject obj = leafsOnly.get(i);
				FieldWrapper wrapper = obj.getWrapper();
				String id = wrapper.getField().getFieldID();
				if (id.equalsIgnoreCase(fieldID)) {
					wrapper.getField().setColor(selectedColor);
				}
			}

//			LabelObject cObject = segment.getRootLabelObject().getCustomLabelObject();
//			ArrayList<LabelObject> gLeafsOnly = cObject.getAllChildren();
//
//			for (int i = 0; i < gLeafsOnly.size(); i++) {
//				LabelObject obj = gLeafsOnly.get(i);
//				FieldWrapper wrapper = obj.getWrapper();
//				String id = wrapper.getField().getFieldID();
//				if (id.equalsIgnoreCase(fieldID)) {
//					wrapper.getField().setColor(selectedColor);
//				}
//			}

			if (!isGroup)
				selectedNode = (MutableTreeTableNode) selectedNode.getParent();
		}
	}
}
	
class manualSelectionListener implements TreeSelectionListener {
	private CSFieldPanel panel = null;
	private SS3CodingScheme scheme = null;

	public manualSelectionListener(CSFieldPanel panel, SS3CodingScheme scheme, SignStreamSegmentPanel segmentPanel) {
		this.panel = panel;
		this.scheme = scheme;
	}
	
	@Override
	public void valueChanged(TreeSelectionEvent se) {
		JTree tree = (JTree) se.getSource();
		TreeTableNode selectedNode = (TreeTableNode) tree.getLastSelectedPathComponent();
		if (selectedNode == null)
		    return;

		panel.setSelectedTreeTableNode((CodingSchemaFieldNode) selectedNode);
	}
}

class SelectionListener implements TreeSelectionListener {
	private CSFieldPanel panel = null;
	private SS3CodingScheme scheme = null;

	public SelectionListener(CSFieldPanel panel, SS3CodingScheme scheme, SignStreamSegmentPanel segmentPanel) {
		this.panel = panel;
		this.scheme = scheme;
	}
	
	@Override
	public void valueChanged(TreeSelectionEvent se) {
		JTree tree = (JTree) se.getSource();
		edu.bu.signstream.ui.panels.codingScheme.CSFieldPanel.CodingSchemaFieldNode selectedNode 
		= (CodingSchemaFieldNode) tree.getLastSelectedPathComponent();
		
		if (selectedNode == null) {
			return;
		}
		
		panel.setSelectedTreeTableNode(selectedNode);
		
		if (selectedNode.isLeaf()) {
			String fieldID = (String) selectedNode.getValueAt(panel.ARRAY_INDEX_FIELD_ID);
			SS3Field field = scheme.getField(fieldID);

			panel.setSelectedNodeID(fieldID);
			
			Color selectedColor = (Color) selectedNode.getValueAt(panel.ARRAY_INDEX_COLOR);
			Segment segment = panel.getSegment();
			field.setColor(selectedColor);

			LabelObject object = segment.getRootLabelObject().getNonManuelLabelObject();
			ArrayList<LabelObject> leafsOnly = object.getAllChildren();

			for (int i = 0; i < leafsOnly.size(); i++) {
				LabelObject obj = leafsOnly.get(i);
				FieldWrapper wrapper = obj.getWrapper();
				String id = wrapper.getField().getFieldID();
				if (id.equalsIgnoreCase(fieldID)) {
					wrapper.getField().setColor(selectedColor);
				}
			}

			LabelObject gObject = segment.getRootLabelObject().getGlossLabelObject();
			ArrayList<LabelObject> gLeafsOnly = gObject.getAllChildren();

			for (int i = 0; i < gLeafsOnly.size(); i++) {
				LabelObject obj = gLeafsOnly.get(i);
				FieldWrapper wrapper = obj.getWrapper();
				String id = wrapper.getField().getFieldID();
				if (id.equalsIgnoreCase(fieldID)) {
					wrapper.getField().setColor(selectedColor);
				}
			}

			if (selectedNode.isLeaf())
				selectedNode = (CodingSchemaFieldNode) selectedNode.getParent();
		}
	}
}
