package edu.bu.signstream.ui.panels.newDatabase;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import edu.bu.signstream.common.db.ss3.SS3Writer;
import edu.bu.signstream.common.db.ss4.SS4Writer;
import edu.bu.signstream.common.graph.Graph;
import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.signbank.SignBankResourceCollection;
import edu.bu.signstream.common.util.vo.Excerpt;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Database;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindow;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowSegment;
import edu.bu.signstream.common.util.vo.ss3.db.SS3TemporalPartition;
import edu.bu.signstream.common.util.vo.ss3.media.SS3MediaFile;
import edu.bu.signstream.common.util.vo.ss3.media.SS3MediaLibrary;
import edu.bu.signstream.common.util.vo.ss3.participants.ParticipantsLibrary;
import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.time.TimeRange;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.media.Audio;
import edu.bu.signstream.media.Video;
import edu.bu.signstream.media.fileNavigation.LoadedMediaFile;
import edu.bu.signstream.navigation.DbNavigationPanel;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.ui.ButtonPanel;
import edu.bu.signstream.ui.DirectoryLookupScrolledView;
import edu.bu.signstream.ui.SS3SignStreamApplication;
import edu.bu.signstream.ui.panels.AnnotatorSelectionDialog;
import edu.bu.signstream.ui.panels.AnnotatorsDialog;

public class CreateUpdateMacroUnit extends JPanel implements ActionListener {

	private static final long serialVersionUID = 1L;

	private ParticipantsLibrary participantsLibrary = new ParticipantsLibrary();
	private ParticipantsLibrary annotatorsLibrary = new ParticipantsLibrary();

	private SignStreamSegmentPanel segmentPanel = null;
	private ArrayList<LoadedMediaFile> mFiles = new ArrayList<LoadedMediaFile>();
    private ArrayList<LoadedMediaFile> mAudioFiles = new ArrayList<LoadedMediaFile>();
	private HashMap<String, Graph> graphs = new HashMap<>();
	private ArrayList<ParticipantView> prtView = new ArrayList<ParticipantView>();
	private ArrayList<ParticipantView> antView = new ArrayList<ParticipantView>();
	
	private ParticipantsPaneScrolledView participantsPaneScrolledView = null;
	private ParticipantsPaneScrolledView annotatorsPaneScrolledView = null;
	
	private AnnotatorsDialog participantsDialog = null;
	private AnnotatorsDialog annotatorsDialog = null;
	
	private SignBankResourceCollectionView signBankResourceCollectionView = null;

	private SS3MediaLibrary mLibrary = null;

	private JTextField macroUnitTxt = new JTextField(29);

	private JButton cancelBtn = new JButton("Cancel");
	private JButton submitBtn = new JButton("");
	private JButton lookupBtn = new JButton("Lookup");

	private final int CANCEL = 1;
	private final int SUBMIT = 2;
	private final int LOOKUP = 3;

	public static final int FUNCTION_ADD = 10;
	public static final int FUNCTION_UPDATE = 11;
	public static final int FUNCTION_DELETE = 12;

	private int function = FUNCTION_ADD;

	private VideoPanelScrolledView videoPanel;
	private final AudioPanelScrolledView audioPanel;
	private JFrame frame = null;
	private JDialog parentDialog = null;

	private SS3Database db = null;
	
	private boolean didMakeChanges = false;

	public CreateUpdateMacroUnit(int function, SS3Database db,
			Excerpt excerptToUpdate, ArrayList<LoadedMediaFile> mFiles, ArrayList<LoadedMediaFile> mAudioFiles,
			SS3MediaLibrary mLibrary, ParticipantsLibrary participantsLibrary,
			ParticipantsLibrary annotatorsLibrary,
			SignBankResourceCollection resourceCollection,
			SignStreamSegmentPanel segmentPanel, JDialog parentDialog,
			JFrame frame) {

		this.mFiles = mFiles;
		this.mAudioFiles = mAudioFiles;
		this.mLibrary = mLibrary;
		this.participantsLibrary = participantsLibrary;
		this.annotatorsLibrary = annotatorsLibrary;
		this.segmentPanel = segmentPanel;
		this.parentDialog = parentDialog;
		this.frame = frame;
		this.function = function;
		this.db = db;
		this.function = function;

		if (this.db == null)
			this.db = new SS3Database();

		videoPanel = new VideoPanelScrolledView(mFiles, lookupBtn,
				new Dimension(396, 135), frame);

		audioPanel = new AudioPanelScrolledView(mAudioFiles, new Dimension(396, 135), frame);
		
		signBankResourceCollectionView = new SignBankResourceCollectionView(resourceCollection);

		switch (function) {
		case FUNCTION_ADD:
			submitBtn.setText("Confirm Selections");
			submitBtn.setActionCommand("" + SUBMIT);
			submitBtn.addActionListener(this);
			cancelBtn.setText("Cancel");
			cancelBtn.setActionCommand("" + CANCEL);
			cancelBtn.addActionListener(this);

			macroUnitTxt.setText(getDefaultMacroUnitLabel());

			break;
		case FUNCTION_UPDATE:
			submitBtn.setText("Confirm Selections");
			submitBtn.setActionCommand("" + SUBMIT);
			submitBtn.addActionListener(this);
			cancelBtn.setText("Cancel");
			cancelBtn.setActionCommand("" + CANCEL);
			cancelBtn.addActionListener(this);

			macroUnitTxt.setText(db.getExcerpt());

			ArrayList<SS3TemporalPartition> tPartitions = db
					.getSS3GlossWindow().getTemporalPartitions();

			ArrayList<SS3Participant> selectedParticipants = db
					.getSelectedParticipants();
			ArrayList<SS3Participant> selectedAnnotators = db
					.getSelectedAnnotators();

			for (SS3Participant participant : selectedParticipants) {
				boolean isAssociatedParticipant = false;

				for (SS3TemporalPartition partition : tPartitions) {
					ArrayList<SS3GlossWindowSegment> segments = partition
							.getTemporalPartitionSegments();
					for (SS3GlossWindowSegment segment : segments) {
						SS3Participant segmentParticipant = segment.getParticipant();
						if (segmentParticipant != null && participant.getId().equals(segmentParticipant.getId())) {
							isAssociatedParticipant = true;
						}
					}
				}

				participant.setSelected(true);
				
				
				ParticipantView pv = new ParticipantView(participant, true,
						isAssociatedParticipant, this.participantsLibrary,
						this.annotatorsLibrary, prtView);

				prtView.add(pv);
				participant.setSelected(true);
			}

			for (SS3Participant annotator : selectedAnnotators) {
				ParticipantView pv = new ParticipantView(annotator, false,
						false, this.participantsLibrary,
						this.annotatorsLibrary, antView);
				antView.add(pv);
			}

			for (var g : db.getGraphs())
			    graphs.put(g.id(), g);

			break;
		case FUNCTION_DELETE:
			submitBtn.setText("Delete");
			break;
		}

			lookupBtn.setActionCommand("" + LOOKUP);
			lookupBtn.addActionListener(this);

			JLabel macroUnitLbl = new JLabel("Macro Unit: Label");
			UserInterfaceUtil.setLabelLNF(macroUnitLbl);
			UserInterfaceUtil.setTextFieldLNF(macroUnitTxt);

			Dimension prtD = new Dimension(400, 91);
			
			participantsPaneScrolledView = new ParticipantsPaneScrolledView(prtView,
					participantsLibrary, db, "Participants", prtD, true);
 
			annotatorsPaneScrolledView = new ParticipantsPaneScrolledView(antView,
					annotatorsLibrary, db, "Annotators", prtD, false);

			participantsDialog = participantsPaneScrolledView.getAnnotatorsDialog();
			annotatorsDialog = annotatorsPaneScrolledView.getAnnotatorsDialog();
			
			JPanel centerPanel = new JPanel();
			centerPanel.setLayout(new BoxLayout(centerPanel, BoxLayout.Y_AXIS));
			
			JPanel labelPanel = new JPanel();
			labelPanel.setLayout(new BoxLayout(labelPanel, BoxLayout.X_AXIS));
			labelPanel.add(macroUnitLbl);
			labelPanel.add(macroUnitTxt);
			
			centerPanel.add(labelPanel);
			
			lookupBtn.setAlignmentX(Component.RIGHT_ALIGNMENT);
			// TODO: Re-add lookup label when it is fixed
//			centerPanel.add(lookupBtn);
			
			centerPanel.add(videoPanel);
//			centerPanel.add(audioPanel);
			
			centerPanel.add(participantsPaneScrolledView);
			centerPanel.add(annotatorsPaneScrolledView);

			centerPanel.setBorder(BorderFactory.createEmptyBorder(20, 25, 10, 25));

			ButtonPanel bPanel = new ButtonPanel();
			bPanel.add(cancelBtn);
			bPanel.add(submitBtn);

			setLayout(new BorderLayout());
			add(centerPanel, BorderLayout.CENTER);
			add(bPanel, BorderLayout.SOUTH);
	}
	
	public void update(
		int function, SS3Database db,
		ArrayList<LoadedMediaFile> mFiles,
		ArrayList<LoadedMediaFile> mAudioFiles,
		SS3MediaLibrary mLibrary, ParticipantsLibrary participantsLibrary,
		ParticipantsLibrary annotatorsLibrary,
		SignBankResourceCollection resourceCollection) {

		this.mFiles = mFiles;
		this.mLibrary = mLibrary;
		this.participantsLibrary = participantsLibrary;
		this.annotatorsLibrary = annotatorsLibrary;
		this.function = function;
		this.db = db;
		this.function = function;

        if (this.db == null)
            this.db = new SS3Database();
//		if (this.db == null)
//			this.db = new SS3Database(new TimeInfo(), new TimeInfo());

		videoPanel.update(mFiles);
		videoPanel.repaint();

        audioPanel.update(mAudioFiles);
        audioPanel.repaint();

		signBankResourceCollectionView.update(resourceCollection);
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		String str = ae.getActionCommand();

		switch (Integer.parseInt(str)) {
		case LOOKUP:
			JDialog lDialog = new JDialog();

			DirectoryLookupScrolledView directoryPanel = new DirectoryLookupScrolledView(
					videoPanel, mLibrary,
					new Dimension(400, 180), frame, lDialog);

			Container contentPane = lDialog.getContentPane();
			contentPane.setLayout(new BorderLayout(50, 50));
			contentPane.add(directoryPanel, BorderLayout.CENTER);
			lDialog.setTitle("Lookup Directories ");
			lDialog.pack();
			lDialog.setVisible(true);
			break;
		case SUBMIT:
			if (function == CreateUpdateMacroUnit.FUNCTION_ADD) {
				enterEntry();
				SS3Singleton.getApplicationStateController()
						.setLoadedCollectionUpdated(true);
			} else if (function == CreateUpdateMacroUnit.FUNCTION_UPDATE) {
				editEntry();
				SS3Singleton.getApplicationStateController()
						.setLoadedCollectionUpdated(true);
			} else if (function == CreateUpdateMacroUnit.FUNCTION_DELETE) {
			}

			break;

		case CANCEL:
			cancelEntry();
			break;
		}
	}

	public boolean allFilesValid() {
        var video = videoPanel.getVideoPanels()
            .stream()
            .filter(p -> p.getMediaFile() != null)
            .allMatch(p -> p.getMediaFile().getMediaFile().getFile().exists());

        var audio = audioPanel.getAudioPanels()
            .stream()
            .filter(p -> p.getMediaFile() != null)
            .allMatch(p -> p.getMediaFile().getMediaFile().getFile().exists());

        var graph = graphs.values()
            .stream()
            .allMatch(g -> Paths.get(g.uri()).toFile().exists());

        return video && audio && graph;
    }

	public void cancelEntry() {
	    if (function == FUNCTION_ADD
            && (videoPanel.getVideoPanels().stream().allMatch(p -> p.getMediaFile() == null))
            && (audioPanel.getAudioPanels().stream().allMatch(p -> p.getMediaFile() == null))) {
	        parentDialog.dispose();
	        SS3Singleton.getSignStreamApplication().block();
            return;
        } else if (function == FUNCTION_ADD) {
            final JDialog dialog = new JDialog(parentDialog, "Unsaved Data", true);

            var discardBtn = new JButton("Discard");
            discardBtn.addActionListener(arg0 -> {
                dialog.dispose();
                parentDialog.dispose();
                SS3Singleton.getSignStreamApplication().block();
            });
            var cancelBtn = new JButton("Cancel");
            cancelBtn.addActionListener(l -> dialog.dispose());

            ButtonPanel buttonPanel = new ButtonPanel();
            buttonPanel.add(cancelBtn);
            buttonPanel.add(discardBtn);

            JPanel centerPanel = new JPanel();
            centerPanel.setLayout(new GridLayout(1, 1));
            JLabel l1 = new JLabel("Warning:  Unless you enter this information");
            JLabel l2 = new JLabel("before proceeding, it will be discarded.");
            centerPanel.add(l1);
            centerPanel.add(l2);

            JPanel panel = new JPanel();
            panel.setLayout(new BorderLayout());
            panel.add(centerPanel, BorderLayout.CENTER);
            panel.add(buttonPanel, BorderLayout.SOUTH);
            panel.add(new JLabel(" "), BorderLayout.WEST);

            Dimension d = panel.getPreferredSize();
            Dimension nD = new Dimension(d.width + 10, d.height + 40);
            panel.setPreferredSize(nD);

            Container contentPane = dialog.getContentPane();
            contentPane.setLayout(new BorderLayout(40, 40));
            contentPane.add(panel, BorderLayout.CENTER);

            dialog.setLocation(new Point(250, 250));
            dialog.pack();
            dialog.setVisible(true);
            return;
        }

	    var videoMissing = videoPanel.getVideoPanels().stream()
            .anyMatch(p -> p.getMediaFile() != null && !p.getMediaFile().getMediaFile().getFile().exists());
        var audioMissing = audioPanel.getAudioPanels().stream()
            .anyMatch(p -> p.getMediaFile() != null && !p.getMediaFile().getMediaFile().getFile().exists());
        var missing = videoMissing || audioMissing;

        if (function == FUNCTION_UPDATE && missing) {
            if (!videoPanel.isAtLeastOneValidVideoFileAvailable()) {
                SS3Singleton
                    .getSignStreamApplication()
                    .confirmationDialog("Macro Unit Warning",
                        "Warning:  Please select at least one valid available video file.");
                return;
            } else if (!allFilesValid()) {
                SS3Singleton
                    .getSignStreamApplication()
                    .confirmationDialog("Macro Unit Warning",
                        "Warning:  Please select a valid location for all files.");
                return;
            } else if (participantsPaneScrolledView.isParticipantViewListEmpty()) {
                SS3Singleton
                    .getSignStreamApplication()
                    .confirmationDialog("Macro Unit Warning",
                        "<html>Warning:  You must add at least one participant to the collection before proceeding."
                            + "<br><br>To do this, click the 'Add' button under the 'Participants' section of this window.</html>");
                return;
            } else if (annotatorsPaneScrolledView.isParticipantViewListEmpty()) {
                SS3Singleton
                    .getSignStreamApplication()
                    .confirmationDialog("Macro Unit Warning",
                        "<html>Warning:  You must add at least one annotator to the collection before proceeding."
                            + "<br><br>To do this, click the 'Add' button under the 'Annotators' section of this window.</html>");
                return;
            }
        }

        final JDialog dialog = new JDialog(parentDialog, "Unsaved Data", true);
        JButton cancelBtn = new JButton("Discard");
		JButton saveBtn = new JButton("Save");

		saveBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				switch (function) {
				case FUNCTION_ADD:					
					enterEntry();
					break;
				case FUNCTION_UPDATE:
					editEntry();
					break;
				}
			}
		});

		cancelBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
                parentDialog.dispose();
			}
		});

		ButtonPanel buttonPanel = new ButtonPanel();
		buttonPanel.add(cancelBtn);
		buttonPanel.add(saveBtn);

		JPanel centerPanel = new JPanel();
		centerPanel.setLayout(new GridLayout(1, 1));
		JLabel l1 = new JLabel("Warning:  Unless you save this information");
		JLabel l2 = new JLabel("before proceeding, it will be discarded.");
		centerPanel.add(l1);
		centerPanel.add(l2);

		JPanel panel = new JPanel();
		panel.setLayout(new BorderLayout());
		panel.add(centerPanel, BorderLayout.CENTER);
		panel.add(buttonPanel, BorderLayout.SOUTH);
		panel.add(new JLabel(" "), BorderLayout.WEST);

		Dimension d = panel.getPreferredSize();
		Dimension nD = new Dimension(d.width + 10, d.height + 40);
		panel.setPreferredSize(nD);

		Container contentPane = dialog.getContentPane();
		contentPane.setLayout(new BorderLayout(40, 40));
		contentPane.add(panel, BorderLayout.CENTER);

		dialog.setLocation(new Point(250, 250));
		dialog.pack();
		dialog.setVisible(true);

		updateMediaToolBarAnnotatorList();
	}

	public void editEntry() {
		// at least one valid video file should be selected
        if (macroUnitTxt.getText().isBlank()) {
            SS3Singleton
                .getSignStreamApplication()
                .confirmationDialog("Macro Unit Warning",
                    "Warning:  Please enter a name for the macro unit.");
            return;
        }

		String excTxt = macroUnitTxt.getText().trim();
		boolean matches = excTxt.matches(Util.LABEL_REGEX_PATTERN);
		if (!matches) {
			SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.VALUE_MUST_MATCH_CHARACTERS);
			return;
		}
		
		if (!videoPanel.isAtLeastOneValidVideoFileAvailable()) {
			SS3Singleton
					.getSignStreamApplication()
					.confirmationDialog("Macro Unit Warning",
							"Warning:  Please select at least one valid available video file.");
			return;
        } else if (!allFilesValid()) {
            SS3Singleton
                .getSignStreamApplication()
                .confirmationDialog("Macro Unit Warning",
                    "Warning:  Please select a valid location for all files.");
            return;
		} else if (participantsPaneScrolledView.isParticipantViewListEmpty()) {
			SS3Singleton
			.getSignStreamApplication()
			.confirmationDialog("Macro Unit Warning",
					"<html>Warning:  You must add at least one participant to the collection before proceeding."
					+ "<br><br>To do this, click the 'Add' button under the 'Participants' section of this window.</html>");
			return;	
		} else if (annotatorsPaneScrolledView.isParticipantViewListEmpty()) {
			SS3Singleton
					.getSignStreamApplication()
					.confirmationDialog("Macro Unit Warning",
							"<html>Warning:  You must add at least one annotator to the collection before proceeding."
							+ "<br><br>To do this, click the 'Add' button under the 'Annotators' section of this window.</html>");
			return;			
		} else if (videoPanel.equals("")) {
			return;
		} else {

            // set participants in db and library
            setParticipants();

			// set annotators in db and library
            setAnnotators();

			// save participants, annotators, and search directories to the local and global files
            saveParticipantsAndAnnotatorsLibraries();

            // set macro unit label
            setMacroUnitLabel();

            db.clearGraphs();
            for (var g : graphs.values()) {
                db.addGraph(g);
                if (db.getTimeRange(g.range().name()) != null)
                    g.range(db.getTimeRange(g.range().name()));
                else
                    db.addTimeRange(g.range());
            }

            updateMediaToolBarAnnotatorList();

            // update the segment tier labels to reflect any edits made in the participants
            segmentPanel.update(db);

            var player = SS3Singleton.getMediaPlayer();
            player.close();

            db.clearAudios();
            var audioPanels = audioPanel.getAudioPanels();
            int order = 1;
            for (var ap : audioPanels) {
                var media = ap.getMediaFile();
                if (media != null && media.getMediaFile() != null) {
                    var file = media.getMediaFile();
                    if (file.getId() == null || file.getId().isEmpty())
                        file.setId(Util.getLongUniqueNumber());

                    var uri = file.getUri();

                    var audio = player.audio(uri).orElse(null);
                    if (audio == null)
                        audio = player.addAudio(uri, order);

                    if (!media.isDisplay())
                        audio.hide();

                    db.addAudio(audio, order);

                    order++;
                }
            }

            db.clearVideos();
            var videoPanels = videoPanel.getVideoPanels();
            order = 1;
            for (var vp : videoPanels) {
                var media = vp.getMediaFile();
                if (media != null && media.getMediaFile() != null) {
                    var file = media.getMediaFile();
                    if (file.getId() == null || file.getId().isEmpty())
                        file.setId(Util.getLongUniqueNumber());

                    var uri = file.getUri();

                    var video = player.video(uri).orElse(null);
                    if (video == null)
                        video = player.addVideo(uri, order);

                    if (!media.isDisplay())
                        video.hide();

                    db.addVideo(video, order);
                    
                    order++;
                }
            }

            // Add the audio time range if we now have an audio file
            if (SS3Singleton.getAudioTimeRange() == null && !player.audios().isEmpty()) {
                var audio = player.audios().get(0).range();
                SS3Singleton.setAudioTimeRange(
                    TimeRange.ABSOLUTE.subrange(TimeRange.AUDIO_ID, "Audio",
                        audio.unit(), audio.offset(), audio.duration(), audio.step()));
                db.addTimeRange(SS3Singleton.getAudioTimeRange());
            }

            // Register audio/video time ranges with database
            player.media().forEach(m -> db.addTimeRange(m.range()));

            parentDialog.dispose();

            player.preroll();

            var v = player.videos().get(0);
            var timestep = v.timestep();
            var timebase = v.timebase();
            var framerate = v.framerate();
            var duration = player.duration().as(TimeUnit.SECOND) * framerate * timestep;
            SS3Singleton.getVideoControlManager()
                .getZoomer()
                .getCoordinateTimeConvertor()
                .setTimeScale(timebase, timestep, duration);

            updateZooming();
        }
    }

    public AnnotatorsDialog getParticipantsDialog() {
        return participantsDialog;
    }

    public AnnotatorsDialog getAnnotatorsDialog() {
        return annotatorsDialog;
    }

	private void setMacroUnitLabel() {

        String macroUnitLabel = macroUnitTxt.getText().trim();

        // update Macro Unit Excerpt
        if (!db.getExcerpt().equals(macroUnitLabel)) {

            DbNavigationPanel navigation = segmentPanel
                    .getDbNavigationPanel();
            Object root = navigation.getJTree().getModel().getRoot();

            if (root == null)
                return;
            
            var dbcol = SS3Singleton.getSS3DatabaseCollection();
            dbcol.deleteSS3Database(db.getExcerptObj());

            DefaultMutableTreeNode rootNode = (DefaultMutableTreeNode) root;
            DefaultMutableTreeNode node;

            if (rootNode.getChildCount() > 0) {
                Object obj = rootNode.getChildAt(0);
                node = (DefaultMutableTreeNode) obj;

                String str = (String) node.getUserObject();

                SS3Singleton.getSignStreamApplication().setTitle(str + ": " + macroUnitLabel);

                Enumeration children = node.children();
                while (children.hasMoreElements()) {
                    DefaultMutableTreeNode fNode = (DefaultMutableTreeNode) children
                            .nextElement();
                    Excerpt excerpt = (Excerpt) fNode.getUserObject();

                    if (excerpt.getExcerpt().equals(db.getExcerpt())) {
                        excerpt.setExcerpt(macroUnitLabel);
                        db.setExcerptObj(excerpt);
                        dbcol.addSS3SDatabase(db);

                        navigation.refresh(rootNode);
                        expandAll(navigation.getJTree(), new TreePath(
                                rootNode), true);

                        int rowToSelect = node.getChildCount() + fNode.getLevel() - 1;
                        navigation.getJTree().setSelectionRow(rowToSelect);
                    }
                }
            }
        }
    }

    private void saveParticipantsAndAnnotatorsLibraries() {
        // save participants, annotators, and search directories
        edu.bu.signstream.common.db.UtilityFilesXMLWriter writer = new edu.bu.signstream.common.db.UtilityFilesXMLWriter();
//        var dbWriter = new SS4Writer();
        var dbWriter = new SS3Writer();
        try {
			participantsLibrary = participantsPaneScrolledView.getParticipantLibrary();
			annotatorsLibrary = annotatorsPaneScrolledView.getParticipantLibrary();
			
            writer.saveParticipantsLibrary(participantsLibrary);
            writer.saveAnnotatorsLibrary(annotatorsLibrary);
            dbWriter.saveMediaLibrary(mLibrary);
        } catch (Exception ex) {
            ex.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);
        }
    }

    private void setParticipants() {

        String prtName = "";
        if (prtView.size() > 0) {
            ParticipantView participantView = prtView.get(0);
            prtName = participantView.getParticipant().getFullName();
        }

        for (ParticipantView view : prtView) {
            SS3Participant prt = view.getParticipant();
            if (db.getSelectedParticipant(prt.getId()) == null) {	// if new participant, add it
                db.addSelectedParticipant(prt);
            }
            else {	// if edited participant, reset in db and library
            		participantsPaneScrolledView.getParticipantLibrary().editParticipant(prt);
            }
        }
    }

    private void setAnnotators() {
        // set annotators
        for (ParticipantView view : antView) {
            SS3Participant annotator = view.getParticipant();
            SS3Participant databaseAnnotator 
            		= db.getSelectedAnnotator(annotator.getId());
            		
            if (databaseAnnotator == null) {	// if new annotator, add it
                db.addSelectedAnnotator(annotator);
            }
            else { 	// if edited annotator, reset in db and library
            		databaseAnnotator = annotator;            		
            		annotatorsPaneScrolledView.getParticipantLibrary().editParticipant(annotator);
            }
        }
    }

    private void updateDatabase(ArrayList<SS3MediaFile> filesToLoad) {

        db.getMediaFiles().clear();

        for(SS3MediaFile file : filesToLoad) {
            db.getMediaFiles().put(file.getId(), file);
        }

        try {
            db.save();

            String dbDirectory = db.getParentDirectoryPath();
            String dir = db.getDirectory();
            String directory = dbDirectory + File.separator + dir;

            zipDirectory(directory);

        } catch (Exception e) {
            e.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", e);
        }
    }

    private ArrayList<SS3MediaFile> getFilesToLoad(ArrayList<LoadedMediaFile> selectedFiles) {

        ArrayList<SS3MediaFile> filesToLoad = new ArrayList<SS3MediaFile>();

        for (LoadedMediaFile file : selectedFiles) {
            SS3MediaFile mFile = file.getMediaFile();
            String id = mFile.getId();

            if (id == null || id.isEmpty())
                mFile.setId(Util.getLongUniqueNumber());

            db.addMediaFile(mFile);

            if (file.isDisplay())
                filesToLoad.add(mFile);
        }

        return filesToLoad;
    }
    
    private ArrayList<SS3MediaFile> getAllFiles(ArrayList<LoadedMediaFile> selectedFiles) {

        ArrayList<SS3MediaFile> allFiles = new ArrayList<SS3MediaFile>();

        for (LoadedMediaFile file : selectedFiles) {
            SS3MediaFile mFile = file.getMediaFile();
            String id = mFile.getId();

            if (id == null || id.isEmpty())
                mFile.setId(Util.getLongUniqueNumber());

            db.addMediaFile(mFile);

            allFiles.add(mFile);
        }

        return allFiles;
    }

    private ArrayList<LoadedMediaFile> getSelectedFiles() {

        ArrayList<LoadedMediaFile> selectedFiles = new ArrayList<LoadedMediaFile>();

        ArrayList<VideoPanel> videoPanels = videoPanel.getVideoPanels();

        for (VideoPanel vp : videoPanels) {
            LoadedMediaFile lFile = vp.getMediaFile();
            if (lFile != null && lFile.getMediaFile() != null
                    && lFile.getMediaFile().getFile() != null) {
                selectedFiles.add(lFile);
            }
        }

        return selectedFiles;
    }

    private void updateZooming() {

//        MultipleMovieController controller = SS3Singleton.getVideoControlManager().getMultipleMovieController();
//
//        double movieDuration = controller.getMovieDuration();
//        double packetDuration = controller.getPacketDuration();
//
//        segmentPanel.getZoomer().getCoordinateTimeConvertor()
//                .setPacketDuration(packetDuration);

        double width;

        if(segmentPanel.isUtteranceLoaded()){
            width = segmentPanel.getUtteranceView().getVisibleWidth2();
        } else {
            width = segmentPanel.getDatabaseView().getVisibleWidth2();
        }

        if (width > -1) {

            // Only width is used
            segmentPanel.getZoomer().fillEntirePage(0, 0, width);

            segmentPanel.updateJMenuBar();

            segmentPanel.repaint();
        }
    }

    /**
     * Zips a directory into a ".ss3" archive file.
     * Assumes the input has already had its ".ss3" extension added.
     * @param dirName
     * @return True if directory was zipped and deleted and false if an exception was thrown
     */
	public boolean zipDirectory(String dirName) {
		try {
			File dirFile = new File(dirName);
			String cDirName = dirFile.getName();

			if (cDirName.startsWith(SS3Database.tempDirectoryPrefix)) {
				cDirName = cDirName.substring(4);
			}

			String zipFile = dirFile.getParentFile().getPath()
					+ File.separatorChar + cDirName;

			byte[] buffer = new byte[1024];

			try {
				FileOutputStream fout = new FileOutputStream(zipFile);
				ZipOutputStream zout = new ZipOutputStream(fout);

				File dir = new File(dirName);

				File[] children = dir.listFiles();
				if (children != null) {
					for (int i = 0; i < children.length; i++) {
						File file = children[i];

						FileInputStream fin = new FileInputStream(file);
						zout.putNextEntry(new ZipEntry(file.getName()));
						int length;

						while ((length = fin.read(buffer)) > 0) {
							zout.write(buffer, 0, length);
						}

						zout.closeEntry();
						fin.close();
					}
				}

				// close the ZipOutputStream
				zout.close();
				fout.close();

				// remove unziped files
				if (children != null) {
					for (int i = 0; i < children.length; i++) {
						File file = children[i];
						file.delete();
					}
				}
				dir.delete();

				if (zipFile != null) {
					String fileToDelete = zipFile + SS3Database.zipFileExtension;
					File file = new File(fileToDelete);
					if (file.exists()) {
						file.delete();
					}
				}

			} catch (FileNotFoundException ex) {
				ex.printStackTrace();
				SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);

				return false;
			}
		} catch (IOException ioe) {
			ioe.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ioe);
			return false;
		}
		return true;
	}

	private String getDefaultMacroUnitLabel() {
		DbNavigationPanel navigation = segmentPanel.getDbNavigationPanel();
		Object root = navigation.getJTree().getModel().getRoot();

		if (root == null)
			return "";

		DefaultMutableTreeNode rootNode = (DefaultMutableTreeNode) root;
		DefaultMutableTreeNode node;

		String txt = macroUnitTxt.getText().trim();

		if (rootNode.getChildCount() > 0) {
			Object obj = rootNode.getChildAt(0);
			node = (DefaultMutableTreeNode) obj;

			int counter = 0;
			Enumeration children = node.children();
			while (children.hasMoreElements()) {
				DefaultMutableTreeNode fNode = (DefaultMutableTreeNode) children
						.nextElement();
				Excerpt excerpt = (Excerpt) fNode.getUserObject();
				int fnID = Integer.parseInt(excerpt.getID());
				counter = fnID > counter ? fnID : counter;
			}

			if (txt.isEmpty())
				txt = "macro_unit_" + (counter + 1);

			return txt;
		}
		return "";
	}

	private void enterEntry() {
		// at least one valid video file should be selected
        if (macroUnitTxt.getText().isBlank()) {
            SS3Singleton
                .getSignStreamApplication()
                .confirmationDialog("Macro Unit Warning",
                    "Warning:  Please enter a valid name for the macro unit.");
            return;
        }

		String excTxt = macroUnitTxt.getText().trim();
		boolean matches = excTxt.matches(Util.LABEL_REGEX_PATTERN);
		if (!matches) {
			SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.VALUE_MUST_MATCH_CHARACTERS);
			return;
		}
		
		if (!videoPanel.isAtLeastOneValidVideoFileAvailable()) {
			SS3Singleton
					.getSignStreamApplication()
					.confirmationDialog("Macro Unit Warning",
							"Warning:  Please select at least one valid available video file.");

			return;
        } else if (!allFilesValid()) {
            SS3Singleton
                .getSignStreamApplication()
                .confirmationDialog("Macro Unit Warning",
                    "Warning:  Please select a valid location for all files.");
            return;
		} else if (participantsPaneScrolledView.isParticipantViewListEmpty()) {
			SS3Singleton
			.getSignStreamApplication()
			.confirmationDialog("Macro Unit Warning",
					"<html>Warning:  You must add at least one participant to the collection before proceeding."
					+ "<br><br>To do this, click the 'Add' button under the 'Participants' section of this window.</html>");
			return;	
		} else if (annotatorsPaneScrolledView.isParticipantViewListEmpty()) {
			SS3Singleton
					.getSignStreamApplication()
					.confirmationDialog("Macro Unit Warning",
							"<html>Warning:  You must add at least one annotator to the collection before proceeding."
							+ "<br><br>To do this, click the 'Add' button under the 'Annotators' section of this window.</html>");
			return;			
		} else {
            // save participants, annotators, and search directories
            saveParticipantsAndAnnotatorsLibraries();

			db = null;
			Excerpt excerpt = null;

			DbNavigationPanel navigation = segmentPanel.getDbNavigationPanel();
			Object root = navigation.getJTree().getModel().getRoot();

			if (root == null)
				return;

			DefaultMutableTreeNode rootNode = (DefaultMutableTreeNode) root;
			DefaultMutableTreeNode cNode = null;
			DefaultMutableTreeNode node = null;

			if (rootNode.getChildCount() > 0) {
				Object obj = rootNode.getChildAt(0);
				node = (DefaultMutableTreeNode) obj;

				int counter = 0;
				Enumeration children = node.children();
				while (children.hasMoreElements()) {
					DefaultMutableTreeNode fNode = (DefaultMutableTreeNode) children
							.nextElement();
					Excerpt ex = (Excerpt) fNode.getUserObject();
					int fnID = Integer.parseInt(ex.getID());
					counter = fnID > counter ? fnID : counter;
				}

				if (excTxt.length() == 0)
					excTxt = "macro_unit_" + (counter + 1);

				excerpt = new Excerpt("" + (counter + 1), excTxt);

				cNode = new DefaultMutableTreeNode(excerpt);
				node.add(cNode);
			}

            db = new SS3Database();

			// copy over graph files
            for (var g : graphs.values()) {
                db.addTimeRange(g.range());
                db.addGraph(g);
            }

			SS3GlossWindow glsWindow = new SS3GlossWindow();

			SS3GlossWindowSegment displayTimePeriod = new SS3GlossWindowSegment();
			displayTimePeriod.setLabel("Display Time Period");
			glsWindow.setDisplayTimePeriod(displayTimePeriod);

			SS3TemporalPartition temporalPartition = new SS3TemporalPartition();
			temporalPartition.setLabel("Temporal Partition 1");

			SS3GlossWindowSegment tpSegment = new SS3GlossWindowSegment();
			tpSegment.setLabel("Segment Tier 1");
			tpSegment.setID("" + Util.getUniqueNumber());
			if (prtView.size() == 1) {
				tpSegment.setParticipant(prtView.get(0).getParticipant());
			}
			else {
				ArrayList<SS3Participant> participants = new ArrayList<SS3Participant>(prtView.size());
				for (ParticipantView pView : prtView) {
					participants.add(pView.getParticipant());
				}
				AnnotatorSelectionDialog annotatorSelectionDialog = new AnnotatorSelectionDialog(
						participants, 
						getParticipantsDialog(), parentDialog);
				annotatorSelectionDialog.setLocation(250, 250);
				annotatorSelectionDialog.setVisible(true);
				
				// get last selected item in list of participants before confirmation, then pass it along to createSegment()
				SS3Participant participant = annotatorSelectionDialog.getAnnotatorSelectionPanel().getSelectedAnnotator();
				tpSegment.setParticipant(participant);
			}
			temporalPartition.addTemporalPartitionSegment(tpSegment);
			temporalPartition.setDefaultSegment(tpSegment);

			ArrayList<SS3TemporalPartition> partitions = new ArrayList<SS3TemporalPartition>();
			partitions.add(temporalPartition);
			glsWindow.setTemporalPartitions(partitions);

			db.setSS3GlossWindow(glsWindow);

			db.setFileName(excTxt + ".xml");
			db.setDirectory(node.getUserObject().toString());

			db.setExcerpt(excerpt.getExcerpt());
			db.setExcerptObj(excerpt);
			SS3Singleton.getSS3DatabaseCollection()
					.addSS3SDatabase(db);
			
			segmentPanel.updateLoadedDatabase();

			for (ParticipantView view : prtView) {
				db.addSelectedParticipant(view.getParticipant());
			}

			// set annotators
			for (ParticipantView annot : antView) {
				db.addSelectedAnnotator(annot.getParticipant());
			}

			// set participant
			String prtName = "";
			if (prtView.size() > 0) {
				ParticipantView participantView = prtView.get(0);
				prtName = participantView.getParticipant().getFullName();
			}

			var player = SS3Singleton.getMediaPlayer();
            player.close();

            db.clearAudios();
            var audioPanels = audioPanel.getAudioPanels();
            int order = 1;
            for (var ap : audioPanels) {
                var media = ap.getMediaFile();
                if (media != null && media.getMediaFile() != null) {
                    var file = media.getMediaFile();
                    if (file.getId() == null || file.getId().isEmpty())
                        file.setId(Util.getLongUniqueNumber());

                    var uri = file.getUri();

                    var audio = player.audio(uri).orElse(null);
                    if (audio == null)
                        audio = player.addAudio(uri, order);

                    if (!media.isDisplay())
                        audio.hide();

                    db.addAudio(audio, order);
                    
                    order++;
                }
            }

            db.clearVideos();
            var videoPanels = videoPanel.getVideoPanels();
            order = 1;
            for (var vp : videoPanels) {
                var media = vp.getMediaFile();
                if (media != null && media.getMediaFile() != null) {
                    var file = media.getMediaFile();
                    if (file.getId() == null || file.getId().isEmpty())
                        file.setId(Util.getLongUniqueNumber());

                    var uri = file.getUri();

                    var video = player.video(uri).orElse(null);
                    if (video == null)
                        video = player.addVideo(uri, order);

                    if (!media.isDisplay())
                        video.hide();

                    db.addVideo(video, order);

                    order++;
                }
            }

			navigation.refresh(rootNode);
			expandAll(navigation.getJTree(), new TreePath(rootNode), true);

			if (cNode != null) {
				int rowToSelect = node.getChildCount() + cNode.getLevel() - 1;

				segmentPanel.updateLoadedDatabase();
				SS3Database nDB = SS3Singleton.getSS3DatabaseCollection()
						.getSS3Database(excerpt);
				segmentPanel.loadDatabase(nDB);
				segmentPanel.repaint();

				JFrame frame = segmentPanel.getParentFrame();
				
				if (frame != null) {
					Object uo = cNode.getUserObject();
					if (uo instanceof String) {
						frame.setTitle((String) uo);
						frame.repaint();
					}
				}

				navigation.getJTree().setSelectionRow(rowToSelect);
				navigation.getJTree().repaint();
			}

			DefaultMutableTreeNode bNode = (DefaultMutableTreeNode) node
					.getChildBefore(cNode);
			
			if (bNode != null) {
				Excerpt ex = (Excerpt) bNode.getUserObject();
				SS3Database ndb = SS3Singleton.getSS3DatabaseCollection()
						.getSS3Database(ex);

				Excerpt cEx = (Excerpt) cNode.getUserObject();
				SS3Database cdb = SS3Singleton.getSS3DatabaseCollection()
						.getSS3Database(cEx);
				cdb.setStartTime(ndb.getEndTime());
				cdb.setEndTime(ndb.getEndTime());
			}

//			SS3Singleton.getMediaToolBar(null).setPlayButtonLabel(MediaToolBar.PLAY_RANGE);

            // set macro unit label
            setMacroUnitLabel();

//            updateDatabase(allFiles);
            
            updateMediaToolBarAnnotatorList();

			segmentPanel.loadDatabase(db);

            // update the segment tier labels to reflect any edits made in the participants
            segmentPanel.update(db);

//            loadMovies(lFiles);

            player.preroll();
            var template = player.videos().get(0);
            var timestep = template.timestep();
            var timebase = template.timebase();
            var framerate = template.framerate();
            var duration = player.duration().as(TimeUnit.SECOND) * framerate * timestep;
            SS3Singleton.getVideoControlManager()
                .getZoomer()
                .getCoordinateTimeConvertor()
                .setTimeScale(timebase, timestep, duration);

            segmentPanel.updateSize();

            db.addTimeRange(TimeRange.ABSOLUTE);

            // Set global audio/video time range to first audio/video file's range
            var video = player.videos().get(0).range();
            SS3Singleton.setVideoTimeRange(
                TimeRange.ABSOLUTE.subrange(TimeRange.VIDEO_ID, "Video",
                    video.unit(), video.offset(), video.duration(), video.step()));
            if (!player.audios().isEmpty()) {
                var audio = player.audios().get(0).range();
                SS3Singleton.setAudioTimeRange(
                    TimeRange.ABSOLUTE.subrange(TimeRange.AUDIO_ID, "Audio",
                        audio.unit(), audio.offset(), audio.duration(), audio.step()));
            }

            db.addTimeRange(SS3Singleton.getVideoTimeRange());
            if (SS3Singleton.getAudioTimeRange() != null)
                db.addTimeRange(SS3Singleton.getAudioTimeRange());

            // Register audio/video time ranges with database
            player.media().forEach(m -> db.addTimeRange(m.range()));

            order = 1;
            // Register audios and videos with database
            for (Video v: player.videos()) {
            		db.addVideo(v, order);
            		order++;
            }
            order = 1;
            for (Audio a: player.audios()) {
	            	db.addAudio(a, order);
	            	order++;
            }
            
            try {
                db.save();

            } catch (Exception e) {
                e.printStackTrace();
                SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", e);
            }

            updateZooming();

			parentDialog.dispose();
			
			// select annotator screen	
			AnnotatorSelectionDialog annotatorSelectionDialog = new AnnotatorSelectionDialog(
					SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().getSS3Database().getSelectedAnnotators(), 
					annotatorsDialog, parentDialog);
			annotatorSelectionDialog.setLocation(250, 250);
			annotatorSelectionDialog.setVisible(true);
			
			SS3Singleton.bringAllVideosToFront();
		}
	}

	private void expandAll(JTree tree, TreePath parent, boolean expand) {

		// Traverse children
		TreeNode node = (TreeNode) parent.getLastPathComponent();
		if (node.getChildCount() >= 0) {
			for (Enumeration e = node.children(); e.hasMoreElements();) {
				TreeNode n = (TreeNode) e.nextElement();
				TreePath path = parent.pathByAddingChild(n);
				expandAll(tree, path, expand);
			}
		}

		// Expansion or collapse must be done bottom-up
		if (expand) {
			tree.expandPath(parent);
		} else {
			tree.collapsePath(parent);
		}
	}

    private void updateMediaToolBarAnnotatorList() {
		ArrayList<SS3Participant> selectedAnnotators = 
				SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().getSS3Database().getSelectedAnnotators();
		ArrayList<String> annotatorStrings = new ArrayList<String>();
		
		for (SS3Participant annotator: selectedAnnotators) {
			annotatorStrings.add(annotator.getLabel());
		}
		
		// update media tool bar annotator combobox
		SS3Singleton.getMediaToolBar(SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel())
				.updateAnnotatorList(annotatorStrings, SS3Singleton.getCurrentAnnotator());			
	}    	
    
    public void madeChange() {
    		this.didMakeChanges = true;
    }

	public void addGraph(Graph g) {
	    graphs.put(g.id(), g);
    }

    public Graph getGraph(String name) {
	    return graphs.get(name);
    }

    public void removeGraph(String id) {
	    graphs.remove(id);
    }
}
