package edu.bu.signstream.ui.panels.signbank;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.signbank.SignBankResource;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.AssociatedFieldsPanel;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.signbank.SignBankCollection;
import edu.bu.signstream.grepresentation.view.signbank.SignBankOccurrence;
import edu.bu.signstream.grepresentation.view.signbank.SignBankResultNode;
import edu.bu.signstream.ui.SS3SignStreamApplication;

import javax.imageio.ImageIO;
import javax.swing.*;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;

public class SignBankResultsListPanel extends JPanel implements TreeSelectionListener, ActionListener {

    private static final long serialVersionUID = 1L;

    private SignBankPanel parent = null;

    private AssociatedFieldsPanel assocFieldsPanel = null;

    private JScrollPane jsp = null;

    private JTree resultsTree = null;
    private DefaultTreeCellRenderer resultsRenderer = null;

    private DefaultMutableTreeNode rootNode = null;
    private DefaultMutableTreeNode selectedNode = null;

    private JPanel labelDisplayPanel = null;

    private String variantLabel;
    
    private JButton deleteButton = null;
    private JButton insertButton = null;
    private JButton expandCollapseButton = null;
    
    private boolean expand = false;
    private final int MAX_ROWS_ON_SCREEN = 10;

    private JLabel mainGlossLabel = null;
    private final int DELETE = 10;
    private final int INSERT = 20;
    private final int EXPAND_COLLAPSE = 30;

    private final String insertPrimaryEntry = "Insert Labels of Primary Entry and Variant";
    private final String mainGlossLabelString = "Primary Entry: ";
    private final String variantLabelString = "Entry/Variant: ";

    public SignBankResultsListPanel(SignBankPanel parent, AssociatedFieldsPanel assocFieldsPanel) {
        this.parent = parent;
        this.assocFieldsPanel = assocFieldsPanel;

        createUI();
    }

    private void createTree(DefaultMutableTreeNode rootNode) {
        this.rootNode = rootNode;

        resultsTree = new JTree(rootNode);
        resultsTree.setCellRenderer(resultsRenderer);
        resultsTree.addTreeSelectionListener(this);
        resultsTree.getSelectionModel().setSelectionMode
                (TreeSelectionModel.SINGLE_TREE_SELECTION);

        jsp = new JScrollPane(resultsTree);
        jsp.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
        jsp.setName("Results:");
        jsp.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
    }

    private void createUI() {
        rootNode = new DefaultMutableTreeNode();
        selectedNode = rootNode;
        resultsRenderer = new MyRenderer();

        createTree(rootNode);

        int fontSize = 10;
        Font font = new Font("Arial", Font.PLAIN, fontSize);

        deleteButton = new JButton("Delete");
        deleteButton.setFont(font);
        deleteButton.addActionListener(this);
        deleteButton.setActionCommand("" + DELETE);

        insertButton = new JButton(insertPrimaryEntry);
        insertButton.setFont(font);
        insertButton.addActionListener(this);
        insertButton.setActionCommand("" + INSERT);
        
        expandCollapseButton = new JButton("Expand/Collapse All");
        expandCollapseButton.setFont(font);
        expandCollapseButton.addActionListener(this);
        expandCollapseButton.setActionCommand("" + EXPAND_COLLAPSE);

        mainGlossLabel = new JLabel(mainGlossLabelString);

        labelDisplayPanel = new JPanel();
        labelDisplayPanel.setLayout(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();

        c.gridy = 0;
        c.gridx = 0;
        c.insets = new Insets(0, 5, 0, 5);
        labelDisplayPanel.add(deleteButton, c);
        c.gridx = 1;
        labelDisplayPanel.add(insertButton, c);
        c.gridx = 2;
        labelDisplayPanel.add(expandCollapseButton, c);

        c.gridx = 0;
        c.gridwidth = 2;
        c.insets = new Insets(0, 0, 0, 0);
        c.anchor = GridBagConstraints.WEST;

        c.gridy++;
        //labelDisplayPanel.add(mainGlossLabel, c);

        c.gridy++;
        //labelDisplayPanel.add(variantLabel, c);
        c.gridx = 1;
        //labelDisplayPanel.add(variantLabelField, c);

        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
        add(jsp);
        add(labelDisplayPanel);

        updateList(null);
    }

    /**
     * Update the existing tree list in place with the new root passed in
     *
     * @param root The new root to reset this tree with
     */
    public void updateList(DefaultMutableTreeNode root) {
        if (root == null || root.getChildCount() == 0) {
            root = new DefaultMutableTreeNode("No Results");
        }

        createTree(root);

        DefaultTreeModel model = (DefaultTreeModel) resultsTree.getModel();
        model.reload(root);
        
        if (getChildCount(root) < MAX_ROWS_ON_SCREEN) {
        	expandTree(expand = true);
        } else {
        	expand = false;
        }

        removeAll();
        add(jsp);
        add(labelDisplayPanel);
        revalidate();
        repaint();
    }
    
    public int getChildCount(TreeNode node) {
    	int count = 1;
        int nChildren = node.getChildCount();  
        for (int i = 0; i < nChildren; i++) {  
            count += getChildCount(node.getChildAt(i));
        }  
        return count;  
    }
    
    public void expandTree(boolean expand) {
    	int startingIndex = 0;
    	if (!expand) { // don't collapse root node
    		startingIndex = 1;
    	}
    	for (int i = startingIndex; i < resultsTree.getRowCount(); i++) {
    		if (expand) {
    			resultsTree.expandRow(i);
    		} else {
    			resultsTree.collapseRow(i);
    		}
        }
    }

    /**
     * Delete the currently selected occurrence node (the sign) and write its origin resource file back to disk.
     * Also re-parse SignBankCollection object completely from all files, and update references.
     */
    public void deleteSign() {

        if (!(selectedNode instanceof SignBankResultNode)) {
            return;
        }

        SignBankResultNode signBankResultNode = (SignBankResultNode) selectedNode;
        SignBankOccurrence signBankOccurrence = (SignBankOccurrence) signBankResultNode.getUserObject();
        String resourceFileName = signBankOccurrence.getSIgnBankResourceFileName();
        SignBankCollection signBankCollection = parent.getSignBankCollection();
        SignBankResource resource = signBankCollection.getSignBankResourceCollection().getResourceByFileName(resourceFileName);
        boolean nonEditable = resource.isNonEditableResource();

        if (nonEditable) {
            // first check whether we can delete this occurrence, by validating it is from a local/editable resource
            String warningDialogText =
                    "This sign is global and cannot be deleted.";
            String warningTitleText = "Global Sign";
            int optionType = JOptionPane.YES_NO_OPTION;
            int messageType = JOptionPane.ERROR_MESSAGE;
            String arr[] = {"OK"};

            try {
                JOptionPane.showOptionDialog(this,
                                             warningDialogText,
                                             warningTitleText,
                                             optionType,
                                             messageType,
                                             null,
                                             arr,
                                             arr[0]);
            } catch (HeadlessException he) {
                System.out.println();
            }

            return;
        }

        String warningDialogText =
                "The selected sign will be permanently deleted from the local sign bank.\n\n"
                        + "Are you absolutely sure you want to delete this sign?";
        String warningTitleText = "WARNING";
        int optionType = JOptionPane.YES_NO_OPTION;
        int messageType = JOptionPane.ERROR_MESSAGE;
        String options[] = {"Proceed", "Cancel"};

        int result = -1;
        try {
            result = JOptionPane.showOptionDialog(this,
                                                  warningDialogText,
                                                  warningTitleText,
                                                  optionType,
                                                  messageType,
                                                  null,
                                                  options,
                                                  options[1]);
        } catch (HeadlessException he) {
            System.out.println();
        }

        if (result != JOptionPane.OK_OPTION)
            return;

        // first delete sign from local signbank memory collection
        signBankCollection.deleteSignBankOccurrence(signBankOccurrence);

        // write the resource file this sign came from back to disk
        SS3Singleton.getSignBankWriter().writeResourceToSignBank(signBankCollection, resourceFileName);
        // TODO think about altenatively passing in resource object (and grab str from there)

        // re-parse the entire sign bank collection fresh, application class reference will be set
        SS3Singleton.getSignStreamApplication().parseSignBank();

        selectedNode = null;

        // update the tree, simulate re-pressing lookup on the newly parsed collection
        Enumeration mainGlossNodes = rootNode.children();
        while (mainGlossNodes.hasMoreElements()) {
            DefaultMutableTreeNode mainGlossNode = (DefaultMutableTreeNode) mainGlossNodes.nextElement();

            // if just one child, delete the main gloss node as well
            int childCount = mainGlossNode.getChildCount();
            if (childCount == 1 && mainGlossNode.getChildAt(0).equals(signBankResultNode)) {
                mainGlossNode.remove((MutableTreeNode) mainGlossNode.getChildAt(0));
                rootNode.remove(mainGlossNode);

            } else {    // more than one child, don't delete the main gloss node
                int indexToDelete = -1;
                for (int i = 0; i < childCount; i++) {
                    if (mainGlossNode.getChildAt(i).equals(signBankResultNode)) {
                        indexToDelete = i;
                        break;
                    }
                }

                if (indexToDelete != -1) {
                    mainGlossNode.remove(indexToDelete);
                    break;
                }
            }
        }

        SwingUtilities.invokeLater(new Runnable() {
            @Override
            public void run() {
                updateList(rootNode);
            }
        });
//		updateList(rootNode);		
    }

    /**
     * Take the currently selected node and check which type it is.
     * If it is an occurrence node, grab the parent variant and parent main gloss, get its label.
     * If it is a main gloss node, just get its label.
     * Then in either case, take that label plus the label for the variant and pass it back to the MP.
     */
    public void insert() {
        // ignore selection on rootNode
        if (selectedNode.equals(rootNode)) {
            return;
        }

        String mainGlossLabel = null;
        if (selectedNode instanceof SignBankResultNode) {    // first case, we have selected a sign bank result node
            SignBankResultNode signBankResultNode = (SignBankResultNode) selectedNode;
            DefaultMutableTreeNode mainGlossParentNode = (DefaultMutableTreeNode) signBankResultNode.getParent();
            mainGlossLabel = (String) mainGlossParentNode.getUserObject();
        } else {    // second case, we have selected a main gloss node
            if (selectedNode.isRoot())
                return;

            mainGlossLabel = (String) selectedNode.getUserObject();
        }
        
        assocFieldsPanel.populateMainGlossAndVariantLabels(mainGlossLabel, variantLabel);

        SwingUtilities.getWindowAncestor(this).dispose();
    }

    @Override
    public void valueChanged(TreeSelectionEvent e) {
        JTree tree = (JTree) e.getSource();
        this.selectedNode = (DefaultMutableTreeNode) tree.getLastSelectedPathComponent();
        if (this.selectedNode == null) {
        	return;
        }
        // ignore selection on rootNode
        if (rootNode.equals(selectedNode)) {
            return;
        }

        if (!(selectedNode instanceof SignBankResultNode)) {
            String nodeMainGlossLabel = (String) selectedNode.getUserObject();
            variantLabel = nodeMainGlossLabel;
            mainGlossLabel.setText(mainGlossLabelString + nodeMainGlossLabel);
            
            return;
        }

        SignBankResultNode signBankResultNode = (SignBankResultNode) selectedNode;
        SignBankOccurrence signBankOccurrence = (SignBankOccurrence) signBankResultNode.getUserObject();

        variantLabel = signBankOccurrence.getParentVariant().getVariantLabel();
        String nodeMainGlossLabel = signBankOccurrence.getParentVariant().getParentMainGloss().getMainGlossLabel();
        mainGlossLabel.setText(mainGlossLabelString + nodeMainGlossLabel);

        parent.setVariantPanelVisible(true);
        parent.populateSignBankVariantPanel(signBankOccurrence);
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        String str = e.getActionCommand();
        switch (Integer.parseInt(str)) {
            case INSERT:
                insert();
                break;
            case DELETE:
                deleteSign();
                break;
            case EXPAND_COLLAPSE:
            	expand = !expand;
            	expandTree(expand);
            	break;
        	default:
        		break;
        }
    }
    
    private class TreeCellPanel extends JPanel {
    	private String variant;
    	private BufferedImage domStart, domEnd, ndomStart, ndomEnd;
    	private boolean selected;
    	
    	private final int width = 500, height = 20;
    	
    	public TreeCellPanel(SignBankOccurrence occurrence, boolean selected) {
    		this.variant = occurrence.getParentVariant().getVariantLabel();
    		this.selected = selected;
    		
    		String domStartHS = occurrence.getDomStartHS();
			String domEndHS = occurrence.getDomEndHS();
			String nondomStartHS = occurrence.getNondomStartHS();
			String nondomEndHS = occurrence.getNondomEndHS();

	        SS3CodingScheme dbScheme = SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().getSS3Database().getSS3CodingScheme();
	        
			ArrayList<SS3FieldValue> rightFieldValues = dbScheme.getFieldImg(Util.DOM_HANDSHAPE_FIELD_ID);
			ArrayList<SS3FieldValue> leftFieldValues = dbScheme.getFieldImg(Util.NDOM_HANDSHAPE_FIELD_ID);

            String domStartHSFilePath = getImageFilePathFromValue(domStartHS, rightFieldValues);
            String domEndHSFilePath = getImageFilePathFromValue(domEndHS, rightFieldValues);
            String nondomStartHSFilePath = getImageFilePathFromValue(nondomStartHS, leftFieldValues);
            String nondomEndHSFilePath = getImageFilePathFromValue(nondomEndHS, leftFieldValues);
            
            try {
            	if (!domStartHSFilePath.isEmpty()) {
            		domStart = SS3Singleton.getHandShapesImageManager().getImage(domStartHSFilePath);
                }
                if (!domEndHSFilePath.isEmpty()) {
            		domEnd = SS3Singleton.getHandShapesImageManager().getImage(domEndHSFilePath);
                }
                if (!nondomStartHSFilePath.isEmpty()) {
            		ndomStart = SS3Singleton.getHandShapesImageManager().getImage(nondomStartHSFilePath);
                }
                if (!nondomEndHSFilePath.isEmpty()) {
            		ndomEnd = SS3Singleton.getHandShapesImageManager().getImage(nondomEndHSFilePath);
                }
            } catch (Exception e) {
            	e.printStackTrace();
            }
    	}
		
    	@Override
		public void paint(Graphics g) {
			Graphics2D g2 = (Graphics2D) g;
			g2.setColor(Color.BLUE);
			if (selected) {
				g2.fillRect(0, 0, width, height);
			}

			if (domStart != null) {
            	g2.drawImage(domStart, 0, 0, 25, height, null);
            }
            if (domEnd != null) {
            	g2.drawImage(domEnd, 25, 0, 25, height, null);
            }
            if (ndomStart != null) {
            	g2.drawImage(ndomStart, 50, 0, 25, height, null);
            }
            if (ndomEnd != null) {
            	g2.drawImage(ndomEnd, 75, 0, 25, height, null);
            }
            
            int strY = g2.getFontMetrics().getHeight();
            if (selected) {
            	g2.setColor(Color.WHITE);
            } else {
            	g2.setColor(Color.BLACK);
            }
			g2.drawString(variant, 102, strY);
		}
    	
    	private String getImageFilePathFromValue(String hsName, ArrayList<SS3FieldValue> values) {
            SS3CodingScheme dbScheme = SS3Singleton.getSignStreamApplication()
            		.getSignStreamSegmentPanel().getSS3Database().getSS3CodingScheme();
            SS3FieldValue value = Util.matchSignBank(values, hsName);
            if (value != null) {
                return dbScheme.getImage(value.getIid()).getPath1();
            }
            return "";
        }
    	
    	@Override
    	public Dimension getPreferredSize() {
    		return new Dimension(width, height);
    	}
    }

    private class MyRenderer extends DefaultTreeCellRenderer {

        private static final long serialVersionUID = 1L;

        @Override
        public Component getTreeCellRendererComponent(JTree tree, Object value, boolean sel,
                                                      boolean expanded, boolean leaf, int row, boolean hasFocus) {
        	if (leaf && value instanceof SignBankResultNode) {
	        	SignBankOccurrence occurrence = (SignBankOccurrence) ((SignBankResultNode)value).getUserObject();
	        	return new TreeCellPanel(occurrence, sel);
        	}
  	        return super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf, row, hasFocus);
		}

    }

    public void clearResultsTree() {
        createUI();
    }
}
 