package edu.bu.signstream.ui.popup;

import edu.bu.signstream.common.util.UIPalette;
import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

import javax.swing.*;
import javax.swing.border.EmptyBorder;
import java.awt.*;
import java.awt.event.*;
import java.util.function.Consumer;

public class FieldValueCreatePopup extends Popup {
	public FieldValueCreatePopup(Field field, SignStreamSegmentPanel segmentPanel, Consumer<String> onEnter, Runnable onCancel) {
        setUndecorated(true);

        var panel = new JPanel();
        panel.setLayout(new GridBagLayout());
        panel.setBorder(new EmptyBorder(5, 0, 0, 0));

        var gbc = new GridBagConstraints();
        gbc.anchor = GridBagConstraints.WEST;

        var title = new JLabel("Field Specification Editor");
        UserInterfaceUtil.setLabelLNF(title);
        gbc.gridwidth = 3;
        gbc.gridx = 0;
        gbc.gridy = 0;
        panel.add(title, gbc);

        var nameLbl = new JLabel("New Value Name:");
        UserInterfaceUtil.setLabelLNF(nameLbl);
        gbc.gridy = 2;
        gbc.gridx = 0;
        gbc.gridwidth = 1;
        panel.add(nameLbl, gbc);

        panel.add(Box.createHorizontalStrut(10));

        var name = new JTextField(15);
        UserInterfaceUtil.setTextFieldLNF(name);
        gbc.gridx = 2;
        panel.add(name, gbc);

        panel.add(Box.createHorizontalStrut(10));

        var ok = new JButton("Done");
        ok.setToolTipText("Add new Value to local coding schema, set Value to selected Event");

        gbc.gridx = 4;
        panel.add(ok, gbc);

        var labelLbl = new JLabel("New Value Label:");
        UserInterfaceUtil.setLabelLNF(labelLbl);
        gbc.gridy = 3;
        gbc.gridx = 0;
        panel.add(labelLbl, gbc);

        panel.add(Box.createHorizontalStrut(10));

        var label = new JTextField(15);
        UserInterfaceUtil.setTextFieldLNF(label);
        gbc.gridx = 2;
        panel.add(label, gbc);

        panel.add(Box.createHorizontalStrut(10));

        var cancel = new JButton("Cancel");
        gbc.gridx = 4;
        panel.add(cancel, gbc);

        setBackground(UIPalette.getBackground());
        add(panel);

        ok.addActionListener(l -> {
        	if (name.getText() == null || name.getText().isBlank() || label.getText() == null || label.getText().isBlank())
        		return;
        	
            SS3FieldValue value = new SS3FieldValue();
            value.setId(Util.getUniqueNumber());
            value.setName(name.getText());
            value.setLabel(label.getText());
            
            

            var dbScheme = segmentPanel.getSS3Database().getSS3CodingScheme();
            var dbField = dbScheme.getField(field.getFieldID());
            dbField.addValue(value);

            onEnter.accept(value.getId());
            setVisible(false);
        });
        cancel.addActionListener(l -> {
            onCancel.run();
            setVisible(false);
        });

        addKeyListener(new KeyAdapter() {
            @Override
            public void keyReleased(KeyEvent e) {
                var key = e.getKeyCode();

                if (key == KeyEvent.VK_ESCAPE) {
                    onCancel.run();
                    setVisible(false);
                }
            }
        });

        addFocusListener(new FocusListener() {
            @Override
            public void focusLost(FocusEvent e) {
                onCancel.run();
                setVisible(false);
            }

            @Override public void focusGained(FocusEvent e) {}
        });

        pack();
        SwingUtilities.invokeLater(name::requestFocus);
    }
}
