package edu.bu.signstream.ui.popup;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.function.Consumer;

import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.UIPalette;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;

public class TextEntryPopupGenerator {
	private static TextEntryPopup popupInstance = null;
	
	public static void getInstance(String old, JPanel panel, Point loc, Consumer<String> onEnter, Runnable onCancel)  {
		if (popupInstance == null) {
			popupInstance = new TextEntryPopup(old, onEnter, onCancel);
			popupInstance.display(panel, loc);
		}
	}
	
	public static void deleteInstance() {
		if (popupInstance != null)
			popupInstance.setVisible(false);
		
		popupInstance = null;	
	}
}

class TextEntryPopup extends Popup {
    public TextEntryPopup(String old, Consumer<String> onEnter, Runnable onCancel) {
        setUndecorated(true);

    	var label = new JTextField(old);
        label.setPreferredSize(new Dimension(250, 17));
        label.setBackground(UIPalette.getBackground());
        add(label);

        label.addKeyListener(new KeyAdapter() {
            @Override
            public void keyReleased(KeyEvent e) {
                var key = e.getKeyCode();

                // still an issue here escape key does not register - still unclear as to why
                // however, the case where escape is clicked ends up getting handled below, in the focusLost
                if (key == KeyEvent.VK_ESCAPE) {
                    onCancel.run();
                	TextEntryPopupGenerator.deleteInstance();
                }

                if (key == KeyEvent.VK_ENTER) {
                	eval(label, onEnter, onCancel);
                }
            }
        });

        label.addFocusListener(new FocusAdapter() {
            @Override
            public void focusLost(FocusEvent e) {
                if (isVisible())
                    eval(label, onEnter, onCancel);
                else
                	onCancel.run();
                
            	TextEntryPopupGenerator.deleteInstance();
            }
        });

        pack();
        SwingUtilities.invokeLater(label::requestFocus);
    }
    
    private void eval(JTextField label, Consumer<String> onEnter, Runnable onCancel) {
    	var text = label.getText().trim();
    	
    	if (text.isEmpty())
    		onCancel.run();
    	else {
    		boolean matches = text.matches(Util.LABEL_REGEX_PATTERN);
            
    		if (!matches) {
    			SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.VALUE_MUST_MATCH_CHARACTERS);
    			onCancel.run();
            	TextEntryPopupGenerator.deleteInstance();
    			return;
    		}
    		
            onEnter.accept(text);
    	}
    	
    	TextEntryPopupGenerator.deleteInstance();
    }
}