package edu.bu.signstream.ui.video;

import edu.bu.signstream.time.Duration;
import edu.bu.signstream.time.Rounding;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.media.MediaPlayer;
import edu.bu.signstream.media.gstreamer.GstMediaPlayer;

import javax.swing.*;

public class MultipleMovieController {
	private final MediaPlayer player;
	// TODO: Remove this. Only used to access temp internal methods of underlying MediaPlayer implementation
    @Deprecated
    private final GstMediaPlayer _player;

	private int speedPointer = 2;

	public static int NUMBER_OF_SLIDER_TICKS = 10000;

	public MultipleMovieController() {
	    player = _player = (GstMediaPlayer)SS3Singleton.getMediaPlayer();

	    player.addTimeListener(this::updateSliders);
	    player.addStateListener(this::updatePlaying);
	}

    private void updateSliders(Time time) {
//        System.out.println("updateSliders " + time);
	    if (!_player.prerolled()) return;

	    var fr = time.duration().divide(player.duration());

        SwingUtilities.invokeLater(() -> {
            var tick = (int)(fr * NUMBER_OF_SLIDER_TICKS);
            for (var v : SS3Singleton.getVideoControlManager().getVideoControlPanels()) {
                v.moveSliderTo(tick);
            }

            SS3Singleton.getVideoControlManager().getSlider().setSliderPosition(time);
            SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().repaint();
        });
    }

    private void updatePlaying(boolean playing) {
	    SwingUtilities.invokeLater(() -> {
	        SS3Singleton.getMediaToolBar().update();
            for (VideoControlPanel videoPanel : SS3Singleton.getVideoControlManager().getVideoControlPanels())
                videoPanel.enableFunctions(!playing);
        });
    }

    public void pause() {
        if (!_player.prerolled()) return;

		if (player.playing()) {
			player.pause();
			SwingUtilities.invokeLater(() -> {
                for (VideoControlPanel videoPanel : SS3Singleton.getVideoControlManager().getVideoControlPanels()) {
                    videoPanel.setStopVideoControls();
                }
            });
		}
	}

	public double stepRate(boolean increase) {
	    if (!_player.prerolled()) return 1;

	    speedPointer = increase ? Math.min(5, speedPointer + 1) : Math.max(0, speedPointer - 1);
        var rate = Math.pow(2, speedPointer - 2);

        player.rate(rate);

        return rate;
	}

	public int getPacketDuration() {
	    return (int)SS3Singleton.getVideoControlManager()
            .getZoomer()
            .getCoordinateTimeConvertor()
            .timestep;
	}

	public double getMovieDuration() {
        return player.duration().as(TimeUnit.TIMESTAMP, Rounding.NEAREST);
	}

	public double getFrameRate() {
	    var c = SS3Singleton.getVideoControlManager()
            .getZoomer()
            .getCoordinateTimeConvertor();

	    return c.timebase / c.timestep;
	}

	public int getMovieTimescale() {
        return (int)SS3Singleton.getVideoControlManager()
            .getZoomer()
            .getCoordinateTimeConvertor()
            .timebase;
	}

	public Time getPrecedingFrameTimeInfo(Time timeInfo) {
	    return timeInfo.subtract(Duration.ts(getPacketDuration()));
	}

	public Time getFollowingFrameTimeInfo(Time timeInfo) {
	    return timeInfo.add(Duration.ts(getPacketDuration()));
	}

	public boolean isPlaying() {
        if (!_player.prerolled()) return false;

		return player.playing();
	}
}
