package edu.bu.signstream.ui.video;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Set;

import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import edu.bu.signstream.common.util.CoordinateTimeConverter;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.time.Duration;
import edu.bu.signstream.time.TimeRange;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.ui.video.controlPanelIcons.*;

public class VideoControlPanel extends JPanel implements ActionListener, ChangeListener {

	private static final long serialVersionUID = 1L;

	private JButton playBtn = new JButton();
	
	private JButton playFastBackBtn = new JButton(); 
	private JButton playFastForwardBtn = new JButton();
	
	private JButton frameForwardBtn = new JButton(); //>
	private JButton frameBackBtn = new JButton();//<	
	private JButton halfSecondForwardBtn = new JButton(); // >>
	private JButton secondForwardBtn = new JButton(); // >>>
	private JButton secondBackBtn = new JButton(); // <<<
	private JButton halfSecondBackBtn = new JButton(); // <<
	private JButton firstFrameBtn = new JButton(); // |<
	private JButton lastFrameBtn = new JButton(); // >|
	
	private JSlider slider = new JSlider(JSlider.HORIZONTAL, 0, CoordinateTimeConverter.getNumberOfSliderTicks(), 1);

	private final int PLAY = 1;
	private final int FRAME_FORWARD = 3;
	private final int FRAME_BACK = 4;
	private final int HALF_SECOND_FORWARD = 7;
	private final int ONE_SECOND_FORWARD = 9;
	private final int ONE_SECOND_BACK = 10;
	private final int HALF_SECOND_BACK = 8;
	private final int FIRT_FRAME = 5;
	private final int LAST_FRAME = 6;
	
	private final int PLAY_FASTER = 11;
	private final int PLAY_SLOWER = 12;

	private VideoControlManager videoControlManager;
	
	private JPanel mainPanel = new JPanel();
	private boolean updateSliderPosition = false;
	private int videoSliderPosition = 0;
	private boolean valueIsAdjusting = false;

	public VideoControlPanel(VideoControlManager videoControlManager) {
		this.videoControlManager = videoControlManager;
		init();		
		populateMainPanel();		
		setLayout(new GridLayout(0, 1));		
		add(mainPanel);
		add(slider);		
		slider.setSize(slider.getPreferredSize().width * 2, slider.getPreferredSize().height);
	}
		
	public VideoControlPanel(VideoControlManager videoControlManager, boolean menuPanel) {
		this.videoControlManager = videoControlManager;
		init();		
		populateMainPanel();
		setLayout(new GridLayout(0, 1));		
		add(mainPanel);
	}
	
	private void populateMainPanel() {
		mainPanel.setLayout(new FlowLayout(FlowLayout.CENTER, 0, 0));
		
		mainPanel.add(playFastBackBtn);
		mainPanel.add(playBtn);
		mainPanel.add(playFastForwardBtn);
		
		mainPanel.add(new JLabel(" "));
		mainPanel.add(new JLabel(" "));
		mainPanel.add(new JLabel(" "));
		mainPanel.add(new JLabel(" "));
		mainPanel.add(new JLabel(" "));
		mainPanel.add(new JLabel(" "));
		mainPanel.add(new JLabel(" "));
		mainPanel.add(new JLabel(" "));
		
		mainPanel.add(firstFrameBtn);
		mainPanel.add(secondBackBtn);
		mainPanel.add(halfSecondBackBtn);
		mainPanel.add(frameBackBtn);
		mainPanel.add(frameForwardBtn);
		mainPanel.add(halfSecondForwardBtn);
		mainPanel.add(secondForwardBtn);
		mainPanel.add(lastFrameBtn);

		mainPanel.setBackground(Color.BLACK);
	}
	
	public JPanel getMainPanel() {
		return mainPanel;
	}	
	
	private void init() {		
		this.addKeyListener(SS3Singleton.getSSKeyListener());
		this.setFocusable(true);
		
		slider.addChangeListener(this);
		slider.setMajorTickSpacing(1);
		slider.setPaintTicks(false);

		playBtn.setActionCommand("" + PLAY);
		frameForwardBtn.setActionCommand("" + FRAME_FORWARD);
		playBtn.addActionListener(this);
		frameForwardBtn.addActionListener(this);
		frameBackBtn.setActionCommand("" + FRAME_BACK);
		frameBackBtn.addActionListener(this);
		firstFrameBtn.setActionCommand("" + FIRT_FRAME);
		firstFrameBtn.addActionListener(this);
		lastFrameBtn.setActionCommand("" + LAST_FRAME);
		lastFrameBtn.addActionListener(this);		
		halfSecondForwardBtn.setActionCommand("" + HALF_SECOND_FORWARD);
		halfSecondForwardBtn.addActionListener(this);
		halfSecondBackBtn.setActionCommand("" + HALF_SECOND_BACK);
		halfSecondBackBtn.addActionListener(this);
		secondBackBtn.setActionCommand("" + ONE_SECOND_BACK);
		secondBackBtn.addActionListener(this);
		secondForwardBtn.setActionCommand("" + ONE_SECOND_FORWARD);
		secondForwardBtn.addActionListener(this);
		playFastBackBtn.setActionCommand("" + PLAY_FASTER);
		playFastBackBtn.addActionListener(this);
		playFastForwardBtn.setActionCommand("" + PLAY_SLOWER);
		playFastForwardBtn.addActionListener(this);

		playBtn.setFocusable(false);
		playFastBackBtn.setFocusable(false); 
		playFastForwardBtn.setFocusable(false); 
		
		frameForwardBtn.setFocusable(false); 
		frameBackBtn.setFocusable(false); 	
		halfSecondForwardBtn.setFocusable(false);
		secondForwardBtn.setFocusable(false); 
		secondBackBtn.setFocusable(false); 
		halfSecondBackBtn.setFocusable(false); 
		firstFrameBtn.setFocusable(false); 
		lastFrameBtn.setFocusable(false); 

		slider.setFocusable(false);

		playBtn.setToolTipText("Play");
		playFastBackBtn.setToolTipText("Slower");
        playFastForwardBtn.setToolTipText("Faster");

		frameForwardBtn.setToolTipText("1 Frame Forward");
		halfSecondForwardBtn.setToolTipText("0.5 Seconds Forward");
		secondForwardBtn.setToolTipText("1 Second Forward");
		
		frameBackBtn.setToolTipText("1 Frame Backward");
		halfSecondBackBtn.setToolTipText("0.5 Seconds Backward");
		secondBackBtn.setToolTipText("1 Second Backward");

		firstFrameBtn.setToolTipText("Start");
        lastFrameBtn.setToolTipText("End");

		Dimension d = new Dimension(24, 22);
		
		playFastBackBtn.setBorderPainted(false);		
		playFastBackBtn.setIcon(new PlayBackFastIcon(true, 22, 21));
		playFastBackBtn.setDisabledIcon(new PlayBackFastIcon(false, 22, 21));
		playFastBackBtn.setPreferredSize(d);
		
		playFastForwardBtn.setBorderPainted(false);		
		playFastForwardBtn.setIcon(new PlayForwardFastIcon(true, 22, 21));
		playFastForwardBtn.setDisabledIcon(new PlayForwardFastIcon(false, 22, 21));
		playFastForwardBtn.setPreferredSize(d);

		frameForwardBtn.setBorderPainted(false);		
		frameForwardBtn.setIcon(new GoFrameForward(true, 22, 21));
		frameForwardBtn.setDisabledIcon(new GoFrameForward(false, 22, 21));
		frameForwardBtn.setPreferredSize(d);
		
		frameBackBtn.setBorderPainted(false);		
		frameBackBtn.setIcon(new GoFrameBackIcon(true, 22, 21));
		frameBackBtn.setDisabledIcon(new GoFrameBackIcon(false, 22, 21));
		frameBackBtn.setPreferredSize(d);
		
		halfSecondForwardBtn.setBorderPainted(false);		
		halfSecondForwardBtn.setIcon(new GoTwoFramesForward(true, 22, 21));
		halfSecondForwardBtn.setDisabledIcon(new GoTwoFramesForward(false, 22, 21));
		halfSecondForwardBtn.setPreferredSize(d);

		secondForwardBtn.setBorderPainted(false);		
		secondForwardBtn.setIcon(new GoThreeFramesForward(true, 22, 21));
		secondForwardBtn.setDisabledIcon(new GoThreeFramesForward(false, 22, 21));
		secondForwardBtn.setPreferredSize(d);
		
		secondBackBtn.setBorderPainted(false);		
		secondBackBtn.setIcon(new GoThreFramesBackIcon(true, 22, 21));
		secondBackBtn.setDisabledIcon(new GoThreFramesBackIcon(false, 22, 21));
		secondBackBtn.setPreferredSize(d);

		halfSecondBackBtn.setBorderPainted(false);		
		halfSecondBackBtn.setIcon(new GoTwoFramesBackIcon(true, 22, 21));
		halfSecondBackBtn.setDisabledIcon(new GoTwoFramesBackIcon(false, 22, 21));
		halfSecondBackBtn.setPreferredSize(d);
		
		firstFrameBtn.setBorderPainted(false);		
		firstFrameBtn.setIcon(new GoFirstFrameIcon(true, 22, 21));
		firstFrameBtn.setDisabledIcon(new GoFirstFrameIcon(false, 22, 21));
		firstFrameBtn.setPreferredSize(d);

		lastFrameBtn.setBorderPainted(false);		
		lastFrameBtn.setIcon(new GoLastFrameIcon(true, 22, 21));
		lastFrameBtn.setDisabledIcon(new GoLastFrameIcon(false, 22, 21));
		lastFrameBtn.setPreferredSize(d);

		playBtn.setBorderPainted(false);		
		playBtn.setIcon(new PlayIcon(true, 22, 21));
		playBtn.setDisabledIcon(new PlayIcon(false, 20, 21));
		playBtn.setPreferredSize(d);

//		int c = JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT; //.WHEN_IN_FOCUSED_WINDOW;
//	    KeyStroke ks = KeyStroke.getKeyStroke(32, 0); 
//	        //.getKeyStroke(KeyEvent.VK_R, InputEvent.CTRL_DOWN_MASK);
//	    this.getInputMap(c).put(ks, "PRESS");
//	    this.getActionMap().put("PRESS", playBtn.getAction());
	}

	public VideoControlManager getVideoControlManager() {
		return videoControlManager;
	}

	public boolean isValueAdjusting() {
		return valueIsAdjusting;
	}

	public synchronized int getVideoSliderPosition() {
		return videoSliderPosition;
	}

	public synchronized void setVideoSLiderPosition(int videoSliderPosition) {
		this.videoSliderPosition = videoSliderPosition;
	}
	
	public void moveSliderToEndFrame()  {
		updateOnlyVideoSlderMark = true;					
		slider.setValue(CoordinateTimeConverter.getNumberOfSliderTicks());		
		updateOnlyVideoSlderMark = false;	
	}   
	
	public void moveSliderTo(int x)  {
		if (!updateSliderPosition) {
			updateOnlyVideoSlderMark = true;
			slider.setValue(x);
			updateOnlyVideoSlderMark = false;
		}
	}   

	@Override
	public void stateChanged(ChangeEvent e) {
		if(updateOnlyVideoSlderMark || updateSliderPosition)
			return;

		updateSliderPosition = true;

		JSlider source = (JSlider) e.getSource();	
		double fr = (double)source.getValue() / MultipleMovieController.NUMBER_OF_SLIDER_TICKS;

        var player = SS3Singleton.getMediaPlayer();
		var time = TimeRange.ABSOLUTE.zero().add(player.end().duration().multiply(fr));

		player.pause();
		player.seek(time);

//		SignStreamSegmentPanel sPanel = SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel();
//		MultipleMovieController controller = sPanel .getMultipleMovieController();
//
//		controller.pause();
//
//		int maxTicks = MultipleMovieController.NUMBER_OF_SLIDER_TICKS;
//	    double movieDuration = getVideoControlManager().getMultipleMovieController().getMovieDuration();
//		double packetDuration = (int)getVideoControlManager().getMultipleMovieController().getPacketDuration();
//
//		double numberOfFrames = movieDuration / packetDuration;
//		double framesInOneTick = numberOfFrames / maxTicks;
//		double goToFrameNumber = sliderValue * framesInOneTick;
//		double usethistime = goToFrameNumber * packetDuration;
//
//
//		SS3Singleton.getVideoControlManager()
//				.getMultipleMovieController()
//				.dragToCurrentTime(usethistime);

//		sPanel.getSlider().setSliderPositionTime(usethistime);
//		sPanel.repaint();

		updateSliderPosition = false;
	}
	
	private boolean updateOnlyVideoSlderMark = false;
	
	public void updateVideoSliderDoNotAdjustMovie(int sliderValue ) {
		updateOnlyVideoSlderMark = true;
		slider.setValue(sliderValue);
		updateOnlyVideoSlderMark = false;
	}
	
	public void setUpdateSliderPosition(boolean updateSliderPosition) {
		this.updateSliderPosition = updateSliderPosition;
	}

	private PlayIcon playIcon = new PlayIcon(true, 22, 21);
	private PlayIcon disabledPlayIcon =  new PlayIcon(false, 20, 21);
	private StopIcon stopIcon = new StopIcon(true, 22, 21);
	private StopIcon disabledStopIcon = new StopIcon(false, 20, 21);
	
	public void updatePlayButton(boolean play) {
		if(play) {
			playBtn.setIcon(playIcon);
			playBtn.setDisabledIcon(disabledPlayIcon);
		} else {
			playBtn.setIcon(stopIcon);
			playBtn.setDisabledIcon(disabledStopIcon);
			playBtn.setEnabled(true);
		}
		
//		ArrayList<VideoControlPanel> videoPanels = SS3Singleton.getVideoControlManager().getVideoControlPanels();
//		for(VideoControlPanel videoPanel: videoPanels) {
//			videoPanel.enableFunctions(play);
//		}
	}
	
	public void setStartVideoControls() {
		playBtn.setIcon(stopIcon);
		playBtn.setDisabledIcon(disabledStopIcon);
		playBtn.setEnabled(true);
	}
	
	public void setStopVideoControls() {
		playBtn.setIcon(playIcon);
		playBtn.setDisabledIcon(disabledPlayIcon);
	}
	
	public void enableFunctions(boolean enable) {
		frameForwardBtn.setEnabled(enable);
		frameBackBtn.setEnabled(enable);
		firstFrameBtn.setEnabled(enable);
		lastFrameBtn.setEnabled(enable);
		halfSecondForwardBtn.setEnabled(enable);
		halfSecondBackBtn.setEnabled(enable);
		secondBackBtn.setEnabled(enable);
		secondForwardBtn.setEnabled(enable);

		if(enable) {
			playing = false;
			playBtn.setIcon(new PlayIcon(true, 22, 21));
			playBtn.setDisabledIcon(new PlayIcon(false, 20, 21));
		} else {
			playing = true;
			playBtn.setIcon(new StopIcon(true, 22, 21));
			playBtn.setDisabledIcon(new StopIcon(false, 20, 21));
			playBtn.setEnabled(true);
		}
	}
	
	boolean playing = false;
	
	public void playPause() {
	    var player = SS3Singleton.getMediaPlayer();

	    if (player.playing())
	        player.pause();
	    else
	        player.play();

//	    updatePlayButton(player.playing());
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		String str = ae.getActionCommand();
		
		SignStreamSegmentPanel segmentPanel = SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel();

        var player = SS3Singleton.getMediaPlayer();
        var range = Util.getCurrentRange(SS3Singleton.getVideoTimeRange());

        switch (Integer.parseInt(str)) {
		case PLAY:
		    playPause();
			break;
		case FRAME_FORWARD:
            SS3Singleton.getMediaToolBar(null);
            var step1 = Util.getStep();
            var range1 = Util.getStepRange();
            player.pause();
            player.seek(player.time().add(step1).in(range1));
            break;
		case FRAME_BACK:
            SS3Singleton.getMediaToolBar(null);
            var step2 = Util.getStep();
            var range2 = Util.getStepRange();
            player.pause();
            player.seek(player.time().subtract(step2).in(range2));
            break;
		case FIRT_FRAME:

            SS3Singleton.getMediaToolBar(null);
            SS3Singleton.getMediaPlayer().seek(SS3Singleton.getMediaPlayer().start());
            break;
		case LAST_FRAME:
            SS3Singleton.getMediaToolBar(null);
            SS3Singleton.getMediaPlayer().seek(SS3Singleton.getMediaPlayer().end());
            break;
		case HALF_SECOND_FORWARD:
            player.seek(player.time().add(Duration.ms(500)).in(range));
			break;
		case HALF_SECOND_BACK:
            player.seek(player.time().subtract(Duration.ms(500)).in(range));
			break;
		case ONE_SECOND_FORWARD:
            player.seek(player.time().add(Duration.ms(1000)).in(range));
			break;
		case ONE_SECOND_BACK:
            player.seek(player.time().subtract(Duration.ms(1000)).in(range));
			break;
		case PLAY_FASTER:
			MultipleMovieController controller1 = videoControlManager.getMultipleMovieController(); 
			var rate1 = controller1.stepRate(false);
//			if(!controller1.isPlaying()) {
//				playMovie();
//			}

			SignStreamSegmentPanel panel1 = SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel();
			Hashtable<JDialog, java.io.File> dialogs1 = panel1.getOpenedMediaDialogs(); 
			Set<JDialog> keysSet1 = dialogs1.keySet();
			Iterator<JDialog> iter1 = keysSet1.iterator();
//			while(iter1.hasNext()) {
//				JDialog dialog1 = iter1.next();
//				VideoImage image1 = (VideoImage)dialog1;
//				image1.setSpeedLabel(rate1 == 1 ? "" : "X" + rate1);
//			}
			break;
		case PLAY_SLOWER:
			MultipleMovieController controller2 = videoControlManager.getMultipleMovieController(); 
			var rate2 = controller2.stepRate(true);
//			if(!controller2.isPlaying()) {
//				playMovie();
//			}
			
			Hashtable<JDialog, java.io.File> dialogs2 = segmentPanel.getOpenedMediaDialogs(); 
			Set<JDialog> keysSet2 = dialogs2.keySet();
			Iterator<JDialog> iter2 = keysSet2.iterator();
//			while(iter2.hasNext()) {
//				JDialog dialog2 = iter2.next();
//				VideoImage image2 = (VideoImage)dialog2;
//				image2.setSpeedLabel(rate2 == 1 ? "" : "X" + rate2);
//			}
			break;
		}
	}

	public JButton getPlayBtn() {
		return playBtn;
	}	
	
	public JButton getLastFrameBtn() {
		return lastFrameBtn;
	}

	public JButton getFirstFrameBtn() {
		return firstFrameBtn;
	}

	public JButton getFrameForwardBtn() {
		return frameForwardBtn;
	}

	public JButton getFrameBackBtn() {
		return frameBackBtn;
	}

	public JSlider getSlider() {
		return slider;
	}
}







