package edu.bu.signstream.common.db.ss3;

import edu.bu.signstream.common.db.UtilityFilesXMLWriter;
import edu.bu.signstream.common.files.SSResources;
import edu.bu.signstream.common.util.IntervalSet;
import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.common.util.vo.ss3.participants.ParticipantsLibrary;
import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.grepresentation.fields.glossField.GlossFieldWrapper;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations.Region;
import edu.bu.signstream.grepresentation.fields.signTypeField.SignType;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.media.gstreamer.GstProbe;
import edu.bu.signstream.time.*;
import edu.bu.signstream.ui.SS3SignStreamApplication;

import org.freedesktop.gstreamer.Fraction;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import javax.swing.*;

import java.awt.HeadlessException;
import java.io.File;
import java.nio.file.Path;
import java.util.*;
import java.util.Map.Entry;
import java.util.logging.Level;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

import static edu.bu.signstream.common.db.ss4.SS4Util.*;

public class SS3Validator extends Validator {
	Document doc = null;
	
	Element rootEl = null, dbEl = null, glossWindowEl = null;
	
	Long timebase, timestep, duration;
	
	TimeUnit frameTimeUnit = null;
	
	// Entity ID - Track ID
	HashMap<String, String> entityTrackMap = new HashMap<String, String>();

	HashMap<String, String> deletedEntities = new HashMap<String, String>();	// map: eid -> fid
	HashMap<String, String> newEntities = new HashMap<String, String>();	// map: eid -> fid	

    HashMap<String, Element> participantElements = new HashMap<String, Element>();
    HashMap<String, Element> annotatorElements = new HashMap<String, Element>();
	
	HashMap<String, Element> partitionElements = new HashMap<String, Element>();
	HashMap<String, Element> tierElements = new HashMap<String, Element>();
	HashMap<String, Element> utteranceElements = new HashMap<String, Element>();

	HashMap<String, String> tierPartitionMap = new HashMap<String, String>();		// map: segmentTierID -> temporalPartitionID
	HashMap<String, String> utteranceTierMap = new HashMap<String, String>();	// map: utteranceID -> segmentTierID
	HashMap<String, String> entityUtteranceMap = new HashMap<String, String>();	// map: entityID -> utteranceID
	
	HashMap<String, Element> trackElements = new HashMap<String, Element>();
	HashMap<String, Element> entityElements = new HashMap<String, Element>();
	// for itemElements: key is entity id, value is list of items in entity keyed by its item text
	HashMap<String, ArrayList<Element>> itemElements = new HashMap<String, ArrayList<Element>>();

	HashMap<String, IntervalSet<String>> utteranceIntervals = new HashMap<String, IntervalSet<String>>();	// key is segmentTierID
	HashMap<String, HashMap<String, IntervalSet<String>>> entityIntervals 
			= new HashMap<String, HashMap<String, IntervalSet<String>>>();		// first key is segmentTierID, second key is field/track id
	
	HashMap<String, ArrayList<String>> utteranceToEntitiesMap = new HashMap<String, ArrayList<String>>();
	
	HashMap<String, Element> localSchemeFieldElements = new HashMap<String, Element>();
	HashMap<String, HashMap<String, Element>> localSchemeFieldValues = new HashMap<String, HashMap<String, Element>>();
	
	private String DEFAULT_PARTICIPANT_ID = "800";
	private String DEFAULT_PARTICIPANT_FULL_NAME = "RightHandParticipant";
	private String DEFAULT_PARTICIPANT_LABEL = "RightHandParticipant";
	
	private boolean addedDefaultParticipantToCollection = false;
	
	private String DEFAULT_ANNOTATOR_ID = "200";
	private String DEFAULT_ANNOTATOR_FULL_NAME = "DefaultAnnotator";
	private String DEFAULT_ANNOTATOR_LABEL = "DefaultAnnotator";
	
	private boolean addedDefaultAnnotatorToCollection = false;
	
	private String PLACEHOLDER_PARTICIPANT_LABEL = "DefaultParticipantLabel";
	private String PLACEHOLDER_PARTICIPANT_FULLNAME = "DefaultParticipantFullName";
	private String PLACEHOLDER_PARTICIPANT_DOMINANT_HAND = "R";

	private String PLACEHOLDER_ANNOTATOR_LABEL = "DefaultAnnotatorLabel";
	
	private boolean onlyOneSegmentTier = false;
	
	/**
	 *  hash: entity_id -> field_id
	 */
	HashMap<String, String> refEntities = new HashMap<String, String>();
	
	SS3CodingScheme defScheme = null, defScheme3_3_5 = null;

    private int corrected = 0;

	private boolean modifyFieldBodyMvmtValue = true;
    
    @Override
	public SS3ValidatorConverterReturns validate(Document doc) {
    	this.doc = doc;

    	returns = new SS3ValidatorConverterReturns();
        returns.errors.clear();
        returns.modifications.clear();
    	
    	rootEl = null; 
    	dbEl = null;
    	glossWindowEl = null;
    	
    	timebase = 0L;
    	timestep = 0L;
    	duration = 0L;
    	
    	entityTrackMap = new HashMap<String, String>();
    	deletedEntities = new HashMap<String, String>();

        participantElements = new HashMap<String, Element>();
        annotatorElements = new HashMap<String, Element>();
    	
    	partitionElements = new HashMap<String, Element>();
    	tierElements = new HashMap<String, Element>();
    	utteranceElements = new HashMap<String, Element>();
    	
    	tierPartitionMap = new HashMap<String, String>();
    	utteranceTierMap = new HashMap<String, String>();
    	entityUtteranceMap = new HashMap<String, String>();
    	
    	trackElements = new HashMap<String, Element>();
    	entityElements = new HashMap<String, Element>();
    	itemElements = new HashMap<String, ArrayList<Element>>();
    	
    	utteranceIntervals = new HashMap<String, IntervalSet<String>>();

        utteranceToEntitiesMap = new HashMap<String, ArrayList<String>>();

		String dv = System.getProperty("file.separator", "/");
		var path = SSResources.xmlPath + dv;
		var scPath = path + "3.3.5.defCodingScheme.xml";
        var reader = new SS3Reader();
        defScheme3_3_5 = reader.parseSS3_3_5CodingScheme(scPath);	// using version from 3.3.5, as there were changes since then
        defScheme = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();
    	
    	SS3ValidatorConverterReturns ret = null;
    	
        this.rootEl = readElOpt(doc, "SIGNSTREAM-DATABASE").orElse(null);
        if (!expect(rootEl != null, 10000, "Missing SIGNSTREAM-DATABASE element.", null))
            return returns;

        var version = readOpt(rootEl, "DATABASE-VERSION").orElse(null);
        if (!expect(isNotBlank(version), 10001, "Database missing DATABASE-VERSION.", null))
            return returns;
        
        ret = validateHeader();
        if (ret != null)
        	return ret;

        ret = validateReferences(); // including participants/annotators
        if (ret != null)
        	return ret;

        ret = validateMediaFiles();
        if (ret != null)
        	return ret;

        ret = validateGraphFiles();
        if (ret != null)
        	return ret;
        
        validateLocalCodingScheme();

        this.dbEl = readChildOpt(rootEl, "DATABASE").orElse(null);
        if (!expect(dbEl != null, 10002, "Missing DATABASE element.", null))
            return returns;

        this.glossWindowEl = readChildOpt(dbEl, "GLOSS-WINDOW").orElse(null);
        if (!expect(glossWindowEl != null, 10003, "Missing GLOSS-WINDOW element.", null))
            return returns;
        
        populateRefEntities();
        
        correctDuplicates();

        correctDanglingEntities();
        
        	correctRemoveOutOfBoundsEntities();
        
        	// rounds down utterance times, and subtracts 1 frame due to past video player issues
        ret = validateUtterances();	// including partitions/segments/utterances
        if (ret != null)
        	return ret;

        	// rounds down entity item time, and subtracts 1 frame due to past video player issues
        ret = validateTracks();
        if (ret != null)
            return ret;
        
        correctDepdendentsProperties();

        correctSignTypesOrTwoHanded();
        
        correctRemoveInvalidLocationEntities();

//        correctTranslations();

        // Corrections for parent entity relationships
        correctParentEntities();        
        
        	validateParticipantDominance();
        
        validateAndCorrectDependents();
        
        correctMultiItemEntityTimes();
        
        validateReversedItemTimes();
        
        correctEntityIDs();
        
        correctUtteranceOverlaps();
        
        correctTranslations();	// call this after utterance overlaps are fixed

        populateEntityIntervals();
        
        correctEntityOverlaps();
        
        correctMultiItemEntityTimes();		// calling for second time...

        return returns;
    }

    private SS3ValidatorConverterReturns validateHeader() {
        expect(readOpt(rootEl, "SIGNSTREAM-VERSION"), 10004, "Database missing SIGNSTREAM-VERSION", null);
        
        var dbVersion = readOpt(rootEl, "DATABASE-VERSION").orElse(null);
        if (expect(isNotBlank(dbVersion), 10001, "Database missing DATABASE-VERSION.", null))
            expect(dbVersion.equals("2.0") || dbVersion.equals("3.0"), 10005, "Unexpected DATABASE-VERSION " + dbVersion, null);
        
        expect(readOpt(rootEl, "EXCERPT"), 10006, "Database missing EXCERPT.", null);
        expect(readOpt(rootEl, "EXCERPT_NUMBER"), 10007, "Database missing EXCERPT_NUMBER.", null);
        
        return null;
    }
    
    private boolean addDefaultParticipant(Element participantsEl) {
    	if (addedDefaultParticipantToCollection)
    		return false;
    	
    	Element newParticipantEl = doc.createElement("REF");
    	
    	write(newParticipantEl, "BACKGROUND", "");
    	write(newParticipantEl, "COMMENTS", "");
    	write(newParticipantEl, "DOMINANT_HAND", "R");
    	write(newParticipantEl, "FULL_NAME", DEFAULT_PARTICIPANT_FULL_NAME);
    	write(newParticipantEl, "GENDER", "M");
    	write(newParticipantEl, "ID", DEFAULT_PARTICIPANT_ID);
    	write(newParticipantEl, "LABEL", DEFAULT_PARTICIPANT_LABEL);
    	write(newParticipantEl, "NATIVE_LANGUAGE", "ASL");
    	write(newParticipantEl, "PARENT_INFO", "");
    	write(newParticipantEl, "YEAR_OF_BIRTH", "");
    	
    	participantsEl.appendChild(newParticipantEl);
    	
    	participantElements.put(DEFAULT_PARTICIPANT_ID, participantsEl);
    	
    	addedDefaultParticipantToCollection = true;
    	
    	return true;
    }
    
    private SS3ValidatorConverterReturns validateReferences() {
        var refsEl = readChildOpt(rootEl, "REFERENCES").orElse(null);
        
        if (expect(refsEl != null, 10008, "Missing REFERENCES element.", null)) {
        	
    		ParticipantsLibrary participantsLibrary = SS3Singleton.getManager().getLoadedParticipantsLibrary();
    		ParticipantsLibrary annotatorsLibrary = SS3Singleton.getManager().getLoadedAnnotatorsLibrary();
        	
            // Validate participants
            var participantsEl = readChildOpt(refsEl, "PARTICIPANTS").orElse(null);
            if (expect(participantsEl != null, 10009, "Missing PARTICIPANTS element.", null)) {
            		ArrayList<Element> participantsToDelete = new ArrayList<Element>();
            		
                for (Element pEl: readChildren(participantsEl, "REF").collect(Collectors.toList())) {
                    var repr = "Participant " + repr(pEl, "LABEL");
                    
                    var id = readOpt(pEl, "ID").orElse(null);
                    if (isBlank(id)) {
                    	addToModifications(20000, "Participant with missing ID was dropped.", null);
                    	participantsEl.removeChild(pEl);
                    	continue;
                    }
                    
                    var label = readOpt(pEl, "LABEL").orElse(null);
                    var name = readOpt(pEl, "FULL_NAME").orElse(null);
                    var domHand =  readOpt(pEl, "DOMINANT_HAND").orElse(null);

                    if (participantElements.containsKey(id)) {
                    	var existingParticipantEl = participantElements.get(id);
                    	
                    	boolean different = false;
                    	
                    	var existingLabel = readOpt(existingParticipantEl, "LABEL").orElse(null);
                    	var existingName = readOpt(existingParticipantEl, "FULL_NAME").orElse(null);
                    	var existingDomHand = readOpt(existingParticipantEl, "DOMINANT_HAND").orElse(null);
                    	
                    	if (isNotBlank(existingLabel) && !existingLabel.equals(label))
                    		different = true;
                    	else if (isBlank(existingLabel) && isNotBlank(label))
                    		different = true;
                    	
                    	if (isNotBlank(existingName) && !existingName.equals(name))
                    		different = true;
                    	else if (isBlank(existingName) && isNotBlank(name))
                    		different = true;
                    	
                    	if (isNotBlank(existingDomHand) && !existingDomHand.equals(domHand))
                    		different = true;
                    	else if (isBlank(existingDomHand) && isNotBlank(domHand))
                    		different = true;
                    	
                    	if (different) {	// ask to choose between the two collection versions
                    		addToErrors(10010, "A participant shares duplicate ID " + id + " with another participant in the collection.", "There may exist duplicate participants.", false);
                    		
                    		String warningDialogText = "The collection you are opening has 2 participants which share an identifier but their information differs.\n"
                    				+ "Please choose which of the two below participants are correct, and the other will be ignored.\n\n"
                    				+ "Version 1: {Label: " + label + ",  Full Name: " + name + ", Dominant Hand: " + domHand + "}\n"
                    				+ "Version 2: {Label: " + existingLabel + ",  Full Name: " + existingName + ", Dominant Hand: " + existingDomHand + "}";
                    		String warningTitleText = "Participant Check";
                    		int optionType = JOptionPane.YES_NO_OPTION;
                    		int messageType = JOptionPane.ERROR_MESSAGE;
                    		String options[] = {"Version 1", "Version 2"};	// version1 == the 'ok' option, version2 == the 'cancel' option

                    		try {
                    			int choice = JOptionPane.showOptionDialog(SS3Singleton.getSignStreamApplication(),
                    					warningDialogText,
                    					warningTitleText,
                    					optionType,
                    					messageType,
                    					null,
                    					options,
                    					options[0]);

                    			if (choice == JOptionPane.OK_OPTION) {	// choice == version1, delete version2
                    				participantsToDelete.add(existingParticipantEl);
                    				
                    				addToModifications(20001, "The collection file contained two participants with the same ID, and one was chosen by the user as the definitive version over the other.", null);
                    			} else { // choice == JOptionPane.CANCEL_OPTION == version2, delete version1
                    				participantsToDelete.add(pEl);

                    				addToModifications(20001, "The collection file contained two participants with the same ID, and one was chosen by the user as the definitive version over the other.", null);
                    			}
                    		} catch (HeadlessException he) {
                    			System.out.println("");
                    		}
                    	} else {
                    		addToModifications(20002, "Participant " + id + " had completely duplicated information as another participant and it was removed.", null);
                    		participantsToDelete.add(pEl);
                    		continue;
                    	}
                    }
                    
                    SS3Participant xmlParticipant = participantsLibrary.getParticipant(id);
                    if (xmlParticipant != null && isNotBlank(xmlParticipant.getId())) {
                        if (participantsLibrary.isLocalParticipant(id)) {	// isLocal
                        	String localLabel = xmlParticipant.getLabel();
                        	String localName = xmlParticipant.getFullName();
                        	String localDomHand = xmlParticipant.getDomHand();
                        	
                        	boolean different = false;

                        	if (isNotBlank(localLabel) && !localLabel.equals(label))
                        		different = true;
                        	else if (isBlank(localLabel) && isNotBlank(label))
                        		different = true;
                        	
                        	if (isNotBlank(localName) && !localName.equals(name))
                        		different = true;
                        	else if (isBlank(localName) && isNotBlank(name))
                        		different = true;
                        	
                        	if (isNotBlank(localDomHand) && !localDomHand.equals(domHand))
                        		different = true;
                        	else if (isBlank(localDomHand) && isNotBlank(domHand))
                        		different = true;
                        	
                        	if (different) {
                            	addToErrors(10011, repr + " shares duplicate ID " + id + " with a local XML participant.", "A participant in the collection needs to be reconciled with a locally defined participant.", false);
                            	
                        		String warningDialogText = "The collection you are opening has a participant which shares an ID with a locally defined participant.\n"
                        				+ "Please choose which of the two below participants are correct, and the other will be overriden.\n\n"
                        				+ "Collection Version: {Label: " + label + ",  Full Name: " + name + ", Dominant Hand: " + domHand + "}\n"
                        				+ "Local Version: {Label: " + localLabel + ",  Full Name: " + localName + ", Dominant Hand: " + localDomHand + "}";
		        				String warningTitleText = "Participant Check";
		        				int optionType = JOptionPane.YES_NO_OPTION;
		        				int messageType = JOptionPane.ERROR_MESSAGE;
		        				String options[] = {"Collection Version", "Local Version"};	// collection == the 'ok' option, local == the 'cancel' option
		        				
		        				try {
		        					int choice = JOptionPane.showOptionDialog(SS3Singleton.getSignStreamApplication(),
		        												 warningDialogText,
		        												 warningTitleText,
		        												 optionType,
		        												 messageType,
		        												 null,
		        												 options,
		        												 options[0]);
		        					
		        					if (choice == JOptionPane.OK_OPTION) {	// choice == collection version, overwrite local xml version
		        						xmlParticipant.setLabel(label);
		        						xmlParticipant.setFullName(name);
		        						xmlParticipant.setDomHand(domHand);
		        						
		        						UtilityFilesXMLWriter writer = new edu.bu.signstream.common.db.UtilityFilesXMLWriter();
		        						try {
		        							writer.saveParticipantsLibrary(participantsLibrary);
		        						} catch (Exception ex) {
		        							ex.printStackTrace();
		        							SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);
		        						}
		        						
		        						addToModifications(20003, "The user chose to keep the collection defined version of partcipant ID: " + id + " over the locally defined version. The localParticipants.xml file was edited to reflect this.", null);
		        					} else { // choice == JOptionPane.CANCEL_OPTION == local version, overwrite collection version
		        						write(pEl, "LABEL", localLabel);
		        						write(pEl, "FULL_NAME", localName);
		        						write(pEl, "DOMINANT_HAND", localDomHand);
		        						
		        						addToModifications(20004, "The user chose to keep a locally defined participant ID: " + id + " over the collection defined version. The collection file was edited to reflect this.", null);
		        					}
		        				} catch (HeadlessException he) {
		        					System.out.println("");
		        				}
                        	} // else case means the collection version completely matches the local xml version, which is fine and correct
                        } else {	// isGlobal
                        	String globalLabel = xmlParticipant.getLabel();
                        	String globalName = xmlParticipant.getFullName();
                        	String globalDomHand = xmlParticipant.getDomHand();
                        	
                        	boolean different = false;

                        	if (isNotBlank(globalLabel) && !globalLabel.equals(label))
                        		different = true;
                        	else if (isBlank(globalLabel) && isNotBlank(label))
                        		different = true;
                        	
                        	if (isNotBlank(globalName) && !globalName.equals(name))
                        		different = true;
                        	else if (isBlank(globalName) && isNotBlank(name))
                        		different = true;
                        	
                        	if (isNotBlank(globalDomHand) && !globalDomHand.equals(domHand))
                        		different = true;
                        	else if (isBlank(globalDomHand) && isNotBlank(domHand))
                        		different = true;
                        	
                        	if (different) {
                            	addToErrors(10012, repr + " shares duplicate ID " + id + " with a global XML participant.", 
                            			"A participant in the collection has irreconcilable differences between it and a globally defined version. "
                            			+ "The collection participant will have its information overriden by the global participant information.", false);
                            	
                        		String warningDialogText = "The collection you are opening has a participant which shares an ID with a globally defined participant.\n"
                        				+ "This information for the below participant from the collection:\n"
                        				+ "Collection Version: {Label: " + label + ",  Full Name: " + name + ", Dominant Hand: " + domHand + "}\n"
                        				+ "Will be updated to match the information of the globally defined participant below:\n"
                        				+ "Global Version: {Label: " + globalLabel + ",  Full Name: " + globalName + ", Dominant Hand: " + globalDomHand + "}";
		        				String warningTitleText = "Global Participant Override";
		        				int optionType = JOptionPane.OK_OPTION;
		        				int messageType = JOptionPane.ERROR_MESSAGE;
		        				String options[] = {"OK"};
		        				
		        				try {
		        					JOptionPane.showOptionDialog(SS3Singleton.getSignStreamApplication(),
		        												 warningDialogText,
		        												 warningTitleText,
		        												 optionType,
		        												 messageType,
		        												 null,
		        												 options,
		        												 options[0]);

		        					write(pEl, "LABEL", globalLabel);
		        					write(pEl, "FULL_NAME", globalName);
		        					write(pEl, "DOMINANT_HAND", globalDomHand);
		        					
		        					addToModifications(20005, "The globally defined version of participant " + id + " is overwriting the collection defined version.", null);
		        				} catch (HeadlessException he) {
		        					System.out.println("");
		        				}
                        	}
                        }
                    }
                    
                    if (!expect(isNotBlank(label), 10013, "Participant " + id +" missing LABEL.", "A participant is missing a label.", false)) {
                    	write(pEl, "LABEL", id + PLACEHOLDER_PARTICIPANT_LABEL);
                    	addToModifications(20006, "Participant " + id + " was missing a label and was set to " + repr(id + PLACEHOLDER_PARTICIPANT_LABEL), 
                    			"A participant was missing a label, so it was given the label of: " + repr(id + PLACEHOLDER_PARTICIPANT_LABEL));
                    }
                    
                    if (!expect(isNotBlank(name), 10014, "Participant " + id + " missing FULL_NAME.", "A participant is missing a full name.", false)) {
                    	write(pEl, "FULL_NAME", PLACEHOLDER_PARTICIPANT_FULLNAME);
                    	addToModifications(20007, "Participant " + id + " was missing a full name and was set to " + repr(id + PLACEHOLDER_PARTICIPANT_FULLNAME), 
                    			repr + " was missing a full name, so it was given the full name: " + repr(id + PLACEHOLDER_PARTICIPANT_FULLNAME));
                    }
                    
                    if (isNotBlank(domHand)) {
                    	if (!expect(domHand.equals("L") || domHand.equals("R"), 10015, "Participant " + id + " has invalid DOMINANT_HAND.", repr + " does not have a dominant hand specified.", false)) {
                        	write(pEl, "DOMINANT_HAND", PLACEHOLDER_PARTICIPANT_DOMINANT_HAND);
                        	addToModifications(20008, "Participant " + id + " was missing a dominant hand value and was set to " + repr(PLACEHOLDER_PARTICIPANT_DOMINANT_HAND), 
                        			repr + " was missing a setting for dominant hand, so it was given the dominant hand: " + repr(PLACEHOLDER_PARTICIPANT_DOMINANT_HAND));
                    	}
                    }

                    participantElements.put(id, pEl);
                }
                
                for (Element e: participantsToDelete) {
                	participantsEl.removeChild(e);
                }
                
                if (participantElements.size() == 0) {
                	// insert the information of the default participant from the global file
                	addDefaultParticipant(participantsEl);
                	
                	var str = "The default participant " + repr(DEFAULT_PARTICIPANT_LABEL) + " was added because there were none existing.";
                	addToModifications(20009, str, str);
                }
            }

            // Validate annotators
            var annotatorsEl = readChildOpt(refsEl, "ANNOTATORS").orElse(null); 
            if (expect(annotatorsEl != null, 10016, "Missing ANNOTATORS element.", null)) {
        			ArrayList<Element> annotatorsToDelete = new ArrayList<Element>();
        		
                for (Element aEl: readChildren(annotatorsEl, "REF").collect(Collectors.toList())) {
                    var repr = "Annotator " + repr(aEl, "LABEL");
                    
                    var id = readOpt(aEl, "ID").orElse(null);
                    if (isBlank(id)) {
                    	addToModifications(20010, "Annotator with missing ID was dropped.", null);
                    	continue;
                    }

                    var label = readOpt(aEl, "LABEL").orElse(null);
                    if (annotatorElements.containsKey(id)) {
                    	var existingAnnotatorEl = annotatorElements.get(id);
                    	
                    	boolean different = false;
                    	
                    	var existingLabel = readOpt(existingAnnotatorEl, "LABEL").orElse(null);
                    	
                    	if (isNotBlank(existingLabel) && !existingLabel.equals(label))
                    		different = true;
                    	else if (isBlank(existingLabel) && isNotBlank(label))
                    		different = true;
                    	
                    	if (different) {	// ask to choose between the two collection versions
                    		addToErrors(10017, "An annotator shares duplicate ID " + id + "  with another annotator in the collection.", "There may exist duplicate annotators.", false);
                    		
                    		String warningDialogText = "The collection you are opening has 2 participants which share an identifier but their information differs.\n"
                    				+ "Please choose which of the two below participants are correct, and the other will be ignored.\n\n"
                    				+ "Version 1: {Label: " + label + "}\n"
                    				+ "Version 2: {Label: " + existingLabel + "}";
                    		String warningTitleText = "Participant Check";
                    		int optionType = JOptionPane.YES_NO_OPTION;
                    		int messageType = JOptionPane.ERROR_MESSAGE;
                    		String options[] = {"Version 1", "Version 2"};	// version1 == the 'ok' option, version2 == the 'cancel' option

                    		try {
                    			int choice = JOptionPane.showOptionDialog(SS3Singleton.getSignStreamApplication(),
                    					warningDialogText,
                    					warningTitleText,
                    					optionType,
                    					messageType,
                    					null,
                    					options,
                    					options[0]);

                    			if (choice == JOptionPane.OK_OPTION) {	// choice == version1, delete version2
                    				annotatorsToDelete.add(existingAnnotatorEl);
                    				
                    				addToModifications(20011, "The collection file contained two annotators with the same ID, and one was chosen by the user as the definitive version over the other.", null);
                    			} else { // choice == JOptionPane.CANCEL_OPTION == version2, delete version1
                    				annotatorsToDelete.add(aEl);

                    				addToModifications(20011, "The collection file contained two annotators with the same ID, and one was chosen by the user as the definitive version over the other.", null);
                    			}
                    		} catch (HeadlessException he) {
                    			System.out.println("");
                    		}
                    	} else {
                    		addToModifications(20012, "Annotator " + id + " had completely duplicated information as another annotator and it was removed.", null);
                    		annotatorsToDelete.add(aEl);
                    		continue;
                    	}
                    }
                    
                    SS3Participant xmlAnnotator = annotatorsLibrary.getParticipant(id);
                    if (xmlAnnotator != null && isNotBlank(xmlAnnotator.getId())) {
                        if (participantsLibrary.isLocalParticipant(id)) {	// isLocal
                        	String localLabel = xmlAnnotator.getLabel();
                        	
                        	boolean different = false;

                        	if (isNotBlank(localLabel) && !localLabel.equals(label))
                        		different = true;
                        	else if (isBlank(localLabel) && isNotBlank(label))
                        		different = true;
                        	
                        	if (different) {
                            	addToErrors(10018, repr + " shares duplicate ID " + id + " with a local XML annotator.", "An annotator in the collection needs to be reconciled with a locally defined annotator.", false);
                            	
                        		String warningDialogText = "The collection you are opening has an annotator which shares an ID with a locally defined annotator.\n"
                        				+ "Please choose which of the two below annotators are correct, and the other will be overriden.\n\n"
                        				+ "Collection Version: {Label: " + label + "}\n"
                        				+ "Local Version: {Label: " + localLabel + "}";
		        				String warningTitleText = "Annotator Check";
		        				int optionType = JOptionPane.YES_NO_OPTION;
		        				int messageType = JOptionPane.ERROR_MESSAGE;
		        				String options[] = {"Collection Version", "Local Version"};	// collection == the 'ok' option, local == the 'cancel' option
		        				
		        				try {
		        					int choice = JOptionPane.showOptionDialog(SS3Singleton.getSignStreamApplication(),
		        												 warningDialogText,
		        												 warningTitleText,
		        												 optionType,
		        												 messageType,
		        												 null,
		        												 options,
		        												 options[0]);
		        					
		        					if (choice == JOptionPane.OK_OPTION) {	// choice == collection version, overwrite local xml version
		        						xmlAnnotator.setLabel(label);
		        						UtilityFilesXMLWriter writer = new edu.bu.signstream.common.db.UtilityFilesXMLWriter();
		        						try {
		        							writer.saveAnnotatorsLibrary(annotatorsLibrary);
		        						} catch (Exception ex) {
		        							ex.printStackTrace();
		        							SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);
		        						}
		        						
		        						addToModifications(20013, "The user chose to keep the collection defined version of annotator ID: " + id + " over the locally defined version. The localAnnotators.xml file was edited to reflect this.", null);
		        					} else { // choice == JOptionPane.CANCEL_OPTION == local version, overwrite collection version
		        						write(aEl, "LABEL", localLabel);
		        						
		        						addToModifications(20014, "The user chose to keep a locally defined annotator ID: " + id + " over the collection defined version. The collection file was edited to reflect this.", null);
		        					}
		        				} catch (HeadlessException he) {
		        					System.out.println("");
		        				}
                        	} // else case means the collection version completely matches the local xml version, which is fine and correct
                        } else {	// isGlobal
                        	String globalLabel = xmlAnnotator.getLabel();
                        	
                        	boolean different = false;

                        	if (isNotBlank(globalLabel) && !globalLabel.equals(label))
                        		different = true;
                        	else if (isBlank(globalLabel) && isNotBlank(label))
                        		different = true;
                        	
                        	if (different) {
                            	addToErrors(10019, repr + " shares duplicate ID " + id + " with a global XML annotator.", 
                            			"An annotator in the collection has irreconcilable differences between it and a globally defined version. "
                            			+ "The collection annotator will have its information overriden by the global annotator information.", false);
                            	
                        		String warningDialogText = "The collection you are opening has a participant which shares an ID with a globally defined participant.\n"
                        				+ "This information for the below participant from the collection:\n"
                        				+ "Collection Version: {Label: " + label + "}\n"
                        				+ "Will be updated to match the information of the globally defined participant below:\n"
                        				+ "Global Version: {Label: " + globalLabel + "}";
		        				String warningTitleText = "Global Participant Override";
		        				int optionType = JOptionPane.OK_OPTION;
		        				int messageType = JOptionPane.ERROR_MESSAGE;
		        				String options[] = {"OK"};
		        				
		        				try {
		        					JOptionPane.showOptionDialog(SS3Singleton.getSignStreamApplication(),
		        												 warningDialogText,
		        												 warningTitleText,
		        												 optionType,
		        												 messageType,
		        												 null,
		        												 options,
		        												 options[0]);

		        					write(aEl, "LABEL", globalLabel);
		        					
		        					addToModifications(20015, "The globally defined version of annotator " + id + " is overwriting the collection defined version.", null);
		        				} catch (HeadlessException he) {
		        					System.out.println("");
		        				}
                        	}
                        }
                    }
                    
                    if (!expect(isNotBlank(label), 10020, "Annotator " + id + " missing LABEL.", "An annotator is missing a label.", false)) {
                    	write(aEl, "LABEL", id + PLACEHOLDER_ANNOTATOR_LABEL);
                    	addToModifications(20016, "Annotator " + id + " was missing a label and was set to " + repr(id + PLACEHOLDER_ANNOTATOR_LABEL), 
                    			"An annotator was missing a label, so it was given the label of: " + repr(id + PLACEHOLDER_ANNOTATOR_LABEL));
                    }

                    annotatorElements.put(id, aEl);
                }
                
                for (Element e: annotatorsToDelete) {
                	annotatorsEl.removeChild(e);
                }
                
                if (annotatorElements.size() == 0) {
                	// insert the information of the default annotator from the global file
                	Element newAnnotatorEl = doc.createElement("REF");
                	
                	write(newAnnotatorEl, "ID", DEFAULT_ANNOTATOR_ID);
                	write(newAnnotatorEl, "LABEL", DEFAULT_ANNOTATOR_LABEL);
                	
                	addedDefaultAnnotatorToCollection = true;
                	
                	annotatorsEl.appendChild(newAnnotatorEl);
                	
                	var str = "The default annotator " + repr(DEFAULT_ANNOTATOR_LABEL) + " was added because there were none existing.";
                	addToModifications(20017, str, str);
                }
            }
        }

    	return null;
    }

    private SS3ValidatorConverterReturns validateMediaFiles() {
        var mediaFilesEl = readChildOpt(rootEl, "MEDIA-FILES").orElse(null);
        var medias = new HashMap<String, Element>();

        if (expect(mediaFilesEl != null, 10021, "Missing MEDIA-FILES element.", null)) {
            var timebase = readLongOpt(mediaFilesEl, "MOVIE_TIMESCALE").orElse(null);
            var timestep = readLongOpt(mediaFilesEl, "PACKET_DURATION").orElse(null);
            var duration = readLongOpt(mediaFilesEl, "MOVIE_DURATION").orElse(null);

            var missingTimebase = timebase == null || timebase == 0;
            var missingTimestep = timestep == null || timestep == 0;
            var missingDuration = duration == null || duration == 0;

            if ((missingTimebase || missingTimestep || missingDuration) && mediaFilesEl.hasChildNodes()) {
            	var mediaFilePath = readChildren(mediaFilesEl, "MEDIA-FILE").findFirst().get().getAttribute("PATH");
                var file = new File(mediaFilePath);
                if (!file.exists()) {
                	File origFile = file;
                	while (file == null || !file.exists()) {
                		JOptionPane.showMessageDialog(
                                null,
                                "The collection contains invalid movie file properties, however SignStream could not locate the\n"
                                + "movie file on your system to update this information.\n\n" +
                                    "After pressing OK, please locate the following movie file in the selection dialog:\n\n" +
                                    "\"" + origFile + "\"",
                                "Error Correction",
                                JOptionPane.INFORMATION_MESSAGE);

                            file = UserInterfaceUtil.selectFile(
                                null,
                                origFile,
                                null,
                                UserInterfaceUtil.VIDEO_FILES_FILTER,
                                "Locate " + origFile.getName());
                	}

                    if (file == null || !file.exists()) {
                        if (missingTimebase) addToErrors(10022, "Media files missing MOVIE_TIMESCALE.", null);
                        if (missingTimestep) addToErrors(10023, "Media files missing PACKET_DURATION.", null);
                        if (missingDuration) addToErrors(10024, "Media files missing MOVIE_DURATION.", null);

                        return returns;
                    }
                    
                    // add selected video to the collection file so it does not need to be selected again after validation/re-parsing completes
                    Element firstMediaElement = readChildren(mediaFilesEl, "MEDIA-FILE").findFirst().get();
                    write(firstMediaElement, "PATH", file.getAbsolutePath());
                }

                var caps = GstProbe.probe(file.toURI(), "video/x-raw");

                var fr = (Fraction)caps.get("framerate");
                var dur = (Duration)caps.get("duration");

                timestep = (long)fr.getDenominator();
                timebase = (long)fr.getNumerator();
                var framerate = (double)timebase / timestep;

                var unit = new TimeUnit("temp", "", TimeUnit.Type.FRAME, framerate);
                duration = dur.as(unit, Rounding.FLOOR) * timestep;

                writeLong(mediaFilesEl, "MOVIE_TIMESCALE", timebase.intValue());
                writeLong(mediaFilesEl, "PACKET_DURATION", timestep.intValue());
                writeLong(mediaFilesEl, "MOVIE_DURATION", duration);

                addToModifications(20018, "MOVIE_TIMESCALE recalculated to " + repr("" + timebase) + " because timescale, duration, and duration were blank.", null);
                addToModifications(20019, "PACKET_DURATION recalculated to " + repr("" + timestep) + " because timescale, duration, and duration were blank.", null);
                addToModifications(20020, "MOVIE_DURATION recalculated to " + repr("" + duration) + " because timescale, duration, and duration were blank.", null);
            }

            this.timebase = timebase;
            this.timestep = timestep;
            this.duration = duration;
            
            SS3Singleton.getVideoControlManager()
            		.getZoomer()
            		.getCoordinateTimeConvertor()
            		.setTimeScale(this.timebase, this.timestep, this.duration);

            this.frameTimeUnit = new TimeUnit("temp", "", TimeUnit.Type.FRAME, (double) this.timebase / this.timestep);

            readChildren(mediaFilesEl, "MEDIA-FILE").forEach(mEl -> {
                var repr = "Media file " + repr(mEl, "ID");
                var id = readOpt(mEl, "ID").orElse(null);
                
                expect(isNotBlank(id), 10025, "Media file missing ID.", null);
                expect(!medias.containsKey(id), 10026, repr + " has duplicate ID.", null);

                var path = readOpt(mEl, "PATH").orElse(null);
                
                expect(isNotBlank(path), 10027, repr + " missing PATH.", null);
                test(() -> Path.of(path), 10028, repr + " has invalid PATH.", null);

                medias.put(id, mEl);
            });
        } else {
            return returns;
        }

    	return null;
    }

    private SS3ValidatorConverterReturns validateGraphFiles() {
        var graphFilesEl = readChildOpt(rootEl, "GRAPH-FILES").orElse(null);
        var graphs = new HashMap<String, Element>();
        
        if (graphFilesEl != null) {
            readChildren(graphFilesEl, "GRAPH-FILE").forEach(gEl -> {
                var repr = "Graph file " + repr(gEl, "ID");
                var id = readOpt(gEl, "ID").orElse(null);
                
                expect(isNotBlank(id), 10029, "Graph file missing ID.", null);
                expect(!graphs.containsKey(id), 10030, repr + " has duplicate ID.", null);

                expect(readOpt(gEl, "NAME"), 10031, repr + " missing NAME.", null);

                var path = readOpt(gEl, "PATH").orElse(null);
                
                expect(isNotBlank(path), 10032, repr + " missing PATH.", null);
                test(() -> Path.of(path), 10033, repr + " has invalid PATH.", null);

                graphs.put(id, gEl);
            });
        }
        
    	return null;
    }
    
    private void validateLocalCodingScheme() {
        var localSchemeEl = readChildOpt(rootEl, "LOCAL-CODING-SCHEME").orElse(null);
        localSchemeFieldElements = new HashMap<String, Element>();
        localSchemeFieldValues = new HashMap<String, HashMap<String, Element>>();

        if (expect(localSchemeEl != null, 10034, "Missing LOCAL-CODING-SCHEME element.", null)) {
    		for (Element fEl: readChildren(localSchemeEl, "FIELD").collect(Collectors.toCollection(ArrayList::new))) {
                var repr = "Local coding scheme field " + repr(fEl, "ID");
                var id = readOpt(fEl, "ID").orElse(null);
                if ("36".equals(id)) {
                	continue;		// this is a hopefully temporary hack -- field id 36 was deleted between 3.3.5 and future versions
                }

    			var type = readOpt(fEl, "TYPE").orElse(null);

                if (expect(isNotBlank(type), 10035, repr + " missing TYPE.", null)) {
	        			if ( ! Constants.EDITABLE_TYPE_VALUES.contains(type)) {
	                		continue;		// handled in conversion
	        			}
                }
                
                SS3Field defaultField3_3_5 = defScheme3_3_5.getField(id);

                var group = readOpt(fEl, "GROUP").orElse(null);
                
                var alignment = readOpt(fEl, "ALIGNMENT").orElse(null);
                var label = readOpt(fEl, "LABEL").orElse(null);
                var name = readOpt(fEl, "NAME").orElse(null);
                var prefix = readOpt(fEl, "PREFIX").orElse(null);

                if (isBlank(group) && defaultField3_3_5 != null) {
                	var defGroup = defaultField3_3_5.getGroup();
                	if (isNotBlank(defGroup) && ! defGroup.isBlank()) {
                		// write out the proper group regardless
                		write(fEl, "GROUP", defGroup);
                		group = defGroup;
                		
                		// but if the field differs in any significant way from the defeault, inform the user of the change
//                		var defAlignment = defaultField3_3_5.getAlignment().getLabel();
//                		boolean alignmentEquals = defAlignment.equals(alignment);
                		// above is commented because alignment is not found in the coding scheme
                		var defLabel = defaultField3_3_5.getLabel();
                		boolean labelEquals = defLabel.equals(label);
                		var defName = defaultField3_3_5.getName();
                		boolean nameEquals = defName.equals(name);
                		var defPrefix = defaultField3_3_5.getPrefix();
                		boolean prefixEquals = defPrefix.equals(prefix);

                		var str = repr + " had GROUP updated from (null) to " + defGroup + " to match the default scheme field.";
                		var public_repr = "Local coding scheme field " + repr(fEl, "NAME");
                		String public_str = null;
                		
                		if (!labelEquals || !nameEquals || !prefixEquals) {
                			public_str = public_repr + " had GROUP updated from (null) to " + defGroup + " to match the default scheme field.";
                		}
                		addToModifications(20021, str, null);
                	}
                }
                
                expect(isNotBlank(group), 10036, repr + " missing GROUP.", null);

                var values = new HashMap<String, Element>();
                readChildren(fEl, "VALUE").forEach(vEl -> {
                    var vrepr = repr + " value " + repr(vEl, "ID");
                    var vid = readOpt(vEl, "ID").orElse(null);
                    
                    expect(isNotBlank(vid), 10037, repr + " value missing ID.", null);
                    expect(!values.containsKey(vid), 10038, vrepr + " has duplicate ID.", null);

                    expect(readOpt(vEl, "LABEL"), 10039, vrepr + " missing LABEL.", "A local field value for field " + name + " is missing a label.");
                    expect(readOpt(vEl, "NAME"), 10040, vrepr + " missing NAME.", "A local field value for field " + name + " is missing a name.");

                    readOpt(vEl, "IID").ifPresent(iid -> {});

                    values.put(vid, vEl);
                });

                localSchemeFieldValues.put(id, values);
                
                localSchemeFieldElements.put(id, fEl);
            }

    		String fieldIDBodyMvmt = "32";
    		String valueIDDown = "0";
    		String valueLabelDown = "down";

    		HashMap<String, Element> localFieldBodyMvmt = localSchemeFieldValues.get(fieldIDBodyMvmt);
    		if (localFieldBodyMvmt != null) {
    			Element localValueDown = localFieldBodyMvmt.get(valueIDDown);
    			var localLabel = readOpt(localValueDown, "LABEL").orElse(null);
    			if (isNotBlank(localLabel) && ! localLabel.equals(valueLabelDown)) {
    				// value was edited locally - do not overwrite values
    				modifyFieldBodyMvmtValue = false;
    			}
    		}
        }
    }
    
    private void populateRefEntities() {
        refEntities = new HashMap<String, String>();		// map entityID -> trackID
        
        var partitionsEl = readChildOpt(glossWindowEl, "TEMPORAL-PARTITIONS").orElse(null);
        if (expect(partitionsEl != null, 10041, "Missing TEMPORAL-PARTITIONS element.", null)) {
            readChildren(partitionsEl, "TEMPORAL-PARTITION").forEach(partitionEl -> {
                var repr = "Temporal Partition " + repr(partitionEl, "LABEL");
                var internal_repr = "Temporal Partition " + repr(partitionEl, "ID");
                var pid = readOpt(partitionEl, "ID").orElse(null);
                
                if (expect(isNotBlank(pid), 10042, "Temporal Partition missing ID.", null) && 
                		expect(!partitionElements.containsKey(pid), 10043, internal_repr + " has duplicate ID.", repr + " has duplicate ID."))
                	partitionElements.put(pid, partitionEl);

                expect(readOpt(partitionEl, "LABEL"), 10044, internal_repr + " missing LABEL.", "A temporal partition is missing a label.");

                readChildren(partitionEl, "TEMPORAL-PARTITION-SEGMENT").forEach(tierEl -> {
                    var trepr = repr + " Segment Tier " + repr(tierEl, "LABEL");
                    var internal_trepr = internal_repr + " Segment Tier " + repr(tierEl, "ID");
                    var tid = readOpt(tierEl, "ID").orElse(null);
                    
                    if (isNotBlank(tid) && isNotBlank(pid)) {
                    	tierPartitionMap.put(tid, pid);
                    }
                    
                    if (expect(isNotBlank(tid), 10045, internal_repr + " Segment Tier missing ID.", repr + " Segment Tier missing ID.") &&
                    		expect(!tierElements.containsKey(tid), 10046, internal_trepr + " has duplicate ID.", trepr + " has duplicate ID."))
                    	tierElements.put(tid, tierEl);
                    
                    String partition = null;
                    var partitionLabel = readOpt(partitionEl, "LABEL").orElse(null);
                    if (partitionLabel != null)
                    	partition = "Temporal Partition (" + partitionLabel + ") ";

                    expect(readOpt(tierEl, "LABEL"), 10047, internal_trepr + " missing LABEL.", "A segment tier is missing a label.");

                    // statement fields in newer collections - directly under tier
                    readChildren(tierEl, "STATEMENT-FIELD").forEach(stmtFieldEl -> {
                        var srepr = trepr + " statement field " + repr(stmtFieldEl, "FIELD-ID");
                        var internal_srepr = internal_trepr + "statement field " + repr(stmtFieldEl, "FIELD-ID");
                        var fid = readOpt(stmtFieldEl, "FIELD-ID").orElse(null);
                        
                        expect(isNotBlank(fid), 10048, internal_trepr + " statement field has missing ID.", trepr + " statement field has missing ID.");

                        var eids = readOpt(stmtFieldEl, "ENTITY-IDS").orElse(null);
                        if (expect(eids != null, 10049, internal_srepr + " missing ENTITY-IDS.", srepr + " missing ENTITY-IDS.")) {
                            if (!eids.equals("")) {
                                for (var eid : eids.split(", ")) {
                                	var existingFID = refEntities.get(eid);
                                	// do not consider entities in the same field as duplicates - they are just crossing utterance boundaries, which is acceptable going forward
                                	if (expect(existingFID == null || existingFID.equals(fid), 
                                			10050, "Duplicate entityID {ID=" + eid + "} detected in: " + internal_srepr + " already found in another segment tier statement field {ID=" + existingFID +"} ENTITY-IDS attribute value.", 
                                			"Duplicate entityID {ID=" + eid + "} detected in: " + srepr + " already found in another segment tier statement field {ID=" + existingFID +"} ENTITY-IDS attribute value.")) {
                                		refEntities.put(eid, fid);
                                	}
                                }
                            }
                        }
                    });
                    
                    readChildren(tierEl, "GLOSS-WINDOW-ITEM").forEach(utteranceEl -> {
                        var urepr = trepr + "Utterance " + repr(utteranceEl, "LABEL", "S", "E");
                        var internal_urepr = internal_trepr + " Utterance " + repr(utteranceEl, "ID");
                        var uid = readOpt(utteranceEl, "ID").orElse(null);
                        
                        if (isNotBlank(uid) && isNotBlank(tid)) {
                        	utteranceTierMap.put(uid, tid);
                        }
                        
                        if (expect(isNotBlank(uid), 10051, internal_trepr + " utterance missing ID.", trepr + " utterance missing ID.") &&
                        		expect(!utteranceElements.containsKey(uid), 10052, internal_urepr + " has duplicate ID.", urepr + " has duplicate ID."))
                        	utteranceElements.put(uid, utteranceEl);

                        expect(readOpt(utteranceEl, "LABEL"), 10053, internal_urepr + " has missing LABEL.", "An utterance is missing a label.");

                        // statement fields in older collections - directly under utterance
                        readChildren(utteranceEl, "STATEMENT-FIELD").forEach(stmtFieldEl -> {
                        	var srepr = urepr + " statement field " + repr(stmtFieldEl, "FIELD-ID");
                        	var internal_srepr = internal_urepr + " statement field " + repr(stmtFieldEl, "FIELD-ID");
                            var fid = readOpt(stmtFieldEl, "FIELD-ID").orElse(null);
                            
                            expect(isNotBlank(fid), 10054, internal_urepr + " statement field has missing ID.", urepr + " statement field missing ID.");

                            var eids = readOpt(stmtFieldEl, "ENTITY-IDS").orElse(null);
                            if (expect(eids != null, 10055, internal_srepr + " missing ENTITY-IDS.", srepr + " missing ENTITY-IDS.")) {
                                if (!eids.equals("")) {
                                    for (var eid : eids.split(", ")) {
                                    	var existingFID = refEntities.get(eid);
                                    	// do not consider entities in the same field as duplicates - they are just crossing utterance boundaries, which is acceptable going forward
                                    	if (expect(existingFID == null || existingFID.equals(fid), 
                                    			10056, "Duplicate entityID {ID=" + eid + "} detected in: " + internal_srepr + " already found in another utterance statement field {ID=" + existingFID +"} ENTITY-IDS attribute value.", 
                                    			"Duplicate entityID {ID=" + eid + "} detected in: " + srepr + " already found in another utterance statement field {ID=" + existingFID +"} ENTITY-IDS attribute value.")) {
                                    		refEntities.put(eid, fid);
                                    	}

                                        entityUtteranceMap.put(eid, uid);
                                    }
                                }
                            }
                        });
                    });
                });
            });
        }
        
        // commenting this out temporarily 
//        	if (tierPartitionMap.keySet().size() == 1) {
//        		onlyOneSegmentTier = true;
//        	} else {
//        		onlyOneSegmentTier = false;
//        	}
    }
    
    private void correctDuplicates() {
    	HashMap<Element, String> dupsToRemove = new HashMap<Element, String>();
    	
        var tracksEl = readChildOpt(dbEl, "TRACKS").orElse(null);
        for (Element tEl: readChildren(tracksEl, "TRACK").collect(Collectors.toList())) {
            var repr = "Track " + repr(tEl, "FID");
            var fid = readOpt(tEl, "FID").orElse(null);

            if (expect(isNotBlank(fid), 10057, "Track missing FID.", null) && 
            		expect(!trackElements.containsKey(fid), 10058, repr + " has duplicate FID.", null))
            	trackElements.put(fid, tEl);
            
            for (Element eEl: readChildren(tEl, "ENTITY").collect(Collectors.toList())) {
                var eid = readOpt(eEl, "ID").orElse(null);
                var erepr = repr + " entity " + repr(eEl, "ID");

                if (expect(isNotBlank(eid), 10059, repr + " entity missing ID.", null)) {
                    var refTrackID = refEntities.get(eid);
                    // checking refTrackID non-null because if it were null, it is handled below in correctDanglingEntities
                    if (isNotBlank(refTrackID) && !refTrackID.equals(fid)) {
                    	// refTrackID is correct/definitive - which means this track (fid and tEl) are not
                    	if (deletedEntities.get(eid) == null || ! deletedEntities.get(eid).equals(fid)) {
                    		deletedEntities.put(eid, fid);
                    		dupsToRemove.put(eEl, fid);
                    		addToModifications(20022, "Duplicate entityID " + eid + " was removed from track " + fid, null);
                    	}
                    } else {
                    	if (!entityElements.containsKey(eid)) {
                    		entityElements.put(eid, eEl);	// need to populate this here for all entities, even if they'll be removed in correctDanglingEntities
                    		entityTrackMap.put(eid, fid);
                    	} else {
                    		String existingFID = entityTrackMap.get(eid);
                    		if (!existingFID.equals(fid)) {
                        		deletedEntities.put(eid, fid);
                        		dupsToRemove.put(eEl, fid);	// see comment below
                    			addToModifications(20022, "Duplicate entityID " + eid + " was removed from track " + fid, null);
                    		} else {
                        		addToErrors(10060, erepr + " has duplicate ID within the same track.", null);		// odd case, not sure if fixable without guessing
                    		}
                    	}
                    }
            		}
            }
        }
        
        // doing this instead of just deleting in place above, because we are looping through the tracks above, so instead of deleting while looping do it here
        	for (Entry<Element, String> entry: dupsToRemove.entrySet()) {
        		Element eEl = entry.getKey();
        		String fid = entry.getValue();
        		trackElements.get(fid).removeChild(eEl);
        	}
    }

    private void correctDanglingEntities() {
        for (var entry : entityTrackMap.entrySet()) {
        	var deleted = new HashMap<String, String>();
        	
            var entityID = entry.getKey();
            var trackID = entry.getValue();
            
        	var entityEl = entityElements.get(entityID);
        	var trackEl = trackElements.get(trackID);
        	
        	if (entityEl == null)
        		continue;

        		// this means the entityID was not found in any statement field (thus, dangling), so we're deleting it and and anything related to it
            if (refEntities.get(entityID) == null) {
            	try {
            		trackEl.removeChild(entityEl);
            		entityElements.remove(entityID);
                	deleted.put(entityID, trackID);
            	} catch (Exception e) {
            		continue;		// means it was deleted in a different loop, as a dependent, so just continue to next loop
            	}

            	if (trackID.equals(Util.DOM_HAND_FIELD_ID) || trackID.equals(Util.NDOM_HAND_FIELD_ID)) {
            		Element referencingGlossEntityEl = null;
            		for (Element el: entityElements.values()) {
            			var depHandID = readOpt(el, "DEPENDENT_HAND_ID").orElse(null);
            			if (isNotBlank(depHandID) && depHandID.equals(entityID)) {
            				referencingGlossEntityEl = el;
            			}
            		}

            		if (referencingGlossEntityEl != null) {
            			var referencingEntityID = readOpt(referencingGlossEntityEl, "ID").orElse(null);
            			write(referencingGlossEntityEl, "DEPENDENT_HAND_ID", "");
            			write(referencingGlossEntityEl, "DEPENDENT_HAND_SHAPE_ID", "");
            			var public_entity = getEntityPublicErrorString(referencingEntityID, entityTrackMap.get(referencingEntityID));

//            			var str = "Gloss {" + referencingEntityID + "} had its DEPENDENT_HAND_ID and DEPENDENT_HAND_SHAPE_ID erased because the hand entity was deleted.";
            			var str = "Gloss {" + referencingEntityID + "} had its DEPENDENT_HAND_ID and DEPENDENT_HAND_SHAPE_ID erased.";
//            			var public_str = public_entity + " had its DEPENDENT_HAND_ID and DEPENDENT_HAND_SHAPE_ID erased since the hand entity was orphaned.";
            			addToModifications(20023, str, null);
            		}            		
            	}
            	
            	var dependentHandID = readOpt(entityEl, "DEPENDENT_HAND_ID").orElse(null);
            	var dependentHandshapeID = readOpt(entityEl, "DEPENDENT_HAND_SHAPE_ID").orElse(null);
            	var dependentLocationID = readOpt(entityEl, "DEPENDENT_LOCATION_ID").orElse(null);
            	var dependentSignTypeID = readOpt(entityEl, "DEPENDENT_SIGN_TYPE_ID").orElse(null);
            	var dependentGlossID = readOpt(entityEl, "DEPENDENT_GLOSS_ID").orElse(null);

            	// for all the below, if `trackEl.removeChild()` throws the error (which means the element was not in that track)
            	// then the next lines will not run, so only the tracks which correctly hold the entity will add to the deleted list
            	if (isNotBlank(dependentHandID) && refEntities.get(dependentHandID) == null) {
            		trackEl = trackElements.get(Util.DOM_HAND_FIELD_ID);
            		entityEl = entityElements.get(dependentHandID);
            		try {
            			trackEl.removeChild(entityEl);
            			deleted.put(dependentHandID, trackID);
            			entityElements.remove(dependentHandID);
            		} catch(Exception e) {
            			;
            		}

            		trackEl = trackElements.get(Util.NDOM_HAND_FIELD_ID);
            		entityEl = entityElements.get(dependentHandID);
            		try {
            			trackEl.removeChild(entityEl);
            			deleted.put(dependentHandID, trackID);
            			entityElements.remove(dependentHandID);
            		} catch(Exception e) {
            			;
            		}
            	}

            	if (isNotBlank(dependentHandshapeID) && refEntities.get(dependentHandshapeID) == null) {
            		trackEl = trackElements.get(Util.DOM_HANDSHAPE_FIELD_ID);
            		entityEl = entityElements.get(dependentHandshapeID);
            		try {
            			trackEl.removeChild(entityEl);
            			deleted.put(dependentHandshapeID, trackID);
            			entityElements.remove(dependentHandshapeID);
            		} catch(Exception e) {
            			;
            		}

            		trackEl = trackElements.get(Util.NDOM_HANDSHAPE_FIELD_ID);
            		entityEl = entityElements.get(dependentHandshapeID);
            		try {
            			trackEl.removeChild(entityEl);
            			deleted.put(dependentHandshapeID, trackID);
            			entityElements.remove(dependentHandshapeID);
            		} catch(Exception e) {
            			;
            		}
            	}

            	if (isNotBlank(dependentLocationID) && refEntities.get(dependentLocationID) == null) {
            		trackEl = trackElements.get(Util.DOM_LOCATION_FIELD_ID);
            		entityEl = entityElements.get(dependentLocationID);
            		try {
            			trackEl.removeChild(entityEl);
            			deleted.put(dependentLocationID, trackID);
            			entityElements.remove(dependentLocationID);
            		} catch(Exception e) {
            			;
            		}

            		trackEl = trackElements.get(Util.NDOM_LOCATION_FIELD_ID);
            		entityEl = entityElements.get(dependentLocationID);
            		try {
            			trackEl.removeChild(entityEl);
            			deleted.put(dependentLocationID, trackID);
            			entityElements.remove(dependentLocationID);
            		} catch(Exception e) {
            			;
            		}
            	}

            	if (isNotBlank(dependentSignTypeID) && refEntities.get(dependentSignTypeID) == null) {
            		trackEl = trackElements.get(Util.DOM_SIGN_TYPE_FIELD_ID);
            		entityEl = entityElements.get(dependentSignTypeID);
            		try {
            			trackEl.removeChild(entityEl);
            			deleted.put(dependentSignTypeID, trackID);
            			entityElements.remove(dependentSignTypeID);
            		} catch(Exception e) {
            			;
            		}

            		trackEl = trackElements.get(Util.NDOM_SIGN_TYPE_FIELD_ID);
            		entityEl = entityElements.get(dependentSignTypeID);
            		try {
            			trackEl.removeChild(entityEl);
            			deleted.put(dependentSignTypeID, trackID);
            			entityElements.remove(dependentSignTypeID);
            		} catch(Exception e) {
            			;
            		}
            	}

            	// special case for corresponding glosses - need to check if cleanup of 1H/2H necessary
            	if (isNotBlank(dependentGlossID)) {
            		if (trackID.equals(Util.DOM_GLOSS_FIELD_ID)) {
            			trackEl = trackElements.get(Util.NDOM_GLOSS_FIELD_ID);
            		} else if (trackID.equals(Util.NDOM_GLOSS_FIELD_ID)) {
            			trackEl = trackElements.get(Util.DOM_GLOSS_FIELD_ID);
            		}

            		var corrGlsEntityEl = entityElements.get(dependentGlossID);
            		if (corrGlsEntityEl != null) {
            			var corrGlsEntityID = readOpt(corrGlsEntityEl, "ID").orElse(null); 
            			if (refEntities.get(corrGlsEntityID) != null) {
            				var corrGlsTrackEl = trackElements.get(entityTrackMap.get(corrGlsEntityID));
            				var repr = "Track " + repr(corrGlsTrackEl, "FID");
            				var erepr = repr + " entity " + repr(corrGlsEntityEl, "ID");

            				var corrGlsDependentGlossID = readOpt(corrGlsEntityEl, "DEPENDENT_GLOSS_ID").orElse(null);
            				if (isNotBlank(corrGlsDependentGlossID)) {
            					// need the below check, as this scenario was encountered at least once:
            					// two glosses, both referencing dependent gloss with id XYZ
            					// gloss XYZ was referencing one of them as dependent, the other was orphaned/inert
            					if (refEntities.get(corrGlsDependentGlossID) == null) {
            						write(corrGlsEntityEl, "DEPENDENT_GLOSS_ID", "");
            						write(corrGlsEntityEl, "CORRESPONDING_GLOSS_ENTITY_ID", "");

//            						var str = erepr + " had dependent/corresponding gloss removed as the referenced gloss was deleted.";
            						var str = erepr + " had dependent/corresponding gloss removed.";
//            						var public_entity = getEntityPublicErrorString(corrGlsEntityID, entityTrackMap.get(corrGlsEntityID));
//            						var public_str = public_entity + " had dependent/corresponding gloss removed as the referenced gloss was deleted.";
            						addToModifications(20024, str, null);

            						var propertiesEl = readChildOpt(corrGlsEntityEl, "PROPERTIES").orElse(null);
            						if (propertiesEl != null) {
            							var noOfHands = readOpt(propertiesEl, "NO_OF_HANDS").orElse(null);
            							if ("2".equals(noOfHands)) {
            								write(propertiesEl, "NO_OF_HANDS", "1");

//            								str = erepr + " had number of hands updated to 1 as its dependent/corresponding gloss was deleted.";
            								str = erepr + " had number of hands updated to \"1\" as its dependent/corresponding gloss is orphaned and/or dropped.";
//            								public_str = public_entity + " had number of hands updated to 1 as its dependent/corresponding gloss was deleted.";
            								addToModifications(20025, str, null);
            							}
            						}
            					}
            				}
            			}	// else: it will get deleted on its own loop
            		}
            	}

                    // GD: not sure on this one
//                    if (refEntityID != null) {
//                    
//                    }
//                }

            }

            corrected += deleted.size();
            for (var e : deleted.entrySet()) {
                deletedEntities.put(e.getKey(), e.getValue());
            }
        }
        
        for (var entry : entityTrackMap.entrySet()) {
        		var deleted = new HashMap<String, String>();
        	
            var entityID = entry.getKey();
            var trackID = entry.getValue();
            
            var entityEl = entityElements.get(entityID);
            var trackEl = trackElements.get(trackID);

            if (entityEl == null)
        			continue;

    		// refEntities contained the id, but could still be an orphaned entity if nothing else references it
            if (refEntities.get(entityID) != null) {
            	// orphaned hand
            	if (trackID.equals(Util.DOM_HAND_FIELD_ID) || trackID.equals(Util.NDOM_HAND_FIELD_ID)) {
            		boolean orphaned = true;

            		for (Element el: entityElements.values()) {
            			var depHandID = readOpt(el, "DEPENDENT_HAND_ID").orElse(null);
            			if (isNotBlank(depHandID) && depHandID.equals(entityID)) {
            				orphaned = false;
            			}
            		}

            		if (orphaned) {
            			trackEl.removeChild(entityEl);
            			deleted.put(entityID, trackID);
            			entityElements.remove(entityID);
            		}
            	} else if (trackID.equals(Util.DOM_HANDSHAPE_FIELD_ID) || trackID.equals(Util.NDOM_HANDSHAPE_FIELD_ID)) {
            		boolean orphaned = true;

            		for (Element el: entityElements.values()) {
            			var depHSID = readOpt(el, "DEPENDENT_HAND_SHAPE_ID").orElse(null);
            			if (isNotBlank(depHSID) && depHSID.equals(entityID)) {
            				orphaned = false;
            			}
            		}

            		if (orphaned) {
            			trackEl.removeChild(entityEl);
            			deleted.put(entityID, trackID);
            			entityElements.remove(entityID);
            		}
            	} else if (trackID.equals(Util.DOM_LOCATION_FIELD_ID) || trackID.equals(Util.NDOM_LOCATION_FIELD_ID)) {
            		boolean orphaned = true;

            		for (Element el: entityElements.values()) {
            			var depHSID = readOpt(el, "DEPENDENT_LOCATION_ID").orElse(null);
            			if (isNotBlank(depHSID) && depHSID.equals(entityID)) {
            				orphaned = false;
            			}
            		}

            		if (orphaned) {
            			trackEl.removeChild(entityEl);
            			deleted.put(entityID, trackID);
            			entityElements.remove(entityID);
            		}
            	} else if (trackID.equals(Util.DOM_SIGN_TYPE_FIELD_ID) || trackID.equals(Util.NDOM_SIGN_TYPE_FIELD_ID)) {
            		boolean orphaned = true;

            		for (Element el: entityElements.values()) {
            			var depHSID = readOpt(el, "DEPENDENT_SIGN_TYPE_ID").orElse(null);
            			if (isNotBlank(depHSID) && depHSID.equals(entityID)) {
            				orphaned = false;
            			}
            		}

            		if (orphaned) {
            			trackEl.removeChild(entityEl);
            			deleted.put(entityID, trackID);
            			entityElements.remove(entityID);
            		}
            	}
            }

            corrected += deleted.size();
            for (var e : deleted.entrySet()) {
                deletedEntities.put(e.getKey(), e.getValue());
            }
        }
        
        if (corrected != 0) {
            addToModifications(20026, "Removed "+ corrected + " dangling entities not referenced by any STATEMENT-FIELD.", null);
        }
    }
    
    private void correctRemoveOutOfBoundsEntities() {
    	// entities which are partially or fully outside utterance bounds will be deleted
    	
    	// need to populate an utterance interval set with the initial times - this is not the global utteranceIntervals
    	HashMap<String, IntervalSet<String>> initialUtteranceIntervals = new HashMap<String, IntervalSet<String>>();	// key is segmentTierID
        var partitionsEl = readChildOpt(glossWindowEl, "TEMPORAL-PARTITIONS").orElse(null);
        if (partitionsEl != null) {
            readChildren(partitionsEl, "TEMPORAL-PARTITION").forEach(partitionEl -> {
                readChildren(partitionEl, "TEMPORAL-PARTITION-SEGMENT").forEach(tierEl -> {
                    var tid = readOpt(tierEl, "ID").orElse(null);

                    IntervalSet<String> tierUtterancesIntervalSet = initialUtteranceIntervals.get(tid);
                    if (tierUtterancesIntervalSet == null) {
                    	tierUtterancesIntervalSet = new IntervalSet<String>();
                    	initialUtteranceIntervals.put(tid, tierUtterancesIntervalSet);
                    }
                    
                	for (Element utteranceEl: readChildren(tierEl, "GLOSS-WINDOW-ITEM").collect(Collectors.toCollection(ArrayList::new))) {
                		var uid = readOpt(utteranceEl, "ID").orElse(null);
                		var start = readLongOpt(utteranceEl, "S").orElse(null);
                		var end = readLongOpt(utteranceEl, "E").orElse(null);
                		var interval = TimeRange.ABSOLUTE.time(start, TimeUnit.TIMESTAMP, Rounding.FLOOR)
                				.to(TimeRange.ABSOLUTE.time(end, TimeUnit.TIMESTAMP, Rounding.FLOOR));
                		tierUtterancesIntervalSet.add(uid, interval, Bound.EXCLUSIVE);
                	}
                });
            });
        }
    	
        var tracksEl = readChildOpt(dbEl, "TRACKS").orElse(null);
        var tracks = readChildren(tracksEl, "TRACK").collect(Collectors.toList());
        	for (Element tEl: tracks) {
            var repr = "Track " + repr(tEl, "FID");
            var fid = readOpt(tEl, "FID").orElse(null);

            var entities = readChildren(tEl, "ENTITY").collect(Collectors.toList());
            	for (Element eEl: entities) {
                var erepr = repr + " entity " + repr(eEl, "ID");
                var eid = readOpt(eEl, "ID").orElse(null);

                if (deletedEntities.get(eid) != null && deletedEntities.get(eid).equals(fid))
                    return;

                var items = readChildren(eEl, "ITEM").collect(Collectors.toList());
                var itemsList = new ArrayList<Element>();
                for (int i = 0; i < items.size(); i++) {
                	var iEl = items.get(i);
                	itemsList.add(iEl);
                }
                
                var assumedUID = entityUtteranceMap.get(eid);
                var assumedTID = utteranceTierMap.get(assumedUID);
                
                IntervalSet<String> tierUtterancesIntervalsSet = initialUtteranceIntervals.get(assumedTID);
                if (tierUtterancesIntervalsSet == null)
                	tierUtterancesIntervalsSet = new IntervalSet<String>();

                if (itemsList != null && itemsList.size() >= 0) {
                	Element firstItem = itemsList.get(0);
                	Element lastItem = itemsList.get(itemsList.size() - 1);
                	var start = readLongOpt(firstItem, "S").orElse(null);
                	var end = readLongOpt(lastItem, "E").orElse(null);
                	
                	boolean toDelete = false;

                    var entityInterval = TimeRange.ABSOLUTE.time(start, TimeUnit.TIMESTAMP, Rounding.FLOOR)
                        .to(TimeRange.ABSOLUTE.time(end, TimeUnit.TIMESTAMP, Rounding.FLOOR));
                    
                	ArrayList<String> utteranceIDsOverlapping = 
                			tierUtterancesIntervalsSet.testOverlap(entityInterval, Bound.INCLUSIVE).orElse(new ArrayList<String>());

                	int sizeOverlapping = utteranceIDsOverlapping.size();
                	if (sizeOverlapping == 0) {
                		addToModifications(20027, erepr + " is contained by no utterance bounds. It was deleted.", null);
                		toDelete = true;
                	} else if (sizeOverlapping == 1) {
                		String utteranceIDOverlapping = utteranceIDsOverlapping.get(0);
                		Element utteranceEl = utteranceElements.get(utteranceIDOverlapping);

                    	var utteranceStart = readLongOpt(utteranceEl, "S").orElse(null);
                    	var utteranceEnd = readLongOpt(utteranceEl, "E").orElse(null);
                    	
                    	if (utteranceStart > start || utteranceEnd < end) {
                    		// means the entity is partially out of bounds to the utterance, and thus should be deleted
                    		toDelete = true;
                    		addToModifications(20028, erepr + " is partially outside of utterance bounds. It was deleted.", null);
                    	}
                	} else {
                		ArrayList<Element> utteranceEls = new ArrayList<Element>();
                		for (String s: utteranceIDsOverlapping) {
                			utteranceEls.add(utteranceElements.get(s));
                		}
                		for (int i = 1; i < sizeOverlapping; i++) {
                			Element utteranceEl1 = utteranceEls.get(i - 1);
                			Element utteranceEl2 = utteranceEls.get(i);

                        	var utterance1End = readLongOpt(utteranceEl1, "E").orElse(null);
                        	var utterance2Start = readLongOpt(utteranceEl2, "S").orElse(null);
                        	
                        	// overlaps of utterances exists in the collection files, so the next utterance start can occur before the prior utterance end
                        	// but if the start is strictly after (not equals) the end, it means there is a gap
                        	if (utterance2Start > utterance1End) {
                        		// means that there is a gap between these utterances, so the entity is over that gap and thus should be deleted
                        		toDelete = true;
                        		addToModifications(20029, erepr + " is outside of utterance bounds. It was deleted.", null);
                        	}
                		}
                	}
                	
                	if (toDelete) {
                		deletedEntities.put(eid, fid);
                		entityElements.remove(eid);
                		tEl.removeChild(eEl);
                	}
                }
            }
    	}
    }
    
    private SS3ValidatorConverterReturns validateUtterances() {
        var partitionsEl = readChildOpt(glossWindowEl, "TEMPORAL-PARTITIONS").orElse(null);

        if (expect(partitionsEl != null, 10041, "Missing TEMPORAL-PARTITIONS element.", null)) {
            readChildren(partitionsEl, "TEMPORAL-PARTITION").forEach(partitionEl -> {
                var internal_repr = "Temporal Partition " + repr(partitionEl, "ID");
                var repr = "Temporal Partition " + repr(partitionEl, "LABEL");
                var pid = readOpt(partitionEl, "ID").orElse(null);
                
                expect(isNotBlank(pid), 10042, "Temporal Partition missing ID.", null);
                
                var partitionLabel = readOpt(partitionEl, "LABEL").orElse(null);
                expect(isNotBlank(partitionLabel), 10044, "Temporal Partition " + pid + " missing LABEL.", "A temporal partition is missing a label.");

                readChildren(partitionEl, "TEMPORAL-PARTITION-SEGMENT").forEach(tierEl -> {
                    var internal_trepr = internal_repr + " Segment Tier " + repr(tierEl, "ID");
                    var trepr = repr + " Segment Tier " + repr(tierEl, "LABEL");
                    var tid = readOpt(tierEl, "ID").orElse(null);
                    
                    expect(isNotBlank(tid), 10045, internal_repr + " Segment Tier missing ID.", repr + " Segment Tier missing ID.");
                    
                    String partition = null;
                    if (partitionLabel != null)
                    	partition = "Temporal Partition (" + partitionLabel + ") ";

                    var segmentLabel = readOpt(tierEl, "LABEL").orElse(null);
                    expect(isNotBlank(segmentLabel), 10047, "Temporal Partition " + pid + " Segment Tier " + tid + " missing LABEL.", "A segment tier is missing a label.");
                    
                    var participantID = readOpt(tierEl, "PARTICIPANT_ID").orElse(null);
//                    expect(isNotBlank(participantID), trepr + " missing PARTICIPANT_ID.", "Temporal Partition (" + partitionLabel + ") Segment Tier ( " + segmentLabel + ") does not have an associated participant.");
                    
                    if (isBlank(participantID)) {
                    	var refsEl = readChildOpt(rootEl, "REFERENCES").orElse(null);
                    	if (refsEl != null) {
                    		var participantsEl = readChildOpt(refsEl, "PARTICIPANTS").orElse(null);
                    		if (participantsEl != null) {
                    			int size = participantElements.size(); 
                    			if (size == 0) {
                        			// insert the information of the default participant from the global file
                        			if (addDefaultParticipant(participantsEl)) {
                                    	var str = "The default participant " + DEFAULT_PARTICIPANT_LABEL + " was added.";
                                    	addToModifications(20030, str, str);
                        			}
                        			
                                	write(tierEl, "PARTICIPANT_ID", DEFAULT_PARTICIPANT_ID);

                                	var str = "This segment tier had a missing associated participant, so it was automatically set to " + DEFAULT_PARTICIPANT_LABEL + ": " + trepr;
                                	addToModifications(20031, str, str);	// report this one to user
                    			} else if (size == 1) {
                    				// use that single participant as the one to associate to the segment tier
                    				Entry<String, Element> participant = participantElements.entrySet().iterator().next();
                    				String existingParticipantID = participant.getKey();
                    				Element existingParticipantEl = participant.getValue();
                    				var existingParticipantLabel = readOpt(existingParticipantEl, "LABEL").orElse(null);
                    				
                    				write(tierEl, "PARTICIPANT_ID", existingParticipantID);

                                	var str = "This segment tier had a missing associated participant, so it was automatically set to " + existingParticipantLabel + ": " + trepr;
                                	addToModifications(20087, str, null);	// do not report this one to user
                    			}
                    		}
                    	}
                    }

                    // statement fields in newer collections - directly under tier
                    readChildren(tierEl, "STATEMENT-FIELD").forEach(stmtFieldEl -> {
                        var internal_srepr = internal_trepr + "statement field " + repr(stmtFieldEl, "FIELD-ID");
                        var srepr = trepr + " statement field " + repr(stmtFieldEl, "FIELD-ID");
                        var fid = readOpt(stmtFieldEl, "FIELD-ID").orElse(null);
                        
                        expect(isNotBlank(fid), 10048, internal_trepr + " statement field has missing ID.", trepr + " statement field has missing ID.");

                        var eids = readOpt(stmtFieldEl, "ENTITY-IDS").orElse(null);
                        if (expect(eids != null, 10049, internal_srepr + " missing ENTITY-IDS.", srepr + " missing ENTITY-IDS.")) {
                            if (!eids.equals("")) {
                                for (var eid : eids.split(", ")) {
                                	if (deletedEntities.get(eid) != null && deletedEntities.get(eid).equals(fid))
                                        continue;

                                    expect(entityTrackMap.containsKey(eid), 10061, internal_srepr + " references an entity not found in any track: " + eid, srepr + " references an entity not found in any track: " + eid);
                                }
                            }
                        }
                    });

                    readChildren(tierEl, "GLOSS-WINDOW-ITEM").forEach(utteranceEl -> {
                        var internal_urepr = internal_trepr + " Utterance " + repr(utteranceEl, "ID");
                        var urepr = trepr + "Utterance " + repr(utteranceEl, "LABEL");
                        var uid = readOpt(utteranceEl, "ID").orElse(null);
                        
                        var start = readLongOpt(utteranceEl, "S").orElse(null);
                        var end = readLongOpt(utteranceEl, "E").orElse(null);

                        // statement fields in older collections - directly under utterance
                        readChildren(utteranceEl, "STATEMENT-FIELD").forEach(stmtFieldEl -> {
                            var internal_srepr = internal_trepr + "statement field " + repr(stmtFieldEl, "FIELD-ID");
                            var srepr = urepr + " statement field " + repr(stmtFieldEl, "FIELD-ID");
                            var fid = readOpt(stmtFieldEl, "FIELD-ID").orElse(null);
                            
                            expect(isNotBlank(fid), 10054, internal_urepr + " statement field has missing ID.", urepr + " statement field has missing ID.");

                            var eids = readOpt(stmtFieldEl, "ENTITY-IDS").orElse(null);
                            if (expect(eids != null, 10055, internal_srepr + " missing ENTITY-IDS attribute.", srepr + " missing ENTITY-IDS attribute.")) {
                                if (!eids.equals("")) {
                                    for (var eid : eids.split(", ")) {
                                    	if (deletedEntities.get(eid) != null && deletedEntities.get(eid).equals(fid))
                                            continue;

                                        expect(entityTrackMap.containsKey(eid), 10061, internal_srepr + " references an entity not found in any track: " + eid, srepr + " references an entity not found in any track: " + eid);
                                    }
                                }
                            }
                        });

                        if (expect(start != null, 10062, internal_urepr + " missing start time.", urepr + " missing start time.")
                            && expect(end != null, 10063, internal_urepr + " missing end time.", urepr + " missing end time.")) {
                        	
                        	long roundedDownTime = -1;

                            if (start % timestep != 0) {
                            	roundedDownTime = (start / timestep) * timestep;
                            	
                            	addToModifications(20032, urepr + " had start time rounded down from " + repr("" + start) + " -> " + repr("" + roundedDownTime), null);
                            	
                            	writeLong(utteranceEl, "S", roundedDownTime);
                            	start = roundedDownTime;
                            }
                            
//                            expect(start % timestep == 0, urepr + " has start time between frames.");

                    	    // video player used in old versions erroneously used the +1 frame for all times - so this is meant to correct that
                        	roundedDownTime = start - timestep;
                        	if (roundedDownTime > 0) {
                            	addToModifications(20033, urepr + " had start time auto-corrected from " + repr("" + start) + " -> " + repr("" + roundedDownTime), null);
                            	
                            	writeLong(utteranceEl, "S", roundedDownTime);
                            	start = roundedDownTime;
                        	}

                            if (end % timestep != 0) {
                            	roundedDownTime = (end / timestep) * timestep;
                            	
                            	addToModifications(20034, urepr + " had end time rounded down from " + repr(""+ end) + " -> " + repr("" + roundedDownTime), null);
                            	
                            	writeLong(utteranceEl, "E", roundedDownTime);
                            	end = roundedDownTime;
                            }
                            
//                            expect(end % timestep == 0, urepr + " has end time between frames.");

                    	    // video player used in old versions erroneously used the +1 frame for all times - so this is meant to correct that
                        	roundedDownTime = end - timestep;
                        	if (roundedDownTime > 0) {
                            	addToModifications(20035, urepr + " had end time auto-corrected from " + repr("" + end) + " -> " + repr("" + roundedDownTime), null);
                            	
                            	writeLong(utteranceEl, "E", roundedDownTime);
                            	end = roundedDownTime;
                        	}

                            if (end >= duration) {
                            	var newEnd = duration - timestep;
                            	
                            	addToModifications(20036, urepr + " had end time adjusted down from " + repr("" + end) + " -> " + repr("" + newEnd) + " because it exceeded the video duration.", null);
                            	writeLong(utteranceEl, "E", newEnd);
                                end = newEnd;
                            }
                            
                            expect(start >= 0 && start <= duration, 10064, internal_urepr + " has out-of-bounds start time.", urepr + " has out-of-bounds start time.");
                            expect(end >= 0 && end <= duration, 10065, internal_urepr + " has out-of-bounds end time.", urepr + " has out-of-bounds end time.");

                            expect(start <= end, 10066, internal_urepr + " has reversed start/end times.", urepr + " has reversed start/end times.", false);
                            
                            IntervalSet<String> tierUtterancesIntervalSet = utteranceIntervals.get(tid);
                            if (tierUtterancesIntervalSet == null) {
                            	tierUtterancesIntervalSet = new IntervalSet<String>();
                            	utteranceIntervals.put(tid, tierUtterancesIntervalSet);
                            }

                            var interval = TimeRange.ABSOLUTE.time(start, TimeUnit.TIMESTAMP, Rounding.FLOOR)
                                .to(TimeRange.ABSOLUTE.time(end, TimeUnit.TIMESTAMP, Rounding.FLOOR));

                            tierUtterancesIntervalSet.add(uid, interval, Bound.EXCLUSIVE)
                                .ifPresent(otherIDs -> {
                                	// GD: commenting as it should be reported as an error only if it not fixed - see below in correctUtteranceOverlaps
//                                	for (String otherID: otherIDs) {
//                                    	var other_urepr = "Utterance " + repr(utteranceElements.get(otherID), "LABEL", "ID");
//                                    	addToErrors(urepr + " overlaps with another utterance with ID " + otherID);
//                                    	addToErrors(other_urepr + " overlaps with another utterance with ID " + uid);
//                                	}
                                });
                        }
                    });
                });
            });
        }
        
    	return null;
    }

    private SS3ValidatorConverterReturns validateTracks() {
        var defScheme = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();
        	var defSchemeFields = defScheme.getFields();
        
        var tracksEl = readChildOpt(dbEl, "TRACKS").orElse(null);

        if (expect(tracksEl != null, 10067, "Missing TRACKS element.", null)) {
        		HashMap<String, Element> mapSignTypeCreations = new HashMap<String, Element>();
        	
            for (Element tEl: readChildren(tracksEl, "TRACK").collect(Collectors.toList())) {
                var repr = "Track " + repr(tEl, "FID");
                var fid = readOpt(tEl, "FID").orElse(null);
                
                var localField = localSchemeFieldValues.get(fid);
                var defaultField = defSchemeFields.get(fid);
                
                if (localField == null && defaultField == null) {
                	if (tEl.hasChildNodes()) {
                    	addToErrors(10068, repr + " is neither a default nor local field.", null);
                	} else {
                		tracksEl.removeChild(tEl);

                		var str = repr + " was removed as it is neither a default nor local field, and had no data within it.";
//                		var public_str = repr + " was removed as it is neither a default nor local field, and had no data within it.";
                		addToModifications(20037, str, null);
                		continue;
                	}
                }

                readChildren(tEl, "ENTITY").forEach(eEl -> {
                    var erepr = "Entity " + repr(eEl, "ID");
                    var eid = readOpt(eEl, "ID").orElse(null);
                    
                    // expect fid (above) and the field_id assoc to this entity's eid (from statement field) is equal
                    String entityFieldID = refEntities.get(eid);
                    expect(fid.equals(entityFieldID), 10069, "Entity " + eid + " was in a track whose FID does not match the entity's associated statement field ID.", null);
                    
                    if (deletedEntities.get(eid) != null && deletedEntities.get(eid).equals(fid))
                        return;
                    
                    boolean entityIsTextOnly = false;

                    var items = readChildren(eEl, "ITEM").collect(Collectors.toList());
                    for (int i = 0; i < items.size(); i++) {
                        var iEl = items.get(i);
                        var irepr = erepr + " item";
                        var start = readLongOpt(iEl, "S").orElse(null);
                        var end = readLongOpt(iEl, "E").orElse(null);
                        
                        var list = itemElements.get(eid);
                        if (list == null) {
                        	list = new ArrayList<Element>();
                        	list.add(iEl);
                        	itemElements.put(eid, list);
                        }
                        else {
                        	list.add(iEl);
                        }

                        boolean isMidHandshape =
                            (Util.DOM_HANDSHAPE_FIELD_ID.equals(fid) || Util.NDOM_HANDSHAPE_FIELD_ID.equals(fid))
                            && i != 0 && i != items.size() - 1;

                        if (start != null) {
                        	long roundedDownTime = -1;
                            if (!isMidHandshape && start % timestep != 0) {
                            	roundedDownTime = (start / timestep) * timestep;
                            	
//                            	if (localSchemeFields.keySet().contains(fid)
//                            			|| Integer.valueOf(fid) < 100) {	// GD: hack for default scheme nonmanual fields - all values under 100
//                            		long roundedUpTime = roundedDownTime + timestep;
//                            		
//                                	long diffDown = Math.abs(start - roundedDownTime);
//                                	long diffUp = Math.abs(start - roundedUpTime);
//                                	
//                                	if (diffUp < diffDown) {
//                                        addToModfications(irepr + " had start time rounded up from " + start + " -> " + roundedUpTime);
//                                        writeLong(iEl, "S", roundedUpTime);
//                                        start = roundedUpTime;
//                                	}
//                            	}
                            	
                                addToModifications(20038, irepr + " had start time rounded down from " + repr(""+ start) + " -> " + repr("" + roundedDownTime), null);
                            	
                                writeLong(iEl, "S", roundedDownTime);
                                start = roundedDownTime;
                            }

                    	    // video player used in old versions erroneously used the +1 frame for all times - so this is meant to correct that
                        	roundedDownTime = start - timestep;
                        	if (roundedDownTime > 0) {
                            	addToModifications(20039, irepr + " had start time auto-corrected from " + repr("" + start) + " -> " + repr("" + roundedDownTime), null);
                            	
                            	writeLong(iEl, "S", roundedDownTime);
                            	start = roundedDownTime;
                        	}
                            
//                            expect(isMidHandshape || start % timestep == 0, irepr + " has start time between frames.");
                            expect(start >= 0 && start <= duration, 10070, irepr + " has out-of-bounds start time.", null);
                        }

                        if (end != null) {
                        	long roundedDownTime = -1;
                            if (!isMidHandshape && end % timestep != 0) {
                            	roundedDownTime = (end / timestep) * timestep;
                            	
//                            	if (localSchemeFields.keySet().contains(fid)
//                            			|| Integer.valueOf(fid) < 100) {	// GD: hack for default scheme nonmanual fields - all values under 100
//                            		long roundedUpTime = roundedDownTime + timestep;
//                            		
//                                	long diffDown = Math.abs(end - roundedDownTime);
//                                	long diffUp = Math.abs(end - roundedUpTime);
//                                	
//                                	if (diffUp < diffDown) {
//                                        addToModifications(irepr + " had end time rounded up from " + end + " -> " + roundedUpTime);
//                                        writeLong(iEl, "E", roundedUpTime);
//                                        end = roundedUpTime;
//                                	}
//                            	}
                            	
                                addToModifications(20040, irepr + " had end time rounded down from " + repr("" + end) + " -> " + repr("" + roundedDownTime), null);
                            	
                                writeLong(iEl, "E", roundedDownTime);
                                end = roundedDownTime;
                            }

                    	    // video player used in old versions erroneously used the +1 frame for all times - so this is meant to correct that
                        	roundedDownTime = end - timestep;
                        	if (roundedDownTime > 0) {
                            	addToModifications(20041, irepr + " had end time auto-corrected from " + repr("" + end) + " -> " + repr("" + roundedDownTime), null);
                            	
                            	writeLong(iEl, "E", roundedDownTime);
                            	end = roundedDownTime;
                        	}

                            if (fid.equals(Util.FREE_TEXT_FIELD_ID)) {
                                if (end >= duration) {
                                	var newEnd = duration - timestep;
                                	
                                	addToModifications(20042, irepr + " had end time adjusted down from " + repr("" + end) + " -> " + repr("" + newEnd) + " because it exceeded the video duration.", null);
                                	writeLong(iEl, "E", newEnd);
                                    end = newEnd;
                                }
                            }
                            
//                            expect(isMidHandshape || end % timestep == 0, irepr + " has end time between frames time.");
                            expect(end >= 0 && end <= duration, 10071, irepr + " has out-of-bounds end time.", null);
                        }

                        var textOnly = readOpt(iEl, "TEXT_ONLY").orElse(null);	// values either "T" or "F"
                        var itemText = getCharacterDataFromElement(iEl);

                        if (isNotBlank(textOnly) && textOnly.equals("T")) {
                        	entityIsTextOnly = true;
                            expect(itemText != null, 10072, irepr + " has a blank text value.", null);
                        }
                        
                        if (fid.equals("22")) {	// field id 22 refers to 'role shift'
                        	if (itemText.equals("rs:other")) {		// 'rs:other' was changed to just 'other'
                        		setCharacterDataOnElement(iEl, "other");

                        		var str = irepr + " had item value changed from " + repr("rs:other") + " to " + repr("other") + ".";
                        		var public_entity = getEntityPublicErrorString(eid, fid);
//                        		var public_str = public_entity + " had item value changed from 'rs:other' to 'other'.";
                        		addToModifications(20043, str, null);
                        	}
                        }
                        
                        String fieldIDBodyMvmt = "32";
                        String valueLabelDown = "down";
                        if (modifyFieldBodyMvmtValue && fid.equals(fieldIDBodyMvmt)) {
                        	var vid = readOpt(iEl, "VID").orElse(null);
                        	if (isNotBlank(vid) && vid.equals("0")) {
                        		// this field value label was completely missing from the scheme - adding here for any missing it
                        		setCharacterDataOnElement(iEl, valueLabelDown);

                        		var str = irepr + " had item value changed from <blank> to " + repr(valueLabelDown) + ".";
                        		var public_entity = getEntityPublicErrorString(eid, fid);
//                        		var public_str = public_entity + " had item value changed from <blank> to 'down'.";
                        		addToModifications(20044, str, null);                        		
                        	}
                        }
                    }

                    if (isNotBlank(fid)) {
                        if ( !entityIsTextOnly && (fid.equals(Util.DOM_GLOSS_FIELD_ID) || fid.equals(Util.NDOM_GLOSS_FIELD_ID))) {

                            var propertiesEl = readChildOpt(eEl, "PROPERTIES").orElse(null);
                            if (expect(propertiesEl != null, 10073, erepr + " missing PROPERTIES element.", null)) {

                                var signType = readOpt(propertiesEl, "SIGN").orElse(null);
                                var dependentSignTypeID = readOpt(eEl, "DEPENDENT_SIGN_TYPE_ID").orElse(null);
                                
                                Element signTypeEl = entityElements.get(dependentSignTypeID);
                                var signTypeTrackID = fid.equals(Util.DOM_GLOSS_FIELD_ID)
                                		? Util.DOM_SIGN_TYPE_FIELD_ID
                                		: Util.NDOM_SIGN_TYPE_FIELD_ID;
                                var trackEl = trackElements.get(signTypeTrackID);

                            	var public_entity = getEntityPublicErrorString(eid, fid);
                            	if (isBlank(signType) && isNotBlank(dependentSignTypeID)) {

                            		String currentEntityText = null;

                            		Element onset = null, initialHold = null, main = null, finalHold = null, offset = null;
                            		for (Element item: items) {
                            			var itemText = getCharacterDataFromElement(item);
                            			if (itemText.equals(Constants.ONSET))
                            				onset = item;
                            			else if (itemText.equals(Constants.INITIAL_HOLD))
                            				initialHold = item;
                            			else if (itemText.equals(Constants.FINAL_HOLD))
                            				finalHold = item;
                            			else if (itemText.equals(Constants.OFFSET))
                            				offset = item;
                            			else
                            				main = item;
                            		}

                            		currentEntityText = getCharacterDataFromElement(main);

                            		SignType currentSignType = null;

                            		String fingerspelledPattern = "(!)?(neu:|([a-z_1-3][p:]))?([(1h)][(2h)][(2h)alt.])?(fs-){1}(.*)?";
                            		Pattern fingerspelledR = Pattern.compile(fingerspelledPattern);
                            		Matcher fingerspelledMatcher = fingerspelledR.matcher(currentEntityText);

                            		if (fingerspelledMatcher.find()) {
                            			currentSignType = SignType.FINGERSPELLED;
                            		} else {
                            			// loan sign
                            			String loanSignPattern = "(!)?(neu:|([a-z_1-3][p:]))?([(1h)][(2h)][(2h)alt.])?(#){1}(.*)?";
                            			Pattern loanSignR = Pattern.compile(loanSignPattern);
                            			Matcher loanSignMatcher = loanSignR.matcher(currentEntityText);

                            			if (loanSignMatcher.find()) {
                            				currentSignType = SignType.LOAN;
                            			} else {
                            				String classifierPattern = "(!)?(neu:|([a-z_1-3][p:]))?([(1h)][`][(2h)alt.])?(ns-)?((CL)|(DCL)|(LCL)|(SCL)|(BCL)|(BPCL)|(PCL)|(ICL)){1}([-_:].*)?$";
                            				Pattern classifierR = Pattern.compile(classifierPattern);
                            				Matcher classifierMatcher = classifierR.matcher(currentEntityText);

                            				if (classifierMatcher.find()) {
                            					currentSignType = SignType.CLASSIFIER;
                            				} else {

                            					String gesturePattern = "(!)?(neu:|([a-z_1-3][p:]))?([(1h)][(2h)][(2h)alt.])?(ns-)?((\"){1}(.){1,}(\"){1})(.*)?";
                            					Pattern gestureR = Pattern.compile(gesturePattern);
                            					Matcher gestureMatcher = gestureR.matcher(currentEntityText);

                            					if (gestureMatcher.find()) {
                            						currentSignType = SignType.GESTURE;
                            					} else {
                            						currentSignType = SignType.LEXICAL;
                            					}
                            				}
                            			}
                            		}

                            		var str = erepr + " had a dependent sign type entity with no value, so the sign type entity was inferred by the text to be " + currentSignType + ".";
//                            		var public_str = "This gloss had an error with its sign type, so the sign type value was inferred by the text to be " + currentSignType + " : " + public_entity;
                            		addToModifications(20045, str, null);

                            		write(propertiesEl, "SIGN", currentSignType.toString());
                            	} else if (isNotBlank(signType) && isBlank(dependentSignTypeID)) {
                                    var noOfHands = readOpt(propertiesEl, "NO_OF_HANDS").orElse(null);
                                	// the following error does not apply when it's 2 handed and on the ndom, as the dep is on the dom hand
                                    if ( (isNotBlank(noOfHands) && "1".equals(noOfHands)) || ! fid.equals(Util.NDOM_GLOSS_FIELD_ID)) {
                                    	var str = erepr + " had a sign type value with no dependent sign type entity, so the sign type entity was created with that value.";
                                    	var public_str = "This gloss had a misconfiguration with its sign type value, so it was provided an assumed sign type: " + public_entity;
                                        addToModifications(20046, str, public_str);
                                    	 
                                        // See below, after for-loop to prevent conflicts, for where this sign type creation actually occurs
                                        mapSignTypeCreations.put(eid, trackEl);
                                	}
                                }
                            }
                        }
                    }
                });

                readChildren(tEl, "NON_COMPLETE_EVENTS").forEach(eEl -> {
                    var erepr = repr + " non complete event " + repr(eEl, "ID");

                    var items = readChildren(eEl, "ITEM").collect(Collectors.toList());
                    for (int i = 0; i < items.size(); i++) {
                        var iEl = items.get(i);
                        var irepr = erepr + " item";
                        var start = readLongOpt(iEl, "S").orElse(null);
                        var end = readLongOpt(iEl, "E").orElse(null);
                        
                        if (start != null) {
                            if (start % timestep != 0) {
                                addToModifications(20038, irepr + " had start time rounded down from " + repr("" + start) + " -> " + repr("" + ((start / timestep) * timestep)), null);
                                writeLong(iEl, "S", (start / timestep) * timestep);
                            }
                            
                            expect(start >= 0 && start <= duration, 10070, irepr + " has out-of-bounds start time.", null);
                        }

                        if (end != null) {
                            if (end % timestep != 0) {
                                addToModifications(20040, irepr + " had end time rounded down from " + repr("" + end) + " -> " + repr("" + ((end / timestep) * timestep)), null);
                                writeLong(iEl, "E", (end / timestep) * timestep);
                            }

                            if (end >= duration) {
                            	var newEnd = duration - timestep;
                            	
                            	addToModifications(20042, irepr + " had end time adjusted down from " + repr("" + end) + " -> " + repr("" + newEnd) + " because it exceeded the video duration.", null);
                            	writeLong(iEl, "E", newEnd);
                                end = newEnd;
                            }
                            
                            expect(end >= 0 && end <= duration, 10071, irepr + " has out-of-bounds end time.", null);
                        }
                    }
                });

                readChildren(tEl, "ENTERED_TEXT").forEach(eEl -> {
                    var erepr = repr + " entered text " + repr(eEl, "ID");
                    
                    var items = readChildren(eEl, "ITEM").collect(Collectors.toList());
                    for (int i = 0; i < items.size(); i++) {
                        var iEl = items.get(i);
                        var irepr = erepr + " item";
                        var start = readLongOpt(iEl, "S").orElse(null);
                        var end = readLongOpt(iEl, "E").orElse(null);
                        
                        if (start != null) {
                            if (start % timestep != 0) {
                                addToModifications(20038, irepr + " had start time rounded down from " + repr("" + start) + " -> " + repr("" + ((start / timestep) * timestep)), null);
                                writeLong(iEl, "S", (start / timestep) * timestep);
                            }
                            
                            expect(start >= 0 && start <= duration, 10070, irepr + " has out-of-bounds start time.", null);
                        }

                        if (end != null) {
                            if (end % timestep != 0) {
                                addToModifications(20040, irepr + " had end time rounded down from " + repr("" + end) + " -> " + repr("" + ((end / timestep) * timestep)), null);
                                writeLong(iEl, "E", (end / timestep) * timestep);
                            }
                            
                            expect(end >= 0 && end <= duration, 10071, irepr + " has out-of-bounds end time.", null);
                        }

                        if (end >= duration) {
                        	var newEnd = duration - timestep;
                        	
                        	addToModifications(20042, irepr + " had end time adjusted down from " + repr("" + end) + " -> " + repr("" + newEnd) + " because it exceeded the video duration.", null);
                        	writeLong(iEl, "E", newEnd);
                            end = newEnd;
                        }
                    }
                });
            }
            
            // below is the manual creation of sign type entities inside the sign type track
            for (Entry entry: mapSignTypeCreations.entrySet()) {
            		String glossEntityID = (String) entry.getKey();
            		Element signTypeTrackEl = (Element) entry.getValue();
            		
            		String fid = read(signTypeTrackEl, "FID");

            		Element glossEntityEl = entityElements.get(glossEntityID);
            		String parentEntityID = read(glossEntityEl, "PARENT_ENTITY_ID");
            		
            		ArrayList<Element> items = itemElements.get(glossEntityID);
            		if (items == null || items.isEmpty()) {
            			continue;
            		}

            		Element onset = null, initialHold = null, main = null, finalHold = null, offset = null;
            		Element operativeStartItem = null, operativeEndItem = null;
            		for (Element item: items) {
            			var itemText = getCharacterDataFromElement(item);
            			if (itemText.equals(Constants.ONSET))
            				onset = item;
            			else if (itemText.equals(Constants.INITIAL_HOLD))
            				initialHold = item;
            			else if (itemText.equals(Constants.FINAL_HOLD))
            				finalHold = item;
            			else if (itemText.equals(Constants.OFFSET))
            				offset = item;
            			else
            				main = item;
            		}

            		if (onset != null)
            			operativeStartItem = onset;
            		if (initialHold != null)
            			operativeStartItem = initialHold;
            		else
            			operativeStartItem = main;

            		if (offset != null)
            			operativeEndItem = offset;
            		if (finalHold != null)
            			operativeEndItem = finalHold;
            		else
            			operativeEndItem = main;

            		var start = readLongOpt(operativeStartItem, "S").orElse(null);
            		var end = readLongOpt(operativeEndItem, "E").orElse(null);

            		String dependentSignTypeID = Util.getLongUniqueNumber();
            		write(glossEntityEl, "DEPENDENT_SIGN_TYPE_ID", dependentSignTypeID);
            		
            		Element signTypeEl = doc.createElement("ENTITY");
            		
            		signTypeEl.setAttribute("ID", dependentSignTypeID);
            		signTypeEl.setAttribute("REF_ENTITY_ID", glossEntityID);
            		signTypeEl.setAttribute("PARENT_ENTITY_ID", parentEntityID);
            		
            		signTypeEl.setAttribute("CODER-ID", "");
            		signTypeEl.setAttribute("COMP_END_ENTITY_ID", "");
            		signTypeEl.setAttribute("COMP_START_ENTITY_ID", "");
            		signTypeEl.setAttribute("PARTICIPANT-ID", "");
            		signTypeEl.setAttribute("TIME_CREATED", "");
            		signTypeEl.setAttribute("TIME_UPDATED", "");

            		Element itemEl = doc.createElement("ITEM");
            		itemEl.setAttribute("S"	, "" + start);
            		itemEl.setAttribute("E", "" + end);
            		itemEl.setAttribute("VID", Util.getUniqueNumber());
            		
            		signTypeEl.appendChild(itemEl);
            		signTypeTrackEl.appendChild(signTypeEl);
            		
            		// add to the maps built so far
            		entityElements.put(dependentSignTypeID, signTypeEl);
            		entityTrackMap.put(dependentSignTypeID, fid);
            		
            		var list = new ArrayList<Element>();
            		list.add(itemEl);
            		itemElements.put(dependentSignTypeID, list);
            		
            		newEntities.put(dependentSignTypeID, fid);
            }
        }

        return null;
    }

    private void correctDepdendentsProperties() {
        // Correct dependents' REF_ENTITY_ID to point to the proper source (gloss in most cases, except hs points to hand)
    	var domTrack = trackElements.get(Util.DOM_GLOSS_FIELD_ID);
    	var ndomTrack = trackElements.get(Util.NDOM_GLOSS_FIELD_ID);
    	if (domTrack == null && ndomTrack == null)
    		return;
    	
    	var glossTrackElements = List.of(domTrack, ndomTrack);
    	
        for (var trackEl : glossTrackElements) {
        	var glossTrackID = readOpt(trackEl, "FID").orElse(null);
            var handTrackID = glossTrackID.equals(Util.DOM_GLOSS_FIELD_ID)
                    ? Util.DOM_HAND_FIELD_ID
                    : Util.NDOM_HAND_FIELD_ID;
            	var altHandTrackID = handTrackID.equals(Util.DOM_HAND_FIELD_ID)
            		? Util.NDOM_HAND_FIELD_ID
            		: Util.DOM_HAND_FIELD_ID;
            
            var hsTrackID = glossTrackID.equals(Util.DOM_GLOSS_FIELD_ID)
                ? Util.DOM_HANDSHAPE_FIELD_ID
                : Util.NDOM_HANDSHAPE_FIELD_ID;
            	var altHSTrackID = hsTrackID.equals(Util.DOM_HANDSHAPE_FIELD_ID)
            		? Util.NDOM_HANDSHAPE_FIELD_ID
            		: Util.DOM_HANDSHAPE_FIELD_ID;
            
            var locTrackID = glossTrackID.equals(Util.DOM_GLOSS_FIELD_ID)
            	? Util.DOM_LOCATION_FIELD_ID
            	: Util.NDOM_LOCATION_FIELD_ID;
            	var altLocTrackID = locTrackID.equals(Util.DOM_LOCATION_FIELD_ID)
            		? Util.NDOM_LOCATION_FIELD_ID
            		: Util.DOM_LOCATION_FIELD_ID;
            
            var signTypeTrackID = glossTrackID.equals(Util.DOM_GLOSS_FIELD_ID)
            		? Util.DOM_SIGN_TYPE_FIELD_ID
            		: Util.NDOM_SIGN_TYPE_FIELD_ID;
            	var altSignTypeTrackID = signTypeTrackID.equals(Util.DOM_SIGN_TYPE_FIELD_ID)
            		? Util.NDOM_SIGN_TYPE_FIELD_ID
            		: Util.DOM_SIGN_TYPE_FIELD_ID;
            

            var handTrackEl = trackElements.get(handTrackID);
            	var altHandTrackEl = trackElements.get(altHandTrackID);
            var hsTrackEl = trackElements.get(hsTrackID);
            var altHSTrackEl = trackElements.get(altHSTrackID);
            var locTrackEl = trackElements.get(locTrackID);
            	var altLocTrackEl = trackElements.get(altLocTrackID);
            var signTypeTrackEl = trackElements.get(signTypeTrackID);
            	var altSignTypeTrackEl = trackElements.get(altSignTypeTrackID);

            for (Element glossEl: readChildren(trackEl, "ENTITY")
            		.collect(Collectors.toCollection(ArrayList::new))) {
                var glossEID = readOpt(glossEl, "ID").orElse(null);

                if (deletedEntities.get(glossEID) != null && deletedEntities.get(glossEID).equals(glossTrackID))
                    continue;

            	String entityPublicErrorString = getEntityPublicErrorString(glossEID, glossTrackID);
    			
            	var dependentHandID = readOpt(glossEl, "DEPENDENT_HAND_ID").orElse(null);
            	var dependentHandshapeID = readOpt(glossEl, "DEPENDENT_HAND_SHAPE_ID").orElse(null);
            	var dependentLocationID = readOpt(glossEl, "DEPENDENT_LOCATION_ID").orElse(null);
            	var dependentSignTypeID = readOpt(glossEl, "DEPENDENT_SIGN_TYPE_ID").orElse(null);

            	Element handEl = null, hsEl = null, locEl = null, signTypeEl = null;
            	
            	for (Element handEntityEl: readChildren(handTrackEl, "ENTITY").collect(Collectors.toCollection(ArrayList::new))) {
            		var handEID = readOpt(handEntityEl, "ID").orElse(null);
            		if (isNotBlank(handEID) && handEID.equals(dependentHandID)) {
            			handEl = handEntityEl;
            			break;
            		}
            	}
            	
            	for (Element hsEntityEl: readChildren(hsTrackEl, "ENTITY").collect(Collectors.toCollection(ArrayList::new))) {
            		var hsEID = readOpt(hsEntityEl, "ID").orElse(null);
            		if (isNotBlank(hsEID) && hsEID.equals(dependentHandshapeID)) {
            			hsEl = hsEntityEl;
            			break;
            		}
            	}

        		var public_entity = getEntityPublicErrorString(glossEID, glossTrackID);
        		
            	if (handEl != null) {
            		var handID = readOpt(handEl, "ID").orElse(null);
            		var handRefEntityID = readOpt(handEl, "REF_ENTITY_ID").orElse(null);
            		
            		if (isBlank(handRefEntityID)) {
            			write(handEl, "REF_ENTITY_ID", glossEID);
            			
            			var str = "Hand {ID=" + handID + "} had REF_ENTITY_ID reset to parent gloss {ID=" + glossEID + "} because the gloss referenced it as its dependent.";
//            			var public_str = public_entity + " had its hand information reset.";
            			addToModifications(20047, str, null);
            		} else {
            			if ( ! expect(handRefEntityID.equals(glossEID), 10074, 
            					"Hand {ID=" + handID + "} has reference to incorrect gloss {ID=" + handRefEntityID + "}. Gloss references {ID=" + dependentHandID + "} as its hand.", 
            					"The  hand configuration information is invalid for: " + entityPublicErrorString,
            					false)) {
                			write(handEl, "REF_ENTITY_ID", glossEID);
                			
                			var str = "Hand {ID=" + handID + "} had REF_ENTITY_ID reset to parent gloss {ID=" + glossEID + "} because the gloss referenced it as its dependent.";
//                			var public_str = public_entity + " had its hand information reset.";
                			addToModifications(20047, str, null);
            			}
            		}

            		if (hsEl != null) {
            			var hsRefEntityID = readOpt(hsEl, "REF_ENTITY_ID").orElse(null);
            			var hsID = readOpt(hsEl, "ID").orElse(null);

            			if (isBlank(hsRefEntityID) && isNotBlank(handID)) {
            				write(hsEl, "REF_ENTITY_ID", handID);
            				
            				var str = "Handshape {ID=" + hsID + "} had REF_ENTITY_ID reset to parent hand {ID=" + handID + "} because the hand referenced it as its dependent.";
//            				var public_str = public_entity + " had its handshape information reset.";
            				addToModifications(20048, str, null);
            			} else {
            				if ( ! expect(hsRefEntityID.equals(handID), 10075, 
            						"Handshape {ID=" + hsID + "} has reference to incorrect hand {ID=" + hsRefEntityID + "}. Hand references {ID=" + dependentHandshapeID + " as its handshape.", 
            						"The hand configuration information is invalid for: " + entityPublicErrorString,
            						false)) {
            					write(hsEl, "REF_ENTITY_ID", handID);
            					
            					var str = "Handshape {ID=" + hsID + "} had REF_ENTITY_ID reset to parent hand {ID=" + handID + "} because the hand referenced it as its dependent.";
//            					var public_str = public_entity + " had its handshape information reset.";
            					addToModifications(20048, str, null);
            				}
            			}

            			// also check here: LOCKED_GLOSS_PHON_ALIGNMENT should all match T/F values on the gloss and hand/hs
            			var glossLocked = readOpt(glossEl, "LOCKED_GLOSS_PHON_ALIGNMENT").orElse(null);
            			var handLocked = readOpt(handEl, "LOCKED_GLOSS_PHON_ALIGNMENT").orElse(null);
            			var hsLocked = readOpt(hsEl, "LOCKED_GLOSS_PHON_ALIGNMENT").orElse(null);
            			if (isNotBlank(glossLocked)) {
            				String lockedPublicMessage = "The information about whether the (right/left) hand gloss should be locked or unlocked with respect to the hand configuration displayed below that gloss should be verified for: " + entityPublicErrorString;
            				expect(glossLocked.equals("T") || glossLocked.equals("F"), 10076, "Gloss {ID=" + glossEID + "} LOCKED_GLOSS_PHON_ALIGNMENT value is neither T nor F.", lockedPublicMessage);
            				var handEID = readOpt(handEl, "ID").orElse(null);
            				var hsEID = readOpt(hsEl, "ID").orElse(null);

            				if (isNotBlank(handLocked))
            					expect(handLocked.equals("T") || handLocked.equals("F"), 10077, "Hand {ID=" + handEID + "} LOCKED_GLOSS_PHON_ALIGNMENT value is neither T nor F.", lockedPublicMessage);
            				expect(glossLocked.equals(handLocked), 10078, "Gloss {ID=" + glossEID + "} and Hand {ID=" + handEID 
											+ "} have mismatching values for locked gloss phon alignment.", lockedPublicMessage);
            				if (isNotBlank(hsLocked))
            					expect(hsLocked.equals("T") || hsLocked.equals("F"), 10079, "Handshape {ID=" + hsEID + "} LOCKED_GLOSS_PHON_ALIGNMENT value is neither T nor F.", lockedPublicMessage);
            				expect(glossLocked.equals(hsLocked), 10080, "Gloss {ID=" + glossEID + "} and Handshape {ID=" + hsEID 
											+ "} have mismatching values for locked gloss phon alignment.", lockedPublicMessage);
            			}
            		} else if (isNotBlank(dependentHandshapeID)) {
                		for (Element hsEntityEl: readChildren(altHSTrackEl, "ENTITY").collect(Collectors.toCollection(ArrayList::new))) {
                			var hsEID = readOpt(hsEntityEl, "ID").orElse(null);
                			if (isNotBlank(hsEID) && hsEID.equals(dependentHandshapeID)) {
                				hsEl = hsEntityEl;
                				break;
                			}
                		}

                		if (hsEl != null) {
                			// the proper element by the id was found in the alt track, so it just means the dependent id on the gloss element was wrong and should be null
                			write(glossEl, "DEPENDENT_HANDSHAPE_ID", "");
                			
                			var str = "Gloss {" + glossEID + "} had its DEPENDENT_HANDSHAPE_ID {" + dependentHandshapeID + "} erased because the handshape was found on the track corresponding to the other hand.";
//                			var public_str = public_entity + " had its handshape information erased.";
                			addToModifications(20049, str, null);
                		} else {
                			addToErrors(10081, "Handshape {ID=" + dependentHandshapeID + "} was not found in Track {ID=" + hsTrackID + "}. It was expected there as the dependent handshape of gloss {ID=" + glossEID + "}.", "The hand configuration information is invalid for: " + entityPublicErrorString);
                		}
            		}
            	} else if (isNotBlank(dependentHandID)) {
            		for (Element handEntityEl: readChildren(altHandTrackEl, "ENTITY").collect(Collectors.toCollection(ArrayList::new))) {
            			var handEID = readOpt(handEntityEl, "ID").orElse(null);
            			if (isNotBlank(handEID) && handEID.equals(dependentHandID)) {
            				handEl = handEntityEl;
            				break;
            			}
            		}

            		if (handEl != null) {
            			// the proper element by the id was found in the alt track, so it just means the dependent id on the gloss element was wrong and should be null
            			write(glossEl, "DEPENDENT_HAND_ID", "");
            			
            			var str = "Gloss {" + glossEID + "} had its DEPENDENT_HAND_ID {" + dependentHandID + "} erased because the hand was found on the track corresponding to the other hand.";
//            			var public_str = public_entity + " had its hand information erased.";
            			addToModifications(20050, str, null);
            		} else {
            			addToErrors(10082, "Hand {ID=" + dependentHandID + "} was not found in Track {ID=" + handTrackID + "}. It was expected there as the dependent hand of gloss {ID=" + glossEID + "}.", "The  hand configuration information is invalid for: " + entityPublicErrorString);
            		}
            	}
            	
            	for (Element locEntityEl: readChildren(locTrackEl, "ENTITY").collect(Collectors.toCollection(ArrayList::new))) {
            		var locEID = readOpt(locEntityEl, "ID").orElse(null);
            		if (isNotBlank(locEID) && locEID.equals(dependentLocationID)) {
            			locEl = locEntityEl;
            			break;
            		}
            	}
            	
            	if (locEl != null) {
            		var locID = readOpt(locEl, "ID").orElse(null);
            		var locRefEntityID = readOpt(locEl, "REF_ENTITY_ID").orElse(null);
            		
            		if (isBlank(locRefEntityID)) {
            			write(locEl, "REF_ENTITY_ID", glossEID);
            			
            			var str = "Location {ID=" + locID + "} had REF_ENTITY_ID reset to parent gloss {ID=" + glossEID + "} because the gloss referenced it as its dependent.";
//            			var public_str = public_entity + " had its location information reset.";
            			addToModifications(20051, str, null);
            		} else {
            			if (! expect(locRefEntityID.equals(glossEID), 10083, 
            					"Location {ID=" + locID + "} has reference to incorrect gloss {ID=" + locRefEntityID + "}. Gloss references {ID=" + dependentLocationID + " as its location.", 
            					"The location information needs to be verified for: " + entityPublicErrorString,
            					false)) {
                			write(locEl, "REF_ENTITY_ID", glossEID);
                			
                			var str = "Location {ID=" + locID + "} had REF_ENTITY_ID reset to parent gloss {ID=" + glossEID + "} because the gloss referenced it as its dependent.";
//                			var public_str = public_entity + " had its location information reset.";
                			addToModifications(20051, str, null);
            			}
            		}
            	} else if (isNotBlank(dependentLocationID)) {
            		for (Element locEntityEl: readChildren(altLocTrackEl, "ENTITY").collect(Collectors.toCollection(ArrayList::new))) {
            			var locEID = readOpt(locEntityEl, "ID").orElse(null);
            			if (isNotBlank(locEID) && locEID.equals(dependentLocationID)) {
            				locEl = locEntityEl;
            				break;
            			}
            		}

            		if (locEl != null) {
            			// the proper element by the id was found in the alt track, so it just means the dependent id on the gloss element was wrong and should be null
            			write(glossEl, "DEPENDENT_LOCATION_ID", "");
            			
            			var str = "Gloss {" + glossEID + "} had its DEPENDENT_LOCATION_ID {" + dependentLocationID + "} erased because the location was found on the track corresponding to the other hand.";
//            			var public_str = public_entity + " had its location information erased.";
            			addToModifications(20052, str, null);
            		} else {
                		addToErrors(10084, "Location {ID=" + dependentLocationID + "} was not found in Track {ID=" + locTrackID + "}. It was expected there as the dependent location of gloss {ID=" + glossEID + "}.", "The location information needs to be verified for: " + entityPublicErrorString);
            		}
            	}
            	
            	for (Element signTypeEntityEl: readChildren(signTypeTrackEl, "ENTITY").collect(Collectors.toCollection(ArrayList::new))) {
            		var signTypeEID = readOpt(signTypeEntityEl, "ID").orElse(null);
            		if (isNotBlank(signTypeEID) && signTypeEID.equals(dependentSignTypeID)) {
            			signTypeEl = signTypeEntityEl;
            			break;
            		}
            	}
            	
            	if (signTypeEl != null) {
            		var signTypeID = readOpt(signTypeEl, "ID").orElse(null);
            		var signTypeRefEntityID = readOpt(signTypeEl, "REF_ENTITY_ID").orElse(null);

            		if (isBlank(signTypeRefEntityID)) {
            			write(signTypeEl, "REF_ENTITY_ID", glossEID);
            			
            			var str = "Sign Type {ID=" + signTypeID + "} had REF_ENTITY_ID reset to parent gloss {ID=" + glossEID + "} because the gloss referenced it as its dependent.";
//            			var public_str = public_entity + " had its sign type information reset.";
            			addToModifications(20053, str, null);
            		} else {
            			if ( ! expect(signTypeRefEntityID.equals(glossEID), 10085, 
            					"Sign Type {ID=" + signTypeID + "} has reference to incorrect gloss {ID=" + signTypeRefEntityID + "}", 
            					"The sign type needs to be verified for: " + entityPublicErrorString,
            					false)) {
                			write(signTypeEl, "REF_ENTITY_ID", glossEID);
                			
                			var str = "Sign Type {ID=" + signTypeID + "} had REF_ENTITY_ID reset to parent gloss {ID=" + glossEID + "} because the gloss referenced it as its dependent.";
//                			var public_str = public_entity + " had its sign type information reset.";
                			addToModifications(20053, str, null);
            			}
            		}
            	} else if (isNotBlank(dependentSignTypeID)) {
            		for (Element signTypeEntityEl: readChildren(altSignTypeTrackEl, "ENTITY").collect(Collectors.toCollection(ArrayList::new))) {
            			var signTypeEID = readOpt(signTypeEntityEl, "ID").orElse(null);
            			if (isNotBlank(signTypeEID) && signTypeEID.equals(dependentSignTypeID)) {
            				signTypeEl = signTypeEntityEl;
            				break;
            			}
            		}

            		if (signTypeEl != null) {
            			// the proper element by the id was found in the alt track, so it just means the dependent id on the gloss element was wrong and should be null
            			write(glossEl, "DEPENDENT_SIGN_TYPE_ID", "");
            			
            			var str = "Gloss {" + glossEID + "} had its DEPENDENT_SIGN_TYPE_ID {" + dependentSignTypeID + "} erased because the sign type was found on the track corresponding to the other hand.";
//            			var public_str = public_entity + " had its sign type information erased.";
            			addToModifications(20054, str, null);
            		} else {
            			addToErrors(10086, "Sign Type {ID=" + dependentSignTypeID + "} was not found in Track {ID=" + signTypeTrackID + "}. It was expected there as the dependent sign type of gloss {ID=" + glossEID + "}.", "The sign type needs to be verified for: " + entityPublicErrorString);
            		}
            	}
            }
        }
    }

    private void correctSignTypesOrTwoHanded() {
    	var ndomGlossTrackEl = trackElements.get(Util.NDOM_GLOSS_FIELD_ID);
        var repr = "Track " + repr(ndomGlossTrackEl, "FID");

        for (Element ndomGlossEntityEl: readChildren(ndomGlossTrackEl, "ENTITY")
        		.collect(Collectors.toCollection(ArrayList::new))) {
            var erepr = repr + " entity " + repr(ndomGlossEntityEl, "ID");

            var propertiesEl = readChildOpt(ndomGlossEntityEl, "PROPERTIES").orElse(null);
            if (propertiesEl != null) {
                var noOfHands = readOpt(propertiesEl, "NO_OF_HANDS").orElse(null);
                if ("2".equals(noOfHands)) {
                	var dependentGlossID = readOpt(ndomGlossEntityEl, "DEPENDENT_GLOSS_ID").orElse(null);
                	if (isBlank(dependentGlossID)) {
                		write(propertiesEl, "NO_OF_HANDS", "1");
                		
                		var str = erepr + " had number of hands updated to \"1\" as its dependent/corresponding gloss is orphaned and/or dropped.";
//		    			var eid = readOpt(ndomGlossEntityEl, "ID").orElse(null);
//		    			var fid = entityTrackMap.get(eid);
//		    			var public_entity = getEntityPublicErrorString(eid, fid);
//                		var public_str = public_entity + " was identified as a two handed gloss, but only one hand appears to exist. Updating to identify as a one-handed gloss. Please verify";
                		addToModifications(20025, str, null);
                	} else {
                    	var dependentSignTypeID = readOpt(ndomGlossEntityEl, "DEPENDENT_SIGN_TYPE_ID").orElse(null);
                    	if (isNotBlank(dependentSignTypeID)) {
                    		write(ndomGlossEntityEl, "DEPENDENT_SIGN_TYPE_ID", "");	// pass empty string to be read as null
                            addToModifications(20055, erepr + " had dependent sign type removed since it is part of a 2H ndom gloss.", null);
                    	}
                	}
                }
            }
        }
    }
    
    private void correctRemoveInvalidLocationEntities() {
    	var domTrack = trackElements.get(Util.DOM_LOCATION_FIELD_ID);
    	var ndomTrack = trackElements.get(Util.NDOM_LOCATION_FIELD_ID);
    	if (domTrack == null && ndomTrack == null)
    		return;
    	
    	HashMap<String, Region> regions = SS3Singleton.getLocationConfig().getRegions();
    	
    	var locationTrackElements = List.of(domTrack, ndomTrack);

    	for (var trackEl : locationTrackElements) {
    		ArrayList<Element> locEntitiesToDelete = new ArrayList<Element>();

    		for (Element locEntityEl: readChildren(trackEl, "ENTITY").collect(Collectors.toCollection(ArrayList::new))) {
    			var locEntityID = readOpt(locEntityEl, "ID").orElse(null);
    			
    			var items = readChildren(locEntityEl, "ITEM").collect(Collectors.toList());
    			for (int i = 0; i < items.size(); i++) {
    				Element itemEl = items.get(i);
    				String locationRegionID = readOpt(itemEl, "LOCATION_REGION_ID").orElse(null);
    				if (isNotBlank(locationRegionID)) {
    					if (! locationRegionID.equals("-1") && !regions.containsKey(locationRegionID)) {
    						locEntitiesToDelete.add(locEntityEl);
    						
                    		var str = "Location {ID=" + locEntityID + "} was deleted due to location region ID {ID=" + locationRegionID + " not being found in the coding scheme.";
//                    		var public_str = glossPublicErrorString + " had its location data deleted due to being saved with an invalid location region."";
    						addToModifications(20056, str, null);
    					}
    				}
    			}
    		}
    		
    		for (Element locTrackEl: locEntitiesToDelete) {
    			trackEl.removeChild(locTrackEl);
    		}
    		
    	}    	
    }
    
    private void correctParentEntities() {
        // run through the tracks again, this time matching entity id's with parent utterance
        var tracksEl = readChildOpt(dbEl, "TRACKS").orElse(null);
        readChildren(tracksEl, "TRACK").forEach(tEl -> {
            var repr = "Track " + repr(tEl, "FID");
            var fid = readOpt(tEl, "FID").orElse(null);

            readChildren(tEl, "ENTITY").forEach(eEl -> {
                var erepr = repr + " entity " + repr(eEl, "ID");
                var eid = readOpt(eEl, "ID").orElse(null);
                
                var assumedUID = entityUtteranceMap.get(eid);
                var assumedTID = utteranceTierMap.get(assumedUID);
                
                if (deletedEntities.get(eid) != null && deletedEntities.get(eid).equals(fid))
                    return;

                var itemsList = itemElements.get(eid);
                if (itemsList != null && itemsList.size() >= 0) {
                	Element firstItem = itemsList.get(0);
                	Element lastItem = itemsList.get(itemsList.size() - 1);
                	var start = readLongOpt(firstItem, "S").orElse(null);
                	var end = readLongOpt(lastItem, "E").orElse(null);
                	
//                	String entityPublicErrorString = getEntityPublicErrorString(eid, fid);
                    
                    IntervalSet<String> tierUtterancesIntervalsSet = utteranceIntervals.get(assumedTID);
                    if (tierUtterancesIntervalsSet == null)
                    	tierUtterancesIntervalsSet = new IntervalSet<String>();              	

                    var entityInterval = TimeRange.ABSOLUTE.time(start, TimeUnit.TIMESTAMP, Rounding.FLOOR)
                        .to(TimeRange.ABSOLUTE.time(end, TimeUnit.TIMESTAMP, Rounding.FLOOR));
                    
                	ArrayList<String> utteranceIDs = 
                			tierUtterancesIntervalsSet.testContains(entityInterval, Bound.INCLUSIVE).orElse(new ArrayList<String>());

            			int size = utteranceIDs.size();
                    
                    var parentEntityID = readOpt(eEl, "PARENT_ENTITY_ID").orElse(null);
                    if (isBlank(parentEntityID)) {
                    	if (utteranceIDs != null) {
                        	if (size == 0) {
                        		addToErrors(10087, erepr + " is contained by no utterance. Parent entity was not set.", erepr + " is contained by no utterance. Parent entity was not set.", false);
                        		// NOTE: adding to the errors array this way is the EXCEPTION, not the rule - only doing this way as a special case where we want to see this error privately, but not publically
                        		
                        		// don't delete, instead keep it around until conversion when multiple parent utterances could be set
//                        		write(eEl, "TAGGED_FOR_DELETION", "T");
                        	} else if (size == 1) {
                            	String utteranceID = utteranceIDs.get(0);

                            	if (isNotBlank(utteranceID)) {
                            		write(eEl, "PARENT_ENTITY_ID", utteranceID);
                            		addToModifications(20057, erepr + " had PARENT_ENTITY_ID updated to: " + repr(utteranceID), null);
                            	}
                        	} else {
                        		addToErrors(10088, erepr + " is contained by multiple utterances. Parent entity was not set.", erepr + " is contained by multiple utterances. Parent entity was not set.", false);
                        		// NOTE: adding to the errors array this way is the EXCEPTION, not the rule - only doing this way as a special case where we want to see this error privately, but not publically
                        			
                        		// don't delete, instead keep around in case utterance overlap is resolved later during validation
//                        		write(eEl, "TAGGED_FOR_DELETION", "T");
                        	}
                    	}
                    } else {
                    	if (size == 1) {
                    		String utteranceID = utteranceIDs.get(0);

                        	if (isNotBlank(utteranceID)) {
                				if ( ! expect(parentEntityID.equals(utteranceID), 10089, 
                						"Entity {ID=" + eid + "} has reference to incorrect parent entity {ID=" + parentEntityID + "}", 
                						null,
                						false)) {
                            		write(eEl, "PARENT_ENTITY_ID", utteranceID);
                            		addToModifications(20057, erepr + " had PARENT_ENTITY_ID updated to: " + utteranceID, null);
                				}
                        	}
                    	}
                    }
                }
            });
        });
        
        	// run through again, this time tagging glosses if any dependents are tagged
        readChildren(tracksEl, "TRACK").forEach(tEl -> {
            var repr = "Track " + repr(tEl, "FID");
            var fid = readOpt(tEl, "FID").orElse(null);

            readChildren(tEl, "ENTITY").forEach(eEl -> {
                var erepr = repr + " entity " + repr(eEl, "ID");
                var eid = readOpt(eEl, "ID").orElse(null);

                if (deletedEntities.get(eid) != null && deletedEntities.get(eid).equals(fid))
                    return;
                
                var dependentHandID = readOpt(eEl, "DEPENDENT_HAND_ID").orElse(null);
                var dependentHandshapeID = readOpt(eEl, "DEPENDENT_HAND_SHAPE_ID").orElse(null);
                var dependentLocationID = readOpt(eEl, "DEPENDENT_LOCATION_ID").orElse(null);
                var dependentSignTypeID = readOpt(eEl, "DEPENDENT_SIGN_TYPE_ID").orElse(null);
                var dependentGlossID = readOpt(eEl, "DEPENDENT_GLOSS_ID").orElse(null);

                Element depHandEl = null, depHandshapeEl = null, depLocationEl = null, depSignTypeEl = null, depGlossEl = null;
                
                if (isNotBlank(dependentHandID))
                	depHandEl = entityElements.get(dependentHandID);
                if (isNotBlank(dependentHandshapeID)) 
                	depHandshapeEl = entityElements.get(dependentHandshapeID);
                if (isNotBlank(dependentLocationID))
                	depLocationEl = entityElements.get(dependentLocationID);
                if (isNotBlank(dependentSignTypeID))
                	depSignTypeEl = entityElements.get(dependentSignTypeID);
                if (isNotBlank(dependentGlossID))
                	depGlossEl = entityElements.get(dependentGlossID);

                if (depHandEl != null) {
                	var tagged = readOpt(depHandEl, "TAGGED_FOR_DELETION").orElse(null);
                	if (isNotBlank(tagged)) {
                		write(eEl, "TAGGED_FOR_DELETION", "T");
                	}
	            }
                if (depHandshapeEl != null) {
                	var tagged = readOpt(depHandshapeEl, "TAGGED_FOR_DELETION").orElse(null);
                	if (isNotBlank(tagged)) {
                		write(eEl, "TAGGED_FOR_DELETION", "T");
                	}
	            }
                if (depLocationEl != null) {
                	var tagged = readOpt(depLocationEl, "TAGGED_FOR_DELETION").orElse(null);
                	if (isNotBlank(tagged)) {
                		write(eEl, "TAGGED_FOR_DELETION", "T");
                	}
	            }
                if (depSignTypeEl != null) {
                	var tagged = readOpt(depSignTypeEl, "TAGGED_FOR_DELETION").orElse(null);
                	if (isNotBlank(tagged)) {
                		write(eEl, "TAGGED_FOR_DELETION", "T");
                	}
	            }
                if (depGlossEl != null) {
                	var tagged = readOpt(depGlossEl, "TAGGED_FOR_DELETION").orElse(null);
                	if (isNotBlank(tagged)) {
                		write(eEl, "TAGGED_FOR_DELETION", "T");
                	}
	            }
            });
        });

        // run through again, this time tagging dependents of any tagged glosses
        readChildren(tracksEl, "TRACK").forEach(tEl -> {
	        	var repr = "Track " + repr(tEl, "FID");
	        	var fid = readOpt(tEl, "FID").orElse(null);
	
	        	readChildren(tEl, "ENTITY").forEach(eEl -> {
	        		var erepr = repr + " entity " + repr(eEl, "ID");
	        		var eid = readOpt(eEl, "ID").orElse(null);
	
	        		if (deletedEntities.get(eid) != null && deletedEntities.get(eid).equals(fid))
	        			return;
	
	        		var dependentHandID = readOpt(eEl, "DEPENDENT_HAND_ID").orElse(null);
	        		var dependentHandshapeID = readOpt(eEl, "DEPENDENT_HAND_SHAPE_ID").orElse(null);
	        		var dependentLocationID = readOpt(eEl, "DEPENDENT_LOCATION_ID").orElse(null);
	        		var dependentSignTypeID = readOpt(eEl, "DEPENDENT_SIGN_TYPE_ID").orElse(null);
	        		var dependentGlossID = readOpt(eEl, "DEPENDENT_GLOSS_ID").orElse(null);
	
	        		Element depHandEl = null, depHandshapeEl = null, depLocationEl = null, depSignTypeEl = null, depGlossEl = null;
	                
	        		if (isNotBlank(dependentHandID))
	        			depHandEl = entityElements.get(dependentHandID);
	        		if (isNotBlank(dependentHandshapeID)) 
	        			depHandshapeEl = entityElements.get(dependentHandshapeID);
	        		if (isNotBlank(dependentLocationID))
	        			depLocationEl = entityElements.get(dependentLocationID);
	        		if (isNotBlank(dependentSignTypeID))
	        			depSignTypeEl = entityElements.get(dependentSignTypeID);
	        		if (isNotBlank(dependentGlossID))
	        			depGlossEl = entityElements.get(dependentGlossID);

                var taggedForDeletion = readOpt(eEl, "TAGGED_FOR_DELETION").orElse(null);
                if (taggedForDeletion != null && taggedForDeletion.equals("T")) {
                	if (depHandEl != null)
                		write(depHandEl, "TAGGED_FOR_DELETION", "T");
                	if (depHandshapeEl != null)
                		write(depHandshapeEl, "TAGGED_FOR_DELETION", "T");
                	if (depLocationEl != null)
                		write(depLocationEl, "TAGGED_FOR_DELETION", "T");
                	if (depSignTypeEl != null)
                		write(depSignTypeEl, "TAGGED_FOR_DELETION", "T");
                	if (depGlossEl != null)
                		write(depGlossEl, "TAGGED_FOR_DELETION", "T");
                }
	        	});
        });
        
        NodeList list = rootEl.getElementsByTagName("ENTITY");
        	int size = list.getLength();
        	for (int i = 0; i < size; i++) {
        		Element entityEl = (Element) list.item(i);
        		if (entityEl != null) {
        			var tagged = readOpt(entityEl, "TAGGED_FOR_DELETION").orElse(null);
        			if (isBlank(tagged))
        				continue;
        			
        			var eid = readOpt(entityEl, "ID").orElse(null);
        			if (isNotBlank(eid)) {
        				var trackID = entityTrackMap.get(eid);
        				if (isNotBlank(trackID)) {
        					var trackEl = trackElements.get(trackID);
        					
        					trackEl.removeChild(entityEl);
        					entityElements.remove(eid);
        					deletedEntities.put(eid, trackID);
        					
        					addToModifications(20058, "Entity {ID=" + eid + "} was removed as it is part of a gloss or entity which is not properly contained by an utterance.", null);	// this should not be called anymore
        				}
        			}
        		}
        	}
        
    }
    
    private void validateParticipantDominance() {
    	var domTrack = trackElements.get(Util.DOM_GLOSS_FIELD_ID);
    	var ndomTrack = trackElements.get(Util.NDOM_GLOSS_FIELD_ID);
    	if (domTrack == null && ndomTrack == null)
    		return;
    	
    	var glossTrackElements = List.of(domTrack, ndomTrack);
    	
        for (var tEl : glossTrackElements) {
        	var glossTrackID = readOpt(tEl, "FID").orElse(null);
            var repr = "Track " + repr(tEl, "FID");
            for (Element eEl: readChildren(tEl, "ENTITY")
            		.collect(Collectors.toCollection(ArrayList::new))) {
                var glossEID = readOpt(eEl, "ID").orElse(null);
                var erepr = repr + " entity " + repr(eEl, "ID");

                if (deletedEntities.get(glossEID) != null && deletedEntities.get(glossEID).equals(glossTrackID))
                    continue;
                
                var dominance = readOpt(eEl, "DOMINANCE").orElse(null);
                expect(isNotBlank(dominance), 10090, erepr + " is missing a value for DOMINANCE.", null);
	            	var parentEntityID = readOpt(eEl, "PARENT_ENTITY_ID").orElse(null);
	            	if (isNotBlank(parentEntityID)) {
	            		var utteranceEl =utteranceElements.get(parentEntityID);
	            		if (utteranceEl != null) {
	            			var segmentEl = (Element) utteranceEl.getParentNode();
	            			if (segmentEl != null) {
	            				var participantID = readOpt(segmentEl, "PARTICIPANT_ID").orElse(null);
	            				if (isNotBlank(participantID)) {
	            					var participantEl = participantElements.get(participantID);
	            					if (participantEl != null) {
	            						var participantDominance = readOpt(participantEl, "DOMINANT_HAND").orElse(null);
	            						if (isNotBlank(participantDominance)) {
	            							if ( !participantDominance.equals(dominance)) {
	            								write(eEl, "DOMINANCE", participantDominance);
	            								addToModifications(20059, erepr + " had DOMINANCE value corrected to match its corresponding Participant.", null);
	            							}
	            						}
	            					}
	            				}
	            			}
	            		}
	            	}
            }
        }
    }
    
    private SS3ValidatorConverterReturns validateAndCorrectDependents() {
    	// run through the tracks again, this time validating dependent id's
        var tracksEl = readChildOpt(dbEl, "TRACKS").orElse(null);
        readChildren(tracksEl, "TRACK").forEach(tEl -> {
            var repr = "Track " + repr(tEl, "FID");
            var fid = readOpt(tEl, "FID").orElse(null);

            readChildren(tEl, "ENTITY").forEach(eEl -> {
                var erepr = repr + " entity " + repr(eEl, "ID");
                var eid = readOpt(eEl, "ID").orElse(null);

                if (deletedEntities.get(eid) != null && deletedEntities.get(eid).equals(fid))
                    return;

                readChildren(eEl, "ITEM").forEach(iEl -> {
                    var irepr = erepr + " item";
                    var start = readLongOpt(iEl, "S").orElse(null);
                    var end = readLongOpt(iEl, "E").orElse(null);

                    var interval = TimeRange.ABSOLUTE.time(start, TimeUnit.NANOSECOND)
                        .to(TimeRange.ABSOLUTE.time(end, TimeUnit.NANOSECOND));
                    
                    boolean entityIsTextOnly = false;
                    var textOnly = readOpt(iEl, "TEXT_ONLY").orElse(null);	// values either "T" or "F"
                    if (isNotBlank(textOnly) && textOnly.equals("T")) {
                    	entityIsTextOnly = true;
                    }
                    
                    if (isNotBlank(fid)) {
                    	if ( !entityIsTextOnly && fid.equals(Util.DOM_GLOSS_FIELD_ID) || fid.equals(Util.NDOM_GLOSS_FIELD_ID)) {
                            
                    		String glossPublicErrorString = getEntityPublicErrorString(eid, fid);
                            
                            var propertiesEl = readChildOpt(eEl, "PROPERTIES").orElse(null);
                            if (expect(propertiesEl != null, 10073, erepr + "missing PROPERTIES element.", null)) {

                            	var dependentGlossID = readOpt(eEl, "DEPENDENT_GLOSS_ID").orElse(null);
                                var correspondingGlossEntityID = readOpt(eEl, "CORRESPONDING_GLOSS_ENTITY_ID").orElse(null);
                                
                            	if (isNotBlank(dependentGlossID)) {
                            		expect(entityTrackMap.containsKey(dependentGlossID),
                            				10091, erepr + " references a gloss entity not found in any track: " + dependentGlossID, "The hand configuration needs to be reset for both hands for: " + glossPublicErrorString);
                            		
                                    if (isNotBlank(correspondingGlossEntityID)) {
                                		expect(entityTrackMap.containsKey(correspondingGlossEntityID),
                                				10091, erepr + " references a gloss entity not found in any track: " + correspondingGlossEntityID, "The hand configuration needs to be reset for both hands for: " + glossPublicErrorString);
                                    	
                                    	if (expect(dependentGlossID.equals(correspondingGlossEntityID),
                                    			10092, erepr + " has mismatching ID's for dependent gloss and corresponding gloss.", "The hand configuration needs to be reset for both hands for: " + glossPublicErrorString)) {
                                    		var dependentEl = entityElements.get(dependentGlossID);
                                    		if (expect(dependentEl != null, 
                                    				10093, "Entity {ID=" + dependentGlossID + "} was not found, likely to have not been in a track", null)) {
                                                var depEid = readOpt(dependentEl, "ID").orElse(null);
                                        		var dependentPropertiesEl = readChildOpt(dependentEl, "PROPERTIES").orElse(null);
                                        		
                                                var dep_erepr = "Track {FID=" + entityTrackMap.get(depEid) + "} " + " entity " + repr(dependentEl, "ID");

                                                var depFid = fid.equals(Util.DOM_GLOSS_FIELD_ID) ? Util.NDOM_GLOSS_FIELD_ID : Util.DOM_GLOSS_FIELD_ID;
                                                String depGlossPublicErrorString = getEntityPublicErrorString(depEid, depFid);
                                                
                                            	var reverseDependentGlossID = readOpt(dependentEl, "DEPENDENT_GLOSS_ID").orElse(null);
                                                var reverseCorrespondingGlossEntityID = readOpt(dependentEl, "CORRESPONDING_GLOSS_ENTITY_ID").orElse(null);
                                                
                                                expect(eid.equals(reverseDependentGlossID) && eid.equals(reverseCorrespondingGlossEntityID), 
                                                		10094, erepr + " and " + dep_erepr + " do not refer to each other as corresponding glosses correctly.", glossPublicErrorString + " and " + depGlossPublicErrorString + " do not reference each other as dependents correctly.");
                                                
                                                var markedNumberHands = readOpt(eEl, "MARKED_NUMBER_OF_HANDS").orElse(null);
                                                
                                                var noOfHands = readOpt(propertiesEl, "NO_OF_HANDS").orElse(null);
                                                var sign = readOpt(propertiesEl, "SIGN").orElse(null);
                                                var nameSign = readOpt(propertiesEl, "NAME_SIGN").orElse(null);
                                                var passiveBaseArm = readOpt(propertiesEl, "PASSIVE_BASE_ARM").orElse(null);
                                                var twoHandedHandshapes = readOpt(propertiesEl, "TWOHANDED_HADSHAPES").orElse(null);
                                                if (twoHandedHandshapes == null) {
                                                	twoHandedHandshapes = readOpt(propertiesEl, "TWOHANDED_HANDSHAPES").orElse(null);
                                                }
                                                var leftHandStartEndHS = readOpt(propertiesEl, "LEFT_HAND_START_END_HANDSHAPE").orElse(null);
                                                var rightHandStartEndHS = readOpt(propertiesEl, "RIGHT_HAND_START_END_HANDSHAPE").orElse(null);
                                                var startEndHS = readOpt(propertiesEl, "START_END_HANDSHAPES").orElse(null);

                                                var depMarkedNumberHands = readOpt(dependentEl, "MARKED_NUMBER_OF_HANDS").orElse(null);
                                                
                                                var depNoOfHands = readOpt(dependentPropertiesEl, "NO_OF_HANDS").orElse(null);
                                                var depSign = readOpt(dependentPropertiesEl, "SIGN").orElse(null);
                                                var depNameSign = readOpt(dependentPropertiesEl, "NAME_SIGN").orElse(null);
                                                var depPassiveBaseArm = readOpt(dependentPropertiesEl, "PASSIVE_BASE_ARM").orElse(null);
                                                var depTwoHandedHandshapes = readOpt(dependentPropertiesEl, "TWOHANDED_HADSHAPES").orElse(null);
                                                if (depTwoHandedHandshapes == null) {
                                                	depTwoHandedHandshapes = readOpt(dependentPropertiesEl, "TWOHANDED_HANDSHAPES").orElse(null);
                                                }
                                                var depLeftHandStartEndHS = readOpt(dependentPropertiesEl, "LEFT_HAND_START_END_HANDSHAPE").orElse(null);
                                                var depRightHandStartEndHS = readOpt(dependentPropertiesEl, "RIGHT_HAND_START_END_HANDSHAPE").orElse(null);
                                                var depStartEndHS = readOpt(dependentPropertiesEl, "START_END_HANDSHAPES").orElse(null);
                                                
                                                if (isNotBlank(markedNumberHands) && isNotBlank(depMarkedNumberHands)) {
                                                	expect(markedNumberHands.equals(depMarkedNumberHands),
                                                			10095, erepr + " and " + dep_erepr + " have mismatching values for marked number of hands.", "The information about whether there is a marked number of hands needs to be verified for: " + glossPublicErrorString);
                                                }
                                                if (isNotBlank(noOfHands) && isNotBlank(depNoOfHands)) {
                                                	expect(noOfHands.equals(depNoOfHands), 
                                                			10096, erepr + " and " + dep_erepr + " have mismatching values for number of hands.", glossPublicErrorString + " has invalid two-handed configuration.");
                                                }
                                                if (isNotBlank(sign) && isNotBlank(depSign)) {
                                                	expect(sign.equals(depSign), 
                                                			10097, erepr + " and " + dep_erepr + " have mismatching values for sign.", "The sign type needs to be verified for: " + glossPublicErrorString);
                                                }
                                                if (isNotBlank(nameSign) && isNotBlank(depNameSign)) {
                                                	expect(nameSign.equals(depNameSign), 
                                                			10098, erepr + " and " + dep_erepr + " have mismatching values for name sign.", "There may be an issue with the setting for \"Name sign\" for: " + glossPublicErrorString);
                                                }
                                                if (isNotBlank(passiveBaseArm) && isNotBlank(depPassiveBaseArm)) {
                                                	expect(passiveBaseArm.equals(depPassiveBaseArm), 
                                                			10099, erepr + " and " + dep_erepr + " have mismatching values for passive base arm.", "There may be an issue with the setting for \"Passive base arm\" for: " + glossPublicErrorString);
                                                }
                                                if (isNotBlank(twoHandedHandshapes) && isNotBlank(depTwoHandedHandshapes)) {
                                                	expect(twoHandedHandshapes.equals(depTwoHandedHandshapes),
                                                			10100, erepr + " and " + dep_erepr + " have mismatching values for twohanded handshapes.", "It is necessary to reset all phonological information for: " + glossPublicErrorString);
                                                }
                                                if (isNotBlank(leftHandStartEndHS) && isNotBlank(depLeftHandStartEndHS)) {
                                                	expect (leftHandStartEndHS.equals(depLeftHandStartEndHS), 
                                                			10101, erepr + " and " + dep_erepr + " have mismatching values for ndom hand start end handshape.", "It is necessary to reset all phonological information for: " + glossPublicErrorString);
                                                }
                                                if (isNotBlank(rightHandStartEndHS) && isNotBlank(depRightHandStartEndHS)) {
                                                	expect(rightHandStartEndHS.equals(depRightHandStartEndHS),
                                                			10102, erepr + " and " + dep_erepr + " have mismatching values for dom hand start end handshape.", "It is necessary to reset all phonological information for: " + glossPublicErrorString);
                                                }
                                                if (isNotBlank(startEndHS) && isNotBlank(depStartEndHS)) {
                                                	expect(startEndHS.equals(depStartEndHS), 
                                                			10103, erepr + " and " + dep_erepr + " have mismatching values for start end handshape.", "It is necessary to reset all phonological information for: " + glossPublicErrorString);
                                                }
                                                
                                                // check for locking issues
                                                var elLocked = readOpt(eEl, "LOCKED_LEF_RIGHT_ALIGNMENT").orElse(null);
                                                if (isBlank(elLocked)) {
                                                    elLocked = readOpt(eEl, "LOCKED_LEFT_RIGHT_ALIGNMENT").orElse(null);
                                                }
                                                var depLocked = readOpt(dependentEl, "LOCKED_LEF_RIGHT_ALIGNMENT").orElse(null);
                                                if (isBlank(depLocked)) {
                                                    depLocked = readOpt(dependentEl, "LOCKED_LEFT_RIGHT_ALIGNMENT").orElse(null);
                                                }
                                                
                                                boolean bothLocked = "T".equals(elLocked) && "T".equals(depLocked);
                                                boolean bothUnlocked = "F".equals(elLocked) && "F".equals(depLocked);
                                                boolean matchingLockedValues = bothLocked || bothUnlocked;
                                                
                                            	var elItemList = itemElements.get(eid).stream().sorted(comp).collect(
                                            			Collectors.toCollection(ArrayList::new));
                                            	var depItemList = itemElements.get(depEid).stream().sorted(comp).collect(
                                            			Collectors.toCollection(ArrayList::new));
                                            	int elItemListSize = elItemList.size();
                                            	int depItemListSize = depItemList.size();
                                            	
                                            	var elStart = readLongOpt(elItemList.get(0), "S");
                                            	var depStart = readLongOpt(depItemList.get(0), "S");
                                            	var elEnd = readLongOpt(elItemList.get(elItemListSize - 1), "E");
                                            	var depEnd = readLongOpt(depItemList.get(depItemListSize - 1), "E");
                                            	
                                            	boolean startsEqual = elStart.equals(depStart);
                                            	boolean endsEqual = elEnd.equals(depEnd);
                                                
                                                if ( ! expect(matchingLockedValues,
                                                		10104, 
                                                		erepr + " and " + dep_erepr + " have mismatching values for locked dom/ndom alignment.", 
                                                		"It is necessary to verify/reset the relationship between the dominant and non-dominant hand (locked vs. unlocked) for: " + glossPublicErrorString, 
                                                		false)) {
                                                	
                                                	if (startsEqual && endsEqual) {
                                                		// mismatching locked values, assume meant to be locked, so set values to locked

                                                		write(eEl, "LOCKED_LEF_RIGHT_ALIGNMENT", "T");
                                                		
                                                		var str = erepr + " had LOCKED_LEF_RIGHT_ALIGNMENT updated to \"T\" since the gloss and its dependent have equal start/end times.";
                                                		addToModifications(20085, str, null);
                                                	} else {
                                                		// mismatching locked values, assume not meant to be locked, so set values to unlocked

                                                		write(eEl, "LOCKED_LEF_RIGHT_ALIGNMENT", "F");
                                                		
                                                		var str = erepr + " had LOCKED_LEF_RIGHT_ALIGNMENT updated to \"F\" since the gloss and its dependent have unequal start/end times.";
                                                		addToModifications(20086, str, null);
                                                	}
                                                } else if (bothLocked) {
                                                	String str = "It is necessary to verify/reset the relationship between the dominant and non-dominant hand for: " 
                                                			+ glossPublicErrorString + ". They appear to be set to be locked, which means that they should have the same start and end frames, but they do not.";
                                                	
                                                	expect(startsEqual,
                                                			10105, erepr + " and " + dep_erepr + " are locked but start/end times are not equal.", str);
                                                	
                                                	expect(endsEqual,
                                                			10105, erepr + " and " + dep_erepr + " are locked but start/end times are not equal.", str);
                                                }
                                    		}
                                    	}
                                    } else {
                                        var noOfHands = readOpt(propertiesEl, "NO_OF_HANDS").orElse(null);
                                        if ("1".equals(noOfHands)) {
                                        	var lockedLeftRight = readOpt(eEl, "LOCKED_LEF_RIGHT_ALIGNMENT").orElse(null);
                                        	if ( ! "F".equals(lockedLeftRight)) {
                                        		write(eEl, "LOCKED_LEF_RIGHT_ALIGNMENT", "F");
                                        		
                                        		var str = erepr + " had LOCKED_LEF_RIGHT_ALIGNMENT updated to \"F\" since it is one handed.";
//                                        		var public_str = "This gloss had Locked-Left-Right value set to unlocked since it is one-handed: " + glossPublicErrorString;
                                        		addToModifications(20060, str, null);
                                        	}
                                        }
                                	}
                            	}
                            	
                            	var dependentHandID = readOpt(eEl, "DEPENDENT_HAND_ID").orElse(null);
                            	if (isNotBlank(dependentHandID)) {
                            		if (!expect(entityTrackMap.containsKey(dependentHandID),
                            				10106, erepr + " references a hand entity not found in any track: " + dependentHandID, null, false)) {
                            			write(eEl, "DEPENDENT_HAND_ID", "");
                            			
                            			var str = erepr + " had DEPENDENT_HAND_ID erased because it referenced an entity not found in any track.";
//                            			var public_str = glossPublicErrorString + " had its hand information erased.";
                            			addToModifications(20061, str, null);
                            		}
                            	}
                            	
                            	var dependentHandshapeID = readOpt(eEl, "DEPENDENT_HAND_SHAPE_ID").orElse(null);
                            	if (isNotBlank(dependentHandshapeID)) {
                            		if (!expect(entityTrackMap.containsKey(dependentHandshapeID),
                            				10107, erepr + " references a handshape entity not found in any track: " + dependentHandshapeID, null, false)) {
                            			write(eEl, "DEPENDENT_HAND_SHAPE_ID", "");
                            			
                            			var str = erepr + " had DEPENDENT_HAND_SHAPE_ID erased because it referenced an entity not found in any track.";
//                            			var public_str = glossPublicErrorString + " had its handshape information erased.";
                            			addToModifications(20062, str, null);
                            		}
                            	}
                            	
                            	var dependentLocationID = readOpt(eEl, "DEPENDENT_LOCATION_ID").orElse(null);
                            	if (isNotBlank(dependentLocationID)) {
                            		if (!expect(entityTrackMap.containsKey(dependentLocationID),
                            				10108, erepr + " references a location entity not found in any track: " + dependentLocationID, null, false)) {
                            			write(eEl, "DEPENDENT_LOCATION_ID", "");
                            			
                            			var str = erepr + " had DEPENDENT_LOCATION_ID erased because it referenced an entity not found in any track.";
//                            			var public_str = glossPublicErrorString + " had its location information erased.";
                            			addToModifications(20063, str, null);
                            		}
                            	}
                            	
                            	var dependentSignTypeID = readOpt(eEl, "DEPENDENT_SIGN_TYPE_ID").orElse(null);
                            	if (isNotBlank(dependentSignTypeID)) {
                            		if (!expect(entityTrackMap.containsKey(dependentSignTypeID),
                            				10109, erepr + " references a sign type entity not found in any track: " + dependentSignTypeID, null, false)) {
                            			write(eEl, "DEPENDENT_SIGN_TYPE_ID", "");
                            			
                            			var str = erepr + " had DEPENDENT_SIGN_TYPE_ID erased because it referenced an entity not found in any track.";
//                            			var public_str = glossPublicErrorString + " had its sign type information erased.";
                            			addToModifications(20064, str, null);
                            		}
                            	}
                            	

                            	var elItemList = itemElements.get(eid);
                            	Element onset = null, initialHold = null, main = null, finalHold = null, offset = null;
                            	Element operativeStartItem = null, operativeEndItem = null;
                            	for (Element item: elItemList) {
                                    var itemText = getCharacterDataFromElement(item);
                            		if (itemText.equals(Constants.ONSET))
                            			onset = item;
                            		else if (itemText.equals(Constants.INITIAL_HOLD))
                            			initialHold = item;
                            		else if (itemText.equals(Constants.FINAL_HOLD))
                            			finalHold = item;
                            		else if (itemText.equals(Constants.OFFSET))
                            			offset = item;
                            		else
                            			main = item;
                            	}
                            	
                            	if (initialHold != null)
                            		operativeStartItem = initialHold;
                            	else
                            		operativeStartItem = main;
                            	
                            	if (finalHold != null)
                            		operativeEndItem = finalHold;
                            	else
                            		operativeEndItem = main;
                            	
                            	var entityStart = readLongOpt(operativeStartItem, "S").orElse(null);
                            	var entityEnd = readLongOpt(operativeEndItem, "E").orElse(null);
                            	
                            	
                                var handTrackID = fid.equals(Util.DOM_GLOSS_FIELD_ID)
                                        ? Util.DOM_HAND_FIELD_ID
                                        : Util.NDOM_HAND_FIELD_ID;
                                
                                var hsTrackID = fid.equals(Util.DOM_GLOSS_FIELD_ID)
                                    ? Util.DOM_HANDSHAPE_FIELD_ID
                                    : Util.NDOM_HANDSHAPE_FIELD_ID;

                                var handTrackEl = trackElements.get(handTrackID);
                                var hsTrackEl = trackElements.get(hsTrackID);
                                
                                var handEl = entityElements.get(dependentHandID);
                            	var hsEl = entityElements.get(dependentHandshapeID);

                            	SS3Field hsField = defScheme3_3_5.getField(hsTrackID);
                            	boolean anyHSValuesNotFound = false;

                            	var hsItemsList = itemElements.get(dependentHandshapeID);
                            	if (hsItemsList != null && hsItemsList.size() > 0) {
                                	for (Element hsItemEl: hsItemsList) {
                                		String vid = read(hsItemEl, "VID");
                                    	SS3FieldValue value = hsField.getFieldValue(vid);
                                    	if (value == null) {
                                			anyHSValuesNotFound = true;
                                    		break;
                                    	}
                                	}
                            	}
                            	
                                var entityText = getCharacterDataFromElement(main);
                                if (entityText.startsWith("(recovered)") || anyHSValuesNotFound) {
                        			// additional cases:
                                	// case 1) check for "?" question mark handshapes, which were added, and remove the handshape entity entirely
                                	// case 2) handshape value is not found within our coding scheme
                                	
                                	int code = 0;
                                	String reason = null;
                                	if (entityText.startsWith("(recovered)")) {
                                    	reason = "because it was incorrectly automatically added by a previous version of SignStream with the tag \"(recovered)\"";

                            			write(eEl, "DEPENDENT_HAND_ID", "");
                            			var str = erepr + " had DEPENDENT_HAND_ID " + dependentHandID + " erased " + reason;
                            			var public_str = "This gloss had its hand information erased " + reason + ": " + glossPublicErrorString;
                            			addToModifications(20065, str, public_str);

                            			write(eEl, "DEPENDENT_HAND_SHAPE_ID", "");
                            			str = erepr + " had DEPENDENT_HAND_SHAPE_ID " + dependentHandshapeID + " erased " + reason;
                            			public_str = "This gloss had its handshape information erased " + reason + ": " + glossPublicErrorString;
                            			addToModifications(20066, str, public_str);
                                    	
                                    	handTrackEl.removeChild(handEl);
                                        addToModifications(20067, "Hand entity " + dependentHandID + " was removed from Track {FID=" + handTrackID + "} " + reason, null);
                                    	
                                        hsTrackEl.removeChild(hsEl);
                                        addToModifications(20068, "Handshape entity " + dependentHandshapeID + " was removed from Track {FID=" + hsTrackID + "} " + reason, null);
                                	} else {	// implies anyValuesNotFound == true
                                		reason = "because it references a handshape which is not found in the coding scheme.";

                            			write(eEl, "DEPENDENT_HAND_ID", "");
                            			var str = erepr + " had DEPENDENT_HAND_ID " + dependentHandID + " erased " + reason;
//                            			var public_str = glossPublicErrorString + " had its hand information erased.";
                            			addToModifications(20069, str, null);

                            			write(eEl, "DEPENDENT_HAND_SHAPE_ID", "");
                            			str = erepr + " had DEPENDENT_HAND_SHAPE_ID " + dependentHandshapeID + " erased " + reason;
//                            			public_str = glossPublicErrorString + " had its handshape information erased.";
                            			addToModifications(20070, str, null);
                                    	
                                    	handTrackEl.removeChild(handEl);
                                        addToModifications(20071, "Hand entity " + dependentHandID + " was removed from Track {FID=" + handTrackID + "} " + reason, null);
                                    	
                                        hsTrackEl.removeChild(hsEl);
                                        addToModifications(20072, "Handshape entity " + dependentHandshapeID + " was removed from Track {FID=" + hsTrackID + "} " + reason, null);
                                	}
                                } else {
                                	// hand sync
                                	var handList = itemElements.get(dependentHandID);
                                	if (handList != null) {
                                    	var depHandItemList = handList.stream().sorted(comp).collect(
                                    			Collectors.toCollection(ArrayList::new));
                                    	int depHandItemListSize = depHandItemList.size();
                                		var dependentHandEl = entityElements.get(dependentHandID);
                                		var trackFID = entityTrackMap.get(dependentHandID);
                                        var dep_hand_erepr = "Track {FID=" + trackFID + "} " 
                                        		+ " hand entity " + repr(dependentHandEl, "ID");
                                        
                                        var firstItem = depHandItemList.get(0);
                                        var lastItem = depHandItemList.get(depHandItemListSize - 1);
                                        var itemsStart = readLongOpt(firstItem, "S").orElse(null);
                                        var itemsEnd = readLongOpt(lastItem, "E").orElse(null);

                            			var glossLocked = readOpt(eEl, "LOCKED_GLOSS_PHON_ALIGNMENT").orElse(null);
                            			var handLocked = readOpt(handEl, "LOCKED_GLOSS_PHON_ALIGNMENT").orElse(null);
                            			
                            			if (isNotBlank(glossLocked) && isNotBlank(handLocked)) {
                            				if (glossLocked.equals("T") && handLocked.equals("T")) {
                                            	if (!entityStart.equals(itemsStart)) {
                                            		writeLong(firstItem, "S", entityStart);
                                            		
                                            		var str = dep_hand_erepr + " hands were locked but time was not synced, so it had start time synced to parent gloss.";
                                            		var public_str = "This gloss had the start time on its hand event synced to the main event: " + glossPublicErrorString;
                                            		addToModifications(20073, str, public_str);
                                            	}
                                            	
                                            	if (!entityEnd.equals(itemsEnd)) {
                                            		writeLong(lastItem, "E", entityEnd);
                                            		
                                            		var str = dep_hand_erepr + " hands were locked but time was not synced, so it had end time synced to parent gloss.";
                                            		var public_str = "This gloss had the end time on its hand event synced to the main event: " + glossPublicErrorString;
                                            		addToModifications(20074, str, public_str);
                                            	}
                            				}
                            			}
                                	}
                                }
                            	
                            	// reset these values for sign types
                            	if (onset != null)
                            		operativeStartItem = onset;
                            	else if (initialHold != null)
                            		operativeStartItem = initialHold;
                            	else
                            		operativeStartItem = main;
                            	
                            	if (offset != null)
                            		operativeEndItem = offset;
                            	else if (finalHold != null)
                            		operativeEndItem = finalHold;
                            	else
                            		operativeEndItem = main;
                            	
                            	entityStart = readLongOpt(operativeStartItem, "S").orElse(null);
                            	entityEnd = readLongOpt(operativeEndItem, "E").orElse(null);
                            	
                            	// sign type sync
                            	var signTypeList = itemElements.get(dependentSignTypeID);
                            	if (signTypeList != null) {
                                	var depSignTypeItemList = signTypeList.stream().sorted(comp).collect(
                                			Collectors.toCollection(ArrayList::new));
                                	int depSignTypeItemListSize = depSignTypeItemList.size();
                            		var dependentSignTypeEl = entityElements.get(dependentSignTypeID);
                            		var trackFID = entityTrackMap.get(dependentSignTypeID);
                                    var dep_signtype_erepr = "Track {FID=" + trackFID + "}" 
                                    		+ " sign type entity " + repr(dependentSignTypeEl, "ID");
                                    
                                    var firstItem = depSignTypeItemList.get(0);
                                    var lastItem = depSignTypeItemList.get(depSignTypeItemListSize - 1);
                                    var itemsStart = readLongOpt(firstItem, "S").orElse(null);
                                    var itemsEnd = readLongOpt(lastItem, "E").orElse(null);
                                    
                                	if (!entityStart.equals(itemsStart)) {
                                		writeLong(firstItem, "S", entityStart);
                                		
                                		var str = dep_signtype_erepr + " had start time synced to parent gloss.";
//                                		var public_str = glossPublicErrorString + " had the start time on its sign type event synced to the main event.";
                                		addToModifications(20075, str, null);
                                	}
                                	
                                	if (!entityEnd.equals(itemsEnd)) {
                                		writeLong(lastItem, "E", entityEnd);
                                		
                                		var str = dep_signtype_erepr + " had end time synced to parent gloss.";
//                                		var public_str = glossPublicErrorString + " had the end time on its sign type event synced to the main event.";
                                		addToModifications(20076, str, null);
                                	}
                            	}
                            }
                            
                            // check compound logic
                            var compStartID = readOpt(eEl, "COMP_START_ENTITY_ID").orElse(null);
                            var compEndID = readOpt(eEl, "COMP_END_ENTITY_ID").orElse(null);
                            
                            String str = "It appears this gloss should be part of a compound, but this connection is not set properly: " + glossPublicErrorString;
                            
                            boolean cond = deletedEntities.get(compStartID) == null || !deletedEntities.get(compStartID).equals(fid);
                            if (expect(cond, 10110, erepr + " references non-existent entity {ID=" + compStartID + "} as its start compound.", null)) {
                            	var compStartEl = entityElements.get(compStartID);
                            	if (compStartEl != null) {
                            		var matchingEndID = readOpt(compStartEl, "COMP_END_ENTITY_ID").orElse(null);
                            		if (isNotBlank(matchingEndID)) {
                            			expect(matchingEndID.equals(eid), 10111, erepr + " references start compound entity {ID=" + compStartID + "} which does not reference it back correctly as the compound end entity.", str);
                            		}
                            	}
                            }
                            
                            cond = deletedEntities.get(compEndID) == null || !deletedEntities.get(compEndID).equals(fid);
                            if (expect(cond, 10112, erepr + " references non-existent entity {ID=" + compEndID + "} as its end compound.", null)) {
                            	var compEndEl = entityElements.get(compEndID);
                            	if (compEndEl != null) {
                            		var matchingStartID = readOpt(compEndEl, "COMP_START_ENTITY_ID").orElse(null);
                            		if (isNotBlank(matchingStartID)) {
                            			expect(matchingStartID.equals(eid), 10113, erepr + " references end compound entity {ID=" + compEndID + "} which does not reference it back correctly as the compound start entity.", str);
                            		}
                            	}
                            }
                            
                            // if handshapes are blank for one hand, copy from the other hand and reset references everywhere
                            boolean isDomHand = false;
                            if (fid.equals(Util.DOM_GLOSS_FIELD_ID)) {
                            	isDomHand = true;
                            }

                            var twoHandedHandshapes = readOpt(propertiesEl, "TWOHANDED_HANDSHAPES").orElse(null);
                            var startEndHandshapes = readOpt(propertiesEl, "START_END_HANDSHAPES").orElse(null);
                            var rightHandStartEndHandshapes = readOpt(propertiesEl, "RIGHT_HAND_START_END_HANDSHAPE").orElse(null);
                            var leftHandStartEndHandshapes = readOpt(propertiesEl, "LEFT_HAND_START_END_HANDSHAPE").orElse(null);
                            var noOfHands = readOpt(propertiesEl, "NO_OF_HANDS").orElse(null);

                            boolean isTwoHanded = false;
                            if ("2".equals(noOfHands)) {
                            	isTwoHanded = true;
                            }

                            if (isTwoHanded) {
                            	var oppositeGlossID = readOpt(eEl, "DEPENDENT_GLOSS_ID").orElse(null);
                            	var oppositeEl = entityElements.get(oppositeGlossID);

                            	if (oppositeEl != null) {                        
                            		var oppositePropertiesEl = readChildOpt(oppositeEl, "PROPERTIES").orElse(null);
                            		var oppositeTwoHandedHandshapes = readOpt(oppositePropertiesEl, "TWOHANDED_HANDSHAPES").orElse(null);
                            		var oppositeStartEndHandshapes = readOpt(oppositePropertiesEl, "START_END_HANDSHAPES").orElse(null);
                            		var oppositeRightHandStartEndHandshapes = readOpt(oppositePropertiesEl, "RIGHT_HAND_START_END_HANDSHAPE").orElse(null);
                            		var oppositeLeftHandStartEndHandshapes = readOpt(oppositePropertiesEl, "LEFT_HAND_START_END_HANDSHAPE").orElse(null);
                            		var oppositeNoOfHands = readOpt(oppositePropertiesEl, "NO_OF_HANDS").orElse(null);

                            		var opposite_erepr = "Track {FID=\"" + entityTrackMap.get(oppositeGlossID) + "\"} " + " entity " + repr(oppositeEl, "ID");

                            		boolean oppositeIsTwoHanded = false;
                            		if ("2".equals(oppositeNoOfHands)) {
                            			oppositeIsTwoHanded = true;
                            		}

                            		if (oppositeIsTwoHanded) {
                            			ArrayList<String> handshapeVIDs = new ArrayList<String>();
                            			var dependentHandshapeID = readOpt(eEl, "DEPENDENT_HAND_SHAPE_ID").orElse(null);
                        				ArrayList<Element> depHandshapeItemElList = itemElements.get(dependentHandshapeID);
                        				
                            			if (isNotBlank(dependentHandshapeID)) {
                            				for (Element itemEl: depHandshapeItemElList) {
                            					var vid = readOpt(itemEl, "VID").orElse(null);
                            					if (isNotBlank(vid)) {
                            						handshapeVIDs.add(vid);
                            					}
                            				}
                            			}

                            			ArrayList<String> oppositeHandshapeVIDs = new ArrayList<String>();
                            			var oppositeDependentHandshapeID = readOpt(oppositeEl, "DEPENDENT_HAND_SHAPE_ID").orElse(null);
                        				ArrayList<Element> oppositeDepHandshapeItemElList = itemElements.get(oppositeDependentHandshapeID);
                        				
                            			if (isNotBlank(oppositeDependentHandshapeID)) {
                            				for (Element itemEl: oppositeDepHandshapeItemElList) {
                            					var vid = readOpt(itemEl, "VID").orElse(null);
                            					if (isNotBlank(vid)) {
                            						oppositeHandshapeVIDs.add(vid);
                            					}
                            				}
                            			}
                            			
                            			// the actual correction below
                            			if (handshapeVIDs.size() != 0 && oppositeHandshapeVIDs.size() == 0) {
                            				// opposite hand is blank, fill in with this hand's handshapes

                                    		Element oppHandEntityEl = doc.createElement("ENTITY");

                                    		String oppHandEntityID = Util.getLongUniqueNumber();
                                    		oppHandEntityEl.setAttribute("ID", oppHandEntityID);
                                    		oppHandEntityEl.setAttribute("REF_ENTITY_ID", oppositeGlossID);
                                    		
                                    		oppHandEntityEl.setAttribute("COMP_END_ENTITY_ID", "");
                                    		oppHandEntityEl.setAttribute("COMP_START_ENTITY_ID", "");
                                    		oppHandEntityEl.setAttribute("LOCKED_GLOSS_PHON_ALIGNMENT", "T");

                                    		Element oppHandEntityItemEl = doc.createElement("ITEM");
                                    		oppHandEntityItemEl.setAttribute("S"	, "" + start);
                                    		oppHandEntityItemEl.setAttribute("E", "" + end);
                                    		oppHandEntityItemEl.setAttribute("VID", Util.getLongUniqueNumber());

                                        	var elItemList = itemElements.get(eid);
                                        	Element onset = null, initialHold = null, main = null, finalHold = null, offset = null;
                                        	Element operativeStartItem = null, operativeEndItem = null;
                                        	for (Element item: elItemList) {
                                                var itemText = getCharacterDataFromElement(item);
                                        		if (itemText.equals(Constants.ONSET))
                                        			onset = item;
                                        		else if (itemText.equals(Constants.INITIAL_HOLD))
                                        			initialHold = item;
                                        		else if (itemText.equals(Constants.FINAL_HOLD))
                                        			finalHold = item;
                                        		else if (itemText.equals(Constants.OFFSET))
                                        			offset = item;
                                        		else
                                        			main = item;
                                        	}
                                            var entityText = getCharacterDataFromElement(main);
                                    		oppHandEntityItemEl.setTextContent(entityText);
                                    		
                                    		oppHandEntityEl.appendChild(oppHandEntityItemEl);

                        					var handItemList = itemElements.get(oppHandEntityID);
                        					if (handItemList == null) {
                        						handItemList = new ArrayList<Element>();
                        						handItemList.add(oppHandEntityItemEl);
                        						itemElements.put(oppHandEntityID, handItemList);
                        					}
                        					else {
                        						handItemList.add(oppHandEntityItemEl);
                        					}
                                    		
                                    		// add hand entity to correct track
                            				String oppositeGlossTrackID = entityTrackMap.get(oppositeGlossID);
                            				String oppositeHandTrackID = oppositeGlossTrackID.equals(Util.DOM_GLOSS_FIELD_ID)
                            						? Util.DOM_HAND_FIELD_ID : Util.NDOM_HAND_FIELD_ID;
                            				Element oppositeHandTrackEl = trackElements.get(oppositeHandTrackID);
                            				oppositeHandTrackEl.appendChild(oppHandEntityEl);
                                    		
                                    		Element oppHandshapeEntityEl = doc.createElement("ENTITY");
                                    		
                                    		String oppHandshapeEntityID = Util.getLongUniqueNumber();
                                    		oppHandshapeEntityEl.setAttribute("ID", oppHandshapeEntityID);
                                    		oppHandshapeEntityEl.setAttribute("REF_ENTITY_ID", oppHandEntityID);
                                    		
                                    		oppHandshapeEntityEl.setAttribute("COMP_END_ENTITY_ID", "");
                                    		oppHandshapeEntityEl.setAttribute("COMP_START_ENTITY_ID", "");
                                    		oppHandshapeEntityEl.setAttribute("LOCKED_GLOSS_PHON_ALIGNMENT", "T");
                                    		
                            				for (Element handshapeItemEl: depHandshapeItemElList) {
                                        		Element copiedHandshapeItemEl = (Element) handshapeItemEl.cloneNode(true);
                                        		oppHandshapeEntityEl.appendChild(copiedHandshapeItemEl);

                            					var handshapeItemList = itemElements.get(oppHandshapeEntityID);
                            					if (handshapeItemList == null) {
                            						handshapeItemList = new ArrayList<Element>();
                            						handshapeItemList.add(copiedHandshapeItemEl);
                            						itemElements.put(oppHandshapeEntityID, handshapeItemList);
                            					}
                            					else {
                            						handshapeItemList.add(copiedHandshapeItemEl);
                            					}
                            				}
                            				
                                    		// add references to hand/handshape to entity itself
                            				oppositeEl.setAttribute("DEPENDENT_HAND_ID", oppHandEntityID);
                            				oppositeEl.setAttribute("DEPENDENT_HAND_SHAPE_ID", oppHandshapeEntityID);
                            				oppositeEl.setAttribute("LOCKED_GLOSS_PHON_ALIGNMENT", "T");
                            				
                            				// add handshape entity to correct track
                            				String oppositeHandshapeTrackID = oppositeGlossTrackID.equals(Util.DOM_GLOSS_FIELD_ID)
                            						? Util.DOM_HANDSHAPE_FIELD_ID : Util.NDOM_HANDSHAPE_FIELD_ID;
                            				Element oppositeHandshapeTrackEl = trackElements.get(oppositeHandshapeTrackID);
                            				oppositeHandshapeTrackEl.appendChild(oppHandshapeEntityEl);

                            				// set to SAME dom/ndom regardless
                            				oppositePropertiesEl.setAttribute("TWOHANDED_HADSHAPES", "SAME HANDSHAPES LR");
                            				propertiesEl.setAttribute("TWOHANDED_HADSHAPES", "SAME HANDSHAPES LR");

                            				// update other properties
                            				oppositeEl.setAttribute("LOCKED_GLOSS_PHON_ALIGNMENT", "T");
                            				oppositePropertiesEl.setAttribute("RIGHT_HAND_START_END_HANDSHAPE", "");
                            				oppositePropertiesEl.setAttribute("LEFT_HAND_START_END_HANDSHAPE", "");
                            				
                            				// need to also set new values into ENTITY-IDS of statement fields
                            				String utteranceID = entityUtteranceMap.get(oppositeGlossID);
                            				entityUtteranceMap.put(oppHandEntityID, utteranceID);
                            				entityUtteranceMap.put(oppHandshapeEntityID, utteranceID);
                            				
                            				entityElements.put(oppHandEntityID, oppHandEntityEl);
                            				entityElements.put(oppHandshapeEntityID, oppHandshapeEntityEl);
                            				entityTrackMap.put(oppHandEntityID, oppositeHandTrackID);
                            				entityTrackMap.put(oppHandshapeEntityID, oppositeHandshapeTrackID);
                            				
                            				Element utteranceEl = utteranceElements.get(utteranceID);
                            				Element handStmtFieldEl = null, handshapeStmtFieldEl = null;
                            				
                            				 for (Element stmtFieldEl: readChildren(utteranceEl, "STATEMENT-FIELD").collect(Collectors.toList())) {
                                                 var fieldID = readOpt(stmtFieldEl, "FIELD-ID").orElse(null);
                                                 if (fieldID.equals(oppositeHandTrackID)) {
                                                	 handStmtFieldEl = stmtFieldEl;
                                                 }
                                                 
                                                 if (fieldID.equals(oppositeHandshapeTrackID)) {
                                                	 handshapeStmtFieldEl = stmtFieldEl;
                                                 }
                            				 }

                                             var handStmtFieldEIDs = readOpt(handStmtFieldEl, "ENTITY-IDS").orElse(null);
                                             if (isNotBlank(handStmtFieldEIDs)) {
                                            	 handStmtFieldEIDs += ", " + oppHandEntityID;
                                             } else {
                                            	 handStmtFieldEIDs = oppHandEntityID;
                                             }
                                             handStmtFieldEl.setAttribute("ENTITY-IDS", handStmtFieldEIDs);
                                             
                                             var handshapeStmtFieldEIDs = readOpt(handshapeStmtFieldEl, "ENTITY-IDS").orElse(null);
                                             if (isNotBlank(handshapeStmtFieldEIDs)) {
                                            	 handshapeStmtFieldEIDs += ", " + oppHandshapeEntityID;
                                             } else {
                                            	 handshapeStmtFieldEIDs = oppHandshapeEntityID;
                                             }
                                             handshapeStmtFieldEl.setAttribute("ENTITY-IDS", handshapeStmtFieldEIDs);

                                             addToErrors(10136, opposite_erepr + " was marked as a two-handed gloss with different handshapes on the right and left hands but it has no handshapes assigned.", 
                                            		 "One hand of a two-handed gloss has no handshapes, so the handshapes from the existing hand will be copied over, and the gloss will be declared as having the same handshapes on the left and right hands.", false);
                                             addToModifications(20098, opposite_erepr + " had the handshapes copied over from the opposing hand, and it has ben declared as having the same handshapes on both hands.", null);
                            			} else if (handshapeVIDs.size() == 0 && oppositeHandshapeVIDs.size() != 0) {
                            				// this hand is blank, fill in with the opposite hand's handshapes
                            				// opposite hand is blank, fill in with this hand's handshapes

                                    		Element handEntityEl = doc.createElement("ENTITY");

                                    		String handEntityID = Util.getLongUniqueNumber();
                                    		handEntityEl.setAttribute("ID", handEntityID);
                                    		handEntityEl.setAttribute("REF_ENTITY_ID", eid);
                                    		
                                    		handEntityEl.setAttribute("COMP_END_ENTITY_ID", "");
                                    		handEntityEl.setAttribute("COMP_START_ENTITY_ID", "");
                                    		handEntityEl.setAttribute("LOCKED_GLOSS_PHON_ALIGNMENT", "T");

                                    		Element handEntityItemEl = doc.createElement("ITEM");
                                    		handEntityItemEl.setAttribute("S"	, "" + start);
                                    		handEntityItemEl.setAttribute("E", "" + end);
                                    		handEntityItemEl.setAttribute("VID", Util.getLongUniqueNumber());

                                        	var oppGlossElItemList = itemElements.get(oppositeGlossID);
                                        	Element onset = null, initialHold = null, main = null, finalHold = null, offset = null;
                                        	Element operativeStartItem = null, operativeEndItem = null;
                                        	for (Element item: oppGlossElItemList) {
                                                var itemText = getCharacterDataFromElement(item);
                                        		if (itemText.equals(Constants.ONSET))
                                        			onset = item;
                                        		else if (itemText.equals(Constants.INITIAL_HOLD))
                                        			initialHold = item;
                                        		else if (itemText.equals(Constants.FINAL_HOLD))
                                        			finalHold = item;
                                        		else if (itemText.equals(Constants.OFFSET))
                                        			offset = item;
                                        		else
                                        			main = item;
                                        	}
                                            var oppGlossEntityText = getCharacterDataFromElement(main);
                                    		handEntityItemEl.setTextContent(oppGlossEntityText);
                                    		
                                    		handEntityEl.appendChild(handEntityItemEl);
                                    		
                                    		// add hand entity to correct track
                            				String glossTrackID = entityTrackMap.get(eid);
                            				String handTrackID = glossTrackID.equals(Util.DOM_GLOSS_FIELD_ID)
                            						? Util.DOM_HAND_FIELD_ID : Util.NDOM_HAND_FIELD_ID;
                            				Element handTrackEl = trackElements.get(handTrackID);
                            				handTrackEl.appendChild(handEntityEl);

                        					var handItemList = itemElements.get(handEntityID);
                        					if (handItemList == null) {
                        						handItemList = new ArrayList<Element>();
                        						handItemList.add(handEntityItemEl);
                        						itemElements.put(handEntityID, handItemList);
                        					}
                        					else {
                        						handItemList.add(handEntityItemEl);
                        					}
                                    		
                                    		Element handshapeEntityEl = doc.createElement("ENTITY");
                                    		
                                    		String handshapeEntityID = Util.getLongUniqueNumber();
                                    		handshapeEntityEl.setAttribute("ID", handshapeEntityID);
                                    		handshapeEntityEl.setAttribute("REF_ENTITY_ID", handEntityID);
                                    		
                                    		handshapeEntityEl.setAttribute("COMP_END_ENTITY_ID", "");
                                    		handshapeEntityEl.setAttribute("COMP_START_ENTITY_ID", "");
                                    		handshapeEntityEl.setAttribute("LOCKED_GLOSS_PHON_ALIGNMENT", "T");
                                    		
                            				for (Element handshapeItemEl: oppositeDepHandshapeItemElList) {
                                        		Element copiedHandshapeItemEl = (Element) handshapeItemEl.cloneNode(true);
                                        		handshapeEntityEl.appendChild(copiedHandshapeItemEl);

                            					var handshapeItemList = itemElements.get(handshapeEntityID);
                            					if (handshapeItemList == null) {
                            						handshapeItemList = new ArrayList<Element>();
                            						handshapeItemList.add(copiedHandshapeItemEl);
                            						itemElements.put(handshapeEntityID, handshapeItemList);
                            					}
                            					else {
                            						handshapeItemList.add(copiedHandshapeItemEl);
                            					}
                            				}
                            				
                                    		// add references to hand/handshape to entity itself
                            				eEl.setAttribute("DEPENDENT_HAND_ID", handEntityID);
                            				eEl.setAttribute("DEPENDENT_HAND_SHAPE_ID", handshapeEntityID);
                            				eEl.setAttribute("LOCKED_GLOSS_PHON_ALIGNMENT", "T");
                            				
                            				// add handshape entity to correct track
                            				String handshapeTrackID = glossTrackID.equals(Util.DOM_GLOSS_FIELD_ID)
                            						? Util.DOM_HANDSHAPE_FIELD_ID : Util.NDOM_HANDSHAPE_FIELD_ID;
                            				Element handshapeTrackEl = trackElements.get(handshapeTrackID);
                            				handshapeTrackEl.appendChild(handshapeEntityEl);

                            				// set to SAME dom/ndom regardless
                            				propertiesEl.setAttribute("TWOHANDED_HADSHAPES", "SAME HANDSHAPES LR");
                            				propertiesEl.setAttribute("TWOHANDED_HADSHAPES", "SAME HANDSHAPES LR");

                            				// update other properties
                            				oppositeEl.setAttribute("LOCKED_GLOSS_PHON_ALIGNMENT", "T");
                            				propertiesEl.setAttribute("RIGHT_HAND_START_END_HANDSHAPE", "");
                            				propertiesEl.setAttribute("LEFT_HAND_START_END_HANDSHAPE", "");
                            				
                            				// need to also set new values into ENTITY-IDS of statement fields
                            				String utteranceID = entityUtteranceMap.get(eid);
                            				entityUtteranceMap.put(handEntityID, utteranceID);
                            				entityUtteranceMap.put(handshapeEntityID, utteranceID);
                            				
                            				entityElements.put(handEntityID, handEntityEl);
                            				entityElements.put(handshapeEntityID, handshapeEntityEl);
                            				entityTrackMap.put(handEntityID, handTrackID);
                            				entityTrackMap.put(handshapeEntityID, handshapeTrackID);
                            				
                            				Element utteranceEl = utteranceElements.get(utteranceID);
                            				Element handStmtFieldEl = null, handshapeStmtFieldEl = null;
                            				
                            				 for (Element stmtFieldEl: readChildren(utteranceEl, "STATEMENT-FIELD").collect(Collectors.toList())) {
                                                 var fieldID = readOpt(stmtFieldEl, "FIELD-ID").orElse(null);
                                                 if (fieldID.equals(handTrackID)) {
                                                	 handStmtFieldEl = stmtFieldEl;
                                                 }
                                                 
                                                 if (fieldID.equals(handshapeTrackID)) {
                                                	 handshapeStmtFieldEl = stmtFieldEl;
                                                 }
                            				 }

                                             var handStmtFieldEIDs = readOpt(handStmtFieldEl, "ENTITY-IDS").orElse(null);
                                             if (isNotBlank(handStmtFieldEIDs)) {
                                            	 handStmtFieldEIDs += ", " + handEntityID;
                                             } else {
                                            	 handStmtFieldEIDs = handEntityID;
                                             }
                                             handStmtFieldEl.setAttribute("ENTITY-IDS", handStmtFieldEIDs);
                                             
                                             var handshapeStmtFieldEIDs = readOpt(handshapeStmtFieldEl, "ENTITY-IDS").orElse(null);
                                             if (isNotBlank(handshapeStmtFieldEIDs)) {
                                            	 handshapeStmtFieldEIDs += ", " + handshapeEntityID;
                                             } else {
                                            	 handshapeStmtFieldEIDs = handshapeEntityID;
                                             }
                                             handshapeStmtFieldEl.setAttribute("ENTITY-IDS", handshapeStmtFieldEIDs);

                                             addToErrors(10136, erepr + " was marked as a two-handed gloss with different handshapes on the right and left hands but it has no handshapes assigned.", 
                                            		 "One hand of a two-handed gloss has no handshapes, so the handshapes from the existing hand will be copied over, and the gloss will be declared as having the same handshapes on the left and right hands.", false);
                                             addToModifications(20098, erepr + " had the handshapes copied over from the opposing hand, and it has ben declared as having the same handshapes on both hands.", null);
                            			}
                            		}
                            	}
                            }
                    	}
                    }
                });
            });
        });
        
        return null;
    }
    
    private void correctMultiItemEntityTimes() {
        // Correct start/end times to be within gloss and equally spaced
        for (Entry<String, Element> entry: entityElements.entrySet()) {
        	var glossEID = entry.getKey();
        	var glossEl = entry.getValue();
        	
        	var trackID = entityTrackMap.get(glossEID);
        	if ( ! trackID.equals(Util.DOM_GLOSS_FIELD_ID) && ! trackID.equals(Util.NDOM_GLOSS_FIELD_ID))
        		continue;
        	
        	if (deletedEntities.get(glossEID) != null && deletedEntities.get(glossEID).equals(trackID))
        		continue;

        	// gloss information to setup
        	var glossItemsList = itemElements.get(glossEID);
        	if (glossItemsList == null || glossItemsList.size() == 0)
        		continue;
        	
        	boolean displayMainValueBlankError = false;
        	
        	Element onset = null, initialHold = null, main = null, finalHold = null, offset = null;
        	Element operativeStartItem = null, operativeEndItem = null;
        	for (Element item: glossItemsList) {
                var itemText = getCharacterDataFromElement(item);
        		if (Constants.ONSET.equals(itemText))
        			onset = item;
        		else if (Constants.INITIAL_HOLD.equals(itemText))
        			initialHold = item;
        		else if (Constants.FINAL_HOLD.equals(itemText))
        			finalHold = item;
        		else if (Constants.OFFSET.equals(itemText))
        			offset = item;
        		else {
        			main = item;
        			
        			// extra check for if the main item has text at all
        			if (isBlank(itemText)) {
        				displayMainValueBlankError = true;
        			}
        		}
        	}
        	
        	if (initialHold != null)
        		operativeStartItem = initialHold;
        	else
        		operativeStartItem = main;
        	
        	if (finalHold != null)
        		operativeEndItem = finalHold;
        	else
        		operativeEndItem = main;

        	var start = readLongOpt(operativeStartItem, "S").orElse(0L);
        	var end = readLongOpt(operativeEndItem, "E").orElse(0L);

        	var field = defScheme3_3_5.getField(trackID);
        	var fieldName = field.getName();
        	
        	String entityPublicErrorString = "Gloss entity {Field: " + fieldName + ", Start: " + start + ", End: " + end + "}";

        	if (displayMainValueBlankError) {
				addToErrors(10114, "Gloss entity {ID=" + glossEID + "} has no text value for its main event.", "There is a manual sign that is missing a text string gloss label: " + entityPublicErrorString);
        	}
        	
        	var dependentHandID = readOpt(glossEl, "DEPENDENT_HAND_ID").orElse(null);
        	var dependentHandshapeID = readOpt(glossEl, "DEPENDENT_HAND_SHAPE_ID").orElse(null);
        	var handEl = entityElements.get(dependentHandID);
        	var hsEl = entityElements.get(dependentHandshapeID);
        	
        	// extra check for the main (and only) item on the hand entity - if it has a text value
        	// also set the mainHandItem if needed below
        	Element mainHandItem = null;
        	if (handEl != null) {
        		var handEID = readOpt(handEl, "ID").orElse(null);
            	var handItemsList = itemElements.get(handEID);
            	if (handItemsList != null && handItemsList.size() > 0) {
            		mainHandItem = handItemsList.get(0);		// only one item for hands
            		var itemText = getCharacterDataFromElement(mainHandItem);
            		if (isBlank(itemText)) {
        				addToErrors(10115, "Hand entity {ID=" + handEID + "} has no text value for its main event.", "There is a manual sign that is missing a text string gloss label: " + entityPublicErrorString);
            		}
            	}
        	}
        	
        	if (frameTimeUnit == null)
        		frameTimeUnit = new TimeUnit("temp", "", TimeUnit.Type.FRAME, (double) this.timebase / this.timestep);
        	
        	// handshapes
        	var lockedGlossPhonStr = readOpt(glossEl, "LOCKED_GLOSS_PHON_ALIGNMENT").orElse(null);
        	if (isBlank(lockedGlossPhonStr) || lockedGlossPhonStr.equals("F")) {		// if unlocked from gloss, hs lock to hand
        		if (mainHandItem == null)
        			continue;
        		else {
        			start = readLongOpt(mainHandItem, "S").orElse(0L);
        			end = readLongOpt(mainHandItem, "E").orElse(0L);
        		}
        	}
        	
        	var intervalDiff = end - start;
        	var framesForHS = (intervalDiff / timestep) + 1;

        	
        	var hsItemsList = itemElements.get(dependentHandshapeID);
        	
        	if (hsItemsList != null && hsItemsList.size() > 0) {
                int hsItemsSize = hsItemsList.size();
                
                if (framesForHS == 1 && hsItemsSize == 2) {
                	var hsItemOne = hsItemsList.get(0);
                	var hsItemTwo = hsItemsList.get(1);
                	var hsItemOneID = readOpt(hsItemOne, "VID").orElse(null);
                	var hsItemTwoID = readOpt(hsItemTwo, "VID").orElse(null);
                	if (isNotBlank(hsItemOneID) 
                			&& isNotBlank(hsItemTwoID))  {
                		if (hsItemOneID.equals(hsItemTwoID)) {
	                		hsEl.removeChild(hsItemTwo);
	                		hsItemsList.remove(hsItemTwo);
	                        addToModifications(20077, "Removed duplicate handshape on 1-frame entity {ID=" + dependentHandshapeID + "}.", null);
                		} else {
                			addToErrors(10116, "Handshape entity {ID=" + dependentHandshapeID + "} is a 1-frame entity but has different start/end handshapes.", "There is a 1-frame gloss with more than 1 handshape, which is not allowed: " + entityPublicErrorString);
                		}
                	}
                } else {
                    var propertiesEl = readChildOpt(glossEl, "PROPERTIES").orElse(null);
                    if (propertiesEl == null) {
                    	addToErrors(10073, "Gloss {ID=" + glossEID + "} missing PROPERTIES element.", null);
                    	continue;
                    }
                    var signType = readOpt(propertiesEl, "SIGN").orElse(null);
                    boolean fingerspelled = false, loan = false;
                    if (isNotBlank(signType) && signType.equals("FINGERSPELLED")) fingerspelled = true;
                    if (isNotBlank(signType) && signType.equals("LOAN")) loan = true;
                    
                	if (!(fingerspelled || loan)) {
                		expect(hsItemsSize <= framesForHS,
                				10117, "Handshapes {ID=" + dependentHandshapeID + "} has more items (" + hsItemsSize + ") than frames (" + framesForHS + ").", "There is a gloss with more handshapes than expected for its duration: " + entityPublicErrorString);
                	}
                }

                if (hsItemsSize == 1) {
                	var hsItem = hsItemsList.get(0);

                    var hsItemStart = readLongOpt(hsItem, "S").orElse(0L);
                    var hsItemEnd = readLongOpt(hsItem, "E").orElse(0L);
                    var hsIntervalDiff = hsItemEnd - hsItemStart;
                    if (hsIntervalDiff != 0 && hsIntervalDiff % timestep != 0) {
                    	addToErrors(10118, "Handshape {ID=" + dependentHandshapeID + "} has incorrectly rounded (not divisible by timestep) time.", null);		// should not throw
                    	continue;
                    }
                    
                    if ( !(hsItemStart.equals(start)) && !(hsItemEnd.equals(end)) ) {
                    	writeLong(hsItem, "S", start);
                    	writeLong(hsItem, "E", end);
                    	
                    	var str = "Handshape {ID=" + dependentHandshapeID + "} had time reset to parent interval.";
                    	var public_entity = getEntityPublicErrorString(glossEID, trackID);
//                    	var public_str = public_entity + " had its handshape time reset to the gloss interval.";
                    	addToModifications(20078, str, null);
                    }
                } else {
                    	// check if need to reset all item times
                    	var startTimeInfo = TimeRange.ABSOLUTE.time(start, TimeUnit.TIMESTAMP, Rounding.FLOOR).convert(TimeUnit.NANOSECOND);
                    	var endTimeInfo = TimeRange.ABSOLUTE.time(end, TimeUnit.TIMESTAMP, Rounding.FLOOR).convert(TimeUnit.NANOSECOND);
                        
                    	Duration dur = endTimeInfo.distance(startTimeInfo).divide(hsItemsList.size() - 1);
                    	Time nextDurTime = startTimeInfo;
                    	boolean anyModificationWasMade = false;
                    	for (int i = 0; i < hsItemsList.size(); i++) {
                    		Element item = hsItemsList.get(i);

                    		var nextDurTimestamp = nextDurTime.convert(frameTimeUnit, Rounding.NEAREST);
                    		var time = nextDurTimestamp.value() * timestep;

                    		var existingStart = (long) readLongOpt(item, "S").orElse(0L);
                    		var existingEnd = (long) readLongOpt(item, "E").orElse(0L);

                    		if (time != existingStart || time != existingEnd) {
                    			writeLong(item, "S", time);
                    			writeLong(item, "E", time);
                    			anyModificationWasMade = true;
                    		}

                    		nextDurTime = nextDurTime.add(dur);
                    	}

                    	if (anyModificationWasMade) {
                        	var str = "Handshapes {ID=" + dependentHandshapeID + "} had time reset to parent interval.";
                        	var public_entity = getEntityPublicErrorString(glossEID, trackID);
//                        	var public_str = public_entity + " had its handshape time reset to the gloss interval.";
                    		addToModifications(20078, str, null);
                    	}

                    	// then check
                    	nextDurTime = startTimeInfo;
                    	boolean timeErrorDetected = false;
                    	for (int i = 0; i < hsItemsList.size(); i++) {
                    		Element item = hsItemsList.get(i);

                    		var nextDurTimestamp = nextDurTime.convert(frameTimeUnit, Rounding.NEAREST);
                    		var time = nextDurTimestamp.value() * timestep;

                    		var existingStart = (long) readLongOpt(item, "S").orElse(0L);
                    		var existingEnd = (long) readLongOpt(item, "E").orElse(0L);

                    		if (time != existingStart || time != existingEnd) {
                    			timeErrorDetected = true;
                    		}

                    		nextDurTime = nextDurTime.add(dur);
                    	}                	
                    	
                    	if (timeErrorDetected)
                    		addToErrors(10119, "Handshapes {ID=" + dependentHandshapeID + "} have invalid item times.", null);
                }
        	}
        	

        	// locations -- back to purely gloss parent items
        	
        	if (onset != null)
        		operativeStartItem = onset;
        	else 	if (initialHold != null)
        		operativeStartItem = initialHold;
        	else
        		operativeStartItem = main;

        	if (offset != null)
        		operativeStartItem = offset;
        	else if (finalHold != null)
        		operativeEndItem = finalHold;
        	else
        		operativeEndItem = main;

        	start = readLongOpt(operativeStartItem, "S").orElse(0L);
        	end = readLongOpt(operativeEndItem, "E").orElse(0L);
        	
        	
        	var dependentLocationID = readOpt(glossEl, "DEPENDENT_LOCATION_ID").orElse(null);
        	var locItemsList = itemElements.get(dependentLocationID);
        	
        	if (locItemsList != null && locItemsList.size() > 0) {
                int locItemsSize = locItemsList.size();
                
                if (locItemsSize == 1) {
                	var locItem = locItemsList.get(0);

                    var locItemStart = readLongOpt(locItem, "S").orElse(0L);
                    var locItemEnd = readLongOpt(locItem, "E").orElse(0L);
                    var locIntervalDiff = locItemEnd - locItemStart;
                    if (locIntervalDiff != 0 && locIntervalDiff % timestep != 0) {
                    	addToErrors(10120, "Location {ID=" + dependentLocationID + "} has incorrectly rounded (not divisible by timestep) time.", null);		// should not throw
                    	continue;
                    }
                    
                    if ( !(locItemStart.equals(start)) && !(locItemEnd.equals(end)) ) {
                    	writeLong(locItem, "S", start);
                    	writeLong(locItem, "E", end);
                    	
                    	var str = "Location {ID=" + dependentLocationID + "} had time reset to parent gloss interval.";
                    	var public_entity = getEntityPublicErrorString(glossEID, trackID);
//                    	var public_str = public_entity + " had its location time reset to the gloss interval.";
                        addToModifications(20079, str, null);
                    }
                } else {
                	// check if need to reset all item times
                	var glossStartTimeInfo = TimeRange.ABSOLUTE.time(start, TimeUnit.TIMESTAMP, Rounding.FLOOR).convert(TimeUnit.NANOSECOND);
                	var glossEndTimeInfo = TimeRange.ABSOLUTE.time(end, TimeUnit.TIMESTAMP, Rounding.FLOOR).convert(TimeUnit.NANOSECOND);

                	Duration dur = glossEndTimeInfo.distance(glossStartTimeInfo).divide(locItemsList.size() - 1);
                	Time nextDurTime = glossStartTimeInfo;
                	boolean anyModificationWasMade = false;
                	for (int i = 0; i < locItemsList.size(); i++) {
                		Element item = locItemsList.get(i);

                		var nextDurTimestamp = nextDurTime.convert(frameTimeUnit, Rounding.NEAREST);
                		var time = nextDurTimestamp.value() * timestep;

                		var existingStart = (long) readLongOpt(item, "S").orElse(0L);
                		var existingEnd = (long) readLongOpt(item, "E").orElse(0L);

                		if (time != existingStart || time != existingEnd) {
                			writeLong(item, "S", time);
                			writeLong(item, "E", time);
                			anyModificationWasMade = true;
                		}

                		nextDurTime = nextDurTime.add(dur);
                	}

                	if (anyModificationWasMade) {
                    	var str = "Locations {ID=" + dependentLocationID + "} had time reset to parent gloss interval.";
                    	var public_entity = getEntityPublicErrorString(glossEID, trackID);
//                    	var public_str = public_entity + " had its location time reset to the gloss interval.";
                		addToModifications(20079, str, null);
                	}

                	// then check
                	nextDurTime = glossStartTimeInfo;
                	boolean timeErrorDetected = false;
                	for (int i = 0; i < locItemsList.size(); i++) {
                		Element item = locItemsList.get(i);

                		var nextDurTimestamp = nextDurTime.convert(frameTimeUnit, Rounding.NEAREST);
                		var time = nextDurTimestamp.value() * timestep;

                		var existingStart = (long) readLongOpt(item, "S").orElse(0L);
                		var existingEnd = (long) readLongOpt(item, "E").orElse(0L);

                		if (time != existingStart || time != existingEnd) {
                			timeErrorDetected = true;
                		}

                		nextDurTime = nextDurTime.add(dur);
                	}                	
                	
                	if (timeErrorDetected)
                		addToErrors(10121, "Locations {ID=" + dependentLocationID + "} have invalid item times.", null);
                }
        	}
        }
    }
    
    private void validateReversedItemTimes() {
    	var tracksEl = readChildOpt(dbEl, "TRACKS").orElse(null);

    	readChildren(tracksEl, "TRACK").forEach(tEl -> {
    		var repr = "Track " + repr(tEl, "FID");
    		var fid = readOpt(tEl, "FID").orElse(null);

    		readChildren(tEl, "ENTITY").forEach(eEl -> {
    			var erepr = repr + " entity " + repr(eEl, "ID");
    			var eid = readOpt(eEl, "ID").orElse(null);

    			if (deletedEntities.get(eid) != null && deletedEntities.get(eid).equals(fid))
    				return;
    			
    			String entityPublicErrorString = getEntityPublicErrorString(eid, fid);

    			var items = readChildren(eEl, "ITEM").collect(Collectors.toList());
    			for (int i = 0; i < items.size(); i++) {
    				var iEl = items.get(i);
    				var irepr = erepr + " item";
    				var start = readLongOpt(iEl, "S").orElse(null);
    				var end = readLongOpt(iEl, "E").orElse(null);

    				if (start != null && end != null)
    					expect(start <= end, 10122, irepr + " has reversed start/end times.", entityPublicErrorString + " has reversed start/end times.", false);
    			}
    		});


            readChildren(tEl, "NON_COMPLETE_EVENTS").forEach(eEl -> {
                var erepr = repr + " non complete event " + repr(eEl, "ID");
    			var eid = readOpt(eEl, "ID").orElse(null);
    			
    			String entityPublicErrorString = getEntityPublicErrorString(eid, fid);

                var items = readChildren(eEl, "ITEM").collect(Collectors.toList());
                for (int i = 0; i < items.size(); i++) {
                    var iEl = items.get(i);
                    var irepr = erepr + " item";
                    var start = readLongOpt(iEl, "S").orElse(null);
                    var end = readLongOpt(iEl, "E").orElse(null);

                    if (start != null && end != null) {
                        expect(start <= end, 10122, irepr + " has reversed start/end times.", entityPublicErrorString + " has reversed start/end times.", false);
                    }
                }
            });

            readChildren(tEl, "ENTERED_TEXT").forEach(eEl -> {
                var erepr = repr + " entered text " + repr(eEl, "ID");
    			var eid = readOpt(eEl, "ID").orElse(null);
    			
    			String entityPublicErrorString = getEntityPublicErrorString(eid, fid);
                
                var items = readChildren(eEl, "ITEM").collect(Collectors.toList());
                for (int i = 0; i < items.size(); i++) {
                    var iEl = items.get(i);
                    var irepr = erepr + " item";
                    var start = readLongOpt(iEl, "S").orElse(null);
                    var end = readLongOpt(iEl, "E").orElse(null);
                    
                    if (start != null && end != null) {
                        expect(start <= end, 10122, irepr + " has reversed start/end times.", entityPublicErrorString + " has reversed start/end times.", false);
                    }
                }
            });
    	});
    }
    
    private void correctEntityIDs() {
        var partitionsEl = readChildOpt(glossWindowEl, "TEMPORAL-PARTITIONS").orElse(null);
        if (expect(partitionsEl != null, 10041, "Missing TEMPORAL-PARTITIONS element.", null)) {
            readChildren(partitionsEl, "TEMPORAL-PARTITION").forEach(partitionEl -> {
                var repr = "Temporal Partition " + repr(partitionEl, "LABEL");
                var pid = readOpt(partitionEl, "ID").orElse(null);
                
                readChildren(partitionEl, "TEMPORAL-PARTITION-SEGMENT").forEach(tierEl -> {
                    var trepr = repr + " Segment Tier " + repr(tierEl, "LABEL");
                    var tid = readOpt(tierEl, "ID").orElse(null);
                    
                    // statement fields in newer collections - directly under tier
                    readChildren(tierEl, "STATEMENT-FIELD").forEach(stmtFieldEl -> {
                        var srepr = trepr + " statement field " + repr(stmtFieldEl, "FIELD-ID");
                        var fid = readOpt(stmtFieldEl, "FIELD-ID").orElse(null);
                        
                        var eids = readOpt(stmtFieldEl, "ENTITY-IDS").orElse(null);
                        if (isNotBlank(eids)) {
                            if (!eids.equals("")) {
                            	ArrayList<String> existingList = new ArrayList<String>(); 
                            	ArrayList<String> deletedList = new ArrayList<String>();
                            	ArrayList<String> newList = new ArrayList<String>();
                            	String[] eidList = eids.split(", ");
                            	for (var eid: eidList) {
                            		if (deletedEntities.get(eid) == null || !deletedEntities.get(eid).equals(fid)){
                                    	existingList.add(eid);
                                    } else {
                                    	deletedList.add(eid);
                                    }
                            	}
                            	
                            	if (deletedList.size() > 0) {
                                	write(stmtFieldEl, "ENTITY-IDS", String.join(", ", existingList));
                                	addToModifications(20080, "Removed " + deletedList + " from ENTITY-IDS in "+ srepr + " due to previous modifications.", null);
                            	}
                            }
                        }
                    });

                    readChildren(tierEl, "GLOSS-WINDOW-ITEM").forEach(utteranceEl -> {
                        var urepr = "Utterance " + repr(utteranceEl, "LABEL", "ID");
                        var uid = readOpt(utteranceEl, "ID").orElse(null);
                        
                        if (utteranceToEntitiesMap.get(uid) == null) {
                        	utteranceToEntitiesMap.put(uid, new ArrayList<String>());
                        }
                        ArrayList<String> entities = utteranceToEntitiesMap.get(uid);
                        
                        // statement fields in older collections - directly under utterance
                        readChildren(utteranceEl, "STATEMENT-FIELD").forEach(stmtFieldEl -> {
                            var srepr = urepr + " statement field " + repr(stmtFieldEl, "FIELD-ID");
                            var fid = readOpt(stmtFieldEl, "FIELD-ID").orElse(null);
                            
                            var eids = readOpt(stmtFieldEl, "ENTITY-IDS").orElse(null);
                            if (isNotBlank(eids)) {
                                if (!eids.equals("")) {
                                	ArrayList<String> entitiesList = new ArrayList<String>(); 
                                	ArrayList<String> deletedList = new ArrayList<String>();
                                	ArrayList<String> newList = new ArrayList<String>();
                                	String[] eidList = eids.split(", ");
                                	for (var eid: eidList) {
                                		if (deletedEntities.get(eid) == null || !deletedEntities.get(eid).equals(fid)) {
                                        	entitiesList.add(eid);
                                        } else {
                                        	deletedList.add(eid);
                                        }
                                	}
                                	
                                	for (Entry entry: newEntities.entrySet()) {
                                		String newEID = (String) entry.getKey();
                                		String newFID = (String) entry.getValue();
                                		
                                		Element entityEl = entityElements.get(newEID);
                                		String parentEntityID = readOpt(entityEl, "PARENT_ENTITY_ID").orElse(null);
                                		if (isNotBlank(parentEntityID) && parentEntityID.equals(uid)) {
                                    		if (newFID.equals(fid)) {
                                    			newList.add(newEID);
                                    		}
                                		}
                                	}
                                	
                                	entitiesList.addAll(newList);
                                	
                                	entities.addAll(entitiesList);
                                	
                                	if (deletedList.size() > 0 || newList.size() > 0) {
                                    	write(stmtFieldEl, "ENTITY-IDS", String.join(", ", entitiesList));
                                    	
                                    	if (deletedList.size() > 0)
                                    		addToModifications(20080, "Removed " + deletedList + " from ENTITY-IDS in "+ srepr + " due to previous modifications.", null);
                                    	
                                    	if (newList.size() > 0)
                                    		addToModifications(20081, "Added " + newList + " to ENTITY-IDS in "+ srepr + " due to previous modifications.", null);
                                	}
                                }
                            }
                        });
                    });
                });
            });
        }
    }
    
    private void correctUtteranceOverlaps() {
    	for (Entry entry: utteranceIntervals.entrySet()) {
	    		String segmentTierID = (String) entry.getKey();
	    		IntervalSet tierUtterancesIntervalSet = (IntervalSet) entry.getValue();
    		
		    	ArrayList<String> uids = tierUtterancesIntervalSet.getKeys();
		    	ArrayList<Interval> intervals = tierUtterancesIntervalSet.getIntervals();
		    	
		    	if (uids.size() != intervals.size()) {
		    		// something wrong happened in building the utterance intervals - needs to be exactly equal
		    		return;
		    	}
		    	
		    	for (int index2 = 1; index2 < uids.size(); index2++) {
		    		int index1 = index2 - 1;
		    		
		    		Interval u1Interval = intervals.get(index1).convert(TimeUnit.TIMESTAMP, Rounding.NEAREST);
		    		Interval u2Interval = intervals.get(index2).convert(TimeUnit.TIMESTAMP, Rounding.NEAREST);
		    		
		    		String uid1 = uids.get(index1);
		    		String uid2 = uids.get(index2);
		    		
		    		// if the intervals do not overlap, do not do any of the below and proceed to the next pair
		    		if ( ! u1Interval.overlaps(u2Interval, Bound.EXCLUSIVE)) {
		    			continue;
		    		}
		    		
		    		Element u1 = utteranceElements.get(uid1);
		    		Element u2 = utteranceElements.get(uid2);
		    		
		    		Interval u1OrigInterval = new Interval(u1Interval.t0, u1Interval.t1);
		    		Interval u2OrigInterval = new Interval(u2Interval.t0, u2Interval.t1);
		    		
					ArrayList<String> u1entityIDs = (ArrayList<String>) utteranceToEntitiesMap.get(uid1).clone();
					ArrayList<String> u2entityIDs = (ArrayList<String>) utteranceToEntitiesMap.get(uid2).clone();
					
					String tid1 = utteranceTierMap.get(uid1);
					String tid2 = utteranceTierMap.get(uid2);
					
					Element tier1 = tierElements.get(tid1);
					Element tier2 = tierElements.get(tid2);
					
					String pid1 = tierPartitionMap.get(tid1);
					String pid2 = tierPartitionMap.get(tid2);
					
					Element part1 = partitionElements.get(pid1);
					Element part2 = partitionElements.get(pid2);
		
		            var u1repr = "Temporal Partition " + repr(part1, "LABEL") + " Segment Tier " + repr(tier1, "LABEL") + " Utterance " + repr(u1, "LABEL");
		            var u2repr = "Temporal Partition " + repr(part2, "LABEL") + " Segment Tier " + repr(tier2, "LABEL") + " Utterance " + repr(u2, "LABEL");
					
					ArrayList<String> entityIDsWithinBothUtterances = new ArrayList<String>();
					for (String u1ID: u1entityIDs)
						for (String u2ID: u2entityIDs)
							if (u1ID.equals(u2ID))
								entityIDsWithinBothUtterances.add(u1ID);
					
					u1entityIDs.removeAll(entityIDsWithinBothUtterances);
					u2entityIDs.removeAll(entityIDsWithinBothUtterances);
					
					String translationElementID1 = null, translationElementID2 = null;
		    		
		    		boolean u1loop = true;
		    		boolean u2loop = true;
		    		boolean flip = true;
		    		while (u1loop || u2loop) {
		    			if (u1Interval.overlaps(u2Interval, Bound.EXCLUSIVE)) {
		    				if (flip && u1loop) {
		    					// try to decrease the end time of u1 one frame at a time
		    					u1Interval = new Interval(u1Interval.t0, u1Interval.t1.subtract(timestep));
		
		    					boolean proceed = true;
		
		    					// then check if any entities within the utterance fell outside the bounds
		    					for (String u1entityID: u1entityIDs) {
		    						String fieldID = entityTrackMap.get(u1entityID);
		    						if (fieldID.equals(Util.FREE_TEXT_FIELD_ID)) {
		    							// ignore translation entities as they always just align to the utterance - if an adjustment is necessary it will be done below during the correction
		    							translationElementID1 = u1entityID;
		    							continue;
		    						}
		
		    						ArrayList<Element> items = itemElements.get(u1entityID);
		    						Element firstItem = items.get(0);
		    						Element lastItem = items.get(items.size() - 1);
		
		    						var start = readLongOpt(firstItem, "S").orElse(0L);
		    						var end = readLongOpt(lastItem, "E").orElse(0L);
		    						var entityInterval = TimeRange.ABSOLUTE.time(start, TimeUnit.TIMESTAMP, Rounding.FLOOR)
		    								.to(TimeRange.ABSOLUTE.time(end, TimeUnit.TIMESTAMP, Rounding.FLOOR)).convert(TimeUnit.TIMESTAMP, Rounding.NEAREST);
		
		    						if ( ! u1Interval.contains(entityInterval, Bound.INCLUSIVE)) {
		    							proceed = false;
		    							break;
		    						}
		    					}
		
		    					if ( ! proceed) {
		    						// revert interval time
		    						u1Interval = new Interval(u1Interval.t0, u1Interval.t1.add(timestep));
		
		    						u1loop = false;
		    					}
		    					
		    				} else if ( !flip && u2loop) {
		    					// try to increase the start time of u2 one frame at a time
		    					u2Interval = new Interval(u2Interval.t0.add(timestep), u2Interval.t1);
		
		    					boolean proceed = true;
		
		    					// then check if any entities within the utterance fell outside the bounds
		    					for (String u2entityID: u2entityIDs) {
		    						String fieldID = entityTrackMap.get(u2entityID);
		    						if (fieldID.equals(Util.FREE_TEXT_FIELD_ID)) {
		    							// ignore translation entities as they always just align to the utterance - if an adjustment is necessary it will be done below during the correction
		    							translationElementID2 = u2entityID;
		    							continue;
		    						}
		
		    						ArrayList<Element> items = itemElements.get(u2entityID);
		    						Element firstItem = items.get(0);
		    						Element lastItem = items.get(items.size() - 1);
		
		    						var start = readLongOpt(firstItem, "S").orElse(0L);
		    						var end = readLongOpt(lastItem, "E").orElse(0L);
		    						var entityInterval = TimeRange.ABSOLUTE.time(start, TimeUnit.TIMESTAMP, Rounding.FLOOR)
		    								.to(TimeRange.ABSOLUTE.time(end, TimeUnit.TIMESTAMP, Rounding.FLOOR)).convert(TimeUnit.TIMESTAMP, Rounding.NEAREST);
		
		    						if ( ! u2Interval.contains(entityInterval, Bound.INCLUSIVE)) {
		    							proceed = false;
		    							break;
		    						}
		    					}
		
		    					if ( ! proceed) {
		    						// revert interval time
		    						u2Interval = new Interval(u2Interval.t0, u2Interval.t1.add(timestep));
		
		    						u2loop = false;
		    					}
		    				}
		    				
		    				flip = !flip;
		    						
		    			} else {
		    				write(u1, "E", "" + u1Interval.t1.value());
		    				if (translationElementID1 != null) {
		    					Element translationItem1 = itemElements.get(translationElementID1).get(0);
		    					write(translationItem1, "E", "" + u1Interval.t1.value());
		    				}
		
		    				write(u2, "S", "" + u2Interval.t0.value());
		    				if (translationElementID2 != null) {
		    					Element translationItem2 = itemElements.get(translationElementID2).get(0);
		    					write(translationItem2, "S", "" + u2Interval.t0.value());
		    				}
		    				
		    				// convert back to original timeunit and set back in the intervals array
		    				Interval u1IntervalCorrected = u1Interval.convert(TimeUnit.NANOSECOND, Rounding.FLOOR);
		    				Interval u2IntervalCorrected = u2Interval.convert(TimeUnit.NANOSECOND, Rounding.FLOOR);
		    				
		    				intervals.set(index1, u1IntervalCorrected);
		    				intervals.set(index2, u2IntervalCorrected);
		    				tierUtterancesIntervalSet.setIntervals(uids, intervals);
		
		    				var str = "Utterance {ID=" + uid1 + "} and Utterance {ID=" + uid2 + "} had their overlap corrected.";
//		    				var public_str = "The following utterance times overlapped, and had their times modified to address the overlap: " + u1repr + " and " + u2repr;
		    				addToModifications(20082, str, null);
		    				u1loop = false;
		    				u2loop = false;
		    			}
		    		}
		
		    		// check if overlap still occurring, and revert intervals if so
		    		if (u1Interval.overlaps(u2Interval, Bound.EXCLUSIVE)) {
		    			u1Interval = new Interval(u1OrigInterval.t0, u1OrigInterval.t1);
		    			u2Interval = new Interval(u2OrigInterval.t0, u2OrigInterval.t1);
		
		    			addToErrors(10123, "Utterance {ID=" + uid1 + "} and Utterance {ID=" + uid2 + "} are overlapping.", "There are 2 overlapping utterances: " + u1repr + " and " + u2repr);
		    		}
		    	}	// for (int index2 = 1; index2 < uids.size(); index2++)
    	}
    }

    private void correctTranslations() {
    	var translationTrackEl = trackElements.get(Util.FREE_TEXT_FIELD_ID);
        var repr = "Track " + repr(translationTrackEl, "FID");

        var utterances = new HashMap<Element, Set<Element>>();

        for (Element utteranceEl: utteranceElements.values()) {
            var urepr = "Utterance " + repr(utteranceEl, "LABEL", "S", "E");
            	var uid = readOpt(utteranceEl, "ID").orElse(null);
            	var tid = utteranceTierMap.get(uid);

            var ustart = readLongOpt(utteranceEl, "S").orElse(null);
            var uend = readLongOpt(utteranceEl, "E").orElse(null);
            var uInterval = TimeRange.ABSOLUTE.time(ustart, TimeUnit.NANOSECOND)
                .to(TimeRange.ABSOLUTE.time(uend, TimeUnit.NANOSECOND));
            
            var translationElements = new HashSet<Element>();
            
            for (Element translationEntityEl: readChildren(translationTrackEl, "ENTITY")
            		.collect(Collectors.toCollection(ArrayList::new))) {
            	
            		var eid = readOpt(translationEntityEl, "ID").orElse(null);
            		var assumedUID = entityUtteranceMap.get(eid);
            		var assumedTID = utteranceTierMap.get(assumedUID);
            		
            		if ( ! tid.equals(assumedTID)) {
            			continue;		// this means the utterance is not from the same segment tier as this translation entity
            		}
            		
                var items = readChildren(translationEntityEl, "ITEM").collect(Collectors.toList());
                var itemEl = items.get(0);

                var tstart = readLongOpt(itemEl, "S").orElse(null);
                var tend = readLongOpt(itemEl, "E").orElse(null);
                var tInterval = TimeRange.ABSOLUTE.time(tstart, TimeUnit.NANOSECOND)
                    .to(TimeRange.ABSOLUTE.time(tend, TimeUnit.NANOSECOND));
                
                if (uInterval.overlaps(tInterval, Bound.EXCLUSIVE)) {
                	translationElements.add(translationEntityEl);
                }
            }
            
            utterances.put(utteranceEl, translationElements);
        }
        
        for (var entry: utterances.entrySet()) {
        	var utteranceEl = entry.getKey();
            var uid = readOpt(utteranceEl, "ID").orElse(null);
            var label = readOpt(utteranceEl, "LABEL").orElse(null);

            var ustart = readLongOpt(utteranceEl, "S").orElse(null);
            var uend = readLongOpt(utteranceEl, "E").orElse(null);
            var uInterval = TimeRange.ABSOLUTE.time(ustart, TimeUnit.NANOSECOND)
                .to(TimeRange.ABSOLUTE.time(uend, TimeUnit.NANOSECOND));
            
            var translationEls = entry.getValue();
            var size = translationEls.size();
            
            String tid1 = utteranceTierMap.get(uid);
			
			Element tier1 = tierElements.get(tid1);
			
			String pid1 = tierPartitionMap.get(tid1);
			
			Element part1 = partitionElements.get(pid1);

            var urepr = "Temporal Partition " + repr(part1, "LABEL") + " Segment Tier " + repr(tier1, "LABEL") + " Utterance " + repr(utteranceEl, "LABEL");

            expect(size <= 1, 10124, "Utterance " + uid + " has " + size + " overlapping translations.", "There is an utterance that has " + size + " overlapping translations: " + urepr);
        	
            if (size == 1) {
            	var translationEntityEl = translationEls.iterator().next();
                var tid = readOpt(translationEntityEl, "ID").orElse(null);
                
                var items = readChildren(translationEntityEl, "ITEM").collect(Collectors.toList());
                var itemEl = items.get(0);

                var tstart = readLongOpt(itemEl, "S").orElse(null);
                var tend = readLongOpt(itemEl, "E").orElse(null);
                var tInterval = TimeRange.ABSOLUTE.time(tstart, TimeUnit.NANOSECOND)
                    .to(TimeRange.ABSOLUTE.time(tend, TimeUnit.NANOSECOND));
                
                if (!tInterval.equals(uInterval)) {
                	writeLong(itemEl, "S", ustart);
                	writeLong(itemEl, "E", uend);
                	
                	var str = "Translation {ID=" + tid + "} had times reset to utterance interval.";
//                	var public_str = "Translation from " + urepr + " had times reset to the utterance interval.";
                    addToModifications(20083, str, null);
                }
            }
        }
    }
    
    private void populateEntityIntervals() {
    	// just set this up for each possible segment tier located below
    	for (String tid: tierPartitionMap.keySet()) {
    		entityIntervals.put(tid, new HashMap<String, IntervalSet<String>>());
    	}
    	
        var tracksEl = readChildOpt(dbEl, "TRACKS").orElse(null);

        if (expect(tracksEl != null, 10067, "Missing TRACKS element.", null)) {
            readChildren(tracksEl, "TRACK").forEach(tEl -> {
                var repr = "Track " + repr(tEl, "FID");
                var fid = readOpt(tEl, "FID").orElse(null);
                
                readChildren(tEl, "ENTITY").forEach(eEl -> {
                    var erepr = repr + " entity " + repr(eEl, "ID");
                    var eid = readOpt(eEl, "ID").orElse(null);
                    
                    if (deletedEntities.get(eid) != null && deletedEntities.get(eid).equals(fid))
                        return;
                    
                    var assumedUID = entityUtteranceMap.get(eid);
                    var assumedTID = utteranceTierMap.get(assumedUID);
                    
                    var segmentTierMap = entityIntervals.get(assumedTID);
                    
                    if (segmentTierMap.get(fid) == null) {
                    	segmentTierMap.put(fid, new IntervalSet<String>());
                    }

                    IntervalSet<String> intervals = segmentTierMap.get(fid);
                    
                    if (utteranceIntervals.get(assumedTID) == null) {
                    	utteranceIntervals.put(assumedTID,  new IntervalSet<String>());
                    }
                    
                    IntervalSet<String> tierUtterancesIntervalSet = utteranceIntervals.get(assumedTID);
                    
                    String entityPublicErrorString = getEntityPublicErrorString(eid, fid);
					
					boolean textOnly = readChildren(eEl, "ITEM").anyMatch(iEl -> readOpt(iEl, "TEXT_ONLY").orElse("F").equals("T"));
					if (textOnly)
						return;	// continue to next entity, do not add this to the intervals

                    var items = readChildren(eEl, "ITEM").collect(Collectors.toList());
					Element firstItem = items.get(0);
					Element lastItem = items.get(items.size() - 1);

					var start = readLongOpt(firstItem, "S").orElse(0L);
					var end = readLongOpt(lastItem, "E").orElse(0L);
                    
                    if (start != null && end != null) {
                    	var videoInterval = TimeRange.ABSOLUTE.time(start, TimeUnit.TIMESTAMP, Rounding.FLOOR)
                    			.to(TimeRange.ABSOLUTE.time(end, TimeUnit.TIMESTAMP, Rounding.FLOOR));
                    	expect(tierUtterancesIntervalSet.testOverlap(videoInterval, Bound.INCLUSIVE).isPresent(),
                    			10125, erepr + " does not overlap with any utterance.", entityPublicErrorString + " does not overlap with any utterance.");

                    	var interval = TimeRange.ABSOLUTE.time(start, TimeUnit.TIMESTAMP, Rounding.FLOOR)
                    			.to(TimeRange.ABSOLUTE.time(end, TimeUnit.TIMESTAMP, Rounding.FLOOR));
                    	intervals.add(eid, interval, Bound.EXCLUSIVE)
                    	.ifPresent(otherIDs -> {
                        	// GD: commenting as it should be reported as an error only if it not fixed - see below in correctEntityOverlaps
//                    		for (String otherID: otherIDs) {
//                        		var other_erepr = repr + " entity " + repr(entityElements.get(otherID), "ID");
//                        		addToErrors(erepr + " overlaps with another entity with ID " + otherID);
//                        		addToErrors(other_erepr + " overlaps with another entity with ID " + eid );
//                    		}
                    	})
                    	;
                    }
                });
            });
        }
    }
    
    private void correctEntityOverlaps() {
    	for (String tierID: tierPartitionMap.keySet()) {
	        	for (String fid: trackElements.keySet()) {
	        		if (fid.equals(Util.FREE_TEXT_FIELD_ID)) {
	    				// ignore translation entities as they always just align to the utterance - occurs elsewhere
	        			continue;
	        		}
	        		
	        		if (fid.equals(Util.DOM_HANDSHAPE_FIELD_ID) || fid.equals(Util.NDOM_HANDSHAPE_FIELD_ID)) {
	        			continue;
	        		}
	        		
	        		if (fid.equals(Util.DOM_LOCATION_FIELD_ID) || fid.equals(Util.NDOM_LOCATION_FIELD_ID)) {
	        			continue;
	        		}

	        		var segmentTierMap = entityIntervals.get(tierID);

	        		IntervalSet<String> fieldIntervalSet = segmentTierMap.get(fid);
	        		
	        		if (fieldIntervalSet == null)
	        			continue;
	
	        		ArrayList<String> uids = fieldIntervalSet.getKeys();
	        		ArrayList<Interval> intervals = fieldIntervalSet.getIntervals();
	
	        		if (uids.size() != intervals.size()) {
	        			// something wrong happened in building the entity intervals - needs to be exactly equal
	        			return;
	        		}
	
	        		for (int index2 = 1; index2 < uids.size(); index2++) {
	        			int index1 = index2 - 1;
	
	        			Interval e1Interval = intervals.get(index1).convert(TimeUnit.TIMESTAMP, Rounding.NEAREST);
	        			Interval e2Interval = intervals.get(index2).convert(TimeUnit.TIMESTAMP, Rounding.NEAREST);
	
	        			String eid1 = uids.get(index1);
	        			String eid2 = uids.get(index2);
	
	    				String entityPublicErrorString1 = getEntityPublicErrorString(eid1, fid);
	    				String entityPublicErrorString2 = getEntityPublicErrorString(eid2, fid);
	        			
	        			// if the intervals do not overlap, do not do any of the below and proceed to the next pair
	        			if ( ! e1Interval.overlaps(e2Interval, Bound.EXCLUSIVE)) {
	        				continue;
	        			}
	
	        			Element e1 = entityElements.get(eid1);
	        			Element e2 = entityElements.get(eid2);
	        			
	    				ArrayList<Element> e1items = itemElements.get(eid1);
	
	    				Element e1onset = null, e1initialHold = null, e1main = null, e1finalHold = null, e1offset = null;
	    				Element e1operativeStartItem = null, e1operativeEndItem = null;
	    				for (Element item: e1items) {
	    					var itemText = getCharacterDataFromElement(item);
	    					if (itemText.equals(Constants.ONSET))
	    						e1onset = item;
	    					else if (itemText.equals(Constants.INITIAL_HOLD))
	    						e1initialHold = item;
	    					else if (itemText.equals(Constants.FINAL_HOLD))
	    						e1finalHold = item;
	    					else if (itemText.equals(Constants.OFFSET))
	    						e1offset = item;
	    					else
	    						e1main = item;
	    				}
	
	    				if (e1onset != null)
	    					e1operativeStartItem = e1onset;
	    				else if (e1initialHold != null)
	    					e1operativeStartItem = e1initialHold;
	    				else
	    					e1operativeStartItem = e1main;
	
	    				if (e1offset != null)
	    					e1operativeEndItem = e1offset;
	    				else if (e1finalHold != null)
	    					e1operativeEndItem = e1finalHold;
	    				else
	    					e1operativeEndItem = e1main;
	                	
	    				Element e1LastItem = e1items.get(e1items.size() - 1);
	
	    				var e1LastItemStart = readLongOpt(e1LastItem, "S").orElse(0L);
	    				var e1LastItemEnd = readLongOpt(e1LastItem, "E").orElse(0L);
	    				var e1LastItemInterval = TimeRange.ABSOLUTE.time(e1LastItemStart, TimeUnit.TIMESTAMP)
	                			.to(TimeRange.ABSOLUTE.time(e1LastItemEnd, TimeUnit.TIMESTAMP)).convert(TimeUnit.TIMESTAMP, Rounding.NEAREST);
	        			
	    				ArrayList<Element> e2items = itemElements.get(eid2);
	
	    				Element e2onset = null, e2initialHold = null, e2main = null, e2finalHold = null, e2offset = null;
	    				Element e2operativeStartItem = null, e2operativeEndItem = null;
	    				for (Element item: e2items) {
	    					var itemText = getCharacterDataFromElement(item);
	    					if (itemText.equals(Constants.ONSET))
	    						e2onset = item;
	    					else if (itemText.equals(Constants.INITIAL_HOLD))
	    						e2initialHold = item;
	    					else if (itemText.equals(Constants.FINAL_HOLD))
	    						e2finalHold = item;
	    					else if (itemText.equals(Constants.OFFSET))
	    						e2offset = item;
	    					else
	    						e2main = item;
	    				}
	
	    				if (e2onset != null)
	    					e2operativeStartItem = e2onset;
	    				else if (e2initialHold != null)
	    					e2operativeStartItem = e2initialHold;
	    				else
	    					e2operativeStartItem = e2main;
	
	    				if (e2offset != null)
	    					e2operativeEndItem = e2offset;
	    				else if (e2finalHold != null)
	    					e2operativeEndItem = e2finalHold;
	    				else
	    					e2operativeEndItem = e2main;
	    				
	    				Element e2FirstItem = e2items.get(0);
	
	    				var e2FirstItemStart = readLongOpt(e2FirstItem, "S").orElse(0L);
	    				var e2FirstItemEnd = readLongOpt(e2FirstItem, "E").orElse(0L);
	    				var e2FirstItemInterval = TimeRange.ABSOLUTE.time(e2FirstItemStart, TimeUnit.TIMESTAMP)
	                			.to(TimeRange.ABSOLUTE.time(e2FirstItemEnd, TimeUnit.TIMESTAMP)).convert(TimeUnit.TIMESTAMP, Rounding.NEAREST);
	
	        			Interval e1OrigInterval = new Interval(e1Interval.t0, e1Interval.t1);
	        			Interval e2OrigInterval = new Interval(e2Interval.t0, e2Interval.t1);
	
	        			boolean e1loop = true;
	        			boolean e2loop = true;
	    				
	    				boolean flip = true;
	    				boolean doFlip = true;
	    				
	    				// this is to force the phase item to be shortened as much as possible, before trying to move the main item
	    				if (e1LastItem == e1main) {
	    					if (e2FirstItem != e2main) {
	    						// only e2 is a phase
	    						e1loop = false;
	    						doFlip = false;
	    					}
	    				} else {
	    					if (e2FirstItem == e2main) {
	    						// only e1 is a phase
	    						e2loop = false;
	    						doFlip = false;
	    					}
	    				}
	    				
	        			while (e1loop || e2loop) {
	        				if (e1Interval.overlaps(e2Interval, Bound.EXCLUSIVE)) {
	        					if (flip && e1loop) {
	        						// try to decrease the end time of e1 one frame at a time, using the last item of entity e1
	        						e1LastItemInterval = new Interval(e1LastItemInterval.t0, e1LastItemInterval.t1.subtract(timestep));
	
	        						if (e1LastItemInterval.t0.greaterThanOrEq(e1LastItemInterval.t1)) {
	        							// cannot move phase to zero-time, stop
	        							e1LastItemInterval = new Interval(e1LastItemInterval.t0, e1LastItemInterval.t1.add(timestep));
	
	        							e1loop = false;
	        							
	        							if (!doFlip) {
	        								doFlip = true;
	        								e2loop = true;
	        							}
	        						} else {
	        							// update the main entity interval and check on the next loop
	        							e1Interval = new Interval(e1Interval.t0, e1Interval.t1.subtract(timestep));
	        						}
	        					} else if (!flip && e2loop) {
	        						// try to increase the start time of e2 one frame at a time, using the first item of entity e2
	        						e2FirstItemInterval = new Interval(e2FirstItemInterval.t0.add(timestep), e2FirstItemInterval.t1);
	
	        						if (e2FirstItemInterval.t0.greaterThanOrEq(e2FirstItemInterval.t1)) {
	        							// cannot move phase to zero-time, stop
	        							e2FirstItemInterval = new Interval(e2FirstItemInterval.t0.subtract(timestep), e2FirstItemInterval.t1);
	
	        							e2loop = false;
	        							
	        							if (!doFlip) {
	        								doFlip = true;
	        								e1loop = true;
	        							}
	        						} else {
	        							// update the main entity interval and check on the next loop
	        							e2Interval = new Interval(e2Interval.t0.add(timestep), e2Interval.t1);
	        						}
	        					}
	
	    						if (doFlip)
	    							flip = !flip;
	        				} else {
	        					write(e1LastItem, "E", "" + e1Interval.t1.value());
	        					write(e2FirstItem, "S", "" + e2Interval.t0.value());
	
	        					var str = "Entity {ID=" + eid1 + "} and Entity {ID=" + eid2 + "} had their overlap corrected.";
//	        					var public_str = "The following entity times overlapped, and had their times modified to address the overlap: " + entityPublicErrorString1 + " and " + entityPublicErrorString2;
	        					addToModifications(20084, str, null);
	        					e1loop = false;
	        					e2loop = false;
	        				}
	        			}
	
	        			// check if overlap still occurring, and revert intervals if so
	        			if (e1Interval.overlaps(e2Interval, Bound.EXCLUSIVE)) {
	        				// else revert the intervals back to orig
	        				e1Interval = new Interval(e1OrigInterval.t0, e1OrigInterval.t1);
	        				e2Interval = new Interval(e2OrigInterval.t0, e2OrigInterval.t1);
	        				
	        				addToErrors(10126, "Entity {ID=" + eid1 + "} and Entity {ID=" + eid2 + "} are overlapping.", "There are 2 overlapping entities: " + entityPublicErrorString1 + " and " + entityPublicErrorString2);
	        			}
	        			
	        		}	// for (int index2 = 1; index2 < uids.size(); index2++)
	        		
	        	}	// 	for (String fid: trackElements.keySet())
	        	
    	}	// for (String tid: tierPartitionMap.keySet())
    	
    }
    
    private String getEntityPublicErrorString(String eid, String fid) {
    	
    	if (isBlank(fid))
    		return null;
    	
        String entityPublicErrorStr ="";

    	String partitionLabel = null, segmentTierLabel = null, utteranceLabel = null;
        	Element entityEl = entityElements.get(eid);
        	if (entityEl != null) {
	        	var parentEntityID = readOpt(entityEl, "PARENT_ENTITY_ID").orElse(null);
	        	if (isNotBlank(parentEntityID)) {
	        		Element utteranceEl = utteranceElements.get(parentEntityID);
	        		if (utteranceEl != null) {
	        			utteranceLabel = readOpt(utteranceEl, "LABEL").orElse(null);
	        			Element segmentTierEl = (Element) utteranceEl.getParentNode();
	        			if (segmentTierEl != null) {
	        				segmentTierLabel = readOpt(segmentTierEl, "LABEL").orElse(null);
	        				Element partitionEl = (Element) segmentTierEl.getParentNode();
	        				if (partitionEl != null) {
	        					partitionLabel = readOpt(partitionEl, "LABEL").orElse(null);
	        				}
	        			}
	        		}
	        	} else {
	        		String assumedUID = entityUtteranceMap.get(eid);
	        		if (isNotBlank(assumedUID)) {
	        			Element utteranceEl = utteranceElements.get(assumedUID);
	        			if (utteranceEl != null) {
	        				utteranceLabel = readOpt(utteranceEl, "LABEL").orElse(null);
	        			}
	        			
	        			String assumedTID = utteranceTierMap.get(assumedUID);
	        			if (isNotBlank(assumedTID)) {
	        				Element segmentTierEl = tierElements.get(assumedTID);
	        				if (segmentTierEl != null) {
	        					segmentTierLabel = readOpt(segmentTierEl, "LABEL").orElse(null);
	        				}
	        				
	        				String assumedPID = tierPartitionMap.get(assumedTID);
	        				if (isNotBlank(assumedTID)) {
	        					Element partitionEl = (Element) partitionElements.get(assumedPID);
	        					if (partitionEl != null) {
	        						partitionLabel = readOpt(partitionEl, "LABEL").orElse(null);
	        					}
	        				}
	        			}
	        		}
	        	}
        	}
    	
		ArrayList<Element> items = itemElements.get(eid);
		if (items != null && !items.isEmpty()) {
			Element onset = null, initialHold = null, main = null, finalHold = null, offset = null;
			Element operativeStartItem = null, operativeEndItem = null;
			for (Element item: items) {
				var itemText = getCharacterDataFromElement(item);
				if (itemText.equals(Constants.ONSET))
					onset = item;
				else if (itemText.equals(Constants.INITIAL_HOLD))
					initialHold = item;
				else if (itemText.equals(Constants.FINAL_HOLD))
					finalHold = item;
				else if (itemText.equals(Constants.OFFSET))
					offset = item;
				else
					main = item;
			}

			if (onset != null)
				operativeStartItem = onset;
			if (initialHold != null)
				operativeStartItem = initialHold;
			else
				operativeStartItem = main;

			if (offset != null)
				operativeEndItem = offset;
			if (finalHold != null)
				operativeEndItem = finalHold;
			else
				operativeEndItem = main;
			
			if (main != null) {
				String entityDescriptor = "Entity";
				if (isNotBlank(fid) && (fid.equals(Util.DOM_GLOSS_FIELD_ID) || fid.equals(Util.NDOM_GLOSS_FIELD_ID))){
					entityDescriptor = "Gloss";
				}
				
				var mainText = getCharacterDataFromElement(main);
				
				if (isNotBlank(mainText)) {
					entityPublicErrorStr += entityDescriptor + " (" + repr(mainText) + ") ";
				} else {
					entityPublicErrorStr += entityDescriptor + " ";
				}
			}
			
			if (operativeStartItem != null && operativeEndItem != null) {
				var start = readLongOpt(operativeStartItem, "S").orElse(null);
				var end = readLongOpt(operativeEndItem, "E").orElse(null);

				var startTimeInfo = TimeRange.ABSOLUTE.time(start, TimeUnit.TIMESTAMP, Rounding.FLOOR).convert(TimeUnit.NANOSECOND);
				var endTimeInfo = TimeRange.ABSOLUTE.time(end, TimeUnit.TIMESTAMP, Rounding.FLOOR).convert(TimeUnit.NANOSECOND);
				Interval intervalOrig = new Interval(startTimeInfo, endTimeInfo);
				Interval interval = intervalOrig.convert(frameTimeUnit, Rounding.NEAREST);

				if (interval != null) {
					entityPublicErrorStr += "(Start: " + repr("" + interval.t0.value()) + " frames, End: " + repr("" + interval.t1.value()) + " frames) ";
				}
			}
		}

        String fieldName = null;
        var field = defScheme3_3_5.getField(fid);
        if (field != null) {
        		fieldName = field.getName();
        } else {
            var localFieldElement = localSchemeFieldElements.get(fid);
            if (localFieldElement != null) {
                fieldName = readOpt(localFieldElement, "NAME").orElse(null);
            }
        }
        
        if (isNotBlank(fieldName)) {
        		entityPublicErrorStr += "of Field (" + repr(fieldName) + ") ";
        }
    	
    	if (isNotBlank(utteranceLabel)) {
    		entityPublicErrorStr += "of Utterance (" + repr(utteranceLabel) + ") ";
    	}
    	
    	if (isNotBlank(segmentTierLabel) && !onlyOneSegmentTier) {
    		entityPublicErrorStr += "of Segment Tier (" + repr(segmentTierLabel) + ") ";
    	}
    	
    	if (isNotBlank(partitionLabel) && !onlyOneSegmentTier) {
    		entityPublicErrorStr += "of Temporal Partition (" + repr(partitionLabel) + ") ";
    	}

        return entityPublicErrorStr;
    }
}
